Package: netmeta
Title: Network Meta-Analysis using Frequentist Methods
Version: 3.6-1
Date: 2026-06-19
Depends: R (>= 4.0.0), meta (>= 8.5-0)
Imports: Matrix, metafor, MASS, mvtnorm, magic, igraph, ggplot2,
        colorspace, grid, dplyr, magrittr, methods, stringr, tidyr,
        ggrepel
Suggests: Rgraphviz, graph, rgl, gridExtra, tictoc, writexl, R.rsp,
        cccp, brglm2, crossnma, gemtc, rmarkdown, knitr
Authors@R: c(person("Gerta", "Rücker",
                    role = "aut",
                    comment = c(ORCID = "0000-0002-2192-2560")),
             person("Ulrike", "Krahn",
                    role = "aut"),
             person("Jochem", "König",
                    role = "aut",
                    comment = c(ORCID = "0000-0003-4683-0360")),
             person("Orestis", "Efthimiou",
                    role = "aut",
                    comment = c(ORCID = "0000-0002-0955-7572")),
             person("Annabel", "Davies",
                    role = "aut",
                    comment = c(ORCID = "0000-0003-2320-7701")),
             person("Theodoros", "Papakonstantinou",
                    role = "aut",
                    comment = c(ORCID = "0000-0002-6630-6817")),
             person("Theodoros", "Evrenoglou",
                    role = "ctb",
                    comment = c(ORCID = "0000-0003-3336-8058")),
             person("Krzysztof", "Ciomek",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-2293-2146",
                    paste("Author of the original MIT-licensed",
                          "R package hasseDiagram"))),
             person("Nana-adjoa", "Kwarteng",
                    role = "ctb",
                    comment = c(ORCID = "0009-0009-0502-6887")),
             person("Guido", "Schwarzer",
                    role = c("aut", "cre"),
                    email = "guido.schwarzer@uniklinik-freiburg.de",
                    comment = c(ORCID = "0000-0001-6214-9087")))
URL: https://github.com/guido-s/netmeta
Description: A comprehensive set of functions providing frequentist methods for network meta-analysis (Balduzzi et al., 2023) <doi:10.18637/jss.v106.i02> and supporting Schwarzer et al. (2015) <doi:10.1007/978-3-319-21416-0>, Chapter 8 "Network Meta-Analysis":
 - frequentist network meta-analysis following Rücker (2012) <doi:10.1002/jrsm.1058>;
 - additive network meta-analysis for combinations of treatments (Rücker et al., 2020) <doi:10.1002/bimj.201800167>;
 - network meta-analysis of binary data using the Mantel-Haenszel or non-central hypergeometric distribution method (Efthimiou et al., 2019) <doi:10.1002/sim.8158>, or penalised logistic regression (Evrenoglou et al., 2022) <doi:10.1002/sim.9562>;
 - rankograms and ranking of treatments by the Surface under the cumulative ranking curve (SUCRA) (Salanti et al., 2013) <doi:10.1016/j.jclinepi.2010.03.016>;
 - ranking of treatments using P-scores (frequentist analogue of SUCRAs without resampling) according to Rücker & Schwarzer (2015) <doi:10.1186/s12874-015-0060-8>;
 - split direct and indirect evidence to check consistency (Dias et al., 2010) <doi:10.1002/sim.3767>, (Efthimiou et al., 2019) <doi:10.1002/sim.8158>;
 - scatter plot to visualize local inconsistency (Wilson et al., 2026) <doi:10.1017/rsm.2026.10082>;
 - league table with network meta-analysis results;
 - 'comparison-adjusted' funnel plot (Chaimani & Salanti, 2012) <doi:10.1002/jrsm.57>;
 - net heat plot and design-based decomposition of Cochran's Q according to Krahn et al. (2013) <doi:10.1186/1471-2288-13-35>;
 - measures characterizing the flow of evidence between two treatments by König et al. (2013) <doi:10.1002/sim.6001>;
 - automated drawing of network graphs described in Rücker & Schwarzer (2016) <doi:10.1002/jrsm.1143>;
 - partial order of treatment rankings ('poset') and Hasse diagram for 'poset' (Carlsen & Bruggemann, 2014) <doi:10.1002/cem.2569>; (Rücker & Schwarzer, 2017) <doi:10.1002/jrsm.1270>;
 - contribution matrix as described in Papakonstantinou et al. (2018) <doi:10.12688/f1000research.14770.3> and Davies et al. (2022) <doi:10.1002/sim.9346>;
 - network meta-regression with a single continuous or binary covariate (Kwarteng et al., 2026) <doi:10.21203/rs.3.rs-8235913/v1>;
 - subgroup network meta-analysis.
License: GPL (>= 2) | file LICENSE
Encoding: UTF-8
VignetteBuilder: R.rsp, knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-19 16:53:32 UTC; sc
Author: Gerta Rücker [aut] (ORCID: <https://orcid.org/0000-0002-2192-2560>),
  Ulrike Krahn [aut],
  Jochem König [aut] (ORCID: <https://orcid.org/0000-0003-4683-0360>),
  Orestis Efthimiou [aut] (ORCID:
    <https://orcid.org/0000-0002-0955-7572>),
  Annabel Davies [aut] (ORCID: <https://orcid.org/0000-0003-2320-7701>),
  Theodoros Papakonstantinou [aut] (ORCID:
    <https://orcid.org/0000-0002-6630-6817>),
  Theodoros Evrenoglou [ctb] (ORCID:
    <https://orcid.org/0000-0003-3336-8058>),
  Krzysztof Ciomek [ctb] (ORCID: <https://orcid.org/0000-0002-2293-2146>,
    Author of the original MIT-licensed R package hasseDiagram),
  Nana-adjoa Kwarteng [ctb] (ORCID:
    <https://orcid.org/0009-0009-0502-6887>),
  Guido Schwarzer [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6214-9087>)
Maintainer: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
Repository: CRAN
Date/Publication: 2026-06-20 05:10:21 UTC
Built: R 4.5.2; ; 2026-06-20 06:48:44 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,netconnection)
S3method(as.data.frame,netmeta)
S3method(baujat,netpairwise)
S3method(createC,default)
S3method(createC,matrix)
S3method(createC,netcomb)
S3method(createC,netconnection)
S3method(createC,netmeta)
S3method(drawDetails,hasseGrob)
S3method(forest,netbind)
S3method(forest,netcomb)
S3method(forest,netcomparison)
S3method(forest,netcomplex)
S3method(forest,netmeta)
S3method(forest,netpairwise)
S3method(forest,netsplit)
S3method(forest,subgroup.netmeta)
S3method(funnel,netmeta)
S3method(funnel,netpairwise)
S3method(hasse,netposet)
S3method(heatplot,netmeta)
S3method(merge,pairwise)
S3method(metabias,netmeta)
S3method(metabias,netpairwise)
S3method(metacum,netpairwise)
S3method(metainf,netpairwise)
S3method(metareg,netpairwise)
S3method(netconnection,default)
S3method(netconnection,netcomb)
S3method(netconnection,netmeta)
S3method(netconnection,pairwise)
S3method(netdistance,default)
S3method(netdistance,netcomb)
S3method(netdistance,netconnection)
S3method(netdistance,netmeta)
S3method(netgraph,discomb)
S3method(netgraph,netcomb)
S3method(netgraph,netconnection)
S3method(netgraph,netimpact)
S3method(netgraph,netmeta)
S3method(netmetareg,default)
S3method(netmetareg,netmeta)
S3method(netpairwise,netmeta)
S3method(netpairwise,netmetabin)
S3method(plot,netbind)
S3method(plot,netcomb)
S3method(plot,netcomparison)
S3method(plot,netcomplex)
S3method(plot,netmeta)
S3method(plot,netpairwise)
S3method(plot,netposet)
S3method(plot,netrank)
S3method(plot,netsplit)
S3method(plot,rankogram)
S3method(plot,subgroup.netmeta)
S3method(print,decomp.design)
S3method(print,hatmatrix)
S3method(print,metabias.netpairwise)
S3method(print,metacum.netpairwise)
S3method(print,metainf.netpairwise)
S3method(print,metareg.netpairwise)
S3method(print,netbind)
S3method(print,netcomb)
S3method(print,netcomparison)
S3method(print,netcomplex)
S3method(print,netconnection)
S3method(print,netcontrib)
S3method(print,netdistance)
S3method(print,netimpact)
S3method(print,netleague)
S3method(print,netmeta)
S3method(print,netmetareg)
S3method(print,netpairwise)
S3method(print,netposet)
S3method(print,netrank)
S3method(print,netsplit)
S3method(print,nettable)
S3method(print,plot.netsplit)
S3method(print,rankogram)
S3method(print,subgroup.netmeta)
S3method(print,summary.netcomb)
S3method(print,summary.netconnection)
S3method(print,summary.netmeta)
S3method(print,summary.netmetareg)
S3method(print,summary.netpairwise)
S3method(print,summary.rankogram)
S3method(print,trimfill.netpairwise)
S3method(radial,netmeta)
S3method(radial,netpairwise)
S3method(rankogram,default)
S3method(rankogram,netmeta)
S3method(subgroup,netmeta)
S3method(summary,netcomb)
S3method(summary,netconnection)
S3method(summary,netmeta)
S3method(summary,netmetareg)
S3method(summary,netpairwise)
S3method(summary,rankogram)
S3method(trimfill,netpairwise)
export(combinations)
export(comps)
export(createC)
export(crossnma2netmeta)
export(decomp.design)
export(discomb)
export(gemtc2netmeta)
export(hasse)
export(hatmatrix)
export(heatplot)
export(invmat)
export(netbind)
export(netcomb)
export(netcomparison)
export(netcomplex)
export(netconnection)
export(netcontrib)
export(netdistance)
export(netgraph)
export(netheat)
export(netimpact)
export(netleague)
export(netmatrix)
export(netmeasures)
export(netmeta)
export(netmetabin)
export(netmetareg)
export(netpairwise)
export(netposet)
export(netrank)
export(netsplit)
export(nettable)
export(rankogram)
export(settings.netmeta)
export(subgroup)
export(treats)
importFrom(MASS,ginv)
importFrom(Matrix,bdiag)
importFrom(colorspace,choose_palette)
importFrom(colorspace,diverge_hcl)
importFrom(colorspace,diverge_hsv)
importFrom(colorspace,heat_hcl)
importFrom(colorspace,rainbow_hcl)
importFrom(colorspace,sequential_hcl)
importFrom(colorspace,terrain_hcl)
importFrom(dplyr,"%>%")
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,count)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,last_col)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_step)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_dark)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colours)
importFrom(grDevices,heat.colors)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,xy.coords)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(grid,arrow)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,convertX)
importFrom(grid,convertY)
importFrom(grid,drawDetails)
importFrom(grid,gpar)
importFrom(grid,grid.clip)
importFrom(grid,grid.draw)
importFrom(grid,grid.lines)
importFrom(grid,grid.newpage)
importFrom(grid,grid.rect)
importFrom(grid,grid.roundrect)
importFrom(grid,grid.text)
importFrom(grid,grob)
importFrom(grid,popViewport)
importFrom(grid,pushViewport)
importFrom(grid,stringWidth)
importFrom(grid,unit)
importFrom(grid,viewport)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,all_simple_paths)
importFrom(igraph,delete_edges)
importFrom(igraph,get.shortest.paths)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_edgelist)
importFrom(igraph,gsize)
importFrom(igraph,head_of)
importFrom(igraph,set_edge_attr)
importFrom(igraph,set_vertex_attr)
importFrom(igraph,tail_of)
importFrom(magic,adiag)
importFrom(magrittr,"%<>%")
importFrom(meta,backtransf)
importFrom(meta,baujat)
importFrom(meta,ci)
importFrom(meta,cilayout)
importFrom(meta,forest)
importFrom(meta,funnel)
importFrom(meta,gs)
importFrom(meta,longarm)
importFrom(meta,metabias)
importFrom(meta,metabin)
importFrom(meta,metacont)
importFrom(meta,metacum)
importFrom(meta,metagen)
importFrom(meta,metainc)
importFrom(meta,metainf)
importFrom(meta,metareg)
importFrom(meta,pairwise)
importFrom(meta,radial)
importFrom(meta,settings.meta)
importFrom(meta,trimfill)
importFrom(metafor,bldiag)
importFrom(metafor,contrmat)
importFrom(metafor,rma.mv)
importFrom(metafor,rma.uni)
importFrom(methods,as)
importFrom(mvtnorm,rmvnorm)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,dist)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,hclust)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(stats,reshape)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(stringr,str_length)
importFrom(tidyr,pivot_longer)
importFrom(utils,capture.output)
importFrom(utils,installed.packages)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)

isCol <- function(data, varname) {
  !is.null(data) & varname %in% names(data)
}
is_log_effect <- function(x)
  x %in% c("PLN", "IRLN", "MLN")
is_relative_effect <- function(x)
  x %in% c("HR", "OR", "RR", "IRR", "ROM", "DOR")
is_untransformed <- function(x)
  x %in% c("COR", "PRAW", "IR", "RD", "IRD", "MD", "SMD", "MRAW")
is_wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
  if (is.numeric(x))
    res <- abs(x - round(x)) < tol
  else
    res <- NA
  ##
  res
}
is_zero <- function(x, n = 10)
  abs(x) < n * .Machine$double.eps
is_wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
  if (is.numeric(x))
    res <- abs(x - round(x)) < tol
  else
    res <- NA
  ##
  res
}

nmr_results <- function(x,
                        covar = x$.netmeta$covar,
                        covar.name = x$.netmeta$covar.name) {
  
  # Link abbreviated treatment names with original treatment names
  #
  trts <- x$.netmeta$trts
  trts.abbr <- x$.netmeta$trts.abbr
  #
  assumption <- x$.netmeta$assumption
  reference.group <- x$.netmeta$reference.group
  
  # Get rid of warning "no visible binding for global variable"
  #
  treat <- treat_temp <- NULL
  
  dat <- data.frame(treat = "",
                    coef = x$b, se = x$se,
                    lower = x$ci.lb, upper = x$ci.ub,
                    z = x$zval, pval = x$pval)
  #
  # Remove back ticks which were created in the manual model matrix
  #
  rownames(dat) <- gsub("^`|`$", "", rownames(dat))
  #
  dat$treat <- rnam <- rownames(dat)
  #
  # Data set with main effects
  #
  sel_d <- !grepl(":", rnam)
  dat_d <- dat[sel_d, , drop = FALSE]
  dat_d$type <- "d"
  #
  rownames(dat_d) <- paste0("d[", rnam[sel_d], "]")
  #
  if (is.null(covar)) {
    return(dat_d)
  }
  else {
    is_num <- is.numeric(covar)
  }
  #
  # Add variables relevant for interaction terms
  #
  dat_d$covar <- NA
  #
  if (!is_num) {
    dat_d$cov_lvl <- NA
    dat_d$cov_ref <- NA
  }
  #
  # Data set with interaction effects
  #
  dat_beta <- dat[!sel_d, , drop = FALSE]
  #
  # Split 'rownames' using strsplit()
  #
  split_beta <- strsplit(rownames(dat_beta), "[:]")
  #
  trts_beta <- sapply(split_beta, first)
  lvls_beta <- sapply(split_beta, second)
  #
  dat_beta$type <- "beta"
  dat_beta$treat <- trts_beta
  dat_beta$covar <- covar.name
  #
  if (is_num) {
    row.names(dat_beta) <- paste0("beta[", rnam[!sel_d], "]")
  }
  else {
    # Add original treatment names to merge results
    # (for the common assumption only the covariates observed for the
    #  reference treatment are relevant)
    #
    dat_beta %<>%
      mutate(treat_temp = ifelse(treat == "nonref", reference.group, treat),
             treat_long =
               as.character(
                 factor(treat_temp, levels = trts.abbr, labels = trts))) %>%
      select(-treat_temp)
    #
    # Get observed covariate levels for each treatment
    #
    dat_cov <- x$.netmeta$x$data[, c("treat1", "treat2", covar.name)]
    #
    # Convert to long format
    #
    unique_lvls <- rbind(setNames(dat_cov[, c("treat1", covar.name)],
                                  c("treat_long", "lvl")),
                         setNames(dat_cov[, c("treat2", covar.name)],
                                  c("treat_long", "lvl")))
    #
    unique_lvls <- unique(unique_lvls)
    # Add treatment abbreviations
    unique_lvls <- merge(unique_lvls,
                         data.frame(treat_long = trts,
                                    treat = trts.abbr),
                         by = "treat_long", all.x = TRUE)
    #
    if (assumption == "common") {
      unique_lvls$treat <-
        ifelse(unique_lvls$treat == reference.group,
               "nonref",
               unique_lvls$treat)
    }
    #
    # Capture estimated factor levels
    #
    dat_beta$lvl <- gsub(covar.name, "", lvls_beta)
    #
    # Get the non-included covariate levels aka covariate references
    #
    ref_lvls <-
      merge(unique_lvls, dat_beta,
            by = c("treat_long", "lvl", "treat"),
            all.x = TRUE)
    #
    ref_lvls <- ref_lvls[is.na(ref_lvls$coef), c("treat_long", "lvl", "treat")]
    colnames(ref_lvls)[colnames(ref_lvls) == "lvl"] <- "cov_ref"
    #
    # Should be 1 to many. If there is an issue then it is likely in rma.mv()
    #
    dat_beta <-
      merge(dat_beta, ref_lvls,
            by = c("treat_long", "treat"), all.x = TRUE)
    colnames(dat_beta)[colnames(dat_beta) == "lvl"] <- "cov_lvl"
    #
    # Remove variable treat_long which we only used for merging
    #
    dat_beta <- dat_beta[, names(dat_beta) != "treat_long"]
    #
    rnam_beta <- paste0("beta[", rnam[!sel_d], "]")
    #
    for (i in seq_along(dat_beta$cov_lvl)) {
      rnam_beta[i] <- gsub(dat_beta$cov_lvl[i], "", rnam_beta[i])
    }
    #
    row.names(dat_beta) <- rnam_beta
  }
  #
  res <- rbind(dat_d, dat_beta)
  #
  attr(res, "reference.group") <- reference.group
  attr(res, "covar.name") <- covar.name
  #
  res <- res[, c("type", "treat",
                 if (!is_num) "cov_lvl", if (!is_num) "cov_ref",
                 "coef", "se", "lower", "upper", "z", "pval")]
  #
  res
}

nmr_full_results <- function(x) {
  # Extract NMR attributes / assumptions
  #
  assumption <- x$.netmeta$assumption
  reference.group <- x$.netmeta$reference.group
  consistency <- x$.netmeta$consistency
  covar <- x$.netmeta$covar
  # The issues with factors with more than two levels are the same for
  # character covariates with more than two levels
  #
  covar_is_factor <- is.factor(covar) | is.character(covar)
  #
  sep.trts <- x$.netmeta$x$sep.trts
  #
  # Extract estimates from rma.mv()
  #
  results <- x$results
  
  # Get rid of warning "no visible binding for global variable"
  #
  comparison <- cov <- cov.default <- interim.x <- interim.y <-
    ref.x <- ref.y <- treat <- type <- value <- treat1 <- treat2 <- NULL
  
  #
  # Generate matrix combinations for consistent parameters
  #
  sel.d <- results$type == "d"
  mat_d <- outer(c(0, results$coef[sel.d]), c(0, results$coef[sel.d]), "-")
  rownames(mat_d) <- colnames(mat_d) <- c(reference.group, results$treat[sel.d])
  #
  Cov <- vcov(x)
  #
  Cov_d <- Cov[sel.d, sel.d, drop = FALSE]
  Cov_d <- cbind(0, Cov_d)
  Cov_d <- rbind(0, Cov_d)
  rownames(Cov_d)[1] <- colnames(Cov_d)[1] <- reference.group
  #
  mat_se.d <- sqrt(outer(diag(Cov_d), diag(Cov_d), "+") - 2 * Cov_d)
  #
  # Convert to data frame
  #
  dat_d <- dat_se.d <- NULL
  #
  for (i in colnames(mat_d)) {
    # Data set with estimates
    #
    dat_d.i <- as.data.frame(mat_d)[i]
    names(dat_d.i) <- "d"
    #
    dat_d.i %<>%
      mutate(comparison = paste(rownames(dat_d.i), i, sep = sep.trts)) %>%
      select("comparison", "d")
    #
    dat_d <- rbind(dat_d, dat_d.i)
    #
    # Data set with standard errors
    #
    dat_se.d.i <- as.data.frame(mat_se.d)[i]
    names(dat_se.d.i) <- "se.d"
    #
    dat_se.d.i %<>%
      mutate(comparison = paste(rownames(dat_se.d.i), i, sep = sep.trts)) %>%
      select("comparison", "se.d")
    #
    dat_se.d <- rbind(dat_se.d, dat_se.d.i)
  }
  #
  rownames(dat_d) <- seq_len(nrow(dat_d))
  rownames(dat_se.d) <- seq_len(nrow(dat_se.d))
  
  #
  # Covariance for interaction terms
  #
  if (is.null(covar) | (covar_is_factor & length(unique(covar)) > 2)) {
    # We don't have a solution for factor / categorical covariates with more
    # than two levels. We lose the information on comparator / reference group
    # assignment which will later be important for more than two levels
    #
    return(merge(dat_d, dat_se.d, by = "comparison", all.x = TRUE))
  }  else if (consistency) {
    if (assumption == "common") {
      # The common assumption has one beta per treatment
      #
      mat_beta <-
        outer(c(0, rep(results$coef[!sel.d], sum(sel.d))),
              c(0, rep(results$coef[!sel.d], sum(sel.d))), "-")
      rownames(mat_beta) <- colnames(mat_beta) <-
        c(reference.group, results$treat[sel.d])
      #
      # Single beta coefficient, i.e., no covariances
      #
      var_beta <- Cov[!sel.d, !sel.d]
      #
      mat_se.beta <-
        sqrt(outer(c(0, rep(var_beta, sum(sel.d))),
                   c(0, rep(var_beta, sum(sel.d))),
                   FUN = function(x, y) abs(x - y)))
      rownames(mat_se.beta) <- colnames(mat_se.beta) <-
        c(reference.group, results$treat[sel.d])
      #
      # Covariances
      #
      Cov_d_beta <- Cov[!sel.d, sel.d]
      #
      all_se.beta <- dat_d %>% select(comparison) %>% mutate(cov.default = 0)
      #
      dat_cov_d_beta <-
        data.frame(comparison =
                     paste(names(Cov_d_beta), reference.group, sep = sep.trts),
                   cov = Cov_d_beta) %>%
        # Include reverse comparisons by reversing the label; values are
        # equivalent.
        #
        bind_rows(data.frame(comparison =
                               paste(reference.group, names(Cov_d_beta),
                                     sep = sep.trts),
                             cov = Cov_d_beta))
      #
      dat_cov_d_beta <-
        merge(all_se.beta, dat_cov_d_beta, by = "comparison", all.x = TRUE) %>%
        mutate(cov = if_else(is.na(cov), cov.default, cov)) %>%
        select(-cov.default)
    }
    else if (assumption == "independent") {
      mat_beta <-
        outer(c(0, results$coef[!sel.d]), c(0, results$coef[!sel.d]), "-")
      rownames(mat_beta) <- colnames(mat_beta) <-
        c(reference.group, results$treat[!sel.d])
      #
      Cov_beta <- Cov[!sel.d, !sel.d, drop = FALSE]
      Cov_beta <- cbind(0, Cov_beta)
      Cov_beta <- rbind(0, Cov_beta)
      rownames(Cov_beta)[1] <- colnames(Cov_beta)[1] <- reference.group
      #
      mat_se.beta <-
        sqrt(outer(diag(Cov_beta), diag(Cov_beta), "+") - 2 * Cov_beta)
      rownames(mat_se.beta) <- colnames(mat_se.beta) <-
        c(reference.group, results$treat[!sel.d])
      #
      # Covariances
      #
      # Make the interaction vs treatment matrix a proper square matrix,
      # interactions in rows, treats in cols
      #
      Cov_d_beta <- Cov[!sel.d, sel.d, drop = FALSE]
      #
      # Drop interaction part from row names
      #
      rownames(Cov_d_beta) <- gsub(":.*|`", "", rownames(Cov_d_beta))
      #
      # Only keep treatments with interaction term
      #
      Cov_d_beta <- Cov_d_beta[, rownames(Cov_d_beta), drop = FALSE]
      #
      dat_cov_d_beta <- as.data.frame(Cov_d_beta)
      dat_cov_d_beta <- reshape(dat_cov_d_beta, 
                                idvar = "interaction", 
                                ids = rownames(dat_cov_d_beta), 
                                direction = "long", 
                                varying = list(names(dat_cov_d_beta)), 
                                v.names = "value", 
                                timevar = "treat", 
                                times = names(dat_cov_d_beta)) %>%
        mutate(type = if_else(treat == interaction, "ref", "interim"))
      #
      interim <- dat_cov_d_beta %>% select(-type) %>%
        rename(interim = value)
      
      ref <- dat_cov_d_beta %>% filter(type == "ref") %>%
        select(-treat, -type) %>%
        rename(ref = value)
      #
      comb1 <- merge(ref, interim, by = "interaction")
      comb1 <- merge(comb1, interim,
                     by.x = c("treat", "interaction"),
                     by.y = c("interaction", "treat"))
      comb1 <- merge(comb1, ref,
                     by.x = "treat", by.y = "interaction")
      #
      comb1 %<>%
        mutate(cov =
                 if_else(interaction == treat,
                         ref.x,
                         ref.x + ref.y - interim.x - interim.y))
      comb1 %<>%
        mutate(comparison =
                 if_else(interaction == treat,
                         paste(interaction, reference.group, sep = sep.trts),
                         paste(interaction, treat, sep = sep.trts))) %>%
        bind_rows(
          comb1 %<>%
            mutate(comparison =
                     if_else(interaction == treat,
                             paste(reference.group, interaction,
                                   sep = sep.trts),
                             paste(treat, interaction, sep = sep.trts)))) %>%
        select(comparison, cov)
      #
      dat_cov_d_beta <- unique(comb1)
    }
    #
    # Transform from matrix to data frame in long format
    #
    dat_beta <- dat_se.beta <- NULL
    #
    for (i in colnames(mat_beta)) {
      dat_beta.i <- as.data.frame(mat_beta)[i]
      names(dat_beta.i) <- "beta"
      dat_beta.i$comparison <- paste(rownames(dat_beta.i), i, sep = sep.trts)
      #
      dat_beta <- rbind(dat_beta, dat_beta.i)
      #
      dat_se.beta.i <- as.data.frame(mat_se.beta)[i]
      names(dat_se.beta.i) <- "se.beta"
      dat_se.beta.i$comparison <-
        paste(rownames(dat_se.beta.i), i, sep = sep.trts)
      dat_se.beta <- rbind(dat_se.beta, dat_se.beta.i)
    }
  } else if(consistency==FALSE){
    if(assumption=="common"){# common
      mat_beta<-matrix(nrow = length(x$results[x$results$type =="d","treat"])+1,
                       ncol=length(x$results[x$results$type =="d","treat"])+1,
                       data=x$results[x$results$type =="beta","coef"])
      diag(mat_beta)<-0
      dimnames(mat_beta)<-list(c(reference.group,x$results[x$results$type =="d","treat"]),
                               c(reference.group,x$results[x$results$type =="d","treat"]))
      
      mat_se.beta<-matrix(nrow = length(x$results[x$results$type =="d","treat"])+1,
                          ncol=length(x$results[x$results$type =="d","treat"])+1,
                          data=x$results[x$results$type =="beta","se"])
      diag(mat_se.beta)<-0
      dimnames(mat_se.beta)<-list(c(reference.group,x$results[x$results$type =="d","treat"]),
                                  c(reference.group,x$results[x$results$type =="d","treat"]))
      
      # covariances ###############
      temp_vcov<-Cov_d_beta <- Cov[!sel.d, sel.d]
      # rownames(temp_vcov)<-gsub(":.*|`","",rownames(temp_vcov))
      temp_vcov[reference.group]<-0
      mat_vcov<-as.numeric(temp_vcov)
      names(mat_vcov)<-names(temp_vcov)
      mat_vcov<-outer(mat_vcov,mat_vcov,"-")
      mat_vcov<-mat_vcov[order(rownames(mat_vcov)),order(colnames(mat_vcov))]
      
      mat_vcov<-as.data.frame(mat_vcov)
      
      
    }else if(assumption=="independent"){
      mat_beta<-x$results[x$results$type =="beta",c("coef","treat")]
      mat_se.beta<-x$results[x$results$type =="beta",c("se","treat")]
      # #reverse direction in case reverse direction occurs in original data
      mat_beta<-rbind.data.frame(mat_beta,
                                 data.frame("coef"=mat_beta$coef,# reversal depends on interpretation of ivals. Which is why users need the ival warning. Treatment direction is reversible
                                            "treat"=paste(gsub(".*_vs_","",mat_beta$treat), # without the vs this can get messy
                                                          gsub("_vs_.*","",mat_beta$treat),
                                                          sep="_vs_")))
      mat_beta$treat<-gsub(".comp_", "", mat_beta$treat) # since i added comp
      mat_beta$treat<-gsub("_vs_", ":", mat_beta$treat) #standardize with treatment effect matrices
      mat_se.beta<-rbind.data.frame(mat_se.beta)
      mat_se.beta<-rbind.data.frame(mat_se.beta,
                                    data.frame("se"=mat_se.beta$se,
                                               "treat"=paste(gsub(".*_vs_","",mat_se.beta$treat), # without the vs this can get messy
                                                             gsub("_vs_.*","",mat_se.beta$treat),
                                                             sep="_vs_")))
      mat_se.beta$treat<-gsub(".comp_", "", mat_se.beta$treat) # since i added comp
      mat_se.beta$treat<-gsub("_vs_", ":", mat_se.beta$treat) #standardize with treatment effect matrices
      names(mat_beta)<-c("beta","comparison")
      names(mat_se.beta)<-c("se.beta","comparison")
      
      # it is already a joinable list
      dat_beta<-mat_beta
      dat_se.beta<-mat_se.beta
      
      # covariances ###############
      temp_vcov<-Cov_beta <- Cov[!sel.d, sel.d]
      rownames(temp_vcov)<-gsub("\\.comp_|:.*|`","",rownames(temp_vcov))
      temp_vcov_interactions<-rownames(temp_vcov)
      temp_vcov<-as.data.frame(temp_vcov)
      temp_vcov[colnames(temp_vcov)[!colnames(temp_vcov)%in%temp_vcov_interactions],]<-0 # the dataframe appends rows whereas the matrix will not
      
      temp_vcov <- cbind(0, temp_vcov)
      temp_vcov <- rbind(0, temp_vcov)
      rownames(temp_vcov)[1] <- reference.group
      colnames(temp_vcov)[1] <- reference.group
      
      temp_vcov<-temp_vcov[order(rownames(temp_vcov)),order(colnames(temp_vcov))]
      mat_vcov<-temp_vcov
      
      mat_vcov<-as.data.frame(mat_vcov)
      mat_vcov<-reshape(mat_vcov,
                        idvar = "interaction",
                        ids=rownames(mat_vcov),
                        direction = "long", 
                        varying = list(names(mat_vcov)),
                        v.names = "value", 
                        timevar = "treat", 
                        times = names(mat_vcov))
      mat_vcov$int_treat1<-gsub(".*_vs_","",mat_vcov$interaction)
      mat_vcov$int_treat2<-gsub("_vs_.*","",mat_vcov$interaction)
      
      mat_vcov$type<-ifelse(mat_vcov$treat==mat_vcov$int_treat1&mat_vcov$treat==mat_vcov$int_treat2,"ref","interim")
      
      interim<-mat_vcov[mat_vcov$type=="interim"&
                          (mat_vcov$treat==mat_vcov$int_treat1|
                             mat_vcov$treat==mat_vcov$int_treat2),
                        !names(mat_vcov)%in%c("type")]
      names(interim)<-gsub("value","interim",names(interim))
      
      ref<-mat_vcov[mat_vcov$type=="ref",!names(mat_vcov)%in%c("treat","type","int_treat1", "int_treat2")]
      names(ref)<-gsub("value","ref",names(ref))
      
      comb1<-merge(interim,ref, by.x="int_treat2",by.y="interaction")
      comb1<-merge(comb1,ref, by.x="int_treat1",by.y="interaction")
      comb1<-merge(comb1,interim, by.x=c("interaction","int_treat1","int_treat2"),by.y=c("interaction","int_treat1","int_treat2"))
      comb1<-comb1[comb1$treat.x==comb1$int_treat2,]# only in the same order as the covariate
      
      comb1$cov<-comb1$ref.x+comb1$ref.y+comb1$interim.x-comb1$interim.y
      comb1$.comp<-paste(comb1$treat.x,comb1$treat.y,sep=":")
      combrev<-comb1
      combrev$.comp<-paste(combrev$treat.y,combrev$treat.x,sep=":")#reverse, does not change covariance
      comb1<-rbind(comb1,combrev)
      dat_cov_d_beta<-unique(comb1[,c(".comp","cov")])
      names(dat_cov_d_beta)<-c("comparison", "cov")
    }
    
    
    if (assumption == "common" | consistency) {
      dat_beta <- dat_se.beta<- dat_cov_d_beta<-NULL
    }
    #
    for (i in colnames(mat_beta)) {
      if (assumption == "common" | consistency) {
        dat_beta.i <- as.data.frame(mat_beta)[i]
        names(dat_beta.i) <- "beta"
        dat_beta.i$comparison <- paste(rownames(dat_beta.i), i, sep = sep.trts)
        #
        dat_beta <- rbind(dat_beta, dat_beta.i)
        #
        dat_se.beta.i <- as.data.frame(mat_se.beta)[i]
        names(dat_se.beta.i) <- "se.beta"
        dat_se.beta.i$comparison <-
          paste(rownames(dat_se.beta.i), i, sep = sep.trts)
        dat_se.beta <- rbind(dat_se.beta, dat_se.beta.i)
        #
        dat_cov_d_beta.i <- as.data.frame(mat_vcov)[i]
        names(dat_cov_d_beta.i) <- "cov"
        dat_cov_d_beta.i$comparison <-
          paste(rownames(dat_cov_d_beta.i), i, sep = sep.trts)
        dat_cov_d_beta <- rbind(dat_cov_d_beta, dat_cov_d_beta.i)
      }
    }
  }
  #
  # Combine elements for summary and all combinations
  #
  res <- merge(dat_d, dat_se.d, by = "comparison", all.x = TRUE)
  res <- merge(res, dat_beta, by = "comparison", all.x = TRUE)
  res <- merge(res, dat_se.beta, by = "comparison", all.x = TRUE)
  res <- merge(res, dat_cov_d_beta, by = "comparison", all.x = TRUE)
  #
  # Drop comparisons with themselves
  #
  split_comps <- strsplit(res$comparison, "[:]")
  res$treat1 <- sapply(split_comps, first)
  res$treat2 <- sapply(split_comps, second)
  res %<>% filter(treat1 != treat2) %>% filter(!duplicated(comparison))
  #
  res
}

mkIval <- function(x) {
  # Create directionality parameter as treatment order
  #
  ref <- sort(unique(c(x$treat1, x$treat2)))[1]
  #
  x$Ival1 <- ifelse(x$treat1 == ref, 0, 1)
  x$Ival2 <- ifelse(x$treat2 == ref, 0, 1)
  #
  return(x)
}

any_invalid_I <- function(x) {
  # Get rid of warning "no visible binding for global variable"
  #
  treat1 <- treat2 <- treat1_new <- treat2_new <- NULL
  #
  # Check Ival with ability to specify two custom variables
  #
  edges2 <- x %>%
    mutate(w = case_when(Ival_diff > 0 ~ -Ival_diff, TRUE ~ Ival_diff),
           treat1_new =
             case_when(Ival_diff %in% c(0, 1) ~ treat2, TRUE ~ treat1),
           treat2_new =
             case_when(Ival_diff %in% c(0, 1) ~ treat1, TRUE ~ treat2)
           ) %>%
    select(-treat1, -treat2) %>%
    rename(treat1 = treat1_new, treat2 = treat2_new)
  #
  treats <- unique(c(edges2$treat1, edges2$treat2))
  n <- length(treats)
  #
  dist <- rep(0, n)
  names(dist) <- treats
  #
  # For each treatment
  #
  for (i in seq_len(n)) {
    updated <- FALSE
    #
    # For each comparison
    #
    for (e in seq_len(nrow(edges2))) {
      # Distance for comparator treatment larger than the distance for
      # reference + the length
      if (dist[edges2$treat1[e]] > dist[edges2$treat2[e]] + edges2$w[e]) {
        dist[edges2$treat1[e]] <- dist[edges2$treat2[e]] + edges2$w[e]
        updated <- TRUE
      }
    }
    if (!updated)
      return(FALSE)
  }
  #
  return(TRUE)
}

check_Ival <- function(dat) {
  # Get rid of warning "no visible binding for global variable"
  #
  studlab <- treat <- treat1 <- treat2 <- Ival1 <- Ival2 <- NULL
  #
  # Check for invalid values
  #
  if (sum(!dat$Ival1 %in% c(-1, 0, 1)) > 0)
    stop("Error in directionality assignment. ",
         "Ival1 only accepts values of -1, 0, or 1.")
  #
  if (sum(!dat$Ival2 %in%c(-1, 0, 1)) > 0)
    stop("Error in directionality assignment. ",
         "Ival2 only accepts values of -1, 0, or 1.")
  #
  multiple_Ival <- dat %>%
    select(studlab, treat = treat1, Ival = Ival1) %>%
    bind_rows(dat %>% select(studlab, treat = treat2, Ival = Ival2)) %>%
    distinct() %>%
    count(studlab, treat)
  #
  longtrt_Ival <-
    unique(
      rbind(setNames(dat[c("studlab", "treat1", "Ival1")],
                     c("studlab", "treat", "Ival")),
            setNames(dat[c("studlab", "treat2", "Ival2")],
                     c("studlab", "treat", "Ival"))
            )
      )
  #
  ct_unique_Ival <-
    as.data.frame(table(longtrt_Ival$studlab, longtrt_Ival$treat))
  names(ct_unique_Ival) <- c("studlab", "treat", "n")
  #
  # Multiple treatment specific Ivalues within a study
  #
  if (sum(ct_unique_Ival$n > 1) > 0) 
    stop(paste("Error in directionality assignment. ",
               "Multiple directions defined for identical treatments in the ",
               "following studies: ",
               paste(unique(ct_unique_Ival[ct_unique_Ival$n > 1, "studlab"]),
                     collapse = ", ")))
  #
  # Inconsistent Ivalues between arms within a study
  #
  dat$Ival_diff <- dat$Ival1 - dat$Ival2
  Ival_consistency_list <-
    lapply(split(dat, dat$studlab), function(x) any_invalid_I(x))
  #
  if (length(Ival_consistency_list[Ival_consistency_list==TRUE]) > 0)
    stop(paste("Error in directionality assignment. ",
               "Inconsistent directions defined in the following studies: ",
               paste(names(Ival_consistency_list[Ival_consistency_list==TRUE]),
                     collapse = ", ")))
  #
  if (sum(!dat$Ival_diff %in% c(-1, 0, 1)) > 0)
    stop("Error in directionality assignment. ",
         "At least one study has a difference between directionality values ",
         "which is not in the accepted values of -1, 0, 1.")
  #
  NULL
}

#' Design-based decomposition of Cochran's Q in network meta-analysis
#' 
#' @description
#' This function performs a design-based decomposition of Cochran's Q
#' for assessing the homogeneity in the whole network, the homogeneity
#' within designs, and the homogeneity/consistency between designs. It
#' allows also an assessment of the consistency assumption after
#' detaching the effect of single designs.
#' 
#' @details
#' In the context of network meta-analysis and the assessment of the
#' homogeneity and consistency assumption, a generalised Cochran's Q
#' statistic for multivariate meta-analysis can be used as shown in
#' Krahn et al. (2013). This Q statistic can be decomposed in a sum
#' of within-design Q statistics and one between-designs Q statistic
#' that incorporates the concept of design inconsistency, see Higgins
#' et al. (2012).
#' 
#' For assessing the inconsistency in a random effects model, the
#' between-designs Q statistic can be calculated based on a full
#' design-by-treatment interaction random effects model (see Higgins
#' et al., 2012). This Q statistic will be automatically given in the
#' output (\eqn{\tau^2} estimated by the method of moments (see Jackson
#' et al., 2012). Alternatively, the square-root of the between-study
#' variance can be prespecified by argument \code{tau.preset} to
#' obtain a between-designs Q statistic (in \code{Q.inc.random}), its
#' design-specific contributions \code{Q.inc.design.random.preset}) as
#' well as residuals after detaching of single designs
#' (\code{residuals.inc.detach.random.preset}).
#' 
#' Since an inconsistent treatment effect of one design can
#' simultaneously inflate several residuals, Krahn et al. (2013)
#' suggest for locating the inconsistency in a network to fit a set of
#' extended models allowing for example for a deviating effect of each
#' study design in turn. The recalculated between-designs Q statistics
#' are given in list component \code{Q.inc.detach}. The change of the
#' inconsistency contribution of single designs can be investigated in
#' more detail by a net heat plot (see function
#' \link{netheat}). Designs where only one treatment is involved in
#' other designs of the network or where the removal of corresponding
#' studies would lead to a splitting of the network do not contribute
#' to the inconsistency assessment. These designs are not included in
#' \code{Q.inc.detach}.
#' 
#' @param x An object of class \code{netmeta}.
#' @param tau.preset An optional value for the square-root of the
#'   between-study variance \eqn{\tau^2} (see Details).
#' @param warn A logical indicating whether warnings should be
#'   printed.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' 
#' @return
#' Network meta-analysis with a single design: \code{NULL}. Otherwise,
#' a list containing the following components:
#' \item{Q.decomp}{Data frame with Q statistics (variable \code{Q})
#'   based on the common effects model to assess the
#'   homogeneity/consistency in the whole network, within designs, and
#'   between designs. Corresponding degrees of freedom (\code{df}) and
#'   p-values (\code{p.val}) are also given.}
#' \item{Q.het.design}{Data frame with design-specific decomposition
#'   of the within-designs Q statistic (\code{Q}) of the common effects
#'   model, corresponding degrees of freedom (\code{df}) and p-values
#'   (\code{p.val}) are given.}
#' \item{Q.inc.detach}{Data frame with between-designs Q statistics
#'   (\code{Q}) of the common effects model after detaching of single
#'   designs, corresponding degrees of freedom (\code{df}) and
#'   p-values (\code{p.val}) are given.}
#' \item{Q.inc.design}{A named vector with contributions of single
#'   designs to the between design Q statistic given in
#'   \code{Q.decomp}.}
#' \item{Q.inc.random}{Data frame with between-designs Q statistic
#'   (\code{Q}) based on a random effects model with square-root of
#'   between-study variance \code{tau.within} estimated embedded in a
#'   full design-by-treatment interaction model, corresponding degrees
#'   of freedom (\code{df}) and p-value (\code{p.val}).}
#' \item{Q.inc.random.preset}{Data frame with between-designs Q
#'   statistic (\code{Q}) based on a random effects model with
#'   prespecified square-root of between-study variance
#'   \code{tau.preset} in the case if argument \code{tau.preset} is
#'   not NULL, corresponding degrees of freedom (\code{df}) and
#'   p-value (\code{p.val}).}
#' \item{Q.inc.design.random.preset}{A named vector with contributions
#'   of single designs to the between design Q statistic based on a
#'   random effects model with prespecified square-root of
#'   between-study variance \code{tau.preset} in the case if argument
#'   \code{tau.preset} is given.}
#' \item{residuals.inc.detach}{Matrix with residuals,
#'   i.e. design-specific direct estimates minus the corresponding
#'   network estimates after detaching the design of the column.}
#' \item{residuals.inc.detach.random.preset}{Matrix with residuals
#'   analogous to \code{residuals.inc.detach} but based on a random
#'   effects model with prespecified square-root of between-study
#'   variance \code{tau.preset} in the case if argument
#'   \code{tau.preset} is not NULL.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package netmeta used to create object.}
#' 
#' @author Ulrike Krahn \email{ulrike.krahn@@bayer.com}, Jochem König
#'   \email{koenigjo@@uni-mainz.de}
#' 
#' @seealso \link{netmeta}, \link{netheat}
#' 
#' @references
#' Higgins JPT, Jackson D, Barrett JK, Lu G, Ades AE, White IR (2012):
#' Consistency and inconsistency in network meta-analysis: concepts
#' and models for multi-arm studies.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 98--110
#' 
#' Krahn U, Binder H, König J (2013):
#' A graphical tool for locating inconsistency in network meta-analyses.
#' \emph{BMC Medical Research Methodology},
#' \bold{13}, 35
#' 
#' Jackson D, White IR and Riley RD (2012):
#' Quantifying the impact of between-study heterogeneity in
#' multivariate meta-analyses.
#' \emph{Statistics in Medicine},
#' \bold{31}, 3805--20
#' 
#' @examples
#' \donttest{
#' # Only consider first five studies (to reduce runtime of example)
#' #
#' studies <- unique(Senn2013$study)
#' Senn2013.5 <- subset(Senn2013, study %in% studies[1:5])
#' 
#' # Transform data from long arm-based to contrast-based format
#' #
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013.5,
#'   varnames = c("MD", "seMD"))
#' 
#' # Conduct network meta-analysis
#' #
#' nma <- netmeta(pw)
#' 
#' # Decomposition of Cochran's Q
#' #
#' decomp.design(nma)
#' }
#' 
#' @export decomp.design

decomp.design <- function(x, tau.preset = x$tau.preset, warn = gs("warn"),
                          nchar.trts = x$nchar.trts) {
  
  
  chkclass(x, "netmeta")
  chksuitable(x, "Decomposition of designs", classes = gs(".other_nma"))
  ##
  if (x$n == 2) {
    warning("No decomposition possible for network meta-analysis ",
            "with only two treatments.",
            call. = FALSE)
    return(NULL)
  }
  ##
  if (!is.null(tau.preset))
    chknumeric(tau.preset, min = 0, length = 1)
  chklogical(warn)
  chknumeric(nchar.trts, min = 1, length = 1)
  ##
  if (inherits(x, "netmetabin")) {
    warning("Decomposition of designs not implemented for ",
            "objects created with netmetabin().",
            call. = FALSE)
    ##
    Q.decomp <- data.frame(Q = x$Q.inconsistency,
                           df = x$df.Q.inconsistency,
                           pval = x$pval.Q.inconsistency)
    ##
    Q.decomp$pval[Q.decomp$df == 0] <- NA
    ##
    rownames(Q.decomp) <- "Between designs"
    res <- list(Q.decomp = Q.decomp,
                call = match.call(),
                version = packageDescription("netmeta")$Version
                )
    ##
    attr(res, "netmetabin") <- TRUE
    class(res) <- "decomp.design"
    ##
    return(res)
  }
  
  
  tau.within <- tau.within(x)
  ##
  decomp.random <- decomp.tau(x, tau.preset = tau.within, warn = warn)
  ##
  Q.inc.random <- decomp.random$Q.decomp["Between designs", ]
  Q.inc.design.random <- decomp.random$Q.inc.design
  residuals.detach.random <- decomp.random$residuals.detach
  
  
  if (length(tau.preset) == 1) {
    decomp.random.preset <-
      decomp.tau(x, tau.preset = tau.preset, warn = warn)
    Q.inc.random.preset <-
      decomp.random.preset$Q.decomp["Between designs",]
    Q.inc.design.random.preset <- decomp.random.preset$Q.inc.design
    residuals.inc.detach.random.preset <-
      decomp.random.preset$residuals.inc.detach
  }
  else {
    tau.preset <- NULL
    Q.inc.random.preset <- NULL
    Q.inc.design.random.preset <- NULL
    residuals.inc.detach.random.preset <- NULL
  }
  
  
  dct <- decomp.tau(x, warn = warn)
  
  
  res <- list(
    Q.decomp = dct$Q.decomp,
    Q.het.design = dct$Q.het.design,
    Q.inc.detach = dct$Q.inc.detach,
    Q.inc.design = dct$Q.inc.design,
    ##
    Q.inc.random = data.frame(Q.inc.random, tau.within),
    Q.inc.random.preset = data.frame(Q.inc.random.preset, tau.preset),
    Q.inc.design.random.preset = Q.inc.design.random.preset,
    ##
    residuals.inc.detach = dct$residuals.inc.detach,
    residuals.inc.detach.random.preset = residuals.inc.detach.random.preset,
    ##
    tau.preset = tau.preset,
    nchar.trts = nchar.trts,
    ##
    x = x,
    ##
    call = match.call(),
    version = packageDescription("netmeta")$Version
    )
  ##
  class(res) <- "decomp.design"
  ##
  res
}

#' Forest plot for additive network meta-analysis
#' 
#' @description
#' Draws a forest plot in the active graphics window (using grid
#' graphics system).
#'
#' @aliases forest.netcomb plot.netcomb
#' 
#' @param x An object of class \code{netcomb}.
#' @param reference.group Reference treatment(s).
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param equal.size A logical indicating whether all squares should
#'   be of equal size. Otherwise, the square size is proportional to
#'   the precision of estimates.
#' @param leftcols A character vector specifying (additional) columns
#'   to be plotted on the left side of the forest plot or a logical
#'   value (see \code{\link[meta]{forest.meta}} help page for details).
#' @param leftlabs A character vector specifying labels for
#'   (additional) columns on left side of the forest plot (see
#'   \code{\link[meta]{forest.meta}} help page for details).
#' @param rightcols A character vector specifying (additional) columns
#'   to be plotted on the right side of the forest plot or a logical
#'   value (see \code{\link[meta]{forest.meta}} help page for details).
#' @param rightlabs A character vector specifying labels for
#'   (additional) columns on right side of the forest plot (see
#'   \code{\link[meta]{forest.meta}} help page for details).
#' @param digits Minimal number of significant digits for treatment
#'   effects and confidence intervals, see \code{print.default}.
#' @param smlab A label printed at top of figure. By default, text
#'   indicating either common or random effects model is printed.
#' @param sortvar An optional vector used to sort the individual
#'   studies (must be of same length as the total number of
#'   treatments).
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param lab.NA A character string to label missing values.
#' @param add.data An optional data frame with additional columns to
#'   print in forest plot (see Details).
#' @param addrows.below.overall A numeric value indicating how many
#'   empty rows are printed between meta-analysis results and
#'   heterogeneity statistics.
#' @param drop.reference.group A logical indicating whether the
#'   reference group should be printed in the forest plot.
#' @param \dots Additional arguments for \code{\link[meta]{forest.meta}}
#'   function.
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window.
#' 
#' Argument \code{sortvar} can be either a numeric or character vector
#' with length of number of treatments. If \code{sortvar} is numeric
#' the \code{\link[base]{order}} function is utilised internally to
#' determine the order of values. If \code{sortvar} is character it
#' must be a permutation of the treatment names. It is also possible
#' to to sort by treatment comparisons (\code{sortvar = TE}, etc.),
#' standard error (\code{sortvar = seTE}), and number of studies with
#' direct treatment comparisons (\code{sortvar = k}).
#' 
#' Argument \code{add.data} can be used to add additional columns to
#' the forest plot. This argument must be a data frame with the same
#' row names as the treatment effects matrices in R object \code{x},
#' i.e., \code{x$TE.common} or \code{x$TE.random}.
#' 
#' For more information see help page of \code{\link[meta]{forest.meta}}
#' function.
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{discomb}},
#'   \code{\link[meta]{forest.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netcomb)
#' 
#' @method forest netcomb
#' @export

forest.netcomb <- function(x,
                           pooled = ifelse(x$random, "random", "common"),
                           reference.group = x$reference.group,
                           baseline.reference = x$baseline.reference,
                           equal.size = gs("equal.size"),
                           leftcols = "studlab",
                           leftlabs = "Treatment",
                           rightcols = c("effect", "ci"),
                           rightlabs = NULL,
                           digits = gs("digits.forest"),
                           smlab = NULL,
                           sortvar = x$seq,
                           overall.hetstat = gs("overall.hetstat"),
                           backtransf = x$backtransf,
                           lab.NA = gs("lab.NA"),
                           add.data,
                           addrows.below.overall =
                             if (x$overall.hetstat) 2 else
                               gs("addrows.below.overall"),
                           drop.reference.group = gs("drop.reference.group"),
                           ...) {
  
  
  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netcomb")
  x <- updateversion(x)
  ##
  is.discomb <- inherits(x, "discomb")
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  ##
  chknumeric(digits, min = 0, length = 1)
  ##
  chklogical(baseline.reference)
  chklogical(drop.reference.group)
  ##
  overall.hetstat <- replaceNULL(overall.hetstat, FALSE)
  #
  chklogical(backtransf)
  chkchar(lab.NA)
  
  
  ##
  ##
  ## (2) Extract results for common and random effects model
  ##     and calculate P-scores
  ##
  ##
  labels <- colnames(x$TE.common)
  ##
  if (reference.group == "") {
    warning("First treatment used as reference as ",
            "argument 'reference.group' is unspecified.")
    reference.group <- labels[1]
  }
  else
    reference.group <- setref(reference.group, labels)
  ##
  if (pooled == "common") {
    TE   <- x$TE.common
    seTE <- x$seTE.common
    ##
    text.common <- "(Common Effects Model)"
    ##
    if (is.null(smlab))
      if (baseline.reference)
        smlab <- paste0("Comparison: other vs '", reference.group, "'\n",
                        text.common)
      else
        smlab <- paste0("Comparison: '", reference.group, "' vs other\n",
                       text.common)
  }
  ##
  if (pooled == "random") {
    TE   <- x$TE.random
    seTE <- x$seTE.random
    if (is.null(smlab))
      if (baseline.reference)
        smlab <- paste0("Comparison: other vs '", reference.group,
                        "'\n(Random Effects Model)")
      else
        smlab <- paste0("Comparison: '", reference.group,
                        "' vs other\n(Random Effects Model)")
  }
  
  
  ##
  ##
  ## (3) Extract comparisons with reference group
  ##
  ##
  if (baseline.reference) {
    dat <- data.frame(TE = TE[, colnames(TE) == reference.group],
                      seTE = seTE[, colnames(seTE) == reference.group],
                      trts = colnames(TE),
                      k = NA,
                      row.names = colnames(TE),
                      as.is = TRUE)
    if (!is.discomb)
      dat$k <- x$A.matrix[, colnames(TE) == reference.group]
  }
  else {
    dat <- data.frame(TE = TE[rownames(TE) == reference.group, ],
                      seTE = seTE[rownames(seTE) == reference.group, ],
                      trts = rownames(TE),
                      k = NA,
                      row.names = colnames(TE),
                      as.is = TRUE)
    if (!is.discomb)
      dat$k <- x$A.matrix[rownames(TE) == reference.group, ]
  }
  ##
  rm(TE)
  rm(seTE)
  ##
  if (!missing(add.data)) {
    if (!is.data.frame(add.data))
      stop("Argument 'add.data' must be a data frame.",
           call. = FALSE)
    if (nrow(add.data) != length(labels))
      stop("Dataset 'add.data' must have ", nrow(dat),
           " rows (corresponding to number of treatments)",
           call. = FALSE)
    if (any(rownames(add.data) != labels))
      stop("Dataset 'add.data' must have the following row names:\n",
           paste(paste0("'", labels, "'"), collapse = " - "),
           call. = FALSE)
    ##
    dat <- cbind(dat, add.data)
  }
  
  
  ##
  ##
  ## (4) Sort dataset according to argument sortvar
  ##
  ##
  sortvar.c <- deparse(substitute(sortvar))
  sortvar.c <- gsub("\"", "", sortvar.c)
  ##
  idx5 <- charmatch(tolower(sortvar.c), "te", nomatch = NA)
  sel5 <- !is.na(idx5) & idx5 == 1
  if (any(sel5))
    sortvar <- dat$TE
  ##
  idx6 <- charmatch(tolower(sortvar.c), "-te", nomatch = NA)
  sel6 <- !is.na(idx6) & idx6 == 1
  if (any(sel6))
    sortvar <- -dat$TE
  ##
  idx7 <- charmatch(tolower(sortvar.c), "sete", nomatch = NA)
  sel7 <- !is.na(idx7) & idx7 == 1
  if (any(sel7))
    sortvar <- dat$seTE
  ##
  idx8 <- charmatch(tolower(sortvar.c), "-sete", nomatch = NA)
  sel8 <- !is.na(idx8) & idx8 == 1
  if (any(sel8))
    sortvar <- -dat$seTE
  ##
  if (!is.discomb) {
    idx9 <- charmatch(tolower(sortvar.c), "k", nomatch = NA)
    sel9 <- !is.na(idx9) & idx9 == 1
    if (any(sel9))
      sortvar <- dat$k
    ##
    idx10 <- charmatch(tolower(sortvar.c), "-k", nomatch = NA)
    sel10 <- !is.na(idx10) & idx10 == 1
    if (any(sel10))
      sortvar <- -dat$k
  }
  ##  
  if (!is.null(sortvar)) {
    if (is.character(sortvar))
      sort <- setseq(sortvar, labels)
    else
      sort <- order(sortvar)
    ##
    dat <- dat[sort, ]
  }
  
  
  ##
  ##
  ## (5) Generate forest plot
  ##
  ##
  if (drop.reference.group)
    dat <- subset(dat, trts != reference.group)
  ##
  trts <- dat$trts
  m1 <-
    suppressWarnings(metagen(TE, seTE, data = dat,
                             sm = x$sm,
                             studlab = trts, backtransf = backtransf,
                             method.tau = "DL", method.tau.ci = "",
                             warn = FALSE))
  ##
  forest(m1,
         digits = digits,
         #
         overall = FALSE, common = FALSE, random = FALSE,
         overall.hetstat = overall.hetstat,
         test.subgroup = FALSE,
         #
         leftcols = leftcols,
         leftlabs = leftlabs,
         rightcols = rightcols,
         rightlabs = rightlabs,
         #
         smlab = smlab, lab.NA = lab.NA,
         #
         weight.study = if (equal.size) "same" else pooled,
         #
         addrows.below.overall = addrows.below.overall,
         #
         ...)
  
  
  invisible(NULL)
}


#' @rdname forest.netcomb
#' @method plot netcomb
#' @export

plot.netcomb <- function(x, ...)
  forest(x, ...)

ranksampling <- function(x, nsim,
                         pooled = "random", small.values = "desirable",
                         keep.samples = FALSE) {
  
  chknumeric(nsim, min = 1, length = 1)
  pooled <- setchar(pooled, c("common", "random"))
  small.values <- setsv(small.values)
  chklogical(keep.samples)
  
  
  #
  # Generate samples
  #
  
  samples <- samples_netmeta(x, nsim, pooled)
  
  
  #
  # Calculate rankings
  #
  
  if (small.values == "desirable")
    rankings <- rankings(samples$samples)
  else
    rankings <- rankings(-samples$samples)
  
  
  #
  # Return results
  #
  
  res <- list(sucras = rankings$sucras,
              rankogram = rankings$rankogram,
              cumrank = rankings$cumrank,
              #
              meanranks = rankings$meanranks,
              medianranks = rankings$medianranks,
              #
              nsim = nsim,
              pooled = pooled,
              small.values = small.values,
              keep.samples = keep.samples,
              #
              compMatrix = samples$compMatrix)
  #
  if (keep.samples)
    res$samples <- samples$samples
  res
}

# Auxiliary functions
#
# Package: meta
# Author: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
# License: GPL (>= 2)
#

bylevs <- function(x) {
  if (is.factor(x))
    res <- levels(factor(x))
  else
    res <- unique(x)
  res
}

byvarname <- function(argname, matchcall) {
  #
  # Determine name of subgroup variable
  #
  res <- as.character(matchcall[[match(argname, names(matchcall))]])
  #
  if (length(res) > 1 & res[1] == "$")
    res <- res[length(res)]
  #
  if (length(res) == 0 || length(res) > 1)
    res <- "subgroup"
  #
  res
}

catch <- function(argname, matchcall, data, encl) {
  #
  # Catch value for argument
  #
  eval(matchcall[[match(argname, names(matchcall))]], data, enclos = encl)
}

int2num <- function(x) {
  #
  # Convert integer to numeric
  #
  if (is.integer(x))
    res <- as.numeric(x)
  else
    res <- x
  #
  res
}

npn <- function(x) {
  #
  # Check for non-positive values in vector
  #
  selNA <- is.na(x)
  res <- selNA
  if (sum(!selNA) > 0)
    x[!selNA] <- x[!selNA] <= 0
  #
  res
}

replaceNULL <- function(x, replace = NA) {
  if (is.null(x))
    return(replace)
  x
}

replaceNA <- function(x, replace = NA) {
  if (is.null(x))
    return(x)
  else
    x[is.na(x)] <- replace
  x
}

warnarg <- function(x, y, fun, cl, otherarg) {
  if (x %in% y)
    if (!missing(cl))
      warning("Argument '", x, "' has been removed from R function ", fun,
              ".\nThis argument can be used in R function ", cl, ".",
              call. = FALSE)
    else if (!missing(otherarg))
      warning("Argument '", x, "' has been replaced by argument '", otherarg,
              "' in R function ", fun, ".\nSee help page of R function ",
              fun, " for information on the use of the new argument.",
              call. = FALSE)
  #
  invisible(NULL)
}

catchvar <- function(varname, x, mf) {
  res <- NULL
  error <-
    try(res <- eval(mf[[match(varname, names(mf))]],
                    x,
                    enclos = sys.frame(sys.parent())),
        silent = TRUE)
  #
  if (inherits(error, "try-error")) {
    res <- eval(mf[[match(varname, names(mf))]],
                x$data, enclos = NULL)
  }
  #
  res
}

augment <- function(x, len, fun) {
  if (length(x) > 1)
    chklength(x, len, fun)
  else
    x <- rep(x, len)
  x
}

stoponly <- function(arg, val, func)
  stop("Argument ", arg, " =\"", val, "\"",
       " only defined for meta-analysis conducted with ",
       func, ".",
       call. = FALSE)

deprecated <- function(newvar, newmiss, args, old, warn = TRUE) {
  #
  new <- deparse(substitute(newvar))
  #
  if (length(args) == 0)
    return(newvar)
  #
  if (is.list(args[[1]]))
    args <- args[[1]]
  #
  additional.arguments <- names(args)
  #
  if (!is.na(charmatch(old, additional.arguments)))
    if (!newmiss) {
      if (warn)
        warning("Deprecated argument '", old, "' ignored as ",
                "'", new, "' is also provided.",
                call. = FALSE)
      return(newvar)
    }
    else {
      if (warn)
        warning("Use argument '", new, "' instead of '",
                old, "' (deprecated).",
                call. = FALSE)
      return(args[[charmatch(old, additional.arguments)]])
    }
  else
    return(newvar)
}

deprecated2 <- function(newvar, newmiss, oldvar, oldmiss, warn = TRUE) {
  #
  new <- deparse(substitute(newvar))
  old <- deparse(substitute(oldvar))
  #
  if (newmiss & oldmiss)
    return(newvar)
  else if (!newmiss & oldmiss)
    return(newvar)
  else if (!newmiss & !oldmiss) {
    if (warn)
      warning("Deprecated argument '", old, "' ignored as ",
              "'", new, "' is also provided.",
              call. = FALSE)
    return(newvar)
  }
  else if (newmiss & !oldmiss) {
    if (warn)
      warning("Use argument '", new, "' instead of '",
              old, "' (deprecated).",
              call. = FALSE)
    return(oldvar)
  }
}

runNN <- function(func, args, warn = TRUE) {
  args <- args[!sapply(args, is.null)]
  if (warn)
    do.call(func, args)
  else
    suppressWarnings(do.call(func, args))
}

# Estimate the heterogeneity parameter phi using the
# modified version of Pearson's statistic.
#
phi <- function(x) {
  # Extract number of trials
  n.trials <- x$prior.weights
  #
  if (identical(unique(n.trials), 1))
    stop("The number of successes must be summarized for valid computation of ",
         "c-hat.")
  
  # Pearson chi-square
  chisq <- sum(residuals(x, type = "pearson")^2)
  
  # Extract raw residuals
  raw.res <- residuals(x, type = "response")
  
  # Extract fitted values
  fit.vals <- fitted(x)
  
  # Estimate s.bar
  s.bar <- mean((1 - 2 * fit.vals) / ((n.trials * fit.vals) * (1 - fit.vals)))
  
  # Calculate estimate based on Fletcher estimator
  phi <- (chisq / x$df.residual) / (1 + s.bar)
  
  # Set phi = 1 if phi < 1 to remain consistent with common effect model
  phi <- max(phi, 1)
  
  phi
}

# Gets as arguments
# - 'trts' = treatment names,
# - 'TE.basic' = basic treatment effect estimates,
# - 'vcov.basic' = variance-covariance matrix of the estimates.
#
# Returns all NMA estimates (basic + functional) and their standard errors.
#
basic2all <- function(trts, TE.basic, vcov.basic) {
  
  n.trts <- length(trts)
  
  # Get rid of warning 'no visible binding for global variable'
  treat1 <- treat2 <- NULL
  
  # Identify all comparisons
  #
  comps <- expand.grid(treat1 = seq_len(n.trts), treat2 = seq_len(n.trts))
  comps <- subset(comps, treat2 < treat1)
  #
  comps.basic <- subset(comps, treat2 == 1)
  comps.other <- subset(comps, treat2 > 1)
  #
  n.comps.basic <- nrow(comps.basic)
  n.comps.other <- nrow(comps.other)
  
  # Construct X matrix, i.e., the treatment comparison matrix
  #
  X.basic <- matrix(0, nrow = n.comps.other, ncol = n.comps.basic)
  #
  for (i in seq_len(n.comps.other)) {
    for (j in seq_len(n.comps.basic)) {
      if (comps.other$treat1[i] == comps.basic$treat1[j])
        X.basic[i, j] <- 1
      #
      if (comps.other$treat2[i] == comps.basic$treat1[j])
        X.basic[i, j] <- -1
    }
  }
  
  dat.basic <-
    data.frame(comps.basic,
               TE = TE.basic,
               seTE = sqrt(diag(vcov.basic)))
  #
  dat.other <-
    data.frame(comps.other,
               TE = X.basic %*% TE.basic,
               seTE = sqrt(diag(X.basic %*% vcov.basic %*% t(X.basic))))
  #
  dat.comps <- rbind(dat.basic, dat.other)
  
  TEs <- matrix(NA, ncol = n.trts, nrow = n.trts)
  seTEs <- matrix(NA, ncol = n.trts, nrow = n.trts)
  #
  for (i in seq_len(nrow(dat.comps))) {
    TEs[dat.comps$treat1[i], dat.comps$treat2[i]] <-  dat.comps$TE[i]
    TEs[dat.comps$treat2[i], dat.comps$treat1[i]] <- -dat.comps$TE[i]
    #
    seTEs[dat.comps$treat1[i], dat.comps$treat2[i]] <- dat.comps$seTE[i]
    seTEs[dat.comps$treat2[i], dat.comps$treat1[i]] <- dat.comps$seTE[i]
  }
  
  diag(TEs) <- diag(seTEs) <- 0
  #
  rownames(TEs) <- rownames(seTEs) <- colnames(TEs) <- colnames(seTEs) <- trts
  
  res <- list(TEs = TEs, seTEs = seTEs)
  #
  res
}

setNA <- function(x) {
  x[!is.na(x)] <- NA
  x  
}

setNA_subset <- function(x, subset) {
  x[subset] <- NA
  x
}

setNA_vars <- function(x, varnames) {
  x[names(x) %in% varnames] <- NA
  x
}

setNA_nma <- function(x) {
   x$seTE.adj <- setNA(x$seTE.adj)
   x$seTE.adj.common <- setNA(x$seTE.adj.common)
   x$seTE.adj.random <- setNA(x$seTE.adj.random)
   x$correlated <- setNA(x$correlated)
   #
   x$TE.nma.common <- setNA(x$TE.nma.common)
   x$seTE.nma.common <- setNA(x$seTE.nma.common)
   x$lower.nma.common <- setNA(x$lower.nma.common)
   x$upper.nma.common <- setNA(x$upper.nma.common)
   x$statistic.nma.common <- setNA(x$statistic.nma.common)
   x$pval.nma.common <- setNA(x$pval.nma.common)
   #
   x$leverage.common <- setNA(x$leverage.common)
   x$w.common <- setNA(x$w.common)
   x$Q.common <- setNA(x$Q.common)
   #
   x$statistic.common <- setNA(x$statistic.common)
   x$pval.common <- setNA(x$pval.common)
   #
   x$TE.nma.random <- setNA(x$TE.nma.random)
   x$seTE.nma.random <- setNA(x$seTE.nma.random)
   x$lower.nma.random <- setNA(x$lower.nma.random)
   x$upper.nma.random <- setNA(x$upper.nma.random)
   x$statistic.nma.random <- setNA(x$statistic.nma.random)
   x$pval.nma.random <- setNA(x$pval.nma.random)
   #
   x$w.random <- setNA(x$w.random)
   #
   x$statistic.random <- setNA(x$statistic.random)
   x$pval.random <- setNA(x$pval.random)
   #
   x$seTE.predict <- setNA(x$seTE.predict)
   x$lower.predict <- setNA(x$lower.predict)
   x$upper.predict <- setNA(x$upper.predict)
   #
   x$prop.direct.common <- setNA(x$prop.direct.common)
   x$prop.direct.random <- setNA(x$prop.direct.random)
   #
   x$TE.indirect.common <- setNA(x$TE.indirect.common)
   x$seTE.indirect.common <- setNA(x$seTE.indirect.common)
   x$lower.indirect.common <- setNA(x$lower.indirect.common)
   x$upper.indirect.common <- setNA(x$upper.indirect.common)
   x$statistic.indirect.common <- setNA(x$statistic.indirect.common)
   x$pval.indirect.common <- setNA(x$pval.indirect.common)
   #
   x$TE.indirect.random <- setNA(x$TE.indirect.random)
   x$seTE.indirect.random <- setNA(x$seTE.indirect.random)
   x$lower.indirect.random <- setNA(x$lower.indirect.random)
   x$upper.indirect.random <- setNA(x$upper.indirect.random)
   x$statistic.indirect.random <- setNA(x$statistic.indirect.random)
   x$pval.indirect.random <- setNA(x$pval.indirect.random)
   #
   x$Q <- setNA(x$Q)
   x$df.Q <- setNA(x$df.Q)
   x$pval.Q <- setNA(x$pval.Q)
   x$I2 <- setNA(x$I2)
   x$lower.I2 <- setNA(x$lower.I2)
   x$upper.I2 <- setNA(x$upper.I2)
   x$tau <- setNA(x$tau)
   x$tau2 <- setNA(x$tau2)
   #
   x$Q.heterogeneity <- setNA(x$Q.heterogeneity)
   x$df.Q.heterogeneity <- setNA(x$df.Q.heterogeneity)
   x$pval.Q.heterogeneity <- setNA(x$pval.Q.heterogeneity)
   #
   x$Q.inconsistency <- setNA(x$Q.inconsistency)
   x$df.Q.inconsistency <- setNA(x$df.Q.inconsistency)
   x$pval.Q.inconsistency <- setNA(x$pval.Q.inconsistency)
   #
   x$Q.decomp <- setNA(x$Q.decomp)
   #
   x$W.matrix.common <- setNA(x$W.matrix.common)
   x$W.matrix.random <- setNA(x$W.matrix.random)
   #
   x$L.matrix.common <- setNA(x$L.matrix.common)
   x$Lplus.matrix.common <- setNA(x$Lplus.matrix.common)
   x$L.matrix.random <- setNA(x$L.matrix.random)
   x$Lplus.matrix.random <- setNA(x$Lplus.matrix.random)
   #
   x$Q.matrix <- setNA(x$Q.matrix)
   #
   x$G.matrix <- setNA(x$G.matrix)
   #
   x$H.matrix.common <- setNA(x$H.matrix.common)
   x$H.matrix.random <- setNA(x$H.matrix.random)
   #
   x$P.common <- setNA(x$P.common)
   x$P.random <- setNA(x$P.random)
   #
   x$Cov.common <- setNA(x$Cov.common)
   x$Cov.random <- setNA(x$Cov.random)
   #
   x
}



if (FALSE) {
  TE.direct.common = TE.direct
  seTE.direct.common = seTE.direct
  lower.direct.common = lower.direct
  upper.direct.common = upper.direct
  statistic.direct.common = statistic.direct
  pval.direct.common = pval.direct
  #
  TE.direct.random = res.r$TE.direct
  seTE.direct.random = res.r$seTE.direct
  lower.direct.random = res.r$lower.direct
  upper.direct.random = res.r$upper.direct
  statistic.direct.random = res.r$statistic.direct
  pval.direct.random = res.r$pval.direct
  #
  Q.direct = res.r$Q.direct
  tau.direct = sqrt(res.r$tau2.direct)
  tau2.direct = res.r$tau2.direct
  I2.direct = res.r$I2.direct
}

#' Scatter plot or biplot showing partially order of treatment ranks
#' 
#' @description
#' This function generates a scatter plot or biplot of P-scores with
#' an overlay describing partial order of treatment ranks.
#' 
#' @param x An object of class \code{netmeta} (mandatory).
#' @param plottype A character string indicating whether a scatter
#'   plot or biplot should be produced, either \code{"scatter"} or
#'   \code{"biplot"}. Can be abbreviated.
#' @param pooled A character string indicating whether scatter plot
#'   should be drawn for common (\code{"common"}) or random effects
#'   model (\code{"random"}). Can be abbreviated.
#' @param dim A character string indicating whether a 2- or
#'   3-dimensional plot should be produced, either \code{"2d"} or
#'   \code{"3d"}. Can be abbreviated.
#' @param sel.x A numeric specifying number of outcome to use for the
#'   x-axis in a scatterplot (argument is not considered for a
#'   biplot).
#' @param sel.y A numeric specifying number of outcome to use for the
#'   y-axis in a scatterplot (argument is not considered for a
#'   biplot).
#' @param sel.z A numeric specifying number of outcome to use for the
#'   z-axis in a scatterplot (argument is not considered for a
#'   biplot).
#' @param cex The magnification to be used for treatment labels and
#'   points.
#' @param col Colour(s) of treatment labels and points.
#' @param cex.text The magnification to be used for treatment labels.
#' @param col.text Colour(s) of treatment labels.
#' @param adj.x Value(s) in [0, 1] to specify adjustment of treatment
#'   labels on x-axis (only considered in 2-D plots); see
#'   \code{\link[graphics]{text}}.
#' @param adj.y Value(s) in [0, 1] to specify adjustment of treatment
#'   labels on y-axis (only considered in 2-D plots); see
#'   \code{\link[graphics]{text}}.
#' @param offset.x Offset(s) of treatment labels on x-axis (only
#'   considered in 2-D plots).
#' @param offset.y Offset(s) of treatment labels on y-axis (only
#'   considered in 2-D plots).
#' @param pch Plot symbol(s) for points; no points printed if equal to
#'   \code{NULL}.
#' @param cex.points Magnification(s) to be used for points.
#' @param col.points Colour(s) of points.
#' @param col.lines Line colour.
#' @param lty.lines Line type.
#' @param lwd.lines Line width.
#' @param arrows A logical indicating whether arrows should be printed
#'   (only considered in 2-D plots).
#' @param length Length of arrows; see \code{\link[graphics]{arrows}}.
#' @param grid A logical indicating whether grid lines should be added
#'   to plot.
#' @param col.grid Colour of grid lines.
#' @param lty.grid Line type of grid lines.
#' @param lwd.grid Line width of grid lines.
#' @param \dots Additional graphical arguments.
#' 
#' @details
#' By default (arguments \code{plottype = "scatter"} and \code{dim =
#' "2d"}), a scatter plot is created showing P-scores (see
#' \code{\link{netrank}}) for the first two outcomes considered in the
#' generation of a partially ordered set of treatment ranks (using
#' \code{\link{netposet}}). In addition to the P-scores, the partially
#' order of treatment ranks is shown as lines connecting treatments
#' which is analogous to a Hasse diagram. If argument \code{dim =
#' "3d"}), a 3-D scatter plot is generated showing P-scores for the
#' first three outcomes.
#' 
#' To overcome the restriction of two or three dimension, a biplot
#' (Gabriel, 1971) can be generated using argument \code{plottype =
#' "biplot"}. This is essentially a scatter plot using the first two
#' (\code{dim = "2d"}) or three (\code{dim = "3d"}) components in a
#' principal components analysis (using
#' \code{\link[stats]{prcomp}}). Note, if only two / three outcomes
#' are considered in a \code{netposet} object, a 2-D / 3-D scatter
#' plot is generated instead of a biplot as a principal component
#' analysis is superfluous in such a situation.
#' 
#' Arguments \code{sel.x} and \code{sel.y} can be used to select
#' different outcomes to show on x- and y-axis in a 2-D scatter plot;
#' argument \code{sel.z} can be used accordingly in a 3-D scatter
#' plot. These arguments are ignored for a biplot.
#' 
#' Note, in order to generate 3-D plots (argument \code{dim = "3d"}),
#' R package \bold{rgl} is necessary. Note, under macOS the X.Org X
#' Window System must be available (see
#' \url{https://www.xquartz.org}).
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netposet}}, \code{\link{hasse}},
#'   \code{\link{netrank}}, \code{\link{netmeta}},
#'   \code{\link[metabook]{Linde2015}}
#'   
#' @references
#' Carlsen L, Bruggemann R (2014):
#' Partial order methodology: a valuable tool in chemometrics.
#' \emph{Journal of Chemometrics},
#' \bold{28}, 226--34
#' 
#' Gabriel KR (1971):
#' The biplot graphic display of matrices with application to
#' principal component analysis.
#' \emph{Biometrika},
#' \bold{58}, 453--67
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netposet)
#' 
#' @method plot netposet
#' @export

plot.netposet <- function(x,
                          plottype = "scatter",
                          pooled = ifelse(x$random, "random", "common"),
                          dim = "2d",
                          sel.x = 1, sel.y = 2, sel.z = 3,
                          cex = 1, col = "black",
                          cex.text = cex, col.text = col,
                          adj.x = 0, adj.y = 1,
                          offset.x = 0.005, offset.y = -0.005,
                          pch = NULL, cex.points = cex, col.points = col,
                          col.lines = "black", lty.lines = 1, lwd.lines = 1,
                          arrows = FALSE,
                          length = 0.05,
                          grid = TRUE,
                          col.grid = "gray", lty.grid = 2, lwd.grid = 1,
                          ...) {
  
  
  chkclass(x, "netposet")
  x <- updateversion(x)
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  
  
  if (pooled == "common") {
    p.matrix <- x$P.common
    M0 <- x$M0.common
  }
  else {
    p.matrix <- x$P.random
    M0 <- x$M0.random
  }
  ##
  outcomes <- colnames(p.matrix)
  treatments <- rownames(p.matrix)
  ##
  n.outcomes   <- length(outcomes)
  n.treatments <- length(treatments)
  
  
  dim <- setchar(dim, c("2d", "3d"))
  is_2d <- dim == "2d"
  is_3d <- !is_2d
  ##
  if (is_3d & n.outcomes == 2) {
    warning("Scatter plot in 2-D generated as only two outcomes considered ",
            "in netposet().")
    is_2d <- TRUE
    is_3d <- FALSE
  }
  ##
  plottype <- setchar(plottype, c("scatter", "biplot"))
  is_biplot <- plottype == "biplot"
  ##
  if (is_biplot) {
    if (is_2d & n.outcomes == 2) {
      is_biplot <- FALSE
      warning("Scatter plot instead of biplot generated as only ",
              "two outcomes considered in netposet().")
    }
    if (is_3d & n.outcomes == 3) {
      is_biplot <- FALSE
      warning("Scatter plot instead of biplot generated as only ",
              "three outcomes considered in netposet().")
    }
  }
  ##
  if (is_biplot) {
    outcomes <- paste("Principal Component", 1:3)
    sel.x <- 2
    sel.y <- 1
    ##
    if (n.outcomes > 2 & is_3d) {
      sel.x <- 2
      sel.y <- 3
      sel.z <- 1
    }
    p.matrix <- prcomp(p.matrix, scale = TRUE)$x
  }
  else {
    sel.x <- as.numeric(setchar(sel.x, seq_len(n.outcomes)))
    sel.y <- as.numeric(setchar(sel.y, seq_len(n.outcomes)))
    ##
    if (n.outcomes > 2)
      sel.z <- as.numeric(setchar(sel.z, seq_len(n.outcomes)))
  }
  ##
  if (is_3d & !is_installed_package("rgl", stop = FALSE)) {
    warning(paste0("2-D plot generated as package 'rgl' is missing.",
                   "\n  ",
                   "Please install package 'rgl' in order to ",
                   "produce 3-D plots\n  ",
                   "(R command: 'install.packages(\"rgl\")').",
                   if (length(grep("darwin", R.Version()$os)) == 1)
                     paste0("\n  Note, macOS users have to install ",
                            "XQuartz, see https://www.xquartz.org/.")
                   ))
    dim <- "2d"
    is_2d <- TRUE
    is_3d <- FALSE
  }
  ##
  use_pch <- !is.null(pch)
  ##
  chknumeric(cex.text, min = 0, zero = 0)
  chknumeric(adj.x)
  chknumeric(adj.y)
  chknumeric(offset.x)
  chknumeric(offset.y)
  chknumeric(lty.lines, min = 0, zero = 0, length = 1)
  chknumeric(lwd.lines, min = 0, zero = 0, length = 1)
  chklogical(arrows)
  chknumeric(length, min = 0, zero = 0, length = 1)
  if (use_pch)
    chknumeric(cex.points, min = 0, zero = 0)
  chklogical(grid)
  chknumeric(lty.grid, min = 0, zero = 0, length = 1)
  chknumeric(lwd.grid, min = 0, zero = 0, length = 1)
  
  
  if (!(length(cex.text) %in% c(1, n.treatments)))
    stop("Argument 'cex.text' must be a single numeric or vector of length ",
         n.treatments, ".")
  ##
  if (!(length(col.text) %in% c(1, n.treatments)))
    stop("Argument 'col.text' must be a character string or vector of length ",
         n.treatments, ".")
  ##
  if (!(length(adj.x) %in% c(1, n.treatments)))
    stop("Argument 'adj.x' must be a single numeric or vector of length ",
         n.treatments, ".")
  ##
  if (!(length(adj.y) %in% c(1, n.treatments)))
    stop("Argument 'adj.y' must be a single numeric or vector of length ",
         n.treatments, ".")
  ##
  if (!(length(offset.x) %in% c(1, n.treatments)))
    stop("Argument 'offset.x' must be a single numeric or vector of length ",
         n.treatments, ".")
  ##
  if (!(length(offset.y) %in% c(1, n.treatments)))
    stop("Argument 'offset.y' must be a single numeric or vector of length ",
         n.treatments, ".")
  ##
  if (use_pch) {
    if (!(length(pch) %in% c(1, n.treatments)))
      stop("Argument 'pch' must be a single numeric or vector of length ",
           n.treatments, ".")
    ##
    if (!(length(cex.points) %in% c(1, n.treatments)))
      stop("Argument 'cex.points' must be a single numeric or vector of length ",
           n.treatments, ".")
    ##
    if (!(length(col.points) %in% c(1, n.treatments)))
      stop("Argument 'col.points' must be a character string or vector of length ",
           n.treatments, ".")
  }
  
  
  if (length(cex.text) == 1)
    cex.text <- rep(cex.text, n.treatments)
  ##
  if (length(col.text) == 1)
    col.text <- rep(col.text, n.treatments)
  ##
  if (length(adj.x) == 1)
    adj.x <- rep(adj.x, n.treatments)
  ##
  if (length(adj.y) == 1)
    adj.y <- rep(adj.y, n.treatments)
  ##
  if (length(offset.x) == 1)
    offset.x <- rep(offset.x, n.treatments)
  ##
  if (length(offset.y) == 1)
    offset.y <- rep(offset.y, n.treatments)
  ##
  if (use_pch) {
    if (length(pch) == 1)
      pch <- rep(pch, n.treatments)
    ##
    if (length(cex.points) == 1)
      cex.points <- rep(cex.points, n.treatments)
    ##
    if (length(col.points) == 1)
      col.points <- rep(col.points, n.treatments)
  }
  
  
  seq.treats <- seq_len(n.treatments)
  ##
  if (is_2d) {
    ##
    ## 2-D plot
    ##
    xvals <- p.matrix[, sel.x]
    yvals <- p.matrix[, sel.y]
    ##
    if (is_biplot) {
      plot(xvals, yvals,
           type = "n",
           xlab = outcomes[sel.x], ylab = outcomes[sel.y],
           ...)
    }
    else {
      ##
      plot(xvals, yvals,
           type = "n",
           xlab = outcomes[sel.x], ylab = outcomes[sel.y],
           xlim = c(0, 1), ylim = c(0, 1),
           ...)
    }
    
    if (grid & !is_biplot) {
      for (i in seq.treats) {
        lines(x = c(xvals[i], xvals[i]),
              y = c(0, yvals[i]),
              col = col.grid, lty = lty.grid, lwd = lwd.grid)
        ##
        lines(x = c(0, xvals[i]),
              y = c(yvals[i], yvals[i]),
              col = col.grid, lty = lty.grid, lwd = lwd.grid)
      }
    }
    ##
    if (use_pch)
      for (i in seq.treats)
        points(xvals[i], yvals[i],
               pch = pch[i], col = col.points[i], cex = cex.points[i])
    ##
    for (i in seq.treats) {
      for (j in seq.treats) {
        if (M0[i, j] == 1) {
          if (arrows)
            arrows(xvals[i], yvals[i],
                   xvals[j], yvals[j],
                   col = col.lines, lty = lty.lines, lwd = lwd.lines,
                   length = length)
          else
            lines(c(xvals[i], xvals[j]),
                  c(yvals[i], yvals[j]),
                   col = col.lines, lty = lty.lines, lwd = lwd.lines)
        }
      }
    }
    ##
    for (i in seq.treats)
      text(xvals[i] + offset.x[i],
           yvals[i] + offset.y[i],
           treatments[i],
           adj = c(adj.x[i], adj.y[i]),
           col = col.text[i], cex = cex.text[i])
  }
  else {
    ##
    ## 3-D plot
    ##
    xvals <- p.matrix[, sel.x]
    yvals <- p.matrix[, sel.y]
    zvals <- p.matrix[, sel.z]
    ##
    rgl::plot3d(xvals, yvals, zvals,
                xlab = outcomes[sel.x],
                ylab = outcomes[sel.y],
                zlab = outcomes[sel.z])
    ##
    if (use_pch)
      for (i in seq.treats)
        rgl::points3d(xvals[i], yvals[i], zvals[i],
                      pch = pch[i], col = col.points[i], cex = cex.points[i])
    ##
    for (i in seq.treats)
      for (j in seq.treats)
        if (M0[i, j] == 1)
          rgl::lines3d(x = c(xvals[i], xvals[j]),
                       y = c(yvals[i], yvals[j]),
                       z = c(zvals[i], zvals[j]),
                       lwd = 2)
    ##
    for (i in seq.treats)
      rgl::text3d(xvals[i], yvals[i], zvals[i],
                  treatments[i],
                  col = col.text[i], cex = cex.text[i])
  }
  
  invisible(NULL)
}

rankings <- function(x) {
  
  #
  # Calculate ranks and rankogram
  #
  
  nsim <- nrow(x)
  n.trts <- ncol(x)
  #
  rnk <- apply(x, 1, rank, ties.method = "random")
  #
  tab <- apply(rnk, 1, table)
  #
  if (is.list(tab)) {
    rankogram <- matrix(0, nrow = n.trts, ncol = n.trts)
    rownames(rankogram) <- names(tab)
    colnames(rankogram) <- seq_len(n.trts)
    ##
    for (i in names(tab))
      rankogram[i, names(tab[[i]])] <- tab[[i]][names(tab[[i]])]
  }
  else
    rankogram <- t(as.data.frame(tab))
  #
  rankogram <- rankogram / nsim
  
  #
  # Cumulative ranks and SUCRAs
  #
  
  cumrank <- t(apply(rankogram, 1, cumsum))
  sucras <- apply(cumrank[, -n.trts], 1, sum) / (n.trts - 1)
  
  #
  # Mean and median ranks
  #
  
  meanranks <- apply(col(rankogram) * rankogram, 1, sum)
  #
  getmed <- function(x) which(x >= 0.5)[1]
  medianranks <- apply(cumrank, 1, getmed)
  
  #
  # Return results
  #
  
  res <- list(sucras = sucras,
              rankogram = rankogram,
              cumrank = cumrank,
              #
              meanranks = meanranks,
              medianranks = medianranks,
              #
              nsim = nsim)
  #
  res
}

prepare <- function(TE, seTE, treat1, treat2, studlab, tau = 0,
                    correlated = FALSE, func.inverse) {
  
  if (is.na(tau))
    tau <- 0
  
  data <- data.frame(studlab,
                     treat1, treat2,
                     treat1.pos = NA, treat2.pos = NA,
                     TE, seTE, weights = 1 / (seTE^2 + tau^2), correlated,
                     narms = NA, stringsAsFactors = FALSE)
  #
  # Ordering dataset
  #
  o <- order(data$studlab, data$treat1, data$treat2)
  data <- data[o, ]
  #
  # Adapt numbers to treatment IDs
  #
  names.treat <- sort(unique(c(data$treat1, data$treat2)))
  data$treat1.pos <- match(data$treat1, names.treat)
  data$treat2.pos <- match(data$treat2, names.treat)
  #
  data$order <- o
  
  sl <- unique(data$studlab)
  #
  # List with weight matrices
  #
  W.list <- vector("list", length(sl))
  names(W.list) <- sl
  #
  # List with covariance matrices
  #
  C.list <- vector("list", length(sl))
  names(C.list) <- sl
  #
  # Determining number of arms and adjusting weights of multi-arm studies
  #
  for (s in sl) {
    sel.s <- data$studlab == s
    correlated.s <- unique(data$correlated[sel.s]) 
    #
    if (length(correlated.s) != 1)
      stop("Different values for argument 'correlated' for study '", s, "'.",
           call. = FALSE)
    # Only treatment arms from multi-arm studies can be correlated
    if (correlated.s & sum(sel.s) == 1)
      correlated.s <- FALSE
    #
    res.s <- covar_study(1 / data$weights[sel.s], s, correlated.s, func.inverse)
    #
    W.list[[s]] <- res.s$W
    C.list[[s]] <- res.s$Cov
    #
    data$narms[sel.s] <- res.s$n
    data$weights[sel.s] <- diag(res.s$W)
  }
  #
  res <- list(W = bdiag(W.list), Cov = bdiag(C.list), data = data)
  #
  res
}


covar_study <- function(v, studlab, correlated, func.inverse) {
  m <- length(v)
  n <- (1 + sqrt(8 * m + 1)) / 2
  #
  if (correlated) {
    B <- createB(ncol = n)
    V <- diag(diag(t(B) %*% diag(v, nrow = m) %*% B)) - t(B) %*%
      diag(v, nrow = m) %*% B
    #
    Cov <- matrix(0, nrow = m, ncol = m)
    edges <- matrix(nrow = m, ncol = 2)
    #
    r <- 0
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        r <- r + 1
        edges[r, ] <- c(i, j)
      }
    }
    #
    for (p in 1:(m - 1)) {
      i <- edges[p, 1]
      j <- edges[p, 2]
      #
      for (q in (p+1):m) {
        k <- edges[q, 1]
        l <- edges[q, 2] 
        #
        Cov[p, q] <- 0.5 * (V[i, l] - V[i, k] + V[j, k] - V[j, l])
        Cov[q, p] <- 0.5 * (V[i, l] - V[i, k] + V[j, k] - V[j, l])
      }
    }
    #
    for (p in 1:m) {
      i <- edges[p, 1]
      j <- edges[p, 2]
      #
      Cov[p, p] <- V[i, j]
    }
    #
    if (qr(Cov)$rank == n - 1)
      W <- ginv(as.matrix(Cov))
    else {
      if (length(v) > 1)
        W <- diag(1 / v)
      else {
        Cov <- matrix(v)
        W <- 1 / Cov
      }
    }
  }
  else {
    if (length(v) > 1) {
      v <- multiarm(v, studlab, func.inverse)$v
      Cov <- diag(v)
      W <- diag(1 / v)
    }
    else {
      Cov <- matrix(v)
      W <- 1 / Cov
    }
  }
  #
  res <- list(v = v, n = n, m = m, Cov = as.matrix(Cov), W = W)
}

get_unident <- function(X.matrix, details.chkident) {
  comps.unident <- character(0)
  #
  Xplus <- ginv(X.matrix)
  colnames(Xplus) <- rownames(X.matrix)
  rownames(Xplus) <- colnames(X.matrix)
  e <- eigen(Xplus %*% X.matrix)$values
  E <- eigen(Xplus %*% X.matrix)$vectors
  rownames(E) <- rownames(Xplus %*% X.matrix)
  M <- as.matrix(E[, is_zero(e, n = 100)])
  #
  # Identify all unidentifiable components
  #
  if (dim(M)[2] > 0) {
    for (i in 1:dim(M)[2])
      comps.unident <-
        c(comps.unident, names(M[, i])[!is_zero(M[, i], n = 100)])
    #
    comps.unident <- unique(sort(comps.unident))
  }
  #
  # Identify components only appearing in combination
  #
  Xid <- X.matrix[, is_identical(X.matrix), drop = FALSE]
  #
  ncols <- ncol(Xid)
  list.id <- vector("list")
  j <- 0
  #
  while (ncols > 1) {
    sel.i <- vector()
    for (i in 1 + seq_len(ncols - 1)) {
      if (all(Xid[, 1] == Xid[, i]))
        sel.i <- c(sel.i, i)
    }
    #
    if (length(sel.i) > 0) {
      j <- j + 1
      list.id[[j]] <- colnames(Xid[, c(1, sel.i), drop = FALSE])
      Xid <- Xid[, -c(1, sel.i), drop = FALSE]
      ncols <- ncol(Xid)
    }
  }
  #
  if (length(list.id) > 0)
    warning(
      paste0("The following components are not uniquely identifiable as ",
             "they always appear together in a treatment: ",
             paste0("'", lapply(list.id, paste, collapse = " + "),
                    "'", collapse = "; "),
             ". The combined effect of these components is identifiable ",
             "by creating a combined component."),
      call. = FALSE)
  #
  # Warn about remaining unidentifiable components
  #
  if (dim(M)[2] > 0) {
    if (length(list.id) == 0)
      comps.unident.rest <- comps.unident
    else
      comps.unident.rest <-
        comps.unident[!(comps.unident %in% unique(unlist(list.id)))]
    #
    if (length(comps.unident.rest) > 0)
      warning(paste0("The following component",
                     if (length(comps.unident.rest) > 1)
                       "s are " else " is ",
                     if (length(list.id) > 0) "also ",
                     "not uniquely identifiable: ",
                     paste(paste0("'", comps.unident.rest, "'"),
                           collapse = ", ")),
              call. = FALSE)
    #
    if (details.chkident) {
      M[is_zero(M, n = 100)] <- 0
      prmatrix(M, quote = FALSE, right = TRUE)
    }
  }
  #
  comps.unident
}

armlab <- function(sm, backtransf, big.mark = gs("big.mark")) {
  
  lab <- ""
  
  if (sm == "SMD")
    lab <- "Std. Mean"
  ##
  else if (sm %in% c("MD", "ROM"))
    lab <- "Mean"
  ##
  else if (sm %in% c("RD", "RR"))
    lab <- "Risk"
  ##
  else if (sm %in% c("OR", "DOR"))
    lab <- "Odds"
  ##
  else if (sm == "HR")
    lab <- "Hazard"
  ##
  else if (sm == "ASD")
    lab <- "AS"
  ##
  else if (sm %in% c("IRD", "IRR"))
    lab <- "Rate"
  ##
  else if (sm == "VE")
    lab <- "Effectiveness"
  ##
  if (!backtransf) {
    if (is_relative_effect(sm))
      lab <- paste("Log", lab)
    ##
    else if (sm == "VE")
      lab <- "Log Risk"
  }
  
  lab
}

#' Print and change default network meta-analysis settings in R package
#' \bold{netmeta}
#' 
#' @description
#' Print and change default settings to conduct and print or plot
#' network meta-analyses in R package \bold{netmeta}.
#' 
#' @param ... Arguments to change default settings.
#' @param quietly A logical indicating whether information on settings
#'   should be printed.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link[meta]{settings.meta}}, \code{\link[meta]{gs}}
#' 
#' @export settings.netmeta

settings.netmeta <- function(..., quietly = TRUE) {
  
  #
  # Check argument
  #
  
  missing.quietly <- missing(quietly)
  chklogical(quietly)
  
  
  #
  # Save object with current settings
  #
  
  argslist <- gs(".argslist.netmeta")
  #
  oldset <- vector("list", 0)
  #
  for (i in argslist)
    oldset[[i]] <- gs(i)
  #
  print.settings <- FALSE
  reset.settings <- FALSE
  #
  args  <- list(...)
  
  names.all <- names(args)
  #
  sel.netmeta <- names.all %in% argslist
  
  
  #
  # Determine whether to print, reset or use specific settings
  #
  if (any(names.all == "print") && args[["print"]]) {
    print.settings <- TRUE
    quietly <- FALSE
  }
  if (any(names.all == "reset") && args[["reset"]])
    reset.settings <- TRUE
  
  
  #
  # Print settings if no argument is provided
  #
  
  if (length(args) == 0) {
    if (missing.quietly || !quietly)
      settings.netmeta("print", quietly = FALSE)
    #
    return(invisible(oldset))
  }
  
  
  #
  # Check argument names
  #
  
  names.all <- names(args)
  #
  if (length(names.all) != length(unique(names.all)))
    stop("Arguments must be unique.")
  #
  sel.netmeta <- names.all %in% argslist
  names.netmeta <- names.all[sel.netmeta]
  names.other <- names.all[!sel.netmeta]
  #
  args.netmeta <- args.other <- vector("list", 0)
  #
  if (length(names.netmeta) > 0) {
    for (i in names.netmeta) {
      args.netmeta[[i]] <- args[[i]]
    }
  }
  #
  if (length(names.other) > 0) {
    for (i in names.other) {
      args.other[[i]] <- args[[i]]
    }
  }
  
  
  #
  # Check whether first argument is a list. In this case only use
  # this list as input.
  #
  
  warn.depr <- TRUE
  if (length(args) > 0 && is.list(args[[1]])) {
    if (!is.null(names(args))) {
      print(names(args))
      warning("Additional arguments ignored as first argument is a list.",
              call. = FALSE)
    }
    warn.depr <- FALSE
    args <- args[[1]]
  }
  
  
  #
  # Unnamed first (and only) argument must be character string or a
  # logical
  #
  
  if (length(args) == 1 & is.null(names(args))) {
    if (is.character(unlist(args)))
      action <- setchar(unlist(args), c("reset", "print"),
                        stop.at.error = FALSE)
    else
      action <- unlist(args)
    #
    if (is.null(action))
      stop("First argument can be one of the following character strings:",
           "\n 'reset', 'print'",
           call. = FALSE)
    else if (action == "reset")
      settings.netmeta(reset = TRUE, quietly = quietly)
    else if (action == "print" | (is.logical(action) && action))
      settings.netmeta(print = TRUE, quietly = FALSE)
    #
    return(invisible(oldset))
  }
  #
  else if (length(args) > 1 & names(args)[1] == "") {
    if (is.character(unlist(args[[1]])))
      action <- setchar(unlist(args[[1]]), c("reset", "print"),
                        stop.at.error = FALSE)
    else
      action <- unlist(args[[1]])
    #
    if (is.null(action))
      stop("First argument can be one of the following character strings:",
           "\n 'reset', 'print'",
           call. = FALSE)
    else if (action == "reset")
      settings.netmeta(reset = TRUE, quietly = quietly)
    else if (action == "print")
      settings.netmeta(print = TRUE, quietly = FALSE)
  }
    
  
  #
  # Reset settings
  #
  
  if (reset.settings) {
    if (!quietly)
      cat("\n** Reset all network meta-analysis settings (R package netmeta). ",
          "**\n\n")
    #
    settings.meta(baseline.reference = TRUE, small.values = "desirable",
                  all.treatments = NULL, seq = NULL,
                  method.tau.netmeta = "DL",
                  drop.reference.group = TRUE, equal.size = TRUE,
                  show = "both",
                  #
                  nsim = 1000, lump.comparator = FALSE,
                  #
                  plastic = FALSE, col.netgraph = NULL,
                  number.of.studies = TRUE, thickness = "number.of.studies",
                  multiarm = FALSE,
                  #
                  tol.multiarm = 0.001, tol.multiarm.se = NULL,
                  details.chkmultiarm = FALSE,
                  #
                  na.unident = TRUE,
                  sep.trts = ":", sep.comps = "+", sep.ia = "*",
                  nchar.trts = 666, nchar.studlab = 666,
                  #
                  sort.distance = TRUE,
                  #
                  legend = TRUE)
  }
  
  
  #
  # Print settings
  #
  
  if (print.settings & !quietly) {
    cat(paste0("\n** Settings for network meta-analysis method (",
               "R package netmeta, version ",
               packageDescription("netmeta")$Version,
               ") **\n\n"))
    #
    cat(paste0("* General settings *\n"))
    catarg("baseline.reference   ")
    catarg("small.values         ")
    catarg("all.treatments       ")
    catarg("seq                  ")
    catarg("method.tau.netmeta   ")
    catarg("tol.multiarm         ")
    catarg("tol.multiarm.se      ")
    catarg("details.chkmultiarm  ")
    catarg("nchar.studlab        ")
    catarg("legend               ")
    #
    cat("\n* Additional settings for network meta-analysis *\n")
    catarg("sep.trts             ")
    catarg("nchar.trts           ")
    #
    cat("\n* Additional settings for component network meta-analysis *\n")
    catarg("sep.comps            ")
    catarg("sep.ia               ")
    catarg("na.unident           ")
    #
    cat("\n* Additional settings for forest plots *\n")
    catarg("drop.reference.group ")
    catarg("equal.size           ")
    #
    cat("\n* Additional settings for network graphs *\n")
    catarg("plastic              ")
    catarg("col.netgraph         ")
    catarg("number.of.studies    ")
    catarg("thickness            ")
    catarg("multiarm             ")
    #
    cat("\n* Additional setting for netsplit() *\n")
    catarg("show                 ")
    #
    cat("\n* Additional setting to evaluate small study effects *\n")
    catarg("lump.comparator      ")
    #
    cat("\n* Number of samples to calculate ranking metrics *\n")
    catarg("nsim                 ")
    #
    cat("\n* Additional setting for netdistance() *\n")
    catarg("sort.distance        ")
    #
    return(invisible(oldset))
  }
  
  
  #
  # Set settings
  #

  if (length(names.netmeta) > 0) {
    setlogical("baseline.reference", args.netmeta)
    setcharacter("small.values", args.netmeta, c("desirable", "undesirable"))
    setlogical("all.treatments", args.netmeta, NULL.ok = TRUE)
    setcharacter("seq", args.netmeta, NULL.ok = TRUE)
    setcharacter("method.tau.netmeta", args.netmeta, c("DL", "REML", "ML"))
    setlogical("drop.reference.group", args.netmeta)
    setlogical("equal.size", args.netmeta)
    setcharacter("show", args.netmeta,
                 c("all", "both", "with.direct", "direct.only", "indirect.only"))
    setnumeric("nsim", args.netmeta)
    setlogical("lump.comparator", args.netmeta)
    setlogical("plastic", args.netmeta)
    setcharacter("col.netgraph", args.netmeta, NULL.ok = TRUE)
    setlogical("number.of.studies", args.netmeta)
    setcharacter("thickness", args.netmeta)
    setlogical("multiarm", args.netmeta)
    setnumeric("tol.multiarm", args.netmeta)
    setnumeric("tol.multiarm.se", args.netmeta)
    setlogical("details.chkmultiarm", args.netmeta)
    setlogical("na.unident", args.netmeta)
    setcharacter("sep.trts", args.netmeta)
    setcharacter("sep.comps", args.netmeta)
    setcharacter("sep.ia", args.netmeta)
    setnumeric("nchar.trts", args.netmeta)
    setnumeric("nchar.studlab", args.netmeta)
    setlogical("sort.distance", args.netmeta)
    setlogical("legend", args.netmeta)
  }
  
  
  #
  # Return current settings
  #
  
  res <- vector("list", 0)
  #
  for (i in argslist)
    res[[i]] <- gs(i)
  #
  invisible(res)
}

#' Print method for objects of class netmeta
#' 
#' @description
#' Print method for objects of class \code{netmeta}.
#' 
#' @param x An object of class \code{netmeta}.
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param prediction A logical indicating whether prediction intervals
#'   should be printed.
#' @param reference.group Reference treatment.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
#'   matrices with all treatment effects, and confidence limits will
#'   be printed.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param header A logical indicating whether information on title of
#'   meta-analysis, comparison and outcome should be printed at the
#'   beginning of the printout.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for tests
#'   of overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall effects, see \code{print.default}.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity tests, see \code{print.default}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistics, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for square
#'   root of between-study variance, see \code{print.default}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   statistic, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.Q A logical value indicating whether to print the
#'   results of the test of heterogeneity.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param text.I2 Text printed to identify heterogeneity statistic
#'   I\eqn{^2}.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}
#' 
#' @keywords print
#' 
#' @examples
#' # Examples: example(netmeta)
#' 
#' @method print netmeta
#' @export

print.netmeta <- function(x,
                          common = x$common,
                          random = x$random,
                          prediction = x$prediction,
                          reference.group = x$reference.group,
                          baseline.reference = x$baseline.reference,
                          all.treatments = x$all.treatments,
                          overall.hetstat = x$overall.hetstat,
                          backtransf = x$backtransf,
                          nchar.trts = x$nchar.trts,
                          header = TRUE,
                          digits = gs("digits"),
                          digits.stat = gs("digits.stat"),
                          digits.pval = max(gs("digits.pval"), 2),
                          digits.pval.Q = max(gs("digits.pval.Q"), 2),
                          digits.Q = gs("digits.Q"),
                          digits.tau2 = gs("digits.tau2"),
                          digits.tau = gs("digits.tau"),
                          digits.I2 = gs("digits.I2"),
                          #
                          big.mark = gs("big.mark"),
                          scientific.pval = gs("scientific.pval"),
                          zero.pval = gs("zero.pval"),
                          JAMA.pval = gs("JAMA.pval"),
                          #
                          print.tau2 = gs("print.tau2"),
                          print.tau = gs("print.tau"),
                          print.Q = gs("print.Q"),
                          print.I2 = gs("print.I2"),
                          print.I2.ci = gs("print.I2.ci"),
                          #
                          text.tau2 = gs("text.tau2"),
                          text.tau = gs("text.tau"),
                          text.I2 = gs("text.I2"),
                          #
                          details.methods = gs("details"),
                          legend = gs("legend"),
                          #
                          warn.deprecated = gs("warn.deprecated"),
                          ##
                          ...) {
  
  
  ##
  ##
  ## (1) Check for netmeta object and upgrade object
  ##
  ##
  chkclass(x, "netmeta")
  x <- updateversion(x)
  ##
  is.mh.nch <- inherits(x, "netmetabin") && x$method %in% c("MH", "NCH")
  ##  
  if (is.null(x$df.Q))
    oldversion <- TRUE
  else
    oldversion <- FALSE
  #
  if (is.null(x$lower.predict))
    prediction <- FALSE
  #
  nchar.trts <- replaceNULL(nchar.trts, 666)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chklogical(prediction)
  chklogical(baseline.reference)
  ##
  chknumeric(nchar.trts, min = 1, length = 1)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  chknumeric(digits.pval.Q, min = 1, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  #
  chkchar(big.mark, length = 1)
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chklogical(print.tau2)
  chklogical(print.tau)
  chklogical(print.Q)
  chklogical(print.I2)
  chklogical(print.I2.ci)
  #
  chkchar(text.tau2)
  chkchar(text.tau)
  chkchar(text.I2)
  ##
  chklogical(details.methods)
  chklogical(legend)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  chklogical(overall.hetstat)
  #
  backtransf <-
    deprecated(backtransf, missing(backtransf), args, "logscale")
  if (is_untransformed(x$sm))
    backtransf <- TRUE
  backtransf <- replaceNULL(backtransf, TRUE)
  chklogical(backtransf)
  
  
  ##
  ##
  ## (3) Some additional settings
  ##
  ##
  k <- x$k
  m <- x$m
  n <- x$n
  sm <- sm.lab <- x$sm
  #
  if (!backtransf & (is_relative_effect(sm) | sm == "VE"))
    sm.lab <- paste0("log", if (sm == "VE") "VR" else sm)
  ##
  ci.lab <- paste0(round(100 * x$level.ma, 1), "% CI")
  #
  zlab <- "z"
  
  
  ##
  ##
  ## (4) Set and backtransform results of meta-analysis
  ##
  ##
  TE.common <- x$TE.common
  seTE.common <- x$seTE.common
  lowTE.common <- x$lower.common
  uppTE.common <- x$upper.common
  statistic.common <- replaceNULL(x$statistic.common, x$zval.common)
  pval.common <- x$pval.common
  ##
  TE.random <- x$TE.random
  seTE.random <- x$seTE.random
  lowTE.random <- x$lower.random
  uppTE.random <- x$upper.random
  statistic.random <- replaceNULL(x$statistic.random, x$zval.random)
  pval.random <- x$pval.random
  ##
  lower.predict <- x$lower.predict
  upper.predict <- x$upper.predict
  ##
  if (!is.null(x$seq)) {
    TE.common <- TE.common[x$seq, x$seq]
    seTE.common <- seTE.common[x$seq, x$seq]
    lowTE.common <- lowTE.common[x$seq, x$seq]
    uppTE.common <- uppTE.common[x$seq, x$seq]
    statistic.common <- statistic.common[x$seq, x$seq]
    pval.common <- pval.common[x$seq, x$seq]
    ##
    if (!all(is.na(TE.random))) {
      TE.random <- TE.random[x$seq, x$seq]
      seTE.random <- seTE.random[x$seq, x$seq]
      lowTE.random <- lowTE.random[x$seq, x$seq]
      uppTE.random <- uppTE.random[x$seq, x$seq]
      statistic.random <- statistic.random[x$seq, x$seq]
      pval.random <- pval.random[x$seq, x$seq]
      ##
      lower.predict <- lower.predict[x$seq, x$seq]
      upper.predict <- upper.predict[x$seq, x$seq]
    }
  }
  ##  
  noeffect <- 1L * (backtransf & is_relative_effect(sm))
  #
  if (backtransf) {
    TE.common    <- backtransf(TE.common, sm)
    lowTE.common <- backtransf(lowTE.common, sm)
    uppTE.common <- backtransf(uppTE.common, sm)
    #
    TE.random    <- backtransf(TE.random, sm)
    lowTE.random <- backtransf(lowTE.random, sm)
    uppTE.random <- backtransf(uppTE.random, sm)
    #
    if (prediction) {
      lower.predict <- backtransf(lower.predict, sm)
      upper.predict <- backtransf(upper.predict, sm)
    }
    #
    # Switch lower and upper limit for VE if results have been
    # backtransformed
    #
    if (sm == "VE") {
      tmp.l <- lowTE.common
      lowTE.common <- uppTE.common
      uppTE.common <- tmp.l
      #
      tmp.l <- lowTE.random
      lowTE.random <- uppTE.random
      uppTE.random <- tmp.l
      #
      if (prediction) {
        tmp.l <- lower.predict
        lower.predict <- upper.predict
        upper.predict <- tmp.l
      }
    }
  }
  ##
  TE.common <- round(TE.common, digits)
  lowTE.common <- round(lowTE.common, digits)
  uppTE.common <- round(uppTE.common, digits)
  statistic.common <- round(statistic.common, digits.stat)
  ##
  TE.random    <- round(TE.random, digits)
  lowTE.random <- round(lowTE.random, digits)
  uppTE.random <- round(uppTE.random, digits)
  statistic.random <- round(statistic.random, digits.stat)
  ##
  if (prediction) {
    lower.predict <- round(lower.predict, digits)
    upper.predict <- round(upper.predict, digits)
  }
  ##
  I2 <- round(100 * x$I2, digits.I2)
  lower.I2 <- round(100 * x$lower.I2, digits.I2)
  upper.I2 <- round(100 * x$upper.I2, digits.I2)
  
  
  ##
  ##
  ## (5) Print result for network meta-analysis
  ##
  ##
  if (header)
    matitle(x)
  ##  
  if (reference.group != "" & is.null(all.treatments))
    all.treatments <- FALSE
  ##
  if (reference.group != "")
    reference.group <- setref(reference.group, rownames(TE.common))
  ##  
  if (common | random) {
    cat("Number of studies: k = ", k, "\n", sep = "")
    #
    if (!is.na(m))
      cat("Number of pairwise comparisons: m = ", m, "\n", sep = "")
    #
    if (!is.null(x$n.trts))
      cat("Number of observations: o = ",
          round(sum(x$n.trts, na.rm = TRUE), 1),
          "\n",
          sep = "")
    #
    cat("Number of treatments: n = ", n, "\n", sep = "")
    #
    if (!oldversion)
      cat("Number of designs: d = ", x$d, "\n", sep = "")
    #
    if (reference.group != "") {
      if (baseline.reference)
        comptext <-
          paste0(if (x$n == 2)
            paste0("'",
                   treats(rownames(TE.common),
                          nchar.trts)[rownames(TE.common)
                                      != reference.group],
                   "'")
            else
              "other treatments",
            " vs '",
            treats(rownames(TE.common),
                   nchar.trts)[rownames(TE.common)
                               == reference.group],
            "'")
      else
        comptext <-
          paste0("'",
                 treats(rownames(TE.common),
                        nchar.trts)[rownames(TE.common)
                                    == reference.group],
                 "' vs ",
                 if (x$n == 2)
                   paste0("'",
                          treats(rownames(TE.common),
                                 nchar.trts)[rownames(TE.common)
                                             != reference.group],
                          "'")
                 else
                   "other treatments")
    }
    ##
    if (common) {
      if (all.treatments | reference.group != "") {
        text.common <- "Common effects model"
        ##
        if (x$method == "MH")
          text.common <-
            paste(text.common, "(Mantel-Haenszel method)")
        else if (x$method == "NCH")
          text.common <-
            paste(text.common, "(Non-central hypergeometric distribution)")
        ##
        cat("\n", text.common, "\n", sep = "")
      }
      ##
      if (all.treatments) {
        cat("\nTreatment estimate (sm = '", sm.lab, "'):\n", sep = "")
        ##
        TEf <- formatN(TE.common, digits = digits)
        rownames(TEf) <- treats(TEf, nchar.trts)
        colnames(TEf) <- treats(TEf, nchar.trts, FALSE)
        ##
        if (all(diag(TE.common) == noeffect))
          diag(TEf) <- "."
        ##
        prmatrix(TEf, quote = FALSE, right = TRUE)
        ##
        cat("\nLower ", 100 * x$level.ma, "%-confidence limit:\n", sep = "")
        ##
        lowTEf <- formatN(lowTE.common, digits = digits)
        rownames(lowTEf) <- treats(lowTEf, nchar.trts)
        colnames(lowTEf) <- treats(lowTEf, nchar.trts, FALSE)
        ##
        if (all(diag(lowTE.common) == noeffect))
          diag(lowTEf) <- "."
        ##
        prmatrix(lowTEf, quote = FALSE, right = TRUE)
        ##
        cat("\nUpper ", 100 * x$level.ma, "%-confidence limit:\n", sep = "")
        ##
        uppTEf <- formatN(uppTE.common, digits = digits)
        rownames(uppTEf) <- treats(uppTEf, nchar.trts)
        colnames(uppTEf) <- treats(uppTEf, nchar.trts, FALSE)
        ##
        if (all(diag(uppTE.common) == noeffect))
          diag(uppTEf) <- "."
        ##
        prmatrix(uppTEf, quote = FALSE, right = TRUE)
        ##
        ## Print prediction intervals
        ##
        if (!random && prediction & x$df.Q >= 1) {
          cat("\nPrediction intervals\n")
          ##
          cat("\nLower ", 100 * x$level.predict, "%-prediction limit:\n",
              sep = "")
          ##
          lowTEp <- formatN(lower.predict, digits = digits)
          rownames(lowTEp) <- treats(lowTEp, nchar.trts)
          colnames(lowTEp) <- treats(lowTEp, nchar.trts, FALSE)
          ##
          if (all(diag(lower.predict) == noeffect))
            diag(lowTEp) <- "."
          ##
          prmatrix(lowTEp, quote = FALSE, right = TRUE)
          ##
          cat("\nUpper ", 100 * x$level.predict, "%-prediction limit:\n",
              sep = "")
          ##
          uppTEp <- formatN(upper.predict, digits = digits)
          rownames(uppTEp) <- treats(uppTEp, nchar.trts)
          colnames(uppTEp) <- treats(uppTEp, nchar.trts, FALSE)
          ##
          if (all(diag(upper.predict) == noeffect))
            diag(uppTEp) <- "."
          ##
          prmatrix(uppTEp, quote = FALSE, right = TRUE)
        }
      }
      if (reference.group != "") {
        if (all(colnames(TE.common) != reference.group))
          stop("Argument 'reference.group' must match any of ",
               "the following values: ",
               paste(paste0("'", colnames(TE.common), "'"),
                     collapse = " - "))
        ##
        if (baseline.reference) {
          TE.common.b <- TE.common[, colnames(TE.common) == reference.group]
          lowTE.common.b <-
            lowTE.common[, colnames(lowTE.common) == reference.group]
          uppTE.common.b <-
            uppTE.common[, colnames(uppTE.common) == reference.group]
          statistic.common.b <-
            statistic.common[, colnames(statistic.common) == reference.group]
          pval.common.b <-
            pval.common[, colnames(pval.common) == reference.group]
        }
        else {
          TE.common.b <- TE.common[rownames(TE.common) == reference.group, ]
          lowTE.common.b <-
            lowTE.common[rownames(lowTE.common) == reference.group, ]
          uppTE.common.b <-
            uppTE.common[rownames(uppTE.common) == reference.group, ]
          statistic.common.b <-
            statistic.common[rownames(statistic.common) == reference.group, ]
          pval.common.b <-
            pval.common[rownames(pval.common) == reference.group, ]
        }
        ##
        res <- cbind(formatN(TE.common.b, digits, text.NA = "NA",
                             big.mark = big.mark),
                     formatCI(formatN(round(lowTE.common.b, digits),
                                      digits, "NA", big.mark = big.mark),
                              formatN(round(uppTE.common.b, digits),
                                      digits, "NA", big.mark = big.mark)),
                     if (!all(is.na(statistic.common.b)))
                       formatN(statistic.common.b, digits.stat, text.NA = "NA",
                               big.mark = big.mark),
                     if (!all(is.na(pval.common.b)))
                       formatPT(pval.common.b, digits = digits.pval,
                              scientific = scientific.pval,
                              zero = zero.pval, JAMA = JAMA.pval)
                     )
        dimnames(res) <-
          list(colnames(TE.common),
               c(sm.lab, ci.lab,
                 if (!all(is.na(statistic.common.b)))
                   zlab,
                 if (!all(is.na(pval.common.b)))
                   "p-value"))
        ##
        ## Add prediction interval (or not)
        ##
        if (!random && prediction & x$df.Q >= 1) {
          if (baseline.reference) {
            lower.predict.b <-
              lower.predict[, colnames(lower.predict) == reference.group]
            upper.predict.b <-
              upper.predict[, colnames(upper.predict) == reference.group]
          }
          else {
            lower.predict.b <-
              lower.predict[rownames(lower.predict) == reference.group, ]
            upper.predict.b <-
              upper.predict[rownames(upper.predict) == reference.group, ]
          }
          ##
          pi.lab <- paste0(round(100 * x$level.predict, 1), "% PI")
          ##
          res <- cbind(res,
                       rep_len("", nrow(res)),
                       formatCI(formatN(round(lower.predict.b, digits),
                                        digits, "NA", big.mark = big.mark),
                                formatN(round(upper.predict.b, digits),
                                        digits, "NA", big.mark = big.mark)))
          colnames(res)[ncol(res) - 1] <- ".not.a.column."
          colnames(res)[ncol(res)] <- pi.lab
        }
        ##
        if (TE.common.b[rownames(res) == reference.group] == noeffect) {
          res[rownames(res) == reference.group, ] <- "."
          res[, colnames(res) == ".not.a.column."] <- ""
          colnames(res)[colnames(res) == ".not.a.column."] <- ""
        }
        ##
        rownames(res) <- treats(rownames(res), nchar.trts)
        ##
        cat("\nTreatment estimate (", comptext, "):\n", sep = "")
        ##
        prmatrix(res, quote = FALSE, right = TRUE)
      }
    }
    ##
    if (random) {
      if (all.treatments | reference.group != "")
        cat("\nRandom effects model\n")
      if (all.treatments) {
        cat("\nTreatment estimate (sm = '", sm.lab, "'):\n", sep = "")
        ##
        TEr <- formatN(TE.random, digits = digits)
        rownames(TEr) <- treats(TEr, nchar.trts)
        colnames(TEr) <- treats(TEr, nchar.trts, FALSE)
        ##
        if (all(diag(TE.random) == noeffect))
          diag(TEr) <- "."
        ##
        prmatrix(TEr, quote = FALSE, right = TRUE)
        ##
        cat("\nLower ", 100 * x$level.ma, "%-confidence limit:\n", sep = "")
        ##
        lowTEr <- formatN(lowTE.random, digits = digits)
        rownames(lowTEr) <- treats(lowTEr, nchar.trts)
        colnames(lowTEr) <- treats(lowTEr, nchar.trts, FALSE)
        ##
        if (all(diag(lowTE.random) == noeffect))
          diag(lowTEr) <- "."
        ##
        prmatrix(lowTEr, quote = FALSE, right = TRUE)
        ##
        cat("\nUpper ", 100 * x$level.ma, "%-confidence limit:\n", sep = "")
        ##
        uppTEr <- formatN(uppTE.random, digits = digits)
        rownames(uppTEr) <- treats(uppTEr, nchar.trts)
        colnames(uppTEr) <- treats(uppTEr, nchar.trts, FALSE)
        ##
        if (all(diag(uppTE.random) == noeffect))
          diag(uppTEr) <- "."
        ##
        prmatrix(uppTEr, quote = FALSE, right = TRUE)
        ##
        ## Print prediction intervals
        ##
        if (prediction & x$df.Q >= 1) {
          cat("\nPrediction intervals\n")
          ##
          cat("\nLower ", 100 * x$level.predict, "%-prediction limit:\n",
              sep = "")
          ##
          lowTEp <- formatN(lower.predict, digits = digits)
          rownames(lowTEp) <- treats(lowTEp, nchar.trts)
          colnames(lowTEp) <- treats(lowTEp, nchar.trts, FALSE)
          ##
          if (all(diag(lower.predict) == noeffect))
            diag(lowTEp) <- "."
          ##
          prmatrix(lowTEp, quote = FALSE, right = TRUE)
          ##
          cat("\nUpper ", 100 * x$level.predict, "%-prediction limit:\n",
              sep = "")
          ##
          uppTEp <- formatN(upper.predict, digits = digits)
          rownames(uppTEp) <- treats(uppTEp, nchar.trts)
          colnames(uppTEp) <- treats(uppTEp, nchar.trts, FALSE)
          ##
          if (all(diag(upper.predict) == noeffect))
            diag(uppTEp) <- "."
          ##
          prmatrix(uppTEp, quote = FALSE, right = TRUE)
        }
      }
      if (reference.group != "") {
        if (all(colnames(TE.random) != reference.group))
          stop("Argument 'reference.group' must match any of ",
               "the following values: ",
               paste(paste0("'", colnames(TE.random), "'"),
                     collapse = " - "))
        ##
        if (baseline.reference) {
          TE.random.b <-
            TE.random[, colnames(TE.random) == reference.group]
          lowTE.random.b <-
            lowTE.random[, colnames(lowTE.random) == reference.group]
          uppTE.random.b <-
            uppTE.random[, colnames(uppTE.random) == reference.group]
          statistic.random.b <-
            statistic.random[, colnames(statistic.random) == reference.group]
          pval.random.b <-
            pval.random[, colnames(pval.random) == reference.group]
        }
        else {
          TE.random.b <-
            TE.random[colnames(TE.random) == reference.group]
          lowTE.random.b <-
            lowTE.random[rownames(lowTE.random) == reference.group, ]
          uppTE.random.b <-
            uppTE.random[rownames(uppTE.random) == reference.group, ]
          statistic.random.b <-
            statistic.random[rownames(statistic.random) == reference.group, ]
          pval.random.b <-
            pval.random[rownames(pval.random) == reference.group, ]
        }
        ##
        res <- cbind(formatN(TE.random.b, digits, text.NA = "NA",
                             big.mark = big.mark),
                     formatCI(formatN(round(lowTE.random.b, digits),
                                      digits, "NA", big.mark = big.mark),
                              formatN(round(uppTE.random.b, digits),
                                      digits, "NA", big.mark = big.mark)),
                     if (!all(is.na(statistic.random.b)))
                       formatN(statistic.random.b, digits.stat, text.NA = "NA",
                             big.mark = big.mark),
                     if (!all(is.na(pval.random.b)))
                       formatPT(pval.random.b, digits = digits.pval,
                              scientific = scientific.pval,
                              zero = zero.pval, JAMA = JAMA.pval)
                     )
        dimnames(res) <-
          list(colnames(TE.random),
               c(sm.lab, ci.lab,
                 if (!all(is.na(statistic.random.b)))
                   zlab,
                 if (!all(is.na(pval.random.b)))
                   "p-value"))
        ##
        ## Add prediction interval (or not)
        ##
        if (prediction & x$df.Q >= 1) {
          if (baseline.reference) {
            lower.predict.b <-
              lower.predict[, colnames(lower.predict) == reference.group]
            upper.predict.b <-
              upper.predict[, colnames(upper.predict) == reference.group]
          }
          else {
            lower.predict.b <-
              lower.predict[rownames(lower.predict) == reference.group, ]
            upper.predict.b <-
              upper.predict[rownames(upper.predict) == reference.group, ]
          }
          ##
          pi.lab <- paste0(round(100 * x$level.predict, 1), "% PI")
          ##
          res <- cbind(res,
                       rep_len("", nrow(res)),
                       formatCI(formatN(round(lower.predict.b, digits),
                                        digits, "NA", big.mark = big.mark),
                                formatN(round(upper.predict.b, digits),
                                        digits, "NA", big.mark = big.mark))
                       )
          colnames(res)[ncol(res) - 1] <- ".not.a.column."
          colnames(res)[ncol(res)] <- pi.lab
        }
        ##
        if (!is.na(TE.random.b[rownames(res) == reference.group]) &&
            TE.random.b[rownames(res) == reference.group] == noeffect) {
          res[rownames(res) == reference.group, ] <- "."
          res[, colnames(res) == ".not.a.column."] <- ""
          colnames(res)[colnames(res) == ".not.a.column."] <- ""
        }
        ##
        rownames(res) <- treats(rownames(res), nchar.trts)
        ##
        cat("\nTreatment estimate (", comptext, "):\n", sep = "")
        ##
        prmatrix(res, quote = FALSE, right = TRUE)
      }
    }
    #
    if (!is.null(x$tau.preset))
      tau <- x$tau.preset
    else
      tau <- x$tau
    #
    print.I2 <- print.I2 & !is.na(I2)
    #
    if (overall.hetstat & !x$method == "LRP") {
      #
      if (is.mh.nch)
        txt.hetinc <- "inconsistency (between designs)"
      else if (x$d == 1)
        txt.hetinc <- "heterogeneity"
      else
        txt.hetinc <- "heterogeneity / inconsistency"
      #
      if (!is.mh.nch & (print.tau2 | print.tau | print.I2)) {
        print.I2.ci <- print.I2.ci & print.I2
        #
        text.hetstat <- paste0("\nQuantifying ", txt.hetinc, ":\n")
        #
        if (print.tau2)
          text.hetstat <- paste0(
            text.hetstat,
            formatPT(tau^2,
                     lab = TRUE, labval = text.tau2,
                     digits = digits.tau2,
                     lab.NA = "NA", big.mark = big.mark))
        #
        if (print.tau)
          text.hetstat <- paste0(
            text.hetstat,
            if (print.tau2) "; ",
            formatPT(tau,
                     lab = TRUE, labval = text.tau,
                     digits = digits.tau,
                     lab.NA = "NA", big.mark = big.mark))
        #
        if (print.I2)
          text.hetstat <- paste0(
            text.hetstat,
            if (print.tau2 | print.tau) "; ",
            paste0(text.I2, " = ", round(I2, digits.I2), "%"),
            if (print.I2.ci & (!(is.na(lower.I2) | is.na(upper.I2))))
              pasteCI(lower.I2, upper.I2,
                      digits.I2, big.mark, unit = "%"))
        #
        text.hetstat <- paste0(text.hetstat, "\n")
        #
        cat(text.hetstat)
      }
      #
      print.Q <- print.Q &
        ((is.mh.nch & !is.na(x$Q.inconsistency)) |
           (!is.mh.nch & !is.na(x$Q)))
      #
      if (print.Q && m > 1) {
        if (is.mh.nch) {
          Q.overall <- x$Q.inconsistency
          df.Q.overall <- x$df.Q.inconsistency
          pval.Q.overall <-
            formatPT(x$pval.Q.inconsistency,
                     digits = digits.pval.Q,
                     scientific = scientific.pval,
                     zero = zero.pval, JAMA = JAMA.pval)
        }
        else {
          Q.overall <- x$Q
          if (oldversion) {
            df.Q.overall <- x$df
            pval.Q.overall <-
              ifelse(df.Q.overall == 0, "--",
                     formatPT(x$pval.Q,
                              digits = digits.pval.Q,
                              scientific = scientific.pval,
                              zero = zero.pval, JAMA = JAMA.pval))
          }
          else {
            df.Q.overall <- x$df.Q
            pval.Q.overall <-
              formatPT(x$pval.Q,
                       digits = digits.pval.Q,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval)
          }
        }
        ##
        if (is.mh.nch & x$d == 1)
          cat("")
        else if (x$d == 1 | is.mh.nch |
                 is.na(x$Q.heterogeneity) | is.na(x$Q.inconsistency)) {
          Qdata <- cbind(round(Q.overall, digits.Q), df.Q.overall,
                         pval.Q.overall)
          ##
          dimnames(Qdata) <- list("", c("Q", "d.f.", "p-value"))
          ##
          cat("\nTest of ", txt.hetinc, ":\n", sep = "")
          prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
        }
        else {
          Qs <- c(x$Q, x$Q.heterogeneity, x$Q.inconsistency)
          df.Qs <- c(x$df.Q, x$df.Q.heterogeneity, x$df.Q.inconsistency)
          pval.Qs <- c(x$pval.Q, x$pval.Q.heterogeneity, x$pval.Q.inconsistency)
          pval.Qs <- formatPT(pval.Qs,
                              digits = digits.pval.Q,
                              scientific = scientific.pval,
                              zero = zero.pval, JAMA = JAMA.pval)
          cat("\nTests of heterogeneity (within designs) and inconsistency",
              if (options()$width < 77) "\n" else " ",
              "(between designs):\n",
              sep = "")
          Qdata <- data.frame(Q = round(Qs, digits.Q),
                              df = df.Qs,
                              pval = pval.Qs)
          names(Qdata) <- c("Q", "d.f.", "p-value")
          rownames(Qdata) <- c("Total",
                               "Within designs",
                               "Between designs")
          prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
        }
      }
    }
  }
  #
  # Print details of network meta-analysis methods
  #
  if (details.methods)
    cat(textmeth(x, random,
                 overall.hetstat & print.tau2, overall.hetstat & print.tau,
                 text.tau2, text.tau, digits.tau2, digits.tau,
                 overall.hetstat & print.I2, text.I2,
                 big.mark))
  #
  # Add legend with abbreviated treatment labels
  #
  legendabbr(unique(rownames(TE.common)),
             treats(TE.common, nchar.trts),
             legend)
  
  invisible(NULL)
}

samples_netmeta <- function(x, nsim, pooled) {
  
  chkclass(x, "netmeta")
  #
  pooled <- setchar(pooled, c("common", "random"))
  #
  if (pooled == "common") {
    TE.pooled <- x$TE.common
    Cov.pooled <- x$Cov.common
  }
  else {
    TE.pooled <- x$TE.random
    Cov.pooled <- x$Cov.random
  }
  #
  theta <- TE.pooled[, 1]
  
  compMatrix <- matrix(0, nrow = nrow(Cov.pooled), ncol = length(x$trts))
  rownames(compMatrix) <- rownames(Cov.pooled)
  colnames(compMatrix) <- x$trts
  #
  allcomps <- compsplit(rownames(compMatrix), x$sep.trts)
  for (i in seq_len(nrow(Cov.pooled)))
    compMatrix[i, allcomps[[i]]] <- 1
  #
  var.theta <- as.vector(ginv(compMatrix) %*% diag(Cov.pooled))
  
  samples <- rmvnorm(nsim, theta, diag(var.theta))
  rownames(samples) <- seq_len(nrow(samples))
  colnames(samples) <- x$trts
  
  res <- list(samples = samples, compMatrix = compMatrix)
  #
  res
}

#' Generic function for heat plots
#' 
#' @description
#' Generic function for heat plots
#' 
#' @param x An R object.
#' @param \dots Additional arguments.
#' 
#' @details
#' 
#' For more details, look at the following function to generate heat
#' plots:
#' \itemize{
#' \item \code{\link{heatplot.netmeta}}
#' \item \code{\link[crossnma]{heatplot.crossnma}} (if installed)
#' }
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de
#' }
#' 
#' @keywords hplot
#'
#' @examples
#' # Only consider first five studies (to reduce runtime of example)
#' #
#' studies <- unique(Senn2013$study)
#' Senn2013.5 <- subset(Senn2013, study %in% studies[1:5])
#' 
#' # Transform data from long arm-based to contrast-based format
#' #
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013.5,
#'   varnames = c("MD", "seMD"))
#' 
#' # Conduct random effects network meta-analysis with
#' # placebo as reference treatment
#' #
#' nma <- netmeta(pw, common = FALSE, reference = "plac")
#' 
#' # Generate a heat plot (with abbreviated treatment labels)
#' #
#' heatplot(nma, nchar.trts = 4) 
#' 
#' @keywords hplot
#' 
#' @rdname heatplot
#' @export heatplot

heatplot <- function(x, ...)
  UseMethod("heatplot")

#' Forest plot for network meta-analysis
#' 
#' @description
#' Draws a forest plot in the active graphics window (using grid
#' graphics system).
#'
#' @aliases forest.netmeta plot.netmeta
#' 
#' @param x An object of class \code{netmeta}.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model
#'   (\code{"random"}) should be plotted. Can be abbreviated.
#' @param reference.group Reference treatment(s).
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa.
#' @param labels An optional vector with treatment labels.
#' @param equal.size A logical indicating whether all squares should
#'   be of equal size. Otherwise, the square size is proportional to
#'   the precision of estimates.
#' @param leftcols A character vector specifying columns to be plotted
#'   on the left side of the forest plot or a logical value (see
#'   Details).
#' @param leftlabs A character vector specifying labels for
#'   (additional) columns on left side of the forest plot (see
#'   Details).
#' @param rightcols A character vector specifying columns to be
#'   plotted on the right side of the forest plot or a logical value
#'   (see Details).
#' @param rightlabs A character vector specifying labels for
#'   (additional) columns on right side of the forest plot (see
#'   Details).
#' @param digits Minimal number of significant digits for treatment
#'   effects and confidence intervals, see \code{print.default}.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect, can be abbreviated; see
#'   \code{\link{netrank}}.
#' @param nsim Number of repetitions to calculate SUCRAs.
#' @param digits.prop Minimal number of significant digits for
#'   P-scores, SUCRAs and direct evidence proportions, see
#'   \code{\link{print.default}} and \code{\link{netrank}}.
#' @param smlab A label printed at top of figure. By default, text
#'   indicating either common or random effects model is printed.
#' @param sortvar An optional vector used to sort treatments (must be
#'   of same length as the total number of treatments).
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param print.tau2 A logical value indicating whether to print the
#'   value of the between-study variance \eqn{\tau^2}.
#' @param print.tau A logical value indicating whether to print
#'   \eqn{\tau}, the square root of the between-study variance
#'   \eqn{\tau^2}.
#' @param print.I2 A logical value indicating whether to print the
#'   value of the I-squared statistic.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param lab.NA A character string to label missing values.
#' @param add.data An optional data frame with additional columns to
#'   print in forest plot (see Details).
#' @param addrows.below.overall A numeric value indicating how many
#'   empty rows are printed between meta-analysis results and
#'   heterogeneity statistics.
#' @param drop.reference.group A logical indicating whether the
#'   reference group should be printed in the forest plot.
#' @param col.subgroup The colour to print information on subgroups.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the grouping variable should be printed in front of the group
#'   labels.
#' @param \dots Additional arguments for \code{\link[meta]{forest.meta}}
#'   function.
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window.
#' 
#' Argument \code{sortvar} can be either a numeric or character vector
#' with length of number of treatments. If \code{sortvar} is numeric
#' the \code{\link[base]{order}} function is utilised internally to
#' determine the order of values. If \code{sortvar} is character it
#' must be a permutation of the treatment names. It is also possible
#' to provide either \code{sortvar = Pscore}, \code{sortvar =
#' "Pscore"}, \code{sortvar = -Pscore}, or \code{sortvar = "-Pscore"}
#' in order to sort treatments according to the ranking generated by
#' \code{\link{netrank}} which is called internally. It is also
#' possible to use "SUCRA" instead of "Pscore". Similar expressions
#' are possible to sort by treatment comparisons (\code{sortvar = TE},
#' etc.), standard error (\code{sortvar = seTE}), number of studies
#' with direct treatment comparisons (\code{sortvar = k}), and direct
#' evidence proportion (\code{sortvar = prop.direct}, see also
#' \code{\link{netmeasures}}).
#'
#' The arguments \code{leftcols} and \code{rightcols} can be used to
#' specify columns which are plotted on the left and right side of the
#' forest plot, respectively. The following columns are available:
#' \tabular{ll}{
#' \bold{Name} \tab \bold{Definition} \cr
#' \code{"studlab"} \tab Treatments \cr
#' \code{"TE"} \tab Network estimates (either from common or random
#'   effects model) \cr
#' \code{"seTE"} \tab Corresponding standard errors \cr
#' \code{"Pscore"} \tab P-scores (see \code{\link{netrank}}) \cr
#' \code{"SUCRA"} \tab SUCRAs (see \code{\link{netrank}}) \cr
#' \code{"n.trts"} \tab Number of participants per treatment arm \cr
#' \code{"k"} \tab Number of studies in pairwise comparisons \cr
#' \code{"prop.direct"} \tab Direct evidence proportions (see
#'   \code{\link{netmeasures}}) \cr
#' \code{"effect"} \tab (Back-transformed) network estimates \cr
#' \code{"ci"} \tab Confidence intervals \cr
#' \code{"effect.ci"} \tab (Back-transformed) network estimates and
#'   confidence intervals
#' }
#'
#' As a sidenote, the rather odd column name \code{"studlab"} to
#' describe the treatment comparisons comes from internally calling
#' \code{\link[meta]{forest.meta}} which uses study labels as the essential
#' information.
#' 
#' Argument \code{add.data} can be used to add additional columns to
#' the forest plot. This argument must be a data frame with row names
#' equal to the treatment names in R object \code{x}, i.e.,
#' \code{x$trts}.
#' 
#' See help page of \code{\link[meta]{forest.meta}} for more information on
#' the generation of forest plots and additional arguments.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link[meta]{forest.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netmeta)
#' 
#' @method forest netmeta
#' @export

forest.netmeta <- function(x,
                           pooled = ifelse(x$random, "random", "common"),
                           reference.group = x$reference.group,
                           baseline.reference = x$baseline.reference,
                           labels = x$trts,
                           equal.size = gs("equal.size"),
                           leftcols = "studlab",
                           leftlabs,
                           rightcols = c("effect", "ci"),
                           rightlabs,
                           digits = gs("digits.forest"),
                           small.values = x$small.values,
                           nsim = gs("nsim"),
                           digits.prop = 2,
                           smlab = NULL,
                           sortvar = x$seq,
                           overall.hetstat = gs("overall.hetstat"),
                           print.tau2 = gs("forest.tau2"),
                           print.tau = gs("forest.tau"),
                           print.I2 = gs("forest.I2"),
                           backtransf = x$backtransf,
                           lab.NA = gs("lab.NA"),
                           add.data,
                           addrows.below.overall =
                             if (x$overall.hetstat) 2 else
                               gs("addrows.below.overall"),
                           drop.reference.group = gs("drop.reference.group"),
                           ##
                           col.subgroup = "black",
                           print.subgroup.name = FALSE,
                           ##
                           ...) {
  
  
  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netmeta")
  x <- updateversion(x)
  ##
  is.bin <- inherits(x, "netmetabin")
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  ##
  chklogical(equal.size)
  ##
  chknumeric(digits, min = 0, length = 1)
  ##
  small.values <- setsv(small.values)
  ##
  chknumeric(nsim, min = 1, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  ##
  chklogical(baseline.reference)
  ##
  trts <- x$trts
  ##
  if (!missing(labels)) {
    ##
    labels <- catch("labels", match.call(), x, sys.frame(sys.parent()))
    ##
    if (is.null(labels))
      stop("Argument 'labels' must be not NULL.")
    ##
    if (length(labels) != length(trts))
      stop("Length of argument 'labels' must be equal to number of treatments.")
    ##
    names(labels) <- trts
  }
  ##
  chklogical(drop.reference.group)
  chklogical(print.subgroup.name)
  #
  overall.hetstat <- replaceNULL(overall.hetstat, FALSE)
  print.tau2 <- replaceNULL(print.tau2, overall.hetstat)
  print.tau <- replaceNULL(print.tau, overall.hetstat)
  print.I2 <- replaceNULL(print.I2, overall.hetstat)
  #
  chklogical(print.tau2)
  chklogical(print.tau)
  chklogical(print.I2)
  #
  if (print.tau)
    print.tau2 <- FALSE
  #
  chklogical(backtransf)
  chkchar(lab.NA)
  ##
  stdlabs <- c("event.e", "n.e", "event.c", "n.c",
               "mean.e", "sd.e", "mean.c", "sd.c",
               "n", "time", "event",
               "TE", "seTE",
               "time.e", "time.c",
               "effect", "ci", "effect.ci",
               "w.common", "w.random")
  #
  missing.leftlabs <- missing(leftlabs)
  if (missing.leftlabs) {
    leftlabs <- leftcols
    leftlabs[leftcols %in% stdlabs] <- NA
  }
  #
  missing.rightlabs <- missing(rightlabs)
  if (missing.rightlabs) {
    rightlabs <- rightcols
    rightlabs[rightcols %in% stdlabs] <- NA
  }
  #
  for (i in names(list(...))) {
    if (!is.null(setchar(i, "weight.study", stop.at.error = FALSE)))
      stop("Argument 'weight.study' set internally.", call. = TRUE)
    if (!is.null(setchar(i, "prediction", stop.at.error = FALSE)))
      stop("For prediction intervals see example in help file of ",
           "forest.netsplit().", call. = TRUE)
  }
  
  
  ##
  ##
  ## (2) Extract results for common and random effects model and
  ##     calculate P-scores and SUCRAs if calcSUCRA == TRUE
  ##
  ##
  one.rg <- length(reference.group) == 1
  ##
  sortvar.c <- deparse(substitute(sortvar))
  sortvar.c <- gsub("\"", "", sortvar.c)
  ##
  calcPscore <-
    anyCol(rightcols, "Pscore") || anyCol(leftcols, "Pscore") ||
    any(matchVar(sortvar.c, "Pscore")) || any(matchVar(sortvar.c, "-Pscore"))
  ##
  calcSUCRA <-
    anyCol(rightcols, "SUCRA") || anyCol(leftcols, "SUCRA") ||
    any(matchVar(sortvar.c, "SUCRA")) || any(matchVar(sortvar.c, "-SUCRA"))
  ##
  if (one.rg && reference.group == "") {
    warning("First treatment used as reference as argument ",
            "'reference.group' is unspecified.",
            call. = FALSE)
    reference.group <- trts[1]
  }
  ##
  reference.group <- setref(reference.group, trts, length = 0)
  ##
  if (pooled == "common") {
    TE   <- x$TE.common
    seTE <- x$seTE.common
    ##
    prop.direct <- x$P.common
    ##
    if (calcPscore)
      Pscore <- netrank(x, small.values = small.values,
                        method = "P-score")$ranking.common
    if (calcSUCRA) {
      x$common <- TRUE
      x$random <- FALSE
      SUCRA <- netrank(x, small.values = small.values,
                       method = "SUCRA", nsim = nsim)$ranking.common
    }
    ##
    text.pooled <- "Common Effects Model"
  }
  ##
  if (pooled == "random") {
    TE   <- x$TE.random
    seTE <- x$seTE.random
    ##
    prop.direct <- x$P.random
    ##
    ##
    if (calcPscore)
      Pscore <- netrank(x, small.values = small.values,
                        method = "P-score")$ranking.random
    if (calcSUCRA) {
      x$common <- FALSE
      x$random <- TRUE
      SUCRA <- netrank(x, small.values = small.values,
                       method = "SUCRA", nsim = nsim)$ranking.random
    }
    ##
    text.pooled <- "Random Effects Model"
  }
  ##
  if (is.null(smlab)) {
    if (one.rg) {
      if (baseline.reference)
        smlab <- paste0("Comparison: other vs '",
                        reference.group, "'\n(",
                        text.pooled,
                        ")")
      else
        smlab <- paste0("Comparison: '",
                        reference.group,
                        "' vs other \n(",
                        text.pooled,
                        ")")
    }
    else
      smlab  <- text.pooled
  }
  #
  if (!missing.rightlabs && length(rightlabs) > length(rightcols))
    stop("Too many labels defined in argument 'rightlabs': ",
         length(rightlabs), " label", if (length(rightlabs) > 1) "s",
         " for ", length(rightcols), " column",
         if (length(rightcols) > 1) "s",
         ".",
         call. = FALSE)
  #
  rightcols <- setCol(rightcols, "Pscore")
  rightcols <- setCol(rightcols, "SUCRA")
  rightcols <- setCol(rightcols, "n.trts")
  rightcols <- setCol(rightcols, "k")
  rightcols <- setCol(rightcols, "prop.direct")
  #
  if (missing.rightlabs || (length(rightlabs) < length(rightcols))) {
    rightlabs <- setLab(rightlabs, rightcols, "Pscore", "P-score")
    rightlabs <- setLab(rightlabs, rightcols, "SUCRA", "SUCRA")
    rightlabs <- setLab(rightlabs, rightcols, "n.trts",
                        "Number of\nParticipants")
    rightlabs <- setLab(rightlabs, rightcols, "k", "Direct\nComparisons")
    rightlabs <- setLab(rightlabs, rightcols, "prop.direct",
                        "Direct Evidence\nProportion")
  }
  else if (length(rightlabs) == length(rightcols) && any(is.na(rightlabs))) {
    if (naLab(rightlabs[matchVar(rightcols, "Pscore")]))
      rightlabs <- setLab(rightlabs, rightcols, "Pscore", "P-score")
    #
    if (naLab(rightlabs[matchVar(rightcols, "SUCRA")]))
      rightlabs <- setLab(rightlabs, rightcols, "SUCRA", "SUCRA")
    
    if (naLab(rightlabs[matchVar(rightcols, "n.trts")]))
      rightlabs <- 
        setLab(rightlabs, rightcols, "n.trts", "Number of\nParticipants")
    #
    if (naLab(rightlabs[matchVar(rightcols, "k")]))
      rightlabs <- setLab(rightlabs, rightcols, "k", "Direct\nComparisons")
    #
    if (naLab(rightlabs[matchVar(rightcols, "prop.direct")]))
      rightlabs <- setLab(rightlabs, rightcols, "prop.direct",
                          "Direct Evidence\nProportion")
  }
  #
  if (!missing.leftlabs && length(leftlabs) > length(leftcols))
    stop("Too many labels defined in argument 'leftlabs': ",
         length(leftlabs), " label", if (length(leftlabs) > 1) "s",
         " for ", length(leftcols), " column",
         if (length(leftcols) > 1) "s",
         ".",
         call. = FALSE)
  #
  leftcols <- setCol(leftcols, "Pscore")
  leftcols <- setCol(leftcols, "SUCRA")
  leftcols <- setCol(leftcols, "n.trts")
  leftcols <- setCol(leftcols, "k")
  leftcols <- setCol(leftcols, "prop.direct")
  #
  if (missing.leftlabs || (length(leftlabs) < length(leftcols))) {
    if (length(reference.group) > 1)
      leftlabs[matchVar(leftcols, "studlab")] <- "Comparison"
    else
      leftlabs[matchVar(leftcols, "studlab")] <- "Treatment"
    #
    leftlabs <- setLab(leftlabs, leftcols, "Pscore", "P-score")
    leftlabs <- setLab(leftlabs, leftcols, "SUCRA", "SUCRA")
    leftlabs <- setLab(leftlabs, leftcols, "n.trts", "Number of\nParticipants")
    leftlabs <- setLab(leftlabs, leftcols, "k", "Direct\nComparisons")
    leftlabs <- setLab(leftlabs, leftcols, "prop.direct",
                       "Direct Evidence\nProportion")
  }
  else if (length(leftlabs) == length(leftcols) && any(is.na(leftlabs))) {
    if (is.na(leftlabs[matchVar(leftcols, "studlab")])) {
      if (length(reference.group) > 1)
        leftlabs[matchVar(leftcols, "studlab")] <- "Comparison"
      else
        leftlabs[matchVar(leftcols, "studlab")] <- "Treatment"
    }
    #
    if (naLab(leftlabs[matchVar(leftcols, "Pscore")]))
      leftlabs <- setLab(leftlabs, leftcols, "Pscore", "P-score")
    #
    if (naLab(leftlabs[matchVar(leftcols, "SUCRA")]))
      leftlabs <- setLab(leftlabs, leftcols, "SUCRA", "SUCRA")
    
    if (naLab(leftlabs[matchVar(leftcols, "n.trts")]))
      leftlabs <- 
        setLab(leftlabs, leftcols, "n.trts", "Number of\nParticipants")
    #
    if (naLab(leftlabs[matchVar(leftcols, "k")]))
      leftlabs <- setLab(leftlabs, leftcols, "k", "Direct\nComparisons")
    #
    if (naLab(leftlabs[matchVar(leftcols, "prop.direct")]))
      leftlabs <- setLab(leftlabs, leftcols, "prop.direct",
                         "Direct Evidence\nProportion")
  }
  
  
  ##
  ##
  ## (3) Extract comparisons with reference group
  ##
  ##
  dat <- data.frame(comparison = character(0),
                    treat = character(0),
                    TE = numeric(0), seTE = numeric(0),
                    Pscore = numeric(0),
                    SUCRA = numeric(0),
                    k = numeric(0),
                    prop.direct = numeric(0),
                    stringsAsFactors = FALSE)
  ##
  for (i in seq_along(reference.group)) {
    rg.i <- reference.group[i]
    ##
    if (baseline.reference)
      dat.i <- data.frame(comparison = rg.i,
                          treat = colnames(TE),
                          labels = labels,
                          TE = TE[, colnames(TE) == rg.i],
                          seTE = seTE[, colnames(seTE) == rg.i],
                          Pscore = if (calcPscore) Pscore else NA,
                          SUCRA = if (calcSUCRA) SUCRA else NA,
                          k = x$A.matrix[, colnames(TE) == rg.i],
                          prop.direct =
                            if (is.bin) prop.direct
                            else prop.direct[, colnames(TE) == rg.i],
                          stringsAsFactors = FALSE)
    else
      dat.i <- data.frame(comparison = rg.i,
                          treat = rownames(TE),
                          labels = labels,
                          TE = TE[rownames(TE) == rg.i, ],
                          seTE = seTE[rownames(seTE) == rg.i, ],
                          Pscore = if (calcPscore) Pscore else NA,
                          SUCRA = if (calcSUCRA) SUCRA else NA,
                          k = x$A.matrix[rownames(TE) == rg.i, ],
                          prop.direct =
                            if (is.bin) prop.direct
                            else prop.direct[rownames(TE) == rg.i, ],
                          stringsAsFactors = FALSE)
    ##
    if (!is.null(x$n.trts))
      dat.i$n.trts <- x$n.trts
    ##
    if (!missing(add.data)) {
      if (!is.data.frame(add.data))
        stop("Argument 'add.data' must be a data frame.",
             call. = FALSE)
      if (nrow(add.data) != length(trts))
        stop("Dataset 'add.data' must have ", nrow(dat.i),
             " rows (corresponding to number of treatments)",
             call. = FALSE)
      if (any(rownames(add.data) != trts))
        stop("Dataset 'add.data' must have the following row names:\n",
             paste(paste0("'", trts, "'"), collapse = " - "),
             call. = FALSE)
      ##
      dat.i <- cbind(dat.i, add.data)
    }
    ##
    ## Sort dataset according to argument sortvar
    ##
    if (any(matchVar(sortvar.c, "Pscore")))
      sortvar <- Pscore
    else if (any(matchVar(sortvar.c, "-Pscore")))
      sortvar <- -Pscore
    else if (any(matchVar(sortvar.c, "SUCRA")))
      sortvar <- SUCRA
    else if (any(matchVar(sortvar.c, "-SUCRA")))
      sortvar <- -SUCRA
    else if (any(matchVar(sortvar.c, "TE")))
      sortvar <- dat.i$TE
    else if (any(matchVar(sortvar.c, "-TE")))
      sortvar <- -dat.i$TE
    else if (any(matchVar(sortvar.c, "seTE")))
      sortvar <- dat.i$seTE
    else if (any(matchVar(sortvar.c, "-seTE")))
      sortvar <- -dat.i$seTE
    else if (any(matchVar(sortvar.c, "k")))
      sortvar <- dat.i$k
    else if (any(matchVar(sortvar.c, "-k")))
      sortvar <- -dat.i$k
    else if (any(matchVar(sortvar.c, "n.trts")))
      sortvar <- dat.i$n.trts
    else if (any(matchVar(sortvar.c, "-n.trts")))
      sortvar <- -dat.i$n.trts
    else if (any(matchVar(sortvar.c, "prop.direct")))
      sortvar <- dat.i$prop.direct
    else if (any(matchVar(sortvar.c, "-prop.direct")))
      sortvar <- -dat.i$prop.direct
    ##
    if (!is.null(sortvar)) {
      if (is.character(sortvar))
        sort <- setseq(sortvar, trts)
      else
        sort <- order(sortvar)
      ##
      dat.i <- dat.i[sort, ]
    }
    ##
    if (drop.reference.group)
      dat.i <- subset(dat.i, treat != rg.i)
    ##
    if (baseline.reference)
      dat.i$comparison <- paste0("Other vs '", dat.i$comparison, "'")
    else
      dat.i$comparison <- paste0("'", dat.i$comparison, "' vs other")
    ##
    dat <- rbind(dat, dat.i)
  }
  ##
  dat.out <- dat
  ##
  if ("Pscore" %in% names(dat))
    dat$Pscore <- formatN(dat$Pscore, digits = digits.prop,
                          text.NA = lab.NA)
  ##
  if ("SUCRA" %in% names(dat))
    dat$SUCRA <- formatN(dat$SUCRA, digits = digits.prop,
                         text.NA = lab.NA)
  ##
  if ("prop.direct" %in% names(dat))
    dat$prop.direct <- formatN(dat$prop.direct,
                               digits = digits.prop, text.NA = lab.NA)
  ##
  rm(TE)
  rm(seTE)
  
  
  ##
  ##
  ## (5) Generate forest plot
  ##
  ##
  treat <- dat$treat
  ##
  if (one.rg)
    m1 <- suppressWarnings(metagen(TE, seTE, data = dat,
                                   sm = x$sm,
                                   studlab = labels, backtransf = backtransf,
                                   method.tau = "DL", method.tau.ci = "",
                                   warn = FALSE))
  else
    m1 <- suppressWarnings(metagen(TE, seTE, data = dat,
                                   subgroup = dat$comparison,
                                   sm = x$sm,
                                   studlab = labels, backtransf = backtransf,
                                   method.tau = "DL", method.tau.ci = "",
                                   warn = FALSE))
  #
  m1 <- setHet(m1, x)
  m1$.text.details.methods <-
    textmeth(x, pooled == "random", print.tau2, print.tau,
             "", "", gs("digits.tau2"), gs("digits.tau"),
             print.I2, gs("text.I2"),
             big.mark = gs("big.mark"), forest = TRUE)
  #
  forest(m1,
         digits = digits,
         #
         overall = FALSE, common = FALSE, random = FALSE,
         overall.hetstat = overall.hetstat,
         print.tau2 = print.tau2, print.tau = print.tau,
         print.I2 = print.I2,
         test.subgroup = FALSE,
         #
         leftcols = leftcols,
         leftlabs = leftlabs,
         rightcols = rightcols,
         rightlabs = rightlabs,
         #
         smlab = smlab, lab.NA = lab.NA,
         ##
         col.subgroup = col.subgroup,
         print.subgroup.name = print.subgroup.name,
         ##
         weight.study = if (equal.size) "same" else pooled,
         #
         addrows.below.overall = addrows.below.overall,
         #
         ...)
  
  rownames(dat.out) <- seq_len(nrow(dat.out))
  #
  attr(dat.out, "pooled") <- pooled
  attr(dat.out, "small.values") <- small.values
  #
  invisible(dat.out)
}


#' @rdname forest.netmeta
#' @method plot netmeta
#' @export

plot.netmeta <- function(x, ...)
  forest(x, ...)

#' Network meta-analysis of treatments for depression
#' 
#' @description
#' Network meta-analysis of nine classes of antidepressants including
#' placebo for the primary care setting; partly shown in Linde et
#' al. (2015), supplementary Table 2.
#' 
#' @name Linde2015
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{id}}\tab Study ID \cr
#' \bold{\emph{author}}\tab First author \cr
#' \bold{\emph{year}}\tab Publication year \cr
#' \bold{\emph{treatment1}}\tab First treatment \cr
#' \bold{\emph{treatment2}}\tab Second treatment \cr
#' \bold{\emph{treatment3}}\tab Third treatment \cr
#' \bold{\emph{n1}}\tab Number of patients receiving first treatment
#'   \cr
#' \bold{\emph{resp1}}\tab Number of early responder (treatment 1) \cr
#' \bold{\emph{remi1}}\tab Number of early remissions (treatment 1)
#'   \cr
#' \bold{\emph{loss1}}\tab Number of patients loss to follow-up
#'   (treatment 1) \cr
#' \bold{\emph{loss.ae1}}\tab Number of patients loss to follow-up due
#'   to adverse events (treatment 1) \cr
#' \bold{\emph{ae1}}\tab Number of patients with adverse events
#'   (treatment 1) \cr
#' \bold{\emph{n2}}\tab Number of patients receiving second treatment
#'   \cr
#' \bold{\emph{resp2}}\tab Number of early responder (treatment 2) \cr
#' \bold{\emph{remi2}}\tab Number of early remissions (treatment 2)
#'   \cr
#' \bold{\emph{loss2}}\tab Number of patients loss to follow-up
#'   (treatment 2) \cr
#' \bold{\emph{loss.ae2}}\tab Number of patients loss to follow-up due
#'   to adverse events (treatment 2) \cr
#' \bold{\emph{ae2}}\tab Number of patients with adverse events
#'   (treatment 2) \cr
#' \bold{\emph{n3}}\tab Number of patients receiving third treatment
#'   \cr
#' \bold{\emph{resp3}}\tab Number of early responder (treatment 3) \cr
#' \bold{\emph{remi3}}\tab Number of early remissions (treatment 3)
#'   \cr
#' \bold{\emph{loss3}}\tab Number of patients loss to follow-up
#'   (treatment 3) \cr
#' \bold{\emph{loss.ae3}}\tab Number of patients loss to follow-up due
#'   to adverse events (treatment 3) \cr
#' \bold{\emph{ae3}}\tab Number of patients with adverse events
#'   (treatment 3) 
#' }
#' 
#' @note
#' The dataset Linde2015 is identical to dataset
#' \code{\link[metabook]{Linde2015}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[metabook]{Dong2013}},
#'   \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
#'   \code{\link{netmeta}}, \code{\link{netposet}}
#' 
#' @source
#' Linde K, Kriston L, Rücker G, et al. (2015):
#' Efficacy and acceptability of pharmacological treatments for
#' depressive disorders in primary care: Systematic review and network
#' meta-analysis.
#' \emph{Annals of Family Medicine},
#' \bold{13}, 69--79
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Linde2015)
#' 
#' # Example using pairwise(), netmeta(), netrank() and netposet():
#' # example(Linde2015, package = "metabook")

NULL

nettable_internal <- function(x, method,
                              ##
                              upper, reference.group, baseline.reference,
                              order, tol.direct, backtransf,
                              ##
                              digits, digits.I2, digits.pval,
                              ##
                              scientific.pval, zero.pval, JAMA.pval,
                              big.mark, text.NA,
                              bracket, separator, lower.blank, upper.blank,
                              ##
                              writexl,
                              ##
                              warn, verbose) {
  
  
  ##
  ##
  ## (1) Some checks and assignments
  ##
  ##
  is.bin <- inherits(x, "netmetabin")
  ##
  if (is.null(method))
    if (is.bin)
      method <- "SIDDE"
    else
      method <- "Back-calculation"
  ##
  if (is.null(reference.group))
    reference.group <- x$reference.group
  else
    reference.group <- setref(reference.group, x$trts)
  ##
  if (is.null(baseline.reference))
    baseline.reference <- x$baseline.reference
  ##
  if (is.null(backtransf))
    backtransf <- x$backtransf
  ##
  if (!is.null(order)) {
    order <- setseq(order, x$trts, equal.length = FALSE)
    baseline.reference <- FALSE
    reference.group <- ""
  }
  
  
  ##
  ##
  ## (2) Create dat.trts
  ##
  ##
  dat.trts <- comptrts(x, upper, reference.group, baseline.reference, order)
  
  
  ##
  ##
  ## (4) Change order of prop.direct.common and prop.direct.random
  ##
  ##
  if (!(is.bin & method == "SIDDE")) {
    prop.common <- sortprop(x$prop.direct.common, dat.trts, x$sep.trts)
    prop.random <- sortprop(x$prop.direct.random, dat.trts, x$sep.trts)
  }
  else
    prop.common <- prop.random <- NULL
  
  
  ##
  ##
  ## (5) Calculate / extract indirect estimates
  ##
  ##
  x.direct.indirect <- x
  ##
  if (method == "SIDDE") {
    ind <- sidde(x.direct.indirect, x$sep.trts, verbose, warn, FALSE)
    ##    
    x.direct.indirect$TE.indirect.common <- ind$TE.indirect.common
    x.direct.indirect$seTE.indirect.common <- ind$seTE.indirect.common
    ##
    if (!is.bin) {
      x.direct.indirect$TE.indirect.random <- ind$TE.indirect.random
      x.direct.indirect$seTE.indirect.random <- ind$seTE.indirect.random
    } 
  }
  ##
  direct.indirect <- direct.indirect(x.direct.indirect, tol.direct)
  
  
  ##
  ##
  ## (6) Transform matrices to data frames
  ##
  ##
  common <- mat2dat.table(direct.indirect, "common", dat.trts,
                          backtransf,
                          digits, digits.I2, digits.pval,
                          scientific.pval, zero.pval, JAMA.pval,
                          big.mark, text.NA,
                          writexl)
  ##
  random <- mat2dat.table(direct.indirect, "random", dat.trts,
                          backtransf,
                          digits, digits.I2, digits.pval,
                          scientific.pval, zero.pval, JAMA.pval,
                          big.mark, text.NA,
                          writexl)
  
  
  res <- list(common = common, random = random)
  ##
  res
}

#' Network graph for objects of class netimpact
#' 
#' @description
#' This function generates a graph of the evidence network.
#' 
#' @param x An object of class \code{netimpact}.
#' @param col.ignore A character string indicating color for
#'   comparisons removed from network, either \code{"transparent"} or
#'   any color defined in \code{\link[grDevices]{colours}}.
#' @param number.of.studies A logical indicating whether number of
#'   studies should be added to network graph.
#' @param main Main title.
#' @param sub Subtitle.
#' @param multiarm A logical indicating whether multi-arm studies
#'   should be marked in plot.
#' @param col.multiarm Either a function from R package
#'   \bold{colorspace} or \bold{grDevice} to define colors for
#'   multi-arm studies or a character vector with colors to highlight
#'   multi-arm studies.
#' @param alpha.transparency The alpha transparency of colors used to
#'   highlight multi-arm studies (0 means transparent and 1 means
#'   opaque).
#' @param col.ignore.multiarm A character string indicating color to
#'   mark multi-arm studies removed from network, either
#'   \code{"transparent"} or any color defined in
#'   \code{\link[grDevices]{colours}}.
#' @param col A single color (or vector of colors) for lines
#'   connecting treatments if argument \code{plastic =
#'   FALSE}. Length of the vector must be equal to the number of
#'   lines / edges.
#' @param plastic A logical indicating whether the appearance of the
#'   comparisons should be in '3D look'.
#' @param \dots Additional arguments passed on to
#'   \code{\link{netgraph.netmeta}}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netimpact}}, \code{\link{netgraph.netmeta}},
#'   \code{\link[metabook]{Franchini2012}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netimpact)
#' 
#' @method netgraph netimpact
#' @export

netgraph.netimpact <- function(x,
                               col.ignore = "red",
                               number.of.studies = TRUE,
                               main, sub,
                               multiarm = gs("multiarm"),
                               col.multiarm = NULL,
                               alpha.transparency = 0.5,
                               col.ignore.multiarm =  "transparent",
                               col = "black",
                               plastic = gs("plastic"),
                               ...) {
  
  
  chkclass(x, "netimpact")
  ##
  x <- updateversion(x)
  
  
  col.ignore <- setchar(col.ignore,
                               c("transparent", colours()),
                               text = paste0("should be any color ",
                                             "defined in colours()"))
  ##
  col.ignore.multiarm <- setchar(col.ignore.multiarm,
                                        c("transparent", colours()),
                                        text = paste0("should be any color ",
                                                      "defined in colours()"))
  
  
  studlab <- x$x$studlab
  treat1  <- x$x$treat1
  treat2  <- x$x$treat2
  TE      <- x$x$TE
  seTE    <- x$x$seTE
  ##
  sep.trts <- x$x$sep.trts
  comparison <- paste(treat1, sep = sep.trts, treat2)
  
  
  comparisons <- names(x$x$prop.direct.common)
  studies <- x$x$studies
  narms <- x$x$narms
  ##  
  impact <- matrix(NA, ncol = x$x$k, nrow = length(comparisons))
  ##
  rownames(impact) <- comparisons
  colnames(impact) <- studies
  ##
  ## Run network meta-analyses "excluding" one study at a time
  ##
  if (multiarm) {
    mc <- multicols(studies, narms,
                    missing(col.multiarm),
                    col.multiarm, alpha.transparency)
    col.polygon <- mc$col
    multiarm.studies <- mc$studlab
  }
  else
    col.polygon <- col.ignore.multiarm
  ##
  res <- list()
  ##
  for (i in studies) {
    ##
    seTE.i <- seTE
    seTE.i[studlab == i] <- x$seTE.ignore
    ##
    ignore.i <- x$ignored.comparisons[[i]]
    col.ignore.i <- rep(col.ignore, length(ignore.i))
    ##
    mat <- matrix(unlist(strsplit(ignore.i, split = sep.trts)),
                  ncol = 2, byrow = TRUE)
    treat1.i <- mat[, 1]
    treat2.i <- mat[, 2]
    ##
    col.polygon.i <- col.polygon
    ##
    if (multiarm)
      col.polygon.i[multiarm.studies == i] <- col.ignore.multiarm
    ##
    net.i <- x$nets[[i]]
    ##
    for (j in seq_along(treat1.i)) {
      ##
      net.i$A.matrix[treat1.i[j], treat2.i[j]] <-
        net.i$A.matrix[treat1.i[j], treat2.i[j]] - 1
      net.i$A.matrix[treat2.i[j], treat1.i[j]] <-
        net.i$A.matrix[treat2.i[j], treat1.i[j]] - 1
      ##
      if (net.i$A.matrix[treat1.i[j], treat2.i[j]] == 0)
        col.ignore.i[j] <- "transparent"
    }
    ##
    n.i <- netgraph(net.i,
                    highlight = ignore.i, col.highlight = col.ignore.i,
                    multiarm = multiarm, col.multiarm = col.polygon.i,
                    alpha.transparency = alpha.transparency,
                    number.of.studies = number.of.studies,
                    plastic = plastic,
                    col = col, ...)
    ##
    res[[i]] <- list(nodes = n.i$nodes, edges = n.i$edges)
    ##
    if (!missing(main)) {
      if (!(is.logical(main) && length(main) == 1 && !main))
        title(main = main)
    }
    else
      title(main = paste0("Study removed: ", i))
    ##
    if (!missing(sub)) {
      if (!(is.logical(sub) && length(sub) == 1 && !sub))
        title(sub = sub)
    }
    else
      title(sub = paste0("Comparison",
                         if (length(ignore.i) > 1) "s",
                         ": ",
                         paste(paste0("'", ignore.i, "'"), collapse = ", ")))
  }
  
  
  invisible(res)
}

#' Scatter plot to evaluate local inconsistency in network meta-analysis
#' 
#' @description
#' Scatter plot to show the difference between direct and indirect evidence in
#' network meta-analysis to assess local inconsistency.
#'
#' @aliases plot.netsplit
#' 
#' @param x An object of class \code{netsplit}.
#' @param type A character string indicating which figure type is to be used.
#'   Either \code{"Bland-Altman"} or \code{"Wilson"}, can be abbreviated.
#'   See Details.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param subset An optional logical vector specifying a subset of
#'   comparisons to consider (must be of same length as the total number of
#'   comparisons).
#' @param only.reference A logical indicating whether only comparisons
#'   with the reference group should be considered.
#' @param level The level used to calculate confidence intervals for local
#'   local inconsistency.
#' @param vertical A logical indicating whether confidence intervals should be
#'   plotted vertically or horizontally; only considered if argument
#'   \code{type = "Wilson"}. The interpretation is the same.
#' @param size.points A single numeric defining the point size. If \code{NULL},
#'   point sizes are proportional to the proportion of indirect evidence.
#' @param points.only.inc A logical indicating whether (larger) points should
#'   be only shown for comparisons with local inconsistency.
#' @param col Colour of points and confidence intervals.
#' @param col.inc Colour of points and confidence intervals for comparisons
#'   with local inconsistency.
#' @param labels A logical value indicating whether to show comparisons labels,
#'   or an expression or character vector specifying the labels to use.
#' @param labels.only.inc A logical indicating whether labels are only shown
#'   for comparisons with local inconsistency.
#' @param size.labels A single numeric defining the size of comparison labels.
#' @param col.labels Colour of comparison labels.
#' @param col.labels.inc Colour of labels for comparisons with local
#'   inconsistency.
#' @param max_overlap An integer specifying the maximum number of overlapping
#'   labels allowed in the scatter plot.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names in comparisons.
#' @param text.pooled A character string used to describe the statistical model.
#' @param main Main title.
#' @param xlab A label for the x-axis.
#' @param ylab A label for the y-axis.
#' @param xlim The x limits (min, max) of the plot.
#' @param ylim The y limits (min, max) of the plot.
#' @param tag The text for the tag label (see \code{\link[ggplot2]{labs}}).
#' @param \dots Additional arguments (ignored).
#' 
#' @details
#' A scatter plot is drawn in the active graphics window.
#' 
#' By default (argument \code{type = "Bland-Altman"}), a variant of the
#' Bland-Altman plot is shown (Bland & Altman, 1995). In this case, the
#' network estimates are shown on the horizontal axis and the difference between
#' direct and indirect effect estimates on the vertical axis. The confidence
#' intervals are those from the differences.
#' 
#' If argument \code{type = "Wilson"}, a scatter plot with direct effect
#' estimates on the horizontal and indirect estimates on the vertical axis
#' is produced (Wilson et al., 2026). The confidence intervals are
#' calculated from the indirect (argument \code{vertical = TRUE}) or direct
#' effect estimates and the standard error of the difference between indirect
#' and direct effect estimates.
#' 
#' For both plot types, the dashed green line corresponds to comparisons with
#' equal direct and indirect effect estimates.
#'
#' @return
#' An object of class \code{"plot.netsplit"}. The object is a list containing
#' the following components:
#' \item{plot}{A ggplot2 object.}
#' \item{data}{Data set used to create the plot.}
#' 
#' @author Federico Bonofiglio \email{bonostat@@gmx.de},
#'   Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest.netsplit}}
#' 
#' @references
#' Bland JM, Altman DG (1995):
#' Comparing methods of measurement: Why plotting difference against standard
#' method is misleading.
#' \emph{The Lancet},
#' \bold{346}, 1085--87
#' 
#' Wilson H, Schönstein A, Robson S, Bonofiglio F (2026):
#' A novel approach for visualizing local consistency in
#' network meta-analysis.
#' \emph{Research Synthesis Methods},
#' 1--15
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netsplit)
#' 
#' @method plot netsplit
#' @export

plot.netsplit <- function(x,
                          type = "Bland-Altman",
                          pooled = ifelse(x$x$random, "random", "common"),
                          subset = NULL,
                          only.reference = x$only.reference,
                          level = x$level.ma,
                          #
                          vertical = TRUE,
                          #
                          size.points = NULL,
                          points.only.inc = FALSE,
                          col = "gray50",
                          col.inc = "red",
                          #
                          labels = TRUE,
                          labels.only.inc = FALSE,
                          size.labels = 5,
                          col.labels = col,
                          col.labels.inc = col.inc,
                          max_overlap = 10,
                          #
                          nchar.trts = x$nchar.trts,
                          #
                          text.pooled,
                          main,
                          xlab = NULL,
                          ylab = NULL,
                          xlim = NULL,
                          ylim = NULL,
                          tag = NULL,
                          ...) {
  
  #
  #
  # (1) Check and set arguments
  #
  #
  
  chkclass(x, "netsplit")
  x <- updateversion(x)
  #
  type <- setchar(type, c("Wilson", "Bland-Altman"))
  #
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  #
  missing.only.reference <- missing(only.reference)
  if (!missing.only.reference)
    chklogical(only.reference)
  #
  if (missing(text.pooled)) {
    if (pooled == "common")
      text.pooled <- gsub(" effect ", " effects ", gs("text.common"))
    else
      text.pooled <- gs("text.random")
  }
  else if (!is.null(text.pooled))
    chkchar(text.pooled, length = 1)
  #
  chklevel(level, length = 1)
  chklogical(vertical)
  #
  same.size <- !is.null(size.points)
  if (same.size)
    chknumeric(size.points, min = 0, zero = TRUE, length = 1)
  #
  chklogical(points.only.inc)
  #
  chkcolor(col, length = 1)
  chkcolor(col.inc, length = 1)
  #
  chklogical(labels)
  chknumeric(size.labels, min = 0, zero = TRUE, length = 1)
  chkcolor(col.labels, length = 1)
  chkcolor(col.labels.inc, length = 1)
  chklogical(labels.only.inc)
  chknumeric(max_overlap, min = 0, zero = TRUE, length = 1)
  #
  nchar.trts <- replaceNULL(nchar.trts, 666)
  chknumeric(nchar.trts, length = 1)
  #
  if (missing(main)) {
    if (type == "Wilson")
      main <- paste0("Direct against indirect estimates with ",
                     round(100 * level, 1),
                     "% confidence intervals for their differences")
    else if (type == "Bland-Altman")
      main <- paste0("Bland-Altman plot with ",
                     round(100 * level, 1),
                     "% confidence intervals for differences")
  }
  else if (!is.null(main))
    chkchar(main, length = 1)
  #
  if (!is.null(xlab))
    chkchar(xlab, length = 1)
  if (!is.null(ylab))
    chkchar(ylab, length = 1)
  #
  if (!is.null(xlim))
    chknumeric(xlim, length = 2)
  if (!is.null(ylim))
    chknumeric(ylim, length = 2)
  if (!is.null(tag))
    chkchar(tag, length = 1)
  #
  calculate.ci <- level != x$level.ma
  #
  # Get rid of warning 'Undefined global functions or variables'
  #
  comparison <- TE <- prop.direct<- prop.indirect <-
    direct <- indirect <- mixed <- lower <- upper <-
    pval <- label_width <- Value <- Bound <- signif_label <- NULL
  
  
  #
  #
  # (2) Extract results for common or random effects model
  #
  #
  
  if (pooled == "common") {
    dat.mixed <- x$common
    #
    dat.direct <- x$direct.common
    #
    dat.indirect <- x$indirect.common %>% mutate(prop.direct = x$prop.common)
    #
    dat.diff <- x$compare.common
  }
  else {
    dat.mixed <- x$random
    #
    dat.direct <- x$direct.random
    #
    dat.indirect <- x$indirect.random %>% mutate(prop.direct = x$prop.random)
    dat.indirect$prop.indirect <- 1 - x$prop.random
    #
    dat.diff <- x$compare.random
  }
  #
  dat.indirect %<>% mutate(prop.indirect = 1 - prop.direct)
  #
  dat.mixed %<>%
    select(comparison, TE) %>%
    rename(mixed = TE)
  #
  dat.direct %<>%
    select(comparison, TE) %>%
    rename(direct = TE)
  #
  dat.indirect %<>%
    select(comparison, TE, prop.direct, prop.indirect) %>%
    rename(indirect = TE)
  #
  if (calculate.ci) {
    ci.diff <- ci(dat.diff$TE, dat.diff$seTE, level = level)
    dat.diff %<>% mutate(lower = ci.diff$lower, upper = ci.diff$upper)
  }
  #
  dat.diff %<>%
    select(comparison, TE, lower, upper, p) %>%
    rename(diff = TE, pval = p)
  #
  dat <- inner_join(dat.mixed, dat.direct, by = "comparison")
  dat <- inner_join(dat, dat.indirect, by = "comparison")
  dat <- inner_join(dat, dat.diff, by = "comparison")
  #
  if (type == "Bland-Altman") {
    tmp.lower <- dat$lower
    #
    dat %<>% mutate(direct = mixed, indirect = diff)
    #
    vertical <- TRUE
  }
  else {
    if (vertical) {
      dat %<>%
        mutate(lower = direct - lower, upper = direct - upper)
    }
    else {
      dat %<>%
        mutate(lower = indirect + lower,
               upper = indirect + upper)
    }
  }
  #
  dat %<>%
    mutate(signif =
             factor(if_else(pval < 1 - level, "Y", "N"), levels = c("N", "Y")),
           signif_label =
             factor(if_else(pval < 1 - level, "Yl", "Nl"),
                    levels = c("Nl", "Yl")))
    
  
  #
  #
  # (3) Select treatment comparisons to show in scatter plot
  #
  #
  
  if (!missing(subset)) {
    #
    # Catch subset from data:
    #
    mc <- match.call()
    sfsp <- sys.frame(sys.parent())
    #
    error <- try(subset.x <- catch("subset", mc, dat, sfsp), silent = TRUE)
    if (!any(class(error) == "try-error"))
      subset <- subset.x
    #
    if (!is.null(subset)) {
      chklength(subset, length(dat$comparison),
                text = paste0("Argument 'subset' must be of length ",
                              length(dat$comparison), "."))
      if (!is.logical(subset))
        stop("Argument 'subset' must be a logical vector.")
    }
    #
    dat <- dat %>% filter(subset)
  }
  #
  if (only.reference) {
    if (x$reference.group == "") {
      warning("First treatment used as reference as argument ",
              "'reference.group' was unspecified in netsplit().",
              call. = FALSE)
      x$reference.group <- compsplit(x$comparison, x$sep.trts)[[1]][1]
    }
    #
    sel <- apply(!is.na(sapply(compsplit(dat$comparison, x$sep.trts),
                               match, x$reference.group)), 2, sum) >= 1
    #
    dat <- dat %>% filter(sel)
  }
  #
  # Keep only comparisons for which we have both direct and indirect.
  #
  dat %<>% filter(!is.na(direct), !is.na(indirect))
  #
  any_sign <- any(dat$pval < 1 - level)
  #
  trts <- unique(sort(unlist(compsplit(dat$comparison, split = x$sep.trts))))
  trts.abbr <- treats(trts, nchar.trts = nchar.trts)
  #
  for (i in seq_len(nrow(dat))) {
    trts.i <- unlist(compsplit(dat$comparison[i], split = x$sep.trts))
    trts.i <- factor(trts.i, levels = trts, labels = trts.abbr)
    dat$comparison[i] <- paste(trts.i, collapse = x$sep.trts)
  }
  #
  # Show (larger) points only for comparisons with local inconsistency
  #
  if (same.size)
    size.points <- rep(size.points, nrow(dat))
  #
  all.inc <- all(dat$signif == "Y", na.rm = TRUE)
  all.noninc <- all(dat$signif == "N", na.rm = TRUE)
  #
  if (all.inc | all.noninc)
    points.only.inc <- FALSE
  #
  if (points.only.inc) {
    if (same.size)
      size.points[dat$signif == "N"] <- 0
    else
      #
      dat %<>% mutate(prop.indirect = if_else(signif == "Y", prop.indirect, 0))
  }
  #
  any.inc <- any(dat$signif == "Y", na.rm = TRUE)
  
  
  dat.lines <- dat %>%
    select(comparison, direct, indirect, lower, upper, signif, signif_label) %>%
    pivot_longer(cols = c(lower, upper),
                 names_to = "Bound", values_to = "Value") %>%
    mutate(label_width = str_length(comparison))
  
  
  #
  #
  # (4) Create scatter plot
  #
  #
  
  xylim <- max(abs(dat.lines$Value))
  axis.color <- "gray40"
  #
  if (vertical) {
    maps <- aes(x = direct, y = Value, group = comparison, color = signif)
    mapslab <- aes(x = direct, y = Value, label = comparison, color = signif_label)
  }
  else {
    maps <- aes(x = Value, y = indirect, group = comparison, color = signif)
    mapslab <- aes(x = Value, y = indirect, label = comparison, color = signif_label)
  }
  #
  if (same.size)
    mapspoint <- aes(x = direct, y = indirect, color = signif,
                     size = size.points)
  else
    mapspoint <- aes(x = direct, y = indirect, color = signif,
                     size = 100 * prop.indirect)
  #
  p <- ggplot()
  #
  if (type == "Wilson") {
    if (is.null(xlab)){
      xlab <- "Direct effect estimate"
      if (x$sm != "")
        xlab <- paste0(xlab, " (", tolower(meta:::xlab_meta(x$sm, FALSE)), ")")
    }
    #
    if (is.null(ylab)){
      ylab <- "Indirect effect estimate"
      if (x$sm != "")
        ylab <- paste0(ylab, " (", tolower(meta:::xlab_meta(x$sm, FALSE)), ")")
    }
    #
    p <- p +
      geom_abline(intercept = 0, slope = 1,
                  linetype = "dashed", color = "green4") +
      #
      geom_hline(yintercept = 0) + geom_vline(xintercept = 0)
  }
  else {
    if (is.null(xlab)){
      xlab <- "Network estimate"
      if (x$sm != "")
        xlab <- paste0(xlab, " (", tolower(meta:::xlab_meta(x$sm, FALSE)), ")")
    }
    #
    if (is.null(ylab)){
      ylab <- "Difference between direct and indirect evidence"
      if (x$sm != "")
        ylab <- paste0(ylab, "\n(", tolower(meta:::xlab_meta(x$sm, FALSE)), ")")
    }
    #
    p <- p +
      geom_vline(xintercept = 0) +
      geom_abline(intercept = 0, slope = 0,
                  linetype = "dashed", color = "green4")
  }
  #
  p <- p +
    geom_line(data = dat.lines, mapping = maps) +
    geom_point(data = dat, mapping = mapspoint) +
    # Significance colors
    # If only significant data shown then stick to 'col.inc'
    scale_color_manual(
      values =
        c("N" = if (all(dat$pval < 1 - level)) col.inc else col,
          "Y" = col.inc,
          "Nl" = if (all(dat$pval < 1 - level)) col.labels.inc else col.labels,
          "Yl" = col.labels.inc)) +
    # Define theme
    theme(
      axis.title.x = element_text(margin = margin(t = 15)),
      axis.title.y = element_text(margin = margin(r = 10)),
      plot.margin = margin(20, 40, 20, 20),
      # panel.background = element_rect(fill = alpha("white", 1)),
      axis.ticks = element_line(color = axis.color),
      axis.line = element_line(color = axis.color),
      # panel.grid = element_line(color = alpha("white", 0)),
      axis.text = element_text(color = axis.color),
      axis.title = element_text(size  = 9, color = axis.color),
      panel.background = element_rect(fill = "gray99"),
      panel.grid.major = element_line(color = "gray95"),
      panel.grid.minor = element_blank(),
      #
      legend.position = "bottom")
  #
  if (!same.size) {
    p <- p +
      guides(colour = "none", size = guide_legend(title = "Indirect (%)"))
  }
  else {
    p <- p + guides(colour = "none", size = "none")
  }
  #
  # Add plot labels
  #
  if (any_sign) {
    p <- p +
      labs(title = main, subtitle = text.pooled, x = xlab, y = ylab,
           caption = paste0(
             "Significant results (p < ", 1 - level, ") are displayed in ",
             col.inc),
           tag = tag)
  }
  else {
    p <- p +
      labs(title = main, subtitle = text.pooled, x = xlab, y = ylab, tag = tag)
  }
  #
  if (labels) {
    dat.labels <- dat.lines %>%
      filter(Bound == "lower")
    #
    if (labels.only.inc)
      dat.labels %<>% filter(signif == "Y")
    #
    p <- p +
      geom_text_repel(
        data = dat.labels,
        mapping = mapslab,
        size = size.labels,
        max.overlaps = max_overlap)
  }
  #
  if (!is.null(xlim) & is.null(ylim))
    p <- p + coord_cartesian(xlim = xlim)
  #
  else if (is.null(xlim) & !is.null(ylim))
    p <- p + coord_cartesian(ylim = ylim)
  #
  else if (!is.null(xlim) & !is.null(ylim))
    p <- p + coord_cartesian(xlim = xlim, ylim = ylim)
  #
  else if (type == "Wilson") {
    p <- p +
      coord_cartesian(xlim = c(-xylim, xylim), ylim = c(-xylim, xylim))
  }
  #
  res <- list(plot = p, data = dat)
  #
  if (any(trts != trts.abbr)) {
    res$trts <- trts
    res$trts.abbr <- trts.abbr
  }
  #
  class(res) <- "plot.netsplit"
  #
  res
}

#' @method print plot.netsplit
#' @export

print.plot.netsplit <- function(x, ...) {
  print(x$plot)
  #
  invisible(NULL)
}

#' Additive network meta-analysis for combinations of treatments
#' (disconnected networks)
#' 
#' @description
#' Some treatments in a network meta-analysis may be combinations of
#' other treatments or have common components. The influence of
#' individual components can be evaluated in an additive network
#' meta-analysis model assuming that the effect of treatment
#' combinations is the sum of the effects of its components. This
#' function implements this additive model in a frequentist way and is
#' particularly intended for disconnected networks.
#' 
#' @param TE Estimate of treatment effect, i.e. difference between
#'   first and second treatment (e.g. log odds ratio, mean difference,
#'   or log hazard ratio). Or an R object created with
#'   \code{\link[meta]{pairwise}}.
#' @param seTE Standard error of treatment estimate.
#' @param treat1 Label/Number for first treatment.
#' @param treat2 Label/Number for second treatment.
#' @param studlab An optional - but important! - vector with study
#'   labels (see \code{\link{netmeta}}).
#' @param data An optional data frame containing the study
#'   information.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param inactive A character string defining the inactive treatment
#'   component (see Details).
#' @param sep.comps A single character to define separator between
#'   treatment components.
#' @param C.matrix C matrix (see Details).
#' @param sm A character string indicating underlying summary measure,
#'   e.g., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"},
#'   \code{"HR"}, \code{"MD"}, \code{"SMD"}, or \code{"ROM"}.
#' @param level The level used to calculate confidence intervals for
#'   individual comparisons.
#' @param level.ma The level used to calculate confidence intervals
#'   for network estimates.
#' @param common A logical indicating whether a common effects /
#'   common effects network meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects network
#'   meta-analysis should be conducted.
#' @param reference.group Reference treatment (first treatment is used
#'   if argument is missing).
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments in printouts.
#' @param tau.preset An optional value for the square-root of the
#'   between-study variance \eqn{\tau^2}.
#' @param tol.multiarm A numeric for the tolerance for consistency of
#'   treatment estimates in multi-arm studies which are consistent by
#'   design.
#' @param tol.multiarm.se A numeric for the tolerance for consistency
#'   of standard errors in multi-arm studies which are consistent by
#'   design. This check is not conducted if the argument is
#'   \code{NULL}.
#' @param details.chkmultiarm A logical indicating whether treatment
#'   estimates and / or variances of multi-arm studies with
#'   inconsistent results or negative multi-arm variances should be
#'   printed.
#' @param details.chkident A logical indicating whether details on
#'   unidentifiable components should be printed.
#' @param sep.trts A character used in comparison names as separator
#'   between treatment labels.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique names for components (see
#'   Details).
#' @param sep.ia A single character to define separator for interactions.
#' @param func.inverse R function used to calculate the pseudoinverse
#'   of the Laplacian matrix L (see \code{\link{netmeta}}).
#' @param n1 Number of observations in first treatment group.
#' @param n2 Number of observations in second treatment group.
#' @param event1 Number of events in first treatment group.
#' @param event2 Number of events in second treatment group.
#' @param incr Numerical value added to cell frequencies.
#' @param na.unident A logical indicating whether unidentifiable
#'   components and combinations should be set to missing values.
#' @param keepdata A logical indicating whether original data(set)
#'   should be kept in netmeta object.
#' @param title Title of meta-analysis / systematic review.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if studies are excluded from meta-analysis due to zero
#'   standard errors).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param nchar.trts Deprecated argument (replaced by
#'   \code{nchar.comps}).
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' Treatments in network meta-analysis (NMA) can be complex
#' interventions. Some treatments may be combinations of others or
#' have common components. The standard analysis provided by
#' \code{\link{netmeta}} is a NMA where all existing (single or
#' combined) treatments are considered as different nodes in the
#' network. Exploiting the fact that some treatments are combinations
#' of common components, an additive component network meta-analysis
#' (CNMA) model can be used to evaluate the influence of individual
#' components. This model assumes that the effect of a treatment
#' combination is the sum of the effects of its components which
#' implies that common components cancel out in comparisons.
#' 
#' This R function can be used for disconnected networks. Use
#' \code{\link{netmeta}} and \code{\link{netcomb}} for connected
#' networks.
#' 
#' The additive CNMA model has been implemented using Bayesian methods
#' (Mills et al., 2012; Welton et al., 2013). This function implements
#' the additive model in a frequentist way (Rücker et al., 2020).
#' 
#' The underlying multivariate model is given by
#' 
#' \deqn{\bold{\delta} = \bold{B} \bold{\theta}, \bold{\theta} =
#' \bold{C} \bold{\beta}}
#' 
#' with
#' \describe{
#' \item{\eqn{\bold{\delta}}}{vector of true treatment effects
#'   (differences) from individual studies,}
#' \item{\eqn{\bold{B}}}{design matrix describing the structure of the
#'   network,}
#' \item{\eqn{\bold{\theta}}}{parameter vector that represents the
#'   existing combined treatments,}
#' \item{\eqn{\bold{C}}}{matrix describing how the treatments are
#'   composed,}
#' \item{\eqn{\bold{\beta}}}{parameter vector representing the
#'   treatment components.}
#' }
#' All parameters are estimated using weighted least squares
#' regression.
#' 
#' Argument \code{inactive} can be used to specify a single component
#' that does not have any therapeutic value. Accordingly, it is
#' assumed that the treatment effect of the combination of this
#' component with an additional treatment component is equal to the
#' treatment effect of the additional component alone.
#' 
#' Argument \code{sep.comps} can be used to specify the separator
#' between individual components. By default, the matrix \strong{C} is
#' calculated internally from treatment names. However, it is possible
#' to specify a different matrix using argument \code{C.matrix}.
#' 
#' By default, component names are not abbreviated in
#' printouts. However, in order to get more concise printouts,
#' argument \code{nchar.comps} can be used to define the minimum
#' number of characters for abbreviated component names (see
#' \code{\link{abbreviate}}, argument \code{minlength}). R function
#' \code{\link{treats}} is utilised internally to create abbreviated
#' component names.
#' 
#' @note
#' This function calculates effects for individual components and
#' complex interventions present in the network.
#'
#' R function \code{\link{netcomplex}} can be used to calculate the
#' effect for arbitrary complex interventions in a component network
#' meta-analysis. Furthermore, R function \code{\link{netcomparison}}
#' can be used to calculate the effect for comparisons of two
#' arbitrary complex intervention in a component network
#' meta-analysis.
#' 
#' @return
#' An object of classes \code{discomb} and \code{netcomb} with
#' corresponding \code{print}, \code{summary}, and \code{forest}
#' functions. The object is a list containing the following
#' components:
#' \item{studlab}{Study labels.}
#' \item{treat1}{Label/Number for first treatment.}
#' \item{treat2}{Label/Number for second treatment.}
#' \item{TE}{Estimate of treatment effect, i.e. difference between
#'   first and second treatment.}
#' \item{seTE}{Standard error of treatment estimate.}
#' \item{seTE.adj.common, seTE.adj.random}{Standard error of treatment
#'   estimate, adjusted for multi-arm studies.}
#' \item{event1}{Number of events in first treatment group.}
#' \item{event2}{Number of events in second treatment group.}
#' \item{n1}{Number of observations in first treatment group.}
#' \item{n2}{Number of observations in second treatment group.}
#' \item{k}{Total number of studies.}
#' \item{m}{Total number of pairwise comparisons.}
#' \item{n}{Total number of treatments.}
#' \item{d}{Total number of designs (corresponding to the unique set
#'   of treatments compared within studies).}
#' \item{c}{Total number of components.}
#' \item{trts}{Treatments included in network meta-analysis.}
#' \item{comps}{Unique list of components present in the network.}
#' \item{TE.cnma.common, TE.cnma.random}{A vector of length \emph{m} of
#'   consistent treatment effects estimated by the additive (common and
#'   random effects) model.}
#' \item{seTE.cnma.common, seTE.cnma.random}{A vector of length
#'   \emph{m} with standard errors estimated by the additive (common
#'   and random effects) model.}
#' \item{lower.cnma.common, lower.cnma.random}{A vector of length
#'   \emph{m} of lower confidence interval limits for consistent
#'   treatment effects estimated by the additive (common and random
#'   effects) model.}
#' \item{upper.cnma.common, upper.cnma.random}{A vector of length
#'   \emph{m} of upper confidence interval limits for consistent
#'   treatment effects estimated by the additive (common and random
#'   effects) model.}
#' \item{statistic.cnma.common, statistic.cnma.random}{A vector of
#'   length \emph{m} of z-values for the test of an overall effect
#'   estimated by the additive (common and random effects) model.}
#' \item{pval.cnma.common, pval.cnma.random}{A vector of length
#'   \emph{m} of p-values for the test of an overall effect estimated
#'   by the additive (common and random effects) model.}
#' \item{TE.common, TE.random}{\emph{n}x\emph{n} matrix with overall
#'   treatment effects estimated by the additive (common and random
#'   effects) model.}
#' \item{seTE.common, seTE.random}{\emph{n}x\emph{n} matrix with
#'   standard errors estimated by the additive (common and random
#'   effects) model.}
#' \item{lower.common, upper.common, lower.random,
#'   upper.random}{\emph{n}x\emph{n} matrices with lower and upper
#'   confidence interval limits estimated by the additive (common and
#'   random effects) model.}
#' \item{statistic.common, pval.common, statistic.random,
#'   pval.random}{\emph{n}x\emph{n} matrices with z-values and
#'   p-values for test of overall effect estimated by the additive
#'   (common and random effects) model.}
#' \item{Comp.common, Comp.random}{A vector of component effects (common
#'   and random effects model).}
#' \item{seComp.common, seComp.random}{A vector with corresponding
#'   standard errors (common and random effects model).}
#' \item{lower.Comp.common, lower.Comp.random}{A vector with lower
#'   confidence limits for components (common and random effects
#'   model).}
#' \item{upper.Comp.common, upper.Comp.random}{A vector with upper
#'   confidence limits for components (common and random effects
#'   model).}
#' \item{statistic.Comp.common, statistic.Comp.random}{A vector with
#'   z-values for the overall effect of components (common and random
#'   effects model).}
#' \item{pval.Comp.common, pval.Comp.random}{A vector with p-values for
#'   the overall effect of components (common and random effects
#'   model).}
#' \item{Comb.common, Comb.random}{A vector of combination effects (common
#'   and random effects model).}
#' \item{seComb.common, seComb.random}{A vector with corresponding
#'   standard errors (common and random effects model).}
#' \item{lower.Comb.common, lower.Comb.random}{A vector with lower
#'   confidence limits for combinations (common and random effects
#'   model).}
#' \item{upper.Comb.common, upper.Comb.random}{A vector with upper
#'   confidence limits for combinations (common and random effects
#'   model).}
#' \item{statistic.Comb.common, statistic.Comb.random}{A vector with
#'   z-values for the overall effect of combinations (common and random
#'   effects model).}
#' \item{pval.Comb.common, pval.Comb.random}{A vector with p-values for
#'   the overall effect of combinations (common and random effects
#'   model).}
#' \item{Q.additive}{Overall heterogeneity / inconsistency statistic
#'   (additive model).}
#' \item{df.Q.additive}{Degrees of freedom for test of heterogeneity /
#'   inconsistency (additive model).}
#' \item{pval.Q.additive}{P-value for test of heterogeneity /
#'   inconsistency (additive model).}
#' \item{tau}{Square-root of between-study variance (additive model).}
#' \item{I2}{I-squared (additive model).}
#' \item{Q.standard}{Overall heterogeneity / inconsistency statistic
#'   (standard model).}
#' \item{df.Q.standard}{Degrees of freedom for test of heterogeneity /
#'   inconsistency (standard model).}
#' \item{pval.Q.standard}{P-value for test of heterogeneity /
#'   inconsistency (standard model).}
#' \item{Q.diff}{Test statistic for difference in goodness of fit
#'   between standard and additive model.}
#' \item{df.Q.diff}{Degrees of freedom for difference in goodness of
#'   fit between standard and additive model.}
#' \item{pval.Q.diff}{P-value for difference in goodness of fit
#'   between standard and additive model.}
#' \item{X.matrix}{Design matrix (\emph{m}x\emph{n}).}
#' \item{B.matrix}{Edge-vertex incidence matrix (\emph{m}x\emph{n}).}
#' \item{C.matrix}{As defined above.}
#' \item{sm}{Summary measure.}
#' \item{level.ma}{Level for confidence intervals.}
#' \item{common, random, tau.preset}{As defined above.}
#' \item{sep.trts}{A character used in comparison names as separator
#'   between treatment labels.}
#' \item{nchar.comps}{A numeric defining the minimum number of
#'   characters used to create unique component names.}
#' \item{inactive, sep.comps}{As defined above.}
#' \item{backtransf}{A logical indicating whether results should be
#'   back transformed in printouts and forest plots.}
#' \item{title}{Title of meta-analysis / systematic review.}
#' \item{x}{As defined above.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package netmeta used to create
#'   object.}
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{forest.netcomb}},
#'   \code{\link{summary.netcomb}}, \code{\link{netmeta}},
#'   \code{\link{netconnection}}, \code{\link{netcomplex}},
#'   \code{\link{netcomparison}}
#' 
#' @references
#' König J, Krahn U, Binder H (2013):
#' Visualizing the flow of evidence in network meta-analysis and
#' characterizing mixed treatment comparisons.
#' \emph{Statistics in Medicine},
#' \bold{32}, 5414--29
#' 
#' Mills EJ, Thorlund K, Ioannidis JP (2012):
#' Calculating additive treatment effects from multiple randomized
#' trials provides useful estimates of combination therapies.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{65}, 1282--8
#' 
#' Rücker G, Petropoulou M, Schwarzer G (2020):
#' Network meta-analysis of multicomponent interventions.
#' \emph{Biometrical Journal},
#' \bold{62}, 808--21
#' 
#' Welton NJ, Caldwell DM, Adamopoulos E, Vedhara K (2009):
#' Mixed treatment comparison meta-analysis of complex interventions:
#' psychological interventions in coronary heart disease.
#' \emph{American Journal of Epidemiology},
#' \bold{169}: 1158--65
#' 
#' @examples
#' # Artificial dataset
#' #
#' t1 <- c("A + B", "A + C", "A"    , "A"    , "D", "D", "E")
#' t2 <- c("C"    , "B"    , "B + C", "A + D", "E", "F", "F")
#' #
#' mean    <- c(4.1, 2.05, 0, 0, 0.1, 0.1, 0.05)
#' se.mean <- rep(0.1, 7)
#' #
#' study <- paste("study", c(1:4, 5, 5, 5))
#' #
#' dat <- data.frame(mean, se.mean, t1, t2, study,
#'   stringsAsFactors = FALSE)
#' #
#' trts <- c("A", "A + B", "A + C", "A + D",
#'   "B", "B + C", "C", "D", "E", "F")
#' #
#' comps <- LETTERS[1:6]
#' 
#' # Use netconnection() to display network information
#' #
#' netconnection(t1, t2, study)
#' 
#' dc1 <- discomb(mean, se.mean, t1, t2, study, seq = trts)
#' dc1
#' 
#' \donttest{
#' forest(dc1, ref = "F")
#' 
#' # Define C matrix manually (which will produce the same results)
#' #
#' C <- rbind(c(1, 0, 0, 0, 0, 0),  # A
#'   c(1, 1, 0, 0, 0, 0),  # A + B
#'   c(1, 0, 1, 0, 0, 0),  # A + C
#'   c(1, 0, 0, 1, 0, 0),  # A + D
#'   c(0, 1, 0, 0, 0, 0),  # B
#'   c(0, 1, 1, 0, 0, 0),  # B + C
#'   c(0, 0, 1, 0, 0, 0),  # C
#'   c(0, 0, 0, 1, 0, 0),  # D
#'   c(0, 0, 0, 0, 1, 0),  # E
#'   c(0, 0, 0, 0, 0, 1))  # F
#' #                  
#' colnames(C) <- comps
#' rownames(C) <- trts
#' #
#' dc2 <- discomb(mean, se.mean, t1, t2, study, seq = trts,
#'   C.matrix = C)
#' #
#' # Compare C matrices
#' #
#' all.equal(dc1$C.matrix, dc2$C.matrix)
#' }
#' 
#' @export discomb

discomb <- function(TE, seTE,
                    treat1, treat2,
                    studlab, data = NULL, subset = NULL,
                    ##
                    inactive = NULL,
                    sep.comps = gs("sep.comps"),
                    C.matrix,
                    ##
                    sm,
                    level = gs("level"),
                    level.ma = gs("level.ma"),
                    common = gs("common"),
                    random = gs("random") | !is.null(tau.preset),
                    ##
                    reference.group,
                    baseline.reference = gs("baseline.reference"),
                    seq = gs("sep"),
                    ##
                    tau.preset = NULL,
                    ##
                    tol.multiarm = gs("tol.multiarm"),
                    tol.multiarm.se = gs("tol.multiarm.se"),
                    details.chkmultiarm = gs("details.chkmultiarm"),
                    ##
                    details.chkident = FALSE,
                    ##
                    sep.trts = gs("sep.trts"),
                    nchar.comps = gs("nchar.comps"),
                    #
                    sep.ia = gs("sep.ia"),
                    #
                    func.inverse = invmat,
                    #
                    n1 = NULL,
                    n2 = NULL,
                    event1 = NULL,
                    event2 = NULL,
                    incr = NULL,
                    #
                    overall.hetstat = gs("overall.hetstat"),
                    backtransf = gs("backtransf"),
                    #
                    na.unident = gs("na.unident"),
                    ##
                    title = gs("title"),
                    keepdata = gs("keepdata"),
                    #
                    warn = gs("warn"), warn.deprecated = gs("warn.deprecated"),
                    nchar.trts = nchar.comps,
                    ...) {
  
  
  ##
  ##
  ## (1) Check arguments
  ##
  ##
  chkchar(sep.comps, nchar = 1, length = 1)
  ##
  chklevel(level)
  ##
  missing.reference.group <- missing(reference.group) 
  chklogical(baseline.reference)
  ##
  if (!is.null(tau.preset))
    chknumeric(tau.preset, min = 0, length = 1)
  ##
  chknumeric(tol.multiarm, min = 0, length = 1)
  if (!is.null(tol.multiarm.se))
    chknumeric(tol.multiarm.se, min = 0, length = 1)
  chklogical(details.chkmultiarm)
  ##
  chklogical(details.chkident)
  ##
  missing.sep.trts <- missing(sep.trts)
  chkchar(sep.trts)
  #
  overall.hetstat <- replaceNULL(overall.hetstat, TRUE)
  chklogical(overall.hetstat)
  chklogical(backtransf)
  chklogical(na.unident)
  ##
  chkchar(title)
  chklogical(keepdata)
  chklogical(warn)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  level.ma <- deprecated(level.ma, missing(level.ma), args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  ##
  nchar.comps <-
    deprecated2(nchar.comps, missing(nchar.comps),
                nchar.trts, missing(nchar.trts),
                warn.deprecated)
  nchar.comps <- replaceNULL(nchar.comps, 666)
  chknumeric(nchar.comps, min = 1, length = 1)
  #
  missing.sep.ia <- missing(sep.ia)
  chkchar(sep.ia, nchar = 0:1, length = 1)
  
  
  ##
  ##
  ## (2) Read data
  ##
  ##
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  if (nulldata)
    data <- sfsp
  ##
  ## Catch TE, treat1, treat2, seTE, studlab from data:
  ##
  TE <- catch("TE", mc, data, sfsp)
  ##
  avail.reference.group.pairwise <- FALSE
  ##
  if (is.data.frame(TE) & !is.null(attr(TE, "pairwise"))) {
    is.pairwise <- TRUE
    ##
    sm <- attr(TE, "sm")
    #
    if (missing.reference.group) {
      reference.group <- attr(TE, "reference.group")
      #
      if (is.null(reference.group))
        reference.group <- ""
      else
        avail.reference.group.pairwise <- TRUE
    }
    #
    keep.all.comparisons <- attr(TE, "keep.all.comparisons")
    if (!is.null(keep.all.comparisons) && !keep.all.comparisons)
      stop("First argument is a pairwise object created with ",
           "'keep.all.comparisons = FALSE'.",
           call. = TRUE)
    #
    if (is.null(attr(TE, "varnames")))
      seTE <- TE$seTE
    else
      seTE <- TE[[attr(TE, "varnames")[2]]]
    #
    treat1 <- TE$treat1
    treat2 <- TE$treat2
    studlab <- TE$studlab
    #
    if (!is.null(TE$n1))
      n1 <- TE$n1
    if (!is.null(TE$n2))
      n2 <- TE$n2
    if (!is.null(TE$event1))
      event1 <- TE$event1
    if (!is.null(TE$event2))
      event2 <- TE$event2
    if (!is.null(TE$incr))
      incr <- TE$incr
    #
    pairdata <- TE
    data <- TE
    #
    if (is.null(attr(TE, "varnames")))
      TE <- TE$TE
    else
      TE <- TE[[attr(TE, "varnames")[1]]]
  }
  else {
    is.pairwise <- FALSE
    if (missing(sm))
      if (!is.null(data) && !is.null(attr(data, "sm")))
        sm <- attr(data, "sm")
      else
        sm <- ""
    ##
    seTE <- catch("seTE", mc, data, sfsp)
    #
    treat1 <- catch("treat1", mc, data, sfsp)
    treat2 <- catch("treat2", mc, data, sfsp)
    #
    studlab <- catch("studlab", mc, data, sfsp)
    #
    if (length(studlab) == 0) {
      if (warn)
        warning("No information given for argument 'studlab'. ",
                "Assuming that comparisons are from independent studies.",
                call. = FALSE)
      studlab <- seq(along = TE)
    }
    studlab <- as.character(studlab)
    #
    n1 <- catch("n1", mc, data, sfsp)
    n2 <- catch("n2", mc, data, sfsp)
    #
    event1 <- catch("event1", mc, data, sfsp)
    event2 <- catch("event2", mc, data, sfsp)
    #
    incr <- catch("incr", mc, data, sfsp)
  }
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  studlab <- as.character(studlab)
  #
  chknumeric(TE)
  chknumeric(seTE)
  ##
  if (!any(!is.na(TE) & !is.na(seTE)))
    stop("Missing data for estimates (argument 'TE') and ",
         "standard errors (argument 'seTE') in all studies.\n  ",
         "No network meta-analysis possible.",
         call. = FALSE)
  ##
  k.Comp <- length(TE)
  ##
  if (is.factor(treat1))
    treat1 <- as.character(treat1)
  if (is.factor(treat2))
    treat2 <- as.character(treat2)
  if (is.factor(studlab))
    studlab <- as.character(studlab)
  ##
  ## Remove leading and trailing whitespace
  ##
  treat1 <- rmSpace(rmSpace(treat1, end = TRUE))
  treat2 <- rmSpace(rmSpace(treat2, end = TRUE))
  #
  #
  if (!is.null(event1) & !is.null(event2))
    available.events <- TRUE
  else
    available.events <- FALSE
  #
  if (!is.null(n1) & !is.null(n2))
    available.n <- TRUE
  else
    available.n <- FALSE
  #
  if (available.events & is.null(incr))
    incr <- rep(0, length(event2))
  #
  # Store complete dataset in list object data
  # (if argument keepdata is TRUE)
  #
  if (keepdata) {
    if (nulldata & !is.pairwise)
      data <- data.frame(.studlab = studlab, stringsAsFactors = FALSE)
    else if (nulldata & is.pairwise) {
      data <- pairdata
      data$.studlab <- studlab
    }
    else
      data$.studlab <- studlab
    #
    data$.order <- seq_along(studlab)
    #
    data$.treat1 <- treat1
    data$.treat2 <- treat2
    #
    data$.TE <- TE
    data$.seTE <- seTE
    #
    data$.event1 <- event1
    data$.n1 <- n1
    data$.event2 <- event2
    data$.n2 <- n2
    data$.incr <- incr
    #
    # Check for correct treatment order within comparison
    #
    wo <- data$.treat1 > data$.treat2
    #
    if (any(wo)) {
      data$.TE[wo] <- -data$.TE[wo]
      ttreat1 <- data$.treat1
      data$.treat1[wo] <- data$.treat2[wo]
      data$.treat2[wo] <- ttreat1[wo]
      #
      if (isCol(data, ".n1") & isCol(data, ".n2")) {
        tn1 <- data$.n1
        data$.n1[wo] <- data$.n2[wo]
        data$.n2[wo] <- tn1[wo]
      }
      #
      if (isCol(data, ".event1") & isCol(data, ".event2")) {
        tevent1 <- data$.event1
        data$.event1[wo] <- data$.event2[wo]
        data$.event2[wo] <- tevent1[wo]
      }
      #
      #if (isCol(data, ".mean1") & isCol(data, ".mean2")) {
      #  tmean1 <- data$.mean1
      #  data$.mean1[wo] <- data$.mean2[wo]
      #  data$.mean2[wo] <- tmean1[wo]
      #}
      #
      if (isCol(data, ".sd1") & isCol(data, ".sd2")) {
        tsd1 <- data$.sd1
        data$.sd1[wo] <- data$.sd2[wo]
        data$.sd2[wo] <- tsd1[wo]
      }
      #
      if (isCol(data, ".time1") & isCol(data, ".time2")) {
        ttime1 <- data$.time1
        data$.time1[wo] <- data$.time2[wo]
        data$.time2[wo] <- ttime1[wo]
      }
    }
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
  }
  
  
  ##
  ##
  ## (3) Use subset for analysis
  ##
  ##
  if (!is.null(subset)) {
    if ((is.logical(subset) & (sum(subset) > k.Comp)) ||
        (length(subset) > k.Comp))
      stop("Length of subset is larger than number of studies.")
    ##
    TE <- TE[subset]
    seTE <- seTE[subset]
    treat1 <- treat1[subset]
    treat2 <- treat2[subset]
    studlab <- studlab[subset]
    #
    if (!is.null(n1))
      n1 <- n1[subset]
    if (!is.null(n2))
      n2 <- n2[subset]
    if (!is.null(event1))
      event1 <- event1[subset]
    if (!is.null(event2))
      event2 <- event2[subset]
    if (!is.null(incr))
      incr <- incr[subset]
  }
  ##
  trts <- sort(unique(c(treat1, treat2)))
  ##
  if (compmatch(trts, sep.trts)) {
    if (!missing.sep.trts)
      warning("Separator '", sep.trts, "' used in at least ",
              "one treatment label. Try to use predefined separators: ",
              "':', '-', '_', '/', '+', '.', '|', '*'.",
              call. = FALSE)
    ##
    if (!compmatch(trts, ":"))
      sep.trts <- ":"
    else if (!compmatch(trts, "-"))
      sep.trts <- "-"
    else if (!compmatch(trts, "_"))
      sep.trts <- "_"
    else if (!compmatch(trts, "/"))
      sep.trts <- "/"
    else if (!compmatch(trts, "+"))
      sep.trts <- "+"
    else if (!compmatch(trts, "."))
      sep.trts <- "-"
    else if (!compmatch(trts, "|"))
      sep.trts <- "|"
    else if (!compmatch(trts, "*"))
      sep.trts <- "*"
    else
      stop("All predefined separators ",
           "(':', '-', '_', '/', '+', '.', '|', '*') ",
           "are used in at least one treatment label.",
           "\n   Please specify a different character that ",
           "should be used as separator (argument 'sep.trts').",
           call. = FALSE)
  }
  ##
  if (!is.null(seq))
    seq <- setseq(seq, trts)
  else {
    seq <- trts
    if (is.numeric(seq))
      seq <- as.character(seq)
  }
  
  
  ##
  ##
  ## (4) Additional checks
  ##
  ##
  if (!(any(grepl(sep.comps, treat1, fixed = TRUE)) |
        any(grepl(sep.comps, treat2, fixed = TRUE))))
    warning("No treatment contains the component separator '", sep.comps, "'.",
            call. = FALSE)
  ##
  if (any(treat1 == treat2))
    stop("Treatments must be different (arguments 'treat1' and 'treat2').",
         call. = FALSE)
  ##
  if (length(studlab) != 0)
    studlab <- as.character(studlab)
  else {
    if (warn)
      warning("No information given for argument 'studlab'. ",
              "Assuming that comparisons are from independent studies.",
              call. = FALSE)
    studlab <- seq(along = TE)
  }
  ##
  ## Check for correct number of comparisons
  ##
  tabnarms <- table(studlab)
  sel.narms <- !is_wholenumber((1 + sqrt(8 * tabnarms + 1)) / 2)
  ##
  if (sum(sel.narms) == 1)
    stop(paste0("Study '", names(tabnarms)[sel.narms],
                "' has a wrong number of comparisons.",
                "\n  Please provide data for all treatment comparisons",
                " (two-arm: 1; three-arm: 3; four-arm: 6, ...)."))
  if (sum(sel.narms) > 1)
    stop(paste0("The following studies have a wrong number of comparisons: ",
                paste(paste0("'", names(tabnarms)[sel.narms], "'"),
                      collapse = ", "),
                "\n  Please provide data for all treatment comparisons",
                " (two-arm: 1; three-arm: 3; four-arm: 6, ...)."))
  ##
  ## Check NAs and zero standard errors
  ##
  excl <- is.na(TE) | is.na(seTE) | seTE <= 0
  ##
  if (any(excl)) {
    if (keepdata)
      data$.excl <- excl
    #
    dat.NAs <- data.frame(studlab = studlab[excl],
                          treat1 = treat1[excl],
                          treat2 = treat2[excl],
                          TE = format(round(TE[excl], 4)),
                          seTE = format(round(seTE[excl], 4))
                          )
    warning("Comparison",
            if (sum(excl) > 1) "s",
            " with missing TE / seTE or zero seTE not considered in",
            " network meta-analysis.",
            call. = FALSE)
    cat("Comparison", if (sum(excl) > 1) "s",
        " not considered in network meta-analysis:\n",
        sep = "")
    prmatrix(dat.NAs, quote = FALSE, right = TRUE,
             rowlab = rep("", sum(excl)))
    cat("\n")
    ##
    studlab <- studlab[!excl]
    treat1  <- treat1[!excl]
    treat2  <- treat2[!excl]
    TE      <- TE[!excl]
    seTE    <- seTE[!excl]
    #
    if (!is.null(n1))
      n1 <- n1[!excl]
    if (!is.null(n2))
      n2 <- n2[!excl]
    if (!is.null(event1))
      event1 <- event1[!excl]
    if (!is.null(event2))
      event2 <- event2[!excl]
    if (!is.null(incr))
      incr <- incr[!excl]
    #
    seq <- seq[seq %in% unique(c(treat1, treat2))]
    trts <- trts[trts %in% unique(c(treat1, treat2))]
  }
  ##
  ## Check for correct number of comparisons (after removing
  ## comparisons with missing data)
  ##
  tabnarms <- table(studlab)
  sel.narms <- !is_wholenumber((1 + sqrt(8 * tabnarms + 1)) / 2)
  ##
  if (sum(sel.narms) == 1)
    stop(paste0("After removing comparisons with missing treatment effects",
                " or standard errors,\n  study '",
                names(tabnarms)[sel.narms],
                "' has a wrong number of comparisons.",
                " Please check data and\n  consider to remove study",
                " from network meta-analysis."))
  if (sum(sel.narms) > 1)
    stop(paste0("After removing comparisons with missing treatment effects",
                " or standard errors,\n  the following studies have",
                " a wrong number of comparisons: ",
                paste(paste0("'", names(tabnarms)[sel.narms], "'"),
                      collapse = ", "),
                "\n  Please check data and consider to remove studies",
                " from network meta-analysis."))
  ##
  ## Check for correct treatment order within comparison
  ##
  wo <- treat1 > treat2
  ##
  if (any(wo)) {
    TE[wo] <- -TE[wo]
    ttreat1 <- treat1
    treat1[wo] <- treat2[wo]
    treat2[wo] <- ttreat1[wo]
    #
    if (available.n) {
      tn1 <- n1
      n1[wo] <- n2[wo]
      n2[wo] <- tn1[wo]
    }
    #
    if (available.events) {
      tevent1 <- event1
      event1[wo] <- event2[wo]
      event2[wo] <- tevent1[wo]
    }
  }
  #
  # Set reference group
  #
  if (missing.reference.group & !avail.reference.group.pairwise) {
    go.on <- TRUE
    i <- 0
    while (go.on) {
      i <- i + 1
      sel.i <-
        !is.na(TE) & !is.na(seTE) &
        (treat1 == trts[i] | treat2 == trts[i])
      if (sum(sel.i) > 0) {
        go.on <- FALSE
        reference.group <- trts[i]
      }
      else if (i == length(trts)) {
        go.on <- FALSE
        reference.group <- ""
      }
    }
  }
  #
  # Check reference group
  #
  if (reference.group != "")
    reference.group <- setref(reference.group, trts)
  
  
  ##
  ##
  ## (5) Create C.matrix, B.matrix and Design matrix
  ##
  ##
  netc <- netconnection(treat1, treat2, studlab)
  ##
  if (missing(C.matrix)) {
    C.matrix <- createC(netc, inactive = inactive, sep.comps = sep.comps)
    inactive <- attr(C.matrix, "inactive")
    C.matrix <- C.matrix[trts, , drop = FALSE]
  }
  else {
    ##
    if (!(is.matrix(C.matrix) | is.data.frame(C.matrix))) 
      stop("Argument 'C.matrix' must be a matrix or data frame.", 
           call. = FALSE)
    ##
    wrong.trts <- FALSE
    if (is.null(rownames(C.matrix)))
      wrong.trts <- TRUE
    else {
      if (length(unique(trts)) ==
          length(unique(tolower(trts))) &&
          length(unique(rownames(C.matrix))) ==
          length(unique(tolower(rownames(C.matrix))))
          )
        idx <- charmatch(tolower(rownames(C.matrix)), 
                         tolower(trts), nomatch = NA)
      else
        idx <- charmatch(rownames(C.matrix), trts, nomatch = NA)
      ##
      if (any(is.na(idx)) || any(idx == 0)) 
        wrong.trts <- TRUE
    }
    if (wrong.trts) 
      stop(paste0("Row names of argument 'C.matrix' must be a ", 
                  "permutation of treatment names:\n  ",
                  paste(paste0("'", trts, "'"), collapse = " - ")),
           call. = FALSE)
    ##
    C.matrix <- C.matrix[trts, , drop = FALSE]
  }
  if (is.data.frame(C.matrix))
    C.matrix <- as.matrix(C.matrix)
  ##
  c <- ncol(C.matrix) # number of components
  #
  # Create B.matrix
  #
  p0 <- prepare(TE, seTE, treat1, treat2, studlab,
                func.inverse = func.inverse)
  #
  W.matrix.common <- p0$W
  dat.c <- p0$data
  #
  B.matrix <- createB(dat.c$treat1.pos, dat.c$treat2.pos)
  #
  colnames(B.matrix) <- trts
  rownames(B.matrix) <- studlab
  ##
  ## Design matrix based on treatment components
  ##
  X.matrix <- B.matrix %*% C.matrix
  ##
  colnames(X.matrix) <- colnames(C.matrix)
  rownames(X.matrix) <- studlab
  #
  A.matrix <- diag(diag(t(B.matrix) %*% B.matrix)) - t(B.matrix) %*% B.matrix
  #
  # Identify and warn about not uniquely identifiable components
  #
  if (qr(X.matrix)$rank < c)
    comps.unident <- get_unident(X.matrix, details.chkident)
  else
    comps.unident <- NULL
  
    
  
  ##
  ##
  ## (6) Conduct network meta-analyses
  ##
  ##
  tdata <- data.frame(studies = dat.c$studlab, narms = dat.c$narms,
                      order = dat.c$order,
                      stringsAsFactors = FALSE)
  #
  tdata <- tdata[!duplicated(tdata[, c("studies", "narms")]), , drop = FALSE]
  studies <- tdata$studies[order(tdata$order)]
  narms <- tdata$narms[order(tdata$order)]
  n.a <- sum(narms)  
  ##
  comps <- colnames(C.matrix) # treatment components
  ##
  n <- length(trts)
  m <- length(TE)
  k <- length(unique(studlab))
  ##
  ## Common effects models
  ##
  df.Q.additive <- n.a - k - qr(X.matrix)$rank
  ##
  if (netc$n.subnets == 1) {
    net <- netmeta(TE, seTE, treat1, treat2, studlab,
                   tol.multiarm = tol.multiarm,
                   tol.multiarm.se = tol.multiarm.se,
                   details.chkmultiarm = details.chkmultiarm)
    ##
    Q <- net$Q
    df.Q <- net$df.Q
    pval.Q <- net$pval.Q
    ##
    df.Q.diff <- n - 1 - qr(X.matrix)$rank
  }
  else {
    Q <- df.Q <- pval.Q <- df.Q.diff <- NA
  }
  #
  res.c <- nma_additive(dat.c$TE, W.matrix.common,
                        dat.c$treat1, dat.c$treat2,
                        dat.c$studlab,
                        level.ma,
                        X.matrix, C.matrix, B.matrix,
                        df.Q.additive, n, sep.trts)
  #
  # Random effects models
  #
  if (!is.null(tau.preset))
    tau <- tau.preset
  else
    tau <- res.c$tau
  #
  p1 <- prepare(TE, seTE, treat1, treat2, studlab,
                tau = if (is.na(tau)) 0 else tau,
                func.inverse = func.inverse)
  #
  W.matrix.random <- p1$W
  dat.r <- p1$data
  #
  res.r <- nma_additive(dat.r$TE, W.matrix.random,
                        dat.r$treat1, dat.r$treat2,
                        dat.r$studlab,
                        level.ma,
                        X.matrix, C.matrix, B.matrix,
                        df.Q.additive, n, sep.trts)
  #
  # Difference to standard network meta-analysis model
  #
  Q.diff <- res.c$Q.additive - Q
  if (!is.na(Q.diff) && abs(Q.diff) < .Machine$double.eps^0.75)
    Q.diff <- 0
  #
  if (is.na(df.Q.diff) | df.Q.diff == 0)
    pval.Q.diff <- NA
  else
    pval.Q.diff <- 1 - pchisq(Q.diff, df.Q.diff)
  #
  # Set unidentifiable components and combinations to NA
  #
  if (na.unident & length(comps.unident) > 0) {
    trts <- names(res.c$combinations$TE)
    unident.pattern <- paste0("^", comps.unident, "$", collapse = "|")
    unident.combs <-
      trts[unlist(lapply(lapply(compsplit(trts, sep.comps),
                                grepl,
                                pattern = unident.pattern), any))]
    ##
    res.c$components <- lapply(res.c$components, setNA_vars, comps.unident)
    res.c$combinations <- lapply(res.c$combinations, setNA_vars, unident.combs)
    ##
    res.r$components <- lapply(res.r$components, setNA_vars, comps.unident)
    res.r$combinations <- lapply(res.r$combinations, setNA_vars, unident.combs)
  }
  ##
  if (length(comps.unident) == 0)
    comps.unident <- NULL
  #
  comps <- names(res.c$components$TE)
  #
  if (sep.comps == sep.ia)
    stop("Input for arguments 'sep.comps' and 'sep.ia' must be different.",
         call. = FALSE)
  #
  sep.ia <- setsep(comps, sep.ia, missing = missing.sep.ia)
  
  
  ##
  ##
  ## (7) Generate CNMA object
  ##
  ##
  n.comps <- table(dat.c$studlab)
  designs <- designs(dat.c$treat1, dat.c$treat2, dat.c$studlab,
                     sep.trts = sep.trts)
  ##  
  NAs <- rep(NA, length(res.c$comparisons$TE))
  #
  o <- order(dat.c$order)
  #
  W.matrix.common <- W.matrix.common[o, o, drop = FALSE]
  rownames(W.matrix.common) <- colnames(W.matrix.common) <- res.c$studlab[o]
  #
  W.matrix.random <- W.matrix.random[o, o, drop = FALSE]
  rownames(W.matrix.random) <- colnames(W.matrix.random) <- res.c$studlab[o]
  #
  res <- list(studlab = dat.c$studlab,
              treat1 = dat.c$treat1,
              treat2 = dat.c$treat2,
              ##
              TE = dat.c$TE,
              seTE = dat.c$seTE,
              seTE.adj = sqrt(1 / dat.c$weights),
              seTE.adj.common = sqrt(1 / dat.c$weights),
              seTE.adj.random = sqrt(1 / dat.r$weights),
              ##
              design = designs$design,
              ##
              event1 = event1,
              event2 = event2,
              n1 = n1,
              n2 = n2,
              incr = incr,
              ##
              k = k,
              m = m,
              n = n,
              d = length(unique(designs$design)),
              c = c,
              s = netc$n.subnets,
              ##
              trts = trts,
              k.trts = NA,
              n.trts = if (available.n) NA else NULL,
              events.trts = if (available.events) NA else NULL,
              ##
              n.arms = NA,
              multiarm = NA,
              ##
              studies = studies,
              narms = narms,
              ##
              designs = unique(sort(designs$design)),
              ##
              comps = comps,
              k.comps = NA,
              n.comps = NA,
              events.comps = NA,
              na.unident = na.unident,
              comps.unident = comps.unident,
              ##
              TE.nma.common = NAs,
              seTE.nma.common = NAs,
              lower.nma.common = NAs,
              upper.nma.common = NAs,
              statistic.nma.common = NAs,
              pval.nma.common = NAs,
              ##
              TE.cnma.common = res.c$comparisons$TE[o],
              seTE.cnma.common = res.c$comparisons$seTE[o],
              lower.cnma.common = res.c$comparisons$lower[o],
              upper.cnma.common = res.c$comparisons$upper[o],
              statistic.cnma.common = res.c$comparisons$statistic[o],
              pval.cnma.common = res.c$comparisons$p[o],
              ##
              TE.common = res.c$all.comparisons$TE,
              seTE.common = res.c$all.comparisons$seTE,
              lower.common = res.c$all.comparisons$lower,
              upper.common = res.c$all.comparisons$upper,
              statistic.common = res.c$all.comparisons$statistic,
              pval.common = res.c$all.comparisons$p,
              ##
              TE.nma.random = NAs,
              seTE.nma.random = NAs,
              lower.nma.random = NAs,
              upper.nma.random = NAs,
              statistic.nma.random = NAs,
              pval.nma.random = NAs,
              ##
              TE.cnma.random = res.r$comparisons$TE[o],
              seTE.cnma.random = res.r$comparisons$seTE[o],
              lower.cnma.random = res.r$comparisons$lower[o],
              upper.cnma.random = res.r$comparisons$upper[o],
              statistic.cnma.random = res.r$comparisons$statistic[o],
              pval.cnma.random = res.r$comparisons$p[o],
              ##
              TE.random = res.r$all.comparisons$TE,
              seTE.random = res.r$all.comparisons$seTE,
              lower.random = res.r$all.comparisons$lower,
              upper.random = res.r$all.comparisons$upper,
              statistic.random = res.r$all.comparisons$statistic,
              pval.random = res.r$all.comparisons$p,
              ##
              Comp.common = unname(res.c$components$TE),
              seComp.common = unname(res.c$components$seTE),
              lower.Comp.common = unname(res.c$components$lower),
              upper.Comp.common = unname(res.c$components$upper),
              statistic.Comp.common = unname(res.c$components$statistic),
              pval.Comp.common = unname(res.c$components$p),
              ##
              Comp.random = unname(res.r$components$TE),
              seComp.random = unname(res.r$components$seTE),
              lower.Comp.random = unname(res.r$components$lower),
              upper.Comp.random = unname(res.r$components$upper),
              statistic.Comp.random = unname(res.r$components$statistic),
              pval.Comp.random = unname(res.r$components$p),
              ##
              Comb.common = unname(res.c$combinations$TE),
              seComb.common = unname(res.c$combinations$seTE),
              lower.Comb.common = unname(res.c$combinations$lower),
              upper.Comb.common = unname(res.c$combinations$upper),
              statistic.Comb.common = unname(res.c$combinations$statistic),
              pval.Comb.common = unname(res.c$combinations$p),
              ##
              Comb.random = unname(res.r$combinations$TE),
              seComb.random = unname(res.r$combinations$seTE),
              lower.Comb.random = unname(res.r$combinations$lower),
              upper.Comb.random = unname(res.r$combinations$upper),
              statistic.Comb.random = unname(res.r$combinations$statistic),
              pval.Comb.random = unname(res.r$combinations$p),
              ##
              Q.additive = res.c$Q.additive, 
              df.Q.additive = df.Q.additive, 
              pval.Q.additive = res.c$pval.Q.additive,
              tau = tau,
              I2 = res.c$I2,
              lower.I2 = res.c$lower.I2, upper.I2 = res.c$upper.I2,
              ##
              Q.standard = Q,
              df.Q.standard = df.Q,
              pval.Q.standard = pval.Q, 
              ##
              Q.diff = Q.diff,
              df.Q.diff = df.Q.diff,
              pval.Q.diff = pval.Q.diff, 
              #
              W.matrix.common = W.matrix.common,
              W.matrix.random = W.matrix.random,
              #
              A.matrix = A.matrix,
              X.matrix = X.matrix,
              B.matrix = B.matrix,
              C.matrix = C.matrix,
              ##
              L.matrix.common = res.c$L.matrix,
              Lplus.matrix.common = res.c$Lplus.matrix,
              L.matrix.random = res.r$L.matrix,
              Lplus.matrix.random = res.r$Lplus.matrix,
              ##
              H.matrix.common = res.c$H.matrix[o, o, drop = FALSE],
              H.matrix.random = res.r$H.matrix[o, o, drop = FALSE],
              ##
              n.matrix = NULL,
              events.matrix = NULL,
              #
              Cov.common = res.c$Cov,
              Cov.random = res.r$Cov,
              #
              sm = sm,
              method = "Inverse",
              level = level,
              level.ma = level.ma,
              common = common,
              random = random, 
              ##
              reference.group = reference.group,
              baseline.reference = baseline.reference,
              all.treatments = NULL,
              seq = seq,
              ##
              method.tau = "DL",
              tau.preset = tau.preset,
              #
              tol.multiarm = tol.multiarm,
              details.chkmultiarm = details.chkmultiarm,
              #
              sep.trts = sep.trts,
              sep.comps = sep.comps,
              nchar.comps = nchar.comps,
              sep.ia = sep.ia,
              #
              inactive = inactive,
              #
              func.inverse = deparse(substitute(func.inverse)),
              #
              overall.hetstat = overall.hetstat,
              backtransf = backtransf, 
              ##
              title = title,
              ##
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )
  #
  class(res) <- c("discomb", "netcomb")
  #
  # Add information on multi-arm studies
  #
  if (any(res$narms > 2)) {
    tdata1 <- data.frame(studlab = res$studlab,
                         .order = seq(along = res$studlab))
    tdata2 <- data.frame(studlab = res$studies, narms = res$narms)
    ##
    tdata12 <- merge(tdata1, tdata2,
                     by = "studlab", all.x = TRUE, all.y = FALSE,
                     sort = FALSE)
    tdata12 <- tdata12[order(tdata12$.order), ]
    res$n.arms <- tdata12$narms
    res$multiarm <- tdata12$narms > 2
  }
  else {
    res$n.arms <- rep(2, length(res$studlab))
    res$multiarm <- rep(FALSE, length(res$studlab))
  }
  #
  # Additional assignments
  #
  l1 <- length(res$treat1)
  tab.trts <-
    table(longarm(res$treat1, res$treat2,
                  rep(1, l1), rep(2, l1), rep(1, l1), rep(2, l1),
                  studlab = res$studlab)$treat)
  res$k.trts <- as.numeric(tab.trts)
  names(res$k.trts) <- names(tab.trts)
  #
  if (keepdata) {
    data$.design <- designs(data$.treat1, data$.treat2, data$.studlab,
                            sep = sep.trts)$design
    #
    res$data <- merge(data,
                      data.frame(.studlab = res$studies, .narms = res$narms),
                      by = ".studlab",
                      stringsAsFactors = FALSE)
    #
    # Store adjusted standard errors in dataset
    #
    if (isCol(res$data, ".subset"))
      sel.s <- res$data$.subset
    else
      sel.s <- rep(TRUE, nrow(res$data))
    #
    res$data$.seTE.adj.common <- NA
    res$data$.seTE.adj.random <- NA
    #
    res$data$.seTE.adj.common[sel.s] <- res$seTE.adj.common
    res$data$.seTE.adj.random[sel.s] <- res$seTE.adj.random
    #
    res$data <- res$data[order(res$data$.order), ]
    res$data$.order <- NULL
  }
  #
  # Add information on events and sample sizes
  #
  if (available.n) {
    res$n.matrix <- netmatrix(res, n1 + n2, func = "sum")
    #
    dat.n <- data.frame(studlab = c(studlab, studlab),
                        treat = c(treat1, treat2),
                        n = c(n1, n2))
    dat.n <- dat.n[!duplicated(dat.n[, c("studlab", "treat")]), ]
    dat.n <- by(dat.n$n, dat.n$treat, sum, na.rm = TRUE)
    res$n.trts <- as.vector(dat.n[trts])
    names(res$n.trts) <- trts
  }
  #
  if (available.events) {
    res$events.matrix <- netmatrix(res, event1 + event2, func = "sum")
    #
    dat.e <- data.frame(studlab = c(studlab, studlab),
                        treat = c(treat1, treat2),
                        n = c(event1, event2))
    dat.e <- dat.e[!duplicated(dat.e[, c("studlab", "treat")]), ]
    dat.e <- by(dat.e$n, dat.e$treat, sum, na.rm = TRUE)
    res$events.trts <- as.vector(dat.e[trts])
    names(res$events.trts) <- trts
  }
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  res$comb.fixed <- res$common
  res$comb.random <- res$random
  ##
  res$seTE.adj.fixed <- res$seTE.adj.common
  res$TE.nma.fixed <- res$TE.nma.common
  res$seTE.nma.fixed <- res$seTE.nma.common
  res$lower.nma.fixed <- res$lower.nma.common
  res$upper.nma.fixed <- res$upper.nma.common
  res$statistic.nma.fixed <- res$statistic.nma.common
  res$pval.nma.fixed <- res$pval.nma.common
  res$TE.cnma.fixed <- res$TE.cnma.common
  res$seTE.cnma.fixed <- res$seTE.cnma.common
  res$lower.cnma.fixed <- res$lower.cnma.common
  res$upper.cnma.fixed <- res$upper.cnma.common
  res$statistic.cnma.fixed <- res$statistic.cnma.common
  res$pval.cnma.fixed <- res$pval.cnma.common
  ##
  res$TE.fixed <- res$TE.common
  res$seTE.fixed <- res$seTE.common
  res$lower.fixed <- res$lower.common
  res$upper.fixed <- res$upper.common
  res$statistic.fixed <- res$statistic.common
  res$pval.fixed <- res$pval.common
  ##
  res$Comp.fixed <- res$Comp.common
  res$seComp.fixed <- res$seComp.common
  res$lower.Comp.fixed <- res$lower.Comp.common
  res$upper.Comp.fixed <- res$upper.Comp.common
  res$statistic.Comp.fixed <- res$statistic.Comp.common
  res$pval.Comp.fixed <- res$pval.Comp.common
  ##
  res$Comb.fixed <- res$Comb.common
  res$seComb.fixed <- res$seComb.common
  res$lower.Comb.fixed <- res$lower.Comb.common
  res$upper.Comb.fixed <- res$upper.Comb.common
  res$statistic.Comb.fixed <- res$statistic.Comb.common
  res$pval.Comb.fixed <- res$pval.Comb.common
  ##
  res$L.matrix.fixed <- res$L.matrix.common
  res$Lplus.matrix.fixed <- res$Lplus.matrix.common
  res$H.matrix.fixed <- res$H.matrix.common        
  
  
  res
}

#' Generic function for rankograms
#' 
#' @description
#' Generic function to calculate the probabilities of each treatment being
#' at each possible rank and the SUCRAs (Surface Under the Cumulative
#' RAnking curve) in network meta-analysis.
#' 
#' @param x An R object.
#' @param \dots Additional arguments.
#' 
#' @details
#' For more details, look at the following functions to generate
#' rankograms:
#' \itemize{
#' \item \code{\link{rankogram.netmeta}}
#' \item \code{\link{rankogram.default}}
#' }
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{netmeta}}, \code{\link{netrank}},
#'   \code{\link{plot.rankogram}}, \code{\link[metabook]{Woods2010}}
#'
#' @references
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#'
#' @examples
#' # Examples:
#' # example(rankogram.netmeta)
#' # example(rankogram.default)
#'
#' @rdname rankogram
#' @export rankogram

rankogram <- function(x, ...)
  UseMethod("rankogram")

#' Network meta-analysis of treatments for Parkinson's disease
#' 
#' @description
#' Network meta-analysis comparing the effects of a number of
#' treatments for Parkinson's disease.
#' 
#' The data are the mean lost work-time reduction in patients given
#' dopamine agonists as adjunct therapy in Parkinson’s disease
#' (Franchini et al. 2012). The data are given as sample size, mean
#' and standard deviation in each trial arm. Treatments are placebo
#' and four active drugs. These data are used as an example in the
#' supplemental material of Dias et al. (2013) where placebo is coded
#' as 1 and the four active drugs as 2 to 5.
#' 
#' @name Franchini2012
#' @aliases parkinson
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{Study}}\tab study label \cr
#' \bold{\emph{Treatment1}}\tab treatment 1 \cr
#' \bold{\emph{y1}}\tab treatment effect arm 1 \cr
#' \bold{\emph{sd1}}\tab Standard deviation arm 1 \cr
#' \bold{\emph{n1}}\tab Sample size arm 1 \cr
#' \bold{\emph{Treatment2}}\tab treatment 2 \cr
#' \bold{\emph{y2}}\tab treatment effect arm 2 \cr
#' \bold{\emph{sd2}}\tab Standard deviation arm 2 \cr
#' \bold{\emph{n2}}\tab Sample size arm 2 \cr
#' \bold{\emph{Treatment3}}\tab treatment 3 \cr
#' \bold{\emph{y3}}\tab treatment effect arm 3 \cr
#' \bold{\emph{sd3}}\tab Standard deviation arm 3 \cr
#' \bold{\emph{n3}}\tab Sample size arm 3
#' }
#' 
#' @note
#' The dataset Franchini2012 is identical to dataset
#' \code{\link[metabook]{Franchini2012}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[metabook]{Franchini2012}},
#'   \code{\link[meta]{pairwise}}, \code{\link[meta]{metacont}},
#'   \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
#' 
#' @source
#' Dias S, Sutton AJ, Ades AE and Welton NJ (2013):
#' Evidence synthesis for decision making 2: A generalized linear
#' modeling framework for pairwise and network meta-analysis of
#' randomized controlled trials.
#' \emph{Medical Decision Making},
#' \bold{33}, 607--17
#'
#' Franchini AJ, Dias S, Ades AE, Jansen JP, Welton NJ (2012):
#' Accounting for correlation in network meta-analysis with multi-arm
#' trials.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 142--60
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Franchini2012)
#' 
#' # Example using pairwise() and netmeta():
#' # example(Franchini2012, package = "metabook")

NULL

#' Network graph for objects of class netcomb
#' 
#' @description
#' This function generates a graph of the evidence network.
#' 
#' @param x An object of class \code{netcomb}.
#' @param \dots Additional arguments passed on to
#'   \code{\link{netgraph.netmeta}}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{netgraph.netmeta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Only consider studies including Face-to-face PST (to reduce
#' # runtime of example)
#' #
#' face <- subset(Linde2016, id %in% c(16, 24, 49, 118))
#' 
#' # Use pairwise() to transform data to comparison-based format
#' #
#' pw <- pairwise(treat = int,
#'   event = resp, n = n, studlab = paste(author, year),
#'   data = face, reference = "plac", sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma <- netmeta(pw, common = FALSE)
#' 
#' # Additive model for treatment components (with placebo as inactive
#' # treatment)
#' #
#' cnma <- netcomb(nma, inactive = "placebo")
#'
#' netgraph(cnma)
#' 
#' @method netgraph netcomb
#' @export

netgraph.netcomb <- function(x, ...) {
  
  
  chkclass(x, "netcomb")
  
  
  res <- netgraph(x$x, ...)
  
  
  invisible(res)
}

#' Network meta-analysis of binary outcome data
#' 
#' @description
#' Provides four models for the network meta-analysis of binary data
#' (Mantel-Haenszel method, based on the non-central hypergeometric
#' distribution, penalised logistic regression, and the
#' inverse variance method).
#' 
#' @param event1 Number of events (first treatment).
#' @param n1 Number of observations (first treatment).
#' @param event2 Number of events (second treatment).
#' @param n2 Number of observations (second treatment)
#' @param treat1 Label/Number for first treatment.
#' @param treat2 Label/Number for second treatment.
#' @param studlab An optional - but important! - vector with study
#'   labels (see Details).
#' @param data An optional data frame containing the study
#'   information.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param sm A character string indicating underlying summary measure,
#'   i.e., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"}.
#' @param method A character string indicating which method is to be
#'   used for pooling of studies. One of \code{"MH"},
#'   \code{"NCH"}, \code{"LRP"}, or \code{"Inverse"}, can be abbreviated.
#' @param cc.pooled A logical indicating whether \code{incr} should be
#'   used as a continuity correction, when calculating the network
#'   meta-analysis estimates (ignored for penalised logistic regression and
#'   inverse variance method).
#' @param incr A numerical value which is added to each cell count,
#'   i.e., to the numbers of events and non-events, of all treatment
#'   arms in studies with zero events or non-events in any of the
#'   treatment arms ("continuity correction").
#' @param method.incr A character string indicating which continuity
#'   correction method should be used (\code{"only0"},
#'   \code{"if0all"}, or \code{"all"}), see \code{\link[meta]{metabin}}.
#' @param allstudies A logical indicating whether studies with zero
#'   events or non-events in all treatment arms should be included in
#'   an inverse variance meta-analysis (applies only if \code{method =
#'   "Inverse"} and \code{sm} is equal to either \code{"RR"} or
#'   \code{"OR"}).
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param level.ma The level used to calculate confidence intervals
#'   for network estimates.
#' @param common A logical indicating whether a common effects network
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects network
#'   meta-analysis should be conducted.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed (only considered if \code{method =
#'   "Inverse"}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study (only considered if \code{method =
#'   "Inverse"}).
#' @param reference.group Reference treatment.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect (passed on to
#'   \code{\link{netrank}}, can be abbreviated.
#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
#'   matrices with all treatment effects, and confidence limits will
#'   be printed.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments in printouts.
#' @param tau.preset An optional value for manually setting the
#'   square-root of the between-study variance \eqn{\tau^2} (only
#'   considered if \code{method = "Inverse"}).
#' @param tol.multiarm A numeric for the tolerance for consistency of
#'   treatment estimates in multi-arm studies which are consistent by
#'   design (only considered if \code{method = "Inverse"}).
#' @param tol.multiarm.se A numeric for the tolerance for consistency
#'   of standard errors in multi-arm studies which are consistent by
#'   design (only considered if the argument is not \code{NULL} and
#'   \code{method = "Inverse"}).
#' @param details.chkmultiarm A logical indicating whether treatment
#'   estimates and / or variances of multi-arm studies with
#'   inconsistent results or negative multi-arm variances should be
#'   printed (only considered if \code{method = "Inverse"}).
#' @param details.chkdata A logical indicating whether number of
#'   events and participants of studies with inconsistent data should
#'   be printed.
#' @param sep.trts A character used in comparison names as separator
#'   between treatment labels.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param func.inverse R function used to calculate the pseudoinverse
#'   of the Laplacian matrix L (see \code{\link{netmeta}}).
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param title Title of meta-analysis / systematic review.
#' @param keepdata A logical indicating whether original data(set)
#'   should be kept in netmeta object.
#' @param addincr Deprecated argument (replaced by 'method.incr');
#'   see \code{\link[meta]{metabin}}.
#' @param allincr Deprecated argument (replaced by 'method.incr');
#'   see \code{\link[meta]{metabin}}.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if studies are excluded from meta-analysis due to zero
#'   standard errors).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' This function implements four models for the network meta-analysis
#' of binary data:
#' \itemize{
#' \item The Mantel-Haenszel network meta-analysis model, as described
#'   in Efthimiou et al. (2019) (\code{method = "MH"});
#' \item a network meta-analysis model using the non-central
#'   hypergeometric distribution with the Breslow approximation, as
#'   described in Stijnen et al. (2010) (\code{method = "NCH"});
#' \item a logistic regression with penalised likelihood, as described
#'   in Evrenoglou et al. (2022) (\code{method = "LRP"});
#' \item the inverse variance method for network meta-analysis
#'   (\code{method = "Inverse"}), also provided by
#'   \code{\link{netmeta}}.
#' }
#' 
#' Comparisons belonging to multi-arm studies are identified by
#' identical study labels (argument \code{studlab}). It is therefore
#' important to use identical study labels for all comparisons
#' belonging to the same multi-arm study.
#' 
#' Data entry for this function is in \emph{contrast-based} format,
#' that is, each line of the data corresponds to a single pairwise
#' comparison between two treatments (arguments \code{treat1},
#' \code{treat2}, \code{event1}, \code{n1}, \code{event2}, and
#' \code{n2}). If data are provided in \emph{arm-based} format, that
#' is, number of events and participants are given for each treatment
#' arm separately, function \code{\link[meta]{pairwise}} can be used to
#' transform the data to \emph{contrast-based} format (see help page
#' of function \code{\link[meta]{pairwise}}).
#' 
#' Note, all pairwise comparisons must be provided for a multi-arm
#' study. Consider a multi-arm study of \emph{p} treatments with known
#' variances. For this study, the number of events and observations
#' must be provided for each treatment, for each of \emph{p}(\emph{p}
#' - 1) / 2 possible comparisons in separate lines in the data. For
#' instance, a three-arm study contributes three pairwise comparisons,
#' a four-arm study even six pairwise comparisons. Function
#' \code{\link[meta]{pairwise}} automatically calculates all pairwise
#' comparisons for multi-arm studies.
#' 
#' For \code{method = "Inverse"}, both common and random effects
#' models are calculated regardless of values chosen for arguments
#' \code{common} and \code{random}. Accordingly, the network estimates
#' for the random effects model can be extracted from component
#' \code{TE.random} of an object of class \code{"netmeta"} even if
#' argument \code{random = FALSE}.  However, all functions in R
#' package \bold{netmeta} will adequately consider the values for
#' \code{common} and \code{random}. E.g. function
#' \code{\link{print.summary.netmeta}} will not print results for the
#' random effects model if \code{random = FALSE}.
#' 
#' For the random-effects model, the direct treatment estimates are
#' based on the common between-study variance \eqn{\tau^2} from the
#' network meta-analysis.
#'
#' For \code{method = "MH"} and \code{method = "NCH"}, only a common
#' effects model is available.
#' 
#' By default, treatment names are not abbreviated in
#' printouts. However, in order to get more concise printouts,
#' argument \code{nchar.trts} can be used to define the minimum number
#' of characters for abbreviated treatment names (see
#' \code{\link{abbreviate}}, argument \code{minlength}). R function
#' \code{\link{treats}} is utilised internally to create abbreviated
#' treatment names.
#' 
#' Names of treatment comparisons are created by concatenating
#' treatment labels of pairwise comparisons using \code{sep.trts} as
#' separator (see \code{\link{paste}}). These comparison names are
#' used in the covariance matrices \code{Cov.common} and
#' \code{Cov.random} and in some R functions, e.g,
#' \code{\link{decomp.design}}. By default, a colon is used as the
#' separator. If any treatment label contains a colon the following
#' characters are used as separator (in consecutive order):
#' \code{"-"}, \code{"_"}, \code{"/"}, \code{"+"}, \code{"."},
#' \code{"|"}, and \code{"*"}. If all of these characters are used in
#' treatment labels, a corresponding error message is printed asking
#' the user to specify a different separator.
#'
#' @return
#' An object of class \code{netmetabin} and \code{netmeta} with
#' corresponding \code{print}, \code{summary}, \code{forest}, and
#' \code{netrank} functions. The object is a list containing the
#' following components:
#' \item{studlab, treat1, treat2}{As defined above.}
#' \item{n1, n2, event1, event2}{As defined above.}
#' \item{TE}{Estimate of treatment effect, i.e. difference between
#'   first and second treatment (e.g. log odds ratio).}
#' \item{seTE}{Standard error of treatment estimate.}
#' \item{k}{Total number of studies.}
#' \item{m}{Total number of pairwise comparisons.}
#' \item{n}{Total number of treatments.}
#' \item{d}{Total number of designs (corresponding to the unique set
#'   of treatments compared within studies).}
#' \item{trts}{Treatments included in network meta-analysis.}
#' \item{k.trts}{Number of studies evaluating a treatment.}
#' \item{n.trts}{Number of observations receiving a treatment.}
#' \item{events.trts}{Number of events observed for a treatment.}
#' \item{studies}{Study labels coerced into a factor with its levels
#'   sorted alphabetically.}
#' \item{narms}{Number of arms for each study.}
#' \item{designs}{Unique list of designs present in the network. A
#'   design corresponds to the set of treatments compared within a
#'   study.}
#' \item{TE.common, seTE.common}{\emph{n}x\emph{n} matrix with estimated
#'   overall treatment effects and standard errors for common effects
#'   model.}
#' \item{lower.common, upper.common}{\emph{n}x\emph{n} matrices with
#'   lower and upper confidence interval limits for common effects
#'   model.}
#' \item{statistic.common, pval.common}{\emph{n}x\emph{n} matrices with
#'   z-value and p-value for test of overall treatment effect under
#'   common effects model.}
#' \item{TE.random, seTE.random}{\emph{n}x\emph{n} matrix with
#'   estimated overall treatment effects and standard errors for
#'   random effects model (only available if \code{method =
#'   "Inverse"}).}
#' \item{lower.random, upper.random}{\emph{n}x\emph{n} matrices with
#'   lower and upper confidence interval limits for random effects
#'   model (only available if \code{method = "Inverse"}).}
#' \item{statistic.random, pval.random}{\emph{n}x\emph{n} matrices
#'   with z-value and p-value for test of overall treatment effect
#'   under random effects model (only available if \code{method =
#'   "Inverse"}).}
#' \item{TE.direct.common, seTE.direct.common}{\emph{n}x\emph{n} matrix
#'   with estimated treatment effects and standard errors from direct
#'   evidence under common effects model.}
#' \item{lower.direct.common, upper.direct.common}{\emph{n}x\emph{n}
#'   matrices with lower and upper confidence interval limits from
#'   direct evidence under common effects model.}
#' \item{statistic.direct.common, pval.direct.common}{\emph{n}x\emph{n}
#'   matrices with z-value and p-value for test of overall treatment
#'   effect from direct evidence under common effects model.}
#' \item{TE.direct.random, seTE.direct.random}{\emph{n}x\emph{n}
#'   matrix with estimated treatment effects and standard errors from
#'   direct evidence under random effects model (only available if
#'   \code{method = "Inverse"}).}
#' \item{lower.direct.random, upper.direct.random}{\emph{n}x\emph{n}
#'   matrices with lower and upper confidence interval limits from
#'   direct evidence under random effects model (only available if
#'   \code{method = "Inverse"}).}
#' \item{statistic.direct.random,
#'   pval.direct.random}{\emph{n}x\emph{n} matrices with z-value and
#'   p-value for test of overall treatment effect from direct evidence
#'   under random effects model (only available if \code{method =
#'   "Inverse"}).}
#' \item{Q}{Overall heterogeneity / inconsistency statistic. (only
#'   available if \code{method = "Inverse"})}
#' \item{df.Q}{Degrees of freedom for test of heterogeneity /
#'   inconsistency.}
#' \item{pval.Q}{P-value for test of heterogeneity / inconsistency.}
#' \item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
#'   limits (only available if \code{method = "Inverse"}).}
#' \item{tau}{Square-root of between-study variance (only available if
#'   \code{method = "Inverse"}).}
#' \item{Q.heterogeneity}{Overall heterogeneity statistic. (only
#'   available if \code{method = "Inverse"})}
#' \item{df.Q.heterogeneity}{Degrees of freedom for test of overall
#'   heterogeneity.}
#' \item{pval.Q.heterogeneity}{P-value for test of overall
#'   heterogeneity.}
#' \item{Q.inconsistency}{Overall inconsistency statistic.}
#' \item{df.Q.inconsistency}{Degrees of freedom for test of overall
#'   inconsistency.}
#' \item{pval.Q.inconsistency}{P-value for test of overall
#'   inconsistency.}
#' \item{A.matrix}{Adjacency matrix (\emph{n}x\emph{n}).}
#' \item{H.matrix.common}{Hat matrix (\emph{m}x\emph{m})}
#' \item{n.matrix}{\emph{n}x\emph{n} matrix with number of
#'   observations in direct comparisons.}
#' \item{events.matrix}{\emph{n}x\emph{n} matrix with number of events
#'   in direct comparisons.}
#' \item{sm, method, level, level.ma}{As defined above.}
#' \item{incr, method.incr, allstudies, cc.pooled}{As defined
#'   above.}
#' \item{addincr, allincr}{As defined above.}
#' \item{common, random}{As defined above.}
#' \item{prediction, level.predict}{As defined above.}
#' \item{reference.group, baseline.reference, small.values, all.treatments}{As
#'   defined above.}
#' \item{seq, tau.preset, tol.multiarm, tol.multiarm.se}{As defined
#'   above.}
#' \item{details.chkmultiarm, details.chkdata}{As defined above.}
#' \item{sep.trts, nchar.trts, overall.hetstat}{As defined above.}
#' \item{backtransf, title, warn, warn.deprecated}{As defined above.}
#' \item{data}{Dataset (in contrast-based format).}
#' \item{data.design}{List with data in arm-based format (each list
#'   element corresponds to a single design).}
#' \item{call}{Function call.}
#' \item{version}{Version of R package netmeta used to create object.}
#' 
#' @author Orestis Efthimiou \email{oremiou@@gmail.com}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Theodoros Evrenoglou \email{theodoros.evrenoglou@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link[meta]{pairwise}}, \code{\link{netmeta}},
#'   \code{\link[metabook]{Dong2013}}
#' 
#' @references
#' Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
#' (2019):
#' A Mantel-Haenszel model for network meta-analysis of rare events.
#' \emph{Statistics in Medicine},
#' \bold{38}, 2992--3012
#' 
#' Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A (2022):
#' Network Meta-analysis of rare events using penalized likelihood regression.
#' \emph{Statistics in Medicine},
#' \bold{41}, 5203--19.
#' 
#' Senn S, Gavini F, Magrez D, Scheen A (2013):
#' Issues in performing a network meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{22}, 169--89
#' 
#' Stijnen T, Hamza TH, Ozdemir P (2010):
#' Random effects meta-analysis of event outcome in the framework of
#' the generalized linear mixed model with applications in sparse
#' data.
#' \emph{Statistics in Medicine},
#' \bold{29}, 3046--67
#' 
#' @examples
#' \donttest{
#' # Only consider first four studies (to reduce runtime of example)
#' #
#' first4 <- subset(Dong2013, id <= 4)
#' 
#' # Transform data from long arm-based format to contrast-based
#' # format. Argument 'sm' has to be used for odds ratio as summary
#' # measure; by default the risk ratio is used in the metabin
#' # function called internally.
#' #
#' pw1 <- pairwise(treatment, death, randomized, studlab = id,
#'   data = first4, sm = "OR")
#' 
#' # Conduct Mantel-Haenszel network meta-analysis (without continuity
#' # correction)
#' #
#' nb1 <- netmetabin(pw1, ref = "plac")
#' nb1
#' 
#' # Obtain the league table
#' #
#' netleague(nb1)
#' 
#' # Mantel-Haenszel network meta-analysis for the whole
#' # dataset: example(Dong2013)
#' }
#' 
#' @export netmetabin

netmetabin <- function(event1, n1, event2, n2,
                       treat1, treat2, studlab,
                       data = NULL, subset = NULL,
                       sm,
                       method = "MH",
                       cc.pooled = FALSE,
                       #
                       incr, method.incr, allstudies,
                       #
                       level = gs("level"),
                       level.ma = gs("level.ma"),
                       common = gs("common"),
                       random = method %in% c("Inverse", "LRP") &
                         (gs("random") | !is.null(tau.preset)),
                       ##
                       prediction = gs("prediction"),
                       level.predict = gs("level.predict"),
                       ##
                       reference.group,
                       baseline.reference = gs("baseline.reference"),
                       small.values = gs("small.values"),
                       all.treatments = gs("all.treatments"),
                       seq = gs("seq"),
                       #
                       tau.preset = NULL,
                       ##
                       tol.multiarm = 0.001,
                       tol.multiarm.se = NULL,
                       details.chkmultiarm = FALSE,
                       details.chkdata = TRUE,
                       ##
                       sep.trts = ":",
                       nchar.trts = 666,
                       ##
                       func.inverse = invmat,
                       ##
                       overall.hetstat = gs("overall.hetstat"),
                       backtransf = gs("backtransf"),
                       ##
                       title = gs("title"),
                       keepdata = gs("keepdata"),
                       #
                       addincr, allincr,
                       #
                       warn = gs("warn"),
                       warn.deprecated = gs("warn.deprecated"),
                       ...) {
  
  
  ##
  ##
  ## (1) Check arguments
  ##
  ##
  
  missing.common <- missing(common)
  missing.random <- missing(random)
  #
  missing.level.ma <- missing(level.ma)
  missing.sep.trts <- missing(sep.trts)
  #
  missing.incr <- missing(incr)
  missing.method.incr <- missing(method.incr)
  missing.allstudies <- missing(allstudies)
  #
  missing.addincr <- missing(addincr)
  missing.allincr <- missing(allincr)
  #
  modtext <-
    paste0("must be equal to 'MH' (Mantel-Haenszel, the default), ",
           "'NCH' (common-effects non-central hypergeometric), ",
           "'LRP' (penalised logistic regression), or ",
           "'Inverse' (classic network meta-analysis).")
  method <- setchar(method, c("Inverse", "MH", "NCH", "LRP"), modtext)
  is.mh.nch <- !(method %in% c("Inverse", "LRP"))
  is.lrp <- method == "LRP"
  is.iv <- method == "Inverse"
  #
  chklogical(cc.pooled)
  ##
  chklevel(level)
  chklevel(level.predict)
  ##
  chklogical(prediction)
  #
  missing.reference.group <- missing(reference.group)
  #
  baseline.reference <- replaceNULL(baseline.reference, TRUE)
  chklogical(baseline.reference)
  #
  small.values <- setsv(small.values)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  level.ma <- deprecated(level.ma, missing.level.ma, args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  ##
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <-
    deprecated(random, missing.random, args, "comb.random", warn.deprecated)
  chklogical(random)
  ##
  if (is.mh.nch & !common) {
    warning("Argument 'common' set to TRUE for Mantel-Haenszel ",
            "method and non-central hypergeometric distribution.",
            call. = FALSE)
    common <- TRUE
  }
  ##
  if (is.mh.nch & random) {
    warning("Argument 'random' set to FALSE for Mantel-Haenszel ",
            "method and non-central hypergeometric distribution.",
            call. = FALSE)
    random <- FALSE
  }
  ##
  if (is.mh.nch & prediction) {
    warning("Argument 'prediction' set to FALSE for Mantel-Haenszel ",
            "method and non-central hypergeometric distribution.",
            call. = FALSE)
    prediction <- FALSE
  }
  #
  if (!is.null(tau.preset))
    chknumeric(tau.preset, min = 0, length = 1)
  ##
  chknumeric(tol.multiarm, min = 0, length = 1)
  if (!is.null(tol.multiarm.se))
    chknumeric(tol.multiarm.se, min = 0, length = 1)
  chklogical(details.chkmultiarm)
  chklogical(details.chkdata)
  ##
  chkchar(sep.trts)
  chknumeric(nchar.trts, min = 1, length = 1)
  #
  overall.hetstat <- replaceNULL(overall.hetstat, TRUE)
  chklogical(overall.hetstat)
  chklogical(backtransf)
  #
  chkchar(title)
  chklogical(keepdata)
  chklogical(warn)
  
  
  ##
  ##
  ## (2) Read data
  ##
  ##
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  if (nulldata)
    data <- sfsp
  ##
  ## Catch 'event1', 'event2', 'n1', 'n2', 'treat1', 'treat2', and
  ## 'studlab' from data:
  ##
  event1 <- catch("event1", mc, data, sfsp)
  #
  avail.reference.group.pairwise <- FALSE
  #
  if (inherits(event1, "pairwise")) {
    is.pairwise <- TRUE
    ##
    if (missing.incr)
      incr <- replaceNULL(attr(event1, "incr"), gs("incr"))
    if (missing.method.incr)
      method.incr <- replaceNULL(attr(event1, "method.incr"), gs("method.incr"))
    if (missing(allstudies))
      allstudies <- replaceNULL(attr(event1, "allstudies"), gs("allstudies"))
    #
    missing.incr <- FALSE
    missing.method.incr <- FALSE
    missing.allstudies <- FALSE
    #
    if (missing(sm) & is.iv)
      sm <- replaceNULL(attr(event1, "sm"), gs("smbin"))
    else if (method != "Inverse") {
      if (!missing(sm) && tolower(sm) != "or")
        warning("Argument 'sm' set to 'OR'.", call. = FALSE)
      sm <- "OR"
    }
    #
    if (missing.reference.group) {
      reference.group <- attr(event1, "reference.group")
      #
      if (is.null(reference.group))
        reference.group <- ""
      else
        avail.reference.group.pairwise <- TRUE
    }
    #
    keep.all.comparisons <- attr(event1, "keep.all.comparisons")
    if (!is.null(keep.all.comparisons) && !keep.all.comparisons)
      stop("First argument is a pairwise object created with ",
           "'keep.all.comparisons = FALSE'.",
           call. = TRUE)
    #
    n1 <- event1$n1
    event2 <- event1$event2
    n2 <- event1$n2
    treat1 <- event1$treat1
    treat2 <- event1$treat2
    studlab <- event1$studlab
    ##
    pairdata <- event1
    data <- event1
    ##
    event1 <- event1$event1
    ##
    chknull(event1, text = "Variable")
    chknull(n1, text = "Variable")
    chknull(event2, text = "Variable")
    chknull(n2, text = "Variable")
    chknull(treat1, text = "Variable")
    chknull(treat2, text = "Variable")
    chknull(studlab, text = "Variable")
  }
  else {
    is.pairwise <- FALSE
    ##
    if (missing.incr)
      incr <- gs("incr")
    if (missing(method.incr))
      method.incr <- gs("method.incr")
    if (missing(allstudies))
      allstudies <- gs("allstudies")
    #
    if (missing(sm) & is.iv) {
      if (!nulldata && !is.null(attr(data, "sm")))
        sm <- attr(data, "sm")
      else
        sm <- ""
    }
    else if (method != "Inverse") {
      if (!missing(sm) && tolower(sm) != "or")
        warning("Argument 'sm' set to 'OR'.", call. = FALSE)
      sm <- "OR"
    }
    ##
    event2 <- catch("event2", mc, data, sfsp)
    ##
    n1 <- catch("n1", mc, data, sfsp)
    n2 <- catch("n2", mc, data, sfsp)
    ##
    treat1 <- catch("treat1", mc, data, sfsp)
    treat2 <- catch("treat2", mc, data, sfsp)
    ##
    studlab <- catch("studlab", mc, data, sfsp)
    #
    if (!missing.incr)
      incr <- catch("incr", mc, data, sfsp)
  }
  #
  addincr <-
    deprecated2(method.incr, missing.method.incr, addincr, missing.addincr,
                warn.deprecated)
  allincr <-
    deprecated2(method.incr, missing.method.incr, allincr, missing.allincr,
                warn.deprecated)
  #
  if (missing.method.incr) {
    method.incr <- gs("method.incr")
    ##
    if (is.logical(addincr) && addincr)
      method.incr <- "all"
    else if (is.logical(allincr) && allincr)
      method.incr <- "if0all"
  }
  #
  addincr <- allincr <- FALSE
  if (!(sm == "ASD" | method %in% c("Peto", "GLMM"))) {
    if (method.incr == "all")
      addincr <- TRUE
    else if (method.incr == "if0all")
      allincr <- TRUE
  }
  #
  chknull(event1)
  chknull(n1)
  chknull(event2)
  chknull(n2)
  chknull(treat1)
  chknull(treat2)
  chknull(studlab)
  #
  chknumeric(event1)
  chknumeric(n1)
  chknumeric(event2)
  chknumeric(n2)
  ##
  k.Comp <- length(event1)
  ##
  if (is.factor(treat1))
    treat1 <- as.character(treat1)
  if (is.factor(treat2))
    treat2 <- as.character(treat2)
  if (is.factor(studlab))
    studlab <- as.character(studlab)
  ##
  ## Remove leading and trailing whitespace
  ##
  treat1 <- rmSpace(rmSpace(treat1, end = TRUE))
  treat2 <- rmSpace(rmSpace(treat2, end = TRUE))
  ##
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  # Drop comparisons with missing events
  #
  if (!is.iv & (any(is.na(event1)) | any(is.na(event2)))) {
    keepEv <- !(is.na(event1) | is.na(event2))
    #
    if (warn) {
      prt_s <- sum(!keepEv) > 1
      #
      dat_Ev <- data.frame(studlab, treat1, treat2, event1, event2)[!keepEv, ]
      #
      warning("Comparison", if (prt_s) "s",
              " with missing events not considered ",
              "in network meta-analysis.",
              call. = FALSE)
      #
      cat("Comparison", if (prt_s) "s",
          " not considered in network meta-analysis:\n",
          sep = "")
      prmatrix(dat_Ev,
               quote = FALSE, right = TRUE,
               rowlab = rep("", sum(!keepEv)))
    }
    #
    studlab <- studlab[keepEv]
    treat1 <- treat1[keepEv]
    treat2 <- treat2[keepEv]
    #
    event1 <- event1[keepEv]
    event2 <- event2[keepEv]
    n1 <- n1[keepEv]
    n2 <- n2[keepEv]
    #
    if (!missing.subset)
      subset <- subset[keepEv]
    #
    if (nulldata & is.pairwise)
      pairdata <- pairdata[keepEv, , drop = FALSE]
  }
  #
  # Drop comparisons with missing sample sizes
  #
  if (!is.iv & (any(is.na(n1)) | any(is.na(n2)))) {
    keepN <- !(is.na(n1) | is.na(n2))
    #
    if (warn) {
      prt_s <- sum(!keepN) > 1
      #
      dat_N <- data.frame(studlab, treat1, treat2, n1, n2)[!keepN, ]
      #
      warning("Comparison", if (prt_s) "s",
              " with missing sample sizes not considered ",
              "in network meta-analysis.",
              call. = FALSE)
      #
      cat("Comparison", if (prt_s) "s",
          " not considered in network meta-analysis:\n",
          sep = "")
      prmatrix(dat_N,
               quote = FALSE, right = TRUE,
               rowlab = rep("", sum(!keepN)))
    }
    #
    studlab <- studlab[keepN]
    treat1 <- treat1[keepN]
    treat2 <- treat2[keepN]
    #
    event1 <- event1[keepN]
    event2 <- event2[keepN]
    n1 <- n1[keepN]
    n2 <- n2[keepN]
    #
    if (!missing.subset)
      subset <- subset[keepN]
    #
    if (nulldata & is.pairwise)
      pairdata <- pairdata[keepN, , drop = FALSE]
  }
  #
  # Keep original order of studies
  #
  .order <- seq_along(studlab)
  
  
  ##
  ##
  ## (2b) Store complete dataset in list object data
  ##
  ##
  if (nulldata & !is.pairwise)
    data <- data.frame(.event1 = event1)
  else if (nulldata & is.pairwise) {
    data <- pairdata
    data$.order <- .order
    data$.event1 <- event1
  }
  else
    data$.event1 <- event1
  ##
  data$.n1 <- n1
  data$.event2 <- event2
  data$.n2 <- n2
  data$.treat1 <- treat1
  data$.treat2 <- treat2
  data$.studlab <- studlab
  data$.order <- .order
  ##
  if (!missing.subset) {
    if (length(subset) == dim(data)[1])
      data$.subset <- subset
    else {
      data$.subset <- FALSE
      data$.subset[subset] <- TRUE
    }
  }
  ##
  chklogical(incr)
  chklogical(allstudies)
  ##
  m.data <- metabin(event1, n1, event2, n2,
                    sm = sm, method = "Inverse",
                    incr = incr, method.incr = method.incr,
                    allstudies = allstudies,
                    method.tau = "DL", method.tau.ci = "",
                    warn = FALSE,
                    warn.deprecated = FALSE)
  ##
  data$.TE <- m.data$TE
  data$.seTE <- m.data$seTE
  
  
  ##
  ##
  ## (3) Use subset for analysis
  ##
  ##
  if (!missing.subset) {
    if ((is.logical(subset) & (sum(subset) > k.Comp)) ||
        (length(subset) > k.Comp))
      stop("Length of subset is larger than number of studies.",
           call. = FALSE)
    ##
    studlab <- studlab[subset]
    treat1 <- treat1[subset]
    treat2 <- treat2[subset]
    ##
    event1 <- event1[subset]
    event2 <- event2[subset]
    n1 <- n1[subset]
    n2 <- n2[subset]
    ##
    .order <- .order[subset]
  }
  ##
  ## Check for correct treatment order within comparison
  ##
  wo <- treat1 > treat2
  ##
  if (any(wo)) {
    tevent1 <- event1
    event1[wo] <- event2[wo]
    event2[wo] <- tevent1[wo]
    ##
    tn1 <- n1
    n1[wo] <- n2[wo]
    n2[wo] <- tn1[wo]
    ##
    ttreat1 <- treat1
    treat1[wo] <- treat2[wo]
    treat2[wo] <- ttreat1[wo]
  }
  ##
  labels <- sort(unique(c(treat1, treat2)))
  ##
  if (compmatch(labels, sep.trts)) {
    if (!missing.sep.trts)
      warning("Separator '", sep.trts, "' used in at least ",
              "one treatment label. Try to use predefined separators: ",
              "':', '-', '_', '/', '+', '.', '|', '*'.",
              call. = FALSE)
    ##
    if (!compmatch(labels, ":"))
      sep.trts <- ":"
    else if (!compmatch(labels, "-"))
      sep.trts <- "-"
    else if (!compmatch(labels, "_"))
      sep.trts <- "_"
    else if (!compmatch(labels, "/"))
      sep.trts <- "/"
    else if (!compmatch(labels, "+"))
      sep.trts <- "+"
    else if (!compmatch(labels, "."))
      sep.trts <- "-"
    else if (!compmatch(labels, "|"))
      sep.trts <- "|"
    else if (!compmatch(labels, "*"))
      sep.trts <- "*"
    else
      stop("All predefined separators (':', '-', '_', '/', '+', '.', '|', '*') ",
           "are used in at least one treatment label.\n   ",
           "Please specify a different character that should be used ",
           " as separator (argument 'sep.trts').",
           call. = FALSE)
  }
  #
  # Set reference group
  #
  if (missing.reference.group & !avail.reference.group.pairwise) {
    go.on <- TRUE
    i <- 0
    while (go.on) {
      i <- i + 1
      sel.i <-
        !is.na(event1) & !is.na(n1) & !is.na(event2) & !is.na(n2) &
        (treat1 == labels[i] | treat2 == labels[i])
      if (sum(sel.i) > 0) {
        go.on <- FALSE
        reference.group <- labels[i]
      }
      else if (i == length(labels)) {
        go.on <- FALSE
        reference.group <- ""
      }
    }
  }
  #
  if (is.null(all.treatments))
    if (reference.group == "")
      all.treatments <- TRUE
  else
    all.treatments <- FALSE
  #
  # Check reference group
  #
  if (reference.group != "")
    reference.group <- setref(reference.group, labels)
  #
  rm(labels)


  ##
  ##
  ## (4) Additional checks
  ##
  ##
  if (any(treat1 == treat2))
    stop("Treatments must be different (arguments 'treat1' and 'treat2').",
         call. = FALSE)
  ##
  if (length(studlab) != 0)
    studlab <- as.character(studlab)
  else {
    if (warn)
      warning("No information given for argument 'studlab'. ",
              "Assuming that comparisons are from independent studies.",
              call. = FALSE)
    studlab <- seq(along = event1)
  }
  ##
  ## Check for correct number of comparisons
  ##
  tabnarms <- table(studlab)
  sel.narms <- !is_wholenumber((1 + sqrt(8 * tabnarms + 1)) / 2)
  ##
  if (sum(sel.narms) == 1)
    stop(paste0("Study '", names(tabnarms)[sel.narms],
                "' has a wrong number of comparisons.",
                "\n  Please provide data for all treatment comparisons",
                " (two-arm: 1; three-arm: 3; four-arm: 6, ...)."),
         call. = FALSE)
  if (sum(sel.narms) > 1)
    stop(paste0("The following studies have a wrong number of comparisons: ",
                paste(paste0("'", names(tabnarms)[sel.narms], "'"),
                      collapse = " - "),
                "\n  Please provide data for all treatment comparisons",
                " (two-arm: 1; three-arm: 3; four-arm: 6, ...)."),
         call. = FALSE)
  ##
  ## Check number of subgraphs
  ##
  n.subnets <- netconnection(treat1, treat2, studlab)$n.subnets
  ##
  if (n.subnets > 1)
    stop(paste0("Network consists of ", n.subnets, " separate sub-networks.\n",
                "  Use R function 'netconnection' to identify sub-networks."),
         call. = FALSE)
  #
  chknumeric(incr, min = 0)
  ##
  if (is.mh.nch & length(incr) > 1) {
    warning("Argument 'incr' must be a single value for ",
            "Mantel-Haenszel and common-effects non-central ",
            "hypergeometric method. Set to zero (default).",
            call. = FALSE)
    incr <- 0
  }
  ##
  if (is.mh.nch & cc.pooled == TRUE & all(incr == 0))
    cc.pooled <- FALSE
  ##
  lengthunique <- function(x) length(unique(x))


  ##
  ##
  ## (5) Create analysis dataset
  ##
  ##
  dat.long <- rbind(data.frame(studlab, treat = treat1, event = event1, n = n1,
                               .order,
                               stringsAsFactors = FALSE),
                    data.frame(studlab, treat = treat2, event = event2, n = n2,
                               .order,
                               stringsAsFactors = FALSE))
  ##
  dat.wide <- data.frame(studlab = studlab, treat1 = treat1, treat2 = treat2,
                         event1 = event1, n1 = n1, event2 = event2, n2 = n2,
                         .order,
                         stringsAsFactors = FALSE)
  dat.wide <- dat.wide[order(dat.wide$studlab,
                             dat.wide$treat1, dat.wide$treat2), ]
  ##
  ## Check for correct number of treatments
  ##
  d.long <- unique(dat.long[, c("studlab", "treat", "event", "n")])
  d.long <- d.long[order(d.long$studlab, d.long$treat), , drop = FALSE]
  ##
  tabnarms <- table(d.long$studlab, d.long$treat)
  larger.one <- function(x) any(x > 1)
  sel.study <- apply(tabnarms, 1, larger.one)
  ##
  if (sum(sel.study) == 1) {
    if (details.chkdata) {
      cat("Inconsistent number of events and participants:\n")
      d.l <- d.long[d.long$studlab == rownames(tabnarms)[sel.study], ]
      prmatrix(d.l,
               quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(d.l)))
    }
    stop(paste0("Study '", rownames(tabnarms)[sel.study],
                "' has inconsistent data. Please check the original data."),
         call. = FALSE)
  }
  if (sum(sel.study) > 1) {
    if (details.chkdata) {
      cat("Inconsistent number of events and participants:\n")
      d.l <- d.long[d.long$studlab %in% rownames(tabnarms)[sel.study], ]
      prmatrix(d.l,
               quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(d.l)))
    }
    ##
    stop(paste0("The following studies have inconsistent data: ",
                paste(paste0("'", rownames(tabnarms)[sel.study], "'"),
                      collapse = " - "),
                "\n  Please check the original data."),
         call. = FALSE)
  }
  ##
  get.designs <- function(x) {
    net1 <-
      netmeta(pairwise(studlab = x$studlab, treat = x$treat,
                       event = x$event, n = x$n,
                       sm = "RD"),
              warn = FALSE)
    ##
    if (net1$n == 2)
      res <- data.frame(studlab = net1$studlab, design = net1$designs)
    else
      res <- nma_krahn(net1)$studies[, c("studlab", "design")]
    ##
    res$design <- as.character(res$design)
    res <- unique(res)
    ##
    res
  }
  ##
  data$.drop <- rep(FALSE, nrow(data))
  ##
  ## Add variable 'non.event'
  ##
  dat.long$non.event <- dat.long$n - dat.long$event
  ##
  dat.wide$non.event1 <- dat.wide$n1 - dat.wide$event1
  dat.wide$non.event2 <- dat.wide$n2 - dat.wide$event2
  ##
  ## Remove duplicated rows from dataset (due to multi-arm studies)
  ##
  dupl <- duplicated(dat.long[, c("studlab", "treat", "event", "n")])
  ##
  if (any(dupl))
    dat.long <- dat.long[!dupl, ]
  ##
  rm(dupl)
  ##
  all.designs <- get.designs(dat.long)
  ##
  ## Add variable 'design' with study design
  ##
  tdat.design <- get.designs(dat.long)
  ##
  dat.long <- merge(dat.long, tdat.design, by = "studlab",
                    all.x = TRUE)
  dat.wide <- merge(dat.wide, tdat.design, by = "studlab",
                    all.x = TRUE)
  ##
  dat.long <- dat.long[order(dat.long$.order), ]
  dat.wide <- dat.wide[order(dat.wide$.order), ]
  ##
  names(tdat.design) <- c("studlab", ".design")
  ##
  data <- merge(data, tdat.design,
                by.x = ".studlab", by.y = "studlab",
                all.x = TRUE)
  data <- data[order(data$.order), ]
  ##
  rm(tdat.design)
  
  
  ##
  ##
  ## (6) Stage I: setting up the data (Efthimiou et al., 2019)
  ##
  ##
  ## Step i. Remove all-zero or all-event studies (only MH and NCH methods)
  ##
  if (is.mh.nch) {
    events.study <- with(dat.long, tapply(event, studlab, sum))
    nonevents.study <- with(dat.long, tapply(n - event, studlab, sum))
    ##
    if (any(events.study == 0) | any(nonevents.study == 0)) {
      zeroevents <- events.study == 0
      allevents <- nonevents.study == 0 & events.study != 0
      keep <- !(zeroevents | allevents)
      ##
      if (warn) {
        if (sum(zeroevents) == 1)
          warning("Study '", names(events.study)[zeroevents],
                  "' without any events excluded from network meta-analysis.",
                  call. = FALSE)
        else if (sum(zeroevents) > 1)
          warning("Studies without any events excluded ",
                  "from network meta-analysis: ",
                  paste(paste0("'", names(events.study)[zeroevents], "'"),
                        collapse = " - "),
                  call. = FALSE)
        ##
        if (sum(allevents) == 1)
          warning("Study '", names(nonevents.study)[allevents],
                  "' with all events excluded from network meta-analysis.",
                  call. = FALSE)
        else if (sum(allevents) > 1)
          warning("Studies with all events excluded ",
                  "from network meta-analysis: ",
                  paste(paste0("'", names(nonevents.study)[allevents], "'"),
                        collapse = " - "),
                  call. = FALSE)
      }
      ##
      dat.long <- dat.long[dat.long$studlab %in% names(events.study)[keep], ,
                           drop = FALSE]
      dat.wide <- dat.wide[dat.wide$studlab %in% names(events.study)[keep], ,
                           drop = FALSE]
      ##
      data$.drop <- data$.drop | data$studlab %in% names(events.study)[!keep]
      ##
      rm(zeroevents, allevents, keep)
    }
    ##
    rm(events.study)
  }
  ##
  ## Add variable 'study' with study numbers
  ##
  dat.long$study <- as.numeric(factor(dat.long$studlab,
                                      levels = unique(dat.long$studlab)))
  dat.wide$study <- as.numeric(factor(dat.wide$studlab,
                                      levels = unique(dat.wide$studlab)))
  ##
  ## Sort by study and treatment
  ##
  dat.long <- dat.long[order(dat.long$studlab, dat.long$treat), ]
  dat.wide <- dat.wide[order(dat.wide$studlab,
                              dat.wide$treat1, dat.wide$treat2), ]
  ##
  dat.long$incr <- 0
  dat.wide$incr <- 0
  data$.incr <- 0
  ##
  ## Step iii. Drop treatment arms without or all events from individual
  ##           designs (argument 'cc.pooled' is FALSE) or add
  ##           increment if argument 'cc.pooled' is TRUE and argument
  ##           'incr' is larger than zero
  ##
  if (is.mh.nch) {
    ##
    events.arm <- with(dat.long, tapply(event, list(design, treat), sum))
    nonevents.arm <- with(dat.long, tapply(n - event, list(design, treat), sum))
    ##
    if (any(events.arm == 0, na.rm = TRUE) |
        any(nonevents.arm == 0, na.rm = TRUE)) {
      ##
      ## Identify entries without events
      ##
      zeroevents <- events.arm == 0
      zerocells <- as.data.frame(which(zeroevents, arr.ind = TRUE),
                                 stringsAsFactors = FALSE)
      ##
      zerocells$design <- rownames(zeroevents)[zerocells$row]
      zerocells$treat <- colnames(zeroevents)[zerocells$col]
      ##
      zero.long <- rep(0, nrow(dat.long))
      zero.wide <- rep(0, nrow(dat.wide))
      zero.data <- rep(0, nrow(data))
      ##
      for (i in seq_along(zerocells$design)) {
        if (!cc.pooled)
          zero.long <- zero.long + (dat.long$design == zerocells$design[i] &
                                    dat.long$treat == zerocells$treat[i])
        else
          zero.long <- zero.long + (dat.long$design == zerocells$design[i])
        ##
        zero.wide <- zero.wide + (dat.wide$design == zerocells$design[i] &
                                  (dat.wide$treat1 == zerocells$treat[i] |
                                   dat.wide$treat2 == zerocells$treat[i]))
        ##
        zero.data <- zero.data + (data$.design == zerocells$design[i] &
                                  (data$.treat1 == zerocells$treat[i] |
                                   data$.treat2 == zerocells$treat[i]))
      }
      ##
      zero.long <- zero.long > 0
      zero.wide <- zero.wide > 0
      zero.data <- zero.data > 0
      ##
      ## Identify entries with all events
      ##
      allevents <- nonevents.arm == 0
      allcells <- as.data.frame(which(allevents, arr.ind = TRUE),
                                 stringsAsFactors = FALSE)
      ##
      allcells$design <- rownames(allevents)[allcells$row]
      allcells$treat <- colnames(allevents)[allcells$col]
      ##
      all.long <- rep(0, nrow(dat.long))
      all.wide <- rep(0, nrow(dat.wide))
      all.data <- rep(0, nrow(data))
      ##
      for (i in seq_along(allcells$design)) {
        if (!cc.pooled)
          all.long <- all.long + (dat.long$design == allcells$design[i] &
                                  dat.long$treat == allcells$treat[i])
        else
          all.long <- all.long + (dat.long$design == allcells$design[i])
        ##
        all.wide <- all.wide + (dat.wide$design == allcells$design[i] &
                                (dat.wide$treat1 == allcells$treat[i] |
                                 dat.wide$treat2 == allcells$treat[i]))
        ##
        all.data <- all.data + (data$.design == allcells$design[i] &
                                (data$.treat1 == allcells$treat[i] |
                                 data$.treat2 == allcells$treat[i]))
      }
      ##
      all.long <- all.long > 0
      all.wide <- all.wide > 0
      all.data <- all.data > 0
      ##
      ## Identify entries with sparse binary data (i.e., with zero or
      ## all events)
      ##
      sparse.long <- zero.long | all.long
      sparse.wide <- zero.wide | all.wide
      sparse.data <- zero.data | all.data
      ##
      if (!cc.pooled) {
        if (warn) {
          if (sum(zeroevents, na.rm = TRUE) == 1)
            warning("Treatment arm '", zerocells$treat,
                    "' without events in design '",
                    zerocells$design, "' excluded from network meta-analysis.",
                    call. = FALSE)
          else if (sum(zeroevents, na.rm = TRUE) > 1)
            warning("Treatment arms without events in a design excluded ",
                    "from network meta-analysis:\n    ",
                    paste0("'",
                           paste0(paste0(zerocells$treat, " in "),
                                  zerocells$design),
                           "'",
                           collapse = " - "),
                    call. = FALSE)
          ##
          if (sum(allevents, na.rm = TRUE) == 1)
            warning("Treatment arm '", allcells$treat,
                    "' with all events in design '",
                    allcells$design, "' excluded from network meta-analysis.",
                    call. = FALSE)
          else if (sum(allevents, na.rm = TRUE) > 1)
            warning("Treatment arms with all events in a design excluded ",
                    "from network meta-analysis:\n    ",
                    paste0("'",
                           paste0(paste0(allcells$treat, " in "),
                                  allcells$design),
                           "'",
                           collapse = " - "),
                    call. = FALSE)
        }
        ##
        dat.long <- dat.long[!sparse.long, , drop = FALSE]
        dat.wide <- dat.wide[!sparse.wide, , drop = FALSE]
        data$.drop <- data$.drop | sparse.data
      }
      else {
        dat.long$incr[sparse.long] <- incr
        ##
        dat.long$event[sparse.long] <- dat.long$event[sparse.long] + incr
        dat.long$non.event[sparse.long] <- dat.long$non.event[sparse.long] + incr
        dat.long$n[sparse.long] <- dat.long$n[sparse.long] + 2 * incr
        ##
        dat.wide$incr[sparse.wide] <- incr
        ##
        data$.incr[sparse.data] <- incr
      }
      ##
      rm(zeroevents, zerocells, zero.long, zero.wide, zero.data,
         allevents, allcells, all.long, all.wide, all.data,
         sparse.long, sparse.wide, sparse.data)
    }
    ##
    rm(events.arm, nonevents.arm)
  }
  ##
  ## Step iv. Remove designs with single treatment arm from dataset
  ##
  if (is.mh.nch) {
    d.single <- with(dat.long, tapply(treat, design, lengthunique))
    ##
    if (any(d.single == 1, na.rm = TRUE)) {
      single <- !is.na(d.single) & d.single == 1
      design.single <- names(d.single)[single]
      ##
      if (warn)
        if (sum(single) == 1)
          warning("Design '", design.single,
                  "' with single treatment arm excluded ",
                  "from network meta-analysis.",
                  call. = FALSE)
        else
          warning("Designs with single treatment arm excluded ",
                  "from network meta-analysis: ",
                  paste(paste0("'", design.single, "'"),
                        collapse = " - "),
                  call. = FALSE)
      ##
      dat.long <- dat.long[!(dat.long$design %in% design.single), , drop = FALSE]
      dat.wide <- dat.wide[!(dat.wide$design %in% design.single), , drop = FALSE]
      ##
      data$.drop <- data$.drop | data$.design %in% design.single
      ##
      rm(single, design.single)
    }
    ##
    rm(d.single)
  }
  ##
  dat.long <- dat.long[, c("studlab", "treat",
                           "event", "non.event", "n", "incr",
                           "design", "study", ".order")]
  dat.wide <- dat.wide[, c("studlab", "treat1", "treat2",
                           "event1", "event2", "non.event1", "non.event2",
                           "n1", "n2", "incr",
                           "design", "study", ".order")]
  ##
  dat.long$studlab <- as.character(dat.long$studlab)
  dat.long$design <- as.character(dat.long$design)
  dat.long$treat <- as.character(dat.long$treat)
  ##
  dat.wide$studlab <- as.character(dat.wide$studlab)
  dat.wide$design <- as.character(dat.wide$design)
  dat.wide$treat1 <- as.character(dat.wide$treat1)
  dat.wide$treat2 <- as.character(dat.wide$treat2)
  ##
  o <- order(dat.wide$.order)
  studlab <- dat.wide$studlab[o]
  treat1 <- dat.wide$treat1[o]
  treat2 <- dat.wide$treat2[o]
  ##
  event1 <- dat.wide$event1[o]
  event2 <- dat.wide$event2[o]
  n1 <- dat.wide$n1[o]
  n2 <- dat.wide$n2[o]
  ##
  trts <- sort(unique(c(treat1, treat2)))
  n.treat <- length(trts)
  n.studies <- length(unique(studlab))
  m <- length(studlab)
  ##
  treat1.pos <- match(treat1, trts)
  treat2.pos <- match(treat2, trts)
  ##
  ## Calculate adjacency matrix
  ##
  B <- createB(treat1.pos, treat2.pos, ncol = n.treat)
  ##
  M <- t(B) %*% B    # unweighted Laplacian matrix
  D <- diag(diag(M)) # diagonal matrix
  A <- D - M         # adjacency matrix (n x n)
  ##
  rownames(A) <- colnames(A) <- trts
  ##
  rm(B, M, D)
  ##
  ## Empty matrices for results
  ##
  NAmatrix <- matrix(NA, nrow = n.treat, ncol = n.treat)
  rownames(NAmatrix) <- colnames(NAmatrix) <- trts
  ##
  TE.common <- seTE.common <- NAmatrix
  TE.direct.common <- seTE.direct.common <- NAmatrix
  Q.direct <- tau2.direct <- I2.direct <- NAmatrix
  #
  if (is.lrp) {
    TE.direct.random <- seTE.direct.random <- phi.direct <- NAmatrix
  }
  
  
  ##
  ##
  ## (7) Conduct classic network meta-analysis using inverse variance
  ##     method
  ##
  ##
  dat.iv <- dat.long[order(dat.long$.order), ]
  ##
  if (is.iv) {
    if (missing(warn))
      warn <- gs("warn")
    #
    p.iv <- pairwise(studlab = dat.iv$studlab,
                     treat = dat.iv$treat,
                     event = dat.iv$event,
                     n = dat.iv$n,
                     data = dat.iv,
                     sm = sm,
                     incr = incr,
                     method.incr = method.incr,
                     allstudies = allstudies)
    ##
    net.iv <- netmeta(p.iv,
                      level = level, level.ma = level.ma,
                      common = common, random = random,
                      prediction = prediction, level.predict = level.predict,
                      reference.group = reference.group,
                      baseline.reference = baseline.reference,
                      all.treatments = all.treatments,
                      seq = seq,
                      tau.preset = tau.preset,
                      tol.multiarm = tol.multiarm,
                      tol.multiarm.se = tol.multiarm.se,
                      details.chkmultiarm = details.chkmultiarm,
                      sep.trts = sep.trts,
                      nchar.trts = nchar.trts,
                      overall.hetstat = overall.hetstat,
                      backtransf = backtransf,
                      title = title,
                      keepdata = keepdata,
                      warn = warn)
    #
    return(net.iv)
  }
  
  
  ##
  ##
  ## (8) Stage 2: Direct meta-analyses per design (MH and NCH methods)
  ##
  ##
  n.d <- tapply(dat.long$treat,   dat.long$design, lengthunique)
  k.d <- tapply(dat.long$studlab, dat.long$design, lengthunique)
  ##
  designs <- names(k.d)
  d <- length(designs)
  seq.d <- seq_len(d)
  ##
  dat.design <- vector("list", d)
  names(dat.design) <- designs
  ##
  for (i in seq.d)
    dat.design[[i]] <- dat.long[dat.long$design == designs[i], , drop = FALSE]
  ##
  if (method == "MH") {
    ##
    ## MH method
    ##
    treat.per.design <- vector("list", d)
    studies.in.design <- vector("list", d)
    ##
    c.xy <- vector("list", d)
    d.xy <- vector("list", d)
    C.xy <- vector("list", d)
    L.xy <- vector("list", d)
    ##
    U.xyy <- vector("list", d)
    U.xyz <- vector("list", d)
    ##
    t.pl <- vector("list", d)
    ##
    ps1 <- ps2 <- ps3 <- ps4 <- vector("list", d)
    ##
    L.bar.xy <- vector("list", d)
    ##
    U.plus.xx <- vector("list", d)
    ##
    U.new <- vector("list", d)
    ##
    U.plus.xy <- vector("list", d)
    ##
    Var.Lbar <- vector("list", d)
    ##
    CoVar.Lbar <- vector("list", d)
    ##
    length.y <- sum(n.d - 1)
    ##
    y <- c(rep(0, length.y))
    ##
    V1 <- matrix(rep(0, length.y * length.y), nrow = length.y)
    V2 <- matrix(rep(0, length.y * length.y), nrow = length.y)
    ##
    X <- matrix(0, nrow = length.y, ncol = n.treat - 1)
    ##
    counter1 <- counter2 <- counter3 <- 0
    ##
    list1 <- matrix(0, nrow = length.y, ncol = 2)
    ##
    N.j <- rep(0, d)
    ##
    if (d > 1)
      for (i in 2:d)
        N.j[i] <- N.j[i - 1] + n.d[i - 1] - 1
    ##
    ## Loop for designs
    ##
    for (i in seq.d) {
      treat.per.design[[i]] <- unique(dat.design[[i]]$treat)
      studies.in.design[[i]] <- unique(dat.design[[i]]$study)
      ##
      ## List of studies in each design
      ##
      ## Recode the studies in each design
      ##
      for (j in seq_along(dat.design[[i]]$study))
        for (k in seq_len(k.d[i]))
          if (dat.design[[i]]$study[j] == studies.in.design[[i]][k])
            dat.design[[i]]$id.s[j] <- k
      ##
      ## Recode the treatments in each design
      ##
      for (j in seq_along(dat.design[[i]]$study))
        for (k in seq_len(n.d[i]))
          if (dat.design[[i]]$treat[j] == treat.per.design[[i]][k])
            dat.design[[i]]$id.t[j] <- k
      ##
      ## Calculate total number of patients per studies
      ##
      for (j in seq_along(dat.design[[i]]$studlab))
        dat.design[[i]]$n.study[j] <-
          with(dat.design[[i]], sum(n[which(study == study[j])]))
      ##
      ## Define c.xy and d.xy
      ##
      c.xy[[i]] <- array(rep(0, k.d[i] * n.d[i] * n.d[i]),
                         dim = c(n.d[i], n.d[i], k.d[i]))
      ##
      d.xy[[i]] <- array(rep(0, k.d[i] * n.d[i] * n.d[i]),
                         dim = c(n.d[i], n.d[i], k.d[i]))
      ##
      for (st in seq_len(k.d[i]))
        for (t1 in seq_len(n.d[i]))
          for (t2 in seq_len(n.d[i])) {
            c.xy[[i]][t1, t2, st] <-
              with(dat.design[[i]],
                   sum(    event[which(id.s == st & id.t == t1)]) *
                   sum(non.event[which(id.s == st & id.t == t2)]) /
                   n.study[id.s == st][1])
            ##
            d.xy[[i]][t1, t2, st] <-
              with(dat.design[[i]],
              (sum(    event[which(id.s == st & id.t == t1)]) +
               sum(non.event[which(id.s == st & id.t == t2)])) /
              n.study[id.s == st][1])
          }
      ##
      ## Define C.xy
      ##
      C.xy[[i]] <- matrix(rep(0, n.d[i] * n.d[i]), nrow = n.d[i])
      ##
      for (j in seq_len(n.d[i]))
        for (k in seq_len(n.d[i]))
          C.xy[[i]][j, k] <- sum(c.xy[[i]][j, k, ])
      ##
      ## Define L.xy
      ##
      L.xy[[i]] <- matrix(rep(0, n.d[i] * n.d[i]), nrow = n.d[i])
      for (j in seq_len(n.d[i]))
        for (k in seq_len(n.d[i]))
          L.xy[[i]][j, k] <- log(C.xy[[i]][j, k] / C.xy[[i]][k, j])
      ##
      ## Calculate the variance of L.xy (dimension n.d[i] x n.d[i])
      ##
      U.xyy[[i]] <- matrix(rep(0, n.d[i] * n.d[i]), nrow = n.d[i])
      for (j in seq_len(n.d[i]))
        for (k in seq_len(n.d[i]))
          U.xyy[[i]][j, k] <-
            sum(c.xy[[i]][j, k, ] * d.xy[[i]][j, k, ]) /
            (2 * C.xy[[i]][j, k]^2) +
            sum(c.xy[[i]][j, k, ] * d.xy[[i]][k, j, ] + c.xy[[i]][k, j, ] *
                d.xy[[i]][j, k, ]) / (2 * C.xy[[i]][j, k] * C.xy[[i]][k, j]) +
            sum(c.xy[[i]][k, j, ] * d.xy[[i]][k, j, ]) / (2 * C.xy[[i]][k, j]^2)
      ##
      ## Calculate the covariance matrix U.xyz
      ##
      t.pl[[i]] = array(rep(0, k.d[i] * n.d[i] * n.d[i]),
                        dim = c(n.d[i], n.d[i], k.d[i]))
      ##
      for (j in seq_len(k.d[i]))
        t.pl[[i]][ , , j] <- with(dat.design[[i]], n.study[id.s == j][1])
      ##
      U.xyz[[i]] <- array(rep(0, n.d[i] * n.d[i] * n.d[i]),
                          dim = c(n.d[i], n.d[i], n.d[i]))
      ##
      ## Per study ...
      ##
      for (t1 in seq_len(n.d[i])) {
        for (t2 in seq_len(n.d[i])) {
          for (t3 in seq_len(n.d[i])) {
            for (st in seq_len(k.d[i])) {
              sel1 <- with(dat.design[[i]], which(id.s == st & id.t == t1))
              sel2 <- with(dat.design[[i]], which(id.s == st & id.t == t2))
              sel3 <- with(dat.design[[i]], which(id.s == st & id.t == t3))
              ##
              ps1[[i]][st] <-
                with(dat.design[[i]],
                     event[sel1] /
                     (t.pl[[i]][1, 1, st])^2 *
                     non.event[sel2] * non.event[sel3] *
                     (t1 != t2) * (t1 != t3) * (t2 != t3))
              ##
              ps2[[i]][st] <-
                with(dat.design[[i]],
                     n[sel1] / (t.pl[[i]][1, 1, st])^2 *
                     non.event[sel2] * event[sel3] *
                     (t1 != t2) * (t1 != t3) * (t2 != t3))
              ##
              ps3[[i]][st] <-
                with(dat.design[[i]],
                     n[sel1] / (t.pl[[i]][1, 1, st])^2 *
                     event[sel2] * non.event[sel3] *
                     (t1 != t2) * (t1 != t3) * (t2 != t3))
              ##
              ps4[[i]][st] <-
                with(dat.design[[i]],
                     non.event[sel1] / (t.pl[[i]][1, 1, st])^2 *
                     event[sel2] * event[sel3] *
                     (t1 != t2) * (t1 != t3) * (t2 != t3))
            }
            ##
            U.xyz[[i]][t1, t2, t3] <-
              sum(ps1[[i]][]) / (3 * C.xy[[i]][t1, t2] * C.xy[[i]][t1, t3]) +
              sum(ps2[[i]][]) / (3 * C.xy[[i]][t1, t2] * C.xy[[i]][t3, t1]) +
              sum(ps3[[i]][]) / (3 * C.xy[[i]][t2, t1] * C.xy[[i]][t1, t3]) +
              sum(ps4[[i]][]) / (3 * C.xy[[i]][t2, t1] * C.xy[[i]][t3, t1])
          }
        }
      }
      ##
      ## Calculate L.bar.xy
      ##
      L.bar.xy[[i]] <- matrix(rep(0, n.d[i] * n.d[i]), nrow = n.d[i])
      for (t1 in seq_len(n.d[i]))
        for (t2 in seq_len(n.d[i]))
          L.bar.xy[[i]][t1, t2] <-
            (sum(L.xy[[i]][t1, ]) - sum(L.xy[[i]][t2, ])) / n.d[i]
      ##
      ## Calculate U.plus.xx
      ##
      for (t1 in seq_len(n.d[i]))
        U.xyy[[i]][t1, t1] <- 0
      ##
      U.plus.xx[[i]] <- rep(0, n.d[i])
      for (t1 in seq_len(n.d[i]))
        U.plus.xx[[i]][t1] <-
          sum(U.xyy[[i]][t1, seq_len(n.d[i])]) + sum(U.xyz[[i]][t1, , ])
      ##
      ## Calculate U.plus.xy
      ##
      U.new[[i]] <- array(rep(0, n.d[i] * n.d[i] * n.d[i]),
                          dim = c(n.d[i], n.d[i], n.d[i]))
      ##
      for (t1 in seq_len(n.d[i]))
        for (t2 in seq_len(n.d[i]))
          for (t3 in seq_len(n.d[i]))
            U.new[[i]][t1, t2, t3] <-
              (t1 != t2) * (t1 != t3) * (t2 != t3) *
              U.xyz[[i]][t1, t2, t3] +
              (t1 != t2) * (t2 == t3) * U.xyy[[i]][t1, t2]
      ##
      U.plus.xy[[i]] = matrix(rep(0, n.d[i] * n.d[i]), nrow = n.d[i])
      ##
      for (t1 in seq_len(n.d[i]))
        for (t2 in seq_len(n.d[i]))
          U.plus.xy[[i]][t1, t2] <-
            sum(U.new[[i]][seq_len(n.d[i]), t1, t2]) -
            sum(U.new[[i]][t1, t2, seq_len(n.d[i])]) -
            sum(U.new[[i]][t2, t1, seq_len(n.d[i])]) +
            U.new[[i]][t1, t2, t2]
      ##
      ## Variance of L.bar.xy
      ##
      Var.Lbar[[i]] <- matrix(rep(0, n.d[i] * n.d[i]), nrow = n.d[i])
      for (t1 in seq_len(n.d[i]))
        for (t2 in seq_len(n.d[i]))
          Var.Lbar[[i]][t1, t2] <-
            (U.plus.xx[[i]][t1] - 2 * U.plus.xy[[i]][t1, t2] +
             U.plus.xx[[i]][t2]) / n.d[i]^2
      ##
      ## Covariance of L.bar.xy. Only a subset of covariances are
      ## calculate here, i.e. cov(L.bar_(1, t1), L.bar_(1, t2))
      ##
      CoVar.Lbar[[i]] <- matrix(rep(0, n.d[i]^2), nrow = n.d[i])
      for (t1 in seq_len(n.d[i]))
        for (t2 in seq_len(n.d[i]))
          CoVar.Lbar[[i]][t1, t2] <-
            (U.plus.xx[[i]][1] - U.plus.xy[[i]][1, t2] -
             U.plus.xy[[i]][t1, 1] +
             U.plus.xy[[i]][t1, t2]) / n.d[i]^2
      ##
      ## Define matrix X
      ##
      for (j in seq_along(dat.design[[i]]$studlab))
        for (k in seq_along(trts))
          if (dat.design[[i]]$treat[j] == trts[k])
            dat.design[[i]]$id.t2[j] <- k
      ##
      ##
      ## Create y, the vector of treatment effects from each study. Only
      ## effects vs the first treatment are needed. y is coded as OR
      ## 1vsX
      ##
      for (j in seq_len(n.d[i] - 1))
        y[j + counter1] <- L.bar.xy[[i]][1, j + 1]
      ##
      counter1 <- counter1 + n.d[i] - 1
      ##
      ## Create V, the matrix of covariances of treatment effects from
      ## each study. Only covariances of effects vs the first treatment
      ## are needed.
      ##
      for (j in seq_len(n.d[i] - 1))
        for (k in seq_len(n.d[i] - 1))
          V1[j + counter2, k + counter2] <- (k == j) * Var.Lbar[[i]][1, j + 1]
      ##
      counter2 <- counter2 + n.d[i] - 1
      ##
      for (j in 2:(n.d[i]))
        for (k in 2:(n.d[i]))
          V2[j + counter3 - 1, k + counter3 - 1] <-
            (k != j) * CoVar.Lbar[[i]][j, k]
      ##
      counter3 <- counter3 + n.d[i] - 1
      ##
      for (j in seq_len(n.d[i] - 1)) {
        list1[N.j[i] + j, 1] <- dat.design[[i]]$id.t2[[1]]
        list1[N.j[i] + j, 2] <- dat.design[[i]]$id.t2[[j + 1]]
      }
      ##
      ## Drop unnecessary variables
      ##
      dat.design[[i]]$id.s <- NULL
      dat.design[[i]]$id.t <- NULL
      dat.design[[i]]$id.t2 <- NULL
      dat.design[[i]]$n.study <- NULL
    }
    ##
    V <- V1 + V2
    ##
    basic.contrasts <- 2:n.treat
    ##
    for (i in 1:length.y)
      for (k in 1:(n.treat - 1)) {
        if (list1[i, 1] == basic.contrasts[k])
          X[i, k] = -1
        if (list1[i, 2] == basic.contrasts[k])
          X[i, k] = 1
      }
    ##
    W <- solve(V)
    ##
    TE.basic <- solve(t(X) %*% W %*% X) %*% t(X) %*% W %*% y
  }
  else if (method == "NCH") {
    ##
    ## NCH method
    ##
    dat.long$ttt <- 0
    ##
    for (k in 1:n.treat)
      for (i in seq_along(dat.long$studlab))
        if (dat.long$treat[i] == trts[k])
          dat.long$ttt[i] <- k
    ##
    dat.long$treat <- as.numeric(dat.long$ttt)
    dat.long$ttt <- NULL
    ##
    dat.long$treat <- as.numeric(dat.long$treat)
    ##
    dat.long <- dat.long[order(dat.long$studlab, dat.long$treat), ] # necessary ???
    ##
    for (i in unique(dat.long$studlab)) {
      counter <- 1
      for (j in seq_along(dat.long$studlab))
        if (dat.long$studlab[j] == i) {
          dat.long$count[j] <- counter
          counter <- counter + 1
        }
    }
    ##
    max.arms <- max(dat.long$count)
    ##
    d1 <- reshape(dat.long, idvar = "studlab",
                  timevar = "count", direction = "wide")
    ##
    d1 <- d1[order(d1$studlab), ]
    ##
    d1$N1 <- 0 # d1$n.all = 0
    d1$narms <- 0
    ##
    for (i in seq_len(max.arms)) {
      ev <- paste0("event.", i)
      tot <- paste0("n.", i)
      d1$N1 <- rowSums(cbind(d1$N1, d1[, colnames(d1) == ev]),
                       na.rm = TRUE)
      ## d1$n.all <- rowSums(cbind(d1$n.all, d1[, colnames(d1) == tot]), na.rm = TRUE)
    }
    ##
    for (i in seq_along(d1$studlab))
      for (k in 1:max.arms) {
        ev <- paste0("event.", k)
        if (!is.na(d1[, colnames(d1) == ev][i]))
          d1$narms[i] <- k
      }
    ##
    for (i in seq_along(colnames(d1)))
      for (k in seq_along(colnames(d1))) {
        if (colnames(d1)[k] == paste0("treat.", i))
          colnames(d1)[k] = paste0("t", i)
        ##
        if (colnames(d1)[k] == paste0("event.", i))
          colnames(d1)[k] = paste0("r", i)
        ##
        if (colnames(d1)[k] == paste0("n.", i))
          colnames(d1)[k] = paste0("n", i)
      }
    ##
    ## Likelihood function
    ##
    myLik1 <- function(mypar) {
      x <- mypar
      ##
      myLogLik1 <- myLogLik2 <- myLogLik3 <- rep(0, length(d1$studlab))
      ##
      for (i in seq_along(d1$studlab)) {
        for (k in 2:d1$narms[i]) {
          myLogLik1[i] <-
            myLogLik1[i] +
            d1[, colnames(d1) == paste0("r", k)][i] *
            (x[d1[, colnames(d1) == paste0("t", k)][i]] -
             x[d1$t1[i]] * (d1$t1[i] != 1))
          ##
          myLogLik2[i] <-
            myLogLik2[i] +
            d1[, colnames(d1) == paste0("n", k)][i] *
            exp(x[d1[, colnames(d1) == paste0("t", k)][i]] -
                x[d1$t1[i]] * (d1$t1[i] != 1))
          ##
          myLogLik3[i] <- -d1$N1[i] * log(d1$n1[i] + myLogLik2[i])
        }
      }
      ##
      myLogLik <- sum(myLogLik1 + myLogLik3)
      ##
      myLogLik
    }
    ##
    opt <- optim(rep(0, n.treat), myLik1, method = "L-BFGS-B",
                 lower = -Inf, upper = Inf,
                 control = list(fnscale = -1, maxit = 10000),
                 hessian = TRUE)
    ##
    W <- solve(-opt$hessian[2:n.treat, 2:n.treat])
    ##
    TE.basic <- -(opt$par)[2:n.treat]
  }
  else if (is.lrp) {
    #
    # Penalised logistic regression
    #
    is_installed_package("brglm2", "netmetabin", "method", " = \"LRP\"")
    #
    phi <- 1
    text.formula <- "cbind(event, non.event) ~ as.factor(treat)"
    #
    if (length(unique(dat.long$studlab)) > 1)
      text.formula <- paste(text.formula, "+ as.factor(studlab)")
    #
    res.lrp <- glm(as.formula(text.formula), data = dat.long,
                   family = binomial(link = "logit"),
                   method = brglm2::brglmFit, type = "MPL_Jeffreys",
                   ...)
    #
    if (length(unique(dat.long$studlab)) > 1)
      phi <- phi(res.lrp)
    #
    # Basic estimates (all treats vs reference)
    #
    ests.lrp <- summary(res.lrp)$coefficients
    ests.lrp <-
      ests.lrp[grepl("as.factor(treat)", rownames(ests.lrp), fixed = TRUE), ]
    rownames(ests.lrp) <-
      gsub("as.factor(treat)", "", rownames(ests.lrp), fixed = TRUE)
    #
    trts.basic <- rownames(ests.lrp)
    #
    # Get the variance-covariance matrix for the basic estimates
    #
    var_cov_common <- vcov(res.lrp)
    var_cov_common <-
      var_cov_common[
        grepl("as.factor(treat)", rownames(var_cov_common), fixed = TRUE),
        grepl("as.factor(treat)", colnames(var_cov_common), fixed = TRUE)]
    #
    rownames(var_cov_common) <-
      gsub("as.factor(treat)", "", rownames(var_cov_common), fixed = TRUE)
    colnames(var_cov_common) <-
      gsub("as.factor(treat)", "", colnames(var_cov_common), fixed = TRUE)
    #
    # Inflate the elements of the whole variance covariance matrix from the
    # common effect model to get the respective matrix for the random effects
    # model
    #
    var_cov_random <- var_cov_common * phi
    #
    TE.basic <- ests.lrp[, 1]
    #
    # Use the basic2all() function to calculate the functional parameters using
    # the basic parameters
    #
    all_ests_common <- basic2all(trts, TE.basic, var_cov_common)
    #
    all_ests_random <- basic2all(trts, TE.basic, phi * var_cov_common)
    #
    TE.common <- all_ests_common$TEs
    seTE.common <- all_ests_common$seTEs
    #
    TE.random <- all_ests_random$TEs
    seTE.random <- all_ests_random$seTE
  }
  #
  if (!is.lrp) {
    #
    # Define H matrix
    #
    H <- matrix(0,
                nrow = n.treat * ((n.treat - 1) / 2),
                ncol = n.treat - 1)
    #
    diag(H) <- 1
    #
    if (n.treat > 2) {
      t1 <- c()
      t2 <- c()
      for (i in 2:(n.treat - 1))
        for (j in (i + 1):(n.treat)) {
          t1 <- rbind(t1, i)
          t2 <- rbind(t2, j)
        }
      #
      h1 <- matrix(c(t1, t2), nrow = nrow(t1))
      #
      for (i in 1:((n.treat - 1) * (n.treat - 2) / 2))
        for (j in 1:(n.treat - 1))
          H[i + n.treat - 1, j] <- -(h1[i, 1] == j + 1) + (h1[i, 2] == j + 1)
    }
    #
    colnames(H) <- trts[-1]
    #
    # Common effects matrix
    #
    d.hat <- H %*% TE.basic
    #
    TE.common[lower.tri(TE.common, diag = FALSE)] <-  d.hat
    TE.common <- t(TE.common)
    TE.common[lower.tri(TE.common, diag = FALSE)] <- -d.hat
    diag(TE.common) <- 0
    #
    # Matrix with standard errors
    #
    if (method == "MH")
      cov.d.hat <- H %*% solve(t(X) %*% W %*% X) %*% t(H)
    else
      cov.d.hat <- H %*% W %*% t(H)
    ##
    seTE.common[lower.tri(seTE.common, diag = FALSE)] <- sqrt(diag(cov.d.hat))
    seTE.common <- t(seTE.common)
    seTE.common[lower.tri(seTE.common, diag = FALSE)] <- sqrt(diag(cov.d.hat))
    diag(seTE.common) <- 0
  }
  else
    H <- matrix(NA,
                nrow = n.treat * ((n.treat - 1) / 2),
                ncol = n.treat - 1)
  #
  # Confidence intervals
  #
  ci.f <- ci(TE.common, seTE.common, level = level.ma)
  #
  if (is.lrp)
    ci.r <- ci(TE.random, seTE.random, level = level.ma)
  ##
  ## Inconsistency global
  ##
  if (method == "MH") {
    Q <- as.vector(t(y - X %*% TE.basic) %*% solve(V) %*%
                    (y - X %*% TE.basic))
    df.Q <- sum(n.d - 1) - (n.treat - 1)
    pval.Q <- pvalQ(Q, df.Q)
  }
  else {
    Q <- NA
    df.Q <- NA
    pval.Q <- NA
  }
  
    
  ##
  ##
  ## (9) Inconsistency evaluation: direct MH estimates
  ##
  ##
  B.matrix <- createB(treat1.pos, treat2.pos)
  B.matrix <- unique(B.matrix)
  colnames(B.matrix) <- trts
  ##
  for (i in seq_len(nrow(B.matrix))) {
    sel.treat1 <- colnames(B.matrix)[B.matrix[i, ] ==  1]
    sel.treat2 <- colnames(B.matrix)[B.matrix[i, ] == -1]
    selstud <- treat1 == sel.treat1 & treat2 == sel.treat2
    ##
    if (!is.lrp)
      m.i <- metabin(event1, n1, event2, n2, subset = selstud,
                     method = "MH", sm = "OR",
                     incr = incr,
                     method.incr = method.incr,
                     allstudies = allstudies, MH.exact = !cc.pooled,
                     method.tau = "DL", method.tau.ci = "",
                     Q.Cochrane = FALSE,
                     warn.deprecated = FALSE)
    else
      m.i <- metabin(event1, n1, event2, n2, subset = selstud,
                     method = "LRP",
                     warn.deprecated = FALSE, ...)
    #
    TE.direct.common[sel.treat1, sel.treat2]   <- m.i$TE.common
    seTE.direct.common[sel.treat1, sel.treat2] <- m.i$seTE.common
    #
    TE.direct.common[sel.treat2, sel.treat1]   <- -m.i$TE.common
    seTE.direct.common[sel.treat2, sel.treat1] <- m.i$seTE.common
    #
    Q.direct[sel.treat1, sel.treat2] <- m.i$Q
    tau2.direct[sel.treat1, sel.treat2] <- m.i$tau2
    I2.direct[sel.treat1, sel.treat2] <- m.i$I2
    #
    Q.direct[sel.treat2, sel.treat1] <- m.i$Q
    tau2.direct[sel.treat2, sel.treat1] <- m.i$tau2
    I2.direct[sel.treat2, sel.treat1] <- m.i$I2
    #
    if (is.lrp) {
      TE.direct.random[sel.treat1, sel.treat2]   <- m.i$TE.random
      seTE.direct.random[sel.treat1, sel.treat2] <- sqrt(phi) * m.i$seTE.common
      #
      TE.direct.random[sel.treat2, sel.treat1]   <- -m.i$TE.random
      seTE.direct.random[sel.treat2, sel.treat1] <- sqrt(phi) * m.i$seTE.common
      #
      phi.direct[sel.treat1, sel.treat2] <- m.i$phi
      phi.direct[sel.treat2, sel.treat1] <- m.i$phi
    }
  }
  #
  rm(sel.treat1, sel.treat2, selstud, m.i)
  #
  ci.direct.c <- ci(TE.direct.common, seTE.direct.common, level = level.ma)
  #
  if (is.lrp)
    ci.direct.r <- ci(TE.direct.random, seTE.direct.random, level = level.ma)
  
  
  labels <- sort(unique(c(treat1, treat2)))
  ##
  if (!is.null(seq))
    seq <- setseq(seq, labels)
  else {
    seq <- labels
    if (is.numeric(seq))
      seq <- as.character(seq)
  }
  #
  comps <- vector("character")
  #
  for (i in seq_along(trts))
    for (j in seq_along(trts))
      if (i < j)
        comps <- c(comps, paste(trts[i], trts[j], sep = sep.trts))
  #
  Cov <- matrix(1, nrow = length(comps), ncol = length(comps),
                dimnames = list(comps, comps))
  Cov <- setNA(Cov)
  #
  # Row names in the H matrix are second vs first, not first vs second
  #
  secondfirst <- function(x, sep)
    paste(x[2], x[1], sep = sep)
  #
  if (nrow(H) == length(comps))
    rownames(H) <- sapply(compsplit(comps, sep.trts), secondfirst,
                          sep = sep.trts)
  
  
  res <- list(studlab = studlab,
              treat1 = treat1,
              treat2 = treat2,
              ##
              TE = data$.TE[!data$.drop],
              seTE = data$.seTE[!data$.drop],
              seTE.adj = rep(NA, sum(!data$.drop)),
              seTE.adj.common = rep(NA, sum(!data$.drop)),
              seTE.adj.random = rep(NA, sum(!data$.drop)),
              ##
              design = designs(treat1, treat2, studlab)$design,
              ##
              event1 = event1,
              event2 = event2,
              n1 = n1,
              n2 = n2,
              ##
              k = n.studies,
              m = m,
              n = length(trts),
              d = d,
              ##
              trts = trts,
              k.trts = rowSums(A),
              n.trts = NA,
              events.trts = NA,
              ##
              studies = NA,
              narms = NA,
              ##
              designs = designs,
              ##
              TE.common = TE.common,
              seTE.common = seTE.common,
              lower.common = ci.f$lower,
              upper.common = ci.f$upper,
              statistic.common = ci.f$statistic,
              pval.common = ci.f$p,
              ##
              TE.random = if (is.lrp) TE.random else NAmatrix,
              seTE.random = if (is.lrp) seTE.random else NAmatrix,
              lower.random = if (is.lrp) ci.r$lower else NAmatrix,
              upper.random = if (is.lrp) ci.r$upper else NAmatrix,
              statistic.random = if (is.lrp) ci.r$statistic else NAmatrix,
              pval.random = if (is.lrp) ci.r$p else NAmatrix,
              ##
              seTE.predict = NAmatrix,
              lower.predict = NAmatrix,
              upper.predict = NAmatrix,
              method.predict = "V",
              ##
              prop.direct.common = NA,
              prop.direct.random = NA,
              ##
              TE.direct.common = TE.direct.common,
              seTE.direct.common = seTE.direct.common,
              lower.direct.common = ci.direct.c$lower,
              upper.direct.common = ci.direct.c$upper,
              statistic.direct.common = ci.direct.c$statistic,
              pval.direct.common = ci.direct.c$p,
              ##
              TE.direct.random = if (is.lrp) TE.direct.random else NAmatrix,
              seTE.direct.random = if (is.lrp) seTE.direct.random else NAmatrix,
              lower.direct.random = if (is.lrp) ci.direct.c$lower else NAmatrix,
              upper.direct.random = if (is.lrp) ci.direct.c$upper else NAmatrix,
              statistic.direct.random =
                if (is.lrp) ci.direct.c$statistic else NAmatrix,
              pval.direct.random = if (is.lrp) ci.direct.c$p else NAmatrix,
              #
              Q.direct = Q.direct,
              tau.direct = sqrt(tau2.direct),
              tau2.direct = tau2.direct,
              I2.direct = I2.direct,
              #
              phi = if (is.lrp) phi else NA,
              phi.direct = if (is.lrp) phi.direct else NAmatrix,
              #
              TE.indirect.common = NA,
              seTE.indirect.common = NA,
              lower.indirect.common = NA,
              upper.indirect.common = NA,
              statistic.indirect.common = NA,
              pval.indirect.common = NA,
              ##
              TE.indirect.random = NA,
              seTE.indirect.random = NA,
              lower.indirect.random = NA,
              upper.indirect.random = NA,
              statistic.indirect.random = NA,
              pval.indirect.random = NA,
              ##
              Q = NA,
              df.Q = NA,
              pval.Q = NA,
              I2 = NA, lower.I2 = NA, upper.I2 = NA,
              tau = NA,
              ##
              Q.heterogeneity = NA,
              df.Q.heterogeneity = NA,
              pval.Q.heterogeneity = NA,
              Q.inconsistency = Q,
              df.Q.inconsistency = df.Q,
              pval.Q.inconsistency = pval.Q,
              ##
              Q.decomp = NA,
              ##
              A.matrix = A,
              H.matrix.common = H,
              H.matrix.random = NA,
              ##
              n.matrix = NA,
              events.matrix = NA,
              ##
              P.common = NA,
              P.random = NA,
              ##
              Cov.common = Cov,
              Cov.random = Cov,
              ##
              sm = sm,
              method = method,
              ##
              incr = incr,
              method.incr = method.incr,
              allstudies = allstudies,
              cc.pooled = cc.pooled,
              ##
              level = level,
              level.ma = level.ma,
              common = common,
              random = random,
              ##
              prediction = prediction,
              level.predict = level.predict,
              ##
              reference.group = reference.group,
              baseline.reference = baseline.reference,
              small.values = small.values,
              all.treatments = all.treatments,
              seq = seq,
              ##
              method.tau = "DL",
              tau.preset = tau.preset,
              ##
              tol.multiarm = tol.multiarm,
              tol.multiarm.se = tol.multiarm.se,
              details.chkmultiarm = details.chkmultiarm,
              details.chkdata = details.chkdata,
              ##
              sep.trts = sep.trts,
              nchar.trts = nchar.trts,
              ##
              func.inverse = deparse(substitute(func.inverse)),
              #
              overall.hetstat = overall.hetstat,
              backtransf = backtransf,
              #
              title = title,
              ##
              data = if (keepdata) data else NULL,
              data.design = dat.design,
              ##
              warn = warn,
              call = match.call(),
              version = packageDescription("netmeta")$Version,
              #
              addincr = addincr,
              allincr = allincr
              )
  ##
  class(res) <- c("netmetabin", "netmeta")
  ##
  ## Add results for indirect treatment estimates
  ##
  n <- res$n
  ##
  res$prop.direct.common <-
    netmeasures(res, random = FALSE, warn = warn)$proportion
  if (is.logical(res$prop.direct.common))
    res$prop.direct.common <- as.numeric(res$prop.direct.common)
  #
  if (is.lrp) {
    res$prop.direct.random <-
      netmeasures(res, random = TRUE, warn = warn)$proportion
    if (is.logical(res$prop.direct.random))
      res$prop.direct.random <- as.numeric(res$prop.direct.random)
    #
    res$Cov.common <- matrix(NA, nrow = length(res$prop.direct.common),
                             ncol = length(res$prop.direct.common))
    res$Cov.random <- matrix(NA, nrow = length(res$prop.direct.random),
                             ncol = length(res$prop.direct.random))
    #
    rownames(res$Cov.common) <- colnames(res$Cov.common) <-
      names(res$prop.direct.common)
    rownames(res$Cov.random) <- colnames(res$Cov.random) <-
      names(res$prop.direct.random)
  }
  #
  P.common <- P.random <- matrix(NA, n, n)
  colnames(P.common) <- rownames(P.common) <-
    colnames(P.random) <- rownames(P.random) <- trts
  ##
  if (n == 2) {
    ##
    ## For two treatments only direct evidence is available
    ##
    res$prop.direct.common <- 1
    names(res$prop.direct.common) <- paste(labels, collapse = sep.trts)
    ##
    sel <- row(P.common) != col(P.common)
    P.common[sel] <- 1
    #
    if (is.lrp) {
      res$prop.direct.random <- 1
      names(res$prop.direct.random) <- paste(labels, collapse = sep.trts)
      ##
      sel <- row(P.random) != col(P.random)
      P.random[sel] <- 1
    }
  }
  else {
    k <- 0
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        k <- k + 1
        P.common[i, j] <- P.common[j, i] <- res$prop.direct.common[k]
        #
        if (is.lrp)
          P.random[i, j] <- P.random[j, i] <- res$prop.direct.random[k]
      }
    }
  }
  ##
  ## Set direct evidence estimates to 0 if only indirect evidence is available
  ## (otherwise indirect estimates would be NA as direct estimates are NA)
  ##
  TE.direct.common <- res$TE.direct.common
  ##
  TE.direct.common[is_zero(P.common)] <- 0
  ##
  ## Indirect estimate is NA if only direct evidence is available
  ##
  res$P.common <- P.common
  ##
  P.common[is_zero(P.common - 1)] <- NA
  P.common[P.common > 1] <- NA
  #
  if (is.lrp) {
    TE.direct.random <- res$TE.direct.random
    TE.direct.random[is_zero(P.random)] <- 0
    #
    # Indirect estimate is NA if only direct evidence is available
    #
    res$P.random <- P.random
    #
    P.random[is_zero(P.random - 1)] <- NA
    P.random[P.random > 1] <- NA
  }
  ##
  ## Common effects model
  ##
  ci.indirect.c <-
    ci((res$TE.common - P.common * TE.direct.common) / (1 - P.common),
       sqrt(res$seTE.common^2 / (1 - P.common)), level = level)
  ##
  res$TE.indirect.common   <- ci.indirect.c$TE
  res$seTE.indirect.common <- ci.indirect.c$seTE
  ##
  res$lower.indirect.common <- ci.indirect.c$lower
  res$upper.indirect.common <- ci.indirect.c$upper
  ##
  res$statistic.indirect.common <- ci.indirect.c$statistic
  res$pval.indirect.common <- ci.indirect.c$p
  ##
  ## Random effects model
  ##
  if (!is.lrp) {
    res$prop.direct.random <- res$prop.direct.common
    res$prop.direct.random[!is.na(res$prop.direct.random)] <- NA
    res$P.random <- P.random
    ##
    res$TE.indirect.random <- res$seTE.indirect.random <-
      res$lower.indirect.random <- res$upper.indirect.random <-
      res$statistic.indirect.random <- res$pval.indirect.random <-
      NAmatrix
  }
  else {
    ci.indirect.r <-
      ci((res$TE.random - P.random * TE.direct.random) / (1 - P.random),
         sqrt(res$seTE.random^2 / (1 - P.random)), level = level)
    ##
    res$TE.indirect.random   <- ci.indirect.r$TE
    res$seTE.indirect.random <- ci.indirect.r$seTE
    ##
    res$lower.indirect.random <- ci.indirect.r$lower
    res$upper.indirect.random <- ci.indirect.r$upper
    ##
    res$statistic.indirect.random <- ci.indirect.r$statistic
    res$pval.indirect.random <- ci.indirect.r$p
    #
    res$P.random <- P.random
  }
  
  
  ##
  ## Study overview
  ##
  p0 <- prepare(rep(1, nrow(dat.wide)), rep(1, nrow(dat.wide)),
                 dat.wide$treat1, dat.wide$treat2, dat.wide$studlab,
                 func.inverse = func.inverse)
  ##
  tdata <- data.frame(studies = p0$data$studlab, narms = p0$data$narms,
                      stringsAsFactors = FALSE)
  ##
  tdata <- tdata[!duplicated(tdata[, c("studies", "narms")]), , drop = FALSE]
  res$studies <- tdata$studies
  res$narms <- tdata$narms
  ##
  if (all(suppressWarnings(!is.na(as.numeric(res$studies))))) {
    o <- order(as.numeric(res$studies))
    res$studies <- res$studies[o]
    res$narms <- res$narms[o]
  }
  ##
  res$data <- merge(res$data,
                    data.frame(.studlab = res$studies,
                               .narms = res$narms),
                    by = ".studlab", all.x = TRUE)
  
  
  res$data <- res$data[order(res$data$.order), ]
  res$data$.order <- NULL
  rownames(res$data) <- seq_len(nrow(res$data))
  
  
  res$n.matrix <- netmatrix(res, n1 + n2, func = "sum")
  ##
  dat.n <- data.frame(studlab = c(studlab, studlab),
                      treat = c(treat1, treat2),
                      n = c(n1, n2))
  dat.n <- dat.n[!duplicated(dat.n[, c("studlab", "treat")]), ]
  dat.n <- by(dat.n$n, dat.n$treat, sum, na.rm = TRUE)
  res$n.trts <- as.vector(dat.n[trts])
  names(res$n.trts) <- trts
  ##
  res$events.matrix <- netmatrix(res, event1 + event2, func = "sum")
  ##
  dat.e <- data.frame(studlab = c(studlab, studlab),
                      treat = c(treat1, treat2),
                      n = c(event1, event2))
  dat.e <- dat.e[!duplicated(dat.e[, c("studlab", "treat")]), ]
  dat.e <- by(dat.e$n, dat.e$treat, sum, na.rm = TRUE)
  res$events.trts <- as.vector(dat.e[trts])
  names(res$events.trts) <- trts
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  res$comb.fixed <- res$common
  res$comb.random <- res$random
  ##
  res$seTE.adj.fixed <- res$seTE.adj.common
  ##
  res$TE.fixed <- res$TE.common
  res$seTE.fixed <- res$seTE.common
  res$lower.fixed <- res$lower.common
  res$upper.fixed <- res$upper.common
  res$statistic.fixed <- res$statistic.common
  res$pval.fixed <- res$pval.common
  ##
  res$prop.direct.fixed <- res$prop.direct.common
  ##
  res$TE.direct.fixed <- res$TE.direct.common
  res$seTE.direct.fixed <- res$seTE.direct.common
  res$lower.direct.fixed <- res$lower.direct.common
  res$upper.direct.fixed <- res$upper.direct.common
  res$statistic.direct.fixed <- res$statistic.direct.common
  res$pval.direct.fixed <- res$pval.direct.common
  ##
  res$TE.indirect.fixed <- res$TE.indirect.common
  res$seTE.indirect.fixed <- res$seTE.indirect.common
  res$lower.indirect.fixed <- res$lower.indirect.common
  res$upper.indirect.fixed <- res$upper.indirect.common
  res$statistic.indirect.fixed <- res$statistic.indirect.common
  res$pval.indirect.fixed <- res$pval.indirect.common
  ##
  res$P.fixed <- res$P.common
  res$Cov.fixed <- res$Cov.common               
  
  res
}

#' Network meta-analysis of primary care depression treatments
#' 
#' @description
#' Network meta-analysis of 22 treatments (including placebo and usual
#' care) for the primary care of depression.
#' 
#' @name Linde2016
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{id}}\tab Study ID \cr
#' \bold{\emph{author}}\tab First author \cr
#' \bold{\emph{year}}\tab Year of publication \cr
#' \bold{\emph{treat1}}\tab First treatment (abbreviated) \cr
#' \bold{\emph{treat2}}\tab Second treatment (abbreviated) \cr
#' \bold{\emph{treat1.long}}\tab First treatment \cr
#' \bold{\emph{treat2.long}}\tab Second treatment \cr
#' \bold{\emph{lnOR}}\tab Response after treatment (log odds ratio) \cr
#' \bold{\emph{selnOR}}\tab Standard error of log odds ratio \cr
#' \bold{\emph{resp1}}\tab Responder (first treatment) \cr
#' \bold{\emph{n1}}\tab Sample size (first treatment) \cr
#' \bold{\emph{resp2}}\tab Responder (second treatment) \cr
#' \bold{\emph{n2}}\tab Sample size (second treatment) \cr
#' }
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netcomb}}
#' 
#' @source
#' Linde K, Rücker G, Schneider A et al. (2016):
#' Questionable assumptions hampered interpretation of a network
#' meta-analysis of primary care depression treatments.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{71}, 86--96
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Linde2016)
#' 
#' # Example using pairwise(), netmeta() and netcomb():
#' # example(Linde2016, package = "metabook")

NULL

## Auxiliary functions to check function arguments
##
## Package: meta
## Author: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
## License: GPL (>= 2)
##
chkchar <- function(x, length = 0, name = NULL, nchar = NULL, single = FALSE) {
  if (!missing(single) && single)
    length <- 1
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  if (length && length(x) != length)
    stop("Argument '", name, "' must be a character vector of length ",
         length, ".",
         call. = FALSE)
  ##
  if (length == 1) {
    if (!is.null(nchar) && !(nchar(x) %in% nchar))
      if (length(nchar) == 1 && nchar == 1)
        stop("Argument '", name, "' must be a single character.",
             call. = FALSE)
      else
        stop("Argument '", name, "' must be a character string of length ",
             if (length(nchar) == 2)
               paste0(nchar, collapse = " or ")
             else
               paste0(nchar, collapse = ", "),
             ".",
             call. = FALSE)
  }
  ##
  if (!is.character(x))
    stop("Argument '", name, "' must be a character vector.")
  else {
    if (!is.null(nchar) & any(!(nchar(x) %in% nchar)))
      if (length(nchar) == 1 && nchar == 1)
        stop("Argument '", name, "' must be a vector of single characters.",
             call. = FALSE)
      else
        stop("Argument '", name, "' must be a character vector where ",
             "each element has ",
             if (length(nchar) == 2)
               paste0(nchar, collapse = " or ")
             else
               paste0(nchar, collapse = ", "),
             " characters.",
             call. = FALSE)
  }
}
chkclass <- function(x, class, name = NULL) {
  ##
  ## Check class of R object
  ##
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  n.class <- length(class)
  if (n.class == 1)
    text.class <- paste0('"', class, '"')
  else if (n.class == 2)
    text.class <- paste0('"', class, '"', collapse = " or ")
  else
    text.class <- paste0(paste0('"', class[-n.class], '"', collapse = ", "),
                    ', or ', '"', class[n.class], '"')
  ##
  if (!inherits(x, class))
    stop("Argument '", name,
         "' must be an object of class \"",
         text.class, "\".", call. = FALSE)
  ##
  invisible(NULL)
}
chkcolor <- function(x, length = 0, name = NULL, single = FALSE) {
  if (!missing(single) && single)
    length <- 1
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  if (length && length(x) != length)
    stop("Argument '", name, "' must must be a character or ",
         "numeric vector of length ", length, ".",
         call. = FALSE)
  else if (!(is.character(x) || is.numeric(x)))
    stop("Argument '", name, "' must be a character or numeric vector.",
         call. = FALSE)
}
chklength <- function(x, k.all, fun = "", text, name = NULL) {
  ##
  ## Check length of vector
  ##
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  if (length(x) != k.all) {
    funcs <- c("metabin", "metacont", "metacor",
               "metagen", "metainc", "metamean",
               "metaprop", "metarate",
               "funnel", "forest.meta")
    args <- c("event.e", "n.e", "cor",
              "TE", "event.e", "n",
              "event", "event",
              "TE", "TE")
    ##
    idx <- charmatch(fun, funcs, nomatch = NA)
    if (!is.na(idx))
      argname <- args[idx]
    else
      argname <- fun
    ##
    if (missing(text))
      stop("Arguments '", argname, "' and '", name,
           "' must have the same length.",
           call. = FALSE)
    else
      stop(text, call. = FALSE)
  }
  ##
  invisible(NULL)
}
chklevel <- function(x, length = 0, ci = TRUE, name = NULL, single = FALSE) {
  if (!missing(single) && single)
    length <- 1
  ##
  ## Check for levels of confidence interval / contour level
  ##
  if (is.null(name))
    name <- deparse(substitute(x))
  if (ci)
    "level for confidence interval (range: 0-1)"
  else
    "contour levels (range: 0-1)"
  ##
  if (!is.numeric(x))
    if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
    else
      stop("Argument '", name, "' must be numeric.",
           call. = FALSE)
  ##
  if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
  ##
  if (any(x <= 0, na.rm = TRUE) | any(x >= 1, na.rm = TRUE))
    stop("Argument '", name, "' must be a numeric between 0 and 1.",
         call. = FALSE)
  ##
  invisible(NULL)
}
chklogical <- function(x, name = NULL) {
  ##
  ## Check whether argument is logical
  ##
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  if (is.numeric(x))
    x <- as.logical(x)
  ##
  if (length(x) !=  1 || !is.logical(x) || is.na(x))
    stop("Argument '", name, "' must be a logical.", call. = FALSE)
  ##
  invisible(NULL)
}
chkmiss <- function(x, name = NULL) {
  ##
  ## Check for missing values
  ##
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  if (anyNA(x))
    stop("Missing values in argument '", name, "'.",
         call. = FALSE)
  ##
  invisible(NULL)
}
chknull <- function(x, name = NULL, text = "Argument") {
  ##
  ## Check whether argument is NULL
  ##
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  if (is.null(x))
    stop(text, " '", name, "' is NULL.", call. = FALSE)
  ##
  invisible(NULL)
}
chknumeric <- function(x, min, max, zero = FALSE, length = 0,
                       name = NULL, single = FALSE) {
  if (!missing(single) && single)
    length <- 1
  ##
  ## Check numeric variable
  ##
  if (is.null(name))
    name <- deparse(substitute(x))
  ##
  x <- x[!is.na(x)]
  if (length(x) == 0)
    return(NULL)
  ##
  if (!is.numeric(x))
    stop("Non-numeric value for argument '", name, "'.",
         call. = FALSE)
  ##
  if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
  ##
  if (!missing(min) & missing(max)) {
    if (zero & min == 0 & any(x <= min, na.rm = TRUE))
      stop("Argument '", name, "' must be positive.",
           call. = FALSE)
    else if (any(x < min, na.rm = TRUE))
      stop("Argument '", name, "' must be larger equal ",
           min, ".", call. = FALSE)
  }
  ##
  if (missing(min) & !missing(max)) {
    if (zero & max == 0 & any(x >= max, na.rm = TRUE))
      stop("Argument '", name, "' must be negative.",
           call. = FALSE)
    else if (any(x > max, na.rm = TRUE))
      stop("Argument '", name, "' must be smaller equal ",
           min, ".", call. = FALSE)
  }
  ##
  if ((!missing(min) & !missing(max)) &&
      (any(x < min, na.rm = TRUE) | any(x > max, na.rm = TRUE)))
    stop("Argument '", name, "' must be between ",
         min, " and ", max, ".", call. = FALSE)
  ##
  invisible(NULL)
}
argid <- function(x, value) {
  if (any(x == value))
    res <- seq(along = x)[x == value]
  else
    res <- NA
  res
}
chkdeprecated <- function(x, new, old, warn = TRUE) {
  depr <- !is.na(argid(x, old))
  if (depr & warn)
    warning("Deprecated argument '", old, "' ignored. ",
            "Use argument '", new, "' instead.",
            call. = FALSE)
  invisible(depr)
}

chksuitable <- function(x, method,
                        classes = c("netmetabin", gs(".other_nma")),
                        addtext = "",
                        stop = TRUE,
                        status = "implemented") {
  func <- if (stop) base::stop else base::warning
  #
  if (length(addtext) != length(classes) & length(addtext) == 1)
    addtext <- rep_len(addtext, length(classes))
  #
  for (i in seq_along(classes)) {
    if (inherits(x, classes[i])) {
      func(method, " not ", status, " for an object of class \"",
           classes[i], "\"", addtext[i], ".",
           call. = FALSE)
      #
      return(FALSE)
    }
  }
  #
  return(TRUE)
}

sortprop <- function(prop, dat.trts, sep.trts) {
  res <- rep_len(NA, length(prop))
  seq.comps <- names(prop)
  trts <-
    matrix(unlist(compsplit(seq.comps, sep.trts)),
           ncol = 2, byrow = TRUE)
  trts <- as.data.frame(trts)
  names(trts) <- c("treat1", "treat2")
  ##
  for (i in seq_len(nrow(dat.trts))) {
    sel.i <-
      (trts$treat1 == dat.trts$treat1[i] &
       trts$treat2 == dat.trts$treat2[i]) |
      (trts$treat1 == dat.trts$treat2[i] &
       trts$treat2 == dat.trts$treat1[i])
    ##
    res[i] <- prop[sel.i]
  }
  ##
  res
}

#' Contribution matrix in network meta-analysis
#' 
#' @description
#' This function generates the contribution of direct comparisons to
#' every network treatment comparison. The output is a matrix where
#' rows represent network treatment effects and columns represent the
#' contribution of direct treatment effects.
#' 
#' @aliases netcontrib print.netcontrib
#' 
#' @param x An object of class \code{netmeta} or \code{netcontrib}.
#' @param method A character string indicating which method is to
#'   calculate the contribution matrix. Either \code{"randomwalk"},
#'   \code{"shortestpath"}, \code{"cccp"}, or \code{"pseudoinverse"},
#'   can be abbreviated.
#' @param hatmatrix.F1000 A logical indicating whether hat matrix
#'   given in F1000 article should be used for \code{method =
#'   "shortestpath"}.
#' @param common A logical indicating whether a contribution matrix
#'   should be printed for the common effects network meta-analysis.
#' @param random A logical indicating whether a contribution matrix
#'   should be printed for the random effects network meta-analysis.
#' @param study A logical indicating whether contributions of individual studies
#'    for each comparison should be returned (currently only supported for the
#'   \code{"shortestpath"} method).
#' @param path A logical indicating whether contributions of individual paths
#'   for each comparison should be returned (currently only supported for the
#'   \code{"shortestpath"} method).
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param verbose A logical indicating whether progress information
#'   should be printed.
#' @param \dots Additional arguments.
#' 
#' @details
#' In network meta-analysis (NMA), it is important to assess the
#' influence of limitations or other characteristics of individual
#' studies on the estimates obtained from the network. To this end,
#' the contribution matrix shows how much each direct treatment effect
#' contributes to each treatment effect estimate from network
#' meta-analysis.
#' 
#' We use ideas from graph theory to derive the proportion that is
#' contributed by each direct treatment effect using R package
#' \bold{igraph}. We start
#' with the 'projection' matrix in a two-step network meta-analysis model,
#' called the H matrix, which is analogous to the hat matrix in a
#' linear regression model. H entries are translated to proportion
#' contributions based on the observation that the rows of H can be
#' interpreted as flow networks. A stream is defined as the
#' composition of a path and its associated flow (Papakonstantinou et
#' al., 2018).
#'
#' To account for multi-arm trials, we use the H matrix from a
#' two-step (aggregate) version of the graph theoretical NMA model
#' (Davies et al., 2022). This H matrix can be obtained from
#' \code{\link{hatmatrix}} with argument \code{method = "davies"}.
#' 
#' Four methods are implemented to estimate the proportion
#' contributions:
#' 
#' (1) If argument \code{method = "randomwalk"}, an analytical
#' random-walk (RW) approach is used (Davies et al., 2022). Here, the
#' "full" version of the aggregate H matrix (\code{\link{hatmatrix}}
#' with arguments \code{method = "davies"} and \code{type = "full"})
#' is used to define RW transition matrices. For each pair of
#' treatments (ij) in the network, the elements in the corresponding
#' row of H-full define a transition matrix from node i to node j. We
#' use the \bold{igraph} package to find every (directed) path from
#' node i to node j. The flow through each path is then equal to the
#' probability that a walker takes that path. This is simply the
#' product of the transition probabilities associated with each edge
#' along the path.
#' 
#' (2) If argument \code{method = "shortestpath"}, an iterative
#' algorithm is used (Papakonstantinou et al., 2018). Broadly
#' speaking, each iteration of the algorithm consists of the following
#' steps:
#' \itemize{
#' \item A path in the evidence flow network is selected.
#' \item The minimum flow through the edges making up the path is
#'   identified. This is assigned as the flow associated with the
#'   path.
#' \item The flow of the path is subtracted from the values of
#'   flow in the edges that make up that path. This means that the edge
#'   corresponding to the minimum flow in that path is removed from the
#'   graph.
#' \item A new path is then selected from the remaining graph. The process
#'   repeats until all the evidence flow in the edges has been assigned to a
#'   path.
#' }
#' In the original F1000 paper (Papakonstantinou et al., 2018), the
#' hat matrix used did not account for correlations due to multi-arm
#' trials. For reproducibility the result of this version can be
#' obtained by specifying \code{hatmatrix.F1000 = TRUE} for
#' \code{method = "shortestpath"}. For other purposes, this method is
#' not recommended.
#'
#' (3) If argument \code{method = "cccp"}, contributions are estimated
#' using \code{\link[cccp]{l1}} from R package \bold{cccp} which must be
#' available (Rücker et al., 2024).
#'
#' (4) If argument \code{method = "pseudoinverse"}, contributions are
#' derived from an L2 solution based on a Moore-Penrose pseudoinverse
#' (Rücker et al., 2024).
#' 
#' Once the streams have been identified by any method, the proportion
#' contribution of each direct comparison is equal to the sum over the
#' flow of evidence in each path containing that edge divided by the
#' number of edges that make up that path.
#'
#' By default, treatment names are not abbreviated in
#' printouts. However, in order to get more concise printouts,
#' argument \code{nchar.trts} can be used to define the minimum number
#' of characters for abbreviated treatment names (see
#' \code{\link{abbreviate}}, argument \code{minlength}). R function
#' \code{\link{treats}} is utilised internally to create abbreviated
#' treatment names.
#'
#' Calculation of network contributions can be compute-intensive for
#' the random-walk approach in large networks. Crude information on
#' the computation progress is printed if argument \code{verbose} is
#' \code{TRUE}. In addition, computation times are printed if R
#' package \bold{tictoc} is installed.
#' 
#' @return
#' An object of class \code{netcontrib} with corresponding
#' \code{print} function. The object is a list containing the
#' following components:
#' \item{common}{Numeric matrix of percentage contributions of direct
#'   comparisons for each network comparison for the common effects
#'   model.}
#' \item{random}{Numeric matrix of percentage contributions of direct
#'   comparisons for each network comparison for the random effects
#'   model.}
#' \item{study.common}{Data frame containing study contributions for each
#'   comparison for the common effects model.}
#' \item{study.random}{Data frame containing study contributions for each
#'   comparison for the random effects model.}
#' \item{path.common}{Data frame containing path contributions for each
#'   comparison for the common effects model.}
#' \item{path.random}{Data frame containing path contributions for each
#'   comparison for the random effects model.}
#' \item{x}{As defined above.}
#' \item{tictoc.common}{Computation times under common effects model
#'   (if R package \bold{tictoc} is installed).}
#' \item{tictoc.random}{Computation times under random effects model
#'   (if R package \bold{tictoc} is installed).}
#' with the contribution matrices for common and random NMA. Each
#' matrix has the percentage contributions of each direct comparison
#' as columns for each network comparison, direct or indirect as rows.
#' 
#' @author Theodoros Papakonstantinou \email{dev@@tpapak.com}, Annabel
#'   Davies \email{annabel.davies@@manchester.ac.uk}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link[metabook]{Woods2010}}
#' 
#' @references
#' Davies AL, Papakonstantinou T, Nikolakopoulou A, Rücker G, Galla T
#' (2022):
#' Network meta-analysis and random walks.
#' \emph{Statistics in Medicine},
#' \bold{41}, 2091--2114
#' 
#' Papakonstantinou T, Nikolakopoulou A, Rücker G, Chaimani A, Schwarzer G,
#' Egger M, Salanti G (2018):
#' Estimating the contribution of studies in network meta-analysis:
#' paths, flows and streams.
#' \emph{F1000Research}
#' 
#' Rücker G, Papakonstantinou T, Nikolakopoulou A, Schwarzer G, Galla T,
#' Davies AL (2024):
#' Shortest path or random walks? A framework for path weights in network
#' meta-analysis.
#' \emph{Statistics in Medicine}.
#' 
#' @keywords contribution
#' 
#' @examples
#' \donttest{
#' # Use the Woods dataset
#' #
#' pw <- pairwise(treatment, event = r, n = N,
#'   studlab = author, data = Woods2010, sm = "OR")
#' 
#' nma <- netmeta(pw)
#' 
#' # Shortest path approach (default)
#' netcontrib(nma)
#' }
#'
#' \dontrun{ 
#' # Random walk approach
#' netcontrib(nma, method = "r")
#' }
#' 
#' @rdname netcontrib
#' @export netcontrib

netcontrib <- function(x,
                       method = "shortestpath",
                       hatmatrix.F1000 = FALSE,
                       #
                       common = x$common, random = x$random,
                       #
                       study = FALSE, path = FALSE,
                       #
                       nchar.trts = x$nchar.trts,
                       #
                       warn.deprecated = gs("warn.deprecated"),
                       verbose = FALSE,
                       ...) {
  
  ##
  ##
  ## (1) Check for netmeta object and upgrade object
  ##
  ##
  chkclass(x, "netmeta")
  x <- updateversion(x)
    
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  method <-
    setchar(method, c("randomwalk", "shortestpath", "cccp", "pseudoinverse"))
  chklogical(hatmatrix.F1000)
  if (method == "randomwalk" & hatmatrix.F1000) {
    warning("Argument 'hatmatrix.F1000' ignored for random walk method.",
            call. = FALSE)
    hatmatrix.F1000 <- FALSE
  }
  ##
  if (method == "cccp")
    is_installed_package("cccp")
  
  chknumeric(nchar.trts, min = 1, length = 1)
  #
  if (method != "shortestpath") {
    study <- FALSE
    path <- FALSE
  }
  #
  chklogical(study)
  chklogical(path)
  #
  chklogical(verbose)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (3) Create netcontrib object
  ##
  ##
  x$common <- common
  x$random <- random
  ##
  cm.f <- contribution.matrix(x, method, "common", hatmatrix.F1000,
                              study, path, verbose)
  cm.r <- contribution.matrix(x, method, "random", hatmatrix.F1000,
                              study, path, verbose)
  ##
  res <- list(common = cm.f$weights,
              random = cm.r$weights,
              study.common = cm.f$studyContribution,
              study.random = cm.r$studyContribution,
              path.common = cm.f$pathContribution,
              path.random = cm.r$pathContribution,
              method = method,
              hatmatrix.F1000 = hatmatrix.F1000,
              nchar.trts = nchar.trts,
              x = x,
              version = packageDescription("netmeta")$Version
              )
  ##
  if (!is.null(cm.f$tictoc))
    res$tictoc.common <- cm.f$tictoc
  if (!is.null(cm.r$tictoc))
    res$tictoc.random <- cm.r$tictoc
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  if (!is.null(res$tictoc.common))
    res$tictoc.fixed <- res$tictoc.common
  ##
  class(res) <- "netcontrib"
  ##
  res
}


#' @rdname netcontrib
#' 
#' @method print netcontrib
#' 
#' @export

print.netcontrib <- function(x,
                             common = x$x$common,
                             random = x$x$random,
                             digits = 4,
                             nchar.trts = x$nchar.trts,
                             legend = gs("legend"),
                             warn.deprecated = gs("warn.deprecated"),
                             ...) {
  
  ##
  ##
  ## (1) Check for netcontrib object and upgrade object
  ##
  ##
  chkclass(x, "netcontrib")
  chksuitable(x, "Network contributions", classes = gs(".other_nma"))
  x <- updateversion(x)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chknumeric(nchar.trts, length = 1)
  chklogical(legend)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (3) Print results for contribution matrix
  ##
  ##
  matitle(x$x)
  ##
  cat("Contribution matrix (",
      if (is.null(x$method) | x$method == "shortestpath")
        "Papakonstantinou et al., 2018, F1000Research"
      else if (x$method == "randomwalk")
        "Davies et al., 2022, Stat Med"
      else if (x$method == "cccp")
        paste0("Ruecker et al., 2023, ",
               "L1 solution based on R package cccp")
      else if (x$method == "pseudoinverse")
        paste0("Ruecker et al., 2023, ",
               "L2 solution based on Moore-Penrose pseudoinverse")
      else
        "unknown method",
      ")",
      sep = "")
  ##
  if ((is.null(x$method) | x$method == "shortestpath") & x$hatmatrix.F1000)
    cat(",\nhat matrix does not take correlation of",
        "multi-arm studies into account")
  ##
  cat("\n\n")
  
  
  ##
  trts <- x$x$trts
  ##
  if (common) {
    rownames(x$common) <- comps(x$common, trts, x$x$sep.trts, nchar.trts)
    colnames(x$common) <- comps(x$common, trts, x$x$sep.trts, nchar.trts,
                                row = FALSE)
    ##
    cat("Common effects model:\n\n")
    prmatrix(round(x$common, digits))
    if (random)
      cat("\n")
  }
  if (random) {
    rownames(x$random) <- comps(x$random, trts, x$x$sep.trts, nchar.trts)
    colnames(x$random) <- comps(x$random, trts, x$x$sep.trts, nchar.trts,
                                row = FALSE)
    ##
    cat("Random effects model:\n\n")
    prmatrix(round(x$random, digits))
  }
  ##
  ## Add legend with abbreviated treatment labels
  ##
  legendabbr(trts, treats(trts, nchar.trts), legend & (common | random))
  ##
  invisible(NULL)
}

va.image <- function(x,
                     design, Hpn, h1, wi,
                     ##
                     size = abs(Hpn), labels = NULL,
                     cex = 1, col, bg = "grey", box.bg = TRUE,
                     min.size = 0.10,
                     xlab = NULL, ylab = NULL, cex.axis = 1.1) {
  
  
  if (length(wi) > 0)
    design <- design[-wi, ]
  ##
  design.comb <- rep(NA, length(as.character(design$comparison)))
  ##
  for (i in 1:(length(as.character(design$comparison)))) {
    if ((((design$narms)[h1$order])[i]) > 2)
      design.comb[i] <- paste(as.character(design$comparison)[h1$order][i],
                              as.character(design$design)[h1$order][i],
                              sep = "_")
    else
      design.comb[i] <-as.character(design$comparison)[h1$order][i]
  }
  ##
  sc <- max(strwidth(design.comb, "inches")) / par("cin")[2]
  oldpar <- par(mar = c(1, 1.5 + sc, 1.5 + sc,
                        1.5 + strwidth("-4", "inches") / par("cin")[2]))
  ##
  plot(0, type = "n", xlim = c(0.036, 0.963), ylim = c(0.036, 0.963),
       bty = "n", xlab = "", ylab = "", axes = FALSE)
  ##
  if (is.null(xlab) && !is.null(rownames(x))) xlab <- rownames(x)
  if (is.null(ylab) && !is.null(colnames(x))) ylab <- colnames(x)
  ##
  rect(0, 0, 1, 1, col = bg, border = NA)
  ##
  runit <- 1 / ncol(x)
  ##
  center.x <- seq(from = runit / 2, to = 1 - runit / 2, len = ncol(x))
  center.y <- seq(from = runit / 2, to = 1 - runit / 2, len = ncol(x))
  ##
  val.grid <- seq(from = min(x), to = max(x), len = length(col) + 1)
  val.grid <- val.grid[-length(val.grid)]
  if (min(x) == max(x)) val.grid <- val.grid[1]
  ##
  col[unlist(lapply(x[, 1], function(arg) sum(arg < val.grid)))]
  ##
  min.sv <- min(size)
  max.sv <- max(size)

  fg.col <- bg.col <- apply(col2rgb(col) / 255, 2,
                            function(arg) rgb(arg[1], arg[2], arg[3]))
  if (box.bg)
    fg.col <- apply(col2rgb(col) / 255 * 0.8, 2,
                    function(arg) rgb(arg[1], arg[2], arg[3]))
  
  
  for (i in 1:ncol(x)) {
    actual.size <-
      (sqrt((size[, i] - min.sv) / (max.sv - min.sv)) + min.size) /
      (1 + 4 * min.size) * runit / 2
    ##
    rect(center.x - runit / 2, center.y[i] - runit / 2,
         center.x + runit / 2, center.y[i] + runit / 2,
         col = bg.col[unlist(lapply(x[, i],
                                    function(arg) sum(arg >= val.grid)))],
         border = NA)
    ##
    if (box.bg)
      rect(center.x - actual.size, center.y[i] - actual.size,
           center.x + actual.size, center.y[i] + actual.size,
           col = fg.col[unlist(lapply(x[, i],
                                      function(arg) sum(arg >= val.grid)))],
           border = NA)
    if (!is.null(labels))
      text(center.x, center.y[i], labels[, i], cex = cex)
  }
  
  
  if (!is.null(ylab))
    axis(side = 2, at = center.y, labels = ylab, tick = FALSE,
         line = NA, las = 2, cex.axis = cex.axis)
  ##
  if (!is.null(xlab))
    axis(side = 3, at = center.x, labels = xlab, tick = FALSE,
         line = NA, las = 2, cex.axis = cex.axis)

  axis(2, at = center.x, lty = 0, las = 2,
       labels = design.comb[length(design.comb):1], cex.axis = 1)
  axis(3, at = center.y, lty = 0, las = 3,
       labels = design.comb, cex.axis = 1)
  
  oldpar
}


legend.col <- function(col, lev, oldpar) {
  n <- length(col)
  bx <- par()$usr
  box.cx <- c(bx[2] + (bx[2] - bx[1]) / 1000,
              bx[2] + (bx[2] - bx[1]) / 1000 + (bx[2] - bx[1]) / 50)
  box.cy <- c(bx[3], bx[3])
  box.sy <- (bx[4] - bx[3]) / n
  xx <- rep(box.cx, each = 2)
  oldpar <- c(par(xpd = TRUE), oldpar)
  ##
  for (i in 1:n) {
    yy <- c(box.cy[1] + (box.sy * (i - 1)),
            box.cy[1] + (box.sy * i),
            box.cy[1] + (box.sy * i),
            box.cy[1] + (box.sy * (i - 1)))
    polygon(xx, yy, col = col[i], border = col[i])
  }
  ##
  par(new = TRUE)
  ##
  plot(0, 0, type = "n",
       ylim = c(min(lev), max(lev)),
       yaxt = "n", ylab = "", xaxt = "n", xlab = "",
       frame.plot = FALSE, yaxs = "i")
  ##
  axis(side = 4, tick = FALSE, line = .25, las = 1)
  
  invisible(NULL)
}

#' Network meta-analysis of adjuvant treatments to levodopa therapy
#' for Parkinson's disease
#' 
#' @description
#' This dataset contains data from a Cochrane review assessing
#' efficacy and safety of three drug classes as adjuvant treatment to
#' levodopa therapy in patients with Parkinson’s disease and motor
#' complications (Stowe et al., 2010). The authors conducted three
#' pairwise meta-analyses comparing dopamine agonists,
#' catechol-O-methyl transferase inhibitors (COMTIs), and monoamine
#' oxidase type B inhibitors (MAOBIs), respectively, with placebo.
#' 
#' The primary outcome was the mean reduction of the time spent in a
#' relatively immobile ’off’ phase (mean off-time), calculated in
#' hours per day. Relative treatment effects were expressed as mean
#' difference. Data on this outcome were available for 5,331 patients
#' from 28 studies comparing an active treatment with placebo and one
#' three-arm study comparing two active treatments with placebo.
#' 
#' @name Stowe2010
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{id}}\tab study id \cr
#' \bold{\emph{t1}}\tab treatment 1 \cr
#' \bold{\emph{y1}}\tab treatment effect arm 1 \cr
#' \bold{\emph{sd1}}\tab Standard deviation arm 1 \cr
#' \bold{\emph{n1}}\tab Sample size arm 1 \cr
#' \bold{\emph{t2}}\tab treatment 2 \cr
#' \bold{\emph{y2}}\tab treatment effect arm 2 \cr
#' \bold{\emph{sd2}}\tab Standard deviation arm 2 \cr
#' \bold{\emph{n2}}\tab Sample size arm 2 \cr
#' \bold{\emph{t3}}\tab treatment 3 \cr
#' \bold{\emph{y3}}\tab treatment effect arm 3 \cr
#' \bold{\emph{sd3}}\tab Standard deviation arm 3 \cr
#' \bold{\emph{n3}}\tab Sample size arm 3
#' }
#' 
#' @note
#' The dataset Stowe2010 is identical to dataset
#' \code{\link[metabook]{Stowe2010}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[meta]{pairwise}}, \code{\link[meta]{metacont}},
#'   \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
#' 
#' @source
#' Stowe R, Ives N, Clarke CE, Deane K, Hilten V, Wheatley K, et
#' al. (2010):
#' Evaluation of the efficacy and safety of adjuvant treatment to
#' levodopa therapy in Parkinson's disease patients with motor
#' complications.
#' \emph{Cochrane Database of Systematic Reviews}
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Stowe2010)
#' 
#' # Example using pairwise(), netmeta() and netrank():
#' # example(Stowe2010, package = "metabook")

NULL

#' \sQuote{Comparison-adjusted} radial plot
#' 
#' @description
#' Draw a \sQuote{comparison-adjusted} radial plot to assess funnel
#' plot asymmetry in network meta-analysis.
#' 
#' @param x An object of class \code{netmeta}.
#' @param order A mandatory character or numerical vector specifying
#'   the order of treatments or list of comparators (see Details).
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param level The confidence level utilised in the plot.
#' @param pch The plotting symbol(s) used for individual studies
#'   within direct comparisons.
#' @param col The colour(s) used for individual studies within direct
#'   comparisons.
#' @param legend A logical indicating whether a legend with
#'   information on direct comparisons should be added to the plot.
#' @param pos.legend The position of the legend describing plotting
#'   symbols and colours for direct comparisons.
#' @param pos.tests The position of results for test(s) of funnel plot
#'   asymmetry.
#' @param lump.comparator A logical indicating whether comparators
#'   should be lumped, e.g., to specify inactive treatments.
#'   information on direct comparisons should be added to the plot.
#' @param text.comparator A character string used in the plot to label
#'   the comparator if \code{lump.comparator} is \code{TRUE}.
#' @param method.bias A character vector indicating which test(s) for
#'   funnel plot asymmetry to use. Admissible values are
#'   \code{"Begg"}, \code{"Egger"}, and \code{"Thompson"}, can be
#'   abbreviated. See function \code{\link[meta]{metabias}}.
#' @param text.linreg A character string used in the plot to label the
#'   Egger test for funnel plot asymmetry.
#' @param text.rank A character string used in the plot to label the
#'   Begg test for funnel plot asymmetry.
#' @param text.mm A character string used in the plot to label the
#'   Thompson-Sharp test for funnel plot asymmetry.
#' @param sep.trts A character used in comparison names as separator
#'   between treatment labels.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see
#'   \code{\link{netmeta}}).
#' @param digits.pval Minimal number of significant digits for p-value
#'   of test(s) for funnel plot asymmetry.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param linreg Deprecated argument (replaced by \code{method.bias}).
#' @param rank Deprecated argument (replaced by \code{method.bias}).
#' @param mm Deprecated argument (replaced by \code{method.bias}).
#' @param \dots Additional graphical arguments passed as arguments to
#'   \code{\link[meta]{radial.meta}}.
#' 
#' @details
#' 
#' A \sQuote{comparison-adjusted} radial plot is drawn in the active
#' graphics window. The idea of this radial plot is similar to a
#' \sQuote{comparison-adjusted} funnel plot (Chaimani & Salanti,
#' 2012).
#' 
#' Argument \code{order} is mandatory to determine the order of
#' treatments (Chaimani et al., 2013):
#' 
#' \emph{\dQuote{Before using this plot, investigators should order
#' the treatments in a meaningful way and make assumptions about how
#' small studies differ from large ones. For example, if they
#' anticipate that newer treatments are favored in small trials, then
#' they could name the treatments from oldest to newest so that all
#' comparisons refer to \sQuote{old versus new intervention}. Other
#' possibilities include defining the comparisons so that all refer to
#' an active treatment versus placebo or sponsored versus
#' non-sponsored intervention.}}
#' 
#' Alternatively, it is possible to only provide a single or few
#' treatment name(s) in argument \code{order} to define the
#' comparator(s). In this case only comparisons with this / these
#' treatment(s) will be considered. If argument \code{lump.comparator}
#' is \code{TRUE}, all comparators will be lumped into a single
#' group. The text for this group can be specified with argument
#' \code{text.comparator}.
#'
#' @return
#' A data frame with the following columns:
#' \item{studlab}{Study label.}
#' \item{treat1}{Label/Number for first treatment.}
#' \item{treat2}{Label/Number for second treatment.}
#' \item{comparison}{Treatment comparison.}
#' \item{TE}{Estimate of treatment effect, e.g., log odds ratio.}
#' \item{TE.direct}{Pooled estimate from direct evidence.}
#' \item{TE.adj}{\sQuote{Comparison-adjusted} treatment effect (TE -
#'   TE.direct).}
#' \item{seTE}{Standard error of treatment estimate.}
#' \item{pch}{Plotting symbol(s).}
#' \item{col}{Colour of plotting symbol(s).}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link[meta]{radial.meta}},
#'   \code{\link[meta]{funnel.meta}}, \code{\link[meta]{metabias}}
#' 
#' @references
#' Chaimani A & Salanti G (2012):
#' Using network meta-analysis to evaluate the existence of
#' small-study effects in a network of interventions.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 161--76
#' 
#' Chaimani A, Higgins JP, Mavridis D, Spyridonos P, Salanti G (2013):
#' Graphical tools for network meta-analysis in STATA.
#' PLOS ONE,
#' \bold{8}, e76654
#' 
#' @keywords hplot
#' 
#' @examples
#' \donttest{
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' #
#' nma <- netmeta(pw)
#' 
#' # 'Comparison-adjusted' radial plot not created as argument 'order'
#' # is missing
#' #
#' try(radial(nma))
#' 
#' # Only show comparisons with placebo
#' #
#' radial(nma, order = "pl")
#' 
#' # Add result for Egger test of funnel plot asymmetry
#' #
#' radial(nma, order = "pl", method.bias = "Egger",
#'   digits.pval = 2)
#' 
#' # (Non-sensical) alphabetic order of treatments with placebo as
#' # last treatment
#' #
#' ord <- c("a", "b", "me", "mi", "pi", "r", "si", "su", "v", "pl")
#' radial(nma, order = ord)
#'
#' # Add results for tests of funnel plot asymmetry and use different
#' # plotting symbols and colours
#' #
#' radial(nma, order = ord,
#'   pch = rep(c(15:18, 1), 3), col = 1:3,
#'   method.bias = c("Egger", "Begg", "Thompson"), digits.pval = 2)
#' 
#' # Same results for tests of funnel plot asymmetry using reversed
#' # order of treatments
#' #
#' radial(nma, order = rev(ord),
#'   pch = rep(c(15:18, 1), 3), col = 1:3,
#'   method.bias = c("Egger", "Begg", "Thompson"), digits.pval = 2)
#' 
#' # Calculate tests for funnel plot asymmetry
#' #
#' f1 <- radial(nma, order = ord)
#' #
#' metabias(metagen(TE.adj, seTE, data = f1))
#' metabias(metagen(TE.adj, seTE, data = f1), method = "Begg")
#' metabias(metagen(TE.adj, seTE, data = f1), method = "Thompson")
#' }
#'
#' @method radial netmeta
#' @export

radial.netmeta <- function(x,
                           order,
                           pooled = ifelse(x$random, "random", "common"),
                           ##
                           level = x$level,
                           ##
                           pch,
                           col = "black",
                           ##
                           legend = FALSE,
                           ##
                           pos.legend = "topright",
                           pos.tests = "topleft",
                           ##
                           lump.comparator = FALSE,
                           text.comparator = "comparator",
                           ##
                           method.bias,
                           text.linreg = "(Egger)",
                           text.rank = "(Begg-Mazumdar)",
                           text.mm = "(Thompson-Sharp)",
                           ##
                           sep.trts = x$sep.trts,
                           nchar.trts = x$nchar.trts,
                           ##
                           digits.pval = gs("digits.pval"),
                           ##
                           warn.deprecated = gs("warn.deprecated"),
                           linreg = FALSE,
                           rank = FALSE,
                           mm = FALSE,
                           ##
                           ...) {
  
  
  ##
  ##
  ## Generate 'comparison-adjusted' radial plot
  ##
  ##
  
  
  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netmeta")
  x <- updateversion(x)
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  chklogical(lump.comparator)
  ##
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  if (missing(order))
    stop("Argument 'order' with a meaningful order of treatments ",
         "must be provided.\n  ",
         "(see help page of radial.netmeta for some examples).")
  else {
    order <- catch("order", mc, x, sfsp)
    ##
    if (length(order) != length(x$trts)) {
      order <- setchar(order, x$trts)
      order.all <- c(x$trts[!(x$trts %in% order)], order)
    }
    else
      order.all <- order <- setseq(order, x$trts)
  }
  ##
  if ((length(order) == length(order.all)) & lump.comparator) {
    warning("Argument 'lump.comparator' ignored as full treatment order is ",
            "provided.",
            call. = FALSE)
    lump.comparator <- FALSE
  }
  ##
  chklogical(warn.deprecated)
  ##
  missing.method.bias <- missing(method.bias)
  if (missing.method.bias)
    method.bias <- character(0)
  ##
  chklogical(legend)
  ##
  deprecated2(method.bias, missing.method.bias,
              linreg, missing(linreg), warn.deprecated)
  deprecated2(method.bias, missing.method.bias,
              rank, missing(rank), warn.deprecated)
  deprecated2(method.bias, missing.method.bias,
              mm, missing(mm), warn.deprecated)
  ##
  chklogical(linreg)
  chklogical(rank)
  chklogical(mm)
  ##
  if (missing.method.bias) {
    if (linreg)
      method.bias <- "Egger"
    if (rank)
      method.bias <- c(method.bias, "Begg")
    if (mm)
      method.bias <- c(method.bias, "Thompson")
  }
  else
    method.bias <- setmethodbias(method.bias, 1:3)
  ##
  linreg <- "Egger" %in% method.bias
  rank <- "Begg" %in% method.bias
  mm <- "Thompson" %in% method.bias
  
  
  chkchar(text.comparator)
  chkchar(text.linreg)
  chkchar(text.rank)
  chkchar(text.mm)
  ##
  chkchar(sep.trts)
  chknumeric(nchar.trts, min = 1, length = 1)
  ##
  chknumeric(digits.pval, min = 1, length = 1)
  
  
  ##
  ##
  ## (2) Get data
  ##
  ##
  TE <- x$TE
  seTE <- x$seTE
  studlab <- x$studlab
  ##
  treat1 <- x$treat1
  treat2 <- x$treat2
  ##
  comparison <- paste(treat1, treat2, sep = sep.trts)
  comparison21 <- paste(treat2, treat1, sep = sep.trts)
  ##
  treat1.pos <- as.numeric(factor(treat1, levels = order.all))
  treat2.pos <- as.numeric(factor(treat2, levels = order.all))
  ##
  trts.abbr <- treats(x$trts, nchar.trts)
  trt1 <- as.character(factor(treat1, levels = x$trts, labels = trts.abbr))
  trt2 <- as.character(factor(treat2, levels = x$trts, labels = trts.abbr))
  ##
  comp <- paste(trt1, trt2, sep = sep.trts)
  comp21 <- paste(trt2, trt1, sep = sep.trts)
  ##
  wo <- treat1.pos > treat2.pos
  ##
  if (any(wo)) {
    ttreat1.pos <- treat1.pos
    treat1.pos[wo] <- treat2.pos[wo]
    treat2.pos[wo] <- ttreat1.pos[wo]
    ##
    TE[wo] <- -TE[wo]
    ##
    ttreat1 <- treat1
    treat1[wo] <- treat2[wo]
    treat2[wo] <- ttreat1[wo]
    ##
    comparison[wo] <- comparison21[wo]
    ##
    ttrt1 <- trt1
    trt1[wo] <- trt2[wo]
    trt2[wo] <- ttrt1[wo]
    ##
    comp[wo] <- comp21[wo]
  }
  ##
  o <- order(treat1.pos, treat2.pos)
  ##
  TE <- TE[o]
  seTE <- seTE[o]
  ##
  studlab <- studlab[o]
  ##
  treat1 <- treat1[o]
  treat2 <- treat2[o]
  comparison <- comparison[o]
  ##
  trt1 <- trt1[o]
  trt2 <- trt2[o]
  comp <- comp[o]
  ##
  res <- data.frame(studlab,
                    treat1, treat2, comparison,
                    trt1, trt2, comp,
                    TE, TE.direct = NA, TE.adj = NA, seTE)
  
  
  ##
  ##
  ## (3) Calculate 'comparison-adjusted' treatment effects
  ##
  ##
  if (is.numeric(treat1))
    treat1 <- as.character(treat1)
  if (is.numeric(treat2))
    treat2 <- as.character(treat2)
  ##
  if (pooled == "common")
    for (i in seq_along(res$TE))
      res$TE.direct[i] <- x$TE.direct.common[treat1[i], treat2[i]]
  else
    for (i in seq_along(res$TE))
      res$TE.direct[i] <- x$TE.direct.random[treat1[i], treat2[i]]
  ##
  res$TE.adj <- res$TE - res$TE.direct
  ##
  if (length(order) != length(order.all))
    res <- subset(res, treat1 %in% order | treat2 %in% order)
  ##
  if (lump.comparator) {
    res <- subset(res, !(treat1 %in% order & treat2 %in% order))
    ##
    res$trt2[res$treat2 %in% order] <- text.comparator
    res$treat2[res$treat2 %in% order] <- text.comparator
    ##
    res$comp <- paste(res$trt1, res$trt2, sep = sep.trts)
    res$comparison <- paste(res$treat1, res$treat2, sep = sep.trts)
  }
  
  
  ##
  ##
  ## (4) Calculate necessary data for radial plot
  ##
  ##
  m.adj <-
    suppressWarnings(metagen(res$TE.adj, res$seTE,
                             studlab = res$studlab, sm = x$sm,
                             method.tau = "DL", method.tau.ci = ""))
  ##
  n.comps <- length(unique(res$comparison))
  ##
  ## Argument 'pch'
  ##
  if (missing(pch))
    pch <- seq_len(n.comps)
  else {
    bad <- FALSE
    ##
    if (length(pch) != n.comps) {
      if (length(pch) > n.comps)
        bad <- TRUE
      else if (n.comps %% length(pch) != 0)
        bad <- TRUE
      ##
      if (bad)
        stop("Length of argument 'pch' (", length(pch),
             ") does not match the number of direct pairwise comparisons (",
             n.comps, ").")
      ##
      pch <- cbind(seq_len(n.comps), pch)[, 2]
    }
  }
  ##
  ## Argument 'col'
  ##
  bad <- FALSE
  ##
  if (length(col) != n.comps) {
    if (length(col) > n.comps)
      bad <- TRUE
    else if (n.comps %% length(col) != 0)
      bad <- TRUE
    ##
    if (bad)
      stop("Length of argument 'col' (", length(col),
           ") does not match the number of direct pairwise comparisons (",
           n.comps, ").")
    ##
    col <- cbind(seq_len(n.comps), col)[, 2]
  }
  ##
  res$col <- res$pch <- as.numeric(factor(res$comparison,
                                          levels = unique(res$comparison)))
  ##
  res$pch <- pch[res$pch]
  res$col <- col[res$col]
  ##
  if (is.numeric(col))
    res$col <- as.numeric(res$col)
  ##
  if (is.numeric(pch))
    res$pch <- as.numeric(res$pch)
  
  
  ##
  ##
  ## (5) Radial plot
  ##
  ##
  radial(m.adj,
         pch = res$pch,
         col = res$col,
         level = level,
         common = FALSE,
         ...
         )
  ##
  if (legend) {
    d1 <- unique(res[, c("comp", "pch", "col"), drop = FALSE])
    legend(pos.legend, legend = d1$comp,
           pch = d1$pch, col = d1$col)
  }
  ##
  if (linreg)
    mb.linreg <- suppressWarnings(metabias(m.adj, method = "Egger"))
  if (rank)
    mb.rank <- suppressWarnings(metabias(m.adj, method = "Begg"))
  if (mm)
    mb.mm <- suppressWarnings(metabias(m.adj, method = "Thompson"))
  ##
  if (linreg | rank | mm)
    legend(pos.tests,
           legend = paste(formatPT(c(if (linreg) mb.linreg$p.value,
                                            if (rank) mb.rank$p.value,
                                            if (mm) mb.mm$p.value),
                                          digits = digits.pval, lab = TRUE),
                          c(if (linreg) text.linreg,
                            if (rank) text.rank,
                            if (mm) text.mm)))
  
  
  ##
  if (all(res$comparison == res$comp)) {
    res$trt1 <- NULL
    res$trt2 <- NULL
    res$comp <- NULL
  }
  ##
  attr(res, "pooled") <- pooled
  attr(res, "order") <- order
  attr(res, "lump.comparator") <- lump.comparator
  
  
  invisible(res)
}

#' Network meta-regression with a single continuous or binary covariate
#'
#' @description
#' Network meta-regression with a single continuous or binary covariate for
#' objects of class \code{netmeta} (Kwarteng et al., 2026). This is a wrapper
#' function for \code{\link[metafor]{rma.mv}} in the R package \bold{metafor}
#' (Viechtbauer 2010).
#'
#' @details
#' This R function is a wrapper function for \code{\link[metafor]{rma.mv}} in
#' the R package \bold{metafor} (Viechtbauer 2010).
#'
#' Note, results are not back-transformed in printouts of network meta-analyses
#' using summary measures with transformations, e.g., log risk ratios are
#' printed instead of the risk ratio if argument \code{sm = "RR"}.
#'
#' Argument '\dots{}' can be used to pass additional arguments to R
#' function \code{\link[metafor]{rma.mv}}. For example, argument
#' \code{control} to provide a list of control values for the
#' iterative estimation algorithm. See help page of R function
#' \code{\link[metafor]{rma.mv}} for more details.
#'
#' @param x An object of class \code{netmeta}.
#' @param covar Continuous or binary covariate.
#' @param consistency A logical indicating whether a consistency or
#'   unrelated mean interaction effect (UMIE) model should be assumed.
#' @param assumption A character string indicating which assumption is done
#'   for the covariate; either "independent" or "common" (can be abbreviated).
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance tau-squared. Either
#'   \code{"FE"}, \code{"REML"}, or \code{"ML"}.
#' @param level The level used to calculate confidence intervals for regression
#'   coefficients.
#' @param reference.group Reference treatment.
#' @param direction1 Directionality parameter for the first treatment group
#'   when using UMIE models.
#' @param direction2 Directionality parameter for the second treatment group
#'   when using UMIE models.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   errors.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param print.se A logical specifying whether standard errors should be
#'   printed.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param \dots Additional arguments passed to \code{\link[metafor]{rma.mv}}.
#'
#' @return
#' An object of class \code{c("netmetareg", "rma.mv", "rma")}. Please
#' look at the help page of R function \code{\link[metafor]{rma.mv}}
#' for more details on the output from this function.
#'
#' In addition, a list \code{.netmeta} is added to the output containing
#' the following components:
#' \item{x, covar, method.tau}{As defined above.}
#' \item{dots}{Information provided in argument '\dots{}'.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package \bold{netmeta} used to create
#'   object.}
#' \item{version.metafor}{Version of R package \bold{metafor} used to
#'   create object.}
#'
#' @author Nana-adjoa Kwarteng
#'   \email{nana-adjoa.kwarteng@@uniklinik-freiburg.de},
#'   Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{netmeta}}
#'
#' @references
#' Kwarteng N, Evrenoglou T, Mueller J, Elsaesser M, Schramm E, Schwarzer G,
#' Nikolakopoulou A (2026):
#' Illustrating the assumptions of meta-regression in treatment networks.
#' Preprint available at \emph{Research Square},
#' \doi{10.21203/rs.3.rs-8235913/v1}
#' 
#' Viechtbauer W (2010):
#' Conducting Meta-Analyses in R with the Metafor Package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#'
#' @keywords models regression
#' 
#' @examples
#' #
#' # 1) Smoking cessation example
#' #
#' data(smokingcessation)
#' # Add variable with (fictitious) risk of bias values
#' # with 1 = "low risk" and 2 = "high risk"
#' #
#' smokingcessation$rob <- rep(1:2, 12)
#' 
#' pw1 <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' 
#' # Run network meta-analysis (NMA)
#' nma1 <- netmeta(pw1, common = FALSE, ref = "A")
#' 
#' # Network meta-regression with continuous covariate and assumption of
#' # independent slopes
#' nr1 <- netmetareg(nma1, rob)
#' nr1
#' 
#' \donttest{
#' # 2) Pain Prevention of Propofol Injection Example (Jalota2011)
#' #
#' # Create pairwise object
#' pw2 <- pairwise(treat = trt, event =  pain, n = n,
#'   studlab = id, data = Jalota2011, sm = "OR")
#' 
#' # Run network meta-analysis (NMA)
#' nma2 <- netmeta(pw2, common = FALSE, ref = "Hand vein")
#' 
#' # NMR with independent and consistent assumption (default)
#' nmr2_i <- netmetareg(nma2, covar = seTE, consistency = TRUE,
#'   assumption = "i")
#' # NMR with common and consistent assumption
#' nmr2_c <- netmetareg(nma2, covar = seTE, consistency = TRUE,
#'   assumption = "c")
#' # NMR with independent UMIE model (no consistency in interactions) using
#' # default treatment order
#' nmr2_i_umie <- netmetareg(nma2, covar = seTE, consistency = FALSE,
#'   assumption = "i")
#'
#' # 3) Physical therapy example (Hong 2015)
#' #
#' dat3 <- Hong2015
#' # Externally create a customizable treatment order indicator variable,
#' # which will later be modified
#' dat3$trtnum <- as.numeric(as.factor(dat3$trt))
#' dat3 <- do.call(rbind, lapply(split(dat3, dat3$id),
#'   function(x) {
#'     x$direction <- ifelse(x$trtnum == min(x$trtnum), 0, 1)
#'   x}))
#' 
#' pw3 <- pairwise(treat = trt, mean = mean_pain, n = n, sd = sd_pain,
#'   studlab=id, data = dat3, sm = "MD")
#' # Create difference variable for covariate
#' pw3$disability_diff <- pw3$mean_disability1 - pw3$mean_disability2
#' 
#' # Update directionality parameters so that they will exclude imputed values
#' sel.NA <- is.na(pw3$disability_diff)
#' pw3$direction1[sel.NA] <- 0
#' pw3$direction2[sel.NA] <- 0
#' 
#' # Mean imputations for missing covariate values
#' pw3$disability_diff[sel.NA] <- mean(pw3$disability_diff, na.rm = TRUE)
#' 
#' # NMA
#' nma3 <- netmeta(pw3, common = FALSE, ref = "No treatment")
#' 
#' # NMR with independent and consistent assumption (default)
#' nmr3_i <- netmetareg(nma3, covar = disability_diff, consistency = TRUE,
#'   assumption = "i")
#' # NMR with common and consistent assumption
#' nmr3_c <- netmetareg(nma3, covar = disability_diff, consistency = TRUE,
#'   assumption = "c")
#' # NMR with independent UMIE model (no consistency in interactions) using
#' # default treatment order variable
#' nmr3_i_umie <- netmetareg(nma3, covar = disability_diff, consistency = FALSE,
#'   assumption = "i")
#' # NMR with independent UMIE model (no consistency in interactions) using
#' # custom treatment order variable which excludes the mean imputations from
#' # interaction estimation
#' nmr3_i_umie_noimp <- netmetareg(nma3, covar = disability_diff,
#'   consistency = FALSE, assumption = "i",
#'    direction1 = "direction1", direction2 = "direction2")
#' # NMR with common UMIE model (no consistency in interactions) using default
#' # treatment order variable
#' nmr3_c_umie <- netmetareg(nma3, covar = disability_diff, consistency = FALSE,
#'   assumption = "c")
#' # NMR with common UMIE model (no consistency in interactions) using custom
#' # treatment order variable which excludes the mean imputations from
#' # interaction estimation
#' nmr3_c_umie_noimp <- netmetareg(nma3, covar = disability_diff,
#'   consistency = FALSE, assumption = "c",
#'   direction1 = "direction1", direction2 = "direction2")
#' }
#' 
#' @rdname netmetareg
#' @method netmetareg netmeta
#' @export 

netmetareg.netmeta <- function(x, covar = NULL,
                               consistency = TRUE,
                               assumption = "independent",
                               method.tau = if (!x$random) "FE" else "REML",
                               level = x$level.ma,
                               reference.group = x$reference.group,
                               direction1 = NULL, direction2 = NULL,
                               nchar.trts = x$nchar.trts, ...) {
  
  #
  #
  # (1) Checks and assignments
  #
  #
  
  chkclass(x, "netmeta")
  #
  x <- updateversion(x)
  #
  chklogical(consistency)
  assumption <- setchar(assumption, c("independent", "common"))
  #
  reference.group <- setref(reference.group, x$trts)
  method.tau <- setchar(method.tau, c("REML", "ML", "FE"))
  chklevel(level)
  #
  sm <- x$sm
  #
  args <- list(...)
  # Check whether first argument is a list. In this case only use
  # this list as input.
  if (length(args) > 0 && is.list(args[[1]]))
    args <- args[[1]]
  #
  max.ia <- replaceNULL(args[["max.ia"]], FALSE)
  chklogical(max.ia)
  
  
  #
  #
  # (2) Return network meta-analysis object if covariate is missing
  #
  #
  
  if (missing(covar)) {
    warning("No network meta-regresssion conducted as argument 'covar'",
            "is missing.")
    return(x)
  }
  
  
  #
  #
  # (3) Create analysis data set
  #
  #
  
  data <- x$data
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (is.null(data))
    data <- sfsp
  #
  # Catch covariate
  #
  covar.name <- gsub("\"", "", deparse(substitute(covar)), fixed = TRUE)
  covar.name <- gsub("$", ".", covar.name, fixed = TRUE)
  covar.name <- gsub("+", "_", covar.name, fixed = TRUE)
  covar.name <- gsub("-", "_", covar.name, fixed = TRUE)
  covar.name <- gsub(":", "_", covar.name, fixed = TRUE)
  #
  covar <- catch("covar", mc, data, sfsp)
  #
  trts <- x$trts
  trts.abbr <- treats(x$trts, nchar.trts)
  #
  reference.group <- trts.abbr[trts == reference.group]
  
  dat <- data.frame(studlab = x$data$.studlab,
                    treat1 = x$data$.treat1,
                    treat2 = x$data$.treat2,
                    TE = x$data$.TE,
                    seTE = x$data$.seTE)
  #
  dat$treat1 <-
    as.character(factor(dat$treat1, levels = trts, labels = trts.abbr))
  dat$treat2 <-
    as.character(factor(dat$treat2, levels = trts, labels = trts.abbr))
  #
  dat$treat1 <- gsub(" ", "_", dat$treat1, fixed = TRUE)
  dat$treat2 <- gsub(" ", "_", dat$treat2, fixed = TRUE)
  #
  if (length(unique(c(dat$treat1, dat$treat2))) != length(x$trts))
    stop("Number of treatments changes after replacing whitespaces with '_'.",
         call. = FALSE)
  #
  if (is.factor(covar) && length(levels(covar)) > 2)
    stop("Network meta-regression not available for a covariate of ",
         "class 'factor' with more than two categories.",
         call. = FALSE)
  #
  # if (is.logical(covar)) # deprecated because model matrix is now constructed manually
  #  covar <- as.numeric(covar)
  #
  dat[[covar.name]] <- covar
  #
  available.n <- available.events <- available.times <- available.sds <- FALSE
  if (!is.null(x$data$.n1) & !is.null(x$data$.n2)) {
    dat$n1 <- x$data$.n1
    dat$n2 <- x$data$.n2
    available.n <- TRUE
  }
  #
  if (!is.null(x$data$.mean1) & !is.null(x$data$.mean2)) {
    dat$mean1 <- x$data$.mean1
    dat$mean2 <- x$data$.mean2
  }
  #
  if (!is.null(x$data$.sd1) & !is.null(x$data$.sd2)) {
    dat$sd1 <- x$data$.sd1
    dat$sd2 <- x$data$.sd2
    available.sds <- TRUE
  }
  #
  if (!is.null(x$data$.event1) & !is.null(x$data$.event2)) {
    dat$event1 <- x$data$.event1
    dat$event2 <- x$data$.event2
    available.events <- TRUE
  }
  #
  if (!is.null(x$data$.time1) & !is.null(x$data$.time2)) {
    dat$time1 <- x$data$.time1
    dat$time2 <- x$data$.time2
    available.times <- TRUE
  }
  #
  if (!is.null(x$data$.incr1) & !is.null(x$data$.incr2)) {
    dat$incr1 <- x$data$.incr1
    dat$incr2 <- x$data$.incr2
  }
  
  # import or create directionality parameters
  if (!consistency) {
    # Handle directionality parameters for inconsistency model
    has_treat1_dir <- !is.null(direction1)
    has_treat2_dir <- !is.null(direction2)
    
    if (has_treat1_dir & has_treat2_dir) {
      # Use provided directionality parameters
      dat$Ival1 <-
        ifelse(x$data$treat1 == x$data$.treat1,
               x$data[[substitute(direction1)]],
               x$data[[substitute(direction2)]])
      #
      dat$Ival2 <-
        ifelse(x$data$treat2 == x$data$.treat2,
               x$data[[substitute(direction2)]],
               x$data[[substitute(direction1)]])
    }
    else if (!has_treat1_dir & !has_treat2_dir) {
      # Default to treatment order when no directionality specified
      dat <- do.call(rbind, lapply(split(dat, dat$studlab), mkIval))
      rownames(dat) <- NULL
      warning("No directionality parameters specified. ",
              "Defaulting to treatment order.")
    }
    # Validate directionality parameters
    check_Ival(dat)
  }
  #
  dat <- dat[order(dat$studlab, dat$treat1, dat$treat2), , drop = FALSE]
  #
  keep <- logical(0)
  wo <- logical(0)
  #
  if (consistency) {
    # Check for constant covariate in reference treatment
    if (length(unique(dat[dat$treat1 == reference.group |
                          dat$treat2 == reference.group,
                          covar.name])) == 1)
      stop("Invalid reference treatment for interaction. ",
           "Insufficient variation in observed covariate values for ",
           "reference treatment.")
    #
    for (i in unique(dat$studlab)) {
      d.i <- dat[dat$studlab == i, , drop = FALSE]
      trts.i <- unique(sort(c(d.i$treat1, d.i$treat2)))
      if (reference.group %in% trts.i)
        ref.i <- reference.group
      else
        ref.i <- rev(trts.i)[1]
      #
      keep.i <- !(d.i$treat1 != ref.i & d.i$treat2 != ref.i)
      wo.i <- d.i$treat1 == ref.i
      #
      keep <- c(keep, keep.i)
      wo <- c(wo, wo.i)
    }
    #
  }
  else if (!consistency) {
    # calcV() is sensitive to the choice of reference treatment, but if we
    # have study specific references we have to update the keep and wo
    # statements
    #
    dat$Ival_diff <- dat$Ival1 - dat$Ival2
    # Whether a row contains a reference.
    # Equivalent !(d.i$treat1 != ref.i & d.i$treat2 != ref.i)
    keep.i <- dat$Ival_diff != 0
    # Whether the reference treatment is treat1.
    # Equivalent to d.i$treat1 == ref.i
    wo.i <- dat$Ival1 == 0
    #
    keep <- c(keep, keep.i)
    wo <- c(wo, wo.i)
  }
  #
  dat <- dat[keep, , drop = FALSE]
  #
  wo <- wo[keep]
  if (sum(wo) > 0) {
    dat$TE[wo] <- -dat$TE[wo]
    #
    t2.tmp <- dat$treat2
    e2.tmp <- dat$event2
    n2.tmp <- dat$n2
    mean2.tmp <- dat$mean2
    sd2.tmp <- dat$sd2
    time2.tmp <- dat$time2
    incr2.tmp <- dat$incr2
    Ival2.tmp <- dat$Ival2
    #
    dat$treat2[wo] <- dat$treat1[wo]
    dat$event2[wo] <- dat$event1[wo]
    dat$n2[wo] <- dat$n1[wo]
    dat$mean2[wo] <- dat$mean1[wo]
    dat$sd2[wo] <- dat$sd1[wo]
    dat$time2[wo] <- dat$time1[wo]
    dat$incr2[wo] <- dat$incr1[wo]
    dat$Ival2[wo] <- dat$Ival1[wo]
    #
    dat$treat1[wo] <- t2.tmp[wo]
    dat$event1[wo] <- e2.tmp[wo]
    dat$n1[wo] <- n2.tmp[wo]
    dat$mean1[wo] <- mean2.tmp[wo]
    dat$sd1[wo] <- sd2.tmp[wo]
    dat$time1[wo] <- time2.tmp[wo]
    dat$incr1[wo] <- incr2.tmp[wo]
    dat$Ival1[wo] <- Ival2.tmp[wo]
  }
  #
  ncols1 <- ncol(dat)
  dat <- contrmat(dat, grp1 = "treat1", grp2 = "treat2")
  dat <- dat[order(dat$studlab), ]
  trts.all <- names(dat)[(ncols1 + 1):ncol(dat)]
  #
  # Ensure that the reference treatment is correctly assigned
  trts <- trts.all[trts.all != reference.group]
  
  
  #
  #
  # (4) Create model matrix
  #
  #
  
  if (consistency) {
    if (assumption == "independent") {
      formula.nmr_default <- # renamed for construction of manual matrix below
        as.formula(paste("~ 0 + ",
                         paste(trts, collapse = " + "),
                         if (!is.null(covar))
                           paste0( " + ",
                                   paste(paste0(trts, ":", covar.name),
                                         collapse = " + "))))
    }
    else {
      #
      # The interaction is all non-reference treatments vs reference,
      # so we can define a variable indicating whether it is reference
      # or non-reference.
      #
      error <-
        try(dat$nonref <- as.numeric(dat[[make.names(reference.group)]] != 0),
            silent = TRUE)
      #
      # Necessary, if reference treatment contains a white space
      #
      if (inherits(error, "try-error"))
        dat$nonref <- as.numeric(dat[[gsub(" ", "_", reference.group)]] != 0)
      #
      # Then get the interaction using the colon(:). The output has a
      # colon which is in the same format as the independent. Helpful
      # for later extraction
      #
      formula.nmr_default <- # renamed for construction of manual matrix below
        as.formula(paste("~ 0 + ",
                         paste(trts, collapse = " + "),
                         if (!is.null(covar))
                           paste0( " + ",
                                   paste(paste0("nonref:", covar.name),
                                         collapse = " + "))))
    }
  }
  else {
    dat$Ival <- dat$Ival1 - dat$Ival2
    if (assumption == "independent") {
      dat$.comp_ <- paste(pmin(dat$treat1, dat$treat2),
                          pmax(dat$treat1, dat$treat2),
                          sep = "_vs_")
      count_comp <- table(dat$.comp_)
      # for dev use
      if (!max.ia) {
        # Requires at least two observations
        dat$.comp_ <-
          ifelse(dat$.comp_ %in% names(count_comp[count_comp >= 2]),
                 dat$.comp_, "insufficient_data")
      }
      #
      formula.nmr_default <-
        as.formula(paste("~ 0 + ", paste(trts, collapse = " + "),
                         if (!is.null(covar))
                           paste0("+ .comp_:Ival:", covar.name))
        )
    }
    else {
      formula.nmr_default <-
        as.formula(paste("~ 0 + ", paste(trts, collapse = " + "),
                         if (!is.null(covar))
                           paste0("+ Ival:", covar.name))
        )
    }
  }
  #
  # Checks for non-numeric covariate
  #
  mm <- model.matrix(formula.nmr_default, data = dat) # default model matrix
  mm <- mm[, !grepl("insufficient_data", colnames(mm))]
  #
  # Drop interactions which contain the reference covariate level if covariate
  # is of mode factor, character, or logical
  #
  if (is.factor(covar))
    mm <-
    mm[, !grepl(paste0(levels(covar)[1], "$"), colnames(mm)), drop = FALSE]
  else if (is.character(covar))
    mm <- mm[, !grepl(paste0(min(covar, na.rm = TRUE), "$"), colnames(mm)),
             drop = FALSE]
  else if (is.logical(covar))
    mm <- mm[, !grepl(paste0(as.logical(min(covar)), "$"), colnames(mm)),
             drop = FALSE]
  #
  dat[colnames(mm)] <- mm # update model matrix columns
  #
  # The back ticks are necessary if the covariate levels / values contain
  # spaces or special characters
  #
  formula.nmr <-
    as.formula(paste("~ 0 ",
                     paste0(" + ",
                            paste0("`", colnames(mm), "`", collapse = " + "))))
  
  
  #
  #
  # (5) Run network meta-regression
  #
  #
  
  # Get rid of warning 'Undefined global functions or variables'
  #
  treat1 <- treat2 <- comparison <- NULL
  #
  # Covariate 'x' makes problems without removing network meta-analysis object x
  #
  ..x <- x
  rm(x)
  #
  dat.TE <- dat$TE
  dat$comparison <- paste(dat$treat1, dat$treat2, sep = " vs ")
  #
  # Calculate Variance-Covariance matrix
  #
  if (available.n & (available.events | available.times | (available.sds)))
    V <- bldiag(lapply(split(dat, dat$studlab), calcV, sm = sm))
  else
    V <- dat$seTE^2
  #
  suppressWarnings(
    res <-
      runNN(rma.mv,
            list(yi = dat.TE, V = V,
                 data = dat,
                 mods = formula.nmr,
                 random = as.call(~ factor(comparison) | studlab),
                 rho = 0.5,
                 method = method.tau,
                 level = 100 * level,
                 ...)))
  #
  X <- res$X.f
  rownames(X) <- dat$studlab
  X <- X[rev(do.call(order, as.data.frame(X))), , drop = FALSE]
  #
  res$.netmeta <- list(x = ..x,
                       covar.name = covar.name,
                       covar = covar,
                       X = X,
                       consistency = consistency,
                       assumption = assumption,
                       method.tau = method.tau,
                       level = level,
                       reference.group = reference.group,
                       trts = ..x$trts,  # keep the long name of the treatment
                       trts.abbr = trts.abbr,
                       nchar.trts = nchar.trts,
                       dots = list(...),
                       call = match.call(),
                       version = packageDescription("netmeta")$Version,
                       version.metafor = packageDescription("metafor")$Version)
  #
  res$results <- nmr_results(res)
  res$full_results <- nmr_full_results(res)
  #
  class(res) <- c("netmetareg", class(res))
  res$call <- NULL
  #
  res
}


#' @rdname netmetareg
#' @export netmetareg

netmetareg <- function(x, ...)
  UseMethod("netmetareg")


#' @rdname netmetareg
#' @method netmetareg default
#' @export

netmetareg.default <- function(x, ...)
  stop("Network meta-regression not available for an object of class '",
       class(x)[1], "'.",
       call. = FALSE)


#' @rdname netmetareg
#' @method print netmetareg
#' @export

print.netmetareg <- function(x,
                             digits = gs("digits"),
                             digits.se = gs("digits.se"),
                             digits.stat = gs("digits.stat"),
                             digits.pval = gs("digits.pval"),
                             print.se = FALSE,
                             details.methods = TRUE, ...) {
  
  chkclass(x, "netmetareg")
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 0, length = 1)
  chklogical(print.se)
  chklogical(details.methods)
  #
  covar <- x$.netmeta$covar
  #
  if (is.null(covar))
    cat("Network Meta-Analysis\n\n")
  else
    cat("Network Meta-Regression\n\n")
  #
  x.orig <- x
  #
  consistency <- x$.netmeta$consistency
  assumption <- x$.netmeta$assumption
  #
  lower <- NULL
  #
  if (is.null(covar) || is.numeric(covar))
    dat <- x$results[ , c("coef", if (print.se) "se", "lower", "upper",
                          "z", "pval")]
  else
    dat <- x$results[ , c("cov_lvl", "cov_ref",
                          "coef", if (print.se) "se", "lower", "upper",
                          "z", "pval")]
  #
  dat$coef <- formatN(dat$coef, digits = digits)
  #
  if (print.se)
    dat$se <- formatN(dat$se, digits = digits.se, ...)
  #
  dat$lower <- formatCI(formatN(dat$lower, digits = digits),
                        formatN(dat$upper, digits = digits))
  dat$upper <- NULL
  #
  dat <- replaceNA(dat, ".")
  #
  names(dat)[names(dat) == "lower"] <-
    paste0(round(100 * x$.netmeta$level, 1), "% CI")
  #
  dat$z <- formatN(dat$z, digits = digits.stat)
  dat$pval <- formatPT(dat$pval, digits = digits.pval)
  names(dat)[names(dat) == "pval"] <- "p-value"
  #
  prmatrix(dat, quote = FALSE, right = TRUE, ...)
  #
  # Get rid of warning 'Undefined global functions or variables'
  comparison <- NULL
  #
  if (!is.null(covar) & details.methods) {
    cat("\nDetails on network meta-regression methods:\n")
    #
    if (x$.netmeta$method.tau == "FE")
      cat("- Common effects model\n")
    else
      cat("- Random effects model\n")
    #
    if (consistency)
      cat("- Consistency model\n")
    else
      cat("- Unrelated mean interaction effect (UMIE) model\n")
    #
    if (assumption == "independent")
      cat("- Independent slopes\n")
    else
      cat("- Common slope\n")
    #
    cat(paste0("- Reference group: ", x$.netmeta$reference.group, "\n"))
  }
  else if (details.methods) {
    cat("\nDetails on network meta-analysis methods:\n")
    #
    if (x$.netmeta$method.tau == "FE")
      cat("- Common effects model\n")
    else
      cat("- Random effects model\n")
  }
  #
  invisible(NULL)
}

#' Generic function for network graphs
#' 
#' @description
#' Generic function for network graphs
#' 
#' @param x An R object.
#' @param \dots Additional arguments.
#' 
#' @details
#' 
#' For more details, look at the following functions to generate
#' network graphs:
#' \itemize{
#' \item \code{\link{netgraph.netmeta}}
#' \item \code{\link{netgraph.netimpact}}
#' \item \code{\link{netgraph.netconnection}}
#' \item \code{\link{netgraph.netcomb}}
#' \item \code{\link{netgraph.discomb}}
#' }
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de
#' }
#' 
#' @keywords hplot
#'
#' @examples
#' # Examples: example(netgraph.netmeta)
#' 
#' @rdname netgraph
#' @export netgraph

netgraph <- function(x, ...)
  UseMethod("netgraph")

#' Summary method for objects of class rankogram
#' 
#' @description
#' Summary method for objects of class \code{rankogram} to print
#' list of studies in subnetworks.
#' 
#' @param object An object of class \code{rankogram}.
#' @param x An object of class \code{summary.rankogram}.
#' @param common A logical indicating to print ranking probabilities
#'   and SUCRAs for the common effects model.
#' @param random A logical indicating to print ranking probabilities
#'   and SUCRAs for the random effects model.
#' @param sort A logical indicating whether treatments should be
#'   sorted by decreasing SUCRAs.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param digits Minimal number of significant digits for ranking probabilities
#'   and SUCRAs, see \code{\link{print.default}}.
#' @param digits.mean Minimal number of significant digits for mean ranks, see
#'   \code{\link{print.default}}.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param \dots Additional arguments (passed on to
#'   \code{\link{print.rankogram}}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' @seealso \code{\link{rankogram}}
#' 
#' @examples
#' # Examples:
#' # example(rankogram.netmeta)
#'
#' @method summary rankogram 
#' @export

summary.rankogram <- function(object, ...) {
  
  chkclass(object, "rankogram")
  
  res <- object
  #
  class(res) <- c("summary.rankogram", class(res))
  #
  res
}


#' @rdname summary.rankogram
#' @method print summary.rankogram 
#' @export

print.summary.rankogram <- function(x,
                                    common = x$common,
                                    random = x$random,
                                    sort = TRUE,
                                    nchar.trts = x$nchar.trts,
                                    digits = gs("digits.prop"),
                                    digits.mean = 2,
                                    details.methods = gs("details"),
                                    legend = gs("legend"),
                                    ...) {
  
  chkclass(x, "summary.rankogram")
  #
  chklogical(common)
  chklogical(random)
  #
  if (is.character(sort)) {
    sort <- setchar(sort, c("common", "random", "fixed"))
    sort[sort == "fixed"] <- "common"
  }
  else
    chklogical(sort)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.mean, min = 0, length = 1)
  #
  both <- (common + random) == 2
  #
  if (!both & is.character(sort)) {
    if (common & sort == "random") {
      warning("Argument 'sort=\"random\"' ignored for common effects model.",
              call. = FALSE)
      sort <- TRUE
    }
    if (random & sort == "common") {
      warning("Argument 'sort=\"common\"' ignored for random effects model.",
              call. = FALSE)
      sort <- TRUE
    }
  }
  else if (both & !is.character(sort) && sort)
    sort <- "random"
  #
  sort.logical <- if (is.character(sort)) TRUE else sort
  #
  chknumeric(nchar.trts, length = 1)
  chklogical(details.methods)
  chklogical(legend)
  #
  class(x) <- "rankogram"
  #
  print(x,
        common = common,
        random = random,
        sort = sort.logical,
        nchar.trts = nchar.trts,
        digits = digits,
        details.methods = FALSE, legend = FALSE,
        ...)
  #
  if (common | random) {
    cat("\nSurface Under the Cumulative RAnking curve\n\n")
    #
    dat.c <- dat.r <- NULL
    #
    if (common) {
      dat.c <- data.frame(mean = formatN(x$ranking.common, digits))
      names(dat.c) <- paste0("SUCRA", if (random) " (common)")
    }
    #
    if (random) {
      dat.r <- data.frame(mean = formatN(x$ranking.random, digits))
      names(dat.r) <- paste0("SUCRA", if (common) " (random)")
    }
    #
    if (both) {
      dat <- merge(dat.c, dat.r, by = "row.names")
      rownames(dat) <- dat$Row.names
      dat$Row.names <- NULL
      #
      if (sort == "common")
        dat <- dat[rev(order(x$ranking.common)), , drop = FALSE]
      else if (sort == "random")
        dat <- dat[rev(order(x$ranking.random)), , drop = FALSE]
      else
        dat <- dat[x$x$seq, , drop = FALSE]
    }
    else if (common) {
      if (sort.logical)
        dat <- dat.c[rev(order(x$ranking.common)), , drop = FALSE]
      else
        dat <- dat.c[x$x$seq, , drop = FALSE]
    }
    else if (random) {
      if (sort.logical)
        dat <- dat.r[rev(order(x$ranking.random)), , drop = FALSE]
      else
        dat <- dat.r[x$x$seq, , drop = FALSE]
    }
    #
    rownames(dat) <- treats(rownames(dat), nchar.trts)
    #
    prmatrix(dat, quote = FALSE, right = TRUE, ...)
  }
  #
  #
  if (common | random) {
    cat("\nMean ranks\n\n")
    #
    dat.c <- dat.r <- NULL
    #
    if (common) {
      dat.c <- data.frame(mean = formatN(x$meanranks.common, digits.mean))
      names(dat.c) <- paste0("Mean", if (random) " (common)")
    }
    #
    if (random) {
      dat.r <- data.frame(mean = formatN(x$meanranks.random, digits.mean))
      names(dat.r) <- paste0("Mean", if (common) " (random)")
    }
    #
    if (both) {
      dat <- merge(dat.c, dat.r, by = "row.names")
      rownames(dat) <- dat$Row.names
      dat$Row.names <- NULL
      #
      if (sort == "common")
        dat <- dat[rev(order(x$ranking.common)), , drop = FALSE]
      else if (sort == "random")
        dat <- dat[rev(order(x$ranking.random)), , drop = FALSE]
      else
        dat <- dat[x$x$seq, , drop = FALSE]
    }
    else if (common) {
      if (sort.logical)
        dat <- dat.c[rev(order(x$ranking.common)), , drop = FALSE]
      else
        dat <- dat.c[x$x$seq, , drop = FALSE]
    }
    else if (random) {
      if (sort.logical)
        dat <- dat.r[rev(order(x$ranking.random)), , drop = FALSE]
      else
        dat <- dat.r[x$x$seq, , drop = FALSE]
    }
    #
    rownames(dat) <- treats(rownames(dat), nchar.trts)
    #
    prmatrix(dat, quote = FALSE, right = TRUE, ...)
  }
  #
  # Print details of network meta-analysis methods
  #
  if (details.methods) {
    text.details <-
      textmeth(x, random, TRUE)
    #
    cat(text.details)
  }
  #
  # Add legend with abbreviated treatment labels
  #
  if ((common | random) & legend) {
    if (random)
      trts <- rownames(x$ranking.matrix.random)
    else
      trts <- rownames(x$ranking.matrix.common)
    #
    legendabbr(trts, treats(trts, nchar.trts), TRUE)
  }
  #
  invisible(NULL)
}

#' Net heat plot
#' 
#' @description
#' This function creates a net heat plot, a graphical tool for
#' locating inconsistency in network meta-analyses.
#' 
#' @param x An object of class \code{netmeta}.
#' @param random A logical indicating whether the net heat plot should
#'   be based on a random effects model.
#' @param tau.preset An optional value for the square-root of the
#'   between-study variance \eqn{\tau^2} for a random effects model on
#'   which the net heat plot will be based.
#' @param showall A logical indicating whether results should be shown
#'   for all designs or only a sensible subset (see Details).
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param \dots Additional arguments.
#' 
#' @details
#' The net heat plot is a matrix visualization proposed by Krahn et
#' al. (2013) that highlights hot spots of inconsistency between
#' specific direct evidence in the whole network and renders
#' transparent possible drivers.
#' 
#' In this plot, the area of a gray square displays the contribution
#' of the direct estimate of one design in the column to a network
#' estimate in a row. In combination, the colors show the detailed
#' change in inconsistency when relaxing the assumption of consistency
#' for the effects of single designs. The colors on the diagonal
#' represent the inconsistency contribution of the corresponding
#' design. The colors on the off-diagonal are associated with the
#' change in inconsistency between direct and indirect evidence in a
#' network estimate in the row after relaxing the consistency
#' assumption for the effect of one design in the column. Cool colors
#' indicate an increase and warm colors a decrease: the stronger the
#' intensity of the color, the greater the difference between the
#' inconsistency before and after the detachment. So, a blue colored
#' element indicates that the evidence of the design in the column
#' supports the evidence in the row. A clustering procedure is applied
#' to the heat matrix in order to find warm colored hot spots of
#' inconsistency. In the case that the colors of a column
#' corresponding to design \eqn{d} are identical to the colors on the
#' diagonal, the detaching of the effect of design \eqn{d} dissolves
#' the total inconsistency in the network.
#' 
#' The pairwise contrasts corresponding to designs of three- or
#' multi-arm studies are marked by '_' following the treatments of the
#' design.
#' 
#' Designs where only one treatment is involved in other designs of
#' the network or where the removal of corresponding studies would
#' lead to a splitting of the network do not contribute to the
#' inconsistency assessment. By default (\code{showall = TRUE}), these
#' designs are not incorporated into the net heat plot. If
#' \code{showall = FALSE}, additional designs with minimal
#' contribution to the inconsistency Q statistic are not incorporated
#' (i.e., designs with \code{abs(Q.inc.design)} \code{<=}
#' \code{.Machine$double.eps^0.5)}.).
#' 
#' In the case of \code{random = TRUE}, the net heat plot is based on
#' a random effects model generalised for multivariate meta-analysis
#' in which the between-study variance \eqn{\tau^2} is estimated by the
#' method of moments (see Jackson et al., 2012) and embedded in a full
#' design-by-treatment interaction model (see Higgins et al., 2012).
#'
#' @author Ulrike Krahn \email{ulrike.krahn@@bayer.com}
#'
#' @seealso \link{netmeta}
#' 
#' @references
#' Krahn U, Binder H, König J (2013):
#' A graphical tool for locating inconsistency in network meta-analyses.
#' \emph{BMC Medical Research Methodology},
#' \bold{13}, 35
#' 
#' Jackson D, White IR and Riley RD (2012):
#' Quantifying the impact of between-study heterogeneity in
#' multivariate meta-analyses.
#' \emph{Statistics in Medicine},
#' \bold{31}, 3805--20
#' 
#' Higgins JPT, Jackson D, Barrett JK, Lu G, Ades AE, White IR (2012):
#' Consistency and inconsistency in network meta-analysis: concepts
#' and models for multi-arm studies.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 98--110
#' 
#' @keywords hplot
#' 
#' @examples
#' # Only consider first five studies (to reduce runtime of example)
#' #
#' studies <- unique(Senn2013$study)
#' Senn2013.5 <- subset(Senn2013, study %in% studies[1:5])
#' 
#' # Transform data from long arm-based to contrast-based format
#' #
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013.5,
#'   varnames = c("MD", "seMD"))
#' 
#' # Conduct common effects network meta-analysis with
#' # placebo as reference treatment
#' #
#' nma1 <- netmeta(pw, random = FALSE, reference = "plac")
#' 
#' # Generate a net heat plot based on the common effects model
#' #
#' netheat(nma1) 
#' 
#' \donttest{
#' # Conduct common and random effects network meta-analysis with placebo as
#' # reference treatment
#' #
#' nma2 <- netmeta(pw, reference = "plac")
#'         
#' # Generate a net heat plot based on the random effects model
#' #
#' netheat(nma2) 
#' }
#'
#' @export netheat

netheat <- function(x, random = FALSE, tau.preset = NULL,
                    showall = TRUE,
                    nchar.trts = x$nchar.trts,
                    ...) {
  
  
  chkclass(x, "netmeta")
  chksuitable(x, "Net heat plot", classes = gs(".other_nma"))
  x <- updateversion(x)
  ##
  if (x$d <= 2) {
    warning("Net heat plot not available due to small number of designs: ",
            x$d,
            call. = FALSE)
    return(invisible(NULL))
  }
  ##
  if (inherits(x, "netmetabin")) {
    warning("Net heat plot not available for objects created ",
            "with netmetabin().",
            call. = FALSE)
    return(invisible(NULL))
  }
  ##
  chklogical(random)
  missing.showall <- missing(showall)
  chklogical(showall)
  chknumeric(nchar.trts, min = 1, length = 1)
  
  
  if (is.null(x$nchar.trts))
    nchar.trts <- 666
  ##
  trts <- colnames(x$A.matrix)
  trts.abbr <- treats(trts, nchar.trts)
  ##
  x$treat1 <- as.character(factor(x$treat1,
                                  levels = trts,
                                  labels = trts.abbr))
  ##
  x$treat2 <- as.character(factor(x$treat2,
                                  levels = trts,
                                  labels = trts.abbr))
  ##
  colnames(x$A.matrix) <- trts.abbr
  ##
  if (x$reference.group != "")
    x$reference.group <- trts.abbr[trts == x$reference.group]
  
  
  if (random == FALSE & length(tau.preset) == 0) {
    nmak <- nma_krahn(x)
    decomp <- decomp.design(x)
    residuals <- decomp$residuals.inc.detach
    Q.inc.design <- decomp$Q.inc.design
  }
  ##
  if (length(tau.preset) == 1) {
    nmak <- nma_krahn(x, tau.preset = tau.preset)
    decomp <- decomp.design(x, tau.preset = tau.preset)
    residuals <- decomp$residuals.inc.detach.random.preset
    Q.inc.design <- decomp$Q.inc.design.random.preset
  }
  ##
  if (random == TRUE & length(tau.preset) == 0) {
    tau.within <- tau.within(x)
    nmak <- nma_krahn(x, tau.preset = tau.within)
    decomp <- decomp.design(x, tau.preset = tau.within)
    residuals <- decomp$residuals.inc.detach.random.preset
    Q.inc.design <- decomp$Q.inc.design.random.preset
  }
  ##
  df.Q.between.designs <- decomp$Q.decomp["Between designs", "df"]
  ##
  if (df.Q.between.designs == 0) {
    warning("Net heat plot not available because the network ",
            "does not contain any loop",
            if (any(x$narms > 2)) " along more than one design." else ".",
            call. = FALSE)
    return(invisible(NULL))
  }
  
  
  if (!showall) {
    sel.Q.inc.design <-
      unlist(lapply(split(Q.inc.design, names(Q.inc.design)),
                    function(x)
                      sum(abs(x)) < .Machine$double.eps^0.5))
    drop.designs <- names(sel.Q.inc.design)[sel.Q.inc.design]
  }
  
  
  H <- nmak$H
  V <- nmak$V
  ##
  if (!showall)
    design <- nmak$design[!(nmak$design$design %in% drop.designs), ]
  else
    design <- nmak$design
  
  
  Q.inc.design.typ <- apply(residuals, 2, function(x) t(x) %*% solve(V) * x)
  inc <- matrix(Q.inc.design,
                nrow = nrow(Q.inc.design.typ),
                ncol = ncol(Q.inc.design.typ))
  diff <- inc - Q.inc.design.typ
  colnames(diff) <- colnames(Q.inc.design.typ)
  rownames(diff) <- rownames(residuals)
  ##
  if (!showall)
    diff <- diff[!(rownames(diff) %in% drop.designs),
                 !(colnames(diff) %in% drop.designs), drop = FALSE]
  ##
  if (all(is.na(diff))) {
    warning("Net heat plot not available as ",
            "no between-design heterogeneity exists.",
            call. = FALSE)
    return(invisible(NULL))
  }
  ##
  if (length(diff) == 1) {
    warning("Net heat plot not available due to small number of ",
            "informative designs.",
            call. = FALSE)
    return(invisible(NULL))
  }
  
  
  t1 <- -diff
  wi <- which(apply(t1, 2, function(x) sum(is.na(x)) == nrow(t1)))
  if (length(wi) > 0) {
    t1 <- t1[-wi, -wi, drop = FALSE]
  }
  dmat <- t1
  d1 <- dist(dmat, method = "manhattan")
  d1 <- d1 + dist(t(dmat), method = "manhattan")
  ##
  if (length(d1) > 0)
    h1 <- hclust(d1)
  else {
    warning("Insufficient number of designs ",
            "(available or selected by the program  specification) ",
            "for a net heat plot.",
            call. = FALSE)
    return(invisible(NULL))
  }
  ##
  t1 <- t1[h1$order, h1$order]
  
  
  tn <- matrix(NA, nrow = nrow(t1), ncol = nrow(t1))
  
  
  for (i in 1:(nrow(t1)))
    tn[i, ] <- t1[nrow(t1) - (i - 1), ]
  
  
  tn[tn < -8] <- -8
  tn[tn >  8] <-  8
  
  
  ncolo <- 40
  sat <- min(max(abs(tn)) / 8, 1)
  nblue <- ifelse((max(max(tn), 0) - min(min(tn), 0)) != 0,
                  round(max(max(tn), 0) /
                        (max(max(tn), 0) - min(min(tn), 0)) * ncolo),
                  0)
  bf <- min(max(max(tn), 0) / abs(min(min(tn), 0)), 1) * sat
  clev <- seq(from = 1, to = 1 - bf, len = nblue)
  heat.vec <- heat.colors(round((ncolo - nblue - 1) / max(sat, 0.00001)))
  mycol <- c(heat.vec[(length(heat.vec) -
                       (ncolo - nblue - 1) + 1):length(heat.vec)],
             rgb(1, 1, 1), rgb(clev, clev, 1))
  
  
  if (length(wi) > 0)
    Hp <- H[(as.character(design$comparison)[-wi]), -wi]
  else
    Hp <- H[as.character(design$comparison), ]
  ##
  if (!showall)
    Hp <- Hp[!(rownames(Hp) %in% drop.designs),
             !(colnames(Hp) %in% drop.designs), drop = FALSE]
  ##
  Hp <- Hp[h1$order, h1$order]
  
  
  Hpn <- matrix(NA, nrow = nrow(Hp), ncol = ncol(Hp))
  ##
  for (i in 1:(nrow(Hp)))
    Hpn[i, ] <- Hp[ncol(Hp) - (i - 1), ]
  ##
  Hpn <- t(Hpn)
  
  
  oldpar <- va.image(t(tn) + max(abs(tn)),
                     design, Hpn, h1, wi,
                     col = mycol)
  ##  
  on.exit(par(oldpar))
  
  
  if (min(round(t1, 10)) != max(round(t1, 10)))
    legend.col(rev(mycol),
               seq(from = max(-max(t1), -8), to = min(-min(t1), 8),
                   len = length(mycol)),
               oldpar)
  ##
  box(lwd = 1.1)
  
  
  invisible(NULL)
}

#' Forest plot showing results of network meta-analysis with subgroups
#' 
#' @description
#' Forest plot to show subgroup estimates of network meta-analysis.
#'
#' @aliases forest.subgroup.netmeta plot.subgroup.netmeta
#' 
#' @param x An object of class \code{subgroup.netmeta}.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param equal.size A logical indicating whether all squares should
#'   be of equal size. Otherwise, the square size is proportional to
#'   the precision of estimates.
#' @param leftcols A character vector specifying columns to be plotted
#'   on the left side of the forest plot (see Details).
#' @param leftlabs A character vector specifying labels for columns on
#'   left side of the forest plot.
#' @param rightcols A character vector specifying columns to be
#'   plotted on the right side of the forest plot (see Details).
#' @param rightlabs A character vector specifying labels for columns
#'   on right side of the forest plot.
#' @param calcwidth.subgroup A logical indicating whether text with
#'   comparison labels should be considered to calculate width of the
#'   column with treatment labels.
#' @param digits Minimal number of significant digits for treatment
#'   effects and confidence intervals, see \code{print.default}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistic Q, see \code{print.default}.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity test, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance \eqn{\tau^2}, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for
#'   \eqn{\tau}, the square root of the between-study variance
#'   \eqn{\tau^2}.
#' @param sep.trts A character string used to label treatment comparisons.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param lab.NA A character string to label missing values.
#' @param smlab A label printed at top of figure. By default, text
#'   indicating either common or random effects model is printed.
#' @param col.subgroup The colour to print information on subgroups.
#' @param \dots Additional arguments for \code{\link[meta]{forest.meta}}
#'   function.
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window.
#' 
#' The arguments \code{leftcols} and \code{rightcols} can be used to
#' specify columns which are plotted on the left and right side of the
#' forest plot, respectively. If argument \code{rightcols} is
#' \code{FALSE}, no columns will be plotted on the right side.
#' 
#' For more information see help page of \code{\link[meta]{forest.meta}}
#' function.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{subgroup.netmeta}}, \code{\link{netmeta}},
#'   \code{\link[meta]{forest.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' \dontrun{
#' data("Senn2013")
#' # Add variable with (fictitious) risk of bias values
#' Senn2013$rob <- NA
#' set.seed(1909)
#' for (i in unique(Senn2013$studlab))
#'   Senn2013$rob[Senn2013$studlab == i] <- sample(1:3, 1)
#' Senn2013$rob <- factor(Senn2013$rob, levels = 1:3,
#'   labels = c("low", "moderate", "high"))
#' # Conduct network meta-analysis
#' net <- netmeta(TE, seTE, treat1.long, treat2.long, studlab,
#'   data = Senn2013, sm = "MD", reference = "plac", nchar.trts = 4)
#' # Conduct subgroup network meta-analysis
#' sg <- subgroup(net, rob, common = FALSE)
#' sg
#' # Forest plot
#' forest(sg)
#' }
#' 
#' @method forest subgroup.netmeta
#' @export

forest.subgroup.netmeta <- function(x,
                                    pooled =
                                      ifelse(x$x$random, "random", "common"),
                                    #
                                    equal.size = gs("equal.size"),
                                    #
                                    leftcols =
                                      c("studlab",
                                        "Q", "df.Q", "pval.Q"),
                                    leftlabs =
                                      c("Comparison /\nSubgroup",
                                        "Q", "d.f.", "p-value"),
                                    rightcols =
                                      c("effect", "ci", "k",
                                        if (pooled == "random") "tau"),
                                    rightlabs =
                                      c(NA, NA, "Number of\nStudies",
                                        if (pooled == "random") "Tau"),
                                    #
                                    calcwidth.subgroup =
                                      gs("calcwidth.subgroup"),
                                    #
                                    digits = gs("digits.forest"),
                                    digits.Q = gs("digits.Q"),
                                    digits.pval.Q = gs("digits.pval.Q"),
                                    digits.tau2 = gs("digits.tau2"),
                                    digits.tau = gs("digits.tau"),
                                    #
                                    sep.trts = " vs ",
                                    backtransf = x$x$backtransf,
                                    lab.NA = ".",
                                    smlab,
                                    #
                                    col.subgroup = "black",
                                    ...) {
  
  
  #
  #
  # (1) Check and set arguments
  #
  #
  
  chkclass(x, "subgroup.netmeta")
  #
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  #
  chklogical(equal.size)
  chklogical(calcwidth.subgroup)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.pval.Q, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  #
  chkchar(sep.trts, length = 1)
  chklogical(backtransf)
  chkchar(lab.NA, length = 1)
  
  
  #
  #
  # (2) Extract results for common and random effects model
  #
  #
  
  if (pooled == "common") {
    common <- x$common %>% filter(is.na(df.Q) | df.Q > 0)
    #
    common$Q <- formatN(common$Q, digits = digits.Q, text.NA = "")
    common$df.Q <- formatN(common$df.Q, digits = 0, text.NA = "")
    common$pval.Q <-
      formatPT(common$pval.Q, digits = digits.pval.Q, lab.NA = "")
    #
    m <-
      suppressWarnings(metagen(TE, seTE, studlab = subgroup, data = common,
                               subgroup = paste(treat1, treat2, sep = sep.trts),
                               print.subgroup.name = FALSE,
                               sm = x$x$sm,
                               overall = FALSE, overall.hetstat = FALSE,
                               common = FALSE, random = FALSE))
    #
    text.pooled <- "Common Effects Model"
  }
  else {
    random <- x$random %>% filter(is.na(df.Q) | df.Q > 0)
    #
    random$Q <- formatN(random$Q, digits = digits.Q, text.NA = "")
    random$df.Q <- formatN(random$df.Q, digits = 0, text.NA = "")
    random$pval.Q <-
      formatPT(random$pval.Q, digits = digits.pval.Q, lab.NA = "")
    random$tau2 <- formatPT(random$tau2, digits = digits.tau2, lab.NA = lab.NA)
    random$tau <- formatPT(random$tau, digits = digits.tau, lab.NA = lab.NA)
    #
    m <-
      suppressWarnings(metagen(TE, seTE, studlab = subgroup, data = random,
                               subgroup = paste(treat1, treat2, sep = " vs "),
                               print.subgroup.name = FALSE,
                               sm = x$x$sm,
                               overall = FALSE, overall.hetstat = FALSE,
                               common = FALSE, random = FALSE))
    #
    text.pooled <- "Random Effects Model"
  }
  #
  if (missing(smlab))
      smlab <- text.pooled

  
  #
  #
  # (3) Forest plot
  #
  #
  
  # Get rid of warning 'Undefined global functions or variables'
  .seTE <- .studlab <- .TE <- .treat1 <- .treat2 <- comparison <-
    df.Q <- subnet <- TE <- seTE <- treat1 <- treat2 <- NULL
  
  forest(m,
         digits = digits,
         #
         overall = FALSE, common = FALSE, random = FALSE,
         hetstat = FALSE, test.subgroup = FALSE,
         #
         subgroup.hetstat = FALSE,
         prediction.subgroup = FALSE,
         calcwidth.subgroup = calcwidth.subgroup,
         #
         leftcols = leftcols,
         leftlabs = leftlabs,
         rightcols = rightcols,
         rightlabs = rightlabs,
         #
         lab.NA = lab.NA,
         smlab = smlab,
         backtransf = backtransf,
         #
         col.subgroup = col.subgroup,
         #
         weight.study = if (equal.size) "same" else pooled,
         ...)
  
  ret <- m
  #
  ret$leftcols <- leftcols
  ret$rightcols <- rightcols
  ret$leftlabs <- leftlabs
  ret$rightlabs <- rightlabs
  #
  invisible(ret)
}


#' @rdname forest.subgroup.netmeta
#' @method plot subgroup.netmeta
#' @export

plot.subgroup.netmeta <- function(x, ...)
  forest(x, ...)

#' Network graph for objects of class netconnection
#' 
#' @description
#' This function generates a graph of the evidence network.
#' 
#' @param x An object of class \code{netconnection}.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments arrangement (anticlockwise if \code{start.layout =
#'   "circle"}).
#' @param col A single color (or vector of colors) for lines
#'   connecting treatments if argument \code{plastic = FALSE}
#'   (see Details).
#' @param reference.group Reference treatment (only relevant for
#'   disconnected networks).
#' @param plastic A logical indicating whether the appearance of the
#'   comparisons should be in '3D look'.
#' @param start.layout A character string indicating which starting
#'   layout is used (see \code{\link{netgraph.netmeta}}).
#' @param iterate A logical indicating whether the stress majorization
#'   algorithm is carried out for optimization of the layout
#'   (see \code{\link{netgraph.netmeta}}).
#' @param \dots Additional arguments passed on to
#'   \code{\link{netgraph.netmeta}} (see Details).
#' 
#' @details
#' Argument \code{col} can be a single color for all lines / edges, a vector
#' of length equal to the number of edges, or a vector of length equal
#' to the number of subnetworks. Argument \code{reference.group} is
#' only considered in disconnected networks, i.e., if more than one
#' (sub)network exists, and if argument \code{col} provides colors for
#' subnetworks. In this case, the first color provided in argument
#' \code{col} defines the color for the subnetwork with the reference
#' treatment.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netconnection}}, \code{\link{netgraph.netmeta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Artificial example with two subnetworks
#' #
#' t1 <- c("G", "B", "B", "D", "A", "F")
#' t2 <- c("B", "C", "E", "E", "H", "A")
#' #
#' nc1 <- netconnection(t1, t2)
#' print(nc1, details = TRUE)
#' 
#' netgraph(nc1, points = TRUE, adj = 0.5, bg.points = "lightgray")
#' netgraph(nc1, points = TRUE, adj = 0.5, bg.points = "lightgray",
#'   plastic = TRUE)
#' 
#' @method netgraph netconnection
#' @export

netgraph.netconnection <- function(x, seq,
                                   col = seq_len(x$n.subnets),
                                   reference.group = NULL,
                                   plastic = gs("plastic"),
                                   start.layout = "circle", iterate = FALSE,
                                   ...) {
  
  
  chkclass(x, "netconnection")
  
  if (missing(seq))
    seq <- rownames(x$A.matrix)
  else {
    if (length(seq) == 1)
      seq <- setchar(seq, "optimal", "should be equal to ",
                     "'optimal' or a permutation of treatments")
    else
      seq <- setseq(seq, rownames(x$A.matrix))
  }
  #
  x$d <- 2
  x$trts <- rownames(x$A.matrix)
  ##
  class(x) <- "netmeta"
  
  
  ## Check length of argument 'col'
  ##
  col.subnets <- FALSE
  ##
  if (length(col) == 1) {
    col.subnets <- TRUE
    col <- rep_len(col, x$n.subnets)
  }
  else if (length(col) == x$n.subnets)
    col.subnets <- TRUE
  else
    chklength(col, length(x$subnet.comparisons),
              text = paste0("Length of argument 'col' must be equal to ",
                            "number of edges (",
                            length(x$subnet.comparisons), ")",
                            if (x$n.subnets > 1)
                              paste0(" or subnetworks (",
                                     x$n.subnets, ").")
                            else "."))
  
  
  ## Reorder subnetworks if reference treatment is not part of first
  ## subnetwork
  ##
  if (!is.null(reference.group)) {
    if (!col.subnets)
      warning("Argument 'reference.group' not considered as",
              "a color is defined for each edge.",
              call. = FALSE)
    else {
      reference.group <- setref(reference.group, x$trts)
      ##
      sel.ref <-
        x$treat1 == reference.group | x$treat2 == reference.group
      sel.ref <- unique(x$subnet[sel.ref])
      ##
      if (sel.ref != 1)
        x$subnet.comparisons <-
          as.numeric(relevel(as.factor(x$subnet.comparisons),
                             ref = sel.ref))
    }
  }
  
  
  ## Set color for edges in (sub)networks
  ##
  if (col.subnets) {
    col <-
      as.character(factor(x$subnet.comparisons,
                          levels = seq_len(x$n.subnet),
                          labels = col))
  }
  
  
  res <- netgraph(x, plastic = plastic, col = col,
                  seq = seq, start.layout = start.layout, iterate = iterate,
                  ...)

  
  invisible(res)
}

createC_full <- function(n, k) {
  if (k > n)
    stop("Error: k may not exceed n")
  else if (k == 0)
    res <- as.matrix(t(rep(0, n)))
  else if (k == n)
    res <- as.matrix(t(rep(1, n)))
  else
    res <-
      rbind(cbind(rep(1, choose(n - 1, k - 1)), createC_full(n - 1, k - 1)),
            cbind(rep(0, choose(n - 1, k)), createC_full(n - 1, k)))
  #
  res
}


createC_trts_inactive <- function(trts, inactive = NULL, sep.comps) {
  trts.all <- trts
  #
  # Set of all components
  #
  components <- unique(sort(unlist(compsplit(trts, sep.comps))))
  #
  # Inactive treatment must be empty or one of the treatments
  #
  avail.inactive <- !is.null(inactive)
  #
  if (avail.inactive) {
    inactive <- setchar(inactive, components)
    #
    trts <- trts[trts != inactive]
    #
    if (length(trts) == 0)
      stop("All treatments equal to reference treatment ",
           "(argument 'inactive')",
           call. = FALSE)
    #
    # Remove inactive treatment from list of components
    #
    components <- components[components != inactive]
  }
  #
  # Create list with all treatment components
  #
  components.list <- compsplit(trts, sep.comps)
  #
  # Remove blanks (at start and end)
  #
  components.list <- lapply(components.list, rmSpace)
  components.list <- lapply(components.list, rmSpace, end = TRUE)
  #
  # Create C matrix (without interaction terms)
  #
  C.matrix <- matrix(NA, nrow = length(trts), ncol = length(components))
  #
  # Each list element
  # - corresponds to a treatment combination
  # - consists of a vector with treatment components
  #
  equal <- function(x, pattern) x == pattern
  #
  for (i in seq(along = components)) {
    # Logical grep whether components[i] is part of a treatment combination
    C.matrix[, i] <- unlist(lapply(lapply(components.list, equal,
                                          pattern = components[i]),
                                   sum)) > 0
  }
  #
  # Add row for reference group (and convert to matrix with numeric values)
  #
  if (avail.inactive && inactive %in% trts.all)
    C.matrix <- rbind(C.matrix, rep(0, length(components)))
  else
    C.matrix <- 1L * C.matrix
  #
  colnames(C.matrix) <- components
  #
  if (avail.inactive && inactive %in% trts.all)
    rownames(C.matrix) <- c(trts, inactive)
  else
    rownames(C.matrix) <- trts
  #
  attr(C.matrix, "inactive") <- inactive
  attr(C.matrix, "sep.comps") <- sep.comps
  #
  C.matrix
}

#' Forest plot showing results of two or more network meta-analyses
#' 
#' @description
#' Forest plot to show network estimates of two or more network
#' meta-analyses.
#'
#' @aliases forest.netbind plot.netbind
#' 
#' @param x An object of class \code{netbind}.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param equal.size A logical indicating whether all squares should
#'   be of equal size. Otherwise, the square size is proportional to
#'   the precision of estimates.
#' @param leftcols A character vector specifying columns to be plotted
#'   on the left side of the forest plot (see Details).
#' @param leftlabs A character vector specifying labels for columns on
#'   left side of the forest plot.
#' @param rightcols A character vector specifying columns to be
#'   plotted on the right side of the forest plot (see Details).
#' @param rightlabs A character vector specifying labels for columns
#'   on right side of the forest plot.
#' @param subset.treatments A character vector specifying treatments
#'   to show in forest plot as comparators to the reference.
#' @param digits Minimal number of significant digits for treatment
#'   effects and confidence intervals, see \code{print.default}.
#' @param digits.prop Minimal number of significant digits for the
#'   direct evidence proportion.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param lab.NA A character string to label missing values.
#' @param smlab A label printed at top of figure. By default, text
#'   indicating either common or random effects model is printed.
#' @param \dots Additional arguments for \code{\link[meta]{forest.meta}}
#'   function.
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window.
#' 
#' The arguments \code{leftcols} and \code{rightcols} can be used to
#' specify columns which are plotted on the left and right side of the
#' forest plot, respectively. If argument \code{rightcols} is
#' \code{FALSE}, no columns will be plotted on the right side.
#' 
#' For more information see help page of \code{\link[meta]{forest.meta}}
#' function.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netbind}}, \code{\link{netcomb}},
#'   \code{\link[meta]{forest.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netbind)
#' 
#' @method forest netbind
#' @export

forest.netbind <- function(x,
                           pooled = ifelse(x$x$random, "random", "common"),
                           ##
                           equal.size = gs("equal.size"),
                           ##
                           leftcols = "studlab",
                           leftlabs = "Treatment",
                           rightcols = c("effect", "ci"),
                           rightlabs = NULL,
                           ##
                           subset.treatments,
                           ##
                           digits = gs("digits.forest"),
                           digits.prop = max(gs("digits.pval") - 2, 2),
                           ##
                           backtransf = x$backtransf,
                           lab.NA = gs("lab.NA"),
                           smlab,
                           ...) {
  
  
  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netbind")
  x <- updateversion(x)
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  ##
  chklogical(equal.size)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  chklogical(backtransf)
  ##
  chkchar(lab.NA)
  #
  commons <- replaceNULL(x$x$commons, 1)
  randoms <- replaceNULL(x$x$randoms, 1)
  
  
  ##
  ##
  ## (2) Extract results for common and random effects model
  ##
  ##
  if (pooled == "common") {
    if (!missing(subset.treatments)) {
      subset.treatments <- setchar(subset.treatments, unique(x$common$treat))
      sel <- x$common$treat %in% subset.treatments
    }
    else
      sel <- x$common$treat != x$reference.group
    ##
    m <-
      suppressWarnings(metagen(x$common$TE, x$common$seTE,
                               studlab = x$common$name,
                               sm = x$sm,
                               common = FALSE, random = FALSE,
                               subgroup = x$common$treat,
                               print.subgroup.name = FALSE,
                               subset = sel,
                               method.tau = "DL", method.tau.ci = ""))
    ##
    m$studlab <- x$common$name[sel]
    m$TE <- x$common$TE[sel]
    m$seTE <- x$common$seTE[sel]
    m$lower <- x$common$lower[sel]
    m$upper <- x$common$upper[sel]
    m$statistic <- x$common$statistic[sel]
    m$pval <- x$common$pval[sel]
    m$zval <- x$common$statistic[sel]
    ##
    m$col.study <- x$common$col.study[sel]
    m$col.square <- x$common$col.square[sel]
    m$col.square.lines <- x$common$col.square.lines[sel]
    m$col.inside <- x$common$col.inside[sel]
    #
    if (length(unique(commons)) == 1)
      text.pooled <- "\n(Common Effects Model)"
    else
      text.pooled <- ""
    #
    if (!is.null(x$common$method))
      x$method <- unique(x$common$method[sel])
  }
  else {
    if (!missing(subset.treatments)) {
      subset.treatments <- setchar(subset.treatments, unique(x$random$treat))
      sel <- x$random$treat %in% subset.treatments
    }
    else
      sel <- x$random$treat != x$reference.group
    ##
    m <-
      suppressWarnings(metagen(x$random$TE, x$random$seTE,
                               studlab = x$random$name,
                               sm = x$sm,
                               common = FALSE, random = FALSE,
                               subgroup = x$random$treat,
                               print.subgroup.name = FALSE,
                               subset = sel,
                               method.tau = "DL", method.tau.ci = ""))
    ##
    m$studlab <- x$random$name[sel]
    m$TE <- x$random$TE[sel]
    m$seTE <- x$random$seTE[sel]
    m$lower <- x$random$lower[sel]
    m$upper <- x$random$upper[sel]
    m$statistic <- x$random$statistic[sel]
    m$pval <- x$random$pval[sel]
    m$zval <- x$random$statistic[sel]
    ##
    m$col.study <- x$random$col.study[sel]
    m$col.square <- x$random$col.square[sel]
    m$col.square.lines <- x$random$col.square.lines[sel]
    m$col.inside <- x$random$col.inside[sel]
    #
    if (length(unique(randoms)) == 1)
      text.pooled <- "\n(Random Effects Model)"
    else
      text.pooled <- ""
    #
    if (!is.null(x$random$method))
      x$method <- unique(x$random$method[sel])
  }
  ##
  if (missing(smlab))
    if (x$baseline.reference)
      smlab <- paste0("Comparison: other vs '",
                      x$reference.group, "'",
                      text.pooled)
    else
      smlab <- paste0("Comparison: '",
                      x$reference.group, "' vs other",
                      text.pooled)
  #
  m$.text.details.methods <-
    textmeth(x, pooled == pooled, FALSE, FALSE,
             "", "", gs("digits.tau2"), gs("digits.tau"),
             FALSE, gs("text.I2"),
             big.mark = gs("big.mark"), forest = TRUE)
  
  
  ##
  ##
  ## (3) Forest plot
  ##
  ##
  forest(m,
         digits = digits,
         ##
         overall = FALSE, common = FALSE, random = FALSE,
         hetstat = FALSE, test.subgroup = FALSE,
         ##
         subgroup.hetstat = FALSE,
         prediction.subgroup = FALSE,
         calcwidth.subgroup = TRUE,
         ##
         leftcols = leftcols,
         leftlabs = leftlabs,
         rightcols = rightcols,
         rightlabs = rightlabs,
         ##
         lab.NA = lab.NA,
         smlab = smlab,
         backtransf = backtransf,
         ##
         col.study = m$col.study,
         col.square = m$col.square,
         col.square.lines = m$col.square.lines,
         col.inside = m$col.inside,
         col.inside.common = "black",
         col.inside.random = "black",
         ##
         weight.study = if (equal.size) "same" else pooled,
         ...)
  
  ret <- m
  ##
  ret$leftcols <- leftcols
  ret$rightcols <- rightcols
  ret$leftlabs <- leftlabs
  ret$rightlabs <- rightlabs
  ##
  invisible(ret)
}


#' @rdname forest.netbind
#' @method plot netbind
#' @export

plot.netbind <- function(x, ...)
  forest(x, ...)

tau.within <- function(x) {
  
  
  chkclass(x, "netmeta")
  
  
  nmak <- nma_krahn(x)
  ##
  if (nmak$n == 2)
    return(NA)
  
  
  t <- diag(rep(1, length(as.character(nmak$design$design))))
  rownames(t) <- paste(as.character(nmak$design$comparison),
                       as.character(nmak$design$design), sep = "_")
  colnames(t) <- paste(as.character(nmak$design$comparison),
                       as.character(nmak$design$design), sep = "_")
  X_incon_studies <- t[paste(as.character(nmak$studies$comparison),
                             as.character(nmak$studies$design), sep = "_"), ]
  
  
  DiagonalBlocks <- function(V) {
    if (length(V) == 1) B <- 1
    else {
      m <- NULL
      b <- NULL
      d <- NULL
      B <- NULL
      for (i in 1:nrow(V)) {
        h <- which(abs(V[i, ]) > 0)
        b[i] <- h[1]
        d[i] <- max(h) - min(h) + 1
      }
      ##
      m <- d[as.numeric(names(table(b)))]
      b <- b[as.numeric(names(table(b)))]
      ##
      for (i in 1:length(m))
        B[i] <- list(b[i]:(b[i] + m[i] - 1))
    }
    B
  }

  
  V0 <- function(m)
    diag(m) / 2 + matrix(.5, m, m)
  
  
  W <- function(V, tausq) {
    W <- diag(nrow(as.matrix(V)))
    b <- DiagonalBlocks(V)
    ##
    for (i in 1:length(b))
      W[b[[i]], b[[i]]] <-
        solve(as.matrix(V)[b[[i]], b[[i]]] + tausq * V0(length(b[[i]])))
    ##
    W
  }
  
  
  tau.DL <- function(X, y, V) {
    C <- solve(t(X) %*% (W0 <- W(V, 0)) %*% X)
    theta <- C %*% t(X) %*% W0 %*% y
    r <- y - X %*% theta
    Q <- t(r) %*% W0 %*% r
    V.0 <- diag(nrow(V))
    b <- DiagonalBlocks(V)
    ##
    for (i in 1:length(b))
      V.0[b[[i]], b[[i]]] <- V0(length(b[[i]]))
    ##
    trace1 <- sum(diag(W0 %*% V.0))
    trace2 <- sum(diag(C %*% t(X) %*% W0 %*% V.0 %*% W0 %*% X))
    ##
    if (trace1 == trace2)
      tausq <- 0
    else
      tausq <- max(0, (Q - nrow(V) + ncol(X)) / (trace1 - trace2))
    ##
    sqrt(tausq)
  }
  
  
  tau.within <- tau.DL(X_incon_studies, nmak$studies$TE, nmak$V.studies)
  ##
  tau.within
}

#' Frequentist method to rank treatments in network
#' 
#' @description
#' Ranking treatments in frequentist network meta-analysis with and
#' without resampling methods.
#' 
#' @aliases netrank print.netrank
#' 
#' @param x An object of class \code{netmeta} or \code{rankogram}.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect, can be abbreviated.
#' @param method A character string specifying whether the
#'   \code{"P-score"} or \code{"SUCRA"} ranking metric will be
#'   calculated.
#' @param nsim Number of repetitions to calculate SUCRAs.
#' @param common A logical indicating whether to print P-scores or
#'   SUCRAs for the common effects model.
#' @param random A logical indicating whether to print P-scores or
#'   SUCRAs for the random effects model.
#' @param sort A logical indicating whether printout should be sorted
#'   by decreasing P-score.
#' @param digits Minimal number of significant digits, see
#'   \code{\link{print.default}}.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments passed on to
#'   \code{\link{print.data.frame}} function (used internally).
#' 
#' @details
#' 
#' Treatments are ranked based on a network meta-analysis. Ranking is
#' performed by a ranking metric: P-score or SUCRA.
#'
#' P-scores are based solely on the point estimates and standard
#' errors of the network estimates. They measure the extent of
#' certainty that a treatment is better than another treatment,
#' averaged over all competing treatments (Rücker and Schwarzer 2015).
#'
#' The Surface Under the Cumulative RAnking curve (SUCRA) is the rank
#' of treatment \emph{i} within the range of treatments, measured on a
#' scale from 0 (worst) to 1 (best) (Salanti et al. 2011). A
#' resampling method is used to calculate SUCRAs for frequentist
#' network meta-analysis. The number of repetitions is determine by
#' argument \code{nsim}.
#'
#' The interpretation of P-scores and SUCRAs is comparable.
#' 
#' The P-score of treatment \emph{i} is defined as the mean of all 1 -
#' P[\emph{j}] where P[\emph{j}] denotes the one-sided P-value of
#' accepting the alternative hypothesis that treatment \emph{i} is
#' better than one of the competing treatments \emph{j}. Thus, if
#' treatment \emph{i} is better than many other treatments, many of
#' these P-values will be small and the P-score will be large. Vice
#' versa, if treatment \emph{i} is worse than most other treatments,
#' the P-score is small.
#' 
#' The P-score of treatment \emph{i} can be interpreted as the mean
#' extent of certainty that treatment \emph{i} is better than another
#' treatment.
#'
#' @return
#' An object of class \code{netrank} with corresponding \code{print}
#' function. The object is a list containing the following components:
#' \item{ranking.common}{A named numeric vector with P-scores or SUCRAs
#'   for the common effects model.}
#' \item{Pmatrix.common}{Numeric matrix based on pairwise one-sided
#'   p-values for the common effects model.}
#' \item{ranking.random}{A named numeric vector with P-scores or
#'   SUCRAs for the random effects model.}
#' \item{Pmatrix.random}{Numeric matrix based on pairwise one-sided
#'   p-values of the random effects model.}
#' \item{small.values, method, x}{As defined above.}
#' \item{version}{Version of R package netmeta used to create object.}
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}, Theodoros
#'   Papakonstantinou \email{dev@@tpapak.com}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{rankogram}},
#'   \code{\link{plot.rankogram}}
#' 
#' @references
#' Rücker G, Schwarzer G (2017):
#' Resolve conflicting rankings of outcomes in network meta-analysis:
#' Partial ordering of treatments.
#' \emph{Research Synthesis Methods},
#' \bold{8}, 526--36
#' 
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#' 
#' @examples
#' \donttest{
#' # Define order of treatments in depression dataset Linde2015
#' #
#' trts <- c("TCA", "SSRI", "SNRI", "NRI",
#'   "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")
#'
#' # Outcome labels
#' #
#' outcomes <- c("Early response", "Early remission")
#' 
#' # (1) Early response
#' #
#' pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma1 <- netmeta(pw1, common = FALSE, seq = trts, ref = "Placebo",
#'   small.values = "undesirable")
#' netrank(nma1)
#' 
#' # (2) Early remission
#' #
#' pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma2 <- netmeta(pw2, common = FALSE, seq = trts, ref = "Placebo",
#'   small.values = "undesirable")
#' netrank(nma2)
#' 
#' # Image plot of treatment rankings (two outcomes)
#' #
#' plot(netrank(nma1), netrank(nma2), name = outcomes, digits = 2)
#'
#' 
#' # Outcome labels
#' #
#' outcomes <- c("Early response", "Early remission",
#'   "Lost to follow-up", "Lost to follow-up due to AEs",
#'   "Adverse events (AEs)")
#' 
#' # (3) Loss to follow-up
#' #
#' pw3 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(loss1, loss2, loss3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma3 <- netmeta(pw3, common = FALSE, seq = trts, ref = "Placebo",
#'   small.values = "desirable")
#' 
#' # (4) Loss to follow-up due to adverse events
#' #
#' pw4 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(loss.ae1, loss.ae2, loss.ae3), n = list(n1, n2, n3),
#'   studlab = id, data = subset(Linde2015, id != 55), sm = "OR")
#' #
#' nma4 <- netmeta(pw4, common = FALSE, seq = trts, ref = "Placebo",
#'   small.values = "desirable")
#' 
#' # (5) Adverse events
#' #
#' pw5 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(ae1, ae2, ae3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma5 <- netmeta(pw5, common = FALSE, seq = trts, ref = "Placebo",
#'   small.values = "desirable")
#' 
#' # Image plot of treatment rankings (two outcomes)
#' #
#' plot(netrank(nma1), netrank(nma2), netrank(nma3),
#'   netrank(nma4), netrank(nma5),
#'   name = outcomes, digits = 2)
#' }
#' 
#' @rdname netrank
#' @export netrank

netrank <- function(x, small.values = x$small.values, method, nsim,
                    common = x$common, random = x$random,
                    warn.deprecated = gs("warn.deprecated"),
                    ...) {
  
  ##
  ##
  ## (1) Check and upgrade object
  ##
  ##
  chkclass(x, c("netmeta", "netcomb", "rankogram"))
  x <- updateversion(x)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  if (missing(method)) {
    if (inherits(x, "netcomb"))
      method <- "P-score"
    else if (inherits(x, "netmeta") & !inherits(x, gs(".other_nma")))
      method <- "P-score"
    else
      method <- "SUCRA"
  }
  else
    method <- setchar(method, c("P-score", "SUCRA"))
  #
  small.values <- setsv(small.values)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  # Additional checks for crossnma, gemtc and multinma objects
  #
  if (inherits(x, gs(".other_nma"))) {
    if (method == "P-score")
      warning("Argument 'method = \"SUCRA\"' for netmeta.", x$method,
              " object.",
              call. = FALSE)
    #
    if (common != x$common)
      warning("Argument 'common = ", x$common, "' as netmeta.", x$method,
              " object is based on ", if (x$common) "common" else "random",
              " effects model.",
              call. = FALSE)
    #
    if (random != x$random)
      warning("Argument 'random = ", x$random, "' as netmeta.", x$method,
              " object is based on ", if (x$random) "random" else "common",
              " effects model.",
              call. = FALSE)
    #
    #
    if (!missing(nsim) & x$keep.samples)
      warning("Argument 'nsim' ignored for netmeta.", x$method, " object.",
              call. = FALSE)
    #
    method <- "SUCRA"
    common <- x$common
    random <- x$random
  }
  
  
  if (method == "SUCRA") {
    ##
    ## SUCRAs
    ##
    if (inherits(x, "netcomb"))
      stop("netcomb object is not compatible with SUCRAs.",
           call. = FALSE)
    else if (inherits(x, "netmeta")) {
      if (inherits(x, gs(".other_nma")) && x$keep.samples) {
        rnk <- rankogram(x$samples$d,
                         pooled = if (common) "common" else "random",
                         small.values = small.values)
        #
        nsim <- rnk$nsim
        #
        if (common)
          rnk$ranking.random <- setNA(rnk$ranking.common)
        else
          rnk$ranking.common <- setNA(rnk$ranking.random)
      }
      else {
        if (missing(nsim))
          nsim <- gs("nsim")
        ##
        rnk <- rankogram(x,
                         nsim = nsim,
                         small.values = small.values,
                         common = common, random = random)
      }
    }
    else {
      if (!missing(nsim))
        warning("Argument 'nsim' ignored for rankogram object.",
                call. = FALSE)
      rnk <- x
      x <- rnk$x
      nsim <- rnk$nsim
    }
    ##
    P.common <- NULL
    P.random <- NULL
    ##
    ranking.common <- rnk$ranking.common
    ranking.random <- rnk$ranking.random
  }
  else {
    ##
    ## P-scores
    ##
    nsim <- NULL
    ##
    TE.common <- x$TE.common
    pval.common <- x$pval.common
    ##
    TE.random <- x$TE.random
    pval.random <- x$pval.random
    
    
    ## Calculate one-sided p-values
    ##
    w.common <- (1 + sign(TE.common)) / 2
    p.common <- pval.common
    ##
    if (small.values == "desirable")
      P.common <-
        w.common * p.common / 2 + (1 - w.common) * (1 - p.common / 2)
    else
      P.common <-
        w.common * (1 - p.common / 2) + (1 - w.common) * p.common / 2
    ##
    w.random <- (1 + sign(TE.random)) / 2
    p.random <- pval.random
    ##
    if (small.values == "desirable")
      P.random <-
        w.random * p.random / 2 + (1 - w.random) * (1 - p.random / 2)
    else
      P.random <-
        w.random * (1 - p.random / 2) + (1 - w.random) * p.random / 2
    
    
    ## Row means provide P-scores
    ##
    ranking.common <- rowMeans(P.common, na.rm = TRUE)
    ##
    if (!all(is.na(TE.random)))
      ranking.random <- rowMeans(P.random, na.rm = TRUE)
    else
      ranking.random <- NA
  }
  
  
  res <- list(ranking.common = ranking.common,
              Pmatrix.common = P.common,
              ##
              ranking.random = ranking.random,
              Pmatrix.random = P.random,
              ##
              method = method,
              small.values = small.values,
              ##
              nsim = nsim,
              ##
              common = common,
              random = random,
              ##
              x = x,
              title = x$title,
              version = packageDescription("netmeta")$Version,
              ##
              Pscore = paste("'Pscore' replaced by 'ranking.common'",
                             "and 'ranking.random'."),
              Pmatrix = paste("'Pmatrix' replaced by 'Pmatrix.common'",
                              "and 'Pmatrix.random'.")
              )
  ##
  ## Backward compatibility
  ##
  res$fixed <- common
  ##
  res$ranking.fixed <- res$ranking.common
  res$Pmatrix.fixed <- res$Pmatrix.common
  ##
  ## For R package NMAoutlier:
  ##
  res$Pscore.fixed <-
    if (method == "P-score") ranking.common else NULL
  res$Pscore.random <-
    if (method == "P-score") ranking.random else NULL
  
  
  class(res) <- "netrank"
  
  res
}


#' @rdname netrank
#' @method print netrank
#' @export

print.netrank <- function(x,
                          common = x$common,
                          random = x$random,
                          sort = TRUE,
                          digits = gs("digits.prop"),
                          warn.deprecated = gs("warn.deprecated"),
                          ...) {
  
  ##
  ##
  ## (1) Check netrank object and upgrade object
  ##
  ##
  chkclass(x, "netrank")
  x <- updateversion(x)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  if (is.character(sort)) {
    sort <- setchar(sort, c("common", "random", "fixed"))
    sort[sort == "fixed"] <- "common"
  }
  else
    chklogical(sort)
  ##
  chknumeric(digits, length = 1)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  
  
  both <- (common + random) == 2
  ##
  if (!both & is.character(sort)) {
    if (common & sort == "random") {
      warning("Argument 'sort=\"random\"' ignored for common effects model.",
              call. = FALSE)
      sort <- TRUE
    }
    if (random & sort == "common") {
      warning("Argument 'sort=\"common\"' ignored for random effects model.",
              call. = FALSE)
      sort <- TRUE
    }
  }
  else if (both & !is.character(sort) && sort)
    sort <- "random"


  if (is.null(x$method)) {
    x$method <- "P-score"
    x$ranking.common <- x$Pscore.common
    x$ranking.random <- x$Pscore.random
  }
  
  
  if (both) {
    if (is.character(sort)) {
      res.both <- data.frame(common = round(x$ranking.common, digits),
                             random = round(x$ranking.random, digits))
      res.both <- res.both[order(-res.both[, sort]), ]
    }
    else if (!sort) {
      res.both <- data.frame(common = round(x$ranking.common[x$x$seq], digits),
                             random = round(x$ranking.random[x$x$seq], digits))
    }
    ##
    colnames(res.both) <-
      paste(x$method,
            paste0("(", c(gs("text.w.common"), gs("text.w.random")), ")"))
  }
  else {
    if (sort) {
      if (common)
        res.common <-
          as.data.frame(round(x$ranking.common[order(-x$ranking.common)],
                              digits))
      if (random)
        res.random <-
          as.data.frame(round(x$ranking.random[order(-x$ranking.random)],
                              digits))
    }
    else {
      if (common)
        res.common <- as.data.frame(round(x$ranking.common[x$x$seq], digits))
      if (random)
        res.random <- as.data.frame(round(x$ranking.random[x$x$seq], digits))
    }
    ##
    if (common)
      colnames(res.common)  <- x$method
    if (random)
      colnames(res.random) <- x$method
  }
  ##
  matitle(x)
  ##
  if (both)
    prmatrix(res.both, quote = FALSE, ...)
  ##
  else if (common) {
    prmatrix(res.common, quote = FALSE, ...)
    if (random)
      cat("\n")
  }
  ##
  else if (random) {
    prmatrix(res.random, quote = FALSE, ...)
  }

  if (x$method == "SUCRA" & !is.null(x$nsim))
    cat("\n- based on ", x$nsim,
        " repetition", if (x$nsim > 1) "s", "\n",
        sep = "")
  
  
  invisible(NULL)
}

nma_krahn <- function(x, reference.group = x$reference.group,
                      tau.preset = 0, sep.trts = x$sep.trts) {
  
  
  mysolve <- function(x) {
    res <- try(solve(x), silent = TRUE)
    #
    if (inherits(res, "try-error")) {
      res <- x
      res[!is.na(res)] <- NA
    }
    #
    res
  }
  
  chkclass(x, "netmeta")
  
  if (is.na(tau.preset))
    tau.preset <- 0
  
  if (is.null(sep.trts))
    sep.trts <- ":"
  
  n <- x$n


  if (reference.group == "")
    trts <- colnames(x$A.matrix)
  else
    trts <- c(reference.group,
              colnames(x$A.matrix)[colnames(x$A.matrix) != reference.group])


  studies.pre <- data.frame(studlab = x$studlab,
                            treat1 = x$treat1, treat2 = x$treat2,
                            TE = -x$TE,
                            seTE = sqrt(x$seTE^2 + tau.preset^2),
                            narms = x$narms[match(x$studlab, x$studies)],
                            stringsAsFactors = FALSE)
  ##
  studies <- studies.pre <- studies.pre[order(studies.pre$studlab), ]


  twoarm   <- any(studies$narms == 2)
  multiarm <- any(studies$narms > 2)
  selmulti <- studies$narms > 2


  sel <- studies.pre$treat2 == reference.group
  ##
  studies$treat1[sel] <- studies.pre$treat2[sel]
  studies$treat2[sel] <- studies.pre$treat1[sel]
  studies$TE[sel] <- -studies.pre$TE[sel]
  studies <- data.frame(studies,
                        comparison = paste(studies$treat1, studies$treat2, sep = sep.trts))


  comparison.num.poss <- n * (n - 1) / 2
  comparisons <- levels(factor(as.character(studies$comparison)))
  comparison.num <- length(comparisons)


  trts.poss <- rep(NA, comparison.num.poss)
  k <- 1
  for (i in 1:(n - 1))
    for (j in (i + 1):n) {
      trts.poss[k] <- paste(trts[i], trts[j], sep = sep.trts)
      k <- k + 1
    }


  direct <- matrix(NA, nrow = comparison.num, ncol = 6)
  ##
  colnames(direct) <- c("comparison", "TE", "seTE",
                        "TE.2arm", "seTE.2arm", "n.2arm")
  ##
  direct <- data.frame(direct)
  j <- 0
  ##
  for (i in names(table(studies$comparison))) {
    j <- j + 1
    ##
    TE.i <- studies$TE[studies$comparison == i]
    seTE.i <- studies$seTE[studies$comparison == i]
    #
    m1 <-
      suppressWarnings(metagen(TE.i, seTE.i, sm = x$sm,
                               method.tau = "DL", method.tau.ci = "",
                               warn = FALSE))
    ##
    direct$comparison[j] <- i
    direct$TE[j] <- m1$TE.common
    direct$seTE[j] <- m1$seTE.common
    ##
    if (sum(studies$comparison == i & !selmulti) > 0) {
      TE.i   <- studies$TE[studies$comparison == i & studies$narms == 2]
      seTE.i <- studies$seTE[studies$comparison == i & studies$narms == 2]
      #
      m2 <-
        suppressWarnings(metagen(TE.i, seTE.i, sm = x$sm,
                                 method.tau = "DL", method.tau.ci = "",
                                 warn = FALSE))
      ##
      direct$TE.2arm[j] <- m2$TE.common
      direct$seTE.2arm[j] <- m2$seTE.common
      direct$n.2arm[j] <- m2$k
    }
  }


  if (multiarm) {
    multistudies <-
      split(studies[selmulti, ], as.character(studies$studlab[selmulti]))
    multistudies <-
      lapply(multistudies,
             function(x)
               x[which(x$treat1 == names(which.max(table(x$treat1)))), ]
             )
    multistudies <-
      lapply(multistudies,
             function(x)
               x[order(x$treat2), ]
             )
    ##
    des <-
      lapply(multistudies,
             function(x)
               paste(c(x$treat1[1], x$treat2), collapse = sep.trts)
             )
    multistudies <-
      data.frame(unsplit(multistudies,
                         rep(names(multistudies),
                             unlist(lapply(multistudies,
                                           function(x)
                                             nrow(x)
                                           )
                                    )
                             )
                         ),
                 design = unsplit(des,
                                  rep(names(multistudies),
                                      unlist(lapply(multistudies,
                                                    function(x)
                                                      nrow(x)
                                                    )
                                             )
                                      )
                                  )
                 )
    ##
    multistudies <-
      data.frame(multistudies,
                 des = paste(multistudies$comparison, multistudies$design,
                             sep = "_"))
    ##
    row.names(studies) <- NULL
    multistudies2 <-
      split(studies[selmulti, ], as.character(studies$studlab[selmulti]))
    multistudies2 <-
      lapply(multistudies2,
             function(x)
               x[do.call(order, x[, c("treat1","treat2")]), ]
             )
    multistudies2 <-
      lapply(multistudies2,
             function(x)
               rbind(
                 x[x$treat1 == names(which.max(table(x$treat1))), ],
                 x[x$treat1 %in%
                   names(table(x$treat1)[-which.max(table(x$treat1))]), ])
             )
    multistudies2 <-
      unsplit(multistudies2,
              rep(names(multistudies2),
                  unlist(lapply(multistudies2,
                                function(x)
                                  nrow(x)
                                )
                         )
                  )
              )
  }
  
  studies <- data.frame(studies, design = studies$comparison)
  if (multiarm & sum(is.na(direct$seTE.2arm)) > 0)
    direct2 <- data.frame(direct[!is.na(direct$seTE.2arm), ])
  else
    direct2 <- direct
  ##
  direct2 <- data.frame(direct2)
  
  if (length(unique(studies$comparison)) == 1) {
    res <- list(n = 2)
    ##
    class(res) <- "nma_krahn"
    ##
    return(res)
  }
  
  V.design <- diag(direct2$seTE.2arm^2,
                   nrow = length(direct2$seTE.2arm),
                   ncol = length(direct2$seTE.2arm))
  
  
  if (multiarm) {
    sp <- split(multistudies2, multistudies2$studlab)
    armM <- unlist(lapply(split(multistudies2$narms, multistudies2$studlab),
                          function(x)
                            x[1]
                          )
                   )
    ##
    covs <-
      lapply(sp,
             function(x) {
               n <- x$narms[1]
               k <- 0
               m <- matrix(NA, nrow = n - 1, ncol = n - 1)
               for (i in 1:(n - 2)) {
                 for (j in (i + 1):(n - 1)) {
                   m[i, j] <-
                     (x$seTE[i]^2 + x$seTE[j]^2 - x$seTE[n + k]^2) / 2
                   m[j, i] <-
                     (x$seTE[i]^2 + x$seTE[j]^2 - x$seTE[n + k]^2) / 2
                   k <- k + 1
                 }
               }
               diag(m) <- x$seTE[1:(n - 1)]^2
               m
             }
             )
    ##
    V3 <- NA
    ##
    for (i in 1:length(covs))
      V3 <- adiag(V3, covs[[i]])
    ##
    V3 <- V3[-1, -1]
    ##
    if (sum(!selmulti) == 0) {
      V.studies <- V3
    }
    else {
      if (sum(!selmulti) < 2)
        V.2arm <- matrix(studies$seTE[!selmulti]^2)
      else
        V.2arm <- diag(studies$seTE[!selmulti]^2)
      ##
      V.studies <- adiag(V.2arm, V3)
    }
    colnames(V.studies) <- c(as.character(studies$design[!selmulti]),
                             as.character(multistudies$design))
    rownames(V.studies) <- c(as.character(studies$design[!selmulti]),
                             as.character(multistudies$design))
    ##
    multicomp <- names(which(table(multistudies$design) > 0))
    V3.agg <- NA
    TE.agg <- NA
    #
    for (i in 1:length(multicomp)) {
      studlabM <-
        unique(multistudies$studlab[multistudies$design == multicomp[i]])
      ncovs <- covs[names(covs) %in% studlabM]
      l <- sapply(ncovs, mysolve)
      dim <- multistudies$narms[multistudies$studlab == studlabM[1]][1] - 1
      covs3 <- solve(matrix(apply(l, 1, sum), nrow = dim))
      V3.agg <- adiag(V3.agg, covs3)
      m <- matrix(NA, nrow = dim, ncol = length(studlabM))
      for (j in 1:length(studlabM))
        m[, j] <- matrix(l[, j], nrow = dim) %*%
          multistudies$TE[multistudies$studlab == studlabM[j]]
      ##
      TE.agg <- c(TE.agg, covs3 %*% apply(m, 1, sum))
    }
    
    TE.agg <- TE.agg[-1]
    V3.agg <- V3.agg[-1, -1]

    V <- adiag(V.design, V3.agg)
    ##
    nam <-
      rep(multicomp,
          unlist(lapply(split(multistudies, multistudies$design),
                        function(x)
                          x$narms[1]
                        )
                 ) - 1
          )
    ##
    if (any(twoarm))
      rownames(V) <- colnames(V) <- c(direct2$comparison, nam)
    else
      rownames(V) <- colnames(V) <- nam
    ##
    TE.dir <- c(direct2$TE.2arm, TE.agg)
  }
  else {
    V <- adiag(V.design)
    rownames(V) <- direct2$comparison
    colnames(V) <- direct2$comparison
    TE.dir <- direct2$TE.2arm
    V.studies <- diag(studies$seTE[!selmulti]^2)
    colnames(V.studies) <- rownames(V.studies) <-
      as.character(studies$comparison[!selmulti])
  }
  #
  if (!all(is.na(V)) && min(eigen(V, only.values = TRUE)$values) < 0)
    stop("Covariance matrix is not non-negative definite.")


  fX <- function(n) {
    possK <- n * (n - 1) / 2
    X <- matrix(0, nrow = possK, ncol = n - 1)
    X[1:(n - 1), 1:(n - 1)] <- diag(rep(-1, n - 1))
    X[n * (n - 1) / 2, (n - 2):(n - 1)] <- cbind(1, -1)
    if (n * (n - 1) / 2 - (n - 1) > 1) {
      l <- n
      j <- n - 2
      u <- n + j - 1
      for (k in 1:(n - 3)) {
        X[l:u, k:(n - 1)] <- cbind(1, diag(rep(-1, n - k - 1)))
        j <- j - 1
        l <- u + 1
        u <- l + j - 1
      }
    }
    X
  }
  ##
  X.full <- fX(n)
  rownames(X.full) <- trts.poss
  colnames(X.full) <- trts.poss[1:n - 1]
  ##
  X.obs2.design <- X.full[direct2$comparison, , drop = FALSE]
  
  
  if (multiarm) {
    num.basics.design <-
      unlist(lapply(split(multistudies, multistudies$design),
                    function(x)
                      x$narms[1]
                    )
             ) - 1
    ##
    basics <-
      lapply(split(multistudies, multistudies$design),
             function(x)
               split(x, x$studlab)[[1]]$comparison
             )
    basics <- unsplit(basics, rep(1:length(multicomp), num.basics.design))
    ##
    X.obs3.design <- X.full[as.character(basics), ]
    rownames(X.obs3.design) <- rep(multicomp, num.basics.design)
    X.obs <- rbind(X.obs2.design, X.obs3.design)
  }
  else
    X.obs <- X.obs2.design
  ##
  H <-
    X.full %*% solve(t(X.obs) %*% solve(V) %*% X.obs) %*%
    t(X.obs) %*% solve(V)
  ##
  TE.net <- H %*% TE.dir
  
  
  covTE.net.base <- solve(t(X.obs) %*% solve(V) %*% X.obs)
  co <- NA
  for (i in 1:(n - 2)) {
    for (j in 2:(n - 1)) {
      if (i != j && i < j) {
        co <- c(co,
                diag(covTE.net.base)[i] +
                diag(covTE.net.base)[j] -
                2 * covTE.net.base[i, j])
      }
    }
  }
  ##
  covTE.net <- c(diag(covTE.net.base), co[-1])


  comps <- as.character(studies$comparison[!selmulti])
  studlabs <- as.character(studies$studlab[!selmulti])
  ##
  if (multiarm) {
    comps <- c(comps, as.character(multistudies$comparison))
    studlabs <- c(studlabs, as.character(multistudies$studlab))
  }


  X.obs.studies <- X.full[comps, ]

  
  H.studies <- X.full %*%
    solve(t(X.obs.studies) %*% mysolve(V.studies) %*% X.obs.studies) %*%
      t(X.obs.studies) %*% mysolve(V.studies)
  ##
  colnames(H.studies) <- studlabs


  network <- data.frame(TE = TE.net, seTE = sqrt(covTE.net))


  if (multiarm) {
    len.designs <-
      c(rep(1, length(direct2$comparison)),
        unlist(lapply(compsplit(multicomp, sep.trts),
                      function(x)
                        length(x) - 1
                      )
               )
        )
    freq <-
      rep(c(direct2$n.2arm,
            unlist(lapply(split(multistudies, multistudies$design),
                          function(x)
                            length(names(table(x$studlab)))
                          )
                   )
            ),
          len.designs)
    narms <-
      rep(c(rep(2, nrow(direct2)),
            unlist(lapply(compsplit(multicomp, sep.trts),
                          function(x)
                            length(x)
                          )
                   )
            ),
          len.designs)
    ##
    design <-
      data.frame(design = c(direct2$comparison,
                            rep(multicomp,
                                unlist(lapply(compsplit(multicomp, sep.trts),
                                              function(x)
                                                length(x)
                                              )
                                       ) - 1
                                )
                            ),
                 comparison = c(direct2$comparison,
                                as.character(unlist(
                                  lapply(split(multistudies,
                                               multistudies$design),
                                         function(x)
                                           as.character(
                                             unlist(
                                               split(x, x$studlab)[[1]]["comparison"]
                                             )
                                           )
                                         )
                                )
                                )
                                ),
                 narms = narms,
                 freq = freq,
                 TE.dir = TE.dir,
                 seTE.dir = sqrt(diag(V)))
  }
  else {
    len.designs <- c(rep(1, length(direct2$comparison)))
    freq <- rep(c(direct2$n.2arm), len.designs)
    narms <- rep(c(rep(2, nrow(direct2))), len.designs)
    ##
    design <- data.frame(design = colnames(V),
                         comparison = colnames(V),
                         narms = rep(2, length(direct2$comparison)),
                         freq = direct2$n.2arm,
                         TE.dir = TE.dir,
                         seTE.dir = sqrt(diag(V)))
  }
  ##
  rownames(design) <- NULL
  ##
  design <-
    data.frame(design,
               TE.net = network[as.character(design$comparison), "TE"],
               seTE.net = network[as.character(design$comparison), "seTE"])
  

  if (multiarm)
    studies <- rbind(studies[!selmulti, ], multistudies[, 1:8])
  ##
  studies <- studies[, c("studlab", "treat1", "treat2",
                         "TE", "seTE", "narms",
                         "design", "comparison")]
  ##
  studies <- merge(studies, design[, names(design) != "narms"],
                   by = c("design", "comparison"))
  ##
  studies <- studies[, c("studlab", "design", "comparison", "treat1", "treat2",
                         "narms", "freq", "TE", "seTE",
                         "TE.dir", "seTE.dir", "TE.net", "seTE.net")]
  ##
  studies_lim <- studies[which(studies$narms == 2), ]
  studies_mult <- studies[which(studies$narms > 2), ]
  studies <-
    rbind(studies_lim[order(studies_lim$studlab), ],
          studies_mult[
            do.call(order, studies_mult[, c("studlab","treat1","treat2")]), ])

  res <- list(n = n,
              k = x$k,
              d = length(unique(design$design)),
              trts = trts,
              comparisons = comparisons,
              studies = studies,
              direct = direct,
              network = network,
              design = design,
              multicomp = if (multiarm) multicomp else NULL,
              X.obs = X.obs,
              X.full = X.full,
              V = V,
              V.studies = V.studies,
              H = H,
              H.studies = H.studies,
              sep.trts = sep.trts)

  class(res) <- "nma_krahn"

  res
}

#' Network meta-analysis of antithrombotic treatments in patients with
#' non-valvular atrial fibrillation
#' 
#' @description
#' This dataset comes from a systematic review aiming to determine
#' the effects of eight antithrombotic treatments in reducing the
#' incidence of major thrombotic events in patients with non-valvular
#' atrial fibrillation (Dogliotti et al., 2014). The review included 20
#' studies (79,808 participants), four of which were three-arm
#' studies. The primary outcome is stroke reduction.
#' 
#' @name Dogliotti2014
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{id}}\tab study ID \cr
#' \bold{\emph{treatment}}\tab treatment \cr
#' \bold{\emph{stroke}}\tab number of strokes \cr
#' \bold{\emph{total}}\tab number of individuals in treatment arm
#' }
#' 
#' @note
#' The dataset Dogliotti2014 is identical to dataset
#' \code{\link[metabook]{Dogliotti2014}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[metabook]{Dogliotti2014}},
#'   \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
#'   \code{\link{netmetabin}}
#' 
#' @source
#' Dogliotti A, Paolasso E, Giugliano RP (2014):
#' Current and new oral antithrombotics in non-valvular atrial
#' fibrillation: a network meta-analysis of 79 808 patients.
#' \emph{Heart},
#' \bold{100}, 396--405
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Dogliotti2014)
#' 
#' # Example using pairwise(), netmeta() and netmetabin():
#' # example(Dogliotti2014, package = "metabook")

NULL

#' Auxiliary functions for component network meta-analysis
#' 
#' @description
#' Auxiliary functions to (i) create a combination / C matrix with information
#' on treatment combinations and interaction terms and (ii) a vector with all
#' combinations in a component network meta-analysis.
#' 
#' @param x A \code{\link{netcomb}}, \code{\link{netmeta}} or
#'   \code{\link{netconnection}} object, a matrix or the number of components.
#' @param inactive A character string defining the inactive treatment
#'   component (see \code{\link{netcomb}}).
#' @param comb.ia A character vector specifying treatment combinations which
#'   will be considered as interactions.
#' @param sep.comps A single character to define separator between
#'   treatment components.
#' @param sep.ia A single character to define separator for interactions.
#' @param n A single number specifying the number of components in combinations.
#' @param \dots Additional arguments.
#'
#' @return
#' R function \code{createC} returns a combination matrix / C matrix with
#' studies in rows and component and interaction terms in columns.
#' 
#' R function \code{combinations} returns a character vector with combinations.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{discomb}}
#' 
#' @examples
#' # Examples: example(netcomb)
#' 
#' @export createC

createC <- function(x, ...)
  UseMethod("createC")


#' @rdname createC
#' @method createC matrix
#' @export

createC.matrix <- function(x, comb.ia, inactive = NULL,
                           sep.comps = gs("sep.comps"),
                           sep.ia = gs("sep.ia"), ...) {
  if (!is.matrix(x))
    stop("Argument 'x' must be a matrix.", call. = FALSE)
  #
  comps.all <- colnames(x)
  comps <- comps.all[!grepl(sep.ia, comps.all, fixed = TRUE)]
  #
  chkchar(comb.ia, length = 1)
  #
  if (!is.null(inactive)) {
    inactive <- setchar(inactive, comps.all, stop.at.error = FALSE)
    comps.all <- comps.all[!(comps.all %in% inactive)]
    comps <- comps[!(comps %in% inactive)]
    x <- x[, comps.all, drop = FALSE]
  }
  #
  missing.sep.ia <- missing(sep.ia)
  chkchar(sep.ia, nchar = 0:1, length = 1)
  if (sep.comps == sep.ia)
    stop("Input for arguments 'sep.comps' and 'sep.ia' must be different.",
         call. = FALSE)
  sep.ia <- setsep(comps, sep.ia, missing = missing.sep.ia)
  #
  mat.int <- x
  #
  for (i in length(comb.ia)) {
    comb.ia.i <- unlist(compsplit(comb.ia[i], sep.comps))
    #
    if (length(comb.ia.i) == 1)
      stop("Entry ", i, " of argument 'comb.ia' contains a single component.")
    #
    if (any(grepl(sep.ia, comb.ia.i, fixed = TRUE)))
      stop("Input for argument 'comb.ia' contains interaction separator '",
           sep.ia, "'.")
    #
    comb.ia.i <- setchar(comb.ia.i, comps, name = "comb.ia",
                         text = paste0("must be a combination of ",
                                       paste0("'", comps, "'",
                                              collapse = ", "),
                                       "; separated by '", sep.comps,
                                       "' (argument 'sep.comps')"))
    #
    if (length(comb.ia.i) != length(unique(comb.ia.i)))
      stop("Identical components provided in argument 'comb.ia'.")
    #
    mat.int.i <- x[, comb.ia.i[1], drop = FALSE]
    #
    for (i in (1 + seq_len(length(comb.ia.i) - 1)))
      mat.int.i <- mat.int.i * x[, comb.ia.i[i], drop = FALSE]
    #
    colnames(mat.int.i) <- paste(comb.ia.i, collapse = paste0(" ", sep.ia, " "))
    #
    if (colnames(mat.int.i) %in% comps.all) {
      warning("Interaction '", colnames(mat.int.i),
              "' is already part of the C matrix.",
              call. = FALSE)
      mat.int.i <- NULL
    }
    #
    if (!is.null(mat.int.i)) {
      i <- 1
      while (i <= ncol(x)) {
        if (all(unname(x[, i]) == unname(mat.int.i[, ncol(mat.int.i)]))) {
          warning("Information on interaction '", colnames(mat.int.i),
                  "' is already available in column '",
                  colnames(x)[i], "'.")
          mat.int.i <- NULL
          i <- ncol(x) + 1
        }
        else
          i <- i + 1
      }
    }
    #
    if (!is.null(mat.int.i)) {
      if (apply(abs(mat.int.i), 2, sum) == 0) {
        warning("Interaction '",
                colnames(mat.int.i),
                "' is ignored as it is inestimable ",
                "(column with zeros in C matrix).",
                call. = FALSE)
        mat.int.i <- NULL
      }
      #
      mat.int <- cbind(mat.int, mat.int.i)
    }
  }
  #
  attr(mat.int, "inactive") <- inactive
  attr(mat.int, "sep.comps") <- sep.comps
  attr(mat.int, "sep.ia") <- sep.ia
  #
  mat.int
}


#' @rdname createC
#' @method createC netcomb
#' @export

createC.netcomb <- function(x, comb.ia = NULL, inactive = NULL,
                            sep.ia = x$sep.ia,
                            ...) {
  chkclass(x, "netcomb")
  x <- updateversion(x)
  #
  C.matrix <- x$C.matrix
  #
  comps.all <- colnames(C.matrix)
  comps <- comps.all[!grepl(sep.ia, comps.all, fixed = TRUE)]
  #
  if (is.null(comb.ia))
    comb.ia <- combinations(x)
  #
  if (!is.null(inactive) & !is.null(x$inactive))
    stop("Inactive treatment was already defined in component NMA provided in ",
         "argument 'x'.",
         call. = FALSE)
  #
  missing.sep.ia <- missing(sep.ia)
  chkchar(sep.ia, nchar = 0:1, length = 1)
  if (x$sep.comps == sep.ia)
    stop("Input for argument 'sep.ia' identical to separator for components",
         call. = FALSE)
  sep.ia <- setsep(comps, sep.ia, missing = missing.sep.ia)
  #
  for (i in comb.ia)
    C.matrix <- createC(C.matrix, i, sep.comps = x$sep.comps, sep.ia = sep.ia,
                        inactive = inactive)
  #
  C.matrix
}

#' @rdname createC
#' @method createC netmeta
#' @export

createC.netmeta <- function(x, inactive = NULL,
                            sep.comps = gs("sep.comps"), ...) {
  chkclass(x, "netmeta")
  #
  chkchar(sep.comps, length = 1)
  #
  C.matrix <-
    createC_trts_inactive(x$trts, inactive = inactive, sep.comps = sep.comps)
  #
  C.matrix
}


#' @rdname createC
#' @method createC netconnection
#' @export

createC.netconnection <- function(x, inactive = NULL,
                                  sep.comps = gs("sep.comps"), ...) {
  chkclass(x, "netconnection")
  #
  chkchar(sep.comps, length = 1)
  #
  C.matrix <-
    createC_trts_inactive(rownames(x$D.matrix),
                          inactive = inactive, sep.comps = sep.comps)
  #
  C.matrix
}


#' @rdname createC
#' @method createC default
#' @export

createC.default <- function(x, n = 1, ...) {
  if (missing(x))
    stop("Argument 'x' must be provided.", call. = TRUE)
  #
  chknumeric(x, min = 1, length = 1)
  chknumeric(n, min = 1, max = x, length = 1)
  #
  C.matrix <- createC_full(x, n)
  #
  C.matrix
}


#' @rdname createC
#' @export combinations

combinations <- function(x, n = NULL) {
  chkclass(x, "netcomb")
  #
  comps <- x$comps
  trts <- x$trts
  #
  combs <- trts[!(trts %in% comps)]
  #
  if (!is.null(x$inactive))
    combs <- combs[!(combs %in% x$inactive)]
  
  if (!is.null(n)) {
    if (!is_wholenumber(n))
      stop("Argument 'n' must be a whole number.")
    sel <-
      unlist(lapply(strsplit(combs, x$sep.comps, fixed = TRUE), length)) == n
    combs <- combs[sel]
  }
  #
  combs
}

optcircle <- function(x, start.layout = "eigen") {
  
  n <- x$n
  n1 <- netgraph(x, start.layout = start.layout,
                 iterate = TRUE, figure = FALSE)
  
  xpos <- n1$nodes$xpos
  ypos <- n1$nodes$ypos
  ##
  slope <- ypos / xpos
  
  angle <- rep(0, n)
  ##
  for (i in 1:n) {
    if (xpos[i] >= 0 & ypos[i] >= 0)
      angle[i] <- atan(slope[i])
    else if (xpos[i] >= 0 & ypos[i] <  0)
      angle[i] <- atan(slope[i]) + 2 * pi 
    else if (xpos[i] < 0)
      angle[i] <- atan(slope[i]) + pi 
  }
  ##
  seq <- order(angle)
  ##
  ## Permutation matrix P
  ##
  P <- diag(n)
  ##
  for (i in 1:n) {
    k <- seq[i]
    if (k != i) {
      P[i, i] <- P[k, k] <- 0
      P[i, k] <- 1
    }
  }
  
  #
  # Distance matrix D of n-circle, here used to penalise distances
  #
  if (n > 676)
    stop("Maximum number of 676 treatments", call. = FALSE)
  else if (n > 26) {
    trts <- paste0(rep(LETTERS[seq_len(n %/% 26)], rep(26, n %/% 26)), LETTERS)
    if (n %% 26)
      trts <- c(trts,
                paste0(LETTERS[max(n %/% 26) + 1], LETTERS[seq_len(n %% 26)]))
  }
  else
    trts <- LETTERS[1:n]
  #
  D <- t(P) %*% netconnection(trts, trts[c(2:n, 1)])$D.matrix %*% P
  # A.matrix is the adjacency matrix, edges penalised
  A.matrix <- sign(x$A.matrix) * D
  # Weighted distance sum
  distsum <- sum(A.matrix) / 2
  
  res <- list(xpos = xpos, ypos = ypos, seq = seq,
              A.matrix = A.matrix, distsum = distsum,
              start.layout = start.layout, x = x)
  ##
  res
}

#' Print method for objects of class netimpact
#' 
#' @description
#' Print method for objects of class \code{netimpact}.
#' 
#' @param x An object of class \code{netimpact}.
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param digits Minimal number of significant digits.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param nchar.studlab A numeric defining the minimum number of
#'   characters used to create unique study labels.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param legend.studlab A logical indicating whether a legend should
#'   be printed for abbreviated study labels.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netimpact}}
#' 
#' @keywords print
#' 
#' @examples
#' # Examples: example(netimpact)
#' 
#' @method print netimpact
#' @export

print.netimpact <- function(x,
                            common = x$x$common,
                            random = x$x$random,
                            #
                            digits = gs("digits.prop"),
                            #
                            nchar.trts = x$nchar.trts,
                            nchar.studlab = x$nchar.studlab,
                            #
                            details.methods = gs("details"),
                            legend = gs("legend"),
                            legend.studlab = TRUE,
                            warn.deprecated = gs("warn.deprecated"),
                            ##
                            ...) {
  
  ##
  ##
  ## (1) Check for netimpact object and upgrade object
  ##
  ##
  chkclass(x, "netimpact")
  x <- updateversion(x)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chknumeric(digits, min = 0, length = 1)
  chklogical(details.methods)
  chklogical(legend)
  chklogical(legend.studlab)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                      warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (3) Generate (abbreviated) column names
  ##
  ##
  sep.trts <- x$x$sep.trts
  ##
  cn <- colnames(x$impact.common)
  mat <- matrix(unlist(strsplit(cn, split = sep.trts)),
                ncol = 2, byrow = TRUE)
  treat1.long <- mat[, 1]
  treat2.long <- mat[, 2]
  ##
  trts <- x$x$trts
  ##
  treat1 <- as.character(factor(treat1.long, levels = trts,
                                labels = treats(trts, nchar.trts)))
  treat2 <- as.character(factor(treat2.long, levels = trts,
                                labels = treats(trts, nchar.trts)))
  
  
  ##
  ##
  ## (4) Print results
  ##
  ##
  if (common) {
    cat("Common effects model: \n\n")
    impact.common <- formatN(x$impact.common, digits = digits)
    #
    studlab <- rownames(impact.common)
    studlab.abbr <- treats(studlab, nchar.studlab)
    rownames(impact.common) <- studlab.abbr
    #
    colnames(impact.common) <- paste(treat1, treat2, sep = sep.trts)
    ##
    prmatrix(impact.common, quote = FALSE, right = TRUE)
    if (random)
      cat("\n")
  }
  ##
  ## Print results for random effects model
  ##
  if (random) {
    cat("Random effects model: \n\n")
    impact.random <- formatN(x$impact.random, digits = digits)
    #
    studlab <- rownames(impact.random)
    studlab.abbr <- treats(studlab, nchar.studlab)
    rownames(impact.random) <- studlab.abbr
    #
    colnames(impact.random) <- paste(treat1, treat2, sep = sep.trts)
    ##
    prmatrix(impact.random, quote = FALSE, right = TRUE)
  }
  #
  # Print details of network meta-analysis methods
  #
  if (details.methods & (common | random))
    cat(textmeth(x, random))
  ##
  ## Add legend with abbreviated treatment labels
  ##
  if (legend & (common | random))
    legendabbr(trts, treats(trts, nchar.trts), legend)
  #
  # Add legend with abbreviated study labels
  #
  if (legend.studlab & (common | random))
    legendabbr(studlab, studlab.abbr, TRUE, "Study label",
               if (legend) "\n" else "\nLegend:\n")
  
  
  invisible(NULL)
}

#' Abbreviate treatment names
#' 
#' @description
#' Auxiliary functions to create uniquely abbreviated treatment names.
#' 
#' @details
#' These auxiliary functions can be used to create uniquely
#' abbreviated treatment names (and are used internally in several R
#' functions for this purpose).
#' 
#' In order to construct uniquely abbreviated treatment names,
#' \code{treats} uses \code{\link{substring}} to extract the first
#' \code{nchar.trts} characters. If these abbreviated treatment names
#' are not unique, \code{\link{abbreviate}} with argument
#' \code{minlength = nchar.trts} is used.
#'
#' In order to construct comparisons with uniquely abbreviated
#' treatment names, \code{comps} calls \code{treats} internally.
#' 
#' @aliases treats comps
#' 
#' @param x A vector with treatment or comparison names or a matrix
#'   with treatment or comparison names as row and / or column names.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param row A logical indicating whether row or column names should
#'   be used (only considered if argument \code{x} is a matrix).
#' @param trts A character vector with treatment names.
#' @param sep.trts A character used in comparison names as separator
#'   between treatment labels.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{print.netmeta}},
#'   \code{\link{print.summary.netmeta}}
#' 
#' @examples
#' \donttest{
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' nma <- netmeta(pw)
#' 
#' # Full treatment names
#' #
#' nma$trts
#' 
#' # Treatment names with four characters
#' #
#' treats(nma$trts, nchar.trts = 4)
#' 
#' # With two characters
#' #
#' treats(nma$trts, nchar.trts = 2)
#' 
#' # With one character (if possible)
#' #
#' treats(nma$trts, nchar.trts = 1)
#'
#' # Full comparison names
#' #
#' nma$comparisons
#' 
#' # Abbreviated comparison names
#' #
#' with(nma, comps(comparisons, trts, sep.trts, nchar = 4))
#' }
#' 
#' @export treats

treats <- function(x, nchar.trts = 8, row = TRUE) {
  
  chknumeric(nchar.trts, min = 1, length = 1)
  chklogical(row)
  
  if (is.matrix(x)) {
    if (row)
      trts <- rownames(x)
    else
      trts <- colnames(x)
  }
  else
    trts <- x
  ##
  ## Default: first 'nchar.trts' character of treatment names
  ##
  res <- substring(trts, 1, nchar.trts)
  ##
  ## Use abbreviated treatment names if necessary
  ##
  if (length(unique(res)) != length(unique(trts)))
    res <- as.character(abbreviate(trts, nchar.trts))
  ##
  res
}


#' @rdname treats
#' 
#' @export comps

comps <- function(x, trts, sep.trts, nchar.trts = 8, row = TRUE) {
  
  chknumeric(nchar.trts, min = 1, length = 1)
  chklogical(row)
  chkchar(sep.trts)
  ##
  trts.abbr <- treats(trts, nchar.trts)  
  
  if (is.matrix(x)) {
    if (row)
      comps <- rownames(x)
    else
      comps <- colnames(x)
  }
  else
    comps <- x
  
  trts.list <- compsplit(comps, sep.trts)
  trts.list.c <- lapply(trts.list, charfac, levels = trts, labels = trts.abbr)
  ##
  res <- sapply(trts.list.c, paste, collapse = sep.trts)

  res
}

# Copied from the R package hasseDiagram, version 0.2.0
# Original author: Krzysztof Ciomek <k.ciomek@gmail.com>
# Source: https://github.com/kciomek/hasseDiagram
# License: MIT
# 
# R functions are included in accordance with the terms of the
# MIT license and have been modified to work with the Hasse matrix
# created with netposet().
# 
# Arguments:
# - M          (logical Hasse matrix)
# - parameters (list with named elements)
# 
# Named elements in parameters:
# - newpage   (whether to call \code{grid.newpage()} before drawing)
# - shape     (shape of diagram nodes)
# - col.lines (edge colour)
# - col.nodes (node colour)

hasse_hasseDiagram <- function(M, parameters = list()) {
  
  stopifnot(is.matrix(M))
  stopifnot(nrow(M) > 0)
  stopifnot(nrow(M) == ncol(M))
  #
  stopifnot(is.list(parameters))
  
  # Set defaults
  #
  parameters$newpage <- replaceNULL(parameters$newpage, TRUE)
  #
  parameters$cluster <- TRUE
  parameters$clusterMerge <- FALSE
  parameters$clusterNonAdjacent <- FALSE
  parameters$arrow <- "forward"
  #
  parameters$shape <- replaceNULL(parameters$shape, "roundrect")
  parameters$col.lines <- replaceNULL(parameters$col.lines, "black")
  parameters$col.nodes <- replaceNULL(parameters$col.nodes, "black")
  parameters$lwd <- replaceNULL(parameters$lwd, 1)
  #
  # Node margins as a list with four numerical items:
  # - "tb" and "rl" for top-bottom and right-left margin,
  # - "otb" and "orl" for outer margin when multiple labels are present.
  #
  parameters$margin <- list()
  parameters$margin$rl <- parameters$margin$tb <- 0.125
  parameters$margin$orl <- parameters$margin$otb <- 0.08
  
  # Convert labels to list with named elements
  #
  labels <- as.list(rownames(M))
  names(labels) <- rownames(M)
  
  # Remove self-loops 
  #
  nrNodes <- nrow(M)
  #
  for (i in seq_len(nrNodes)) {
    M[i, i] <- FALSE
  }
  
  # Cluster
  #
  groups <- extractGroups(M, parameters$clusterNonAdjacent)
  toRemove <- c()
  
  for (group in groups) {
    for (i in group) {
      for (j in group) {
        M[i, j] <- FALSE
      }
    }
    
    if (parameters$cluster) {
      first <- group[1]
      rest <- group[-1]
      
      rownames(M)[first] <-
        colnames(M)[first] <-
        names(labels)[first] <- paste(rownames(M)[group], collapse = "")
      
      toRemove <- c(toRemove, rest)
      labels[[first]] <- c(unlist(labels[group]))
    }
  }
  
  if (!is.null(toRemove)) {
    M <- M[-toRemove, -toRemove]
    labels <- labels[-toRemove]
  }
  #
  nrNodes <- nrow(M)
  
  # Detect cycles
  #
  tmpM <- M
  toVisit <- which(sapply(1:nrow(M),
                          function(x) {length(which(tmpM[, x])) == 0}) == TRUE)
  
  while (length(toVisit) > 0) {
    n <- toVisit[1]
    toVisit <- toVisit[-1]
    
    for (m in which(tmpM[n, ] == TRUE)) {
      tmpM[n, m] <- FALSE
      
      if (length(which(tmpM[, m])) == 0) {
        toVisit <- c(toVisit, m)
      }
    }
  }
  
  notRemovedEdges <- which (tmpM == TRUE, arr.ind = TRUE)
  
  if (nrow(notRemovedEdges) > 0) {
    stop(paste("Cycle detected. Check edges: ",
               paste(sapply(seq_len(nrow(notRemovedEdges)),
                            function(x) {
                              paste(rownames(notRemovedEdges)[
                                notRemovedEdges[x, ]], collapse = "-")} ),
                     collapse = ", "),
               sep = ""))
  }
  
  # Perform transitive reduction
  #
  for (source in seq_len(nrNodes)) {
    stack <- which(M[source, ])
    visited <- rep(F, nrNodes)
    visited[stack] <- T
    
    while (length(stack) > 0) {
      element <- stack[1]
      stack <- stack[-1]
      
      children <- which(M[element, ])
      for (child in children) {
        M[source, child] = FALSE
        if (!visited[child]) {
          stack <- c(child, stack)
        }
      }
    }
  }
  
  # Calculate node levels
  #
  ranks <- rep(1, nrNodes)
  queue <- which(sapply(1:nrow(M),
                        function(x) {length(which(M[, x])) == 0}) == TRUE)
  distances <- rep(1, length(queue))
  
  while (length(queue) > 0) {
    element <- queue[1]
    queue <- queue[-1]
    dist <- distances[1]
    distances <- distances[-1]
    children <- which(M[element, ])
    
    for (i in seq_len(length(children))) {
      idx <- which(queue == children[i])
      
      if (length(idx) == 0) {
        ranks[children[i]] <- dist + 1
        queue <- c(queue, children[i])
        distances <- c(distances, dist + 1)
      } else {
        distances[idx] <- max(distances[idx], dist + 1)
        ranks[children[i]] <- max(ranks[children[i]], dist + 1)
      }
    }
  }
  
  # Build graph
  #
  graph <- as(graph::graphAM(M, "directed"), "graphNEL")
  
  nAttrs <- list()
  nAttrs$width <- sapply(labels, function(x) { nWi(x, parameters) })
  nAttrs$height <- sapply(labels, function(x) { nHi(x, parameters) })
  nAttrs$fixedsize <- rep(TRUE, nrNodes)
  nAttrs <- lapply(nAttrs, function(x) { names(x) <- rownames(M); x})
  
  subGList <- list()
  
  for (i in seq_len(max(ranks))) {
    subGList[[length(subGList) + 1]] <-
      list(graph =
             graph::subGraph(rownames(M)[which(ranks == i)], graph),
           cluster = FALSE)
  }
  
  ragraph <-
    Rgraphviz::agopen(graph,
                      name = "graph",
                      subGList = subGList,
                      attrs = list(node = list(shape = "box"),
                                   graph = list(rank = "same", rankdir = "TB")),
                      nodeAttrs = nAttrs)
  # Draw graph
  #
  if (parameters$newpage) {
    grid.newpage()
  }
  hGrob <- hasseGrob(ragraph, labels, parameters)
  grid.draw(hGrob)
  #return (hGrob)
}

isGroup <- function(data, i, j, groupNonAdjacent) {
  if ((data[i, j] == TRUE && data[j, i] == TRUE) || groupNonAdjacent == TRUE) {
    iParents <- data[, i]
    jParents <- data[, j]
    iChildren <- data[i, ]
    jChildren <- data[j, ]
    
    iParents[j] <- FALSE
    jParents[i] <- FALSE
    iChildren[j] <- FALSE
    jChildren[i] <- FALSE
    
    if (all(iParents == jParents) && all(iChildren == jChildren)) {
      return (TRUE)
    }
  }
  
  return (FALSE)
}

extractGroups <- function(data, groupNonAdjacent) {
  result <- list()
  itemGroup <- seq_len(nrow(data))
  
  for (i in seq_len(nrow(data))) {
    for (j in seq_len(nrow(data))) {
      if (isGroup(data, i, j, groupNonAdjacent)) {
        iGroup <- which(itemGroup == itemGroup[i])
        mergable <- TRUE
        
        for (k in iGroup) {
          if (k != i) {
            if (!isGroup(data, j, k, groupNonAdjacent)) {
              mergable <- FALSE
              break
            }
          }
        }
        
        if (mergable) {
          itemGroup[j] <- itemGroup[i]
        }
      }
    }
  }
  
  for (g in unique(itemGroup)) {
    items <- which(itemGroup == g)
    if (length(items) > 1) {
      result[[length(result) + 1]] <- items
    }
  }
  
  return (result)
}

# Node height by labels (in inches)
#
nHi <- function(labels, parameters) {
  result <- unit(1, "lines") + unit(parameters$margin$tb * 2, "inch")
  if (length(labels) > 1 && parameters$clusterMerge == FALSE)
    result <- result + unit(parameters$margin$otb * 2, "inch")
  
  return (convertY(result, "inches", TRUE))
}

# Node width by labels (in inches)
#
nWi <- function(labels, parameters) {
  result <- unit(0, "inch")
  for (label in labels)
    result <-
      result + stringWidth(label) + unit(parameters$margin$rl * 2, "inch")
  if (length(labels) > 1 && parameters$clusterMerge == FALSE)
    result <-
      result + (length(labels) + 1) * unit(parameters$margin$orl, "inch")
  
  return (convertX(result, "inches", TRUE))
}

drawNode <- function(x, y, width, height, labels, parameters, isInner=FALSE) {
  vp <- viewport(x,
                 y,
                 width,
                 height,
                 xscale = c(0, nWi(labels, parameters)),
                 yscale = c(0, nHi(labels, parameters)))
  pushViewport(vp)
  
  if (parameters$shape != "none" &&
      (isInner == FALSE || parameters$clusterMerge == FALSE)) {
    gp <- gpar(col = parameters$col.nodes, lwd = parameters$lwd)
    
    if (parameters$shape == "rect")
      grid.rect(gp = gp)
    else if (parameters$shape == "roundrect")
      grid.roundrect(gp = gp)
    else
      stop(paste("Unsupported node shape '", parameters$shape, "'.", sep = ""))
  }
  
  grid.clip()
  
  if (length(labels) == 1) {
    cex <- min(1.0 / (convertWidth(stringWidth(labels) +
                                     unit(parameters$margin$rl * 2, "inch"), 
                                   "npc", TRUE)),
               1.0 / (convertHeight(unit(1, "lines") +
                                      unit(parameters$margin$tb * 2, "inch"),
                                    "npc", TRUE)))
    
    grid.text(labels[1], gp = gpar(cex = cex))
  }
  else {
    xCenter <- unit(ifelse(parameters$clusterMerge == FALSE,
                           parameters$margin$orl, 0.0), "native")
    yCenter <- unit(0.5, "npc")
    
    for (label in labels) {
      drawNode(xCenter + unit(nWi(label, parameters), "native") * 0.5,
               yCenter,
               unit(nWi(label, parameters), "native"),
               unit(nHi(label, parameters), "native"),
               label,
               parameters,
               TRUE)
      xCenter <- xCenter + unit(nWi(label, parameters), "native") 
      if (parameters$clusterMerge == FALSE)
        xCenter <- xCenter + unit(parameters$margin$orl, "native")
    }
  }
  
  popViewport()
}


hasseGrob <- function(graph, labels, parameters) {
  grob(graph = graph, labels = labels,
       parameters = parameters, cl = "hasseGrob")
}


#' @method drawDetails hasseGrob
#' @export

drawDetails.hasseGrob <- function(x, recording) {
  g <- x$graph
  ur <- Rgraphviz::upRight(Rgraphviz::boundBox(g))
  bl <- Rgraphviz::botLeft(Rgraphviz::boundBox(g))
  
  vp <- viewport(width = unit(0.96, "npc"),
                 height = unit(0.96, "npc"),
                 xscale = c(Rgraphviz::getX(bl), Rgraphviz::getX(ur)),
                 yscale = c(Rgraphviz::getY(bl), Rgraphviz::getY(ur)))
  
  pushViewport(vp)
    
  # Draw edges before nodes
  #
  dir <- x$parameters$arrow
  gp <- gpar(col = x$parameters$col.lines, lwd = x$parameters$lwd)
  
  for (edge in Rgraphviz::AgEdge(g)) {
    nrLines <- length(edge@splines)
    
    for (i in seq_len(nrLines)) {
      arrow <- NULL
      arrowEnds <- NULL
      
      if (dir == "forward" && i == nrLines) {
        arrowEnds = "last"
      }
      else if (dir == "backward" && i == 1) {
        arrowEnds = "first"
      }
      else if (dir == "both") {
        if (nrLines == 1)
          arrowEnds = "both"
        else if (i == 1)
          arrowEnds = "first"
        else if (i == nrLines)
          arrowEnds = "last"
      }
      
      if (!is.null(arrowEnds)) {
        arrow <- arrow(angle = 30,
                       length = min(unit(4, "mm"), unit(0.02, "npc")),
                       ends = arrowEnds,
                       type = "open")
      }
      
      bp <- Rgraphviz::bezierPoints(edge@splines[[i]])
      grid.lines(bp[, 1], bp[, 2], default.units = "native",
                 arrow = arrow, gp = gp)
    }
  }
  
  # Draw nodes
  #
  for (agNode in Rgraphviz::AgNode(g)) {
    center <- Rgraphviz::getNodeCenter(agNode)
    centerX <- unit(Rgraphviz::getX(center), "native")
    centerY <- unit(Rgraphviz::getY(center), "native")
    width <- unit(Rgraphviz::getNodeRW(agNode) + Rgraphviz::getNodeLW(agNode),
                  "native")
    height <- unit(Rgraphviz::getNodeHeight(agNode), "native")
    
    drawNode(centerX, centerY, width, height,
             unlist(x$labels[agNode@name]), x$parameters)
  }
    
  popViewport()
}

nma_additive <- function(TE, W,
                         treat1, treat2,
                         studlab,
                         level,
                         X, C.matrix, B.matrix,
                         df.Q.additive, n, sep.trts) {
  
  
  m <- length(TE)
  #
  # Drop Matrix attributes
  #
  W <- as.matrix(W)
  class(W) <- "matrix"
  #
  # Laplacian matrix and pseudoinverse of L
  #
  L <- t(X) %*% W %*% X
  ##
  Lplus <- ginv(L) # = Cov matrix of beta (components)
  Lplus[is_zero(Lplus)] <- 0
  colnames(Lplus) <- colnames(L)
  rownames(Lplus) <- rownames(L)
  #
  # H matrix
  #
  H <- X %*% Lplus %*% t(X) %*% W
  H[is_zero(H)] <- 0
  
  
  ##
  ## beta = effects of components
  ##
  beta <- as.vector(Lplus %*% t(X) %*% W %*% TE)
  se.beta <- sqrt(diag(Lplus))
  names(beta) <- names(se.beta)
  ##
  ## theta = estimates for combination treatments
  ##
  theta <- as.vector(C.matrix %*% beta)
  se.theta <- sqrt(diag(C.matrix %*% Lplus %*% t(C.matrix)))
  names(theta) <- names(se.theta)
  ##
  ## delta = treatment estimates for observed comparisons
  ##
  delta <- as.vector(X %*% beta) # = B.matrix %*% theta = H %*% TE
  se.delta <- unname(sqrt(diag(X %*% Lplus %*% t(X))))
  ##
  ## delta.all(.matrix) = all direct and indirect treatment estimates
  ##
  B.full <- createB(ncol = n)
  X.all <- B.full %*% C.matrix
  colnames(X.all) <- colnames(C.matrix)
  ##
  labels <- colnames(B.matrix)
  ##
  k <- 0
  lab <- vector(mode = "numeric", length = choose(n, 2))
  ##
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      k <- k + 1
      lab[k] <- paste(labels[i], labels[j], sep = sep.trts)
    }
  }
  ##
  rownames(X.all) <- lab
  #
  # Variance-covariance matrix for all comparisons
  #
  Cov <- X.all %*% Lplus %*% t(X.all)
  Cov[is_zero(Cov)] <- 0
  #
  delta.all <- as.vector(X.all %*% beta)
  se.delta.all <- sqrt(diag(X.all %*% Lplus %*% t(X.all)))
  names(delta.all) <- names(se.delta.all)
  ##
  delta.all.matrix <- se.delta.all.matrix <- matrix(0, ncol = n, nrow = n)
  ##
  k <- 0
  ##
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      k <- k + 1
      delta.all.matrix[i, j] <-  delta.all[k]
      delta.all.matrix[j, i] <- -delta.all[k]
      se.delta.all.matrix[i, j] <-
        se.delta.all.matrix[j, i] <- se.delta.all[k]
    }
  }
  ##
  colnames(delta.all.matrix) <- rownames(delta.all.matrix) <-
    colnames(se.delta.all.matrix) <- rownames(se.delta.all.matrix) <- labels
  
  
  comparisons <- c(list(studlab = studlab, treat1 = treat1, treat2 = treat2),
                   ci(delta, se.delta, level = level))
  ##
  all.comparisons <- ci(delta.all.matrix, se.delta.all.matrix,
                        level = level)
  ##
  components <- ci(beta, se.beta, level = level)
  ##
  combinations <- ci(theta, se.theta, level = level)
  ##
  ## Test of total heterogeneity / inconsistency:
  ##
  Q.additive <- as.vector(t(delta - TE) %*% W %*% (delta - TE))
  ##
  if (is.na(df.Q.additive) | df.Q.additive == 0)
    pval.Q.additive <- NA
  else
    pval.Q.additive <- 1 - pchisq(Q.additive, df.Q.additive)
  ##
  ## Heterogeneity variance
  ##
  I <- diag(m)
  E <- matrix(0, nrow = m, ncol = m)
  for (i in 1:m)
    for (j in 1:m)
      E[i, j] <- as.numeric(studlab[i] ==  studlab[j])
  ##
  if (df.Q.additive == 0) {
    tau2 <- NA
    tau <- NA
    I2 <- NA
    lower.I2 <- NA
    upper.I2 <- NA
  }
  else {
    tau2 <- max(0, (Q.additive - df.Q.additive) /
                   sum(diag((I - H) %*%
                            (B.matrix %*% t(B.matrix) * E / 2) %*% W)))
    tau <- sqrt(tau2)
    ci.I2 <- isquared(Q.additive, df.Q.additive, level)
    I2 <- ci.I2$TE
    lower.I2 <- ci.I2$lower
    upper.I2 <- ci.I2$upper
  }
  
  H.matrix <- H
  #
  rownames(H.matrix) <- colnames(H.matrix) <- studlab
  #
  res <- list(comparisons = comparisons,
              all.comparisons = all.comparisons,
              components = components,
              combinations = combinations,
              ##
              Q.additive = Q.additive,
              df.Q.additive = df.Q.additive,
              pval.Q.additive = pval.Q.additive,
              ##
              tau = tau,
              I2 = I2, lower.I2 = lower.I2, upper.I2 = upper.I2,
              ##
              L.matrix = L,
              Lplus.matrix = Lplus,
              H.matrix = H.matrix,
              #
              Cov = Cov
              )
  
  
  res
}

#' Forest plot for complex interventions in component network
#' meta-analysis
#' 
#' @description
#' Draws a forest plot in the active graphics window (using grid
#' graphics system).
#'
#' @aliases forest.netcomplex plot.netcomplex
#' 
#' @param x An object of class \code{netcomplex}.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param leftcols A character vector specifying (additional) columns
#'   to be plotted on the left side of the forest plot or a logical
#'   value (see \code{\link[meta]{forest.meta}} help page for details).
#' @param leftlabs A character vector specifying labels for
#'   (additional) columns on left side of the forest plot (see
#'   \code{\link[meta]{forest.meta}} help page for details).
#' @param rightcols A character vector specifying (additional) columns
#'   to be plotted on the right side of the forest plot or a logical
#'   value (see \code{\link[meta]{forest.meta}} help page for details).
#' @param rightlabs A character vector specifying labels for
#'   (additional) columns on right side of the forest plot (see
#'   \code{\link[meta]{forest.meta}} help page for details).
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique names for components.
#' @param digits Minimal number of significant digits for treatment
#'   effects and confidence intervals, see \code{print.default}.
#' @param digits.stat Minimal number of significant digits for tests
#'   of overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall effects, see \code{print.default}.
#' @param smlab A label printed at top of figure. By default, text
#'   indicating either common or random effects model is printed.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param lab.NA A character string to label missing values.
#' @param equal.size A logical indicating whether all squares should
#'   be of equal size. Otherwise, the square size is proportional to
#'   the precision of estimates.
#' @param \dots Additional arguments for \code{\link[meta]{forest.meta}}
#'   function.
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window. For more information see help page of
#' \code{\link[meta]{forest.meta}} function.
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomplex}}, \code{\link{netcomb}},
#'   \code{\link{discomb}}, \code{\link[meta]{forest.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' \dontrun{
#' # Only consider studies including Face-to-face PST (to reduce
#' # runtime of example)
#' #
#' face <- subset(Linde2016, id %in% c(16, 24, 49, 118))
#' 
#' # Use pairwise() to transform data to comparison-based format
#' #
#' pw <- pairwise(treat = int,
#'   event = resp, n = n, studlab = paste(author, year),
#'   data = face, reference = "plac", sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma <- netmeta(pw, common = FALSE)
#' 
#' # Additive model for treatment components (with placebo as inactive
#' # treatment)
#' #
#' cnma <- netcomb(nma, inactive = "placebo")
#'
#' # Some complex interventions
#' #
#' ints <- c("F + TCA", "F + Plac", "SSRI + Plac + TCA")
#' netcomplex(cnma, ints)
#' #
#' forest(netcomplex(cnma, ints))
#' forest(netcomplex(cnma, ints), nchar.comps = 4)
#'
#' # Component effects
#' #
#' forest(netcomplex(cnma, cnma$comps))
#' }
#' 
#' @method forest netcomplex
#' @export

forest.netcomplex <- function(x,
                              pooled = ifelse(x$random, "random", "common"),
                              leftcols = "studlab",
                              leftlabs = NULL,
                              rightcols =
                                c("effect", "ci", "statistic", "pval"),
                              rightlabs = c(NA, NA, "z", "p-value"),
                              nchar.comps = x$nchar.trts,
                              digits = gs("digits.forest"),
                              digits.stat = gs("digits.stat"),
                              digits.pval = gs("digits.pval"),
                              smlab = NULL,
                              backtransf = x$backtransf,
                              lab.NA = gs("lab.NA"),
                              equal.size = gs("equal.size"),
                              ...) {
  
  
  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netcomplex")
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  ##
  nchar.comps <- replaceNULL(nchar.comps, 666)
  chknumeric(nchar.comps, min = 1, length = 1)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 0, length = 1)
  ##
  chklogical(backtransf)
  chkchar(lab.NA)
  
  
  ##
  ##
  ## (2) Extract results for common and random effects model
  ##
  ##
  if (pooled == "common") {
    TE   <- x$Comb.common
    seTE <- x$seComb.common
    stat <- x$statistic.Comb.common
    pval <- x$pval.Comb.common
    ##
    if (is.null(smlab))
      smlab <- "Common Effects Model"
  }
  ##
  else if (pooled == "random") {
    TE   <- x$Comb.random
    seTE <- x$seComb.random
    stat <- x$statistic.Comb.random
    pval <- x$pval.Comb.random
    ##
    if (is.null(smlab))
      smlab <- "Random Effects Model"
  }
  ##
  statistic <- formatN(stat, digits = digits.stat, text.NA = lab.NA)
  pval <- formatPT(pval, digits = digits.pval, lab.NA = lab.NA)
  ##
  ## Abbreviated component labels
  ##
  n.complex <- length(x$complex)
  complex <- rep("", n.complex)
  ##
  comps <- c(x$comps, x$inactive)
  comps.abbr <- treats(comps, nchar.comps)
  ##
  for (i in seq_len(n.complex))
    complex[i] <- compos(x$complex[i], comps, comps.abbr,
                         x$x$sep.comps, x$add[i] == " ")
  ##
  dat <- data.frame(complex, TE, seTE, statistic, pval)
  ##
  if (is.null(leftlabs))
    if (leftcols == "studlab" &&
        !any(grepl(x$x$sep.comps, complex, fixed = TRUE)))
      leftlabs <- "Component"
    else
      leftlabs <- "Complex intervention"
  
  
  ##
  ##
  ## (3) Generate forest plot
  ##
  ##
  m1 <-
    suppressWarnings(metagen(TE, seTE, data = dat, sm = x$x$sm,
                             studlab = complex, backtransf = backtransf,
                             level = x$level,
                             method.tau = "DL", method.tau.ci = "",
                             warn = FALSE))
  ##
  forest(m1,
         digits = digits,
         common = FALSE, random = FALSE,
         overall = FALSE, hetstat = FALSE, test.subgroup = FALSE,
         leftcols = leftcols,
         leftlabs = leftlabs,
         rightcols = rightcols,
         rightlabs = rightlabs,
         smlab = smlab,
         lab.NA = lab.NA,
         weight.study = if (equal.size) "same" else pooled,
         just.addcols = "right",
         ...)
  
  
  invisible(NULL)
}


#' @rdname forest.netcomplex
#' @method plot netcomplex
#' @export

plot.netcomplex <- function(x, ...)
  forest(x, ...)

contribution.matrix <- function(x, method, model, hatmatrix.F1000,
                                study, path, verbose = FALSE) {
  if (verbose)
    cat("Calculate network contributions (", model, " effects model):\n",
        sep = "")
  ##
  if (method == "randomwalk")
    return(contribution.matrix.davies(x, model, verbose = verbose))
  else if (method == "shortestpath")
    return(contribution.matrix.tpapak(x, model, hatmatrix.F1000,
                                      study = study, path = path,
                                      verbose = verbose))
  else if (method == "cccp")
    return(contribution.matrix.ruecker.cccp(x, model, verbose = verbose))
  else if (method == "pseudoinverse")
    return(contribution.matrix.ruecker.pseudoinv(x, model, verbose = verbose))
  else
    return(NULL)
}


contribution.matrix.tpapak <- function(x, model, hatmatrix.F1000,
                                       study, path, verbose = FALSE
                                       ) {
  
  chkclass(x, "netmeta")
  model <- setchar(model, c("common", "random"))
  chklogical(hatmatrix.F1000)
  chklogical(verbose)
  ##
  old <- hatmatrix.F1000
  
  
  ##
  ## Auxiliary R functions
  ##
  split <- function(dir)
    strsplit(dir, x$sep.trts)
  ##
  ## comparisonToEdge <- function(comp) unlist (split(comp))
  ##
  setWeights <- function(g, comparison, conMat)
    set_edge_attr(g, "weight", value = rep(1, dims[2]))
  ##
  getFlow <- function(g, edge)
    E(g)[edge]$flow
  ##
  sv <- function(comparison)
    split(comparison)[[1]][1][1]
  ##
  tv <- function(comparison)
    split(comparison)[[1]][2][1]
  ##
  initRowGraph <- function(x, comparison) {
    dedgeList <-
      lapply(seq_len(length(directs)),
             function(comp) {
               if (x[comparison, comp] > 0)
                 return(c(sv(directs[comp]), tv(directs[comp])))
               else
                 return(c(tv(directs[comp]), sv(directs[comp])))
             })
    ##
    dedgeList <- matrix(unlist(dedgeList), ncol = 2, byrow = TRUE)
    ##
    flows <- abs(x[comparison, ])
    dg <- graph_from_edgelist(dedgeList, directed = TRUE)
    E(dg)[]$weight <- rep(0, dims[2])
    E(dg)[]$flow <- abs(x[comparison, ])
    V(dg)[]$label <- V(dg)[]$name
    resg <- set_edge_attr(dg, 'label', value = seq_len(gsize(dg)))
    ##
    resg
  }
  ##
  reducePath <- function(g, comparison, spl) {
    pl <- length(spl[[1]])
    #
    splE <- lapply(spl[[1]], function(e) E(g)[e[]])
    flow <- min(unlist(lapply(splE, function(e) e$flow[])))
    #
    pathNames <-
      unlist(
        lapply(
          splE,
          function(e)
            paste(tail_of(g, e)$name, head_of(g, e)$name, sep = "->"))) %>%
      paste(collapse = ",")
    #
    pathContribution <<-
      rbind(pathContribution,
            data.frame(comparison = comparison,
                       path = pathNames,
                       contribution = flow))
    #
    gg <- Reduce(function(g, e) {
      elabel <- e$label
      pfl <- e$flow[]
      g <- set_edge_attr(g, "flow", e, pfl-flow)
      cw <- e$weight[] + (flow[1] / pl) 
      weights[comparison, elabel] <<- cw
      #
      set_edge_attr(g, "weight", e, cw)
      },
      splE, g)
    #
    emptyEdges <-
      Reduce(
        function(removedEdges, e) {
          e <- E(gg)[e[]]
          if (e$flow[[1]][[1]] == 0)
            removedEdges <- c(removedEdges, e)
          #
          removedEdges
          },
      splE, c())
    #
    delete_edges(gg, emptyEdges)
  }
  ##
  reduceGraph <- function(g, comparison, verbose, is.tictoc) {
    if (verbose)
      cat("- ", comparison, "\n", sep = "")
    ##
    if (verbose & is.tictoc)
      tictoc::tic()
    ##
    getshortest <- function(g, comparison) {
      getShortest <- function()
        get.shortest.paths(g, sv(comparison), tv(comparison),
                           mode = "out", output = "epath",
                           weights = NA)$epath
      ##
      res <- suppressWarnings(getShortest())
      #
      res
    }
    spath <- getshortest(g, comparison)
    while (length(unlist(spath)) > 0) {
      g <- reducePath(g, comparison, spath)
      spath <- getshortest(g, comparison)
    }
    ##
    if (verbose & is.tictoc)
      tictoc::toc()
    ##
    g
  }
  
  
  if (old)
    H <- hatmatrix.F1000(x, model)
  else
    H <- hatmatrix.aggr(x, model, "long")
  ##
  directs <- colnames(H)
  
  
  list.direct <- unlist(lapply(lapply(directs, split), unlist))
  list.edge <- matrix(list.direct, ncol = 2, byrow = TRUE)
  ##
  g <- graph_from_edgelist(list.edge , directed = FALSE)
  g <- set_vertex_attr(g, 'label', value = V(g))
  g <- set_edge_attr(g, 'label', value = E(g))
  
  
  is.tictoc <- is_installed_package("tictoc", stop = FALSE)
  #
  dims <- dim(H)
  ##
  contribution <- rep(0, dims[2])
  names(contribution) <- seq_len(dims[2])
  #
  # Weights matrix
  #
  weights <- matrix(rep(0, dims[2] * dims[1]),
                    nrow = dims[1], ncol = dims[2], byrow = TRUE)
  rownames(weights) <- rownames(H)
  colnames(weights) <- seq_len(dims[2])
  #
  # Rows of comparison matrix
  #
  comparisons <- unlist(lapply(rownames(H), unlist))
  #
  # Path contributions
  #
  pathContribution <- data.frame()
  #
  # Both pathContribution and weights are changed internally in reduceGraph()
  # using "<<-"
  #
  lapply(comparisons,
         function(comp)
           reduceGraph(initRowGraph(H, comp), comp, verbose, is.tictoc))
  #
  if (!path)
    pathContribution <- NULL
  #
  colnames(weights) <- directs
  #
  weights[is_zero(weights)] <- 0
  #
  attr(weights, "model") <- model
  attr(weights, "hatmatrix.F1000") <- old
  #
  # Study contributions
  #
  if (study) {
    studyContribution <- 
      lapply(comparisons,
             function(comp) getStudyContribution(x, weights, comp, model))
    #
    studyContribution <- bind_rows(studyContribution)
  }
  else
    studyContribution <- NULL
  #
  res <- list(weights = weights,
              studyContribution = studyContribution,
              pathContribution = pathContribution)
  res
}


contribution.matrix.davies <- function(x, model, verbose = FALSE) {
  
  chkclass(x, "netmeta")
  model <- setchar(model, c("common", "random"))
  chklogical(verbose)
  ##
  is.tictoc <- is_installed_package("tictoc", stop = FALSE)
  
  
  ##
  ## 'Full' aggregated hat matrix
  ##
  H.full <- hatmatrix.aggr(x, model, type = "full")
  ## Total number of possible pairwise comparisons
  comps <- rownames(H.full)
  n.comps <- length(comps)
  
  
  ##
  ## Create prop contribution matrix (square matrix)
  ##
  weights <- matrix(0, nrow = n.comps, ncol = n.comps)
  rownames(weights) <- colnames(weights) <- comps
  
  
  ##
  ## Measure times
  ##
  if (is.tictoc) {
    tictoc <- rep(NA, n.comps)
    names(tictoc) <- comps
  }
  
  
  ##
  ## Cycle through comparisons
  ##
  n <- x$n
  r <- 0
  ##
  for (t1 in seq_len(n - 1)) {
    for (t2 in (t1 + 1):n) {
      r <- r + 1
      ##
      if (is.tictoc)
        tictoc::tic()
      ##
      if (verbose)
        cat("- ",
            paste(x$trts[t1], x$trts[t2], sep = x$sep.trts),
            " (", r, "/", n.comps, ")\n",
            sep = "")
      ##
      ## For each row, t1 is the source and t2 is the sink
      ##
      ## Create a transition matrix (P, n x n) from t1 (source) to t2
      ## (sink) using H.full
      ##
      ## Create a dummy n x n matrix Q (this is a step in getting P)
      ##
      ## Assign Q[i, j] equal to the element of H.full in row r and
      ## column representing comparison ij this defines only the upper
      ## half of the matrix Q
      ##
      Q <- matrix(0, nrow = n, ncol = n)
      idx <- 0
      for (i in seq_len(n - 1))
        for (j in (i + 1):n) {
          idx <- idx + 1
          Q[i, j] <- H.full[r, idx]
        }
      ##
      ## Now use H_ij = -H_ji to define the lower half of Q
      ##
      for (i in seq_len(n))
        for (j in seq_len(n))
          if (i != j)
            Q[j, i] <- -Q[i, j]
      ##
      ## RW can only move in direction of flow therefore if H_ij < 0
      ## then set Q[i, j] = 0
      ##
      for (i in seq_len(n))
        for (j in seq_len(n))
          if (i != j & Q[i, j] <= 0)
            Q[i, j] <- 0.0
      ##
      ## Create the transition matrix by normalising the values in
      ## each row of Q
      ##
      P <- ginv(diag(rowSums(Q))) %*% Q
      P[t2, ] <- rep(0, n)
      P[t2, t2] <- 1
      ##
      P[is_zero(P, n = 1000)] <- 0
      ##
      ## Find all possible paths
      ##
      ## Define an igraph using P as adjacency matrix. The graph is
      ## directed (and acyclic) and weighted
      ##
      Pgraph <-
        graph_from_adjacency_matrix(P, "directed",
                                    weighted = TRUE, diag = FALSE)
      ##
      ## Now find all possible paths from source to sink
      ##
      ## Simple paths means no vertex is visited more than once this
      ## is true for us as the graph is acyclic
      ##
      all.paths <- all_simple_paths(Pgraph, t1, t2, mode = "out")
      ##
      ## Calculate streams
      ##
      R <- matrix(0, nrow = n, ncol = n)
      colnames(R) <- rownames(R) <- x$seq
      ##
      for (p in seq_len(length(all.paths))) {
        path.p <- all.paths[[p]]
        L.p <- length(path.p) - 1
        prob.p <- 1 / L.p
        ##
        ## Prob of taking a path.p = product of transition
        ## probabilities in each edge along the path
        ##
        for (i in seq_len(L.p))
          prob.p <- prob.p * P[path.p[i], path.p[i + 1]]
        ##
        for (i in seq_len(L.p))
          R[path.p[i], path.p[i + 1]] <-
            R[path.p[i], path.p[i + 1]] + prob.p
        ##
        for (i in seq_len(n - 1))
          for (j in (i + 1):n)
            R[i, j] <- R[j, i] <- max(R[i, j], R[j, i])
      }
      ##
      weights[r, ] <- uppertri(R)
      ##
      if (is.tictoc) {
        tictoc.r <- tictoc::toc(func.toc = NULL)
        tictoc[r] <- as.numeric(tictoc.r$toc) - as.numeric(tictoc.r$tic)
        ##
        if (verbose)
          cat(round(tictoc[r], 3), "sec elapsed\n")
      }
    }
  }
  ##
  weights[is_zero(weights)] <- 0
  ##
  weights <- weights[, apply(weights, 2, sum) > 0, drop = FALSE]
  ##
  attr(weights, "model") <- model
  
  
  res <- list(weights = weights)
  ##
  if (is.tictoc)
    res$tictoc <- tictoc
  ##
  res
}





contribution.matrix.ruecker.cccp <- function(x, model, verbose = FALSE) {
  
  chkclass(x, "netmeta")
  ##
  model <- setchar(model, c("common", "random"))
  chklogical(verbose)
  ##
  is.tictoc <- is_installed_package("tictoc", stop = FALSE)
  
  
  H.full <- hatmatrix.aggr(x, model, type = "full")
  ## Total number of possible pairwise comparisons
  comps <- rownames(H.full)
  n.comps <- nrow(H.full)
  ##
  n <- x$n
  
  
  ##
  ## Create prop contribution matrix (square matrix)
  ##
  weights <- matrix(0, nrow = n.comps, ncol = n.comps)
  rownames(weights) <- colnames(weights) <- rownames(H.full)
  
  
  ##
  ## Create phi vector
  ##
  phi <- pl <- vector("list", n.comps)
  names(phi) <- names(pl) <- comps
  
  
  ##
  ## List with Z matrices
  ##
  zlist <- vector("list", choose(n,2))
  
  
  ##
  ## Measure times
  ##
  if (is.tictoc) {
    tictoc <- rep(NA, n.comps)
    names(tictoc) <- comps
  }
  
  
  ##
  ## Cycle through comparisons
  ##
  r <- 0
  ##
  for (t1 in seq_len(n - 1)) {
    for (t2 in (t1 + 1):n) {
      r <- r + 1
      ##
      if (is.tictoc)
        tictoc::tic()
      ##
      if (verbose) {
        cat("*** ",
            paste(x$trts[t1], x$trts[t2], sep = x$sep.trts),
            " (", r, " / ", n.comps, ") ***\n",
            sep = "")
      }
      ##
      Q <- matrix(0, nrow = n, ncol = n)
      idx <- 0
      for (i in seq_len(n - 1)) {
        for (j in (i + 1):n) {
          idx <- idx + 1
          Q[i, j] <- H.full[r, idx]
        }
      }
      ##
      for (i in seq_len(n))
        for (j in seq_len(n))
          if (i != j)
            Q[j, i] <- -Q[i, j]
      ##
      for (i in seq_len(n))
        for (j in seq_len(n))
          if (i != j & Q[i, j] <= 0)
            Q[i, j] <- 0
      ##
      P <- ginv(diag(rowSums(Q))) %*% Q # Transition matrix
      P[t2, ] <- rep(0, n)
      P[t2, t2] <- 1
      P[is_zero(P, n = 1000)] <- 0
      Pgraph <-
        graph_from_adjacency_matrix(P, "directed",
                                    weighted = TRUE, diag = FALSE)
      all.paths <- all_simple_paths(Pgraph, t1, t2, mode = "out")
      ##
      Z <- matrix(0, nrow = length(all.paths), ncol = dim(H.full)[2])
      rownames(Z) <- all.paths
      colnames(Z) <- colnames(H.full)
      ##
      for (p in 1:length(all.paths)) {
        l <- length(all.paths[[p]])
        for (i in 1:(l-1)) {
          s <- 1
          v1 <- all.paths[[p]][i]
          v2 <- all.paths[[p]][i+1]
          if (v2 < v1) {
            s <- -1
            v1 <- all.paths[[p]][i+1]
            v2 <- all.paths[[p]][i]
          }
          Z[p, n * (v1 - 1) - choose(v1 + 1, 2) + v2] <- s
        }
      }
      ##
      zlist[[r]] <- Z
      ##
      ## There is an exact least squares (L2) solution for 
      ## phi %*% Z = H.full[r, ]
      ## given by
      ## phi0 = H.full[r, ] %*% ginv(Z)
      ##
      ## The L1 solution is given by this idea (h = H.full[r, ]):
      ## If phi %*% Z = h is solvable, the full set of solutions is given by
      ##    phi = h %*% ginv(Z) + w %*% (I - Z %*% ginv(Z))
      ##        = phi0 + w %*% (I - Z %*% ginv(Z))
      ## Minimising |phi| means finding w such that 
      ##       |phi0 + w %*% (I - Z %*% ginv(Z))| is minimised
      ## Thus the problem is to minimise |phi0 + w %*% A| where
      ##       A = (I - Z %*% ginv(Z)) and phi0 = h %*% ginv(Z)
      ## If a solution w is found, the solution for phi is given by 
      ##       phi = phi0 + w %*% A
      ##
      ## Find LS solution phi0.r
      ##
      path.length <- rowSums(abs(Z))
      phi0.r <- H.full[r, ] %*% ginv(Z)
      ##
      ## Find L1 solution
      ##
      A <- diag(p) - Z %*% ginv(Z)
      ##
      ## Minimise |wA + phi0.r|
      ##
      invisible(capture.output(mn <-
                                 cccp::l1(A, -phi0.r,
                                          optctrl = cccp::ctrl(trace = FALSE)),
                               type = "message"))
      w <- cccp::getx(mn)[1:p]
      phi.r <- phi0.r + w %*% A
      ##
      weights[r, ] <- (phi.r / path.length) %*% abs(Z)
      phi[[r]] <- as.vector(phi.r)
      pl[[r]] <- path.length
      ##
      if (is.tictoc) {
        tictoc.r <- tictoc::toc(func.toc = NULL)
        tictoc[r] <- as.numeric(tictoc.r$toc) - as.numeric(tictoc.r$tic)
        ##
        if (verbose)
          cat(round(tictoc[r], 3), "sec elapsed\n")
      }
    }
  }
  ##
  w <- weights
  weights[is_zero(weights)] <- 0
  weights <- weights[, apply(weights, 2, sum) > 0, drop = FALSE]
  attr(weights, "model") <- model
  
  
  res <- list(weights = weights, phi = phi, w = w, pl = pl, zlist = zlist)
  ##
  if (is.tictoc) 
    res$tictoc <- tictoc
  ##
  res
}


contribution.matrix.ruecker.pseudoinv <- function(x, model, verbose = FALSE) {
  
  chkclass(x, "netmeta")
  model <- setchar(model, c("common", "random"))
  chklogical(verbose)
  ##
  is.tictoc <- is_installed_package("tictoc", stop = FALSE)
  
  
  ##
  ## 'Full' aggregated hat matrix
  ##
  H.full <- hatmatrix.aggr(x, model, type = "full")
  ## Total number of possible pairwise comparisons
  comps <- rownames(H.full)
  n.comps <- length(comps)
  n <- x$n
  
  
  ##
  ## Create prop contribution matrix (square matrix)
  ##
  weights <- matrix(0, nrow = n.comps, ncol = n.comps)
  rownames(weights) <- colnames(weights) <- comps


  ##
  ## List with phis (path weights) and pl (path lengths)
  ##
  phi <- pl <- vector("list", n.comps)
  names(phi) <- names(pl) <- comps
  
  ##                                       ####
  ## List with Z matrices                  ####
  ##                                       ####
  zlist <- vector("list", choose(n,2))     ####
  
  
  ##
  ## Measure times
  ##
  if (is.tictoc) {
    tictoc <- rep(NA, n.comps)
    names(tictoc) <- comps
  }
  
  
  ##
  ## Cycle through comparisons
  ##
  r <- 0
  ##
  for (t1 in seq_len(n - 1)) {
    for (t2 in (t1 + 1):n) {
      r <- r + 1
      ##
      if (is.tictoc)
        tictoc::tic()
      ##
      if (verbose)
        cat("- ",
            paste(x$trts[t1], x$trts[t2], sep = x$sep.trts),
            " (", r, "/", n.comps, ")\n",
            sep = "")
      ##
      ## For each row, t1 is the source and t2 is the sink
      ##
      ## Create a transition matrix (P, n x n) from t1 (source) to t2
      ## (sink) using H.full
      ##
      ## Create a dummy n x n matrix Q (this is a step in getting P)
      ##
      ## Assign Q[i, j] equal to the element of H.full in row r and
      ## column representing comparison ij this defines only the upper
      ## half of the matrix Q
      ##
      Q <- matrix(0, nrow = n, ncol = n)
      idx <- 0
      for (i in seq_len(n - 1)) {
        for (j in (i + 1):n) {
          idx <- idx + 1
          Q[i, j] <- H.full[r, idx]
        }
      }
      ##
      ## Now use H_ij = -H_ji to define the lower half of Q
      ##
      for (i in seq_len(n))
        for (j in seq_len(n))
          if (i != j)
            Q[j, i] <- -Q[i, j]
      ##
      ## RW can only move in direction of flow therefore if H_ij < 0
      ## then set Q[i, j] = 0
      ##
      for (i in seq_len(n))
        for (j in seq_len(n))
          if (i != j & Q[i, j] <= 0)
            Q[i, j] <- 0.0
      ##
      ## Create the transition matrix by normalising the values in
      ## each row of Q
      ##
      P <- ginv(diag(rowSums(Q))) %*% Q
      P[t2, ] <- rep(0, n)
      P[t2, t2] <- 1
      ##
      P[is_zero(P, n = 1000)] <- 0
      ##
      ## Find all possible paths
      ##
      ## Define an igraph using P as adjacency matrix. The graph is
      ## directed (and acyclic) and weighted
      ##
      Pgraph <-
        graph_from_adjacency_matrix(P, "directed",
                                    weighted = TRUE, diag = FALSE)
      ##
      ## Now find all possible paths from source to sink
      ##
      ## Simple paths means no vertex is visited more than once 
      ## This is true for us as the graph is acyclic
      ##
      all.paths <- all_simple_paths(Pgraph, t1, t2, mode = "out")
      ##
      ## Calculate streams
      ##
      Z <- matrix(0, nrow = length(all.paths), ncol = dim(H.full)[2])
      rownames(Z) <- all.paths
      colnames(Z) <- colnames(H.full)
      ##
      ## The solution is unique <=> Z %*% ginv(Z) is an identity
      ## matrix, see round(Z %*% ginv(Z), 8).
      ##
      for (p in 1:length(all.paths)) {
        l <- length(all.paths[[p]])
        for (i in 1:(l-1)) {
          s <- 1
          v1 <- all.paths[[p]][i]
          v2 <- all.paths[[p]][i + 1]
          if (v2 < v1) {
            s <- -1
            v1 <- all.paths[[p]][i + 1]
            v2 <- all.paths[[p]][i]
          }
          Z[p, n * (v1 - 1) - choose(v1 + 1, 2) + v2] <- s
        }
      }
      zlist[[r]] <- Z               ####
      ##
      ## Calculate the least squares solution of
      ## phi %*% Z = H.full[r, ]
      ##
      path.length <- rowSums(abs(Z))
      phi.r <- H.full[r, ] %*% ginv(Z)
      weights[r, ] <- (phi.r / path.length) %*% abs(Z)
      phi[[r]] <- as.vector(phi.r)
      pl[[r]] <- path.length
      ##
      if (is.tictoc) {
        tictoc.r <- tictoc::toc(func.toc = NULL)
        tictoc[r] <- as.numeric(tictoc.r$toc) - as.numeric(tictoc.r$tic)
        ##
        if (verbose)
          cat(round(tictoc[r], 3), "sec elapsed\n")
      }
    }
  }
  ##  
  w <- weights
  weights[is_zero(weights)] <- 0
  weights <- weights[, apply(weights, 2, sum) > 0, drop = FALSE]
  attr(weights, "model") <- model
  
  
  res <- list(weights = weights, phi = phi, w = w, pl = pl, zlist = zlist)
  ##
  if (is.tictoc)
    res$tictoc <- tictoc
  ##
  res
}

#' Network meta-analysis of dietary fat
#' 
#' @description
#' Network meta-analysis comparing the effects of two diets to control
#' on mortality.
#' 
#' The data are rates, given as the number of deaths and person-years.
#' These data are used as an example in the supplemental material of
#' Dias et al. (2013).
#' 
#' @name dietaryfat
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{treat1}}\tab treatment 1 \cr
#' \bold{\emph{treat2}}\tab treatment 2 \cr
#' \bold{\emph{treat3}}\tab treatment 3 \cr
#' \bold{\emph{years1}}\tab person years arm 1 \cr
#' \bold{\emph{years2}}\tab person years arm 2 \cr
#' \bold{\emph{years3}}\tab person years arm 3 \cr
#' \bold{\emph{d1}}\tab events (deaths) arm 1 \cr
#' \bold{\emph{d2}}\tab events (deaths) arm 2 \cr
#' \bold{\emph{d3}}\tab events (deaths) arm 3 \cr
#' \bold{\emph{ID}}\tab study ID
#' }
#' 
#' @seealso \code{\link[meta]{pairwise}}, \code{\link[meta]{metainc}},
#' \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
#' 
#' @source
#' Dias S, Sutton AJ, Ades AE and Welton NJ (2013):
#' Evidence synthesis for decision making 2: A generalized linear
#' modeling framework for pairwise and network meta-analysis of
#' randomized controlled trials.
#' \emph{Medical Decision Making},
#' \bold{33}, 607--17
#' 
#' @keywords datasets
#' 
#' @examples
#' data(dietaryfat)
#' head(dietaryfat)
#' 
#' # Examples: example(netmeta)

NULL

comptrts <- function(x,
                     upper = TRUE, reference.group = "", baseline.reference,
                     order = NULL, sep.trts = x$sep.trts, quote.trts = "") {
  
  
  seq.comps <- rownames(x$Cov.common)
  ##
  dat.trts <- matrix(unlist(compsplit(seq.comps, x$sep.trts)),
                     ncol = 2, byrow = TRUE)
  dat.trts <- as.data.frame(dat.trts)
  names(dat.trts) <- c("treat1", "treat2")
  ##
  if (!upper) {
    ##
    ## Comparison names are column:row (and must be switched)
    ##
    t1 <- dat.trts$treat1
    dat.trts$treat1 <- dat.trts$treat2
    dat.trts$treat2 <- t1
  }
  ##  
  if (is.null(order)) {
    ##
    ## Change treatment order if
    ## - reference group is specified, i.e., unequal to ""
    ## - reference group is first treatment
    ##   (argument 'baseline.reference' is TRUE)
    ## - reference group is second treatment
    ##   (argument 'baseline.reference' is FALSE)
    ##
    wo <- rep_len(FALSE, length(seq.comps))
    ##
    if (reference.group != "") {
      reference.group <- setref(reference.group, colnames(x$TE.common))
      ##
      if (baseline.reference)
        wo <- dat.trts$treat1 == reference.group
      else
        wo <- dat.trts$treat2 == reference.group
    }
    ##
    if (any(wo)) {
      t1.wo <- dat.trts$treat1[wo]
      dat.trts$treat1[wo] <- dat.trts$treat2[wo]
      dat.trts$treat2[wo] <- t1.wo
    }
  }
  else {
    treat1.pos <- as.numeric(factor(dat.trts$treat1, levels = order))
    treat2.pos <- as.numeric(factor(dat.trts$treat2, levels = order))
    ##
    wo <- treat1.pos > treat2.pos
    ##
    if (any(wo)) {
      ttreat1 <- dat.trts$treat1
      dat.trts$treat1[wo] <- dat.trts$treat2[wo]
      dat.trts$treat2[wo] <- ttreat1[wo]
      ##
      ttreat1.pos <- treat1.pos
      treat1.pos[wo] <- treat2.pos[wo]
      treat2.pos[wo] <- ttreat1.pos[wo]
    }
    ##
    o <- order(treat1.pos, treat2.pos)
    dat.trts <- dat.trts[o, ]
  }
  
  
  comparison <-
    as.character(
      interaction(
        paste0(quote.trts, dat.trts$treat1, quote.trts),
        paste0(quote.trts, dat.trts$treat2, quote.trts),
        sep = sep.trts))
  
  
  res <- cbind(comparison, dat.trts)
  res
}

#' Merge pairwise object with additional data
#' 
#' @description
#' Merge pairwise object with additional data, e.g., information on
#' network connectivity.
#' 
#' @param x An object of class \code{pairwise}.
#' @param y A data frame or an object of class \code{netconnection}.
#' @param all.x A logical indicating whether to keep all observations from the
#'   pairwise object, i.e., also include observations not belonging to a
#'   subnetwork due to missing estimates or standard errors.
#' @param \dots Other arguments (passed on to \code{merge}).
#' 
#' @return An object of class \code{pairwise}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link[meta]{pairwise}}, \code{\link{netconnection}},
#'   \code{\link[metabook]{Woods2010}}
#' 
#' @examples
#' # Transform data from long arm-based format to contrast-based
#' # format Argument 'sm' has to be used for odds ratio as summary
#' # measure; by default the risk ratio is used in the metabin
#' # function called internally.
#' #
#' pw <- pairwise(treatment, event = r, n = N,
#'   studlab = author, data = Woods2010, sm = "OR",
#'   append = FALSE)
#' head(pw)
#' 
#' # Add information on network connectivity
#' nc <- netconnection(pw)
#' pw.nc <- merge(pw, nc)
#' head(pw.nc)
#'
#' @method merge pairwise
#' @export

merge.pairwise <- function(x, y, all.x = TRUE, ...) {
  chkclass(x, "pairwise")
  #
  x$..order <- seq_len(nrow(x))
  xdat <- as.data.frame(x)
  #
  attribs <- attributes(x)
  attribs[["names"]] <- NULL
  attribs[["row.names"]] <- NULL
  #
  # Get rid of warning "no visible binding for global variable"
  #
  subnet <- design <- NULL
  #
  if (inherits(y, "netconnection")) {
    if (isCol(xdat, "subnet"))
      xdat <- rename(xdat, subnet.orig = subnet)
    #
    if (isCol(xdat, "design"))
      xdat <- rename(xdat, design.orig = design)
    #
    ydat <- as.data.frame(y)
    #
    res <- merge(xdat, ydat, by = c("studlab", "treat1", "treat2"),
                 all.x = all.x, ...)
  }
  else if (inherits(y, "data.frame"))
    res <- merge(xdat, y, all.x = all.x, ...)
  else
    stop("Argument 'y' must be a data frame or of class 'netconnection'.",
         call. = FALSE)
  #
  res <- res[order(res$..order), ]
  rownames(res) <- seq_len(nrow(res))
  res$..order <- NULL
  #
  for (i in names(attribs))
    attr(res, i) <- attribs[[i]]
  #
  res
}

#' Heat Plot
#'
#' @description
#' Produces a heat plot containing treatment estimates with confidence
#' intervals for all possible pairwise comparisons.
#'
#' @param x An object of class \code{netmeta}.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model
#'   (\code{"random"}) should be plotted. Can be abbreviated.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments in rows and columns of the heat plot.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param low.colour A string indicating the colour of low relative
#'   treatment effects for the heat plot (e.g odds ratio of ~0.5)
#' @param mid.colour A string indicating the colour of null relative
#'   treatment effects for the heat plot (e.g odds ratio of ~1.0).
#' @param high.colour A string indicating the colour of high relative
#'   treatment effects for the heat plot (e.g odds ratio of ~2.0).
#' @param size The size of cell entries with the relative treatment
#'   effect and confidence intervals.
#' @param size.trt The size of treatment names placed on the top and
#'   left of the plot.
#' @param size.axis The size of labels on the top and left of the plot
#' @param digits Minimal number of significant digits for treatment
#'   effects and confidence intervals, see \code{print.default}.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param \dots Additional arguments.
#'
#' @return
#' League heat plot, where a color scale is used to represent the
#' values of relative treatment effects.
#' 
#' @keywords hplot
#' 
#' @examples
#' # Only consider first five studies (to reduce runtime of example)
#' #
#' studies <- unique(Senn2013$study)
#' Senn2013.5 <- subset(Senn2013, study %in% studies[1:5])
#' 
#' # Transform data from long arm-based to contrast-based format
#' #
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013.5,
#'   varnames = c("MD", "seMD"))
#' 
#' # Conduct random effects network meta-analysis with
#' # placebo as reference treatment
#' #
#' nma <- netmeta(pw, common = FALSE, reference = "plac")
#' 
#' # Generate a heat plot (with abbreviated treatment labels)
#' #
#' heatplot(nma, nchar.trts = 4) 
#'
#' @method heatplot netmeta
#' @export

heatplot.netmeta <- function(x,
                             pooled = ifelse(x$random, "random", "common"),
                             seq = x$seq,
                             nchar.trts = x$nchar.trts,
                             ##
                             low.colour = "red",
                             mid.colour = "white",
                             high.colour = "springgreen4",
                             ##
                             size = 6,
                             size.trt = 16,
                             size.axis = 12,
                             ##
                             digits = gs("digits.forest"),
                             backtransf = x$backtransf,
                             ...) {
  
  chkclass(x, "netmeta")
  x <- updateversion(x)
  sm <- x$sm
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  ##
  if (is.null(x$nchar.trts))
    nchar.trts <- 666
  chknumeric(nchar.trts, min = 1, length = 1)
  ##
  chknumeric(digits, min = 0, length = 1)
  ##
  if (is_untransformed(sm))
    backtransf <- TRUE
  backtransf <- replaceNULL(backtransf, TRUE)
  chklogical(backtransf)
  
  
  trts <- x$trts
  ##
  if (!missing(seq) & is.null(seq))
    stop("Argument 'seq' must be not NULL.")
  ##
  if (is.null(seq) | (length(seq) == 1 & x$d == 1))
    seq1 <- 1:length(trts)
  else
    seq1 <- charmatch(setseq(seq, x$trts), x$trts)
  
  
  if (pooled == "common") {
    TE.nma <- x$TE.common[seq1, seq1]
    lower.nma <- x$lower.common[seq1, seq1]
    upper.nma <- x$upper.common[seq1, seq1]
  }
  else {
    TE.nma <- x$TE.random[seq1, seq1]
    lower.nma <- x$lower.random[seq1, seq1]
    upper.nma <- x$upper.random[seq1, seq1]
  }
  ##
  noeffect <- 1L * (backtransf & is_relative_effect(sm))
  #
  if (backtransf) {
    TE.nma    <- backtransf(TE.nma, sm)
    lower.nma <- backtransf(lower.nma, sm)
    upper.nma <- backtransf(upper.nma, sm)
    #
    # Switch lower and upper limit for VE if results have been
    # backtransformed
    #
    if (sm == "VE") {
      tmp.l <- lower.nma
      lower.nma <- upper.nma
      upper.nma <- tmp.l
    }
  }
  ##
  TE.nma    <- round(TE.nma, digits)
  lower.nma <- round(lower.nma, digits)
  upper.nma <- round(upper.nma, digits)
  ##
  trts <- trts[seq1]
  ct <- heattrts(TE.nma)
  ##
  hdata <- data.frame(Treatment = c(ct$treat2, ct$treat1),
                      Comparator = c(ct$treat1, ct$treat2))
  ##
  hdata$TE.nma <- c(lowertri(TE.nma), uppertri(TE.nma))
  hdata$lower.nma <- c(lowertri(lower.nma), uppertri(lower.nma))
  hdata$upper.nma <- c(lowertri(upper.nma), uppertri(upper.nma))
  ##
  trts.abbr <- treats(trts, nchar.trts)
  ##
  hdata$Treatment <-
    as.character(factor(hdata$Treatment,
                        levels = trts, labels = trts.abbr))
  ##
  hdata$Comparator <-
    as.character(factor(hdata$Comparator,
                        levels = trts, labels = trts.abbr))
  
  
  ##
  ##
  ## Create heat plot
  ##
  ##
  Treatment <- Comparator <- TE.nma <- lower.nma <- upper.nma <- NULL
  ##
  hplot <-
    ggplot(data = hdata,
           aes(x = Treatment, y = Comparator, fill = TE.nma)) +
    geom_tile() +
    geom_text(aes(label = paste0(formatN(TE.nma, digits), "\n",
                                 formatCI(round(lower.nma, digits),
                                          round(upper.nma, digits)))),
              size = size)
  ##
  hplot <- hplot +
    scale_fill_gradient2(low = low.colour,
                         mid = mid.colour,
                         high = high.colour,
                         midpoint = noeffect) +
    theme_dark() +
    theme(panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          legend.position = "none", panel.border = element_blank(),
          axis.ticks.x = element_blank(),
          axis.ticks.y = element_blank(),
          axis.text = element_text(size = size.trt),
          axis.title = element_text(size = size.axis)) +
    scale_x_discrete(limits = trts.abbr, expand = c(0, 0),
                     position = "top") +
    scale_y_discrete(limits = rev(trts.abbr), expand = c(0, 0))
  
  return(hplot)
}


heattrts <- function(x) {
  trts <- rownames(x)
  n <- length(trts)
  ##
  treat1 <- treat2 <- vector("character", 0)
  k <- 0
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      if (i != j) {
        k <- k + 1
        treat1[k] <- trts[j]
        treat2[k] <- trts[i]
      }
    }
  }
  ##
  data.frame(treat1, treat2)
}

setsep <- function(x, sep, type = "treatment label",
                   argname = deparse(substitute(sep)),
                   missing = TRUE) {
  labels <- sort(unique(x))
  #
  if (compmatch(labels, sep)) {
    if (!missing)
      warning("Argument '", argname, "': ",
              "separator '", sep, "' used in at least one ", type, ". ",
              "Trying to use predefined separators: ",
              "':', '-', '_', '/', '+', '.', '|', '*'.",
              call. = FALSE)
    #
    if (!compmatch(labels, ":"))
      sep <- ":"
    else if (!compmatch(labels, "-"))
      sep <- "-"
    else if (!compmatch(labels, "_"))
      sep <- "_"
    else if (!compmatch(labels, "/"))
      sep <- "/"
    else if (!compmatch(labels, "+"))
      sep <- "+"
    else if (!compmatch(labels, "."))
      sep <- "."
    else if (!compmatch(labels, "|"))
      sep <- "|"
    else if (!compmatch(labels, "*"))
      sep <- "*"
    else
      stop("All predefined separators (':', '-', '_', '/', '+', ",
           "'.', '|', '*') are used in at least one ", type, ". ",
           "Please specify a different character that should be ",
           "used as separator (argument '", argname, "').",
           call. = FALSE)
  }
  #
  sep
}

formatCC <- function(x,
                     backtransf, sm, level,
                     comps, comps.abbr, sep.comps,
                     digits, digits.stat, digits.pval.Q,
                     scientific.pval, zero.pval, JAMA.pval,
                     big.mark) {
  
  
  relative <- is_relative_effect(sm)
  
  sm.lab <- sm
  ##
  if (sm != "") {
    sm.lab <- paste0("i", if (sm == "VE") "VR" else sm)
    if (!backtransf & relative)
      sm.lab <- paste0("log(", sm.lab, ")")
  }
  ##  
  ci.lab <- paste0(round(100 * level, 1), "% CI")
  
  
  ## First column contains row names
  ##
  res <- x
  ##
  rownames(res) <- compos(rownames(res), comps, comps.abbr, sep.comps)
  ##
  if (backtransf) {
    res$TE <- backtransf(res$TE, sm)
    res$lower <- backtransf(res$lower, sm)
    res$upper <- backtransf(res$upper, sm)
    #
    # Switch lower and upper limit for VE if results have been
    # backtransformed
    #
    if (sm == "VE") {
      tmp.l <- res$lower
      res$lower <- res$upper
      res$upper <- tmp.l
    }
  }
  ##
  res$TE <- formatN(res$TE, digits, "NA", big.mark)
  res$lower <- formatCI(formatN(round(res$lower, digits),
                                digits, "NA", big.mark),
                        formatN(round(res$upper, digits),
                                digits, "NA", big.mark))
  res$statistic <- formatN(round(res$statistic, digits.stat),
                           digits.stat, big.mark = big.mark)
  res$p <- formatPT(res$p,
                    digits = digits.pval.Q,
                    scientific = scientific.pval,
                    zero = zero.pval,
                    JAMA = JAMA.pval)
  ##
  res$seTE <- res$upper <- res$z <- NULL
  ##
  sel <- names(res) == "TE"
  names(res)[sel] <- sm.lab
  ##
  sel <- names(res) == "lower"
  names(res)[sel] <- ci.lab
  ##
  sel <- names(res) == "statistic"
  names(res)[sel] <- "z"
  ##
  sel <- names(res) == "p"
  names(res)[sel] <- "p-value"
  
  
  res
}

#' Test of funnel plot asymmetry in network meta-analysis
#' 
#' @description
#' Test of funnel plot asymmetry in network meta-analysis
#' 
#' @param x An object of class \code{netmeta}.
#' @param order A mandatory character or numerical vector specifying
#'   the order of treatments or list of comparators (see Details).
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be used in test of funnel plot asymmetry. Can be
#'   abbreviated.
#' @param method.bias A character vector indicating which test(s) for
#'   funnel plot asymmatrx to use. Admissible values are
#'   \code{"Begg"}, \code{"Egger"}, and \code{"Thompson"}, can be
#'   abbreviated. See function \code{\link[meta]{metabias.meta}}.
#' @param lump.comparator A logical indicating whether comparators
#'   should be lumped, e.g., to specify inactive treatments.
#'   information on direct comparisons should be added to the plot.
#' @param \dots Additional arguments (passed on to
#'   \code{\link[meta]{metabias.meta}}).
#' 
#' @details
#' Test of funnel plot asymmetry in network meta-analysis
#' 
#' Argument \code{order} is mandatory to determine the order of
#' treatments (Chaimani et al., 2013):
#' 
#' \emph{\dQuote{[...] investigators should order
#' the treatments in a meaningful way and make assumptions about how
#' small studies differ from large ones. For example, if they
#' anticipate that newer treatments are favored in small trials, then
#' they could name the treatments from oldest to newest so that all
#' comparisons refer to \sQuote{old versus new intervention}. Other
#' possibilities include defining the comparisons so that all refer to
#' an active treatment versus placebo or sponsored versus
#' non-sponsored intervention.}}
#' 
#' Alternatively, it is possible to only provide a single or few
#' treatment name(s) in argument \code{order} to define the
#' comparator(s). In this case only comparisons with this / these
#' treatment(s) will be considered. If argument \code{lump.comparator}
#' is \code{TRUE}, all comparators will be lumped into a single
#' group.
#' 
#' @return
#' A list with class \code{metabias} containing the following
#' components if a test for funnel plot asymmetry is conducted:
#' \item{statistic}{Test statistic.}
#' \item{df}{The degrees of freedom of the test statistic in
#'   the case that it follows a t distribution.}
#' \item{pval}{The p-value for the test.}
#' \item{estimate}{Estimates used to calculate test statisic.}
#' \item{method}{A character string indicating what type of test was
#'   used.}
#' \item{title}{Title of Cochrane review.}
#' \item{complab}{Comparison label.}
#' \item{outclab}{Outcome label.}
#' \item{var.model}{A character string indicating whether none,
#'   multiplicative, or additive residual heterogeneity variance was
#'   assumed.}
#' \item{method.bias}{As defined above.}
#' \item{x}{Network meta-analysis object.}
#' \item{version}{Version of R package \bold{meta} used to create
#'   object.}
#' \item{version.netmeta}{Version of R package \bold{netmeta} used to
#'   create object.}
#'
#' Or a list with the following elements if test is not conducted due
#' to the number of studies:
#' \item{k}{Number of comparisons.}
#' \item{k.min}{Minimum number of comparisons to perform test for
#'   funnel plot asymmetry.}
#' \item{version}{Version of R package \bold{meta} used to create
#'   object.}
#' \item{version.netmeta}{Version of R package \bold{netmeta} used to
#'   create object.}
#' 
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{funnel.netmeta}},
#'   \code{\link[meta]{metabias}}
#' 
#' @references
#' Chaimani A & Salanti G (2012):
#' Using network meta-analysis to evaluate the existence of
#' small-study effects in a network of interventions.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 161--76
#' 
#' Chaimani A, Higgins JP, Mavridis D, Spyridonos P, Salanti G (2013):
#' Graphical tools for network meta-analysis in STATA.
#' PLOS ONE,
#' \bold{8}, e76654
#' 
#' @examples
#' \donttest{
#' # Transform data from long arm-based to contrast-based format
#' #
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' 
#' # Conduct network meta-analysis
#' #
#' nma <- netmeta(pw)
#' 
#' # Test for asymmetry in 'comparison-adjusted' funnel plot not
#' # conducted as argument 'order' is missing
#' #
#' try(metabias(nma))
#' 
#' # Test for funnel plot asymmetry comparing active treatments with
#' # placebo
#' metabias(nma, order = "pl")
#'
#' # Rank test
#' #
#' metabias(nma, order = "pl", method.bias = "Begg")
#'
#' 
#' # Test for funnel plot asymmetry based on (non-sensical) alphabetic
#' # order of treatments with placebo as last treatment
#' #
#' ord <- c("a", "b", "me", "mi", "pi", "r", "si", "su", "v", "pl")
#' metabias(nma, order = ord)
#' }
#'
#' @method metabias netmeta
#' @export

metabias.netmeta <- function(x,
                             order,
                             pooled = ifelse(x$random, "random", "common"),
                             method.bias = "Egger",
                             lump.comparator = gs("lump.comparator"),
                             ...) {
  
  
  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netmeta")
  x <- updateversion(x)
  sep.trts <- x$sep.trts
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  chklogical(lump.comparator)
  ##
  if (missing(order))
    stop("Argument 'order' with a meaningful order of treatments ",
         "must be provided.\n  ",
         "(see help page of funnel.netmeta for some examples).")
  else {
    if (length(order) != length(x$trts)) {
      order <- setchar(order, x$trts)
      order.all <- c(x$trts[!(x$trts %in% order)], order)
    }
    else
      order.all <- order <- setseq(order, x$trts)
  }
  ##
  if ((length(order) == length(order.all)) & lump.comparator) {
    warning("Argument 'lump.comparator' ignored as full treatment order is ",
            "provided.",
            call. = FALSE)
    lump.comparator <- FALSE
  }
  ##
  method.bias <- setmethodbias(method.bias, 1:3)
  
  
  ##
  ##
  ## (2) Get data
  ##
  ##
  TE <- x$TE
  seTE <- x$seTE
  studlab <- x$studlab
  ##
  treat1 <- x$treat1
  treat2 <- x$treat2
  ##
  comparison <- paste(treat1, treat2, sep = sep.trts)
  comparison21 <- paste(treat2, treat1, sep = sep.trts)
  ##
  treat1.pos <- as.numeric(factor(treat1, levels = order.all))
  treat2.pos <- as.numeric(factor(treat2, levels = order.all))
  ##
  wo <- treat1.pos > treat2.pos
  ##
  if (any(wo)) {
    ttreat1.pos <- treat1.pos
    treat1.pos[wo] <- treat2.pos[wo]
    treat2.pos[wo] <- ttreat1.pos[wo]
    ##
    TE[wo] <- -TE[wo]
    ##
    ttreat1 <- treat1
    treat1[wo] <- treat2[wo]
    treat2[wo] <- ttreat1[wo]
    ##
    comparison[wo] <- comparison21[wo]
  }
  ##
  o <- order(treat1.pos, treat2.pos)
  ##
  TE <- TE[o]
  seTE <- seTE[o]
  ##
  studlab <- studlab[o]
  ##
  treat1 <- treat1[o]
  treat2 <- treat2[o]
  comparison <- comparison[o]
  ##
  res <- data.frame(studlab,
                    treat1, treat2, comparison,
                    TE, TE.direct = NA, TE.adj = NA, seTE)
  
  
  ##
  ##
  ## (3) Calculate 'comparison-adjusted' treatment effects
  ##
  ##
  if (is.numeric(treat1))
    treat1 <- as.character(treat1)
  if (is.numeric(treat2))
    treat2 <- as.character(treat2)
  ##
  if (pooled == "common")
    for (i in seq_along(res$TE))
      res$TE.direct[i] <- x$TE.direct.common[treat1[i], treat2[i]]
  else
    for (i in seq_along(res$TE))
      res$TE.direct[i] <- x$TE.direct.random[treat1[i], treat2[i]]
  ##
  res$TE.adj <- res$TE - res$TE.direct
  ##
  if (length(order) != length(order.all))
    res <- subset(res, treat1 %in% order | treat2 %in% order)
  ##
  if (lump.comparator) {
    res <- subset(res, !(treat1 %in% order & treat2 %in% order))
    ##
    res$treat2[res$treat2 %in% order] <- "...inactive..."
    ##
    res$comparison <- paste(res$treat1, res$treat2, sep = sep.trts)
  }
  
  
  ##
  ##
  ## (4) Conduct meta-analysis
  ##
  ##
  m.adj <-
    suppressWarnings(metagen(res$TE.adj, res$seTE,
                             studlab = res$studlab, sm = x$sm,
                             method.tau = "DL", method.tau.ci = ""))
  
  
  ##
  ##
  ## (5) Test of funnel plot asymmetry
  ##
  ##
  res <- metabias(m.adj, ...)
  ##
  res$version.netmeta <- packageDescription("netmeta")$Version
  
  
  res
}

setseq <- function(seq, levs, text, equal.length = TRUE) {
  name <- deparse(substitute(seq))
  if (missing(text))
      text <- paste0("Argument '", name, "'")
  ##
  if (length(levs) != length(seq) & equal.length)
    stop("Length of argument '", name,
         "' different from number of treatments.", call. = FALSE)
  ##
  if (length(unique(seq)) != length(seq))
    stop("Values for argument '", name,
         "' must all be disparate.", call. = FALSE)
  ##
  if (is.numeric(seq)) {
    if (anyNA(seq))
      stop("Missing values not allowed in argument '",
           name, "'.", call. = FALSE)
    if (any(!(seq %in% seq_len(length(levs)))))
      stop(paste0("Argument '", name,
                  "' must be a permutation of the integers from 1 to ",
                  length(levs), "."),
           call. = FALSE)
    res <- levs[seq]
  }
  else if (is.character(seq)) {
    if (length(unique(levs)) == length(unique(tolower(levs))))
      idx <- charmatch(tolower(seq), tolower(levs), nomatch = NA)
    else
      idx <- charmatch(seq, levs, nomatch = NA)
    ##
    if (equal.length && (anyNA(idx) || any(idx == 0)))
      stop(paste0(text,
                  " must be a permutation of the following values:\n  ",
                  paste(paste0("'", levs, "'"), collapse = " - ")),
           call. = FALSE)
    res <- levs[idx]
    if (!equal.length)
      res <- res[!is.na(res)]
  }
  else
    stop("Argument '", name, "' must be either a numeric or character vector.",
         call. = FALSE)
  
  res
}

#' Combine network meta-analysis objects
#' 
#' @description
#' This function can be used to combine network meta-analysis objects
#' which is especially useful to generate a forest plot with results
#' of several network meta-analyses.
#' 
#' @param \dots Any number of network meta-analysis objects or a
#'   single list with network meta-analyses.
#' @param name An optional character vector providing descriptive
#'   names for network meta-analysis objects.
#' @param common A logical indicating whether results for the common
#'   effects model should be reported.
#' @param random A logical indicating whether results for the random
#'   effects model should be reported.
#' @param col.study The colour for network estimates and confidence
#'   limits.
#' @param col.inside The colour for network estimates and confidence
#'   limits if confidence limits are completely within squares.
#' @param col.square The colour for squares.
#' @param col.square.lines The colour for the outer lines of squares.
#' @param backtransf A logical indicating whether results should be
#'   back transformed. If \code{backtransf = TRUE} (default), results
#'   for \code{sm = "OR"} are printed as odds ratios rather than log
#'   odds ratios, for example.
#' @param reference.group Reference treatment.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param fixed Deprecated argument; replaced by \code{common}.
#' @param comb.fixed Deprecated argument; replaced by \code{common}.
#' @param comb.random Deprecated argument; replaced by \code{random}.
#' 
#' @return
#' An object of class \code{"netbind"} with corresponding
#' \code{forest} function. The object is a list containing the
#' following components:
#' \item{common}{A data frame with results for the common effects
#'   model.}
#' \item{random}{A data frame with results for the random effects
#'   model.}
#' \item{sm}{Summary measure used in network meta-analyses.}
#' \item{level.ma}{Level for confidence intervals.}
#' \item{reference.group, baseline.reference}{As defined above.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netcomb}},
#'   \code{\link{discomb}}, \code{\link{forest.netbind}}
#' 
#' @examples
#' \donttest{
#' # Only consider studies including Face-to-face PST (to reduce
#' # runtime of example)
#' #
#' face <- subset(Linde2016, id %in% c(16, 24, 49, 118))
#' 
#' # Use pairwise() to transform data to comparison-based format
#' #
#' pw <- pairwise(treat = int,
#'   event = resp, n = n, studlab = paste(author, year),
#'   data = face, reference = "plac", sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma <- netmeta(pw, common = FALSE)
#' 
#' # Additive CNMA model with placebo as inactive component and
#' # reference
#' #
#' cnma <- netcomb(nma, inactive = "placebo")
#' 
#' # Combine results of standard NMA and CNMA
#' #
#' nb <- netbind(cnma, nma,
#'   name = c("Additive CNMA", "Standard NMA"),
#'   col.study = c("red", "black"), col.square = c("red", "black"))
#' #
#' nb
#' #
#' forest(nb,
#'   col.subgroup = "black", addrow.subgroups = FALSE,
#'   fontsize = 10, spacing = 0.7, squaresize = 0.9,
#'   label.left = "Favours Placebo",
#'   label.right = "Favours other")
#' 
#' # Also show results for common effects model
#' #
#' print(nb, common = TRUE)
#' }
#' 
#' @export netbind

netbind <- function(..., name,
                    common, random,
                    ##
                    col.study = "black",
                    col.inside = "white",
                    col.square = "gray",
                    col.square.lines = col.square,
                    ##
                    backtransf,
                    reference.group, baseline.reference,
                    warn.deprecated = gs("warn.deprecated"),
                    fixed, comb.fixed, comb.random
                    ) {
  
  
  ##
  ##
  ## (1) Extract list elements and basic checks
  ##
  ##
  
  chklogical(warn.deprecated)
  ##
  is.nma <- function(x)
    inherits(x, "netmeta") |
      inherits(x, "netcomb") |
      inherits(x, "discomb")
  ##
  args <- list(...)
  ##
  n.netmeta <- length(args)
  n.i <- seq_len(n.netmeta)
  ##
  if (n.netmeta == 1) {
    if (!is.list(args[[1]]))
      stop("All elements of argument '...' must be of classes ",
           "'netmeta', 'netcomb', or 'discomb'.",
           call. = FALSE)
    ##
    if (!is.nma(args[[1]])) {
      n.netmeta <- length(args[[1]])
      n.i <- seq_len(n.netmeta)
      ##
      args2 <- list()
      for (i in n.i)
        args2[[i]] <- args[[1]][[i]]
    }
    args <- args2
  }
  #
  commons <- randoms <- vector()
  #
  for (i in n.i) {
    if (!is.nma(args[[i]]))
      stop("All elements of argument '...' must be of classes ",
           "'netmeta', 'netcomb', or 'discomb'.",
           call. = FALSE)
    ##
    args[[i]] <- updateversion(args[[i]])
    #
    commons[i] <- args[[i]]$common
    randoms[i] <- args[[i]]$random
  }
  ##
  levs <- numeric(0)
  for (i in n.i)
    levs[i] <- args[[i]]$level.ma
  ##
  if (length(unique(levs)) != 1)
    stop("Different confidence levels used in network meta-analyses ",
         "(see list element 'level.ma').",
         call. = FALSE)
  ##
  sms <- character(0)
  for (i in n.i)
    sms[i] <- args[[i]]$sm
  ##
  if (length(unique(sms)) != 1)
    stop("Different summary measure used in network meta-analyses ",
         "(see list element 'sm').",
         call. = FALSE)
  ##
  if (missing(col.study))
    col.study <- rep(col.study, n.netmeta)
  else
    if (length(col.study) != n.netmeta) 
      stop("Length of argument 'col.study' must be the same as the ",
           "number of network meta-analyses.",
           call. = FALSE)
  ##
  if (missing(col.square))
    col.square <- rep(col.square, n.netmeta)
  else
    if (length(col.square) != n.netmeta) 
      stop("Length of argument 'col.square' must be the same as the ",
           "number of network meta-analyses.",
           call. = FALSE)
  ##
  if (missing(col.square.lines))
    col.square.lines <- rep(col.square.lines, n.netmeta)
  else
    if (length(col.square.lines) != n.netmeta) 
      stop("Length of argument 'col.square.lines' must be the same as the ",
           "number of network meta-analyses.",
           call. = FALSE)
  ##
  if (missing(col.inside))
    col.inside <- rep(col.inside, n.netmeta)
  else
    if (length(col.inside) != n.netmeta) 
      stop("Length of argument 'col.inside' must be the same as the ",
           "number of network meta-analyses.",
           call. = FALSE)
  ##
  print.warning1 <- FALSE
  print.warning2 <- FALSE
  print.warning3 <- FALSE
  print.warning4 <- FALSE


  ##
  ##
  ## (2) Set names of network meta-analysis objects
  ##
  ##
  if (missing(name))
    name <- paste0("netmeta", n.i)
  else {
    if (length(name) != n.netmeta) 
      stop("Number of network meta-analyses and ",
           "names provided in argument 'name' differ.",
           call. = FALSE)
    ##
    if (length(unique(name)) != length(name)) {
      warning1 <-
        paste0("Network meta-analyses are labelled 'netmeta1' to 'netmeta",
               n.netmeta,
               "' as values of argument 'name' are not all disparate.")
      print.warning1 <- TRUE
      ##
      name <- paste0("netmeta", n.i)
    }
  }
  
  
  ##
  ## (3) Determine common
  ##
  missing.common <- missing(common)
  missing.fixed <- missing(fixed)
  missing.comb.fixed <- missing(comb.fixed)
  ##
  if (missing.common & missing.fixed & missing.comb.fixed) {
    cfs <- logical(0)
    ##
    for (i in n.i) {
      if (!is.null(args[[i]]$common))
        cfs[i] <- args[[i]]$common
      else if (!is.null(args[[i]]$fixed))
        cfs[i] <- args[[i]]$fixed
      else
        cfs[i] <- args[[i]]$comb.fixed
    }
    ##
    cfs <- unique(cfs)
    ##
    if (length(cfs) != 1)
      common <- TRUE
    else
      common <- cfs
  }
  else {
    common <-
      deprecated2(common, missing.common, comb.fixed, missing.comb.fixed,
                  warn.deprecated)
    common <-
      deprecated2(common, missing.common, fixed, missing.fixed,
                  warn.deprecated)
    chklogical(common)
  }
  
  
  ##
  ## (4) Determine random
  ##
  missing.random <- missing(random)
  missing.comb.random <- missing(comb.random)
  ##
  if (missing.random & missing.comb.random) {
    crs <- logical(0)
    ##
    for (i in n.i)
      if (!is.null(args[[i]]$random))
        crs[i] <- args[[i]]$random
      else
        crs[i] <- args[[i]]$comb.random
    ##
    crs <- unique(crs)
    ##
    if (length(crs) != 1)
      random <- TRUE
    else
      random <- crs
  }
  else {
    random <-
      deprecated2(random, missing.random, comb.random, missing.comb.random,
                  warn.deprecated)
    chklogical(random)
  }
  
  
  ##
  ## (5) Determine backtransf
  ##
  if (missing(backtransf)) {
    backt <- logical(0)
    ##
    for (i in n.i)
      backt[i] <- args[[i]]$backtransf
    ##
    backt <- unique(backt)
    ##
    if (length(backt) != 1) {
      backtransf <- TRUE
      warning2 <- paste0("Argument 'backtransf' set to TRUE ",
                         "(as it is not unique in network meta-analyses).")
      print.warning2 <- TRUE
    }
    else
      backtransf <- backt
  }
  else
    chklogical(backtransf)
  
  
  ##
  ##
  ## (4) Determine reference group
  ##
  ##
  if (missing(reference.group)) {
    refs <- character(0)
    ##
    for (i in n.i)
      refs[i] <- args[[i]]$reference.group
    ##
    refs <- unique(refs)
    refs <- refs[refs != ""]
    ##
    if (length(refs) == 0) {
      reference.group <- args[[i]]$trts[1]
      warning3 <- paste0("Unspecified argument 'reference.group' is set to '",
                         reference.group,
                         "'.")
      print.warning3 <- TRUE
    }
    else if (length(refs) == 1)
      reference.group <- refs
    else {
      reference.group <- refs[1]
      warning3 <- paste0("Argument 'reference.group' set to '",
                         reference.group,
                         "' (as it is not unique in network meta-analyses).")
      print.warning3 <- TRUE
    }
  }
  ##
  trts.all <- character(0)
  ##
  for (i in n.i)
    trts.all <- c(trts.all, args[[i]]$trts)
  ##
  trts.all <- unique(trts.all)
  ##
  reference.group <- setref(reference.group, trts.all)
  ##
  for (i in n.i)
    if (!(reference.group %in% args[[i]]$trts))
      stop("Reference treatment '", reference.group,
           "' not included in all network meta-analyses.")
  
  
  ##
  ## (6) Determine baseline reference
  ##
  if (missing(baseline.reference)) {
    bref <- logical(0)
    ##
    for (i in n.i)
      bref[i] <- args[[i]]$baseline.reference
    ##
    bref <- unique(bref)
    ##
    if (length(bref) != 1) {
      bref <- TRUE
      warning4 <- paste0("Argument 'baseline.reference' set to TRUE ",
                         "(as it is not unique in network meta-analyses).")
      print.warning4 <- TRUE
    }
    ##
    baseline.reference <- bref
  }
  ##
  chklogical(baseline.reference)
  
  
  common.nma <- common
  common <- NULL
  ##
  for (i in n.i) {
    ##
    rn <- rownames(args[[i]]$TE.common)
    seq1 <- charmatch(setseq(args[[i]]$seq, rn), rn)
    ##
    TE.i <- args[[i]]$TE.common[seq1, seq1]
    seTE.i <- args[[i]]$seTE.common[seq1, seq1]
    lower.i <- args[[i]]$lower.common[seq1, seq1]
    upper.i <- args[[i]]$upper.common[seq1, seq1]
    statistic.i <- args[[i]]$statistic.common[seq1, seq1]
    pval.i <- args[[i]]$pval.common[seq1, seq1]
    ##
    cnam <- colnames(TE.i)
    rnam <- rownames(TE.i)
    selc <- cnam == reference.group
    selr <- rnam == reference.group
    ##
    if (baseline.reference) {
      common <- rbind(common,
                      data.frame(name = name[i],
                                 treat = cnam,
                                 ##
                                 TE = TE.i[, selc],
                                 seTE = seTE.i[, selc],
                                 lower = lower.i[, selc],
                                 upper = upper.i[, selc],
                                 statistic = statistic.i[, selc],
                                 pval = pval.i[, selc],
                                 #
                                 method = args[[i]]$method[1],
                                 #
                                 col.study = col.study[i],
                                 col.square = col.square[i],
                                 col.square.lines = col.square.lines[i],
                                 col.inside = col.inside[i],
                                 ##
                                 stringsAsFactors = FALSE)
                      )
    }
    else {
      common <- rbind(common,
                      data.frame(name = name[i],
                                 treat = cnam,
                                 ##
                                 TE = TE.i[selr, ],
                                 seTE = seTE.i[selr, ],
                                 lower = lower.i[selr, ],
                                 upper = upper.i[selr, ],
                                 statistic = statistic.i[selr, ],
                                 pval = pval.i[selr, ],
                                 #
                                 method = args[[i]]$method[1],
                                 #
                                 col.study = col.study[i],
                                 col.square = col.square[i],
                                 col.square.lines = col.square.lines[i],
                                 col.inside = col.inside[i],
                                 ##
                                 stringsAsFactors = FALSE)
                      )
    }
  }
  ##
  rownames(common) <- seq_len(nrow(common))
  
  
  random.nma <- random
  random <- NULL
  ##
  for (i in n.i) {
    ##
    rn <- rownames(args[[i]]$TE.random)
    seq1 <- charmatch(setseq(args[[i]]$seq, rn), rn)
    ##
    TE.i <- args[[i]]$TE.random[seq1, seq1]
    seTE.i <- args[[i]]$seTE.random[seq1, seq1]
    lower.i <- args[[i]]$lower.random[seq1, seq1]
    upper.i <- args[[i]]$upper.random[seq1, seq1]
    statistic.i <- args[[i]]$statistic.random[seq1, seq1]
    pval.i  <- args[[i]]$pval.random[seq1, seq1]
    ##
    cnam <- colnames(TE.i)
    rnam <- rownames(TE.i)
    selc <- cnam == reference.group
    selr <- rnam == reference.group
    ##
    if (baseline.reference) {
      random <- rbind(random,
                      data.frame(name = name[i],
                                 treat = cnam,
                                 ##
                                 TE = TE.i[, selc],
                                 seTE = seTE.i[, selc],
                                 lower = lower.i[, selc],
                                 upper = upper.i[, selc],
                                 statistic = statistic.i[, selc],
                                 pval = pval.i[, selc],
                                 #
                                 method = args[[i]]$method[1],
                                 #
                                 col.study = col.study[i],
                                 col.square = col.square[i],
                                 col.square.lines = col.square.lines[i],
                                 col.inside = col.inside[i],
                                 ##
                                 stringsAsFactors = FALSE)
                      )
    }
    else {
      random <- rbind(random,
                      data.frame(name = name[i],
                                 treat = cnam,
                                 ##
                                 TE = TE.i[selr, ],
                                 seTE = seTE.i[selr, ],
                                 lower = lower.i[selr, ],
                                 upper = upper.i[selr, ],
                                 statistic = statistic.i[selr, ],
                                 pval = pval.i[selr, ],
                                 #
                                 method = args[[i]]$method[1],
                                 #
                                 col.study = col.study[i],
                                 col.square = col.square[i],
                                 col.square.lines = col.square.lines[i],
                                 col.inside = col.inside[i],
                                 ##
                                 stringsAsFactors = FALSE)
                      )
    }
  }
  #
  rownames(random) <- seq_len(nrow(random))
  
  
  # Replace common effects with random effects results and vice versa
  # if (C)NMA was based on random or common effects model only
  #
  if (missing.common & missing.random) {
    for (i in seq_along(commons)) {
      if (!commons[i] & randoms[i]) {
        common[common$name == name[i], ] <-
          random[random$name == name[i], ]
      }
      else if (commons[i] & !randoms[i]) {
        random[random$name == name[i], ] <-
          common[common$name == name[i], ]
      }
    }
  }
  
  res <- list(common = common,
              random = random,
              sm = sms[1],
              ##
              x = list(common = common.nma, random = random.nma,
                       level.ma = levs[1]),
              ##
              backtransf = backtransf,
              reference.group = reference.group,
              baseline.reference = baseline.reference,
              ##
              version = packageDescription("netmeta")$Version)
  #
  res$x$commons <- commons
  res$x$randoms <- randoms
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  res$x$fixed <- res$x$common
  ##
  class(res) <- "netbind"
  
  
  ##
  ## Print warnings
  ##
  if (print.warning1)
    warning(warning1, call. = FALSE)
  if (print.warning2)
    warning(warning2, call. = FALSE)
  if (print.warning3)
    warning(warning3, call. = FALSE)
  if (print.warning4)
    warning(warning4, call. = FALSE)


  res
}

#' Summary method for objects of class netcomb
#' 
#' @description
#' Summary method for objects of class \code{netcomb}.
#' 
#' @param object An object of class \code{netcomb}.
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots.
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique component names.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @return
#'
#' A list of class "summary.netcomb" is returned with the following elements:
#' 
#' \item{k}{Total number of studies.}
#' \item{m}{Total number of pairwise comparisons.}
#' \item{n}{Total number of treatments.}
#' \item{d}{Total number of designs (corresponding to the unique set
#'   of treatments compared within studies).}
#' \item{c}{Total number of components.}
#' \item{s}{Total number of subnetworks (for \code{\link{discomb}}).}
#' 
#' \item{trts}{Treatments included in network meta-analysis.}
#' \item{k.trts}{Number of studies evaluating a treatment.}
#' \item{n.trts}{Number of observations receiving a treatment (if
#'   available).}
#' \item{events.trts}{Number of events observed for a treatment (if
#'   available).}
#' 
#' \item{studies}{Study labels coerced into a factor with its levels
#'   sorted alphabetically.}
#' \item{narms}{Number of arms for each study.}
#' 
#' \item{designs}{Vector with unique designs present in the network. A
#'   design corresponds to the set of treatments compared within a
#'   study.}
#' 
#' \item{comps}{Components included in network meta-analysis.}
#' \item{k.comps}{Number of studies evaluating a component.}
#' \item{n.comps}{Number of observations receiving a component (if
#'   available).}
#' \item{events.comps}{Number of events observed for a component (if
#'   available).}
#' 
#' \item{comparison}{Results for pairwise comparisons (data frame with
#'   columns studlab, treat1, treat2, TE, seTE, lower, upper, z, p).}
#' \item{comparison.nma.common}{Results for pairwise comparisons based
#'   on common effects NMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p, leverage).}
#' \item{comparison.nma.random}{Results for pairwise comparisons based
#'   on random effects NMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p).}
#' \item{comparison.cnma.common}{Results for pairwise comparisons based
#'   on common effects CNMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p, leverage).}
#' \item{comparison.cnma.random}{Results for pairwise comparisons based
#'   on random effects CNMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p).}
#' 
#' \item{components.common}{Results for components based
#'   on common effects CNMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p, leverage).}
#' \item{components.random}{Results for components based
#'   on random effects CNMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p).}
#' 
#' \item{combinations.common}{Results for available combinations based
#'   on common effects CNMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p, leverage).}
#' \item{combinations.random}{Results for available combinations based
#'   on random effects CNMA model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p).}
#' 
#' \item{common}{Results for common effects model (a list with elements
#'   TE, seTE, lower, upper, z, p).}
#' \item{random}{Results for random effects model (a list with
#'   elements TE, seTE, lower, upper, z, p).}
#' \item{predict}{Prediction intervals (a list with elements seTE,
#'   lower, upper).}
#' 
#' \item{Q.additive}{Overall heterogeneity / inconsistency statistic.}
#' \item{df.Q.additive}{Degrees of freedom for test of heterogeneity /
#'   inconsistency.}
#' \item{pval.Q.additive}{P-value for test of heterogeneity / inconsistency.}
#' \item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
#'   limits.}
#' \item{tau}{Square-root of between-study variance.}
#' 
#' \item{Q.additive}{Overall heterogeneity / inconsistency statistic (CNMA model).}
#' \item{df.Q.additive}{Degrees of freedom for test of heterogeneity /
#'   inconsistency (CNMA model).}
#' \item{pval.Q.additive}{P-value for test of heterogeneity / inconsistency
#'   (CNMA model).}
#' 
#' \item{Q.standard}{Overall heterogeneity statistic (NMA model).}
#' \item{df.Q.heterogeneity}{Degrees of freedom for test of overall
#'   heterogeneity (NMA model).}
#' \item{pval.Q.heterogeneity}{P-value for test of overall
#'   heterogeneity (NMA model).}
#' 
#' \item{Q.diff}{Q statistic for difference between CNMA and NMA model.}
#' \item{df.Q.diff, pval.Q.diff}{Corresponding degrees of freedom and p-value.}
#'
#' \item{sm}{A character string indicating underlying summary
#'   measure.}
#' \item{method}{A character string indicating which method is to be
#'   used for pooling of studies.}
#' \item{level}{The level used to calculate confidence intervals for
#'   individual studies.}
#' \item{level.ma}{The level used to calculate confidence intervals
#'   for pooled estimates.}
#' 
#' \item{ci.lab}{Label for confidence interval.}
#' 
#' \item{reference.group, baseline.reference}{As defined above.}
#' \item{all.treatments}{As defined above.}
#' \item{seq}{A character specifying the sequence of treatments.}
#' 
#' \item{tau.preset}{An optional value for the square-root of the
#'   between-study variance \eqn{\tau^2}.}
#' 
#' \item{sep.comps}{A character used in comparison names as separator
#'   between component labels.}
#' \item{nchar.comps}{A numeric defining the minimum number of
#'   characters used to create unique component names.}
#' 
#' \item{overall.hetstat, backtransf}{As defined above.}
#'  
#' \item{title}{Title of meta-analysis / systematic review.}
#' \item{x}{\code{\link{netmeta}} object (if available).}
#' \item{call}{Function call.}
#' \item{version}{Version of R package \bold{netmeta} used to create object.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{discomb}}
#' 
#' @examples
#' # Examples: example(netcomb)
#' 
#' @method summary netcomb
#' @export

summary.netcomb <- function(object,
                            common = object$common,
                            random = object$random,
                            overall.hetstat = object$overall.hetstat,
                            backtransf = object$backtransf,
                            nchar.comps = object$nchar.comps,
                            warn.deprecated = gs("warn.deprecated"),
                            ...) {
  
  ##
  ##
  ## (1) Check for netcomb object
  ##
  ##
  chkclass(object, "netcomb")
  object <- updateversion(object)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chklogical(overall.hetstat)
  chklogical(backtransf)
  nchar.comps <- replaceNULL(nchar.comps, 666)
  chknumeric(nchar.comps, min = 1, length = 1)
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (3) Summarise results for individual studies and network
  ##     meta-analyses
  ##
  ##
  keepvars <- c("TE", "seTE", "lower", "upper", "statistic", "p")
  ##
  ci.comp <- data.frame(studlab = object$studlab,
                        treat1 = object$treat1, treat2 = object$treat2,
                        ci(object$TE, object$seTE, object$level)[keepvars],
                        stringsAsFactors = FALSE)
  ##
  ci.nma.common <- data.frame(studlab = object$studlab,
                              treat1 = object$treat1,
                              treat2 = object$treat2,
                              TE = object$TE.nma.common,
                              seTE = object$seTE.nma.common,
                              lower = object$lower.nma.common,
                              upper = object$upper.nma.common,
                              statistic = object$statistic.nma.common,
                              p = object$pval.nma.common,
                              stringsAsFactors = FALSE)
  ##
  ci.cnma.common <- data.frame(studlab = object$studlab,
                               treat1 = object$treat1,
                               treat2 = object$treat2,
                               TE = object$TE.cnma.common,
                               seTE = object$seTE.cnma.common,
                               lower = object$lower.cnma.common,
                               upper = object$upper.cnma.common,
                               statistic = object$statistic.cnma.common,
                               p = object$pval.cnma.common,
                               stringsAsFactors = FALSE)
  ##
  ci.nma.random <- data.frame(studlab = object$studlab,
                              treat1 = object$treat1,
                              treat2 = object$treat2,
                              TE = object$TE.nma.random,
                              seTE = object$seTE.nma.random,
                              lower = object$lower.nma.random,
                              upper = object$upper.nma.random,
                              statistic = object$statistic.nma.random,
                              p = object$pval.nma.random,
                              stringsAsFactors = FALSE)
  ##
  ci.cnma.random <- data.frame(studlab = object$studlab,
                               treat1 = object$treat1,
                               treat2 = object$treat2,
                               TE = object$TE.cnma.random,
                               seTE = object$seTE.cnma.random,
                               lower = object$lower.cnma.random,
                               upper = object$upper.cnma.random,
                               statistic = object$statistic.cnma.random,
                               p = object$pval.cnma.random,
                               stringsAsFactors = FALSE)
  ##
  ci.f <- list(TE = object$TE.common,
               seTE = object$seTE.common,
               lower = object$lower.common,
               upper = object$upper.common,
               statistic = object$statistic.common,
               p = object$pval.common)
  ##
  ci.r <- list(TE = object$TE.random,
               seTE = object$seTE.random,
               lower = object$lower.random,
               upper = object$upper.random,
               statistic = object$statistic.random,
               p = object$pval.random)
  ##
  ci.comp.f <- data.frame(TE = object$Comp.common,
                          seTE = object$seComp.common,
                          lower = object$lower.Comp.common,
                          upper = object$upper.Comp.common,
                          statistic = object$statistic.Comp.common,
                          p = object$pval.Comp.common,
                          stringsAsFactors = FALSE)
  rownames(ci.comp.f) <- object$comps
  ##
  ci.comp.r <- data.frame(TE = object$Comp.random,
                          seTE = object$seComp.random,
                          lower = object$lower.Comp.random,
                          upper = object$upper.Comp.random,
                          statistic = object$statistic.Comp.random,
                          p = object$pval.Comp.random,
                          stringsAsFactors = FALSE)
  rownames(ci.comp.r) <- object$comps
  ##
  ci.comb.f <- data.frame(TE = object$Comb.common,
                          seTE = object$seComb.common,
                          lower = object$lower.Comb.common,
                          upper = object$upper.Comb.common,
                          statistic = object$statistic.Comb.common,
                          p = object$pval.Comb.common,
                          stringsAsFactors = FALSE)
  rownames(ci.comb.f) <- object$trts
  ##
  ci.comb.r <- data.frame(TE = object$Comb.random,
                          seTE = object$seComb.random,
                          lower = object$lower.Comb.random,
                          upper = object$upper.Comb.random,
                          statistic = object$statistic.Comb.random,
                          p = object$pval.Comb.random,
                          stringsAsFactors = FALSE)
  rownames(ci.comb.r) <- object$trts
  
  
  ##
  ##
  ## (4) Create summary.netmeta object
  ##
  ##
  res <- list(k = object$k,
              m = object$m,
              n = object$n,
              d = object$d,
              c = object$c,
              s = object$s,
              ##
              trts = object$trts,
              k.trts = object$k.trts,
              n.trts = object$n.trts,
              events.trts = object$events.trts,
              ##
              studies = object$studies,
              narms = object$narms,
              ##
              designs = object$designs,
              ##
              comps = object$comps,
              k.comps = object$k.comps,
              n.comps = object$n.comps,
              events.comps = object$events.comps,
              ##
              comparison = ci.comp,
              comparison.nma.common = ci.nma.common,
              comparison.nma.random = ci.nma.random,
              comparison.cnma.common = ci.cnma.common,
              comparison.cnma.random = ci.cnma.random,
              ##
              components.common = ci.comp.f,
              components.random = ci.comp.r,
              ##
              combinations.common = ci.comb.f,
              combinations.random = ci.comb.r,
              ##
              common = ci.f, random = ci.r,
              ##
              Q.additive = object$Q.additive,
              df.Q.additive = object$df.Q.additive,
              pval.Q.additive = object$pval.Q.additive,
              I2 = object$I2,
              lower.I2 = object$lower.I2, upper.I2 = object$upper.I2,
              tau = object$tau,
              ##
              Q.standard = object$Q.standard,
              df.Q.standard = object$df.Q.standard,
              pval.Q.standard = object$pval.Q.standard,
              ##
              Q.diff = object$Q.diff,
              df.Q.diff = object$df.Q.diff,
              pval.Q.diff = object$pval.Q.diff, 
              ##
              sm = object$sm,
              method = object$method,
              level = object$level,
              level.ma = object$level.ma,
              ##
              ci.lab = paste0(round(100 * object$level.ma, 1),"% CI"),
              ##
              reference.group = object$reference.group,
              baseline.reference = object$baseline.reference,
              all.treatments = object$all.treatments,
              seq = object$seq,
              ##
              tau.preset = object$tau.preset,
              ##
              sep.comps = object$sep.comps,
              nchar.comps = nchar.comps,
              ##
              inactive = object$inactive,
              #
              incr = object$incr,
              method.incr = object$method.incr,
              allstudies = object$allstudies,
              cc.pooled = object$cc.pooled,
              #
              overall.hetstat = overall.hetstat,
              backtransf = backtransf,
              #
              title = object$title,
              ##
              x = object,
              ##
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )
  #
  res$x$common <- common
  res$x$random <- random
  res$x$overall.hetstat <- overall.hetstat
  res$x$backtransf <- backtransf
  res$x$nchar.comps <- nchar.comps
  #
  class(res) <- c(if (inherits(object, "discomb")) "summary.discomb",
                  "summary.netcomb")
  
  res
}

#' Network meta-analysis using graph-theoretical method
#' 
#' @description
#' Network meta-analysis is a generalisation of pairwise meta-analysis
#' that compares all pairs of treatments within a number of treatments
#' for the same condition. The graph-theoretical approach for network
#' meta-analysis uses methods that were originally developed in
#' electrical network theory. It has been found to be equivalent to
#' the frequentist approach to network meta-analysis which is based on
#' weighted least squares regression (Rücker, 2012). An extension for
#' multi-arm studies with correlated treatment arms is also implemented
#' (Rücker & Schwarzer, 2025, unpublished).
#' 
#' @param TE Estimate of treatment effect, i.e. difference between
#'   first and second treatment (e.g. log odds ratio, mean difference,
#'   or log hazard ratio). Or an R object created with
#'   \code{\link[meta]{pairwise}}.
#' @param seTE Standard error of treatment estimate.
#' @param treat1 Label/Number for first treatment.
#' @param treat2 Label/Number for second treatment.
#' @param studlab An optional - but important! - vector with study
#'   labels (see Details).
#' @param data An optional data frame containing the study
#'   information.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param correlated An optional logical vector specifying whether
#'   treatment arms of a multi-arm study are correlated (see Details).
#' @param sm A character string indicating underlying summary measure,
#'   e.g., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"},
#'   \code{"HR"}, \code{"MD"}, \code{"SMD"}, or \code{"ROM"}.
#' @param level The level used to calculate confidence intervals for
#'   individual comparisons.
#' @param level.ma The level used to calculate confidence intervals
#'   for network estimates.
#' @param common A logical indicating whether a common effects network
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects network
#'   meta-analysis should be conducted.
#' @param prediction A logical indicating whether prediction intervals
#'   should be printed.
#' @param level.predict The level used to calculate prediction
#'   intervals for a new study.
#' @param reference.group Reference treatment (first treatment is used
#'   if argument is missing).
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect (passed on to
#'   \code{\link{netrank}}, can be abbreviated.
#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
#'   matrices with all treatment effects, and confidence limits will
#'   be printed.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments in printouts.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau}. Either \code{"DL"}, \code{"REML"}, or
#'   \code{"ML"}, can be abbreviated.
#' @param tau.preset An optional value for manually setting the
#'   square-root of the between-study variance \eqn{\tau^2}.
#' @param tol.multiarm A numeric for the tolerance for consistency of
#'   treatment estimates in multi-arm studies which are consistent by
#'   design.
#' @param tol.multiarm.se A numeric for the tolerance for consistency
#'   of standard errors in multi-arm studies which are consistent by
#'   design. This check is not conducted if the argument is
#'   \code{NULL}.
#' @param details.chkmultiarm A logical indicating whether treatment
#'   estimates and / or variances of multi-arm studies with
#'   inconsistent results or negative multi-arm variances should be
#'   printed.
#' @param sep.trts A character used in comparison names as separator
#'   between treatment labels.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param nchar.studlab A numeric defining the minimum number of
#'   characters used to create unique study labels.
#' @param func.inverse R function used to calculate the pseudoinverse
#'   of the Laplacian matrix L (see Details).
#' @param n1 Number of observations in first treatment group.
#' @param n2 Number of observations in second treatment group.
#' @param event1 Number of events in first treatment group.
#' @param event2 Number of events in second treatment group.
#' @param incr Numerical value added to cell frequencies to construct the
#'   covariance matrix used to calculate the REML or ML estimate of the
#'   between-study variance \eqn{\tau^2} (only for binary or count outcomes).
#' @param mean1 Mean in first treatment group.
#' @param mean2 Mean in second treatment group.
#' @param sd1 Standard deviation in first treatment group.
#' @param sd2 Standard deviation in second treatment group.
#' @param time1 Person time at risk in first treatment group.
#' @param time2 Person time at risk in second treatment group.
#' @param title Title of meta-analysis / systematic review.
#' @param keepdata A logical indicating whether original data(set)
#'   should be kept in netmeta object.
#' @param keeprma A logical indicating whether \code{\link[metafor]{rma.mv}}
#'   object should be stored.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.mv}}.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if studies are excluded from meta-analysis due to zero
#'   standard errors).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param nchar Deprecated argument (replaced by \code{nchar.trts}).
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' Network meta-analysis using R package \bold{netmeta} is described
#' in detail in Schwarzer et al. (2015), Chapter 8.
#' 
#' \subsection{Network meta-analysis model}{
#' Let \emph{n} be the number of different treatments (nodes,
#' vertices) in a network and let \emph{m} be the number of existing
#' comparisons (edges) between the treatments. If there are only
#' two-arm studies, \emph{m} is the number of studies. Let TE and seTE
#' be the vectors of observed effects and their standard errors. Let W
#' be the \emph{m}x\emph{m} diagonal matrix that contains the inverse
#' variance 1 / seTE^2.
#' 
#' The given comparisons define the network structure. Therefrom an
#' \emph{m}x\emph{n} design matrix X (edge-vertex incidence matrix) is
#' formed; for more precise information, see Rücker (2012). Moreover,
#' the \emph{n}x\emph{n} Laplacian matrix L and its Moore-Penrose
#' pseudoinverse L+ are calculated (both matrices play an important
#' role in graph theory and electrical network theory). Using these
#' matrices, the variances based on both direct and indirect
#' comparisons can be estimated. Moreover, the hat matrix H can be
#' estimated by \strong{H = XL+X^tW = X(X^t W X)^+X^tW} and finally
#' consistent treatment effects can be estimated by applying the hat
#' matrix to the observed (potentially inconsistent) effects. H is a
#' projection matrix which maps the observed effects onto the
#' consistent (n-1)-dimensional subspace. This is the Aitken estimator
#' (Senn et al., 2013). As in pairwise meta-analysis, the Q statistic
#' measures the deviation from consistency. Q can be separated into
#' parts for each pairwise meta-analysis and a part for remaining
#' inconsistency between comparisons.
#' 
#' A simple random effects model assuming that a constant
#' heterogeneity variance is added to each comparison of the network
#' can be defined via a generalised methods of moments estimate of the
#' between-studies variance \eqn{\tau^2} (Jackson et al., 2012). This
#' is added to the observed sampling variance \code{seTE^2} of each
#' comparison in the network (before appropriate adjustment for
#' multi-arm studies). Then, as in standard pairwise meta-analysis,
#' the procedure is repeated with the resulting enlarged standard
#' errors.
#' 
#' For the random-effects model, the direct treatment estimates are
#' based on the common between-study variance \eqn{\tau^2} from the
#' network meta-analysis.
#' }
#' 
#' \subsection{Multi-arm studies}{
#' Often multi-arm studies are included in a network meta-analysis.
#' In multi-arm studies, the treatment effects on different
#' comparisons are not independent, but correlated. This is accounted
#' for by reweighting all comparisons of each multi-arm study. The
#' method is described in Rücker (2012) and Rücker and Schwarzer
#' (2014).
#' 
#' Comparisons belonging to multi-arm studies are identified by
#' identical study labels (argument \code{studlab}). It is therefore
#' important to use identical study labels for all comparisons
#' belonging to the same multi-arm study, e.g., study label
#' "Willms1999" for the three-arm study in the data example (Senn et
#' al., 2013). The function netmeta then automatically accounts for
#' within-study correlation by reweighting all comparisons of each
#' multi-arm study.
#' 
#' Note, all pairwise comparisons must be provided for a multi-arm
#' study. Consider a multi-arm study of \emph{p} treatments with known
#' variances. For this study, treatment effects and standard errors
#' must be provided for each of \emph{p}(\emph{p} - 1) / 2 possible
#' comparisons. For instance, a three-arm study contributes three
#' pairwise comparisons, a four-arm study even six pairwise
#' comparisons. Function \code{\link[meta]{pairwise}} automatically
#' calculates all pairwise comparisons for multi-arm studies.
#' }
#' 
#' \subsection{Studies with correlated treatment arms}{
#' The network meta-analysis method described in Rücker (2012) assumes that the
#' treatment arms of a study are independent. This assumption is justified for
#' parallel design trials and leads to variance consistency in multi-arm trials.
#' However, this assumption is violated for trials with correlated arms, for
#' example, split-body trials. For these trials, the variance of a contrast is
#' not the sum of the arm-based variances, but comes with a correlation term
#' which may lead to the violation of variance consistency. Using argument
#' \code{correlated}, an alternative method is used for studies with correlated
#' treatment arms (Rücker & Schwarzer, 2025, unpublished).
#' 
#' Argument \code{correlated}, which must be of the same length as arguments
#' \code{TE}, \code{seTE}, etc., is a logical vector which must be TRUE for
#' trials with correlated arms and FALSE otherwise. This can be easily done
#' using a variable with design information like
#' \code{correlated = design != "Parallel"}.
#' }
#' 
#' \subsection{Data formats}{
#' Data entry for this function is in \emph{contrast-based} format,
#' that is, data are given as contrasts (differences) between two
#' treatments (argument \code{TE}) with standard error (argument
#' \code{seTE}). In principle, meta-analysis functions from R package
#' \bold{meta}, e.g. \code{\link[meta]{metabin}} for binary outcomes or
#' \code{\link[meta]{metacont}} for continuous outcomes, can be used to
#' calculate treatment effects separately for each treatment
#' comparison which is a rather tedious enterprise. If data are
#' provided in \emph{arm-based} format, that is, data are given for
#' each treatment arm separately (e.g. number of events and
#' participants for binary outcomes), a much more convenient way to
#' transform data into contrast-based form is available. Function
#' \code{\link[meta]{pairwise}} can automatically transform data with binary
#' outcomes (using the \code{\link[meta]{metabin}} function from R package
#' \bold{meta}), continuous outcomes (\code{\link[meta]{metacont}}
#' function), incidence rates (\code{\link[meta]{metainc}} function), and
#' generic outcomes (\code{\link[meta]{metagen}} function). Additional
#' arguments of these functions can be provided (see help page of
#' function \code{\link[meta]{pairwise}}).
#' }
#' 
#' \subsection{Additional comments}{
#' Internally, both common and random effects models are calculated
#' regardless of values chosen for arguments \code{common} and
#' \code{random}. Accordingly, the network estimates for the random
#' effects model can be extracted from component \code{TE.random} of
#' an object of class \code{"netmeta"} even if argument \code{random =
#' FALSE}. However, all functions in R package \bold{netmeta} will
#' adequately consider the values for \code{common} and
#' \code{random}. E.g. function \code{\link{print.summary.netmeta}}
#' will not print results for the random effects model if \code{random
#' = FALSE}.
#' 
#' By default, treatment names are not abbreviated in
#' printouts. However, in order to get more concise printouts,
#' argument \code{nchar.trts} can be used to define the minimum number
#' of characters for abbreviated treatment names (see
#' \code{\link{abbreviate}}, argument \code{minlength}). R function
#' \code{\link{treats}} is utilised internally to create abbreviated
#' treatment names.
#' 
#' Names of treatment comparisons are created by concatenating
#' treatment labels of pairwise comparisons using \code{sep.trts} as
#' separator (see \code{\link{paste}}). These comparison names are
#' used in the covariance matrices \code{Cov.common} and
#' \code{Cov.random} and in some R functions, e.g,
#' \code{\link{decomp.design}}. By default, a colon is used as the
#' separator. If any treatment label contains a colon the following
#' characters are used as separator (in consecutive order):
#' \code{"-"}, \code{"_"}, \code{"/"}, \code{"+"}, \code{"."},
#' \code{"|"}, and \code{"*"}. If all of these characters are used in
#' treatment labels, a corresponding error message is printed asking
#' the user to specify a different separator.
#' }
#'
#' @note
#' R function \code{\link[metafor]{rma.mv}} from R package
#' \pkg{metafor} (Viechtbauer 2010) is called internally to estimate
#' the between-study variance \eqn{\tau^2} for the (restricted)
#' maximum likelihood method. For binary outcomes, incidence rates,
#' and the mean difference, the variance-covariance matrix is
#' calculated if arguments \code{event1}, \code{event2}, \code{n1},
#' and \code{n2} (binary outcomes); \code{event1}, \code{event2},
#' \code{time1}, and \code{time2} (incidence rates); \code{n1},
#' \code{n2}, \code{sd1}, and \code{sd2} (mean difference) are
#' provided. For datasets preprocessed with \code{\link[meta]{pairwise}}
#' the respective variables are selected automatically.
#'
#' @return
#' An object of class \code{netmeta} with corresponding \code{print},
#' \code{summary}, \code{forest}, and \code{netrank} functions. The
#' object is a list containing the following components:
#' \item{studlab, treat1, treat2, TE, seTE}{As defined above.}
#' \item{seTE.adj.common, seTE.adj.random}{Standard error of treatment
#'   estimate, adjusted for multi-arm studies.}
#' \item{design}{Design of study providing pairwise comparison.}
#' \item{n1, n2, event1, event2, incr}{As defined above.}
#' \item{mean1, mean2, sd1, sd2, time1, time2}{As defined above.}
#' \item{sd1, sd2, time1, time2}{As defined above.}
#' \item{k}{Total number of studies.}
#' \item{m}{Total number of pairwise comparisons.}
#' \item{n}{Total number of treatments.}
#' \item{d}{Total number of designs (corresponding to the unique set
#'   of treatments compared within studies).}
#' \item{trts}{Treatments included in network meta-analysis.}
#' \item{k.trts}{Number of studies evaluating a treatment.}
#' \item{n.trts}{Number of observations receiving a treatment (if
#'   arguments \code{n1} and \code{n2} are provided).}
#' \item{events.trts}{Number of events observed for a treatment (if
#'   arguments \code{event1} and \code{event2} are provided).}
#' \item{multiarm}{Logical vector to identify pairwise comparisons
#'   from multi-arm studies.}
#' \item{n.arms}{Number of treatment arms in study providing pairwise
#'   comparison.}
#' \item{studies}{Vector with unique study labels.}
#' \item{narms}{Number of arms for each study.}
#' \item{designs}{Vector with unique designs present in the network. A
#'   design corresponds to the set of treatments compared within a
#'   study.}
#' \item{comparisons}{Vector with unique direct comparisons present in the
#'   network.}
#' \item{TE.nma.common, TE.nma.random}{A vector of length \emph{m} of
#'   consistent treatment effects estimated by network meta-analysis
#'   (nma) (common / random effects model).}
#' \item{seTE.nma.common, seTE.nma.random}{A vector of length \emph{m}
#'   of effective standard errors estimated by network meta-analysis
#'   (common / random effects model).}
#' \item{lower.nma.common, lower.nma.random}{A vector of length
#'   \emph{m} of lower confidence interval limits for consistent
#'   treatment effects estimated by network meta-analysis (common
#'   effects / random effects model).}
#' \item{upper.nma.common, upper.nma.random}{A vector of length
#'   \emph{m} of upper confidence interval limits for the consistent
#'   treatment effects estimated by network meta-analysis (common
#'   effects / random effects model).}
#' \item{statistic.nma.common, statistic.nma.random}{A vector of length
#'   \emph{m} of z-values for test of treatment effect for individual
#'   comparisons (common / random effects model).}
#' \item{pval.nma.common, pval.nma.random}{A vector of length \emph{m}
#'   of p-values for test of treatment effect for individual
#'   comparisons (common / random effects model).}
#' \item{leverage.common}{A vector of length \emph{m} of leverages,
#'   interpretable as factors by which variances are reduced using
#'   information from the whole network.}
#' \item{w.common, w.random}{A vector of length \emph{m} of weights of
#'   individual studies (common / random effects model).}
#' \item{Q.common}{A vector of length \emph{m} of contributions to
#'   total heterogeneity / inconsistency statistic.}
#' \item{TE.common, TE.random}{\emph{n}x\emph{n} matrix with estimated
#'   overall treatment effects (common / random effects model).}
#' \item{seTE.common, seTE.random}{\emph{n}x\emph{n} matrix with
#'   standard errors (common / random effects model).}
#' \item{lower.common, upper.common, lower.random,
#'   upper.random}{\emph{n}x\emph{n} matrices with lower and upper
#'   confidence interval limits (common / random effects
#'   model).}
#' \item{statistic.common, pval.common, statistic.random,
#'   pval.random}{\emph{n}x\emph{n} matrices with z-value and p-value
#'   for test of overall treatment effect (common / random
#'   effects model).}
#' \item{seTE.predict}{\emph{n}x\emph{n} matrix with standard errors
#'   for prediction intervals.}
#' \item{lower.predict, upper.predict}{\emph{n}x\emph{n} matrices with
#'   lower and upper prediction interval limits.}
#' \item{prop.direct.common, prop.direct.random}{A named vector of the
#'   direct evidence proportion of each network estimate. (common
#'   effects / random effects model).}
#' \item{TE.direct.common, TE.direct.random}{\emph{n}x\emph{n} matrix
#'   with estimated treatment effects from direct evidence (common
#'   effects / random effects model).}
#' \item{seTE.direct.common, seTE.direct.random}{\emph{n}x\emph{n}
#'   matrix with estimated standard errors from direct evidence (common
#'   effects / random effects model).}
#' \item{lower.direct.common, upper.direct.common, lower.direct.random,
#'   upper.direct.random}{\emph{n}x\emph{n} matrices with lower and upper
#'   confidence interval limits from direct evidence (common / random
#'   effects model).}
#' \item{statistic.direct.common, pval.direct.common, statistic.direct.random,
#'   pval.direct.random}{
#'   \emph{n}x\emph{n} matrices with z-value and p-value for test of overall
#'   treatment effect from direct evidence (common / random effects model).}
#' \item{TE.indirect.common, TE.indirect.random}{\emph{n}x\emph{n}
#'   matrix with estimated treatment effects from indirect evidence
#'   (common / random effects model).}
#' \item{seTE.indirect.common, seTE.indirect.random}{\emph{n}x\emph{n}
#'   matrix with estimated standard errors from indirect evidence
#'   (common / random effects model).}
#' \item{lower.indirect.common, upper.indirect.common,
#'   lower.indirect.random, upper.indirect.random}{
#'   \emph{n}x\emph{n} matrices with lower and upper confidence interval limits
#'   from indirect evidence (common / random effects model).}
#' \item{statistic.indirect.common, pval.indirect.common,
#'   statistic.indirect.random, pval.indirect.random}{
#'   \emph{n}x\emph{n} matrices with z-value and p-value for test of overall
#'   treatment effect from indirect evidence (common / random effects model).}
#' \item{Q}{Overall heterogeneity / inconsistency statistic.}
#' \item{df.Q}{Degrees of freedom for test of heterogeneity /
#'   inconsistency.}
#' \item{pval.Q}{P-value for test of heterogeneity / inconsistency.}
#' \item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
#'   limits.}
#' \item{tau}{Square-root of between-study variance.}
#' \item{Q.heterogeneity}{Overall heterogeneity statistic.}
#' \item{df.Q.heterogeneity}{Degrees of freedom for test of overall
#'   heterogeneity.}
#' \item{pval.Q.heterogeneity}{P-value for test of overall
#'   heterogeneity.}
#' \item{Q.inconsistency}{Overall inconsistency statistic.}
#' \item{df.Q.inconsistency}{Degrees of freedom for test of overall
#'   inconsistency.}
#' \item{pval.Q.inconsistency}{P-value for test of overall
#'   inconsistency.}
#' \item{Q.decomp}{Data frame with columns 'treat1', 'treat2', 'Q',
#'   'df' and 'pval.Q', providing heterogeneity statistics for each
#'   pairwise meta-analysis of direct comparisons.}
#' \item{A.matrix}{Adjacency matrix (\emph{n}x\emph{n}).}
#' \item{X.matrix}{Design matrix (\emph{m}x\emph{n}).}
#' \item{B.matrix}{Edge-vertex incidence matrix (\emph{m}x\emph{n}).}
#' \item{L.matrix.common, L.matrix.random}{Laplacian matrix
#'   (\emph{n}x\emph{n}).}
#' \item{Lplus.matrix.common, Lplus.matrix.random}{Moore-Penrose
#'   pseudoinverse of the Laplacian matrix (\emph{n}x\emph{n}).}
#' \item{Q.matrix}{Matrix of heterogeneity statistics for pairwise
#'   meta-analyses, where direct comparisons exist
#'   (\emph{n}x\emph{n}).}
#' \item{G.matrix}{Matrix with variances and covariances of
#'   comparisons (\emph{m}x\emph{m}). G is defined as
#'   \strong{BL+B^t}.}
#' \item{H.matrix.common, H.matrix.random}{Hat matrix
#'   (\emph{m}x\emph{m}), defined as \strong{H = GW = BL+B^tW}.}
#' \item{n.matrix}{\emph{n}x\emph{n} matrix with number of
#'   observations in direct comparisons (if arguments \code{n1} and
#'   \code{n2} are provided).}
#' \item{events.matrix}{\emph{n}x\emph{n} matrix with number of events
#'   in direct comparisons (if arguments \code{event1} and
#'   \code{event2} are provided).}
#' \item{P.common, P.random}{\emph{n}x\emph{n} matrix with direct
#'   evidence proportions (common / random effects model).}
#' \item{Cov.common}{Variance-covariance matrix (common effects model)}
#' \item{Cov.random}{Variance-covariance matrix (random effects model)}
#' \item{sm, level, level.ma}{As defined above.}
#' \item{common, random}{As defined above.}
#' \item{prediction, level.predict}{As defined above.}
#' \item{reference.group, baseline.reference, small.values,
#'   all.treatments}{As defined above.}
#' \item{seq, tau.preset, tol.multiarm, tol.multiarm.se}{As defined
#'   above.}
#' \item{details.chkmultiarm, sep.trts, nchar.trts}{As defined above.}
#' \item{backtransf, title, warn, warn.deprecated}{As defined above.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package netmeta used to create object.}
#'
#' In addition, the following component is stored if \bold{metafor} is
#' used to calculate the between-study variance and argument
#' \code{keeprma = TRUE}:
#' \item{rma.tau}{R object created with \code{\link[metafor]{rma.mv}}.}
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link[meta]{pairwise}}, \code{\link{forest.netmeta}},
#'   \code{\link{netrank}}, \code{\link[meta]{metagen}},
#'   \code{\link{smokingcessation}}, \code{\link{Senn2013}},
#'   \code{\link{dietaryfat}}
#' 
#' @references
#' Jackson D, White IR, Riley RD (2012):
#' Quantifying the impact of between-study heterogeneity in
#' multivariate meta-analyses.
#' \emph{Statistics in Medicine},
#' \bold{31}, 3805--20
#' 
#' Rücker G (2012):
#' Network meta-analysis, electrical networks and graph theory.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 312--24
#' 
#' Rücker G, Schwarzer G (2014):
#' Reduce dimension or reduce weights? Comparing two approaches to
#' multi-arm studies in network meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{33}, 4353--69
#' 
#' Schwarzer G, Carpenter JR, Rücker G (2015):
#' \emph{Meta-Analysis with R (Use R!)}.
#' Springer International Publishing, Switzerland
#' 
#' Senn S, Gavini F, Magrez D, Scheen A (2013):
#' Issues in performing a network meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{22}, 169--89
#' 
#' Viechtbauer W (2010):
#' Conducting Meta-Analyses in R with the metafor Package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#' 
#' @examples
#' #
#' # 1) Smoking cessation example
#' #
#' 
#' data(smokingcessation)
#' 
#' # Transform data from wide arm-based to contrast-based format
#' #
#' pw1 <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma1 <- netmeta(pw1, common = FALSE)
#' nma1
#' 
#' # Draw network graphs
#' #
#' netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.25)
#' tname <- c("No intervention", "Self-help",
#'   "Individual counselling", "Group counselling")
#' netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.25, labels = tname)
#' 
#' # Forest plot
#' #
#' forest(nma1)
#' 
#' \donttest{
#' #
#' # 2) Diabetes example
#' #
#' 
#' # Transform data from long arm-based to contrast-based format
#' #
#' pw2 <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' 
#' # Conduct common effects network meta-analysis
#' #
#' nma2 <- netmeta(pw2, random = FALSE)
#' nma2
#' 
#' \dontrun{
#' # Senn (2013) data set in comparison-based format
#' data(Senn2013)
#' #
#' nma2.2 <- netmeta(TE, seTE, treat1, treat2, studlab,
#'   data = Senn2013, sm = "MD", random = FALSE)
#' nma2.2
#' rm(Senn2013)
#' }
#' 
#' nma2$Q.decomp
#' 
#' # Comparison with reference group
#' #
#' print(nma2, reference = "plac")
#' forest(nma2, reference = "plac")
#' 
#' # Print detailed results
#' #
#' snma2 <- summary(nma2)
#' print(snma2, digits = 3)
#' 
#' # Only show individual study results for multi-arm studies
#' #
#' print(snma2, digits = 3, truncate = multiarm)
#' 
#' # Only show first three individual study results
#' #
#' print(snma2, digits = 3, truncate = 1:3)
#' 
#' # Only show individual study results for 'Kim 2007' and 'Willms 1999'
#' #
#' print(snma2, digits = 3, truncate = c("Kim 2007", "Willms 1999"))
#' 
#' # Only show individual study results for studies starting with the
#' # letter "W"
#' #
#' print(snma2, ref = "plac", digits = 3,
#'   truncate = substring(studlab, 1, 1) == "W")
#'
#' # Conduct random effects network meta-analysis
#' #
#' nma3 <- netmeta(pw2, common = FALSE, reference = "plac")
#' nma3
#' forest(nma3, xlim = c(-1.5, 1), xlab = "HbA1c difference")
#' 
#' # Add column with P-Scores on right side of forest plot
#' #
#' forest(nma3, xlim = c(-1.5, 1),
#'   xlab = "HbA1c difference",
#'   rightcols = c("effect", "ci", "Pscore"),
#'   just.addcols = "right")
#' 
#' # Add column with P-Scores on left side of forest plot
#' #
#' forest(nma3, xlim = c(-1.5, 1),
#'   xlab = "HbA1c difference",
#'   leftcols = c("studlab", "Pscore"),
#'   just.addcols = "right")
#' 
#' # Sort forest plot by descending P-Score
#' #
#' forest(nma3, xlim = c(-1.5, 1),
#'   xlab = "HbA1c difference",
#'   rightcols = c("effect", "ci", "Pscore"),
#'   just.addcols = "right",
#'   sortvar = -Pscore)
#' 
#' # Sort by and print number of studies with direct treatment comparisons
#' #
#' forest(nma3, xlim = c(-1.5, 1),
#'   xlab = "HbA1c difference",
#'   leftcols = c("studlab", "k"),
#'   leftlabs = c("Contrast\nto Placebo", "Direct\nComparisons"),
#'   sortvar = -k,
#'   smlab = "Random Effects Model")
#' 
#' # Change printing order of treatments with placebo last and use
#' # long treatment names
#' #
#' trts <- c("acar", "benf", "metf", "migl", "piog",
#'   "rosi", "sita", "sulf", "vild", "plac")
#' #
#' nma4 <- netmeta(pw2, common = FALSE, reference = "plac", seq = trts)
#' print(nma4, digits = 2)
#' 
#' #
#' # 3) Dietary fat example
#' #
#' 
#' data(dietaryfat)
#' 
#' # Transform data from arm-based format to contrast-based format
#' # Using incidence rate ratios (sm = "IRR") as effect measure.
#' # Note, the argument 'sm' is not necessary as this is the default
#' # in R function metainc() called internally
#' #
#' pw5 <- pairwise(list(treat1, treat2, treat3),
#'   list(d1, d2, d3), time = list(years1, years2, years3),
#'   studlab = ID, data = dietaryfat, sm = "IRR")
#' pw5
#' 
#' # Conduct network meta-analysis
#' #
#' nma5 <- netmeta(pw5)
#' nma5
#' 
#' # Conduct network meta-analysis using incidence rate differences
#' # (sm = "IRD")
#' #
#' pw6 <- pairwise(list(treat1, treat2, treat3),
#'   list(d1, d2, d3), time = list(years1, years2, years3),
#'   studlab = ID, data = dietaryfat, sm = "IRD")
#' nma6 <- netmeta(pw6)
#' nma6
#' 
#' # Draw network graph
#' #
#' netgraph(nma5, points = TRUE, cex.points = 3, cex = 1.25,
#'   labels = c("Control","Diet", "Diet 2"))
#' }
#' 
#' @export netmeta

netmeta <- function(TE, seTE,
                    treat1, treat2, studlab,
                    data = NULL, subset = NULL,
                    correlated,
                    #
                    sm,
                    level = gs("level"),
                    level.ma = gs("level.ma"),
                    common = gs("common"),
                    random = gs("random") | !is.null(tau.preset),
                    #
                    prediction = gs("prediction"),
                    level.predict = gs("level.predict"),
                    #
                    reference.group,
                    baseline.reference = gs("baseline.reference"),
                    small.values = gs("small.values"),
                    all.treatments = gs("all.treatments"),
                    seq = gs("seq"),
                    #
                    method.tau = gs("method.tau.netmeta"),
                    tau.preset = NULL,
                    #
                    tol.multiarm = gs("tol.multiarm"),
                    tol.multiarm.se = gs("tol.multiarm.se"),
                    details.chkmultiarm = gs("details.chkmultiarm"),
                    #
                    sep.trts = gs("sep.trts"),
                    nchar.trts = gs("nchar.trts"),
                    nchar.studlab = gs("nchar.studlab"),
                    #
                    func.inverse = invmat,
                    #
                    n1 = NULL,
                    n2 = NULL,
                    event1 = NULL,
                    event2 = NULL,
                    incr = NULL,
                    mean1 = NULL,
                    mean2 = NULL,
                    sd1 = NULL,
                    sd2 = NULL,
                    time1 = NULL,
                    time2 = NULL,
                    #
                    overall.hetstat = gs("overall.hetstat"),
                    backtransf = gs("backtransf"),
                    #
                    title = gs("title"),
                    keepdata = gs("keepdata"),
                    keeprma = gs("keeprma"),
                    control = NULL,
                    #
                    warn = gs("warn"), warn.deprecated = gs("warn.deprecated"),
                    #
                    nchar = nchar.trts,
                    ...) {


  #
  #
  # (1) Check arguments
  #
  #
  chklevel(level)
  chklevel(level.predict)
  #
  chklogical(prediction)
  #
  missing.reference.group <- missing(reference.group)
  #
  baseline.reference <- replaceNULL(baseline.reference, TRUE)
  chklogical(baseline.reference)
  #
  small.values <- setsv(replaceNULL(small.values, "desirable"))
  #
  if (!is.null(all.treatments))
    chklogical(all.treatments)
  #
  method.tau <- setchar(method.tau, c("DL", "ML", "REML"))
  #
  if (!is.null(tau.preset))
    chknumeric(tau.preset, min = 0, length = 1)
  #
  tol.multiarm <- replaceNULL(tol.multiarm, 0.001)
  chknumeric(tol.multiarm, min = 0, length = 1)
  if (!is.null(tol.multiarm.se))
    chknumeric(tol.multiarm.se, min = 0, length = 1)
  #
  details.chkmultiarm <- replaceNULL(details.chkmultiarm, FALSE)
  chklogical(details.chkmultiarm)
  #
  missing.sep.trts <- missing(sep.trts)
  sep.trts <- replaceNULL(sep.trts, ":")
  chkchar(sep.trts, length = 1)
  #
  nchar.studlab <- replaceNULL(nchar.studlab, 666)
  chknumeric(nchar.studlab, length = 1)
  #
  overall.hetstat <- replaceNULL(overall.hetstat, TRUE)
  chklogical(overall.hetstat)
  chklogical(backtransf)
  #
  chkchar(title)
  chklogical(keepdata)
  chklogical(keeprma)
  chklogical(warn)
  #
  chklogical(baseline.reference)
  #
  # Check for deprecated arguments in '...'
  #
  args <- list(...)
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing(level.ma), args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  #
  missing.nchar.trts <- missing(nchar.trts)
  nchar.trts <- replaceNULL(nchar.trts, 666)
  nchar.trts <-
    deprecated2(nchar.trts, missing.nchar.trts, nchar, missing(nchar),
                warn.deprecated)
  chknumeric(nchar.trts, min = 1, length = 1)
  
  
  #
  #
  # (2) Read data
  #
  #
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata)
    data <- sfsp
  #
  # Catch TE, treat1, treat2, seTE, studlab from data:
  #
  TE <- catch("TE", mc, data, sfsp)
  #
  avail.reference.group.pairwise <- FALSE
  #
  if (inherits(TE, "pairwise")) {
    is.pairwise <- TRUE
    #
    sm <- attr(TE, "sm")
    allstudies <- replaceNULL(attr(TE, "allstudies"), TRUE)
    #
    if (missing.reference.group) {
      reference.group <- attr(TE, "reference.group")
      #
      if (is.null(reference.group))
        reference.group <- ""
      else
        avail.reference.group.pairwise <- TRUE
    }
    #
    keep.all.comparisons <- attr(TE, "keep.all.comparisons")
    if (!is.null(keep.all.comparisons) && !keep.all.comparisons)
      stop("First argument is a pairwise object created with ",
           "'keep.all.comparisons = FALSE'.",
           call. = TRUE)
    #
    if (is.null(attr(TE, "varnames")))
      seTE <- TE$seTE
    else
      seTE <- TE[[attr(TE, "varnames")[2]]]
    #
    treat1 <- TE$treat1
    treat2 <- TE$treat2
    studlab <- TE$studlab
    #
    if (!is.null(TE$n1))
      n1 <- TE$n1
    if (!is.null(TE$n2))
      n2 <- TE$n2
    if (!is.null(TE$event1))
      event1 <- TE$event1
    if (!is.null(TE$event2))
      event2 <- TE$event2
    #
    incr1 <- TE$incr1
    incr2 <- TE$incr2
    if (!is.null(incr2))
      incr <- incr2
    else
      incr <- TE$incr
    #
    if (!is.null(TE$mean1))
      mean1 <- TE$mean1
    if (!is.null(TE$mean2))
      mean2 <- TE$mean2
    if (!is.null(TE$sd1))
      sd1 <- TE$sd1
    if (!is.null(TE$sd2))
      sd2 <- TE$sd2
    if (!is.null(TE$time1))
      time1 <- TE$time1
    if (!is.null(TE$time2))
      time2 <- TE$time2
    #
    pairdata <- TE
    data <- TE
    #
    if (is.null(attr(TE, "varnames")))
      TE <- TE$TE
    else
      TE <- TE[[attr(TE, "varnames")[1]]]
  }
  else {
    is.pairwise <- FALSE
    if (missing(sm))
      if (!is.null(data) && !is.null(attr(data, "sm")))
        sm <- attr(data, "sm")
      else
        sm <- ""
    #
    seTE <- catch("seTE", mc, data, sfsp)
    #
    treat1 <- catch("treat1", mc, data, sfsp)
    treat2 <- catch("treat2", mc, data, sfsp)
    #
    studlab <- catch("studlab", mc, data, sfsp)
    #
    n1 <- catch("n1", mc, data, sfsp)
    n2 <- catch("n2", mc, data, sfsp)
    #
    event1 <- catch("event1", mc, data, sfsp)
    event2 <- catch("event2", mc, data, sfsp)
    #
    incr1 <- NULL
    incr2 <- catch("incr", mc, data, sfsp)
    #
    mean1 <- catch("mean1", mc, data, sfsp)
    mean2 <- catch("mean2", mc, data, sfsp)
    #
    sd1 <- catch("sd1", mc, data, sfsp)
    sd2 <- catch("sd2", mc, data, sfsp)
    #
    time1 <- catch("time1", mc, data, sfsp)
    time2 <- catch("time2", mc, data, sfsp)
  }
  #
  chknumeric(TE)
  chknumeric(seTE)
  #
  if (!any(!is.na(TE) & !is.na(seTE)))
    stop("Missing data for estimates (argument 'TE') and ",
         "standard errors (argument 'seTE') in all studies.\n  ",
         "No network meta-analysis possible.",
         call. = FALSE)
  #
  k.Comp <- length(TE)
  #
  if (is.factor(treat1))
    treat1 <- as.character(treat1)
  if (is.factor(treat2))
    treat2 <- as.character(treat2)
  #
  # Remove leading and trailing whitespace
  #
  treat1 <- rmSpace(rmSpace(treat1, end = TRUE))
  treat2 <- rmSpace(rmSpace(treat2, end = TRUE))
  #
  if (length(studlab) == 0) {
    if (warn)
      warning("No information given for argument 'studlab'. ",
              "Assuming that comparisons are from independent studies.",
              call. = FALSE)
    studlab <- seq(along = TE)
  }
  studlab <- as.character(studlab)
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  correlated <- catch("correlated", mc, data, sfsp)
  if (is.null(correlated))
    correlated <- FALSE
  if (!is.logical(correlated))
    stop("Argument 'correlated' must be a logical vector.", call. = FALSE)
  if (length(correlated) == 1)
    correlated <- rep(correlated, length(TE))
  else if (length(correlated) != length(TE))
    stop("Different length for arguments 'TE' and 'correlated'.", call. = FALSE)
  #
  if (!is.null(event1) & !is.null(event2))
    available.events <- TRUE
  else
    available.events <- FALSE
  #
  if (!is.null(n1) & !is.null(n2))
    available.n <- TRUE
  else
    available.n <- FALSE
  #
  if (available.events & is.null(incr2))
    incr2 <- rep(0, length(event2))
  #
  if (!is.null(mean1) & !is.null(mean2))
    available.means <- TRUE
  else
    available.means <- FALSE
  #
  if (!is.null(sd1) & !is.null(sd2))
    available.sds <- TRUE
  else
    available.sds <- FALSE
  #
  if (!is.null(time1) & !is.null(time2))
    available.times <- TRUE
  else
    available.times <- FALSE
  
  
  #
  #
  # (2b) Store complete dataset in list object data
  #      (if argument keepdata is TRUE)
  #
  #
  if (keepdata) {
    if (nulldata & !is.pairwise)
      data <- data.frame(.studlab = studlab, stringsAsFactors = FALSE)
    else if (nulldata & is.pairwise) {
      data <- pairdata
      data$.studlab <- studlab
    }
    else
      data$.studlab <- studlab
    #
    data$.order <- seq_along(studlab)
    #
    data$.treat1 <- treat1
    data$.treat2 <- treat2
    #
    data$.TE <- TE
    data$.seTE <- seTE
    #
    data$.correlated <- correlated
    #
    data$.event1 <- event1
    data$.n1 <- n1
    data$.event2 <- event2
    data$.n2 <- n2
    data$.incr1 <- incr1
    data$.incr2 <- incr2
    #
    data$.mean1 <- mean1
    data$.sd1 <- sd1
    data$.mean2 <- mean2
    data$.sd2 <- sd2
    #
    data$.time1 <- time1
    data$.time2 <- time2
    #
    # Check for correct treatment order within comparison
    #
    wo <- data$.treat1 > data$.treat2
    #
    if (any(wo)) {
      data$.TE[wo] <- -data$.TE[wo]
      ttreat1 <- data$.treat1
      data$.treat1[wo] <- data$.treat2[wo]
      data$.treat2[wo] <- ttreat1[wo]
      #
      if (isCol(data, ".n1") & isCol(data, ".n2")) {
        tn1 <- data$.n1
        data$.n1[wo] <- data$.n2[wo]
        data$.n2[wo] <- tn1[wo]
      }
      #
      if (isCol(data, ".event1") & isCol(data, ".event2")) {
        tevent1 <- data$.event1
        data$.event1[wo] <- data$.event2[wo]
        data$.event2[wo] <- tevent1[wo]
      }
      #
      if (isCol(data, ".mean1") & isCol(data, ".mean2")) {
        tmean1 <- data$.mean1
        data$.mean1[wo] <- data$.mean2[wo]
        data$.mean2[wo] <- tmean1[wo]
      }
      #
      if (isCol(data, ".sd1") & isCol(data, ".sd2")) {
        tsd1 <- data$.sd1
        data$.sd1[wo] <- data$.sd2[wo]
        data$.sd2[wo] <- tsd1[wo]
      }
      #
      if (isCol(data, ".time1") & isCol(data, ".time2")) {
        ttime1 <- data$.time1
        data$.time1[wo] <- data$.time2[wo]
        data$.time2[wo] <- ttime1[wo]
      }
    }
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
  }
  
  
  #
  #
  # (3) Use subset for analysis
  #
  #
  if (!missing.subset) {
    if ((is.logical(subset) & (sum(subset) > k.Comp)) ||
        (length(subset) > k.Comp))
      stop("Length of subset is larger than number of studies.",
           call. = FALSE)
    #
    TE <- TE[subset]
    seTE <- seTE[subset]
    treat1 <- treat1[subset]
    treat2 <- treat2[subset]
    studlab <- studlab[subset]
    #
    correlated <- correlated[subset]
    #
    if (!is.null(n1))
      n1 <- n1[subset]
    if (!is.null(n2))
      n2 <- n2[subset]
    if (!is.null(event1))
      event1 <- event1[subset]
    if (!is.null(event2))
      event2 <- event2[subset]
    if (!is.null(incr1))
      incr1 <- incr1[subset]
    if (!is.null(incr2))
      incr2 <- incr2[subset]
    if (!is.null(mean1))
      mean1 <- mean1[subset]
    if (!is.null(mean2))
      mean2 <- mean2[subset]
    if (!is.null(sd1))
      sd1 <- sd1[subset]
    if (!is.null(sd2))
      sd2 <- sd2[subset]
    if (!is.null(time1))
      time1 <- time1[subset]
    if (!is.null(time2))
      time2 <- time2[subset]
  }
  #
  labels <- sort(unique(c(treat1, treat2)))
  #
  if (!is.null(seq))
    seq <- setseq(seq, labels)
  else {
    seq <- labels
    if (is.numeric(seq))
      seq <- as.character(seq)
  }
  #
  sep.trts <- setsep(labels, sep.trts, missing = missing.sep.trts)
  
  
  #
  #
  # (4) Additional checks
  #
  #
  if (any(treat1 == treat2))
    stop("Treatments must be different (arguments 'treat1' and 'treat2').",
         call. = FALSE)
  #
  # Check for correct number of comparisons
  #
  tabnarms <- table(studlab)
  sel.narms <- !is_wholenumber((1 + sqrt(8 * tabnarms + 1)) / 2)
  #
  if (sum(sel.narms) == 1)
    stop("Study '", names(tabnarms)[sel.narms],
         "' has a wrong number of comparisons.",
         "\n  Please provide data for all treatment comparisons",
         " (two-arm: 1; three-arm: 3; four-arm: 6, ...).",
         call. = FALSE)
  if (sum(sel.narms) > 1)
    stop("The following studies have a wrong number of comparisons: ",
         paste(paste0("'", names(tabnarms)[sel.narms], "'"),
               collapse = ", "),
         "\n  Please provide data for all treatment comparisons",
         " (two-arm: 1; three-arm: 3; four-arm: 6, ...).",
         call. = FALSE)
  #
  # Check number of subgraphs
  #
  n.subnets <- netconnection(treat1, treat2, studlab)$n.subnets
  #
  if (n.subnets > 1)
    stop("Network consists of ", n.subnets, " separate sub-networks.\n  ",
         "Use R function 'netconnection' to identify sub-networks.",
         call. = FALSE)
  #
  # Check NAs and zero standard errors
  #
  excl <- is.na(TE) | is.na(seTE) | seTE <= 0
  #
  if (any(excl)) {
    if (keepdata) {
      if (!missing.subset) {
        data$.excl <- NA
        data$.excl[subset] <- excl
      }
      else
        data$.excl <- excl
    }
    #
    dat.NAs <- data.frame(studlab = studlab[excl],
                          treat1 = treat1[excl],
                          treat2 = treat2[excl],
                          TE = format(round(TE[excl], 4)),
                          seTE = format(round(seTE[excl], 4)),
                          stringsAsFactors = FALSE
                          )
    #
    studlab <- studlab[!excl]
    treat1  <- treat1[!excl]
    treat2  <- treat2[!excl]
    TE      <- TE[!excl]
    seTE    <- seTE[!excl]
    #
    correlated <- correlated[!excl]
    #
    if (!is.null(n1))
      n1 <- n1[!excl]
    if (!is.null(n2))
      n2 <- n2[!excl]
    if (!is.null(event1))
      event1 <- event1[!excl]
    if (!is.null(event2))
      event2 <- event2[!excl]
    if (!is.null(incr1))
      incr1 <- incr1[!excl]
    if (!is.null(incr2))
      incr2 <- incr2[!excl]
    if (!is.null(mean1))
      mean1 <- mean1[!excl]
    if (!is.null(mean2))
      mean2 <- mean2[!excl]
    if (!is.null(sd1))
      sd1 <- sd1[!excl]
    if (!is.null(sd2))
      sd2 <- sd2[!excl]
    if (!is.null(time1))
      time1 <- time1[!excl]
    if (!is.null(time2))
      time2 <- time2[!excl]
    #
    seq <- seq[seq %in% unique(c(treat1, treat2))]
    labels <- labels[labels %in% unique(c(treat1, treat2))]
  }
  #
  # Check for correct number of comparisons (after removing
  # comparisons with missing data)
  #
  tabnarms <- table(studlab)
  sel.narms <- !is_wholenumber((1 + sqrt(8 * tabnarms + 1)) / 2)
  #
  if (sum(sel.narms) == 1)
    stop("After removing comparisons with missing treatment effects",
         " or standard errors,\n  study '",
         names(tabnarms)[sel.narms],
         "' has a wrong number of comparisons.\n  ",
         "Please check data and consider to ",
         if (is.pairwise & !allstudies) "\n   (i) ",
         "remove study from network meta-analysis",
         if (is.pairwise & !allstudies)
           " or\n  (ii) use argument 'allstudies = TRUE' in pairwise()",
         ".",
         call. = FALSE)
  #
  if (sum(sel.narms) > 1)
    stop("After removing comparisons with missing treatment effects",
         " or standard errors,\n  the following studies have",
         " a wrong number of comparisons: ",
         paste(paste0("'", names(tabnarms)[sel.narms], "'"), collapse = ", "),
         "\n  ",
         "Please check data and consider to ",
         if (is.pairwise & !allstudies) "\n   (i) ",
         "remove studies from network meta-analysis",
         if (is.pairwise & !allstudies)
           " or\n  (ii) use argument 'allstudies = TRUE' in pairwise()",
         ".",
         call. = FALSE)
  #
  if (any(excl)) {
    if (warn)
      warning("Comparison",
              if (sum(excl) > 1) "s",
              " with missing TE / seTE or zero seTE not considered ",
              "in network meta-analysis.",
              call. = FALSE)
    if (warn) {
      cat("Comparison",
          if (sum(excl) > 1) "s",
          " not considered in network meta-analysis:\n",
          sep = "")
      prmatrix(dat.NAs, quote = FALSE, right = TRUE,
               rowlab = rep("", sum(excl)))
      #cat("\n")
    }
  }
  #
  # Check number of subgraphs
  #
  n.subnets <- netconnection(treat1, treat2, studlab)$n.subnets
  #
  if (n.subnets > 1)
    stop("After removing comparisons with missing treatment effects",
         " or standard errors,\n  network consists of ",
         n.subnets, " separate sub-networks.\n  ",
         "Please check data and consider to remove studies",
         " from network meta-analysis.",
         call. = FALSE)
  #
  # Check for correct treatment order within comparison
  #
  wo <- treat1 > treat2
  #
  if (any(wo)) {
    TE[wo] <- -TE[wo]
    ttreat1 <- treat1
    treat1[wo] <- treat2[wo]
    treat2[wo] <- ttreat1[wo]
    #
    if (available.n) {
      tn1 <- n1
      n1[wo] <- n2[wo]
      n2[wo] <- tn1[wo]
    }
    #
    if (available.events) {
      tevent1 <- event1
      event1[wo] <- event2[wo]
      event2[wo] <- tevent1[wo]
    }
    #
    if (available.means) {
      tmean1 <- mean1
      mean1[wo] <- mean2[wo]
      mean2[wo] <- tmean1[wo]
    }
    #
    if (available.sds) {
      tsd1 <- sd1
      sd1[wo] <- sd2[wo]
      sd2[wo] <- tsd1[wo]
    }
    #
    if (available.times) {
      ttime1 <- time1
      time1[wo] <- time2[wo]
      time2[wo] <- ttime1[wo]
    }
  }
  #
  # Set reference group
  #
  if (missing.reference.group & !avail.reference.group.pairwise) {
    go.on <- TRUE
    i <- 0
    while (go.on) {
      i <- i + 1
      sel.i <-
        !is.na(TE) & !is.na(seTE) &
        (treat1 == labels[i] | treat2 == labels[i])
      if (sum(sel.i) > 0) {
        go.on <- FALSE
        reference.group <- labels[i]
      }
      else if (i == length(labels)) {
        go.on <- FALSE
        reference.group <- ""
      }
    }
  }
  #
  if (is.null(all.treatments))
    if (reference.group == "")
      all.treatments <- TRUE
    else
      all.treatments <- FALSE
  #
  # Check reference group
  #
  if (reference.group != "")
    reference.group <- setref(reference.group, labels)
  
  
  #
  #
  # (5) Generate analysis dataset
  #
  #
  #
  # Calculate weight matrix and generate ordered dataset, with added numbers
  # of arms per study
  #
  p0 <- prepare(TE, seTE, treat1, treat2, studlab,
                correlated = correlated,
                func.inverse = func.inverse)
  #
  W.matrix.common <- as.matrix(p0$W)
  Cov.matrix.common <- as.matrix(p0$Cov)
  dat.c <- p0$data
  #
  # Check consistency of treatment effects and standard errors in
  # multi-arm studies
  #
  chkmultiarm(dat.c$TE, dat.c$seTE, dat.c$treat1, dat.c$treat2, dat.c$studlab,
              dat.c$correlated,
              tol.multiarm = tol.multiarm, tol.multiarm.se = tol.multiarm.se,
              details = details.chkmultiarm)
  #
  # Study overview
  #
  tdata <- data.frame(studies = dat.c$studlab, narms = dat.c$narms,
                      order = dat.c$order,
                      stringsAsFactors = FALSE)
  #
  tdata <- tdata[!duplicated(tdata[, c("studies", "narms")]), , drop = FALSE]
  studies <- tdata$studies[order(tdata$order)]
  narms <- tdata$narms[order(tdata$order)]
  
  
  #
  #
  # (6) Conduct network meta-analysis
  #
  
  # Common effects model
  #
  res.c <- nma_ruecker(dat.c$TE,
                       W.matrix.common,
                       sqrt(1 / dat.c$weights),
                       dat.c$treat1, dat.c$treat2,
                       dat.c$treat1.pos, dat.c$treat2.pos,
                       dat.c$narms, dat.c$studlab,
                       sm,
                       level, level.ma,
                       dat.c$seTE, 0, sep.trts,
                       method.tau,
                       func.inverse, Cov0 = p0$Cov)
  trts <- rownames(res.c$A.matrix)
  #
  #
  # Random effects model
  #
  if (is.null(tau.preset)) {
    if (method.tau %in% c("ML", "REML")) {
      #
      dat.tau <-
        data.frame(studlab = studlab,
                   treat1 = treat1, treat2 = treat2,
                   TE = TE, seTE = seTE)
      if (available.n) {
        dat.tau$n1 <- n1
        dat.tau$n2 <- n2
      }
      if (available.events) {
        dat.tau$event1 <- event1
        dat.tau$event2 <- event2
        dat.tau$incr2 <- incr2
      }
      if (available.means) {
        dat.tau$mean1 <- mean1
        dat.tau$mean2 <- mean2
      }
      if (available.sds) {
        dat.tau$sd1 <- sd1
        dat.tau$sd2 <- sd2
      }
      if (available.times) {
        dat.tau$time1 <- time1
        dat.tau$time2 <- time2
      }
      #
      .order <- NULL
      dat.tau$.order <- seq_len(nrow(dat.tau))
      #
      dat.tau <-
        dat.tau[order(dat.tau$studlab,
                      dat.tau$treat1, dat.tau$treat2), , drop = FALSE]
      #
      keep <- logical(0)
      wo <- logical(0)
      #
      for (i in unique(dat.tau$studlab)) {
        d.i <- dat.tau[dat.tau$studlab == i, , drop = FALSE]
        trts.i <- unique(sort(c(d.i$treat1, d.i$treat2)))
        if (reference.group %in% trts.i)
          ref.i <- reference.group
        else
          ref.i <- rev(trts.i)[1]
        #
        keep.i <- !(d.i$treat1 != ref.i & d.i$treat2 != ref.i)
        wo.i <- d.i$treat1 == ref.i
        #
        keep <- c(keep, keep.i)
        wo <- c(wo, wo.i)
      }
      #
      dat.tau <- dat.tau[keep, , drop = FALSE]
      #
      wo <- wo[keep]
      if (sum(wo) > 0) {
        dat.tau$TE[wo] <- -dat.tau$TE[wo]
        #
        t2.i <- dat.tau$treat2
        e2.i <- dat.tau$event2
        n2.i <- dat.tau$n2
        mean2.i <- dat.tau$mean2
        sd2.i <- dat.tau$sd2
        time2.i <- dat.tau$time2
        #
        dat.tau$treat2[wo] <- dat.tau$treat1[wo]
        dat.tau$event2[wo] <- dat.tau$event1[wo]
        dat.tau$n2[wo] <- dat.tau$n1[wo]
        dat.tau$mean2[wo] <- dat.tau$mean1[wo]
        dat.tau$sd2[wo] <- dat.tau$sd1[wo]
        dat.tau$time2[wo] <- dat.tau$time2[wo]
        #
        dat.tau$treat1[wo] <- t2.i[wo]
        dat.tau$event1[wo] <- e2.i[wo]
        dat.tau$n1[wo] <- n2.i[wo]
        dat.tau$mean1[wo] <- mean2.i[wo]
        dat.tau$sd1[wo] <- sd2.i[wo]
        dat.tau$time1[wo] <- time2.i[wo]
      }
      #
      ncols1 <- ncol(dat.tau)
      dat.tau <- contrmat(dat.tau, grp1 = "treat1", grp2 = "treat2")
      ncols2 <- ncol(dat.tau)
      newnames <- paste0("V", seq_len(ncols2 - ncols1))
      names(dat.tau)[(ncols1 + 1):ncols2] <- newnames
      #
      trts.tau <- newnames[-length(newnames)]
      #
      formula.trts <-
        as.formula(paste("~ ", paste(trts.tau, collapse = " + "), " - 1"))
      #
      # Calculate Variance-Covariance matrix
      #
      if (available.n &
          (available.events | available.times | (available.sds))) {
        V <- bldiag(lapply(split(dat.tau, dat.tau$studlab), calcV, sm = sm))
      }
      else
        V <- dat.tau$seTE^2
      #
      dat.tau.TE <- dat.tau$TE
      dat.tau$comparison <- paste(dat.tau$treat1, dat.tau$treat2, sep = " vs ")
      #
      dat.tau %<>% relocate(.order, .after = last_col())
      #
      if (length(dat.tau.TE) == 1) {
        rma1 <- runNN(rma.uni,
                      list(yi = dat.tau.TE, vi = V, data = dat.tau,
                           method = method.tau, control = control))
        #
        tau <- NA
      }
      else {
        rma1 <- runNN(rma.mv,
                      list(yi = dat.tau.TE, V = V,
                           data = dat.tau,
                           mods = formula.trts,
                           random = as.call(~ factor(comparison) | studlab),
                           rho = 0.5,
                           method = method.tau, control = control))
        #
        tau <- sqrt(rma1$tau2)
      }
    }
    else
      tau <- res.c$tau
  }
  else
    tau <- tau.preset
  #
  p1 <- prepare(TE, seTE, treat1, treat2, studlab,
                tau = tau, correlated = correlated,
                func.inverse = func.inverse)
  #
  W.matrix.random <- as.matrix(p1$W)
  Cov.matrix.random <- as.matrix(p1$Cov)
  dat.r <- p1$data
  #
  res.r <- nma_ruecker(dat.r$TE,
                       W.matrix.random,
                       sqrt(1 / dat.r$weights),
                       dat.r$treat1, dat.r$treat2,
                       dat.r$treat1.pos, dat.r$treat2.pos,
                       dat.r$narms, dat.r$studlab,
                       sm,
                       level, level.ma,
                       dat.r$seTE, tau, sep.trts,
                       method.tau,
                       func.inverse, Cov0 = p1$Cov)
  #
  TE.random <- res.r$TE.pooled
  seTE.random <- res.r$seTE.pooled
  df.Q <- res.c$df
  #
  # Prediction intervals
  #
  if (df.Q == 0)
    prediction <- FALSE
  #
  if (df.Q >= 2) {
    seTE.predict <- sqrt(seTE.random^2 + tau^2)
    ci.p <- ci(TE.random, seTE.predict, level.predict, df.Q)
    p.lower <- ci.p$lower
    p.upper <- ci.p$upper
    diag(p.lower) <- 0
    diag(p.upper) <- 0
  }
  else {
    seTE.predict <- p.lower <- p.upper <- seTE.random
    seTE.predict[!is.na(seTE.predict)] <- NA
    p.lower[!is.na(p.lower)] <- NA
    p.upper[!is.na(p.upper)] <- NA
  }


  #
  #
  # (7) Generate R object
  #
  #
  o <- order(dat.c$order)
  #
  designs <- designs(res.c$treat1, res.c$treat2, res.c$studlab,
                     sep.trts = sep.trts)
  #
  W.matrix.common <- W.matrix.common[o, o, drop = FALSE]
  rownames(W.matrix.common) <- colnames(W.matrix.common) <- res.c$studlab[o]
  #
  W.matrix.random <- W.matrix.random[o, o, drop = FALSE]
  rownames(W.matrix.random) <- colnames(W.matrix.random) <- res.c$studlab[o]
  #
  Cov.matrix.common <- Cov.matrix.common[o, o, drop = FALSE]
  rownames(Cov.matrix.common) <- colnames(Cov.matrix.common) <- res.c$studlab[o]
  #
  Cov.matrix.random <- Cov.matrix.random[o, o, drop = FALSE]
  rownames(Cov.matrix.random) <- colnames(Cov.matrix.random) <- res.c$studlab[o]
  #
  res <- list(studlab = res.c$studlab[o],
              treat1 = res.c$treat1[o],
              treat2 = res.c$treat2[o],
              #
              TE = res.c$TE[o],
              seTE = res.c$seTE.orig[o],
              seTE.adj = res.c$seTE[o],
              seTE.adj.common = res.c$seTE[o],
              seTE.adj.random = res.r$seTE[o],
              correlated = correlated,
              #
              design = designs$design[o],
              #
              event1 = event1,
              event2 = event2,
              n1 = n1,
              n2 = n2,
              incr = incr2,
              #
              mean1 = mean1,
              mean2 = mean2,
              sd1 = sd1,
              sd2 = sd2,
              #
              time1 = time1,
              time2 = time2,
              #
              k = res.c$k,
              m = res.c$m,
              n = res.c$n,
              d = length(unique(designs$design)),
              #
              trts = trts,
              k.trts = NA,
              n.trts = if (available.n) NA else NULL,
              events.trts = if (available.events) NA else NULL,
              #
              n.arms = NA,
              multiarm = NA,
              #
              studies = studies,
              narms = narms,
              #
              designs = unique(sort(designs$design)),
              comparisons = "",
              #
              TE.nma.common = res.c$TE.nma[o],
              seTE.nma.common = res.c$seTE.nma[o],
              lower.nma.common = res.c$lower.nma[o],
              upper.nma.common = res.c$upper.nma[o],
              statistic.nma.common = res.c$statistic.nma[o],
              pval.nma.common = res.c$pval.nma[o],
              #
              leverage.common = res.c$leverage[o],
              w.common = res.c$w.pooled[o],
              Q.common = res.c$Q.pooled[o],
              #
              TE.common = res.c$TE.pooled,
              seTE.common = res.c$seTE.pooled,
              lower.common = res.c$lower.pooled,
              upper.common = res.c$upper.pooled,
              statistic.common = res.c$statistic.pooled,
              pval.common = res.c$pval.pooled,
              #
              TE.nma.random = res.r$TE.nma[o],
              seTE.nma.random = res.r$seTE.nma[o],
              lower.nma.random = res.r$lower.nma[o],
              upper.nma.random = res.r$upper.nma[o],
              statistic.nma.random = res.r$statistic.nma[o],
              pval.nma.random = res.r$pval.nma[o],
              #
              w.random = res.r$w.pooled[o],
              #
              TE.random = TE.random,
              seTE.random = seTE.random,
              lower.random = res.r$lower.pooled,
              upper.random = res.r$upper.pooled,
              statistic.random = res.r$statistic.pooled,
              pval.random = res.r$pval.pooled,
              #
              seTE.predict = seTE.predict,
              lower.predict = p.lower,
              upper.predict = p.upper,
              method.predict = "V",
              #
              prop.direct.common = NA,
              prop.direct.random = NA,
              #
              TE.direct.common = res.c$TE.direct,
              seTE.direct.common = res.c$seTE.direct,
              lower.direct.common = res.c$lower.direct,
              upper.direct.common = res.c$upper.direct,
              statistic.direct.common = res.c$statistic.direct,
              pval.direct.common = res.c$pval.direct,
              #
              TE.direct.random = res.r$TE.direct,
              seTE.direct.random = res.r$seTE.direct,
              lower.direct.random = res.r$lower.direct,
              upper.direct.random = res.r$upper.direct,
              statistic.direct.random = res.r$statistic.direct,
              pval.direct.random = res.r$pval.direct,
              #
              Q.direct = res.r$Q.direct,
              tau.direct = sqrt(res.r$tau2.direct),
              tau2.direct = res.r$tau2.direct,
              I2.direct = res.r$I2.direct,
              #
              TE.indirect.common = NA,
              seTE.indirect.common = NA,
              lower.indirect.common = NA,
              upper.indirect.common = NA,
              statistic.indirect.common = NA,
              pval.indirect.common = NA,
              #
              TE.indirect.random = NA,
              seTE.indirect.random = NA,
              lower.indirect.random = NA,
              upper.indirect.random = NA,
              statistic.indirect.random = NA,
              pval.indirect.random = NA,
              #
              Q = res.c$Q,
              df.Q = df.Q,
              pval.Q = res.c$pval.Q,
              I2 = res.c$I2,
              lower.I2 = res.c$lower.I2,
              upper.I2 = res.c$upper.I2,
              tau = tau,
              tau2 = tau^2,
              #
              Q.heterogeneity = NA,
              df.Q.heterogeneity = NA,
              pval.Q.heterogeneity = NA,
              #
              Q.inconsistency = NA,
              df.Q.inconsistency = NA,
              pval.Q.inconsistency = NA,
              #
              Q.decomp = res.c$Q.decomp,
              #
              W.matrix.common = W.matrix.common,
              W.matrix.random = W.matrix.random,
              #
              Cov.matrix.common = Cov.matrix.common,
              Cov.matrix.random = Cov.matrix.random,
              #
              A.matrix = res.c$A.matrix,
              X.matrix = res.c$B.matrix[o, ],
              B.matrix = res.c$B.matrix[o, ],
              #
              L.matrix.common = res.c$L.matrix,
              Lplus.matrix.common = res.c$Lplus.matrix,
              L.matrix.random = res.r$L.matrix,
              Lplus.matrix.random = res.r$Lplus.matrix,
              #
              Q.matrix = res.c$Q.matrix,
              #
              G.matrix = res.c$G.matrix[o, o, drop = FALSE],
              #
              H.matrix.common = res.c$H.matrix[o, o, drop = FALSE],
              H.matrix.random = res.r$H.matrix[o, o, drop = FALSE],
              #
              n.matrix = if (available.n) NA else NULL,
              events.matrix = if (available.events) NA else NULL,
              #
              P.common = NA,
              P.random = NA,
              #
              Cov.common = res.c$Cov,
              Cov.random = res.r$Cov,
              #
              treat1.pos = res.c$treat1.pos[o],
              treat2.pos = res.c$treat2.pos[o],
              #
              sm = sm,
              method = "Inverse",
              level = level,
              level.ma = level.ma,
              common = common,
              random = random,
              #
              prediction = prediction,
              level.predict = level.predict,
              #
              reference.group = reference.group,
              baseline.reference = baseline.reference,
              all.treatments = all.treatments,
              seq = seq,
              #
              method.tau = method.tau,
              tau.preset = tau.preset,
              #
              tol.multiarm = tol.multiarm,
              tol.multiarm.se = tol.multiarm.se,
              details.chkmultiarm = details.chkmultiarm,
              #
              func.inverse = deparse(substitute(func.inverse)),
              #
              sep.trts = sep.trts,
              nchar.trts = nchar.trts,
              nchar.studlab = nchar.studlab,
              #
              overall.hetstat = overall.hetstat,
              backtransf = backtransf,
              #
              title = title,
              #
              warn = warn,
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )
  #
  class(res) <- "netmeta"
  #
  # Add results for indirect treatment estimates
  #
  n <- res$n
  #
  res$prop.direct.common <-
    netmeasures(res, random = FALSE, warn = warn)$proportion
  # Print warning(s) in call of netmeasures() once
  res$prop.direct.random <-
    suppressWarnings(netmeasures(res, random = TRUE,
                                 tau.preset = res$tau,
                                 warn = FALSE)$proportion)
  if (is.logical(res$prop.direct.common))
    res$prop.direct.common <- as.numeric(res$prop.direct.common)
  if (is.logical(res$prop.direct.random))
    res$prop.direct.random <- as.numeric(res$prop.direct.random)
  #
  res$comparisons <-
    names(res$prop.direct.random)[!is_zero(res$prop.direct.random)]
  #
  # Add P.common and P.random
  #
  P.common <- P.random <- matrix(NA, n, n)
  colnames(P.common) <- rownames(P.common) <-
    colnames(P.random) <- rownames(P.random) <- trts
  #
  if (n == 2) {
    #
    # For two treatments only direct evidence is available
    #
    res$prop.direct.common <- 1
    res$prop.direct.random <- 1
    names(res$prop.direct.common) <-
      names(res$prop.direct.random) <- paste(labels, collapse = sep.trts)
    #
    sel <- row(P.common) != col(P.common)
    P.common[sel] <- 1
    P.random[sel] <- 1
  }
  else {
    k <- 0
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        k <- k + 1
        P.common[i, j] <- P.common[j, i] <- res$prop.direct.common[k]
        P.random[i, j] <- P.random[j, i] <- res$prop.direct.random[k]
      }
    }
  }
  #
  # Set direct evidence estimates to 0 if only indirect evidence is available
  # (otherwise indirect estimates would be NA as direct estimates are NA)
  #
  TE.direct.common <- res$TE.direct.common
  TE.direct.random <- res$TE.direct.random
  #
  TE.direct.common[abs(P.common) < .Machine$double.eps^0.5] <- 0
  TE.direct.random[abs(P.random) < .Machine$double.eps^0.5] <- 0
  #
  # Indirect estimate is NA if only direct evidence is available
  #
  res$P.common <- P.common
  res$P.random <- P.random
  #
  P.common[abs(P.common - 1) < .Machine$double.eps^0.5] <- NA
  P.common[P.common > 1] <- NA
  P.random[abs(P.random - 1) < .Machine$double.eps^0.5] <- NA
  P.random[P.random > 1] <- NA
  #
  # Common effects model
  #
  ci.if <- ci((res$TE.common - P.common * TE.direct.common) / (1 - P.common),
              sqrt(res$seTE.common^2 / (1 - P.common)),
              level = level)
  #
  res$TE.indirect.common   <- ci.if$TE
  res$seTE.indirect.common <- ci.if$seTE
  #
  res$lower.indirect.common <- ci.if$lower
  res$upper.indirect.common <- ci.if$upper
  #
  res$statistic.indirect.common <- ci.if$statistic
  res$pval.indirect.common <- ci.if$p
  #
  # Random effects model
  #
  ci.ir <- ci((res$TE.random - P.random * TE.direct.random) / (1 - P.random),
              sqrt(res$seTE.random^2 / (1 - P.random)),
              level = level)
  #
  res$TE.indirect.random   <- ci.ir$TE
  res$seTE.indirect.random <- ci.ir$seTE
  #
  res$lower.indirect.random <- ci.ir$lower
  res$upper.indirect.random <- ci.ir$upper
  #
  res$statistic.indirect.random <- ci.ir$statistic
  res$pval.indirect.random <- ci.ir$p
  #
  # Additional assignments
  #
  res$small.values <- small.values
  #
  l1 <- length(res$treat1)
  tab.trts <-
    table(longarm(res$treat1, res$treat2,
                  rep(1, l1), rep(2, l1), rep(1, l1), rep(2, l1),
                  studlab = res$studlab)$treat)
  res$k.trts <- as.numeric(tab.trts)
  names(res$k.trts) <- names(tab.trts)
  #
  if (any(res$narms > 2)) {
    tdata1 <- data.frame(studlab = res$studlab,
                         .order = seq(along = res$studlab))
    tdata2 <- data.frame(studlab = as.character(res$studies),
                         narms = res$narms)
    #
    tdata12 <- merge(tdata1, tdata2,
                     by = "studlab", all.x = TRUE, all.y = FALSE,
                     sort = FALSE)
    tdata12 <- tdata12[order(tdata12$.order), ]
    res$n.arms <- tdata12$narms
    res$multiarm <- tdata12$narms > 2
  }
  else {
    res$n.arms <- rep(2, length(res$studlab))
    res$multiarm <- rep(FALSE, length(res$studlab))
  }
  #
  # Set leverage of multi-arm studies to NA
  #
  if (any(res$multiarm))
    res$leverage.common[res$multiarm] <- NA
  
  
  #
  # Calculate heterogeneity and inconsistency statistics
  #
  if (res$d > 1) {
    dd <- decomp.design(res, warn = FALSE)
    res$Q.heterogeneity <- dd$Q.decomp$Q[2]
    res$Q.inconsistency <- dd$Q.decomp$Q[3]
    #
    res$df.Q.heterogeneity <- dd$Q.decomp$df[2]
    res$df.Q.inconsistency <- dd$Q.decomp$df[3]
    #
    res$pval.Q.heterogeneity <- dd$Q.decomp$pval[2]
    res$pval.Q.inconsistency <- dd$Q.decomp$pval[3]
  }
  
  
  if (keepdata) {
    data$.design <- designs(data$.treat1, data$.treat2, data$.studlab,
                            sep = sep.trts)$design
    #
    res$data <- merge(data,
                      data.frame(.studlab = res$studies,
                                 .narms = res$narms),
                      by = ".studlab",
                      stringsAsFactors = FALSE)
    #
    # Store adjusted standard errors in dataset
    #
    res$data <- merge(res$data,
                      data.frame(.studlab = res.c$studlab,
                                 .treat1 = res.c$treat1,
                                 .treat2 = res.c$treat2,
                                 .seTE.adj.common = res.c$seTE),
                      by = c(".studlab", ".treat1", ".treat2"),
                      stringsAsFactors = FALSE)
    #
    res$data <- merge(res$data,
                      data.frame(.studlab = res.r$studlab,
                                 .treat1 = res.r$treat1,
                                 .treat2 = res.r$treat2,
                                 .seTE.adj.random = res.r$seTE),
                      by = c(".studlab", ".treat1", ".treat2"),
                      stringsAsFactors = FALSE)
    #
    res$data <- res$data[order(res$data$.order), ]
    res$data$.order <- NULL
  }
  #
  if (available.n) {
    res$n.matrix <- netmatrix(res, n1 + n2, func = "sum")
    #
    dat.n <- data.frame(studlab = c(studlab, studlab),
                        treat = c(treat1, treat2),
                        n = c(n1, n2))
    dat.n <- dat.n[!duplicated(dat.n[, c("studlab", "treat")]), ]
    dat.n <- by(dat.n$n, dat.n$treat, sum, na.rm = TRUE)
    res$n.trts <- as.vector(dat.n[trts])
    names(res$n.trts) <- trts
  }
  #
  if (available.events) {
    res$events.matrix <- netmatrix(res, event1 + event2, func = "sum")
    #
    dat.e <- data.frame(studlab = c(studlab, studlab),
                        treat = c(treat1, treat2),
                        n = c(event1, event2))
    dat.e <- dat.e[!duplicated(dat.e[, c("studlab", "treat")]), ]
    dat.e <- by(dat.e$n, dat.e$treat, sum, na.rm = TRUE)
    res$events.trts <- as.vector(dat.e[trts])
    names(res$events.trts) <- trts
  }
  
  #
  # Add results from rma.mv()
  #
  if (keeprma & method.tau %in% c("ML", "REML"))
    res$rma.tau <- rma1
  
  #
  # Drop list element 'correlated' if no correlated outcomes are considered 
  #
  if (all(!correlated)) {
    res$correlated <- NULL
    res$data <- res$data[, names(res$data) != ".correlated"]
  }
  
  #
  # Backward compatibility
  #
  res$fixed <- res$common
  res$comb.fixed <- res$common
  res$comb.random <- res$random
  #
  res$seTE.adj.fixed <- res$seTE.adj.common
  res$TE.nma.fixed <- res$TE.nma.common
  res$seTE.nma.fixed <- res$seTE.nma.common
  res$lower.nma.fixed <- res$lower.nma.common
  res$upper.nma.fixed <- res$upper.nma.common
  res$statistic.nma.fixed <- res$statistic.nma.common
  res$pval.nma.fixed <- res$pval.nma.common
  res$leverage.fixed <- res$leverage.common
  res$w.fixed <- res$w.common
  res$Q.fixed <- res$Q.common
  #
  res$TE.fixed <- res$TE.common
  res$seTE.fixed <- res$seTE.common
  res$lower.fixed <- res$lower.common
  res$upper.fixed <- res$upper.common
  res$statistic.fixed <- res$statistic.common
  res$pval.fixed <- res$pval.common
  #
  res$prop.direct.fixed <- res$prop.direct.common
  #
  res$TE.direct.fixed <- res$TE.direct.common
  res$seTE.direct.fixed <- res$seTE.direct.common
  res$lower.direct.fixed <- res$lower.direct.common
  res$upper.direct.fixed <- res$upper.direct.common
  res$statistic.direct.fixed <- res$statistic.direct.common
  res$pval.direct.fixed <- res$pval.direct.common
  #
  res$TE.indirect.fixed <- res$TE.indirect.common
  res$seTE.indirect.fixed <- res$seTE.indirect.common
  res$lower.indirect.fixed <- res$lower.indirect.common
  res$upper.indirect.fixed <- res$upper.indirect.common
  res$statistic.indirect.fixed <- res$statistic.indirect.common
  res$pval.indirect.fixed <- res$pval.indirect.common
  #
  res$L.matrix.fixed <- res$L.matrix.common
  res$Lplus.matrix.fixed <- res$Lplus.matrix.common
  res$H.matrix.fixed <- res$H.matrix.common
  res$P.fixed <- res$P.common
  res$Cov.fixed <- res$Cov.common
  
  res
}

#' Additive network meta-analysis for combinations of treatments
#' 
#' @description
#' Some treatments in a network meta-analysis may be combinations of
#' other treatments or have common components. The influence of
#' individual components can be evaluated in an additive network
#' meta-analysis model assuming that the effect of treatment
#' combinations is the sum of the effects of its components. This
#' function implements this additive model in a frequentist way.
#' 
#' @param x An object of class \code{netmeta}.
#' @param inactive A character string defining the inactive treatment
#'   component (see Details).
#' @param sep.comps A single character to define separator between
#'   treatment components.
#' @param C.matrix C matrix (see Details).
#' @param common A logical indicating whether a common effects network
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects network
#'   meta-analysis should be conducted.
#' @param tau.preset An optional value for the square-root of the
#'   between-study variance \eqn{\tau^2}.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots.
#' @param details.chkident A logical indicating whether details on
#'   unidentifiable components should be printed.
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique names for components (see
#'   Details).
#' @param sep.ia A single character to define separator for interactions.
#' @param func.inverse R function used to calculate the pseudoinverse
#'   of the Laplacian matrix L (see \code{\link{netmeta}}).
#' @param na.unident A logical indicating whether unidentifiable
#'   components and combinations should be set to missing values.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' Treatments in network meta-analysis (NMA) can be complex
#' interventions. Some treatments may be combinations of others or
#' have common components. The standard analysis provided by
#' \code{\link{netmeta}} is a NMA where all existing (single or
#' combined) treatments are considered as different nodes in the
#' network. Exploiting the fact that some treatments are combinations
#' of common components, an additive component network meta-analysis
#' (CNMA) model can be used to evaluate the influence of individual
#' components. This model assumes that the effect of a treatment
#' combination is the sum of the effects of its components which
#' implies that common components cancel out in comparisons.
#' 
#' The additive CNMA model has been implemented using Bayesian methods
#' (Mills et al., 2012; Welton et al., 2013). This function implements
#' the additive model in a frequentist way (Rücker et al., 2020).
#' 
#' The underlying multivariate model is given by
#' 
#' \deqn{\bold{\delta} = \bold{B} \bold{\theta}, \bold{\theta} =
#' \bold{C} \bold{\beta}}
#' 
#' with \describe{
#' \item{\eqn{\bold{\delta}}}{vector of true treatment effects
#'   (differences) from individual studies,}
#' \item{\eqn{\bold{B}}}{design matrix describing the structure of the
#'   network,}
#' \item{\eqn{\bold{\theta}}}{parameter vector that represents the
#'   existing combined treatments,}
#' \item{\eqn{\bold{C}}}{matrix describing how the treatments are
#'   composed,}
#' \item{\eqn{\bold{\beta}}}{parameter vector representing the
#'   treatment components.}
#' }
#' All parameters are estimated using weighted least squares
#' regression.
#' 
#' Argument \code{inactive} can be used to specify a single component
#' that does not have any therapeutic value. Accordingly, it is
#' assumed that the treatment effect of the combination of this
#' component with an additional treatment component is equal to the
#' treatment effect of the additional component alone.
#' 
#' Argument \code{sep.comps} can be used to specify the separator
#' between individual components. By default, the matrix \strong{C} is
#' calculated internally from treatment names. However, it is possible
#' to specify a different matrix using argument \code{C.matrix}.
#' 
#' By default, component names are not abbreviated in
#' printouts. However, in order to get more concise printouts,
#' argument \code{nchar.comps} can be used to define the minimum
#' number of characters for abbreviated component names (see
#' \code{\link{abbreviate}}, argument \code{minlength}). R function
#' \code{\link{treats}} is utilised internally to create abbreviated
#' component names.
#' 
#' @note
#' This function calculates effects for individual components and
#' complex interventions present in the network.
#'
#' R function \code{\link{netcomplex}} can be used to calculate the
#' effect for arbitrary complex interventions in a component network
#' meta-analysis. Furthermore, R function \code{\link{netcomparison}}
#' can be used to calculate the effect for comparisons of two
#' arbitrary complex intervention in a component network
#' meta-analysis.
#'
#' @return
#' An object of class \code{netcomb} with corresponding \code{print},
#' \code{summary}, and \code{forest} functions. The object is a list
#' containing the following components:
#' \item{studlab}{Study labels.}
#' \item{treat1}{Label/Number for first treatment.}
#' \item{treat2}{Label/Number for second treatment.}
#' \item{TE}{Estimate of treatment effect, i.e. difference between
#'   first and second treatment.}
#' \item{seTE}{Standard error of treatment estimate.}
#' \item{seTE.adj.common, seTE.adj.random}{Standard error of treatment
#'   estimate, adjusted for multi-arm studies.}
#' \item{design}{Design of study providing pairwise comparison.}
#' \item{event1}{Number of events in first treatment group.}
#' \item{event2}{Number of events in second treatment group.}
#' \item{n1}{Number of observations in first treatment group.}
#' \item{n2}{Number of observations in second treatment group.}
#' \item{k}{Total number of studies.}
#' \item{m}{Total number of pairwise comparisons.}
#' \item{n}{Total number of treatments.}
#' \item{d}{Total number of designs (corresponding to the unique set
#'   of treatments compared within studies).}
#' \item{c}{Total number of components.}
#' \item{trts}{Treatments included in network meta-analysis.}
#' \item{k.trts}{Number of studies evaluating a treatment.}
#' \item{n.trts}{Number of observations receiving a treatment (if
#'   arguments \code{n1} and \code{n2} are provided).}
#' \item{events.trts}{Number of events observed for a treatment (if
#'   arguments \code{event1} and \code{event2} are provided).}
#' \item{studies}{Study labels coerced into a factor with its levels
#'   sorted alphabetically.}
#' \item{narms}{Number of arms for each study.}
#' \item{designs}{Unique list of designs present in the network. A
#'   design corresponds to the set of treatments compared within a
#'   study.}
#' \item{comps}{Unique list of components present in the network.}
#' \item{TE.nma.common, TE.nma.random}{A vector of length \emph{m} of
#'   consistent treatment effects estimated by network meta-analysis
#'   (nma) (common and random effects model).}
#' \item{seTE.nma.common, seTE.nma.random}{A vector of length \emph{m}
#'   of effective standard errors estimated by network meta-analysis
#'   (common and random effects model).}
#' \item{lower.nma.common, lower.nma.random}{A vector of length
#'   \emph{m} of lower confidence interval limits for consistent
#'   treatment effects estimated by network meta-analysis (common
#'   and random effects model).}
#' \item{upper.nma.common, upper.nma.random}{A vector of length
#'   \emph{m} of upper confidence interval limits for the consistent
#'   treatment effects estimated by network meta-analysis (common
#'   and random effects model).}
#' \item{statistic.nma.common, statistic.nma.random}{A vector of length \emph{m}
#'   of z-values for test of treatment effect for individual
#'   comparisons (common and random effects model).}
#' \item{pval.nma.common, pval.nma.random}{A vector of length \emph{m}
#'   of p-values for test of treatment effect for individual
#'   comparisons (common and random effects model).}
#' \item{TE.cnma.common, TE.cnma.random}{A vector of length \emph{m} of
#'   consistent treatment effects estimated by the additive (common and
#'   random effects) model.}
#' \item{seTE.cnma.common, seTE.cnma.random}{A vector of length
#'   \emph{m} with standard errors estimated by the additive (common
#'   and random effects) model.}
#' \item{lower.cnma.common, lower.cnma.random}{A vector of length
#'   \emph{m} of lower confidence interval limits for consistent
#'   treatment effects estimated by the additive (common and random
#'   effects) model.}
#' \item{upper.cnma.common, upper.cnma.random}{A vector of length
#'   \emph{m} of upper confidence interval limits for consistent
#'   treatment effects estimated by the additive (common and random
#'   effects) model.}
#' \item{statistic.cnma.common, statistic.cnma.random}{A vector of length
#'   \emph{m} of z-values for the test of an overall effect estimated
#'   by the additive (common and random effects) model.}
#' \item{pval.cnma.common, pval.cnma.random}{A vector of length
#'   \emph{m} of p-values for the test of an overall effect estimated
#'   by the additive (common and random effects) model.}
#' \item{TE.common, TE.random}{\emph{n}x\emph{n} matrix with overall
#'   treatment effects estimated by the additive (common and random
#'   effects) model.}
#' \item{seTE.common, seTE.random}{\emph{n}x\emph{n} matrix with
#'   standard errors estimated by the additive (common and random
#'   effects) model.}
#' \item{lower.common, upper.common, lower.random,
#'   upper.random}{\emph{n}x\emph{n} matrices with lower and upper
#'   confidence interval limits estimated by the additive (common and
#'   random effects) model.}
#' \item{statistic.common, pval.common, statistic.random,
#'   pval.random}{\emph{n}x\emph{n} matrices with z-values and
#'   p-values for test of overall effect estimated by the additive
#'   (common and random effects) model.}
#' \item{Comp.common, Comp.random}{A vector of component effects (common
#'   and random effects model).}
#' \item{seComp.common, seComp.random}{A vector with corresponding
#'   standard errors (common and random effects model).}
#' \item{lower.Comp.common, lower.Comp.random}{A vector with lower
#'   confidence limits for components (common and random effects
#'   model).}
#' \item{upper.Comp.common, upper.Comp.random}{A vector with upper
#'   confidence limits for components (common and random effects
#'   model).}
#' \item{statistic.Comp.common, statistic.Comp.random}{A vector with z-values for
#'   the overall effect of components (common and random effects
#'   model).}
#' \item{pval.Comp.common, pval.Comp.random}{A vector with p-values for
#'   the overall effect of components (common and random effects
#'   model).}
#' \item{Comb.common, Comb.random}{A vector of combination effects
#'   (common and random effects model).}
#' \item{seComb.common, seComb.random}{A vector with corresponding
#'   standard errors (common and random effects model).}
#' \item{lower.Comb.common, lower.Comb.random}{A vector with lower
#'   confidence limits for combinations (common and random effects
#'   model).}
#' \item{upper.Comb.common, upper.Comb.random}{A vector with upper
#'   confidence limits for combinations (common and random effects
#'   model).}
#' \item{statistic.Comb.common, statistic.Comb.random}{A vector with
#'   z-values for the overall effect of combinations (common and random
#'   effects model).}
#' \item{pval.Comb.common, pval.Comb.random}{A vector with p-values for
#'   the overall effect of combinations (common and random effects
#'   model).}
#' \item{Q.additive}{Overall heterogeneity / inconsistency statistic
#'   (additive model).}
#' \item{df.Q.additive}{Degrees of freedom for test of heterogeneity /
#'   inconsistency (additive model).}
#' \item{pval.Q.additive}{P-value for test of heterogeneity /
#'   inconsistency (additive model).}
#' \item{tau}{Square-root of between-study variance (additive model).}
#' \item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
#'   limits.}
#' \item{Q.standard}{Overall heterogeneity / inconsistency statistic
#'   (standard model).}
#' \item{df.Q.standard}{Degrees of freedom for test of heterogeneity /
#'   inconsistency (standard model).}
#' \item{pval.Q.standard}{P-value for test of heterogeneity /
#'   inconsistency (standard model).}
#' \item{Q.diff}{Test statistic for difference in goodness of fit
#'   between standard and additive model.}
#' \item{df.Q.diff}{Degrees of freedom for difference in goodness of
#'   fit between standard and additive model.}
#' \item{pval.Q.diff}{P-value for difference in goodness of fit
#'   between standard and additive model.}
#' \item{A.matrix}{Adjacency matrix (\emph{n}x\emph{n}).}
#' \item{B.matrix}{Edge-vertex incidence matrix (\emph{m}x\emph{n}).}
#' \item{C.matrix}{As defined above.}
#' \item{sm}{Summary measure.}
#' \item{level.ma}{Level for confidence intervals.}
#' \item{common, random, tau.preset}{As defined above.}
#' \item{sep.trts}{A character used in comparison names as separator
#'   between treatment labels.}
#' \item{nchar.comps}{A numeric defining the minimum number of
#'   characters used to create unique component names.}
#' \item{inactive, sep.comps, sep.ia}{As defined above.}
#' \item{backtransf}{A logical indicating whether results should be
#'   back transformed in printouts and forest plots.}
#' \item{title}{Title of meta-analysis / systematic review.}
#' \item{x}{As defined above.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package netmeta used to create
#'   object.}
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{discomb}}, \code{\link{netmeta}},
#'   \code{\link{forest.netcomb}}, \code{\link{print.netcomb}},
#'   \code{\link{netcomplex}}, \code{\link{netcomparison}}
#' 
#' @references
#' König J, Krahn U, Binder H (2013):
#' Visualizing the flow of evidence in network meta-analysis and
#' characterizing mixed treatment comparisons.
#' \emph{Statistics in Medicine},
#' \bold{32}, 5414--29
#' 
#' Mills EJ, Thorlund K, Ioannidis JP (2012):
#' Calculating additive treatment effects from multiple randomized
#' trials provides useful estimates of combination therapies.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{65}, 1282--8
#' 
#' Rücker G, Petropoulou M, Schwarzer G (2020):
#' Network meta-analysis of multicomponent interventions.
#' \emph{Biometrical Journal},
#' \bold{62}, 808--21
#' 
#' Welton NJ, Caldwell DM, Adamopoulos E, Vedhara K (2009):
#' Mixed treatment comparison meta-analysis of complex interventions:
#' psychological interventions in coronary heart disease.
#' \emph{American Journal of Epidemiology},
#' \bold{169}: 1158--65
#' 
#' @examples
#' # Only consider studies including Face-to-face PST (to reduce
#' # runtime of example)
#' #
#' face <- subset(Linde2016, id %in% c(16, 24, 49, 118))
#' 
#' # Use pairwise() to transform data to comparison-based format
#' #
#' pw1 <- pairwise(treat = int,
#'   event = resp, n = n, studlab = paste(author, year),
#'   data = face, reference = "plac", sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma1 <- netmeta(pw1, common = FALSE)
#' nma1
#' 
#' # Additive model for treatment components (with placebo as inactive
#' # treatment)
#' #
#' cnma1 <- netcomb(nma1, inactive = "placebo")
#' cnma1
#' 
#' # Print results with two digits and test statistics with three digits
#' print(cnma1, digits = 2, digits.stat = 3)
#'
#' # Print detailed results
#' print(summary(cnma1), digits = 2, digits.stat = 3)
#' 
#' # All available combinations in CNMA
#' combinations(cnma1) # only "Face-to-face PST + SSRI"
#' 
#' # Create C matrix with all available interactions, i.e., one interaction
#' # for each combination
#' colnames(createC(cnma1))
#' 
#' # Run interaction CNMA model with all available interactions
#' # (same result as standard NMA)
#' netcomb(nma1, C.matrix = createC(cnma1))
#' 
#' \donttest{
#' forest(nma1, xlim = c(0.2, 50))
#' forest(cnma1, xlim = c(0.2, 50))
#' 
#' # Specify, order of treatments
#' #
#' trts <- c("TCA", "SSRI", "SNRI", "NRI", "Low-dose SARI", "NaSSa",
#'   "rMAO-A", "Ind drug", "Hypericum", "Face-to-face CBT",
#'   "Face-to-face PST", "Face-to-face interpsy", "Face-to-face psychodyn",
#'   "Other face-to-face", "Remote CBT", "Self-help CBT", "No contact CBT",
#'   "Face-to-face CBT + SSRI", "Face-to-face interpsy + SSRI",
#'   "Face-to-face PST + SSRI", "UC", "Placebo")
#' #
#' # Note, three treatments are actually combinations of 'SSRI' with
#' # other components:
#' # "Face-to-face CBT + SSRI",
#' # "Face-to-face interpsy + SSRI",
#' # "Face-to-face PST + SSRI"
#' 
#' pw2 <- pairwise(treat = int,
#'   event = resp, n = n, studlab = paste(author, year),
#'   data = Linde2016, reference = "plac", sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma2 <- netmeta(pw2, common = FALSE)
#' nma2
#' forest(nma2, xlim = c(0.2, 50))
#' 
#' # Additive model for treatment components (with placebo as inactive
#' # treatment)
#' #
#' cnma2 <- netcomb(nma2, inactive = "placebo")
#' cnma2
#' forest(cnma2, xlim = c(0.2, 50))
#' }
#' 
#' @export netcomb

netcomb <- function(x,
                    inactive = NULL,
                    sep.comps = gs("sep.comps"),
                    C.matrix,
                    common = x$common,
                    random = x$random | !is.null(tau.preset),
                    tau.preset = NULL,
                    details.chkident = FALSE,
                    nchar.comps = x$nchar.trts,
                    #
                    sep.ia = gs("sep.ia"),
                    #
                    func.inverse = invmat,
                    overall.hetstat = x$overall.hetstat,
                    backtransf = x$backtransf,
                    ##
                    na.unident = gs("na.unident"),
                    warn.deprecated = gs("warn.deprecated"),
                    ...) {
  
  
  ##
  ##
  ## (1) Check arguments
  ##
  ##
  chkclass(x, "netmeta")
  chksuitable(x, "Component network meta-analysis", classes = gs(".other_nma"))
  #
  x <- updateversion(x)
  ##
  chkchar(sep.comps, nchar = 0:1, length = 1)
  ##
  if (!is.null(tau.preset))
    chknumeric(tau.preset, min = 0, length = 1)
  ##
  chklogical(details.chkident)
  nchar.comps <- replaceNULL(nchar.comps, 666)
  chknumeric(nchar.comps, min = 1, length = 1)
  #
  missing.sep.ia <- missing(sep.ia)
  chkchar(sep.ia, nchar = 0:1, length = 1)
  #
  chklogical(overall.hetstat)
  chklogical(backtransf)
  chklogical(na.unident)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed", warn.deprecated)
  chklogical(common)
  ##
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (2) Create C.matrix
  ##
  ##
  if (missing(C.matrix)) {
    #
    # Create C-matrix from netmeta object
    #
    if (sep.comps == "")
      C.matrix <-
        createC(x, inactive = inactive,
                sep.comps = "...this_is_not_a_separator...")
    else
      C.matrix <-
        createC(x, inactive = inactive, sep.comps = sep.comps)
    #
    inactive <- attr(C.matrix, "inactive")
    C.matrix <- as.matrix(C.matrix)[x$trts, , drop = FALSE]
  }
  else {
    ##
    ## Check if appropriate C.matrix is provided
    ##
    if (!(is.matrix(C.matrix) | is.data.frame(C.matrix)))
      stop("Argument 'C.matrix' must be a matrix or data frame.",
           call. = FALSE)
    ##
    if (nrow(C.matrix) != x$n)
      stop("Argument 'C.matrix' has wrong number of rows",
           " (must be equal to number of treatments).",
           call. = FALSE)
    ##
    ## Row names must be a permutation of treatments
    ##
    wrong.labels <- FALSE
    ##
    if (is.null(rownames(C.matrix)))
      wrong.labels <- TRUE
    else {
      if (length(unique(x$trts)) ==
          length(unique(tolower(x$trts))) &&
          length(unique(rownames(C.matrix))) ==
          length(unique(tolower(rownames(C.matrix))))
          )
        idx <- charmatch(tolower(rownames(C.matrix)),
                         tolower(x$trts), nomatch = NA)
      else
        idx <- charmatch(rownames(C.matrix), x$trts, nomatch = NA)
      ##
      if (any(is.na(idx)) || any(idx == 0))
        wrong.labels <- TRUE
    }
    ##
    if (wrong.labels)
      stop("Row names of argument 'C.matrix' must be a ",
           "permutation of treatment names:\n  ",
           paste(paste0("'", x$trts, "'"),
                 collapse = " - "),
           call. = FALSE)
    ##
    C.matrix <- C.matrix[x$trts, , drop = FALSE]
    ##
    if (is.data.frame(C.matrix))
      C.matrix <- as.matrix(C.matrix)
  }
  ##
  c <- ncol(C.matrix) # number of components
  
  
  ##
  ##
  ## (3) Create design matrix based on treatment components
  ##
  ##
  X.matrix <- x$B.matrix %*% C.matrix
  colnames(X.matrix) <- colnames(C.matrix)
  rownames(X.matrix) <- x$studlab
  #
  # Identify and warn about not uniquely identifiable components
  #
  if (qr(X.matrix)$rank < c)
    comps.unident <- get_unident(X.matrix, details.chkident)
  else
    comps.unident <- NULL
  
  
  ##
  ##
  ## (4) Conduct CNMA
  ##
  ##
  df.Q.additive <- x$df.Q + x$n - 1 - qr(X.matrix)$rank
  df.Q.diff <- x$n - 1 - qr(X.matrix)$rank
  ##
  p0 <- prepare(x$TE, x$seTE, x$treat1, x$treat2, x$studlab,
                func.inverse = func.inverse)
  #
  W.matrix.common <- p0$W
  dat.c <- p0$data
  #
  # Common effects models
  #
  res.c <- nma_additive(dat.c$TE, W.matrix.common,
                        dat.c$treat1, dat.c$treat2,
                        dat.c$studlab,
                        x$level.ma,
                        X.matrix[dat.c$order, , drop = FALSE],
                        C.matrix, x$B.matrix[dat.c$order, , drop = FALSE],
                        df.Q.additive, x$n, x$sep.trts)
  ##
  ## Calculate heterogeneity statistics (additive model)
  ##
  Q.additive <- res.c$Q.additive
  ##
  if (!is.null(tau.preset))
    tau <- tau.preset
  else
    tau <- res.c$tau
  ##
  I2 <- res.c$I2
  lower.I2 <- res.c$lower.I2
  upper.I2 <- res.c$upper.I2
  #
  # Random effects models
  #
  p1 <- prepare(x$TE, x$seTE, x$treat1, x$treat2, x$studlab,
                tau = if (is.na(tau)) 0 else tau,
                func.inverse = func.inverse)
  #
  W.matrix.random <- p1$W
  dat.r <- p1$data
  #
  res.r <- nma_additive(dat.r$TE, W.matrix.random,
                        dat.r$treat1, dat.r$treat2,
                        dat.r$studlab,
                        x$level.ma,
                        X.matrix[dat.c$order, , drop = FALSE],
                        C.matrix, x$B.matrix[dat.c$order, , drop = FALSE],
                        df.Q.additive, x$n, x$sep.trts)
  #
  # Difference to standard network meta-analysis model
  #
  Q.diff <- res.c$Q.additive - x$Q
  if (!is.na(Q.diff) && abs(Q.diff) < .Machine$double.eps^0.75)
    Q.diff <- 0
  #
  if (is.na(df.Q.diff) | df.Q.diff == 0)
    pval.Q.diff <- NA
  else
    pval.Q.diff <- 1 - pchisq(Q.diff, df.Q.diff)
  ##
  ## Set unidentifiable components and combinations to NA
  ##
  if (na.unident & length(comps.unident) > 0) {
    trts <- names(res.c$combinations$TE)
    unident.pattern <- paste0("^", comps.unident, "$", collapse = "|")
    unident.combs <-
      trts[unlist(lapply(lapply(compsplit(trts, sep.comps),
                                grepl,
                                pattern = unident.pattern), any))]
    ##
    res.c$components <- lapply(res.c$components, setNA_vars, comps.unident)
    res.c$combinations <- lapply(res.c$combinations, setNA_vars, unident.combs)
    ##
    res.r$components <- lapply(res.r$components, setNA_vars, comps.unident)
    res.r$combinations <- lapply(res.r$combinations, setNA_vars, unident.combs)
  }
  ##
  if (length(comps.unident) == 0)
    comps.unident <- NULL
  #
  comps <- names(res.c$components$TE)
  #
  if (sep.comps == sep.ia)
    stop("Input for arguments 'sep.comps' and 'sep.ia' must be different.",
         call. = FALSE)
  #
  sep.ia <- setsep(comps, sep.ia, missing = missing.sep.ia)
  
  
  ##
  ##
  ## (5) Generate CNMA object
  ##
  ##
  o <- order(dat.c$order)
  #
  W.matrix.common <- W.matrix.common[o, o, drop = FALSE]
  rownames(W.matrix.common) <- colnames(W.matrix.common) <- res.c$studlab[o]
  #
  W.matrix.random <- W.matrix.random[o, o, drop = FALSE]
  rownames(W.matrix.random) <- colnames(W.matrix.random) <- res.c$studlab[o]
  #
  res <- list(studlab = x$studlab,
              treat1 = x$treat1,
              treat2 = x$treat2,
              ##
              TE = dat.c$TE[o],
              seTE = dat.c$seTE[o],
              seTE.adj = sqrt(1 / dat.c$weights[o]),
              seTE.adj.common = sqrt(1 / dat.c$weights[o]),
              seTE.adj.random = sqrt(1 / dat.r$weights[o]),
              ##
              design = x$design,
              ##
              event1 = x$event1,
              event2 = x$event2,
              n1 = x$n1,
              n2 = x$n2,
              incr = x$incr,
              ##
              k = x$k,
              m = x$m,
              n = x$n,
              d = x$d,
              c = c,
              ##
              trts = x$trts,
              k.trts = x$k.trts,
              n.trts = x$n.trts,
              events.trts = x$events.trts,
              ##
              n.arms = x$n.arms,
              multiarm = x$multiarm,
              ##
              studies = x$studies,
              narms = x$narms,
              ##
              designs = x$designs,
              ##
              comps = comps,
              k.comps = NA,
              n.comps = NA,
              events.comps = NA,
              na.unident = na.unident,
              comps.unident = comps.unident,
              ##
              TE.nma.common = x$TE.nma.common,
              seTE.nma.common = x$seTE.nma.common,
              lower.nma.common = x$lower.nma.common,
              upper.nma.common = x$upper.nma.common,
              statistic.nma.common = x$statistic.nma.common,
              pval.nma.common = x$pval.nma.common,
              ##
              TE.cnma.common = res.c$comparisons$TE[o],
              seTE.cnma.common = res.c$comparisons$seTE[o],
              lower.cnma.common = res.c$comparisons$lower[o],
              upper.cnma.common = res.c$comparisons$upper[o],
              statistic.cnma.common = res.c$comparisons$statistic[o],
              pval.cnma.common = res.c$comparisons$p[o],
              ##
              TE.common = res.c$all.comparisons$TE,
              seTE.common = res.c$all.comparisons$seTE,
              lower.common = res.c$all.comparisons$lower,
              upper.common = res.c$all.comparisons$upper,
              statistic.common = res.c$all.comparisons$statistic,
              pval.common = res.c$all.comparisons$p,
              ##
              TE.nma.random = x$TE.nma.random,
              seTE.nma.random = x$seTE.nma.random,
              lower.nma.random = x$lower.nma.random,
              upper.nma.random = x$upper.nma.random,
              statistic.nma.random = x$statistic.nma.random,
              pval.nma.random = x$pval.nma.random,
              ##
              TE.cnma.random = res.r$comparisons$TE[o],
              seTE.cnma.random = res.r$comparisons$seTE[o],
              lower.cnma.random = res.r$comparisons$lower[o],
              upper.cnma.random = res.r$comparisons$upper[o],
              statistic.cnma.random = res.r$comparisons$statistic[o],
              pval.cnma.random = res.r$comparisons$p[o],
              ##
              TE.random = res.r$all.comparisons$TE,
              seTE.random = res.r$all.comparisons$seTE,
              lower.random = res.r$all.comparisons$lower,
              upper.random = res.r$all.comparisons$upper,
              statistic.random = res.r$all.comparisons$statistic,
              pval.random = res.r$all.comparisons$p,
              ##
              Comp.common = unname(res.c$components$TE),
              seComp.common = unname(res.c$components$seTE),
              lower.Comp.common = unname(res.c$components$lower),
              upper.Comp.common = unname(res.c$components$upper),
              statistic.Comp.common = unname(res.c$components$statistic),
              pval.Comp.common = unname(res.c$components$p),
              ##
              Comp.random = unname(res.r$components$TE),
              seComp.random = unname(res.r$components$seTE),
              lower.Comp.random = unname(res.r$components$lower),
              upper.Comp.random = unname(res.r$components$upper),
              statistic.Comp.random = unname(res.r$components$statistic),
              pval.Comp.random = unname(res.r$components$p),
              ##
              Comb.common = unname(res.c$combinations$TE),
              seComb.common = unname(res.c$combinations$seTE),
              lower.Comb.common = unname(res.c$combinations$lower),
              upper.Comb.common = unname(res.c$combinations$upper),
              statistic.Comb.common = unname(res.c$combinations$statistic),
              pval.Comb.common = unname(res.c$combinations$p),
              ##
              Comb.random = unname(res.r$combinations$TE),
              seComb.random = unname(res.r$combinations$seTE),
              lower.Comb.random = unname(res.r$combinations$lower),
              upper.Comb.random = unname(res.r$combinations$upper),
              statistic.Comb.random = unname(res.r$combinations$statistic),
              pval.Comb.random = unname(res.r$combinations$p),
              ##
              Q.additive = Q.additive,
              df.Q.additive = df.Q.additive,
              pval.Q.additive = res.c$pval.Q.additive,
              tau = tau,
              I2 = I2, lower.I2 = lower.I2, upper.I2 = upper.I2,
              ##
              Q.standard = x$Q,
              df.Q.standard = x$df.Q,
              pval.Q.standard = x$pval.Q,
              ##
              Q.diff = Q.diff,
              df.Q.diff = df.Q.diff,
              pval.Q.diff = pval.Q.diff, 
              #
              W.matrix.common = W.matrix.common,
              W.matrix.random = W.matrix.random,
              #
              A.matrix = x$A.matrix,
              X.matrix = X.matrix,
              B.matrix = x$B.matrix,
              C.matrix = C.matrix,
              ##
              L.matrix.common = res.c$L.matrix,
              Lplus.matrix.common = res.c$Lplus.matrix,
              L.matrix.random = res.r$L.matrix,
              Lplus.matrix.random = res.r$Lplus.matrix,
              ##
              H.matrix.common = res.c$H.matrix[o, o, drop = FALSE],
              H.matrix.random = res.r$H.matrix[o, o, drop = FALSE],
              ##
              n.matrix = x$n.matrix,
              events.matrix = x$events.matrix,
              ##
              Cov.common = res.c$Cov,
              Cov.random = res.r$Cov,
              ##
              sm = x$sm,
              method = "Inverse",
              level = x$level,
              level.ma = x$level.ma,
              common = common,
              random = random,
              ##
              reference.group = x$reference.group,
              baseline.reference = x$baseline.reference,
              all.treatments = NULL,
              seq = x$seq,
              #
              method.tau = x$method.tau,
              tau.preset = tau.preset,
              ##
              tol.multiarm = x$tol.multiarm,
              details.chkmultiarm = x$details.chkmultiarm,
              ##
              sep.trts = x$sep.trts,
              sep.comps = sep.comps,
              nchar.comps = nchar.comps,
              sep.ia = sep.ia,
              #
              inactive = inactive,
              ##
              func.inverse = deparse(substitute(func.inverse)),
              #
              overall.hetstat = overall.hetstat,
              backtransf = backtransf,
              #
              title = x$title,
              ##
              x = x,
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )
  #
  if (!is.null(x$data))
    res$data <- x$data
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  res$comb.fixed <- res$common
  res$comb.random <- res$random
  ##
  res$seTE.adj.fixed <- res$seTE.adj.common
  res$TE.nma.fixed <- res$TE.nma.common
  res$seTE.nma.fixed <- res$seTE.nma.common
  res$lower.nma.fixed <- res$lower.nma.common
  res$upper.nma.fixed <- res$upper.nma.common
  res$statistic.nma.fixed <- res$statistic.nma.common
  res$pval.nma.fixed <- res$pval.nma.common
  res$TE.cnma.fixed <- res$TE.cnma.common
  res$seTE.cnma.fixed <- res$seTE.cnma.common
  res$lower.cnma.fixed <- res$lower.cnma.common
  res$upper.cnma.fixed <- res$upper.cnma.common
  res$statistic.cnma.fixed <- res$statistic.cnma.common
  res$pval.cnma.fixed <- res$pval.cnma.common
  ##
  res$TE.fixed <- res$TE.common
  res$seTE.fixed <- res$seTE.common
  res$lower.fixed <- res$lower.common
  res$upper.fixed <- res$upper.common
  res$statistic.fixed <- res$statistic.common
  res$pval.fixed <- res$pval.common
  ##
  res$Comp.fixed <- res$Comp.common
  res$seComp.fixed <- res$seComp.common
  res$lower.Comp.fixed <- res$lower.Comp.common
  res$upper.Comp.fixed <- res$upper.Comp.common
  res$statistic.Comp.fixed <- res$statistic.Comp.common
  res$pval.Comp.fixed <- res$pval.Comp.common
  ##
  res$Comb.fixed <- res$Comb.common
  res$seComb.fixed <- res$seComb.common
  res$lower.Comb.fixed <- res$lower.Comb.common
  res$upper.Comb.fixed <- res$upper.Comb.common
  res$statistic.Comb.fixed <- res$statistic.Comb.common
  res$pval.Comb.fixed <- res$pval.Comb.common
  ##
  res$L.matrix.fixed <- res$L.matrix.common
  res$Lplus.matrix.fixed <- res$Lplus.matrix.common
  res$H.matrix.fixed <- res$H.matrix.common        
  ##
  class(res) <- "netcomb"
  
  
  res
}

#' Calculate rankogram from treatment effect samples
#'
#' @description
#' This function calculates the probabilities of each treatment being
#' at each possible rank and the SUCRAs (Surface Under the Cumulative
#' RAnking curve) from a sample of treatment estimates in network
#' meta-analysis.
#'
#' @param x A matrix or data frame with treatment effects in columns and
#'  samples in rows.
#' @param pooled A character string indicating whether samples come from
#'   a common (\code{"common"}), random effects (\code{"random"}), or
#'   \code{"unspecified"} model, can be abbreviated.
#' @param small.values An optional character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect, can be abbreviated.
#' @param cumulative.rankprob A logical indicating whether cumulative
#'   ranking probabilities should be printed.
#' @param keep.samples A logical indicating whether to keep the generated
#'   samples.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param \dots Additional arguments (ignored).
#'
#' @details
#' We derive a matrix showing the probability of each treatment being
#' at each possible rank. To this aim, we use samples and
#' summarise them using the ranking metric SUCRAs (Surface Under
#' the Cumulative RAnking curve).
#' 
#' The matrix / data frame in argument \code{x} must contain the sampled
#' effects for each treatment.
#'
#' @return
#' An object of class \code{rankogram} with corresponding \code{print}
#' and \code{plot} function. The object is a list containing the
#' following components:
#' \item{ranking.matrix.common}{Numeric matrix giving the probability
#'   of each treatment being at each possible rank for the common
#'   effects model.}
#' \item{ranking.common}{SUCRA values for the common effects model.}
#' \item{ranking.matrix.random}{Numeric matrix giving the probability
#'   of each treatment being at each possible rank for the random
#'   effects model.}
#' \item{ranking.random}{SUCRA values for the random effects model.}
#' \item{cumrank.matrix.common}{Numeric matrix giving the cumulative
#'   ranking probability of each treatment for the
#'   common effects model.}
#' \item{cumrank.matrix.random}{Numeric matrix giving the cumulative
#'   ranking probability of each treatment for the random effects
#'   model.}
#' \item{nsim, common, random}{As defined above},
#' \item{small.values, x}{As defined above},
#'
#' @author Theodoros Papakonstantinou \email{dev@@tpapak.com}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{netmeta}}, \code{\link{netrank}},
#'   \code{\link{plot.rankogram}},
#'   \code{\link[metabook]{Woods2010}}
#'
#' @references
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#'
#' @examples
#' pw <- pairwise(treatment, event = r, n = N, studlab = author,
#'   data = Woods2010, sm = "OR")
#' nma <- netmeta(pw, small.values = "desirable")
#'
#' set.seed(1909) # get reproducible results
#' rnk <- rankogram(nma, common = FALSE, nsim = 10, # reduce runtime
#'   keep.samples = TRUE)
#' rnk
#' 
#' rankogram(rnk$samples.random)
#'
#' @rdname rankogram.default
#' @method rankogram default
#' @export

rankogram.default <- function(x, pooled = "unspecified",
                              small.values = "desirable",
                              cumulative.rankprob = FALSE,
                              keep.samples = FALSE,
                              nchar.trts = gs("nchar.trts"),
                              ...) {
  
  #
  #
  # (1) Check for matrix or data frame
  #
  #
  
  if (!inherits(x, c("matrix", "data.frame")))
    stop("Argument 'x' must be a matrix or data frame.",
         call. = FALSE)
  #
  if (is.data.frame(x))
    x <- as.matrix(x)
  #
  if (any(!is.numeric(x)))
    stop("Input for argument 'x' must contain numeric values.",
         call. = FALSE)
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  
  pooled <- setchar(pooled, c("common", "random", "unspecified"))
  small.values <- setsv(small.values)
  chklogical(cumulative.rankprob)
  chklogical(keep.samples)
  #
  if (is.null(nchar.trts))
    nchar.trts <- 666
  else
    chknumeric(nchar.trts, length = 1)
  #
  common <- pooled == "common"
  random <- !common
  
  
  #
  #
  # (3) Resampling to calculate ranking probabilites and SUCRAs
  #
  #
  
  if (small.values == "undesirable")
    x <- -x
  #
  sucras.common  <- ranking.matrix.common  <- cumrank.matrix.common <-
    meanranks.common <- medianranks.common <- NULL
  #
  sucras.random <- ranking.matrix.random <- cumrank.matrix.random <-
    meanranks.random <- medianranks.random <- NULL
  #
  if (common) {
    res.c <- rankings(x)
    #
    sucras.common <- res.c$sucras
    ranking.matrix.common <- res.c$rankogram
    cumrank.matrix.common <- res.c$cumrank
    #
    meanranks.common <- res.c$meanranks
    medianranks.common <- res.c$medianranks
    #
    samples.common <- x
    nsim <- res.c$nsim
  }
  #
  if (random) {
    res.r <- rankings(x)
    #
    sucras.random <- res.r$sucras
    ranking.matrix.random <- res.r$rankogram
    cumrank.matrix.random <- res.r$cumrank
    #
    meanranks.random <- res.r$meanranks
    medianranks.random <- res.r$medianranks
    #
    samples.random <- x
    nsim <- res.r$nsim
  }
  
  
  #
  #
  # (4) Create rankogram object
  #
  #
  
  res <- list(ranking.common = sucras.common,
              ranking.matrix.common = ranking.matrix.common,
              cumrank.matrix.common = cumrank.matrix.common,
              #
              meanranks.common = meanranks.common,
              medianranks.common = medianranks.common,
              #
              samples.common =
                if (common & keep.samples) samples.common else NULL,
              #
              ranking.random = sucras.random,
              ranking.matrix.random = ranking.matrix.random,
              cumrank.matrix.random = cumrank.matrix.random,
              #
              meanranks.random = meanranks.random,
              medianranks.random = medianranks.random,
              #
              samples.random =
                if (random & keep.samples) samples.random else NULL,
              #
              nsim = nsim,
              #
              pooled = pooled,
              common = common,
              random = random,
              small.values = small.values,
              cumulative.rankprob = cumulative.rankprob,
              #
              nchar.trts = nchar.trts,
              #
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )
  #
  # Backward compatibility
  #
  res$fixed <- common
  #
  res$ranking.fixed <- sucras.common
  res$ranking.matrix.fixed <- ranking.matrix.common
  res$cumrank.matrix.fixed <- cumrank.matrix.common
  #
  class(res) <- "rankogram"
  
  res
}

#' Summary method for objects of class netmetareg
#' 
#' @description
#' Summary method for objects of class \code{netmetareg} to print
#' list of studies in subnetworks.
#' 
#' @param object An object of class \code{netmetareg}.
#' @param x An object of class \code{summary.netmetareg}.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   errors.
#' @param big.mark A character used as thousands separator.
#' @param \dots Additional arguments (ignored).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' @seealso \code{\link{netmetareg}}
#' 
#' @examples
#' \dontrun{
#' data(smokingcessation)
#' # Add variable with (fictitious) risk of bias values
#' # with 1 = "low risk" and 2 = "high risk"
#' #
#' smokingcessation$rob <- rep(1:2, 12)
#' 
#' pw <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' 
#' nma <- netmeta(pw, common = FALSE, ref = "A")
#' 
#' # Network meta-regression with continuous covariate and assumption of
#' # independent slopes
#' nr <- netmetareg(nma, rob)
#' nr
#' 
#' summary(nr1)
#' }
#'
#' @method summary netmetareg 
#' @export

summary.netmetareg <- function(object, ...) {
  
  chkclass(object, "netmetareg")
  
  res <- object
  #
  class(res) <- c("summary.netmetareg", class(res))
  #
  res
}


#' @rdname summary.netmetareg
#' @method print summary.netmetareg 
#' @export

print.summary.netmetareg <- function(x,
                                     digits = gs("digits"),
                                     digits.se = gs("digits.se"),
                                     big.mark = gs("big.mark"),
                                     ...) {
  
  chkclass(x, "summary.netmetareg")
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chkchar(big.mark, length = 1)
  #
  dat <- x$full_results
  rownames(dat) <- dat$comparison
  dat$comparison <- NULL
  #
  dat$d <-
    formatN(dat$d, digits = digits, big.mark = big.mark, text.NA = ".")
  dat$se.d <-
    formatN(dat$se.d, digits = digits.se, big.mark = big.mark, text.NA = ".")
  dat$beta <-
    formatN(dat$beta, digits = digits, big.mark = big.mark, text.NA = ".")
  dat$se.beta <-
    formatN(dat$se.beta, digits = digits.se, big.mark = big.mark, text.NA = ".")
  dat$cov <-
    formatN(dat$cov, digits = digits.se, big.mark = big.mark, text.NA = ".")
  #
  # Do not print treatment labels (which are contained in the row names)
  #
  dat$treat1 <- dat$treat2 <- NULL
  #
  prmatrix(dat, quote = FALSE, right = TRUE)
  #
  invisible(NULL)
}

updateversion <- function(x, verbose = FALSE) {
  
  if (is.null(x$version)) {
    major <- 0
    minor <- 0
  }
  else {
    version <- as.numeric(unlist(strsplit(x$version, "-")))
    major <- version[1]
    minor <- version[2]
  }
  ##
  update.0.9.6 <- FALSE
  update.0.9.7 <- FALSE
  ##
  if (!((major == 0.9 & minor > 5) | major > 0.9)) {
    update.0.9.6 <- TRUE
    update.0.9.7 <- TRUE
  }
  if (!((major == 0.9 & minor > 6) | major > 0.9))
    update.0.9.7 <- TRUE
  ##
  if (verbose) {
    if (update.0.9.6)
      message("Update to netmeta, version 0.9-6")
    if (update.0.9.7)
      message("Update to netmeta, version 0.9-7")
  }
  ##
  update.1.3.0 <- update_needed(x$version, 1, 3, verbose)
  update.2.0.0 <- update_needed(x$version, 2, 0, verbose)
  update.2.5.0 <- update_needed(x$version, 2, 5, verbose)
  update.2.8.0 <- update_needed(x$version, 2, 8, verbose)
  update.2.9.0 <- update_needed(x$version, 2, 9, verbose)
  update.3.0.0 <- update_needed(x$version, 3, 0, verbose)
  update.3.3.0 <- update_needed(x$version, 3, 3, verbose)
  
  
  ##
  ##  (1) Update netmeta object
  ##
  if (inherits(x, "netmeta")) {
    ##
    if (update.0.9.6) {
      x$prediction <- FALSE
      x$df.Q <- x$df
      ##
      x$d <- nma_krahn(x)$d
      if (is.null(x$d))
        x$d <- 1
      ##
      if (x$d > 1) {
        dd <- decomp.design(x)
        x$Q.heterogeneity <- dd$Q.decomp$Q[2]
        x$Q.inconsistency <- dd$Q.decomp$Q[3]
        ##
        x$df.Q.heterogeneity <- dd$Q.decomp$df[2]
        x$df.Q.inconsistency <- dd$Q.decomp$df[3]
        ##
        x$pval.Q.heterogeneity <- dd$Q.decomp$pval[2]
        x$pval.Q.inconsistency <- dd$Q.decomp$pval[3]
      }
      else {
        x$Q.heterogeneity <- NA
        x$Q.inconsistency <- NA
        ##
        x$df.Q.heterogeneity <- NA
        x$df.Q.inconsistency <- NA
        ##
        x$pval.Q.heterogeneity <- NA
        x$pval.Q.inconsistency <- NA
      }
      ##
      x$df <- NULL
      ##
      x$baseline.reference <- TRUE
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.0.9.7)
      x$backtransf <- TRUE
    ##
    if (update.1.3.0) {
      x$statistic.fixed <- x$zval.fixed
      x$statistic.random <- x$zval.random
      x$statistic.direct.fixed <- x$zval.direct.fixed
      x$statistic.direct.random <- x$zval.direct.random
      x$statistic.indirect.fixed <- x$zval.indirect.fixed
      x$statistic.indirect.random <- x$zval.indirect.random
      x$statistic.nma.fixed <- x$zval.nma.fixed
      x$statistic.nma.random <- x$zval.nma.random
      ##
      x$zval.fixed <- x$zval.random <-
        x$zval.nma.fixed <- x$zval.nma.random <-
          x$zval.direct.fixed <- x$zval.direct.random <-
            x$zval.indirect.fixed <- x$zval.indirect.random <- NULL
      ##
      if (any(x$narms > 2)) {
        tdata1 <- data.frame(studlab = x$studlab,
                             .order = seq(along = x$studlab))
        tdata2 <- data.frame(studlab = as.character(x$studies),
                             narms = x$narms)
        ##
        tdata12 <- merge(tdata1, tdata2,
                         by = "studlab", all.x = TRUE, all.y = FALSE,
                         sort = FALSE)
        tdata12 <- tdata12[order(tdata12$.order), ]
        ##
        x$n.arms <- tdata12$narms
        x$multiarm <- tdata12$narms > 2
      }
      else {
        x$n.arms <- rep(2, length(x$studlab))
        x$multiarm <- rep(FALSE, length(x$studlab))
      }
    }
    ##
    if (update.2.0.0) {
      x$fixed <- x$comb.fixed
      x$random <- x$comb.random
      x$level.ma <- x$level.comb
      ##
      x$comb.fixed <- x$comb.random <- x$level.comb <- NULL
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      ##
      x$seTE.adj.common <- x$seTE.adj.fixed
      ##
      if (!inherits(x, "netmetabin")) {
        x$TE.nma.common <- x$TE.nma.fixed
        x$seTE.nma.common <- x$seTE.nma.fixed
        x$lower.nma.common <- x$lower.nma.fixed
        x$upper.nma.common <- x$upper.nma.fixed
        x$statistic.nma.common <- x$statistic.nma.fixed
        x$pval.nma.common <- x$pval.nma.fixed
        x$leverage.common <- x$leverage.fixed
        x$w.common <- x$w.fixed
        x$Q.common <- x$Q.fixed
      }
      ##
      x$TE.common <- x$TE.fixed
      x$seTE.common <- x$seTE.fixed
      x$lower.common <- x$lower.fixed
      x$upper.common <- x$upper.fixed
      x$statistic.common <- x$statistic.fixed
      x$pval.common <- x$pval.fixed
      ##
      x$prop.direct.common <- x$prop.direct.fixed
      ##
      x$TE.direct.common <- x$TE.direct.fixed
      x$seTE.direct.common <- x$seTE.direct.fixed
      x$lower.direct.common <- x$lower.direct.fixed
      x$upper.direct.common <- x$upper.direct.fixed
      x$statistic.direct.common <- x$statistic.direct.fixed
      x$pval.direct.common <- x$pval.direct.fixed
      ##
      x$TE.indirect.common <- x$TE.indirect.fixed
      x$seTE.indirect.common <- x$seTE.indirect.fixed
      x$lower.indirect.common <- x$lower.indirect.fixed
      x$upper.indirect.common <- x$upper.indirect.fixed
      x$statistic.indirect.common <- x$statistic.indirect.fixed
      x$pval.indirect.common <- x$pval.indirect.fixed
      ##
      if (!inherits(x, "netmetabin")) {
        x$L.matrix.common <- x$L.matrix.fixed
        x$Lplus.matrix.common <- x$Lplus.matrix.fixed
        x$H.matrix.common <- x$H.matrix.fixed
      }
      x$P.common <- x$P.fixed
      x$Cov.common <- x$Cov.fixed               
    }
    ##
    if (update.2.8.0)
      x$small.values <- setsv(x$small.values)
    ##
    if (update.2.9.0) {
      if (is.null(x$keepdata) || x$keepdata) {
        if (isCol(x$data, "subset"))
          sel.s <- x$data$subset
        else
          sel.s <- rep(TRUE, nrow(x$data))
        ##
        x$data$.seTE.adj.common <- NA
        x$data$.seTE.adj.random <- NA
        ##
        x$data$.seTE.adj.common[sel.s] <- x$seTE.adj.common
        x$data$.seTE.adj.random[sel.s] <- x$seTE.adj.random
      }
    }
    #
    if (update.3.0.0) {
      x$overall.hetstat <- TRUE
      x$method.predict <- "V"
      #
      if (inherits(x, "netmetabin")) {
        x$method.incr <- "only0"
        #
        if (is.logical(x$addincr) && x$addincr)
          method.incr <- "all"
        else if (is.logical(x$allincr) && x$allincr)
          method.incr <- "if0all"
      }
    }
    #
    if (update.3.3.0) {
      if (!is.null(x$data$incr)) {
       x$data$.incr1 <- replaceNULL(x$data$.incr1, x$data$incr) 
       x$data$.incr2 <- replaceNULL(x$data$.incr2, x$data$incr) 
      }
    }
    #
    return(x)
  }
  
  
  ##
  ##  (2) Update summary.netmeta / summary.netcomb object
  ##
  if (inherits(x, c("summary.netmeta", "summary.netcomb"))) {
    if (update.2.0.0) {
      x$level.ma <- x$level.comb
      x$x$fixed <- x$comb.fixed
      x$x$random <- x$comb.random
      ##
      x$comb.fixed <- x$comb.random <- x$level.comb <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.5.0)
      x$x$common <- x$x$fixed
    ##
    if (update.2.8.0)
      x$x$small.values <- setsv(x$x$small.values)
    #
    if (update.3.0.0) {
      x$x$overall.hetstat <- TRUE
      x$x$method.predict <- "V"
      #
      if (inherits(x, "summary.netmetabin")) {
        x$method.incr <- "only0"
        #
        if (is.logical(x$addincr) && x$addincr)
          method.incr <- "all"
        else if (is.logical(x$allincr) && x$allincr)
          method.incr <- "if0all"
      }
    }
    #
    return(x)
  }
  
  
  ##
  ##  (3) Update netcomb object
  ##
  if (inherits(x, "netcomb") && !inherits(x, "discomb")) {
    if (update.1.3.0) {
      x$statistic.fixed <- x$zval.fixed
      x$statistic.random <- x$zval.random
      x$statistic.nma.fixed <- x$zval.nma.fixed
      x$statistic.nma.random <- x$zval.nma.random
      x$statistic.cnma.fixed <- x$zval.cnma.fixed
      x$statistic.cnma.random <- x$zval.cnma.random
      x$statistic.Comb.fixed <- x$zval.Comb.fixed
      x$statistic.Comb.random <- x$zval.Comb.random
      x$statistic.Comp.fixed <- x$zval.Comp.fixed
      x$statistic.Comp.random <- x$zval.Comp.random
      ##
      x$zval.fixed <- x$zval.random <-
        x$zval.nma.fixed <- x$zval.nma.random <-
          x$zval.cnma.fixed <- x$zval.cnma.random <-
            x$zval.Comb.fixed <- x$zval.Comb.random <-
              x$zval.Comp.fixed <- x$zval.Comp.random <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.0.0) {
      x$fixed <- x$comb.fixed
      x$random <- x$comb.random
      x$level.ma <- x$level.comb
      ##
      x$comb.fixed <- x$comb.random <- x$level.comb <- NULL
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      ##
      x$seTE.adj.common <- x$seTE.adj.fixed
      x$TE.nma.common <- x$TE.nma.fixed
      x$seTE.nma.common <- x$seTE.nma.fixed
      x$lower.nma.common <- x$lower.nma.fixed
      x$upper.nma.common <- x$upper.nma.fixed
      x$statistic.nma.common <- x$statistic.nma.fixed
      x$pval.nma.common <- x$pval.nma.fixed
      x$TE.cnma.common <- x$TE.cnma.fixed
      x$seTE.cnma.common <- x$seTE.cnma.fixed
      x$lower.cnma.common <- x$lower.cnma.fixed
      x$upper.cnma.common <- x$upper.cnma.fixed
      x$statistic.cnma.common <- x$statistic.cnma.fixed
      x$pval.cnma.common <- x$pval.cnma.fixed
      ##
      x$TE.common <- x$TE.fixed
      x$seTE.common <- x$seTE.fixed
      x$lower.common <- x$lower.fixed
      x$upper.common <- x$upper.fixed
      x$statistic.common <- x$statistic.fixed
      x$pval.common <- x$pval.fixed
      ##
      x$Comp.common <- x$Comp.fixed
      x$seComp.common <- x$seComp.fixed
      x$lower.Comp.common <- x$lower.Comp.fixed
      x$upper.Comp.common <- x$upper.Comp.fixed
      x$statistic.Comp.common <- x$statistic.Comp.fixed
      x$pval.Comp.common <- x$pval.Comp.fixed
      ##
      x$Comb.common <- x$Comb.fixed
      x$seComb.common <- x$seComb.fixed
      x$lower.Comb.common <- x$lower.Comb.fixed
      x$upper.Comb.common <- x$upper.Comb.fixed
      x$statistic.Comb.common <- x$statistic.Comb.fixed
      x$pval.Comb.common <- x$pval.Comb.fixed
      ##
      x$L.matrix.common <- x$L.matrix.fixed
      x$Lplus.matrix.common <- x$Lplus.matrix.fixed
      x$H.matrix.common <- x$H.matrix.fixed        
    }
    ##
    if (update.2.8.0)
      x$x$small.values <- setsv(x$x$small.values)
    #
    if (update.3.0.0) {
      x$overall.hetstat <- TRUE
      x$method.tau <- x$x$method.tau
    }
    #
    return(x)
  }
  
  
  ##
  ##  (4) Update discomb object
  ##
  if (inherits(x, "discomb")) {
    if (update.1.3.0) {
      x$statistic.fixed <- x$zval.fixed
      x$statistic.random <- x$zval.random
      x$statistic.nma.fixed <- x$zval.nma.fixed
      x$statistic.nma.random <- x$zval.nma.random
      x$statistic.cnma.fixed <- x$zval.cnma.fixed
      x$statistic.cnma.random <- x$zval.cnma.random
      x$statistic.Comb.fixed <- x$zval.Comb.fixed
      x$statistic.Comb.random <- x$zval.Comb.random
      x$statistic.Comp.fixed <- x$zval.Comp.fixed
      x$statistic.Comp.random <- x$zval.Comp.random
      ##
      x$zval.fixed <- x$zval.random <-
        x$zval.nma.fixed <- x$zval.nma.random <-
          x$zval.cnma.fixed <- x$zval.cnma.random <-
            x$zval.Comb.fixed <- x$zval.Comb.random <-
              x$zval.Comp.fixed <- x$zval.Comp.random <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.0.0) {
      x$fixed <- x$comb.fixed
      x$random <- x$comb.random
      x$level.ma <- x$level.comb
      ##
      x$comb.fixed <- x$comb.random <- x$level.comb <- NULL
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      ##
      x$seTE.adj.common <- x$seTE.adj.fixed
      x$TE.nma.common <- x$TE.nma.fixed
      x$seTE.nma.common <- x$seTE.nma.fixed
      x$lower.nma.common <- x$lower.nma.fixed
      x$upper.nma.common <- x$upper.nma.fixed
      x$statistic.nma.common <- x$statistic.nma.fixed
      x$pval.nma.common <- x$pval.nma.fixed
      x$TE.cnma.common <- x$TE.cnma.fixed
      x$seTE.cnma.common <- x$seTE.cnma.fixed
      x$lower.cnma.common <- x$lower.cnma.fixed
      x$upper.cnma.common <- x$upper.cnma.fixed
      x$statistic.cnma.common <- x$statistic.cnma.fixed
      x$pval.cnma.common <- x$pval.cnma.fixed
      ##
      x$TE.common <- x$TE.fixed
      x$seTE.common <- x$seTE.fixed
      x$lower.common <- x$lower.fixed
      x$upper.common <- x$upper.fixed
      x$statistic.common <- x$statistic.fixed
      x$pval.common <- x$pval.fixed
      ##
      x$Comp.common <- x$Comp.fixed
      x$seComp.common <- x$seComp.fixed
      x$lower.Comp.common <- x$lower.Comp.fixed
      x$upper.Comp.common <- x$upper.Comp.fixed
      x$statistic.Comp.common <- x$statistic.Comp.fixed
      x$pval.Comp.common <- x$pval.Comp.fixed
      ##
      x$Comb.common <- x$Comb.fixed
      x$seComb.common <- x$seComb.fixed
      x$lower.Comb.common <- x$lower.Comb.fixed
      x$upper.Comb.common <- x$upper.Comb.fixed
      x$statistic.Comb.common <- x$statistic.Comb.fixed
      x$pval.Comb.common <- x$pval.Comb.fixed
      ##
      x$L.matrix.common <- x$L.matrix.fixed
      x$Lplus.matrix.common <- x$Lplus.matrix.fixed
      x$H.matrix.common <- x$H.matrix.fixed        
    }
    #
    if (update.3.0.0)
      x$overall.hetstat <- TRUE
    #
    return(x)
  }
  
  
  ##
  ##  (5) Update netsplit object
  ##
  if (inherits(x, "netsplit")) {
    if (update.1.3.0) {
      x$statistic.fixed <- x$zval.fixed
      x$statistic.random <- x$zval.random
      x$statistic.nma.fixed <- x$zval.nma.fixed
      x$statistic.nma.random <- x$zval.nma.random
      x$statistic.cnma.fixed <- x$zval.cnma.fixed
      x$statistic.cnma.random <- x$zval.cnma.random
      x$statistic.Comb.fixed <- x$zval.Comb.fixed
      x$statistic.Comb.random <- x$zval.Comb.random
      x$statistic.Comp.fixed <- x$zval.Comp.fixed
      x$statistic.Comp.random <- x$zval.Comp.random
      ##
      x$zval.fixed <- x$zval.random <-
        x$zval.nma.fixed <- x$zval.nma.random <-
          x$zval.cnma.fixed <- x$zval.cnma.random <-
            x$zval.Comb.fixed <- x$zval.Comb.random <-
              x$zval.Comp.fixed <- x$zval.Comp.random <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.0.0) {
      x$x$fixed <- x$comb.fixed
      x$x$random <- x$comb.random
      x$level.ma <- x$level.comb
      ##
      x$comb.fixed <- x$comb.random <- x$level.comb <- NULL
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      ##
      x$prop.common <- x$prop.fixed
      x$direct.common <- x$direct.fixed
      x$indirect.common <- x$indirect.fixed
      x$compare.common <- x$compare.fixed
    }
    ##
    if (update.2.8.0)
      x$x$small.values <- setsv(x$x$small.values)
    ##
    if (update.3.0.0) {
      x$overall <- TRUE
      x$direct <- TRUE
      x$indirect <- TRUE
      #
      x$show <- gs("show.netsplit")
      x$only.reference <- FALSE
    }
    ##
    return(x)
  }
  
  
  ##
  ##  (6) Update netrank object
  ##
  if (inherits(x, "netrank")) {
    if (update.2.0.0) {
      x$x <- updateversion(x$x)
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      ##
      x$ranking.common <- x$ranking.fixed
      x$Pmatrix.common <- x$Pmatrix.fixed
    }
    ##
    if (update.2.8.0)
      x$small.values <- setsv(x$small.values)
    ##
    return(x)
  }
  
  
  ##
  ##  (7) Update rankogram object
  ##
  if (inherits(x, "rankogram")) {
    if (update.2.0.0) {
      if (is.null(x$cumulative.rankprob))
        x$cumulative.rankprob <- FALSE
      if (is.null(x$nchar.trts))
        x$nchar.trts <- 666
      ##
      x$fixed <- x$comb.fixed
      x$random <- x$comb.random
      ##
      x$comb.fixed <- x$comb.random <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      ##
      x$ranking.common <- x$ranking.fixed
      x$ranking.matrix.common <- x$ranking.matrix.fixed
      x$cumrank.matrix.common <- x$cumrank.matrix.fixed
    }
    ##
    if (update.2.8.0)
      x$small.values <- setsv(x$small.values)
    ##
    return(x)
  }
  
  
  ##
  ##  (8) Update netimpact object
  ##
  if (inherits(x, "netimpact")) {
    if (update.2.0.0) {
      x$x <- updateversion(x$x)
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      x$impact.common <- x$impact.fixed
      ##
      x$version <- packageDescription("netmeta")$Version
    }              
    ##
    if (update.2.8.0)
      x$x$small.values <- setsv(x$x$small.values)
    ##
    if (update.3.0.0) {
      x$method.tau <- x$x$method.tau
      x$nchar.trts <- x$x$nchar.trts
      x$nchar.studlab <- x$x$nchar.studlab
    }
    ##
    return(x)
  }
  
  
  ##
  ##  (9) Update netbind object
  ##
  if (inherits(x, "netbind")) {
    if (update.2.0.0) {
      x$x$fixed <- x$comb.fixed
      x$x$random <- x$comb.random
      x$x$level.ma <- x$level.comb
      ##
      x$comb.fixed <- x$comb.random <- x$level.comb <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      x$x$common <- x$x$fixed
      ##
      x$version <- packageDescription("netmeta")$Version
    }              
    ##
    return(x)
  }
  
  
  ##
  ##  (10) Update netposet object
  ##
  if (inherits(x, "netposet")) {
    if (update.2.0.0) {
      x$fixed <- x$comb.fixed
      x$random <- x$comb.random
      ##
      x$comb.fixed <- x$comb.random <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.5.0) {
      x$common <- x$fixed
      ##
      x$P.common <- x$P.fixed
      x$M0.common <- x$M0.fixed
      x$M.common <- x$M.fixed
      x$O.common <- x$O.fixed
      ##
      x$version <- packageDescription("netmeta")$Version
    }              
    ##
    if (update.2.8.0)
      x$small.values <- setsv(x$small.values)
    ##
    return(x)
  }
  
  
  ##
  ##  (11) Update netcontrib object
  ##
  if (inherits(x, "netcontrib")) {
    if (update.2.0.0) {
      x$x$fixed <- x$comb.fixed
      x$x$random <- x$comb.random
      ##
      x$comb.fixed <- x$comb.random <- NULL
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.5.0) {
      res$common <- res$fixed
      ##
      if (!is.null(res$tictoc.fixed))
        res$tictoc.common <- res$tictoc.fixed
    }
    ##
    if (update.2.8.0)
      x$x$small.values <- setsv(x$x$small.values)
    ##
    return(x)
  }
  
  
  ##
  ##  (12) Update netcomparison object
  ##
  if (inherits(x, "netcomparison")) {
    if (update.2.5.0) {
      res$common <- res$fixed
      ##
      res$TE.common <- res$TE.fixed
      res$seTE.common <- res$seTE.fixed
      res$lower.common <- res$lower.fixed
      res$upper.common <- res$upper.fixed
      res$statistic.common <- res$statistic.fixed
      res$pval.common <- res$pval.fixed
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.8.0)
      x$x$small.values <- setsv(x$x$small.values)
    ##
    return(x)
  }
  
  
  ##
  ##  (13) Update netcomplex object
  ##
  if (inherits(x, "netcomplex")) {
    if (update.2.5.0) {
      res$common <- res$fixed
      ##
      res$Comb.common <- res$Comb.fixed
      res$seComb.common <- res$seComb.fixed
      res$lower.Comb.common <- res$lower.Comb.fixed
      res$upper.Comb.common <- res$upper.Comb.fixed
      res$statistic.Comb.common <- res$statistic.Comb.fixed
      res$pval.Comb.common <- res$pval.Comb.fixed
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    if (update.2.8.0)
      x$x$small.values <- setsv(x$x$small.values)
    ##
    return(x)
  }
  
  
  ##
  ##  (14) Update nettable object
  ##
  if (inherits(x, "nettable")) {
    if (update.2.5.0) {
      res$common <- res$fixed
      res$x$common <- res$x$fixed
      ##
      x$version <- packageDescription("netmeta")$Version
    }
    ##
    return(x)
  }
  
  x
}


update_needed <- function(version, major = 0, minor = 0,
                          verbose = FALSE) {
  if (is.null(version)) {
    version <- 0.1
    major.cur <- 0
    minor.cur <- 1
  }
  else {
    version <- unlist(strsplit(version, "-")[1])
    major.cur <-
      as.numeric(unlist(strsplit(version, ".", fixed = TRUE))[1])
    minor.cur <-
      as.numeric(unlist(strsplit(version, ".", fixed = TRUE))[2])
  }
  ##
  res <-
    ifelse(major.cur < major,
           TRUE, ifelse(major.cur > major,
                        FALSE, minor.cur < minor))
  if (res & verbose)
    message(paste0("Update to netmeta, version ", major, ".", minor))
  ##
  res
}

multiarm <- function(r, studlab, func.inverse) {
  ##
  ## Dimension of r and R
  ##
  m <- length(r) # Number of edges
  ##
  k <- (1 + sqrt(8 * m + 1)) / 2 # Number of vertices
  if (!(abs(k - round(k)) < .Machine$double.eps^0.5))
    stop("Wrong number of comparisons in multi-arm study.", call. = FALSE)
  ##
  ## Construct edge-vertex incidence matrix of complete graph of
  ## dimension k
  ##
  B <- createB(ncol = k)
  ##
  ## Distribute the edge variances on a symmetrical matrix R of
  ## dimension k x k
  ##
  R <- diag(diag(t(B) %*% diag(r, nrow = m) %*% B)) -
    t(B) %*% diag(r, nrow = m) %*% B
  ##
  ## Construct pseudoinverse Lt from given variance (resistance) matrix R
  ## using a theorem equivalent to Theorem 7 by Gutman & Xiao
  ## Lt <- -0.5 * (R - (R %*% J + J %*% R) / k + J %*%R %*% J / k^2)
  ##
  Lt <- -0.5 * t(B) %*% B %*% R %*% t(B) %*% B / k^2
  ##
  ## Compute Laplacian matrix L from Lt
  ##
  L <- do.call(func.inverse, list(X = Lt))
  ##
  ## Compute weight matrix W and variance matrix V from Laplacian L
  ## 
  W <- diag(diag(L)) - L
  ##
  ## Replace small negative weights with zeros
  ## (i.e., if an absolute negative weight contributes less than 0.1%)
  ##
  W[W < 0 & (abs(W) / sum(abs(W)[lower.tri(W)])) < 0.001] <- 0
  #
  V <- 1 / W
  ##
  ## Compute original variance vector v from V
  ##
  v <- rep(0, m)
  edge <- 0
  for (i in 1:(k - 1)) {
    for (j in (i + 1):k) {
      edge <- edge + 1
      v[edge] <- V[i, j]
    }
  }
  ##
  ## Result
  ##
  res <- list(k = k, r = r, R = R, Lt = Lt, L = L, W = W, V = V, v = v)
  res
}

multicols <- function(studies, narms, missing.col.multiarm,
                      col.multiarm, alpha.transparency) {
  ##
  ## Define coloured regions for multi-arm studies
  ##
  dat <- data.frame(studies = studies, narms = narms,
                    idx = seq_along(studies))
  dat <- dat[rev(order(dat$narms)), ]
  dat <- dat[dat$narms > 2, ]
  ##
  o <- order(dat$idx)
  multiarm.studies <- dat$studies
  ##
  n.multi <- length(multiarm.studies)
  ##
  missing.col.multiarm <- missing(col.multiarm)
  ##
  o.cols <- TRUE
  ##
  if (missing.col.multiarm | is.null(col.multiarm))
    cols <- sequential_hcl(n.multi, alpha = alpha.transparency)
  else {
    ##
    if (is.function(col.multiarm)) {
      mcname <- deparse(substitute(col.multiarm))
      ##
      colfun <- function(fcall, fname)
        length(grep(fname, fcall)) > 0
      ##
      if (colfun(mcname, "rainbow_hcl"))
        cols <-
        rainbow_hcl(n.multi, start = 240, end = 60, alpha = alpha.transparency)
      else if (colfun(mcname, "sequential_hcl"))
        cols <- sequential_hcl(n.multi, alpha = alpha.transparency)
      else if (colfun(mcname, "diverge_hcl"))
        cols <- diverge_hcl(n.multi, alpha = alpha.transparency)
      else if (colfun(mcname, "heat_hcl"))
        cols <- heat_hcl(n.multi, alpha = alpha.transparency)
      else if (colfun(mcname, "terrain_hcl"))
        cols <- terrain_hcl(n.multi, alpha = alpha.transparency)
      else if (colfun(mcname, "diverge_hsv"))
        cols <- diverge_hsv(n.multi, alpha = alpha.transparency)
      else if (colfun(mcname, "choose_palette")) {
        fcolm <- choose_palette(n = n.multi)
        cols <- fcolm(n = n.multi)
      }
      else
        cols <- sapply(n.multi, col.multiarm, alpha = alpha.transparency)
      ##
      if (colfun(mcname, "sequential_hcl") | colfun(mcname, "diverge_hcl") |
          colfun(mcname, "heat_hcl"))
        cols <- rev(cols)
    }
  }
  ##
  if (!missing.col.multiarm & is.character(col.multiarm)) {
    if (length(col.multiarm) > 1 & length(col.multiarm) != n.multi)
      stop("Length of argument 'col.multiarm' must be equal to one or ",
           "the number of multi-arm studies: ", n.multi)
    cols <- col.multiarm
    o.cols <- FALSE
  }
  
  
  res <- data.frame(studlab = multiarm.studies[o])
  ##
  if (o.cols)
    res$col <- cols[o]
  else
    res$col <- cols
  ##
  res
}

#' Print method for rankograms
#' 
#' @description
#' Print method for objects of class \code{rankogram}.
#' 
#' @param x An R object of class \code{rankogram}.
#' @param common A logical indicating to print ranking probabilities
#'   and SUCRAs for the common effects model.
#' @param random A logical indicating to print ranking probabilities
#'   and SUCRAs for the random effects model.
#' @param cumulative.rankprob A logical indicating whether cumulative
#'   ranking probabilities should be printed.
#' @param sort A logical indicating whether treatments should be
#'   sorted by decreasing SUCRAs.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param digits Minimal number of significant digits, see
#'   \code{\link{print.default}}.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments for printing.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{rankogram}}, \code{\link{plot.rankogram}}
#'
#' @references
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#' 
#' @keywords print
#'
#' @examples
#' # Examples: example(rankogram.netmeta)
#'
#' @rdname print.rankogram
#' @method print rankogram
#' @export

print.rankogram <- function(x,
                            common = x$common,
                            random = x$random,
                            cumulative.rankprob = x$cumulative.rankprob,
                            sort = TRUE,
                            nchar.trts = x$nchar.trts,
                            digits = gs("digits.prop"),
                            details.methods = gs("details"),
                            legend = gs("legend"),
                            warn.deprecated = gs("warn.deprecated"),
                            ...) {
  
  #
  #
  # (1) Check for rankogram object and upgrade object
  #
  #
  
  chkclass(x, "rankogram")
  x <- updateversion(x)
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  
  chklogical(cumulative.rankprob)
  chklogical(sort)
  #
  chknumeric(nchar.trts, length = 1)
  #
  chknumeric(digits, length = 1)
  chklogical(details.methods)
  chklogical(legend)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  
  
  #
  #
  # (3) Print results
  #
  #
  
  if (common | random)
    cat(if (cumulative.rankprob)
      "Cumulative ranking probabilities" else "Rankogram",
      " (based on ", x$nsim, " repetition",
      if (x$nsim > 1) "s", ")\n\n",
      sep = "")
  #
  if (common) {
    if (cumulative.rankprob)
      rank.common <- x$cumrank.matrix.common
    else
      rank.common <- x$ranking.matrix.common
    rownames(rank.common) <- treats(rank.common, nchar.trts)
    #
    if (sort)
      rank.common <-
      rank.common[rev(order(x$ranking.common)), , drop = FALSE]
    #
    cat("Common effects model: \n\n")
    prmatrix(formatN(rank.common, digits), quote = FALSE, right = TRUE, ...)
    if (random)
      cat("\n")
  }
  #
  if (random) {
    if (cumulative.rankprob)
      rank.random <- x$cumrank.matrix.random
    else
      rank.random <- x$ranking.matrix.random
    rownames(rank.random) <-
      treats(rank.random, nchar.trts)
    #
    if (sort)
      rank.random <-
      rank.random[rev(order(x$ranking.random)), , drop = FALSE]
    #
    if (is.null(x$pooled) || x$pooled != "unspecified")
      cat("Random effects model: \n\n")
    else
      random <- FALSE
    #
    prmatrix(formatN(rank.random, digits), quote = FALSE, right = TRUE, ...)
  }
  #
  # Print details of network meta-analysis methods
  #
  if (details.methods) {
    text.details <-
      textmeth(x, random, TRUE)
    #
    cat(text.details)
  }
  #
  # Add legend with abbreviated treatment labels
  #
  if ((common | random) & legend) {
    if (random)
      trts <- rownames(x$ranking.matrix.random)
    else
      trts <- rownames(x$ranking.matrix.common)
    #
    legendabbr(trts, treats(trts, nchar.trts), TRUE)
  }
  
  invisible()
}

#' Subgroup analysis for network meta-analysis
#'
#' @description
#' Subgroup analysis for objects of class \code{netmeta}.
#'
#' @param x An object of class \code{netmeta} (or \code{subgroup.netmeta}).
#' @param subgroup A vector defining the subgroups considered in the
#'   network meta-analysis.
#' @param only.connected A logical indicating whether networks of subgroups
#'   must be connected.
#' @param common A logical indicating whether results for common
#'   effect subgroup network meta-analysis should be printed.
#' @param random A logical indicating whether results for random
#'   effects subgroup network meta-analysis should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau}. Either \code{"DL"}, \code{"REML"}, or
#'   \code{"ML"}, can be abbreviated.
#' @param level.ma The level used to calculate confidence intervals
#'   for network estimates.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names
#'   (see \code{\link{netmeta}}).
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   errors.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistic Q, see \code{print.default}.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity test, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance \eqn{\tau^2}, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for
#'   \eqn{\tau}, the square root of the between-study variance
#'   \eqn{\tau^2}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.se A logical specifying whether standard errors should be
#'   printed.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.Q A logical value indicating whether to print the
#'   results of the test of heterogeneity.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param \dots Additional arguments.
#'
#' @return
#' An object of class \code{"subgroup.netmeta"} with corresponding \code{print}
#'   and \code{forest} function.
#'
#' @seealso \code{\link{forest.subgroup.netmeta}}, \code{\link{forest.netmeta}}
#'
#' @examples
#' \donttest{
#' data("Senn2013")
#' # Add variable with (fictitious) risk of bias values
#' Senn2013$rob <- NA
#' set.seed(1909)
#' for (i in unique(Senn2013$studlab))
#'   Senn2013$rob[Senn2013$studlab == i] <- sample(1:3, 1)
#' Senn2013$rob <- factor(Senn2013$rob, levels = 1:3,
#'   labels = c("low", "moderate", "high"))
#' # Conduct network meta-analysis
#' net <- netmeta(TE, seTE, treat1.long, treat2.long, studlab,
#'   data = Senn2013, sm = "MD", reference = "plac", nchar.trts = 4)
#' # Conduct subgroup network meta-analysis
#' subgroup(net, rob, common = FALSE)
#' }
#'
#' @method subgroup netmeta
#' @export

subgroup.netmeta <- function(x, subgroup, only.connected = FALSE,
                             common = x$common, random = x$random,
                             method.tau = x$method.tau,
                             level.ma = x$level.ma,
                             backtransf = x$backtransf,
                             nchar.trts = x$nchar.trts,
                             ...) {
  
  chkclass(x, "netmeta")
  #
  chklogical(only.connected)
  chklogical(common)
  chklogical(random)
  chklevel(level.ma)
  chklogical(backtransf)
  #
  method.tau <- setchar(method.tau, c("DL", "REML", "ML"))
  #
  chknumeric(nchar.trts, min = 1, length = 1)
  
  
  # Get rid of warning 'Undefined global functions or variables'
  .studlab <- .treat1 <- .treat2 <- .TE <- .seTE <-
    comparison <- subnet <- NULL
    
  trts <- x$seq
  subgroup.name <- deparse(substitute(subgroup))
  #
  subgroup <- catch("subgroup", match.call(), x$data, sys.frame(sys.parent()))
  chknull(subgroup)
  if (is.factor(subgroup))
    levs <- levels(subgroup)
  else
    levs <- sort(unique(subgroup))
  #
  matNA <- x$TE.common[x$seq, x$seq]
  matNA[!is.na(matNA)] <- NA
  #
  TE.common <- seTE.common <- lower.common <- upper.common <-
    TE.random <- seTE.random <- lower.random <- upper.random <-
    tau2.matrix <- tau.matrix <- A.matrix <-
    array(NA, dim = c(nrow(matNA), ncol(matNA), length(levs)),
          dimnames = list(rownames(matNA), colnames(matNA), levs))
  #
  networks <- vector("list", length(levs))
  names(networks) <- levs
  #
  res.c <- res.r <- data.frame()
  #
  for (k in seq_along(levs)) {
    dat.k <- subset(x$data, subgroup == levs[k])
    #
    nc.k <- netconnection(.treat1, .treat2, .studlab, data = dat.k)
    #
    if (only.connected & nc.k$n.subnets > 1)
      stop("Subgroup '", subgroup.name, " = ", levs[k],
           "' does not have a connected network.",
           call. = FALSE)
    #
    dat.nc.k <- as.data.frame(nc.k)
    #
    for (s in seq_len(nc.k$n.subnets)) {
      studies.s <- subset(dat.nc.k, subnet == s)$studlab
      dat.ks <- subset(dat.k, .studlab %in% studies.s)
      #
      net.ks <- netmeta(.TE, .seTE, .treat1, .treat2, .studlab, data = dat.ks,
                        sm = x$sm, method.tau = method.tau,
                        level.ma = level.ma, backtransf = backtransf)
      if (nc.k$n.subnets > 1)
        networks[[k]][[s]] <- net.ks
      else
        networks[[k]] <- net.ks
      #
      for (i in net.ks$trts) {
        for (j in net.ks$trts) {
          TE.common[i, j, k] <- net.ks$TE.common[i, j]
          seTE.common[i, j, k] <- net.ks$seTE.common[i, j]
          lower.common[i, j, k] <- net.ks$lower.common[i, j]
          upper.common[i, j, k] <- net.ks$upper.common[i, j]
          #
          TE.random[i, j, k] <- net.ks$TE.random[i, j]
          seTE.random[i, j, k] <- net.ks$seTE.random[i, j]
          lower.random[i, j, k] <- net.ks$lower.random[i, j]
          upper.random[i, j, k] <- net.ks$upper.random[i, j]
          #
          tau2.matrix[i, j, k] <- net.ks$tau2
          tau.matrix[i, j, k] <- net.ks$tau
          #
          A.matrix[i, j, k] <- net.ks$A.matrix[i, j]
        }
      }
    }
    #
    diag(TE.common[, , k]) <- NA
    diag(seTE.common[, , k]) <- NA
    diag(lower.common[, , k]) <- NA
    diag(upper.common[, , k]) <- NA
    #
    diag(TE.random[, , k]) <- NA
    diag(seTE.random[, , k]) <- NA
    diag(lower.random[, , k]) <- NA
    diag(upper.random[, , k]) <- NA
    #
    diag(tau2.matrix[, , k]) <- NA
    diag(tau.matrix[, , k]) <- NA
    diag(A.matrix[, , k]) <- NA
  }
  #
  for (i in seq_len(length(trts) - 1)) {
    for (j in (i + 1):length(trts)) {
      m.ij <- metagen(TE.common[trts[i], trts[j], ],
                      seTE.common[trts[i], trts[j], ],
                      method.tau = "DL", method.tau.ci = "")
      #
      for (k in levs) {
        res.ijk <-
          data.frame(treat1 = trts[i], treat2 = trts[j],
                     subgroup = k,
                     k = A.matrix[trts[i], trts[j], k],
                     TE = TE.common[trts[i], trts[j], k],
                     seTE = seTE.common[trts[i], trts[j], k],
                     lower = lower.common[trts[i], trts[j], k],
                     upper = upper.common[trts[i], trts[j], k],
                     Q = m.ij$Q, df.Q = m.ij$df.Q, pval.Q = m.ij$pval.Q)
        #
        if (nrow(res.c) == 0)
          res.c <- res.ijk
        else
          res.c <- rbind(res.c, res.ijk)
      }
    }
  }
  #
  for (i in seq_len(length(trts) - 1)) {
    for (j in (i + 1):length(trts)) {
      m.ij <- metagen(TE.random[trts[i], trts[j], ],
                      seTE.random[trts[i], trts[j], ],
                      method.tau = method.tau, method.tau.ci = "")
      #
      for (k in levs) {
        res.ijk <-
          data.frame(treat1 = trts[i], treat2 = trts[j],
                     subgroup = k,
                     k = A.matrix[trts[i], trts[j], k],
                     TE = TE.random[trts[i], trts[j], k],
                     seTE = seTE.random[trts[i], trts[j], k],
                     lower = lower.random[trts[i], trts[j], k],
                     upper = upper.random[trts[i], trts[j], k],
                     tau2 = tau2.matrix[trts[i], trts[j], k],
                     tau = tau.matrix[trts[i], trts[j], k],
                     Q = m.ij$Q, df.Q = m.ij$df.Q, pval.Q = m.ij$pval.Q)
        #
        if (nrow(res.r) == 0)
          res.r <- res.ijk
        else
          res.r <- rbind(res.r, res.ijk)
      }
    }
  }
  #
  res.c <- subset(res.c, !is.na(res.c$TE))
  #
  sel.c <- duplicated(res.c[, c("Q", "df.Q", "pval.Q")]) & res.c$df.Q != 0
  res.c$Q[sel.c] <- NA
  res.c$df.Q[sel.c] <- NA
  res.c$pval.Q[sel.c] <- NA
  #
  res.r <- subset(res.r, !is.na(res.r$TE))
  #
  sel.r <- duplicated(res.r[, c("Q", "df.Q", "pval.Q")]) & res.r$df.Q != 0
  res.r$Q[sel.r] <- NA
  res.r$df.Q[sel.r] <- NA
  res.r$pval.Q[sel.r] <- NA
  #
  res.r <- subset(res.r, !is.na(res.r$TE))
  #
  rownames(res.c) <- seq_len(nrow(res.c))
  rownames(res.r) <- seq_len(nrow(res.r))
  #
  x$common <- common
  x$random <- random
  x$method.tau <- method.tau
  x$level.ma <- level.ma
  x$backtransf <- backtransf
  x$nchar.trts <- nchar.trts
  #
  res <- list(common = res.c, random = res.r,
              #
              TE.common = TE.common, seTE.common = seTE.common,
              lower.common = lower.common, upper.common = upper.common,
              #
              TE.random = TE.random, seTE.random = seTE.random,
              lower.random = lower.random, upper.random = upper.random,
              #
              tau2.matrix = tau2.matrix, tau.matrix = tau.matrix,
              A.matrix = A.matrix,
              networks = networks,
              #
              tau.preset = x$tau.preset,
              method.tau = x$method.tau,
              x = x,
              call = match.call(),
              version = packageDescription("netmeta")$Version)
  #
  class(res) <- "subgroup.netmeta"
  #
  res
}


#' @rdname subgroup.netmeta
#' @export subgroup

subgroup <- function(x, ...)
  UseMethod("subgroup")


#' @rdname subgroup.netmeta
#' @method print subgroup.netmeta
#' @export

print.subgroup.netmeta <- function(x,
                                   common = x$x$common,
                                   random = x$x$random,
                                   backtransf = x$x$backtransf,
                                   #
                                   nchar.trts = x$x$nchar.trts,
                                   #
                                   digits = gs("digits"),
                                   digits.se = gs("digits.se"),
                                   digits.Q = gs("digits.Q"),
                                   digits.pval.Q = gs("digits.pval.Q"),
                                   digits.tau2 = gs("digits.tau2"),
                                   digits.tau = gs("digits.tau"),
                                   #
                                   big.mark = gs("big.mark"),
                                   scientific.pval = gs("scientific.pval"),
                                   zero.pval = gs("zero.pval"),
                                   JAMA.pval = gs("JAMA.pval"),
                                   #
                                   print.se = !backtransf,
                                   print.tau2 = gs("print.tau2"),
                                   print.tau = gs("print.tau"),
                                   print.Q = gs("print.Q"),
                                   #
                                   text.tau2 = gs("text.tau2"),
                                   text.tau = gs("text.tau"),
                                   #
                                   details.methods = gs("details"),
                                   legend = gs("legend"),
                                   #
                                   ...) {
  
  chkclass(x, "subgroup.netmeta")
  #
  chklogical(common)
  chklogical(random)
  chklogical(backtransf)
  #
  if (is.null(nchar.trts))
    nchar.trts <- 666
  else
    chknumeric(nchar.trts, min = 1, length = 1)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.pval.Q, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  #
  chkchar(big.mark, length = 1)
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chklogical(print.se)
  chklogical(print.tau2)
  chklogical(print.tau)
  chklogical(print.Q)
  #
  chkchar(text.tau2)
  chkchar(text.tau)
  #
  chklogical(details.methods)
  chklogical(legend)
  
  # Get rid of warning 'Undefined global functions or variables'
  .seTE <- .studlab <- .TE <- .treat1 <- .treat2 <- comparison <-
    df.Q <- subnet <- treat1 <- treat2 <- k <- TE <- NULL
  #
  trts <- x$x$trts
  trts.abbr <- treats(x$x$trts, nchar.trts)
  sm <- sm.lab <- x$x$sm
  #
  if (!backtransf & (is_relative_effect(sm) | sm == "VE"))
    sm.lab <- paste0("log", if (sm == "VE") "VR" else sm)
  ##
  ci.lab <- paste0(round(100 * x$x$level.ma, 1), "% CI")
  
  if (common) {
    cat("Common effects model: \n\n")
    #
    dat.common <- x$common %>% filter(is.na(df.Q) | df.Q > 0)
    #
    if (backtransf) {
      dat.common$TE    <- backtransf(dat.common$TE, sm)
      dat.common$lower <- backtransf(dat.common$lower, sm)
      dat.common$upper <- backtransf(dat.common$upper, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- dat.common$lower
        dat.common$lower <- dat.common$upper
        dat.common$upper <- tmp.l
      }
    }
    #
    dat.common$TE <-
      formatN(dat.common$TE, digits = digits, text.NA = ".")
    #
    if (print.se)
      dat.common$seTE <-
        formatN(dat.common$seTE, digits = digits.se, text.NA = ".")
    else
      dat.common$seTE <- NULL
    #
    dat.common$ci <-
      formatCI(formatN(round(dat.common$lower, digits),
                       digits, ".", big.mark = big.mark),
               formatN(round(dat.common$upper, digits),
                       digits, ".", big.mark = big.mark))
    #
    if (print.Q) {
      dat.common$Q <-
        formatN(dat.common$Q, digits = digits.Q, text.NA = ".")
      dat.common$df.Q <-
        formatN(dat.common$df.Q, digits = 0, text.NA = ".")
      dat.common$pval.Q <-
        formatPT(dat.common$pval.Q,
                 digits = digits.pval.Q, lab.NA = ".",
                 scientific = scientific.pval,
                 zero = zero.pval, JAMA = JAMA.pval)
    }
    #
    if (any(trts != trts.abbr)) {
      dat.common$treat1 <-
        factor(dat.common$treat1, levels = trts, labels = trts.abbr)
      dat.common$treat2 <-
        factor(dat.common$treat2, levels = trts, labels = trts.abbr)
    }
    #
    rowlabs <- paste(dat.common$treat1, dat.common$treat2, sep = x$x$sep.trts)
    dat.common %<>% select(-treat1, -treat2) %>%
      select(subgroup, k, TE, if (print.se) "seTE", ci,
             if (print.Q) "Q", if (print.Q) "df.Q", if (print.Q) "pval.Q")
    #
    names(dat.common)[names(dat.common) == "TE"] <- sm.lab
    names(dat.common)[names(dat.common) == "ci"] <- ci.lab
    #
    prmatrix(dat.common, quote = FALSE, right = TRUE, rowlab = rowlabs)
    #
    if (random)
      cat("\n")
  }
  #
  # Print results for random effects model
  #
  if (random) {
    cat("Random effects model: \n\n")
    #
    dat.random <- x$random %>% filter(is.na(df.Q) | df.Q > 0)
    #
    if (backtransf) {
      dat.random$TE    <- backtransf(dat.random$TE, sm)
      dat.random$lower <- backtransf(dat.random$lower, sm)
      dat.random$upper <- backtransf(dat.random$upper, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- dat.random$lower
        dat.random$lower <- dat.random$upper
        dat.random$upper <- tmp.l
      }
    }
    #
    dat.random$TE <-
      formatN(dat.random$TE, digits = digits, text.NA = ".",
              big.mark = big.mark)
    #
    if (print.se)
      dat.random$seTE <-
        formatN(dat.random$seTE, digits = digits.se, text.NA = ".")
    else
      dat.random$seTE <- NULL
    #
    dat.random$ci <-
      formatCI(formatN(round(dat.random$lower, digits),
                     digits, ".", big.mark = big.mark),
             formatN(round(dat.random$upper, digits),
                     digits, ".", big.mark = big.mark))
    #
    if (print.Q) {
      dat.random$Q <-
        formatN(dat.random$Q, digits = digits.Q, text.NA = ".",
                big.mark = big.mark)
      dat.random$df.Q <-
        formatN(dat.random$df.Q, digits = 0, text.NA = ".",
                big.mark = big.mark)
      dat.random$pval.Q <-
        formatPT(dat.random$pval.Q,
                 digits = digits.pval.Q, lab.NA = ".",
                 scientific = scientific.pval,
                 zero = zero.pval, JAMA = JAMA.pval)
    }
    #
    if (print.tau2)
      dat.random$tau2 <-
      formatPT(dat.random$tau2, digits = digits.tau2, lab.NA = ".",
               big.mark = big.mark)
    if (print.tau2)
      dat.random$tau <-
      formatPT(dat.random$tau, digits = digits.tau, lab.NA = ".",
               big.mark = big.mark)
    #
    if (any(trts != trts.abbr)) {
      dat.random$treat1 <-
        factor(dat.random$treat1, levels = trts, labels = trts.abbr)
      dat.random$treat2 <-
        factor(dat.random$treat2, levels = trts, labels = trts.abbr)
    }
    #
    rowlabs <- paste(dat.random$treat1, dat.random$treat2, sep = x$x$sep.trts)
    dat.random %<>% select(-treat1, -treat2) %>%
      select(subgroup, k, TE, if (print.se) "seTE", ci,
             if (print.tau2) "tau2", if (print.tau) "tau",
             if (print.Q) "Q", if (print.Q) "df.Q", if (print.Q) "pval.Q")
    #
    names(dat.random)[names(dat.random) == "TE"] <- sm.lab
    names(dat.random)[names(dat.random) == "ci"] <- ci.lab
    #
    prmatrix(dat.random, quote = FALSE, right = TRUE, rowlab = rowlabs)
  }
  #
  # Print details of network meta-analysis methods
  #
  if (details.methods) {
    text.details <-
      textmeth(x, random, print.tau2, print.tau,
               text.tau2, text.tau, digits.tau2, digits.tau,
               big.mark = big.mark)
    #
    cat(text.details)
  }
  #
  # Add legend with abbreviated treatment labels
  #
  legendabbr(trts, trts.abbr, legend)
  #
  invisible(NULL)
}

#' Network meta-analysis of interventions for smoking cessation
#' 
#' @description
#' Network meta-analysis comparing the effects of a number of
#' interventions for smoking cessation.
#' 
#' These data are used as an example in Dias et al. (2013), page 651.
#' 
#' @name smokingcessation
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{event1}}\tab number of individuals with successful
#'   smoking cessation in arm 1 \cr
#' \bold{\emph{n1}}\tab number of individuals in arm 1 \cr
#' \bold{\emph{event2}}\tab number of individuals with successful
#'   smoking cessation in arm 2 \cr
#' \bold{\emph{n2}}\tab number of individuals in arm 2 \cr
#' \bold{\emph{event3}}\tab number of individuals with successful
#'   smoking cessation in arm 3 \cr
#' \bold{\emph{n3}}\tab number of individuals in arm 3 \cr
#' \bold{\emph{treat1}}\tab treatment 1 \cr
#' \bold{\emph{treat2}}\tab treatment 2 \cr \bold{\emph{treat3}}\tab
#'   treatment 3
#' }
#' 
#' @seealso \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
#'   \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
#' 
#' @source
#' Dias S, Welton NJ, Sutton AJ, Caldwell DM, Lu G and Ades AE (2013):
#' Evidence Synthesis for Decision Making 4: Inconsistency in networks
#' of evidence based on randomized controlled trials.
#' \emph{Medical Decision Making},
#' \bold{33}, 641--56
#' 
#' @keywords datasets
#' 
#' @examples
#' data(smokingcessation)
#' head(smokingcessation)
#' 
#' # Examples: example(netmeta)

NULL

#' Print method for objects of class netbind
#' 
#' @description
#' Print method for objects of class \code{netbind}.
#' 
#' @param x An object of class \code{netbind} or
#'   \code{summary.netbind}.
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netbind}}
#' 
#' @keywords print
#' 
#' @examples
#' # Examples: example(netbind)
#' 
#' @method print netbind
#' @export

print.netbind <- function(x,
                          common = x$x$common,
                          random = x$x$random,
                          ##
                          warn.deprecated = gs("warn.deprecated"),
                          ##
                          ...) {
  
  ##
  ##
  ## (1) Check for netbind object and upgrade object
  ##
  ##
  chkclass(x, "netbind")
  x <- updateversion(x)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (3) Print results
  ##
  ##
  
  nam <- c("name", "treat",
           "TE", "seTE", "lower", "upper", "statistic", "pval")
  #
  if (common) {
    cnames <- nam
    if (!is.null(x$common$method) && length(unique(x$common$method)) > 1)
      cnames <- c(cnames, "method")
    #
    cat("Common effects model\n\n")
    print(x$common[, cnames])
    if (random)
      cat("\n")
  }
  ##
  if (random) {
    rnames <- nam
    if (!is.null(x$random$method) && length(unique(x$random$method)) > 1)
      rnames <- c(rnames, "method")
    #
    cat("Random effects model\n\n")
    print(x$random[, rnames])
  }
  
  invisible(NULL)
}

direct.indirect <- function(x, tol.direct = 0.0005) {
  
  chkclass(x, "netmeta")
  chknumeric(tol.direct, min = 0, length = 1)
    
  ##
  ## Direct and indirect estimates from common effects model
  ##
  sel.df <- abs(x$P.common) < tol.direct
  ##
  TE.direct.common <- setNA_subset(x$TE.direct.common, sel.df)
  seTE.direct.common <- setNA_subset(x$seTE.direct.common, sel.df)
  ##
  sel.if <- abs(x$P.common - 1) < tol.direct
  ##
  TE.indirect.common <- setNA_subset(x$TE.indirect.common, sel.if)
  seTE.indirect.common <- setNA_subset(x$seTE.indirect.common, sel.if)
  ## Set indirect estimate to network estimate if k = 0
  TE.indirect.common[x$A.matrix == 0] <- x$TE.common[x$A.matrix == 0]
  seTE.indirect.common[x$A.matrix == 0] <- x$seTE.common[x$A.matrix == 0]
  diag(TE.indirect.common) <- diag(seTE.indirect.common) <- NA
  
  ##
  ## Direct and indirect estimates from random effects model
  ##
  sel.dr <- abs(x$P.random) < tol.direct
  ##
  TE.direct.random <- setNA_subset(x$TE.direct.random, sel.df)
  seTE.direct.random <- setNA_subset(x$seTE.direct.random, sel.df)
  ##
  sel.ir <- abs(x$P.random - 1) < tol.direct
  ##
  TE.indirect.random <- setNA_subset(x$TE.indirect.random, sel.ir)
  seTE.indirect.random <- setNA_subset(x$seTE.indirect.random, sel.ir)
  ## Set indirect estimate to network estimate if k = 0
  TE.indirect.random[x$A.matrix == 0] <- x$TE.random[x$A.matrix == 0]
  seTE.indirect.random[x$A.matrix == 0] <- x$seTE.random[x$A.matrix == 0]
  diag(TE.indirect.random) <- diag(seTE.indirect.random) <- NA
  
  ##
  ## Calculate confidence limits
  ##
  ci.nf <- ci(x$TE.common, x$seTE.common, x$level.ma)
  ci.df <- ci(TE.direct.common, seTE.direct.common, x$level.ma)
  ci.if <- ci(TE.indirect.common, seTE.indirect.common, x$level.ma)
  ##
  ci.nr <- ci(x$TE.random, x$seTE.random, x$level.ma)
  ci.dr <- ci(TE.direct.random, seTE.direct.random, x$level.ma)
  ci.ir <- ci(TE.indirect.random, seTE.indirect.random, x$level.ma)
  ##
  ci.nf$z <- ci.df$z <- ci.if$z <- 
    ci.nr$z <- ci.dr$z <- ci.ir$z <- NULL
  
  res <- list(nma.common = ci.nf, nma.random = ci.nr,
              direct.common = ci.df, direct.random = ci.dr,
              indirect.common = ci.if, indirect.random = ci.ir,
              x = x)
  ##
  res
}

matitle <- function(x) {
  tl <- options()$width - 7
  ##
  if (!is.null(x$title))
    if (x$title != "")
      if (nchar(x$title) <= tl)
        cat("Title: ", x$title, "\n\n", sep="")
      else
        cat("Title: ", substring(x$title, 1, tl - 4),
            " ...\n\n", sep = "")
}

#' Calculate effect of arbitrary complex interventions in component
#' network meta-analysis
#' 
#' @description
#' Calculate effect of arbitrary complex interventions (i.e.,
#' combinations of several components) in component network
#' meta-analysis.
#' 
#' @param x An object of class \code{netcomb} or \code{netcomplex}
#'   (print function).
#' @param complex A matrix, vector or single numeric defining the
#'   complex intervention(s) (see Details).
#' @param common A logical indicating whether results for common
#'   effects model should be conducted.
#' @param random A logical indicating whether results for random
#'   effects model should be conducted.
#' @param level The level used to calculate confidence intervals for
#'   combinations of components.
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique names for components (see
#'   Details).
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. If \code{backtransf=TRUE}, results
#'   for \code{sm="OR"} are printed as odds ratios rather than log
#'   odds ratios.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z-value
#'   of test for overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for
#'   p-values, see \code{print.default}.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   combination effect should be printed according to JAMA reporting
#'   standards.
#' @param big.mark A character used as thousands separator.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#'
#' @details
#' R functions \code{\link{netcomb}} and \code{\link{discomb}} only
#' report results for complex interventions present in the
#' network. This function can be used to calculate the effect for
#' arbitrary complex interventions.
#'
#' Complex interventions can be specified using argument \code{complex}:
#' \itemize{
#' \item a character vector with definition of complex interventions,
#' \item a single numeric defining the number of components to combine
#'   in a complex intervention,
#' \item a dedicated C matrix.
#' }
#' In order to calculate effects of arbitrary complex interventions, a
#' C matrix is needed which describes how the complex interventions
#' are composed by the components (Rücker et al., 2020, Section
#' 3.2). The C matrix is constructed internally if not provided by
#' argument \code{complex}. All complex interventions occuring in the
#' network are considered if argument \code{complex} is missing.

#' 
#' By default, component names are not abbreviated in
#' printouts. However, in order to get more concise printouts,
#' argument \code{nchar.comps} can be used to define the minimum
#' number of characters for abbreviated component names (see
#' \code{\link{abbreviate}}, argument \code{minlength}). R function
#' \code{\link{treats}} is utilised internally to create abbreviated
#' component names.
#' 
#' @note
#' R function \code{\link{netcomparison}} can be used to calculate the
#' effect for comparisons of two arbitrary complex intervention in a
#' component network meta-analysis.
#' 
#' @return
#' A list is returned by the function \code{netcomplex} with the
#' following elements:
#' \item{complex}{Complex intervention(s).}
#' \item{Comb.common, Comb.random}{A vector of combination effects
#'   (common and random effects model).}
#' \item{seComb.common, seComb.random}{A vector with corresponding
#'   standard errors (common and random effects model).}
#' \item{lower.Comb.common, lower.Comb.random}{A vector with lower
#'   confidence limits for combinations (common and random effects
#'   model).}
#' \item{upper.Comb.common, upper.Comb.random}{A vector with upper
#'   confidence limits for combinations (common and random effects
#'   model).}
#' \item{statistic.Comb.common, statistic.Comb.random}{A vector with
#'   z-values for the overall effect of combinations (common and random
#'   effects model).}
#' \item{pval.Comb.common, pval.Comb.random}{A vector with p-values for
#'   the overall effect of combinations (common and random effects
#'   model).}
#' \item{common, random}{A defined above.}
#' \item{level, nchar.comps, backtransf, x}{A defined above.}
#' \item{C.matrix}{C matrix.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{discomb}},
#'   \code{\link{netcomparison}}
#' 
#' @references
#' Rücker G, Petropoulou M, Schwarzer G (2020):
#' Network meta-analysis of multicomponent interventions.
#' \emph{Biometrical Journal},
#' \bold{62}, 808--21
#' 
#' @examples
#' # Only consider studies including Face-to-face PST (to reduce
#' # runtime of example)
#' #
#' face <- subset(Linde2016, id %in% c(16, 24, 49, 118))
#' 
#' # Use pairwise() to transform data to comparison-based format
#' #
#' pw <- pairwise(treat = int,
#'   event = resp, n = n, studlab = paste(author, year),
#'   data = face, reference = "plac", sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma1 <- netmeta(pw, common = FALSE)
#' 
#' # Additive model for treatment components (with placebo as inactive
#' # treatment)
#' #
#' cnma1 <- netcomb(nma1, inactive = "placebo")
#' 
#' # Result for combination Face-to-face PST + SSRI
#' netcomplex(cnma1, "Face-to-face PST + SSRI", nchar.comps = 4)
#' netcomplex(cnma1, "F + S", nchar.comps = 4)
#'
#' # Result for combination Face-to-face PST + SSRI + Placebo
#' netcomplex(cnma1, "Face-to-face PST + SSRI + Plac", nchar.comps = 4)
#'
#' \donttest{
#' # Artificial example
#' t1 <- rep("A", 3)
#' t2 <- c("B+C", "A+C", "C+D")
#' TE <- c(0, 1, 0)
#' seTE <- rep(1, 3)
#' # Conduct (C)NMA
#' nma2 <- netmeta(TE, seTE, t1, t2, random = FALSE)
#' cnma2 <- netcomb(nma2)
#'
#' # Result for combination A + B + C
#' netcomplex(cnma2, "A + B + C")
#' # Same results
#' netcomplex(cnma2, "A+B+C")
#' netcomplex(cnma2, "B+C+A")
#' netcomplex(cnma2, "C+B+A")
#' netcomplex(cnma2, "c+b+a")
#' 
#' # Generated C matrix
#' netcomplex(cnma2, c(LETTERS[1:4], "A+B+C"))$C.matrix
#' 
#' # Results for all possible combinations of two components
#' netcomplex(cnma2, 2)
#' # Results for all possible combinations of three components
#' netcomplex(cnma2, 3)
#' }
#' 
#' @rdname netcomplex
#' @export
#' @export netcomplex

netcomplex <- function(x, complex,
                       common = x$common,
                       random = x$random,
                       level = x$level.ma,
                       nchar.comps = x$nchar.trts,
                       backtransf = x$backtransf,
                       warn.deprecated = gs("warn.deprecated"),
                       ...) {
  
  
  chkclass(x, "netcomb")
  x <- updateversion(x)
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  chklogical(random)
  chklevel(level)
  ##
  nchar.comps <- replaceNULL(nchar.comps, 666)
  chknumeric(nchar.comps, min = 1, length = 1)
  ##
  chklogical(backtransf)
  
  
  if (missing(complex)) {
    complex <- x$trts
    complex.orig <- complex
  }
  else
    complex.orig <- complex    
  ##
  n.comps <- length(x$comps)
  ##
  comps <- x$comps
  ##
  if (is.matrix(complex)) {
    C.matrix <- complex
    complex <- rownames(complex)   
    n.complex <- length(complex)
    add <- rep("", n.complex)
  }
  else if (is.numeric(complex)) {
    chknumeric(complex, min = 1, max = n.comps, length = 1)
    ##
    C.matrix <- createC(n.comps, ncomb = complex)
    colnames(C.matrix) <- nam.comps <- x$comps
    rownames(C.matrix) <- paste0("...", seq_len(nrow(C.matrix)))
    ##
    for (i in seq_len(nrow(C.matrix))) {
      rownames(C.matrix)[i] <-
        paste(colnames(C.matrix[i, C.matrix[i, ] == 1, drop = FALSE]),
              collapse = x$sep.comps)
    }
    ##
    complex <- rownames(C.matrix)
    n.complex <- length(complex)
    add <- rep("", n.complex)
  }
  else if (is.vector(complex)) {
    n.complex <- length(complex)
    ##
    complex.list <- compsplit(complex, x$sep.comps)
    ##
    comps.c <- setref(unique(unlist(complex.list)), c(comps, x$inactive),
                      error.text = "component names", length = 0)
    ##
    complex.list <-
      lapply(complex.list, setref, c(comps, x$inactive), length = 0)
    ##
    complex.orig <- complex
    ##
    add <- rep("", n.complex)
    ##
    for (i in seq_len(n.complex)) {
      add[i] <-
        if (attr(compsplit(complex[i], x$sep.comps), "withspace")) " " else ""
      complex[i] <-
        paste(complex.list[[i]],
              collapse = paste0(add[i], x$sep.comps, add[i]))
    }
    ##
    ## Generate C matrix
    ##
    C.matrix <- matrix(0, ncol = n.comps, nrow = n.complex)
    colnames(C.matrix) <- x$comps
    rownames(C.matrix) <- complex
    ##
    for (i in seq_len(n.complex))
      C.matrix[i, ] <- 1L * colnames(C.matrix) %in% complex.list[[i]]
  }
  else
    stop("Argument 'complex' must be a single numeric, a matrix, or a vector.",
         call. = FALSE)
  
  
  ##
  ## Calculate estimates for combinations
  ##
  Comb.common <- as.vector(C.matrix %*% x$Comp.common)
  seComb.common <-
    sqrt(diag(C.matrix %*% x$Lplus.matrix.common %*% t(C.matrix)))
  ##
  Comb.random <- as.vector(C.matrix %*% x$Comp.random)
  seComb.random <-
    sqrt(diag(C.matrix %*% x$Lplus.matrix.random %*% t(C.matrix)))
  ##
  ci.f <- ci(Comb.common, seComb.common, level = level)
  ci.r <- ci(Comb.random, seComb.random, level = level)
    
  res <- list(complex = complex,
              ##
              Comb.common = ci.f$TE,
              seComb.common = ci.f$seTE,
              lower.Comb.common = ci.f$lower,
              upper.Comb.common = ci.f$upper,
              statistic.Comb.common = ci.f$statistic,
              pval.Comb.common = ci.f$p,
              ##
              Comb.random = ci.r$TE,
              seComb.random = ci.r$seTE,
              lower.Comb.random = ci.r$lower,
              upper.Comb.random = ci.r$upper,
              statistic.Comb.random = ci.r$statistic,
              pval.Comb.random = ci.r$p,
              ##
              common = common,
              random = random,
              level = level,
              ##
              C.matrix = C.matrix,
              ##
              comps = colnames(C.matrix)[apply(C.matrix, 2, sum) > 0],
              inactive = x$inactive,
              nchar.comps = nchar.comps,
              ##
              backtransf = backtransf,
              ##
              x = x,
              ##
              add = add,
              ##
              complex.orig = complex.orig,
              ##
              version = packageDescription("netmeta")$Version
              )
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  ##
  res$Comb.fixed <- res$Comb.common
  res$seComb.fixed <- res$seComb.common
  res$lower.Comb.fixed <- res$lower.Comb.common
  res$upper.Comb.fixed <- res$upper.Comb.common
  res$statistic.Comb.fixed <- res$statistic.Comb.common
  res$pval.Comb.fixed <- res$pval.Comb.common
  ##
  class(res) <- c("netcomplex", class(res))
  
  res
}





#' @rdname netcomplex
#' @method print netcomplex
#' @export

print.netcomplex <- function(x,
                             common = x$common,
                             random = x$random,
                             backtransf = x$backtransf,
                             nchar.comps = x$nchar.comps,
                             ##
                             digits = gs("digits"),
                             digits.stat = gs("digits.stat"),
                             digits.pval = gs("digits.pval"),
                             ##
                             scientific.pval = gs("scientific.pval"),
                                 zero.pval = gs("zero.pval"),
                             JAMA.pval = gs("JAMA.pval"),
                             ##
                             big.mark = gs("big.mark"),
                             ##
                             legend = gs("legend"),
                             warn.deprecated = gs("warn.deprecated"),
                             ##
                             ...) {
  
  chkclass(x, "netcomplex")
  x <- updateversion(x)
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  chklogical(random)
  chklogical(backtransf)
  ##
  nchar.comps <- replaceNULL(nchar.comps, 666)
  chknumeric(nchar.comps, min = 1, length = 1)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 0, length = 1)
  ##
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  ##
  chklogical(legend)
  
  
  ##
  ## Abbreviated component labels
  ##
  n.complex <- length(x$complex)
  complex <- rep("", n.complex)
  ##
  if (common | random) {
    comps <- c(x$comps, x$inactive)
    comps.abbr <- treats(comps, nchar.comps)
     ##
    for (i in seq_len(n.complex))
      complex[i] <- compos(x$complex[i], comps, comps.abbr,
                           x$x$sep.comps, x$add[i] == " ")
  }
  
  
  sm <- sm.lab <- x$x$sm
  ##
  relative <- is_relative_effect(sm) | sm == "VE"
  ##
  if (sm != "") {
    sm.lab <- paste0("i", sm)
    if (!backtransf & relative)
      sm.lab <- paste0("log(", if (sm == "VE") "VR" else sm, ")")
  }
  ##  
  ci.lab <- paste0(round(100 * x$level, 1), "% CI")
  
  
  if (common) {
    Comb.common <- x$Comb.common
    lower.Comb.common <- x$lower.Comb.common
    upper.Comb.common <- x$upper.Comb.common
    ##
    if (backtransf) {
      Comb.common <- backtransf(Comb.common, sm)
      lower.Comb.common <- backtransf(lower.Comb.common, sm)
      upper.Comb.common <- backtransf(upper.Comb.common, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- lower.Comb.common
        lower.Comb.common <- upper.Comb.common
        upper.Comb.common <- tmp.l
      }
    }
  }
  ##
  if (random) {
    Comb.random <- x$Comb.random
    lower.Comb.random <- x$lower.Comb.random
    upper.Comb.random <- x$upper.Comb.random
    ##
    if (backtransf) {
      Comb.random <- backtransf(Comb.random, sm)
      lower.Comb.random <- backtransf(lower.Comb.random, sm)
      upper.Comb.random <- backtransf(upper.Comb.random, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- lower.Comb.random
        lower.Comb.random <- upper.Comb.random
        upper.Comb.random <- tmp.l
      }
    }
  }
  
  
  if (common | random) {
    ##
    if (is.numeric(x$complex.orig) & !is.matrix(x$complex.orig))
      if (all(x$complex.orig == 1))
        msg <- "Results for single components "
      else {
        if (x$complex.orig <= 10)
          complex.orig <- c("two", "three", "four",
                            "five", "six", "seven", "eight", "nine",
                            "ten")[match(x$complex.orig, 2:10)]
        else
          complex.orig <- x$complex.orig
        ##
        msg <- paste("Results for all possible combinations of",
                     complex.orig, "components\n")
      }
    else
      msg <- "Results for combinations "
  }
  
  
  if (common) {
    pval.f <- formatPT(x$pval.Comb.common, digits = digits.pval,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval,
                       lab.NA = "")
    ##
    res.f <-
      cbind(Comb = formatN(Comb.common, digits = digits,
                           "NA", big.mark = big.mark),
            CI = formatCI(formatN(round(lower.Comb.common, digits),
                                  digits, "NA",
                                  big.mark = big.mark),
                          formatN(round(upper.Comb.common, digits),
                                  digits, "NA",
                                  big.mark = big.mark)),
            zval = formatN(x$statistic.Comb.common, digits = digits.stat,
                           "NA", big.mark = big.mark),
            pval = pval.f)
    ##
    cat(msg)
    ##
    dimnames(res.f)[[1]] <- complex
    dimnames(res.f)[[2]] <- c(sm.lab, ci.lab, "z", "p-value")
    ##
    cat("(additive CNMA model, common effects model):\n")
    prmatrix(res.f, quote = FALSE, right = TRUE, na.print = "--")
    ##
    if (random)
      cat("\n")
  }
  
  
  if (random) {
    ##
    pval.r <- formatPT(x$pval.Comb.random, digits = digits.pval,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval,
                       lab.NA = "")
    ##
    res.r <-
      cbind(Comb = formatN(Comb.random, digits = digits,
                           "NA", big.mark = big.mark),
            CI = formatCI(formatN(round(lower.Comb.random, digits),
                                  digits, "NA",
                                  big.mark = big.mark),
                          formatN(round(upper.Comb.random, digits),
                                  digits, "NA",
                                  big.mark = big.mark)),
            zval = formatN(x$statistic.Comb.random, digits = digits.stat,
                           "NA", big.mark = big.mark),
            pval = pval.r)
    ##
    dimnames(res.r)[[1]] <- complex
    dimnames(res.r)[[2]] <- c(sm.lab, ci.lab, "z", "p-value")
    ##
    cat(msg)
    ##
    cat("(additive CNMA model, random effects model):\n")
    prmatrix(res.r, quote = FALSE, right = TRUE, na.print = "--")
  }
  
  
  if (legend && (common | random)) {
    diff.comps <- comps != comps.abbr
    any.comps <- any()
    ##
    if (any(diff.comps)) {
      cat("\nLegend:\n")
      ##
      tmat <- data.frame(comps.abbr, comps)
      tmat <- tmat[diff.comps, ]
      names(tmat) <- c("Abbreviation", " Component name")
      tmat <- tmat[order(tmat$Abbreviation), ]
      ##
      prmatrix(tmat, quote = FALSE, right = TRUE,
               rowlab = rep("", length(comps.abbr))) 
    }
  }
  
  
  invisible(NULL)
}

#' Print detailed information for component network meta-analysis
#' 
#' @description
#' Print detailed information  for component network meta-analysis.
#' 
#' @param x An object of class \code{summary.netcomb}
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf=TRUE}, results for \code{sm="OR"} are presented
#'   as odds ratios rather than log odds ratios, for example.
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique component names.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity tests, see \code{print.default}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistics, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.Q A logical value indicating whether to print the
#'   results of the test of heterogeneity.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{discomb}},
#'   \code{\link{summary.netcomb}}
#' 
#' @keywords print
#' 
#' @examples
#' # Examples: example(summary.netcomb)
#' 
#' @method print summary.netcomb
#' @export

print.summary.netcomb <- function(x,
                                  common = x$x$common,
                                  random = x$x$random,
                                  overall.hetstat = x$overall.hetstat,
                                  backtransf = x$backtransf,
                                  nchar.comps = x$nchar.comps,
                                  ##
                                  digits = gs("digits"),
                                  digits.stat = gs("digits.stat"),
                                  digits.pval = gs("digits.pval"),
                                  digits.pval.Q = max(gs("digits.pval.Q"), 2),
                                  digits.Q = gs("digits.Q"),
                                  #
                                  big.mark = gs("big.mark"),
                                  scientific.pval = gs("scientific.pval"),
                                  zero.pval = gs("zero.pval"),
                                  JAMA.pval = gs("JAMA.pval"),
                                  #
                                  print.tau2 = gs("print.tau2"),
                                  print.tau = gs("print.tau"),
                                  print.Q = gs("print.Q"),
                                  print.I2 = gs("print.I2"),
                                  print.I2.ci = gs("print.I2.ci"),
                                  #
                                  details.methods = gs("details"),
                                  legend = gs("legend"),
                                  ##
                                  warn.deprecated = gs("warn.deprecated"),
                                  ##
                                  ...) {
  
  
  ##
  ##
  ## (1) Check for summary.netcomb object and upgrade object
  ##
  ##
  chkclass(x, "summary.netcomb")
  updateversion(x)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  chknumeric(digits.pval.Q, min = 1, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  #
  chkchar(big.mark, length = 1)
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chklogical(print.tau2)
  chklogical(print.tau)
  chklogical(print.Q)
  chklogical(print.I2)
  chklogical(print.I2.ci)
  #
  chklogical(details.methods)
  chklogical(legend)
  ##
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                      warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  ##
  nchar.comps <-
    deprecated(nchar.comps, missing(nchar.comps), args, "nchar.trts")
  nchar.comps <- replaceNULL(nchar.comps, 666)
  chknumeric(nchar.comps, min = 1, length = 1)
  
  
  ##
  ##
  ## (3) Print summary results
  ##
  ##
  comps <- sort(c(x$comps, x$inactive))
  comps.abbr <- treats(comps, nchar.comps)
  ##
  comp.f <- x$comparison.cnma.common
  comp.f$seTE <- NULL
  ##
  dat.f <- formatComp(comp.f,
                      backtransf, x$sm, x$level.ma,
                      comps, comps.abbr, x$sep.comps,
                      digits, digits.stat, digits.pval.Q,
                      scientific.pval, zero.pval, JAMA.pval,
                      big.mark)
  ##
  comp.r <- x$comparison.cnma.random
  comp.r$seTE <- NULL
  ##
  dat.r <- formatComp(comp.r,
                      backtransf, x$sm, x$level.ma,
                      comps, comps.abbr, x$sep.comps,
                      digits, digits.stat, digits.pval.Q,
                      scientific.pval, zero.pval, JAMA.pval,
                      big.mark)
  ##
  if (common) {
    cat("Additive model (common effects model):\n")
    prmatrix(dat.f, quote = FALSE, right = TRUE, ...)
    cat("\n")
  }
  ##
  if (random) {
    cat("Additive model (random effects model):\n")
    prmatrix(dat.r, quote = FALSE, right = TRUE, ...)
    cat("\n")
  }
  ##  
  if (common | random)
    print.netcomb(x$x,
                  common = common,
                  random = random,
                  overall.hetstat = overall.hetstat,
                  backtransf = backtransf,
                  nchar.comps = nchar.comps,
                  ##
                  digits = digits,
                  digits.stat = digits.stat,
                  digits.pval = digits.pval,
                  digits.pval.Q = digits.pval.Q,
                  digits.Q = digits.Q,
                  #
                  big.mark = big.mark,
                  scientific.pval = scientific.pval,
                  zero.pval = zero.pval,
                  JAMA.pval = JAMA.pval,
                  #
                  print.tau2 = print.tau2,
                  print.tau = print.tau,
                  print.Q = print.Q,
                  print.I2 = print.I2,
                  print.I2.ci = print.I2.ci,
                  #
                  details.methods = details.methods,
                  legend = legend,
                  ##
                  ...)
  else
    cat("Please use argument 'common = TRUE' or 'random = TRUE'",
        "to print component network meta-analysis results.\n")
  
  invisible(NULL)
}

chkmultiarm <- function(TE, seTE, treat1, treat2, studlab, correlated,
                        tol.multiarm = 0.001,
                        tol.multiarm.se = NULL,
                        details = FALSE, debug = FALSE) {
  
  
  #
  # Ordering dataset (if necessary)
  #
  o <- order(studlab, treat1, treat2)
  #
  if (any(o != seq_along(studlab))) {
    TE <- TE[o]
    seTE <- seTE[o]
    treat1 <- treat1[o]
    treat2 <- treat2[o]
    studlab <- studlab[o]
    correlated <- correlated[o]
  }
  
  
  tabnarms <- table(studlab)
  sel.multi <- tabnarms > 1
  #
  if (any(sel.multi)) {
    #
    msgdetails <-
      paste0("  - For more details, re-run netmeta() with argument ",
             "details.chkmultiarm = TRUE.\n")
    #
    studlab.multi <- names(tabnarms)[sel.multi]
    #
    # Check duplicate and incomplete comparisons
    #
    dat.duplicate <- dat.incomplete <-
      data.frame(studlab = "", treat1 = "", treat2 = "",
                 stringsAsFactors = FALSE)
    #
    incomplete <- rep_len(NA, sum(sel.multi))
    duplicate <- rep_len(NA, sum(sel.multi))
    #
    s.idx <- 0
    
    #
    # (1) Check for multi-arm studies with incomplete or duplicate treatment
    #     comparisons
    #
    for (s in studlab.multi) {
      s.idx <- s.idx + 1
      sel <- studlab == s
      #
      TE.s <- TE[sel]
      studlab.s <- studlab[sel]
      treat1.s <- treat1[sel]
      treat2.s <- treat2[sel]
      treats.s <- unique(c(treat1.s, treat2.s))
      #
      n <- (1 + sqrt(8 * length(TE.s) + 1)) / 2
      #
      incomplete[s.idx] <- length(treats.s) != n
      duplicate[s.idx] <- any(table(interaction(treat1.s, treat2.s)) > 1)
      #
      if (incomplete[s.idx])
        dat.incomplete <- rbind(dat.incomplete,
                               data.frame(studlab = studlab.s,
                                          treat1 = treat1.s,
                                          treat2 = treat2.s,
                                          stringsAsFactors = FALSE))
      #
      if (duplicate[s.idx])
        dat.duplicate <- rbind(dat.duplicate,
                               data.frame(studlab = studlab.s,
                                          treat1 = treat1.s,
                                          treat2 = treat2.s,
                                          stringsAsFactors = FALSE))
    }
    #
    if (details & any(incomplete)) {
      dat.incomplete <- dat.incomplete[-1, ]
      cat("\nMulti-arm studies with incomplete treatment comparisons:\n\n")
      prmatrix(dat.incomplete, quote = FALSE, right = TRUE,
               rowlab = rep("", dim(dat.incomplete)[1]))
      cat("\n")
    }
    #
    if (details & any(duplicate)) {
      dat.duplicate <- dat.duplicate[-1, ]
      cat("\nMulti-arm studies with duplicate treatment comparisons:\n\n")
      prmatrix(dat.duplicate, quote = FALSE, right = TRUE,
               rowlab = rep("", dim(dat.duplicate)[1]))
      cat("\n")
    }
    #
    if (any(incomplete)) {
      studlabs <- unique(dat.incomplete$studlab[-1])
      #
      if (length(studlabs) == 1)
        errmsg.incomplete <-
          paste0("  - Study '", studlabs,
                "' has an incomplete set of comparisons.\n")
      else
        errmsg.incomplete <-
          paste0("  - Studies with incomplete set of comparisons: ",
                paste(paste0("'", studlabs, "'"), collapse = ", "),
                "\n")
    }
    else
      errmsg.incomplete <- ""
    #
    if (any(duplicate)) {
      studlabs <- unique(dat.duplicate$studlab[-1])
      #
      if (length(studlabs) == 1)
        errmsg.duplicate <-
          paste0("  - Duplicate comparison in study '", studlabs, "'.\n")
      else
        errmsg.duplicate <-
          paste0("  - Studies with duplicate comparisons: ",
                paste(paste0("'", studlabs, "'"), collapse = ", "),
                "\n")
    }
    else
      errmsg.duplicate <- ""
    #
    if (any(incomplete) | any(duplicate))
      stop("Problem",
           if ((sum(incomplete) + sum(duplicate)) > 1) "s",
           " in multi-arm studies!\n",
           errmsg.incomplete, errmsg.duplicate,
           if (!details) msgdetails, call. = FALSE)
    
    #
    # (2) Check for (i) consistency of TE and varTE or (2) negative or zero
    #     treatment arm variance (zero variance only results in a warning)
    #
    dat.TE <- data.frame(studlab = "", treat1 = "", treat2 = "",
                         TE = NA, resid = NA,
                         stringsAsFactors = FALSE)
    #
    dat.varTE <- data.frame(studlab = "", treat1 = "", treat2 = "",
                            varTE = NA, resid.var = NA,
                            seTE = NA, resid.se = NA,
                            stringsAsFactors = FALSE)
    #
    dat.negative <- data.frame(studlab = "", treat = "", var.treat = NA,
                               stringsAsFactors = FALSE)
    #
    dat.zero <- data.frame(studlab = "", treat = "", var.treat = NA,
                           stringsAsFactors = FALSE)
    #
    inconsistent.TE <- inconsistent.varTE <-
      zero.sigma2 <- negative.sigma2 <- rep_len(NA, sum(sel.multi))
    #
    s.idx <- 0
    #
    for (s in studlab.multi) {
      s.idx <- s.idx + 1
      sel <- studlab == s
      #
      TE.s <- TE[sel]
      seTE.s <- seTE[sel]
      varTE.s <- seTE.s^2
      studlab.s <- studlab[sel]
      treat1.s <- treat1[sel]
      treat2.s <- treat2[sel]
      #
      correlated.s <- unique(correlated[sel])
      #
      n <- (1 + sqrt(8 * length(TE.s) + 1)) / 2
      #
      treats.s <- unique(c(treat1.s, treat2.s))
      studlab.s.arms <- rep_len(s, n)
      #
      # Create full edge-vertex incidence matrix
      #
      B <- createB(ncol = n)
      #
      # Check treatment estimates
      #
      TE.diff <- TE.s - B %*% as.vector(ginv(B) %*% TE.s)
      if (debug) {
        cat("*** TE.diff = TE - B %*% as.vector(ginv(B) %*% TE) ***\n")
        print(data.frame(TE = TE.s,
                         TE.calc =  B %*% as.vector(ginv(B) %*% TE.s),
                         TE.diff))
      }
      #
      inconsistent.TE[s.idx] <- any(abs(TE.diff) > tol.multiarm)
      #
      if (any(abs(TE.diff) > tol.multiarm))
        dat.TE <- rbind(dat.TE,
                        data.frame(studlab = studlab.s,
                                   treat1 = treat1.s,
                                   treat2 = treat2.s,
                                   TE = round(TE.s, 8),
                                   resid = round(TE.diff, 8),
                                   stringsAsFactors = FALSE))
      #
      # Check standard errors
      #
      if (correlated.s) {
        inconsistent.varTE[s.idx] <- 0
        negative.sigma2[s.idx] <- 0
        zero.sigma2[s.idx] <- 0
      }
      else {
        A <- abs(B)
        #
        sigma2 <- as.vector(ginv(A) %*% varTE.s)
        #
        varTE.diff <- varTE.s - A %*% sigma2
        #
        if (debug) {
          cat("*** varTE.diff = varTE - A %*% sigma2 ***\n")
          cat("*** with sigma2 = as.vector(ginv(A) %*% varTE) ***\n")
          print(data.frame(varTE = varTE.s,
                           varTE.calc = A %*% sigma2,
                           varTE.diff))
          print(data.frame(sigma2))
        }
        #
        if (!is.null(tol.multiarm.se))
          inconsistent.varTE[s.idx] <- any(abs(varTE.diff) > tol.multiarm.se^2)
        else
          inconsistent.varTE <- rep_len(FALSE, length(inconsistent.varTE))
        #
        is.negative <- sigma2 < 0
        negative.sigma2[s.idx] <- any(is.negative)
        zero.sigma2[s.idx] <- any(is_zero(sigma2[!is.negative]))
        #
        if (inconsistent.varTE[s.idx])
          dat.varTE <- rbind(dat.varTE,
                             data.frame(studlab = studlab.s,
                                        treat1 = treat1.s,
                                        treat2 = treat2.s,
                                        varTE = round(varTE.s, 8),
                                        resid.var = round(varTE.diff, 8),
                                        seTE = round(sqrt(varTE.s), 8),
                                        resid.se = sign(varTE.diff) *
                                          round(sqrt(abs(varTE.diff)), 8),
                                        stringsAsFactors = FALSE))
        #
        if (negative.sigma2[s.idx])
          dat.negative <- rbind(dat.negative,
                                data.frame(studlab = studlab.s.arms,
                                           treat = treats.s,
                                           var.treat = sigma2,
                                           stringsAsFactors = FALSE))
        #
        if (zero.sigma2[s.idx])
          dat.zero <- rbind(dat.zero,
                            data.frame(studlab = studlab.s.arms,
                                       treat = treats.s,
                                       var.treat = round(sigma2, 8),
                                       stringsAsFactors = FALSE))
      }
    }
    #
    iTE <- sum(inconsistent.TE)
    ivarTE <- sum(inconsistent.varTE)
    inconsistent <- iTE > 0 | ivarTE > 0
    #
    inegative.sigma2 <- sum(negative.sigma2)
    izero.sigma2 <- sum(zero.sigma2)
    zero <- izero.sigma2 > 0
    negative <- inegative.sigma2 > 0
    #
    studlab.inconsistent.TE <- character()
    studlab.inconsistent.varTE <- character()
    studlab.zero.sigma2 <- character()
    studlab.negative.sigma2 <- character()
    if (iTE > 0)
      studlab.inconsistent.TE <- studlab.multi[inconsistent.TE]
    if (ivarTE > 0)
      studlab.inconsistent.varTE <- studlab.multi[inconsistent.varTE]
    if (zero)
      studlab.zero.sigma2 <- studlab.multi[zero.sigma2]
    if (negative)
      studlab.negative.sigma2 <- studlab.multi[negative.sigma2]
    #
    studlab.inconsistent <-
      unique(c(studlab.inconsistent.TE, studlab.inconsistent.varTE,
               studlab.zero.sigma2, studlab.negative.sigma2))
    
    #
    # Print information on deviations from consistency assumption in
    # multi-arm studies
    #
    if (details & (inconsistent | zero | negative)) {
      if (length(dat.TE$studlab) > 1) {
        dat.TE <- dat.TE[-1, ]
        dat.TE$TE <- format(dat.TE$TE)
        dat.TE$resid <- format(dat.TE$resid)
        cat("\nMulti-arm studies with inconsistent treatment effects:\n\n")
        prmatrix(dat.TE, quote = FALSE, right = TRUE,
                 rowlab = rep("", dim(dat.TE)[1]))
        cat("\n")
      }
      if (length(dat.varTE$studlab) > 1 & ivarTE > 0) {
        dat.varTE <- dat.varTE[-1, ]
        dat.varTE$varTE <- format(dat.varTE$varTE)
        dat.varTE$resid.var <- format(dat.varTE$resid.var)
        dat.varTE$seTE <- format(dat.varTE$seTE)
        dat.varTE$resid.se <- format(dat.varTE$resid.se)
        cat("\nMulti-arm studies with inconsistent",
            "variances / standard errors:\n\n")
        prmatrix(dat.varTE, quote = FALSE, right = TRUE,
                 rowlab = rep("", dim(dat.varTE)[1]))
        cat("\n")
      }
      #
      # Negative variance
      #
      if (length(dat.negative$studlab) > 1 & negative) {
        dat.negative <- dat.negative[-1, ]
        cat("\nMulti-arm studies with negative treatment arm variance:\n\n")
        prmatrix(dat.negative, quote = FALSE, right = TRUE,
                 rowlab = rep("", dim(dat.negative)[1]))
        cat("\n")
      }
      #
      # Zero variance
      #
      if (length(dat.zero$studlab) > 1 & zero) {
        dat.zero <- dat.zero[-1, ]
        cat("\nMulti-arm studies with zero treatment arm variance:\n\n")
        prmatrix(dat.zero, quote = FALSE, right = TRUE,
                 rowlab = rep("", dim(dat.zero)[1]))
        cat("\n")
        #
        warning(
          paste0("Note, a zero treatment arm variance has been calculated ",
                 "for the following multi-arm ",
                 if (izero.sigma2 == 1) "study" else "studies",
                 ": ",
                 paste(paste0("'", studlab.zero.sigma2, "'"), collapse = ", ")),
          call. = FALSE)
      }
      #
      cat("Legend:\n")
      if (inconsistent)
        cat(" resid - residual deviation (observed minus expected)\n")
      if (iTE > 0)
        cat(" TE", if (ivarTE > 0) "   ", " - treatment estimate\n", sep = "")
      if (ivarTE > 0) {
        cat(" varTE - variance of treatment estimate\n")
        cat(" seTE  - standard error of treatment estimate\n")
      }
      if (negative | zero)
        cat(" var.treat - treatment arm variance\n")
      cat("\n")
    }
    
    
    #
    # Generate error message
    #
    if (inconsistent | negative) {
      #
      if (iTE > 0)
        msgTE <-
          paste0("  ",
                 if (iTE == 1) "- Study " else "- Studies ",
                 "with inconsistent treatment estimates: ",
                 paste(paste0("'", studlab.inconsistent.TE, "'"),
                       collapse = ", "),
                 "\n")
      else
        msgTE <- ""
      #
      if (ivarTE > 0)
        msgvarTE <-
          paste0("  ",
                 if (ivarTE == 1) "- Study " else "- Studies ",
                 "with inconsistent variances: ",
                 paste(paste0("'", studlab.inconsistent.varTE, "'"),
                       collapse = ", "),
                 "\n")
      else
        msgvarTE <- ""
      #
      if (negative)
        msgsigma2 <-
          paste0("  ",
                 if (inegative.sigma2 == 1) "- Study " else "- Studies ",
                 "with negative treatment arm variance: ",
                 paste(paste0("'", studlab.negative.sigma2, "'"),
                       collapse = ", "),
                 "\n",
                 "    Potential solutions:\n",
                 "    1. Use argument 'func.inverse' to specify a different ",
                 "function for matrix inversion;\n",
                 "    2. Use argument 'correlated' to identify studies with ",
                 "correlated treatment arms, e.g., due to body-split design;\n",
                 "    3. Fix data errors.\n")
      else
        msgsigma2 <- ""
      #
      errmsg <-
        paste0("Problem",
               if ((iTE + ivarTE + inegative.sigma2 + izero.sigma2) > 1) "s",
               " in multi-arm studies!\n",
               msgTE, msgvarTE, msgsigma2,
               "  - Please check original data used as input to netmeta().\n",
               if (!details) msgdetails,
               if (inconsistent)
                 paste0("  - Argument",
                        if (iTE & ivarTE) "s",
                        if (iTE) " 'tol.multiarm'",
                        if (iTE & ivarTE) " and",
                        if (ivarTE) " 'tol.multiarm.se'",
                        " in netmeta() can be used to",
                        if (iTE & ivarTE) "\n   ",
                        " relax consistency",
                        if (!(iTE & ivarTE)) "\n   ",
                        " assumption for multi-arm studies (if appropriate)."))
      #
      stop(errmsg, call. = FALSE)
    }
  }
  
  invisible(NULL)
}

stress <- function(x,
                   A.matrix = x$A.matrix,
                   N.matrix = sign(A.matrix),
                   D.matrix = netdistance(N.matrix),
                   ##
                   dim = "2d",
                   start.layout = "eigen",
                   iterate = TRUE,
                   eig1 = 2, eig2 = 3, eig3 = 4,
                   tol = 0.0001,
                   maxit = 500,
                   ##
                   allfigures = FALSE,
                   ...) {
  
  
  ##
  ## Function stress for optimising 2-D and 3-D representation of a
  ## graph, given its adjacency matrix A.matrix
  ##
  dim <- setchar(dim, c("2d", "3d"))
  is_2d <- dim == "2d"
  is_3d <- !is_2d
  
  
  ##
  ## Calculate Laplacian matrix
  ##
  As.matrix <- sign(A.matrix)
  n <- dim(As.matrix)[1]                            # Dimension
  e <- rep(1, n)                                    # Vector of ones
  L <- diag(as.vector(As.matrix %*% e)) - As.matrix # Laplacian
  Lt <- ginv(L)                                     # Its pseudoinverse
  
  
  ##
  ## Calculate eigenvectors
  ##
  if (start.layout == "eigen") {
    Eig <- eigen(L, symmetric = TRUE)
    eigvectors <- Eig$vectors
  }
  else if (start.layout == "prcomp") {
    Eig <- prcomp(L, scale = TRUE)
    eigvectors <- Eig$rotation
  }
  
  
  ##
  ## Weight matrix and its pseudoinverse from complete graph of n
  ## vertices
  ##
  K <- 1 - diag(rep(1, n))     # K = Complete graph of n vertices
  W <- diag(rep(n - 1, n)) - K # Weight matrix = Laplacian of K
  Wt <- ginv(W)                # Its pseudoinverse
  
  
  ##
  ## Starting layout
  ##
  if (start.layout == "circle")
    coord <- cbind(cos(2 * pi * (1:n) / n),
                   sin(2 * pi * (1:n) / n),
                   rep(0, n)) / sqrt(n)
  else if (start.layout == "eigen" | start.layout == "prcomp")
    coord <- cbind(eigvectors[, n - eig1 + 1],
                   eigvectors[, n - eig2 + 1],
                   if (is_3d) eigvectors[, n - eig3 + 1] else rep(0, n)
                   )
  else if (start.layout == "random")
    coord <- cbind(rnorm(n, 0, 1 / sqrt(n - 1)),
                   rnorm(n, 0, 1 / sqrt(n - 1)),
                   if (is_3d) rnorm(n, 0, 1 / sqrt(n - 1)) else rep(0, n)
                   )
  
  
  ##
  ## Only for 2-D network plots
  ##
  if (allfigures & is_2d)
    netgraph(x,
             xpos = coord[, 1], ypos = coord[, 2],
             iterate = iterate,
             allfigures = FALSE, ...)
  
  
  ##
  ## Apply algorithm
  ##
  convergence <- 10L
  ##
  if (!iterate) {
    coord.new <- coord
    convergence <- NA
  }
  else {
    n.iter <- 0
    ##
    while (convergence != 0L & convergence != 1L) {
      n.iter <- n.iter + 1
      Lwd <- matrix(nrow = n, ncol = n)
      for (i in 1:(n - 1)) {
        for (j in (i + 1):n) {
          Lwd[i, j] <- Lwd[j, i] <- W[i, j] * D.matrix[i, j] /
            sqrt((coord[i, 1] - coord[j, 1])^2 +
                   (coord[i, 2] - coord[j, 2])^2 +
                     (coord[i, 3] - coord[j, 3])^2)
          ##
          if (is.na(Lwd[i, j]))
            Lwd[i, j] <- Lwd[j, i] <-  0
          else if (Lwd[i, j] == Inf)
            Lwd[i, j] <- Lwd[j, i] <-  1
          else if (Lwd[i, j] == -Inf)
            Lwd[i, j] <- Lwd[j, i] <- -1
        }
      }
      ##
      for (i in 1:n)
        Lwd[i, i] <- -sum(Lwd[i, ], na.rm = TRUE)
      ##
      RS <- Lwd %*% coord
      ##
      coord.new <- Wt %*% RS
      coord.new <- cbind(coord.new[, 1] / sqrt(sum(coord.new[, 1]^2)),
                         coord.new[, 2] / sqrt(sum(coord.new[, 2]^2)),
                         coord.new[, 3] / sqrt(sum(coord.new[, 3]^2)))
      ##
      if (is_2d)
        coord.new[, 3] <- 0
      ##
      if (allfigures & is_2d)
        netgraph(x,
                 xpos = coord.new[, 1], ypos = coord.new[, 2],
                 iterate = iterate,
                 allfigures = FALSE,
                 ...)
      ##
      maxdiff <- max(coord - coord.new)
      ##
      if (maxdiff < tol)
        convergence <- 0L
      else if (maxit == n.iter)
        convergence <- 1L
      ##
      coord <- coord.new
    }
  }
  
  res <- data.frame(labels = rownames(As.matrix),
                    x = coord.new[, 1],
                    y = coord.new[, 2])
  ##
  if (is_3d)
    res$z <- coord.new[, 3]
  ##
  attr(res, "convergence") <- convergence
  if (!is.na(convergence) & convergence != 10L) {
    attr(res, "n.iter") <- n.iter
    attr(res, "tol") <- tol
    attr(res, "maxdiff") <- maxdiff
  }
  ##
  res
}

#' Hasse diagram
#' 
#' @description
#' This function generates a Hasse diagram for a partial order of
#' treatment ranks in a network meta-analysis.
#' 
#' @aliases hasse hasse.netposet
#' 
#' @param x An object of class \code{netposet} (mandatory).
#' @param pooled A character string indicating whether Hasse diagram
#'   show be drawn for common (\code{"common"}) or random effects model
#'   (\code{"random"}). Can be abbreviated.
#' @param newpage A logical value indicating whether a new figure
#'   should be printed in an existing graphics window. Otherwise, the
#'   Hasse diagram is added to the existing figure.
#' @param shape Shape of node borders, either \code{"roundrect"},
#'   \code{"rect"}, or \code{"none"}, can be abbreviated.
#' @param col.lines Line colour.
#' @param col.nodes Colour for treatment node borders.
#' @param lwd Width of lines and node borders.
#' @param \dots Additional arguments (ignored).
#' 
#' @details
#' Generate a Hasse diagram (Carlsen & Bruggemann, 2014) for a partial
#' order of treatment ranks in a network meta-analysis (Rücker &
#' Schwarzer, 2017).
#' 
#' This R function is a wrapper function for a modified version of R function
#' \code{hasse} in R package \bold{hasseDiagram}
#' (Krzysztof Ciomek, \url{https://github.com/kciomek/hasseDiagram}) which is
#' available under the MIT license.
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netposet}},
#'   \code{\link{netrank}}, \code{\link{plot.netrank}},
#'   \code{\link[metabook]{Linde2015}}
#' 
#' @references
#' Carlsen L, Bruggemann R (2014):
#' Partial order methodology: a valuable tool in chemometrics.
#' \emph{Journal of Chemometrics},
#' \bold{28}, 226--34
#' 
#' Rücker G, Schwarzer G (2017):
#' Resolve conflicting rankings of outcomes in network meta-analysis:
#' Partial ordering of treatments.
#' \emph{Research Synthesis Methods},
#' \bold{8}, 526--36
#' 
#' @name hasse.netposet
#' 
#' @keywords plot
#' 
#' @examples
#' \dontrun{
#' # Only run example if R package 'Rgraphviz' from Bioconductor is available
#' #
#' if (requireNamespace("Rgraphviz", quietly = TRUE)) {
#' # Define order of treatments in depression dataset Linde2015
#' #
#' trts <- c("TCA", "SSRI", "SNRI", "NRI",
#'   "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")
#' 
#' # Outcome labels
#' #
#' outcomes <- c("Early response", "Early remission")
#' 
#' # (1) Early response
#' #
#' pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(resp1, resp2, resp3),
#'   n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma1 <- netmeta(pw1, common = FALSE,
#'   seq = trts, ref = "Placebo", small.values = "undesirable")
#' 
#' # (2) Early remission
#' #
#' pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(remi1, remi2, remi3),
#'   n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma2 <- netmeta(pw2, common = FALSE,
#'   seq = trts, ref = "Placebo", small.values = "undesirable")
#' 
#' # Partial order of treatment rankings
#' #
#' po <- netposet(netrank(nma1), netrank(nma2), outcomes = outcomes)
#' 
#' # Hasse diagram
#' #
#' hasse(po)
#' }
#' }
#' 
#' @method hasse netposet
#' @export

hasse.netposet <- function(x,
                           pooled = ifelse(x$random, "random", "common"),
                           newpage = TRUE,
                           shape = "roundrect",
                           col.lines = "black", col.nodes = "black",
                           lwd = 1,
                           ...) {
  
  if (!is_installed_package("Rgraphviz", stop = FALSE)) {
    message(paste0("Package 'Rgraphviz' missing.",
                   "\n  ",
                   "Please use the following R commands for installation:",
                   "\n  ",
                   "install.packages(\"BiocManager\")",
                   "\n  ",
                   "BiocManager::install(\"Rgraphviz\")"))
    #
    return(invisible(NULL))
  }
  
  chkclass(x, "netposet")
  x <- updateversion(x)
  #
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  #
  chklogical(newpage)
  #
  shape <- setchar(shape, c("roundrect", "rect", "none"))
  #
  chkcolor(col.lines, length = 1)
  chkcolor(col.nodes, length = 1)
  #
  chknumeric(lwd, min = 0, zero = TRUE, length = 1)
  
  if (pooled == "common")
    M <- x$M.common
  else
    M <- x$M.random
  #
  M <- matrix(as.logical(M), dim(M), dimnames = dimnames(M))
  #
  pars <- list(newpage = newpage,  shape = shape,
               col.lines = col.lines, col.nodes = col.nodes,
               lwd = lwd)
  #
  hasse_hasseDiagram(M, pars)
  
  invisible()
}


#' @rdname hasse.netposet
#' @export

hasse <- function(x, ...)
  UseMethod("hasse")

mat2dat.table <- function(x, pooled = "common", dat.trts,
                          backtransf,
                          digits, digits.I2, digits.pval,
                          scientific.pval, zero.pval, JAMA.pval,
                          big.mark, text.NA,
                          writexl) {
  
  pooled <- setchar(pooled, c("common", "random"))
  ##
  name.n <- paste0("nma.", pooled)
  name.d <- paste0("direct.", pooled)
  name.i <- paste0("indirect.", pooled)
  
  
  res <-
    data.frame(treat1 = rep_len(NA, nrow(dat.trts)),
               treat2 = NA,
               k = NA, n = NA, I2 = NA,
               direct = NA, indirect = NA, nma = NA,
               p.value = NA,
               ##
               TE.direct = NA,
               seTE.direct = NA,
               lower.direct = NA,
               upper.direct = NA,
               TE.indirect = NA,
               seTE.indirect = NA,
               lower.indirect = NA,
               upper.indirect = NA,
               TE.nma = NA,
               lower.nma = NA,
               upper.nma = NA,
               ##
               stringsAsFactors = FALSE)
  ##
  for (i in seq_len(nrow(dat.trts))) {
    t1.i <- dat.trts$treat1[i]
    t2.i <- dat.trts$treat2[i]
    ##
    res$treat1[i] <- t1.i
    res$treat2[i] <- t2.i
    ##
    res$k[i] <- x$x$A.matrix[t1.i, t2.i]
    if (!is.null(x$x$n.matrix))
      res$n[i] <- x$x$n.matrix[t1.i, t2.i]
    res$I2[i] <- x$x$I2.direct[t1.i, t2.i]
    ##
    res$TE.direct[i] <- x[[name.d]]$TE[t1.i, t2.i]
    res$seTE.direct[i] <- x[[name.d]]$seTE[t1.i, t2.i]
    res$lower.direct[i] <- x[[name.d]]$lower[t1.i, t2.i]
    res$upper.direct[i] <- x[[name.d]]$upper[t1.i, t2.i]
    ##
    res$TE.indirect[i] <- x[[name.i]]$TE[t1.i, t2.i]
    res$seTE.indirect[i] <- x[[name.i]]$seTE[t1.i, t2.i]
    res$lower.indirect[i] <- x[[name.i]]$lower[t1.i, t2.i]
    res$upper.indirect[i] <- x[[name.i]]$upper[t1.i, t2.i]
    ##
    res$TE.nma[i] <- x[[name.n]]$TE[t1.i, t2.i]
    res$lower.nma[i] <- x[[name.n]]$lower[t1.i, t2.i]
    res$upper.nma[i] <- x[[name.n]]$upper[t1.i, t2.i]
  }
  ##
  res$p.value <-
    suppressWarnings(
      metagen(res$TE.direct - res$TE.indirect,
              sqrt(res$seTE.direct^2 + res$seTE.indirect^2),
              level = x$x$level.ma,
              method.tau = "DL", method.tau.ci = "")$pval)
  res$p.value <-
    formatPT(res$p.value, digits = digits.pval,
             scientific = scientific.pval,
             zero = zero.pval, JAMA = JAMA.pval,
             lab.NA = text.NA)
  ##
  if (backtransf) {
    sm <- x$x$sm
    #
    res$TE.nma <- backtransf(res$TE.nma, sm)
    res$lower.nma <- backtransf(res$lower.nma, sm)
    res$upper.nma <- backtransf(res$upper.nma, sm)
    #
    res$TE.direct <- backtransf(res$TE.direct, sm)
    res$lower.direct <- backtransf(res$lower.direct, sm)
    res$upper.direct <- backtransf(res$upper.direct, sm)
    #
    res$TE.indirect <- backtransf(res$TE.indirect, sm)
    res$lower.indirect <- backtransf(res$lower.indirect, sm)
    res$upper.indirect <- backtransf(res$upper.indirect, sm)
    #
    if (sm == "VE") {
      tmp.l <- res$lower.nma
      res$lower.nma <- res$upper.nma
      res$upper.nma <- tmp.l
      #
      tmp.l <- res$lower.direct
      res$lower.direct <- res$upper.direct
      res$upper.direct <- tmp.l
      #
      tmp.l <- res$lower.indirect
      res$lower.indirect <- res$upper.indirect
      res$upper.indirect <- tmp.l
    }
  }
  ##
  ## Round and round ...
  ##
  res$TE.nma <- round(res$TE.nma, digits = digits)
  res$lower.nma <- round(res$lower.nma, digits = digits)
  res$upper.nma <- round(res$upper.nma, digits = digits)
  ##
  res$TE.direct <- round(res$TE.direct, digits = digits)
  res$lower.direct <- round(res$lower.direct, digits = digits)
  res$upper.direct <- round(res$upper.direct, digits = digits)
  ##
  res$TE.indirect <- round(res$TE.indirect, digits = digits)
  res$lower.indirect <- round(res$lower.indirect, digits = digits)
  res$upper.indirect <- round(res$upper.indirect, digits = digits)
  ##
  ## Format results
  ##
  res$nma <-
    ifelse(is.na(res$TE.nma),
           text.NA,
           paste(formatN(res$TE.nma, digits = digits,
                         text.NA = text.NA, big.mark = big.mark),
                 formatCI(formatN(res$lower.nma,
                                  digits = digits,
                                  text.NA = text.NA, big.mark = big.mark),
                          formatN(res$upper.nma,
                                  digits = digits,
                                  text.NA = text.NA, big.mark = big.mark))))
  ##
  res$direct <-
    ifelse(is.na(res$TE.direct),
           text.NA,
           paste(formatN(res$TE.direct, digits = digits,
                         text.NA = text.NA, big.mark = big.mark),
                 formatCI(formatN(res$lower.direct,
                                  digits = digits,
                                  text.NA = text.NA, big.mark = big.mark),
                          formatN(res$upper.direct,
                                  digits = digits,
                                  text.NA = text.NA, big.mark = big.mark))))
  ##
  res$indirect <-
    ifelse(is.na(res$TE.indirect),
           text.NA,
           paste(formatN(res$TE.indirect, digits = digits,
                         text.NA = text.NA, big.mark = big.mark),
                 formatCI(formatN(res$lower.indirect,
                                  digits = digits,
                                  text.NA = text.NA, big.mark = big.mark),
                          formatN(res$upper.indirect,
                                  digits = digits,
                                  text.NA = text.NA, big.mark = big.mark))))
  ##
  res$I2 <- round(100 * res$I2, digits = digits.I2)
  res$I2 <-
    ifelse(is.na(res$I2),
           text.NA,
           paste0(formatN(res$I2, digits = digits.I2), "%"))
  res$direct[is.na(res$direct)] <- text.NA
  res$indirect[is.na(res$indirect)] <- text.NA
  res$p.value[is.na(res$p.value)] <- text.NA
  ##
  ## Drop unnecessary columns
  ##
  res$TE.direct <- res$seTE.direct <-
    res$lower.direct <- res$upper.direct <-
      res$TE.indirect <- res$seTE.indirect <-
        res$lower.indirect <- res$upper.indirect <-
          res$TE.nma <- res$lower.nma <- res$upper.nma <- NULL
  ##
  ## Change variable names
  ##
  nam <- c("Arm 1", "Arm 2", "k", "n", "I2",
           "Direct estimate", "Indirect estimate",
           "Network meta-analysis",
           if (writexl) "Incoherence P-value" else "Incoherence")
  names(res) <- nam
  ##
  if (!is.null(x$x$outcome.name)) {
    res$Outcome <- x$x$outcome.name
    res <- res[, c("Outcome", nam)]
  }
  ##
  res
}

#' Network meta-analysis of pharmacologic treatments for chronic
#' obstructive pulmonary disease
#' 
#' @description
#' This dataset comes from a systematic review of randomized
#' controlled trials on pharmacologic treatments for chronic
#' obstructive pulmonary disease (COPD) (Baker et al., 2009).
#'
#' The primary outcome, occurrence of one or more episodes of COPD
#' exacerbation, is binary (yes / no). For this outcome, five drug
#' treatments (fluticasone, budesonide, salmeterol, formoterol,
#' tiotropium) and two combinations (fluticasone + salmeterol,
#' budesonide + formoterol) were compared to placebo. The authors
#' considered the two combinations as separate treatments instead of
#' evaluating the individual components.
#' 
#' @name Baker2009
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{year}}\tab year of publication \cr
#' \bold{\emph{id}}\tab study ID \cr
#' \bold{\emph{treatment}}\tab treatment \cr
#' \bold{\emph{exac}}\tab one or more episodes of COPD exacerbation
#'   \cr
#' \bold{\emph{total}}\tab number of individuals in treatment arm
#' }
#' 
#' @note
#' The dataset Baker2009 is identical to dataset
#' \code{\link[metabook]{Baker2009}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[metabook]{Baker2009}},
#'   \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
#'   \code{\link{netmeta}}, \code{\link{netcomb}}
#' 
#' @source
#' Baker WL, Baker EL, Coleman CI (2009):
#' Pharmacologic Treatments for Chronic Obstructive Pulmonary Disease:
#' A Mixed-Treatment Comparison Meta-analysis.
#' \emph{Pharmacotherapy: The Journal of Human Pharmacology and Drug
#' Therapy},
#' \bold{29}, 891--905
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Baker2009)
#' 
#' # Example using pairwise(), netmeta() and netcomb():
#' # example(Baker2009, package = "metabook")

NULL

designs <- function(treat1, treat2, studlab, sep.trts = ":") {
  
  
  id <- seq_along(studlab)
  o <- order(studlab, treat1, treat2)
  ##
  if (any(o != id)) {
    treat1 <- treat1[o]
    treat2 <- treat2[o]
    studlab <- studlab[o]
    id <- id[o]
  }
  
  
  studies <- unique(studlab)
  n.study <- length(unique(studies))
  ##
  designs <- data.frame(studlab = "", design = rep_len("", n.study))
  
  
  for (i in seq_len(n.study)) {
    designs$studlab[i] <- studies[i]
    designs$design[i] <-
      paste(sort(unique(c(treat1[studlab == studies[i]],
                          treat2[studlab == studies[i]]))),
            collapse = sep.trts)
  }
  
  
  dat <- data.frame(studlab, treat1, treat2, o)
  ##
  res <- merge(dat, designs, by = "studlab")
  res <- res[order(res$o), ]
  res$treat1 <- NULL
  res$treat2 <- NULL
  res$o <- NULL
  ##
  res
}

## Auxiliary functions to format print output
##
## Package: meta
## Author: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
## License: GPL (>= 2)
##
bylabel <- function(subgroup.name, bylevs, print.subgroup.name,
                    sep.subgroup, big.mark = "") {
  if (print.subgroup.name) {
    if (length(subgroup.name) == 0 || subgroup.name == "")
      res <- format(bylevs, big.mark = big.mark)
    else
      res <- paste0(subgroup.name, sep.subgroup,
                    format(bylevs, big.mark = big.mark))
  }
  else
    res <- format(bylevs, big.mark = big.mark)
  ##
  res
}
crtitle <- function(x) {
  tl <- options()$width - 12
  newline <- FALSE
  ##  
  if (!is.null(x$title)) {
    if (x$title != "") {
      newline <- TRUE
      if (nchar(x$title) <= tl)
        cat("Review:     ", x$title, "\n", sep = "")
      else
        cat("Review:     ", substring(x$title, 1, tl - 4), " ...\n", sep = "")
    }
  }
  if (!is.null(x$complab)) {
    if (x$complab != "") {
      newline <- TRUE
      if (nchar(x$complab) <= tl)
        cat("Comparison: ", x$complab, "\n", sep = "")
      else
        cat("Comparison: ", substring(x$complab, 1, tl - 4), " ...\n", sep = "")
    }
  }
  if (!is.null(x$outclab)) {
    if (x$outclab != "") {
      newline <- TRUE
      if (nchar(x$outclab) <= tl)
        cat("Outcome:    ", x$outclab, "\n", sep = "")
      else
        cat("Outcome:    ", substring(x$outclab, 1, tl - 4), " ...\n", sep = "")
    }
  }
  ##
  if (newline)
    cat("\n")
}
formatCI <- function(lower, upper, rmspace = TRUE,
                     bracket.left = gs("CIbracket"),
                     separator = gs("CIseparator"),
                     bracket.right,
                     justify.lower = "right",
                     justify.upper = justify.lower,
                     ...
                     ) {
  
  ## Change layout of CIs
  ##
  bracks <- c("[", "(", "{", "")
  ibracket <- charmatch(bracket.left,
                        bracks,
                        nomatch = NA)
  ##
  if (is.na(ibracket) | ibracket == 0)
    stop("No valid bracket type specified. ",
         "Admissible values: '[', '(', '{', '\"\"'")
  ##
  bracktype <- bracks[ibracket]
  ##
  if (bracktype == "[") {
    bracketLeft <- "["
    bracketRight <- "]"
  }
  else if (bracktype == "(") {
    bracketLeft <- "("
    bracketRight <- ")"
  }
  else if (bracktype == "{") {
    bracketLeft <- "{"
    bracketRight <- "}"
  }
  else if (bracktype == "") {
    bracketLeft <- ""
    bracketRight <- ""
  }
  ##
  if (missing(bracket.left))
    bracket.left <- bracketLeft
  ##
  if (missing(bracket.right))
    bracket.right <- bracketRight
  
  if (rmspace) {
    lower <- rmSpace(lower)
    upper <- rmSpace(upper)
  }
  ##
  res <- ifelse(lower != "NA" & upper != "NA",
                paste0(bracket.left,
                       format(lower, justify = justify.lower),
                       separator,
                       format(upper, justify = justify.upper),
                       bracket.right),
                "")
  ##
  res
}
formatN <- function(x, digits = 2, text.NA = "--", big.mark = "") {
  
  outdec <- options()$OutDec
  
  res <- format(ifelse(is.na(x),
                       text.NA,
                       formatC(x, decimal.mark = outdec,
                               format = "f", digits = digits,
                               big.mark = big.mark)
                       )
                )
  ##
  res <-  rmSpace(res, end = TRUE)
  ##
  res
}
formatPT <- function(x, lab = FALSE, labval = "p", noblanks = FALSE,
                     digits = 4, zero = TRUE, scientific = FALSE,
                     lab.NA = "--", big.mark = "",
                     JAMA = FALSE) {
  
  if (is.null(x))
    return("")
  
  outdec <- options()$OutDec
  
  n.zeros <- digits - 1
  n.zeros[n.zeros < 0] <- 0
  
  if (!scientific) {
    if (lab) {
      if (!JAMA)
        res <- format(ifelse(is.na(x) | is.nan(x),
                             paste(labval, "=", lab.NA),
                      ifelse(x == 0,
                             paste(labval, "= 0"),
                      ifelse(x < 1 / 10^digits,
                             paste0(labval, " < 0", outdec,
                                    paste(rep("0",
                                             n.zeros), collapse = ""),
                                    "1"),
                             paste(paste(labval, "="),
                                   formatC(round(x, digits),
                                           decimal.mark = outdec,
                                           big.mark = big.mark,
                                           format = "f", digits = digits)
                                   )
                             )
                      )
                      )
                      )
      else
        res <- format(ifelse(is.na(x) | is.nan(x),
                             paste(labval, "=", lab.NA),
                      ifelse(x < 0.001,
                             paste0(labval, " < 0", outdec,
                                    paste(rep("0", 2), collapse = ""), "1"),
                      ifelse(x >= 0.001 & x < 0.01,
                             paste(paste(labval, "="),
                                   formatC(x,
                                           decimal.mark = outdec,
                                           big.mark = big.mark,
                                           format = "f", digits = 3)),
                      ifelse(x >= 0.01 & x <= 0.99,
                             paste(paste(labval, "="),
                                   formatC(x,
                                           decimal.mark = outdec,
                                           big.mark = big.mark,
                                           format = "f", digits = 2)),
                             paste(paste(labval, ">"),
                                   formatC(0.99,
                                           decimal.mark = outdec,
                                           big.mark = big.mark,
                                           format = "f", digits = 2)))
                      )
                      )
                      )
                      )
      
    }
    else {
      if (!JAMA)
        res <- format(ifelse(is.na(x) | is.nan(x),
                             lab.NA,
                      ifelse(x == 0,
                             0,
                      ifelse(x < 1 / 10^digits,
                             paste0("< 0", outdec,
                                    paste(rep("0", n.zeros), collapse = ""),
                                    "1"),
                             formatC(round(x, digits),
                                     decimal.mark = outdec,
                                     big.mark = big.mark,
                                     format = "f", digits = digits)
                             )
                      )
                      ),
                      justify = "right")
      else
        res <- format(ifelse(is.na(x) | is.nan(x),
                             lab.NA,
                      ifelse(x < 0.001,
                             paste0("< 0", outdec,
                                    paste(rep("0", 2), collapse = ""), "1"),
                      ifelse(x >= 0.001 & x < 0.01,
                             formatC(x,
                                     decimal.mark = outdec,
                                     big.mark = big.mark,
                                     format = "f", digits = 3),
                      ifelse(x >= 0.01 & x <= 0.99,
                             formatC(x,
                                     decimal.mark = outdec,
                                     big.mark = big.mark,
                                     format = "f", digits = 2),
                             paste(">",
                                   formatC(0.99,
                                           decimal.mark = outdec,
                                           big.mark = big.mark,
                                           format = "f", digits = 2)))
                      )
                      )
                      ),
                      justify = "right")
    }
  }
  else {
    if (lab)
      res <- format(ifelse(is.na(x) | is.nan(x),
                           paste(labval, "=", lab.NA),
                           paste(labval, "=",
                                 formatC(x, decimal.mark = outdec,
                                         big.mark = big.mark,
                                         format = "e", digits = digits)
                                 )
                           )
                    )
    else
      res <- formatC(x, decimal.mark = outdec,
                     big.mark = big.mark, format = "e", digits = digits)
  }
  ##
  if (noblanks)
    res <- gsub(" ", "", res)
  if (!zero)
    res <- gsub("0\\.", "\\.", res)
  ##
  ## Treat NaNs as NAs
  ##
  res[grep("NaN", res)] <- lab.NA
  
  res
}
p.ci <- function(lower, upper, rmspace = TRUE,
                 bracket.left = gs("CIbracket"),
                 separator = gs("CIseparator"),
                 bracket.right = "]",
                 justify.lower = "right",
                 justify.upper = justify.lower,
                 ...
                 ) {
  
  warning("Use of function p.ci() from R package meta is deprecated; ",
          "use instead formatCI().")
  
  ## Change layout of CIs
  ##
  ibracktype <- charmatch(gs("CIbracket"),
                          c("[", "(", "{", ""), nomatch = NA)
  if (is.na(ibracktype) | ibracktype == 0) {
    warning("No valid bracket type specified globally for R package meta: ",
            gs("CIbracket"),
            "\n  Using default bracket type: '['. See help page on ",
            "R command 'cilayout' for further information.")
    bracktype <- "["
  }
  else
    bracktype <- c("[", "(", "{", "")[ibracktype]
  ##
  if (bracktype == "[") {
    bracketLeft <- "["
    bracketRight <- "]"
  }
  else if (bracktype == "(") {
    bracketLeft <- "("
    bracketRight <- ")"
  }
  else if (bracktype == "{") {
    bracketLeft <- "{"
    bracketRight <- "}"
  }
  else if (bracktype == "") {
    bracketLeft <- ""
    bracketRight <- ""
  }
  ##
  if (missing(bracket.left))
    bracket.left <- bracketLeft
  ##
  if (missing(bracket.right))
    bracket.right <- bracketRight
  ##
  
  if (rmspace) {
    lower <- rmSpace(lower)
    upper <- rmSpace(upper)
  }
  ##
  res <- ifelse(lower != "NA" & upper != "NA",
                paste0(bracket.left,
                       format(lower, justify = justify.lower),
                       separator,
                       format(upper, justify = justify.upper),
                       bracket.right),
                "")
  ##
  res
}
pasteCI <- function(lower, upper, digits, big.mark,
                    sign.lower = "", sign.upper = "", text.NA = "NA",
                    unit = "")
  paste0(" ",
         formatCI(paste0(sign.lower,
                         formatN(lower, digits, big.mark = big.mark,
                                 text.NA = text.NA), unit),
                  paste0(sign.upper,
                         formatN(upper, digits, big.mark = big.mark,
                                 text.NA = text.NA), unit)))
rmSpace <- function(x, end = FALSE, pat = " ") {
  
  if (!end) {
    while (any(substring(x, 1, 1) == pat, na.rm = TRUE)) {
      sel <- substring(x, 1, 1) == pat
      x[sel] <- substring(x[sel], 2)
    }
  }
  else {
    last <- nchar(x)
    
    while (any(substring(x, last, last) == pat, na.rm = TRUE)) {
      sel <- substring(x, last, last) == pat
      x[sel] <- substring(x[sel], 1, last[sel] - 1)
      last <- nchar(x)
    }
  }
  
  x
}

#' Print method for objects of class netcomb
#' 
#' @description
#' Print method for objects of class \code{netcomb}.
#' 
#' @param x An object of class \code{netcomb} or
#'   \code{summary.netcomb}.
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique names for components.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity tests, see \code{print.default}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistics, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for square
#'   root of between-study variance, see \code{print.default}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   statistic, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.Q A logical value indicating whether to print the
#'   results of the test of heterogeneity.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param text.I2 Text printed to identify heterogeneity statistic
#'   I\eqn{^2}.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{discomb}}
#' 
#' @keywords print
#' 
#' @examples
#' # Examples: example(netcomb)
#' 
#' @method print netcomb
#' @export

print.netcomb <- function(x,
                          common = x$common,
                          random = x$random,
                          overall.hetstat = x$overall.hetstat,
                          backtransf = x$backtransf,
                          nchar.comps = x$nchar.comps,
                          ##
                          digits = gs("digits"),
                          digits.stat = gs("digits.stat"),
                          digits.pval = gs("digits.pval"),
                          digits.pval.Q = max(gs("digits.pval.Q"), 2),
                          digits.Q = gs("digits.Q"),
                          digits.tau2 = gs("digits.tau2"),
                          digits.tau = gs("digits.tau"),
                          digits.I2 = gs("digits.I2"),
                          #
                          big.mark = gs("big.mark"),
                          scientific.pval = gs("scientific.pval"),
                          zero.pval = gs("zero.pval"),
                          JAMA.pval = gs("JAMA.pval"),
                          #
                          print.tau2 = gs("print.tau2"),
                          print.tau = gs("print.tau"),
                          print.Q = gs("print.Q"),
                          print.I2 = gs("print.I2"),
                          print.I2.ci = gs("print.I2.ci"),
                          #
                          text.tau2 = gs("text.tau2"),
                          text.tau = gs("text.tau"),
                          text.I2 = gs("text.I2"),
                          #
                          details.methods = gs("details"),
                          legend = gs("legend"),
                          ##
                          warn.deprecated = gs("warn.deprecated"),
                          ##
                          ...) {
  
  
  ##
  ##
  ## (1) Check for netcomb object and upgrade object
  ##
  ##
  chkclass(x, "netcomb")
  x <- updateversion(x)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chklogical(overall.hetstat)
  chklogical(backtransf)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  chknumeric(digits.pval.Q, min = 1, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  #
  chkchar(big.mark, length = 1)
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chklogical(print.tau2)
  chklogical(print.tau)
  chklogical(print.Q)
  chklogical(print.I2)
  chklogical(print.I2.ci)
  #
  chkchar(text.tau2)
  chkchar(text.tau)
  chkchar(text.I2)
  #
  chklogical(details.methods)
  chklogical(legend)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                      warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  ##
  nchar.comps <-
    deprecated(nchar.comps, missing(nchar.comps), args, "nchar.trts")
  
  
  ##
  ##
  ## (3) Print results
  ##
  ##
  I2 <- round(100 * x$I2, digits.I2)
  lower.I2 <- round(100 * x$lower.I2, digits.I2)
  upper.I2 <- round(100 * x$upper.I2, digits.I2)
  ##
  if (common | random) {
    cat("Number of studies: k = ", x$k, "\n", sep = "")
    cat("Number of pairwise comparisons: m = ", x$m, "\n", sep = "")
    cat("Number of treatments: n = ", x$n, "\n", sep = "")
    cat("Number of active components: c = ", x$c, "\n", sep = "")
    if (!is.null(x$d))
      cat("Number of designs: d = ", x$d, "\n", sep = "")
    if (inherits(x, "discomb"))
      cat("Number of subnetworks: s = ", x$s, "\n", sep = "")
    ##
    cat("\n")
    ##
    ## (a) Results for comparisons
    ##
    sm <- x$sm
    reference.group <- x$reference.group
    baseline.reference <- x$baseline.reference
    ##
    comps <- sort(c(x$comps, x$inactive))
    comps.abbr <- treats(comps, nchar.comps)
    sep.comps <- x$sep.comps
    ##
    if (!backtransf & (is_relative_effect(sm) | sm == "VE"))
      sm.lab <- paste0("log", if (sm == "VE") "VR" else sm)
    else
      sm.lab <- sm
    ##
    ci.lab <- paste0(round(100 * x$level, 1), "% CI")
    ##
    TE.common <- x$TE.common
    lowTE.common <- x$lower.common
    uppTE.common <- x$upper.common
    statistic.common <- replaceNULL(x$statistic.common, x$zval.common)
    pval.common <- x$pval.common
    ##
    TE.random <- x$TE.random
    lowTE.random <- x$lower.random
    uppTE.random <- x$upper.random
    statistic.random <- replaceNULL(x$statistic.random, x$zval.random)
    pval.random <- x$pval.random
    ##
    if (!is.null(x$seq)) {
      TE.common <- TE.common[x$seq, x$seq]
      lowTE.common <- lowTE.common[x$seq, x$seq]
      uppTE.common <- uppTE.common[x$seq, x$seq]
      statistic.common <- statistic.common[x$seq, x$seq]
      pval.common <- pval.common[x$seq, x$seq]
      ##
      if (!all(is.na(TE.random))) {
        TE.random <- TE.random[x$seq, x$seq]
        lowTE.random <- lowTE.random[x$seq, x$seq]
        uppTE.random <- uppTE.random[x$seq, x$seq]
        statistic.random <- statistic.random[x$seq, x$seq]
        pval.random <- pval.random[x$seq, x$seq]
      }
    }
    ##
    if (reference.group != "") {
      if (all(colnames(TE.common) != reference.group))
        stop("Argument 'reference.group' must match any of ",
             "the following values: ",
             paste(paste0("'", colnames(TE.common), "'"),
                   collapse = " - "))
      ##
      trts <- rownames(TE.common)
      trts.abbr <- compos(trts, comps, comps.abbr, sep.comps)
      ##
      if (baseline.reference)
        comptext <- paste0("comparison: ",
                           if (x$n == 2)
                             paste0("'",
                                    trts.abbr[trts != reference.group],
                                    "'")
                           else
                             "other treatments",
                           " vs '",
                           trts.abbr[trts == reference.group],
                           "'")
      else
        comptext <- paste0("comparison: '",
                           trts.abbr[trts == reference.group],
                           "' vs ",
                           if (x$n == 2)
                             paste0("'",
                                    trts.abbr[trts != reference.group],
                                    "'")
                           else
                             "other treatments")
      ##
      noeffect <- 1L * (backtransf & is_relative_effect(sm))
      #
      if (backtransf) {
        TE.common    <- backtransf(TE.common, sm)
        lowTE.common <- backtransf(lowTE.common, sm)
        uppTE.common <- backtransf(uppTE.common, sm)
        ##
        TE.random    <- backtransf(TE.random, sm)
        lowTE.random <- backtransf(lowTE.random, sm)
        uppTE.random <- backtransf(uppTE.random, sm)
        #
        # Switch lower and upper limit for VE if results have been
        # backtransformed
        #
        if (sm == "VE") {
          tmp.l <- lowTE.common
          lowTE.common <- uppTE.common
          uppTE.common <- tmp.l
          #
          tmp.l <- lowTE.random
          lowTE.random <- uppTE.random
          uppTE.random <- tmp.l
        }
      }
      ##
      TE.common <- round(TE.common, digits)
      lowTE.common <- round(lowTE.common, digits)
      uppTE.common <- round(uppTE.common, digits)
      statistic.common <- round(statistic.common, digits.stat)
      ##
      TE.random    <- round(TE.random, digits)
      lowTE.random <- round(lowTE.random, digits)
      uppTE.random <- round(uppTE.random, digits)
      statistic.random <- round(statistic.random, digits.stat)
      ##
      if (baseline.reference) {
        TE.common.b <- TE.common[, trts == reference.group]
        lowTE.common.b <-
          lowTE.common[, trts == reference.group]
        uppTE.common.b <-
          uppTE.common[, trts == reference.group]
        statistic.common.b <-
          statistic.common[, trts == reference.group]
        pval.common.b <-
          pval.common[, trts == reference.group]
      }
      else {
        TE.common.b <- TE.common[trts == reference.group, ]
        lowTE.common.b <-
          lowTE.common[trts == reference.group, ]
        uppTE.common.b <-
          uppTE.common[trts == reference.group, ]
        statistic.common.b <-
          statistic.common[trts == reference.group, ]
        pval.common.b <-
          pval.common[trts == reference.group, ]
      }
      ##
      dat1.c <-
        cbind(formatN(TE.common.b, digits, text.NA = "NA",
                      big.mark = big.mark),
              formatCI(formatN(round(lowTE.common.b, digits),
                               digits, "NA", big.mark = big.mark),
                       formatN(round(uppTE.common.b, digits),
                               digits, "NA", big.mark = big.mark)),
              formatN(statistic.common.b, digits.stat, text.NA = "NA",
                      big.mark = big.mark),
              formatPT(pval.common.b, digits = digits.pval,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval)
              )
      ##
      dat1.c[trts == reference.group, ] <- rep(".", ncol(dat1.c))
      dimnames(dat1.c) <-
        list(trts, c(sm.lab, ci.lab, "z", "p-value"))
      ##
      if (!is.na(TE.common.b[trts == reference.group]) &&
          TE.common.b[trts == reference.group] == noeffect)
        dat1.c[trts == reference.group, ] <- "."
      rownames(dat1.c) <- trts.abbr
      ##
      if (baseline.reference) {
        TE.random.b <- TE.random[, trts == reference.group]
        lowTE.random.b <-
          lowTE.random[, trts == reference.group]
        uppTE.random.b <-
          uppTE.random[, trts == reference.group]
        statistic.random.b <-
          statistic.random[, trts == reference.group]
        pval.random.b <-
          pval.random[, trts == reference.group]
      }
      else {
        TE.random.b <- TE.random[trts == reference.group, ]
        lowTE.random.b <-
          lowTE.random[trts == reference.group, ]
        uppTE.random.b <-
          uppTE.random[trts == reference.group, ]
        statistic.random.b <-
          statistic.random[trts == reference.group, ]
        pval.random.b <-
          pval.random[trts == reference.group, ]
      }
      ##
      dat1.r <-
        cbind(formatN(TE.random.b, digits, text.NA = "NA",
                      big.mark = big.mark),
              formatCI(formatN(round(lowTE.random.b, digits),
                               digits, "NA", big.mark = big.mark),
                       formatN(round(uppTE.random.b, digits),
                               digits, "NA", big.mark = big.mark)),
              formatN(statistic.random.b, digits.stat, text.NA = "NA",
                      big.mark = big.mark),
              formatPT(pval.random.b, digits = digits.pval,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval)
              )
      ##
      dat1.r[trts == reference.group, ] <- rep(".", ncol(dat1.r))
      dimnames(dat1.r) <-
        list(colnames(TE.random), c(sm.lab, ci.lab, "z", "p-value"))
      ##
      if (!is.na(TE.random.b[trts == reference.group]) &&
          TE.random.b[trts == reference.group] == noeffect)
        dat1.r[trts == reference.group, ] <- "."
      rownames(dat1.r) <- trts.abbr
    }
    ##
    ## (b) Results for combinations
    ##
    ci.comb.c <- data.frame(TE = x$Comb.common,
                            seTE = x$seComb.common,
                            lower = x$lower.Comb.common,
                            upper = x$upper.Comb.common,
                            statistic = x$statistic.Comb.common,
                            p = x$pval.Comb.common,
                            stringsAsFactors = FALSE)
    rownames(ci.comb.c) <- x$trts
    ## Only consider combinations with identifiable components
    ci.comb.c <- ci.comb.c[!is.na(ci.comb.c$TE), ]
    ## Drop result for inactive component (if available)
    if (!is.null(x$inactive))
      ci.comb.c <- subset(ci.comb.c, rownames(ci.comb.c) != x$inactive)
    ## Drop combinations consisting of single components
    ci.comb.c <-
      subset(ci.comb.c, grepl(sep.comps, rownames(ci.comb.c), fixed = TRUE))
    ##
    dat2.c <- formatCC(ci.comb.c,
                       backtransf, sm, x$level,
                       comps, comps.abbr, sep.comps,
                       digits, digits.stat, digits.pval,
                       scientific.pval, zero.pval, JAMA.pval,
                       big.mark)
    ##
    ci.comb.r <- data.frame(TE = x$Comb.random,
                            seTE = x$seComb.random,
                            lower = x$lower.Comb.random,
                            upper = x$upper.Comb.random,
                            statistic = x$statistic.Comb.random,
                            p = x$pval.Comb.random,
                            stringsAsFactors = FALSE)
    rownames(ci.comb.r) <- x$trts
    ## Only consider combinations with identifiable components
    ci.comb.r <- ci.comb.r[!is.na(ci.comb.r$TE), ]
    ## Drop result for inactive component (if available)
    if (!is.null(x$inactive))
      ci.comb.r <- subset(ci.comb.r, rownames(ci.comb.r) != x$inactive)
    ## Drop combinations consisting of single components
    ci.comb.r <-
      subset(ci.comb.r, grepl(sep.comps, rownames(ci.comb.r), fixed = TRUE))
    ##
    dat2.r <- formatCC(ci.comb.r,
                       backtransf, sm, x$level,
                       comps, comps.abbr, sep.comps,
                       digits, digits.stat, digits.pval,
                       scientific.pval, zero.pval, JAMA.pval,
                       big.mark)
    ##
    ## (c) Results for components
    ##
    ci.comp.c <- data.frame(TE = x$Comp.common,
                            seTE = x$seComp.common,
                            lower = x$lower.Comp.common,
                            upper = x$upper.Comp.common,
                            statistic = x$statistic.Comp.common,
                            p = x$pval.Comp.common,
                            stringsAsFactors = FALSE)
    rownames(ci.comp.c) <- x$comps
    ## Only consider components with identifiable components
    sel.comb.c <- !is.na(ci.comp.c$TE)
    ##
    dat3.c <- formatCC(ci.comp.c,
                       backtransf, sm, x$level,
                       comps, comps.abbr, sep.comps,
                       digits, digits.stat, digits.pval,
                       scientific.pval, zero.pval, JAMA.pval,
                       big.mark)
    dat3.c[!sel.comb.c, 2:4] <- "."
    ##
    ci.comp.r <- data.frame(TE = x$Comp.random,
                            seTE = x$seComp.random,
                            lower = x$lower.Comp.random,
                            upper = x$upper.Comp.random,
                            statistic = x$statistic.Comp.random,
                            p = x$pval.Comp.random,
                            stringsAsFactors = FALSE)
    rownames(ci.comp.r) <- x$comps
    ## Only consider components with identifiable components
    sel.comb.r <- !is.na(ci.comp.r$TE)
    ##
    dat3.r <- formatCC(ci.comp.r,
                       backtransf, sm, x$level,
                       comps, comps.abbr, sep.comps,
                       digits, digits.stat, digits.pval,
                       scientific.pval, zero.pval, JAMA.pval,
                       big.mark)
    dat3.r[!sel.comb.r, 2:4] <- "."
    ##
    if (common) {
      if (reference.group != "") {
        cat("Common effects model",
            if (!is.null(x$inactive))
              paste0(" (inactive component: '", x$inactive, "')"),
            "\n\n",
            sep = "")
        ##
        cat("Treatment estimate (sm = '", sm.lab, "', ", comptext, "):\n",
            sep = "")
        prmatrix(dat1.c, quote = FALSE, right = TRUE)
        cat("\n")
      }
      ##
      if (nrow(dat2.c) >= 1) {
        cat("Incremental effect for existing combinations:\n")
        print(dat2.c)
        cat("\n")
      }
      ##
      cat("Incremental effect for components:\n")
      print(dat3.c)
    }
    ##
    if (random) {
      if (common)
        cat("\n")
      if (reference.group != "") {
        cat("Random effects model",
            if (!is.null(x$inactive))
              paste0(" (inactive component: '", x$inactive, "')"),
            "\n\n",
            sep = "")
        ##
        cat("Treatment estimate (sm = '", sm.lab,
            "', ", comptext, "):\n", sep = "")
        prmatrix(dat1.r, quote = FALSE, right = TRUE)
        cat("\n")
      }
      ##
      if (nrow(dat2.r) >= 1) {
        cat("Incremental effect for existing combinations:\n")
        print(dat2.r)
        cat("\n")
      }
      ##
      cat("Incremental effect for components:\n")
      print(dat3.r)
    }
    ##
    ## (d) Heterogeneity / inconsistency
    ##
    print.I2 <- print.I2 & !is.na(I2)
    #
    if (overall.hetstat) {
      if (print.tau2 | print.tau | print.I2 | print.Q)
        cat("\n")
      #
      if (print.tau2 | print.tau | print.I2) {
        print.I2.ci <- print.I2.ci & print.I2
        #
        text.hetstat <- "Quantifying heterogeneity / inconsistency:\n"
        #
        if (print.tau2)
          text.hetstat <- paste0(
            text.hetstat,
            formatPT(x$tau^2,
                     lab = TRUE, labval = text.tau2,
                     digits = digits.tau2,
                     lab.NA = "NA", big.mark = big.mark))
        #
        if (print.tau)
          text.hetstat <- paste0(
            text.hetstat,
            if (print.tau2) "; ",
            formatPT(x$tau,
                     lab = TRUE, labval = text.tau,
                     digits = digits.tau,
                     lab.NA = "NA", big.mark = big.mark))
        #
        if (print.I2)
          text.hetstat <- paste0(
            text.hetstat,
            if (print.tau2 | print.tau) "; ",
            paste0(text.I2, " = ", round(I2, digits.I2), "%"),
            if (print.I2.ci & (!(is.na(lower.I2) | is.na(upper.I2))))
              pasteCI(lower.I2, upper.I2,
                      digits.I2, big.mark, unit = "%"))
        #
        text.hetstat <- paste0(text.hetstat, "\n")
        #
        cat(text.hetstat)
      }
      #
      if (print.Q) {
        if (print.tau2 | print.tau | print.I2)
          cat("\n")
        #
        if (inherits(x, "discomb") && x$s > 1) {
          cat("Heterogeneity statistic:\n")
          #
          hetdat <- 
            data.frame(Q = formatN(x$Q.additive, digits.Q),
                       df.Q = formatN(x$df.Q.additive, 0),
                       pval = formatPT(x$pval.Q.additive,
                                       digits = digits.pval.Q,
                                       scientific = scientific.pval,
                                       zero = zero.pval, JAMA = JAMA.pval),
                       row.names = "")
          #
          names(hetdat) <- c("Q", "df", "p-value")
          #
          print(hetdat)
        }
        else {
          cat("Heterogeneity statistics:\n")
          #
          hetdat <- 
            data.frame(Q = formatN(c(x$Q.additive,
                                     x$Q.standard,
                                     x$Q.diff),
                                   digits.Q),
                       df.Q = formatN(c(x$df.Q.additive,
                                        x$df.Q.standard,
                                        x$df.Q.diff), 0),
                       pval = formatPT(c(x$pval.Q.additive,
                                         x$pval.Q.standard,
                                         x$pval.Q.diff),
                                       digits = digits.pval.Q,
                                       scientific = scientific.pval,
                                       zero = zero.pval, JAMA = JAMA.pval),
                       row.names = c("Additive model", "Standard model",
                                     "Difference"))
          #
          names(hetdat) <- c("Q", "df", "p-value")
          #
          print(hetdat)
        }
      }
    }
    #
    # Print details of component network meta-analysis methods
    #
    if (details.methods)
      cat(textmeth(x, random, print.tau2, print.tau,
                   text.tau2, text.tau, digits.tau2, digits.tau,
                   print.I2, text.I2, big.mark))
    #
    # Add legend with abbreviated component labels
    #
    if (legend) {
      diff.comps <- comps != comps.abbr
      if (any(diff.comps)) {
        tmat <- data.frame(comps.abbr, comps)
        tmat <- tmat[diff.comps, ]
        names(tmat) <- c("Abbreviation", " Component name")
        tmat <- tmat[order(tmat$Abbreviation), ]
        ##
        cat("\nLegend:\n")
        prmatrix(tmat, quote = FALSE, right = TRUE,
                 rowlab = rep("", length(comps.abbr))) 
      }
    }
  }
  
  invisible(NULL)
}

#' netmeta: Brief overview of methods and general hints
#' 
#' @description
#' R package \bold{netmeta} (Balduzzi et al., 2023) provides
#' frequentist methods for network meta-analysis and supports
#' Schwarzer et al. (2015), Chapter 8 on network meta-analysis,
#' \url{https://link.springer.com/book/10.1007/978-3-319-21416-0}.
#' 
#' @details
#' R package \bold{netmeta} is an add-on package for \bold{meta}
#' providing the following network meta-analysis models:
#' \itemize{
#' \item frequentist network meta-analysis (function
#'   \code{\link{netmeta}}) based on Rücker (2012) and Rücker &
#'   Schwarzer (2014);
#' \item additive network meta-analysis for combinations of treatments
#'   (\code{\link{netcomb}} for connected networks,
#'   \code{\link{discomb}} for disconnected networks) (Rücker et al.,
#'   2020a);
#' \item network meta-analysis of binary data
#'   (\code{\link{netmetabin}}) using the Mantel-Haenszel or
#'   non-central hypergeometric distribution method (Efthimiou et al.,
#'   2019), or penalised logistic regression (Evrenoglou et al., 2022);
#' \item network meta-regression (\code{\link{netmetareg}}) with a single
#'   continuous or binary covariate (Kwarteng et al., 2026);
#' \item subgroup network meta-analysis (\code{\link{subgroup.netmeta}}).
#' }
#'
#' The following methods are available to present results of a network
#' meta-analysis:
#' \itemize{
#' \item network graphs (\code{\link{netgraph}}) described in Rücker &
#'   Schwarzer (2016);
#' \item forest plots (\code{\link{forest.netmeta}},
#'   \code{\link{forest.netcomb}});
#' \item league tables with network meta-analysis results
#'   (\code{\link{netleague}});
#' \item tables with network, direct and indirect estimates
#'   (\code{\link{nettable}}) looking similar to the statistical part
#'   of a GRADE table for a network meta-analysis (Puhan et al.,
#'   2014).
#' }
#' 
#' The following methods are implemented to rank treatments:
#' \itemize{
#' \item rankograms (\code{\link{rankogram}}) (Salanti et al., 2011);
#' \item ranking of treatments (\code{\link{netrank}}) based on
#'   P-scores (Rücker & Schwarzer, 2015) or the Surface Under the
#'   Cumulative RAnking curve (SUCRA) (Salanti et al., 2011);
#' \item partial order of treatment rankings (\code{\link{netposet}},
#'   \code{\link{plot.netposet}}) and Hasse diagram
#'   (\code{\link{hasse}}) according to Carlsen & Bruggemann (2014)
#'   and Rücker & Schwarzer (2017).
#' }
#' 
#' Available functions to evaluate network inconsistency:
#' \itemize{
#' \item split direct and indirect evidence (\code{\link{netsplit}})
#'   to check for consistency (Dias et al., 2010; Efthimiou et al.,
#'   2019);
#' \item scatter plot of direct and indirect estimates
#'   (\code{\link{plot.netmeta}}) to visualise local consistency
#'   (Wilson et al., 2026);
#' \item net heat plot (\code{\link{netheat}}) and design-based
#'   decomposition of Cochran's Q (\code{\link{decomp.design}})
#'   described in Krahn et al. (2013).
#' }
#' 
#' Additional methods and functions:
#' \itemize{
#' \item network meta-regression with a single continuous or binary covariate
#'   (\code{\link{netmetareg}});
#' \item subgroup network meta-analysis (\code{\link{subgroup.netmeta}});
#' \item information on network connectivity
#'   (\code{\link{netconnection}});
#' \item contribution of direct comparisons to network estimates
#'   (\code{\link{netcontrib}}) (Papakonstantinou et al., 2018; Davies
#'   et al., 2022);
#' \item importance of individual studies measured by reduction of
#'   precision if removed from network (\code{\link{netimpact}})
#'   (Rücker et al., 2020b);
#' \item\sQuote{comparison-adjusted} funnel plot
#'   (\code{\link{funnel.netmeta}}) to assess funnel plot asymmetry in
#'   network meta-analysis (Chaimani & Salanti, 2012);
#' \item conduct pairwise meta-analyses for all comparisons with
#'   direct evidence in a network meta-analysis
#'   (\code{\link{netpairwise}});
#' \item results of several network meta-analyses can be combined with
#'   \code{\link{netbind}} to show these results in a forest plot
#'   (\code{\link{forest.netbind}}).
#' \item measures characterizing the flow of evidence between two
#'   treatments (\code{\link{netmeasures}}) described in König et
#'   al. (2013);
#' \item calculate comparison effects of two arbitrary complex
#'   interventions in component network meta-analysis
#'   (\code{\link{netcomparison}});
#' \item calculate effect of arbitrary complex interventions in
#'   component network meta-analysis (\code{\link{netcomplex}}).
#' }
#' 
#' R package \bold{netmeta} provides two vignettes:
#' \itemize{
#'  \item \code{vignette("netmeta-workflow")} with an overview of the
#'    work flow and the main functions,
#'  \item \code{vignette("netmeta")} with the PDF file for
#'    Balduzzi et al. (2023).
#' }
#' 
#' Functions and datasets from \bold{netmeta} are utilised in
#' Schwarzer et al. (2015), Chapter 8 "Network Meta-Analysis",
#' \url{https://link.springer.com/book/10.1007/978-3-319-21416-0}.
#' 
#' Type \code{help(package = "netmeta")} for a listing of all R
#' functions available in \bold{netmeta}.
#' 
#' Type \code{citation("netmeta")} on how to cite \bold{netmeta} in
#' publications.
#' 
#' To report problems and bugs
#' \itemize{
#' \item type \code{bug.report(package = "netmeta")} if you do not use
#'   RStudio,
#' \item send an email to Guido Schwarzer
#'   \email{guido.schwarzer@@uniklinik-freiburg.de} if you use RStudio.
#' }
#' 
#' The development version of \bold{netmeta} is available on GitHub
#' \url{https://github.com/guido-s/netmeta}.
#'
#' @name netmeta-package
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}, Gerta
#'   Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @references
#' Balduzzi S, Rücker G, Nikolakopoulou A, Papakonstantinou T, Salanti
#' G, Efthimiou O, Schwarzer G (2023):
#' netmeta: An R Package for network meta-analysis using frequentist
#' methods.
#' \emph{Journal of Statistical Software},
#' \bold{106}, 1--40
#' 
#' Carlsen L, Bruggemann R (2014):
#' Partial order methodology: a valuable tool in chemometrics.
#' \emph{Journal of Chemometrics},
#' \bold{28}, 226--34
#' 
#' Chaimani A & Salanti G (2012):
#' Using network meta-analysis to evaluate the existence of
#' small-study effects in a network of interventions.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 161--76
#' 
#' Davies AL, Papakonstantinou T, Nikolakopoulou A, Rücker G, Galla T
#' (2022):
#' Network meta-analysis and random walks.
#' \emph{Statistics in Medicine},
#' \bold{41}, 2091--2114
#' 
#' Dias S, Welton NJ, Caldwell DM, Ades AE (2010):
#' Checking consistency in mixed treatment comparison meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{29}, 932--44
#' 
#' Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
#' (2019):
#' A Mantel-Haenszel model for network meta-analysis of rare events.
#' \emph{Statistics in Medicine},
#' \bold{38}, 2992--3012
#' 
#' Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A (2022):
#' Network Meta-analysis of rare events using penalized likelihood regression.
#' \emph{Statistics in Medicine},
#' \bold{41}, 5203--19.
#' 
#' König J, Krahn U, Binder H (2013):
#' Visualizing the flow of evidence in network meta-analysis and
#' characterizing mixed treatment comparisons.
#' \emph{Statistics in Medicine},
#' \bold{32}, 5414--29
#' 
#' Krahn U, Binder H, König J (2013):
#' A graphical tool for locating inconsistency in network meta-analyses.
#' \emph{BMC Medical Research Methodology},
#' \bold{13}, 35
#' 
#' Kwarteng N, Evrenoglou T, Mueller J, Elsaesser M, Schramm E, Schwarzer G,
#' Nikolakopoulou A (2026):
#' Illustrating the assumptions of meta-regression in treatment networks.
#' Preprint available at \emph{Research Square},
#' \doi{10.21203/rs.3.rs-8235913/v1}
#' 
#' Papakonstantinou, T., Nikolakopoulou, A., Rücker, G., Chaimani, A.,
#' Schwarzer, G., Egger, M., Salanti, G. (2018):
#' Estimating the contribution of studies in network meta-analysis:
#' paths, flows and streams.
#' \emph{F1000Research}
#' 
#' Puhan MA, Schünemann HJ, Murad MH, et al. (2014):
#' A GRADE working group approach for rating the quality of treatment
#' effect estimates from network meta-analysis.
#' \emph{British Medical Journal},
#' \bold{349}, g5630
#' 
#' Rücker G (2012):
#' Network meta-analysis, electrical networks and graph theory.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 312--24
#' 
#' Rücker G, Schwarzer G (2014):
#' Reduce dimension or reduce weights? Comparing two approaches to
#' multi-arm studies in network meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{33}, 4353--69
#' 
#' Rücker G, Schwarzer G (2015):
#' Ranking treatments in frequentist network meta-analysis works
#' without resampling methods.
#' \emph{BMC Medical Research Methodology},
#' \bold{15}, 58
#' 
#' Rücker G, Schwarzer G (2016):
#' Automated drawing of network plots in network meta-analysis.
#' \emph{Research Synthesis Methods},
#' \bold{7}, 94--107
#' 
#' Rücker G, Schwarzer G (2017):
#' Resolve conflicting rankings of outcomes in network meta-analysis:
#' Partial ordering of treatments.
#' \emph{Research Synthesis Methods},
#' \bold{8}, 526--36
#' 
#' Rücker G, Petropoulou M, Schwarzer G (2020a):
#' Network meta-analysis of multicomponent interventions.
#' \emph{Biometrical Journal},
#' \bold{62}, 808--21
#' 
#' Rücker G, Nikolakopoulou A, Papakonstantinou T, Salanti G, Riley
#' RD, Schwarzer G (2020b):
#' The statistical importance of a study for a network meta-analysis
#' estimate.
#' \emph{BMC Medical Research Methodology},
#' \bold{20}, 190
#' 
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#' 
#' Schwarzer G, Carpenter JR and Rücker G (2015):
#' \emph{Meta-Analysis with R (Use R!)}.
#' Springer International Publishing, Switzerland.
#' 
#' Wilson H, Schönstein A, Robson S, Bonofiglio F (2026):
#' A novel approach for visualizing local consistency in
#' network meta-analysis.
#' \emph{Research Synthesis Methods},
#' 1--15
#'
#' @keywords package
#' 
#' @importFrom meta baujat forest funnel radial trimfill longarm metabias metabin metacont metagen metainc metacum metainf metareg gs ci cilayout backtransf pairwise settings.meta
#'
#' @importFrom metafor bldiag contrmat rma.mv rma.uni
#'
#' @importFrom Matrix bdiag
#'
#' @importFrom MASS ginv
#'
#' @importFrom magic adiag
#' 
#' @importFrom mvtnorm rmvnorm
#'
#' @importFrom stats as.formula dist hclust optim optimize pchisq prcomp relevel reshape rnorm sd coef glm binomial vcov update fitted residuals quantile setNames model.matrix aggregate
#'   
#' @importFrom methods as
#'
#' @importFrom utils installed.packages packageDescription capture.output packageVersion
#'
#' @importFrom igraph E<- V<- E V all_simple_paths delete_edges get.shortest.paths graph_from_adjacency_matrix graph_from_edgelist gsize set_edge_attr set_vertex_attr head_of tail_of
#'
#' @importFrom grDevices colours col2rgb heat.colors rainbow rgb xy.coords
#'
#' @importFrom graphics axis box lines par points plot polygon rect text strheight strwidth title
#'
#' @importFrom ggplot2 ggplot aes xlab ylab labs element_blank element_line element_rect element_text expand_limits geom_col geom_line geom_step geom_tile geom_text ggtitle geom_abline geom_hline geom_point geom_vline scale_fill_gradient2 scale_x_discrete scale_y_discrete scale_x_continuous scale_x_discrete scale_y_discrete theme theme_classic theme_dark scale_color_manual coord_cartesian guide_legend guides margin
#' 
#' @importFrom ggrepel geom_text_repel
#'
#' @importFrom colorspace sequential_hcl rainbow_hcl sequential_hcl diverge_hcl heat_hcl terrain_hcl diverge_hsv choose_palette
#' 
#' @importFrom grid arrow convertHeight convertWidth convertX convertY drawDetails gpar grid.clip grid.draw grid.lines grid.newpage grid.rect grid.roundrect grid.text grob popViewport pushViewport stringWidth unit viewport
#' 
#' @importFrom dplyr %>% filter select rename starts_with relocate last_col mutate if_else bind_rows pull case_when distinct count summarise inner_join
#' 
#' @importFrom stringr str_length
#' 
#' @importFrom tidyr pivot_longer
#' 
#' @importFrom magrittr %<>%

"_PACKAGE"

NULL

#' Count statistics of survival data
#' 
#' @description
#' Count mortality statistics in randomised controlled trials of
#' treatments for chronic obstructive pulmonary disease (Woods et
#' al. (2010), Table 1).
#' 
#' @name Woods2010
#'
#' @docType data
#'
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{author}}\tab first author / study name \cr
#' \bold{\emph{treatment}}\tab treatment \cr
#' \bold{\emph{r}}\tab number of deaths in treatment arm \cr
#' \bold{\emph{N}}\tab number of patients in treatment arm
#' }
#' 
#' @note
#' The dataset Woods2010 is identical to dataset
#' \code{\link[metabook]{Woods2010}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
#'   \code{\link{netmeta}}
#' 
#' @source
#' Woods BS, Hawkins N, Scott DA (2010):
#' Network meta-analysis on the log-hazard scale, combining count and
#' hazard ratio statistics accounting for multi-arm trials: A
#' tutorial.
#' \emph{BMC Medical Research Methodology},
#' \bold{10}, 54
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Woods2010)
#' 
#' # Example using pairwise() and netmeta():
#' # example(Woods2010, package = "metabook")

NULL

#' Create league table with network meta-analysis results
#' 
#' @description
#' A league table is a square matrix showing all pairwise comparisons
#' in a network meta-analysis. Typically, both treatment estimates and
#' confidence intervals are shown.
#' 
#' @aliases netleague print.netleague
#' 
#' @param x An object of created with \code{netmeta}, \code{netcomb},
#'   \code{discomb}, or \code{netleague} (mandatory).
#' @param y An object of class \code{netmeta}, \code{netcomb}, or \code{discomb}
#'   (optional).
#' @param common A logical indicating whether a league table should be
#'   printed for the common effects network meta-analysis.
#' @param random A logical indicating whether a league table should be
#'   printed for the random effects network meta-analysis.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments in rows and columns of a league table.
#' @param ci A logical indicating whether confidence intervals should
#'   be shown.
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. If \code{backtransf = TRUE}, results
#'   for \code{sm = "OR"} are printed as odds ratios rather than log
#'   odds ratios, for example.
#' @param direct A logical indicating whether league tables with
#'   network estimates or estimates from direct comparisons
#'   should be shown if (i) argument \code{y} is provided and the input for
#'   arguments \code{x} and \code{y} was created with \code{netmeta} or (ii)
#'   input for argument \code{x} was created with \code{netmeta} or
#'   \code{netcomb}.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param text.NA A character string to label missing values.
#' @param bracket A character with bracket symbol to print lower
#'   confidence interval: "[", "(", "\{", "".
#' @param separator A character string with information on separator
#'   between lower and upper confidence interval.
#' @param lower.blank A logical indicating whether blanks between left
#'   bracket and lower confidence limit should be printed.
#' @param upper.blank A logical indicating whether blanks between
#'   separator and upper confidence limit should be printed.
#' @param writexl A logical indicating whether an Excel file should be
#'   created (R package \bold{writexl} must be available).
#' @param path A character string specifying the filename of the Excel
#'   file.
#' @param overwrite A logical indicating whether an existing Excel
#'   file should be overwritten.
#' @param details A logical specifying whether details on rows and columns
#'   should be printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (passed on to \code{write_xlsx}
#'   to create Excel file).
#' 
#' @details
#' A league table is a square matrix showing all pairwise comparisons
#' in a network meta-analysis (Hutton et al., 2015). Typically, both
#' treatment estimates and confidence intervals are shown.
#' 
#' If argument \code{y} is not provided, the league table contains
#' \itemize{
#' \item the network estimates in the lower triangle and the direct treatment
#'   estimates from pairwise comparisons in the upper triangle, if the input
#'   for argument \code{x} was created with \code{netmeta};
#' \item the network estimates from the component network meta-analysis in the
#'   lower triangle, if the input for argument \code{x} was created with
#'   \code{netcomb}, and, in the upper triangle, (i) the network estimates from
#'   the network meta-analysis, if argument \code{direct = FALSE}, or
#'   (ii)) the direct treatment estimates from pairwise comparisons, if
#'   argument \code{direct = TRUE};
#' \item the network estimates from the component network meta-analysis in the
#'   lower and upper triangle, if the input for argument \code{x} was created
#'   with \code{discomb}.
#' }
#' Note, for the random-effects model, the direct treatment estimates are based
#' on the common between-study variance \eqn{\tau^2} from the network
#' meta-analysis, i.e. the square of list element \code{x$tau}.
#' 
#' If argument \code{y} is provided, the league table contains
#' information on treatment comparisons from (component) network meta-analysis
#' object \code{x} in the lower triangle and from (component) network
#' meta-analysis object \code{y} in the upper triangle. This is, for
#' example, useful to print information on efficacy and safety in the
#' same league table. If argument \code{direct = TRUE}, direct estimates are
#' shown both in the lower and upper triangle if the input for arguments
#' \code{x} and \code{y} was created with \code{netmeta}.
#'
#' By default, an R object with the league tables is generated. Alternatively,
#' an Excel file is created if argument \code{writexl = TRUE}.
#' 
#' This implementation reports pairwise comparisons of the treatment
#' in the column versus the treatment in the row in the lower triangle
#' and row versus column in the upper triangle. This is a common
#' presentation for network meta-analyses which allows to easily
#' compare direction and magnitude of treatment effects. For example,
#' given treatments A, B, and C, the results reported in the first row
#' and second column as well as second row and first column are from
#' the pairwise comparison A versus B. Note, this presentation is
#' different from the printout of a network meta-analysis object which
#' reports opposite pairwise comparisons in the lower and upper
#' triangle, e.g., A versus B in the first row and second column and B
#' versus A in the second row and first column.
#' 
#' If the same (component) network meta-analysis object is used for arguments
#' \code{x} and \code{y}, reciprocal treatment estimates will be shown
#' in the upper triangle (see examples), e.g., the comparison B versus
#' A.
#' 
#' R function \code{\link{netrank}} can be used to change the order of
#' rows and columns in the league table (see examples).
#'
#' @return
#' An object of class \code{netleague} with corresponding \code{print}
#' function if \code{writexl = FALSE}. The object is a list containing
#' the league tables in list elements 'common' and 'random'. An Excel
#' file is created if \code{writexl = TRUE}. In this case, \code{NULL}
#' is returned in R.
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}, Gerta
#'   Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netcomb}},
#'   \code{\link{discomb}}, \code{\link{netposet}}, \code{\link{netrank}},
#'   \code{\link[metabook]{Woods2010}},
#'   \code{\link[metabook]{Linde2015}}
#'
#' 
#' @references
#' Hutton B, Salanti G, Caldwell DM, et al. (2015):
#' The PRISMA Extension Statement for Reporting of Systematic Reviews
#' Incorporating Network Meta-analyses of Health Care Interventions:
#' Checklist and Explanations.
#' \emph{Annals of Internal Medicine},
#' \bold{162}, 777
#' 
#' @keywords print
#' 
#' @examples
#' # Network meta-analysis of count mortality statistics
#' #
#' pw0 <- pairwise(treatment, event = r, n = N,
#'   studlab = author, data = Woods2010, sm = "OR",
#'   reference.group = "Placebo")
#' nma0 <- netmeta(pw0)
#' 
#' oldopts <- options(width = 100)
#' 
#' # League table for common and random effects model with
#' # - network estimates in lower triangle
#' # - direct estimates in upper triangle
#' #
#' netleague(nma0, digits = 2, bracket = "(", separator = " - ")
#' 
#' \donttest{
#' # League table for common effects model
#' #
#' netleague(nma0, random = FALSE, digits = 2)
#' 
#' # Change order of treatments according to treatment ranking (random
#' # effects model)
#' #
#' netleague(nma0, common = FALSE, digits = 2, seq = netrank(nma0))
#' #
#' print(netrank(nma0), common = FALSE)
#' 
#' # Create a CSV file with league table for random effects model
#' #
#' league0 <- netleague(nma0, digits = 2, bracket = "(", separator = " to ")
#' #
#' write.table(league0$random, file = "league0-random.csv",
#'   row.names = FALSE, col.names = FALSE, sep = ",")
#' #
#' # Create Excel files with league tables
#' # (if R package writexl is available)
#' #
#' netleague(nma0, digits = 2, bracket = "(", separator = " to ",
#'           path = tempfile(fileext = ".xlsx"))
#' 
#' # Define order of treatments in depression dataset Linde2015
#' #
#' trts <- c("TCA", "SSRI", "SNRI", "NRI",
#'   "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")
#' 
#' # Outcome labels
#' #
#' outcomes <- c("Early response", "Early remission")
#' 
#' # (1) Early response
#' #
#' pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma1 <- netmeta(pw1, common = FALSE,
#'   seq = trts, ref = "Placebo", small = "undesirable")
#' 
#' # (2) Early remission
#' #
#' pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma2 <- netmeta(pw2, common = FALSE,
#'   seq = trts, ref = "Placebo", small = "undesirable")
#' 
#' options(width = 200)
#' netleague(nma1, digits = 2)
#' 
#' netleague(nma1, digits = 2, ci = FALSE)
#' netleague(nma2, digits = 2, ci = FALSE)
#' 
#' # League table for two outcomes with
#' # - network estimates of first outcome in lower triangle
#' # - network estimates of second outcome in upper triangle
#' #
#' netleague(nma1, nma2, digits = 2, ci = FALSE)
#' 
#' netleague(nma1, nma2, seq = netrank(nma1), ci = FALSE)
#' netleague(nma1, nma2, seq = netrank(nma2), ci = FALSE)
#' 
#' netrank(nma1)
#' netrank(nma2)
#' 
#' # Report results for network meta-analysis twice
#' #
#' netleague(nma1, nma1, seq = netrank(nma1), ci = FALSE,
#'   backtransf = FALSE)
#' netleague(nma1, nma1, seq = netrank(nma1), ci = FALSE,
#'   backtransf = FALSE, direct = TRUE)
#' }
#' 
#' options(oldopts)
#' 
#' @rdname netleague
#' @export netleague

netleague <- function(x, y,
                      common = x$common, random = x$random,
                      seq = x$seq, ci = TRUE, backtransf = x$backtransf,
                      direct,
                      #
                      nchar.trts = x$nchar.trts,
                      #
                      digits = gs("digits"),
                      ##
                      big.mark = gs("big.mark"),
                      text.NA = ".",
                      ##
                      bracket = gs("CIbracket"),
                      separator = gs("CIseparator"),
                      lower.blank = gs("CIlower.blank"),
                      upper.blank = gs("CIupper.blank"),
                      ##
                      writexl = !missing(path),
                      path = "leaguetable.xlsx",
                      overwrite = FALSE,
                      #
                      details = gs("details"),
                      #
                      warn.deprecated = gs("warn.deprecated"),
                      ...) {
  
  
  ##
  ##
  ## (1) Check class and arguments
  ##
  ##
  
  chkclass(x, c("netmeta", "netcomb"))
  x <- updateversion(x)
  #
  x.netmeta <- inherits(x, "netmeta")
  x.discomb <- inherits(x, "discomb")
  x.netcomb <- !(x.netmeta | x.discomb)
  #
  x.is.y <- FALSE
  #
  avail.y <- !missing(y)
  #
  if (avail.y) {
    chkclass(y, c("netmeta", "netcomb"))
    ##
    if (length(x$seq) != length(y$seq))
      stop("Arguments 'x' and 'y' must have the same number of treatments.")
    if (any(sort(x$seq) != sort(y$seq)))
      stop("Arguments 'x' and 'y' must have the same treatments.")
    #
    y.netmeta <- inherits(y, "netmeta")
    y.discomb <- inherits(y, "discomb")
    y.netcomb <- !(y.netmeta | y.discomb)
    #
    x.is.y <- all.equal(x, y)
    x.is.y <- is.logical(x.is.y) && x.is.y
  }
  #
  if (missing(seq)) {
    seq.c <- seq.r <- seq
  }
  else {
    if (is.null(seq))
      stop("Argument 'seq' must be not NULL.")
    else if (inherits(seq, "netrank")) {
      ranking.c <- seq$ranking.common
      ranking.r <- seq$ranking.random
      ##
      seq.c <- setseq(names(ranking.c)[rev(order(ranking.c))], x$seq)
      seq.r <- setseq(names(ranking.r)[rev(order(ranking.r))], x$seq)
    }
    else
      seq.c <- seq.r <- setseq(seq, x$seq)
  }
  ##
  chklogical(ci)
  #
  chklogical(backtransf)
  #
  if (missing(direct)) {
    if (!avail.y)
      direct <- !x.discomb
    else
      direct <- FALSE
  }
  chklogical(direct)
  #
  if (direct &&
      (inherits(x, "discomb") || (avail.y && inherits(y, "discomb")))) {
    warning("Argument 'direct' set to FALSE for object created with discomb().")
    direct <- FALSE
  }
  #
  if (!direct && (inherits(x, "netmeta")) && !avail.y) {
    warning(paste("Argument 'direct' set to TRUE for single object",
                  "created with netmeta()."))
    direct <- TRUE
  }
  #
  nchar.trts <- replaceNULL(nchar.trts, 666)
  chknumeric(nchar.trts, min = 1, length = 1)
  #
  chknumeric(digits, min = 0, length = 1)
  ##
  chkchar(text.NA)
  chkchar(big.mark)
  ##
  bracket.old <- gs("CIbracket")
  separator.old <- gs("CIseparator")
  lower.blank.old <- gs("CIlower.blank")
  upper.blank.old <- gs("CIupper.blank")
  ##
  cilayout(bracket, separator, lower.blank, upper.blank)
  on.exit(cilayout(bracket.old, separator.old,
                   lower.blank.old, upper.blank.old))
  ##
  chklogical(writexl)
  chkchar(path, length = 1)
  chklogical(overwrite)
  #
  chklogical(details)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  
  
  #
  #
  # (2) Set details
  #
  #
  
  if (details) {
    if (x.netmeta) {
      if (avail.y & direct)
        txt.x <- "\nLower triangle: results from direct comparisons"
      else
        txt.x <- "\nLower triangle: results from network meta-analysis"
    }
    else
      txt.x <- "\nLower triangle: results from component network meta-analysis"
    #
    if (avail.y) {
      if (y.netmeta) {
        if (direct)
          txt.y <- "\nUpper triangle: results from direct comparisons"
        else
          txt.y <- "\nUpper triangle: results from network meta-analysis"
      }
      else
        txt.y <-
          "\nUpper triangle: results from component network meta-analysis"
    }
    else {
      if (x.netmeta)
        txt.y <- "\nUpper triangle: results from direct comparisons"
      else if (x.netcomb) {
        if (direct)
          txt.y <- "\nUpper triangle: results from direct comparisons"
        else
          txt.y <- "\nUpper triangle: results from network meta-analysis"
      }
      else {
          txt.y <-
            "\nUpper triangle: results from component network meta-analysis"
      }
    }    
    #
    txt.x <- paste(txt.x, "(column vs row)")
    txt.y <- paste(txt.y,
                   if (x.discomb & !avail.y)
                     "(column vs row)"
                   else if (!avail.y | (avail.y & !x.is.y)) "(row vs column)"
                   else "(column vs row)")
    #
    text.details <- paste0(txt.x, txt.y, "\n")
  }
  else
    text.details <- NULL
  
  
  ##
  ##
  ## (3) Values in lower triangle
  ##
  ##
  
  if (x.netmeta & avail.y & direct) {
    TE.common.x    <- x$TE.direct.common
    lower.common.x <- x$lower.direct.common
    upper.common.x <- x$upper.direct.common
    #
    TE.random.x    <- x$TE.direct.random
    lower.random.x <- x$lower.direct.random
    upper.random.x <- x$upper.direct.random
  }
  else if (x.netcomb & avail.y & direct) {
    TE.common.x    <- x$x$TE.direct.common
    lower.common.x <- x$x$lower.direct.common
    upper.common.x <- x$x$upper.direct.common
    #
    TE.random.x    <- x$x$TE.direct.random
    lower.random.x <- x$x$lower.direct.random
    upper.random.x <- x$x$upper.direct.random
  }
  else {
    TE.common.x    <- x$TE.common
    lower.common.x <- x$lower.common
    upper.common.x <- x$upper.common
    #
    TE.random.x    <- x$TE.random
    lower.random.x <- x$lower.random
    upper.random.x <- x$upper.random
  }
  
  
  #
  #
  # (4) Values in upper triangle
  #
  #
  
  if (!avail.y) {
    if (x.netmeta) {
      TE.common.y    <- x$TE.direct.common
      lower.common.y <- x$lower.direct.common
      upper.common.y <- x$upper.direct.common
      #
      TE.random.y    <- x$TE.direct.random
      lower.random.y <- x$lower.direct.random
      upper.random.y <- x$upper.direct.random
    }
    else if (inherits(x, "discomb")) {
      TE.common.y    <- x$TE.common
      lower.common.y <- x$lower.common
      upper.common.y <- x$upper.common
      #
      TE.random.y    <- x$TE.random
      lower.random.y <- x$lower.random
      upper.random.y <- x$upper.random
    }
    else {
      if (direct) {
        TE.common.y    <- x$x$TE.direct.common
        lower.common.y <- x$x$lower.direct.common
        upper.common.y <- x$x$upper.direct.common
        #
        TE.random.y    <- x$x$TE.direct.random
        lower.random.y <- x$x$lower.direct.random
        upper.random.y <- x$x$upper.direct.random
      }
      else {
        TE.common.y    <- x$x$TE.common
        lower.common.y <- x$x$lower.common
        upper.common.y <- x$x$upper.common
        #
        TE.random.y    <- x$x$TE.random
        lower.random.y <- x$x$lower.random
        upper.random.y <- x$x$upper.random
      }
    }
  }
  else {
    if (y.netmeta & direct) {
      TE.common.y    <- y$TE.direct.common
      lower.common.y <- y$lower.direct.common
      upper.common.y <- y$upper.direct.common
      #
      TE.random.y    <- y$TE.direct.random
      lower.random.y <- y$lower.direct.random
      upper.random.y <- y$upper.direct.random
    }
    else if (y.netcomb & direct) {
      TE.common.y    <- y$x$TE.direct.common
      lower.common.y <- y$x$lower.direct.common
      upper.common.y <- y$x$upper.direct.common
      #
      TE.random.y    <- y$x$TE.direct.random
      lower.random.y <- y$x$lower.direct.random
      upper.random.y <- y$x$upper.direct.random
    }
    else {
      TE.common.y    <- y$TE.common
      lower.common.y <- y$lower.common
      upper.common.y <- y$upper.common
      #
      TE.random.y    <- y$TE.random
      lower.random.y <- y$lower.random
      upper.random.y <- y$upper.random
    }
  }
  #
  if (x.is.y | (x.discomb & !avail.y)) {
    TE.common.y    <- t(TE.common.y)
    lower.common.y <- t(lower.common.y)
    upper.common.y <- t(upper.common.y)
    #
    TE.random.y    <- t(TE.random.y)
    lower.random.y <- t(lower.random.y)
    upper.random.y <- t(upper.random.y)
  }
  
  
  #
  #
  # (5) Back-transform results
  #
  #
  
  if (backtransf) {
    TE.common.x    <- backtransf(TE.common.x, x$sm)
    lower.common.x <- backtransf(lower.common.x, x$sm)
    upper.common.x <- backtransf(upper.common.x, x$sm)
    #
    # Switch lower and upper limit for VE if results have been
    # backtransformed
    #
    if (x$sm == "VE") {
      tmp.l <- lower.common.x
      lower.common.x <- upper.common.x
      upper.common.x <- tmp.l
    }
    #
    if (random) {
      TE.random.x    <- backtransf(TE.random.x, x$sm)
      lower.random.x <- backtransf(lower.random.x, x$sm)
      upper.random.x <- backtransf(upper.random.x, x$sm)
      #
      if (x$sm == "VE") {
        tmp.l <- lower.random.x
        lower.random.x <- upper.random.x
        upper.random.x <- tmp.l
      }
    }
    #
    TE.common.y    <- backtransf(TE.common.y, x$sm)
    lower.common.y <- backtransf(lower.common.y, x$sm)
    upper.common.y <- backtransf(upper.common.y, x$sm)
    #
    # Switch lower and upper limit for VE if results have been
    # backtransformed
    #
    if (x$sm == "VE") {
      tmp.l <- lower.common.y
      lower.common.y <- upper.common.y
      upper.common.y <- tmp.l
    }
    #
    if (random) {
      TE.random.y    <- backtransf(TE.random.y, x$sm)
      lower.random.y <- backtransf(lower.random.y, x$sm)
      upper.random.y <- backtransf(upper.random.y, x$sm)
      #
      if (x$sm == "VE") {
        tmp.l <- lower.random.y
        lower.random.y <- upper.random.y
        upper.random.y <- tmp.l
      }
    }
  }
  
  
  #
  #
  # (6) Comparisons are column versus row
  #
  #
  
  TE.common.x <- t(TE.common.x)
  lower.common.x <- t(lower.common.x)
  upper.common.x <- t(upper.common.x)
  #
  TE.random.x <- t(TE.random.x)
  lower.random.x <- t(lower.random.x)
  upper.random.x <- t(upper.random.x)
  #
  TE.common.y <- t(TE.common.y)
  lower.common.y <- t(lower.common.y)
  upper.common.y <- t(upper.common.y)
  #
  TE.random.y <- t(TE.random.y)
  lower.random.y <- t(lower.random.y)
  upper.random.y <- t(upper.random.y)
  
  
  #
  #
  # (7) Create league table for common effects model
  #
  #
  
  TE.common.x    <- round(   TE.common.x[seq.c, seq.c], digits)
  lower.common.x <- round(lower.common.x[seq.c, seq.c], digits)
  upper.common.x <- round(upper.common.x[seq.c, seq.c], digits)
  #
  if (ci) {
    nl.NA <- is.na(TE.common.x)
    nl.c <- paste(formatN(TE.common.x, digits = digits,
                          text.NA = text.NA, big.mark = big.mark),
                  formatCI(lower.common.x, upper.common.x, lab.NA = text.NA,
                           big.mark = big.mark))
    nl.c[nl.NA] <- text.NA
  }
  else
    nl.c <- formatN(TE.common.x, digits = digits,
                    text.NA = text.NA, big.mark = big.mark)
  ##
  nl.c <- matrix(nl.c, nrow = nrow(TE.common.x), ncol = ncol(TE.common.x))
  diag(nl.c) <- treats(rownames(TE.common.x), nchar.trts)
  ##
  TE.common.y    <- round(   TE.common.y[seq.c, seq.c], digits)
  lower.common.y <- round(lower.common.y[seq.c, seq.c], digits)
  upper.common.y <- round(upper.common.y[seq.c, seq.c], digits)
  ##
  if (ci) {
    nl.NA <- is.na(TE.common.y)
    nl.c.y <- paste(formatN(TE.common.y,
                            digits = digits, text.NA = text.NA,
                            big.mark = big.mark),
                    formatCI(lower.common.y, upper.common.y, lab.NA = text.NA,
                             big.mark = big.mark))
    nl.c.y[nl.NA] <- text.NA
  }
  else
    nl.c.y <- formatN(TE.common.y, digits = digits,
                      text.NA = text.NA, big.mark = big.mark)
  ##
  nl.c.y <- matrix(nl.c.y, nrow = nrow(TE.common.y), ncol = ncol(TE.common.y))
  ##
  nl.c[upper.tri(nl.c)] <- t(nl.c.y)[upper.tri(nl.c)]
  ##
  nl.c <- as.data.frame(nl.c, stringsAsFactors = FALSE)
  #
  names(nl.c) <- rownames(nl.c) <- seq.c
  
  
  #
  #
  # (8) Create league table for random effects model
  #
  #
  
  if (random) {
    TE.random.x    <- round(   TE.random.x[seq.r, seq.r], digits)
    lower.random.x <- round(lower.random.x[seq.r, seq.r], digits)
    upper.random.x <- round(upper.random.x[seq.r, seq.r], digits)
    ##
    if (ci) {
      nl.NA <- is.na(TE.random.x)
      nl.r <- paste(formatN(TE.random.x, digits = digits,
                            text.NA = text.NA, big.mark = big.mark),
                    formatCI(lower.random.x, upper.random.x, lab.NA = text.NA,
                             big.mark = big.mark))
      nl.r[nl.NA] <- text.NA
    }
    else
      nl.r <- formatN(TE.random.x, digits = digits,
                      text.NA = text.NA, big.mark = big.mark)
    ##
    nl.r <- matrix(nl.r, nrow = nrow(TE.random.x), ncol = ncol(TE.random.x))
    diag(nl.r) <- treats(rownames(TE.random.x), nchar.trts)
    ##
    TE.random.y    <- round(   TE.random.y[seq.r, seq.r], digits)
    lower.random.y <- round(lower.random.y[seq.r, seq.r], digits)
    upper.random.y <- round(upper.random.y[seq.r, seq.r], digits)
    ##
    if (ci) {
      nl.NA <- is.na(TE.random.y)
      nl.r.y <- paste(formatN(TE.random.y, digits = digits,
                              text.NA = text.NA),
                      formatCI(lower.random.y, upper.random.y, lab.NA = text.NA))
      nl.r.y[nl.NA] <- text.NA
    }
    else
      nl.r.y <- formatN(TE.random.y, digits = digits,
                        text.NA = text.NA, big.mark = big.mark)
    ##
    nl.r.y <- matrix(nl.r.y, nrow = nrow(TE.random.y), ncol = ncol(TE.random.y))
    ##
    nl.r[upper.tri(nl.r)] <- t(nl.r.y)[upper.tri(nl.c)]
    ##
    nl.r <- as.data.frame(nl.r, stringsAsFactors = FALSE)
    #
    names(nl.r) <- rownames(nl.r) <- seq.r
  }
  
  
  #
  #
  # (9) Save Excel file
  #
  #
  
  if (writexl) {
    if (!(common | random)) {
      warning("Excel file not generated as neither ",
              "argument 'common' nor 'random' is TRUE.")
      return(invisible(NULL))
    }
    ##
    if (!is_installed_package("writexl", stop = FALSE))
      stop(paste0("Package 'writexl' missing.",
                  "\n  ",
                  "Please use the following R command for installation:",
                  "\n  install.packages(\"writexl\")"),
           call. = FALSE)
    ##
    if (file.exists(path) & !overwrite)
      warning("File '", path, "' exists. ",
              "Use argument 'overwrite = TRUE' to overwrite file.",
              call. = FALSE)
    else {
      if (common & random)
        xlsx <- list(common = nl.c, random = nl.r)
      else if (common)
        xlsx <- list(common = nl.c)
      else
        xlsx <- list(random = nl.r)
      ##
      writexl::write_xlsx(xlsx, path = path, col_names = FALSE, ...)
      message(paste0("League table", if (common & random) "s",
                     " saved in file '", path, "'."))
    }
    ##
    return(invisible(NULL))
  }
  
  
  #
  #
  # (10) Return league tables
  #
  #
  
  res <- list(common = nl.c, random = if (random) nl.r else NA,
              seq = seq, ci = ci, backtransf = backtransf,
              x = list(common = common, random = random),
              details = text.details,
              direct = direct,
              digits = digits,
              version = packageDescription("netmeta")$Version)
  #
  # Backward compatibility
  #
  res$fixed <- res$common
  res$x$fixed <- res$x$common
  ##
  class(res) <- "netleague"
  
  res
}


#' @rdname netleague
#' @method print netleague
#' @export

print.netleague <- function(x,
                            common = x$x$common,
                            random = x$x$random,
                            warn.deprecated = gs("warn.deprecated"),
                            ...) {
  
  
  ##
  ##
  ## (1) Check arguments
  ##
  ##
  chkclass(x, "netleague")
  x <- updateversion(x)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (2) Print league table for common effects model
  ##
  ##
  if (common) {
    cat("League table (common effects model):\n")
    ##
    prmatrix(x$common, quote = FALSE, right = TRUE,
             rowlab = rep("", nrow(x$common)),
             collab = rep("", ncol(x$common)))
    if (random)
      cat("\n")
  }
  
  
  ##
  ##
  ## (3) Print league table for random effects model
  ##
  ##
  if (random) {
    cat("League table (random effects model):\n")
    ##
    prmatrix(x$random, quote = FALSE, right = TRUE,
             rowlab = rep("", nrow(x$random)),
             collab = rep("", ncol(x$random)))
  }
  
  if (!is.null(x$details))
    cat(x$details)
  
  invisible(NULL)
}

#' Plot treatment ranking(s) of network meta-analyses
#' 
#' @description
#' Produce an image plot of treatment ranking(s) generated with R
#' function \code{netrank}.
#' 
#' @param ... A single netrank object or a list of netrank objects.
#' @param name An optional character vector providing descriptive
#'   names for the network meta-analysis objects.
#' @param common A logical indicating whether results for the common
#'   effects model should be plotted.
#' @param random A logical indicating whether results for the random
#'   effects model should be plotted.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments on the x-axis.
#' @param low A character string defining the colour for a P-score of
#'   0, see \code{\link[ggplot2]{scale_fill_gradient2}}.
#' @param mid A character string defining the colour for a P-score of
#'   0.5, see \code{\link[ggplot2]{scale_fill_gradient2}}.
#' @param high A character string defining the colour for a P-score of
#'   1, see \code{\link[ggplot2]{scale_fill_gradient2}}.
#' @param col Colour of text.
#' @param main Title.
#' @param main.size Font size of title, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param main.col Colour of title, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param main.face Font face of title, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param axis.size Font size of axis text, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param axis.col Colour of axis text, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param axis.face Font face of axis text, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param na.value Colour for missing values, see
#'   \code{\link[ggplot2]{scale_fill_gradient2}}.
#' @param angle Angle for text on x-axis, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param hjust.x A numeric between 0 and 1 with horizontal
#'   justification of text on x-axis, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param vjust.x A numeric between 0 and 1 with vertical
#'   justification of text on x-axis, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param hjust.y A numeric between 0 and 1 with horizontal
#'   justification of text on y-axis, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param vjust.y A numeric between 0 and 1 with vertical
#'   justification of text on y-axis, see
#'   \code{\link[ggplot2]{element_text}}.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param fixed Deprecated argument (replaced by 'common').
#' @param comb.fixed Deprecated argument (replaced by 'common').
#' @param comb.random Deprecated argument (replaced by 'random').
#'
#' @details
#' This function produces an image plot of network rankings (Palpacuer
#' et al., 2018, Figure 4). Note, a scatter plot of two network
#' rankings can be generated with \code{\link{plot.netposet}}.
#'
#' By default, treatments are ordered by decreasing P-scores of the
#' first network meta-analysis object. Argument \code{seq} can be used
#' to specify a differenct treatment order.
#' 
#' @return
#' A ggplot2 object or NULL if no ranking was conducted.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}, Clément
#'   Palpacuer \email{clementpalpacuer@@gmail.com}
#' 
#' @seealso \code{\link{netrank}}, \code{\link{netmeta}},
#'   \code{\link{netposet}}, \code{\link{hasse}},
#'   \code{\link[metabook]{Linde2015}}
#' 
#' @references
#' Palpacuer C, Duprez R, Huneau A, Locher C, Boussageon R, Laviolle
#' B, et al. (2018):
#' Pharmacologically controlled drinking in the treatment of alcohol
#' dependence or alcohol use disorders: a systematic review with
#' direct and network meta-analyses on nalmefene, naltrexone,
#' acamprosate, baclofen and topiramate.
#' \emph{Addiction},
#' \bold{113}, 220--37
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netrank)
#' 
#' @method plot netrank
#' @export

plot.netrank <- function(...,
                         name,
                         common, random,
                         ##
                         seq,
                         ##
                         low = "red", mid = "yellow", high = "green",
                         ##
                         col = "black",
                         ##
                         main,
                         main.size = 14, main.col = col,
                         main.face = "bold",
                         ##
                         legend = TRUE,
                         ##
                         axis.size = 12, axis.col = col,
                         axis.face = "plain",
                         ##
                         na.value = "grey50",
                         ##
                         angle = 45,
                         ##
                         hjust.x = 1, vjust.x = 1,
                         hjust.y = 1, vjust.y = 0,
                         ##
                         nchar.trts,
                         ##
                         digits = 3,
                         ##
                         fixed,
                         comb.fixed, comb.random,
                         ##
                         warn.deprecated = gs("warn.deprecated")) {
  
  ##
  ##
  ## (1) Identify list elements of class netrank
  ##
  ##
  args <- list(...)
  names.args <- names(args)
  ##
  n.args <- length(args)
  n.i <- seq_len(n.args)
  ##
  is.netrank <- rep_len(FALSE, n.args)
  ##
  for (i in n.i)
    is.netrank[i] <- inherits(args[[i]], "netrank")
  ##
  n.netrank <- sum(is.netrank)
  ##
  if (n.netrank == 0)
    stop("At least one R object of class 'netrank' must be provided.",
         call. = FALSE)
  ##
  for (i in n.i)
    if (is.netrank[i])
      args[[i]] <- updateversion(args[[i]])
  ##
  missing.name <- missing(name)
  missing.common <- missing(common)
  missing.random <- missing(random)
  missing.seq <- missing(seq)
  missing.main <- missing(main)
  missing.nchar.trts <- missing(nchar.trts)
  missing.fixed <- missing(fixed)
  missing.comb.fixed <- missing(comb.fixed)
  missing.comb.random <- missing(comb.random)
  
  
  ##
  ##
  ## (2) Identify additional arguments
  ##
  ##
  formal.args <- c("name", "common", "random",
                   "seq", "low", "mid", "high", "col",
                   "main", "main.size", "main.col", "main.face",
                   "legend", "axis.size", "axis.col", "axis.face",
                   "na.value", "angle",
                   "hjust.x", "vjust.x", "hjust.y", "vjust.y",
                   "nchar.trts", "digits",
                   "fixed", "comb.fixed", "comb.random", "warn.deprecated")
  ##
  for (i in n.i) {
    if (!is.netrank[i]) {
      cm <- charmatch(names.args[i], formal.args)
      ##
      if (is.na(cm))
        stop("Argument '", names.args[i],
             "' must be an R object of class 'netrank'.",
             call. = FALSE)
      else if (cm == 0)
        stop("Argument '", names.args[i],
             "' matches multiple formal arguments.",
             call. = FALSE)
      else {
        if (cm == 1) {
          missing.name <- FALSE
          name <- args[[i]]
        }
        else if (cm == 2) {
          missing.common <- FALSE
          common <- args[[i]]
        }
        else if (cm == 3) {
          missing.random <- FALSE
          random <- args[[i]]
        }
        else if (cm == 4) {
          missing.seq <- FALSE
          seq <- args[[i]]
        }
        else if (cm == 5)
          low <- args[[i]]
        else if (cm == 6)
          mid <- args[[i]]
        else if (cm == 7)
          high <- args[[i]]
        else if (cm == 8)
          col <- args[[i]]
        else if (cm == 9) {
          missing.main <- FALSE
          main <- args[[i]]
        }
        else if (cm == 10)
          main.size <- args[[i]]
        else if (cm == 11)
          main.col <- args[[i]]
        else if (cm == 12)
          main.face <- args[[i]]
        else if (cm == 13)
          legend <- args[[i]]
        else if (cm == 14)
          axis.size <- args[[i]]
        else if (cm == 15)
          axis.col <- args[[i]]
        else if (cm == 16)
          axis.face <- args[[i]]
        else if (cm == 17)
          na.value <- args[[i]]
        else if (cm == 18)
          angle <- args[[i]]
        else if (cm == 19)
          hjust.x <- args[[i]]
        else if (cm == 20)
          vjust.x <- args[[i]]
        else if (cm == 21)
          hjust.y <- args[[i]]
        else if (cm == 22)
          vjust.y <- args[[i]]
        else if (cm == 23) {
          missing.nchar.trts <- FALSE
          nchar.trts <- args[[i]]
        }
        else if (cm == 24)
          digits <- args[[i]]
        else if (cm == 25) {
          missing.comb.common <- FALSE
          comb.common <- args[[i]]
        }
        else if (cm == 26) {
          missing.comb.random <- FALSE
          comb.random <- args[[i]]
        }
        else if (cm == 27)
          warn.deprecated <- args[[i]]
      }
    }
  }
  
  
  ##
  ##
  ## (3) Check arguments
  ##
  ##
  chkchar(low)
  chkchar(mid)
  chkchar(high)
  ##
  chklogical(legend)
  chknumeric(main.size, min = 0, length = 1)
  chkchar(main.face)
  chknumeric(axis.size, min = 0, length = 1)
  chkchar(axis.face)
  chknumeric(angle, min = -360, max = 360, length = 1)
  chknumeric(hjust.x, min = 0, max = 1, length = 1)
  chknumeric(vjust.x, min = 0, max = 1, length = 1)
  chknumeric(hjust.y, min = 0, max = 1, length = 1)
  chknumeric(vjust.y, min = 0, max = 1, length = 1)
  chknumeric(digits, min = 0, length = 1)
  ##
  print.warning1 <- FALSE
  print.warning2 <- FALSE
  print.warning3 <- FALSE
  print.warning4 <- FALSE
  print.warning5 <- FALSE
  
  
  ##
  ##
  ## (4) Set names of network meta-analysis objects
  ##
  ##
  if (missing.name)
    name <- paste0("netmeta", seq_len(n.netrank))
  else {
    if (length(name) != n.netrank) 
      stop("Number of network meta-analyses and ",
           "names provided in argument 'name' differ.",
           call. = FALSE)
    ##
    if (length(unique(name)) != length(name)) {
      warning1 <-
        paste0("Network meta-analyses are labelled 'netmeta1' to 'netmeta",
               n.netrank,
               "' as values of argument 'name' are not all disparate.")
      print.warning1 <- TRUE
      ##
      name <- paste0("netmeta", seq_len(n.netrank))
    }
  }
  
  
  ##
  ##
  ## (5) Determine common
  ##
  ##
  if (missing.common & (!missing.fixed | !missing.comb.fixed)) {
    chklogical(warn.deprecated)
    common <-
      deprecated2(common, missing.common, comb.fixed, missing.comb.fixed,
                  warn.deprecated)
    common <-
      deprecated2(common, missing.common, fixed, missing.fixed,
                  warn.deprecated)
    missing.common <- FALSE
  }
  ##
  if (missing.common & missing.fixed & missing.comb.fixed) {
    cfs <- logical(0)
    ##
    for (i in n.i)
      if (is.netrank[i])
        cfs[i] <- args[[i]]$x$common
      else
        cfs[i] <- FALSE
    ##
    cfs <- unique(cfs[is.netrank])
    ##
    if (length(cfs) != 1) {
      common <- TRUE
      warning2 <- paste0("Argument 'common' set to TRUE ",
                         "(as it is not unique in network meta-analyses).")
      print.warning2 <- TRUE
    }
    else
      common <- cfs
  }
  else {
    chklogical(warn.deprecated)
    common <-
      deprecated2(common, missing.common, comb.fixed, missing.comb.fixed,
                  warn.deprecated)
    common <-
      deprecated2(common, missing.common, fixed, missing.fixed,
                  warn.deprecated)
    missing.common <- FALSE
  }
  ##
  chklogical(common)
  
  
  ##
  ##
  ## (6) Determine random
  ##
  ##
  if (missing.random & missing.comb.random) {
    crs <- logical(0)
    ##
    for (i in n.i)
      if (is.netrank[i])
        crs[i] <- args[[i]]$x$random
      else
        crs[i] <- FALSE
    ##
    crs <- unique(crs[is.netrank])
    ##
    if (length(crs) != 1) {
      random <- TRUE
      warning3 <- paste0("Argument 'random' set to TRUE ",
                         "(as it is not unique in network meta-analyses).")
      print.warning3 <- TRUE
    }
    else
      random <- crs
  }
  else {
    chklogical(warn.deprecated)
    random <-
      deprecated2(random, missing.random, comb.random, missing.comb.random,
                  warn.deprecated)
    missing.random <- FALSE
  }
  ##
  chklogical(random)
  ##
  if (common & random) {
    warning4 <- paste0("P-scores for random effects model displayed ",
                       "as both common and random effects ",
                       "network meta-analysis was conducted.")
    print.warning4 <- TRUE
    common <- FALSE
  }
  else if (!common & !random) {
    warning("No plot generated as ranking was neither for common nor ",
            "random effects network meta-analysis conducted.")
    return(invisible(NULL))
  }
  
  
  ##
  ##
  ## (7) Determine nchar.trts
  ##
  ##
  if (missing.nchar.trts) {
    cns <- logical(0)
    ##
    for (i in n.i)
      if (is.netrank[i])
        cns[i] <- args[[i]]$x$nchar.trts
      else
        cns[i] <- 666
    ##
    cns <- unique(cns[is.netrank])
    ##
    if (length(cns) != 1) {
      nchar.trts <- 666
      warning5 <- paste0("Argument 'nchar.trts' set to 666 ",
                         "(as it is not unique in network meta-analyses).")
      print.warning5 <- TRUE
    }
    else
      nchar.trts <- cns
  }
  else
    chknumeric(nchar.trts, min = 1, length = 1)
  
  
  ##
  ##
  ## (8) Determine list of all treatments
  ##
  ##
  trts <- character(0)
  ##
  for (i in n.i)
    if (is.netrank[i])
      trts <- c(trts, names(args[[i]]$ranking.common))
  ##
  trts <- unique(trts)
  ##  
  if (!missing.seq)
    seq <- setseq(seq, trts)
  else {
    trts1 <- data.frame(treat = trts, ranking = NA,
                        row.names = trts,
                        stringsAsFactors = FALSE)
    first <- min(seq_along(is.netrank)[is.netrank])
    trts.first <- names(args[[first]]$ranking.common)
    ##
    if (random)
      trts1[trts.first, "ranking"] <- args[[first]]$ranking.random
    else
      trts1[trts.first, "ranking"] <- args[[first]]$ranking.common
    ##
    trts1 <- trts1[rev(order(trts1$ranking, na.last = FALSE)), ]
    seq <- trts1$treat
  }
  ##
  n.trts <- length(trts)
  
  
  ##
  ##
  ## (9) Generate dataset for plot
  ##
  ##
  dat <- data.frame(name = character(0), treat = character(0),
                    ranking = numeric(0),
                    stringsAsFactors = FALSE)
  ##  
  for (i in n.i) {
    if (is.netrank[i]) {
      dat.i <- data.frame(name = rep_len(name[i], n.trts),
                          treat = trts, ranking = NA,
                          row.names = trts,
                          stringsAsFactors = FALSE)
      ##
      trts.i <- names(args[[i]]$ranking.common)
      ##
      if (random)
        dat.i[trts.i, "ranking"] <- args[[i]]$ranking.random
      else 
        dat.i[trts.i, "ranking"] <- args[[i]]$ranking.common
      ##
      dat <- rbind(dat, dat.i)
    }
  }
  ##
  row.names(dat) <- seq_len(nrow(dat))
  ##
  dat$ranking <- as.character(formatN(round(dat$ranking, digits),
                                      digits = digits, text.NA = ""))
  ##
  dat$name <- factor(dat$name, levels = rev(name))
  ##
  dat$treat <- factor(dat$treat, levels = seq,
                      labels = treats(seq, nchar.trts))
  ## Get rid of warnings 'Undefined global functions or variables'
  treat <- dat$treat
  ranking <- dat$ranking
  
  
  ##
  ##
  ## (10) Create ggplot2 object
  ##
  ##
  plt <- ggplot(dat,
                aes(x = treat, y = name, fill = as.numeric(ranking)))
  
  plt <- plt + theme_classic()
  
  plt <- plt + geom_tile() + xlab("") + ylab("") +
    theme(line = element_blank(),
          axis.ticks = element_blank(),
          axis.text.x = element_text(size = axis.size, face = axis.face,
                                     colour = axis.col,
                                     angle = angle,
                                     hjust = hjust.x, vjust = vjust.x),
          axis.text.y = element_text(size = axis.size, face = axis.face,
                                     colour = axis.col,
                                     angle = 0,
                                     hjust = hjust.y, vjust = vjust.y),
          legend.title = element_text(colour = col,
                                      size = 7, face = "bold"),
          legend.text = element_text(colour = col, size = 10)) +
    scale_fill_gradient2(low = low, mid = mid, high = high,
                         midpoint = 0.5, limit = c(0, 1),
                         space = "Lab", name = "P-scores",
                         na.value = na.value)
  
  plt <- plt + geom_text(aes(x = treat, y = name, label = ranking),
                         colour = col)
  
  if (!legend)
    plt <- plt + theme(legend.position = "none")
  
  if (!missing.main)
    plt <- plt + ggtitle(main) +
      theme(plot.title = element_text(size = main.size, face = main.face,
                                      colour = main.col))
  
  
  ##
  ## Print warnings
  ##
  if (print.warning1)
    warning(warning1, call. = FALSE)
  if (print.warning2)
    warning(warning2, call. = FALSE)
  if (print.warning3)
    warning(warning3, call. = FALSE)
  if (print.warning4)
    warning(warning4, call. = FALSE)
  if (print.warning5)
    warning(warning5, call. = FALSE)
  
  
  plt
}

#' Calculate comparison effects of two arbitrary complex interventions
#' in component network meta-analysis
#' 
#' @description
#' Calculate comparison effects of two arbitrary complex interventions
#' (i.e., combinations of several components) in component network
#' meta-analysis.
#' 
#' @param x An object of class \code{netcomb} or \code{netcomparison}
#'   (print function).
#' @param treat1 A character vector defining the first complex
#'   intervention(s).
#' @param treat2 A character vector defining the second complex
#'   intervention(s).
#' @param common A logical indicating whether results for common
#'   effects model should be conducted.
#' @param random A logical indicating whether results for random
#'   effects model should be conducted.
#' @param level The level used to calculate confidence intervals for
#'   combinations of components.
#' @param nchar.comps A numeric defining the minimum number of
#'   characters used to create unique names for components (see
#'   Details).
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. If \code{backtransf=TRUE}, results
#'   for \code{sm="OR"} are printed as odds ratios rather than log
#'   odds ratios.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z-value
#'   of test for overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for
#'   p-values, see \code{print.default}.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   combination effect should be printed according to JAMA reporting
#'   standards.
#' @param big.mark A character used as thousands separator.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#'
#' @details
#' R functions \code{\link{netcomb}} and \code{\link{discomb}}
#' calculate effects for individual components and complex
#' interventions present in the component network meta-analysis
#' (CNMA). This function can be used to calculate the effect for
#' comparisons of two arbitrary complex interventions defined by
#' arguments \code{treat1} and \code{treat2}.
#'
#' All complex interventions occuring in the network are considered
#' for the first complex intervention if argument \code{treat1} is
#' missing. The reference group defined in the (C)NMA is used as
#' second complex intervention if argument \code{treat2} is
#' missing. The first complex intervention in the (C)NMA is used if
#' the reference group is not defined.
#' 
#' The following matrices are needed to calculate comparison effects
#' of arbitrary complex interventions, (Rücker et al., 2020, Section
#' 3.2):
#' \itemize{
#' \item B matrix describing how comparisons are composed by complex
#'   intervetions,
#' \item C matrix describing how the complex interventions are
#'   composed by the components.
#' }
#' Internally, both matrices are constructed based on arguments
#' \code{x}, \code{treat1} and \code{treat2}.
#' 
#' By default, component names are not abbreviated in
#' printouts. However, in order to get more concise printouts,
#' argument \code{nchar.comps} can be used to define the minimum
#' number of characters for abbreviated component names (see
#' \code{\link{abbreviate}}, argument \code{minlength}). R function
#' \code{\link{treats}} is utilised internally to create abbreviated
#' component names.
#' 
#' @note
#' R function \code{\link{netcomplex}} can be used to calculate the
#' effect for arbitrary complex interventions in a component network
#' meta-analysis.
#' 
#' @return
#' A list is returned by the function \code{netcomparison} with the
#' following elements:
#' \item{comparison}{Comparison.}
#' \item{TE.common, TE.random}{A vector of comparison effects (common
#'   and random effects model).}
#' \item{seTE.common, seTE.random}{A vector with corresponding standard
#'   errors (common and random effects model).}
#' \item{lower.common, lower.random}{A vector with lower confidence
#'   limits for comparisons (common and random effects model).}
#' \item{upper.common, upper.random}{A vector with upper confidence
#'   limits for comparisons (common and random effects model).}
#' \item{statistic.common, statistic.random}{A vector with z-values for
#'   the overall effect of comparisons (common and random effects
#'   model).}
#' \item{pval.common, pval.random}{A vector with p-values for the
#'   overall effect of comparisons (common and random effects model).}
#' \item{trts}{Treatments included in comparisons.}
#' \item{comps}{Components included in comparisons.}
#' \item{treat1, treat2}{A defined above.}
#' \item{common, random}{A defined above.}
#' \item{level, nchar.comps, backtransf, x}{A defined above.}
#' \item{B.matrix}{B matrix.}
#' \item{C.matrix}{C matrix.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netcomb}}, \code{\link{discomb}},
#'   \code{\link{netcomplex}}
#' 
#' @references
#' Rücker G, Petropoulou M, Schwarzer G (2020):
#' Network meta-analysis of multicomponent interventions.
#' \emph{Biometrical Journal},
#' \bold{62}, 808--21
#' 
#' @examples
#' # Only consider studies including Face-to-face PST (to reduce
#' # runtime of example)
#' #
#' face <- subset(Linde2016, id %in% c(16, 24, 49, 118))
#' 
#' # Use pairwise() to transform data to comparison-based format
#' #
#' pw <- pairwise(treat = int,
#'   event = resp, n = n, studlab = paste(author, year),
#'   data = face, reference = "plac", sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma1 <- netmeta(pw, common = FALSE)
#' 
#' # Additive model for treatment components (with placebo as inactive
#' # treatment)
#' #
#' cnma1 <- netcomb(nma1, inactive = "placebo")
#' 
#' # Result for comparison Face-to-face PST vs TCA
#' netcomparison(cnma1, "Face-to-face PST", "TCA", nchar.comps = 4)
#' netcomparison(cnma1, "F", "T", nchar.comps = 4)
#'
#' # Result for comparison Face-to-face PST vs TCA + Placebo
#' netcomparison(cnma1, "Face-to-face PST", "TCA + Plac", nchar.comps = 4)
#' 
#' \donttest{
#' # Forest plots
#' #
#' t1 <- c("F + TCA", "F + Plac", "SSRI + Plac + TCA")
#' t2 <- c("UC", "Plac", "UC")
#' #
#' netcomparison(cnma1, t1, t2)
#' #
#' forest(netcomparison(cnma1, t1, t2))
#' forest(netcomparison(cnma1, t1, t2), nchar.comps = 4)
#' forest(netcomparison(cnma1, c("F", "TCA"), "UC"), nchar.comps = 4)
#' 
#' # Artificial example
#' #
#' treat1 <- rep("A", 3)
#' treat2 <- c("B+C", "A+C", "C+D")
#' TE <- c(0, 1, 0)
#' seTE <- rep(1, 3)
#' # Conduct (C)NMA
#' nma2 <- netmeta(TE, seTE, treat1, treat2, random = FALSE)
#' cnma2 <- netcomb(nma2)
#'
#' # Result for comparison A vs B + D
#' netcomparison(cnma2, "A", "B + D")
#' # Same results
#' netcomparison(cnma2, "A", "B+D")
#' netcomparison(cnma2, "A", "D+B")
#' netcomparison(cnma2, "a", "d+b")
#' 
#' # Generated B matrix
#' netcomparison(cnma2, "A", "B + D")$C.matrix
#' # Generated B matrix
#' netcomparison(cnma2, "A", "B + D")$B.matrix
#' }
#' 
#' @rdname netcomparison
#' @export
#' @export netcomparison

netcomparison <- function(x, treat1, treat2,
                          common = x$common,
                          random = x$random,
                          level = x$level.ma,
                          nchar.comps = x$nchar.comps,
                          backtransf = x$backtransf,
                          warn.deprecated = gs("warn.deprecated"),
                          ...) {
  
  
  chkclass(x, "netcomb")
  x <- updateversion(x)
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  chklogical(random)
  chklevel(level)
  ##
  chknumeric(nchar.comps, min = 1, length = 1)
  chklogical(backtransf)
  
  
  missing.treat1 <- missing(treat1)
  missing.treat2 <- missing(treat2)
  ##
  if (missing(treat1)) {
    treat1.orig <- NULL
    treat2.orig <- NULL
    treat1 <- x$trts
  }
  else
    treat1.orig <- treat1
  ##
  if (missing(treat2)) {
    treat2.orig <- NULL
    treat2 <- x$reference.group
    if (treat2 == "") {
      treat2 <- treat1[1]
      treat1 <- treat1[-1]
    }
    else
      treat1 <- treat1[treat1 != treat2]
  }
  else
    treat2.orig <- treat2
  ##
  if (length(treat1) == 1 & length(treat2) > 1)
    treat1 <- rep(treat1, length(treat2))
  else if (length(treat1) > 1 & length(treat2) == 1)
    treat2 <- rep(treat2, length(treat1))
  ##
  n.comparisons <- length(treat1)
  ##
  chklength(treat2, n.comparisons, "treat1")
  ##
  comps <- x$comps
  
  
  ##
  ## Check components
  ##
  comps1.list <- compsplit(treat1, x$sep.comps)
  comps2.list <- compsplit(treat2, x$sep.comps)
  ##
  comps1 <- setref(unique(unlist(comps1.list)), c(comps, x$inactive),
                   error.text = "component names in argument 'treat1'",
                   length = 0)
  comps2 <- setref(unique(unlist(comps2.list)), c(comps, x$inactive),
                   error.text = "component names in argument 'treat2'",
                   length = 0)
  ##
  comps1.list <- lapply(comps1.list, setref, c(comps, x$inactive), length = 0)
  comps2.list <- lapply(comps2.list, setref, c(comps, x$inactive), length = 0)
  ##
  add1 <- add2 <- rep("", n.comparisons)
  ##
  for (i in seq_len(n.comparisons)) {
    add1[i] <-
      if (attr(compsplit(treat1[i], x$sep.comps), "withspace")) " " else ""
    treat1[i] <-
      paste(comps1.list[[i]], collapse = paste0(add1[i], x$sep.comps, add1[i]))
  }
  ##
  for (i in seq_len(n.comparisons)) {
    add2[i] <-
      if (attr(compsplit(treat2[i], x$sep.comps), "withspace")) " " else ""
    treat2[i] <-
      paste(comps2.list[[i]], collapse = paste0(add2[i], x$sep.comps, add2[i]))
  }
  ##
  trts <- sort(unique(c(treat1, treat2)))
  
  
  ##
  ## Extract comparisons
  ##
  comparison <- rep("", n.comparisons)
  ##
  for (i in seq_len(n.comparisons)) {
    sel1.i <- !comps1.list[[i]] %in% comps2.list[[i]]
    sel2.i <- !comps2.list[[i]] %in% comps1.list[[i]]
    #
    if (any(sel1.i) | any(sel2.i)) {
      comb1 <- paste(comps1.list[[i]][sel1.i],
                     collapse = paste0(add1[i], x$sep.comps, add1[i]))
      comb2 <- paste(comps2.list[[i]][sel2.i],
                     collapse = paste0(add2[i], x$sep.comps, add2[i]))
      #
      if (comb1 == "")
        comparison[i] <- comb2
      else if (comb2 == "")
        comparison[i] <- comb1
      else
        comparison[i] <- paste0(comb1, x$sep.trts, comb2)
    }
  }
  
  
  ##
  ## Generate C matrix
  ##
  C.matrix <- matrix(0, nrow = length(trts), ncol = length(comps))
  rownames(C.matrix) <- trts
  colnames(C.matrix) <- comps
  ##
  for (i in seq_len(n.comparisons)) {
    C.matrix[treat1[i], ] <- 1L * colnames(C.matrix) %in% comps1.list[[i]]
    C.matrix[treat2[i], ] <- 1L * colnames(C.matrix) %in% comps2.list[[i]]
  }
  
  
  ##
  ## Generate B matrix
  ##
  B.matrix <- matrix(0, nrow = n.comparisons, ncol = length(trts))
  rownames(B.matrix) <- seq_len(n.comparisons)
  colnames(B.matrix) <- trts
  ##
  for (i in seq_len(n.comparisons)) {
    ##
    B.matrix[i, ] <-
      1L * colnames(B.matrix) %in% treat1[i] +
      -1L * colnames(B.matrix) %in% treat2[i]
  }
  
  
  ##
  ## Calculate estimates for comparisons
  ##
  X.matrix <- B.matrix %*% C.matrix
  #
  # Identify inestimable components and drop them from the design matrix
  #
  compNA <- is.na(x$Comp.common)
  inestimable <- apply(abs(X.matrix[, compNA, drop = FALSE]), 1, sum) > 0
  X.matrix <- X.matrix[, !compNA, drop = FALSE]
  #
  TE.common <- as.vector(X.matrix %*% x$Comp.common[!compNA])
  seTE.common <-
    sqrt(
      diag(X.matrix %*% x$Lplus.matrix.common[!compNA, !compNA] %*%
             t(X.matrix)))
  #
  TE.random <- as.vector(X.matrix %*% x$Comp.random[!compNA])
  seTE.random <-
    sqrt(diag(X.matrix %*% x$Lplus.matrix.random[!compNA, !compNA] %*%
                t(X.matrix)))
  #
  # Set comparisons with inestimable components to NA
  #
  TE.common[inestimable] <- NA
  seTE.common[inestimable] <- NA
  TE.random[inestimable] <- NA
  seTE.random[inestimable] <- NA
  #
  # Calculate confidence intervals
  #
  ci.f <- ci(TE.common, seTE.common, level = level)
  ci.r <- ci(TE.random, seTE.random, level = level)
  
  
  res <- list(comparison = comparison,
              treat1 = treat1, treat2 = treat2,
              ##
              TE.common = ci.f$TE,
              seTE.common = ci.f$seTE,
              lower.common = ci.f$lower,
              upper.common = ci.f$upper,
              statistic.common = ci.f$statistic,
              pval.common = ci.f$p,
              ##
              TE.random = ci.r$TE,
              seTE.random = ci.r$seTE,
              lower.random = ci.r$lower,
              upper.random = ci.r$upper,
              statistic.random = ci.r$statistic,
              pval.random = ci.r$p,
              ##
              common = common,
              random = random,
              level = level,
              ##
              trts = trts,
              comps = colnames(C.matrix)[apply(C.matrix, 2, sum) > 0],
              inactive = x$inactive,
              nchar.comps = nchar.comps,
              backtransf = backtransf,
              ##
              B.matrix = B.matrix,
              C.matrix = C.matrix,
              ##
              x = x,
              ##
              add1 = add1,
              add2 = add2,
              ##
              treat1.orig = treat1.orig,
              treat2.orig = treat2.orig,
              ##
              version = packageDescription("netmeta")$Version
              )
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  ##
  res$TE.fixed <- res$TE.common
  res$seTE.fixed <- res$seTE.common
  res$lower.fixed <- res$lower.common
  res$upper.fixed <- res$upper.common
  res$statistic.fixed <- res$statistic.common
  res$pval.fixed <- res$pval.common
  ##
  class(res) <- c("netcomparison", class(res))
  
  res
}


#' @rdname netcomparison
#' @method print netcomparison
#' @export

print.netcomparison <- function(x,
                                ##
                                common = x$common,
                                random = x$random,
                                backtransf = x$backtransf,
                                ##
                                nchar.comps = x$nchar.comps,
                                ##
                                digits = gs("digits"),
                                digits.stat = gs("digits.stat"),
                                digits.pval = gs("digits.pval"),
                                ##
                                scientific.pval = gs("scientific.pval"),
                                zero.pval = gs("zero.pval"),
                                JAMA.pval = gs("JAMA.pval"),
                                big.mark = gs("big.mark"),
                                ##
                                legend = gs("legend"),
                                warn.deprecated = gs("warn.deprecated"),
                                ...) {
  
  chkclass(x, "netcomparison")
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  chklogical(random)
  chklogical(backtransf)
  ##
  chknumeric(nchar.comps, min = 1, length = 1)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  ##
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  ##
  chklogical(legend)
  
  
  ##
  ## Abbreviated component and treatment labels
  ##
  n.comparisons <- length(x$treat1)
  treat1 <- rep("", n.comparisons)
  treat2 <- rep("", n.comparisons)
  ##
  if (common | random) {
    comps <- x$comps
    if (!is.null(x$inactive) && any(grepl(x$inactive, x$comparison)))
      comps <- c(comps, x$inactive)
    comps.abbr <- treats(comps, nchar.comps)
    ##
    for (i in seq_len(n.comparisons))
      treat1[i] <- compos(x$treat1[i], comps, comps.abbr,
                          x$x$sep.comps, x$add1[i] == " ")
    for (i in seq_len(n.comparisons))
      treat2[i] <- compos(x$treat2[i], comps, comps.abbr,
                          x$x$sep.comps, x$add2[i] == " ")
    ##
    comps1.list <- compsplit(x$treat1, x$x$sep.comps)
    comps2.list <- compsplit(x$treat2, x$x$sep.comps)
    ##
    comps1.list <- lapply(comps1.list, charfac,
                          levels = comps, labels = comps.abbr)
    comps2.list <- lapply(comps2.list, charfac,
                          levels = comps, labels = comps.abbr)
    ##
    comparison <- rep("", n.comparisons)
    ##
    for (i in seq_len(n.comparisons)) {
      sel1.i <- !comps1.list[[i]] %in% comps2.list[[i]]
      sel2.i <- !comps2.list[[i]] %in% comps1.list[[i]]
      ##
      if (any(sel1.i) | any(sel2.i)) {
        comb1 <- paste(comps1.list[[i]][sel1.i],
                       collapse = paste0(x$add1[i], x$x$sep.comps, x$add1[i]))
        comb2 <- paste(comps2.list[[i]][sel2.i],
                       collapse = paste0(x$add2[i], x$x$sep.comps, x$add2[i]))
        #
        if (comb1 == "")
          comparison[i] <- comb2
        else if (comb2 == "")
          comparison[i] <- comb1
        else
          comparison[i] <- paste0(comb1, x$x$sep.trts, comb2)
      }
    }
  }
  
  
  sm <- sm.lab <- x$x$sm
  ##
  relative <- is_relative_effect(sm) | sm == "VE"
  #
  if (!backtransf & relative)
    sm.lab <- paste0("log", if (sm == "VE") "VR" else sm)
  ##  
  ci.lab <- paste0(round(100 * x$level, 1), "% CI")
  
  
  if (common) {
    TE.common <- x$TE.common
    lower.common <- x$lower.common
    upper.common <- x$upper.common
    ##
    if (backtransf) {
      TE.common <- backtransf(TE.common, sm)
      lower.common <- backtransf(lower.common, sm)
      upper.common <- backtransf(upper.common, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- lower.common
        lower.common <- upper.common
        upper.common <- tmp.l
      }
    }
  }
  ##
  if (random) {
    TE.random <- x$TE.random
    lower.random <- x$lower.random
    upper.random <- x$upper.random
    ##
    if (backtransf) {
      TE.random <- backtransf(TE.random, sm)
      lower.random <- backtransf(lower.random, sm)
      upper.random <- backtransf(upper.random, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- lower.random
        lower.random <- upper.random
        upper.random <- tmp.l
      }
    }
  }
  
  
  if (common) {
    pval.f <- formatPT(x$pval.common, digits = digits.pval,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval,
                       lab.NA = "")
    ##
    res.c <-
      cbind(comparison = comparison,
            treat1 = treat1,
            treat2 = treat2,
            Comb = formatN(TE.common, digits = digits,
                           "NA", big.mark = big.mark),
            CI = formatCI(formatN(round(lower.common, digits),
                                  digits, "NA",
                                  big.mark = big.mark),
                          formatN(round(upper.common, digits),
                                  digits, "NA",
                                  big.mark = big.mark)),
            zval = formatN(x$statistic.common, digits = digits.stat,
                           "NA", big.mark = big.mark),
            pval = pval.f)
    ##
    dimnames(res.c) <- list(rep("", nrow(res.c)),
                            c("comparison", "treat1", "treat2",
                              sm.lab, ci.lab, "z", "p-value"))
    ##
    res.c[res.c[, "treat1"] == res.c[, "treat2"], 4:7] <- "."
    ##
    cat("Results for comparisons (additive CNMA model,",
        "common effects model):\n\n")
    ##
    prmatrix(res.c, quote = FALSE, right = TRUE, na.print = "--")
    ##
    if (random)
      cat("\n")
  }
  
  
  if (random) {
    ##
    pval.r <- formatPT(x$pval.random, digits = digits.pval,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval,
                       lab.NA = "")
    ##
    res.r <-
      cbind(comparison = comparison,
            treat1 = treat1,
            treat2 = treat2,
            Comb = formatN(TE.random, digits = digits,
                           "NA", big.mark = big.mark),
            CI = formatCI(formatN(round(lower.random, digits),
                                  digits, "NA",
                                  big.mark = big.mark),
                          formatN(round(upper.random, digits),
                                  digits, "NA",
                                  big.mark = big.mark)),
            zval = formatN(x$statistic.random, digits = digits.stat,
                           "NA", big.mark = big.mark),
            pval = pval.r)
    ##
    dimnames(res.r) <- list(rep("", nrow(res.r)),
                            c("comparison", "treat1", "treat2",
                              sm.lab, ci.lab, "z", "p-value"))
    ##
    res.r[res.r[, "treat1"] == res.r[, "treat2"], 4:7] <- "."
    ##
    cat("Results for comparisons (additive CNMA model,",
        "random effects model):\n\n")
    ##
    prmatrix(res.r, quote = FALSE, right = TRUE, na.print = "--")
  }
  
  
  if (legend && (common | random)) {
    diff.comps <- comps != comps.abbr
    any.comps <- any()
    ##
    if (any(diff.comps)) {
      cat("\nLegend:\n")
      ##
      tmat <- data.frame(comps.abbr, comps)
      tmat <- tmat[diff.comps, ]
      names(tmat) <- c("Abbreviation", " Component name")
      tmat <- tmat[order(tmat$Abbreviation), ]
      ##
      prmatrix(tmat, quote = FALSE, right = TRUE,
               rowlab = rep("", length(comps.abbr))) 
    }
  }
  
  invisible(NULL)
}

.onAttach <- function (libname, pkgname) {
  msg <-
    paste0("Loading 'netmeta' package (version ",
           packageDescription("netmeta")$Version,
           ").",
           "\nType 'help(\"netmeta-package\")' for a brief overview.")
  packageStartupMessage(msg)
}


.onLoad <- function(libname, pkgname) {
  #
  # Default settings for R package netmeta
  #
  argslist <- c("baseline.reference", "small.values",
                "all.treatments", "seq",
                "method.tau.netmeta",
                "drop.reference.group", "equal.size",
                "show",
                #
                "nsim", "lump.comparator",
                #
                "plastic", "col.netgraph",
                "number.of.studies", "thickness",
                "multiarm",
                #
                "tol.multiarm", "tol.multiarm.se",
                "details.chkmultiarm",
                #
                "na.unident",
                "sep.trts", "sep.comps", "sep.ia",
                "nchar.trts", "nchar.studlab",
                #
                "sort.distance",
                #
                "legend")
  #
  suppressWarnings(
    settings.meta(baseline.reference = TRUE, small.values = "desirable",
                  all.treatments = NULL, seq = NULL,
                  method.tau.netmeta = "DL",
                  drop.reference.group = TRUE, equal.size = TRUE,
                  show = "both",
                  #
                  nsim = 1000, lump.comparator = FALSE,
                  #
                  plastic = FALSE, col.netgraph = NULL,
                  number.of.studies = TRUE, thickness = "number.of.studies",
                  multiarm = FALSE,
                  #
                  tol.multiarm = 0.001, tol.multiarm.se = NULL,
                  details.chkmultiarm = FALSE,
                  #
                  na.unident = TRUE,
                  sep.trts = ":", sep.comps = "+", sep.ia = "*",
                  nchar.trts = 666, nchar.studlab = 666,
                  #
                  sort.distance = TRUE,
                  #
                  legend = TRUE,
                  #
                  .other_nma =
                    c("netmeta.crossnma", "netmeta.gemtc", "netmeta.multinma"),
                  #
                  .argslist.netmeta = argslist))
}


calcV <- function(x, sm) {
  p2 <- (x$event2[1] + x$incr2[1]) / (x$n2[1] + 2 * x$incr2[1])
  n2 <- x$n2[1] + 2 * x$incr2[1]
  ##
  if (sm == "OR")
    V <- matrix(1 / (x$event2[1] + x$incr2[1]) +
                1 / (x$n2[1] - x$event2[1] + x$incr2[1]),
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "RR")
    V <- matrix((1 - p2) / (n2 * p2),
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "RD")
    V <- matrix(p2 * (1 - p2) / n2,
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "ASD")
    V <- matrix(0.25 / x$n2[1],
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "MD")
    V <- matrix(x$sd2[1]^2 / x$n2[1],
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "SMD")
    V <- matrix(1 / x$n2[1],
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "IRR")
    V <- matrix(1 / (x$event2[1] + x$incr2[1]),
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "IRD")
    V <- matrix((x$event2[1] + x$incr2[1]) / x$time2[1]^2,
                nrow = nrow(x), ncol = nrow(x))
  else if (sm == "IRSD")
    V <- matrix(0.25 / x$time2[1],
                nrow = nrow(x), ncol = nrow(x))
  else if (nrow(x) == 1)
    V <- as.matrix(x$seTE^2)
  else
    V <- diag(x$seTE^2)
  #
  diag(V) <- x$seTE^2
  #
  V
}


##
## Abbreviated component labels
##
compos <- function(x, lev, abbr, split, add) {
  x.list <- compsplit(x, split = split)
  if (!missing(add))
    add <- ifelse(add, " ", "")
  else
    add <- ifelse(attr(x.list, "withspace"), " ", "")
  ##
  x.list <- lapply(x.list, charfac, levels = lev, labels = abbr)
  x.list <- lapply(x.list, paste, collapse = paste0(add, split, add))
  ##
  unlist(x.list)
}


charfac <- function(x, ...)
  as.character(factor(x, ...))


legendabbr <- function(full, abbr, condition,
                       text = "Treatment name",
                       header = "\nLegend:\n") {
  diff <- full != abbr
  if (condition) {
    if (any(diff)) {
      tmat <- data.frame(abbr, full)
      names(tmat) <- c("Abbreviation", text)
      tmat <- tmat[diff, ]
      tmat <- tmat[order(tmat$Abbreviation), ]
      ##
      cat(header)
      ##
      prmatrix(unique(tmat), quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(unique(tmat))))
    }
  }
  invisible(condition & any(diff))
}


setsv <- function(x, add = NULL) {
  if (is.null(x))
    res <- "desirable"
  else {
    res <- setchar(x, c("good", "bad"), stop.at.error = FALSE)
    ##
    if (!is.null(res))
      res <- switch(res, good = "desirable", bad = "undesirable")
    else
      res <- x
  }
  ##
  setchar(res, c("desirable", "undesirable", add))
}

first <- function(x) x[1]
second <- function(x) x[2]


# For forest plots
#
setHet <- function(meta, netmeta) {
  #
  meta$method.tau <- netmeta$method.tau
  meta$tau2 <- netmeta$tau2
  meta$se.tau2 <- NA
  meta$lower.tau2 <- NA
  meta$upper.tau2 <- NA
  #
  meta$tau <- netmeta$tau
  meta$lower.tau <- NA
  meta$upper.tau <- NA
  #
  meta$method.tau.ci <- ""
  meta$sign.lower.tau <- ""
  meta$sign.upper.tau <- ""
  #
  meta$Q <- netmeta$Q
  meta$df.Q <- netmeta$df.Q
  meta$pval.Q <- netmeta$pval.Q
  #
  meta$method.I2 <- "Q"
  meta$I2 <- netmeta$I2
  meta$lower.I2 <- netmeta$.lower.I2
  meta$upper.I2 <- netmeta$.upper.I2
  #
  meta$H <- netmeta$H
  meta$lower.H <- netmeta$.lower.H
  meta$upper.H <- netmeta$.upper.H
  #
  meta$tau2.resid <- NA
  meta$se.tau2.resid <- NA
  meta$lower.tau2.resid <- NA
  meta$upper.tau2.resid <- NA
  #
  meta$tau.resid <- NA
  meta$lower.tau.resid <- NA
  meta$upper.tau.resid <- NA
  #
  meta$Q.resid <- netmeta$Q.resid
  meta$df.Q.resid <- netmeta$df.Q.resid
  meta$pval.Q.resid <- netmeta$pval.Q.resid
  #
  meta$H.resid <- netmeta$H.resid
  meta$lower.H.resid <- netmeta$.lower.H.resid
  meta$upper.H.resid <- netmeta$.upper.H.resid
  #
  meta$I2.resid <- netmeta$I2.resid
  meta$lower.I2.resid <- netmeta$.lower.I2.resid
  meta$upper.I2.resid <- netmeta$.upper.I2.resid
  #
  meta
}


is_identical <- function(x) {
  duplicated(as.data.frame(t(x))) |
    duplicated(as.data.frame(t(x)), fromLast = TRUE)
}

#' Get information on network connectivity (number of subnetworks,
#' distance matrix)
#' 
#' @description
#' To determine the network structure and to test whether a given
#' network is fully connected. Network information is provided as a
#' triple of vectors \code{treat1}, \code{treat2}, and \code{studlab}
#' where each row corresponds to an existing pairwise treatment
#' comparison (\code{treat1}, \code{treat2}) in a study
#' (\code{studlab}). The function calculates the number of subnetworks
#' (connectivity components; value of 1 corresponds to a fully
#' connected network) and the distance matrix (in block-diagonal form
#' in the case of subnetworks). If some treatments are combinations of
#' other treatments or have common components, an analysis based on
#' the additive network meta-analysis model might be possible, see
#' \link{discomb} function.
#' 
#' @aliases netconnection netconnection.default netconnection.pairwise print.netconnection
#' 
#' @param data A data frame, e.g., created with
#'   \code{\link[meta]{pairwise}}.
#' @param treat1 Label / number for first treatment (ignored if
#'   \code{data} was created with \code{\link[meta]{pairwise}}).
#' @param treat2 Label / number for second treatment (ignored if
#'   \code{data} was created with \code{\link[meta]{pairwise}}).
#' @param studlab Study labels (ignored if \code{data} was created
#'   with \code{\link[meta]{pairwise}}).
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param title Title of meta-analysis / systematic review.
#' @param sep.trts A character used in comparison names as separator
#'   between treatment labels.
#' @param drop.NA A logical indicating whether comparisons with missing
#'   treatment estimates or standard errors should be considered.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param warn A logical indicating whether warnings should be
#'   printed.
#' @param x An object of class \code{netconnection}.
#' @param digits Minimal number of significant digits, see
#'   \code{\link{print.default}}.
#' @param details A logical indicating whether to print the distance
#'   matrix.
#' @param details.disconnected A logical indicating whether to print
#'   more details for disconnected networks.
#' @param \dots Additional arguments (ignored at the moment)
#' 
#' @return
#' An object of class \code{netconnection} with corresponding
#' \code{print} function. The object is a list containing the
#' following components:
#' \item{treat1, treat2, studlab, title, warn, nchar.trts}{As defined
#'   above.}
#' \item{k}{Total number of studies.}
#' \item{m}{Total number of pairwise comparisons.}
#' \item{n}{Total number of treatments.}
#' \item{n.subnets}{Number of subnetworks; equal to 1 for a fully
#'   connected network.}
#' \item{D.matrix}{Distance matrix.}
#' \item{A.matrix}{Adjacency matrix.}
#' \item{L.matrix}{Laplace matrix.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package netmeta used to create object.}
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netdistance}},
#'   \code{\link{discomb}}
#' 
#' @examples
#' \donttest{
#' # Transform data from long arm-based to contrast-based format
#' #
#' pw <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' 
#' nc1 <- netconnection(pw)
#' nc1
#' 
#' # Extract number of (sub)networks
#' #
#' nc1$n.subnets
#' 
#' # Extract distance matrix
#' #
#' nc1$D.matrix
#' 
#' # Artificial example with two subnetworks
#' #
#' t1 <- c("G", "B", "B", "D", "A", "F")
#' t2 <- c("B", "C", "E", "E", "H", "A")
#' #
#' nc2 <- netconnection(t1, t2)
#' print(nc2, details = TRUE)
#' 
#' # Number of subnetworks
#' #
#' nc2$n.subnets
#' 
#' # Extract distance matrix
#' #
#' nc2$D.matrix
#' 
#' # Conduct network meta-analysis (results in an error message due to
#' # unconnected network)
#' try(nma <- netmeta(1:6, 1:6, t1, t2, 1:6))
#' 
#' # Conduct network meta-analysis on first subnetwork
#' #
#' nma.1 <- netmeta(1:6, 1:6, t1, t2, 1:6, subset = nc2$subnet == 1)
#' 
#' # Conduct network meta-analysis on second subnetwork
#' #
#' nma.2 <- netmeta(1:6, 1:6, t1, t2, 1:6, subset = nc2$subnet == 2)
#' 
#' nma.1
#' nma.2
#' }
#' 
#' @rdname netconnection
#' @method netconnection default
#' @export

netconnection.default <- function(data = NULL, treat1, treat2, studlab = NULL,
                                  subset = NULL,
                                  sep.trts = ":",
                                  nchar.trts = 666,
                                  title = "", details.disconnected = FALSE,
                                  warn = FALSE, ...) {
  
  ##
  ##
  ## (1) Check arguments
  ##
  ##
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  ## Catch treat1
  ##
  if (!nulldata & !is.data.frame(data)) {
    if (!missing(treat1) & !missing(treat2) & !missing(studlab))
      stop("Argument 'data' must be a data frame.")
    else if (!missing(treat1) & !missing(treat2)) {
      treat1 <- catch("data", mc, sfsp, sfsp)
      treat2 <- catch("treat1", mc, sfsp, sfsp)
      studlab <- catch("treat2", mc, sfsp, sfsp)
    }
    else if (!missing(treat1)) {
      treat1 <- catch("data", mc, sfsp, sfsp)
      treat2 <- catch("treat1", mc, sfsp, sfsp)
    }
  }
  else {
    ##
    if (nulldata)
      data <- sfsp
    ##
    treat1 <- catch("treat1", mc, data, sfsp)
    treat2 <- catch("treat2", mc, data, sfsp)
    studlab <- catch("studlab", mc, data, sfsp)
    subset <- catch("subset", mc, data, sfsp)
  }
  ##
  if (length(studlab) != 0)
    studlab <- as.character(studlab)
  else {
    if (warn)
      warning("No information given for argument 'studlab'. ",
              "Assuming that comparisons are from independent studies.")
    studlab <- as.character(seq_along(treat1))
  }
  ##
  chknumeric(nchar.trts, min = 1, length = 1)
  ##
  chklogical(details.disconnected)
  chklogical(warn)
  
  
  ##
  ##
  ## (2) Check length of essential variables
  ##
  ##
  
  fun <- "netconnection"
  ##
  k.All <- length(treat1)
  ##
  missing.subset <- is.null(subset)
  ##
  chklength(treat2, k.All, fun,
                   text = paste0("Arguments 'treat1' and 'treat2' ",
                                 "must have the same length."))
  chklength(studlab, k.All, fun,
                   text = paste0("Arguments 'treat1' and 'studlab' ",
                                 "must have the same length."))
  ##
  if (is.factor(treat1))
    treat1 <- as.character(treat1)
  if (is.factor(treat2))
    treat2 <- as.character(treat2)
  
  
  ##
  ##
  ## (3) Use subset for analysis
  ##
  ##
  
  if (!missing.subset) {
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of subset is larger than number of studies.")
    ##
    treat1 <- treat1[subset]
    treat2 <- treat2[subset]
    studlab <- studlab[subset]
  }
  
  
  ##
  ##
  ## (4) Additional checks
  ##
  ##
  
  if (any(treat1 == treat2))
    stop("Treatments must be different (arguments 'treat1' and 'treat2').")
  ##
  ## Check for correct number of comparisons
  ##
  tabnarms <- table(studlab)
  sel.narms <- !is_wholenumber((1 + sqrt(8 * tabnarms + 1)) / 2)
  ##
  if (sum(sel.narms) == 1)
    stop("Study '", names(tabnarms)[sel.narms],
         "' has a wrong number of comparisons.",
         "\n  Please provide data for all treatment comparisons ",
         "(two-arm: 1; three-arm: 3; four-arm: 6, ...).")
  if (sum(sel.narms) > 1)
    stop("The following studies have a wrong number of comparisons: ",
         paste(paste0("'", names(tabnarms)[sel.narms], "'"),
               collapse = ", "),
         "\n  Please provide data for all treatment comparisons ",
         "(two-arm: 1; three-arm: 3; four-arm: 6, ...).")
  ##
  labels <- sort(unique(c(treat1, treat2)))
  ##
  if (compmatch(labels, sep.trts)) {
    if (!missing(sep.trts))
      warning("Separator '", sep.trts,
              "' used in at least one treatment label. ",
              "Try to use predefined separators: ",
              "':', '-', '_', '/', '+', '.', '|', '*'.",
              call. = FALSE)
    ##
    if (!compmatch(labels, ":"))
      sep.trts <- ":"
    else if (!compmatch(labels, "-"))
      sep.trts <- "-"
    else if (!compmatch(labels, "_"))
      sep.trts <- "_"
    else if (!compmatch(labels, "/"))
      sep.trts <- "/"
    else if (!compmatch(labels, "+"))
      sep.trts <- "+"
    else if (!compmatch(labels, "."))
      sep.trts <- "-"
    else if (!compmatch(labels, "|"))
      sep.trts <- "|"
    else if (!compmatch(labels, "*"))
      sep.trts <- "*"
    else
      stop("All predefined separators (':', '-', '_', '/', '+', ",
           "'.', '|', '*') are used in at least one treatment label.",
           "\n   Please specify a different character that should be ",
           "used as separator (argument 'sep.trts').",
           call. = FALSE)
  }
  
  
  ##
  ##
  ## (5) Determine (sub)network(s)
  ##
  ##
  
  treats <- as.factor(c(as.character(treat1), as.character(treat2)))
  trts <- levels(treats)
  ##
  n <- length(trts)   # Number of treatments
  m <- length(treat1) # Number of comparisons
  ##
  ## Edge-vertex incidence matrix
  ##
  treat1.pos <- treats[1:m]
  treat2.pos <- treats[(m + 1):(2 * m)]
  B <- createB(treat1.pos, treat2.pos, ncol = n)
  ##
  rownames(B) <- studlab
  colnames(B) <- trts
  ##
  L.mult <- t(B) %*% B             # Laplacian matrix with multiplicity
  A <- diag(diag(L.mult)) - L.mult # Adjacency matrix
  D <- netdistance(A)              # Distance matrix
  L <- diag(rowSums(A)) - A        # Laplacian matrix without multiplicity
  ##
  n.subsets <- as.integer(table(round(eigen(L)$values, 10) == 0)[2])
  ##
  ## Block diagonal matrix in case of sub-networks
  ##
  maxdist <- nrow(D)
  D2 <- D
  D2[is.infinite(D2)] <- maxdist
  o <- hclust(dist(D2))$order
  ##
  D <- D[o, o]
  A <- A[o, o]
  L <- L[o, o]
  ##
  A.loop <- A
  D.loop <- D
  id.treats <- character(0)
  id.subnets <- numeric(0)
  more.subnets <- TRUE
  subnet.i <- 0
  dat.subnet <- data.frame()
  ##
  while (more.subnets) {
    subnet.i <- subnet.i + 1
    n.i <- seq_len(nrow(D.loop))
    #
    next.subnet <- min(max(n.i) + 1, n.i[is.infinite(D.loop[, 1])])
    sel.i <- seq_len(next.subnet - 1)
    #
    id.treats <- c(id.treats, rownames(D.loop)[sel.i])
    id.subnets <- c(id.subnets, rep(subnet.i, length(sel.i)))
    ##
    A.i <- A.loop[sel.i, sel.i]
    A.i <- A.i[order(rownames(A.i)), order(colnames(A.i))]
    ##
    for (row.i in 1:(ncol(A.i) - 1)) {
      for (col.i in 2:ncol(A.i)) {
        if (col.i > row.i) {
          if (A.i[row.i, col.i] > 0) {
            trt1.i <- rownames(A.i)[row.i]
            trt2.i <- rownames(A.i)[col.i]
            #
            dat.subnet <-
              rbind(
                dat.subnet,
                data.frame(subnet = subnet.i,
                           comparison = paste(trt1.i, trt2.i, sep = sep.trts),
                           treat1 = trt1.i, treat2 = trt2.i))
          }
        }
      }
    }
    #
    A.loop <- A.loop[-sel.i, -sel.i]
    D.loop <- D.loop[-sel.i, -sel.i]
    #
    more.subnets <- nrow(D.loop) > 0
  }
  #
  # Order matrices within subnets by treatments
  # (data frame 'dat.subnet' is already sorted by subnetwork and treatments)
  #
  seq <- NULL
  for (i in unique(dat.subnet$subnet)) {
    dat.i <- dat.subnet[dat.subnet$subnet == i, ]
    seq <- c(seq, unique(c(dat.i$treat1, dat.i$treat2)))
  }
  #
  A <- A[seq, seq]
  D <- D[seq, seq]
  L <- L[seq, seq]
  #
  # Add subnetwork number to comparisons
  #
  subnet <- rep(NA, length(treat1))
  #
  for (i in seq_along(id.treats))
    subnet[treat1 == id.treats[i]] <- id.subnets[i]
  ##
  comparisons <- dat.subnet$comparison
  subnet.comparisons <- dat.subnet$subnet
  #
  designs <- designs(treat1, treat2, studlab)
  
  res <- list(treat1 = treat1,
              treat2 = treat2,
              studlab = studlab,
              design = designs$design,
              subnet = subnet,
              #
              k = length(unique(studlab)),
              m = m,
              n = n,
              n.subnets = n.subsets,
              d = length(unique(designs$design)),
              #
              seq = seq,
              #
              D.matrix = D,
              A.matrix = A,
              L.matrix = L,
              ##
              designs = unique(sort(designs$design)),
              comparisons = comparisons,
              subnet.comparisons = subnet.comparisons,
              ##
              nchar.trts = nchar.trts,
              ##
              title = title,
              ##
              details.disconnected = details.disconnected,
              ##
              warn = warn,
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )

  class(res) <- "netconnection"

  res
}


#' @rdname netconnection
#' @method netconnection pairwise
#' @export

netconnection.pairwise <- function(data,
                                   treat1, treat2, studlab = NULL,
                                   subset = NULL,
                                   sep.trts = ":",
                                   drop.NA = TRUE,
                                   nchar.trts = 666,
                                   title = "", details.disconnected = FALSE,
                                   warn = FALSE,
                                   ...) {
  
  #
  #
  # (1) Check arguments
  #
  #
  
  chkclass(data, "pairwise")
  #
  chklogical(drop.NA)
  chklogical(warn)
  #
  # Arguments 'treat1', 'treat2' and 'studlab' ignored
  #
  if (warn) {
    if (!missing(treat1))
      warning("Argument 'treat1' ignored as argument 'data' is an ",
              "object created with pairwise().",
              call. = FALSE)
    #
    if (!missing(treat2))
      warning("Argument 'treat2' ignored as argument 'data' is an ",
              "object created with pairwise().",
              call. = FALSE)
    #
    if (!missing(studlab))
      warning("Argument 'studlab' ignored as argument 'data' is an ",
              "object created with pairwise().",
              call. = FALSE)
  }
  #
  treat1 <- data$treat1
  treat2 <- data$treat2
  studlab <- data$studlab
  #
  if (is.factor(treat1))
    treat1 <- as.character(treat1)
  if (is.factor(treat2))
    treat2 <- as.character(treat2)
  #
  missing.subset <- missing(subset)
  #
  if (!missing.subset) {
    sfsp <- sys.frame(sys.parent())
    mc <- match.call()
    subset <- catch("subset", mc, data, sfsp)
    #
    k.All <- length(treat1)
    if  (length(subset) > k.All)
      stop("Length of subset is larger than number of studies.")
    #
    #if ((is.logical(subset) & (sum(subset) > k.All)) ||
    #    (length(subset) > k.All))
    #  stop("Length of subset is larger than number of studies.")
    #
    if (is.numeric(subset)) {
      if (any(is.na(subset)))
        stop("No missing values allowed in argument 'subset'.")
      if (length(subset) != length(unique(subset)))
        stop("Duplicate values in argument 'subset'.")
      if (any(subset > k.All | subset <= 0))
        stop("Numerical values in argument 'subset' must be between 1 and ",
             k.All, ".")
      #
      subset1 <- rep_len(FALSE, k.All)
      subset1[subset] <- TRUE
      subset <- subset1
    }
  }
  else
    subset <- rep_len(TRUE, length(treat1))
  #
  if (drop.NA) {
    if (is.null(attr(data, "varnames"))) {
      TE <- data$TE
      seTE <- data$seTE
    }
    else {
      TE <- data[[attr(data, "varnames")[1]]]
      seTE <- data[[attr(data, "varnames")[2]]]
    }
    #
    subset <- subset & (!is.na(TE) & !(is.na(seTE) | seTE == 0))
  }
  #
  treat1 <- treat1[subset]
  treat2 <- treat2[subset]
  studlab <- studlab[subset]
  #
  chknumeric(nchar.trts, min = 1, length = 1)
  #
  chklogical(details.disconnected)
  
  
  #
  #
  # (2) Call netconnection.default()
  #
  #
  
  res <- netconnection(treat1 = treat1, treat2 = treat2,
                       studlab = studlab,
                       sep.trts = sep.trts, nchar.trts = nchar.trts,
                       title = title,
                       details.disconnected = details.disconnected,
                       warn = warn,
                       ...)
  #
  res
}


#' @rdname netconnection
#' @method netconnection netmeta
#' @export

netconnection.netmeta <- function(data,
                                  sep.trts = data$sep.trts,
                                  nchar.trts = data$nchar.trts,
                                  title = data$title,
                                  details.disconnected = FALSE,
                                  warn = FALSE, ...) {
  
  chkclass(data, "netmeta")
    
  res <- netconnection(treat1 = data$treat1, treat2 = data$treat2,
                       studlab = data$studlab,
                       sep.trts = sep.trts, nchar.trts = nchar.trts,
                       title = title)
  #
  res
}


#' @rdname netconnection
#' @method netconnection netcomb
#' @export

netconnection.netcomb <- function(data,
                                  sep.trts = data$sep.trts,
                                  nchar.trts,
                                  title = data$title,
                                  details.disconnected = FALSE,
                                  warn = FALSE, ...) {
  
  chkclass(data, "netcomb")
  
  if (inherits(data, "discomb")) {
    if (missing(nchar.trts))
      nchar.trts <- data$nchar.comps
    #
    return(netconnection(treat1 = data$treat1, treat2 = data$treat2,
                         studlab = data$studlab,
                         sep.trts = sep.trts, nchar.trts = nchar.trts,
                         title = title))
  }
  else {
    if (missing(nchar.trts))
      nchar.trts <- data$nchar.trts
    #
    return(netconnection(treat1 = data$treat1, treat2 = data$treat2,
                         studlab = data$studlab,
                         sep.trts = sep.trts, nchar.trts = nchar.trts,
                         title = title))
  }
}


#' @rdname netconnection
#' @method print netconnection
#' @export

print.netconnection <- function(x,
                                digits = max(4, .Options$digits - 3),
                                nchar.trts = x$nchar.trts,
                                details = FALSE,
                                details.disconnected = x$details.disconnected,
                                ...) {
  
  chkclass(x, "netconnection")
  ##
  if (is.null(nchar.trts))
    nchar.trts <- 666
  
  
  chknumeric(digits, length = 1)
  chknumeric(nchar.trts, min = 1, length = 1)
  chklogical(details)
  details.disconnected <- replaceNULL(details.disconnected, FALSE)
  chklogical(details.disconnected)
  
  
  matitle(x)
  ##
  cat("Number of studies: k = ", x$k, "\n", sep = "")
  cat("Number of pairwise comparisons: m = ", x$m, "\n", sep = "")
  cat("Number of treatments: n = ", x$n, "\n", sep = "")
  if (!is.null(x$d))
    cat("Number of designs: d = ", x$d, "\n", sep = "")
  ##
  cat("Number of networks: ", x$n.subnets, "\n", sep = "")
  ##
  if (x$n.subnets > 1) {
    f <- function(x) length(unique(x))
    d <- as.data.frame(x)
    k.subset <- tapply(d$studlab, d$subnet, f)
    ##
    m <- as.matrix(
      data.frame(subnetwork = names(k.subset),
                 k = as.vector(k.subset),
                 m = as.vector(tapply(d$studlab, d$subnet, length)),
                 n = as.vector(tapply(c(d$treat1, d$treat2),
                                      c(d$subnet, d$subnet), f)))
    )
    rownames(m) <- rep("", nrow(m))
    ##
    cat("\nDetails on subnetworks: \n")
    prmatrix(m, quote = FALSE, right = TRUE)
    ##
    if (details.disconnected) {
      cat("\n")
      for (i in seq_len(x$n.subnets)) {
        d.i <- subset(d, d$subnet == i)
        cat("Subnetwork ", i, ":\n", sep = "")
        print(sort(unique(c(d.i$treat1, d.i$treat2))))
      }
    }
  }
  
  
  if (details) {
    cat("\nDistance matrix:\n")
    
    D <- round(x$D.matrix, digits = digits)
    D[is.infinite(D)] <- "."
    ##
    if (x$n.subnets == 1)
      diag(D) <- "."
    ##
    rownames(D) <- treats(rownames(D), nchar.trts)
    colnames(D) <- treats(colnames(D), nchar.trts)
    ##
    prmatrix(D, quote = FALSE, right = TRUE)
    
    diff.rownames <- rownames(x$D.matrix) != rownames(D)
    if (any(diff.rownames)) {
      abbr <- rownames(D)
      full <- rownames(x$D.matrix)
      ##
      tmat <- data.frame(abbr, full)
      names(tmat) <- c("Abbreviation", "Treatment name")
      tmat <- tmat[diff.rownames, ]
      tmat <- tmat[order(tmat$Abbreviation), ]
      ##
      cat("\nLegend:\n")
      prmatrix(tmat, quote = FALSE, right = TRUE,
               rowlab = rep("", length(abbr)))
    }
  }
  
  invisible(NULL)
}


#' @rdname netconnection
#' @export

netconnection <- function(data, ...)
  UseMethod("netconnection")

bySummary <- function(x, index1, index2,
                      data = NULL, subset = NULL,
                      na.rm = TRUE, ties.method = "random", sep,
                      long = TRUE) {
  
  
  ##
  ##
  ## (1) Check arguments
  ##
  ##
  chklogical(na.rm)
  ##
  ties.method <- setchar(ties.method, c("first", "last", "random"))
  ##
  chklogical(long)
  ##
  nulldata <- is.null(data)
  ##
  if (!nulldata & inherits(data, "netmeta")) {
    data <- data$data
    if (is.null(subset) & !is.null(data$.subset))
      subset <- data$.subset
  }
  
  
  ##
  ##
  ## (2) Read data
  ##
  ##
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  if (nulldata)
    data <- sfsp
  ##
  ## Catch 'x', 'index1', and 'index2' from data:
  ##
  x <- catch("x", mc, data, sfsp)
  chknull(x)
  k.All <- length(x)
  ##
  index1 <- catch("index1", mc, data, sfsp)
  chknull(index1)
  ##
  missing.index2 <- missing(index2)
  if (!missing.index2) {
    index2 <- catch("index2", mc, data, sfsp)
    chknull(index2)
  }
  ##
  ## Catch 'subset' from data:
  ##
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  ##
  if (missing(sep))
    sep <- ":"
  
  
  ##
  ##
  ## (3) Check length of essential variables
  ##
  ##
  fun <- "calcSummary"
  ##
  chklength(index1, k.All, fun)
  if (!missing.index2)
    chklength(index2, k.All, fun)
  
  
  ##
  ##
  ## (4) Subset
  ##
  ##
  if (!missing.subset)
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of argument 'subset' is larger than number of observations.")
  
  
  ##
  ##
  ## (5) Use subset for analysis
  ##
  ##
  if (!missing.subset) {
    x <- x[subset]
    index1 <- index1[subset]
    if (!missing.index2)
      index2 <- index2[subset]
  }
  ##
  ## Determine total number of studies
  ##
  k.all <- length(x)
  ##
  if (k.all == 0)
    stop("No observations in dataset / subset.")
  
  
  ##
  ##
  ## (6) Determine summaries
  ##
  ##
  if (missing(index2))
    indices <- index1
  else
    indices <- paste(index1, index2, sep = sep)
  ##
  sum    <- by(x, indices, sum, na.rm = na.rm)
  min    <- by(x, indices, min, na.rm = na.rm)
  max    <- by(x, indices, max, na.rm = na.rm)
  mean   <- by(x, indices, mean, na.rm = na.rm)
  median <- by(x, indices, median, na.rm = na.rm)
  ##
  ## Mode
  ##
  tab.x <- table(indices, x)
  max.x <- max.col(tab.x, ties.method = ties.method)
  mode <- colnames(tab.x)[max.x]
  if (is.numeric(x))
    mode <- as.numeric(mode)
  ##
  indices.x <- rownames(sum)
  
  
  ##
  ##
  ## (7) Merge summaries with original data / subset
  ##
  ##
  tdat <- data.frame(indices = indices.x,
                     sum = as.vector(sum),
                     min = as.vector(min),
                     max = as.vector(max),
                     mean = as.vector(mean),
                     median = as.vector(median),
                     mode = mode,
                     stringsAsFactors = FALSE)
  ##
  if (long) {
    res <- data.frame(index1, index2 = index1,
                      indices, x, order = seq(along = index1),
                      stringsAsFactors = FALSE)
    ##
    if (missing(index2))
      res$index2 <- NULL
    else
      res$index2 <- index2
    ##
    res <- merge(res, tdat, by = "indices")
    ##
    res <- res[order(res$order), ]
    ##
    res$indices <- NULL
    ## res$order <- NULL
  }
  else
    res <- tdat
  
  
  res
}

#' Calculate rankogram
#'
#' @description
#' This function calculates the probabilities of each treatment being
#' at each possible rank and the SUCRAs (Surface Under the Cumulative
#' RAnking curve) in frequentist network meta-analysis.
#'
#' @param x An object of class \code{\link{netmeta}}.
#' @param nsim Number of repetitions.
#' @param common A logical indicating to compute ranking probabilities
#'   and SUCRAs for the common effects model.
#' @param random A logical indicating to compute ranking probabilities
#'   and SUCRAs for the random effects model.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect, can be abbreviated.
#' @param cumulative.rankprob A logical indicating whether cumulative
#'   ranking probabilities should be printed.
#' @param keep.samples A logical indicating whether to keep the generated
#'   samples.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (ignored).
#'
#' @details
#' We derive a matrix showing the probability of each treatment being
#' at each possible rank. To this aim, we use resampling from a
#' multivariate normal distribution with estimated network effects as
#' means and corresponding estimated variance covariance matrix. We
#' then summarise them using the ranking metric SUCRAs (Surface Under
#' the Cumulative RAnking curve).
#'
#' @return
#' An object of class \code{rankogram} with corresponding \code{print}
#' and \code{plot} function. The object is a list containing the
#' following components:
#' \item{ranking.matrix.common}{Numeric matrix giving the probability
#'   of each treatment being at each possible rank for the common
#'   effects model.}
#' \item{ranking.common}{SUCRA values for the common effects model.}
#' \item{ranking.matrix.random}{Numeric matrix giving the probability
#'   of each treatment being at each possible rank for the random
#'   effects model.}
#' \item{ranking.random}{SUCRA values for the random effects model.}
#' \item{cumrank.matrix.common}{Numeric matrix giving the cumulative
#'   ranking probability of each treatment for the
#'   common effects model.}
#' \item{cumrank.matrix.random}{Numeric matrix giving the cumulative
#'   ranking probability of each treatment for the random effects
#'   model.}
#' \item{nsim, common, random}{As defined above},
#' \item{small.values, x}{As defined above},
#'
#' @author Theodoros Papakonstantinou \email{dev@@tpapak.com}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{netmeta}}, \code{\link{netrank}},
#'   \code{\link{plot.rankogram}}, \code{\link[metabook]{Woods2010}}
#'
#' @references
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#'
#' @examples
#' pw <- pairwise(treatment, event = r, n = N, studlab = author,
#'   data = Woods2010, sm = "OR")
#' nma <- netmeta(pw, small.values = "desirable")
#'
#' set.seed(1909) # get reproducible results
#' rnk <- rankogram(nma, nsim = 100)
#' rnk
#' print(rnk, cumulative.rankprob = TRUE)
#'
#' # Also print mean ranks
#' summary(rnk)
#'
#' plot(rnk)
#'
#' @rdname rankogram.netmeta
#' @method rankogram netmeta
#' @export

rankogram.netmeta <- function(x, nsim = gs("nsim"),
                              common = x$common, random = x$random,
                              small.values = x$small.values,
                              cumulative.rankprob = FALSE,
                              keep.samples = FALSE,
                              nchar.trts = x$nchar.trts,
                              warn.deprecated = gs("warn.deprecated"),
                              ...) {
  
  #
  #
  # (1) Check for netmeta object and upgrade object
  #
  #
  
  chkclass(x, "netmeta")
  chksuitable(x, "Rankograms", "netmetabin")
  x <- updateversion(x)
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  
  chknumeric(nsim, min = 1, length = 1)
  #
  small.values <- setsv(small.values)
  #
  chklogical(cumulative.rankprob)
  chklogical(keep.samples)
  #
  if (is.null(nchar.trts))
    nchar.trts <- 666
  chknumeric(nchar.trts, length = 1)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                      warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  #
  # Additional checks for crossnma, gemtc and multinma objects
  # (and return results)
  #
  if (inherits(x, gs(".other_nma"))) {
    if (!x$keep.samples)
      stop("Input for argument 'x' is a netmeta.", x$method,
           " object without samples; recreate ", x$method,
           " object with argument 'keep.samples = TRUE'.",
           call. = FALSE)
    #
    if (common != x$common)
      warning("Argument 'common = ", x$common, "' as netmeta.", x$method,
              " object is based on ", if (x$common) "common" else "random",
              " effects model.",
              call. = FALSE)
    #
    if (random != x$random)
      warning("Argument 'random = ", x$random, "' as netmeta.", x$method,
              " object is based on ", if (x$random) "random" else "common",
              " effects model.",
              call. = FALSE)
    #
    #
    if (!missing(nsim) & x$keep.samples)
      warning("Argument 'nsim' ignored for netmeta.", x$method, " object.",
              call. = FALSE)
    #
    common <- x$common
    random <- x$random
    #
    return(rankogram(x$samples$d,
                     pooled = if (common) "common" else "random",
                     small.values = small.values,
                     cumulative.rankprob = cumulative.rankprob,
                     keep.samples = keep.samples,
                     nchar.trts = nchar.trts))
  }
  
  
  #
  #
  # (3) Resampling to calculate ranking probabilities and SUCRAs
  #
  #
  
  sucras.common  <- ranking.matrix.common  <- cumrank.matrix.common <-
    meanranks.common <- medianranks.common <- NULL
  #
  sucras.random <- ranking.matrix.random <- cumrank.matrix.random <-
    meanranks.random <- medianranks.random <- NULL
  #
  if (common) {
    res.c <- ranksampling(x, nsim, "common", small.values, keep.samples)
    #
    sucras.common <- res.c$sucras
    ranking.matrix.common <- res.c$rankogram
    cumrank.matrix.common <- res.c$cumrank
    #
    meanranks.common <- res.c$meanranks
    medianranks.common <- res.c$medianranks
    #
    samples.common <- res.c$samples
  }
  #
  if (random) {
    res.r <- ranksampling(x, nsim, "random", small.values, keep.samples)
    #
    sucras.random <- res.r$sucras
    ranking.matrix.random <- res.r$rankogram
    cumrank.matrix.random <- res.r$cumrank
    #
    meanranks.random <- res.r$meanranks
    medianranks.random <- res.r$medianranks
    #
    samples.random <- res.r$samples
  }
  
  
  #
  #
  # (4) Create rankogram object
  #
  #
  
  res <- list(ranking.common = sucras.common,
              ranking.matrix.common = ranking.matrix.common,
              cumrank.matrix.common = cumrank.matrix.common,
              #
              meanranks.common = meanranks.common,
              medianranks.common = medianranks.common,
              #
              samples.common =
                if (common & keep.samples) samples.common else NULL,
              #
              ranking.random = sucras.random,
              ranking.matrix.random = ranking.matrix.random,
              cumrank.matrix.random = cumrank.matrix.random,
              #
              meanranks.random = meanranks.random,
              medianranks.random = medianranks.random,
              #
              samples.random =
                if (random & keep.samples) samples.random else NULL,
              #
              nsim = nsim,
              #
              common = common,
              random = random,
              small.values = small.values,
              cumulative.rankprob = cumulative.rankprob,
              #
              nchar.trts = nchar.trts,
              x = x,
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )
  #
  # Backward compatibility
  #
  res$fixed <- common
  #
  res$ranking.fixed <- sucras.common
  res$ranking.matrix.fixed <- ranking.matrix.common
  res$cumrank.matrix.fixed <- cumrank.matrix.common
  #
  class(res) <- "rankogram"
  
  res
}

textmeth <- function(x, random = FALSE, print.tau2 = FALSE, print.tau = FALSE,
                     text.tau2 = gs("text.tau2"), text.tau = gs("text.tau"),
                     digits.tau2 = gs("digits.tau2"),
                     digits.tau = gs("digits.tau"),
                     print.I2 = FALSE, text.I2 = gs("text.I2"),
                     big.mark = gs("big.mark"), forest = FALSE) {
  text.details <- ""
  #
  if (print.tau2 & print.tau)
    print.tau <- FALSE
  #
  if (forest) {
    text.tau2 <- "tau^2"
    text.tau <- "tau"
    #
    text.I2 <- "I^2"
  }
  #
  if (inherits(x, c("netmeta", "netcomb"))) {
    if (inherits(x, "netmetabin")) {
      if (x$method == "MH") {
        text.details <- "- Mantel-Haenszel method\n"
        #
        if (x$cc.pooled & x$incr != 0)
          text.details <-
            paste0(text.details,
                   paste("- Continuity correction of", x$incr),
                   "\n")
      }
      else if (x$method == "NCH")
        text.details <-
          "- Based on the non-central hypergeometric distribution"
      else if (x$method == "LRP") {
        text.details <- "- Penalised logistic regression model"
        #
        if (random)
          text.details <-
            paste0(text.details, " (sqrt(phi) = ", round(sqrt(x$phi), 4), ")",
                   "\n")
      }
    }
    else if (inherits(x, "netmeta.crossnma")) {
      text.details <- "- Bayesian network meta-analysis with crossnma\n"
    }
    else if (inherits(x, "netmeta.gemtc")) {
      text.details <- "- Bayesian network meta-analysis with gemtc\n"
    }
    else if (inherits(x, "netmeta.multinma")) {
      text.details <- "- Bayesian network meta-analysis with multinma\n"
    }
    else {
      text.details <- "- Frequentist graph-theoretical approach\n"
      if (inherits(x, "netcomb"))
        text.details <-
          paste0(text.details, "- Component network meta-analysis\n")
      #
      if (random | print.tau2 | print.tau | print.I2) {
        if (!is.null(x$tau.preset)) {
          if (print.tau2) {
            tau2 <- x$tau.preset^2
            tau2 <- formatPT(tau2, lab = TRUE, labval = text.tau2,
                             digits = digits.tau2,
                             lab.NA = "NA", big.mark = big.mark)
            #
            text.details <-
              paste0(text.details,
                     "- Preset between-study variance: ", tau2, "\n")
          }
          else if (print.tau) {
            tau <- x$tau.preset
            tau <- formatPT(tau, lab = TRUE, labval = text.tau,
                            digits = digits.tau,
                            lab.NA = "NA", big.mark = big.mark)
            #
            text.details <-
              paste0(text.details,
                     "- Preset between-study standard deviation: ", tau, "\n")
          }
        }
        else if (random | print.tau2 | print.tau) {
          text.details <-
            paste0(text.details,
                   if (x$method.tau == "DL")
                     "- DerSimonian-Laird estimator"
                   #
                   else if (x$method.tau == "REML")
                     "- Restricted maximum-likelihood estimator"
                   ##
                   else if (x$method.tau == "ML")
                     "- Maximum-likelihood estimator")
          #
          text.details <-
            paste0(text.details, " for ",
                   if (print.tau) text.tau else text.tau2, "\n")
        }
      }
      #
      if (print.I2) {
        text.details <-
          paste0(text.details, "- Calculation of ", text.I2, " based on Q\n")
      }
    }
    #
    if (!is.null(x$correlated) && any(x$correlated)) {
      studies.c <- sort(unique(x$studlab[x$correlated]))
      #
      text.details <-
        paste0(text.details,
               "- ", if (length(studies.c) == 1) "Study" else "Studies",
               " with correlated study arms: ",
               paste0("'", studies.c, "'", collapse = ", "),
               "\n")
    }
  }
  else if (inherits(x, "netbind")) {
    text.details <- ""
    #
    if ("Inverse" %in% x$method)
      text.details <- paste0(text.details, "- Inverse variance method\n")
    #
    if ("MH" %in% x$method)
      text.details <- paste0(text.details, "- Mantel-Haenszel method\n")
    #
    if ("NCH" %in% x$method)
      text.details <-
        paste0(text.details,
               "- Based on the non-central hypergeometric distribution\n")
    #
    if ("LRP" %in% x$method)
      text.details <-
        paste0(text.details, "- Penalised logistic regression model\n")
  }
  else if (inherits(x, "netimpact")) {
    text.details <- "- Frequentist graph-theoretical approach\n"
    #
    text.details <-
      paste0(text.details,
             if (x$method.tau == "DL")
               "- DerSimonian-Laird estimator"
             #
             else if (x$method.tau == "REML")
               "- Restricted maximum-likelihood estimator"
             ##
             else if (x$method.tau == "ML")
               "- Maximum-likelihood estimator")
    #
    text.details <-
      paste0(text.details, " for ",
             if (print.tau) text.tau else text.tau2, "\n")
    #
    if (print.I2) {
      text.details <-
        paste0(text.details, "- Calculation of ", text.I2, " based on Q\n")
    }
  }
  else if (inherits(x, "rankogram")) {
    if (x$small.values == "desirable")
      text.details <-
        paste0(text.details,
               "- Small outcome values indicate a beneficial effect\n")
    else if (x$small.values == "undesirable")
      text.details <-
        paste0(text.details,
               "- Small outcome values indicate a harmful effect\n")
    #
    if (random && !is.null(x$x)) {
      if (!is.null(x$x$tau.preset)) {
          tau2 <- formatPT(x$x$tau.preset^2,
                           lab = TRUE, labval = text.tau2,
                           digits = digits.tau2,
                           lab.NA = "NA", big.mark = big.mark)
          #
          text.details <-
            paste0(text.details,
                   "- Preset between-study variance: ", tau2, "\n")
      }
      else {
        text.details <-
          paste0(text.details,
                 if (x$x$method.tau == "DL")
                   "- DerSimonian-Laird estimator"
                 #
                 else if (x$x$method.tau == "REML")
                   "- Restricted maximum-likelihood estimator"
                 ##
                 else if (x$x$method.tau == "ML")
                   "- Maximum-likelihood estimator")
        #
        text.details <-
          paste0(text.details, " for ",
                 if (print.tau) text.tau else text.tau2, "\n")
      }
    }
  }
  else if (inherits(x, "subgroup.netmeta")) {
    if (random) {
      text.details <-
        paste0(text.details,
               if (x$method.tau == "DL")
                 "- DerSimonian-Laird estimator"
               #
               else if (x$method.tau == "REML")
                 "- Restricted maximum-likelihood estimator"
               ##
               else if (x$method.tau == "ML")
                 "- Maximum-likelihood estimator")
      #
      text.details <-
        paste0(text.details, " for ",
               if (print.tau) text.tau else text.tau2, "\n")
    }
  }
  #
  if (text.details != "")
    text.details <-
      paste0("\nDetails of network meta-analysis methods:\n",
             text.details)
  #
  text.details
}

#' Network meta-analysis for chronic obstructive pulmonary disease
#' 
#' @description
#' Network meta-analysis comparing inhaled medications in patients
#' with chronic obstructive pulmonary disease.
#' 
#' @name Dong2013
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{id}}\tab study ID \cr
#' \bold{\emph{treatment}}\tab treatment \cr
#' \bold{\emph{death}}\tab mortality \cr
#' \bold{\emph{randomized}}\tab number of individuals in treatment arm
#' }
#' 
#' @note
#' The dataset Dong2013 is identical to dataset
#' \code{\link[metabook]{Dong2013}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[metabook]{Dong2013}},
#'   \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
#'   \code{\link{netmetabin}}
#' 
#' @source
#' Dong Y-H, Lin H-H, Shau W-Y, Wu Y-C, Chang C-H, Lai M-S (2013):
#' Comparative safety of inhaled medications in patients with chronic
#' obstructive pulmonary disease: systematic review and mixed
#' treatment comparison meta-analysis of randomised controlled trials.
#' \emph{Thorax},
#' \bold{68}, 48--56
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Dong2013)
#' 
#' # Example using pairwise() and netmetabin():
#' # example(Dong2013, package = "metabook")

NULL

#' Network meta-analysis on blood loss during liver transplantation
#' 
#' @description
#' Network meta-analysis comparing the effects of a number of
#' interventions for decreasing blood loss and blood transfusion
#' requirements during liver transplantation.
#' 
#' @name Gurusamy2011
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study information (first author, year) \cr
#' \bold{\emph{treatment}}\tab treatment \cr
#' \bold{\emph{death}}\tab mortality at 60 days post-transplantation
#'   \cr
#' \bold{\emph{n}}\tab number of individuals in treatment arm
#' } 
#' 
#' @note
#' The dataset Gurusamy2011 is identical to dataset
#' \code{\link[metabook]{Gurusamy2011}} in R package \bold{metabook}.
#' 
#' @seealso \code{\link[metabook]{Gurusamy2011}},
#'   \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
#'   \code{\link{netmetabin}}
#' 
#' @source
#' Gurusamy KS, Pissanou T, Pikhart H, Vaughan J, Burroughs AK,
#' Davidson BR (2011):
#' Methods to decrease blood loss and transfusion requirements for
#' liver transplantation.
#' \emph{Cochrane Database of Systematic Reviews},
#' CD009052
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Gurusamy2011)
#' 
#' # Example using pairwise() and netmetabin():
#' # example(Gurusamy2011, package = "metabook")

NULL

#' Print method for objects of class decomp.design
#' 
#' @description
#' Print method for objects of class \code{decomp.design}.
#' 
#' @param x An object of class \code{decomp.design}.
#' @param digits.Q Minimal number of significant digits for Q
#'   statistics, see \code{print.default}.
#' @param showall A logical indicating whether results should be shown
#'   for all designs or only designs contributing to chi-squared
#'   statistics (default).
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity tests, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance, see \code{print.default}.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param big.mark A character used as thousands separator.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param sort A logical indicating whether to sort results by
#'   p-values.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param \dots Additional arguments (ignored at the moment).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}, Ulrike
#'   Krahn \email{ulrike.krahn@@bayer.com}
#' 
#' @seealso \code{\link{decomp.design}}
#' 
#' @keywords print
#' 
#' @examples
#' # Examples: example(decomp.design)
#' 
#' @method print decomp.design
#' @export

print.decomp.design <- function(x,
                                digits.Q = gs("digits.Q"),
                                showall = FALSE,
                                digits.pval.Q = gs("digits.pval.Q"),
                                digits.tau2 = gs("digits.tau2"),
                                scientific.pval = gs("scientific.pval"),
                                big.mark = gs("big.mark"),
                                nchar.trts = x$nchar.trts,
                                sort = TRUE,
                                legend = gs("legend"),
                                ...) {
  
  
  chkclass(x, "decomp.design")
  x <- updateversion(x)
  ##
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.pval.Q, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  ##
  chklogical(showall)
  if (is.null(nchar.trts))
    nchar.trts <- 666
  chknumeric(nchar.trts, min = 1, length = 1)
  chklogical(sort)
  chklogical(legend)
  ##
  if (is.null(x$x$sep.trts))
    sep.trts <- ":"
  else
    sep.trts <- x$x$sep.trts
  
  
  if (!is.null(attributes(x)$netmetabin)) {
    Qdata <- x$Q.decomp
    ##
    Qdata$Q <- round(Qdata$Q, digits.Q)
    Qdata$pval <- formatPT(Qdata$pval,
                           digits = digits.pval.Q,
                           scientific = scientific.pval)
    ##
    dimnames(Qdata) <- list("", c("Q", "d.f.", "p-value"))
    ##
    cat("\nTest of inconsistency (between designs):\n")
    prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
    ##
    return(invisible(NULL))
  }
  
  
  Q.decomp <- x$Q.decomp
  Q.design <- x$Q.het.design
  Q.detach <- x$Q.inc.detach
  ##
  trts1 <- trts2 <- ""
  ##
  if (!showall) {
    Q.design <- Q.design[Q.design$df > 0, ]
    ##
    sel.designs <- Q.detach$design[Q.detach$df !=
                                   Q.decomp["Between designs", "df"]]
    Q.detach <- Q.detach[Q.detach$design %in% sel.designs, ]
  }
  ##
  Q.inc.random <- x$Q.inc.random
  Q.inc.random.preset <- x$Q.inc.random.preset
  ##
  Q.decomp$Q <- formatN(round(Q.decomp$Q, digits.Q), digits.Q, "NA",
                        big.mark = big.mark)
  Q.decomp$pval <- formatPT(Q.decomp$pval, digits = digits.pval.Q,
                            scientific = scientific.pval)
  ##
  Q.design$Q <- formatN(round(Q.design$Q, digits.Q), digits.Q, "NA",
                        big.mark = big.mark)
  if (sort)
    Q.design <- Q.design[order(Q.design$pval), , drop = FALSE]
  Q.design$pval <- formatPT(Q.design$pval, digits = digits.pval.Q,
                            scientific = scientific.pval)
  ##
  Q.detach$Q <- formatN(round(Q.detach$Q, digits.Q), digits.Q, "NA",
                        big.mark = big.mark)
  if (sort)
    Q.detach <- Q.detach[order(-Q.detach$pval), , drop = FALSE]
  Q.detach$pval <- formatPT(Q.detach$pval, digits = digits.pval.Q,
                            scientific = scientific.pval)
  ##
  Q.inc.random$Q <- formatN(round(Q.inc.random$Q, digits.Q), digits.Q, "NA",
                            big.mark = big.mark)
  Q.inc.random$tau2.within <- formatPT(Q.inc.random$tau.within^2,
                                       digits = digits.tau2,
                                       lab.NA = "NA", big.mark = big.mark)
  Q.inc.random$tau.within <- formatPT(Q.inc.random$tau.within,
                                      digits = digits.tau2,
                                      lab.NA = "NA", big.mark = big.mark)
  Q.inc.random$pval <- formatPT(Q.inc.random$pval, digits = digits.pval.Q,
                                scientific = scientific.pval)
  ##
  nam <- names(Q.decomp)
  names(Q.decomp)[nam == "pval"] <- "p-value"
  nam <- names(Q.design)
  names(Q.design)[nam == "design"] <- "Design"
  names(Q.design)[nam == "pval"] <- "p-value"
  nam <- names(Q.detach)
  names(Q.detach)[nam == "design"] <- "Detached design"
  names(Q.detach)[nam == "pval"] <- "p-value"
  nam <- names(Q.inc.random)
  names(Q.inc.random)[nam == "pval"] <- "p-value"
  ##
  Q.design <- as.matrix(Q.design)
  Q.detach <- as.matrix(Q.detach)
  
  cat("Q statistics to assess homogeneity / consistency\n\n")
  print(Q.decomp)
  
  if (nrow(Q.design) > 0) {
    cat("\nDesign-specific decomposition of within-designs Q statistic\n\n")
    ##
    trts1 <- unique(sort(unlist(compsplit(Q.design[, 1], sep.trts))))
    Q.design[, 1] <- comps(Q.design[, 1], trts1, sep.trts, nchar.trts)
    ##
    dimnames(Q.design) <- list(rep("", dim(Q.design)[[1]]),
                               colnames(Q.design))
    prmatrix(Q.design, quote = FALSE, right = TRUE)
  }
  
  if (nrow(Q.detach) > 0) {
    cat("\nBetween-designs Q statistic after detaching of single designs\n",
        "(influential designs have p-value markedly different from ",
        rmSpace(Q.decomp[3, 3]), ")\n\n",
        sep = "")
    ##
    trts2 <- unique(sort(unlist(compsplit(Q.detach[, 1], sep.trts))))
    Q.detach[, 1] <- comps(Q.detach[, 1], trts2, sep.trts, nchar.trts)
    ##
    dimnames(Q.detach) <- list(rep("", dim(Q.detach)[[1]]),
                               colnames(Q.detach))
    prmatrix(Q.detach, quote = FALSE, right = TRUE)
  }

  cat("\nQ statistic to assess consistency under the assumption of\n",
      "a full design-by-treatment interaction random effects model\n\n",
      sep = "")
  print(Q.inc.random)
    
  
  ##
  ## Add legend with abbreviated treatment labels
  ##
  trts <- unique(c(trts1, trts2))
  legendabbr(trts, treats(trts, nchar.trts), legend)
  
  
  invisible(NULL)
}

#' Plot rankograms
#'
#' @description
#' This function produces a rankogram, i.e., an image plot of ranking
#' probabilities for all treatments.
#'
#' @param x An object of class \code{\link{rankogram}}.
#' @param type A character string specifying whether a "bar" chart, a
#'   "line" graph, or "step" functions should be drawn. Can be
#'   abbreviated.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param sort A logical indicating whether treatments should be
#'   sorted by decreasing SUCRAs.
#' @param trts Treatment(s) to show in rankogram.
#' @param cumulative.rankprob A logical indicating whether cumulative
#'   ranking probabilites should be shown.
#' @param ylim The y limits (min, max) of the plot.
#' @param ylab A label for the y-axis.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param \dots Additional graphical arguments (ignored at the
#'   moment).
#' 
#' @details
#' This function produces plots of (cumulative) ranking probabilities for all
#' treatments as a bar graph, a line graph or as step functions
#' (argument \code{type}). All plots will be shown in a single figure if
#' R package \bold{gridExtra} is installed. Otherwise, separate figures will be
#' created for treatments.
#'
#' By default (argument \code{pooled}), results for the random effects
#' model are shown if a network meta-analysis was conducted for both
#' the common and random effects model.
#'
#' Treatments are sorted according to their mean effects if argument
#' \code{sort = TRUE} (default).  A subset of treatments can be
#' specified using argument \code{trts}.
#'
#' Cumulative ranking probabilites are shown if
#' \code{cumulative.rankprob = TRUE}. By default, step functions are
#' shown for cumulative ranking probabilites.
#'
#' @author Theodoros Papakonstantinou \email{dev@@tpapak.com}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{rankogram}}, \code{\link[metabook]{Woods2010}}
#'
#' @references 
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#' 
#' @examples
#' pw <- pairwise(treatment, event = r, n = N, studlab = author,
#'   data = Woods2010, sm = "OR")
#' nma <- netmeta(pw, small.values = "desirable")
#'
#' set.seed(1909) # get reproducible results
#' rnk <- rankogram(nma, nsim = 100)
#' rnk
#'
#' plot(rnk)
#' plot(rnk, type = "l")
#' plot(rnk, cumulative.rankprob = TRUE)
#' 
#' @method plot rankogram
#' @export

plot.rankogram <- function(x,
                           type = if (cumulative.rankprob) "step" else "bar",
                           pooled = ifelse(x$random, "random", "common"),
                           sort = TRUE, trts,
                           cumulative.rankprob = x$cumulative.rankprob,
                           ylim, ylab,
                           nchar.trts = x$nchar.trts, ...) {  
  
  chkclass(x, c("rankogram"))
  x <- updateversion(x)
  
  
  type <- setchar(type, c("bar", "line", "step"))
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  ##
  chklogical(sort)
  ##
  if (is.null(cumulative.rankprob))
    cumulative.rankprob <- FALSE
  chklogical(cumulative.rankprob)
  ##
  missing.ylim <- missing(ylim)
  if (!missing.ylim)
    chknumeric(ylim, min = 0, max = 1, length = 2)
  ##
  if (!missing(ylab))
    chkchar(ylab, length = 1)
  else {
    if (cumulative.rankprob)
      ylab <- "Cumulative\nprobability"
    else
      ylab <- "Probability"
  }
  if (missing(trts))
    trts <- NULL
  ##
  if (is.null(nchar.trts))
    nchar.trts <- 666
  chknumeric(nchar.trts, length = 1)
  
  
  mytheme <-
    theme(axis.line.x =
            element_line(colour = "black", linewidth = 1, linetype = "solid"),
          axis.line.y =
            element_line(colour = "black", linewidth = 1, linetype = "solid"),
          panel.grid.major = element_line(color = "transparent"),
          panel.grid.minor = element_line(color = "transparent"),
          panel.background = element_rect(fill = "transparent"),
          plot.background =
            element_rect(fill = "transparent", colour = "transparent", linewidth = 1)
          )
  
  
  if (pooled == "random") {
    if (cumulative.rankprob)
      rankmatrix <- "cumrank.matrix.random"
    else 
      rankmatrix <- "ranking.matrix.random"
    ##
    sucras <- "ranking.random"
  }
  else if (pooled == "common") {
    if (cumulative.rankprob)
      rankmatrix <- "cumrank.matrix.common"
    else 
      rankmatrix <- "ranking.matrix.common"
    ##
    sucras <- "ranking.common"
  }
  ##
  if (is.null(x[[rankmatrix]]))
    stop("No results for ", pooled, " effect",
         if (pooled == "random") "s", " model available. ",
         "Rerun rankogram() with argument '", pooled, " = TRUE'.",
         call. = FALSE)
  
  
  plotranks <- function(treat) {
    df <- data.frame(pos = 1:nrow(x[[rankmatrix]]),
                     ranks = x[[rankmatrix]][treat, ])
    mymaxvalue <- max(x[[rankmatrix]])
    ##
    if (type == "bar")
      p <- ggplot(mapping = aes(df$pos, df$ranks)) + geom_col()
    else if (type == "line")
      p <- ggplot(mapping = aes(df$pos, df$ranks)) + geom_line()
    else if (type == "step")
      p <- ggplot(mapping = aes(df$pos, df$ranks)) + geom_step()
    ##
    p <- p + scale_x_continuous(breaks = seq(1, nrow(x[[rankmatrix]]), 1))
    p <- p + labs(x = paste("Rank of", treats(treat, nchar.trts)))
    p <- p + labs(y = ylab)
    ##
    if (missing.ylim)
      p <- p + expand_limits(x = c(1, x$x$n), y = c(0, mymaxvalue))
    else
      p <- p + expand_limits(x = c(1, x$x$n), y = ylim)
    ##
    p + mytheme
  }
  ##
  if (sort)
    treatnames <- names(sort(x[[sucras]], decreasing = TRUE))
  else
    treatnames <- names(x[[sucras]])
  ##
  if (!is.null(trts)) {
    trts.c <- setref(trts, treatnames, varname = "trts", length = 0)
    treatnames <- treatnames[treatnames %in% trts.c]
  }
  #
  rankplots <- lapply(treatnames, plotranks)
  names(rankplots) <- treatnames
  #
  if (is_installed_package("gridExtra", stop = FALSE))
    return(do.call(gridExtra::grid.arrange, rankplots))
  else
    return(rankplots)
}

#' Network meta-analysis in diabetes
#' 
#' @description
#' Network meta-analysis in diabetes comparing effects of a number of
#' drugs on the HbA1c value.
#' 
#' These data are used as an example in Senn et al. (2013) and have been
#' preprocessed for use in R package netmeta.
#' 
#' @name Senn2013
#' 
#' @docType data
#' 
#' @details
#' This data set is in comparison-based format. The data in
#' long arm-based format are available in \code{\link[metabook]{Senn2013}} in
#' R package \bold{metabook}.
#' 
#' Treatment labels provided by columns \code{treat1} and
#' \code{treat2} have been abbreviated:
#' \itemize{
#' \item acar = Acarbose
#' \item benf = Benfluorex
#' \item metf = Metformin
#' \item migl = Miglitol
#' \item piog = Pioglitazone
#' \item plac = Placebo
#' \item rosi = Rosiglitazone
#' \item sita = Sitagliptin
#' \item sulf = Sulfonylurea
#' \item vild = Vildagliptin
#' }
#'
#' Full treatment names are available in columns \code{treat1.long}
#' and \code{treat2.long}.
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{TE}}\tab treatment effect \cr
#' \bold{\emph{seTE}}\tab standard error of treatment effect \cr
#' \bold{\emph{treat1}}\tab treatment 1 \cr
#' \bold{\emph{treat2}}\tab treatment 2 \cr
#' \bold{\emph{treat1.long}}\tab treatment 1 (full treatment names)
#'   \cr
#' \bold{\emph{treat2.long}}\tab treatment 2 (full treatment names)
#'   \cr
#' \bold{\emph{studlab}}\tab Study label
#' }
#' 
#' @seealso \code{\link{netmeta}}
#' 
#' @source
#' Senn S, Gavini F, Magrez D, Scheen A (2013):
#' Issues in performing a network meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{22}, 169--89
#' 
#' @keywords datasets
#' 
#' @examples
#' head(Senn2013)
#' 
#' # Example using netmeta():
#' # example(netmeta)

NULL

createB <- function(pos1, pos2, ncol, aggr = FALSE) {
  
  
  if (!aggr) {
    if (missing(pos1) | missing(pos2)) {
      ##
      ## Create full edge-vertex incidence matrix
      ##
      nrow <- choose(ncol, 2)
      B <- matrix(0, nrow = nrow, ncol = ncol)
      ##
      i <- 0
      ##
      for (pos1.i in 1:(ncol - 1)) {
        for (pos2.i in (pos1.i + 1):ncol) {
          i <- i + 1
          B[i, pos1.i] <-  1
          B[i, pos2.i] <- -1
        }
      }
    }
    else {
      ##
      ## Create edge-vertex incidence matrix
      ##
      nrow <- length(pos1)
      ncol <- length(unique(c(pos1, pos2)))
      ##
      B <- matrix(0, nrow = nrow, ncol = ncol)
      ##
      for (i in 1:nrow) {
        B[i, pos1[i]] <-  1
        B[i, pos2[i]] <- -1
      }
    }
  }
  else {
    nrow <- 0
    ##
    ## Determine number of edges (no. of rows of B)
    ##
    for (i in 1:(ncol - 1)) {
      for (j in (i + 1):ncol) {
        ij.count <- 0
        ## Cycle through every possible edge ij
        ## Search pos1 and pos2 to see if at least one of these
        ## combinations is ij
        for (k in seq_along(pos1)) {
          if (pos1[k] == i & pos2[k] == j) {
            ij.count <- ij.count + 1
          }
          else {
            ij.count <- ij.count
          }
        }
        if (ij.count > 0)
          nrow <- nrow + 1
        else
          nrow <- nrow
      }
    }
    ##
    ## Create aggregate B matrix with dimensions e x n
    ##
    B <- matrix(0, nrow = nrow, ncol = ncol)
    ##
    r <- 0
    ## Cycle through each possible pairwise comparison ij
    for (i in 1:(ncol - 1)) {
      for (j in (i + 1):ncol) {
        ij.count <- 0
        for (k in 1:length(pos1)) {
          ## If there is an edge for that pairwise comparison ...
          if (pos1[k] == i & pos2[k] == j)
            ij.count <- ij.count + 1 # ...then ij.count is no longer = 0 ...
          else
            ij.count <- ij.count
        }
        if (ij.count > 0) {
          ## ...and we add this row to B
          r <- r + 1
          B[r, i] <-  1
          B[r, j] <- -1
        }
      }
    }
  }
  
  B
}

#' Create a data frame from an object of class netconnection
#' 
#' @description
#' The \code{as.data.frame} method returns a data frame containing
#' information on membership of studies / pairwise comparisons to a
#' (sub)network.
#' 
#' @param x An object of class \code{netconnection}.
#' @param \dots Additional arguments (ignored).
#' @return A data frame is returned by the function
#'   \code{as.data.frame}.
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' @seealso \code{\link{netconnection}}
#' 
#' @examples
#' # Artificial example with two subnetworks
#' #
#' t1 <- c("G", "B", "B", "D", "A", "F")
#' t2 <- c("B", "C", "E", "E", "H", "A")
#' #
#' nc <- netconnection(t1, t2)
#' print(nc, details = TRUE)
#' 
#' as.data.frame(nc)
#'
#' @method as.data.frame netconnection 
#' @export

as.data.frame.netconnection <- function(x, ...){
  
  chkclass(x, "netconnection")
  
  # Only keep relevant list elements
  #
  res <- data.frame(treat1 = x$treat1,
                    treat2 = x$treat2,
                    studlab = x$studlab,
                    design = x$design,
                    subnet = x$subnet)
  #
  attr(res, "version") <- x$version
  
  res
}

#' Moore-Penrose Pseudoinverse of a Matrix
#' 
#' @description
#' Calculates the Moore-Penrose pseudoinverse of a square matrix
#' \strong{X}.
#' 
#' @param X A square matrix.
#' 
#' @details
#' This function is used by default in R package \strong{netmeta} to
#' calculate the Moore-Penrose pseudoinverse \strong{L\eqn{^+}} of the
#' Laplacian matrix \strong{L} (Rücker, 2012):
#' 
#' \strong{L\eqn{^+} = (X - J / \emph{n})\eqn{^{-1}} + J / \emph{n}}
#' with identity matrix \strong{J} of dimension \emph{n}x\emph{n}.
#'
#' @return
#' The Moore-Penrose pseudoinverse for matrix \strong{X}.
#' 
#' @author
#' Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido Schwarzer
#'   \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{solve}}
#' 
#' @references
#' Rücker G (2012):
#' Network meta-analysis, electrical networks and graph theory.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 312--24
#' 
#' @examples
#' \donttest{
#' data(smokingcessation)
#' 
#' pw <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' net <- netmeta(pw)
#' 
#' L1 <- net$L.matrix.common
#' L2 <- invmat(net$Lplus.matrix.common)
#' all.equal(round(L1, 10), round(L2, 10))
#' }
#' 
#' @export invmat

invmat <- function(X) {
  n <- nrow(X)
  m <- ncol(X)
  ##
  if (n != m)
    stop("Argument 'X' must be a square matrix", call. = FALSE)
  ##
  J <- matrix(1, nrow = n, ncol = n)
  ##
  res <- solve(X - J / n) + J / n
  ##
  res
}

catarg <- function(x, newline = TRUE, end = "") {
  xname <- x
  x <- gsub(" ", "", x)
  ##
  if (newline)
    cat("- ")
  ##
  if (is.null(gs(x)))
    cat(paste0(xname, ' = NULL', end, '\n'))
  else if (is.character(gs(x)))
    cat(paste0(xname, ' = "', gs(x), '"', end, '\n'))
  else
    cat(paste0(xname, ' = ', gs(x), end, "\n"))
  ##
  invisible(NULL)
}


setcharacter <- function(argname, args, set = NULL, length = 1,
                         NULL.ok = FALSE, ignore.other = FALSE,
                         logical.ok = FALSE) {
  id <- argid(names(args), argname)
  ##
  if (!is.na(id)) {
    val <- args[[id]]
    ##
    if (NULL.ok & is.null(val)) {
      l <- list(val)
      names(l) <- argname
      settings.meta(l)
      return(invisible(NULL))
    }
    ##
    if (logical.ok & is.logical(val)) {
      l <- list(val)
      names(l) <- argname
      settings.meta(l)
      return(invisible(NULL))
    }
    ##
    if (!is.character(val) & ignore.other)
      return(invisible(id))
    ##
    if (!is.null(set))
      val <- setchar(val, set, name = argname)
    else
      chkchar(val, length = length, name = argname)
    ##
    l <- list(val)
    names(l) <- argname
    settings.meta(l)
  }
  ##
  invisible(id)
}


setcolor <- function(argname, args) {
  id <- argid(names(args), argname)
  ##
  if (!is.na(id)) {
    val <- args[[id]]
    chkcolor(val, name = argname)
    #
    l <- list(val)
    names(l) <- argname
    settings.meta(l)
  }
  ##
  invisible(id)
}


setlevel <- function(argname, args) {
  id <- argid(names(args), argname)
  ##
  if (!is.na(id)) {
    val <- args[[id]]
    chklevel(val, name = argname)
    #
    l <- list(val)
    names(l) <- argname
    settings.meta(l)
  }
  ##
  invisible(id)
}


setlogical <- function(argname, args, NULL.ok = FALSE,
                       ignore.other = FALSE) {
  id <- argid(names(args), argname)
  ##
  if (!is.na(id)) {
    val <- args[[id]]
    ##
    if (NULL.ok & is.null(val)) {
      l <- list(val)
      names(l) <- argname
      settings.meta(l)
      return(invisible(NULL))
    }
    ##
    if (!is.logical(val) & ignore.other)
      return(invisible(id))
    ##
    chklogical(val, name = argname)
    #
    l <- list(val)
    names(l) <- argname
    settings.meta(l)
  }
  ##
  invisible(id)
}


setnumeric <- function(argname, args, NULL.ok = FALSE) {
  id <- argid(names(args), argname)
  ##
  if (!is.na(id)) {
    val <- args[[id]]
    ##
    if (NULL.ok & is.null(val)) {
      l <- list(val)
      names(l) <- argname
      settings.meta(l)
      return(invisible(NULL))
    }
    ##
    chknumeric(val, min = 0, length = 1, name = argname)
    #
    l <- list(val)
    names(l) <- argname
    settings.meta(l)
  }
  ##
  invisible(id)
}

mat2dat.split <- function(x, pooled = "common", dat.trts) {

  pooled <- setchar(pooled, c("common", "random"))
  ##
  name.n <- paste0("nma.", pooled)
  name.d <- paste0("direct.", pooled)
  name.i <- paste0("indirect.", pooled)
  
  
  nma <- direct <- indirect <-
    data.frame(comparison = dat.trts$comparison,
               TE = NA, seTE = NA, lower = NA, upper = NA,
               statistic = NA, p = NA)
  ##
  direct$I2 <- direct$tau <- direct$tau2 <- direct$Q <-
    direct$n <- NA
  ##
  predict <- data.frame(comparison = dat.trts$comparison,
                        lower = NA, upper = NA)
  
  
  k <- nma$TE
  ##
  for (i in seq_len(nrow(dat.trts))) {
    t1.i <- dat.trts$treat1[i]
    t2.i <- dat.trts$treat2[i]
    ##
    k[i] <- x$x$A.matrix[t1.i, t2.i]
    ##
    nma$TE[i] <- x[[name.n]]$TE[t1.i, t2.i]
    nma$seTE[i] <- x[[name.n]]$seTE[t1.i, t2.i]
    nma$lower[i] <- x[[name.n]]$lower[t1.i, t2.i]
    nma$upper[i] <- x[[name.n]]$upper[t1.i, t2.i]
    nma$statistic[i] <- x[[name.n]]$statistic[t1.i, t2.i]
    nma$p[i] <- x[[name.n]]$p[t1.i, t2.i]
    ##
    direct$TE[i] <- x[[name.d]]$TE[t1.i, t2.i]
    direct$seTE[i] <- x[[name.d]]$seTE[t1.i, t2.i]
    direct$lower[i] <- x[[name.d]]$lower[t1.i, t2.i]
    direct$upper[i] <- x[[name.d]]$upper[t1.i, t2.i]
    direct$statistic[i] <- x[[name.d]]$statistic[t1.i, t2.i]
    direct$p[i] <- x[[name.d]]$p[t1.i, t2.i]
    ##
    if (!is.null(x$x$n.matrix))
      direct$n[i] <- x$x$n.matrix[t1.i, t2.i]
    direct$Q[i] <- x$x$Q.direct[t1.i, t2.i]
    direct$tau2[i] <- x$x$tau2.direct[t1.i, t2.i]
    direct$tau[i] <- x$x$tau.direct[t1.i, t2.i]
    direct$I2[i] <- x$x$I2.direct[t1.i, t2.i]
    ##
    indirect$TE[i] <- x[[name.i]]$TE[t1.i, t2.i]
    indirect$seTE[i] <- x[[name.i]]$seTE[t1.i, t2.i]
    indirect$lower[i] <- x[[name.i]]$lower[t1.i, t2.i]
    indirect$upper[i] <- x[[name.i]]$upper[t1.i, t2.i]
    indirect$statistic[i] <- x[[name.i]]$statistic[t1.i, t2.i]
    indirect$p[i] <- x[[name.i]]$p[t1.i, t2.i]
    ##
    if (pooled == "random") {
      predict$lower[i] <- x$x$lower.predict[t1.i, t2.i]
      predict$upper[i] <- x$x$upper.predict[t1.i, t2.i]
    }
  }
  ##
  m <-
    suppressWarnings(metagen(direct$TE - indirect$TE,
                             sqrt(direct$seTE^2 +
                                  indirect$seTE^2),
                             level = x$x$level.ma,
                             method.tau = "DL", method.tau.ci = ""))
  ##
  compare <-
    data.frame(comparison = dat.trts$comparison,
               TE = m$TE, seTE = m$seTE,
               lower = m$lower, upper = m$upper,
               statistic = m$statistic, p = m$pval,
               z = m$statistic,
               stringsAsFactors = FALSE)

  res <- list(k = k, nma = nma,
              direct = direct, indirect = indirect,
              compare = compare, predict = predict)
  ##
  res
}

#' Print detailed results of network meta-analysis
#' 
#' @description
#' Print method for objects of class \code{summary.netmeta}.
#' 
#' @param x An object of class \code{summary.netmeta}.
#' @param sortvar An optional vector used to sort individual studies
#'   (must be of same length as \code{x$TE}).
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param prediction A logical indicating whether prediction intervals
#'   should be printed.
#' @param reference.group Reference treatment.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
#'   matrices with all treatment effects, and confidence limits will
#'   be printed.
#' @param details A logical indicating whether further details for
#'   individual studies should be printed.
#' @param nma A logical indicating whether summary results of network
#'   meta-analysis should be printed.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param nchar.studlab A numeric defining the minimum number of
#'   characters used to create unique study labels.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   deviations and standard errors, see \code{print.default}.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity tests, see \code{print.default}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistics, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance, see \code{print.default}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   statistic, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.Q A logical value indicating whether to print the
#'   results of the test of heterogeneity.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param truncate An optional vector used to truncate the printout of
#'   results for individual studies (must be a logical vector of
#'   length corresponding to the number of pairwise comparisons
#'   \code{x$TE} or contain numerical values).
#' @param text.truncate A character string printed if study results
#'   were truncated from the printout.
#' @param details.methods A logical specifying whether details on statistical
#'   methods should be printed.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{netmeta}}, \code{\link{summary.netmeta}}
#' 
#' @keywords print
#' 
#' @examples
#' # Examples: example(summary.netmeta)
#' 
#' @method print summary.netmeta
#' @export

print.summary.netmeta <- function(x,
                                  sortvar,
                                  common = x$x$common,
                                  random = x$x$random,
                                  prediction = x$prediction,
                                  reference.group = x$reference.group,
                                  baseline.reference = x$baseline.reference,
                                  all.treatments = x$all.treatments,
                                  details = TRUE, nma = TRUE,
                                  ##
                                  overall.hetstat = x$overall.hetstat,
                                  backtransf = x$backtransf,
                                  nchar.trts = x$nchar.trts,
                                  nchar.studlab = x$nchar.studlab,
                                  digits = gs("digits"),
                                  digits.se = gs("digits.se"),
                                  digits.pval.Q = max(gs("digits.pval.Q"), 2),
                                  digits.Q = gs("digits.Q"),
                                  digits.tau2 = gs("digits.tau2"),
                                  digits.I2 = gs("digits.I2"),
                                  #
                                  big.mark = gs("big.mark"),
                                  scientific.pval = gs("scientific.pval"),
                                  zero.pval = gs("zero.pval"),
                                  JAMA.pval = gs("JAMA.pval"),
                                  #
                                  print.tau2 = gs("print.tau2"),
                                  print.tau = gs("print.tau"),
                                  print.Q = gs("print.Q"),
                                  print.I2 = gs("print.I2"),
                                  print.I2.ci = gs("print.I2.ci"),
                                  #
                                  truncate,
                                  text.truncate = "*** Output truncated ***",
                                  #
                                  details.methods = gs("details"),
                                  legend = gs("legend"),
                                  #
                                  warn.deprecated = gs("warn.deprecated"),
                                  ##
                                  ...) {
  
  
  ##
  ##
  ## (1) Check for summary.netmeta object and upgrade object
  ##
  ##
  chkclass(x, "summary.netmeta")
  x <- updateversion(x)
  ##
  k.all <- length(x$x$TE)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chklogical(prediction)
  chklogical(baseline.reference)
  ##
  chknumeric(nchar.trts, min = 1, length = 1)
  if (is.null(nchar.studlab))
    nchar.studlab <- 666
  chknumeric(nchar.studlab, min = 1, length = 1)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.pval.Q, min = 1, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  #
  chkchar(big.mark, length = 1)
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chklogical(print.tau2)
  chklogical(print.tau)
  chklogical(print.Q)
  chklogical(print.I2)
  chklogical(print.I2.ci)
  #
  chklogical(nma)
  chklogical(details.methods)
  chklogical(legend)
  ##
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  ## Catch 'truncate' from network meta-analysis object:
  ##
  missing.truncate <- missing(truncate)
  if (!missing.truncate) {
    truncate <- catch("truncate", mc, x$x, sfsp)
    ##
    if (is.null(truncate))
      truncate <- catch("truncate", mc, x$x$data, sfsp)
    ##
    if (length(truncate) > k.all)
      stop("Length of argument 'truncate' is too long.",
           call. = FALSE)
    else if (length(truncate) < k.all) {
      if (is.numeric(truncate)) {
        if (any(is.na(truncate)) | max(truncate) > k.all | min(truncate) < 0)
          stop("Numeric values in argument 'truncate' must be between 1 and ",
               k.all, ".",
               call. = FALSE)
        truncate2 <- rep(FALSE, k.all)
        truncate2[truncate] <- TRUE
        truncate <- truncate2
      }
      else if (is.character(truncate)) {
        if (any(!(truncate %in% x$x$studlab)))
          stop("At least one value of argument 'truncate' does not ",
               "match a study label.",
               call. = FALSE)
        truncate2 <- rep(FALSE, k.all)
        truncate2[x$x$studlab %in% truncate] <- TRUE
        truncate <- truncate2
      }
      else
        stop("Argument 'truncate' must contain integers or study labels if ",
             "length differs from number of treatment effects.",
             call. = FALSE)
    }
  }
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                      warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  chklogical(overall.hetstat)
  #
  backtransf <-
    deprecated(backtransf, missing(backtransf), args, "logscale")
  if (is_untransformed(x$sm))
    backtransf <- TRUE
  chklogical(backtransf)
  
  
  ##
  ##
  ## (3) Some additional settings
  ##
  ##
  if (!inherits(x, "summary.netmetabin")) {
    ##
    if (missing(sortvar)) sortvar <- 1:k.all
    ##
    if (length(sortvar) != k.all)
      stop("'x' and 'sortvar' have different length")
    ##
    ci.lab <- paste0(round(100 * x$level, 1), "% CI")
    ##
    sm <- sm.lab <- x$sm
    #
    if (!backtransf & (is_relative_effect(sm) | sm == "VE"))
      sm.lab <- paste0("log", if (sm == "VE") "VR" else sm)
    #
    if (missing.truncate)
      truncate <- rep_len(TRUE, x$x$m)
    #
    treat1 <- x$x$treat1
    treat2 <- x$x$treat2
    #
    trts <- unique(c(treat1[truncate], treat2[truncate]))
    trts.abbr <- treats(trts, nchar.trts)
    #
    studlab <- x$x$studlab
    studlab.tr <- studlab[truncate]
    #
    studies.tr <- unique(studlab.tr)
    labels.s <- treats(studies.tr, nchar.studlab)
    #
    studlab.abbr <-
      as.character(
        factor(studlab, levels = studies.tr, labels = labels.s))
    
    
    ##
    ##
    ## (4) Print title and details
    ##
    ##
    matitle(x)
    ##  
    if (details) {
      multiarm <- any(x$x$narms > 2)
      cat("Original data",
          ifelse(multiarm & (common | random),
                 paste(" (with adjusted standard errors for",
                       "multi-arm studies)"),
                 ""),
          ":\n\n",
          sep = ""
      )
      ##
      res <- data.frame(treat1,
                        treat2,
                        TE = formatN(x$x$TE, digits, text.NA = "NA",
                                     big.mark = big.mark),
                        seTE = formatN(x$x$seTE, digits.se, text.NA = "NA",
                                       big.mark = big.mark))
      ##
      if (multiarm) {
        if (is.null(x$x$seTE.adj.common))
          seTE.adj <- x$x$seTE.adj
        else
          seTE.adj <- x$x$seTE.adj.common
        #
        if (common & random & !is.null(x$x$seTE.adj.random)) {
          if (!all(is.na(seTE.adj)))
            res$seTE.adj.c <- format(round(seTE.adj, digits.se))
          if (!all(is.na(x$x$seTE.adj.random)))
            res$seTE.adj.r <- format(round(x$x$seTE.adj.random, digits.se))
        }
        else if (common) {
          if (!all(is.na(seTE.adj)))
            res$seTE.adj <- format(round(seTE.adj, digits.se))
        }
        else if (random & !is.null(x$x$seTE.adj.random)) {
          if (!all(is.na(x$x$seTE.adj.random)))
            res$seTE.adj <- format(round(x$x$seTE.adj.random, digits.se))
        }
        #
        res$narms <- x$x$n.arms
        res$multiarm <- ifelse(x$x$multiarm, "*", "")
      }
      ##
      res <- as.matrix(res)
      rownames(res) <- studlab
      #
      if (!missing.truncate) {
        sortvar <- sortvar[truncate]
        res <- res[truncate, , drop = FALSE]
      }
      #
      res <- res[order(sortvar), , drop = FALSE]
      #
      rownames(res) <-
        as.character(
          factor(rownames(res), levels = studies.tr, labels = labels.s))
      #
      res[, "treat1"] <-
        as.character(factor(res[, "treat1"], levels = trts, labels = trts.abbr))
      #
      res[, "treat2"] <-
        as.character(factor(res[, "treat2"], levels = trts, labels = trts.abbr))
      #
      prmatrix(res, quote = FALSE, right = TRUE)
      #
      if (!missing.truncate)
        cat(text.truncate, "\n")
      cat("\n")
      ##      
      studyarms <- data.frame(narms = x$x$narms, row.names = x$x$studies)
      #
      if (!missing.truncate)
        studyarms <-
          studyarms[rownames(studyarms) %in% studies.tr, , drop = FALSE]
      #
      rownames(studyarms) <-
        as.character(
          factor(rownames(studyarms), levels = studies.tr, labels = labels.s))
      #
      if (multiarm)
        studyarms$multiarm <- ifelse(studyarms$narms > 2, "*", "")
      #
      if (length(unique(studyarms$narms)) != 1) {
        studyarms <- studyarms[order(-studyarms$narms), , drop = FALSE]
        addtext <- " (by decreasing number of arms)"
      }
      else
        addtext <- ""
      #
      cat(paste0("Number of treatment arms per study", addtext, ":\n"))
      #
      prmatrix(studyarms, quote = FALSE, right = TRUE)
      #
      if (!missing.truncate)
        cat(text.truncate, "\n")
      cat("\n")
    }
    
    
    ##
    ##
    ## (5) Print results for individual studies
    ##
    ##
    TE.c    <- x$comparison.nma.common$TE
    lowTE.c <- x$comparison.nma.common$lower
    uppTE.c <- x$comparison.nma.common$upper
    ##
    if (backtransf) {
      TE.c    <- backtransf(TE.c, sm)
      lowTE.c <- backtransf(lowTE.c, sm)
      uppTE.c <- backtransf(uppTE.c, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- lowTE.c
        lowTE.c <- uppTE.c
        uppTE.c <- tmp.l
      }
    }
    #
    TE.r    <- x$comparison.nma.random$TE
    lowTE.r <- x$comparison.nma.random$lower
    uppTE.r <- x$comparison.nma.random$upper
    ##
    if (backtransf) {
      TE.r    <- backtransf(TE.r, sm)
      lowTE.r <- backtransf(lowTE.r, sm)
      uppTE.r <- backtransf(uppTE.r, sm)
      #
      # Switch lower and upper limit for VE if results have been
      # backtransformed
      #
      if (sm == "VE") {
        tmp.l <- lowTE.r
        lowTE.r <- uppTE.r
        uppTE.r <- tmp.l
      }
    }
    ##
    res.c <- cbind(treat1, treat2,
                   formatN(TE.c, digits, text.NA = "NA", big.mark = big.mark),
                   formatCI(formatN(round(lowTE.c, digits), digits, "NA",
                                    big.mark = big.mark),
                            formatN(round(uppTE.c, digits), digits, "NA",
                                    big.mark = big.mark)),
                   if (common)
                     formatN(round(x$x$Q.common, digits.Q), digits.Q, "NA",
                             big.mark = big.mark),
                   if (common & !all(x$x$narms > 2))
                     formatN(round(x$x$leverage.common, 2), 2, ".")
                   )
    dimnames(res.c) <-
      list(studlab.abbr,
           c("treat1", "treat2",
             sm.lab, ci.lab,
             if (common) "Q",
             if (common & !all(x$x$narms > 2)) "leverage"))
    #
    res.c[, "treat1"] <-
      as.character(factor(res.c[, "treat1"],
                          levels = trts, labels = trts.abbr))
    #
    res.c[, "treat2"] <-
      as.character(factor(res.c[, "treat2"],
                          levels = trts, labels = trts.abbr))
    #
    res.r <- cbind(treat1, treat2,
                   formatN(TE.r, digits, text.NA = "NA", big.mark = big.mark),
                   formatCI(formatN(round(lowTE.r, digits), digits, "NA",
                                    big.mark = big.mark),
                            formatN(round(uppTE.r, digits), digits, "NA",
                                    big.mark = big.mark)))
    dimnames(res.r) <-
      list(studlab.abbr,
           c("treat1", "treat2", sm.lab, ci.lab))
    #
    res.r[, "treat1"] <-
      as.character(factor(res.r[, "treat1"],
                          levels = trts, labels = trts.abbr))
    #
    res.r[, "treat2"] <-
      as.character(factor(res.r[, "treat2"],
                          levels = trts, labels = trts.abbr))
    #
    if (common) {
      cat("Results (common effects model):\n\n")
      ##
      if (!missing.truncate)
        res.c <- res.c[truncate, , drop = FALSE]
      ##
      prmatrix(res.c[order(sortvar), , drop = FALSE],
               quote = FALSE, right = TRUE)
      if (!missing.truncate)
        cat(text.truncate, "\n")      
      cat("\n")
    }
    ##
    if (random) {
      cat("Results (random effects model):\n\n")
      ##
      if (!missing.truncate)
        res.r <- res.r[truncate, , drop = FALSE]
      ##
      prmatrix(res.r[order(sortvar), , drop = FALSE],
               quote = FALSE, right = TRUE)
      if (!missing.truncate)
        cat(text.truncate, "\n")      
      cat("\n")
    }
  }
  ##  
  if (reference.group != "" & missing(all.treatments))
    all.treatments <- FALSE
  ##  
  if (reference.group != "")
    reference.group <- setref(reference.group, rownames(x$x$A.matrix))
  ##  
  if (nma) {
    print.netmeta(x$x,
                  common = common, random = random,
                  prediction = prediction,
                  backtransf = backtransf,
                  overall.hetstat = overall.hetstat,
                  reference.group = reference.group,
                  baseline.reference = baseline.reference,
                  all.treatments = all.treatments,
                  header = FALSE, nchar.trts = nchar.trts,
                  ##
                  digits = digits,
                  digits.pval.Q = digits.pval.Q,
                  digits.Q = digits.Q,
                  digits.tau2 = digits.tau2,
                  digits.I2 = digits.I2,
                  #
                  big.mark = big.mark,
                  scientific.pval = scientific.pval,
                  zero.pval = zero.pval,
                  JAMA.pval = JAMA.pval,
                  #
                  print.tau2 = print.tau2,
                  print.tau = print.tau,
                  print.Q = print.Q,
                  print.I2 = print.I2,
                  print.I2.ci = print.I2.ci,
                  #
                  details.methods = details.methods,
                  legend = legend)
  }
  else {
    ##
    ## Add legend with abbreviated treatment or study labels
    ##
    legendabbr(trts, trts.abbr, legend)
    legendabbr(studies.tr, labels.s, legend,
               text = "Study label",
               header = if (!any(trts != trts.abbr)) "\nLegend:\n" else "\n")
  }
  
  
  invisible(NULL)
}

#' Forest plot for direct and indirect evidence
#' 
#' @description
#' Forest plot to show direct and indirect evidence in network
#' meta-analysis. Furthermore, estimates from network meta-analysis
#' as well as prediction intervals can be printed.
#'
#' @aliases forest.netsplit
#' 
#' @param x An object of class \code{netsplit}.
#' @param pooled A character string indicating whether results for the
#'   common (\code{"common"}) or random effects model (\code{"random"})
#'   should be plotted. Can be abbreviated.
#' @param show A character string indicating which comparisons should
#'   be printed (see Details).
#' @param overall A logical indicating whether network meta-analysis
#'   estimates should be printed.
#' @param direct A logical indicating whether direct estimates should
#'   be printed.
#' @param indirect A logical indicating whether indirect estimates
#'   should be printed.
#' @param prediction A logical indicating whether prediction intervals
#'   should be printed.
#' @param only.reference A logical indicating whether only comparisons
#'   with the reference group should be printed.
#' @param sortvar An optional vector used to sort comparisons (must be
#'   of same length as the total number of comparisons).
#' @param subset An optional logical vector specifying a subset of
#'   comparisons to print (must be of same length as the total number of
#'   comparisons).
#' @param subgroup A character string indicating which layout should
#'   be used in forest plot: subgroups by comparisons
#'   (\code{"comparison"}) or subgroups by estimates
#'   (\code{"estimate"}). Can be abbreviated.
#' @param text.overall A character string used in the plot to label
#'   the network estimates.
#' @param text.direct A character string used in the plot to label the
#'   direct estimates.
#' @param text.indirect A character string used in the plot to label
#'   the indirect estimates.
#' @param text.predict A character string used in the plot to label
#'   the prediction interval.
#' @param type.overall A character string specifying how to plot
#'   treatment effects and confidence intervals for the overall
#'   network evidence.
#' @param type.direct A character string specifying how to plot
#'   treatment effects and confidence intervals for the direct
#'   evidence.
#' @param type.indirect A character string specifying how to plot
#'   treatment effects and confidence intervals for the indirect
#'   evidence.
#' @param col.square The colour for squares.
#' @param col.square.lines The colour for the outer lines of squares.
#' @param col.inside The colour for results and confidence limits if
#'   confidence limits are completely within squares squares.
#' @param col.diamond The colour of diamonds.
#' @param col.diamond.lines The colour of the outer lines of diamonds.
#' @param col.predict Background colour of prediction intervals.
#' @param col.predict.lines Colour of outer lines of prediction
#'   intervals.
#' @param col.subgroup The colour to print information on subgroups, i.e.,
#'   pairwise comparisons.
#' @param equal.size A logical indicating whether all squares should
#'   be of equal size. Otherwise, the square size is proportional to
#'   the precision of estimates.
#' @param leftcols A character vector specifying columns to be plotted
#'   on the left side of the forest plot (see Details).
#' @param leftlabs A character vector specifying labels for columns on
#'   left side of the forest plot.
#' @param rightcols A character vector specifying columns to be
#'   plotted on the right side of the forest plot (see Details).
#' @param rightlabs A character vector specifying labels for columns
#'   on right side of the forest plot.
#' @param digits Minimal number of significant digits for treatment
#'   effects and confidence intervals, see \code{print.default}.
#' @param digits.prop Minimal number of significant digits for the
#'   direct evidence proportion.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param lab.NA A character string to label missing values.
#' @param smlab A label printed at top of figure. By default, text
#'   indicating either common or random effects model is printed.
#' @param \dots Additional arguments for \code{\link[meta]{forest.meta}}
#'   function.
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window.
#' 
#' The arguments \code{leftcols} and \code{rightcols} can be used to
#' specify columns which are plotted on the left and right side of the
#' forest plot, respectively. If argument \code{rightcols} is
#' \code{FALSE}, no columns will be plotted on the right side.
#' 
#' If direct estimates are included in the forest plot (\code{direct =
#' TRUE}, default), the following columns will be printed on the left
#' side of the forest plot: the comparisons (column \code{"studlab"}
#' in \code{\link[meta]{forest.meta}}), number of pairwise comparisons
#' (\code{"k"}), direct evidence proportion (\code{"k"}), and
#' I\eqn{^2} from pairwise comparison (\code{"I2"}).
#' 
#' If direct estimates are not included in the forest plot
#' (\code{direct = FALSE}), only the comparisons (\code{"studlab"})
#' are printed on the left side of the forest plot.
#' 
#' For more information see help page of \code{\link[meta]{forest.meta}}
#' function.
#' 
#' Argument \code{show} determines which comparisons are printed:
#' \tabular{ll}{
#' \dQuote{all} \tab All comparisons \cr
#' \dQuote{both} \tab Only comparisons contributing both direct and
#'   indirect evidence \cr
#' \dQuote{with.direct} \tab Comparisons providing direct evidence \cr
#' \dQuote{direct.only} \tab Comparisons providing only direct
#'   evidence \cr
#' \dQuote{indirect.only} \tab Comparisons providing only indirect
#'   evidence
#' }
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link[meta]{forest.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Examples: example(netsplit)
#' 
#' @method forest netsplit
#' @export

forest.netsplit <- function(x,
                            pooled = ifelse(x$x$random, "random", "common"),
                            show = x$show,
                            #
                            subgroup = "comparison",
                            #
                            overall = x$overall,
                            direct = x$direct,
                            indirect = x$indirect,
                            prediction = x$prediction,
                            #
                            #test = x$test,
                            only.reference = x$only.reference,
                            #
                            sortvar = NULL,
                            subset = NULL,
                            ##
                            text.overall = "Network estimate",
                            text.direct = "Direct estimate",
                            text.indirect = "Indirect estimate",
                            text.predict = "Prediction interval",
                            ##
                            type.overall,
                            type.direct,
                            type.indirect,
                            ##
                            col.square = "gray",
                            col.square.lines = col.square,
                            col.inside = "white",
                            col.diamond = "gray",
                            col.diamond.lines = "black",
                            col.predict = "red",
                            col.predict.lines = "black",
                            col.subgroup = "black",
                            ##
                            equal.size = TRUE,
                            ##
                            leftcols,
                            leftlabs,
                            rightcols = c("effect", "ci"),
                            rightlabs = NULL,
                            ##
                            digits = gs("digits.forest"),
                            digits.prop = max(gs("digits.pval") - 2, 2),
                            ##
                            backtransf = x$backtransf,
                            lab.NA = "",
                            smlab,
                            ...) {


  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netsplit")
  x <- updateversion(x)
  ##
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  ##
  subgroup <- setchar(subgroup, c("comparison", "estimate"))
  ##
  chklogical(overall)
  chklogical(direct)
  chklogical(indirect)
  chklogical(prediction)
  ##
  missing.only.reference <- missing(only.reference)
  if (!missing.only.reference)
    chklogical(only.reference)
  #
  # Catch sortvar and subset from data:
  #
  mc <- match.call()
  sfsp <- sys.frame(sys.parent())
  #
  error <- try(sortvar.x <- catch("sortvar", mc, x, sfsp), silent = TRUE)
  if (!any(class(error) == "try-error"))
    sortvar <- sortvar.x
  ##
  if (!is.null(sortvar)) {
    if (length(dim(sortvar)) == 2) {
      if (dim(sortvar)[1] != length(x$comparison))
        stop("Argument 'sortvar' must be of length ",
             length(x$comparison), ".",
             call. = FALSE)
      ##
      ## Set proportions to 0 or 1
      ##
      if (is.numeric(sortvar)) {
        sortvar[is_zero(abs(sortvar), n = 1000)] <- 0
        sortvar[is_zero(1 - abs(sortvar), n = 1000)] <-
          1 * sign(sortvar)[is_zero(1 - abs(sortvar), n = 1000)]
      }
      sortvar <- order(do.call(order, as.list(as.data.frame(sortvar))))
    }
    else
      chklength(sortvar, length(x$comparison),
                text = paste0("Argument 'sortvar' must be of length ",
                              length(x$comparison), "."))
    ##
    if (!is.numeric(sortvar))
      sortvar <- setchar(sortvar, x$comparison)
  }
  #
  error <- try(subset.x <- catch("subset", mc, x, sfsp), silent = TRUE)
  if (!any(class(error) == "try-error"))
    subset <- subset.x
  ##
  if (!is.null(subset)) {
    chklength(subset, length(x$comparison),
              text = paste0("Argument 'subset' must be of length ",
                            length(x$comparison), "."))
    if (!is.logical(subset))
      stop("Argument 'subset' must be a logical vector.")
  }
  ##
  chkchar(text.overall)
  chkchar(text.direct)
  chkchar(text.indirect)
  chkchar(text.predict)
  ##
  missing.type.overall <- missing(type.overall)
  if (missing.type.overall)
    type.overall <- "diamond"
  else
    type.overall <- setchar(type.overall, c("diamond", "square"))
  ##
  if (missing(type.direct))
    type.direct <- "square"
  else
    type.direct <- setchar(type.direct, c("diamond", "square"))
  if (missing(type.indirect))
    type.indirect <- "square"
  else
    type.indirect <- setchar(type.indirect, c("diamond", "square"))
  ##
  chkcolor(col.square)
  chkcolor(col.square.lines)
  chkcolor(col.inside)
  chkcolor(col.diamond)
  chkcolor(col.diamond.lines)
  chkcolor(col.predict)
  chkcolor(col.predict.lines)
  chkcolor(col.subgroup)
  ##
  chklogical(equal.size)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  chklogical(backtransf)
  ##
  chkchar(lab.NA)
  ##
  if (pooled == "common") {
    if (!(missing(prediction)) & prediction)
      warning("Prediction intervals not shown for estimates ",
              "from common effects model.")
    prediction <- FALSE
  }
  ##
  if (!any(c(overall, direct, indirect)))
    stop("At least, one of the following estimates ",
         "must be included in forest plot:\n",
         "- network estimates (argument 'overall')\n",
         "- direct estimates (argument 'direct')\n",
         "- indirect estimates (argument 'indirect')")
  ##
  missing.leftcols <- missing(leftcols)
  if (missing.leftcols)
    if (direct)
      leftcols <- c("studlab", "k", "prop", "I2")
    else
      leftcols <- "studlab"
  ##
  missing.leftlabs <- missing(leftlabs)
  if (missing.leftlabs) {
    leftlabs <- rep(NA, length(leftcols))
    leftlabs[leftcols == "studlab"] <- "Comparison"
    leftlabs[leftcols == "k"] <- "Number of\nStudies"
    leftlabs[leftcols == "n"] <- "Number of\nParticipants"
    leftlabs[leftcols == "prop"] <- "Direct\nEvidence"
    leftlabs[leftcols == "I2"] <- "I2"
    leftlabs[leftcols == "tau2"] <- "tau2"
    leftlabs[leftcols == "tau"] <- "tau"
    leftlabs[leftcols == "Q"] <- "Q"
  }
  ##
  n.subgroup <- direct + indirect + overall + prediction
  missing.smlab <- missing(smlab)
  ##
  if (n.subgroup == 1 & overall & missing.type.overall)
    type.overall <- "square"
  ##
  if (missing(text.predict))
    if (!(length(x$level.predict) == 0) &&
        x$level.ma != x$level.predict)
      text.predict <- paste0(text.predict, " (",
                             round(x$level.predict * 100), "% PI)")
  ##
  if (overall & n.subgroup > 1) {
    if (text.overall == text.predict)
      stop("Text must be different for arguments 'text.overall' and ",
           "'text.predict'.")
    if (text.overall == text.direct)
      stop("Text must be different for arguments 'text.overall' and ",
           "'text.direct'.")
    if (text.overall == text.indirect)
      stop("Text must be different for arguments 'text.overall' and ",
           "'text.indirect'.")
  }
  ##
  if (prediction & n.subgroup > 1) {
    if (text.predict == text.overall)
      stop("Text must be different for arguments 'text.predict' and ",
           "'text.overall'.")
    if (text.predict == text.direct)
      stop("Text must be different for arguments 'text.predict' and ",
           "'text.direct'.")
    if (text.predict == text.indirect)
      stop("Text must be different for arguments 'text.predict' and ",
           "'text.indirect'.")
  }
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  ## Check whether first argument is a list. In this case only use
  ## this list as input.
  if (length(args) > 0 && is.list(args[[1]]))
    args <- args[[1]]
  ##
  additional.arguments <- names(args)
  ##
  if (length(additional.arguments) > 0) {
    if (!is.na(charmatch("showa", additional.arguments)))
      if (!missing(show))
        warning("Deprecated argument 'showall' ignored as ",
                "argument 'show' is also provided.")
      else {
        warning("Deprecated argument 'showall' has been replaced by ",
                "argument 'show'.")
        show <- args[[charmatch("showa", additional.arguments)]]
        if (show)
          show <- "all"
        else
          show <- "both"
      }
  }
  ##
  show <- setchar(show, c("all", "both", "with.direct",
                          "direct.only", "indirect.only",
                          "reference.only"))
  ##
  if (show == "reference.only") {
    warning("Argument 'show = \"reference.only\" replaced with ",
            "'only.reference = TRUE'.",
            call. = FALSE)
    show <- "both"
    if (missing.only.reference)
      only.reference <- TRUE
  }
  
  
  ##
  ##
  ## (2) Extract results for common and random effects model
  ##
  ##
  if (pooled == "common") {
    dat.direct <- x$direct.common
    ##
    dat.indirect <- x$indirect.common
    dat.indirect$Q <- dat.indirect$tau2 <-
      dat.indirect$tau <- dat.indirect$I2 <- NA
    ##
    dat.overall <- x$common
    dat.overall$Q <- dat.overall$tau2 <-
      dat.overall$tau <- dat.overall$I2 <- NA
    ##
    dat.direct$prop <- formatPT(x$prop.common, digits = digits.prop)
    dat.indirect$prop <- NA
    dat.overall$prop <- NA
    ##
    if (missing.smlab)
      smlab <- "Common Effects Model"
  }
  else {
    dat.direct <- x$direct.random
    ##
    dat.indirect <- x$indirect.random
    dat.indirect$Q <- dat.indirect$tau2 <-
      dat.indirect$tau <- dat.indirect$I2 <- NA
    ##
    dat.overall <- x$random
    dat.overall$Q <- dat.overall$tau2 <-
      dat.overall$tau <- dat.overall$I2 <- NA
    ##
    dat.direct$prop <- formatPT(x$prop.random, digits = digits.prop)
    dat.indirect$prop <- NA
    dat.overall$prop <- NA
    ##
    if (missing.smlab)
      smlab <- "Random Effects Model"
  }
  ##
  if (missing.smlab & n.subgroup == 1)
    smlab <- paste0(if (direct)
      paste0(text.direct, "\n"),
      if (indirect)
        paste0(text.indirect, "\n"),
      if (overall)
        paste0(text.overall, "\n"),
      "(",
      tolower(smlab),
      ")")
  ##
  dat.predict <- x$predict
  ##
  if ( (subgroup == "estimate") | (prediction & !overall) )
    dat.predict$TE <- dat.overall$TE
  else
    dat.predict$TE <- NA
  ##
  dat.predict$seTE <- dat.predict$statistic <- dat.predict$p <-
    dat.predict$Q <- NA
  ##
  dat.predict <- dat.predict[, c("comparison", "TE", "seTE",
                                 "lower", "upper", "statistic", "p", "Q")]
  dat.predict$tau2 <- NA
  dat.predict$tau <- NA
  dat.predict$I2 <- NA
  dat.predict$prop <- NA
  ##
  dat.direct$comps <- dat.indirect$comps <-
    dat.overall$comps <- dat.predict$comps <- x$comparison
  ##
  dat.direct$k <- x$k
  dat.indirect$k <- dat.overall$k <- dat.predict$k <- NA
  ##
  dat.indirect$n <- dat.overall$n <- dat.predict$n <- NA
  ##
  dat.direct$evidence   <- text.direct
  dat.indirect$evidence <- text.indirect
  dat.overall$evidence  <- text.overall
  dat.predict$evidence  <- text.predict
  ##
  dat.direct$type.study <- type.direct
  dat.indirect$type.study <- type.indirect
  dat.overall$type.study <- type.overall
  dat.predict$type.study <- "predict"
  ##
  dat.direct$col.estimate <- if (type.direct == "square")
                               col.square
                             else
                               col.diamond
  dat.indirect$col.estimate <- if (type.indirect == "square")
                                 col.square
                               else
                                 col.diamond
  dat.overall$col.estimate <- if (type.overall == "square")
                                col.square
                              else
                                col.diamond
  ##
  dat.direct$col.lines <- if (type.direct == "square")
                            col.square.lines
                          else
                            col.diamond.lines
  dat.indirect$col.lines <- if (type.indirect == "square")
                              col.square.lines
                            else
                              col.diamond
  dat.overall$col.lines <- if (type.overall == "square")
                             col.square.lines
                           else
                             col.diamond.lines
  ##
  dat.predict$col.estimate <- col.predict
  dat.predict$col.lines <- col.predict.lines
  ##
  ## col.square.lines = col.square,
  ## col.inside = "white",
  ## col.diamond.lines = "black",
  ## col.predict.lines = "black",
  
  
  ##
  ##
  ## (3) Select treatment comparisons to show in forest plot
  ##
  ##
  if (show == "all")
    sel <- rep_len(TRUE, length(x$direct.common$TE))
  else if (show == "with.direct")
    sel <- (!is.na(x$direct.common$TE) & !is.na(x$direct.random$TE))
  else if (show == "both")
    sel <- (!is.na(x$direct.common$TE)  & !is.na(x$indirect.common$TE) &
            !is.na(x$direct.random$TE) & !is.na(x$indirect.random$TE))
  else if (show == "direct.only")
    sel <- (!is.na(x$direct.common$TE)  & is.na(x$indirect.common$TE) &
            !is.na(x$direct.random$TE) & is.na(x$indirect.random$TE))
  else if (show == "indirect.only")
    sel <- (is.na(x$direct.common$TE)  & !is.na(x$indirect.common$TE) &
            is.na(x$direct.random$TE) & !is.na(x$indirect.random$TE))
  ##
  if (only.reference) {
    if (x$reference.group == "") {
      warning("First treatment used as reference as argument ",
              "'reference.group' was unspecified in netsplit().",
              call. = FALSE)
      x$reference.group <-
        compsplit(x$comparison, x$sep.trts)[[1]][1]
    }
    ##
    sel.ref <-
      apply(!is.na(sapply(compsplit(x$comparison, x$sep.trts),
                          match, x$reference.group)), 2, sum) >= 1
    ##
    sel <- sel & sel.ref
  }
  ##
  dat.direct <- dat.direct[sel, ]
  dat.indirect <- dat.indirect[sel, ]
  dat.overall <- dat.overall[sel, ]
  dat.predict <- dat.predict[sel, ]
  #
  if (!is.null(sortvar))
    sortvar <- sortvar[sel]
  #
  if (!is.null(subset))
    subset <- subset[sel]
  #
  if (!is.null(sortvar)) {
    o <- order(sortvar)
    ##
    dat.direct <- dat.direct[o, ]
    dat.indirect <- dat.indirect[o, ]
    dat.overall <- dat.overall[o, ]
    dat.predict <- dat.predict[o, ]
    #
    if (!is.null(subset))
      subset <- subset[o]
  }
  #
  if (!is.null(subset)) {
    dat.direct <- dat.direct[subset, ]
    dat.indirect <- subset(dat.indirect, subset)
    dat.overall <- subset(dat.overall, subset)
    dat.predict <- subset(dat.predict, subset)
  }
  ##
  if (direct & all(is.na(dat.direct$I2)) & missing.leftcols) {
    leftlabs <- leftlabs[leftcols != "I2"]
    leftcols <- leftcols[leftcols != "I2"]
  }
  
  
  ##
  ##
  ## (4) Forest plot
  ##
  ##
  if (subgroup == "comparison") {
    dat <- rbind(
      if (direct) dat.direct,
      if (indirect) dat.indirect,
      if (overall) dat.overall,
      if (prediction) dat.predict
    )
    ##
    if (nrow(dat) == 0) {
      warning("No comparison(s) selected. Consider using argument ",
              "'show = \"all\"'.")
      return(invisible(NULL))
    }
    ##
    if (n.subgroup > 1)
      m <-
        suppressWarnings(metagen(dat$TE, dat$seTE,
                                 studlab = dat$evidence, data = dat,
                                 sm = x$sm,
                                 common = FALSE, random = FALSE,
                                 method.tau = "DL", method.tau.ci = "",
                                 subgroup = dat$comps,
                                 print.subgroup.name = FALSE))
    else
      m <-
        suppressWarnings(metagen(dat$TE, dat$seTE,
                                 studlab = dat$comps, data = dat, sm = x$sm,
                                 common = FALSE, random = FALSE,
                                 method.tau = "DL", method.tau.ci = ""))
    ##
    if (overall) {
      m$w.common[m$studlab == text.overall] <- max(m$w.common, na.rm = TRUE)
      m$w.random[m$studlab == text.overall] <- max(m$w.random, na.rm = TRUE)
    }
    ##
    if (prediction) {
      m$lower[m$studlab == text.predict] <- dat.predict$lower
      m$upper[m$studlab == text.predict] <- dat.predict$upper
      ##
      m$w.common[m$studlab == text.predict] <- max(m$w.common, na.rm = TRUE)
      m$w.random[m$studlab == text.predict] <- max(m$w.random, na.rm = TRUE)
    }
    ##
    forest(m,
           digits = digits,
           common = FALSE, random = FALSE,
           hetstat = FALSE, test.subgroup = FALSE,
           leftcols = leftcols,
           leftlabs = leftlabs,
           rightcols = rightcols,
           rightlabs = rightlabs,
           lab.NA = lab.NA,
           smlab = smlab,
           backtransf = backtransf,
           type.study = dat$type.study,
           col.square = dat$col.estimate,
           col.square.lines = dat$col.lines,
           weight.study = if (equal.size) "same" else "common",
           ...)
  }
  else {
    dat <- rbind(
      if (direct) dat.direct,
      if (indirect) dat.indirect,
      if (overall) dat.overall,
      if (prediction) dat.predict
    )
    ##
    if (nrow(dat) == 0) {
      warning("No comparison(s) selected. Consider using argument ",
              "'show = \"all\"'.")
      return(invisible(NULL))
    }
    ##
    if (n.subgroup > 1)
      m <-
        suppressWarnings(metagen(dat$TE, dat$seTE,
                                 studlab = dat$comps, data = dat,
                                 sm = x$sm,
                                 common = FALSE, random = FALSE,
                                 method.tau = "DL", method.tau.ci = "",
                                 subgroup = dat$evidence,
                                 print.subgroup.name = FALSE))
    else
      m <-
        suppressWarnings(metagen(dat$TE, dat$seTE,
                                 studlab = dat$comps, data = dat, sm = x$sm,
                                 common = FALSE, random = FALSE,
                                 method.tau = "DL", method.tau.ci = ""))
    ##
    if (overall) {
      m$w.common[m$subgroup == text.overall] <- max(m$w.common, na.rm = TRUE)
      m$w.random[m$subgroup == text.overall] <- max(m$w.random, na.rm = TRUE)
    }
    ##
    if (prediction) {
      m$lower[m$subgroup == text.predict] <- dat.predict$lower
      m$upper[m$subgroup == text.predict] <- dat.predict$upper
      ##
      m$w.common[m$subgroup == text.predict] <- max(m$w.common, na.rm = TRUE)
      m$w.random[m$subgroup == text.predict] <- max(m$w.random, na.rm = TRUE)
    }
    ##
    forest(m,
           digits = digits,
           ##
           overall = FALSE, common = FALSE, random = FALSE,
           hetstat = FALSE, test.subgroup = FALSE,
           ##
           subgroup.hetstat = FALSE,
           prediction.subgroup = FALSE,
           ##
           leftcols = leftcols,
           leftlabs = leftlabs,
           rightcols = rightcols,
           rightlabs = rightlabs,
           ##
           lab.NA = lab.NA,
           smlab = smlab,
           backtransf = backtransf,
           ##
           type.study = dat$type.study,
           col.square = dat$col.estimate,
           col.square.lines = dat$col.lines,
           ##
           weight.study = if (equal.size) "same" else "common",
           ...)
  }


  res <- list(dat = dat,
              leftcols = leftcols,
              leftlabs = leftlabs,
              rightcols = rightcols,
              rightlabs = rightlabs,
              lab.NA = lab.NA,
              backtransf = backtransf,
              smlab = smlab,
              weight.study = if (equal.size) "same" else "common",
              ##
              args = list(
                pooled = pooled,
                show = show,
                subgroup = subgroup,
                overall = overall,
                direct = direct,
                indirect = indirect,
                prediction = prediction,
                only.reference = only.reference,
                sortvar = sortvar,
                subset = subset,
                text.overall = text.overall,
                text.direct = text.direct,
                text.indirect = text.indirect,
                text.predict = text.predict,
                type.overall = type.overall,
                type.direct = type.direct,
                type.indirect = type.indirect,
                col.square = col.square,
                col.square.lines = col.square.lines,
                col.inside = col.inside,
                col.diamond = col.diamond,
                col.diamond.lines = col.diamond.lines,
                col.predict = col.predict,
                col.predict.lines = col.predict.lines,
                col.subgroup = col.subgroup,
                equal.size = equal.size,
                digits = digits,
                digits.prop = digits.prop)
              )

  
  invisible(res)
}

shadowtext <- function(x, y = NULL, labels,
                       col = 'white', bg = 'black',
                       theta = seq(pi / 4, 2 * pi, length.out = 8),
                       r = 0.1, ...) {

  ##
  ## R function from TeachingDemos package by Greg Snow
  ##
  
  xy <- xy.coords(x, y)
  xo <- r * strwidth('A')
  yo <- r * strheight('A')
  
  for (i in theta)
    text(xy$x + cos(i) * xo, xy$y + sin(i) * yo, labels, col = bg, ...)
  
  text(xy$x, xy$y, labels, col = col, ...)
}

# Gives contributions of individual studies for a single comparison
# arguments hatmatrix list from getHatMatrix and comparison the string
# identifying the hatmatrix row separated by ":"

getStudyContribution <- function(x, contribution.Matrix, comparison, model) {
  contribution <- contribution.Matrix[comparison, ]
  #
  if (model == "random")
    tau <- x$tau
  else
    tau <- 0
  # main data frame
  dfr <-
    data.frame(studlab = x$studlab,
               treat1 = x$treat1, treat2 = x$treat2,
               seTE.adj = x$seTE.adj)
  dfr$comp <- paste(dfr$treat1, dfr$treat2, sep = ":")
  dfr$w.adj <- 1 / (dfr$seTE.adj^2 + tau^2)
  
  studyContribution <- function(direct){
    aux <- dfr[dfr$comp == direct, ]
    normfac <- sum(aux[, "w.adj"])
    per <- contribution[direct]
    studycontr <-
      lapply(row.names(aux),
             function(row){
               w <- aux[row, "w.adj"]
               out <- data.frame(study = "", contribution = 0, comparison = "")
               out$study <- aux[row,"studlab"]
               out$contribution <- w * per / normfac
               out$comparison <- direct
               #
               out
             })
    #
    studycontr
  }
  
  outlist <- Reduce(
    function(acc, col) c(acc, studyContribution(col)),
  names(contribution),
  list(),
  accumulate = FALSE)
  
  SCR <- data.frame(
    study = sapply(outlist, function(r) r$study),
    contribution = sapply(outlist, function(r) r$contribution),
    comparison = sapply(outlist, function(r) r$comparison))
  #
  studyRow <-
    aggregate(SCR$contribution, by = list(study = SCR$study), FUN = sum)
  #
  studyRow <- cbind(comparison, studyRow) %>% rename(contribution = x)
  #
  studyRow
}

#' Calculate distance matrix for an adjacency matrix
#' 
#' @description
#' Calculate distance matrix for an adjacency matrix based on distance
#' algorithm by Müller et al. (1987).
#' 
#' @aliases netdistance netdistance.default netdistance.netmeta netdistance.netcomb print.netdistance
#' 
#' @param x Either a netmeta or netcomb object or an adjacency matrix.
#' @param sort A logical indicating whether to sort within (sub)networks by
#'   treatment names.
#' @param lab.Inf A character string to label infinite values.
#' @param \dots Additional arguments.
#'
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#'   Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netconnection}}
#' 
#' @references
#' Müller WR, Szymanski K, Knop JV, and Trinajstic N (1987):
#' An algorithm for construction of the molecular distance matrix.
#' \emph{Journal of Computational Chemistry},
#' \bold{8}, 170--73
#' 
#' @examples
#' data(smokingcessation)
#' 
#' pw1 <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' nma1 <- netmeta(pw1, common = FALSE)
#' 
#' netdistance(nma1)
#' 
#' \donttest{
#' pw2 <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' nma2 <- netmeta(pw2)
#' 
#' netdistance(nma2)
#' netdistance(nma2$A.matrix)
#' }
#' 
#' @rdname netdistance
#' @method netdistance default
#' @export

netdistance.default <- function(x, ...) {
  
  # Calculate distance matrix D of adjacency matrix A based on
  # distance algorithm by Mueller et al. (1987) using triangle
  # inequality
  
  chkclass(x, "matrix")
  #
  A <- x
  
  # Starting value for D is sign(A), with 0 replaced by Inf
  #
  n <- nrow(A)
  D <- sign(A)
  #
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      if (D[i, j] == 0) {
        D[i, j] <- Inf
        D[j, i] <- Inf
      }
    }
  }
  #
  for (d in 1:(n - 1)) {
    for (i in 1:n) {
      for (j in 1:n) {
        if (D[i, j] == d) {
          for (k in 1:n) {
            akj <- D[k, i] + d # = D[k, i] + D[i, j]
            D[k, j] <- min(D[k, j], akj)
          }
        }
      }
    }
  }
  #
  maxdist <- nrow(D)
  D2 <- D
  D2[is.infinite(D2)] <- maxdist
  attr(D, "order") <- hclust(dist(D2))$order
  
  class(D) <- c("netdistance", class(D))
  #
  D
}


#' @rdname netdistance
#' @method netdistance netmeta
#' @export

netdistance.netmeta <- function(x, sort = gs("sort.distance"), ...) {
  
  chkclass(x, "netmeta")
  chklogical(sort)
  
  A <- x$A.matrix
  #
  if (sort) {
    seq <- netconnection(x$treat1, x$treat2)$seq
    A <- A[seq, seq]
  }
  
  res <- netdistance(A)
  #
  if (sort)
    attr(res, "order") <- NULL
  #
  res
}


#' @rdname netdistance
#' @method netdistance netcomb
#' @export

netdistance.netcomb <- function(x, sort = gs("sort.distance"), ...) {
  
  chkclass(x, "netcomb")
  chklogical(sort)
  
  if (inherits(x, "discomb")) {
    A <- x$A.matrix
    #
    if (sort)
      seq <- netconnection(x$treat1, x$treat2)$seq
  }
  else {
    A <- x$x$A.matrix
    #
    if (sort)
      seq <- netconnection(x$x$treat1, x$x$treat2)$seq
  }
  #
  if (sort)
    A <- A[seq, seq]
  
  res <- netdistance(A)
  #
  if (sort)
    attr(res, "order") <- NULL
  #
  res
}


#' @rdname netdistance
#' @method netdistance netconnection
#' @export

netdistance.netconnection <- function(x, ...) {
  
  chkclass(x, "netconnection")
    
  netdistance(x$A.matrix)
}


#' @rdname netdistance
#' @method print netdistance
#' @export

print.netdistance <- function(x, lab.Inf = ".", ...) {
  o <- attr(x, "order")
  #
  if (!is.null(o))
    x <- x[o, o]
  #
  x[is.infinite(x)] <- lab.Inf
  #
  prmatrix(x, quote = FALSE, right = TRUE)
  #
  invisible(NULL)
}


#' @rdname netdistance
#' @export

netdistance <- function(x, ...)
  UseMethod("netdistance")

#' Determine the importance of individual studies in network
#' meta-analysis
#' 
#' @description
#' This function measures the importance of individual studies in
#' network meta-analysis by the reduction of the precision if the
#' study is removed / ignored from the network (Rücker et al., 2020).
#' 
#' @param x An object of class \code{netmeta}.
#' @param seTE.ignore Assumed (large) standard error in order to
#'   mimicking the removal of individual studies from the network
#'   meta-analysis (ignored for \code{\link{netmetabin}} objects).
#' @param event.ignore Assumed event number mimicking the removal of
#'   individual studies from the network meta-analysis (considered for
#'   \code{\link{netmetabin}} objects).
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param nchar.studlab A numeric defining the minimum number of
#'   characters used to create unique study labels.
#' @param verbose A logical indicating whether information on the
#'   estimation progress should be printed.
#' 
#' @return
#' An object of class \code{"netimpact"} with corresponding
#' \code{netgraph} and \code{print} function. The object is a list
#' containing the following components:
#' \item{impact.common}{A matrix with contributions of individual
#'   studies (columns) to comparisons (rows) under the common effects
#'   model.}
#' \item{impact.random}{A matrix with contributions of individual
#'   studies (columns) to comparisons (rows) under the random effects
#'   model.}
#' \item{ignored.comparisons}{List with comparisons of ignored study.}
#' \item{seTE.ignore, event.ignore, x}{As defined above.}
#' \item{nets}{List of all network meta-analyses (removing a single
#'   study).}
#' \item{version}{Version of R package netmeta used to create object.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @references
#' Rücker G, Nikolakopoulou A, Papakonstantinou T, Salanti G, Riley
#' RD, Schwarzer G (2020):
#' The statistical importance of a study for a network meta-analysis
#' estimate.
#' \emph{BMC Medical Research Methodology},
#' \bold{20}, 190
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netmetabin}},
#'   \code{\link{netgraph.netimpact}}, \code{\link{print.netimpact}},
#'   \code{\link[metabook]{Franchini2012}}
#' 
#' @examples
#' # Only consider first two studies (to reduce runtime of example)
#' #
#' studies <- unique(Franchini2012$Study)
#' pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
#'   n = list(n1, n2, n3),
#'   mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
#'   data = subset(Franchini2012, Study %in% studies[1:2]),
#'   studlab = Study)
#' 
#' nma1 <- netmeta(pw1)
#' ni1 <- netimpact(nma1, verbose = TRUE)
#' ni1
#' 
#' netgraph(ni1)
#' 
#' \donttest{
#' netgraph(ni1, plastic = TRUE)
#' 
#' pw2 <- pairwise(list(Treatment1, Treatment2, Treatment3),
#'   n = list(n1, n2, n3),
#'   mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
#'   data = Franchini2012,
#'   studlab = Study)
#' 
#' nma2 <- netmeta(pw2)
#' ni2 <- netimpact(nma2, verbose = TRUE)
#' netgraph(ni2)
#' }
#' 
#' @export netimpact

netimpact <- function(x,
                      seTE.ignore = 1000 * max(x$seTE, na.rm = TRUE),
                      event.ignore = 0.01,
                      nchar.trts = x$nchar.trts,
                      nchar.studlab = x$nchar.studlab,
                      verbose = FALSE) {
  
  
  chkclass(x, "netmeta")
  ##
  x <- updateversion(x)
  ##
  chknumeric(seTE.ignore, min = 0, zero = TRUE, length = 1)
  chknumeric(event.ignore, min = 0, zero = TRUE, length = 1)
  #
  chknumeric(nchar.trts, min = 1, length = 1)
  chknumeric(nchar.studlab, min = 1, length = 1)
  
  
  studlab <- x$studlab
  treat1  <- x$treat1
  treat2  <- x$treat2
  TE      <- x$TE
  seTE    <- x$seTE
  ##
  event1 <- x$event1
  event2 <- x$event2
  n1 <- x$n1
  n2 <- x$n2
  ##
  comparison <- paste(treat1, sep = x$sep.trts, treat2)
  
  
  comparisons <- rownames(x$Cov.common)
  ##
  impact.common <- impact.random <-
    matrix(NA, ncol = length(x$studies), nrow = length(comparisons))
  ##
  rownames(impact.common) <- rownames(impact.random) <- comparisons
  colnames(impact.common) <- colnames(impact.random) <- x$studies
  ##
  ## Run network meta-analyses "excluding" one study at a time
  ##
  ignored <- nets <- list()
  ##
  for (i in x$studies) {
    ##
    ignored[[i]] <- comparison[studlab == i]
    ##
    if (verbose) {
      cat("** Removed study: ", i,
          " **\nComparison",
          if (length(comparison[studlab == i]) > 1) "s",
          ": ",
          paste(paste0("'", comparison[studlab == i], "'"), collapse = ", "),
          "\n\n",
          sep = "")
    }
    ##
    if (!inherits(x, "netmetabin")) {
      seTE.i <- seTE
      seTE.i[studlab == i] <- seTE.ignore
      ##
      net.i <- netmeta(TE, seTE.i, treat1, treat2, studlab,
                       tau.preset = x$tau,
                       tol.multiarm = x$tol.multiarm,
                       tol.multiarm.se = x$tol.multiarm.se)
    }
    else {
      event1.i <- event1
      event2.i <- event2
      n1.i <- n1
      n2.i <- n2
      event1.i[studlab == i] <- event.ignore
      event2.i[studlab == i] <- event.ignore
      n1.i[studlab == i] <- 1
      n2.i[studlab == i] <- 1
      ##
      net.i <- netmetabin(event1.i, n1.i, event2.i, n2.i,
                          treat1, treat2, studlab,
                          method = x$method, sm = x$sm)
    }
    nets[[i]] <- net.i
    ##
    seTE.common <- x$seTE.common
    seTE.common.i <- net.i$seTE.common
    ##
    seTE.random <- x$seTE.random
    seTE.random.i <- net.i$seTE.random
    ##
    impact.common.i <- 1 - (lowertri(seTE.common) / lowertri(seTE.common.i))^2
    zero.common <- abs(impact.common.i) < .Machine$double.eps^0.5
    ##
    impact.common[, x$studies == i] <- impact.common.i
    impact.common[zero.common, x$studies == i] <- 0
    ##
    impact.random.i <- 1 - (lowertri(seTE.random) / lowertri(seTE.random.i))^2
    zero.random <- abs(impact.random.i) < .Machine$double.eps^0.5
    ##
    impact.random[, x$studies == i] <- impact.random.i
    impact.random[zero.random, x$studies == i] <- 0
  }
  
  
  res <- list(impact.common = t(impact.common),
              impact.random = t(impact.random),
              ignored.comparisons = ignored,
              seTE.ignore = seTE.ignore,
              #
              x = x,
              nets = nets,
              method.tau = x$method.tau,
              nchar.trts = nchar.trts,
              nchar.studlab = nchar.studlab,
              #
              call = match.call(),
              version = packageDescription("netmeta")$Version)
  ##
  ## Backward compatibility
  ##
  res$impact.fixed <- res$impact.common
  ##
  class(res) <- "netimpact"
  ##
  res
}

#' Create a data frame from an object of class netmeta
#' 
#' @description
#' The \code{as.data.frame} method returns a data frame containing
#' information on individual studies, e.g., estimated treatment effect
#' and its standard error.
#' 
#' @param x An object of class \code{netmeta}.
#' @param row.names \code{NULL} or a character vector giving the row
#'   names for the data frame.
#' @param optional A logical. If \code{TRUE}, setting row names and
#'   converting column names (to syntactic names) is optional.
#' @param details A logical. If \code{TRUE}, additional variables of
#'   less interest are included in data frame.
#' @param \dots Additional arguments.
#' @return A data frame is returned by the function
#'   \code{as.data.frame}.
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' @seealso \code{\link{netmeta}}
#' 
#' @examples
#' \donttest{
#' data(smokingcessation)
#' 
#' # Transform data from arm-based format to contrast-based format
#' #
#' pw <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' 
#' # Conduct random effects network meta-analysis and show data frame
#' #
#' nma <- netmeta(pw, common = FALSE)
#' head(as.data.frame(nma))
#' head(as.data.frame(nma, details = TRUE))
#' }
#'
#' @method as.data.frame netmeta 
#' @export

as.data.frame.netmeta <- function(x, row.names = NULL,
                                  optional = FALSE,
                                  details = FALSE, ...){
  
  
  chkclass(x, "netmeta")
  ##
  x <- updateversion(x)
  
  
  ## Remove element 'call' from object of class meta to get rid
  ## of an error message in meta-analyses with six studies:
  ## 'Error: evaluation nested too deeply: infinite recursion ...'
  ##
  ## NB: Element 'call' which is of length six contains information
  ##     on the function call.
  ##
  x$call <- NULL
  
  sel1 <- as.vector(lapply(x, length) == length(x$studlab))
  sel2 <- as.vector(unlist(lapply(x, is.vector)))
  sel <- sel1 & sel2
  
  res <- as.data.frame(x[names(x)[sel]], ...)
  ##
  res$studies <- NULL
  res$narms <- NULL
  ##
  res$lower.nma.fixed <- NULL
  res$upper.nma.fixed <- NULL
  res$leverage.fixed <- NULL
  
  if (!details)
    res <- res[, !(names(res) %in% c("treat1.pos", "treat2.pos",
                                     "lower.nma.common", "upper.nma.common",
                                     "lower.nma.random", "upper.nma.random",
                                     "leverage.common"))]
  
  attr(res, "version") <- packageDescription("netmeta")$Version
  
  res
}

#' Derive hat matrix from network meta-analysis
#' 
#' @description
#' Auxiliary function to derive hat matrix from network meta-analysis
#' 
#' @param x A \code{\link{netmeta}} object.
#' @param method A character string indicating which method is used to
#'   derive the hat matrix. Either \code{"Ruecker"}, \code{"Krahn"} or
#'   \code{"Davies"} (can be abbreviated, see Details).
#' @param type A character string indicating which specific hat matrix
#'   should be derived (can be abbreviated, see Details).
#' @param common A logical indicating whether a hat matrix should be
#'   printed for the common effects network meta-analysis.
#' @param random A logical indicating whether a hat matrix should be
#'   printed for the random effects network meta-analysis.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param nchar.studlab A numeric defining the minimum number of
#'   characters used to create unique study labels.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param legend.studlab A logical indicating whether a legend should
#'   be printed for abbreviated study labels.
#' @param \dots Additional arguments (ignored).
#' 
#' @details
#' This auxiliary function can be used to derive various hat matrices
#' from a network meta-analysis object.
#' 
#' \subsection{Hat matrix by Rücker (2012)}{
#' 
#' This hat matrix is estimated if \code{method = "Ruecker"}.
#' 
#' Let \emph{n} be the number of different treatments (nodes,
#' vertices) in a network and let \emph{m} be the number of existing
#' comparisons (edges) between the treatments. If there are only
#' two-arm studies, \emph{m} is equal to the number of studies,
#' \emph{k}. Let seTE.adj.common and seTE.adj.random be the vectors of
#' adjusted standard errors under the common and random effects model
#' (see \code{\link{netmeta}}). Let \strong{W} be the \emph{m} x
#' \emph{m} diagonal matrix that contains the inverse variance 1 /
#' seTE.adj.common\eqn{^2} or 1 / seTE.adj.random\eqn{^2}.
#'
#' The given comparisons define the network structure. Therefrom an
#' \emph{m} x \emph{n} design matrix X (edge-vertex incidence matrix) is
#' formed; for more precise information, see Rücker (2012). Moreover,
#' the \emph{n} x \emph{n} Laplacian matrix \strong{L} and its
#' Moore-Penrose pseudoinverse \strong{L}\eqn{^+} are calculated (both
#' matrices play an important role in graph theory and electrical
#' network theory). Using these matrices, the variances based on both
#' direct and indirect comparisons can be estimated. The hat matrix
#' \strong{H} is estimated by \strong{H =
#' XL}\eqn{^+}\strong{X}\eqn{^T}\strong{W = X(X}\eqn{^T}\strong{W
#' X)}\eqn{^+}\strong{X}\eqn{^T}\strong{W}.
#' }
#' 
#' \subsection{Hat matrices by Krahn et al. (2013)}{
#' 
#' One of the following hat matrices is estimated if \code{method
#' = "Krahn"}.
#'
#' Use of \code{type = "design"} (default) results in a hat matrix of
#' dimension \emph{n(n-1)/2} x \emph{s}, where \emph{s} is the sum of
#' the number of independent comparisons from each design.
#'
#' Use of \code{type = "studies"} results in a hat matrix of dimension
#' \emph{n(n-1)/2} x \emph{l}, where \emph{l} is the number of
#' independent pairwise comparisons, i.e., a three-arm study
#' contributes two pairwise comparisons.
#' }
#' 
#' \subsection{Hat matrices by Davies et al. (2022)}{
#' 
#' One of three hat matrices is estimated if \code{method = "Davies"}.
#'
#' Here, we focus on the hat matrix of the aggregate (two-step)
#' version of the graph theoretical NMA model. In the first step, a
#' pairwise meta-analysis is performed across each edge using the
#' adjusted weights (these account for correlations due to multi-arm
#' trials). From this we obtain direct treatment effect estimates (and
#' corresponding aggregate weights) associated with each edge. In step
#' two, we combine these direct estimates in a network meta analysis
#' to obtain the network estimates.  This is done using weighted least
#' squares regression. The hat matrix associated with this second step
#' is called the \emph{aggregate hat matrix}.
#' 
#' All three versions of the aggregate hat matrix contain the same
#' information: the second two can be derived directly from the
#' first. They differ in their dimensionality.
#' 
#' Each row of the hat matrix that represents a treatment comparison
#' (\emph{ij}) describes the flow of evidence through each edge for
#' that comparison. This defines a directed acyclic 'flow graph' from
#' node \emph{i} to node \emph{j}.
#'
#' (1) Use of \code{type = "short"} (default) results in a hat matrix
#' of dimension \emph{e} x \emph{e}, where \emph{e} is the number of
#' (unique) edges (direct comparisons) in the network. This is the
#' aggregate hat matrix described in Davies et al. (2022). Each row
#' and column represents a pair of treatments for which there is at
#' least one direct comparison.
#'
#' (2) Use of \code{type = "long"} results in a hat matrix of
#' dimension \emph{n(n-1)/2} x \emph{e}. There is a row for every
#' possible pair of treatments in the network - regardless of whether
#' there is direct evidence for this comparison. Each column
#' represents a pair of treatments for which there is at least one
#' direct comparison. The extra rows can be calculated from the short
#' hat matrix using consistency equations.
#' 
#'
#' (3) Use of \code{type = "full"} results in a hat matrix of
#' dimension \emph{n(n-1)/2} x \emph{n(n-1)/2}. In comparison to the
#' long hat matrix, columns of zeroes are added for comparisons that
#' do not have any direct evidence. Therefore, there is a row and
#' column for every pair of treatments in the network. This hat matrix
#' is used to calculate the transition matrices for the random walk in
#' \code{\link{netcontrib}}.
#' }
#'
#' @return
#' A list with two hat matrices: \code{common} (common effects model)
#' and \code{random} (random effects model).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netcontrib}},
#'   \code{\link{netheat}}, \code{\link[metabook]{Dong2013}}
#' 
#' @references
#' Davies AL, Papakonstantinou T, Nikolakopoulou A, Rücker G, Galla T
#' (2022):
#' Network meta-analysis and random walks.
#' \emph{Statistics in Medicine},
#' \bold{41}, 2091--2114
#' 
#' Krahn U, Binder H, König J (2013):
#' A graphical tool for locating inconsistency in network meta-analyses.
#' \emph{BMC Medical Research Methodology},
#' \bold{13}, 35
#'
#' Rücker G (2012):
#' Network meta-analysis, electrical networks and graph theory.
#' \emph{Research Synthesis Methods},
#' \bold{3}, 312--24
#' 
#' @examples
#' # Only consider first ten studies for concise output
#' first10 <- subset(Dong2013, id <= 10)
#' pw <- pairwise(treatment, death, randomized, studlab = id,
#'   data = first10, sm = "OR")
#' nma <- netmeta(pw, common = FALSE)
#' 
#' hatmatrix(nma)
#' hatmatrix(nma, method = "k")
#' hatmatrix(nma, method = "k", type = "studies")
#' hatmatrix(nma, method = "d")
#' hatmatrix(nma, method = "d", type = "long")
#' hatmatrix(nma, method = "d", type = "full")
#' 
#' @export hatmatrix

hatmatrix <- function(x, method = "Ruecker", type,
                      common = x$common,
                      random = x$random,
                      nchar.trts = x$nchar.trts,
                      nchar.studlab = x$nchar.studlab) {
  
  chkclass(x, "netmeta")
  x <- updateversion(x)
  ##
  method <- setchar(method, c("Ruecker", "Krahn", "Davies"))
  ##
  if (x$n == 2 & method == "Krahn") {
    warning("Hat matrix by Krahn et al. (2013) not available for ",
            "network meta-analysis with only two treatments.",
            call. = FALSE)
    return(NULL)
  }
  ##
  if (missing(type)) {
    if (method == "Ruecker")
      type <- ""
    else if (method == "Krahn")
      type <- "design"
    else if (method == "Davies")
      type <- "short"
  }
  else {
    if (method == "Ruecker") {
      warning("Argument 'type' ignored for argument method = \"Ruecker\".",
              call. = FALSE)
      type <- ""
    }
    else if (method == "Krahn")
      type <- setchar(type, c("design", "studies"))
    else if (method == "Davies")
      type <- setchar(type, c("short", "long", "full"))
  }
  #
  common <- replaceNULL(common, TRUE)
  chklogical(common)
  #
  random <- replaceNULL(random, TRUE)
  chklogical(random)
  #
  nchar.trts <- replaceNULL(nchar.trts, 666)
  chknumeric(nchar.trts, min = 1, length = 1)
  ##
  nchar.studlab <- replaceNULL(nchar.studlab, 666)
  chknumeric(nchar.studlab, min = 1, length = 1)
  
  
  res <- list()
  ##
  if (method == "Ruecker") {
    res$common <- x$H.matrix.common
    res$random <- x$H.matrix.random
  }
  else if (method == "Krahn") {
    if (type == "design") {
      res$common <- nma_krahn(x)$H
      res$random <- nma_krahn(x, tau.preset = x$tau)$H
    }
    else if (type == "studies") {
      res$common <- nma_krahn(x)$H.studies
      res$random <- nma_krahn(x, tau.preset = x$tau)$H.studies
    }
  }
  else if (method == "Davies") {
    res$common <- hatmatrix.aggr(x, "common", type)
    res$random <- hatmatrix.aggr(x, "random", type)
  }
  ##
  res$method <- method
  res$type <- type
  ##
  res$x <- x
  res$x$common <- common
  res$x$random <- random
  res$x$nchar.trts <- nchar.trts
  res$x$nchar.studlab <- nchar.studlab
  #
  res$call <- match.call()
  res$version <- packageDescription("netmeta")$Version
  #
  class(res) <- "hatmatrix"
  ##
  res
}


#' @rdname hatmatrix
#' @method print hatmatrix
#' @export

print.hatmatrix <- function(x,
                            common = x$x$common,
                            random = x$x$random,
                            nchar.trts = x$x$nchar.trts,
                            nchar.studlab = x$x$nchar.studlab,
                            digits = gs("digits"),
                            legend = gs("legend"),
                            legend.studlab = TRUE,
                            ...) {
  
  chkclass(x, "hatmatrix")
  x <- updateversion(x)
  ##
  chklogical(common)
  common.logical <- common
  chklogical(random)
  random.logical <- random
  ##
  chknumeric(nchar.trts, length = 1)
  nchar.studlab <- replaceNULL(nchar.studlab, 666)
  chknumeric(nchar.studlab, length = 1)
  chknumeric(digits, length = 1)
  chklogical(legend)
  chklogical(legend.studlab)
  ##
  matitle(x$x)
  ##
  cat("Hat matrix (",
      if (x$method == "Ruecker")
        "R\u00FCcker, 2012, Research Synthesis Method"
      else if (x$method == "Krahn")
        "Krahn et al., 2013"
      else if (x$method == "Davies")
        "Davies et al., 2021",
      ")\n\n",
      sep = "")
  #
  trts <- x$x$trts
  sep.trts <- x$x$sep.trts
  anystudy.r <- anystudy.c <- FALSE
  anycomp.r <- anycomp.c <- FALSE
  ##
  common <- x$common
  random <- x$random
  ##
  legend <- legend & (common.logical | random.logical)
  legend.studlab <- legend.studlab & (common.logical | random.logical)
  ##
  if (common.logical) {
    compnames <- any(grepl(sep.trts, rownames(common), fixed = TRUE))
    anystudy.r <- anystudy.r | !compnames
    anycomp.r  <- anycomp.r  | compnames
    ##
    if (compnames)
      rownames(common) <- comps(common, trts, sep.trts, nchar.trts)
    else
      rownames(common) <- treats(common, nchar.studlab)
    ##
    compnames <- any(grepl(sep.trts, colnames(common), fixed = TRUE))
    anystudy.c <- anystudy.c | !compnames
    anycomp.c  <- anycomp.c  | compnames
    ##
    if (compnames)
      colnames(common) <- comps(common, trts, sep.trts, nchar.trts,
                                row = FALSE)
    else
      colnames(common) <- treats(common, nchar.studlab, row = FALSE)
    ##
    cat("Common effects model:\n\n")
    prmatrix(round(common, digits))
    if (random.logical)
      cat("\n")
  }
  if (random.logical) {
    compnames <- any(grepl(sep.trts, rownames(random), fixed = TRUE))
    anystudy.r <- anystudy.r | !compnames
    anycomp.r  <- anycomp.r  | compnames
    ##
    if (compnames)
      rownames(random) <- comps(random, trts, sep.trts, nchar.trts)
    else
      rownames(random) <- treats(random, nchar.studlab)
    ##
    compnames <- any(grepl(sep.trts, colnames(random), fixed = TRUE))
    anystudy.c <- anystudy.c | !compnames
    anycomp.c  <- anycomp.c  | compnames
    ##
    if (compnames)
      colnames(random) <- comps(random, trts, sep.trts, nchar.trts,
                                row = FALSE)
    else
      colnames(random) <- treats(random, nchar.studlab, row = FALSE)
    ##
    cat("Random effects model:\n\n")
    prmatrix(round(random, digits))
  }
  ##
  ## Add legend with abbreviated treatment labels
  ##
  legend <- legendabbr(trts, treats(trts, nchar.trts),
                       legend && (anycomp.r | anycomp.c))
  ##
  ## Add legend with abbreviated study labels
  ##
  if (legend.studlab && (anystudy.r | anystudy.c)) {
    if (anystudy.r) {
      if (common.logical) {
        studlab <- rownames(x$common)
        studlab.abbr <- rownames(common)
      }
      else {
        studlab <- rownames(x$random)
        studlab.abbr <- rownames(random)
      }
    }
    ##
    if (anystudy.c) {
      if (common.logical) {
        studlab <- colnames(x$common)
        studlab.abbr <- colnames(common)
      }
      else {
        studlab <- colnames(x$random)
        studlab.abbr <- colnames(random)
      }
    }
    ##
    if (legend)
      legendabbr(studlab, studlab.abbr, TRUE, "Study label", "\n")
    else
      legendabbr(studlab, studlab.abbr, TRUE, "Study label")
  }
  ##
  invisible(NULL)
}


hatmatrix.aggr <- function(x, model, type) {
  
  model <- setchar(model, c("common", "random"))
  type <- setchar(type, c("full", "long", "short"))
  
  ## Create aggregate B matrix
  if (!is.null(x$B.matrix.aggr))
    B <- x$B.matrix.aggr
  else
    B <- createB(x$treat1.pos, x$treat2.pos, x$n, aggr = TRUE)
  ## Number of edges (direct comparisons) = no. of rows of B
  e <- nrow(B)
  ##
  ## Aggregate weights
  ##
  ## Create a matrix of the aggregate weights e x e
  ##
  W <- matrix(0, nrow = e, ncol = e)
  for (i in 1:e){
    ## idxA element of row i of B that corresponds to baseline treatment
    ## idxB element of row i of B that corresponds to the other treatment
    for (j in seq_len(x$n))
      if (B[i, j] == 1)
        idxA <- j
      else if (B[i, j] == -1)
        idxB <- j
    ## Find all rows of x$B.matrix (dimension m x n) that have the 
    ## same elements as row i of B and calculate the inverse variance 
    ## of the weighted mean
    WAB <- 0.0
    for (k in seq_len(x$m)) {
      if (x$B.matrix[k, idxA] == 1 & x$B.matrix[k, idxB] == -1) {
        if (model == "common")
          WAB <- WAB + 1.0 / (x$seTE.adj.common[k])^2
        else if (model == "random")
          WAB <- WAB + 1.0 / (x$seTE.adj.random[k])^2
      }
    }
    ##
    W[i, i] = WAB
  }
  ##
  ## Laplacian of the aggregate model
  ##
  L <- t(B) %*% W %*% B
  ## Pseudo-Inverse of L 
  L.plus <- invmat(L)
  L.plus[is_zero(L.plus)] <- 0
  ##
  ## Aggregate Hat matrix
  ##
  ## type = "short"
  ## e x e hat matrix
  H.short <- B %*% L.plus %*% t(B) %*% W
  ##
  ## type = "long"
  ## n (n - 1) / 2 x e hat matrix
  ## Get edge incidence matrix for a fully connected network with n
  ## nodes
  B.full <- createB(ncol = x$n)
  H.long <- B.full %*% L.plus %*% t(B) %*% W
  ## type = "full"
  ## n (n - 1) / 2 x n (n - 1) / 2 hat matrix
  ## Insert columns of zeroes for missing edges
  c <- (x$n) * (x$n - 1.0) / 2.0
  H.full <- matrix(0, nrow = c, ncol = c)
  ##
  k <- 0 # counts H.full rows and columns
  l <- 1 # counts B rows and H.long columns
  for (i in 1:(x$n - 1)) {
    for (j in (i + 1):x$n){
      k <- k + 1
      ## is there a direct comparison of ij?
      if (l <= e){
        ## once l is > e (no. of edges) we have reached the end of B
        if (B[l, i] == 1 & B[l, j] == -1){
          for (m in 1:c)
            H.full[m, k] <- H.long[m, l]
          l <- l+1
        }
      }
    }
  }
  ##
  if (type == "short")
    H <- H.short
  else if (type == "long")
    H <- H.long
  else if (type == "full")
    H <- H.full
  else
    return(NULL)
  ##
  ## Row and col names
  ##
  if (type == "short")
    rownames(H) <- colnames(H) <- x$comparisons
  else {
    allcomps <- ""
    k <- 0
    for (i in 1:(x$n - 1)) {
      for (j in (i + 1):x$n) {
        k <- k + 1
        allcomps[k] <- paste(x$trts[i], x$trts[j], sep = x$sep.trts)
      }
    }
    ##
    if (type == "long") {
      rownames(H) <- allcomps
      colnames(H) <- x$comparisons
    }
    else if (type == "full")
      rownames(H) <- colnames(H) <- allcomps
  }
  #
  attr(H, "model") <- model
  attr(H, "type") <- type
  
  H
}


hatmatrix.F1000 <- function(x, model) {
  ##
  ## H matrix
  ##
  if (model == "common")
    krahn <- nma_krahn(x)
  else if (model == "random")
    krahn <- nma_krahn(x, tau.preset = x$tau)
  ##
  X.full <- krahn$X.full
  direct <- krahn$direct
  X <- krahn$X.full[direct$comparison, , drop = FALSE]
  Vd <- diag(direct$seTE^2,
             nrow = length(direct$seTE),
             ncol = length(direct$seTE))
  
  H <-
    X.full %*% solve(t(X) %*% solve(Vd) %*% X) %*% t(X) %*% solve(Vd)
  #
  colnames(H) <- rownames(X)
  attr(H, "model") <- model
  
  H
}

#' Summary method for objects of class netmeta
#' 
#' @description
#' Summary method for objects of class \code{netmeta}.
#' 
#' @param object An object of class \code{netmeta}.
#' @param common A logical indicating whether results for the common
#'   effects model should be printed.
#' @param random A logical indicating whether results for the random
#'   effects model should be printed.
#' @param prediction A logical indicating whether prediction intervals
#'   should be printed.
#' @param reference.group Reference treatment.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
#'   matrices with all treatment effects, and confidence limits will
#'   be printed.
#' @param overall.hetstat A logical indicating whether to print heterogeneity
#'   measures.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @return
#'
#' A list of class "summary.netmeta" is returned with the following elements:
#' 
#' \item{k}{Total number of studies.}
#' \item{m}{Total number of pairwise comparisons.}
#' \item{n}{Total number of treatments.}
#' \item{d}{Total number of designs (corresponding to the unique set
#'   of treatments compared within studies).}
#' 
#' \item{trts}{Treatments included in network meta-analysis.}
#' \item{k.trts}{Number of studies evaluating a treatment.}
#' \item{n.trts}{Number of observations receiving a treatment (if
#'   available).}
#' \item{events.trts}{Number of events observed for a treatment (if
#'   available).}
#' 
#' \item{studies}{Study labels coerced into a factor with its levels
#'   sorted alphabetically.}
#' \item{narms}{Number of arms for each study.}
#' 
#' \item{designs}{Vector with unique designs present in the network. A
#'   design corresponds to the set of treatments compared within a
#'   study.}
#' \item{comparisons}{Vector with unique direct comparisons present in the
#'   network.}
#' 
#' \item{comparison}{Results for pairwise comparisons (data frame with
#'   columns studlab, treat1, treat2, TE, seTE, lower, upper, z, p).}
#' \item{comparison.nma.common}{Results for pairwise comparisons based
#'   on common effects model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p, leverage).}
#' \item{comparison.nma.random}{Results for pairwise comparisons based
#'   on random effects model (data frame with columns studlab, treat1,
#'   treat2, TE, seTE, lower, upper, z, p).}
#' 
#' \item{common}{Results for common effects model (a list with elements
#'   TE, seTE, lower, upper, z, p).}
#' \item{random}{Results for random effects model (a list with
#'   elements TE, seTE, lower, upper, z, p).}
#' \item{predict}{Prediction intervals (a list with elements seTE,
#'   lower, upper).}
#' 
#' \item{Q}{Overall heterogeneity / inconsistency statistic.}
#' \item{df.Q}{Degrees of freedom for test of heterogeneity /
#'   inconsistency.}
#' \item{pval.Q}{P-value for test of heterogeneity / inconsistency.}
#' \item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
#'   limits.}
#' \item{tau}{Square-root of between-study variance.}
#' 
#' \item{Q.heterogeneity}{Overall heterogeneity statistic.}
#' \item{df.Q.heterogeneity}{Degrees of freedom for test of overall
#'   heterogeneity.}
#' \item{pval.Q.heterogeneity}{P-value for test of overall
#'   heterogeneity.}
#' 
#' \item{Q.inconsistency}{Overall inconsistency statistic.}
#' \item{df.Q.inconsistency}{Degrees of freedom for test of overall
#'   inconsistency.}
#' \item{pval.Q.inconsistency}{P-value for test of overall
#'   inconsistency.}
#'
#' \item{Q.decomp}{Data frame with columns 'treat1', 'treat2', 'Q',
#'   'df' and 'pval.Q', providing heterogeneity statistics for each
#'   pairwise meta-analysis of direct comparisons.}
#' 
#' \item{sm}{A character string indicating underlying summary
#'   measure.}
#' \item{method}{A character string indicating which method is to be
#'   used for pooling of studies.}
#' \item{level}{The level used to calculate confidence intervals for
#'   individual studies.}
#' \item{level.ma}{The level used to calculate confidence intervals
#'   for pooled estimates.}
#' \item{level.predict}{The level used to calculate prediction
#'   intervals for a new study.}
#' 
#' \item{ci.lab}{Label for confidence interval.}
#' 
#' \item{incr}{Numerical value added to cell frequencies
#'   (if applicable).}
#' \item{method.incr}{A character string indicating which continuity
#'   correction method was used (if applicable).}
#' \item{allstudies}{A logical indicating whether studies with zero
#'   events or non-events in all treatment arms should be included in
#'   an inverse variance meta-analysis (if applicable).}
#' \item{cc.pooled}{A logical indicating whether \code{incr} should be
#'   used as a continuity correction (if applicable).}
#' 
#' \item{reference.group, baseline.reference}{As defined above.}
#' \item{all.treatments}{As defined above.}
#' \item{seq}{A character specifying the sequence of treatments.}
#' 
#' \item{tau.preset}{An optional value for the square-root of the
#'   between-study variance \eqn{\tau^2}.}
#' 
#' \item{sep.trts}{A character used in comparison names as separator
#'   between treatment labels.}
#' \item{nchar.trts}{A numeric defining the minimum number of
#'   characters used to create unique treatment names.}
#' 
#' \item{prediction, overall.hetstat, backtransf}{As defined above.}
#'  
#' \item{title}{Title of meta-analysis / systematic review.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package \bold{netmeta} used to create object.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}
#' 
#' @keywords summary
#' 
#' @examples
#' # Examples: example(netmeta)
#' 
#' @method summary netmeta
#' @export

summary.netmeta <- function(object,
                            common = object$common,
                            random = object$random,
                            prediction = object$prediction,
                            reference.group = object$reference.group,
                            baseline.reference = object$baseline.reference,
                            all.treatments = object$all.treatments,
                            overall.hetstat = object$overall.hetstat,
                            backtransf = object$backtransf,
                            nchar.trts = object$nchar.trts,
                            warn.deprecated = gs("warn.deprecated"),
                            ...) {
  
  ##
  ##
  ## (1) Check for netmeta object and upgrade object
  ##
  ##
  chkclass(object, "netmeta")
  object <- updateversion(object)
  ##
  is.bin <- inherits(object, "netmetabin")
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  chklogical(prediction)
  chklogical(baseline.reference)
  chklogical(overall.hetstat)
  chklogical(backtransf)
  chknumeric(nchar.trts, min = 1, length = 1)
  ##
  fun <- "summary.netmeta"
  ##
  ## Check for deprecated arguments in '...'
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  
  
  ##
  ##
  ## (3) Summarise results for individual studies and network
  ##     meta-analyses
  ##
  ##
  keepvars <- c("TE", "seTE", "lower", "upper", "statistic", "p")
  ##
  ci.comp <- data.frame(studlab = object$studlab,
                        treat1 = object$treat1, treat2 = object$treat2,
                        ci(object$TE, object$seTE, object$level)[keepvars],
                        stringsAsFactors = FALSE)
  ##
  ci.nma.common <-
    data.frame(studlab = object$studlab,
               treat1 = object$treat1,
               treat2 = object$treat2,
               TE = if (!is.bin) object$TE.nma.common else NA,
               seTE = if (!is.bin) object$seTE.nma.common else NA,
               lower = if (!is.bin) object$lower.nma.common else NA,
               upper = if (!is.bin) object$upper.nma.common else NA,
               statistic = if (!is.bin) object$statistic.nma.common else NA,
               p = if (!is.bin) object$pval.nma.common else NA,
               leverage = if (!is.bin) object$leverage.common else NA,
               stringsAsFactors = FALSE)
  ##
  ci.nma.random <-
    data.frame(studlab = object$studlab,
               treat1 = object$treat1,
               treat2 = object$treat2,
               TE = if (!is.bin) object$TE.nma.random else NA,
               seTE = if (!is.bin) object$seTE.nma.random else NA,
               lower = if (!is.bin) object$lower.nma.random else NA,
               upper = if (!is.bin) object$upper.nma.random else NA,
               statistic = if (!is.bin) object$statistic.nma.random else NA,
               p = if (!is.bin) object$pval.nma.random else NA,
               stringsAsFactors = FALSE)
  ##
  ci.c <- list(TE = object$TE.common,
               seTE = object$seTE.common,
               lower = object$lower.common,
               upper = object$upper.common,
               statistic = object$statistic.common,
               p = object$pval.common)
  ##
  ci.r <- list(TE = object$TE.random,
               seTE = object$seTE.random,
               lower = object$lower.random,
               upper = object$upper.random,
               statistic = object$statistic.random,
               p = object$pval.random)
  ##
  ci.p <- list(seTE = object$seTE.predict,
               lower = object$lower.predict,
               upper = object$upper.predict)
  
  
  ##
  ##
  ## (4) Create summary.netmeta object
  ##
  ##
  object$common <- common
  object$random <- random
  ##
  res <- list(k = object$k,
              m = object$m,
              n = object$n,
              d = object$d,
              #
              trts = object$trts,
              k.trts = object$k.trts,
              n.trts = object$n.trts,
              events.trts = object$events.trts,
              #
              studies = object$studies,
              narms = object$narms,
              #
              designs = object$designs,
              comparisons = object$comparisons,
              #
              comparison = ci.comp,
              comparison.nma.common = ci.nma.common,
              comparison.nma.random = ci.nma.random,
              #
              common = ci.c,
              random = ci.r,
              predict = ci.p,
              #
              Q = object$Q,
              df.Q = object$df.Q,
              pval.Q = object$pval.Q,
              I2 = object$I2,
              lower.I2 = object$lower.I2, upper.I2 = object$upper.I2,
              tau = object$tau,
              #
              Q.heterogeneity = object$Q.heterogeneity,
              df.Q.heterogeneity = object$df.Q.heterogeneity,
              pval.Q.heterogeneity = object$pval.Q.heterogeneity,
              #
              Q.inconsistency = object$Q.inconsistency,
              df.Q.inconsistency = object$df.Q.inconsistency,
              pval.Q.inconsistency = object$pval.Q.inconsistency,
              #
              Q.decomp = object$Q.decomp,
              #
              sm = object$sm,
              method = object$method,
              level = object$level,
              level.ma = object$level.ma,
              level.predict = object$level.predict,
              #
              ci.lab = paste0(round(100 * object$level.ma, 1),"% CI"),
              #
              incr = object$incr,
              method.incr = object$method.incr,
              allstudies = object$allstudies,
              cc.pooled = object$cc.pooled,
              #
              reference.group = NA,
              baseline.reference = NA,
              all.treatments = NA,
              seq = object$seq,
              #
              tau.preset = object$tau.preset,
              #
              sep.trts = object$sep.trts,
              nchar.trts = nchar.trts,
              #
              prediction = prediction,
              overall.hetstat = overall.hetstat,
              backtransf = backtransf,
              #
              title = object$title,
              #
              call = match.call(),
              version = packageDescription("netmeta")$Version
              )
  ##
  if (reference.group != "" & missing(all.treatments))
    all.treatments <- FALSE
  ##
  if (reference.group != "")
    reference.group <- setref(reference.group, rownames(object$A.matrix))
  ##
  res$reference.group <- reference.group
  res$baseline.reference <- baseline.reference
  res$all.treatments <- all.treatments
  ##
  res$x <- object
  #
  res$x$common <- common
  res$x$random <- random
  res$x$prediction <- prediction
  res$x$reference.group <- reference.group
  res$x$baseline.reference <- baseline.reference
  res$x$all.treatments <- all.treatments
  res$x$overall.hetstat <- overall.hetstat
  res$x$backtransf <- backtransf
  res$x$nchar.trts <- nchar.trts
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  ##
  if (is.bin)
    class(res) <- c("summary.netmeta", "summary.netmetabin")
  else
    class(res) <- "summary.netmeta"
  
  res
}

is_installed_package <- function(pkg, func, argument, value,
                                 chksettings = FALSE, stop = TRUE,
                                 version = NULL) {
  
  old <- options(rgl.useNULL = TRUE)
  on.exit(options(old), add = TRUE)

  pkginstalled <- requireNamespace(pkg, quietly = TRUE)
  
  oldpkg <- pkginstalled && !is.null(version) && packageVersion(pkg) < version
  
  if (stop & (!pkginstalled) | oldpkg) {
    
    if (oldpkg) {
      oldmsg <- paste0("Library '", pkg, "' is too old. ")
      oldinst <- "re"
    }
    else {
      oldmsg <- ""
      oldinst <- ""
    }
    
    if (chksettings)
      warning(oldmsg,
              "Argument '", argument, "' not changed.\n  ",
              "Please ", oldinst, "install library '", pkg,
              "' in order to use argument '", argument, " = \"",
              value, "\"'\n  ",
              "(R command: 'install.packages(\"", pkg, "\")').")
    else
      if (missing(func))
        stop(oldmsg,
             "Please ", oldinst, "install library '", pkg,
             "'\n       ",
             "(R command: 'install.packages(\"", pkg, "\")').",
             call. = FALSE)
      else
        stop(oldmsg,
             "Please ", oldinst, "install library '", pkg,
             "' before using function '", func,
             if (missing(argument))
               "'."
             else
               paste0("' with argument '", argument,
                      ifelse(missing(value), "'.",
                             paste0(" ", value, "'."))),
             "\n       ",
             "(R command: 'install.packages(\"", pkg, "\")').",
             call. = FALSE)
  }
  
  invisible(pkginstalled)
}

## Auxiliary functions to set arguments
##
## Package: meta
## Author: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
## License: GPL (>= 2)
##

setchar <- function(x, val, text, list = FALSE, name = NULL,
                    stop.at.error = TRUE, addtext = "") {
  if (is.null(name))
    name <- deparse(substitute(x))
  nval <- length(val)
  ##
  if (is.numeric(x)) {
    numeric.x <- TRUE
    idx <- x
    idx[idx < 1] <- NA
    idx[idx >= nval + 1] <- NA
  }
  else {
    numeric.x <- FALSE
    ##
    if (length(unique(tolower(x))) != length(unique(x)) |
        length(unique(tolower(val))) != length(unique(val)))
      idx <- charmatch(x, val, nomatch = NA)
    else
      idx <- charmatch(tolower(x), tolower(val), nomatch = NA)
  }
  ##
  if (anyNA(idx) || any(idx == 0)) {
    if (list)
      first <- "List element '"
    else
      first <- "Argument '"
    ##
    if (missing(text)) {
      if (numeric.x) {
        if (nval == 1)
          vlist <- "1"
        else if (nval == 2)
          vlist <- "1 or 2"
        else
          vlist <- paste("between 1 and", nval)
      }
      else {
        if (nval == 1)
          vlist <- paste0('"', val, '"')
        else if (nval == 2)
          vlist <- paste0('"', val, '"', collapse = " or ")
        else
          vlist <- paste0(paste0('"', val[-nval], '"', collapse = ", "),
                          ', or ', '"', val[nval], '"')
      }
      ##
      if (stop.at.error)
        stop(first, name, "' must be ", vlist, addtext, ".", call. = FALSE)
      else
        return(NULL)
    }
    else {
      if (stop.at.error)
        stop(first, name, "' ", text, ".", call. = FALSE)
      else
        return(NULL)
    }
  }
  ##
  val[idx]
}

setstudlab <- function(x, k) {
  ##
  ## Set study labels
  ##
  if (is.null(x)) {
    if (k == 1)
      x <- ""
    else
      x <- seq(along = rep(1, k))
  }
  ##
  if (is.factor(x))
    x <- as.character(x)
  ##
  x
}

setunit <- function(x) {
  xname <- deparse(substitute(x))
  
  if (is.character(x)) {
    if (length(x) != 1)
      stop("Argument '", xname, "' must be a character string.")
    ##
    plotunit <- ""
    if (length(grep("cm", tolower(x))) == 1)
      plotunit <- "cm"
    else if (length(grep("inch", tolower(x))) == 1)
      plotunit <- "inch"
    else if (length(grep("mm", tolower(x))) == 1)
      plotunit <- "mm"
    ##
    if (plotunit == "")
      stop("Argument '", xname, "' must contain 'cm', 'inch', or 'mm'.")
    ##
    if (plotunit == "cm") {
      plotval <- substring(x, 1, nchar(x) - 2)
      if (length(plotval) == 0 | is.na(as.numeric(plotval)))
        stop("Argument '", xname, "' must contain at least one number.")
      ##
      res <- unit(as.numeric(plotval), "cm")
    }
    else if (plotunit == "inch") {
      plotval <- substring(x, 1, nchar(x) - 4)
      if (length(plotval) == 0 | is.na(as.numeric(plotval)))
        stop("Argument '", xname, "' must contain at least one number.")
      ##
      res <- unit(as.numeric(plotval), "inch")
    }
    else if (plotunit == "mm") {
      plotval <- substring(x, 1, nchar(x) - 2)
      if (length(plotval) == 0 | is.na(as.numeric(plotval)))
        stop("Argument '", xname, "' must contain at least one number.")
      ##
      res <- unit(as.numeric(plotval), "mm")
    }
  }
  else
    res <- x
  
  res
}

setmethodbias <- function(x, subset) {
  oldmethod <- setchar(x, gs("meth4bias.old"),
                       stop.at.error = FALSE)
  ##
  if (is.null(oldmethod))
    if (missing(subset))
      res <- setchar(x, gs("meth4bias"), name = "method.bias")
    else
      res <- setchar(x, gs("meth4bias")[subset], name = "method.bias")
  else
    res <- switch(oldmethod,
                  rank = "Begg",
                  linreg = "Egger",
                  mm = "Thompson",
                  count = "Schwarzer",
                  score = "Harbord")
  ##
  res
}

#' Create a netmeta object from a crossnma object
#' 
#' @description
#' Auxiliary function to create a netmeta object from a crossnma object.
#' 
#' @param x A \code{crossnma} object created with R package \bold{crossnma}.
#' @param keep.samples A logical indicating whether to keep the generated
#'   samples.
#' @param level The level used to calculate confidence intervals for
#'   individual comparisons.
#' @param level.ma The level used to calculate confidence intervals
#'   for network estimates.
#' @param reference.group Reference treatment (first treatment is used
#'   if argument is missing).
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect (passed on to
#'   \code{\link{netrank}}, can be abbreviated.
#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
#'   matrices with all treatment effects, and confidence limits will
#'   be printed.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments in printouts.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param nchar.studlab A numeric defining the minimum number of
#'   characters used to create unique study labels.
#'
#' @return
#' A netmeta object with additional class "netmeta.crossnma".
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}
#' 
#' @examples
#' \donttest{
#' if (requireNamespace("crossnma", quietly = TRUE)) {
#' library("crossnma")
#' # Create a JAGS model
#' mod <- crossnma.model(treat, id, relapse, n, design,
#'   prt.data = ipddata, std.data = stddata,
#'   reference = "A", trt.effect = "random", method.bias = "naive")
#' # Fit JAGS model
#' set.seed(1909)
#' fit <- crossnma(mod)
#'
#' # Print results in netmeta layout
#' nma <- crossnma2netmeta(fit)
#' nma
#' # SUCRAs
#' netrank(nma)
#' # Rankogram
#' rankogram(nma)
#' }
#' }
#' 
#' @export crossnma2netmeta

crossnma2netmeta <- function(x,
                             keep.samples = TRUE,
                             level = gs("level"), level.ma = x$model$level.ma,
                             reference.group = x$model$reference,
                             baseline.reference = gs("baseline.reference"),
                             small.values = x$model$small.values,
                             all.treatments = gs("all.treatments"),
                             seq = gs("seq"),
                             backtransf = x$model$backtransf,
                             nchar.trts = gs("nchar.trts"),
                             nchar.studlab = gs("nchar.studlab")) {
  
  #
  # (1) Check for crossnma object and extract data / settings
  #
  
  chkclass(x, "crossnma")
  #
  if (!is.null(x$model$covariate))
    stop("R function crossnma2netmeta() not suitable for ",
         "network meta-regression.",
         call. = FALSE)
  #
  chklogical(keep.samples)
  chklevel(level)
  chklevel(level.ma)
  chklogical(baseline.reference)
  small.values <- setsv(replaceNULL(small.values, gs("small.values")))
  if (!is.null(all.treatments))
    chklogical(all.treatments)
  chklogical(backtransf)
  chknumeric(nchar.trts, min = 1, length = 1)
  chknumeric(nchar.studlab, min = 1, length = 1)
  #
  common <- x$model$trt.effect == "common"
  random <- x$model$trt.effect == "random"
  sm <- x$model$sm
  #
  trts <- as.character(x$trt.key$trt.ini)
  labels <- sort(trts)
  #
  if (!is.null(seq))
    seq <- setseq(seq, labels)
  else {
    seq <- labels
    if (is.numeric(seq))
      seq <- as.character(seq)
  }
  #
  dat <- x$model$all.data.ad
  trt <- dat$trt
  n <- dat$n
  outcome <- dat$outcome
  study <- dat$study
  se <- dat$se
  
  
  #
  # (2) Calculate matrices with treatment effects, standard errors, etc.
  #
  
  dmat <- do.call(rbind, x$samples) %>% data.frame() %>%
    select(starts_with("d."))
  names(dmat) <- trts
  #
  if (random) {
    tau <- NULL
    tmat <- do.call(rbind, x$samples) %>% data.frame() %>% select(tau)
  }
  #
  TE.matrix <- seTE.matrix <-
    lower.matrix <- upper.matrix <-
    matrix(NA, nrow = ncol(dmat), ncol = ncol(dmat),
           dimnames = list(trts, trts))
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        TE.matrix[i, j] <- mean(dmat[, i] - dmat[, j])
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        seTE.matrix[i, j] <- sd(dmat[, i] - dmat[, j])
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        lower.matrix[i, j] <- quantile(dmat[, i] - dmat[, j],
                                       (1 - level.ma) / 2)
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        upper.matrix[i, j] <- quantile(dmat[, i] - dmat[, j],
                                       1 - (1 - level.ma) / 2)
  #
  diag(TE.matrix) <- diag(seTE.matrix) <-
    diag(lower.matrix) <- diag(upper.matrix) <- 0
  
  
  #
  # (3) Create pairwise object from crossnma data and run network meta-analysis
  #
  
  if (sm %in% c("OR", "RR")) {
    suppressWarnings(
      dat <-
        pairwise(treat = trt, event = outcome, n = n,
                 studlab = study, sm = sm, warn = FALSE)
    )
  }
  else if (sm %in% c("MD", "SMD")) {
    suppressWarnings(
      dat <-
        pairwise(treat = trt,
                 mean = outcome, n = n, sd = se,
                 studlab = study, sm = sm, warn = FALSE)
    )
  }
  #
  res <- suppressWarnings(netmeta(dat, common = common, random = random,
                                  warn = FALSE))
  
  
  #
  # (4) Add crossnma results to netmeta object
  #
  
  res <- setNA_nma(res)
  #
  res$keep.samples <- keep.samples
  #
  if (keep.samples) {
    res$samples <- list(d = dmat)
    #
    if (random)
      res$samples$tau <- tmat
  }
  #
  res$method <- "crossnma"
  res$method.tau <- ""
  res$m <- NA
  #
  res$level <- gs("level")
  res$level.ma <- level.ma
  res$reference.group <- reference.group
  res$baseline.reference <- baseline.reference
  res$small.values <- small.values
  res$all.treatments <- all.treatments
  res$seq <- seq
  res$backtransf <- backtransf
  res$nchar.trts <- nchar.trts
  res$nchar.studlab <- nchar.studlab
  #
  if (common) {
    res$TE.common <- TE.matrix[labels, labels]
    res$seTE.common <- seTE.matrix[labels, labels]
    res$lower.common <- lower.matrix[labels, labels]
    res$upper.common <- upper.matrix[labels, labels]
    #
    for (i in seq_along(res$treat1)) {
      res$TE.nma.common[i] <- res$TE.common[res$treat1[i], res$treat2[i]]
      res$seTE.nma.common[i] <- res$seTE.common[res$treat1[i], res$treat2[i]]
      res$lower.nma.common[i] <- res$lower.common[res$treat1[i], res$treat2[i]]
      res$upper.nma.common[i] <- res$upper.common[res$treat1[i], res$treat2[i]]
    }
    #
    res$tau2 <- res$tau <- NA
    #
    res$TE.random[!is.na(res$TE.random)] <- NA
    res$seTE.random <- res$lower.random <- res$upper.random <- res$TE.random
  }
  else if (random) {
    res$TE.random <- TE.matrix[labels, labels]
    res$seTE.random <- seTE.matrix[labels, labels]
    res$lower.random <- lower.matrix[labels, labels]
    res$upper.random <- upper.matrix[labels, labels]
    #
    for (i in seq_along(res$treat1)) {
      res$TE.nma.random[i] <- res$TE.random[res$treat1[i], res$treat2[i]]
      res$seTE.nma.random[i] <- res$seTE.random[res$treat1[i], res$treat2[i]]
      res$lower.nma.random[i] <- res$lower.random[res$treat1[i], res$treat2[i]]
      res$upper.nma.random[i] <- res$upper.random[res$treat1[i], res$treat2[i]]
    }
    #
    res$tau2 <- mean(tmat %>% unlist() %>% unname())
    res$tau <- sqrt(res$tau2)
    #
    res$TE.common[!is.na(res$TE.common)] <- NA
    res$seTE.common <- res$lower.common <- res$upper.common <- res$TE.common
  }
  
  
  #
  # (5) Return netmeta object
  #
  
  class(res) <- c(class(res), "netmeta.crossnma")
  #
  res
}

#' Conduct pairwise meta-analyses for all comparisons with direct
#' evidence in a network meta-analysis
#' 
#' @description
#' Conduct pairwise meta-analyses for all comparisons with direct
#' evidence in a network meta-analysis.
#' 
#' @param x An object of class \code{netmeta} or \code{netpairwise}.
#' @param object An object of class \code{netpairwise}.
#' @param separate A logical indicating whether results for pairwise
#'   comparisons should be printed as separate meta-analyses or as
#'   subgroups which is more concise.
#' @param common A logical indicating whether a common effects network
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects network
#'   meta-analysis should be conducted.
#' @param level The level used to calculate confidence intervals for
#'   individual comparisons.
#' @param level.ma The level used to calculate confidence intervals
#'   for pooled estimates.
#' @param prediction A logical indicating whether prediction intervals
#'   should be printed.
#' @param level.predict The level used to calculate prediction
#'   intervals for a new study.
#' @param reference.group Reference treatment.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau}. Either \code{"DL"}, \code{"REML"}, or
#'   \code{"ML"}, can be abbreviated.
#' @param method A character string indicating which method is to be
#'   used for pooling of studies, see \code{\link[meta]{metabin}}.
#' @param incr A numerical value which is added to cell counts,
#'   see \code{\link[meta]{metabin}}.
#' @param method.incr A character string indicating which continuity
#'   correction method should be used (\code{"only0"},
#'   \code{"if0all"}, or \code{"all"}), see \code{\link[meta]{metabin}}.
#' @param allstudies A logical indicating whether studies with zero
#'   events or non-events in all treatment arms should be included in
#'   the meta-analysis, see \code{\link[meta]{metabin}}.
#' @param order An optional character or numerical vector specifying
#'   the order of treatments.
#' @param sep.trts A character used in comparison names as separator
#'   between treatment labels.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param k.min Minimum number of studies in pairwise comparison to
#'   show funnel plot, radial plot or conduct test for funnel plot
#'   asymmetry.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (passed on to \code{metagen} or
#'   print functions and to catch deprecated arguments).
#' 
#' @details
#' Conduct pairwise meta-analyses for all comparisons with direct
#' evidence in a network meta-analysis. In contrast to
#' \code{\link{netmeta}} and \code{\link{netsplit}}, unadjusted
#' standard errors are used in the calculations and the between-study
#' heterogeneity variance is allowed to differ between comparisons.
#' 
#' The R function \code{\link[meta]{metagen}} is called internally.
#' 
#' @note
#' This function must not be confused with \code{\link[meta]{pairwise}}
#' which can be used as a pre-processing step to convert data from
#' arm-based to contrast-based format by calculating all pairwise
#' comparisons within a study.
#' 
#' @return
#' Either a single \code{\link[meta]{metagen}} object with pairwise
#' comparisons as subgroups or a list with \code{\link[meta]{metagen}}
#' objects for each direct pairwise comparison.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netsplit}},
#'   \code{\link[meta]{pairwise}}
#' 
#' @examples
#' oldsets <- settings.meta(digits = 2, digits.tau2 = 2, digits.tau = 2)
#' 
#' data(smokingcessation)
#' 
#' # Transform data from arm-based format to contrast-based format
#' #
#' pw1 <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma1 <- netmeta(pw1, common = FALSE)
#' 
#' # Calculate and print concise results for all pairwise
#' # meta-analyses
#' #
#' np1 <- netpairwise(nma1)
#' np1
#' print(np1, details.method = FALSE)
#'
#' \donttest{
#' # Random effects model
#' #
#' pw2 <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' nma2 <- netmeta(pw2, common = FALSE, reference = "plac")
#' 
#' # Calculate and print concise results for all pairwise
#' # meta-analyses
#' #
#' np2 <- netpairwise(nma2)
#' np2
#' print(np2, details.method = FALSE)
#'
#' forest(np2)
#' 
#' # Print detailed information for each pairwise comparison
#' #
#' np3 <- netpairwise(nma2, separate = TRUE)
#' forest(np3)
#' funnel(np3)
#' radial(np3)
#' funnel(np3, k.min = 1)
#' }
#'
#' settings.meta(oldsets)
#' 
#' @rdname netpairwise
#' @export netpairwise

netpairwise <- function(x, ...)
  UseMethod("netpairwise")


#' @rdname netpairwise
#' @method netpairwise netmeta
#' @export

netpairwise.netmeta <- function(x,
                                separate = FALSE,
                                common = x$common,
                                random = x$random,
                                level = x$level,
                                level.ma = x$level.ma,
                                prediction = x$prediction,
                                level.predict = x$level.predict,
                                reference.group =
                                  if (missing(order)) x$reference.group else "",
                                baseline.reference = x$baseline.reference,
                                method.tau = x$method.tau,
                                order = NULL,
                                sep.trts = x$sep.trts,
                                nchar.trts = x$nchar.trts,
                                backtransf = x$backtransf,
                                warn.deprecated = gs("warn.deprecated"),
                                ...) {
  
  
  ##
  ##
  ## (1) Check and set arguments
  ##
  ##
  chkclass(x, "netmeta")
  x <- updateversion(x)
  ##
  chklogical(separate)
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  chklogical(random)
  chklogical(prediction)
  ##
  chklevel(level)
  chklevel(level.ma)
  chklevel(level.predict)
  ##
  reference.group <- setref(reference.group, c(x$trts, ""))
  chklogical(baseline.reference)
  ##
  method.tau <- setchar(method.tau, c("DL", "ML", "REML"))
  ##
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  if (!missing(order)) {
    order <- catch("order", mc, x, sfsp)
    ##
    if (length(order) != length(x$trts))
      order <- setchar(order, x$trts)
    else
      order <- setseq(order, x$trts)
  }
  ##
  chkchar(sep.trts)
  chknumeric(nchar.trts, min = 1, length = 1)
  chklogical(backtransf)
  
  
  ##
  ##
  ## (2) Get data
  ##
  ##
  TE <- x$data$.TE
  seTE <- x$data$.seTE
  studlab <- x$data$.studlab
  ##
  n1 <- x$data$.n1
  n2 <- x$data$.n2
  ##
  treat1 <- x$data$.treat1
  treat2 <- x$data$.treat2
  #
  trts <-  unique(sort(c(treat1, treat2)))
  #
  if (is.null(order))
    order <- trts
  ##
  comparison <- paste(treat1, treat2, sep = sep.trts)
  comparison21 <- paste(treat2, treat1, sep = sep.trts)
  ##
  treat1.pos <- as.numeric(factor(treat1, levels = order))
  treat2.pos <- as.numeric(factor(treat2, levels = order))
  ##
  trts.abbr <- treats(trts, nchar.trts)
  trt1 <- as.character(factor(treat1, levels = trts, labels = trts.abbr))
  trt2 <- as.character(factor(treat2, levels = trts, labels = trts.abbr))
  ##
  comp <- paste(trt1, trt2, sep = sep.trts)
  comp21 <- paste(trt2, trt1, sep = sep.trts)
  ##
  wo <- treat1.pos > treat2.pos
  ##
  if (any(wo)) {
    ttreat1.pos <- treat1.pos
    treat1.pos[wo] <- treat2.pos[wo]
    treat2.pos[wo] <- ttreat1.pos[wo]
    ##
    TE[wo] <- -TE[wo]
    ##
    tn1 <- n1
    n1[wo] <- n2[wo]
    n2[wo] <- tn1[wo]
    ##
    ttreat1 <- treat1
    treat1[wo] <- treat2[wo]
    treat2[wo] <- ttreat1[wo]
    ##
    comparison[wo] <- comparison21[wo]
    ##
    ttrt1 <- trt1
    trt1[wo] <- trt2[wo]
    trt2[wo] <- ttrt1[wo]
    ##
    comp[wo] <- comp21[wo]
  }
  ##
  o <- order(treat1.pos, treat2.pos)
  ##
  TE <- TE[o]
  seTE <- seTE[o]
  studlab <- studlab[o]
  ##
  n1 <- n1[o]
  n2 <- n2[o]
  ##
  treat1 <- treat1[o]
  treat2 <- treat2[o]
  comparison <- comparison[o]
  ##
  trt1 <- trt1[o]
  trt2 <- trt2[o]
  comp <- comp[o]
  ##
  if (reference.group != "") {
    if (baseline.reference) {
      wo1 <- trt1 == reference.group & !is.na(TE)
      if (any(wo1)) {
        TE[wo1] <- -TE[wo1]
        ttrt1 <- trt1
        trt1[wo1] <- trt2[wo1]
        trt2[wo1] <- ttrt1[wo1]
        tn1 <- n1
        n1[wo1] <- n2[wo1]
        n2[wo1] <- tn1[wo1]
      }
    }
    else {
      wo2 <- trt2 == reference.group & !is.na(TE)
      if (any(wo2)) {
        TE[wo2] <- -TE[wo2]
        ttrt1 <- trt1
        trt1[wo2] <- trt2[wo2]
        trt2[wo2] <- ttrt1[wo2]
        tn1 <- n1
        n1[wo2] <- n2[wo2]
        n2[wo2] <- tn1[wo2]
      }
    }
  }
  
  
  ##
  ##
  ## (3) Run pairwise meta-analyses
  ##
  ##
  if (!separate) {
    res <- metagen(TE, seTE, studlab = studlab,
                   n.e = n1, n.c = n2,
                   sm = x$sm,
                   subgroup = paste0(trt1, sep.trts, trt2),
                   subgroup.name = "comparison",
                   print.subgroup.name = FALSE,
                   common = common,
                   random = random,
                   level = level,
                   level.ma = level.ma,
                   prediction = prediction,
                   level.predict = level.predict,
                   method.tau = method.tau,
                   overall = FALSE, overall.hetstat = FALSE,
                   test.subgroup = FALSE,
                   warn.deprecated = FALSE,
                   ...)
    ##
    res$k.study <- x$k
    res$k <- x$m
    res$w.common[!is.na(res$w.common)] <- NA
    res$w.random[!is.na(res$w.random)] <- NA
    ##
    res$order <- order
    ##
    class(res) <- c(class(res), "netpairwise")
  }
  else {
    comps <- unique(data.frame(trt1, trt2))
    comps <- comps[order(comps$trt1, comps$trt2), ]
    n.comps <- nrow(comps)
    ##
    res <- vector("list", length = n.comps)
    ##
    for (i in seq_len(n.comps)) {
      comp.i <- paste0(comps$trt1[i], sep.trts, comps$trt2[i])
      res[[i]] <-
        metagen(TE, seTE, studlab = studlab,
                n.e = n1, n.c = n2,
                sm = x$sm,
                subset = trt1 == comps$trt1[i] & trt2 == comps$trt2[i],
                complab = comp.i,
                common = common,
                random = random,
                level = level,
                level.ma = level.ma,
                prediction = prediction,
                level.predict = level.predict,
                method.tau = method.tau,
                label.e = comps$trt1[i], label.c = comps$trt2[i],
                warn.deprecated = FALSE,
                ...)
    }
    ##
    attr(res, "order") <- order
    attr(res, "version") <- packageDescription("netmeta")$Version
    ##
    class(res) <- "netpairwise"
  }
  
  res
}


#' @rdname netpairwise
#' @method netpairwise netmetabin
#' @export

netpairwise.netmetabin <- function(x,
                                   separate = FALSE,
                                   common = x$common,
                                   random = x$random,
                                   level = x$level,
                                   level.ma = x$level.ma,
                                   prediction = x$prediction,
                                   level.predict = x$level.predict,
                                   reference.group =
                                     if (missing(order))
                                       x$reference.group else "",
                                   baseline.reference = x$baseline.reference,
                                   #
                                   method = x$method,
                                   incr = x$incr,
                                   method.incr = x$method.incr,
                                   allstudies = x$allstudies,
                                   #
                                   method.tau = x$method.tau,
                                   #
                                   order = NULL,
                                   sep.trts = x$sep.trts,
                                   nchar.trts = x$nchar.trts,
                                   backtransf = x$backtransf,
                                   warn.deprecated = gs("warn.deprecated"),
                                   ...) {
  
  
  #
  #
  # (1) Check and set arguments
  #
  #
  
  chkclass(x, "netmetabin")
  x <- updateversion(x)
  #
  chklogical(separate)
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  chklogical(random)
  chklogical(prediction)
  #
  chklevel(level)
  chklevel(level.ma)
  chklevel(level.predict)
  #
  reference.group <- setref(reference.group, c(x$trts, ""))
  chklogical(baseline.reference)
  #
  method <- setchar(method, c("MH", "NCH", "LRP"))
  if (method == "NCH")
    method <- "MH"
  #
  method.tau <- setchar(method.tau, c("DL", "ML", "REML"))
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (!missing(order)) {
    order <- catch("order", mc, x, sfsp)
    #
    if (length(order) != length(x$trts))
      order <- setchar(order, x$trts)
    else
      order <- setseq(order, x$trts)
  }
  #
  chkchar(sep.trts)
  chknumeric(nchar.trts, min = 1, length = 1)
  chklogical(backtransf)
  
  
  #
  #
  # (2) Get data
  #
  #
  
  studlab <- x$data$.studlab
  #
  event1 <- x$data$.event1
  event2 <- x$data$.event2
  #
  n1 <- x$data$.n1
  n2 <- x$data$.n2
  #
  treat1 <- x$data$.treat1
  treat2 <- x$data$.treat2
  #
  trts <-  unique(sort(c(treat1, treat2)))
  #
  if (is.null(order))
    order <- trts
  #
  comparison <- paste(treat1, treat2, sep = sep.trts)
  comparison21 <- paste(treat2, treat1, sep = sep.trts)
  #
  treat1.pos <- as.numeric(factor(treat1, levels = order))
  treat2.pos <- as.numeric(factor(treat2, levels = order))
  #
  trts.abbr <- treats(trts, nchar.trts)
  trt1 <- as.character(factor(treat1, levels = trts, labels = trts.abbr))
  trt2 <- as.character(factor(treat2, levels = trts, labels = trts.abbr))
  #
  comp <- paste(trt1, trt2, sep = sep.trts)
  comp21 <- paste(trt2, trt1, sep = sep.trts)
  #
  wo <- treat1.pos > treat2.pos
  #
  if (any(wo)) {
    ttreat1.pos <- treat1.pos
    treat1.pos[wo] <- treat2.pos[wo]
    treat2.pos[wo] <- ttreat1.pos[wo]
    #
    tevent1 <- event1
    event1[wo] <- event2[wo]
    event2[wo] <- tevent1[wo]
    #
    tn1 <- n1
    n1[wo] <- n2[wo]
    n2[wo] <- tn1[wo]
    #
    ttreat1 <- treat1
    treat1[wo] <- treat2[wo]
    treat2[wo] <- ttreat1[wo]
    #
    comparison[wo] <- comparison21[wo]
    #
    ttrt1 <- trt1
    trt1[wo] <- trt2[wo]
    trt2[wo] <- ttrt1[wo]
    #
    comp[wo] <- comp21[wo]
  }
  #
  o <- order(treat1.pos, treat2.pos)
  #
  studlab <- studlab[o]
  #
  event1 <- event1[o]
  event2 <- event2[o]
  #
  n1 <- n1[o]
  n2 <- n2[o]
  #
  treat1 <- treat1[o]
  treat2 <- treat2[o]
  comparison <- comparison[o]
  #
  trt1 <- trt1[o]
  trt2 <- trt2[o]
  comp <- comp[o]
  #
  if (reference.group != "") {
    if (baseline.reference) {
      wo1 <- trt1 == reference.group
      if (any(wo1)) {
        ttrt1 <- trt1
        trt1[wo1] <- trt2[wo1]
        trt2[wo1] <- ttrt1[wo1]
        #
        tevent1 <- event1
        event1[wo1] <- event2[wo1]
        event2[wo1] <- tevent1[wo1]
        #
        tn1 <- n1
        n1[wo1] <- n2[wo1]
        n2[wo1] <- tn1[wo1]
      }
    }
    else {
      wo2 <- trt2 == reference.group
      if (any(wo2)) {
        ttrt1 <- trt1
        trt1[wo2] <- trt2[wo2]
        trt2[wo2] <- ttrt1[wo2]
        #
        tevent1 <- event1
        event1[wo2] <- event2[wo2]
        event2[wo2] <- tevent1[wo2]
        #
        tn1 <- n1
        n1[wo2] <- n2[wo2]
        n2[wo2] <- tn1[wo2]
      }
    }
  }
  
  
  #
  #
  # (3) Run pairwise meta-analyses
  #
  #
  if (!separate) {
    res <- metabin(event1, n1, event2, n2, studlab = studlab,
                   method = method, sm = x$sm,
                   incr = incr, method.incr = method.incr,
                   allstudies = allstudies,
                   subgroup = paste0(trt1, sep.trts, trt2),
                   subgroup.name = "comparison",
                   print.subgroup.name = FALSE,
                   common = common,
                   random = random,
                   level = level,
                   level.ma = level.ma,
                   prediction = prediction,
                   level.predict = level.predict,
                   method.tau = method.tau,
                   overall = FALSE, overall.hetstat = FALSE,
                   test.subgroup = FALSE,
                   warn = FALSE,
                   warn.deprecated = FALSE,
                   ...)
    #
    res$k.study <- x$k
    res$k <- x$m
    res$w.common[!is.na(res$w.common)] <- NA
    res$w.random[!is.na(res$w.random)] <- NA
    #
    res$order <- order
    #
    class(res) <- c(class(res), "netpairwise")
  }
  else {
    comps <- unique(data.frame(trt1, trt2))
    comps <- comps[order(comps$trt1, comps$trt2), ]
    n.comps <- nrow(comps)
    #
    res <- vector("list", length = n.comps)
    #
    for (i in seq_len(n.comps)) {
      comp.i <- paste0(comps$trt1[i], sep.trts, comps$trt2[i])
      res[[i]] <-
        metabin(event1, n1, event2, n2, studlab = studlab,
                method = method, sm = x$sm,
                incr = incr, method.incr = method.incr,
                allstudies = allstudies,
                subset = trt1 == comps$trt1[i] & trt2 == comps$trt2[i],
                complab = comp.i,
                common = common,
                random = random,
                level = level,
                level.ma = level.ma,
                prediction = prediction,
                level.predict = level.predict,
                method.tau = method.tau,
                label.e = comps$trt1[i], label.c = comps$trt2[i],
                warn = FALSE,
                warn.deprecated = FALSE,
                ...)
    }
    #
    attr(res, "order") <- order
    attr(res, "version") <- packageDescription("netmeta")$Version
    #
    class(res) <- "netpairwise"
  }
  
  res
}


#' @rdname netpairwise
#' @method print netpairwise
#' @export

print.netpairwise <- function(x, ...) {
  
  chkclass(x, "netpairwise")
  
  if (inherits(x, "metagen")) {
    print(x, ...)
  }
  else {
    n <- 1
    for (i in 1:length(x)) {
      if (n > 1)
        cat("\n*****\n\n")
      print(x[[i]], ...)
      n <- n + 1
    }
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method summary netpairwise
#' @export

summary.netpairwise <- function(object, ...) {
  
  chkclass(object, "netpairwise")
  
  if (inherits(object, "metagen")) {
    res <- summary(object)
  }
  else {
    res <- object
    for (i in seq_len(length(object)))
      res[[i]] <- summary(object[[i]])
    ##
    class(res) <- "summary.netpairwise"
  }
  
  res
}


#' @rdname netpairwise
#' @method print summary.netpairwise
#' @export

print.summary.netpairwise <- function(x, ...) {
  
  chkclass(x, "summary.netpairwise")
  
  if (inherits(x, "metagen")) {
    print(x, ...)
  }
  else {
    n <- 1
    for (i in seq_len(length(x))) {
      if (n > 1)
        cat("\n*****\n\n")
      print(x[[i]], ...)
      n <- n + 1
    }
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method forest netpairwise
#' @export

forest.netpairwise <- function(x, ...) {
  
  chkclass(x, "netpairwise")
  
  if (inherits(x, "metagen")) {
    res <- forest(x, ...)
    return(invisible(res))
  }
  else {
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      forest(m.i, smlab = paste0("Comparison:\n", m.i$complab), ...)
    }
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method plot netpairwise
#' @export

plot.netpairwise <- function(x, ...)
  forest(x, ...)


#' @rdname netpairwise
#' @method funnel netpairwise
#' @export

funnel.netpairwise <- function(x, k.min = 3, ...) {
  
  chkclass(x, "netpairwise")

  if (inherits(x, "metagen")) {
     stop("Funnel plot not suitable for object of class \"netpairwise\" ",
          "without argument 'separate = TRUE'")
  }
  else {
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        funnel(m.i, ...)
        title(m.i$complab)
      }
    }
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method radial netpairwise
#' @export

radial.netpairwise <- function(x, k.min = 3, ...) {
  
  chkclass(x, "netpairwise")
  
  if (inherits(x, "metagen")) {
     stop("Radial plot not suitable for object of class \"netpairwise\" ",
          "without argument 'separate = TRUE'")
  }
  else {
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        radial(m.i, ...)
        title(m.i$complab)
      }
    }
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method baujat netpairwise
#' @export

baujat.netpairwise <- function(x, k.min = 3, ...) {
  
  chkclass(x, "netpairwise")

  if (inherits(x, "metagen")) {
     stop("Baujat plot not suitable for object of class \"netpairwise\" ",
          "without argument 'separate = TRUE'")
  }
  else {
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        baujat(m.i, ...)
        title(m.i$complab)
      }
    }
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method metabias netpairwise
#' @export

metabias.netpairwise <- function(x, k.min = 10, ...) {
  
  chkclass(x, "netpairwise")
  res <- list()

  if (inherits(x, "metagen")) {
     stop("Test for funnel plot asymmetry not suitable for object of ",
          "class \"netpairwise\" without argument 'separate = TRUE'.")
  }
  else {
    n <- 0
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        n <- n + 1
        res[[n]] <- metabias(m.i, k.min = k.min, ...)
      }
    }
    ##
    if (n > 0)
      class(res) <- "metabias.netpairwise"
    else {
      warning("No pairwise comparison with at least ", k.min, " studies.",
              call. = FALSE)
      return(invisible(NULL))
    }
  }
  
  res
}


#' @rdname netpairwise
#' @method print metabias.netpairwise
#' @export

print.metabias.netpairwise <- function(x, ...) {
  
  chkclass(x, "metabias.netpairwise")
  
  n <- 1
  for (i in seq_len(length(x))) {
    if (n > 1)
      cat("\n*****\n\n")
    print(x[[i]], ...)
    n <- n + 1
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method trimfill netpairwise
#' @export

trimfill.netpairwise <- function(x, k.min = 3, ...) {
  
  chkclass(x, "netpairwise")
  res <- list()
  
  if (inherits(x, "metagen")) {
     stop("Trim-and-fill method not suitable for object of ",
          "class \"netpairwise\" without argument 'separate = TRUE'.")
  }
  else {
    n <- 0
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        n <- n + 1
        res[[n]] <- trimfill(m.i, ...)
      }
    }
    ##
    if (n > 0)
      class(res) <- c("trimfill.netpairwise", "netpairwise")
    else {
      warning("No pairwise comparison with least ", k.min, " studies.",
              call. = FALSE)
      res <- NULL
    }
  }
  
  res
}


#' @rdname netpairwise
#' @method print trimfill.netpairwise
#' @export

print.trimfill.netpairwise <- function(x, ...) {
  
  chkclass(x, "trimfill.netpairwise")
  
  n <- 1
  for (i in seq_len(length(x))) {
    if (n > 1)
      cat("\n*****\n\n")
    print(x[[i]], ...)
    n <- n + 1
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method metainf netpairwise
#' @export

metainf.netpairwise <- function(x, k.min = 2, ...) {
  
  chkclass(x, "netpairwise")
  res <- list()
  
  if (inherits(x, "metagen")) {
     stop("Trim-and-fill method not suitable for object of ",
          "class \"netpairwise\" without argument 'separate = TRUE'.")
  }
  else {
    n <- 0
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        n <- n + 1
        res[[n]] <- metainf(m.i, ...)
      }
    }
    ##
    if (n > 0)
      class(res) <- c("metainf.netpairwise", "netpairwise")
    else {
      warning("No pairwise comparison with least ", k.min, " studies.",
              call. = FALSE)
      res <- NULL
    }
  }
  
  ##
  res
}


#' @rdname netpairwise
#' @method print metainf.netpairwise
#' @export

print.metainf.netpairwise <- function(x, ...) {
  
  chkclass(x, "metainf.netpairwise")
  
  n <- 1
  for (i in seq_len(length(x))) {
    if (n > 1)
      cat("\n*****\n\n")
    print(x[[i]], ...)
    n <- n + 1
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method metacum netpairwise
#' @export

metacum.netpairwise <- function(x, k.min = 2, ...) {
  
  chkclass(x, "netpairwise")
  res <- list()
  
  if (inherits(x, "metagen")) {
     stop("Trim-and-fill method not suitable for object of ",
          "class \"netpairwise\" without argument 'separate = TRUE'.")
  }
  else {
    n <- 0
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        n <- n + 1
        res[[n]] <- metacum(m.i, ...)
      }
    }
    ##
    if (n > 0)
      class(res) <- c("metacum.netpairwise", "netpairwise")
    else {
      warning("No pairwise comparison with least ", k.min, " studies.",
              call. = FALSE)
      res <- NULL
    }
  }
  
  res
}


#' @rdname netpairwise
#' @method print metacum.netpairwise
#' @export

print.metacum.netpairwise <- function(x, ...) {
  
  chkclass(x, "metacum.netpairwise")
  
  n <- 1
  for (i in seq_len(length(x))) {
    if (n > 1)
      cat("\n*****\n\n")
    print(x[[i]], ...)
    n <- n + 1
  }
  
  invisible(NULL)
}


#' @rdname netpairwise
#' @method metareg netpairwise
#' @export

metareg.netpairwise <- function(x, ..., k.min = 2) {
  
  chkclass(x, "netpairwise")
  res <- list()
  
  if (inherits(x, "metagen")) {
     stop("Trim-and-fill method not suitable for object of ",
          "class \"netpairwise\" without argument 'separate = TRUE'.")
  }
  else {
    n <- 0
    for (i in seq_len(length(x))) {
      m.i <- x[[i]]
      if (m.i$k >= k.min) {
        n <- n + 1
        res[[n]] <- metareg(m.i, ...)
      }
    }
    ##
    if (n > 0)
      class(res) <- c("metareg.netpairwise", "netpairwise")
    else {
      warning("No pairwise comparison with least ", k.min, " studies.",
              call. = FALSE)
      res <- NULL
    }
  }
  
  res
}


#' @rdname netpairwise
#' @method print metareg.netpairwise
#' @export

print.metareg.netpairwise <- function(x, ...) {
  
  chkclass(x, "metareg.netpairwise")
  
  n <- 1
  for (i in seq_len(length(x))) {
    if (n > 1)
      cat("\n*****\n\n")
    print(x[[i]], ...)
    n <- n + 1
  }
  
  invisible(NULL)
}

nma.ruecker <- function(TE, W, seTE,
                        treat1, treat2,
                        treat1.pos, treat2.pos,
                        narms, studlab,
                        sm = "",
                        level, level.ma,
                        seTE.orig, tau.direct = 0, sep.trts = ":",
                        method.tau = "DL",
                        func.inverse) {
  
  
  w.pooled <- 1 / seTE^2
  
  
  m <- length(TE)                        # Number of pairwise comparisons (edges)
  n <- length(unique(c(treat1, treat2))) # Number of treatments (vertices)
  df1 <- 2 * sum(1 / narms)              # Sum of degrees of freedom per study
  
  
  ##
  ## B is the edge-vertex incidence matrix (m x n)
  ##
  B <- createB(treat1.pos, treat2.pos, ncol = n)
  ##
  ## B.full is the full edge-vertex incidence matrix (m x n)
  ##
  B.full <- createB(ncol = n)
  ##
  ## M is the unweighted Laplacian, D its diagonal,
  ## A is the adjacency matrix
  ##
  M <- t(B) %*% B    # unweighted Laplacian matrix
  D <- diag(diag(M)) # diagonal matrix
  A <- D - M         # adjacency matrix (n x n)
  ##
  ## L is the weighted Laplacian (Kirchhoff) matrix (n x n)
  ## Lplus is its Moore-Penrose pseudoinverse
  ##
  L <- t(B) %*% W %*% B
  Lplus <- do.call(func.inverse, list(X = L))
  Lplus[is_zero(Lplus)] <- 0
  ##
  ## R resistance distance (variance) matrix (n x n)
  ##
  R <- matrix(0, nrow = n, ncol = n) 
  for (i in 1:n) {
    for (j in 1:n) {
      R[i, j] <- Lplus[i, i] + Lplus[j, j] - 2 * Lplus[i, j]
    }
  }
  ##
  ## V is the vector of effective variances
  ##
  V <- vector(length = m, mode = "numeric")
  for (i in 1:m) {
    V[i] <- R[treat1.pos[i], treat2.pos[i]]
  }
  ##
  ## G is the matrix B %*% Lplus %*% t(B)
  ## H is the projection matrix (also called "hat matrix")
  ##
  ## Interpretation:
  ## (i)    diag(G) = V                 The effective variances
  ## (ii)   diag(H) = V %*% W = V * w   The leverages
  ## (iii)  sum(diag(H)) = n - 1        Rank of projection
  ## (iv)   mean(diag(H)) = (n - 1) / m Mean leverage = average efficiency
  ##
  G <- B %*% Lplus %*% t(B)
  H <- G %*% W
  H[is_zero(H)] <- 0
  ##
  ## Variance-covariance matrix for all comparisons
  ##
  Cov <- B.full %*% Lplus %*% t(B.full)
  Cov[is_zero(Cov)] <- 0
  ##
  ## Resulting effects and variances at numbered edges
  ##
  v <- as.vector(H %*% TE)
  ci.v <- ci(v, sqrt(V), level = level)
  ##
  ## Resulting effects, all edges, as a n x n matrix:
  ##
  all <- matrix(NA, nrow = n, ncol = n)
  ##
  for (i in 1:m) {
    all[treat1.pos[i], treat2.pos[i]] <- v[i]
  }
  ##
  for (i in 1:n) {
    for (j in 1:n) {
      for (k in 1:n) {
        if (!is.na(all[i, k]) & !is.na(all[j, k])) {
          all[i, j] <- all[i, k] - all[j, k]
          all[j, i] <- all[j, k] - all[i, k]
        }
        if (!is.na(all[i, j]) & !is.na(all[k, j])) {
          all[i, k] <- all[i, j] - all[k, j]
          all[k, i] <- all[k, j] - all[i, j]
        }
        if (!is.na(all[i, k]) & !is.na(all[i, j])) {
          all[j, k] <- all[i, k] - all[i, j]
          all[k, j] <- all[i, j] - all[i, k]
        }
      }
    }
  }
  ##
  ## Test of total heterogeneity / inconsistency:
  ##
  Q <- as.vector(t(TE - v) %*% W %*% (TE - v))
  df <- df1 - (n - 1)
  if (df == 0)
    pval.Q <-  NA
  else
    pval.Q <- pchisq(Q, df, lower.tail = FALSE)
  ##
  ## Heterogeneity variance
  ##
  I <- diag(m)
  E <- matrix(0, nrow = m, ncol = m)
  for (i in 1:m)
    for (j in 1:m)
      E[i, j] <- as.numeric(studlab[i] == studlab[j])
  ##
  if (df == 0) {
    tau2 <- NA
    tau <- NA
    I2 <- lower.I2 <- upper.I2 <- NA
  }
  else {
    tau2 <-
      max(0, (Q - df) / sum(diag((I - H) %*% (B %*% t(B) * E / 2) %*% W)))
    tau <- sqrt(tau2)
    ci.I2 <- isquared(Q, df, level.ma)
    I2 <- ci.I2$TE
    lower.I2 <- ci.I2$lower
    upper.I2 <- ci.I2$upper
  }
  ##
  ## Decomposition of total Q into parts from pairwise meta-analyses
  ## and residual inconsistency
  ##
  Q.matrix <- matrix(0, nrow = n, ncol = n)
  n.pairwise <- 0
  data <- data.frame(treat1.pos, treat2.pos,
                     TE, w.pooled,
                     stringsAsFactors = FALSE)
  ##
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      if (A[i, j] > 1) {
        n.pairwise <- n.pairwise + 1
        sub <- data[(data$treat1.pos == i & data$treat2.pos == j) |
                    (data$treat1.pos == j & data$treat2.pos == i), ]
        l <- nrow(sub)
        Q.matrix[i, j] <-
          suppressWarnings(metagen(TE = sub$TE, seTE = 1 / sqrt(sub$w.pooled),
                                   method.tau = "DL", method.tau.ci = "")$Q)
      }
    }
  }
  q <- t1 <- t2 <- dfs <- vector(length = n.pairwise, mode = "numeric")
  p <- 0
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      if (A[i, j] > 0) {
        p <- p + 1
        t1[p] <- i
        t2[p] <- j
        q[p] <- Q.matrix[i, j]
        dfs[p] <- A[i, j] - 1
      }
    }
  }
  ##
  Q.heterogeneity <- sum(Q.matrix)
  Q.inconsistency <- Q - Q.heterogeneity
  ##
  names.treat <- sort(unique(c(treat1, treat2)))
  ##
  Q.decomp <- data.frame(treat1 = names.treat[t1],
                         treat2 = names.treat[t2],
                         Q = q,
                         df = dfs,
                         pval.Q = pchisq(q, dfs, lower.tail = FALSE),
                         stringsAsFactors = FALSE)
  ##
  Q.decomp$pval.Q[Q.decomp$df == 0] <- NA
  
  
  TE.pooled <- all
  seTE.pooled <- sqrt(R)
  ##
  ci.pooled <- ci(all, sqrt(R), level = level.ma)
  ##
  lower.pooled <- ci.pooled$lower
  upper.pooled <- ci.pooled$upper
  statistic.pooled <- ci.pooled$statistic
  pval.pooled <- ci.pooled$p
  ##
  rownames(TE.pooled) <- colnames(TE.pooled) <- names.treat
  rownames(seTE.pooled) <- colnames(seTE.pooled) <- names.treat
  rownames(lower.pooled) <- colnames(lower.pooled) <- names.treat
  rownames(upper.pooled) <- colnames(upper.pooled) <- names.treat
  rownames(statistic.pooled) <- colnames(statistic.pooled) <- names.treat
  rownames(pval.pooled) <- colnames(pval.pooled) <- names.treat
  
  
  A.matrix <- A
  L.matrix <- L
  Lplus.matrix <- Lplus
  ##
  rownames(A.matrix) <- colnames(A.matrix) <- names.treat
  rownames(L.matrix) <- colnames(L.matrix) <- names.treat
  rownames(Lplus.matrix) <- colnames(Lplus.matrix) <- names.treat
  rownames(Q.matrix) <- colnames(Q.matrix) <- names.treat
  
  G.matrix <- G
  H.matrix <- H
  ##
  rownames(G.matrix) <- colnames(G.matrix) <- studlab
  rownames(H.matrix) <- colnames(H.matrix) <- studlab
  
  names.Cov <- rep("", nrow(B.full))
  ##
  k <- 0
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      k <- k + 1
      names.Cov[k] <- paste(names.treat[i], names.treat[j], sep = sep.trts)
    }
  }
  ##
  rownames(Cov) <- colnames(Cov) <- names.Cov
  
  
  ## Calculate direct treatment estimates
  ##
  Bo <- B
  Bo[Bo == 1] <- -2
  rownames(B) <- studlab
  colnames(B) <- names.treat
  ##
  B.matrix <- B[do.call(order, data.frame(Bo)), , drop = FALSE]
  ##
  TE.direct <- seTE.direct <-
    Q.direct <- tau2.direct <- I2.direct <-
      matrix(NA, ncol = length(names.treat), nrow = length(names.treat))
  ##
  rownames(TE.direct) <- colnames(TE.direct) <-
    rownames(seTE.direct) <- colnames(seTE.direct) <-
    rownames(Q.direct) <- colnames(Q.direct) <-
    rownames(tau2.direct) <- colnames(tau2.direct) <-
    rownames(I2.direct) <- colnames(I2.direct) <-
    names.treat
  ##
  C.matrix <- B.matrix[!duplicated(B.matrix), , drop = FALSE]
  ##
  for (i in 1:dim(C.matrix)[1]) {
    sel1 <- C.matrix[i, ] ==  1
    sel2 <- C.matrix[i, ] == -1
    sel.treat1 <- colnames(C.matrix)[sel1]
    sel.treat2 <- colnames(C.matrix)[sel2]
    ##
    selstud <- treat1 == sel.treat1 & treat2 == sel.treat2
    ##
    m.i.tau.preset <-
      suppressWarnings(metagen(TE, seTE.orig, subset = selstud,
                               tau.preset = tau.direct,
                               method.tau = "DL", method.tau.ci = ""))
    ##
    if (is.na(tau.direct) | tau.direct == 0) {
      TE.i   <- m.i.tau.preset$TE.common
      seTE.i <- m.i.tau.preset$seTE.common
    }
    else {
      TE.i   <- m.i.tau.preset$TE.random
      seTE.i <- m.i.tau.preset$seTE.random
    }
    ##
    TE.direct[sel.treat1, sel.treat2]   <- TE.i
    seTE.direct[sel.treat1, sel.treat2] <- seTE.i
    ##
    TE.direct[sel.treat2, sel.treat1]   <- -TE.i
    seTE.direct[sel.treat2, sel.treat1] <- seTE.i
    ##
    m.i <-
      suppressWarnings(metagen(TE, seTE.orig, subset = selstud,
                               method.tau = method.tau, method.tau.ci = ""))
    ##
    Q.direct[sel.treat1, sel.treat2] <-
      Q.direct[sel.treat2, sel.treat1] <- m.i$Q
    tau2.direct[sel.treat1, sel.treat2] <-
      tau2.direct[sel.treat2, sel.treat1] <- m.i$tau2
    I2.direct[sel.treat1, sel.treat2] <-
      I2.direct[sel.treat2, sel.treat1] <- m.i$I2
    
  }
  ##
  ci.direct <- ci(TE.direct, seTE.direct, level = level.ma)
  ##
  lower.direct <- ci.direct$lower
  upper.direct <- ci.direct$upper
  statistic.direct <- ci.direct$statistic
  pval.direct <- ci.direct$p
  
  
  ##
  ## Contribution of individual studies to Q
  ##
  Q.pooled <- w.pooled * (TE - v)^2
  
  
  res <- list(studlab = studlab,
              treat1 = treat1, treat2 = treat2,
              TE = TE, seTE = seTE,
              seTE.orig = seTE.orig,
              TE.nma = ci.v$TE,
              seTE.nma = ci.v$seTE,
              lower.nma = ci.v$lower,
              upper.nma = ci.v$upper,
              statistic.nma = ci.v$statistic,
              pval.nma = ci.v$p,
              leverage = diag(H),
              w.pooled = w.pooled,
              Q.pooled = Q.pooled,
              treat1.pos = treat1.pos,
              treat2.pos = treat2.pos,
              ##
              TE.pooled = TE.pooled,
              seTE.pooled = seTE.pooled,
              lower.pooled = lower.pooled,
              upper.pooled = upper.pooled,
              statistic.pooled = statistic.pooled,
              pval.pooled = pval.pooled,
              ##
              k = length(unique(studlab)),
              m = length(TE),
              n = dim(TE.pooled)[[1]],
              Q = Q,
              df = df,
              pval.Q = pval.Q,
              I2 = I2,
              lower.I2 = lower.I2,
              upper.I2 = upper.I2,
              tau = tau,
              Q.heterogeneity = Q.heterogeneity,
              Q.inconsistency = Q.inconsistency,
              ##
              sm = sm,
              level = level,
              level.ma = level.ma,
              ##
              A.matrix = A.matrix,
              B.matrix = B,
              L.matrix = L.matrix,
              Lplus.matrix = Lplus.matrix,
              Q.matrix = Q.matrix,
              G.matrix = G.matrix,
              H.matrix = H.matrix,
              ##
              Cov = Cov,
              ##
              TE.direct = TE.direct,
              seTE.direct = seTE.direct,
              lower.direct = lower.direct,
              upper.direct = upper.direct,
              statistic.direct = statistic.direct,
              pval.direct = pval.direct,
              ##
              Q.direct = Q.direct,
              tau2.direct = tau2.direct,
              I2.direct = I2.direct,
              ##
              Q.decomp = Q.decomp
              )
  
  res$version <- packageDescription("netmeta")$Version
  
  res
}


nma_ruecker <- function(TE, W, seTE,
                        treat1, treat2,
                        treat1.pos, treat2.pos,
                        narms, studlab,
                        sm = "",
                        level, level.ma,
                        seTE.orig, tau.direct = 0, sep.trts = ":",
                        method.tau = "DL",
                        func.inverse,
                        Cov0) {
  
  
  w.pooled <- 1 / seTE^2
  
  m <- length(TE)                        # Number of pairwise comparisons (edges)
  n <- length(unique(c(treat1, treat2))) # Number of treatments (vertices)
  df1 <- 2 * sum(1 / narms)              # Sum of degrees of freedom per study
  
  # Drop Matrix attributes
  W <- as.matrix(W)
  class(W) <- "matrix"
  #
  Cov0 <- as.matrix(Cov0)
  class(Cov0) <- "matrix"
  
  ##
  ## B is the edge-vertex incidence matrix (m x n)
  ##
  B <- createB(treat1.pos, treat2.pos, ncol = n)
  ##
  ## B.full is the full edge-vertex incidence matrix (m x n)
  ##
  B.full <- createB(ncol = n)
  ##
  ## M is the unweighted Laplacian, D its diagonal,
  ## A is the adjacency matrix
  ##
  M <- t(B) %*% B    # unweighted Laplacian matrix
  D <- diag(diag(M)) # diagonal matrix
  A <- D - M         # adjacency matrix (n x n)
  ##
  ## L is the weighted Laplacian (Kirchhoff) matrix (n x n)
  ## Lplus is its Moore-Penrose pseudoinverse
  ##
  L <- t(B) %*% W %*% B
  Lplus <- do.call(func.inverse, list(X = L))
  Lplus[is_zero(Lplus)] <- 0
  #
  L1 <- t(B) %*% ginv(Cov0) %*% B
  Lplus1 <- do.call(func.inverse, list(X = L1))
  Lplus1[is_zero(Lplus1)] <- 0
  #
  # R resistance distance (variance) matrix (n x n)
  #
  R <- matrix(0, nrow = n, ncol = n) 
  for (i in 1:n) {
    for (j in 1:n) {
      R[i, j] <- Lplus1[i, i] + Lplus1[j, j] - 2 * Lplus1[i, j]
    }
  }
  ##
  ## V is the vector of effective variances
  ##
  V <- vector(length = m, mode = "numeric")
  for (i in 1:m) {
    V[i] <- R[treat1.pos[i], treat2.pos[i]]
  }
  ##
  ## G is the matrix B %*% Lplus %*% t(B)
  ## H is the projection matrix (also called "hat matrix")
  ##
  ## Interpretation:
  ## (i)    diag(G) = V                 The effective variances
  ## (ii)   diag(H) = V %*% W = V * w   The leverages
  ## (iii)  sum(diag(H)) = n - 1        Rank of projection
  ## (iv)   mean(diag(H)) = (n - 1) / m Mean leverage = average efficiency
  ##
  G <- B %*% Lplus %*% t(B)
  H <- G %*% W
  ##
  ## Variance-covariance matrix for all comparisons
  ##
  Cov <- B.full %*% Lplus1 %*% t(B.full)
  ##
  ## Resulting effects and variances at numbered edges
  ##
  v <- as.vector(H %*% TE)
  ci.v <- ci(v, sqrt(V), level = level)
  ##
  ## Resulting effects, all edges, as a n x n matrix:
  ##
  all <- matrix(NA, nrow = n, ncol = n)
  ##
  for (i in 1:m) {
    all[treat1.pos[i], treat2.pos[i]] <- v[i]
  }
  ##
  for (i in 1:n) {
    for (j in 1:n) {
      for (k in 1:n) {
        if (!is.na(all[i, k]) & !is.na(all[j, k])) {
          all[i, j] <- all[i, k] - all[j, k]
          all[j, i] <- all[j, k] - all[i, k]
        }
        if (!is.na(all[i, j]) & !is.na(all[k, j])) {
          all[i, k] <- all[i, j] - all[k, j]
          all[k, i] <- all[k, j] - all[i, j]
        }
        if (!is.na(all[i, k]) & !is.na(all[i, j])) {
          all[j, k] <- all[i, k] - all[i, j]
          all[k, j] <- all[i, j] - all[i, k]
        }
      }
    }
  }
  ##
  ## Test of total heterogeneity / inconsistency:
  ##
  Q <- as.vector(t(TE - v) %*% W %*% (TE - v))
  df <- df1 - (n - 1)
  if (df == 0)
    pval.Q <-  NA
  else
    pval.Q <- pchisq(Q, df, lower.tail = FALSE)
  ##
  ## Heterogeneity variance
  ##
  I <- diag(m)
  E <- matrix(0, nrow = m, ncol = m)
  for (i in 1:m)
    for (j in 1:m)
      E[i, j] <- as.numeric(studlab[i] == studlab[j])
  ##
  if (df == 0) {
    tau2 <- NA
    tau <- NA
    I2 <- lower.I2 <- upper.I2 <- NA
  }
  else {
    tau2 <-
      max(0, (Q - df) / sum(diag((I - H) %*% (B %*% t(B) * E / 2) %*% W)))
    tau <- sqrt(tau2)
    ci.I2 <- isquared(Q, df, level.ma)
    I2 <- ci.I2$TE
    lower.I2 <- ci.I2$lower
    upper.I2 <- ci.I2$upper
  }
  ##
  ## Decomposition of total Q into parts from pairwise meta-analyses
  ## and residual inconsistency
  ##
  Q.matrix <- matrix(0, nrow = n, ncol = n)
  n.pairwise <- 0
  data <- data.frame(treat1.pos, treat2.pos,
                     TE, w.pooled,
                     stringsAsFactors = FALSE)
  ##
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      if (A[i, j] > 1) {
        n.pairwise <- n.pairwise + 1
        sub <- data[(data$treat1.pos == i & data$treat2.pos == j) |
                      (data$treat1.pos == j & data$treat2.pos == i), ]
        l <- nrow(sub)
        Q.matrix[i, j] <-
          suppressWarnings(metagen(TE = sub$TE, seTE = 1 / sqrt(sub$w.pooled),
                                   method.tau = "DL", method.tau.ci = "")$Q)
      }
    }
  }
  q <- t1 <- t2 <- dfs <- vector(length = n.pairwise, mode = "numeric")
  p <- 0
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      if (A[i, j] > 0) {
        p <- p + 1
        t1[p] <- i
        t2[p] <- j
        q[p] <- Q.matrix[i, j]
        dfs[p] <- A[i, j] - 1
      }
    }
  }
  ##
  Q.heterogeneity <- sum(Q.matrix)
  Q.inconsistency <- Q - Q.heterogeneity
  ##
  names.treat <- sort(unique(c(treat1, treat2)))
  ##
  Q.decomp <- data.frame(treat1 = names.treat[t1],
                         treat2 = names.treat[t2],
                         Q = q,
                         df = dfs,
                         pval.Q = pchisq(q, dfs, lower.tail = FALSE),
                         stringsAsFactors = FALSE)
  ##
  Q.decomp$pval.Q[Q.decomp$df == 0] <- NA
  
  
  TE.pooled <- all
  seTE.pooled <- sqrt(R)
  ##
  ci.pooled <- ci(all, sqrt(R), level = level.ma)
  ##
  lower.pooled <- ci.pooled$lower
  upper.pooled <- ci.pooled$upper
  statistic.pooled <- ci.pooled$statistic
  pval.pooled <- ci.pooled$p
  ##
  rownames(TE.pooled) <- colnames(TE.pooled) <- names.treat
  rownames(seTE.pooled) <- colnames(seTE.pooled) <- names.treat
  rownames(lower.pooled) <- colnames(lower.pooled) <- names.treat
  rownames(upper.pooled) <- colnames(upper.pooled) <- names.treat
  rownames(statistic.pooled) <- colnames(statistic.pooled) <- names.treat
  rownames(pval.pooled) <- colnames(pval.pooled) <- names.treat
  
  
  A.matrix <- A
  L.matrix <- L
  Lplus.matrix <- Lplus
  ##
  rownames(A.matrix) <- colnames(A.matrix) <- names.treat
  rownames(L.matrix) <- colnames(L.matrix) <- names.treat
  rownames(Lplus.matrix) <- colnames(Lplus.matrix) <- names.treat
  rownames(Q.matrix) <- colnames(Q.matrix) <- names.treat
  
  G.matrix <- G
  H.matrix <- H
  ##
  rownames(G.matrix) <- colnames(G.matrix) <- studlab
  rownames(H.matrix) <- colnames(H.matrix) <- studlab
  
  names.Cov <- rep("", nrow(B.full))
  ##
  k <- 0
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      k <- k + 1
      names.Cov[k] <- paste(names.treat[i], names.treat[j], sep = sep.trts)
    }
  }
  ##
  rownames(Cov) <- colnames(Cov) <- names.Cov
  
  
  ## Calculate direct treatment estimates
  ##
  Bo <- B
  Bo[Bo == 1] <- -2
  rownames(B) <- studlab
  colnames(B) <- names.treat
  ##
  B.matrix <- B[do.call(order, data.frame(Bo)), , drop = FALSE]
  ##
  TE.direct <- seTE.direct <-
    Q.direct <- tau2.direct <- I2.direct <-
    matrix(NA, ncol = length(names.treat), nrow = length(names.treat))
  ##
  rownames(TE.direct) <- colnames(TE.direct) <-
    rownames(seTE.direct) <- colnames(seTE.direct) <-
    rownames(Q.direct) <- colnames(Q.direct) <-
    rownames(tau2.direct) <- colnames(tau2.direct) <-
    rownames(I2.direct) <- colnames(I2.direct) <-
    names.treat
  ##
  C.matrix <- B.matrix[!duplicated(B.matrix), , drop = FALSE]
  ##
  for (i in 1:dim(C.matrix)[1]) {
    sel1 <- C.matrix[i, ] ==  1
    sel2 <- C.matrix[i, ] == -1
    sel.treat1 <- colnames(C.matrix)[sel1]
    sel.treat2 <- colnames(C.matrix)[sel2]
    ##
    selstud <- treat1 == sel.treat1 & treat2 == sel.treat2
    ##
    m.i.tau.preset <-
      suppressWarnings(metagen(TE, seTE.orig, subset = selstud,
                               tau.preset = tau.direct,
                               method.tau = "DL", method.tau.ci = ""))
    ##
    if (is.na(tau.direct) | tau.direct == 0) {
      TE.i   <- m.i.tau.preset$TE.common
      seTE.i <- m.i.tau.preset$seTE.common
    }
    else {
      TE.i   <- m.i.tau.preset$TE.random
      seTE.i <- m.i.tau.preset$seTE.random
    }
    ##
    TE.direct[sel.treat1, sel.treat2]   <- TE.i
    seTE.direct[sel.treat1, sel.treat2] <- seTE.i
    ##
    TE.direct[sel.treat2, sel.treat1]   <- -TE.i
    seTE.direct[sel.treat2, sel.treat1] <- seTE.i
    ##
    m.i <-
      suppressWarnings(metagen(TE, seTE.orig, subset = selstud,
                               method.tau = method.tau, method.tau.ci = ""))
    ##
    Q.direct[sel.treat1, sel.treat2] <-
      Q.direct[sel.treat2, sel.treat1] <- m.i$Q
    tau2.direct[sel.treat1, sel.treat2] <-
      tau2.direct[sel.treat2, sel.treat1] <- m.i$tau2
    I2.direct[sel.treat1, sel.treat2] <-
      I2.direct[sel.treat2, sel.treat1] <- m.i$I2
    
  }
  ##
  ci.direct <- ci(TE.direct, seTE.direct, level = level.ma)
  ##
  lower.direct <- ci.direct$lower
  upper.direct <- ci.direct$upper
  statistic.direct <- ci.direct$statistic
  pval.direct <- ci.direct$p
  
  
  ##
  ## Contribution of individual studies to Q
  ##
  Q.pooled <- w.pooled * (TE - v)^2
  
  
  res <- list(studlab = studlab,
              treat1 = treat1, treat2 = treat2,
              TE = TE, seTE = seTE,
              seTE.orig = seTE.orig,
              TE.nma = ci.v$TE,
              seTE.nma = ci.v$seTE,
              lower.nma = ci.v$lower,
              upper.nma = ci.v$upper,
              statistic.nma = ci.v$statistic,
              pval.nma = ci.v$p,
              leverage = diag(H),
              w.pooled = w.pooled,
              Q.pooled = Q.pooled,
              treat1.pos = treat1.pos,
              treat2.pos = treat2.pos,
              ##
              TE.pooled = TE.pooled,
              seTE.pooled = seTE.pooled,
              lower.pooled = lower.pooled,
              upper.pooled = upper.pooled,
              statistic.pooled = statistic.pooled,
              pval.pooled = pval.pooled,
              ##
              k = length(unique(studlab)),
              m = length(TE),
              n = dim(TE.pooled)[[1]],
              Q = Q,
              df = df,
              pval.Q = pval.Q,
              I2 = I2,
              lower.I2 = lower.I2,
              upper.I2 = upper.I2,
              tau = tau,
              Q.heterogeneity = Q.heterogeneity,
              Q.inconsistency = Q.inconsistency,
              ##
              sm = sm,
              level = level,
              level.ma = level.ma,
              ##
              A.matrix = A.matrix,
              B.matrix = B,
              L.matrix = L.matrix,
              Lplus.matrix = Lplus.matrix,
              Q.matrix = Q.matrix,
              G.matrix = G.matrix,
              H.matrix = H.matrix,
              ##
              Cov = Cov,
              ##
              TE.direct = TE.direct,
              seTE.direct = seTE.direct,
              lower.direct = lower.direct,
              upper.direct = upper.direct,
              statistic.direct = statistic.direct,
              pval.direct = pval.direct,
              ##
              Q.direct = Q.direct,
              tau2.direct = tau2.direct,
              I2.direct = I2.direct,
              ##
              Q.decomp = Q.decomp
  )
  
  res$version <- packageDescription("netmeta")$Version
  
  res
}

#' Network graph
#' 
#' @description
#' This function generates a graph of the evidence network.
#' 
#' @param x An object of class \code{netmeta} (mandatory).
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments arrangement (anticlockwise if \code{start.layout =
#'   "circle"}).
#' @param labels An optional vector with treatment labels.
#' @param cex The magnification to be used for treatment labels.
#' @param col A single color (or vector of colors) for lines
#'   connecting treatments (edges) if argument \code{plastic =
#'   FALSE}. Length of the vector must be equal to the number of edges
#'   (see list element 'comparisons' in \code{\link{netmeta}}).
#' @param adj One, two, or three values in [0, 1] (or a vector /
#'   matrix with length / number of rows equal to the number of
#'   treatments) specifying the x (and optionally y and z) adjustment
#'   for treatment labels.
#' @param offset Distance between nodes (i.e. treatments) in graph and
#'   treatment labels for 2-D plots (value of 0.0175 corresponds to a
#'   difference of 1.75\% of the range on x- and y-axis).
#' @param srt.labels The character string \code{"orthogonal"} (can be
#'   abbreviated), a single numeric or numerical vector with value(s)
#'   between -180 and 180 specifying the angle to rotate treatment
#'   labels (see Details).
#' @param scale Additional space added outside of nodes
#'   (i.e. treatments).  Increase this value for larger treatment
#'   labels (value of 1.10 corresponds to an additional space of 10\%
#'   around the network graph).
#' @param plastic A logical indicating whether the appearance of the
#'   comparisons should be in '3D look' (not to be confused with
#'   argument \code{dim}).
#' @param thickness Either a character variable to determine the
#'   method to plot line widths (see Details) or a matrix of the same
#'   dimension and row and column names as argument \code{A.matrix}
#'   with information on line width.
#' @param lwd A numeric for scaling the line width of comparisons.
#' @param lwd.min Minimum line width in network graph. All connections
#'   with line widths below this values will be set to \code{lwd.min}.
#' @param lwd.max Maximum line width in network graph. The connection
#'   with the largest value according to argument \code{thickness}
#'   will be set to this value.
#' @param rescale.thickness A logical value or R function to scale the
#'   thickness of lines (see Details).
#' @param dim A character string indicating whether a 2- or
#'   3-dimensional plot should be produced, either \code{"2d"} or
#'   \code{"3d"}.
#' @param rotate A single numeric with value between -180 and 180
#'   specifying the degree to rotate nodes in a circular network clockwise or
#'   anticlockwise.
#' @param highlight A character vector identifying comparisons that
#'   should be marked in the network graph, e.g. \code{highlight =
#'   "treat1:treat2"}.
#' @param col.highlight Color(s) to highlight the comparisons given by
#'   \code{highlight}.
#' @param scale.highlight Scaling factor(s) for the line width(s) to
#'   highlight the comparisons given by \code{highlight}.
#' @param multiarm A logical indicating whether multi-arm studies
#'   should be marked in plot.
#' @param col.multiarm Either a function from R package
#'   \bold{colorspace} or \bold{grDevice} to define colors for
#'   multi-arm studies or a character vector with colors to highlight
#'   multi-arm studies.
#' @param alpha.transparency The alpha transparency of colors used to
#'   highlight multi-arm studies (0 means transparent and 1 means
#'   opaque).
#' @param points A logical indicating whether points should be printed
#'   at nodes (i.e. treatments) of the network graph.
#' @param cex.points,pch.points,col.points,bg.points Corresponding
#'   size, type, color, and background color for points. Can be a
#'   vector with length equal to the number of treatments.
#' @param points.min Minimum point size. All points with size below
#'   this values will be set to \code{points.min}.
#' @param points.max Maximum point size in network graph. The node
#'   with the largest value according to argument \code{cex.points}
#'   will be set to this value.
#' @param rescale.pointsize A logical value or R function to scale the
#'   point size (see Details).
#' @param number.of.studies A logical indicating whether number of
#'   studies should be added to network graph.
#' @param cex.number.of.studies The magnification to be used for
#'   number of studies.
#' @param col.number.of.studies Color for number of studies.
#' @param bg.number.of.studies Color for shadow around number of
#'   studies.
#' @param pos.number.of.studies A single value (or vector of values)
#'   in [0, 1] specifying the position of the number of studies on the
#'   lines connecting treatments (edges). Length of the vector must be
#'   equal to the number of edges.
#' @param start.layout A character string indicating which starting
#'   layout is used if \code{iterate = TRUE}. If "circle" (default),
#'   the iteration starts with a circular ordering of the vertices; if
#'   "eigen", eigenvectors of the Laplacian matrix are used,
#'   calculated via generic function \code{\link{eigen}} (spectral
#'   decomposition); if "prcomp", eigenvectors of the Laplacian matrix
#'   are calculated via generic function \code{\link{prcomp}}
#'   (principal component analysis); if "random", a random layout is
#'   used, drawn from a bivariate normal.
#' @param eig1 A numeric indicating which eigenvector is used as x
#'   coordinate if \code{start = "eigen"} or \code{"prcomp"} and
#'   \code{iterate = TRUE}.  Default is 2, the eigenvector to the
#'   second-smallest eigenvalue of the Laplacian matrix.
#' @param eig2 A numeric indicating which eigenvector is used as
#'   y-coordinate if \code{start = "eigen"} or \code{"prcomp"} and
#'   \code{iterate = TRUE}.  Default is 3, the eigenvector to the
#'   third-smallest eigenvalue of the Laplacian matrix.
#' @param eig3 A numeric indicating which eigenvector is used as
#'   z-coordinate if \code{start = "eigen"} or \code{"prcomp"} and
#'   \code{iterate = TRUE}.  Default is 4, the eigenvector to the
#'   fourth-smallest eigenvalue of the Laplacian matrix.
#' @param iterate A logical indicating whether the stress majorization
#'   algorithm is carried out for optimization of the layout.
#' @param tol A numeric for the tolerance for convergence if
#'   \code{iterate = TRUE}.
#' @param maxit An integer defining the maximum number of iteration
#'   steps if \code{iterate = TRUE}.
#' @param allfigures A logical indicating whether all iteration steps
#'   are shown if \code{iterate = TRUE}. May slow down computations if
#'   set to \code{TRUE} (especially if \code{plastic = TRUE}).
#' @param A.matrix Adjacency matrix (\emph{n}x\emph{n}) characterizing
#'   the structure of the network graph. Row and column names must be
#'   the same set of values as provided by argument \code{seq}.
#' @param N.matrix Neighborhood matrix (\emph{n}x\emph{n}) replacing
#'   A.matrix if neighborhood is to be specified differently from node
#'   adjacency in the network graph, for example content-based. Row
#'   and column names must be the same set of values as provided by
#'   argument \code{seq}.
#' @param D.matrix Distance matrix (\emph{n}x\emph{n}) replacing
#'   A.matrix and N.matrix if distances should be provided
#'   directly. Row and column names must be the same set of values as
#'   provided by argument \code{seq}.
#' @param xpos Vector (\emph{n}) of x coordinates.
#' @param ypos Vector (\emph{n}) of y coordinates.
#' @param zpos Vector (\emph{n}) of z coordinates.
#' @param figure A logical indicating whether network graph should be
#'   shown.
#' @param \dots Additional graphical arguments (passed on to
#'   \code{\link{plot.default}}).
#' 
#' @details
#' This function generates a network graph for an R object created
#' with \code{\link{netmeta}}. R function \code{\link{plot.default}} is used
#' to create the network graph.
#'
#' \subsection{Layout of network graph}{
#' The network is laid out in the plane, where the nodes in the graph
#' layout correspond to the treatments and edges display the observed
#' treatment comparisons. For the default setting, nodes are placed on
#' a circle.  Other starting layouts are "eigen", "prcomp", and
#' "random" (Rücker & Schwarzer 2015). If \code{iterate = TRUE}, the
#' layout is further optimized using the stress majorization
#' algorithm. This algorithm specifies an 'ideal' distance (e.g., the
#' graph distance) between two nodes in the plane. In the optimal
#' layout, these distances are best approximated in the sense of least
#' squares. Starting from an initial layout, the optimum is
#' approximated in an iterative process called stress majorization
#' (Kamada and Kawai 1989, Michailidis and de Leeuw 2001, Hu
#' 2012). The starting layout can be chosen as a circle or coming from
#' eigenvectors of the Laplacian matrix (corresponding to Hall's
#' algorithm, Hall 1970), calculated in different ways, or
#' random. Moreover, it can be chosen whether the iteration steps are
#' shown (argument \code{allfigures = TRUE}).
#' 
#' An optimized circular presentation which typically has a reduced
#' (sometimes minimal) number of crossings can be achieved by using
#' argument \code{seq = "optimal"} in combination with argument
#' \code{start.layout}. Note, is is not possible of prespecify the
#' best value for argument \code{start.layout} for any situation as
#' the result depends on the network structure.
#' }
#' 
#' \subsection{Definition of line widths}{
#' Argument \code{thickness} providing the line width of edges
#' (comparisons) can be a matrix of the same dimension as argument
#' \code{A.matrix} or any of the following character strings (which
#' can be abbreviated):
#' \itemize{
#' \item Proportional to number of studies comparing two treatments
#'   (\code{thickness = "number.of.studies"}, default)
#' \item Proportional to inverse standard error of common effects model
#'   comparing two treatments (\code{thickness = "se.common"})
#' \item Proportional to inverse standard error of random effects
#'   model comparing two treatments (\code{thickness = "se.random"})
#' \item Weight from common effects model comparing two treatments
#'   (\code{thickness = "w.common"})
#' \item Weight from random effects model comparing two treatments
#'   (\code{thickness = "w.random"})
#' \item Same line width for all comparisons (\code{thickness =
#'   "equal"})
#' }
#'
#' Only evidence from direct treatment comparisons is considered to
#' determine the line width if argument \code{thickness} is equal to
#' any but the last method.
#'
#' Line widths are determined by argument \code{lwd} if all lines have
#' the same width. This is possible if either argument \code{thickness
#' = "equal"}, all pairwise comparisons have the same number of
#' studies for \code{thickness = "number.of.studies"} or all direct
#' comparisons are equally precise.
#'
#' Otherwise, the line width of the thickest line is equal to the
#' value of argument \code{lwd.max} and all lines with a thickness
#' below the value of argument \code{lwd.min} are set to this
#' value. Default for argument \code{lwd.max} is \code{4 * lwd}.
#' 
#' Argument \code{rescale.thickness} can be used to provide a function
#' to specify the relative line width of edges (comparisons). By
#' default, the square root function \code{\link[base]{sqrt}} is used
#' in order to lessen differences in line widths. Argument
#' \code{rescale.thickness = FALSE} or \code{rescale.thickness = I},
#' i.e., the identity function \code{\link[base]{I}}, can be used to
#' not rescale line widths.
#' }
#' 
#' \subsection{Definition of point sizes}{
#' Points are printed at nodes (treatments) if argument \code{points =
#' TRUE} or argument \code{cex.points} is provided.
#'
#' Point sizes are equal to the value of argument \code{cex.points} if
#' all points are of equal size. 
#'
#' Otherwise, the point size of the largest point is equal to the
#' value of argument \code{points.max} and all points smaller than the
#' value of argument \code{points.min} are set to this value. The
#' default for argument \code{points.max} is equal to the largest
#' value provided in argument \code{cex.points} if this largest value
#' is below or equal to 25. Otherwise the default is \code{points.max
#' = 8}.
#' 
#' Argument \code{rescale.pointsize} can be used to provide a function
#' to specify relative point sizes. Point sizes are not rescaled at
#' all if they are all equal or the largest \code{cex.points} value is
#' below or equal to 25. Otherwise, the square root function
#' \code{\link[base]{sqrt}} is used in order to lessen the differences
#' in point sizes. Argument \code{rescale.pointsize = FALSE} or
#' \code{rescale.pointsize = I}, i.e., the identity function
#' \code{\link[base]{I}}, can be used to not rescale point sizes.
#' }
#' 
#' \subsection{Other settings}{
#' Argument \code{srt.labels} can be used to specific the rotation (in
#' degrees) of the treatment labels. If \code{srt.labels} is equal to
#' \code{"orthogonal"}, treatment labels are orthogonal to the
#' circle. If \code{srt.labels} is a single numeric, all labels are
#' rotated by this degree. If \code{srt.labels} is a numeric vector,
#' it must be of the same length as the number of treatments and
#' labels are rotated counter-clockwise starting on the right
#' side. Finally, if \code{srt.labels} is a named numeric vector, it
#' must be of the same length as the number of treatments and the
#' names must be equal to the treatment names (and treatment labels
#' are rotated according to the specified values).
#' 
#' Further, a couple of graphical parameters can be specified, such as
#' color and appearance of the nodes (treatments) and the edges
#' (comparisons), whether special comparisons should be highlighted
#' and whether multi-arm studies should be indicated as colored
#' polygons. By default, the \code{\link[colorspace]{sequential_hcl}}
#' function is used to highlight multi-arm studies.
#' 
#' In order to generate 3-D plots (argument \code{dim = "3d"}), R
#' package \bold{rgl} is necessary. Note, under macOS the X.Org X
#' Window System must be available (see
#' \url{https://www.xquartz.org}).
#' }
#'
#' @return
#' A list containing two data frames with information on nodes and
#' edges.
#' 
#' \bold{List element 'nodes'}
#' \item{trts}{Treatment names.}
#' \item{labels}{Treatment labels.}
#' \item{seq}{Sequence of treatment labels.}
#' \item{srt}{String rotation.}
#' \item{xpos}{Position of treatment / node on x-axis.}
#' \item{ypos}{Position of treatment / node on y-axis.}
#' \item{zpos}{Position of treatment / node on z-axis (for 3-D
#'   plots).}
#' \item{xpos.labels}{Position of treatment labels on x-axis (for 2-D
#'   plots).}
#' \item{ypos.labels}{Position of treatment labels on y-axis (for 2-D
#'   plots).}
#' \item{offset.x}{Offset of treatment labels on x-axis (for 2-D
#'   plots).}
#' \item{offset.y}{Offset of treatment labels on y-axis (for 2-D
#'   plots).}
#' \item{cex}{Point size of treatments / nodes}
#' \item{col}{Color for points.}
#' \item{pch}{Point type.}
#' \item{bg}{Background color for points.}
#' \item{adj.x}{Adjustment for treatment label on x-axis.}
#' \item{adj.y}{Adjustment for treatment label on y-axis.}
#' \item{adj.z}{Adjustment for treatment label on z-axis (for 3-D
#'   plots).}
#' 
#' \bold{List element 'edges'}
#' \item{treat1}{Name of first treatment.}
#' \item{treat2}{Name of second treatment.}
#' \item{n.stud}{Number of studies directly comparing treatments.}
#' \item{xpos}{Position of number of studies on x-axis.}
#' \item{ypos}{Position of number of studies on y-axis.}
#' \item{adj}{Adjustment of number of studies.}
#' \item{pos.number.of.studies}{Position of number of studies on
#'   edge.}
#' \item{col}{Color for lines / edges.}
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Ulrike
#'   Krahn \email{ulrike.krahn@@bayer.com}, Jochem König
#'   \email{koenigjo@@uni-mainz.de}, Guido Schwarzer
#'   \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}
#' 
#' @references
#'
#' Hall KM (1970):
#' An r-dimensional quadratic placement algorithm.
#' \emph{Management Science},
#' \bold{17}, 219--29
#' 
#' Hu Y (2012):
#' \emph{Combinatorial Scientific Computing}, Chapter Algorithms for
#' Visualizing Large Networks, pages 525--49.
#' Chapman and Hall / CRC,  Computational Science.
#' 
#' Kamada T, Kawai S (1989):
#' An algorithm for drawing general undirected graphs.
#' \emph{Information Processing Letters},
#' \bold{31}, 7--15
#' 
#' Krahn U, Binder H, König J (2013):
#' A graphical tool for locating inconsistency in network meta-analyses.
#' \emph{BMC Medical Research Methodology},
#' \bold{13}, 35
#' 
#' Michailidis G, de Leeuw J (2001):
#' Data visualization through graph drawing.
#' \emph{Computational Statistics},
#' \bold{16}, 435--50
#' 
#' Rücker G, Schwarzer G (2016):
#' Automated drawing of network plots in network meta-analysis.
#' \emph{Research Synthesis Methods},
#' \bold{7}, 94--107
#' 
#' @keywords hplot
#'
#' @examples
#' data(smokingcessation)
#' 
#' # Transform data from arm-based format to contrast-based format
#' #
#' pw1 <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' 
#' # Conduct random effects network meta-analysis
#' #
#' nma1 <- netmeta(pw1, common = FALSE)
#' 
#' # Network graph with default settings
#' #
#' netgraph(nma1)
#' 
#' \donttest{
#' # Generation of an object of class 'netmeta' with reference
#' # treatment 'plac'
#' #
#' pw2 <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' nma2 <- netmeta(pw2, reference = "plac")
#' 
#' # Network graph with default settings
#' #
#' netgraph(nma2)
#' 
#' # Network graph with specified order of the treatments and one
#' # highlighted comparison
#' #
#' trts <- c("plac", "benf", "migl", "acar", "sulf",
#'   "metf", "rosi", "piog", "sita", "vild")
#' netgraph(nma2, highlight = "rosi:plac", seq = trts)
#' 
#' # Same network graph using argument 'seq' in netmeta function
#' #
#' nma3 <- netmeta(pw2, reference = "plac", seq = trts)
#' netgraph(nma3, highlight = "rosi:plac")
#' 
#' # Network graph optimized, starting from a circle, with multi-arm
#' # study colored
#' #
#' netgraph(nma2, start = "circle", iterate = TRUE,
#'   multiarm = TRUE, col.multiarm = "purple")
#'
#' # Network graph optimized, starting from a circle, with multi-arm
#' # study colored and all intermediate iteration steps visible
#' #
#' netgraph(nma2, start = "circle", iterate = TRUE,
#'   multiarm = TRUE, col.multiarm = "purple",
#'   allfigures = TRUE)
#' 
#' # Network graph optimized, starting from Laplacian eigenvectors,
#' # with multi-arm study colored
#' #
#' netgraph(nma2, start = "eigen",
#'   multiarm = TRUE, col.multiarm = "purple")
#' 
#' # Network graph optimized, starting from different Laplacian
#' # eigenvectors, with multi-arm study colored
#' #
#' netgraph(nma2, start = "prcomp",
#'   multiarm = TRUE, col.multiarm = "purple")
#' 
#' # Network graph optimized, starting from random initial layout,
#' # with multi-arm study colored
#' #
#' netgraph(nma2, start = "random",
#'   multiarm = TRUE, col.multiarm = "purple")
#' 
#' # Network graph without plastic look and one highlighted comparison
#' #
#' netgraph(nma2, plastic = FALSE, highlight = "rosi:plac")
#' 
#' # Network graph with same thickness for all comparisons
#' #
#' netgraph(nma2, thickness = "equal")
#' 
#' # Network graph with changed labels and specified order of the
#' # treatments
#' #
#' netgraph(nma2, seq = c(1, 3, 5, 2, 9, 4, 7, 6, 8, 10),
#'   labels = LETTERS[1:10])
#' 
#' # Rotate treatment labels (orthogonal to circle)
#' #
#' netgraph(nma2, srt.labels = "o")
#' 
#' # Network graph in 3-D (opens a new device, where you may rotate and
#' # zoom the plot using the mouse / the mouse wheel).
#' # The rgl package must be installed for 3-D plots.
#' #
#' if (requireNamespace("rgl", quietly = TRUE))
#'   netgraph(nma2, dim = "3d")
#' }
#' 
#' @method netgraph netmeta
#' @export

netgraph.netmeta <- function(x, seq = x$seq,
                             labels = x$trts,
                             cex = 1, adj = NULL, srt.labels = 0,
                             offset =
                               if (!is.null(adj) && all(unique(adj) == 0.5))
                                 0
                               else
                                 0.0175,
                             scale = 1.10,
                             ##
                             col = gs("col.netgraph"),
                             plastic = gs("plastic"),
                             thickness = gs("thickness"),
                             lwd = 5, lwd.min = lwd / 2.5, lwd.max,
                             rescale.thickness,
                             ##
                             dim = "2d",
                             rotate = 0,
                             ##
                             highlight = NULL, col.highlight = "red2",
                             scale.highlight = 1,
                             ##
                             multiarm = gs("multiarm"),
                             col.multiarm = NULL,
                             alpha.transparency = 0.5,
                             ##
                             points = !missing(cex.points),
                             cex.points = 1,
                             pch.points = 20,
                             col.points =
                               if (length(pch.points) == 1 && pch.points == 21)
                                 "black" else "red",
                             bg.points = "red",
                             points.min, points.max, rescale.pointsize,
                             #
                             number.of.studies = gs("number.of.studies"),
                             cex.number.of.studies = cex,
                             col.number.of.studies = "white",
                             bg.number.of.studies = "black",
                             pos.number.of.studies = 0.5,
                             ##
                             start.layout =
                               ifelse(dim == "2d", "circle", "eigen"),
                             eig1 = 2, eig2 = 3, eig3 = 4,
                             iterate = FALSE,
                             tol = 0.0001, maxit = 500, allfigures = FALSE,
                             ##
                             A.matrix = x$A.matrix,
                             N.matrix = sign(A.matrix),
                             D.matrix = netdistance(N.matrix),
                             ##
                             xpos = NULL, ypos = NULL, zpos = NULL,
                             figure = TRUE,
                             ...) {
  
  
  chkclass(x, "netmeta")
  x <- updateversion(x)
  ##
  n.edges <- sum(x$A.matrix[upper.tri(x$A.matrix)] > 0)
  n.trts <- length(x$trts)
  ##
  chklogical(points)
  chknumeric(lwd, min = 0, zero = TRUE, length = 1)
  chknumeric(lwd.min, min = 0, zero = TRUE, length = 1)
  ##
  missing.lwd.max <- missing(lwd.max)
  if (missing.lwd.max) {
    if (!is.matrix(thickness) && thickness == "equal")
      lwd.max <- 2 * lwd
    else
      lwd.max <- 4 * lwd
  }
  ##
  if (lwd.min > lwd.max)
    stop("Argument 'lwd.min' must be smaller than 'lwd.max'.")
  ##
  if (missing(rescale.thickness)) {
    if (!is.matrix(thickness) && thickness == "equal")
      rescale.thickness <- I
    else
      rescale.thickness <- sqrt
  }
  else {
    if (is.logical(rescale.thickness)) {
      if (rescale.thickness) {
        if (!is.matrix(thickness) && thickness == "equal")
          rescale.thickness <- I
        else
          rescale.thickness <- sqrt
      }
      else
        rescale.thickness <- I
    }
    else if (!is.function(rescale.thickness))
      stop("Argument 'rescale.thickness' must be a logical value or ",
           "an R function to rescale line widths.",
           call. = FALSE)
  }
  ##
  chknumeric(scale.highlight, min = 0, zero = TRUE)
  ##
  dim <- setchar(dim, c("2d", "3d"))
  is_2d <- dim == "2d"
  is_3d <- !is_2d
  ##
  if (is_3d & !is_installed_package("rgl", stop = FALSE)) {
    warning(paste0("2-D plot generated as package 'rgl' is missing.",
                   "\n  ",
                   "Please install package 'rgl' in order to ",
                   "produce 3-D plots\n  ",
                   "(R command: 'install.packages(\"rgl\")').",
                   if (length(grep("darwin", R.Version()$os)) == 1)
                     paste0("\n  Note, macOS users have to install ",
                            "XQuartz, see https://www.xquartz.org/.")
                   ),
            call. = FALSE)
    dim <- "2d"
    is_2d <- TRUE
    is_3d <- FALSE
  }
  ##
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  ##
  missing.rotate <- missing(rotate)
  if (!missing.rotate)
    rotate <- catch("rotate", mc, x, sfsp)
  chknumeric(rotate, min = -180, max = 180, length = 1)
  ##
  missing.start.layout <- missing(start.layout)
  start.layout <-
    setchar(start.layout, c("eigen", "prcomp", "circle", "random"))
  ##
  if (!missing(seq) & is.null(seq))
    stop("Argument 'seq' must be not NULL.")
  ##
  if (is.null(seq) | (length(seq) == 1 & x$d == 1))
    seq1 <- 1:length(labels)
  else if (length(seq) == 1 & x$d > 1) {
    seq <- setchar(seq, "optimal", "should be equal to ",
                   "'optimal' or a permutation of treatments")
    ##
    if (missing.start.layout)
      start.layout <- "eigen"
    ##
    seq1 <- optcircle(x, start.layout = start.layout)$seq
    ##
    start.layout <- "circle"
  }
  else if (!(start.layout == "circle" &
             (missing(iterate) || iterate == FALSE))) {
    seq1 <- 1:length(labels)
    ##
    if (!missing(seq) & !is.null(seq) & (is.null(xpos) & is.null(ypos)))
      warning("Argument 'seq' only considered if ",
              "start.layout=\"circle\" and iterate=FALSE.",
              call. = FALSE)
  }
  else
    seq1 <- charmatch(setseq(seq, x$trts), x$trts)
  ##
  if (missing(iterate))
    iterate <- ifelse(start.layout == "circle", FALSE, TRUE)
  else {
    chklogical(iterate)
    #
    if (iterate && length(seq) == 1 && seq == "optimal") {
      warning("Argument 'iterate' ignored as argument 'seq' is ",
              "equal to \"optimal\".",
              call. = FALSE)
      #
      iterate <- FALSE
    }
  }
  #
  col <- replaceNULL(col, if (iterate) "slateblue" else "black")
  #
  plastic <- replaceNULL(plastic, !(iterate & allfigures))
  chklogical(plastic)
  ##
  if (!missing(labels)) {
    ##
    labels <- catch("labels", mc, x, sfsp)
    ##
    if (is.null(labels))
      stop("Argument 'labels' must be not NULL.")
  }
  ##
  ## Colors of edges
  ##
  if (is.matrix(col)) {
    if ((dim(col)[1] != dim(A.matrix)[1]) |
        (dim(col)[2] != dim(A.matrix)[2]))
      stop("Dimension of argument 'A.matrix' and 'col' are different.")
    if (is.null(dimnames(col)))
      stop("Matrix 'col' must have row and column names identical ",
           "to argument 'A.matrix'.")
    else {
      if (any(rownames(col) != rownames(A.matrix)))
        stop("Row names of matrix 'col' must be identical to ",
             "argument 'A.matrix'.")
      if (any(colnames(col) != colnames(A.matrix)))
        stop("Column names of matrix 'col' must be identical to ",
             "argument 'A.matrix'.")
    }
    ##
    col <- col[lower.tri(col)]
    col <- col[!is.na(col)]
  }
  ##
  n.col <- length(col)
  ##
  if (n.col == 1)
    col <- rep(col, n.edges)
  else if (n.col != n.edges)
    stop("Length of argument 'col' (",
         n.col, ") is different from the number of ",
         "direct pairwise comparisons (", n.edges, ")")
  ##
  n.pos <- length(pos.number.of.studies)
  if (n.pos == 1)
    pos.number.of.studies <- rep(pos.number.of.studies, n.edges)
  else if (n.pos != n.edges)
    stop("Length of argument 'pos.number.of.studies' (",
         n.pos, ") is different from the number of ",
         "direct pairwise comparisons (", n.edges, ")")
  ##
  if (!missing(adj)) {
    adj <- catch("adj", mc, x, sfsp)
    if (is.data.frame(adj))
      adj <- as.matrix(adj)
    if (is.logical(adj))
      adj <- 1L * adj
  }
  ##
  if (!missing(offset))
    offset <- catch("offset", mc, x, sfsp)
  ##
  if (!missing(cex.points))
    cex.points <- catch("cex.points", mc, x, sfsp)
  ##
  if (length(cex.points) == 1)
    cex.points <- rep(cex.points, n.trts)
  else if (length(cex.points) != n.trts)
    stop("Length of argument 'cex.points' must be equal to the ",
         "number of treatments.")
  ##
  if (missing(points.min)) {
    if (length(unique(cex.points)) == 1 | max(cex.points, na.rm = TRUE) <= 25)
      points.min <- NULL
    else
      points.min <- 1
  }
  if (!is.null(points.min))
    chknumeric(points.min, 0, zero = TRUE, length = 1)
  ##
  if (missing(points.max) ) {
    if (length(unique(cex.points)) == 1)
      points.max <- NULL
    else if (max(cex.points, na.rm = TRUE) <= 25)
      points.max <- max(cex.points, na.rm = TRUE)
    else {
      if (length(pch.points) == 1 && pch.points == 21)
        points.max <- 8
      else
        points.max <- 12
    }
  }
  if (!is.null(points.max))
    chknumeric(points.max, 0, zero = TRUE, length = 1)
  ##
  if (missing(rescale.pointsize)) {
    if (length(unique(cex.points)) == 1 | max(cex.points, na.rm = TRUE) <= 25)
      rescale.pointsize <- NULL
    else
      rescale.pointsize <- sqrt
  }
  else {
    if (is.logical(rescale.pointsize)) {
      if (rescale.pointsize) {
        if (length(unique(cex.points)) == 1 | max(cex.points, na.rm = TRUE) <= 25)
          rescale.pointsize <- NULL
        else
          rescale.pointsize <- sqrt
      }
      else
        rescale.pointsize <- I
    }
    else if (!is.function(rescale.pointsize) && !is.null(rescale.pointsize))
      stop("Argument 'rescale.pointsize' must be a logical value or ",
           "an R function to rescale point sizes.",
           call. = FALSE)
  }
  ##
  if (length(col.points) == 1)
    col.points <- rep(col.points, n.trts)
  else if (length(col.points) != n.trts)
    stop("Length of argument 'col.points' must be equal to the ",
         "number of treatments.")
  ##
  if (length(pch.points) == 1)
    pch.points <- rep(pch.points, n.trts)
  else if (length(pch.points) != n.trts)
    stop("Length of argument 'pch.points' must be equal to ",
         "number of treatments.")
  ##
  if (length(bg.points) == 1)
    bg.points <- rep(bg.points, n.trts)
  else if (length(bg.points) != n.trts)
    stop("Length of argument 'bg.points' must be equal to ",
         "the number of treatments.")
  ##
  chklogical(figure)
  ##
  if (!missing(allfigures) && length(seq) == 1 && seq == "optimal") {
    warning("Argument 'allfigures' ignored as argument 'seq' is ",
            "equal to \"optimal\".",
            call. = FALSE)
    allfigures <- FALSE
  }
  ##
  if (allfigures | is_3d) {
    if (allfigures & !missing.rotate & rotate != 0)
      warning("Argument 'rotate' set to 0 as argument 'allfigures' is TRUE.",
              call. = FALSE)
    if (is_3d & !missing.rotate & rotate != 0)
      warning("Argument 'rotate' set to 0 as argument 'dim' is equal to ",
              "\"3d\".",
              call. = FALSE)
    rotate <- 0
  }
  ##
  rotn <- rotate * x$n / 360
  
  
  addargs <- names(list(...))
  ##
  if ("highlight.split" %in% addargs)
    warning("Argument 'highlight.split' has been removed from ",
            "R function netgraph.\n  This argument has been replaced by ",
            "argument 'sep.trts' in R function netmeta.",
            call. = FALSE)
  ##
  highlight.split <- x$sep.trts
  ##
  if (is.null(highlight.split))
    highlight.split <- ":"
  ##
  n.high <- 1
  if (!is.null(highlight)) {
    n.high <- length(highlight)
    ##
    if (!missing(col.highlight))
      if (length(col.highlight) != 1 && length(col.highlight) != n.high)
        stop("Argument 'col.highlight' must be a single value or ",
             "of same length as argument 'highlight'.", call. = FALSE)
    ##
    if (!missing(scale.highlight))
      if (length(scale.highlight) != 1 && length(scale.highlight) != n.high)
        stop("Argument 'scale.highlight' must be a single value or ",
             "of same length as argument 'highlight'.", call. = FALSE)
  }
  ##
  if (length(col.highlight) == 1 & n.high > 1)
    col.highlight <- rep(col.highlight, n.high)
  
  
  if (missing(thickness)) {
    thick <- thickness
  }
  else {
    if (!is.matrix(thickness)) {
      if (length(thickness) == 1 & is.character(thickness)) {
        thick <- setchar(thickness,
                         c("number.of.studies",
                           "se.common", "se.random", "w.common", "w.random",
                           "equal",
                           "se.fixed", "w.fixed"))
        thick[thick == "se.fixed"] <- "se.common"
        thick[thick == "w.fixed"] <- "w.common"
      }
      else if (length(thickness) == 1 & is.logical(thickness)) {
        if (thickness)
          thick <- "number.of.studies"
        else
          thick <- "equal"
      }
    }
    else {
      if ((dim(thickness)[1] != dim(A.matrix)[1]) |
          (dim(thickness)[2] != dim(A.matrix)[2]))
        stop("Dimension of argument 'A.matrix' and 'thickness' are different.")
      if (is.null(dimnames(thickness)))
        stop("Matrix 'thickness' must have row and column names identical to ",
             "argument 'A.matrix'.")
      else {
        if (any(rownames(thickness) != rownames(A.matrix)))
          stop("Row names of matrix 'thickness' must be identical to ",
               "argument 'A.matrix'.")
        if (any(colnames(thickness) != colnames(A.matrix)))
          stop("Column names of matrix 'thickness' must be identical to ",
               "argument 'A.matrix'.")
      }
      ##
      W.matrix <- thickness
      thick <- "matrix"
    }
  }
  
  
  if (allfigures & is_3d) {
    warning("Argument 'allfigures' set to FALSE for 3-D network plot.",
            call. = FALSE)
    allfigures <- FALSE
  }


  col.matrix <- matrix("", nrow = n.trts, ncol = n.trts)
  dimnames(col.matrix) <- dimnames(A.matrix)
  col.matrix <- t(col.matrix)
  col.matrix[lower.tri(col.matrix) & t(A.matrix) > 0] <- col
  tcm <- col.matrix
  col.matrix <- t(col.matrix)
  col.matrix[lower.tri(col.matrix)] <- tcm[lower.tri(tcm)]
  ##
  pos.matrix <- matrix(NA, nrow = n.trts, ncol = n.trts)
  dimnames(pos.matrix) <- dimnames(A.matrix)
  pos.matrix <- t(pos.matrix)
  pos.matrix[lower.tri(pos.matrix) & t(A.matrix) > 0] <- pos.number.of.studies
  tam <- pos.matrix
  pos.matrix <- t(pos.matrix)
  pos.matrix[lower.tri(pos.matrix)] <- tam[lower.tri(tam)]
  ##
  A.matrix <- A.matrix[seq1, seq1]
  N.matrix <- N.matrix[seq1, seq1]
  D.matrix <- D.matrix[seq1, seq1]
  ##
  col.matrix <- col.matrix[seq1, seq1]
  pos.matrix <- pos.matrix[seq1, seq1]
  ##
  if (thick == "matrix")
    W.matrix <- W.matrix[seq1, seq1]
  ##
  trts <- x$trts[seq1]
  labels <- labels[seq1]
  ##
  col.points <- col.points[seq1]
  cex.points <- cex.points[seq1]
  pch.points <- pch.points[seq1]
  bg.points  <- bg.points[seq1]
  ##
  if (!is.null(points.max) & !is.null(rescale.pointsize))
    cex.pointsize <- points.max *
      do.call(rescale.pointsize, list(cex.points)) /
      do.call(rescale.pointsize, list(max(cex.points, na.rm = TRUE)))
  else
    cex.pointsize <- cex.points
  ##
  if (!is.null(points.min))
    cex.pointsize[cex.pointsize < points.min & cex.pointsize != 0] <- points.min
  
  
  A.sign <- sign(A.matrix)
  ##
  if ((is_2d & (is.null(xpos) & is.null(ypos))) |
      (is_3d & (is.null(xpos) & is.null(ypos) & is.null(zpos)))) {
    stressdata <- stress(x,
                         ##
                         A.matrix = A.matrix,
                         N.matrix = N.matrix,
                         D.matrix = D.matrix,
                         ##
                         start.layout = start.layout,
                         eig1 = eig1, eig2 = eig2, eig3 = eig3,
                         iterate = iterate,
                         tol = tol, maxit = maxit, allfigures = allfigures,
                         dim = dim,
                         ##
                         ## Additional settings - argument '...' in stress()
                         ##
                         seq = seq,
                         ##
                         labels = labels,
                         cex = cex, adj = adj, srt.labels = srt.labels,
                         offset = offset,
                         scale = scale,
                         ##
                         col = col,
                         plastic = plastic,
                         thickness = thickness,
                         lwd = lwd, lwd.min = lwd.min, lwd.max = lwd.max,
                         rescale.thickness = rescale.thickness,
                         ##
                         highlight = highlight, col.highlight = col.highlight,
                         scale.highlight = scale.highlight,
                         ##
                         multiarm = multiarm,
                         col.multiarm = col.multiarm,
                         alpha.transparency = alpha.transparency,
                         ##
                         points = points,
                         cex.points = cex.points, pch.points = pch.points,
                         col.points = col.points,
                         bg.points = bg.points,
                         points.max = points.max,
                         points.min = points.min,
                         rescale.pointsize = rescale.pointsize,
                         ##
                         number.of.studies = number.of.studies,
                         cex.number.of.studies = cex.number.of.studies,
                         col.number.of.studies = col.number.of.studies,
                         bg.number.of.studies = bg.number.of.studies,
                         pos.number.of.studies = pos.number.of.studies,
                         ##
                         ...)
    ##
    xpos <- stressdata$x
    ypos <- stressdata$y
    ##
    if (rotate != 0) {
      val <- pi * rotate / 180
      xpos.old <- xpos
      xpos <-  xpos * cos(val) + ypos * sin(val)
      ypos <- -xpos.old * sin(val) + ypos * cos(val)
    }
    ##
    if (is_3d)
      zpos <- stressdata$z
  }
  
  
  if (allfigures)
    return(invisible(NULL))


  n <- dim(A.matrix)[1]
  d <- scale * max(abs(c(min(c(xpos, ypos), na.rm = TRUE),
                         max(c(xpos, ypos), na.rm = TRUE))))


  ##
  ##
  ## Generate datasets for plotting
  ##
  ##
  ##
  ## Dataset for nodes
  ##
  dat.nodes <- data.frame(trts, labels, seq, srt = NA,
                          xpos, ypos, zpos = NA,
                          xpos.labels = NA, ypos.labels = NA,
                          offset.x = NA, offset.y = NA,
                          cex = cex.pointsize,
                          col = col.points,
                          pch = pch.points,
                          bg = bg.points,
                          stringsAsFactors = FALSE)
  if (is_2d)
    dat.nodes$zpos <- NULL
  else {
    dat.nodes$zpos <- zpos
    dat.nodes$zpos.labels <- NA
  }
  ##
  if (is.null(adj)) {
    dat.nodes$adj.x <- NA
    dat.nodes$adj.y <- NA
    if (!is_2d)
      dat.nodes$adj.z <- NA
    ##
    dat.nodes$adj.x[dat.nodes$xpos >= 0] <- 0
    dat.nodes$adj.x[dat.nodes$xpos <  0] <- 1
    ##
    dat.nodes$adj.y[dat.nodes$ypos >  0] <- 0
    dat.nodes$adj.y[dat.nodes$ypos <= 0] <- 1
    ##
    if (!is_2d) {
      dat.nodes$adj.z[dat.nodes$zpos >  0] <- 0
      dat.nodes$adj.z[dat.nodes$zpos <= 0] <- 1
    }
  }
  else {
    dat.nodes$adj.x <- NA
    dat.nodes$adj.y <- NA
    ##
    if (length(adj) == 1) {
      dat.nodes$adj.x <- adj
      dat.nodes$adj.y <- adj
      if (!is_2d)
        dat.nodes$adj.z <- adj
    }
    else if (length(adj) == 2) {
      dat.nodes$adj.x <- adj[1]
      dat.nodes$adj.y <- adj[2]
      if (!is_2d)
        dat.nodes$adj.z <- 0.5
    }
    else if (length(adj) == 3 & !is_2d) {
      dat.nodes$adj.x <- adj[1]
      dat.nodes$adj.y <- adj[2]
      dat.nodes$adj.z <- adj[3]
    }
    else if (is.vector(adj)) {
      if (length(adj) != length(labels))
        stop("Length of vector 'adj' must be equal to number of treatments.")
      ##
      names(adj) <- x$trts
      dat.nodes$adj.x <- adj[seq1]
      dat.nodes$adj.y <- adj[seq1]
      ##
      if (!is_2d)
        dat.nodes$adj.z <- adj[seq1]
    }
    else if (is.matrix(adj)) {
      if (nrow(adj) != length(labels))
        stop("Number of rows of matrix 'adj' must be equal to ",
             "number of treatments.")
      rownames(adj) <- x$trts
      dat.nodes$adj.x <- adj[seq1, 1]
      dat.nodes$adj.y <- adj[seq1, 2]
      ##
      if (!is_2d & ncol(adj) >= 3)
        dat.nodes$adj.z <- adj[seq1, 3]
    }
  }
  ##
  if (is_2d) {
    offset <- offset * 2 * d
    ##
    if (length(offset) == 1) {
      offset.x <- offset
      offset.y <- offset
    }
    else if (length(offset) == 2) {
      offset.x <- offset[1]
      offset.y <- offset[2]
    }
    else if (is.vector(offset)) {
      if (length(offset) != length(labels))
        stop("Length of vector 'offset' must be equal to ",
             "number of treatments.")
      ##
      names(offset) <- x$trts
      offset.x <- offset[seq1]
      offset.y <- offset[seq1]
    }
    else if (is.matrix(offset)) {
      if (nrow(offset) != length(labels))
        stop("Number of rows of matrix 'offset' must be equal to ",
             "number of treatments.")
      ##
      rownames(offset) <- x$trts
      offset.x <- offset[seq1, 1]
      offset.y <- offset[seq1, 2]
    }
    ##
    dat.nodes$xpos.labels <- dat.nodes$xpos - offset.x +
      2 * (dat.nodes$adj.x == 0) * offset.x
    dat.nodes$ypos.labels <- dat.nodes$ypos - offset.y +
      2 * (dat.nodes$adj.y == 0) * offset.y
    ##
    dat.nodes$offset.x <- offset.x
    dat.nodes$offset.y <- offset.y
  }
  else {
    dat.nodes$xpos.labels <- dat.nodes$xpos
    dat.nodes$ypos.labels <- dat.nodes$ypos
    dat.nodes$zpos.labels <- dat.nodes$zpos
  }
  ##  
  if (length(srt.labels) == 1 && is.character(srt.labels)) {
    srt.labels <-
      setchar(srt.labels, "orthogonal",
              "should be equal to 'orthogonal' or numeric (vector)")
    if (!missing(iterate) && iterate == TRUE) {
      warning("Orthogonal labels not supported if argument 'iterate = TRUE'.",
              call. = FALSE)
      srt.labels <- 0
    }
    ##
    srtfunc <- function(ntrt) {
      s <- 180 * (2 * (1:ntrt) / ntrt - 1)
      for (i in 1:ntrt) {
        if (i < ntrt / 4)
          s[i] <- 360 * i / ntrt
        if (i > 3 * ntrt / 4)
          s[i] <- 360 * (i / ntrt - 1)
      }
      s
    }
    srt.labels <- srtfunc(x$n)
  }
  ##
  chknumeric(srt.labels, min = -180, max = 180)
  ##
  if (length(srt.labels) == 1)
    dat.nodes$srt <- srt.labels
  else {
    if (length(srt.labels) != length(labels))
      stop("Length of vector 'srt.labels' must be equal to",
           "number of treatments.",
           eval. = FALSE)
    if (is.null(names(srt.labels))) {
      if (is_wholenumber(rotn) & abs(rotn) < x$n) {
        srt1 <- seq_len(x$n)
        srt1 <- srt1 - rotn
        srt1[srt1 > x$n] <- srt1[srt1 > x$n] - x$n
        srt1[srt1 <= 0] <- x$n + srt1[srt1 <= 0]
        dat.nodes$srt <- srt.labels[srt1]
      }
      else
        dat.nodes$srt <- srt.labels
    }
    else {
      ## Check names of named vector 'srt.labels'
      names.srt.labels <- names(srt.labels)
      names.srt.labels <- setseq(names.srt.labels, dat.nodes$trts,
                                 paste0("Names of vector provided in ",
                                        "argument 'srt.labels'"))
      dat.nodes$srt <- srt.labels[dat.nodes$trts]
    }
  }
  ##
  ## Dataset for edges
  ##
  dat.edges <- data.frame(treat1 = rep("", n.edges),
                          treat2 = "",
                          n.stud = NA,
                          xpos = NA, ypos = NA,
                          adj = NA, pos.number.of.studies,
                          col = "",
                          stringsAsFactors = FALSE)
  ##
  comp.i <- 1
  ##
  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      if (A.sign[i, j] > 0) {
        ##
        dat.edges$treat1[comp.i] <- rownames(A.matrix)[i]
        dat.edges$treat2[comp.i] <- colnames(A.matrix)[j]
        dat.edges$n.stud[comp.i] <- A.matrix[i, j]
        dat.edges$adj[comp.i] <- lambda <- pos.matrix[i, j]
        dat.edges$xpos[comp.i] <- lambda * xpos[i] + (1 - lambda) * xpos[j]
        dat.edges$ypos[comp.i] <- lambda * ypos[i] + (1 - lambda) * ypos[j]
        dat.edges$col[comp.i] <- col.matrix[i, j]
        ##
        comp.i <- comp.i + 1
      }
    }
  }
  
  
  ##
  ## Define coloured regions for multi-arm studies
  ##
  if (multiarm) {
    mc <- multicols(x$studies, x$narms, missing(col.multiarm),
                    col.multiarm, alpha.transparency)
    col.polygon <- mc$col
    multiarm.studies <- mc$studlab
    n.multi <- length(multiarm.studies)
  }
  ##
  ## Define line width
  ##
  if (thick == "number.of.studies") {
    W.matrix <- lwd.max *
      do.call(rescale.thickness, list(A.matrix)) /
      do.call(rescale.thickness, list(max(A.matrix, na.rm = TRUE)))
    W.matrix[W.matrix < lwd.min & W.matrix != 0] <- lwd.min
  }
  else if (thick == "equal") {
    W.matrix <- lwd.max * A.sign
  }
  else if (thick == "se.common") {
    IV.matrix <- x$seTE.direct.common[seq1, seq1]
    IV.matrix[is.infinite(IV.matrix)] <- NA
    W.matrix <- lwd.max *
      do.call(rescale.thickness, list(min(IV.matrix, na.rm = TRUE))) /
      do.call(rescale.thickness, list(IV.matrix))
    W.matrix[W.matrix < lwd.min & W.matrix != 0] <- lwd.min
  }
  else if (thick == "se.random") {
    IV.matrix <- x$seTE.direct.random[seq1, seq1]
    IV.matrix[is.infinite(IV.matrix)] <- NA
    W.matrix <- lwd.max *
      do.call(rescale.thickness, list(min(IV.matrix, na.rm = TRUE))) /
      do.call(rescale.thickness, list(IV.matrix))
    W.matrix[W.matrix < lwd.min & W.matrix != 0] <- lwd.min
  }
  else if (thick == "w.common") {
    IV.matrix <- 1 / x$seTE.direct.common[seq1, seq1]^2
    IV.matrix[is.infinite(IV.matrix)] <- NA
    W.matrix <- lwd.max *
      do.call(rescale.thickness, list(IV.matrix)) /
      do.call(rescale.thickness, list(max(IV.matrix, na.rm = TRUE)))
    W.matrix[W.matrix < lwd.min & W.matrix != 0] <- lwd.min
  }
  else if (thick == "w.random") {
    IV.matrix <- 1 / x$seTE.direct.random[seq1, seq1]^2
    IV.matrix[is.infinite(IV.matrix)] <- NA
    W.matrix <- lwd.max *
      do.call(rescale.thickness, list(IV.matrix)) /
      do.call(rescale.thickness, list(max(IV.matrix, na.rm = TRUE)))
    W.matrix[W.matrix < lwd.min & W.matrix != 0] <- lwd.min
  }
  else if (thick == "matrix") {
    W.matrix[is.infinite(W.matrix)] <- NA
    if (min(W.matrix[W.matrix != 0], na.rm = TRUE) ==
        max(W.matrix[W.matrix != 0], na.rm = TRUE))
      W.matrix <- lwd * W.matrix
    else
      W.matrix <- lwd.max *
        do.call(rescale.thickness, list(W.matrix)) /
        do.call(rescale.thickness, list(max(W.matrix, na.rm = TRUE)))
    W.matrix[W.matrix < lwd.min & W.matrix != 0] <- lwd.min
  }
  ##
  if (missing.lwd.max & !is.matrix(thickness) && thickness != "equal" &
      length(unique(W.matrix[W.matrix != 0])) == 1)
    W.matrix <- W.matrix / 2
  
  
  ##
  ##
  ## Plot graph
  ##
  ##
  if (figure) {
    range <- c(-d, d)
    ##
    if (is_2d) {
      oldpar <- par(xpd = TRUE, pty = "s")
      on.exit(par(oldpar))
      ##
      plot(xpos, ypos,
           xlim = range, ylim = range,
           type = "n", axes = FALSE, bty = "n",
           xlab = "", ylab = "",
           ...)
      ##
      ## Add coloured regions for multi-arm studies
      ##
      if (multiarm) {
        ##
        if (n.multi > 0) {
          multiarm.treat <- vector("list", n.multi)
          if (length(col.polygon) == 1)
            col.polygon <- rep(col.polygon, n.multi)
          for (i in 1:n.multi) {
            treat1 <- x$treat1[x$studlab %in% multiarm.studies[i]]
            treat2 <- x$treat2[x$studlab %in% multiarm.studies[i]]
            multiarm.treat[[i]] <- sort(unique(c(treat2, treat1)))
            ##
            dat.multi <- dat.nodes[dat.nodes$trts %in% multiarm.treat[[i]], ]
            if (nrow(dat.multi) == 0)
              dat.multi <- dat.nodes[dat.nodes$trts %in% multiarm.treat[[i]], ]
            ##
            ## Clockwise ordering of polygon coordinates
            ##
            polysort <- function(x, y) {
              xnorm <- (x - mean(x)) / sd(x) # Normalise coordinate x
              ynorm <- (y - mean(y)) / sd(y) # Normalise coordinate y
              r <- sqrt(xnorm^2 + ynorm^2)   # Calculate polar coordinates
              cosphi <- xnorm / r
              sinphi <- ynorm / r
              s <- as.numeric(sinphi > 0) # Define angles to lie in [0, 2 * pi]
              phi <- acos(cosphi)
              alpha <- s * phi + (1 - s) * (2 * pi - phi)
              ##
              res <- order(alpha)
              res
            }
            ##
            dat.multi <- dat.multi[polysort(dat.multi$xpos, dat.multi$ypos), ]
            ##
            polygon(dat.multi$xpos, dat.multi$ypos,
                    col = col.polygon[i], border = NA)
          }
        }
      }
      ##
      ## Define lines
      ##
      if (plastic) {
        n.plastic <- 30
        lwd.multiply <- rep(NA, n.plastic)
        cols <- rep("", n.plastic)
        cols.highlight <- matrix("", nrow = n.high, ncol = n.plastic)
        scales.highlight <- matrix(scale.highlight,
                                   nrow = n.high, ncol = n.plastic)
        ##
        j <- 0
        for (i in n.plastic:1) {
          j <- j + 1
          lwd.multiply[j] <- sin(pi * i / 2 / n.plastic)
          cols[j] <- paste0("gray", round(100 * (1 - i / n.plastic)))
          cols.highlight[, j] <-
            paste0("gray", round(100 * (1 - i / n.plastic)))
        }
        ##
        for (h in seq_len(n.high)) {
          col.high.h <- col.highlight[h]
          if (col.high.h != "transparent") {
            if (nchar(col.high.h) > 1 &
                substring(col.high.h, nchar(col.high.h)) %in% 1:4)
              col.high.h <- substring(col.high.h, 1, nchar(col.high.h) - 1)
            ##
            cols.highlight[h, 1:12] <- rep(paste0(col.high.h, 4:1), rep(3, 4))
            cols.highlight[h, 13:15] <- rep(col.high.h, 3)
          }
          else {
            cols.highlight[h, ] <- col.high.h
          }
        }
      }
      else {
        lwd.multiply <- 1
        cols <- col
        cols.highlight <- matrix(col.highlight, nrow = n.high, ncol = 1)
        scales.highlight <- matrix(scale.highlight, nrow = n.high, ncol = 1)
      }
      ##
      ## Add highlighted comparisons
      ##
      A.sign.add.lines <- A.sign
      ##
      if (!is.null(highlight)) {
        high.i <- 0
        for (high in highlight) {
          high.i <- high.i + 1
          highs <- unlist(compsplit(high, split = highlight.split))
          if (length(highs) != 2)
            stop("Wrong format for argument 'highlight' ",
                 "(see helpfile of plotgraph command).")
          #
          highs <- setseq(highs, dat.nodes$trts, equal.length = FALSE)
          #
          if (sum(dat.nodes$trts %in% highs) != 2)
            stop("Argument 'highlight' must contain two of ",
                 "the following values (separated by \":\"):\n  ",
                 paste(paste0("'", dat.nodes$trts, "'"), collapse = " - "))
          ##
          dat.high <- dat.nodes[dat.nodes$trts %in% highs, ]
          ##
          if (is_2d)
            for (n.plines in 1:length(lwd.multiply)) {
              lines(dat.high$xpos, dat.high$ypos,
                    lwd = W.matrix[trts == highs[1], trts == highs[2]] *
                      lwd.multiply[n.plines] *
                      scales.highlight[high.i, n.plines],
                    col = cols.highlight[high.i, n.plines])
            }
          ##
          A.sign.add.lines[trts == highs[1], trts == highs[2]] <- 0
          A.sign.add.lines[trts == highs[2], trts == highs[1]] <- 0
        }
      }
      ##
      ## Add lines
      ##
      comp.i <- 1
      ##
      for (n.plines in 1:length(lwd.multiply)) {
        for (i in 1:(n - 1)) {
          for (j in (i + 1):n) {
            ##
            if (plastic)
              col.ij <- cols[n.plines]
            else
              col.ij <- col.matrix[i, j]
            ##
            if (A.sign.add.lines[i, j] > 0) {
              lines(c(xpos[i], xpos[j]), c(ypos[i], ypos[j]),
                    lwd = W.matrix[i, j] * lwd.multiply[n.plines],
                    col = col.ij)
              ##
              comp.i <- comp.i + 1
            }
          }
        }
      }
      ##
      ## Add points for labels
      ##
      if (points)
        points(xpos, ypos,
               pch = pch.points, cex = cex.pointsize, col = col.points,
               bg = bg.points)
      ##
      ## Print treatment labels
      ##
      if (!is.null(labels))
        for (i in 1:n)
          text(dat.nodes$xpos.labels[i], dat.nodes$ypos.labels[i],
               labels = dat.nodes$labels[i],
               cex = cex,
               adj = c(dat.nodes$adj.x[i], dat.nodes$adj.y[i]),
               srt = dat.nodes$srt[i])
      ##
      ## Print number of treatments
      ##
      if (number.of.studies) {
        comp.i <- 1
        ##
        for (i in 1:(n - 1)) {
          for (j in (i + 1):n) {
            if (A.sign[i, j] > 0) {
              ##
              shadowtext(dat.edges$xpos[comp.i],
                         dat.edges$ypos[comp.i],
                         labels = dat.edges$n.stud[comp.i],
                         cex = cex.number.of.studies,
                         col = col.number.of.studies,
                         bg = bg.number.of.studies)
              ##
              comp.i <- comp.i + 1
            }
          }
        }
      }
    }
    else {
      rgl::plot3d(xpos, ypos, zpos,
                  size = 10, col = col.points, cex = cex.pointsize,
                  bg = bg.points,
                  axes = FALSE, box = FALSE,
                  xlab = "", ylab = "", zlab = "")
      ##
      ## Add points for labels
      ##
      if (points)
        rgl::points3d(xpos, ypos, zpos,
                      pch = pch.points, cex = cex.pointsize, col = col.points,
                      bg = bg.points)
      ##
      ## Print treatment labels
      ##
      if (!is.null(labels))
        for (i in 1:n)
          rgl::text3d(dat.nodes$xpos.labels[i], dat.nodes$ypos.labels[i],
                      dat.nodes$zpos.labels[i],
                      texts = dat.nodes$labels[i],
                      cex = cex,
                      adj = c(dat.nodes$adj.x[i], dat.nodes$adj.y[i]))
      ##
      ## Add highlighted comparisons
      ##
      if (!is.null(highlight)) {
        for (high in highlight) {
          highs <- unlist(compsplit(high, split = highlight.split))
          if (length(highs) != 2)
            stop("Wrong format for argument 'highlight' ",
                 "(see helpfile of plotgraph command).")
          ##
          if (sum(dat.nodes$trts %in% highs) != 2)
            stop("Argument 'highlight' must contain two of the ",
                 "following values (separated by \":\"):\n  ",
                 paste(paste0("'", dat.nodes$trts, "'"),
                       collapse = " - "))
          ##
          dat.high <- dat.nodes[dat.nodes$trts %in% highs, ]
          ##
          rgl::lines3d(dat.high$xpos * (1 + 1e-4), dat.high$ypos * (1 + 1e-4),
                       dat.high$zpos * (1 + 1e-4),
                       lwd = W.matrix[trts == highs[1], trts == highs[2]],
                       col = col.highlight)
        }
      }
      ##
      ## Add coloured regions for multi-arm studies
      ##
      if (multiarm) {
        ##
        morethan3 <- FALSE
        ##
        if (n.multi > 0) {
          multiarm.treat <- vector("list", n.multi)
          if (length(col.polygon) == 1)
            col.polygon <- rep(col.polygon, n.multi)
          for (i in 1:n.multi) {
            treat1 <- x$treat1[x$studlab %in% multiarm.studies[i]]
            treat2 <- x$treat2[x$studlab %in% multiarm.studies[i]]
            multiarm.treat[[i]] <- sort(unique(c(treat2, treat1)))
            ##
            dat.multi <- dat.nodes[dat.nodes$trts %in% multiarm.treat[[i]], ]
            if (nrow(dat.multi) == 0)
              dat.multi <- dat.nodes[dat.nodes$trts %in% multiarm.treat[[i]], ]
            if (nrow(dat.multi) == 3)
              rgl::triangles3d(dat.multi$xpos, dat.multi$ypos, dat.multi$zpos,
                               col = col.polygon[i])
            else
              morethan3 <- TRUE
          }
        }
        if (morethan3)
          warning("Multi-arm studies with more than three treatments ",
                  "not shown in 3-D plot.",
                  call. = FALSE)
      }
      ##
      ## Draw lines
      ##
      for (i in 1:(n - 1)) {
        for (j in (i + 1):n) {
          if (A.sign[i, j] > 0) {
            rgl::lines3d(c(xpos[i], xpos[j]),
                         c(ypos[i], ypos[j]),
                         c(zpos[i], zpos[j]),
                         lwd = W.matrix[i, j],
                         col = col)
          }
        }
      }
    }
  }
  
  
  dat.nodes$xpos[is_zero(dat.nodes$xpos)] <- 0
  dat.nodes$ypos[is_zero(dat.nodes$ypos)] <- 0
  ##
  if (!is_2d) {
    dat.nodes$zpos[is_zero(dat.nodes$zpos)] <- 0
    ##
    dat.nodes$xpos.labels <- NULL
    dat.nodes$ypos.labels <- NULL
  }
  else {
    dat.nodes$xpos.labels[is_zero(dat.nodes$xpos.labels)] <- 0
    dat.nodes$ypos.labels[is_zero(dat.nodes$ypos.labels)] <- 0
  }
  ##
  dat.nodes$zpos.labels <- NULL
  ##
  rownames(dat.nodes) <- dat.nodes$trts
  rownames(dat.edges) <-
    paste(dat.edges$treat1, dat.edges$treat2, sep = highlight.split)
  ##  
  dat.edges$xpos[is_zero(dat.edges$xpos)] <- 0
  dat.edges$ypos[is_zero(dat.edges$ypos)] <- 0

  res <- list(nodes = dat.nodes, edges = dat.edges)
  ##
  if (multiarm)
    res$multiarm <- mc
  
  invisible(res)
}

decomp.tau <- function(x, tau.preset = 0, warn = TRUE) {
  
  
  nmak <- nma_krahn(x, tau.preset = tau.preset)
  ##
  if (nmak$n == 2) {
    res <- list(d = 1, n = 2)
    return(res)
  }
  ##
  design <- nmak$design
  studies <- nmak$studies
  V <- nmak$V
  V.studies <- nmak$V.studies
  X.obs <- nmak$X.obs
  multicomp <- nmak$multicomp
  
  
  multiarm <- any(studies$narms > 2)
  
  
  designs <- unique(as.character(design$design))
  TE.net.minus <- matrix(NA, nrow = nrow(X.obs), ncol = length(designs))
  colnames(TE.net.minus) <- designs
  ##
  df.net.minus <- numeric(length(designs))
  names(df.net.minus) <- designs
  ##
  isMat <- function(x)
    "matrix" %in% class(try(solve(x), silent = TRUE))
  ##
  for (i in designs) {
    narms <- design$narms[design$design == i][1]
    Xb <- cbind(X.obs,matrix(0, nrow(X.obs), narms - 1))
    Xb[(rownames(X.obs) == i), ] <- 0
    Xb[(rownames(X.obs) == i), ncol(X.obs) + (1:(narms - 1))] <-
      diag(narms - 1)
    ##
    XVX <- t(Xb) %*% solve(V) %*% Xb
    if (isMat(XVX))
      TE.net.minus[, i] <-
        Xb %*% solve(XVX) %*% t(Xb) %*% solve(V) %*% design$TE.dir
    else
      TE.net.minus[, i] <- rep(NA, nrow(X.obs))
    ##
    df.net.minus[i] <- qr(Xb)$rank
  }
  ##
  rownames(TE.net.minus) <- design$design
  
  
  freq.d <- rep_len(NA, nrow(design))
  Q.het.design <- rep_len(NA, nrow(design))
  ##
  for (i in unique(sort(as.character(studies$design)))) {
    Vs <- V.studies[rownames(V.studies) == i, colnames(V.studies) == i]
    ##
    TE.i <- studies$TE[as.character(studies$design) == i]
    TE.dir.i <- studies$TE.dir[as.character(studies$design) == i]
    ##
    Q.het.design[which(design$design == i)[1]] <- t(TE.i - TE.dir.i) %*%
      solve(Vs) %*% (TE.i - TE.dir.i)
    freq.d[which(design$design == i)[1]] <- length(TE.i)
  }
  ##
  NAfd <- is.na(freq.d)
  design$narms[NAfd] <- NA
  design$freq[NAfd] <- NA
  
  
  df.het.design <- freq.d - (design$narms - 1)
  pval.het.design <- pchisq(Q.het.design, df = df.het.design,
                            lower.tail = FALSE)
  pval.het.design[df.het.design == 0] <- NA
  ##
  Q.het <- t(studies$TE - studies$TE.dir) %*% solve(V.studies) %*%
    (studies$TE - studies$TE.dir)
  ## Formula (7) in Krahn et al. (2013)
  Q.net <- t(studies$TE - studies$TE.net) %*% solve(V.studies) %*%
    (studies$TE - studies$TE.net)
  ## Formula (8) in Krahn et al. (2013)
  Q.inc <- Q.net - Q.het
  ##
  df.het <- sum(df.het.design, na.rm = TRUE)
  df.net <- sum(freq.d, na.rm = TRUE) - ncol(X.obs)
  df.inc <- df.net - df.het
  ##
  Qid <- t(design$TE.dir - design$TE.net) %*% solve(V) *
    (design$TE.dir - design$TE.net)
  nam <- colnames(Qid)
  Q.inc.design <- as.vector(Qid)
  names(Q.inc.design) <- nam
  
  
  residuals <- apply(TE.net.minus, 2, function(x) design$TE.dir - x)
  residuals <- residuals[, rownames(residuals)]
  diag(residuals) <- rep_len(0, nrow(residuals))
  ##
  if (multiarm)
    for (i in 1:length(multicomp))
      residuals[rownames(residuals) == multicomp[i],
                colnames(residuals) == multicomp[i]] <- 0
  
  
  Q.inc.detach <- apply(residuals, 2, function(x) t(x) %*% solve(V) %*% x)
  Q.inc.detach[NAfd] <- NA
  ##
  df.inc.detach <- rep_len(NA, length(NAfd))
  df.inc.detach[!NAfd]  <- nrow(X.obs) - df.net.minus 
  ##
  pval.inc.detach <- pchisq(Q.inc.detach, df = df.inc.detach,
                            lower.tail = FALSE)
  pval.inc.detach[df.inc.detach == 0] <- NA
  
  
  Q.decomp <- data.frame(Q = c(Q.net, Q.het, Q.inc),
                         df = c(df.net, df.het, df.inc),
                         pval = pchisq(c(Q.net, Q.het, Q.inc),
                                       df = c(df.net, df.het, df.inc),
                                       lower.tail = FALSE))
  ##
  Q.decomp$pval[c(df.net, df.het, df.inc) == 0] <- NA
  ##
  rownames(Q.decomp) <- c("Total",
                          "Within designs",
                          "Between designs")
  
  
  Q.het.design <- data.frame(design = design$design,
                             Q = round(Q.het.design, 15),
                             df = df.het.design,
                             pval = pval.het.design)
  ##
  Q.inc.detach <- data.frame(design = design$design,
                             Q = Q.inc.detach,
                             df = df.inc.detach,
                             pval = pval.inc.detach)
  ##
  if (multiarm) {
    Q.het.design <- Q.het.design[!(duplicated(Q.het.design$design)), ]
    Q.inc.detach <- Q.inc.detach[!(duplicated(Q.inc.detach$design)), ]
  }
  ##
  if (anyNA(Q.inc.detach$Q))
    Q.inc.detach <- Q.inc.detach[!(is.na(Q.inc.detach$Q)), ]
  
  res <- list(
    Q.decomp = Q.decomp,
    Q.het.design = Q.het.design,
    Q.inc.detach = Q.inc.detach,
    Q.inc.design = Q.inc.design,
    residuals.inc.detach = residuals,
    d = nmak$d
  )
  
  res
}

#' Network graph for objects of class discomb
#' 
#' @description
#' This function generates a graph of the evidence network.
#' 
#' @param x An object of class \code{discomb}.
#' @param labels An optional vector with treatment labels.
#' @param adj One, two, or three values in [0, 1] (or a vector /
#'   matrix with length / number of rows equal to the number of
#'   treatments) specifying the x (and optionally y and z) adjustment
#'   for treatment labels.
#' @param offset Distance between nodes (i.e. treatments) in graph and
#'   treatment labels for 2-D plots (value of 0.0175 corresponds to a
#'   difference of 1.75\% of the range on x- and y-axis).
#' @param rotate A single numeric with value between -180 and 180
#'   specifying the degrees to rotate nodes in a circular network clockwise or
#'   anticlockwise.
#' @param points A logical indicating whether points should be printed
#'   at nodes (i.e. treatments) of the network graph.
#' @param cex.points Corresponding point size. Can be a vector with length
#'   equal to the number of treatments.
#' @param \dots Additional arguments passed on to
#'   \code{\link{netgraph.netmeta}}.
#' 
#' @details
#' The arguments \code{seq} and \code{iterate} are used internally and
#' cannot be specified by the user.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{discomb}}, \code{\link{netgraph.netmeta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Artificial dataset
#' #
#' t1 <- c("A + B", "A + C", "A"    , "A"    , "D", "D", "E")
#' t2 <- c("C"    , "B"    , "B + C", "A + D", "E", "F", "F")
#' #
#' mean <- c(4.1, 2.05, 0, 0, 0.1, 0.1, 0.05)
#' se.mean <- rep(0.1, 7)
#' #
#' study <- paste("study", c(1:4, 5, 5, 5))
#' #
#' dat <- data.frame(mean, se.mean, t1, t2, study,
#'                   stringsAsFactors = FALSE)
#' #
#' trts <- c("A", "A + B", "A + C", "A + D",
#'   "B", "B + C", "C", "D", "E", "F")
#' #
#' comps <- LETTERS[1:6]
#' 
#' # Use netconnection() to display network information
#' #
#' netconnection(t1, t2, study)
#' 
#' dc1 <- discomb(mean, se.mean, t1, t2, study, seq = trts)
#'
#' netgraph(dc1)
#' 
#' @method netgraph discomb
#' @export

netgraph.discomb <- function(x,
                             labels = x$trts,
                             adj = NULL,
                             offset =
                               if (!is.null(adj) && all(unique(adj) == 0.5))
                                 0
                             else
                               0.0175,
                             rotate = 0,
                             points = !missing(cex.points),
                             cex.points = 1, ...) {
  
  
  chkclass(x, "discomb")
  
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (!missing(labels))
    labels <- catch("labels", mc, x, sfsp)
  if (!missing(adj))
    adj <- catch("adj", mc, x, sfsp)
  if (!missing(offset))
    offset <- catch("offset", mc, x, sfsp)
  if (!missing(rotate))
    rotate <- catch("rotate", mc, x, sfsp)
  if (!missing(cex.points))
    cex.points <- catch("cex.points", mc, x, sfsp)
  
  y <- netconnection(x$treat1, x$treat2, x$studlab)
  #
  y$trts <- x$trts
  y$D.matrix <- y$D.matrix[x$trts, x$trts]
  y$A.matrix <- y$A.matrix[x$trts, x$trts]
  y$L.matrix <- y$L.matrix[x$trts, x$trts]
  #
  class(y) <- "netmeta"
  
  res <- netgraph(y, seq = y$seq, iterate = FALSE,
                  #
                  labels = labels,
                  adj = adj,
                  offset = offset,
                  rotate = rotate,
                  points = points,
                  cex.points = cex.points,
                  #
                  ...)
  
  invisible(res)
}

#' Measures for characterizing a network meta-analysis
#' 
#' @description
#' This function provides measures for quantifying the direct evidence
#' proportion, the mean path length and the minimal parallelism (the
#' latter on aggregated and study level) of mixed treatment
#' comparisons (network estimates) as well as the evidence flow per
#' design, see König et al. (2013).  These measures support the
#' critical evaluation of the network meta-analysis results by
#' rendering transparent the process of data pooling.
#' 
#' @param x An object of class \code{netmeta}.
#' @param random A logical indicating whether random effects model
#'   should be used to calculate network measures.
#' @param tau.preset An optional value for the square-root of the
#'   between-study variance \eqn{\tau^2}.
#' @param warn A logical indicating whether warnings should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' The direct evidence proportion gives the absolute contribution of
#' direct effect estimates combined for two-arm and multi-arm studies
#' to one network estimate.
#' 
#' Concerning indirectness, comparisons with a mean path length beyond
#' two should be interpreted with particular caution, as more than two
#' direct comparisons have to be combined serially on average.
#' 
#' Large indices of parallelism, either on study-level or on
#' aggregated level, can be considered as supporting the validity of a
#' network meta-analysis if there is only a small amount of
#' heterogeneity.
#' 
#' The network estimates for two treatments are linear combinations of
#' direct effect estimates comparing these or other treatments. The
#' linear coefficients can be seen as the generalization of weights
#' known from classical meta-analysis. These coefficients are given in
#' the projection matrix \eqn{H} of the underlying model. For
#' multi-arm studies, the coefficients depend on the choice of the
#' study-specific baseline treatment, but the absolute flow of
#' evidence can be made explicit for each design as shown in König et
#' al. (2013) and is given in \code{H.tilde}.
#' 
#' All measures are calculated based on the common effects
#' meta-analysis by default. In the case that in function
#' \code{netmeta} the argument \code{random = TRUE}, all measures
#' are calculated for a random effects model. The value of the
#' square-root of the between-study variance \eqn{\tau^2} can also be
#' prespecified by argument \code{tau.preset} in function
#' \code{netmeta}.
#'
#' @return
#' A list containing the following components:
#' \item{random, tau.preset}{As defined above.}
#' \item{proportion}{A named vector of the direct evidence proportion
#'   of each network estimate.}
#' \item{meanpath}{A named vector of the mean path length of each
#'   network estimate.}
#' \item{minpar}{A named vector of the minimal parallelism on
#'   aggregated level of each network estimate.}
#' \item{minpar.study}{A named vector of the minimal parallelism on
#'   study level of each network estimate.}
#' \item{H.tilde }{Design-based hat matrix with information on
#'   absolute evidence flow per design. The number of rows is equal to
#'   the number of possible pairwise treatment comparisons and the
#'   number of columns is equal to the number of designs.}
#' 
#' @author Ulrike Krahn \email{ulrike.krahn@@bayer.com}, Jochem König
#'   \email{koenigjo@@uni-mainz.de}
#' 
#' @seealso \link{netmeta}
#' 
#' @references
#' König J, Krahn U, Binder H (2013):
#' Visualizing the flow of evidence in network meta-analysis and
#' characterizing mixed treatment comparisons.
#' \emph{Statistics in Medicine},
#' \bold{32}, 5414--29
#' 
#' @examples
#' data(smokingcessation)
#' 
#' # Transform data from arm-based format to contrast-based format
#' #
#' pw <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' 
#' # Conduct network meta-analysis
#' #
#' nma <- netmeta(pw)
#' 
#' # Calculate measures based on a common effects model
#' #        
#' nm <- netmeasures(nma)
#' 
#' # Plot of minimal parallelism versus mean path length
#' #
#' plot(nm$meanpath, nm$minpar, type = "n",
#'   xlab = "Mean path length", ylab = "Minimal parallelism")
#' text(nm$meanpath, nm$minpar, names(nm$meanpath), cex = 0.8)
#' 
#' @export netmeasures

netmeasures <- function(x,
                        random = x$random | !missing(tau.preset),
                        tau.preset = x$tau.preset,
                        warn = gs("warn"),
                        warn.deprecated = gs("warn.deprecated"),
                        ...) {
  
  ##
  ##
  ## (1) Check for netmeta object and upgrade object
  ##
  ##
  chkclass(x, "netmeta")
  chksuitable(x, "Network measures", classes = gs(".other_nma"))
  x <- updateversion(x)
  ##
  is.bin <- inherits(x, "netmetabin")


  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  args  <- list(...)
  chklogical(warn.deprecated)
  ##
  random <-
    deprecated(random, missing(random), args, "comb.random", warn.deprecated)
  chklogical(random)
  ##
  if (is.bin && random && x$method != "LRP") {
    txt <-
      paste0("Argument 'random' set to FALSE for ",
             if (x$method == "MH")
               "Mantel-Haenszel method"
             else
               "non-central hypergeometric distribution",
             ".")
    if (warn)
      warning(txt, call. = FALSE)
    random <- FALSE
  }
  ##
  if (!missing(random) & !random) {
    if (!is.null(tau.preset)) {
      if (!missing(tau.preset) & tau.preset > 0)
        stop("Argument 'tau.preset' must be equal 0 if random=FALSE.")
      tau.preset <- NULL
    }
  }
  ##
  if (!is.null(tau.preset)) {
    chknumeric(tau.preset, min = 0, length = 1)
    if (!random & warn) {
      warning("Measures calculated for random effects model ",
              "(argument random=TRUE) as argument 'tau.preset' is provided.")
    }
  }
  
  
  ##
  ##
  ## (3) Calculate measures
  ##
  ##
  if (is.bin) {
    ##
    ## Direct evidence proportion
    ## (Rücker et al. 2020, BMC Med Res Meth, equation 7)
    ##
    if (!random) {
      se.direct  <- uppertri(x$seTE.direct.common)
      se.network <- uppertri(x$seTE.common)
      #
      nam <- rep_len("", length(se.direct))
      idx <- 0
      for (i in seq_len(ncol(x$seTE.common)))
        for (j in seq_len(nrow(x$seTE.common)))
          if (i < j) {
            idx <- idx + 1
            nam[idx] <- paste(rownames(x$seTE.direct.common)[i],
                              colnames(x$seTE.direct.common)[j],
                              sep = x$sep.trts)
          }
    }
    else {
      se.direct  <- uppertri(x$seTE.direct.random)
      se.network <- uppertri(x$seTE.random)
      #
      nam <- rep_len("", length(se.direct))
      idx <- 0
      for (i in seq_len(ncol(x$seTE.random)))
        for (j in seq_len(nrow(x$seTE.random)))
          if (i < j) {
            idx <- idx + 1
            nam[idx] <- paste(rownames(x$seTE.direct.random)[i],
                              colnames(x$seTE.direct.random)[j],
                              sep = x$sep.trts)
          }
    }
    #
    proportion <- se.network^2 / se.direct^2
    proportion[is.na(proportion)] <- 0
    names(proportion) <- nam
    ##
    meanpath <- NA
    minpar <- NA
    minpar.study <- NA
    H.tilde <- NA
  }
  else {
    x$reference.group <- ""
    ##
    if (random == FALSE & length(tau.preset) == 0) {
      nmak <- nma_krahn(x)
      if (nmak$n == 2) {
        prop <- mpath <- 1
        names(prop) <- names(mpath) <- x$designs
        ##
        res <- list(proportion = prop,
                    meanpath = mpath,
                    minpar = NULL,
                    minpar.study = NULL,
                    H.tilde = NULL,
                    random = random,
                    tau.preset = tau.preset)
        ##
        return(res)
      }
    }
    ##                                                             
    if (length(tau.preset) == 1) {
      nmak <- nma_krahn(x, tau.preset = tau.preset)
      if (nmak$n == 2) {
        prop <- mpath <- 1
        names(prop) <- names(mpath) <- x$designs
        ##
        res <- list(proportion = prop,
                    meanpath = mpath,
                    minpar = NULL,
                    minpar.study = NULL,
                    H.tilde = NULL,
                    random = random,
                    tau.preset = tau.preset)
        ##
        return(res)
      }
    }
    ##
    if (random == TRUE & length(tau.preset) == 0) {
      nmak <- nma_krahn(x, tau.preset = x$tau)
      if (nmak$n == 2) {
        prop <- mpath <- 1
        names(prop) <- names(mpath) <- x$designs
        ##
        res <- list(proportion = prop,
                    meanpath = mpath,
                    minpar = NULL,
                    minpar.study = NULL,
                    H.tilde = NULL,
                    random = random,
                    tau.preset = tau.preset)
        ##
        return(res)
      }
    }
    ##
    comparisons <- nmak$direct[, "comparison"]
    direct  <- nmak$direct
    network <- nmak$network
    ##
    H <- nmak$H
    H.studies <- nmak$H.studies
    ##
    ## Direct evidence proportion (Koenig 2013, subsection 3.4.1 and 3.4.2)
    ##
    proportion <- rep(0, nrow(H))
    names(proportion) <- rownames(H)
    proportion[comparisons] <- network[comparisons,"seTE"]^2 / direct$seTE^2
    ##
    ## Minimal parallelism (Koenig 2013, subsection 3.4.3)
    ## Mean path length (Koenig 2013, subsection 3.4.4)
    ##
    l <- lapply(split(H,
                      matrix(colnames(H), nrow = nrow(H),
                             ncol = length(colnames(H)), byrow = TRUE)),
                function(x) matrix(x, nrow = nrow(H))
                )
    ##
    H.tilde <- sapply(l,
                      function(x)
                        apply(x, 1,
                              function(x)
                                0.5 * (abs(sum(x)) + sum(abs(x)))
                              )
                      )
    ##
    rownames(H.tilde) <- rownames(H)
    ##  
    minpar  <- apply(H.tilde, 1, function(x) 1 / max(abs(x)))
    meanpath <- apply(H.tilde, 1, sum)
    ##
    ## Minimal parallelism (study-level)
    ##
    l.studies <- lapply(split(H.studies,
                              matrix(colnames(H.studies), nrow = nrow(H.studies),
                                     ncol = length(colnames(H.studies)), byrow = TRUE)),
                        function(x) matrix(x, nrow = nrow(H.studies))
                        )
    ##
    H.tilde.studies <- sapply(l.studies,
                              function(x)
                                apply(x, 1,
                                      function(x)
                                        0.5 * (abs(sum(x)) + sum(abs(x)))
                                      )
                              )
    ##
    rownames(H.tilde.studies) <- rownames(H)
    ##  
    minpar.study <- apply(H.tilde.studies, 1,
                          function(x)
                            1 / max(abs(x))
                          )
  }
  
  
  ##
  ##
  ## (3) Create netmeasures object
  ##
  ##
  res <- list(proportion = proportion,
              meanpath = meanpath,
              minpar = minpar,
              minpar.study = minpar.study,
              H.tilde = H.tilde,
              random = random,
              tau.preset = tau.preset,
              version = packageDescription("netmeta")$Version
              )
  ##
  res
}

#' Create a netmeta object from a gemtc object
#' 
#' @description
#' Auxiliary function to create a netmeta object from a gemtc object
#' (experimental feature).
#' 
#' @param x A \code{gemtc} object created with R package \bold{gemtc}.
#' @param keep.samples A logical indicating whether to keep the generated
#'   samples.
#' @param level The level used to calculate confidence intervals for
#'   individual comparisons.
#' @param level.ma The level used to calculate confidence intervals
#'   for network estimates.
#' @param reference.group Reference treatment (first treatment is used
#'   if argument is missing).
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment (default) or vice versa. This argument is
#'   only considered if \code{reference.group} has been specified.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect (passed on to
#'   \code{\link{netrank}}, can be abbreviated.
#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
#'   matrices with all treatment effects, and confidence limits will
#'   be printed.
#' @param seq A character or numerical vector specifying the sequence
#'   of treatments in printouts.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and forest plots. If
#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
#'   presented as odds ratios rather than log odds ratios, for
#'   example.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names (see Details).
#' @param nchar.studlab A numeric defining the minimum number of
#'   characters used to create unique study labels.
#'
#' @return
#' A netmeta object with additional class "netmeta.gemtc".
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}
#' 
#' @examples
#' \donttest{
#' if (requireNamespace("gemtc", quietly = TRUE)) {
#' library("gemtc")
#' # Load the example network and generate a consistency model:
#' model <- mtc.model(smoking, type="consistency")
#' results <- mtc.run(model, thin = 10)
#' detach("package:gemtc")
#'
#' # Print results in netmeta layout
#' mtc <- gemtc2netmeta(results)
#' mtc
#' # SUCRAs
#' netrank(mtc)
#' # Rankogram
#' rankogram(mtc)
#' }
#' }
#' 
#' @export gemtc2netmeta

gemtc2netmeta <- function(x,
                          keep.samples = TRUE,
                          level = gs("level"),
                          level.ma = gs("level.ma"),
                          reference.group =
                            levels(x$model$network$treatments$id)[1],
                          baseline.reference = gs("baseline.reference"),
                          small.values = gs("small.values"),
                          all.treatments = gs("all.treatments"),
                          seq = gs("seq"),
                          backtransf = gs("backtransf"),
                          nchar.trts = gs("nchar.trts"),
                          nchar.studlab = gs("nchar.studlab")) {
  
  #
  #
  # (1) Check for gemtc object and extract data / settings
  #
  #
  
  chkclass(x, "mtc.result")
  #
  if (!is.null(x$model$regressor))
    stop("R function gemtc2netmeta() not suitable for network meta-regression.",
         call. = FALSE)
  #
  chklogical(keep.samples)
  chklevel(level)
  chklevel(level.ma)
  chklogical(baseline.reference)
  small.values <- setsv(replaceNULL(small.values, gs("small.values")))
  if (!is.null(all.treatments))
    chklogical(all.treatments)
  chklogical(backtransf)
  chknumeric(nchar.trts, min = 1, length = 1)
  chknumeric(nchar.studlab, min = 1, length = 1)
  #
  random <- x$model$linearModel == "random"
  common <- !random
  #
  likelihood <- x$model$likelihood
  link <- x$model$link
  #
  # Determine summary measure
  #
  assign_rules <- data.frame(
    lik = rep(c("normal", "binom", "poisson"), c(2, 3, 1)),
    lin = c("identity", "smd", "logit", "log", "cloglog", "log"),
    sm = c("MD", "SMD", "OR", "RR", "HR", "IRR")
  )
  #
  # Get rid of warning 'Undefined global functions or variables'
  #
  lik <- lin <- NULL
  #
  sm <- assign_rules %>% filter(lik == likelihood, lin == link) %>%
    select(sm) %>% pull()
  #
  if (sm == "HR")
    stop("Not yet implemented.")
  #
  trts <- levels(x$model$network$treatments$id)
  labels <- sort(trts)
  #
  trts.long <- x$model$network$treatments$description
  #
  if (!is.null(seq))
    seq <- setseq(seq, labels)
  else {
    seq <- labels
    if (is.numeric(seq))
      seq <- as.character(seq)
  }
  
  
  #
  #
  # (2) Calculate matrices with treatment effects, standard errors, etc.
  #
  #
  
  dmat <- do.call(rbind, x$samples) %>% data.frame() %>%
    select(starts_with("d."))
  dmat <- cbind(0, dmat)
  names(dmat) <- trts
  #
  tau <- NULL
  tmat <- do.call(rbind, x$samples) %>%
    data.frame() %>% select(starts_with("sd."))
  #
  TE.matrix <- seTE.matrix <-
    lower.matrix <- upper.matrix <-
    matrix(NA, nrow = ncol(dmat), ncol = ncol(dmat),
           dimnames = list(trts, trts))
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        TE.matrix[i, j] <- mean(dmat[, i] - dmat[, j])
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        seTE.matrix[i, j] <- sd(dmat[, i] - dmat[, j])
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        lower.matrix[i, j] <- quantile(dmat[, i] - dmat[, j],
                                       (1 - level.ma) / 2)
  #
  for (i in seq_len(ncol(dmat)))
    for (j in seq_len(ncol(dmat)))
      if (i != j)
        upper.matrix[i, j] <- quantile(dmat[, i] - dmat[, j],
                                       1 - (1 - level.ma) / 2)
  #
  diag(TE.matrix) <- diag(seTE.matrix) <-
    diag(lower.matrix) <- diag(upper.matrix) <- 0
  
  
  #
  #
  # (3) Create pairwise object from gemtc data and run network meta-analysis
  #
  #
  
  #
  # Arm-level data
  #
  dat.gemtc <- x$model$network$data.ab
  #
  if (!is.null(dat.gemtc)) {
    #
    # Get rid of warning 'Undefined global functions or variables'
    #
    treatment <- study <- responders <- sampleSize <- exposure <-
      mean <- std.err <- std.dev <- NULL
    #
    if (sm %in% c("OR", "RR")) {
      suppressWarnings(
        dat <-
          pairwise(treat = treatment, studlab = study,
                   event = responders, n = sampleSize,
                   data = dat.gemtc,
                   sm = sm, warn = FALSE))
    }
    else if (sm == "MD") {
      #
      # Only means and standard errors provided
      #
      if (is.null(dat.gemtc$std.dev)) {
        suppressWarnings(
          dat <-
            pairwise(treat = treatment,
                     TE = mean, seTE = std.err,
                     studlab = study,
                     data = dat.gemtc,
                     sm = sm, warn = FALSE))
      }
      else {
        suppressWarnings(
          dat <-
            pairwise(treat = treatment,
                     n = sampleSize, mean = mean, sd = std.dev,
                     studlab = study,
                     data = dat.gemtc,
                     sm = sm, warn = FALSE))
      }
    }
    else if (sm == "SMD") {
      suppressWarnings(
        dat <-
          pairwise(treat = treatment,
                   n = sampleSize, mean = mean, sd = std.dev,
                   studlab = study,
                   data = dat.gemtc,
                   sm = sm, warn = FALSE))
    }
    else if (sm == "IRR") {
      suppressWarnings(
        dat <-
          pairwise(treat = treatment, studlab = study,
                   event = responders, time = exposure,
                   data = dat.gemtc,
                   sm = sm, warn = FALSE))
    }
  }
  else
    stop("R function gemtc2netmeta() not suitable for relative effect data.",
         call. = FALSE)
  #
  res <- suppressWarnings(
    netmeta(dat, common = common, random = random, warn = FALSE))
  
  
  #
  #
  # (4) Add gemtc results to netmeta object
  #
  #
  
  res <- setNA_nma(res)
  #
  res$keep.samples <- keep.samples
  #
  if (keep.samples) {
    res$samples <- list(d = dmat)
    #
    if (random)
      res$samples$tau <- tmat
  }
  #
  res$method <- "gemtc"
  res$method.tau <- ""
  res$m <- NA
  #
  res$level <- gs("level")
  res$level.ma <- level.ma
  res$reference.group <- setchar(reference.group, trts)
  res$baseline.reference <- baseline.reference
  res$small.values <- small.values
  res$all.treatments <- all.treatments
  res$seq <- seq
  res$backtransf <- backtransf
  res$nchar.trts <- nchar.trts
  res$nchar.studlab <- nchar.studlab
  #
  if (common) {
    res$TE.common <- TE.matrix[labels, labels]
    res$seTE.common <- seTE.matrix[labels, labels]
    res$lower.common <- lower.matrix[labels, labels]
    res$upper.common <- upper.matrix[labels, labels]
    #
    for (i in seq_along(res$treat1)) {
      res$TE.nma.common[i] <- res$TE.common[res$treat1[i], res$treat2[i]]
      res$seTE.nma.common[i] <- res$seTE.common[res$treat1[i], res$treat2[i]]
      res$lower.nma.common[i] <- res$lower.common[res$treat1[i], res$treat2[i]]
      res$upper.nma.common[i] <- res$upper.common[res$treat1[i], res$treat2[i]]
    }
    #
    res$tau2 <- res$tau <- NA
    #
    res$TE.random[!is.na(res$TE.random)] <- NA
    res$seTE.random <- res$lower.random <- res$upper.random <- res$TE.random
  }
  else if (random) {
    res$TE.random <- TE.matrix[labels, labels]
    res$seTE.random <- seTE.matrix[labels, labels]
    res$lower.random <- lower.matrix[labels, labels]
    res$upper.random <- upper.matrix[labels, labels]
    #
    for (i in seq_along(res$treat1)) {
      res$TE.nma.random[i] <- res$TE.random[res$treat1[i], res$treat2[i]]
      res$seTE.nma.random[i] <- res$seTE.random[res$treat1[i], res$treat2[i]]
      res$lower.nma.random[i] <- res$lower.random[res$treat1[i], res$treat2[i]]
      res$upper.nma.random[i] <- res$upper.random[res$treat1[i], res$treat2[i]]
    }
    #
    res$tau2 <- mean(tmat %>% unlist() %>% unname())
    res$tau <- sqrt(res$tau2)
    #
    res$TE.common[!is.na(res$TE.common)] <- NA
    res$seTE.common <- res$lower.common <- res$upper.common <- res$TE.common
  }
  
  
  #
  #
  # (5) Return netmeta object
  #
  #
  
  class(res) <- c(class(res), "netmeta.gemtc")
  #
  res
}

#' Split direct and indirect evidence in network meta-analysis
#' 
#' @description
#' Methods to split network estimates into the contribution of direct
#' and indirect evidence and to test for local inconsistency in
#' network meta-analysis.
#' 
#' @aliases netsplit print.netsplit
#' 
#' @param x An object of class \code{netmeta} or \code{netsplit}.
#' @param method A character string indicating which method to split
#'   direct and indirect evidence is to be used. Either
#'   \code{"Back-calculation"} or \code{"SIDDE"}, can be abbreviated.
#'   See Details.
#' @param upper A logical indicating whether treatment comparisons
#'   should be selected from the lower or upper triangle of the
#'   treatment effect matrices (see list elements \code{TE.common} and
#'   \code{TE.random} in the \code{netmeta} object). Ignored if
#'   argument \code{order} is provided.
#' @param reference.group Reference treatment. Ignored if argument
#'   \code{order} is provided.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment or vice versa. This argument is only
#'   considered if \code{reference.group} is not equal to \code{""}
#'   and argument\code{order} is not provided.
#' @param order A optional character or numerical vector specifying
#'   the order of treatments in comparisons.
#' @param sep.trts A character string used in comparison names as
#'   separator between treatment labels, e.g., " vs ".
#' @param quote.trts A character used to print around treatment
#'   labels.
#' @param tol.direct A numeric defining the maximum deviation of the
#'   direct evidence proportion from 0 or 1 to classify a comparison
#'   as providing only indirect or direct evidence, respectively.
#' @param common A logical indicating whether results for the common
#'   effects network meta-analysis should be printed.
#' @param random A logical indicating whether results for the random
#'   effects network meta-analysis should be printed.
#' @param show A character string indicating which comparisons should
#'   be printed (see Details).
#' @param overall A logical indicating whether estimates from network
#'   meta-analysis should be printed.
#' @param direct A logical indicating whether direct estimates should
#'   be printed.
#' @param indirect A logical indicating whether indirect estimates
#'   should be printed.
#' @param ci A logical indicating whether confidence intervals should
#'   be printed in addition to treatment estimates.
#' @param test A logical indicating whether results of a test
#'   comparing direct and indirect estimates should be printed.
#' @param only.reference A logical indicating whether only comparisons
#'   with the reference group should be printed.
#' @param sortvar An optional vector used to sort comparisons (must be
#'   of same length as the total number of comparisons).
#' @param subset An optional logical vector specifying a subset of
#'   comparisons to print (must be of same length as the total number of
#'   comparisons) .
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z-value
#'   of test of agreement between direct and indirect evidence, see
#'   \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of test of agreement between direct and indirect evidence, see
#'   \code{print.default}.
#' @param digits.prop Minimal number of significant digits for direct
#'   evidence proportions, see \code{print.default}.
#' @param text.NA A character string specifying text printed for
#'   missing values.
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. For example, if \code{backtransf =
#'   TRUE}, results for \code{sm = "OR"} are printed as odds ratios
#'   rather than log odds ratios.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param big.mark A character used as thousands separator.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param indent A logical indicating whether items in the legend
#'   should be indented.
#' @param warn A logical indicating whether warnings should be
#'   printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param verbose A logical indicating whether progress information
#'   should be printed.
#' @param \dots Additional arguments.
#' 
#' @details
#' A comparison of direct and indirect treatment estimates can serve
#' as check for consistency of network meta-analysis (Dias et al.,
#' 2010).
#' 
#' This function provides two methods to derive indirect estimates:
#' \itemize{
#' \item Separate Indirect from Direct Evidence (SIDE) using a
#'   back-calculation method (\code{method = "Back-calculation"})
#'   based on the \emph{direct evidence proportion} to calculate the
#'   indirect evidence (König et al., 2013);
#' \item Separate Indirect from Direct Design Evidence (SIDDE) as
#'   described in Efthimiou et al. (2019).
#' }
#' 
#' Note, for the back-calculation method, indirect treatment estimates
#' are already calculated in \code{\link{netmeta}} and this function
#' combines and prints these estimates in a user-friendly
#' way. Furthermore, this method is not available for the
#' Mantel-Haenszel and non-central hypergeometric distribution
#' approach implemented in \code{\link{netmetabin}}.
#' 
#' For the random-effects model, the direct treatment estimates are
#' based on the common between-study variance \eqn{\tau^2} from the
#' network meta-analysis, i.e. the square of list element
#' \code{x$tau}.
#' 
#' Argument \code{show} determines which comparisons are printed:
#' \tabular{ll}{
#' \dQuote{all} \tab All comparisons \cr
#' \dQuote{both} \tab Only comparisons contributing both direct and
#'   indirect evidence \cr
#' \dQuote{with.direct} \tab Comparisons providing direct evidence \cr
#' \dQuote{direct.only} \tab Comparisons providing only direct
#'   evidence \cr
#' \dQuote{indirect.only} \tab Comparisons providing only indirect
#'   evidence
#' }
#' 
#' Wilson et al. (2026) introduced a scatter plot of direct and indirect
#' estimates to visualise local consistency in a network meta-analysis.
#' Alternatively, a forest plot with direct and indirect estimates can be
#' generated.
#' 
#' The node-splitting method and SIDDE can be compute-intensive in
#' large networks. Crude information on the computation progress is
#' printed if argument \code{verbose = TRUE}. In addition, computation
#' times are printed if R package \bold{tictoc} is installed.
#'
#' @return
#' An object of class \code{netsplit} with corresponding \code{print}
#' \code{forest}, and \code{plot} functions. The object is a list containing
#' the following components:
#' \item{common, random}{As defined above.}
#' \item{comparison}{A vector with treatment comparisons.}
#' \item{prop.common, prop.random}{A vector with direct evidence
#'   proportions (common / random effects model).}
#' \item{common, random}{Results of network meta-analysis (common /
#'   random effects model), i.e., data frame with columns comparison,
#'   TE, seTE, lower, upper, z, and p.}
#' \item{direct.common, direct.random}{Network meta-analysis results
#'   based on direct evidence (common / random effects model), i.e.,
#'   data frame with columns comparison, TE, seTE, lower, upper, z,
#'   and p.}
#' \item{indirect.common, indirect.random}{Network meta-analysis
#'   results based on indirect evidence (common / random effects
#'   model), i.e., data frame with columns comparison, TE, seTE,
#'   lower, upper, z, and p.}
#' \item{compare.common, compare.random}{Comparison of direct and
#'   indirect evidence in network meta-analysis (common / random
#'   effects model), i.e., data frame with columns comparison, TE,
#'   seTE, lower, upper, z, and p.}
#' \item{sm}{A character string indicating underlying summary measure}
#' \item{level.ma}{The level used to calculate confidence intervals
#'   for pooled estimates.}
#' \item{tictoc}{Computation times for node-splitting method or SIDDE
#'   (if R package \bold{tictoc} is installed).}
#' \item{version}{Version of R package netmeta used to create object.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}, Gerta
#'   Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Orestis Efthimiou
#'   \email{oremiou@@gmail.com}
#' 
#' @seealso \code{\link{forest.netsplit}}, \code{\link{plot.netsplit}},
#'   \code{\link{netmeta}}, \code{\link{netmetabin}},
#'   \code{\link{netmeasures}},
#'   \code{\link[metabook]{Woods2010}}, \code{\link{Senn2013}},
#'   \code{\link[metabook]{Dong2013}}
#' 
#' @references
#' Dias S, Welton NJ, Caldwell DM, Ades AE (2010):
#' Checking consistency in mixed treatment comparison meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{29}, 932--44
#' 
#' Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
#' (2019):
#' A Mantel-Haenszel model for network meta-analysis of rare events.
#' \emph{Statistics in Medicine},
#' \bold{38}, 2992--3012
#' 
#' König J, Krahn U, Binder H (2013):
#' Visualizing the flow of evidence in network meta-analysis and
#' characterizing mixed treatment comparisons.
#' \emph{Statistics in Medicine},
#' \bold{32}, 5414--29
#' 
#' Puhan MA, Schünemann HJ, Murad MH, et al. (2014):
#' A GRADE working group approach for rating the quality of treatment
#' effect estimates from network meta-analysis.
#' \emph{British Medical Journal},
#' \bold{349}, g5630
#' 
#' Wilson H, Schönstein A, Robson S, Bonofiglio F (2026):
#' A novel approach for visualizing local consistency in
#' network meta-analysis.
#' \emph{Research Synthesis Methods},
#' 1--15
#' 
#' @examples
#' #
#' # 1) COPD example
#' #
#' 
#' pw1 <- pairwise(treatment, event = r, n = N,
#'   studlab = author, data = Woods2010, sm = "OR")
#' #
#' nma1 <- netmeta(pw1)
#' #
#' print(netsplit(nma1), digits = 2)
#' 
#' \donttest{
#' print(netsplit(nma1), digits = 2,
#'   backtransf = FALSE, common = FALSE)
#'
#' # Sort by increasing number of studies in direct comparisons
#' print(netsplit(nma1), digits = 2, sortvar = k)
#' # Sort by decreasing number of studies in direct comparisons
#' print(netsplit(nma1), digits = 2, sortvar = -k)
#' 
#' # Sort by increasing evidence proportion under common effects model
#' print(netsplit(nma1), digits = 2, sortvar = prop.common)
#' # Sort by decreasing evidence proportion under common effects model
#' print(netsplit(nma1), digits = 2, sortvar = -prop.common)
#' 
#' # Sort by decreasing evidence proportion under common effects model
#' # and number of studies
#' print(netsplit(nma1), digits = 2, sortvar = cbind(-prop.common, -k))
#' 
#' #
#' # 2) Diabetes example
#' #
#' 
#' pw2 <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"))
#' nma2 <- netmeta(pw2, reference = "plac")
#' #
#' ns2 <- netsplit(nma2)
#' #
#' print(ns2, digits = 2)
#' # Layout of Puhan et al. (2014), Table 1
#' print(ns2, digits = 2, ci = TRUE, test = FALSE)
#' 
#' # Forest plot showing comparisons contributing both direct and
#' # indirect evidence
#' #
#' forest(ns2, fontsize = 6, spacing = 0.5, addrow.subgroups = FALSE)
#' 
#' # Scatter plot to evaluate local inconsistency
#' #
#' plot(ns2)
#' 
#' # Forest plot showing comparisons contributing direct evidence
#' #
#' forest(ns2, fontsize = 6, spacing = 0.5, addrow.subgroups = FALSE,
#'   show = "with.direct")
#' 
#' # Forest plot only showing network estimates compared to reference
#' # group and prediction intervals
#' #
#' forest(ns2, fontsize = 8, spacing = 0.75, show = "all",
#'   only.reference = TRUE, prediction = TRUE,
#'   direct = FALSE, indirect = FALSE)
#' 
#' #
#' # 3) Another COPD example
#' #
#' 
#' pw3 <- pairwise(treatment, death, randomized, studlab = id,
#'   data = Dong2013, sm = "OR")
#' nma3 <- netmetabin(pw3)
#' netsplit(nma3)
#' }
#' 
#' @rdname netsplit
#' @export netsplit

netsplit <- function(x, method,
                     upper = TRUE,
                     reference.group = x$reference.group,
                     baseline.reference = x$baseline.reference,
                     #
                     show = gs("show"),
                     overall = TRUE,
                     direct = TRUE,
                     indirect = TRUE,
                     only.reference = FALSE,
                     ci = FALSE,
                     test = show %in% c("all", "with.direct", "both"),
                     #
                     order = NULL,
                     #
                     nchar.trts = x$nchar.trts,
                     sep.trts = x$sep.trts, quote.trts = "",
                     tol.direct = 0.0005,
                     common = x$common,
                     random = x$random,
                     backtransf = x$backtransf,
                     warn = FALSE, warn.deprecated = gs("warn.deprecated"),
                     verbose = FALSE,
                     ...) {
  
  ##
  ##
  ## (1) Check for netmeta object and upgrade object
  ##
  ##
  chkclass(x, "netmeta")
  chksuitable(x, "Methods to split network estimates",
              classes = gs(".other_nma"))
  x <- updateversion(x)
  ##
  is.mh.nch <- inherits(x, "netmetabin") && x$method %in% c("MH", "NCH")
  ##
  is.tictoc <- is_installed_package("tictoc", stop = FALSE)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  if (!missing(method))
    method <- setchar(method, c("Back-calculation", "SIDDE"))
  else {
    if (is.mh.nch)
      method <- "SIDDE"
    else
      method <- "Back-calculation"
  }
  ##
  chklogical(upper)
  chklogical(baseline.reference)
  ##
  if (!is.null(order)) {
    order <- setseq(order, x$trts)
    baseline.reference <- FALSE
    reference.group <- ""
  }
  #
  show <- setchar(show, c("all", "both", "with.direct",
                          "direct.only", "indirect.only",
                          "reference.only"))
  chklogical(overall)
  chklogical(direct)
  chklogical(indirect)
  chklogical(ci)
  chklogical(test)
  chklogical(only.reference)
  #
  nchar.trts <- replaceNULL(nchar.trts, 666)
  chknumeric(nchar.trts, length = 1)
  #
  chkchar(sep.trts)
  chkchar(quote.trts)
  chknumeric(tol.direct, min = 0, length = 1)
  if (!is.null(backtransf))
    chklogical(backtransf)
  chklogical(warn)
  chklogical(verbose)
  ##
  ## Check for deprecated arguments in '...'
  ##
  args <- list(...)
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  ##
  x$common <- common
  x$random <- random
  
  
  ##
  ##
  ## (3) Create dat.trts
  ##
  ##
  dat.trts <- comptrts(x, upper, reference.group, baseline.reference,
                       order, sep.trts, quote.trts)
  
  
  ##
  ##
  ## (4) Change order of prop.direct.common and prop.direct.random
  ##
  ##
  if (!(is.mh.nch & method == "SIDDE")) {
    prop.common <- sortprop(x$prop.direct.common, dat.trts, x$sep.trts)
    prop.random <- sortprop(x$prop.direct.random, dat.trts, x$sep.trts)
  }
  else
    prop.common <- prop.random <- NULL
  
  
  ##
  ##
  ## (5) Calculate / extract indirect estimates
  ##
  ##
  x.direct.indirect <- x
  ##
  if (method == "SIDDE") {
    ind <- sidde(x.direct.indirect, sep.trts, verbose, warn, is.tictoc)
    ##
    x.direct.indirect$TE.indirect.common <- ind$TE.indirect.common
    x.direct.indirect$seTE.indirect.common <- ind$seTE.indirect.common
    ##
    if (!is.mh.nch) {
      x.direct.indirect$TE.indirect.random <- ind$TE.indirect.random
      x.direct.indirect$seTE.indirect.random <- ind$seTE.indirect.random
    }
  }
  ##
  direct.indirect <- direct.indirect(x.direct.indirect, tol.direct)
  
  
  ##
  ##
  ## (6) Transform matrices to data frames
  ##
  ##
  m2d.f <- mat2dat.split(direct.indirect, "common", dat.trts)
  m2d.r <- mat2dat.split(direct.indirect, "random", dat.trts)
  
  
  ##
  ##
  ## (7) Return results
  ##
  ##
  res <- list(comparison = dat.trts$comparison,
              #
              k = m2d.f$k,
              #
              show = show,
              overall = overall,
              direct = direct,
              indirect = indirect,
              ci = ci,
              test = test,
              only.reference = only.reference,
              #
              prop.common = prop.common,
              #
              common = m2d.f$nma,
              direct.common = m2d.f$direct,
              indirect.common = m2d.f$indirect,
              compare.common = m2d.f$compare,
              ##
              prop.random = prop.random,
              ##
              random = m2d.r$nma,
              direct.random = m2d.r$direct,
              indirect.random = m2d.r$indirect,
              compare.random = m2d.r$compare,
              ##
              predict = m2d.r$predict,
              ##
              method = method,
              ##
              sm = x$sm,
              level.ma = x$level.ma,
              ##
              prediction = x$prediction,
              level.predict = x$level.predict,
              tau = x$tau,
              ##
              reference.group = reference.group,
              baseline.reference = baseline.reference,
              order = order,
              sep.trts = sep.trts,
              quote.trts = quote.trts,
              nchar.trts = nchar.trts,
              #
              tol.direct = tol.direct,
              backtransf = backtransf,
              ##
              x = x,
              ##
              version = packageDescription("netmeta")$Version
              )
  ##
  if (method == "SIDDE" & is.tictoc)
    res$tictoc <- ind$tictoc
  ##
  ## Backward compatibility
  ##
  res$prop.fixed <- res$prop.common
  res$fixed <- res$common
  res$direct.fixed <- res$direct.common
  res$indirect.fixed <- res$indirect.common
  res$compare.fixed <- res$compare.common
  ##
  class(res) <-
    c("netsplit",
      if (is.mh.nch & method == "SIDDE")
        "netsplit.netmetabin")
  
  res
}


#' @rdname netsplit
#' @method print netsplit
#' @export

print.netsplit <- function(x,
                           common = x$x$common,
                           random = x$x$random,
                           ##
                           show = x$show,
                           overall = x$overall,
                           direct = x$direct,
                           indirect = x$indirect,
                           ci = x$ci,
                           test = x$test,
                           only.reference = x$only.reference,
                           #
                           sortvar = NULL,
                           subset = NULL,
                           ##
                           nchar.trts = x$nchar.trts,
                           ##
                           digits = gs("digits"),
                           digits.stat = gs("digits.stat"),
                           digits.pval = gs("digits.pval"),
                           digits.prop = max(gs("digits.pval") - 2, 2),
                           ##
                           text.NA = gs("lab.NA"),
                           backtransf = x$backtransf,
                           scientific.pval = gs("scientific.pval"),
                           big.mark = gs("big.mark"),
                           legend = gs("legend"),
                           ##
                           indent = TRUE,
                           warn.deprecated = gs("warn.deprecated"),
                           ##
                           ...) {
  
  ##
  ##
  ## (1) Check for netsplit object and upgrade object
  ##
  ##
  chkclass(x, "netsplit")
  x <- updateversion(x)
  ##
  is.bin <- inherits(x, "netsplit.netmetabin")
  ##
  ## All individual results in a single row - be on the save side:
  ##
  oldopts <- options(width = 200)
  on.exit(options(oldopts))
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  ##
  args <- list(...)
  ##
  show <-
    deprecated(show, missing(show), args, "showall")
  if (is.logical(show))
    if (show)
      show <- "all"
    else
      show <- "both"
  ##
  show <- setchar(show, c("all", "both", "with.direct",
                          "direct.only", "indirect.only",
                          "reference.only"))
  ##
  if (show == "reference.only") {
    warning("Argument 'show = \"reference.only\" replaced with ",
            "'only.reference = TRUE'.",
            call. = FALSE)
    show <- "both"
    if (missing.only.reference)
      only.reference <- TRUE
  }
  ##
  chklogical(overall)
  chklogical(direct)
  chklogical(indirect)
  chklogical(ci)
  chklogical(test)
  ##
  missing.only.reference <- missing(only.reference)
  if (!missing.only.reference)
    chklogical(only.reference)
  #
  # Catch sortvar and subset from data:
  #
  mc <- match.call()
  sfsp <- sys.frame(sys.parent())
  #
  error <- try(sortvar.x <- catch("sortvar", mc, x, sfsp), silent = TRUE)
  if (!any(class(error) == "try-error"))
    sortvar <- sortvar.x
  ##
  if (!is.null(sortvar)) {
    if (length(dim(sortvar)) == 2) {
      if (dim(sortvar)[1] != length(x$comparison))
        stop("Argument 'sortvar' must be of length ",
             length(x$comparison), ".",
             call. = FALSE)
      ##
      ## Set proportions to 0 or 1
      ##
      if (is.numeric(sortvar)) {
        sortvar[is_zero(abs(sortvar), n = 1000)] <- 0
        sortvar[is_zero(1 - abs(sortvar), n = 1000)] <-
          1 * sign(sortvar)[is_zero(1 - abs(sortvar), n = 1000)]
      }
      sortvar <- order(do.call(order, as.list(as.data.frame(sortvar))))
    }
    else
      chklength(sortvar, length(x$comparison),
                text = paste0("Argument 'sortvar' must be of length ",
                              length(x$comparison), "."))
    ##
    if (!is.numeric(sortvar))
      sortvar <- setchar(sortvar, x$comparison)
  }
  #
  error <- try(subset.x <- catch("subset", mc, x, sfsp), silent = TRUE)
  if (!any(class(error) == "try-error"))
    subset <- subset.x
  ##
  if (!is.null(subset)) {
    chklength(subset, length(x$comparison),
              text = paste0("Argument 'subset' must be of length ",
                            length(x$comparison), "."))
    if (!is.logical(subset))
      stop("Argument 'subset' must be a logical vector.")
  }
  #
  nchar.trts <- replaceNULL(nchar.trts, 666)
  chknumeric(nchar.trts, length = 1)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  ##
  if (is.null(backtransf))
    backtransf <- TRUE
  chklogical(backtransf)
  chklogical(scientific.pval)
  chklogical(legend)
  chklogical(indent)
  ##
  ## Check for deprecated arguments in '...'
  ##
  fun <- "print.netmeta"
  ##
  chklogical(warn.deprecated)
  ##
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  common.logical <- common
  ##
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  random.logical <- random
  
  
  ##
  ##
  ## (3) Some additional settings and checks
  ##
  ##
  sm <- sm.lab <- x$sm
  #
  relative <- is_relative_effect(sm) | sm == "VE"
  ##
  if (!backtransf & relative)
    sm.lab <- paste0("log", if (sm == "VE") "VR" else sm)
  ##
  if (!(sm.lab == "" | sm.lab == "log"))
    sm.lab <- paste0("(", sm.lab, ") ")
  else
    sm.lab <- ""
  ##
  level.ma <- x$level.ma
  ci.lab <- paste(100 * level.ma, "% CI", sep ="")
  ##  
  random.available <- !is.null(x$random)
  ##
  if (!random.available & random) {
    warning("No results for random effects model available. ",
            "Argument 'random' set to FALSE.",
            call. = FALSE)
    ##
    random <- FALSE
  }
  
  
  if (show == "all")
    sel <- rep_len(TRUE, length(x$direct.common$TE))
  else if (show == "with.direct")
    sel <- !is.na(x$direct.common$TE)
  else if (show == "both")
    sel <- !is.na(x$direct.common$TE) & !is.na(x$indirect.common$TE)
  else if (show == "direct.only")
    sel <- !is.na(x$direct.common$TE) & is.na(x$indirect.common$TE)
  else if (show == "indirect.only")
    sel <- is.na(x$direct.common$TE) & !is.na(x$common$TE)
  ##
  if (sum(sel) == 0) {
    warning("No comparisons selected with argument 'show = ", show, "'. ",
            "You could use 'show = \"all\" to show all comparisons.",
            call. = FALSE)
    return(invisible(NULL))
  }
  ##
  if (only.reference) {
    if (x$reference.group == "") {
      warning("First treatment used as reference as argument ",
              "'reference.group' was unspecified in netsplit().",
              call. = FALSE)
      x$reference.group <-
        compsplit(x$comparison, x$sep.trts)[[1]][1]
    }
    ##
    sel.ref <-
      apply(!is.na(sapply(compsplit(x$comparison, x$sep.trts),
                          match, x$reference.group)), 2, sum) >= 1
    ##
    sel <- sel & sel.ref
  }
  ##
  comp <- x$comparison[sel]
  ##
  k <- x$k[sel]
  ##
  prop.common <- x$prop.common[sel]
  ##
  TE.common <- x$common$TE[sel]
  lower.common <- x$common$lower[sel]
  upper.common <- x$common$upper[sel]
  ##
  TE.direct.common <- x$direct.common$TE[sel]
  lower.direct.common <- x$direct.common$lower[sel]
  upper.direct.common <- x$direct.common$upper[sel]
  ##
  TE.indirect.common <- x$indirect.common$TE[sel]
  lower.indirect.common <- x$indirect.common$lower[sel]
  upper.indirect.common <- x$indirect.common$upper[sel]
  ##
  TE.compare.common <- x$compare.common$TE[sel]
  lower.compare.common <- x$compare.common$lower[sel]
  upper.compare.common <- x$compare.common$upper[sel]
  statistic.compare.common <- x$compare.common$statistic[sel]
  pval.compare.common <- x$compare.common$p[sel]
  ##
  if (random.available) {
    prop.random <- x$prop.random[sel]
    ##
    TE.random <- x$random$TE[sel]
    lower.random <- x$random$lower[sel]
    upper.random <- x$random$upper[sel]
    ##
    TE.direct.random <- x$direct.random$TE[sel]
    lower.direct.random <- x$direct.random$lower[sel]
    upper.direct.random <- x$direct.random$upper[sel]
    ##
    TE.indirect.random <- x$indirect.random$TE[sel]
    lower.indirect.random <- x$indirect.random$lower[sel]
    upper.indirect.random <- x$indirect.random$upper[sel]
    ##
    TE.compare.random <- x$compare.random$TE[sel]
    lower.compare.random <- x$compare.random$lower[sel]
    upper.compare.random <- x$compare.random$upper[sel]
    statistic.compare.random <- x$compare.random$statistic[sel]
    pval.compare.random <- x$compare.random$p[sel]
  }
  
  
  if (backtransf) {
    TE.common <- backtransf(TE.common, sm)
    lower.common <- backtransf(lower.common, sm)
    upper.common <- backtransf(upper.common, sm)
    ##
    TE.direct.common <- backtransf(TE.direct.common, sm)
    lower.direct.common <- backtransf(lower.direct.common, sm)
    upper.direct.common <- backtransf(upper.direct.common, sm)
    ##
    TE.indirect.common <- backtransf(TE.indirect.common, sm)
    lower.indirect.common <- backtransf(lower.indirect.common, sm)
    upper.indirect.common <- backtransf(upper.indirect.common, sm)
    ##
    TE.compare.common <- backtransf(TE.compare.common, sm)
    lower.compare.common <- backtransf(lower.compare.common, sm)
    upper.compare.common <- backtransf(upper.compare.common, sm)
    ##
    if (random.available) {
      TE.random <- backtransf(TE.random, sm)
      lower.random <- backtransf(lower.random, sm)
      upper.random <- backtransf(upper.random, sm)
      ##
      TE.direct.random <- backtransf(TE.direct.random, sm)
      lower.direct.random <- backtransf(lower.direct.random, sm)
      upper.direct.random <- backtransf(upper.direct.random, sm)
      ##
      TE.indirect.random <- backtransf(TE.indirect.random, sm)
      lower.indirect.random <- backtransf(lower.indirect.random, sm)
      upper.indirect.random <- backtransf(upper.indirect.random, sm)
      ##
      TE.compare.random <- backtransf(TE.compare.random, sm)
      lower.compare.random <- backtransf(lower.compare.random, sm)
      upper.compare.random <- backtransf(upper.compare.random, sm)
    }
    #
    # Switch lower and upper limit for VE if results have been
    # backtransformed
    #
    if (sm == "VE") {
      tmp.l <- lower.common
      lower.common <- upper.common
      upper.common <- tmp.l
      #
      tmp.l <- lower.direct.common
      lower.direct.common <- upper.direct.common
      upper.direct.common <- tmp.l
      #
      tmp.l <- lower.indirect.common
      lower.indirect.common <- upper.indirect.common
      upper.indirect.common <- tmp.l
      #
      tmp.l <- lower.compare.common
      lower.compare.common <- upper.compare.common
      upper.compare.common <- tmp.l
      #
      if (random.available) {
        tmp.l <- lower.random
        lower.random <- upper.random
        upper.random <- tmp.l
        #
        tmp.l <- lower.direct.random
        lower.direct.random <- upper.direct.random
        upper.direct.random <- tmp.l
        #
        tmp.l <- lower.indirect.random
        lower.indirect.random <- upper.indirect.random
        upper.indirect.random <- tmp.l
        #
        tmp.l <- lower.compare.random
        lower.compare.random <- upper.compare.random
        upper.compare.random <- tmp.l
      }
    }
  }
  
  
  common <- list(comp = comp,
                 k = k,
                 prop = formatPT(prop.common, digits = digits.prop))
  names.common <- c("comparison", "k", "prop")
  ##
  if (overall) {
    common$TE.common <- formatN(TE.common, digits, text.NA = text.NA,
                                big.mark = big.mark)
    names.common <- c(names.common, "nma")
    if (ci) {
      common$ci.common <- formatCI(round(lower.common, digits),
                                   round(upper.common, digits))
      common$ci.common[is.na(common$ci.common)] <- text.NA
      names.common <- c(names.common, ci.lab)
    }
  }
  #
  if (direct) {
    common$TE.direct.common <-
      formatN(TE.direct.common, digits, text.NA = text.NA,
              big.mark = big.mark)
    names.common <- c(names.common, "direct")
    if (ci) {
      common$ci.direct.common <-
        formatCI(round(lower.direct.common, digits),
                 round(upper.direct.common, digits))
      common$ci.direct.common[is.na(common$ci.direct.common)] <- text.NA
      names.common <- c(names.common, ci.lab)
    }
  }
  #
  if (indirect) {
    common$TE.indirect.common <-
      formatN(TE.indirect.common, digits,
              text.NA = text.NA, big.mark = big.mark)
    names.common <- c(names.common, "indir.")
    #
    if (ci) {
      common$ci.indirect.common <-
        formatCI(round(lower.indirect.common, digits),
                 round(upper.indirect.common, digits))
      common$ci.indirect.common[is.na(common$ci.indirect.common)] <- text.NA
      names.common <- c(names.common, ci.lab)
    }
  }
  #
  if (test) {
    common$diff <- formatN(TE.compare.common, digits, text.NA = text.NA,
                           big.mark = big.mark)
    names.common <-
      c(names.common, if (backtransf & relative) "RoR" else "Diff")
    if (ci) {
      common$ci.diff <- formatCI(round(lower.compare.common, digits),
                                 round(upper.compare.common, digits))
      common$ci.diff[is.na(common$ci.diff)] <- text.NA
      names.common <- c(names.common, ci.lab)
    }
    ##
    common$statistic <- formatN(statistic.compare.common, digits.stat,
                                big.mark = big.mark)
    common$statistic[common$statistic == "--"] <- text.NA
    common$p <- formatPT(pval.compare.common, digits = digits.pval,
                         scientific = scientific.pval)
    common$p[rmSpace(common$p) == "--"] <- text.NA
    names.common <- c(names.common, c("z", "p-value"))
  }
  #
  common <- as.data.frame(common)
  names(common) <- names.common
  
  
  if (random.available) {
    random <- list(comp = comp,
                   k = k,
                   prop = formatPT(prop.random, digits = digits.prop))
    names.random <- c("comparison", "k", "prop")
    ##
    if (overall) {
      random$TE.random <- formatN(TE.random, digits, text.NA = text.NA,
                                  big.mark = big.mark)
      names.random <- c(names.random, "nma")
      if (ci) {
        random$ci.random <- formatCI(round(lower.random, digits),
                                     round(upper.random, digits))
        random$ci.random[is.na(random$ci.random)] <- text.NA
        names.random <- c(names.random, ci.lab)
      }
    }
    #
    if (direct) {
      random$TE.direct.random <- formatN(TE.direct.random, digits,
                                         text.NA = text.NA,
                                         big.mark = big.mark)
      names.random <- c(names.random, "direct")
      if (ci) {
        random$ci.direct.random <- formatCI(round(lower.direct.random, digits),
                                            round(upper.direct.random, digits))
        random$ci.direct.random[is.na(random$ci.direct.random)] <- text.NA
        names.random <- c(names.random, ci.lab)
      }
    }
    #
    if (indirect) {
      random$TE.indirect.random <- formatN(TE.indirect.random, digits,
                                           text.NA = text.NA,
                                           big.mark = big.mark)
      names.random <- c(names.random, "indir.")
      if (ci) {
        random$ci.indirect.random <-
          formatCI(round(lower.indirect.random, digits),
                   round(upper.indirect.random, digits))
        random$ci.indirect.random[is.na(random$ci.indirect.random)] <- text.NA
        names.random <- c(names.random, ci.lab)
      }
    }
    #
    if (test) {
      random$diff <- formatN(TE.compare.random, digits, text.NA = text.NA,
                             big.mark = big.mark)
      names.random <- c(names.random,
                        if (backtransf & relative) "RoR" else "Diff")
      if (ci) {
        random$ci.diff <- formatCI(round(lower.compare.random, digits),
                                   round(upper.compare.random, digits))
        random$ci.diff[is.na(random$ci.diff)] <- text.NA
        names.random <- c(names.random, ci.lab)
      }
      ##
      random$statistic <- formatN(statistic.compare.random, digits.stat,
                                  big.mark = big.mark)
      random$statistic[random$statistic == "--"] <- text.NA
      random$p <- formatPT(pval.compare.random, digits = digits.pval,
                           scientific = scientific.pval)
      random$p[rmSpace(random$p) == "--"] <- text.NA
      names.random <- c(names.random, c("z", "p-value"))
    }
    random <- as.data.frame(random)
    names(random) <- names.random
  }
  
  
  ## Do not print direct evidence proportion for node-splitting method
  ## or SIDDE
  ## 
  ##
  noprop <- is.bin | x$method == "SIDDE" | all(common$prop == "")
  ##
  if (noprop) {
    common <- common[, !(names(common) %in% "prop")]
    if (random.available)
      random <- random[, !(names(random) %in% "prop")]
  }
  

  if (!is.null(sortvar))
    sortvar <- sortvar[sel]
  #
  if (!is.null(subset))
    subset <- subset[sel]
  #
  if (!is.null(sortvar)) {
    o <- order(sortvar)
    #
    if (common.logical)
      common <- common[o, ]
    if (random.logical)
      random <- random[o, ]
    #
    if (!is.null(subset))
      subset <- subset[o]
  }
  #
  if (!is.null(subset)) {
    if (common.logical)
      common <- common[subset, ]
    if (random.logical)
      random <- random[subset, ]
  }
  
  
  if (common.logical | random.logical) {
    if (x$method == "SIDDE")
      cat("Separate indirect from direct design evidence (SIDDE)\n\n")
    else
      cat("Separate indirect from direct evidence (SIDE)",
          "using back-calculation method\n\n")
  }
  else
    legend <- FALSE
  
  
  if (common.logical) {
    cat("Common effects model: \n\n")
    common[is.na(common)] <- text.NA
    trts <- unique(sort(unlist(compsplit(common$comparison, x$sep.trts))))
    common$comparison <- comps(common$comparison, trts, x$sep.trts, nchar.trts)
    prmatrix(common, quote = FALSE, right = TRUE,
             rowlab = rep("", dim(common)[1]))
    if (random.logical)
      cat("\n")
  }
  ##
  if (random.logical) {
    cat("Random effects model: \n\n")
    random[is.na(random)] <- text.NA
    trts <- unique(sort(unlist(compsplit(random$comparison, x$sep.trts))))
    random$comparison <- comps(random$comparison, trts, x$sep.trts, nchar.trts)
    prmatrix(random, quote = FALSE, right = TRUE,
             rowlab = rep("", dim(random)[1]))
  }
  ##
  if (legend) {
    cat("\nLegend:\n")
    cat(" comparison - Treatment comparison\n")
    cat(" k", if (indent) "          " else " ",
        "- Number of studies providing direct evidence\n",
        sep = "")
    if (!noprop)
      cat(" prop", if (indent) "       " else " ",
          "- Direct evidence proportion\n",
          sep = "")
    if (overall)
      cat(" nma", if (indent) "        " else " ",
          "- Estimated treatment effect ", sm.lab,
          "in network meta-analysis\n",
          sep = "")
    cat(" direct", if (indent) "     " else " ",
        "- Estimated treatment effect ", sm.lab,
        "derived from direct evidence\n",
        sep = "")
    cat(" indir.", if (indent) "     " else " ",
        "- Estimated treatment effect ", sm.lab,
        "derived from indirect evidence\n",
        sep = "")
    if (test) {
      if (backtransf & relative)
        cat(" RoR", if (indent) "        " else " ",
            "- Ratio of Ratios ",
            "(direct versus indirect)\n",
            sep = "")
      else
        cat(" Diff", if (indent) "       " else " ",
            "- Difference between direct and ",
            "indirect treatment estimates\n",
            sep = "")
      ##
      cat(" z", if (indent) "          " else " ",
          "- z-value of test for disagreement (direct versus indirect)\n",
          sep = "")
      cat(" p-value", if (indent) "    " else " ",
          "- p-value of test for disagreement (direct versus indirect)\n",
          sep = "")
    }
    ##
    ## Add legend with abbreviated treatment labels
    ##
    legendabbr(trts, treats(trts, nchar.trts), TRUE, header = "\n")
  }
  
  
  invisible(NULL)
}

#' Table with network meta-analysis results
#' 
#' @description
#' Construct a table with network, direct and indirect estimates from
#' one or more network meta-analyses.
#' 
#' @aliases nettable print.nettable
#' 
#' @param \dots Any number of network meta-analysis objects or a
#'   single list with network meta-analyses.
#' @param x An object of class \code{nettable}.
#' @param name An optional character vector providing descriptive
#'   names for network meta-analysis objects.
#' @param method A character string indicating which method to split
#'   direct and indirect evidence is to be used. Either
#'   \code{"Back-calculation"} or \code{"SIDDE"}, can be abbreviated.
#'   See Details.
#' @param order A optional character or numerical vector specifying
#'   the order of treatments in comparisons.
#' @param common A logical indicating whether table for the common
#'   effects network meta-analysis should be printed.
#' @param random A logical indicating whether table for the random
#'   effects network meta-analysis should be printed.
#' @param upper A logical indicating whether treatment comparisons
#'   should be selected from the lower or upper triangle of the
#'   treatment effect matrices (see list elements \code{TE.common} and
#'   \code{TE.random} in the \code{netmeta} object). Ignored if
#'   argument \code{order} is provided.
#' @param reference.group Reference treatment. Ignored if argument
#'   \code{order} is provided.
#' @param baseline.reference A logical indicating whether results
#'   should be expressed as comparisons of other treatments versus the
#'   reference treatment or vice versa. This argument is only
#'   considered if \code{reference.group} is not equal to \code{""}
#'   and argument\code{order} is not provided.
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. For example, if \code{backtransf =
#'   TRUE}, results for \code{sm = "OR"} are printed as odds ratios
#'   rather than log odds ratios.
#' @param nchar.trts A numeric defining the minimum number of
#'   characters used to create unique treatment names.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   statistic, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of test of agreement between direct and indirect evidence, see
#'   \code{print.default}.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param big.mark A character used as thousands separator.
#' @param text.NA A character string specifying text printed for
#'   missing values.
#' @param bracket A character with bracket symbol to print lower
#'   confidence interval: "[", "(", "\{", "".
#' @param separator A character string with information on separator
#'   between lower and upper confidence interval.
#' @param lower.blank A logical indicating whether blanks between left
#'   bracket and lower confidence limit should be printed.
#' @param upper.blank A logical indicating whether blanks between
#'   separator and upper confidence limit should be printed.
#' @param tol.direct A numeric defining the maximum deviation of the
#'   direct evidence proportion from 0 or 1 to classify a comparison
#'   as providing only indirect or direct evidence, respectively.
#' @param writexl A logical indicating whether an Excel file should be
#'   created (R package \bold{writexl} must be available).
#' @param path A character string specifying the filename of the Excel
#'   file.
#' @param overwrite A logical indicating whether an existing Excel
#'   file should be overwritten.
#' @param legend A logical indicating whether a legend should be
#'   printed for abbreviated treatment names.
#' @param warn A logical indicating whether warnings should be
#'   printed.
#' @param verbose A logical indicating whether progress information
#'   should be printed.
#' 
#' @details
#' Construct a table with network, direct and indirect estimates from
#' one or more network meta-analyses. The table looks very similar to
#' the statistical part of a GRADE table for a network meta-analysis
#' (Puhan et al., 2014).
#'
#' By default, an R object with the network tables is
#' generated. Alternatively, an Excel file is created if argument
#' \code{writexl = TRUE}.
#' 
#' Two methods to derive indirect estimates are available:
#' \itemize{
#' \item Separate Indirect from Direct Evidence (SIDE) using a
#'   back-calculation method (\code{method = "Back-calculation"})
#'   based on the \emph{direct evidence proportion} to calculate the
#'   indirect evidence (König et al., 2013);
#' \item Separate Indirect from Direct Design Evidence (SIDDE) as
#'   described in Efthimiou et al. (2019).
#' }
#' 
#' Note, for the back-calculation method, indirect treatment estimates
#' are already calculated in \code{\link{netmeta}} and this function
#' combines and prints these estimates in a user-friendly
#' way. Furthermore, this method is not available for the
#' Mantel-Haenszel and non-central hypergeometric distribution
#' approach implemented in \code{\link{netmetabin}}.
#' 
#' For the random-effects model, the direct treatment estimates are
#' based on the common between-study variance \eqn{\tau^2} from the
#' network meta-analysis, i.e. the square of list element
#' \code{x$tau}.
#'
#' The SIDDE approach can be compute-intensive in large
#' networks. Crude information on the computation progress is printed
#' for SIDDE if argument \code{verbose} is \code{TRUE}.
#'
#' @return
#' An object of class \code{nettable} with corresponding \code{print}
#' function if argument \code{writexl = FALSE}. The object is a list
#' containing the network tables in list elements 'common' and
#' 'random'. An Excel file is created if \code{writexl = TRUE}. In
#' this case, \code{NULL} is returned in R.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netsplit}}, \code{\link{netmeta}},
#'   \code{\link{netmetabin}}, \code{\link{netmeasures}},
#'   \code{\link[metabook]{Woods2010}}
#' 
#' @references
#' Dias S, Welton NJ, Caldwell DM, Ades AE (2010):
#' Checking consistency in mixed treatment comparison meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{29}, 932--44
#' 
#' Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
#' (2019):
#' A Mantel-Haenszel model for network meta-analysis of rare events.
#' \emph{Statistics in Medicine},
#' \bold{38}, 2992--3012
#' 
#' König J, Krahn U, Binder H (2013):
#' Visualizing the flow of evidence in network meta-analysis and
#' characterizing mixed treatment comparisons.
#' \emph{Statistics in Medicine},
#' \bold{32}, 5414--29
#' 
#' Puhan MA, Schünemann HJ, Murad MH, et al. (2014):
#' A GRADE working group approach for rating the quality of treatment
#' effect estimates from network meta-analysis.
#' \emph{British Medical Journal},
#' \bold{349}, g5630
#' 
#' @examples
#' pw <- pairwise(treatment, event = r, n = N,
#'   studlab = author, data = Woods2010, sm = "OR")
#' #
#' nma <- netmeta(pw)
#' #
#' nt <- nettable(nma, digits = 2)
#' nt
#' 
#' \donttest{
#' print(nt, common = FALSE)
#' print(nt, random = FALSE)
#' 
#' # Create a CSV file with network table from random effects model
#' #
#' table <- nettable(nma, digits = 2, bracket = "(", separator = " to ")
#' #
#' write.table(table$random, file = "table-random.csv",
#'   row.names = FALSE, col.names = TRUE, sep = ",")
#' #
#' # Create Excel files with network tables
#' # (if R package writexl is available)
#' #
#' if (requireNamespace("writexl", quietly = TRUE))
#'   nettable(nma, digits = 2, bracket = "(", separator = " to ",
#'            path = tempfile(fileext = ".xlsx"))
#' }
#' 
#' @rdname nettable
#' @export nettable

nettable <- function(...,
                     name = NULL,
                     method = NULL,
                     ##
                     order = NULL, common, random,
                     ##
                     upper = TRUE,
                     reference.group = NULL,
                     baseline.reference = NULL,
                     ##
                     backtransf = NULL,
                     ##
                     nchar.trts = if (writexl) 666 else NULL,
                     ##
                     digits = gs("digits"),
                     digits.I2 = gs("digits.I2"),
                     digits.pval = gs("digits.pval"),
                     ##
                     scientific.pval = gs("scientific.pval"),
                     zero.pval = gs("zero.pval"),
                     JAMA.pval = gs("JAMA.pval"),
                     ##
                     big.mark = gs("big.mark"),
                     text.NA = ".",
                     ##
                     bracket = gs("CIbracket"),
                     separator = gs("CIseparator"),
                     lower.blank = gs("CIlower.blank"),
                     upper.blank = gs("CIupper.blank"),
                     ##
                     tol.direct = 0.0005,
                     ##
                     writexl = !missing(path),
                     path = "nettable.xlsx",
                     overwrite = FALSE,
                     ##
                     warn = FALSE,
                     verbose = FALSE) {
  
  
  ##
  ##
  ## (1) Extract list elements and basic checks
  ##
  ##
  is.nma <- function(x)
    inherits(x, "netmeta")
  ##
  missing.common <- missing(common)
  missing.random <- missing(random)
  ##
  args <- list(...)
  ##
  n.netmeta <- length(args)
  n.i <- seq_len(n.netmeta)
  ##
  if (length(args) == 1) {
    if (!is.list(args[[1]]))
      stop("All elements of argument '...' must be of class 'netmeta'.",
           call. = FALSE)
    ##
    if (!is.nma(args[[1]])) {
      n.netmeta <- length(args[[1]])
      n.i <- seq_len(n.netmeta)
      ##
      args2 <- list()
      fix <- ran <- rep_len(NA, n.netmeta)
      for (i in n.i) {
        args2[[i]] <- args[[1]][[i]]
        fix[i] <- args[[1]][[i]]$common
        ran[i] <- args[[1]][[i]]$random
      }
      if (missing.common)
        common <- any(fix, na.rm = TRUE)
      if (missing.random)
        random <- any(ran, na.rm = TRUE)
      ##
      args <- args2
    }
    else {
      if (missing.common)
        common <- args[[1]]$common
      if (is.null(common))
        common <- args[[1]]$fixed
      if (is.null(common))
        common <- args[[1]]$comb.fixed
      if (missing.random)
        random <- args[[1]]$random
      if (is.null(random))
        random <- args[[1]]$comb.random
    }
  }
  ##
  for (i in n.i) {
    if (!is.nma(args[[i]]))
      stop("All elements of argument '...' must be of class 'netmeta'.",
           call. = FALSE)
    ##
    args[[i]] <- updateversion(args[[i]])
  }
  ##
  levs <- numeric(0)
  for (i in n.i)
    levs[i] <- args[[i]]$level.ma
  ##
  if (length(unique(levs)) != 1)
    stop("Different confidence levels used in network meta-analyses ",
         "(see list element 'level.ma').",
         call. = FALSE)
  ##
  if (n.netmeta > 1 & (missing.common | missing.random)) {
    fix <- ran <- rep_len(NA, n.netmeta)
    for (i in n.i) {
      if (is.nma(args[[i]])) {
        fix[i] <- args[[i]]$common
        ran[i] <- args[[i]]$random
      }
    }
    if (missing.common)
      common <- any(fix, na.rm = TRUE)
    if (missing.random)
      random <- any(ran, na.rm = TRUE)
  }
  ##
  sms <- character(0)
  for (i in n.i)
    sms[i] <- args[[i]]$sm
  ##
  if (length(unique(sms)) == 1)
    sms <- unique(sms)
  ##
  backtransfs <- logical(0)
  for (i in n.i)
    backtransfs[i] <- args[[i]]$backtransf
  ##
  if (length(unique(backtransfs)) == 1)
    backtransfs <- unique(backtransfs)
  ##
  if (is.null(nchar.trts)) {
    nchar.trts <- vector("numeric", n.netmeta)
    for (i in n.i)
      nchar.trts[i] <- args[[i]]$nchar.trts
    ##
    if (length(unique(nchar.trts)) == 1)
      nchar.trts <- unique(nchar.trts)
    else
      nchar.trts <- min(nchar.trts, na.rm = TRUE)
  }
  chknumeric(nchar.trts, min = 1, length = 1)
  
  
  ##
  ##
  ## (2) Check other arguments
  ##
  ##
  if (!is.null(name))
    chklength(name, n.netmeta,
              paste("Argument 'name' must be of same length as",
                    "number of network meta-analyses"))
  else if (n.netmeta > 1)
    name <- paste("Outcome", n.i)
  ##
  if (!is.null(name))
    for (i in n.i)
      args[[i]]$outcome.name <- name[i]
  ##
  if (!missing(method))
    method <- setchar(method, c("Back-calculation", "SIDDE"))
  ##
  chklogical(common)
  chklogical(random)
  ##
  chklogical(upper)
  if (!is.null(baseline.reference))
    chklogical(baseline.reference)
  ##
  chknumeric(tol.direct, min = 0, length = 1)
  if (!is.null(backtransf))
    chklogical(backtransf)
  ##
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  ##
  chklogical(writexl)
  chkchar(path, length = 1)
  chklogical(overwrite)
  ##
  chklogical(warn)
  chklogical(verbose)
  ##
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  ##
  chkchar(text.NA)
  chkchar(big.mark)
  ##
  bracket.old <- gs("CIbracket")
  separator.old <- gs("CIseparator")
  lower.blank.old <- gs("CIlower.blank")
  upper.blank.old <- gs("CIupper.blank")
  ##
  cilayout(bracket, separator, lower.blank, upper.blank)
  on.exit(cilayout(bracket.old, separator.old,
                   lower.blank.old, upper.blank.old))
  
  
  ##
  ##
  ## (3) Generate network tables
  ##
  ##
  vars <- c(if (!is.null(name)) "Outcome",
            "Arm 1", "Arm 2", "k", "n", "I2",
            "Direct estimate", "Indirect estimate",
            "Network meta-analysis", "Incoherence")
  ##
  table.common <- table.random <-
    data.frame(matrix(nrow = 0, ncol = length(vars)))
  colnames(table.common) <- colnames(table.random) <- vars
  ##
  for (i in n.i) {
    table.i <- 
      nettable_internal(args[[i]], method,
                        upper, reference.group, baseline.reference,
                        order, tol.direct, backtransf,
                        digits, digits.I2, digits.pval,
                        scientific.pval, zero.pval, JAMA.pval,
                        big.mark, text.NA,
                        bracket, separator, lower.blank, upper.blank,
                        writexl,
                        warn, verbose)
    ##
    table.common <- rbind(table.common, table.i$common)
    table.random <- rbind(table.random, table.i$random)
  }
  ##
  if (all(is.na(table.common$n)))
    table.common$n <- NULL
  else
    table.common$n <- formatN(round(table.common$n), digits = 0,
                              text.NA = text.NA, big.mark = big.mark)
  ##
  if (all(is.na(table.random$n)))
    table.random$n <- NULL
  else
    table.random$n <- formatN(round(table.random$n), digits = 0,
                              text.NA = text.NA, big.mark = big.mark)
  ##
  trts <- c(table.common[["Arm 1"]], table.common[["Arm 2"]])
  ##
  if (!(all(nchar.trts > nchar(trts)))) {
    trts.abbr <- treats(trts, nchar.trts)
    ##
    table.common[["Arm 1"]] <-
      as.character(factor(table.common[["Arm 1"]],
                          levels = trts, labels = trts.abbr))
    table.common[["Arm 2"]] <-
      as.character(factor(table.common[["Arm 2"]],
                          levels = trts, labels = trts.abbr))
    ##
    table.random[["Arm 1"]] <-
      as.character(factor(table.random[["Arm 1"]],
                          levels = trts, labels = trts.abbr))
    table.random[["Arm 2"]] <-
      as.character(factor(table.random[["Arm 2"]],
                          levels = trts, labels = trts.abbr))
  }
  else
    trts.abbr <- trts

  
  ##
  ##
  ## (4) Save Excel file
  ##
  ##
  if (writexl) {
    if (!(common | random)) {
      warning("Excel file not generated as neither ",
              "argument 'common' nor 'random' is TRUE.")
      return(invisible(NULL))
    }
    ##
    if (!is_installed_package("writexl", stop = FALSE))
      stop(paste0("Package 'writexl' missing.",
                  "\n  ",
                  "Please use the following R command for installation:",
                  "\n  install.packages(\"writexl\")"),
           call. = FALSE)
    ##
    if (file.exists(path) & !overwrite)
      warning("File '", path, "' exists. ",
              "Use argument 'overwrite = TRUE' to overwrite file.",
              call. = FALSE)
    else {
      if (common & random)
        xlsx <- list(common = table.common, random = table.random)
      else if (common)
        xlsx <- list(common = table.common)
      else
        xlsx <- list(random = table.random)
      ##
      writexl::write_xlsx(xlsx, path = path, col_names = TRUE)
      message(paste0("Network table", if (common & random) "s",
                     " saved in file '", path, "'."))
    }
    ##
    return(invisible(NULL))
  }
  
  
  ##
  ##
  ## (5) Return network tables
  ##
  ##
  res <- list(common = table.common,
              random = table.random,
              ##
              upper = upper,
              reference.group = reference.group,
              baseline.reference = baseline.reference,
              order = order,
              ##
              tol.direct = tol.direct,
              ##
              digits = digits,
              digits.I2 = digits.I2,
              digits.pval = digits.pval,
              ##
              scientific.pval = scientific.pval,
              zero.pval = zero.pval,
              JAMA.pval = JAMA.pval,
              ##
              big.mark = big.mark,
              text.NA = text.NA,
              ##
              bracket = bracket,
              separator = gs("CIseparator"),
              lower.blank = separator,
              upper.blank = upper.blank,
              ##
              x = list(common = common, random = random),
              ##
              backtransf = backtransfs,
              sm = sms,
              level.ma = unique(levs),
              ##
              nchar.trts,
              trts = unique(trts),
              trts.abbr = unique(trts.abbr),
              ##
              version = packageDescription("netmeta")$Version
              )
  ##
  ## Backward compatibility
  ##
  res$fixed <- res$common
  res$x$fixed <- res$x$common
  ##
  class(res) <- "nettable"
  
  res
}


#' @rdname nettable
#' @method print nettable
#' @export

print.nettable <- function(x, common = x$x$common, random = x$x$random,
                           legend = gs("legend"), ...) {
  
  ##
  ##
  ## (1) Check arguments
  ##
  ##
  chkclass(x, "nettable")
  x <- updateversion(x)
  ##  
  ## All individual results in a single row - be on the save side:
  ##
  oldopts <- options(width = 200)
  on.exit(options(oldopts))
  ##
  args <- list(...)
  common <- deprecated(common, missing(common), args, "fixed", FALSE)
  chklogical(common)
  chklogical(random)
  chklogical(legend)
  
  
  ##
  ##
  ## (2) Print network table for common effects model
  ##
  ##
  if (common) {
    cat("Network table (", gs("text.w.common"), ") effects model):\n", sep = "")
    ##
    if (isCol(x$common, "Outcome")) {
      outcomes <- unique(x$common$Outcome)
      n.netmeta <- length(outcomes)
      backtransf <- x$backtransf
      if (length(backtransf) != n.netmeta)
        backtransf <- rep(backtransf, n.netmeta)
      for (i in seq_len(n.netmeta)) {
        mat.i <- x$common[x$common$Outcome == outcomes[i],
                          names(x$common) != "Outcome"]
        outcome.txt <- paste0("\nOutcome: ", outcomes[i])
        if (n.netmeta > 1 & length(x$sm) != 1)
          outcome.txt <-
            paste0(outcome.txt," (sm = '",
                   if (is_relative_effect(x$sm[i]) & !backtransf[i]) "log",
                   x$sm[i], "')")
        cat(outcome.txt, "\n", sep = "")
        prmatrix(mat.i,
                 quote = FALSE, right = TRUE,
                 rowlab = rep("", nrow(mat.i)), ...)
      }
    }
    else {
      cat("\n")
      prmatrix(x$common, quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(x$common)), ...)
    }
    if (random)
      cat("\n")
  }
  
  
  ##
  ##
  ## (3) Print network table for random effects model
  ##
  ##
  if (random) {
    cat("Network table (", gs("text.w.random"), ") effects model):\n", sep = "")
    ##
    if (isCol(x$random, "Outcome")) {
      outcomes <- unique(x$random$Outcome)
      n.netmeta <- length(outcomes)
      backtransf <- x$backtransf
      if (length(backtransf) != n.netmeta)
        backtransf <- rep(backtransf, n.netmeta)
      for (i in seq_len(n.netmeta)) {
        mat.i <- x$random[x$random$Outcome == outcomes[i],
                          names(x$random) != "Outcome"]
        outcome.txt <- paste0("\nOutcome: ", outcomes[i])
        if (n.netmeta > 1 & length(x$sm) != 1)
          outcome.txt <-
            paste0(outcome.txt," (sm = '",
                   if (is_relative_effect(x$sm[i]) & !backtransf[i]) "log",
                   x$sm[i], "')")
        cat(outcome.txt, "\n", sep = "")
        prmatrix(mat.i,
                 quote = FALSE, right = TRUE,
                 rowlab = rep("", nrow(mat.i)), ...)
      }
    }
    else {
      cat("\n")
      prmatrix(x$random, quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(x$random)), ...)
    }
  }
  
  
  ##
  ##
  ## (4) Legend
  ##
  ##
  if (!is.null(x$trts) && !is.null(x$trts.abbr))
    legendabbr(x$trts, x$trts.abbr, legend)
  
  invisible(NULL)
}

#' Summary method for objects of class netconnection
#' 
#' @description
#' Summary method for objects of class \code{netconnection} to print
#' list of studies in subnetworks.
#' 
#' @param object An object of class \code{netconnection}.
#' @param x An object of class \code{summary.netconnection}.
#' @param \dots Additional arguments (passed on to
#'   \code{\link{print.netconnection}}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' @seealso \code{\link{netconnection}}
#' 
#' @examples
#' # Artificial example with two subnetworks
#' #
#' t1 <- c("G", "B", "B", "D", "A", "F")
#' t2 <- c("B", "C", "E", "E", "H", "A")
#' #
#' nc2 <- netconnection(t1, t2)
#' print(nc2, details = TRUE)
#' 
#' summary(nc2)
#'
#' @method summary netconnection 
#' @export

summary.netconnection <- function(object, ...) {
  
  chkclass(object, "netconnection")
  
  res <- object
  #
  class(res) <- c("summary.netconnection", class(res))
  #
  res
}


#' @rdname summary.netconnection
#' @method print summary.netconnection 
#' @export

print.summary.netconnection <- function(x, ...) {
  
  chkclass(x, "summary.netconnection")
  #
  class(x) <- "netconnection"
  print(x, ...)
  #
  for (i in seq_len(x$n.subnets)) {
    cat(paste0("\n*** Subnet ", i, " ***\n"))
    #
    cat("\nTreatments:\n ")
    cat(paste(paste0("'", sort(unique(c(x$treat1[x$subnet == i],
                                        x$treat2[x$subnet == i]))), "'"),
              collapse = ", "))
    #
    cat("\nStudies:\n ")
    cat(paste(paste0("'", sort(unique(x$studlab[x$subnet == i])), "'"),
              collapse = ", "))
    cat("\n")
  }
  #
  invisible(NULL)
}

#' Partial order of treatments in network meta-analysis
#' 
#' @description
#' Partial order of treatments in network meta-analysis. The set of
#' treatments in a network is called a partially ordered set (in
#' short, a \emph{poset}), if different outcomes provide different
#' treatment ranking lists.
#' 
#' @aliases netposet print.netposet
#' 
#' @param \dots See details.
#' @param outcomes A character vector with outcome names.
#' @param treatments A character vector with treatment names.
#' @param small.values See details.
#' @param common A logical indicating whether to show results for the
#'   common effects model.
#' @param random A logical indicating whether to show results for the
#'   random effects model.
#' @param x An object of class \code{netposet}.
#' @param pooled A character string indicating whether Hasse diagram
#'   should be drawn for common (\code{"common"}) or random effects
#'   model (\code{"random"}). Can be abbreviated.
#' @param fixed Ignored deprecated argument (replaced by
#'   \code{common}).
#' @param comb.fixed Ignored deprecated argument (replaced by
#'   \code{common}).
#' @param comb.random Ignored deprecated argument (replaced by
#'   \code{random}).
#' 
#' @details
#' In network meta-analysis, frequently different outcomes are
#' considered which may each provide a different ordering of
#' treatments. The concept of a partially ordered set (in short, a
#' \emph{poset}, Carlsen & Bruggemann, 2014) of treatments can be used
#' to gain further insights in situations with apparently conflicting
#' orderings. This implementation for rankings in network meta-analyis
#' is described in Rücker & Schwarzer (2017).
#' 
#' In function \code{netposet}, argument \code{\dots{}} can be any of the
#' following:
#' \itemize{
#' \item arbitrary number of \code{netrank} objects providing
#'   P-scores;
#' \item arbitrary number of \code{netmeta} objects;
#' \item single ranking matrix with each column providing P-scores
#'   (Rücker & Schwarzer 2015) or SUCRA values (Salanti et al. 2011)
#'   for an outcome and rows corresponding to treatments.
#' }
#' Note, albeit in general a ranking matrix is not constrained to have
#' values between 0 and 1, \code{netposet} stops with an error in this
#' case as this function expects a matrix with P-scores or SUCRA
#' values.
#' 
#' Argument \code{outcomes} can be used to label outcomes. If argument
#' \code{outcomes} is missing,
#' \itemize{
#' \item column names of the ranking matrix are used as outcome labels
#'   (if first argument is a ranking matrix and column names are
#'   available);
#' \item capital letters 'A', 'B', \dots{} are used as outcome labels
#'   and a corresponding warning is printed.
#' }
#'
#' Argument \code{treatments} can be used to provide treatment labels
#' if the first argument is a ranking matrix. If argument
#' \code{treatment} is missing,
#' \itemize{
#' \item row names of the ranking matrix are used as treatment labels
#'   (if available);
#' \item letters 'a', 'b', \dots{} are used as treatment labels and a
#'   corresponding warning is printed.
#' }
#' If argument \code{\dots{}} consists of \code{netmeta} objects,
#' \code{netrank} is called internally to calculate P-scores. In this
#' case, argument \code{small.values} can be used to specify for each
#' outcome whether small values are beneficial (\code{"desirable"}) or
#' harmfull (\code{"undesirable"}); see \code{\link{netrank}}. This
#' argument is ignored for a ranking matrix and \code{netrank}
#' objects.
#' 
#' Arguments \code{common} and \code{random} can be used to define
#' whether results should be printed and plotted for common and random
#' effects model. If netmeta and netrank objects are provided in
#' argument \code{\dots{}}, values for \code{common} and \code{random}
#' within these objects are considered; if these values are not
#' unique, argument \code{common} or \code{random} are set to
#' \code{TRUE}.
#' 
#' In function \code{print.netposet}, argument \code{\dots{}} is
#' passed on to the printing function.
#'
#' @return
#' An object of class \code{netposet} with corresponding \code{print},
#' \code{plot}, and \code{hasse} functions. The object is a list
#' containing the following components:
#' \item{P.common}{Ranking matrix with rows corresponding to treatments
#'   and columns corresponding to outcomes (common effects model).}
#' \item{M0.common}{Hasse matrix skipping unnecessary paths (common
#'   effects model).}
#' \item{M.common}{"Full" Hasse matrix (common effects model).}
#' \item{O.common}{Matrix with information about partial ordering
#'   (common effects model).}
#' \item{P.random}{Ranking matrix with rows corresponding to
#'   treatments and columns corresponding to outcomes (random effects
#'   model).}
#' \item{M0.random}{Hasse matrix skipping unnecessary paths (random
#'   effects model).}
#' \item{M.random}{"Full" Hasse matrix (random effects model).}
#' \item{O.random}{Matrix with information about partial ordering
#'   (random effects model).}
#' \item{small.values, common, random}{As.defined above.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package netmeta used to create object.}
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netrank}},
#'   \code{\link{plot.netrank}}, \code{\link{hasse}},
#'   \code{\link{plot.netposet}}, \code{\link[metabook]{Linde2015}}
#' 
#' @references
#' Carlsen L, Bruggemann R (2014):
#' Partial order methodology: a valuable tool in chemometrics.
#' \emph{Journal of Chemometrics},
#' \bold{28}, 226--34
#' 
#' Rücker G, Schwarzer G (2015):
#' Ranking treatments in frequentist network meta-analysis works
#' without resampling methods.
#' \emph{BMC Medical Research Methodology},
#' \bold{15}, 58
#' 
#' Rücker G, Schwarzer G (2017):
#' Resolve conflicting rankings of outcomes in network meta-analysis:
#' Partial ordering of treatments.
#' \emph{Research Synthesis Methods},
#' \bold{8}, 526--36
#' 
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#' 
#' @examples
#' \donttest{
#' # Define order of treatments in depression data set Linde2015
#' #
#' trts <- c("TCA", "SSRI", "SNRI", "NRI",
#'   "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")
#' 
#' # Outcome labels
#' #
#' outcomes <- c("Early response", "Early remission")
#' 
#' # (1) Early response
#' #
#' pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma1 <- netmeta(pw1, common = FALSE,
#'   seq = trts, ref = "Placebo", small.values = "undesirable")
#' 
#' # (2) Early remission
#' #
#' pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma2 <- netmeta(pw2, common = FALSE,
#'   seq = trts, ref = "Placebo", small.values = "undesirable")
#' 
#' # Partial order of treatment rankings (two outcomes)
#' #
#' po12 <- netposet(netrank(nma1), netrank(nma2), outcomes = outcomes)
#' 
#' # Scatter plot
#' #
#' plot(po12)
#' 
#' # Biplot (same scatter plot as only two outcomes considered)
#' #
#' plot(po12, "biplot")
#' 
#' # Hasse diagram for outcomes early response and early remission
#' #
#' \dontrun{
#' if (requireNamespace("Rgraphviz", quietly = TRUE))
#'   hasse(po12)
#' }
#' 
#' # Scatter plot for outcomes early response and early remission
#' #
#' oldpar <- par(pty = "s")
#' plot(po12)
#' par(oldpar)
#'
#' 
#' # Consider five outcomes
#' #
#' # Outcome labels
#' #
#' outcomes <- c("Early response", "Early remission",
#'   "Lost to follow-up", "Lost to follow-up due to AEs",
#'    "Adverse events (AEs)")
#' 
#' # (3) Loss to follow-up
#' #
#' pw3 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(loss1, loss2, loss3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma3 <- netmeta(pw3, common = FALSE,
#'   seq = trts, ref = "Placebo", small.values = "desirable")
#' 
#' # (4) Loss to follow-up due to adverse events
#' #
#' pw4 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(loss.ae1, loss.ae2, loss.ae3), n = list(n1, n2, n3),
#'   studlab = id, data = subset(Linde2015, id != 55), sm = "OR")
#' #
#' nma4 <- netmeta(pw4, common = FALSE,
#'   seq = trts, ref = "Placebo", small.values = "desirable")
#' 
#' # (5) Adverse events
#' #
#' pw5 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(ae1, ae2, ae3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' #
#' nma5 <- netmeta(pw5, common = FALSE,
#'   seq = trts, ref = "Placebo", small.values = "desirable")
#' 
#' # Partial order of treatment rankings (based on netrank() objects)
#' #
#' po.ranks <- netposet(netrank(nma1), netrank(nma2),
#'   netrank(nma3), netrank(nma4), netrank(nma5), outcomes = outcomes)
#' 
#' # Same result (based on netmeta() objects)
#' #
#' po.nets <- netposet(nma1, nma2, nma3, nma4, nma5,
#'   outcomes = outcomes)
#' #
#' all.equal(po.ranks, po.nets)
#' 
#' # Print matrix with P-scores (random effects model)
#' #
#' po.nets$P.random
#' 
#' \dontrun{
#' # Hasse diagram for all outcomes (random effects model)
#' #
#' if (requireNamespace("Rgraphviz", quietly = TRUE))
#'   hasse(po.ranks)
#' }
#' }
#' 
#' # Example using ranking matrix with P-scores
#' #
#' # Ribassin-Majed L, Marguet S, Lee A.W., et al. (2017):
#' # What is the best treatment of locally advanced nasopharyngeal
#' # carcinoma? An individual patient data network meta-analysis.
#' # Journal of Clinical Oncology, 35, 498-505
#' #
#' outcomes <- c("OS", "PFS", "LC", "DC")
#' treatments <- c("RT", "IC-RT", "IC-CRT", "CRT",
#'   "CRT-AC", "RT-AC", "IC-RT-AC")
#' #
#' # P-scores (from Table 1)
#' #
#' pscore.os  <- c(15, 33, 63, 70, 96, 28, 45) / 100
#' pscore.pfs <- c( 4, 46, 79, 52, 94, 36, 39) / 100
#' pscore.lc  <- c( 9, 27, 47, 37, 82, 58, 90) / 100
#' pscore.dc  <- c(16, 76, 95, 48, 72, 32, 10) / 100
#' #
#' pscore.matrix <- data.frame(pscore.os, pscore.pfs, pscore.lc, pscore.dc)
#' rownames(pscore.matrix) <- treatments
#' colnames(pscore.matrix) <- outcomes
#' pscore.matrix
#' #
#' po <- netposet(pscore.matrix)
#' po12 <- netposet(pscore.matrix[, 1:2])
#' po
#' po12
#' #
#' \dontrun{
#' if (requireNamespace("Rgraphviz", quietly = TRUE)) {
#'   hasse(po)
#'   hasse(po12)
#' }
#' }
#' #
#' oldpar <- par(pty = "s")
#' plot(po12)
#' par(oldpar)
#' 
#' @rdname netposet
#' @export netposet

netposet <- function(..., outcomes, treatments, small.values,
                     common, random, fixed, comb.fixed, comb.random) {
  
  
  args <- list(...)
  
  
  if (!missing(common))
    chklogical(common)
  if (!missing(random))
    chklogical(random)
  ##
  missing.small.values <- missing(small.values)
  
  
  any.netmeta <- any.netrank <- FALSE
  
  
  ##
  ## First argument is expected to be a ranking matrix if only a
  ## single argument is provided
  ##
  if (length(args) == 1) {
    ##
    ## (1) Check arguments
    ##
    if (!(is.matrix(args[[1]]) | is.data.frame(args[[1]])))
      stop("First argument must be a matrix or ",
           "data frame if only a single argument is provided.",
           call. = FALSE)
    ##
    if (!missing.small.values)
      warning("Argument 'small.values' is ignored as first argument ",
              "is a ranking matrix.",
              call. = FALSE)
    ##
    ranking.matrix <- args[[1]]
    ##
    if (any(ranking.matrix[!is.na(ranking.matrix)] > 1) |
        any(ranking.matrix[!is.na(ranking.matrix)] < 0))
      stop("All elements of ranking matrix must be between 0 and 1.",
           call. = FALSE)
    ##
    n.outcomes <- ncol(ranking.matrix)
    n.treatments <- nrow(ranking.matrix)
    ##
    if (n.outcomes == 1)
      stop("Minimum number of two outcomes ",
           "(equal to number of columns of ranking matrix).",
           call. = FALSE)
    ##
    if (!missing(outcomes)) {
      if (length(outcomes) != n.outcomes)
        stop("Number of outcomes provided by argument 'outcomes' ",
             "is different from number of columns of ranking matrix.",
             call. = FALSE)
    }
    else
      outcomes <- colnames(ranking.matrix)
    ##
    if (!missing(treatments)) {
      if (length(treatments) != n.treatments)
        stop("Number of treatments provided by argument 'treatments' ",
             "is different from number of rows of ranking matrix.",
             call. = FALSE)
    }
    else
      treatments <- rownames(ranking.matrix)
    ##
    ## (2) P-Score matrices
    ##
    if (is.null(outcomes)) {
      warning("Outcomes are labelled 'A' to '", LETTERS[n.outcomes],
              "' as argument 'outcomes' is missing and column names are NULL.",
              call. = FALSE)
      outcomes <- LETTERS[1:n.outcomes]
    }
    ##
    if (is.null(treatments)) {
      warning("Treatments are labelled 'a' to '", letters[n.treatments],
              "' as row names are NULL.",
              call. = FALSE)
      treatments <- letters[1:n.treatments]
    }
    ##
    rownames(ranking.matrix) <- treatments
    colnames(ranking.matrix) <- outcomes
    ##
    ranking.matrix.common <- ranking.matrix
    ranking.matrix.random <- ranking.matrix
    ##
    if (missing(common))
      common <- FALSE
    if (missing(random))
      random <- FALSE
  }
  ##
  ## More than one element provided in '...' (netmeta or netrank objects)
  ##
  else {
    ##
    ## (1) Check arguments
    ##
    n.outcomes <- length(args)
    ##
    if (!missing(treatments))
      warning("Argument 'treatments' ignored (only needed if ",
              "first argument is a ranking matrix).",
              call. = FALSE)
    ##
    if (missing(outcomes)) {
      warning("Outcomes are labelled 'A' to '", LETTERS[n.outcomes],
              "' as argument 'outcomes' is missing.",
              call. = FALSE)
      outcomes <- LETTERS[1:n.outcomes]  
    }
    ##
    if (length(outcomes) != n.outcomes)
      stop("Number of outcomes provided by argument 'outcomes' ",
           "and number of rankings differ.",
           call. = FALSE)
    ##
    if (!missing.small.values) {
      if (length(small.values) != n.outcomes)
        stop("Number of values provided by argument 'small.values' ",
             "and number of rankings differ.",
             call. = FALSE)
    }
    else
      small.values <- rep("", n.outcomes)
    ##
    for (i in seq_along(args)) {
      if (inherits(args[[i]], "netmeta"))
        any.netmeta <- TRUE
      else if (inherits(args[[i]], "netrank"))
        any.netrank <- TRUE
      else
        stop("All elements of argument '...' must be of class ",
             "'netmeta' or 'netrank'.",
             call. = FALSE)
      ##
      args[[i]] <- updateversion(args[[i]])
      ##
      if (!missing.small.values)
        small.values[i] <- setsv(small.values[i])
      else {
        small.values[i] <-
          ifelse(is.null(args[[i]]$small.values),
                 "desirable", args[[i]]$small.values)
      }
      ##
      if (length(small.values) != n.outcomes)
        stop("Length of argument 'small.values' must be equal to ",
             "number of outcomes.",
             call. = FALSE)
    }
    ##
    ## (2) Extract P-Scores
    ##
    ranking.list.common <- ranking.list.random <- list()
    commons <- randoms <- rep_len(NA, length(args))
    ##
    for (i in seq_along(args)) {
      ##
      args.i <- args[[i]]
      ##
      if (inherits(args.i, "netmeta")) {
        ranking.list.common[[i]] <- netrank(args.i,
                                            small.values =
                                              small.values[i])$ranking.common
        ranking.list.random[[i]] <- netrank(args.i,
                                            small.values =
                                              small.values[i])$ranking.random
        commons[i]  <- args.i$common
        randoms[i] <- args.i$random
      }
      else if (inherits(args.i, "netrank")) {
        ranking.list.common[[i]] <- args.i$ranking.common
        ranking.list.random[[i]] <- args.i$ranking.random
        commons[i]  <- args.i$x$common
        randoms[i] <- args.i$x$random
      }
    }
    
    
    ranking.treatments <- lapply(ranking.list.common, names)
    n.treatments <- unlist(lapply(ranking.treatments, length))
    ##
    if (length(unique(n.treatments)) != 1) {
      sel.max <- seq_along(n.treatments)[n.treatments == max(n.treatments)][1]
      ##
      treatments <- ranking.treatments[[sel.max]]
      ##
      ## Act on rankings with missing treatments
      ##
      for (j in seq_along(n.treatments)[n.treatments < max(n.treatments)]) {
        treatments.j <- ranking.treatments[[j]]
        missing.j <- !(treatments %in% treatments.j)
        ##
        if (any(treatments.j != treatments[!missing.j]))
          stop("Treatment names of all rankings must be in same order.",
               call. = FALSE)
        ##
        ranking.j <- ranking.list.common[[j]]
        ranking.list.common[[j]] <- ranking.list.common[[sel.max]]
        ranking.list.common[[j]][treatments[!missing.j]] <- ranking.j
        ranking.list.common[[j]][treatments[missing.j]]  <- NA
        ##
        ranking.j <- ranking.list.random[[j]]
        ranking.list.random[[j]] <- ranking.list.random[[sel.max]]
        ranking.list.random[[j]][treatments[!missing.j]] <- ranking.j
        ranking.list.random[[j]][treatments[missing.j]]  <- NA
        ##
        ranking.treatments[[j]] <- treatments
      }
    }
    else
      treatments <- ranking.treatments[[1]]
    ##
    for (i in seq_along(ranking.treatments))
      if (any(ranking.treatments[[i]] != treatments)) {
        if (all(sort(ranking.treatments[[i]]) == sort(treatments)))
          stop("Different order of treatments provided:\n ",
               paste(paste0("'", treatments, "'"), collapse = " - "),
               "\n ",
               paste(paste0("'", ranking.treatments[[i]], "'"),
                     collapse = " - "),
               call. = FALSE)
        else
          stop("Different treatments provided:\n ",
               paste(paste0("'", treatments, "'"), collapse = " - "),
               "\n ",
               paste(paste0("'", ranking.treatments[[i]], "'"),
                     collapse = " - "),
               call. = FALSE)
      }
    ##
    ranking.matrix.common <- matrix(unlist(ranking.list.common,
                                          use.names = FALSE),
                                   ncol = length(outcomes), byrow = FALSE)
    ranking.matrix.random <- matrix(unlist(ranking.list.random,
                                           use.names = FALSE),
                                    ncol = length(outcomes), byrow = FALSE)
    rownames(ranking.matrix.common) <- treatments
    rownames(ranking.matrix.random) <- treatments
    colnames(ranking.matrix.common) <- outcomes
    colnames(ranking.matrix.random) <- outcomes
    ##
    text.netmeta.netrank <- if (any.netmeta) "netmeta" else ""
    text.netmeta.netrank <- if (any.netmeta & any.netrank) "netmeta and netrank"
    text.netmeta.netrank <- if (!any.netmeta & any.netrank) "netrank"
    text.netmeta.netrank <- paste(text.netmeta.netrank, "objects.")
    ##
    if (missing(common)) {
      common <- unique(commons)
      if (length(common) != 1) {
        warning("Argument 'common' set to TRUE as different values are ",
                "available in ", text.netmeta.netrank,
                call. = FALSE)
        common <- TRUE
      }
    }
    ##
    if (missing(random)) {
      random <- unique(randoms)
      if (length(random) != 1) {
        warning("Argument 'random' set to TRUE as different values are ",
                "available in ", text.netmeta.netrank,
                call. = FALSE)
        random <- TRUE
      }
    }
  } 
  
  
  n <- nrow(ranking.matrix.common)
  o <- ncol(ranking.matrix.common)
  
  
  Pos.common <- M.common <- matrix(0, nrow = n, ncol = n)
  ##
  rownames(Pos.common) <- colnames(Pos.common) <-
    rownames(ranking.matrix.common)
  rownames(M.common) <- colnames(M.common) <- rownames(ranking.matrix.common)
  ##
  Pos.random <- M.random <- matrix(0, nrow = n, ncol = n)
  ##
  rownames(Pos.random) <- colnames(Pos.random) <-
    rownames(ranking.matrix.random)
  rownames(M.random) <- colnames(M.random) <- rownames(ranking.matrix.random)
  
  
  ## Pos[i, j] counts how many rankings judge treatment i superior to
  ## treatment j (number between 0 and o)
  ##
  for (i in 1:n)
    for (j in 1:n)
      if (i != j)
        for (k in 1:o)
          if (!is.na(ranking.matrix.common[i, k]) &
              !is.na(ranking.matrix.common[j, k]) &
              ranking.matrix.common[i, k] >= ranking.matrix.common[j, k])
            Pos.common[i, j] <- Pos.common[i, j] + 1
  ##
  for (i in 1:n)
    for (j in 1:n)
      if (i != j)
        for (k in 1:o)
          if (!is.na(ranking.matrix.random[i, k]) &
              !is.na(ranking.matrix.random[j, k]) &
              ranking.matrix.random[i, k] >= ranking.matrix.random[j, k])
            Pos.random[i, j] <- Pos.random[i, j] + 1
  
  
  ## Calculate "full" Hasse matrix M
  ##
  M.common[Pos.common == o] <- 1
  M.random[Pos.random == o] <- 1
  
  
  ## Matrix with information about partial ordering
  ##
  PO.common <- M.common[1:2, ]
  PO.common[1, ] <- rowSums(M.common)
  PO.common[2, ] <- colSums(M.common)
  rownames(PO.common) <- c("inferior", "superior")
  ##
  PO.random <- M.random[1:2, ]
  PO.random[1, ] <- rowSums(M.random)
  PO.random[2, ] <- colSums(M.random)
  rownames(PO.random) <- c("inferior", "superior")
  
  
  ## Skipping each direct path where a path of length 2 is found
  ##
  M0.common <- M.common - sign(M.common %*% M.common)
  ##
  M0.random <- M.random - sign(M.random %*% M.random)
  
  
  if (missing.small.values | !any.netmeta | length(args) == 1)
    small.values <- NULL
  
  
  res <- list(P.common = ranking.matrix.common,
              M0.common = M0.common,
              M.common = M.common,
              O.common = PO.common,
              P.random = ranking.matrix.random,
              M0.random = M0.random,
              M.random = M.random,
              O.random = PO.random,
              small.values = small.values,
              common = common,
              random = random,
              call = match.call(),
              version = packageDescription("netmeta")$Version)
  ##
  class(res) <- "netposet"
  
  res
}


#' @rdname netposet
#' @method print netposet
#' @export

print.netposet <- function(x,
                           pooled = ifelse(x$random, "random", "common"),
                           ...) {
  
  chkclass(x, "netposet")
  x <- updateversion(x)
  
  
  pooled <- setchar(pooled, c("common", "random", "fixed"))
  pooled[pooled == "fixed"] <- "common"
  
  if (pooled == "common") {
    if (!(!x$common & !x$random))
      cat("Common effects model\n")
    print(x$M0.common, ...)
    if (x$random)
      cat("\n")
  }
  ##
  else {
    cat("Random effects model\n")
    print(x$M0.random, ...)
  }
  
  ## diag(M0) <- NA
  ## M0[is.na(M0)] <- "."
  ## prmatrix(M0, quote = FALSE, right = TRUE)
  
  invisible(NULL)
}

#' Create a matrix with additional information for pairwise
#' comparisons
#' 
#' @description
#' Auxiliary function to create a matrix with additional information
#' for pairwise comparisons
#' 
#' @param x A \code{\link{netmeta}} object.
#' @param var Variable with additional information.
#' @param levels An optional vector of the values that \code{var}
#'   might have taken (see \code{\link{factor}}).
#' @param labels An optional vector with labels for \code{var} (see
#'   \code{\link{factor}}).
#' @param func A character string with the function name to summarize
#'   values within pairwise comparisons; see Details.
#' @param ties.method A character string describing how ties are
#'   handled if \code{func = "mode"}; see Details.
#' 
#' @details
#' For each pairwise comparison, unique values will be calculated for
#' the variable \code{var} based on the argument \code{func}: "mode"
#' (most common value), "min" (minimum value), "max", "mean",
#' "median", and "sum". In order to determine the most common value,
#' the argument \code{ties.method} can be used in the case of ties
#' with "first" meaning that the first / smallest value will be
#' selected; similar for "last" (last / largest value) and "random"
#' (random selection).
#'
#' @return
#' A matrix with the same row and column names as the adjacency matrix
#' \code{x$A.matrix}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
#' 
#' @examples
#' data(smokingcessation)
#' # Add variable with (fictitious) risk of bias values
#' # with 1 = "low risk" and 2 = "high risk"
#' #
#' smokingcessation$rob <- rep(1:2, 12)
#' 
#' pw <- pairwise(list(treat1, treat2, treat3),
#'   event = list(event1, event2, event3), n = list(n1, n2, n3),
#'   data = smokingcessation, sm = "OR")
#' nma <- netmeta(pw, common = FALSE, ref = "A")
#' 
#' # Generate network graph with information on risk of bias
#' #
#' col.rob <- netmatrix(nma, rob, ties.method = "last",
#'   levels = 1:2, labels = c("green", "yellow"))
#' #
#' netgraph(nma, col = col.rob, cex.points = 5,
#'   bg.points = "gray", adj = 0.5,
#'   pos.number.of.studies = c(0.5, 0.25, 0.5, 0.5, 0.25, 0.5))
#' 
#' netgraph(nma, col = col.rob, cex.points = n.trts,
#'   bg.points = "blue", labels = paste0(trts, " (n=", n.trts, ")"),
#'   pos.number.of.studies = c(0.5, 0.25, 0.5, 0.5, 0.25, 0.5),
#'   offset = c(0.05, 0.035, 0.05, 0.025))
#' 
#' @export netmatrix

netmatrix <- function(x, var, levels, labels = levels,
                      func = "mode", ties.method = "random") {
  
  
  chkclass(x, c("netmeta", "discomb"))
  ##
  func <- setchar(func,
                  c("mode", "min", "max", "mean", "median", "sum"))
  ties.method <- setchar(ties.method, c("first", "last", "random"))
  
  
  ##
  ## Catch var, treat1, treat2 from data:
  ##
  data <- x$data
  ##
  var <- catch("var", match.call(), data, sys.frame(sys.parent()))
  ##
  treat1 <- data[[".treat1"]]
  treat2 <- data[[".treat2"]]
  ##
  if (length(var) == 1 & length(treat1) > 1)
    var <- rep(var, length(treat1))
  ##
  subset <- data$.subset
  excl <- data$.excl
  drop <- data$.drop
  ##
  if (is.null(subset))
    subset <- rep(TRUE, nrow(data))
  if (is.null(excl))
    excl <- rep(FALSE, nrow(data))
  if (is.null(drop))
    drop <- rep(FALSE, nrow(data))
  ##
  treat1 <- treat1[subset & !excl & !drop]
  treat2 <- treat2[subset & !excl & !drop]
  var <- var[subset & !excl & !drop]
  
  
  dat <- bySummary(var, treat1, treat2,
                   ties.method = ties.method,
                   sep = x$sep.trts, long = FALSE)
  ##
  split <- strsplit(dat$indices, x$sep.trts)
  dat$idx1 <- unlist(lapply(split, first))
  dat$idx2 <- unlist(lapply(split, second))
  ##
  dat <- dat[, c("idx1", "idx2", func)]
  ##
  if (missing(levels))
    levels <- sort(unique(dat[[func]]))
  ##
  if (length(levels) != length(labels))
    stop("Different lengths of arguments 'levels' and 'labels'.")
  ##
  dat[[func]] <- as.character(factor(dat[[func]],
                                     levels = levels,
                                     labels = labels))
  ##
  if (is.numeric(labels))
    dat[[func]] <- as.numeric(dat[[func]])
  
  
  res <- x$A.matrix
  res[!is.na(res)] <- NA
  ##
  for (i in seq(along = dat$idx1)) {
    res[dat$idx1[i], dat$idx2[i]] <- dat[[func]][i]
    res[dat$idx2[i], dat$idx1[i]] <- dat[[func]][i]
  }
  ##
  attr(res, "version") <- packageDescription("netmeta")$Version
  ##
  res
}

sidde <- function(x,
                  sep.trts = " vs ", verbose = TRUE, warn = TRUE,
                  is.tictoc = FALSE) {
  
  chkclass(x, "netmeta")
  is.bin <- inherits(x, "netmetabin")
  
  if (is.null(x$data))
    stop("SIDDE method only available for network meta-analysis ",
         "objects created with argument 'keepdata' equal to TRUE.")
  ##
  if (verbose)
    cat("Start computations for SIDDE method\n")
  ##
  dat <- x$data
  dat <- dat[order(dat$.studlab, dat$.treat1, dat$.treat2), ]
  ##
  if (!is.null(dat$.subset))
    dat <- dat[dat$.subset, , drop = FALSE]
  ##
  if (!is.null(dat$.drop))
    dat <- dat[!dat$.drop, , drop = FALSE]
  ##
  ## Determine comparisons with direct evidence
  ##
  idx.d <- which(!is.na(x$TE.direct.common), arr.ind = TRUE)
  idx.d <- idx.d[idx.d[, 1] < idx.d[, 2], , drop = FALSE]
  ##
  rownames(idx.d) <- seq_len(nrow(idx.d))
  idx1 <- idx.d[, 1]
  idx2 <- idx.d[, 2]
  ##
  n.comps <- nrow(idx.d)
  ##
  trts <- x$trts
  ##
  ## Perform network meta-analyses for indirect evidence
  ## (by dropping one direct comparison at a time)
  ##
  TE.indirect.common <- x$TE.direct.common
  TE.indirect.common[!is.na(TE.indirect.common)] <- NA
  seTE.indirect.common <- TE.indirect.common
  ##
  TE.indirect.random <- x$TE.direct.random
  TE.indirect.random[!is.na(TE.indirect.random)] <- NA
  seTE.indirect.random <- TE.indirect.random
  ##
  if (is.tictoc) {
    tictoc <- rep(NA, n.comps)
    names.tictoc <- ""
  }
  ##
  for (i in seq_len(n.comps)) {
    ##
    idx1.i <- idx1[i]
    idx2.i <- idx2[i]
    ##
    if (is.tictoc)
      tictoc::tic()
    ##
    if (verbose)
      cat("- ",
          paste(trts[idx1.i], trts[idx2.i], sep = sep.trts),
          " (", i, "/", n.comps, ")\n",
          sep = "")
    ##
    ## Determine all pairwise comparisons of trts[idx1.i] vs trts[idx2.i]
    ##
    direct.i <-
      (dat$.treat1 == trts[idx1.i] & dat$.treat2 == trts[idx2.i]) |
      (dat$.treat2 == trts[idx1.i] & dat$.treat1 == trts[idx2.i])
    ##
    ## Determine all studies with pairwise comparison trts[idx1.i] vs
    ## trts[idx2.i]
    ##
    study.direct.i <- unique(dat$.studlab[direct.i])
    ##
    ## Drop all studies containing the comparison trts[idx1.i] vs
    ## trts[idx2.i], i.e., all comparisons of a multi-arm study
    ## containing [idx1.i] and trts[idx2.i] are dropped
    ##
    dat.i <- dat[!(dat$.studlab %in% study.direct.i), , drop = FALSE]
    ##
    dat.i$.design <- NULL
    ##
    if (nrow(dat.i) > 0)
      con <- netconnection(dat.i$.treat1, dat.i$.treat2, dat.i$.studlab)
    else
      con <- list(n.subnets = 0)
    ##
    if (con$n.subnets == 1) {
      ##
      if (is.bin)
        net.i <- netmetabin(dat.i$.event1, dat.i$.n1,
                            dat.i$.event2, dat.i$.n2,
                            dat.i$.treat1, dat.i$.treat2,
                            dat.i$.studlab,
                            data = dat.i,
                            sm = x$sm, method = x$method,
                            warn = warn)
      else
        net.i <- netmeta(dat.i$.TE, dat.i$.seTE,
                         dat.i$.treat1, dat.i$.treat2,
                         dat.i$.studlab,
                         data = dat.i,
                         method.tau = x$method.tau,
                         warn = warn)
      ##
      if (trts[idx1.i] %in% rownames(net.i$TE.common) &
          trts[idx2.i] %in% colnames(net.i$TE.common)) {
        TE.indirect.common[idx1.i, idx2.i] <-
          net.i$TE.common[trts[idx1.i], trts[idx2.i]]
        TE.indirect.common[idx2.i, idx1.i] <-
          net.i$TE.common[trts[idx2.i], trts[idx1.i]]
        ##
        seTE.indirect.common[idx1.i, idx2.i] <-
          seTE.indirect.common[idx2.i, idx1.i] <-
          net.i$seTE.common[trts[idx1.i], trts[idx2.i]]
      }
      ##
      if (!is.bin) {
        if (trts[idx1.i] %in% rownames(net.i$TE.random) &
            trts[idx2.i] %in% colnames(net.i$TE.random)) {
          TE.indirect.random[idx1.i, idx2.i] <-
            net.i$TE.random[trts[idx1.i], trts[idx2.i]]
          TE.indirect.random[idx2.i, idx1.i] <-
            net.i$TE.random[trts[idx2.i], trts[idx1.i]]
          ##
          seTE.indirect.random[idx1.i, idx2.i] <-
            seTE.indirect.random[idx2.i, idx1.i] <-
            net.i$seTE.random[trts[idx1.i], trts[idx2.i]]
        }
      }
    }
    ##
    if (is.tictoc) {
      tictoc.i <- tictoc::toc(func.toc = NULL)
      tictoc[i] <- as.numeric(tictoc.i$toc) - as.numeric(tictoc.i$tic)
      names.tictoc[i] <- paste(trts[idx1.i], trts[idx2.i], sep = sep.trts)
      ##
      if (verbose)
        cat(round(tictoc[i], 3), "sec elapsed\n")
    }
  }
  
  
  res <- list(TE.indirect.common = TE.indirect.common,
              seTE.indirect.common = seTE.indirect.common)
  ##
  if (!is.bin) {
    res$TE.indirect.random <- TE.indirect.random
    res$seTE.indirect.random <- seTE.indirect.random
  }
  ##
  if (is.tictoc) {
    names(tictoc) <- names.tictoc
    res$tictoc <- tictoc
  }
  ##
  res
}

formatComp <- function(x,
                       backtransf, sm, level,
                       comps, comps.abbr, sep.comps,
                       digits, digits.stat, digits.pval,
                       scientific.pval, zero.pval, JAMA.pval,
                       big.mark) {
  
  
  relative <- is_relative_effect(sm)
  
  
  sm.lab <- sm
  ##
  if (!backtransf & relative)
    sm.lab <- paste0("log", if (sm == "VE") "VR" else sm)
  ##  
  ci.lab <- paste0(round(100 * level, 1), "% CI")
  
  
  ## First column contains row names
  ##
  rnam <- x[, 1]
  res <- x[, -1]
  ##
  if (backtransf) {
    res$TE <- backtransf(res$TE, sm)
    res$lower <- backtransf(res$lower, sm)
    res$upper <- backtransf(res$upper, sm)
    #
    # Switch lower and upper limit for VE if results have been
    # backtransformed
    #
    if (sm == "VE") {
      tmp.l <- res$lower
      res$lower <- res$upper
      res$upper <- tmp.l
    }
  }
  ##
  res$treat1 <- compos(res$treat1, comps, comps.abbr, sep.comps)
  res$treat2 <- compos(res$treat2, comps, comps.abbr, sep.comps)
  ##
  res$TE <- formatN(res$TE, digits, "NA", big.mark)
  res$lower <- formatCI(formatN(round(res$lower, digits),
                                       digits, "NA", big.mark),
                               formatN(round(res$upper, digits),
                                       digits, "NA", big.mark))
  res$statistic <- formatN(res$statistic, digits.stat, big.mark = big.mark)
  res$p <- formatPT(res$p,
                           digits = digits.pval,
                           scientific = scientific.pval,
                           zero = zero.pval,
                           JAMA = JAMA.pval)
  ##
  res$upper <- res$z <- NULL
  ##
  sel <- names(res) == "TE"
  names(res)[sel] <- sm.lab
  ##
  sel <- names(res) == "lower"
  names(res)[sel] <- ci.lab
  ##
  sel <- names(res) == "statistic"
  names(res)[sel] <- "z"
  ##
  sel <- names(res) == "p"
  names(res)[sel] <- "p-value"
  ##
  colnames <- names(res)
  ##
  res <- as.matrix(res)
  ##
  dimnames(res) <- list(rnam, colnames)
  
  
  res
}

##
##
## Definition of auxiliary functions for forest plots
##
##


setLab <- function(lab, col, match, value) {
  idx <- charmatch(tolower(col), tolower(match), nomatch = NA)
  sel <- !is.na(idx) & idx == 1
  if (any(sel) & length(lab) == length(col))
    lab[sel] <- value
  ##
  lab
}


setCol <- function(col, match) {
  idx <- charmatch(tolower(col), tolower(match), nomatch = NA)
  sel <- !is.na(idx) & idx == 1
  if (any(sel))
    col[sel] <- match
  ##
  col
}


matchVar <- function(x, match) {
  idx <- charmatch(tolower(x), tolower(match), nomatch = NA)
  sel <- !is.na(idx) & idx == 1
  ##
  sel
}  


anyCol <- function(col, match) {
  idx <- charmatch(tolower(col), tolower(match), nomatch = NA)
  any(!is.na(idx) & idx == 1)
}


naLab <- function(x) {
  if (length(x) == 0)
    return(FALSE)
  else
    return(is.na(x))
}  

setref <- function(reference.group, levs, length = 1,
                   varname = "reference.group", error.text) {

  if (missing(error.text)) {
    text.start <- paste0("Argument '", varname, "'")
    text.within <- paste0("argument '", varname, "'")
  }
  else {
    text.start <- paste0(toupper(substring(error.text, 1, 1)),
                         substring(error.text, 2))
    text.within <- error.text
  }
  
  
  if (length && length(reference.group) != length)
    stop(text.start,
         if (length == 1)
           " must be a numeric or a character string"
         else
           paste(" must be a numeric of character vector of length", length),
         ".",
         call. = FALSE)
  ##
  if (is.numeric(reference.group)) {
    if (any(is.na(reference.group)))
      stop("Missing value not allowed in ", text.within, ".",
           call. = FALSE)
    if (!all(reference.group %in% seq_len(length(levs))))
      stop(paste0(text.start, " must ",
                  if (length == 1) "be any of the " else "contain ",
                  "integers from 1 to ",
                  length(levs), "."),
           call. = FALSE)
    res <- levs[reference.group]
  }
  else if (is.character(reference.group)) {
    if (any(is.na(reference.group)))
      stop("Missing value not allowed in ", text.within, ".",
           call. = FALSE)
    ##
    if (length(unique(levs)) == length(unique(tolower(levs))))
      idx <- charmatch(tolower(reference.group), tolower(levs), nomatch = NA)
    else {
      idx1 <- charmatch(reference.group, levs, nomatch = NA)
      idx2 <- charmatch(tolower(reference.group), tolower(levs), nomatch = NA)
      if (anyNA(idx1) & !anyNA(idx2))
        idx <- idx2
      else
        idx <- idx1
    }
    ##
    if (anyNA(idx) || any(idx == 0))
      stop("Admissible values for ", text.within, ":\n  ",
           paste(paste0("'", levs, "'"), collapse = " - "),
           "\n  (unmatched value", if (sum(is.na(idx)) > 1) "s",
           ": ",
           paste(paste0("'", reference.group[is.na(idx)], "'"),
                 collapse = " - "),
           ")",
           call. = FALSE)
    res <- levs[idx]
  }
  
  res
}

---
#title: "Workflow in R package netmeta"
#author: "Guido Schwarzer"
output:
  rmarkdown::pdf_document:
   number_sections: true
  rmarkdown::html_vignette:
   toc: false
   number_sections: true
vignette: >
  %\VignetteIndexEntry{netmeta: Network Meta-Analysis using Frequentist Methods - Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = TRUE,
  warning = FALSE
  )
options(knitr.kable.NA = ".")
```

```{r, echo = FALSE, out.height = "99%"}
knitr::include_graphics("netmeta-workflow-diagram.pdf")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.netcomb.R
\name{forest.netcomb}
\alias{forest.netcomb}
\alias{plot.netcomb}
\title{Forest plot for additive network meta-analysis}
\usage{
\method{forest}{netcomb}(
  x,
  pooled = ifelse(x$random, "random", "common"),
  reference.group = x$reference.group,
  baseline.reference = x$baseline.reference,
  equal.size = gs("equal.size"),
  leftcols = "studlab",
  leftlabs = "Treatment",
  rightcols = c("effect", "ci"),
  rightlabs = NULL,
  digits = gs("digits.forest"),
  smlab = NULL,
  sortvar = x$seq,
  overall.hetstat = gs("overall.hetstat"),
  backtransf = x$backtransf,
  lab.NA = gs("lab.NA"),
  add.data,
  addrows.below.overall = if (x$overall.hetstat) 2 else gs("addrows.below.overall"),
  drop.reference.group = gs("drop.reference.group"),
  ...
)

\method{plot}{netcomb}(x, ...)
}
\arguments{
\item{x}{An object of class \code{netcomb}.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{reference.group}{Reference treatment(s).}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa.}

\item{equal.size}{A logical indicating whether all squares should
be of equal size. Otherwise, the square size is proportional to
the precision of estimates.}

\item{leftcols}{A character vector specifying (additional) columns
to be plotted on the left side of the forest plot or a logical
value (see \code{\link[meta]{forest.meta}} help page for details).}

\item{leftlabs}{A character vector specifying labels for
(additional) columns on left side of the forest plot (see
\code{\link[meta]{forest.meta}} help page for details).}

\item{rightcols}{A character vector specifying (additional) columns
to be plotted on the right side of the forest plot or a logical
value (see \code{\link[meta]{forest.meta}} help page for details).}

\item{rightlabs}{A character vector specifying labels for
(additional) columns on right side of the forest plot (see
\code{\link[meta]{forest.meta}} help page for details).}

\item{digits}{Minimal number of significant digits for treatment
effects and confidence intervals, see \code{print.default}.}

\item{smlab}{A label printed at top of figure. By default, text
indicating either common or random effects model is printed.}

\item{sortvar}{An optional vector used to sort the individual
studies (must be of same length as the total number of
treatments).}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{lab.NA}{A character string to label missing values.}

\item{add.data}{An optional data frame with additional columns to
print in forest plot (see Details).}

\item{addrows.below.overall}{A numeric value indicating how many
empty rows are printed between meta-analysis results and
heterogeneity statistics.}

\item{drop.reference.group}{A logical indicating whether the
reference group should be printed in the forest plot.}

\item{\dots}{Additional arguments for \code{\link[meta]{forest.meta}}
function.}
}
\description{
Draws a forest plot in the active graphics window (using grid
graphics system).
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window.

Argument \code{sortvar} can be either a numeric or character vector
with length of number of treatments. If \code{sortvar} is numeric
the \code{\link[base]{order}} function is utilised internally to
determine the order of values. If \code{sortvar} is character it
must be a permutation of the treatment names. It is also possible
to to sort by treatment comparisons (\code{sortvar = TE}, etc.),
standard error (\code{sortvar = seTE}), and number of studies with
direct treatment comparisons (\code{sortvar = k}).

Argument \code{add.data} can be used to add additional columns to
the forest plot. This argument must be a data frame with the same
row names as the treatment effects matrices in R object \code{x},
i.e., \code{x$TE.common} or \code{x$TE.random}.

For more information see help page of \code{\link[meta]{forest.meta}}
function.
}
\examples{
# Examples: example(netcomb)

}
\seealso{
\code{\link{netcomb}}, \code{\link{discomb}},
  \code{\link[meta]{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netbind.R
\name{netbind}
\alias{netbind}
\title{Combine network meta-analysis objects}
\usage{
netbind(
  ...,
  name,
  common,
  random,
  col.study = "black",
  col.inside = "white",
  col.square = "gray",
  col.square.lines = col.square,
  backtransf,
  reference.group,
  baseline.reference,
  warn.deprecated = gs("warn.deprecated"),
  fixed,
  comb.fixed,
  comb.random
)
}
\arguments{
\item{\dots}{Any number of network meta-analysis objects or a
single list with network meta-analyses.}

\item{name}{An optional character vector providing descriptive
names for network meta-analysis objects.}

\item{common}{A logical indicating whether results for the common
effects model should be reported.}

\item{random}{A logical indicating whether results for the random
effects model should be reported.}

\item{col.study}{The colour for network estimates and confidence
limits.}

\item{col.inside}{The colour for network estimates and confidence
limits if confidence limits are completely within squares.}

\item{col.square}{The colour for squares.}

\item{col.square.lines}{The colour for the outer lines of squares.}

\item{backtransf}{A logical indicating whether results should be
back transformed. If \code{backtransf = TRUE} (default), results
for \code{sm = "OR"} are printed as odds ratios rather than log
odds ratios, for example.}

\item{reference.group}{Reference treatment.}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{fixed}{Deprecated argument; replaced by \code{common}.}

\item{comb.fixed}{Deprecated argument; replaced by \code{common}.}

\item{comb.random}{Deprecated argument; replaced by \code{random}.}
}
\value{
An object of class \code{"netbind"} with corresponding
\code{forest} function. The object is a list containing the
following components:
\item{common}{A data frame with results for the common effects
  model.}
\item{random}{A data frame with results for the random effects
  model.}
\item{sm}{Summary measure used in network meta-analyses.}
\item{level.ma}{Level for confidence intervals.}
\item{reference.group, baseline.reference}{As defined above.}
}
\description{
This function can be used to combine network meta-analysis objects
which is especially useful to generate a forest plot with results
of several network meta-analyses.
}
\examples{
\donttest{
# Only consider studies including Face-to-face PST (to reduce
# runtime of example)
#
face <- subset(Linde2016, id \%in\% c(16, 24, 49, 118))

# Use pairwise() to transform data to comparison-based format
#
pw <- pairwise(treat = int,
  event = resp, n = n, studlab = paste(author, year),
  data = face, reference = "plac", sm = "OR")

# Conduct random effects network meta-analysis
#
nma <- netmeta(pw, common = FALSE)

# Additive CNMA model with placebo as inactive component and
# reference
#
cnma <- netcomb(nma, inactive = "placebo")

# Combine results of standard NMA and CNMA
#
nb <- netbind(cnma, nma,
  name = c("Additive CNMA", "Standard NMA"),
  col.study = c("red", "black"), col.square = c("red", "black"))
#
nb
#
forest(nb,
  col.subgroup = "black", addrow.subgroups = FALSE,
  fontsize = 10, spacing = 0.7, squaresize = 0.9,
  label.left = "Favours Placebo",
  label.right = "Favours other")

# Also show results for common effects model
#
print(nb, common = TRUE)
}

}
\seealso{
\code{\link{netmeta}}, \code{\link{netcomb}},
  \code{\link{discomb}}, \code{\link{forest.netbind}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Senn2013.R
\docType{data}
\name{Senn2013}
\alias{Senn2013}
\title{Network meta-analysis in diabetes}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{TE}}\tab treatment effect \cr
\bold{\emph{seTE}}\tab standard error of treatment effect \cr
\bold{\emph{treat1}}\tab treatment 1 \cr
\bold{\emph{treat2}}\tab treatment 2 \cr
\bold{\emph{treat1.long}}\tab treatment 1 (full treatment names)
  \cr
\bold{\emph{treat2.long}}\tab treatment 2 (full treatment names)
  \cr
\bold{\emph{studlab}}\tab Study label
}
}
\source{
Senn S, Gavini F, Magrez D, Scheen A (2013):
Issues in performing a network meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{22}, 169--89
}
\description{
Network meta-analysis in diabetes comparing effects of a number of
drugs on the HbA1c value.

These data are used as an example in Senn et al. (2013) and have been
preprocessed for use in R package netmeta.
}
\details{
This data set is in comparison-based format. The data in
long arm-based format are available in \code{\link[metabook]{Senn2013}} in
R package \bold{metabook}.

Treatment labels provided by columns \code{treat1} and
\code{treat2} have been abbreviated:
\itemize{
\item acar = Acarbose
\item benf = Benfluorex
\item metf = Metformin
\item migl = Miglitol
\item piog = Pioglitazone
\item plac = Placebo
\item rosi = Rosiglitazone
\item sita = Sitagliptin
\item sulf = Sulfonylurea
\item vild = Vildagliptin
}

Full treatment names are available in columns \code{treat1.long}
and \code{treat2.long}.
}
\examples{
head(Senn2013)

# Example using netmeta():
# example(netmeta)
}
\seealso{
\code{\link{netmeta}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Franchini2012.R
\docType{data}
\name{Franchini2012}
\alias{Franchini2012}
\alias{parkinson}
\title{Network meta-analysis of treatments for Parkinson's disease}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{Study}}\tab study label \cr
\bold{\emph{Treatment1}}\tab treatment 1 \cr
\bold{\emph{y1}}\tab treatment effect arm 1 \cr
\bold{\emph{sd1}}\tab Standard deviation arm 1 \cr
\bold{\emph{n1}}\tab Sample size arm 1 \cr
\bold{\emph{Treatment2}}\tab treatment 2 \cr
\bold{\emph{y2}}\tab treatment effect arm 2 \cr
\bold{\emph{sd2}}\tab Standard deviation arm 2 \cr
\bold{\emph{n2}}\tab Sample size arm 2 \cr
\bold{\emph{Treatment3}}\tab treatment 3 \cr
\bold{\emph{y3}}\tab treatment effect arm 3 \cr
\bold{\emph{sd3}}\tab Standard deviation arm 3 \cr
\bold{\emph{n3}}\tab Sample size arm 3
}
}
\source{
Dias S, Sutton AJ, Ades AE and Welton NJ (2013):
Evidence synthesis for decision making 2: A generalized linear
modeling framework for pairwise and network meta-analysis of
randomized controlled trials.
\emph{Medical Decision Making},
\bold{33}, 607--17

Franchini AJ, Dias S, Ades AE, Jansen JP, Welton NJ (2012):
Accounting for correlation in network meta-analysis with multi-arm
trials.
\emph{Research Synthesis Methods},
\bold{3}, 142--60
}
\description{
Network meta-analysis comparing the effects of a number of
treatments for Parkinson's disease.

The data are the mean lost work-time reduction in patients given
dopamine agonists as adjunct therapy in Parkinson’s disease
(Franchini et al. 2012). The data are given as sample size, mean
and standard deviation in each trial arm. Treatments are placebo
and four active drugs. These data are used as an example in the
supplemental material of Dias et al. (2013) where placebo is coded
as 1 and the four active drugs as 2 to 5.
}
\note{
The dataset Franchini2012 is identical to dataset
\code{\link[metabook]{Franchini2012}} in R package \bold{metabook}.
}
\examples{
head(Franchini2012)

# Example using pairwise() and netmeta():
# example(Franchini2012, package = "metabook")
}
\seealso{
\code{\link[metabook]{Franchini2012}},
  \code{\link[meta]{pairwise}}, \code{\link[meta]{metacont}},
  \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funnel.netmeta.R
\name{funnel.netmeta}
\alias{funnel.netmeta}
\title{\sQuote{Comparison-adjusted} funnel plot}
\usage{
\method{funnel}{netmeta}(
  x,
  order,
  pooled = ifelse(x$random, "random", "common"),
  xlab = NULL,
  level = x$level,
  pch,
  col = "black",
  legend = TRUE,
  pos.legend = "topright",
  pos.tests = "topleft",
  lump.comparator = FALSE,
  text.comparator = "comparator",
  method.bias,
  text.linreg = "(Egger)",
  text.rank = "(Begg-Mazumdar)",
  text.mm = "(Thompson-Sharp)",
  sep.trts = x$sep.trts,
  nchar.trts = x$nchar.trts,
  backtransf = x$backtransf,
  digits.pval = gs("digits.pval"),
  warn.deprecated = gs("warn.deprecated"),
  linreg = FALSE,
  rank = FALSE,
  mm = FALSE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{order}{A mandatory character or numerical vector specifying
the order of treatments or list of comparators (see Details).}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{xlab}{A label for the x-axis.}

\item{level}{The confidence level utilised in the plot. For the
funnel plot, confidence limits are not drawn if \code{yaxis =
"size"}.}

\item{pch}{The plotting symbol(s) used for individual studies
within direct comparisons.}

\item{col}{The colour(s) used for individual studies within direct
comparisons.}

\item{legend}{A logical indicating whether a legend with
information on direct comparisons should be added to the plot.}

\item{pos.legend}{The position of the legend describing plotting
symbols and colours for direct comparisons.}

\item{pos.tests}{The position of results for test(s) of funnel plot
asymmetry.}

\item{lump.comparator}{A logical indicating whether comparators
should be lumped, e.g., to specify inactive treatments.
information on direct comparisons should be added to the plot.}

\item{text.comparator}{A character string used in the plot to label
the comparator if \code{lump.comparator} is \code{TRUE}.}

\item{method.bias}{A character vector indicating which test(s) for
funnel plot asymmetry to use. Admissible values are
\code{"Begg"}, \code{"Egger"}, and \code{"Thompson"}, can be
abbreviated. See function \code{\link[meta]{metabias}}.}

\item{text.linreg}{A character string used in the plot to label the
Egger test for funnel plot asymmetry.}

\item{text.rank}{A character string used in the plot to label the
Begg test for funnel plot asymmetry.}

\item{text.mm}{A character string used in the plot to label the
Thompson-Sharp test for funnel plot asymmetry.}

\item{sep.trts}{A character used in comparison names as separator
between treatment labels.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see
\code{\link{netmeta}}).}

\item{backtransf}{A logical indicating whether results for relative
summary measures (argument \code{sm} equal to \code{"OR"},
\code{"RR"}, \code{"HR"}, or \code{"IRR"}) should be back
transformed in funnel plots. If \code{backtransf = TRUE}, results
for \code{sm = "OR"} are printed as odds ratios rather than log
odds ratios, for example.}

\item{digits.pval}{Minimal number of significant digits for p-value
of test(s) for funnel plot asymmetry.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{linreg}{Deprecated argument (replaced by \code{method.bias}).}

\item{rank}{Deprecated argument (replaced by \code{method.bias}).}

\item{mm}{Deprecated argument (replaced by \code{method.bias}).}

\item{\dots}{Additional graphical arguments passed as arguments to
\code{\link[meta]{funnel.meta}}.}
}
\value{
A data frame with the following columns:
\item{studlab}{Study label.}
\item{treat1}{Label/Number for first treatment.}
\item{treat2}{Label/Number for second treatment.}
\item{comparison}{Treatment comparison.}
\item{TE}{Estimate of treatment effect, e.g., log odds ratio.}
\item{TE.direct}{Pooled estimate from direct evidence.}
\item{TE.adj}{\sQuote{Comparison-adjusted} treatment effect (TE -
  TE.direct).}
\item{seTE}{Standard error of treatment estimate.}
\item{pch}{Plotting symbol(s).}
\item{col}{Colour of plotting symbol(s).}
}
\description{
Draw a \sQuote{comparison-adjusted} funnel plot to assess funnel
plot asymmetry in network meta-analysis.
}
\details{
A \sQuote{comparison-adjusted} funnel plot (Chaimani & Salanti,
2012) is drawn in the active graphics window.

Argument \code{order} is mandatory to determine the order of
treatments (Chaimani et al., 2013):

\emph{\dQuote{Before using this plot, investigators should order
the treatments in a meaningful way and make assumptions about how
small studies differ from large ones. For example, if they
anticipate that newer treatments are favored in small trials, then
they could name the treatments from oldest to newest so that all
comparisons refer to \sQuote{old versus new intervention}. Other
possibilities include defining the comparisons so that all refer to
an active treatment versus placebo or sponsored versus
non-sponsored intervention.}}

Alternatively, it is possible to only provide a single or few
treatment name(s) in argument \code{order} to define the
comparator(s). In this case only comparisons with this / these
treatment(s) will be considered. If argument \code{lump.comparator}
is \code{TRUE}, all comparators will be lumped into a single
group. The text for this group can be specified with argument
\code{text.comparator}.

In the funnel plot, if \code{yaxis} is \code{"se"}, the standard
error of the treatment estimates is plotted on the y-axis which is
likely to be the best choice (Sterne & Egger, 2001). Other possible
choices for \code{yaxis} are \code{"invvar"} (inverse of the
variance), \code{"invse"} (inverse of the standard error), and
\code{"size"} (study size).
}
\examples{
\donttest{
# Transform data from long arm-based to contrast-based format
#
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))

# Conduct network meta-analysis
#
nma <- netmeta(pw)

# 'Comparison-adjusted' funnel plot not created as argument 'order'
# is missing
#
try(funnel(nma))

# Only show comparisons with placebo
#
funnel(nma, order = "pl")

# Add result for Egger test of funnel plot asymmetry
#
funnel(nma, order = "pl", method.bias = "Egger",
  digits.pval = 2)

# (Nonsensical) alphabetic order of treatments with placebo as
# last treatment
#
ord <- c("a", "b", "me", "mi", "pi", "r", "si", "su", "v", "pl")
funnel(nma, order = ord)

# Add results for tests of funnel plot asymmetry and use different
# plotting symbols and colours
#
funnel(nma, order = ord,
  pch = rep(c(15:18, 1), 3), col = 1:3,
  method.bias = c("Egger", "Begg", "Thompson"), digits.pval = 2)

# Same results for tests of funnel plot asymmetry using reversed
# order of treatments
#
funnel(nma, order = rev(ord),
  pch = rep(c(15:18, 1), 3), col = 1:3,
  method.bias = c("Egger", "Begg", "Thompson"), digits.pval = 2)

# Calculate tests for funnel plot asymmetry
#
f1 <- funnel(nma, order = ord)
#
metabias(metagen(TE.adj, seTE, data = f1))
metabias(metagen(TE.adj, seTE, data = f1), method = "Begg")
metabias(metagen(TE.adj, seTE, data = f1), method = "Thompson")
}

}
\references{
Chaimani A & Salanti G (2012):
Using network meta-analysis to evaluate the existence of
small-study effects in a network of interventions.
\emph{Research Synthesis Methods},
\bold{3}, 161--76

Chaimani A, Higgins JP, Mavridis D, Spyridonos P, Salanti G (2013):
Graphical tools for network meta-analysis in STATA.
PLOS ONE,
\bold{8}, e76654

Sterne JAC & Egger M (2001):
Funnel plots for detecting bias in meta-analysis: Guidelines on
choice of axis.
\emph{Journal of Clinical Epidemiology},
\bold{54}, 1046--55
}
\seealso{
\code{\link{netmeta}}, \code{\link[meta]{funnel.meta}},
  \code{\link[meta]{metabias}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.decomp.design.R
\name{print.decomp.design}
\alias{print.decomp.design}
\title{Print method for objects of class decomp.design}
\usage{
\method{print}{decomp.design}(
  x,
  digits.Q = gs("digits.Q"),
  showall = FALSE,
  digits.pval.Q = gs("digits.pval.Q"),
  digits.tau2 = gs("digits.tau2"),
  scientific.pval = gs("scientific.pval"),
  big.mark = gs("big.mark"),
  nchar.trts = x$nchar.trts,
  sort = TRUE,
  legend = gs("legend"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{decomp.design}.}

\item{digits.Q}{Minimal number of significant digits for Q
statistics, see \code{print.default}.}

\item{showall}{A logical indicating whether results should be shown
for all designs or only designs contributing to chi-squared
statistics (default).}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity tests, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance, see \code{print.default}.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{big.mark}{A character used as thousands separator.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{sort}{A logical indicating whether to sort results by
p-values.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{\dots}{Additional arguments (ignored at the moment).}
}
\description{
Print method for objects of class \code{decomp.design}.
}
\examples{
# Examples: example(decomp.design)

}
\seealso{
\code{\link{decomp.design}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}, Ulrike
  Krahn \email{ulrike.krahn@bayer.com}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netgraph.discomb.R
\name{netgraph.discomb}
\alias{netgraph.discomb}
\title{Network graph for objects of class discomb}
\usage{
\method{netgraph}{discomb}(
  x,
  labels = x$trts,
  adj = NULL,
  offset = if (!is.null(adj) && all(unique(adj) == 0.5)) 0 else 0.0175,
  rotate = 0,
  points = !missing(cex.points),
  cex.points = 1,
  ...
)
}
\arguments{
\item{x}{An object of class \code{discomb}.}

\item{labels}{An optional vector with treatment labels.}

\item{adj}{One, two, or three values in [0, 1] (or a vector /
matrix with length / number of rows equal to the number of
treatments) specifying the x (and optionally y and z) adjustment
for treatment labels.}

\item{offset}{Distance between nodes (i.e. treatments) in graph and
treatment labels for 2-D plots (value of 0.0175 corresponds to a
difference of 1.75\% of the range on x- and y-axis).}

\item{rotate}{A single numeric with value between -180 and 180
specifying the degrees to rotate nodes in a circular network clockwise or
anticlockwise.}

\item{points}{A logical indicating whether points should be printed
at nodes (i.e. treatments) of the network graph.}

\item{cex.points}{Corresponding point size. Can be a vector with length
equal to the number of treatments.}

\item{\dots}{Additional arguments passed on to
\code{\link{netgraph.netmeta}}.}
}
\description{
This function generates a graph of the evidence network.
}
\details{
The arguments \code{seq} and \code{iterate} are used internally and
cannot be specified by the user.
}
\examples{
# Artificial dataset
#
t1 <- c("A + B", "A + C", "A"    , "A"    , "D", "D", "E")
t2 <- c("C"    , "B"    , "B + C", "A + D", "E", "F", "F")
#
mean <- c(4.1, 2.05, 0, 0, 0.1, 0.1, 0.05)
se.mean <- rep(0.1, 7)
#
study <- paste("study", c(1:4, 5, 5, 5))
#
dat <- data.frame(mean, se.mean, t1, t2, study,
                  stringsAsFactors = FALSE)
#
trts <- c("A", "A + B", "A + C", "A + D",
  "B", "B + C", "C", "D", "E", "F")
#
comps <- LETTERS[1:6]

# Use netconnection() to display network information
#
netconnection(t1, t2, study)

dc1 <- discomb(mean, se.mean, t1, t2, study, seq = trts)

netgraph(dc1)

}
\seealso{
\code{\link{discomb}}, \code{\link{netgraph.netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netrank.R
\name{netrank}
\alias{netrank}
\alias{print.netrank}
\title{Frequentist method to rank treatments in network}
\usage{
netrank(
  x,
  small.values = x$small.values,
  method,
  nsim,
  common = x$common,
  random = x$random,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{print}{netrank}(
  x,
  common = x$common,
  random = x$random,
  sort = TRUE,
  digits = gs("digits.prop"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta} or \code{rankogram}.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect, can be abbreviated.}

\item{method}{A character string specifying whether the
\code{"P-score"} or \code{"SUCRA"} ranking metric will be
calculated.}

\item{nsim}{Number of repetitions to calculate SUCRAs.}

\item{common}{A logical indicating whether to print P-scores or
SUCRAs for the common effects model.}

\item{random}{A logical indicating whether to print P-scores or
SUCRAs for the random effects model.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments passed on to
\code{\link{print.data.frame}} function (used internally).}

\item{sort}{A logical indicating whether printout should be sorted
by decreasing P-score.}

\item{digits}{Minimal number of significant digits, see
\code{\link{print.default}}.}
}
\value{
An object of class \code{netrank} with corresponding \code{print}
function. The object is a list containing the following components:
\item{ranking.common}{A named numeric vector with P-scores or SUCRAs
  for the common effects model.}
\item{Pmatrix.common}{Numeric matrix based on pairwise one-sided
  p-values for the common effects model.}
\item{ranking.random}{A named numeric vector with P-scores or
  SUCRAs for the random effects model.}
\item{Pmatrix.random}{Numeric matrix based on pairwise one-sided
  p-values of the random effects model.}
\item{small.values, method, x}{As defined above.}
\item{version}{Version of R package netmeta used to create object.}
}
\description{
Ranking treatments in frequentist network meta-analysis with and
without resampling methods.
}
\details{
Treatments are ranked based on a network meta-analysis. Ranking is
performed by a ranking metric: P-score or SUCRA.

P-scores are based solely on the point estimates and standard
errors of the network estimates. They measure the extent of
certainty that a treatment is better than another treatment,
averaged over all competing treatments (Rücker and Schwarzer 2015).

The Surface Under the Cumulative RAnking curve (SUCRA) is the rank
of treatment \emph{i} within the range of treatments, measured on a
scale from 0 (worst) to 1 (best) (Salanti et al. 2011). A
resampling method is used to calculate SUCRAs for frequentist
network meta-analysis. The number of repetitions is determine by
argument \code{nsim}.

The interpretation of P-scores and SUCRAs is comparable.

The P-score of treatment \emph{i} is defined as the mean of all 1 -
P[\emph{j}] where P[\emph{j}] denotes the one-sided P-value of
accepting the alternative hypothesis that treatment \emph{i} is
better than one of the competing treatments \emph{j}. Thus, if
treatment \emph{i} is better than many other treatments, many of
these P-values will be small and the P-score will be large. Vice
versa, if treatment \emph{i} is worse than most other treatments,
the P-score is small.

The P-score of treatment \emph{i} can be interpreted as the mean
extent of certainty that treatment \emph{i} is better than another
treatment.
}
\examples{
\donttest{
# Define order of treatments in depression dataset Linde2015
#
trts <- c("TCA", "SSRI", "SNRI", "NRI",
  "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")

# Outcome labels
#
outcomes <- c("Early response", "Early remission")

# (1) Early response
#
pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma1 <- netmeta(pw1, common = FALSE, seq = trts, ref = "Placebo",
  small.values = "undesirable")
netrank(nma1)

# (2) Early remission
#
pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma2 <- netmeta(pw2, common = FALSE, seq = trts, ref = "Placebo",
  small.values = "undesirable")
netrank(nma2)

# Image plot of treatment rankings (two outcomes)
#
plot(netrank(nma1), netrank(nma2), name = outcomes, digits = 2)


# Outcome labels
#
outcomes <- c("Early response", "Early remission",
  "Lost to follow-up", "Lost to follow-up due to AEs",
  "Adverse events (AEs)")

# (3) Loss to follow-up
#
pw3 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(loss1, loss2, loss3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma3 <- netmeta(pw3, common = FALSE, seq = trts, ref = "Placebo",
  small.values = "desirable")

# (4) Loss to follow-up due to adverse events
#
pw4 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(loss.ae1, loss.ae2, loss.ae3), n = list(n1, n2, n3),
  studlab = id, data = subset(Linde2015, id != 55), sm = "OR")
#
nma4 <- netmeta(pw4, common = FALSE, seq = trts, ref = "Placebo",
  small.values = "desirable")

# (5) Adverse events
#
pw5 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(ae1, ae2, ae3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma5 <- netmeta(pw5, common = FALSE, seq = trts, ref = "Placebo",
  small.values = "desirable")

# Image plot of treatment rankings (two outcomes)
#
plot(netrank(nma1), netrank(nma2), netrank(nma3),
  netrank(nma4), netrank(nma5),
  name = outcomes, digits = 2)
}

}
\references{
Rücker G, Schwarzer G (2017):
Resolve conflicting rankings of outcomes in network meta-analysis:
Partial ordering of treatments.
\emph{Research Synthesis Methods},
\bold{8}, 526--36

Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
\code{\link{netmeta}}, \code{\link{rankogram}},
  \code{\link{plot.rankogram}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}, Theodoros
  Papakonstantinou \email{dev@tpapak.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netcomplex.R
\name{netcomplex}
\alias{netcomplex}
\alias{print.netcomplex}
\title{Calculate effect of arbitrary complex interventions in component
network meta-analysis}
\usage{
netcomplex(
  x,
  complex,
  common = x$common,
  random = x$random,
  level = x$level.ma,
  nchar.comps = x$nchar.trts,
  backtransf = x$backtransf,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{print}{netcomplex}(
  x,
  common = x$common,
  random = x$random,
  backtransf = x$backtransf,
  nchar.comps = x$nchar.comps,
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  big.mark = gs("big.mark"),
  legend = gs("legend"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netcomb} or \code{netcomplex}
(print function).}

\item{complex}{A matrix, vector or single numeric defining the
complex intervention(s) (see Details).}

\item{common}{A logical indicating whether results for common
effects model should be conducted.}

\item{random}{A logical indicating whether results for random
effects model should be conducted.}

\item{level}{The level used to calculate confidence intervals for
combinations of components.}

\item{nchar.comps}{A numeric defining the minimum number of
characters used to create unique names for components (see
Details).}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. If \code{backtransf=TRUE}, results
for \code{sm="OR"} are printed as odds ratios rather than log
odds ratios.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z-value
of test for overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for
p-values, see \code{print.default}.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
combination effect should be printed according to JAMA reporting
standards.}

\item{big.mark}{A character used as thousands separator.}

\item{legend}{A logical indicating whether a legend should be
printed.}
}
\value{
A list is returned by the function \code{netcomplex} with the
following elements:
\item{complex}{Complex intervention(s).}
\item{Comb.common, Comb.random}{A vector of combination effects
  (common and random effects model).}
\item{seComb.common, seComb.random}{A vector with corresponding
  standard errors (common and random effects model).}
\item{lower.Comb.common, lower.Comb.random}{A vector with lower
  confidence limits for combinations (common and random effects
  model).}
\item{upper.Comb.common, upper.Comb.random}{A vector with upper
  confidence limits for combinations (common and random effects
  model).}
\item{statistic.Comb.common, statistic.Comb.random}{A vector with
  z-values for the overall effect of combinations (common and random
  effects model).}
\item{pval.Comb.common, pval.Comb.random}{A vector with p-values for
  the overall effect of combinations (common and random effects
  model).}
\item{common, random}{A defined above.}
\item{level, nchar.comps, backtransf, x}{A defined above.}
\item{C.matrix}{C matrix.}
}
\description{
Calculate effect of arbitrary complex interventions (i.e.,
combinations of several components) in component network
meta-analysis.
}
\details{
R functions \code{\link{netcomb}} and \code{\link{discomb}} only
report results for complex interventions present in the
network. This function can be used to calculate the effect for
arbitrary complex interventions.

Complex interventions can be specified using argument \code{complex}:
\itemize{
\item a character vector with definition of complex interventions,
\item a single numeric defining the number of components to combine
  in a complex intervention,
\item a dedicated C matrix.
}
In order to calculate effects of arbitrary complex interventions, a
C matrix is needed which describes how the complex interventions
are composed by the components (Rücker et al., 2020, Section
3.2). The C matrix is constructed internally if not provided by
argument \code{complex}. All complex interventions occuring in the
network are considered if argument \code{complex} is missing.

By default, component names are not abbreviated in
printouts. However, in order to get more concise printouts,
argument \code{nchar.comps} can be used to define the minimum
number of characters for abbreviated component names (see
\code{\link{abbreviate}}, argument \code{minlength}). R function
\code{\link{treats}} is utilised internally to create abbreviated
component names.
}
\note{
R function \code{\link{netcomparison}} can be used to calculate the
effect for comparisons of two arbitrary complex intervention in a
component network meta-analysis.
}
\examples{
# Only consider studies including Face-to-face PST (to reduce
# runtime of example)
#
face <- subset(Linde2016, id \%in\% c(16, 24, 49, 118))

# Use pairwise() to transform data to comparison-based format
#
pw <- pairwise(treat = int,
  event = resp, n = n, studlab = paste(author, year),
  data = face, reference = "plac", sm = "OR")

# Conduct random effects network meta-analysis
#
nma1 <- netmeta(pw, common = FALSE)

# Additive model for treatment components (with placebo as inactive
# treatment)
#
cnma1 <- netcomb(nma1, inactive = "placebo")

# Result for combination Face-to-face PST + SSRI
netcomplex(cnma1, "Face-to-face PST + SSRI", nchar.comps = 4)
netcomplex(cnma1, "F + S", nchar.comps = 4)

# Result for combination Face-to-face PST + SSRI + Placebo
netcomplex(cnma1, "Face-to-face PST + SSRI + Plac", nchar.comps = 4)

\donttest{
# Artificial example
t1 <- rep("A", 3)
t2 <- c("B+C", "A+C", "C+D")
TE <- c(0, 1, 0)
seTE <- rep(1, 3)
# Conduct (C)NMA
nma2 <- netmeta(TE, seTE, t1, t2, random = FALSE)
cnma2 <- netcomb(nma2)

# Result for combination A + B + C
netcomplex(cnma2, "A + B + C")
# Same results
netcomplex(cnma2, "A+B+C")
netcomplex(cnma2, "B+C+A")
netcomplex(cnma2, "C+B+A")
netcomplex(cnma2, "c+b+a")

# Generated C matrix
netcomplex(cnma2, c(LETTERS[1:4], "A+B+C"))$C.matrix

# Results for all possible combinations of two components
netcomplex(cnma2, 2)
# Results for all possible combinations of three components
netcomplex(cnma2, 3)
}

}
\references{
Rücker G, Petropoulou M, Schwarzer G (2020):
Network meta-analysis of multicomponent interventions.
\emph{Biometrical Journal},
\bold{62}, 808--21
}
\seealso{
\code{\link{netcomb}}, \code{\link{discomb}},
  \code{\link{netcomparison}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netgraph.netimpact.R
\name{netgraph.netimpact}
\alias{netgraph.netimpact}
\title{Network graph for objects of class netimpact}
\usage{
\method{netgraph}{netimpact}(
  x,
  col.ignore = "red",
  number.of.studies = TRUE,
  main,
  sub,
  multiarm = gs("multiarm"),
  col.multiarm = NULL,
  alpha.transparency = 0.5,
  col.ignore.multiarm = "transparent",
  col = "black",
  plastic = gs("plastic"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netimpact}.}

\item{col.ignore}{A character string indicating color for
comparisons removed from network, either \code{"transparent"} or
any color defined in \code{\link[grDevices]{colours}}.}

\item{number.of.studies}{A logical indicating whether number of
studies should be added to network graph.}

\item{main}{Main title.}

\item{sub}{Subtitle.}

\item{multiarm}{A logical indicating whether multi-arm studies
should be marked in plot.}

\item{col.multiarm}{Either a function from R package
\bold{colorspace} or \bold{grDevice} to define colors for
multi-arm studies or a character vector with colors to highlight
multi-arm studies.}

\item{alpha.transparency}{The alpha transparency of colors used to
highlight multi-arm studies (0 means transparent and 1 means
opaque).}

\item{col.ignore.multiarm}{A character string indicating color to
mark multi-arm studies removed from network, either
\code{"transparent"} or any color defined in
\code{\link[grDevices]{colours}}.}

\item{col}{A single color (or vector of colors) for lines
connecting treatments if argument \code{plastic =
FALSE}. Length of the vector must be equal to the number of
lines / edges.}

\item{plastic}{A logical indicating whether the appearance of the
comparisons should be in '3D look'.}

\item{\dots}{Additional arguments passed on to
\code{\link{netgraph.netmeta}}.}
}
\description{
This function generates a graph of the evidence network.
}
\examples{
# Examples: example(netimpact)

}
\seealso{
\code{\link{netimpact}}, \code{\link{netgraph.netmeta}},
  \code{\link[metabook]{Franchini2012}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Stowe2010.R
\docType{data}
\name{Stowe2010}
\alias{Stowe2010}
\title{Network meta-analysis of adjuvant treatments to levodopa therapy
for Parkinson's disease}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{id}}\tab study id \cr
\bold{\emph{t1}}\tab treatment 1 \cr
\bold{\emph{y1}}\tab treatment effect arm 1 \cr
\bold{\emph{sd1}}\tab Standard deviation arm 1 \cr
\bold{\emph{n1}}\tab Sample size arm 1 \cr
\bold{\emph{t2}}\tab treatment 2 \cr
\bold{\emph{y2}}\tab treatment effect arm 2 \cr
\bold{\emph{sd2}}\tab Standard deviation arm 2 \cr
\bold{\emph{n2}}\tab Sample size arm 2 \cr
\bold{\emph{t3}}\tab treatment 3 \cr
\bold{\emph{y3}}\tab treatment effect arm 3 \cr
\bold{\emph{sd3}}\tab Standard deviation arm 3 \cr
\bold{\emph{n3}}\tab Sample size arm 3
}
}
\source{
Stowe R, Ives N, Clarke CE, Deane K, Hilten V, Wheatley K, et
al. (2010):
Evaluation of the efficacy and safety of adjuvant treatment to
levodopa therapy in Parkinson's disease patients with motor
complications.
\emph{Cochrane Database of Systematic Reviews}
}
\description{
This dataset contains data from a Cochrane review assessing
efficacy and safety of three drug classes as adjuvant treatment to
levodopa therapy in patients with Parkinson’s disease and motor
complications (Stowe et al., 2010). The authors conducted three
pairwise meta-analyses comparing dopamine agonists,
catechol-O-methyl transferase inhibitors (COMTIs), and monoamine
oxidase type B inhibitors (MAOBIs), respectively, with placebo.

The primary outcome was the mean reduction of the time spent in a
relatively immobile ’off’ phase (mean off-time), calculated in
hours per day. Relative treatment effects were expressed as mean
difference. Data on this outcome were available for 5,331 patients
from 28 studies comparing an active treatment with placebo and one
three-arm study comparing two active treatments with placebo.
}
\note{
The dataset Stowe2010 is identical to dataset
\code{\link[metabook]{Stowe2010}} in R package \bold{metabook}.
}
\examples{
head(Stowe2010)

# Example using pairwise(), netmeta() and netrank():
# example(Stowe2010, package = "metabook")
}
\seealso{
\code{\link[meta]{pairwise}}, \code{\link[meta]{metacont}},
  \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.rankogram.R
\name{plot.rankogram}
\alias{plot.rankogram}
\title{Plot rankograms}
\usage{
\method{plot}{rankogram}(
  x,
  type = if (cumulative.rankprob) "step" else "bar",
  pooled = ifelse(x$random, "random", "common"),
  sort = TRUE,
  trts,
  cumulative.rankprob = x$cumulative.rankprob,
  ylim,
  ylab,
  nchar.trts = x$nchar.trts,
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{rankogram}}.}

\item{type}{A character string specifying whether a "bar" chart, a
"line" graph, or "step" functions should be drawn. Can be
abbreviated.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{sort}{A logical indicating whether treatments should be
sorted by decreasing SUCRAs.}

\item{trts}{Treatment(s) to show in rankogram.}

\item{cumulative.rankprob}{A logical indicating whether cumulative
ranking probabilites should be shown.}

\item{ylim}{The y limits (min, max) of the plot.}

\item{ylab}{A label for the y-axis.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{\dots}{Additional graphical arguments (ignored at the
moment).}
}
\description{
This function produces a rankogram, i.e., an image plot of ranking
probabilities for all treatments.
}
\details{
This function produces plots of (cumulative) ranking probabilities for all
treatments as a bar graph, a line graph or as step functions
(argument \code{type}). All plots will be shown in a single figure if
R package \bold{gridExtra} is installed. Otherwise, separate figures will be
created for treatments.

By default (argument \code{pooled}), results for the random effects
model are shown if a network meta-analysis was conducted for both
the common and random effects model.

Treatments are sorted according to their mean effects if argument
\code{sort = TRUE} (default).  A subset of treatments can be
specified using argument \code{trts}.

Cumulative ranking probabilites are shown if
\code{cumulative.rankprob = TRUE}. By default, step functions are
shown for cumulative ranking probabilites.
}
\examples{
pw <- pairwise(treatment, event = r, n = N, studlab = author,
  data = Woods2010, sm = "OR")
nma <- netmeta(pw, small.values = "desirable")

set.seed(1909) # get reproducible results
rnk <- rankogram(nma, nsim = 100)
rnk

plot(rnk)
plot(rnk, type = "l")
plot(rnk, cumulative.rankprob = TRUE)

}
\references{
Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
\code{\link{rankogram}}, \code{\link[metabook]{Woods2010}}
}
\author{
Theodoros Papakonstantinou \email{dev@tpapak.com}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.netrank.R
\name{plot.netrank}
\alias{plot.netrank}
\title{Plot treatment ranking(s) of network meta-analyses}
\usage{
\method{plot}{netrank}(
  ...,
  name,
  common,
  random,
  seq,
  low = "red",
  mid = "yellow",
  high = "green",
  col = "black",
  main,
  main.size = 14,
  main.col = col,
  main.face = "bold",
  legend = TRUE,
  axis.size = 12,
  axis.col = col,
  axis.face = "plain",
  na.value = "grey50",
  angle = 45,
  hjust.x = 1,
  vjust.x = 1,
  hjust.y = 1,
  vjust.y = 0,
  nchar.trts,
  digits = 3,
  fixed,
  comb.fixed,
  comb.random,
  warn.deprecated = gs("warn.deprecated")
)
}
\arguments{
\item{...}{A single netrank object or a list of netrank objects.}

\item{name}{An optional character vector providing descriptive
names for the network meta-analysis objects.}

\item{common}{A logical indicating whether results for the common
effects model should be plotted.}

\item{random}{A logical indicating whether results for the random
effects model should be plotted.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments on the x-axis.}

\item{low}{A character string defining the colour for a P-score of
0, see \code{\link[ggplot2]{scale_fill_gradient2}}.}

\item{mid}{A character string defining the colour for a P-score of
0.5, see \code{\link[ggplot2]{scale_fill_gradient2}}.}

\item{high}{A character string defining the colour for a P-score of
1, see \code{\link[ggplot2]{scale_fill_gradient2}}.}

\item{col}{Colour of text.}

\item{main}{Title.}

\item{main.size}{Font size of title, see
\code{\link[ggplot2]{element_text}}.}

\item{main.col}{Colour of title, see
\code{\link[ggplot2]{element_text}}.}

\item{main.face}{Font face of title, see
\code{\link[ggplot2]{element_text}}.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{axis.size}{Font size of axis text, see
\code{\link[ggplot2]{element_text}}.}

\item{axis.col}{Colour of axis text, see
\code{\link[ggplot2]{element_text}}.}

\item{axis.face}{Font face of axis text, see
\code{\link[ggplot2]{element_text}}.}

\item{na.value}{Colour for missing values, see
\code{\link[ggplot2]{scale_fill_gradient2}}.}

\item{angle}{Angle for text on x-axis, see
\code{\link[ggplot2]{element_text}}.}

\item{hjust.x}{A numeric between 0 and 1 with horizontal
justification of text on x-axis, see
\code{\link[ggplot2]{element_text}}.}

\item{vjust.x}{A numeric between 0 and 1 with vertical
justification of text on x-axis, see
\code{\link[ggplot2]{element_text}}.}

\item{hjust.y}{A numeric between 0 and 1 with horizontal
justification of text on y-axis, see
\code{\link[ggplot2]{element_text}}.}

\item{vjust.y}{A numeric between 0 and 1 with vertical
justification of text on y-axis, see
\code{\link[ggplot2]{element_text}}.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{fixed}{Deprecated argument (replaced by 'common').}

\item{comb.fixed}{Deprecated argument (replaced by 'common').}

\item{comb.random}{Deprecated argument (replaced by 'random').}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}
}
\value{
A ggplot2 object or NULL if no ranking was conducted.
}
\description{
Produce an image plot of treatment ranking(s) generated with R
function \code{netrank}.
}
\details{
This function produces an image plot of network rankings (Palpacuer
et al., 2018, Figure 4). Note, a scatter plot of two network
rankings can be generated with \code{\link{plot.netposet}}.

By default, treatments are ordered by decreasing P-scores of the
first network meta-analysis object. Argument \code{seq} can be used
to specify a differenct treatment order.
}
\examples{
# Examples: example(netrank)

}
\references{
Palpacuer C, Duprez R, Huneau A, Locher C, Boussageon R, Laviolle
B, et al. (2018):
Pharmacologically controlled drinking in the treatment of alcohol
dependence or alcohol use disorders: a systematic review with
direct and network meta-analyses on nalmefene, naltrexone,
acamprosate, baclofen and topiramate.
\emph{Addiction},
\bold{113}, 220--37
}
\seealso{
\code{\link{netrank}}, \code{\link{netmeta}},
  \code{\link{netposet}}, \code{\link{hasse}},
  \code{\link[metabook]{Linde2015}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}, Clément
  Palpacuer \email{clementpalpacuer@gmail.com}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rankogram.netmeta.R
\name{rankogram.netmeta}
\alias{rankogram.netmeta}
\title{Calculate rankogram}
\usage{
\method{rankogram}{netmeta}(
  x,
  nsim = gs("nsim"),
  common = x$common,
  random = x$random,
  small.values = x$small.values,
  cumulative.rankprob = FALSE,
  keep.samples = FALSE,
  nchar.trts = x$nchar.trts,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{netmeta}}.}

\item{nsim}{Number of repetitions.}

\item{common}{A logical indicating to compute ranking probabilities
and SUCRAs for the common effects model.}

\item{random}{A logical indicating to compute ranking probabilities
and SUCRAs for the random effects model.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect, can be abbreviated.}

\item{cumulative.rankprob}{A logical indicating whether cumulative
ranking probabilities should be printed.}

\item{keep.samples}{A logical indicating whether to keep the generated
samples.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (ignored).}
}
\value{
An object of class \code{rankogram} with corresponding \code{print}
and \code{plot} function. The object is a list containing the
following components:
\item{ranking.matrix.common}{Numeric matrix giving the probability
  of each treatment being at each possible rank for the common
  effects model.}
\item{ranking.common}{SUCRA values for the common effects model.}
\item{ranking.matrix.random}{Numeric matrix giving the probability
  of each treatment being at each possible rank for the random
  effects model.}
\item{ranking.random}{SUCRA values for the random effects model.}
\item{cumrank.matrix.common}{Numeric matrix giving the cumulative
  ranking probability of each treatment for the
  common effects model.}
\item{cumrank.matrix.random}{Numeric matrix giving the cumulative
  ranking probability of each treatment for the random effects
  model.}
\item{nsim, common, random}{As defined above},
\item{small.values, x}{As defined above},
}
\description{
This function calculates the probabilities of each treatment being
at each possible rank and the SUCRAs (Surface Under the Cumulative
RAnking curve) in frequentist network meta-analysis.
}
\details{
We derive a matrix showing the probability of each treatment being
at each possible rank. To this aim, we use resampling from a
multivariate normal distribution with estimated network effects as
means and corresponding estimated variance covariance matrix. We
then summarise them using the ranking metric SUCRAs (Surface Under
the Cumulative RAnking curve).
}
\examples{
pw <- pairwise(treatment, event = r, n = N, studlab = author,
  data = Woods2010, sm = "OR")
nma <- netmeta(pw, small.values = "desirable")

set.seed(1909) # get reproducible results
rnk <- rankogram(nma, nsim = 100)
rnk
print(rnk, cumulative.rankprob = TRUE)

# Also print mean ranks
summary(rnk)

plot(rnk)

}
\references{
Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
\code{\link{netmeta}}, \code{\link{netrank}},
  \code{\link{plot.rankogram}}, \code{\link[metabook]{Woods2010}}
}
\author{
Theodoros Papakonstantinou \email{dev@tpapak.com}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netsplit.R
\name{netsplit}
\alias{netsplit}
\alias{print.netsplit}
\title{Split direct and indirect evidence in network meta-analysis}
\usage{
netsplit(
  x,
  method,
  upper = TRUE,
  reference.group = x$reference.group,
  baseline.reference = x$baseline.reference,
  show = gs("show"),
  overall = TRUE,
  direct = TRUE,
  indirect = TRUE,
  only.reference = FALSE,
  ci = FALSE,
  test = show \%in\% c("all", "with.direct", "both"),
  order = NULL,
  nchar.trts = x$nchar.trts,
  sep.trts = x$sep.trts,
  quote.trts = "",
  tol.direct = 5e-04,
  common = x$common,
  random = x$random,
  backtransf = x$backtransf,
  warn = FALSE,
  warn.deprecated = gs("warn.deprecated"),
  verbose = FALSE,
  ...
)

\method{print}{netsplit}(
  x,
  common = x$x$common,
  random = x$x$random,
  show = x$show,
  overall = x$overall,
  direct = x$direct,
  indirect = x$indirect,
  ci = x$ci,
  test = x$test,
  only.reference = x$only.reference,
  sortvar = NULL,
  subset = NULL,
  nchar.trts = x$nchar.trts,
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  digits.prop = max(gs("digits.pval") - 2, 2),
  text.NA = gs("lab.NA"),
  backtransf = x$backtransf,
  scientific.pval = gs("scientific.pval"),
  big.mark = gs("big.mark"),
  legend = gs("legend"),
  indent = TRUE,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta} or \code{netsplit}.}

\item{method}{A character string indicating which method to split
direct and indirect evidence is to be used. Either
\code{"Back-calculation"} or \code{"SIDDE"}, can be abbreviated.
See Details.}

\item{upper}{A logical indicating whether treatment comparisons
should be selected from the lower or upper triangle of the
treatment effect matrices (see list elements \code{TE.common} and
\code{TE.random} in the \code{netmeta} object). Ignored if
argument \code{order} is provided.}

\item{reference.group}{Reference treatment. Ignored if argument
\code{order} is provided.}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment or vice versa. This argument is only
considered if \code{reference.group} is not equal to \code{""}
and argument\code{order} is not provided.}

\item{show}{A character string indicating which comparisons should
be printed (see Details).}

\item{overall}{A logical indicating whether estimates from network
meta-analysis should be printed.}

\item{direct}{A logical indicating whether direct estimates should
be printed.}

\item{indirect}{A logical indicating whether indirect estimates
should be printed.}

\item{only.reference}{A logical indicating whether only comparisons
with the reference group should be printed.}

\item{ci}{A logical indicating whether confidence intervals should
be printed in addition to treatment estimates.}

\item{test}{A logical indicating whether results of a test
comparing direct and indirect estimates should be printed.}

\item{order}{A optional character or numerical vector specifying
the order of treatments in comparisons.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{sep.trts}{A character string used in comparison names as
separator between treatment labels, e.g., " vs ".}

\item{quote.trts}{A character used to print around treatment
labels.}

\item{tol.direct}{A numeric defining the maximum deviation of the
direct evidence proportion from 0 or 1 to classify a comparison
as providing only indirect or direct evidence, respectively.}

\item{common}{A logical indicating whether results for the common
effects network meta-analysis should be printed.}

\item{random}{A logical indicating whether results for the random
effects network meta-analysis should be printed.}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. For example, if \code{backtransf =
TRUE}, results for \code{sm = "OR"} are printed as odds ratios
rather than log odds ratios.}

\item{warn}{A logical indicating whether warnings should be
printed.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{verbose}{A logical indicating whether progress information
should be printed.}

\item{\dots}{Additional arguments.}

\item{sortvar}{An optional vector used to sort comparisons (must be
of same length as the total number of comparisons).}

\item{subset}{An optional logical vector specifying a subset of
comparisons to print (must be of same length as the total number of
comparisons) .}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z-value
of test of agreement between direct and indirect evidence, see
\code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of test of agreement between direct and indirect evidence, see
\code{print.default}.}

\item{digits.prop}{Minimal number of significant digits for direct
evidence proportions, see \code{print.default}.}

\item{text.NA}{A character string specifying text printed for
missing values.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{big.mark}{A character used as thousands separator.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{indent}{A logical indicating whether items in the legend
should be indented.}
}
\value{
An object of class \code{netsplit} with corresponding \code{print}
\code{forest}, and \code{plot} functions. The object is a list containing
the following components:
\item{common, random}{As defined above.}
\item{comparison}{A vector with treatment comparisons.}
\item{prop.common, prop.random}{A vector with direct evidence
  proportions (common / random effects model).}
\item{common, random}{Results of network meta-analysis (common /
  random effects model), i.e., data frame with columns comparison,
  TE, seTE, lower, upper, z, and p.}
\item{direct.common, direct.random}{Network meta-analysis results
  based on direct evidence (common / random effects model), i.e.,
  data frame with columns comparison, TE, seTE, lower, upper, z,
  and p.}
\item{indirect.common, indirect.random}{Network meta-analysis
  results based on indirect evidence (common / random effects
  model), i.e., data frame with columns comparison, TE, seTE,
  lower, upper, z, and p.}
\item{compare.common, compare.random}{Comparison of direct and
  indirect evidence in network meta-analysis (common / random
  effects model), i.e., data frame with columns comparison, TE,
  seTE, lower, upper, z, and p.}
\item{sm}{A character string indicating underlying summary measure}
\item{level.ma}{The level used to calculate confidence intervals
  for pooled estimates.}
\item{tictoc}{Computation times for node-splitting method or SIDDE
  (if R package \bold{tictoc} is installed).}
\item{version}{Version of R package netmeta used to create object.}
}
\description{
Methods to split network estimates into the contribution of direct
and indirect evidence and to test for local inconsistency in
network meta-analysis.
}
\details{
A comparison of direct and indirect treatment estimates can serve
as check for consistency of network meta-analysis (Dias et al.,
2010).

This function provides two methods to derive indirect estimates:
\itemize{
\item Separate Indirect from Direct Evidence (SIDE) using a
  back-calculation method (\code{method = "Back-calculation"})
  based on the \emph{direct evidence proportion} to calculate the
  indirect evidence (König et al., 2013);
\item Separate Indirect from Direct Design Evidence (SIDDE) as
  described in Efthimiou et al. (2019).
}

Note, for the back-calculation method, indirect treatment estimates
are already calculated in \code{\link{netmeta}} and this function
combines and prints these estimates in a user-friendly
way. Furthermore, this method is not available for the
Mantel-Haenszel and non-central hypergeometric distribution
approach implemented in \code{\link{netmetabin}}.

For the random-effects model, the direct treatment estimates are
based on the common between-study variance \eqn{\tau^2} from the
network meta-analysis, i.e. the square of list element
\code{x$tau}.

Argument \code{show} determines which comparisons are printed:
\tabular{ll}{
\dQuote{all} \tab All comparisons \cr
\dQuote{both} \tab Only comparisons contributing both direct and
  indirect evidence \cr
\dQuote{with.direct} \tab Comparisons providing direct evidence \cr
\dQuote{direct.only} \tab Comparisons providing only direct
  evidence \cr
\dQuote{indirect.only} \tab Comparisons providing only indirect
  evidence
}

Wilson et al. (2026) introduced a scatter plot of direct and indirect
estimates to visualise local consistency in a network meta-analysis.
Alternatively, a forest plot with direct and indirect estimates can be
generated.

The node-splitting method and SIDDE can be compute-intensive in
large networks. Crude information on the computation progress is
printed if argument \code{verbose = TRUE}. In addition, computation
times are printed if R package \bold{tictoc} is installed.
}
\examples{
#
# 1) COPD example
#

pw1 <- pairwise(treatment, event = r, n = N,
  studlab = author, data = Woods2010, sm = "OR")
#
nma1 <- netmeta(pw1)
#
print(netsplit(nma1), digits = 2)

\donttest{
print(netsplit(nma1), digits = 2,
  backtransf = FALSE, common = FALSE)

# Sort by increasing number of studies in direct comparisons
print(netsplit(nma1), digits = 2, sortvar = k)
# Sort by decreasing number of studies in direct comparisons
print(netsplit(nma1), digits = 2, sortvar = -k)

# Sort by increasing evidence proportion under common effects model
print(netsplit(nma1), digits = 2, sortvar = prop.common)
# Sort by decreasing evidence proportion under common effects model
print(netsplit(nma1), digits = 2, sortvar = -prop.common)

# Sort by decreasing evidence proportion under common effects model
# and number of studies
print(netsplit(nma1), digits = 2, sortvar = cbind(-prop.common, -k))

#
# 2) Diabetes example
#

pw2 <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))
nma2 <- netmeta(pw2, reference = "plac")
#
ns2 <- netsplit(nma2)
#
print(ns2, digits = 2)
# Layout of Puhan et al. (2014), Table 1
print(ns2, digits = 2, ci = TRUE, test = FALSE)

# Forest plot showing comparisons contributing both direct and
# indirect evidence
#
forest(ns2, fontsize = 6, spacing = 0.5, addrow.subgroups = FALSE)

# Scatter plot to evaluate local inconsistency
#
plot(ns2)

# Forest plot showing comparisons contributing direct evidence
#
forest(ns2, fontsize = 6, spacing = 0.5, addrow.subgroups = FALSE,
  show = "with.direct")

# Forest plot only showing network estimates compared to reference
# group and prediction intervals
#
forest(ns2, fontsize = 8, spacing = 0.75, show = "all",
  only.reference = TRUE, prediction = TRUE,
  direct = FALSE, indirect = FALSE)

#
# 3) Another COPD example
#

pw3 <- pairwise(treatment, death, randomized, studlab = id,
  data = Dong2013, sm = "OR")
nma3 <- netmetabin(pw3)
netsplit(nma3)
}

}
\references{
Dias S, Welton NJ, Caldwell DM, Ades AE (2010):
Checking consistency in mixed treatment comparison meta-analysis.
\emph{Statistics in Medicine},
\bold{29}, 932--44

Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
(2019):
A Mantel-Haenszel model for network meta-analysis of rare events.
\emph{Statistics in Medicine},
\bold{38}, 2992--3012

König J, Krahn U, Binder H (2013):
Visualizing the flow of evidence in network meta-analysis and
characterizing mixed treatment comparisons.
\emph{Statistics in Medicine},
\bold{32}, 5414--29

Puhan MA, Schünemann HJ, Murad MH, et al. (2014):
A GRADE working group approach for rating the quality of treatment
effect estimates from network meta-analysis.
\emph{British Medical Journal},
\bold{349}, g5630

Wilson H, Schönstein A, Robson S, Bonofiglio F (2026):
A novel approach for visualizing local consistency in
network meta-analysis.
\emph{Research Synthesis Methods},
1--15
}
\seealso{
\code{\link{forest.netsplit}}, \code{\link{plot.netsplit}},
  \code{\link{netmeta}}, \code{\link{netmetabin}},
  \code{\link{netmeasures}},
  \code{\link[metabook]{Woods2010}}, \code{\link{Senn2013}},
  \code{\link[metabook]{Dong2013}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}, Gerta
  Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Orestis Efthimiou
  \email{oremiou@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rankogram.default.R
\name{rankogram.default}
\alias{rankogram.default}
\title{Calculate rankogram from treatment effect samples}
\usage{
\method{rankogram}{default}(
  x,
  pooled = "unspecified",
  small.values = "desirable",
  cumulative.rankprob = FALSE,
  keep.samples = FALSE,
  nchar.trts = gs("nchar.trts"),
  ...
)
}
\arguments{
\item{x}{A matrix or data frame with treatment effects in columns and
samples in rows.}

\item{pooled}{A character string indicating whether samples come from
a common (\code{"common"}), random effects (\code{"random"}), or
\code{"unspecified"} model, can be abbreviated.}

\item{small.values}{An optional character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect, can be abbreviated.}

\item{cumulative.rankprob}{A logical indicating whether cumulative
ranking probabilities should be printed.}

\item{keep.samples}{A logical indicating whether to keep the generated
samples.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{\dots}{Additional arguments (ignored).}
}
\value{
An object of class \code{rankogram} with corresponding \code{print}
and \code{plot} function. The object is a list containing the
following components:
\item{ranking.matrix.common}{Numeric matrix giving the probability
  of each treatment being at each possible rank for the common
  effects model.}
\item{ranking.common}{SUCRA values for the common effects model.}
\item{ranking.matrix.random}{Numeric matrix giving the probability
  of each treatment being at each possible rank for the random
  effects model.}
\item{ranking.random}{SUCRA values for the random effects model.}
\item{cumrank.matrix.common}{Numeric matrix giving the cumulative
  ranking probability of each treatment for the
  common effects model.}
\item{cumrank.matrix.random}{Numeric matrix giving the cumulative
  ranking probability of each treatment for the random effects
  model.}
\item{nsim, common, random}{As defined above},
\item{small.values, x}{As defined above},
}
\description{
This function calculates the probabilities of each treatment being
at each possible rank and the SUCRAs (Surface Under the Cumulative
RAnking curve) from a sample of treatment estimates in network
meta-analysis.
}
\details{
We derive a matrix showing the probability of each treatment being
at each possible rank. To this aim, we use samples and
summarise them using the ranking metric SUCRAs (Surface Under
the Cumulative RAnking curve).

The matrix / data frame in argument \code{x} must contain the sampled
effects for each treatment.
}
\examples{
pw <- pairwise(treatment, event = r, n = N, studlab = author,
  data = Woods2010, sm = "OR")
nma <- netmeta(pw, small.values = "desirable")

set.seed(1909) # get reproducible results
rnk <- rankogram(nma, common = FALSE, nsim = 10, # reduce runtime
  keep.samples = TRUE)
rnk

rankogram(rnk$samples.random)

}
\references{
Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
\code{\link{netmeta}}, \code{\link{netrank}},
  \code{\link{plot.rankogram}},
  \code{\link[metabook]{Woods2010}}
}
\author{
Theodoros Papakonstantinou \email{dev@tpapak.com}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Woods2010.R
\docType{data}
\name{Woods2010}
\alias{Woods2010}
\title{Count statistics of survival data}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{author}}\tab first author / study name \cr
\bold{\emph{treatment}}\tab treatment \cr
\bold{\emph{r}}\tab number of deaths in treatment arm \cr
\bold{\emph{N}}\tab number of patients in treatment arm
}
}
\source{
Woods BS, Hawkins N, Scott DA (2010):
Network meta-analysis on the log-hazard scale, combining count and
hazard ratio statistics accounting for multi-arm trials: A
tutorial.
\emph{BMC Medical Research Methodology},
\bold{10}, 54
}
\description{
Count mortality statistics in randomised controlled trials of
treatments for chronic obstructive pulmonary disease (Woods et
al. (2010), Table 1).
}
\note{
The dataset Woods2010 is identical to dataset
\code{\link[metabook]{Woods2010}} in R package \bold{metabook}.
}
\examples{
head(Woods2010)

# Example using pairwise() and netmeta():
# example(Woods2010, package = "metabook")
}
\seealso{
\code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
  \code{\link{netmeta}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.rankogram.R
\name{print.rankogram}
\alias{print.rankogram}
\title{Print method for rankograms}
\usage{
\method{print}{rankogram}(
  x,
  common = x$common,
  random = x$random,
  cumulative.rankprob = x$cumulative.rankprob,
  sort = TRUE,
  nchar.trts = x$nchar.trts,
  digits = gs("digits.prop"),
  details.methods = gs("details"),
  legend = gs("legend"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An R object of class \code{rankogram}.}

\item{common}{A logical indicating to print ranking probabilities
and SUCRAs for the common effects model.}

\item{random}{A logical indicating to print ranking probabilities
and SUCRAs for the random effects model.}

\item{cumulative.rankprob}{A logical indicating whether cumulative
ranking probabilities should be printed.}

\item{sort}{A logical indicating whether treatments should be
sorted by decreasing SUCRAs.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{digits}{Minimal number of significant digits, see
\code{\link{print.default}}.}

\item{details.methods}{A logical specifying whether details on statistical
methods should be printed.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments for printing.}
}
\description{
Print method for objects of class \code{rankogram}.
}
\examples{
# Examples: example(rankogram.netmeta)

}
\references{
Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
\code{\link{rankogram}}, \code{\link{plot.rankogram}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.summary.netcomb.R
\name{print.summary.netcomb}
\alias{print.summary.netcomb}
\title{Print detailed information for component network meta-analysis}
\usage{
\method{print}{summary.netcomb}(
  x,
  common = x$x$common,
  random = x$x$random,
  overall.hetstat = x$overall.hetstat,
  backtransf = x$backtransf,
  nchar.comps = x$nchar.comps,
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  digits.pval.Q = max(gs("digits.pval.Q"), 2),
  digits.Q = gs("digits.Q"),
  big.mark = gs("big.mark"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  print.tau2 = gs("print.tau2"),
  print.tau = gs("print.tau"),
  print.Q = gs("print.Q"),
  print.I2 = gs("print.I2"),
  print.I2.ci = gs("print.I2.ci"),
  details.methods = gs("details"),
  legend = gs("legend"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{summary.netcomb}}

\item{common}{A logical indicating whether results for the common
effects model should be printed.}

\item{random}{A logical indicating whether results for the random
effects model should be printed.}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf=TRUE}, results for \code{sm="OR"} are presented
as odds ratios rather than log odds ratios, for example.}

\item{nchar.comps}{A numeric defining the minimum number of
characters used to create unique component names.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-value, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall treatment effect, see \code{print.default}.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity tests, see \code{print.default}.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistics, see \code{print.default}.}

\item{big.mark}{A character used as thousands separator.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{print.tau2}{A logical specifying whether between-study
variance \eqn{\tau^2} should be printed.}

\item{print.tau}{A logical specifying whether \eqn{\tau}, the
square root of the between-study variance \eqn{\tau^2}, should be
printed.}

\item{print.Q}{A logical value indicating whether to print the
results of the test of heterogeneity.}

\item{print.I2}{A logical specifying whether heterogeneity
statistic I\eqn{^2} should be printed.}

\item{print.I2.ci}{A logical specifying whether confidence interval for
heterogeneity statistic I\eqn{^2} should be printed.}

\item{details.methods}{A logical specifying whether details on statistical
methods should be printed.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments.}
}
\description{
Print detailed information  for component network meta-analysis.
}
\examples{
# Examples: example(summary.netcomb)

}
\seealso{
\code{\link{netcomb}}, \code{\link{discomb}},
  \code{\link{summary.netcomb}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.netcomb.R
\name{summary.netcomb}
\alias{summary.netcomb}
\title{Summary method for objects of class netcomb}
\usage{
\method{summary}{netcomb}(
  object,
  common = object$common,
  random = object$random,
  overall.hetstat = object$overall.hetstat,
  backtransf = object$backtransf,
  nchar.comps = object$nchar.comps,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{object}{An object of class \code{netcomb}.}

\item{common}{A logical indicating whether results for the common
effects model should be printed.}

\item{random}{A logical indicating whether results for the random
effects model should be printed.}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots.}

\item{nchar.comps}{A numeric defining the minimum number of
characters used to create unique component names.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
A list of class "summary.netcomb" is returned with the following elements:

\item{k}{Total number of studies.}
\item{m}{Total number of pairwise comparisons.}
\item{n}{Total number of treatments.}
\item{d}{Total number of designs (corresponding to the unique set
  of treatments compared within studies).}
\item{c}{Total number of components.}
\item{s}{Total number of subnetworks (for \code{\link{discomb}}).}

\item{trts}{Treatments included in network meta-analysis.}
\item{k.trts}{Number of studies evaluating a treatment.}
\item{n.trts}{Number of observations receiving a treatment (if
  available).}
\item{events.trts}{Number of events observed for a treatment (if
  available).}

\item{studies}{Study labels coerced into a factor with its levels
  sorted alphabetically.}
\item{narms}{Number of arms for each study.}

\item{designs}{Vector with unique designs present in the network. A
  design corresponds to the set of treatments compared within a
  study.}

\item{comps}{Components included in network meta-analysis.}
\item{k.comps}{Number of studies evaluating a component.}
\item{n.comps}{Number of observations receiving a component (if
  available).}
\item{events.comps}{Number of events observed for a component (if
  available).}

\item{comparison}{Results for pairwise comparisons (data frame with
  columns studlab, treat1, treat2, TE, seTE, lower, upper, z, p).}
\item{comparison.nma.common}{Results for pairwise comparisons based
  on common effects NMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p, leverage).}
\item{comparison.nma.random}{Results for pairwise comparisons based
  on random effects NMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p).}
\item{comparison.cnma.common}{Results for pairwise comparisons based
  on common effects CNMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p, leverage).}
\item{comparison.cnma.random}{Results for pairwise comparisons based
  on random effects CNMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p).}

\item{components.common}{Results for components based
  on common effects CNMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p, leverage).}
\item{components.random}{Results for components based
  on random effects CNMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p).}

\item{combinations.common}{Results for available combinations based
  on common effects CNMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p, leverage).}
\item{combinations.random}{Results for available combinations based
  on random effects CNMA model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p).}

\item{common}{Results for common effects model (a list with elements
  TE, seTE, lower, upper, z, p).}
\item{random}{Results for random effects model (a list with
  elements TE, seTE, lower, upper, z, p).}
\item{predict}{Prediction intervals (a list with elements seTE,
  lower, upper).}

\item{Q.additive}{Overall heterogeneity / inconsistency statistic.}
\item{df.Q.additive}{Degrees of freedom for test of heterogeneity /
  inconsistency.}
\item{pval.Q.additive}{P-value for test of heterogeneity / inconsistency.}
\item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
  limits.}
\item{tau}{Square-root of between-study variance.}

\item{Q.additive}{Overall heterogeneity / inconsistency statistic (CNMA model).}
\item{df.Q.additive}{Degrees of freedom for test of heterogeneity /
  inconsistency (CNMA model).}
\item{pval.Q.additive}{P-value for test of heterogeneity / inconsistency
  (CNMA model).}

\item{Q.standard}{Overall heterogeneity statistic (NMA model).}
\item{df.Q.heterogeneity}{Degrees of freedom for test of overall
  heterogeneity (NMA model).}
\item{pval.Q.heterogeneity}{P-value for test of overall
  heterogeneity (NMA model).}

\item{Q.diff}{Q statistic for difference between CNMA and NMA model.}
\item{df.Q.diff, pval.Q.diff}{Corresponding degrees of freedom and p-value.}

\item{sm}{A character string indicating underlying summary
  measure.}
\item{method}{A character string indicating which method is to be
  used for pooling of studies.}
\item{level}{The level used to calculate confidence intervals for
  individual studies.}
\item{level.ma}{The level used to calculate confidence intervals
  for pooled estimates.}

\item{ci.lab}{Label for confidence interval.}

\item{reference.group, baseline.reference}{As defined above.}
\item{all.treatments}{As defined above.}
\item{seq}{A character specifying the sequence of treatments.}

\item{tau.preset}{An optional value for the square-root of the
  between-study variance \eqn{\tau^2}.}

\item{sep.comps}{A character used in comparison names as separator
  between component labels.}
\item{nchar.comps}{A numeric defining the minimum number of
  characters used to create unique component names.}

\item{overall.hetstat, backtransf}{As defined above.}
 
\item{title}{Title of meta-analysis / systematic review.}
\item{x}{\code{\link{netmeta}} object (if available).}
\item{call}{Function call.}
\item{version}{Version of R package \bold{netmeta} used to create object.}
}
\description{
Summary method for objects of class \code{netcomb}.
}
\examples{
# Examples: example(netcomb)

}
\seealso{
\code{\link{netcomb}}, \code{\link{discomb}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netgraph.netcomb.R
\name{netgraph.netcomb}
\alias{netgraph.netcomb}
\title{Network graph for objects of class netcomb}
\usage{
\method{netgraph}{netcomb}(x, ...)
}
\arguments{
\item{x}{An object of class \code{netcomb}.}

\item{\dots}{Additional arguments passed on to
\code{\link{netgraph.netmeta}}.}
}
\description{
This function generates a graph of the evidence network.
}
\examples{
# Only consider studies including Face-to-face PST (to reduce
# runtime of example)
#
face <- subset(Linde2016, id \%in\% c(16, 24, 49, 118))

# Use pairwise() to transform data to comparison-based format
#
pw <- pairwise(treat = int,
  event = resp, n = n, studlab = paste(author, year),
  data = face, reference = "plac", sm = "OR")

# Conduct random effects network meta-analysis
#
nma <- netmeta(pw, common = FALSE)

# Additive model for treatment components (with placebo as inactive
# treatment)
#
cnma <- netcomb(nma, inactive = "placebo")

netgraph(cnma)

}
\seealso{
\code{\link{netcomb}}, \code{\link{netgraph.netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Linde2015.R
\docType{data}
\name{Linde2015}
\alias{Linde2015}
\title{Network meta-analysis of treatments for depression}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{id}}\tab Study ID \cr
\bold{\emph{author}}\tab First author \cr
\bold{\emph{year}}\tab Publication year \cr
\bold{\emph{treatment1}}\tab First treatment \cr
\bold{\emph{treatment2}}\tab Second treatment \cr
\bold{\emph{treatment3}}\tab Third treatment \cr
\bold{\emph{n1}}\tab Number of patients receiving first treatment
  \cr
\bold{\emph{resp1}}\tab Number of early responder (treatment 1) \cr
\bold{\emph{remi1}}\tab Number of early remissions (treatment 1)
  \cr
\bold{\emph{loss1}}\tab Number of patients loss to follow-up
  (treatment 1) \cr
\bold{\emph{loss.ae1}}\tab Number of patients loss to follow-up due
  to adverse events (treatment 1) \cr
\bold{\emph{ae1}}\tab Number of patients with adverse events
  (treatment 1) \cr
\bold{\emph{n2}}\tab Number of patients receiving second treatment
  \cr
\bold{\emph{resp2}}\tab Number of early responder (treatment 2) \cr
\bold{\emph{remi2}}\tab Number of early remissions (treatment 2)
  \cr
\bold{\emph{loss2}}\tab Number of patients loss to follow-up
  (treatment 2) \cr
\bold{\emph{loss.ae2}}\tab Number of patients loss to follow-up due
  to adverse events (treatment 2) \cr
\bold{\emph{ae2}}\tab Number of patients with adverse events
  (treatment 2) \cr
\bold{\emph{n3}}\tab Number of patients receiving third treatment
  \cr
\bold{\emph{resp3}}\tab Number of early responder (treatment 3) \cr
\bold{\emph{remi3}}\tab Number of early remissions (treatment 3)
  \cr
\bold{\emph{loss3}}\tab Number of patients loss to follow-up
  (treatment 3) \cr
\bold{\emph{loss.ae3}}\tab Number of patients loss to follow-up due
  to adverse events (treatment 3) \cr
\bold{\emph{ae3}}\tab Number of patients with adverse events
  (treatment 3) 
}
}
\source{
Linde K, Kriston L, Rücker G, et al. (2015):
Efficacy and acceptability of pharmacological treatments for
depressive disorders in primary care: Systematic review and network
meta-analysis.
\emph{Annals of Family Medicine},
\bold{13}, 69--79
}
\description{
Network meta-analysis of nine classes of antidepressants including
placebo for the primary care setting; partly shown in Linde et
al. (2015), supplementary Table 2.
}
\note{
The dataset Linde2015 is identical to dataset
\code{\link[metabook]{Linde2015}} in R package \bold{metabook}.
}
\examples{
head(Linde2015)

# Example using pairwise(), netmeta(), netrank() and netposet():
# example(Linde2015, package = "metabook")
}
\seealso{
\code{\link[metabook]{Dong2013}},
  \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
  \code{\link{netmeta}}, \code{\link{netposet}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rankogram.R
\name{rankogram}
\alias{rankogram}
\title{Generic function for rankograms}
\usage{
rankogram(x, ...)
}
\arguments{
\item{x}{An R object.}

\item{\dots}{Additional arguments.}
}
\description{
Generic function to calculate the probabilities of each treatment being
at each possible rank and the SUCRAs (Surface Under the Cumulative
RAnking curve) in network meta-analysis.
}
\details{
For more details, look at the following functions to generate
rankograms:
\itemize{
\item \code{\link{rankogram.netmeta}}
\item \code{\link{rankogram.default}}
}
}
\examples{
# Examples:
# example(rankogram.netmeta)
# example(rankogram.default)

}
\references{
Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
\code{\link{netmeta}}, \code{\link{netrank}},
  \code{\link{plot.rankogram}}, \code{\link[metabook]{Woods2010}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.subgroup.netmeta.R
\name{forest.subgroup.netmeta}
\alias{forest.subgroup.netmeta}
\alias{plot.subgroup.netmeta}
\title{Forest plot showing results of network meta-analysis with subgroups}
\usage{
\method{forest}{subgroup.netmeta}(
  x,
  pooled = ifelse(x$x$random, "random", "common"),
  equal.size = gs("equal.size"),
  leftcols = c("studlab", "Q", "df.Q", "pval.Q"),
  leftlabs = c("Comparison /\\nSubgroup", "Q", "d.f.", "p-value"),
  rightcols = c("effect", "ci", "k", if (pooled == "random") "tau"),
  rightlabs = c(NA, NA, "Number of\\nStudies", if (pooled == "random") "Tau"),
  calcwidth.subgroup = gs("calcwidth.subgroup"),
  digits = gs("digits.forest"),
  digits.Q = gs("digits.Q"),
  digits.pval.Q = gs("digits.pval.Q"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  sep.trts = " vs ",
  backtransf = x$x$backtransf,
  lab.NA = ".",
  smlab,
  col.subgroup = "black",
  ...
)

\method{plot}{subgroup.netmeta}(x, ...)
}
\arguments{
\item{x}{An object of class \code{subgroup.netmeta}.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{equal.size}{A logical indicating whether all squares should
be of equal size. Otherwise, the square size is proportional to
the precision of estimates.}

\item{leftcols}{A character vector specifying columns to be plotted
on the left side of the forest plot (see Details).}

\item{leftlabs}{A character vector specifying labels for columns on
left side of the forest plot.}

\item{rightcols}{A character vector specifying columns to be
plotted on the right side of the forest plot (see Details).}

\item{rightlabs}{A character vector specifying labels for columns
on right side of the forest plot.}

\item{calcwidth.subgroup}{A logical indicating whether text with
comparison labels should be considered to calculate width of the
column with treatment labels.}

\item{digits}{Minimal number of significant digits for treatment
effects and confidence intervals, see \code{print.default}.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistic Q, see \code{print.default}.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity test, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance \eqn{\tau^2}, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for
\eqn{\tau}, the square root of the between-study variance
\eqn{\tau^2}.}

\item{sep.trts}{A character string used to label treatment comparisons.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{lab.NA}{A character string to label missing values.}

\item{smlab}{A label printed at top of figure. By default, text
indicating either common or random effects model is printed.}

\item{col.subgroup}{The colour to print information on subgroups.}

\item{\dots}{Additional arguments for \code{\link[meta]{forest.meta}}
function.}
}
\description{
Forest plot to show subgroup estimates of network meta-analysis.
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window.

The arguments \code{leftcols} and \code{rightcols} can be used to
specify columns which are plotted on the left and right side of the
forest plot, respectively. If argument \code{rightcols} is
\code{FALSE}, no columns will be plotted on the right side.

For more information see help page of \code{\link[meta]{forest.meta}}
function.
}
\examples{
\dontrun{
data("Senn2013")
# Add variable with (fictitious) risk of bias values
Senn2013$rob <- NA
set.seed(1909)
for (i in unique(Senn2013$studlab))
  Senn2013$rob[Senn2013$studlab == i] <- sample(1:3, 1)
Senn2013$rob <- factor(Senn2013$rob, levels = 1:3,
  labels = c("low", "moderate", "high"))
# Conduct network meta-analysis
net <- netmeta(TE, seTE, treat1.long, treat2.long, studlab,
  data = Senn2013, sm = "MD", reference = "plac", nchar.trts = 4)
# Conduct subgroup network meta-analysis
sg <- subgroup(net, rob, common = FALSE)
sg
# Forest plot
forest(sg)
}

}
\seealso{
\code{\link{subgroup.netmeta}}, \code{\link{netmeta}},
  \code{\link[meta]{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hasse.R
\name{hasse.netposet}
\alias{hasse.netposet}
\alias{hasse}
\title{Hasse diagram}
\usage{
\method{hasse}{netposet}(
  x,
  pooled = ifelse(x$random, "random", "common"),
  newpage = TRUE,
  shape = "roundrect",
  col.lines = "black",
  col.nodes = "black",
  lwd = 1,
  ...
)

hasse(x, ...)
}
\arguments{
\item{x}{An object of class \code{netposet} (mandatory).}

\item{pooled}{A character string indicating whether Hasse diagram
show be drawn for common (\code{"common"}) or random effects model
(\code{"random"}). Can be abbreviated.}

\item{newpage}{A logical value indicating whether a new figure
should be printed in an existing graphics window. Otherwise, the
Hasse diagram is added to the existing figure.}

\item{shape}{Shape of node borders, either \code{"roundrect"},
\code{"rect"}, or \code{"none"}, can be abbreviated.}

\item{col.lines}{Line colour.}

\item{col.nodes}{Colour for treatment node borders.}

\item{lwd}{Width of lines and node borders.}

\item{\dots}{Additional arguments (ignored).}
}
\description{
This function generates a Hasse diagram for a partial order of
treatment ranks in a network meta-analysis.
}
\details{
Generate a Hasse diagram (Carlsen & Bruggemann, 2014) for a partial
order of treatment ranks in a network meta-analysis (Rücker &
Schwarzer, 2017).

This R function is a wrapper function for a modified version of R function
\code{hasse} in R package \bold{hasseDiagram}
(Krzysztof Ciomek, \url{https://github.com/kciomek/hasseDiagram}) which is
available under the MIT license.
}
\examples{
\dontrun{
# Only run example if R package 'Rgraphviz' from Bioconductor is available
#
if (requireNamespace("Rgraphviz", quietly = TRUE)) {
# Define order of treatments in depression dataset Linde2015
#
trts <- c("TCA", "SSRI", "SNRI", "NRI",
  "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")

# Outcome labels
#
outcomes <- c("Early response", "Early remission")

# (1) Early response
#
pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(resp1, resp2, resp3),
  n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma1 <- netmeta(pw1, common = FALSE,
  seq = trts, ref = "Placebo", small.values = "undesirable")

# (2) Early remission
#
pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(remi1, remi2, remi3),
  n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma2 <- netmeta(pw2, common = FALSE,
  seq = trts, ref = "Placebo", small.values = "undesirable")

# Partial order of treatment rankings
#
po <- netposet(netrank(nma1), netrank(nma2), outcomes = outcomes)

# Hasse diagram
#
hasse(po)
}
}

}
\references{
Carlsen L, Bruggemann R (2014):
Partial order methodology: a valuable tool in chemometrics.
\emph{Journal of Chemometrics},
\bold{28}, 226--34

Rücker G, Schwarzer G (2017):
Resolve conflicting rankings of outcomes in network meta-analysis:
Partial ordering of treatments.
\emph{Research Synthesis Methods},
\bold{8}, 526--36
}
\seealso{
\code{\link{netmeta}}, \code{\link{netposet}},
  \code{\link{netrank}}, \code{\link{plot.netrank}},
  \code{\link[metabook]{Linde2015}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.netmeta.R
\name{print.netmeta}
\alias{print.netmeta}
\title{Print method for objects of class netmeta}
\usage{
\method{print}{netmeta}(
  x,
  common = x$common,
  random = x$random,
  prediction = x$prediction,
  reference.group = x$reference.group,
  baseline.reference = x$baseline.reference,
  all.treatments = x$all.treatments,
  overall.hetstat = x$overall.hetstat,
  backtransf = x$backtransf,
  nchar.trts = x$nchar.trts,
  header = TRUE,
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = max(gs("digits.pval"), 2),
  digits.pval.Q = max(gs("digits.pval.Q"), 2),
  digits.Q = gs("digits.Q"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.I2 = gs("digits.I2"),
  big.mark = gs("big.mark"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  print.tau2 = gs("print.tau2"),
  print.tau = gs("print.tau"),
  print.Q = gs("print.Q"),
  print.I2 = gs("print.I2"),
  print.I2.ci = gs("print.I2.ci"),
  text.tau2 = gs("text.tau2"),
  text.tau = gs("text.tau"),
  text.I2 = gs("text.I2"),
  details.methods = gs("details"),
  legend = gs("legend"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{common}{A logical indicating whether results for the common
effects model should be printed.}

\item{random}{A logical indicating whether results for the random
effects model should be printed.}

\item{prediction}{A logical indicating whether prediction intervals
should be printed.}

\item{reference.group}{Reference treatment.}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{all.treatments}{A logical or \code{"NULL"}. If \code{TRUE},
matrices with all treatment effects, and confidence limits will
be printed.}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf = TRUE}, results for \code{sm = "OR"} are
presented as odds ratios rather than log odds ratios, for
example.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{header}{A logical indicating whether information on title of
meta-analysis, comparison and outcome should be printed at the
beginning of the printout.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for tests
of overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall effects, see \code{print.default}.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity tests, see \code{print.default}.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistics, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for square
root of between-study variance, see \code{print.default}.}

\item{digits.I2}{Minimal number of significant digits for I-squared
statistic, see \code{print.default}.}

\item{big.mark}{A character used as thousands separator.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{print.tau2}{A logical specifying whether between-study
variance \eqn{\tau^2} should be printed.}

\item{print.tau}{A logical specifying whether \eqn{\tau}, the
square root of the between-study variance \eqn{\tau^2}, should be
printed.}

\item{print.Q}{A logical value indicating whether to print the
results of the test of heterogeneity.}

\item{print.I2}{A logical specifying whether heterogeneity
statistic I\eqn{^2} should be printed.}

\item{print.I2.ci}{A logical specifying whether confidence interval for
heterogeneity statistic I\eqn{^2} should be printed.}

\item{text.tau2}{Text printed to identify between-study variance
\eqn{\tau^2}.}

\item{text.tau}{Text printed to identify \eqn{\tau}, the square
root of the between-study variance \eqn{\tau^2}.}

\item{text.I2}{Text printed to identify heterogeneity statistic
I\eqn{^2}.}

\item{details.methods}{A logical specifying whether details on statistical
methods should be printed.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments.}
}
\description{
Print method for objects of class \code{netmeta}.
}
\examples{
# Examples: example(netmeta)

}
\seealso{
\code{\link{netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discomb.R
\name{discomb}
\alias{discomb}
\title{Additive network meta-analysis for combinations of treatments
(disconnected networks)}
\usage{
discomb(
  TE,
  seTE,
  treat1,
  treat2,
  studlab,
  data = NULL,
  subset = NULL,
  inactive = NULL,
  sep.comps = gs("sep.comps"),
  C.matrix,
  sm,
  level = gs("level"),
  level.ma = gs("level.ma"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  reference.group,
  baseline.reference = gs("baseline.reference"),
  seq = gs("sep"),
  tau.preset = NULL,
  tol.multiarm = gs("tol.multiarm"),
  tol.multiarm.se = gs("tol.multiarm.se"),
  details.chkmultiarm = gs("details.chkmultiarm"),
  details.chkident = FALSE,
  sep.trts = gs("sep.trts"),
  nchar.comps = gs("nchar.comps"),
  sep.ia = gs("sep.ia"),
  func.inverse = invmat,
  n1 = NULL,
  n2 = NULL,
  event1 = NULL,
  event2 = NULL,
  incr = NULL,
  overall.hetstat = gs("overall.hetstat"),
  backtransf = gs("backtransf"),
  na.unident = gs("na.unident"),
  title = gs("title"),
  keepdata = gs("keepdata"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  nchar.trts = nchar.comps,
  ...
)
}
\arguments{
\item{TE}{Estimate of treatment effect, i.e. difference between
first and second treatment (e.g. log odds ratio, mean difference,
or log hazard ratio). Or an R object created with
\code{\link[meta]{pairwise}}.}

\item{seTE}{Standard error of treatment estimate.}

\item{treat1}{Label/Number for first treatment.}

\item{treat2}{Label/Number for second treatment.}

\item{studlab}{An optional - but important! - vector with study
labels (see \code{\link{netmeta}}).}

\item{data}{An optional data frame containing the study
information.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{inactive}{A character string defining the inactive treatment
component (see Details).}

\item{sep.comps}{A single character to define separator between
treatment components.}

\item{C.matrix}{C matrix (see Details).}

\item{sm}{A character string indicating underlying summary measure,
e.g., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"},
\code{"HR"}, \code{"MD"}, \code{"SMD"}, or \code{"ROM"}.}

\item{level}{The level used to calculate confidence intervals for
individual comparisons.}

\item{level.ma}{The level used to calculate confidence intervals
for network estimates.}

\item{common}{A logical indicating whether a common effects /
common effects network meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects network
meta-analysis should be conducted.}

\item{reference.group}{Reference treatment (first treatment is used
if argument is missing).}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments in printouts.}

\item{tau.preset}{An optional value for the square-root of the
between-study variance \eqn{\tau^2}.}

\item{tol.multiarm}{A numeric for the tolerance for consistency of
treatment estimates in multi-arm studies which are consistent by
design.}

\item{tol.multiarm.se}{A numeric for the tolerance for consistency
of standard errors in multi-arm studies which are consistent by
design. This check is not conducted if the argument is
\code{NULL}.}

\item{details.chkmultiarm}{A logical indicating whether treatment
estimates and / or variances of multi-arm studies with
inconsistent results or negative multi-arm variances should be
printed.}

\item{details.chkident}{A logical indicating whether details on
unidentifiable components should be printed.}

\item{sep.trts}{A character used in comparison names as separator
between treatment labels.}

\item{nchar.comps}{A numeric defining the minimum number of
characters used to create unique names for components (see
Details).}

\item{sep.ia}{A single character to define separator for interactions.}

\item{func.inverse}{R function used to calculate the pseudoinverse
of the Laplacian matrix L (see \code{\link{netmeta}}).}

\item{n1}{Number of observations in first treatment group.}

\item{n2}{Number of observations in second treatment group.}

\item{event1}{Number of events in first treatment group.}

\item{event2}{Number of events in second treatment group.}

\item{incr}{Numerical value added to cell frequencies.}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf = TRUE}, results for \code{sm = "OR"} are
presented as odds ratios rather than log odds ratios, for
example.}

\item{na.unident}{A logical indicating whether unidentifiable
components and combinations should be set to missing values.}

\item{title}{Title of meta-analysis / systematic review.}

\item{keepdata}{A logical indicating whether original data(set)
should be kept in netmeta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if studies are excluded from meta-analysis due to zero
standard errors).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{nchar.trts}{Deprecated argument (replaced by
\code{nchar.comps}).}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of classes \code{discomb} and \code{netcomb} with
corresponding \code{print}, \code{summary}, and \code{forest}
functions. The object is a list containing the following
components:
\item{studlab}{Study labels.}
\item{treat1}{Label/Number for first treatment.}
\item{treat2}{Label/Number for second treatment.}
\item{TE}{Estimate of treatment effect, i.e. difference between
  first and second treatment.}
\item{seTE}{Standard error of treatment estimate.}
\item{seTE.adj.common, seTE.adj.random}{Standard error of treatment
  estimate, adjusted for multi-arm studies.}
\item{event1}{Number of events in first treatment group.}
\item{event2}{Number of events in second treatment group.}
\item{n1}{Number of observations in first treatment group.}
\item{n2}{Number of observations in second treatment group.}
\item{k}{Total number of studies.}
\item{m}{Total number of pairwise comparisons.}
\item{n}{Total number of treatments.}
\item{d}{Total number of designs (corresponding to the unique set
  of treatments compared within studies).}
\item{c}{Total number of components.}
\item{trts}{Treatments included in network meta-analysis.}
\item{comps}{Unique list of components present in the network.}
\item{TE.cnma.common, TE.cnma.random}{A vector of length \emph{m} of
  consistent treatment effects estimated by the additive (common and
  random effects) model.}
\item{seTE.cnma.common, seTE.cnma.random}{A vector of length
  \emph{m} with standard errors estimated by the additive (common
  and random effects) model.}
\item{lower.cnma.common, lower.cnma.random}{A vector of length
  \emph{m} of lower confidence interval limits for consistent
  treatment effects estimated by the additive (common and random
  effects) model.}
\item{upper.cnma.common, upper.cnma.random}{A vector of length
  \emph{m} of upper confidence interval limits for consistent
  treatment effects estimated by the additive (common and random
  effects) model.}
\item{statistic.cnma.common, statistic.cnma.random}{A vector of
  length \emph{m} of z-values for the test of an overall effect
  estimated by the additive (common and random effects) model.}
\item{pval.cnma.common, pval.cnma.random}{A vector of length
  \emph{m} of p-values for the test of an overall effect estimated
  by the additive (common and random effects) model.}
\item{TE.common, TE.random}{\emph{n}x\emph{n} matrix with overall
  treatment effects estimated by the additive (common and random
  effects) model.}
\item{seTE.common, seTE.random}{\emph{n}x\emph{n} matrix with
  standard errors estimated by the additive (common and random
  effects) model.}
\item{lower.common, upper.common, lower.random,
  upper.random}{\emph{n}x\emph{n} matrices with lower and upper
  confidence interval limits estimated by the additive (common and
  random effects) model.}
\item{statistic.common, pval.common, statistic.random,
  pval.random}{\emph{n}x\emph{n} matrices with z-values and
  p-values for test of overall effect estimated by the additive
  (common and random effects) model.}
\item{Comp.common, Comp.random}{A vector of component effects (common
  and random effects model).}
\item{seComp.common, seComp.random}{A vector with corresponding
  standard errors (common and random effects model).}
\item{lower.Comp.common, lower.Comp.random}{A vector with lower
  confidence limits for components (common and random effects
  model).}
\item{upper.Comp.common, upper.Comp.random}{A vector with upper
  confidence limits for components (common and random effects
  model).}
\item{statistic.Comp.common, statistic.Comp.random}{A vector with
  z-values for the overall effect of components (common and random
  effects model).}
\item{pval.Comp.common, pval.Comp.random}{A vector with p-values for
  the overall effect of components (common and random effects
  model).}
\item{Comb.common, Comb.random}{A vector of combination effects (common
  and random effects model).}
\item{seComb.common, seComb.random}{A vector with corresponding
  standard errors (common and random effects model).}
\item{lower.Comb.common, lower.Comb.random}{A vector with lower
  confidence limits for combinations (common and random effects
  model).}
\item{upper.Comb.common, upper.Comb.random}{A vector with upper
  confidence limits for combinations (common and random effects
  model).}
\item{statistic.Comb.common, statistic.Comb.random}{A vector with
  z-values for the overall effect of combinations (common and random
  effects model).}
\item{pval.Comb.common, pval.Comb.random}{A vector with p-values for
  the overall effect of combinations (common and random effects
  model).}
\item{Q.additive}{Overall heterogeneity / inconsistency statistic
  (additive model).}
\item{df.Q.additive}{Degrees of freedom for test of heterogeneity /
  inconsistency (additive model).}
\item{pval.Q.additive}{P-value for test of heterogeneity /
  inconsistency (additive model).}
\item{tau}{Square-root of between-study variance (additive model).}
\item{I2}{I-squared (additive model).}
\item{Q.standard}{Overall heterogeneity / inconsistency statistic
  (standard model).}
\item{df.Q.standard}{Degrees of freedom for test of heterogeneity /
  inconsistency (standard model).}
\item{pval.Q.standard}{P-value for test of heterogeneity /
  inconsistency (standard model).}
\item{Q.diff}{Test statistic for difference in goodness of fit
  between standard and additive model.}
\item{df.Q.diff}{Degrees of freedom for difference in goodness of
  fit between standard and additive model.}
\item{pval.Q.diff}{P-value for difference in goodness of fit
  between standard and additive model.}
\item{X.matrix}{Design matrix (\emph{m}x\emph{n}).}
\item{B.matrix}{Edge-vertex incidence matrix (\emph{m}x\emph{n}).}
\item{C.matrix}{As defined above.}
\item{sm}{Summary measure.}
\item{level.ma}{Level for confidence intervals.}
\item{common, random, tau.preset}{As defined above.}
\item{sep.trts}{A character used in comparison names as separator
  between treatment labels.}
\item{nchar.comps}{A numeric defining the minimum number of
  characters used to create unique component names.}
\item{inactive, sep.comps}{As defined above.}
\item{backtransf}{A logical indicating whether results should be
  back transformed in printouts and forest plots.}
\item{title}{Title of meta-analysis / systematic review.}
\item{x}{As defined above.}
\item{call}{Function call.}
\item{version}{Version of R package netmeta used to create
  object.}
}
\description{
Some treatments in a network meta-analysis may be combinations of
other treatments or have common components. The influence of
individual components can be evaluated in an additive network
meta-analysis model assuming that the effect of treatment
combinations is the sum of the effects of its components. This
function implements this additive model in a frequentist way and is
particularly intended for disconnected networks.
}
\details{
Treatments in network meta-analysis (NMA) can be complex
interventions. Some treatments may be combinations of others or
have common components. The standard analysis provided by
\code{\link{netmeta}} is a NMA where all existing (single or
combined) treatments are considered as different nodes in the
network. Exploiting the fact that some treatments are combinations
of common components, an additive component network meta-analysis
(CNMA) model can be used to evaluate the influence of individual
components. This model assumes that the effect of a treatment
combination is the sum of the effects of its components which
implies that common components cancel out in comparisons.

This R function can be used for disconnected networks. Use
\code{\link{netmeta}} and \code{\link{netcomb}} for connected
networks.

The additive CNMA model has been implemented using Bayesian methods
(Mills et al., 2012; Welton et al., 2013). This function implements
the additive model in a frequentist way (Rücker et al., 2020).

The underlying multivariate model is given by

\deqn{\bold{\delta} = \bold{B} \bold{\theta}, \bold{\theta} =
\bold{C} \bold{\beta}}

with
\describe{
\item{\eqn{\bold{\delta}}}{vector of true treatment effects
  (differences) from individual studies,}
\item{\eqn{\bold{B}}}{design matrix describing the structure of the
  network,}
\item{\eqn{\bold{\theta}}}{parameter vector that represents the
  existing combined treatments,}
\item{\eqn{\bold{C}}}{matrix describing how the treatments are
  composed,}
\item{\eqn{\bold{\beta}}}{parameter vector representing the
  treatment components.}
}
All parameters are estimated using weighted least squares
regression.

Argument \code{inactive} can be used to specify a single component
that does not have any therapeutic value. Accordingly, it is
assumed that the treatment effect of the combination of this
component with an additional treatment component is equal to the
treatment effect of the additional component alone.

Argument \code{sep.comps} can be used to specify the separator
between individual components. By default, the matrix \strong{C} is
calculated internally from treatment names. However, it is possible
to specify a different matrix using argument \code{C.matrix}.

By default, component names are not abbreviated in
printouts. However, in order to get more concise printouts,
argument \code{nchar.comps} can be used to define the minimum
number of characters for abbreviated component names (see
\code{\link{abbreviate}}, argument \code{minlength}). R function
\code{\link{treats}} is utilised internally to create abbreviated
component names.
}
\note{
This function calculates effects for individual components and
complex interventions present in the network.

R function \code{\link{netcomplex}} can be used to calculate the
effect for arbitrary complex interventions in a component network
meta-analysis. Furthermore, R function \code{\link{netcomparison}}
can be used to calculate the effect for comparisons of two
arbitrary complex intervention in a component network
meta-analysis.
}
\examples{
# Artificial dataset
#
t1 <- c("A + B", "A + C", "A"    , "A"    , "D", "D", "E")
t2 <- c("C"    , "B"    , "B + C", "A + D", "E", "F", "F")
#
mean    <- c(4.1, 2.05, 0, 0, 0.1, 0.1, 0.05)
se.mean <- rep(0.1, 7)
#
study <- paste("study", c(1:4, 5, 5, 5))
#
dat <- data.frame(mean, se.mean, t1, t2, study,
  stringsAsFactors = FALSE)
#
trts <- c("A", "A + B", "A + C", "A + D",
  "B", "B + C", "C", "D", "E", "F")
#
comps <- LETTERS[1:6]

# Use netconnection() to display network information
#
netconnection(t1, t2, study)

dc1 <- discomb(mean, se.mean, t1, t2, study, seq = trts)
dc1

\donttest{
forest(dc1, ref = "F")

# Define C matrix manually (which will produce the same results)
#
C <- rbind(c(1, 0, 0, 0, 0, 0),  # A
  c(1, 1, 0, 0, 0, 0),  # A + B
  c(1, 0, 1, 0, 0, 0),  # A + C
  c(1, 0, 0, 1, 0, 0),  # A + D
  c(0, 1, 0, 0, 0, 0),  # B
  c(0, 1, 1, 0, 0, 0),  # B + C
  c(0, 0, 1, 0, 0, 0),  # C
  c(0, 0, 0, 1, 0, 0),  # D
  c(0, 0, 0, 0, 1, 0),  # E
  c(0, 0, 0, 0, 0, 1))  # F
#                  
colnames(C) <- comps
rownames(C) <- trts
#
dc2 <- discomb(mean, se.mean, t1, t2, study, seq = trts,
  C.matrix = C)
#
# Compare C matrices
#
all.equal(dc1$C.matrix, dc2$C.matrix)
}

}
\references{
König J, Krahn U, Binder H (2013):
Visualizing the flow of evidence in network meta-analysis and
characterizing mixed treatment comparisons.
\emph{Statistics in Medicine},
\bold{32}, 5414--29

Mills EJ, Thorlund K, Ioannidis JP (2012):
Calculating additive treatment effects from multiple randomized
trials provides useful estimates of combination therapies.
\emph{Journal of Clinical Epidemiology},
\bold{65}, 1282--8

Rücker G, Petropoulou M, Schwarzer G (2020):
Network meta-analysis of multicomponent interventions.
\emph{Biometrical Journal},
\bold{62}, 808--21

Welton NJ, Caldwell DM, Adamopoulos E, Vedhara K (2009):
Mixed treatment comparison meta-analysis of complex interventions:
psychological interventions in coronary heart disease.
\emph{American Journal of Epidemiology},
\bold{169}: 1158--65
}
\seealso{
\code{\link{netcomb}}, \code{\link{forest.netcomb}},
  \code{\link{summary.netcomb}}, \code{\link{netmeta}},
  \code{\link{netconnection}}, \code{\link{netcomplex}},
  \code{\link{netcomparison}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netmeta-package.R
\docType{package}
\name{netmeta-package}
\alias{netmeta-package}
\title{netmeta: Brief overview of methods and general hints}
\description{
R package \bold{netmeta} (Balduzzi et al., 2023) provides
frequentist methods for network meta-analysis and supports
Schwarzer et al. (2015), Chapter 8 on network meta-analysis,
\url{https://link.springer.com/book/10.1007/978-3-319-21416-0}.
}
\details{
R package \bold{netmeta} is an add-on package for \bold{meta}
providing the following network meta-analysis models:
\itemize{
\item frequentist network meta-analysis (function
  \code{\link{netmeta}}) based on Rücker (2012) and Rücker &
  Schwarzer (2014);
\item additive network meta-analysis for combinations of treatments
  (\code{\link{netcomb}} for connected networks,
  \code{\link{discomb}} for disconnected networks) (Rücker et al.,
  2020a);
\item network meta-analysis of binary data
  (\code{\link{netmetabin}}) using the Mantel-Haenszel or
  non-central hypergeometric distribution method (Efthimiou et al.,
  2019), or penalised logistic regression (Evrenoglou et al., 2022);
\item network meta-regression (\code{\link{netmetareg}}) with a single
  continuous or binary covariate (Kwarteng et al., 2026);
\item subgroup network meta-analysis (\code{\link{subgroup.netmeta}}).
}

The following methods are available to present results of a network
meta-analysis:
\itemize{
\item network graphs (\code{\link{netgraph}}) described in Rücker &
  Schwarzer (2016);
\item forest plots (\code{\link{forest.netmeta}},
  \code{\link{forest.netcomb}});
\item league tables with network meta-analysis results
  (\code{\link{netleague}});
\item tables with network, direct and indirect estimates
  (\code{\link{nettable}}) looking similar to the statistical part
  of a GRADE table for a network meta-analysis (Puhan et al.,
  2014).
}

The following methods are implemented to rank treatments:
\itemize{
\item rankograms (\code{\link{rankogram}}) (Salanti et al., 2011);
\item ranking of treatments (\code{\link{netrank}}) based on
  P-scores (Rücker & Schwarzer, 2015) or the Surface Under the
  Cumulative RAnking curve (SUCRA) (Salanti et al., 2011);
\item partial order of treatment rankings (\code{\link{netposet}},
  \code{\link{plot.netposet}}) and Hasse diagram
  (\code{\link{hasse}}) according to Carlsen & Bruggemann (2014)
  and Rücker & Schwarzer (2017).
}

Available functions to evaluate network inconsistency:
\itemize{
\item split direct and indirect evidence (\code{\link{netsplit}})
  to check for consistency (Dias et al., 2010; Efthimiou et al.,
  2019);
\item scatter plot of direct and indirect estimates
  (\code{\link{plot.netmeta}}) to visualise local consistency
  (Wilson et al., 2026);
\item net heat plot (\code{\link{netheat}}) and design-based
  decomposition of Cochran's Q (\code{\link{decomp.design}})
  described in Krahn et al. (2013).
}

Additional methods and functions:
\itemize{
\item network meta-regression with a single continuous or binary covariate
  (\code{\link{netmetareg}});
\item subgroup network meta-analysis (\code{\link{subgroup.netmeta}});
\item information on network connectivity
  (\code{\link{netconnection}});
\item contribution of direct comparisons to network estimates
  (\code{\link{netcontrib}}) (Papakonstantinou et al., 2018; Davies
  et al., 2022);
\item importance of individual studies measured by reduction of
  precision if removed from network (\code{\link{netimpact}})
  (Rücker et al., 2020b);
\item\sQuote{comparison-adjusted} funnel plot
  (\code{\link{funnel.netmeta}}) to assess funnel plot asymmetry in
  network meta-analysis (Chaimani & Salanti, 2012);
\item conduct pairwise meta-analyses for all comparisons with
  direct evidence in a network meta-analysis
  (\code{\link{netpairwise}});
\item results of several network meta-analyses can be combined with
  \code{\link{netbind}} to show these results in a forest plot
  (\code{\link{forest.netbind}}).
\item measures characterizing the flow of evidence between two
  treatments (\code{\link{netmeasures}}) described in König et
  al. (2013);
\item calculate comparison effects of two arbitrary complex
  interventions in component network meta-analysis
  (\code{\link{netcomparison}});
\item calculate effect of arbitrary complex interventions in
  component network meta-analysis (\code{\link{netcomplex}}).
}

R package \bold{netmeta} provides two vignettes:
\itemize{
 \item \code{vignette("netmeta-workflow")} with an overview of the
   work flow and the main functions,
 \item \code{vignette("netmeta")} with the PDF file for
   Balduzzi et al. (2023).
}

Functions and datasets from \bold{netmeta} are utilised in
Schwarzer et al. (2015), Chapter 8 "Network Meta-Analysis",
\url{https://link.springer.com/book/10.1007/978-3-319-21416-0}.

Type \code{help(package = "netmeta")} for a listing of all R
functions available in \bold{netmeta}.

Type \code{citation("netmeta")} on how to cite \bold{netmeta} in
publications.

To report problems and bugs
\itemize{
\item type \code{bug.report(package = "netmeta")} if you do not use
  RStudio,
\item send an email to Guido Schwarzer
  \email{guido.schwarzer@uniklinik-freiburg.de} if you use RStudio.
}

The development version of \bold{netmeta} is available on GitHub
\url{https://github.com/guido-s/netmeta}.
}
\references{
Balduzzi S, Rücker G, Nikolakopoulou A, Papakonstantinou T, Salanti
G, Efthimiou O, Schwarzer G (2023):
netmeta: An R Package for network meta-analysis using frequentist
methods.
\emph{Journal of Statistical Software},
\bold{106}, 1--40

Carlsen L, Bruggemann R (2014):
Partial order methodology: a valuable tool in chemometrics.
\emph{Journal of Chemometrics},
\bold{28}, 226--34

Chaimani A & Salanti G (2012):
Using network meta-analysis to evaluate the existence of
small-study effects in a network of interventions.
\emph{Research Synthesis Methods},
\bold{3}, 161--76

Davies AL, Papakonstantinou T, Nikolakopoulou A, Rücker G, Galla T
(2022):
Network meta-analysis and random walks.
\emph{Statistics in Medicine},
\bold{41}, 2091--2114

Dias S, Welton NJ, Caldwell DM, Ades AE (2010):
Checking consistency in mixed treatment comparison meta-analysis.
\emph{Statistics in Medicine},
\bold{29}, 932--44

Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
(2019):
A Mantel-Haenszel model for network meta-analysis of rare events.
\emph{Statistics in Medicine},
\bold{38}, 2992--3012

Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A (2022):
Network Meta-analysis of rare events using penalized likelihood regression.
\emph{Statistics in Medicine},
\bold{41}, 5203--19.

König J, Krahn U, Binder H (2013):
Visualizing the flow of evidence in network meta-analysis and
characterizing mixed treatment comparisons.
\emph{Statistics in Medicine},
\bold{32}, 5414--29

Krahn U, Binder H, König J (2013):
A graphical tool for locating inconsistency in network meta-analyses.
\emph{BMC Medical Research Methodology},
\bold{13}, 35

Kwarteng N, Evrenoglou T, Mueller J, Elsaesser M, Schramm E, Schwarzer G,
Nikolakopoulou A (2026):
Illustrating the assumptions of meta-regression in treatment networks.
Preprint available at \emph{Research Square},
\doi{10.21203/rs.3.rs-8235913/v1}

Papakonstantinou, T., Nikolakopoulou, A., Rücker, G., Chaimani, A.,
Schwarzer, G., Egger, M., Salanti, G. (2018):
Estimating the contribution of studies in network meta-analysis:
paths, flows and streams.
\emph{F1000Research}

Puhan MA, Schünemann HJ, Murad MH, et al. (2014):
A GRADE working group approach for rating the quality of treatment
effect estimates from network meta-analysis.
\emph{British Medical Journal},
\bold{349}, g5630

Rücker G (2012):
Network meta-analysis, electrical networks and graph theory.
\emph{Research Synthesis Methods},
\bold{3}, 312--24

Rücker G, Schwarzer G (2014):
Reduce dimension or reduce weights? Comparing two approaches to
multi-arm studies in network meta-analysis.
\emph{Statistics in Medicine},
\bold{33}, 4353--69

Rücker G, Schwarzer G (2015):
Ranking treatments in frequentist network meta-analysis works
without resampling methods.
\emph{BMC Medical Research Methodology},
\bold{15}, 58

Rücker G, Schwarzer G (2016):
Automated drawing of network plots in network meta-analysis.
\emph{Research Synthesis Methods},
\bold{7}, 94--107

Rücker G, Schwarzer G (2017):
Resolve conflicting rankings of outcomes in network meta-analysis:
Partial ordering of treatments.
\emph{Research Synthesis Methods},
\bold{8}, 526--36

Rücker G, Petropoulou M, Schwarzer G (2020a):
Network meta-analysis of multicomponent interventions.
\emph{Biometrical Journal},
\bold{62}, 808--21

Rücker G, Nikolakopoulou A, Papakonstantinou T, Salanti G, Riley
RD, Schwarzer G (2020b):
The statistical importance of a study for a network meta-analysis
estimate.
\emph{BMC Medical Research Methodology},
\bold{20}, 190

Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71

Schwarzer G, Carpenter JR and Rücker G (2015):
\emph{Meta-Analysis with R (Use R!)}.
Springer International Publishing, Switzerland.

Wilson H, Schönstein A, Robson S, Bonofiglio F (2026):
A novel approach for visualizing local consistency in
network meta-analysis.
\emph{Research Synthesis Methods},
1--15
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/guido-s/netmeta}
}

}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}, Gerta
  Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.netmeta.R
\name{as.data.frame.netmeta}
\alias{as.data.frame.netmeta}
\title{Create a data frame from an object of class netmeta}
\usage{
\method{as.data.frame}{netmeta}(x, row.names = NULL, optional = FALSE, details = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{row.names}{\code{NULL} or a character vector giving the row
names for the data frame.}

\item{optional}{A logical. If \code{TRUE}, setting row names and
converting column names (to syntactic names) is optional.}

\item{details}{A logical. If \code{TRUE}, additional variables of
less interest are included in data frame.}

\item{\dots}{Additional arguments.}
}
\value{
A data frame is returned by the function
  \code{as.data.frame}.
}
\description{
The \code{as.data.frame} method returns a data frame containing
information on individual studies, e.g., estimated treatment effect
and its standard error.
}
\examples{
\donttest{
data(smokingcessation)

# Transform data from arm-based format to contrast-based format
#
pw <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")

# Conduct random effects network meta-analysis and show data frame
#
nma <- netmeta(pw, common = FALSE)
head(as.data.frame(nma))
head(as.data.frame(nma, details = TRUE))
}

}
\seealso{
\code{\link{netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netgraph.netmeta.R
\name{netgraph.netmeta}
\alias{netgraph.netmeta}
\title{Network graph}
\usage{
\method{netgraph}{netmeta}(
  x,
  seq = x$seq,
  labels = x$trts,
  cex = 1,
  adj = NULL,
  srt.labels = 0,
  offset = if (!is.null(adj) && all(unique(adj) == 0.5)) 0 else 0.0175,
  scale = 1.1,
  col = gs("col.netgraph"),
  plastic = gs("plastic"),
  thickness = gs("thickness"),
  lwd = 5,
  lwd.min = lwd/2.5,
  lwd.max,
  rescale.thickness,
  dim = "2d",
  rotate = 0,
  highlight = NULL,
  col.highlight = "red2",
  scale.highlight = 1,
  multiarm = gs("multiarm"),
  col.multiarm = NULL,
  alpha.transparency = 0.5,
  points = !missing(cex.points),
  cex.points = 1,
  pch.points = 20,
  col.points = if (length(pch.points) == 1 && pch.points == 21) "black" else "red",
  bg.points = "red",
  points.min,
  points.max,
  rescale.pointsize,
  number.of.studies = gs("number.of.studies"),
  cex.number.of.studies = cex,
  col.number.of.studies = "white",
  bg.number.of.studies = "black",
  pos.number.of.studies = 0.5,
  start.layout = ifelse(dim == "2d", "circle", "eigen"),
  eig1 = 2,
  eig2 = 3,
  eig3 = 4,
  iterate = FALSE,
  tol = 1e-04,
  maxit = 500,
  allfigures = FALSE,
  A.matrix = x$A.matrix,
  N.matrix = sign(A.matrix),
  D.matrix = netdistance(N.matrix),
  xpos = NULL,
  ypos = NULL,
  zpos = NULL,
  figure = TRUE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta} (mandatory).}

\item{seq}{A character or numerical vector specifying the sequence
of treatments arrangement (anticlockwise if \code{start.layout =
"circle"}).}

\item{labels}{An optional vector with treatment labels.}

\item{cex}{The magnification to be used for treatment labels.}

\item{adj}{One, two, or three values in [0, 1] (or a vector /
matrix with length / number of rows equal to the number of
treatments) specifying the x (and optionally y and z) adjustment
for treatment labels.}

\item{srt.labels}{The character string \code{"orthogonal"} (can be
abbreviated), a single numeric or numerical vector with value(s)
between -180 and 180 specifying the angle to rotate treatment
labels (see Details).}

\item{offset}{Distance between nodes (i.e. treatments) in graph and
treatment labels for 2-D plots (value of 0.0175 corresponds to a
difference of 1.75\% of the range on x- and y-axis).}

\item{scale}{Additional space added outside of nodes
(i.e. treatments).  Increase this value for larger treatment
labels (value of 1.10 corresponds to an additional space of 10\%
around the network graph).}

\item{col}{A single color (or vector of colors) for lines
connecting treatments (edges) if argument \code{plastic =
FALSE}. Length of the vector must be equal to the number of edges
(see list element 'comparisons' in \code{\link{netmeta}}).}

\item{plastic}{A logical indicating whether the appearance of the
comparisons should be in '3D look' (not to be confused with
argument \code{dim}).}

\item{thickness}{Either a character variable to determine the
method to plot line widths (see Details) or a matrix of the same
dimension and row and column names as argument \code{A.matrix}
with information on line width.}

\item{lwd}{A numeric for scaling the line width of comparisons.}

\item{lwd.min}{Minimum line width in network graph. All connections
with line widths below this values will be set to \code{lwd.min}.}

\item{lwd.max}{Maximum line width in network graph. The connection
with the largest value according to argument \code{thickness}
will be set to this value.}

\item{rescale.thickness}{A logical value or R function to scale the
thickness of lines (see Details).}

\item{dim}{A character string indicating whether a 2- or
3-dimensional plot should be produced, either \code{"2d"} or
\code{"3d"}.}

\item{rotate}{A single numeric with value between -180 and 180
specifying the degree to rotate nodes in a circular network clockwise or
anticlockwise.}

\item{highlight}{A character vector identifying comparisons that
should be marked in the network graph, e.g. \code{highlight =
"treat1:treat2"}.}

\item{col.highlight}{Color(s) to highlight the comparisons given by
\code{highlight}.}

\item{scale.highlight}{Scaling factor(s) for the line width(s) to
highlight the comparisons given by \code{highlight}.}

\item{multiarm}{A logical indicating whether multi-arm studies
should be marked in plot.}

\item{col.multiarm}{Either a function from R package
\bold{colorspace} or \bold{grDevice} to define colors for
multi-arm studies or a character vector with colors to highlight
multi-arm studies.}

\item{alpha.transparency}{The alpha transparency of colors used to
highlight multi-arm studies (0 means transparent and 1 means
opaque).}

\item{points}{A logical indicating whether points should be printed
at nodes (i.e. treatments) of the network graph.}

\item{cex.points, pch.points, col.points, bg.points}{Corresponding
size, type, color, and background color for points. Can be a
vector with length equal to the number of treatments.}

\item{points.min}{Minimum point size. All points with size below
this values will be set to \code{points.min}.}

\item{points.max}{Maximum point size in network graph. The node
with the largest value according to argument \code{cex.points}
will be set to this value.}

\item{rescale.pointsize}{A logical value or R function to scale the
point size (see Details).}

\item{number.of.studies}{A logical indicating whether number of
studies should be added to network graph.}

\item{cex.number.of.studies}{The magnification to be used for
number of studies.}

\item{col.number.of.studies}{Color for number of studies.}

\item{bg.number.of.studies}{Color for shadow around number of
studies.}

\item{pos.number.of.studies}{A single value (or vector of values)
in [0, 1] specifying the position of the number of studies on the
lines connecting treatments (edges). Length of the vector must be
equal to the number of edges.}

\item{start.layout}{A character string indicating which starting
layout is used if \code{iterate = TRUE}. If "circle" (default),
the iteration starts with a circular ordering of the vertices; if
"eigen", eigenvectors of the Laplacian matrix are used,
calculated via generic function \code{\link{eigen}} (spectral
decomposition); if "prcomp", eigenvectors of the Laplacian matrix
are calculated via generic function \code{\link{prcomp}}
(principal component analysis); if "random", a random layout is
used, drawn from a bivariate normal.}

\item{eig1}{A numeric indicating which eigenvector is used as x
coordinate if \code{start = "eigen"} or \code{"prcomp"} and
\code{iterate = TRUE}.  Default is 2, the eigenvector to the
second-smallest eigenvalue of the Laplacian matrix.}

\item{eig2}{A numeric indicating which eigenvector is used as
y-coordinate if \code{start = "eigen"} or \code{"prcomp"} and
\code{iterate = TRUE}.  Default is 3, the eigenvector to the
third-smallest eigenvalue of the Laplacian matrix.}

\item{eig3}{A numeric indicating which eigenvector is used as
z-coordinate if \code{start = "eigen"} or \code{"prcomp"} and
\code{iterate = TRUE}.  Default is 4, the eigenvector to the
fourth-smallest eigenvalue of the Laplacian matrix.}

\item{iterate}{A logical indicating whether the stress majorization
algorithm is carried out for optimization of the layout.}

\item{tol}{A numeric for the tolerance for convergence if
\code{iterate = TRUE}.}

\item{maxit}{An integer defining the maximum number of iteration
steps if \code{iterate = TRUE}.}

\item{allfigures}{A logical indicating whether all iteration steps
are shown if \code{iterate = TRUE}. May slow down computations if
set to \code{TRUE} (especially if \code{plastic = TRUE}).}

\item{A.matrix}{Adjacency matrix (\emph{n}x\emph{n}) characterizing
the structure of the network graph. Row and column names must be
the same set of values as provided by argument \code{seq}.}

\item{N.matrix}{Neighborhood matrix (\emph{n}x\emph{n}) replacing
A.matrix if neighborhood is to be specified differently from node
adjacency in the network graph, for example content-based. Row
and column names must be the same set of values as provided by
argument \code{seq}.}

\item{D.matrix}{Distance matrix (\emph{n}x\emph{n}) replacing
A.matrix and N.matrix if distances should be provided
directly. Row and column names must be the same set of values as
provided by argument \code{seq}.}

\item{xpos}{Vector (\emph{n}) of x coordinates.}

\item{ypos}{Vector (\emph{n}) of y coordinates.}

\item{zpos}{Vector (\emph{n}) of z coordinates.}

\item{figure}{A logical indicating whether network graph should be
shown.}

\item{\dots}{Additional graphical arguments (passed on to
\code{\link{plot.default}}).}
}
\value{
A list containing two data frames with information on nodes and
edges.

\bold{List element 'nodes'}
\item{trts}{Treatment names.}
\item{labels}{Treatment labels.}
\item{seq}{Sequence of treatment labels.}
\item{srt}{String rotation.}
\item{xpos}{Position of treatment / node on x-axis.}
\item{ypos}{Position of treatment / node on y-axis.}
\item{zpos}{Position of treatment / node on z-axis (for 3-D
  plots).}
\item{xpos.labels}{Position of treatment labels on x-axis (for 2-D
  plots).}
\item{ypos.labels}{Position of treatment labels on y-axis (for 2-D
  plots).}
\item{offset.x}{Offset of treatment labels on x-axis (for 2-D
  plots).}
\item{offset.y}{Offset of treatment labels on y-axis (for 2-D
  plots).}
\item{cex}{Point size of treatments / nodes}
\item{col}{Color for points.}
\item{pch}{Point type.}
\item{bg}{Background color for points.}
\item{adj.x}{Adjustment for treatment label on x-axis.}
\item{adj.y}{Adjustment for treatment label on y-axis.}
\item{adj.z}{Adjustment for treatment label on z-axis (for 3-D
  plots).}

\bold{List element 'edges'}
\item{treat1}{Name of first treatment.}
\item{treat2}{Name of second treatment.}
\item{n.stud}{Number of studies directly comparing treatments.}
\item{xpos}{Position of number of studies on x-axis.}
\item{ypos}{Position of number of studies on y-axis.}
\item{adj}{Adjustment of number of studies.}
\item{pos.number.of.studies}{Position of number of studies on
  edge.}
\item{col}{Color for lines / edges.}
}
\description{
This function generates a graph of the evidence network.
}
\details{
This function generates a network graph for an R object created
with \code{\link{netmeta}}. R function \code{\link{plot.default}} is used
to create the network graph.

\subsection{Layout of network graph}{
The network is laid out in the plane, where the nodes in the graph
layout correspond to the treatments and edges display the observed
treatment comparisons. For the default setting, nodes are placed on
a circle.  Other starting layouts are "eigen", "prcomp", and
"random" (Rücker & Schwarzer 2015). If \code{iterate = TRUE}, the
layout is further optimized using the stress majorization
algorithm. This algorithm specifies an 'ideal' distance (e.g., the
graph distance) between two nodes in the plane. In the optimal
layout, these distances are best approximated in the sense of least
squares. Starting from an initial layout, the optimum is
approximated in an iterative process called stress majorization
(Kamada and Kawai 1989, Michailidis and de Leeuw 2001, Hu
2012). The starting layout can be chosen as a circle or coming from
eigenvectors of the Laplacian matrix (corresponding to Hall's
algorithm, Hall 1970), calculated in different ways, or
random. Moreover, it can be chosen whether the iteration steps are
shown (argument \code{allfigures = TRUE}).

An optimized circular presentation which typically has a reduced
(sometimes minimal) number of crossings can be achieved by using
argument \code{seq = "optimal"} in combination with argument
\code{start.layout}. Note, is is not possible of prespecify the
best value for argument \code{start.layout} for any situation as
the result depends on the network structure.
}

\subsection{Definition of line widths}{
Argument \code{thickness} providing the line width of edges
(comparisons) can be a matrix of the same dimension as argument
\code{A.matrix} or any of the following character strings (which
can be abbreviated):
\itemize{
\item Proportional to number of studies comparing two treatments
  (\code{thickness = "number.of.studies"}, default)
\item Proportional to inverse standard error of common effects model
  comparing two treatments (\code{thickness = "se.common"})
\item Proportional to inverse standard error of random effects
  model comparing two treatments (\code{thickness = "se.random"})
\item Weight from common effects model comparing two treatments
  (\code{thickness = "w.common"})
\item Weight from random effects model comparing two treatments
  (\code{thickness = "w.random"})
\item Same line width for all comparisons (\code{thickness =
  "equal"})
}

Only evidence from direct treatment comparisons is considered to
determine the line width if argument \code{thickness} is equal to
any but the last method.

Line widths are determined by argument \code{lwd} if all lines have
the same width. This is possible if either argument \code{thickness
= "equal"}, all pairwise comparisons have the same number of
studies for \code{thickness = "number.of.studies"} or all direct
comparisons are equally precise.

Otherwise, the line width of the thickest line is equal to the
value of argument \code{lwd.max} and all lines with a thickness
below the value of argument \code{lwd.min} are set to this
value. Default for argument \code{lwd.max} is \code{4 * lwd}.

Argument \code{rescale.thickness} can be used to provide a function
to specify the relative line width of edges (comparisons). By
default, the square root function \code{\link[base]{sqrt}} is used
in order to lessen differences in line widths. Argument
\code{rescale.thickness = FALSE} or \code{rescale.thickness = I},
i.e., the identity function \code{\link[base]{I}}, can be used to
not rescale line widths.
}

\subsection{Definition of point sizes}{
Points are printed at nodes (treatments) if argument \code{points =
TRUE} or argument \code{cex.points} is provided.

Point sizes are equal to the value of argument \code{cex.points} if
all points are of equal size. 

Otherwise, the point size of the largest point is equal to the
value of argument \code{points.max} and all points smaller than the
value of argument \code{points.min} are set to this value. The
default for argument \code{points.max} is equal to the largest
value provided in argument \code{cex.points} if this largest value
is below or equal to 25. Otherwise the default is \code{points.max
= 8}.

Argument \code{rescale.pointsize} can be used to provide a function
to specify relative point sizes. Point sizes are not rescaled at
all if they are all equal or the largest \code{cex.points} value is
below or equal to 25. Otherwise, the square root function
\code{\link[base]{sqrt}} is used in order to lessen the differences
in point sizes. Argument \code{rescale.pointsize = FALSE} or
\code{rescale.pointsize = I}, i.e., the identity function
\code{\link[base]{I}}, can be used to not rescale point sizes.
}

\subsection{Other settings}{
Argument \code{srt.labels} can be used to specific the rotation (in
degrees) of the treatment labels. If \code{srt.labels} is equal to
\code{"orthogonal"}, treatment labels are orthogonal to the
circle. If \code{srt.labels} is a single numeric, all labels are
rotated by this degree. If \code{srt.labels} is a numeric vector,
it must be of the same length as the number of treatments and
labels are rotated counter-clockwise starting on the right
side. Finally, if \code{srt.labels} is a named numeric vector, it
must be of the same length as the number of treatments and the
names must be equal to the treatment names (and treatment labels
are rotated according to the specified values).

Further, a couple of graphical parameters can be specified, such as
color and appearance of the nodes (treatments) and the edges
(comparisons), whether special comparisons should be highlighted
and whether multi-arm studies should be indicated as colored
polygons. By default, the \code{\link[colorspace]{sequential_hcl}}
function is used to highlight multi-arm studies.

In order to generate 3-D plots (argument \code{dim = "3d"}), R
package \bold{rgl} is necessary. Note, under macOS the X.Org X
Window System must be available (see
\url{https://www.xquartz.org}).
}
}
\examples{
data(smokingcessation)

# Transform data from arm-based format to contrast-based format
#
pw1 <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")

# Conduct random effects network meta-analysis
#
nma1 <- netmeta(pw1, common = FALSE)

# Network graph with default settings
#
netgraph(nma1)

\donttest{
# Generation of an object of class 'netmeta' with reference
# treatment 'plac'
#
pw2 <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))
nma2 <- netmeta(pw2, reference = "plac")

# Network graph with default settings
#
netgraph(nma2)

# Network graph with specified order of the treatments and one
# highlighted comparison
#
trts <- c("plac", "benf", "migl", "acar", "sulf",
  "metf", "rosi", "piog", "sita", "vild")
netgraph(nma2, highlight = "rosi:plac", seq = trts)

# Same network graph using argument 'seq' in netmeta function
#
nma3 <- netmeta(pw2, reference = "plac", seq = trts)
netgraph(nma3, highlight = "rosi:plac")

# Network graph optimized, starting from a circle, with multi-arm
# study colored
#
netgraph(nma2, start = "circle", iterate = TRUE,
  multiarm = TRUE, col.multiarm = "purple")

# Network graph optimized, starting from a circle, with multi-arm
# study colored and all intermediate iteration steps visible
#
netgraph(nma2, start = "circle", iterate = TRUE,
  multiarm = TRUE, col.multiarm = "purple",
  allfigures = TRUE)

# Network graph optimized, starting from Laplacian eigenvectors,
# with multi-arm study colored
#
netgraph(nma2, start = "eigen",
  multiarm = TRUE, col.multiarm = "purple")

# Network graph optimized, starting from different Laplacian
# eigenvectors, with multi-arm study colored
#
netgraph(nma2, start = "prcomp",
  multiarm = TRUE, col.multiarm = "purple")

# Network graph optimized, starting from random initial layout,
# with multi-arm study colored
#
netgraph(nma2, start = "random",
  multiarm = TRUE, col.multiarm = "purple")

# Network graph without plastic look and one highlighted comparison
#
netgraph(nma2, plastic = FALSE, highlight = "rosi:plac")

# Network graph with same thickness for all comparisons
#
netgraph(nma2, thickness = "equal")

# Network graph with changed labels and specified order of the
# treatments
#
netgraph(nma2, seq = c(1, 3, 5, 2, 9, 4, 7, 6, 8, 10),
  labels = LETTERS[1:10])

# Rotate treatment labels (orthogonal to circle)
#
netgraph(nma2, srt.labels = "o")

# Network graph in 3-D (opens a new device, where you may rotate and
# zoom the plot using the mouse / the mouse wheel).
# The rgl package must be installed for 3-D plots.
#
if (requireNamespace("rgl", quietly = TRUE))
  netgraph(nma2, dim = "3d")
}

}
\references{
Hall KM (1970):
An r-dimensional quadratic placement algorithm.
\emph{Management Science},
\bold{17}, 219--29

Hu Y (2012):
\emph{Combinatorial Scientific Computing}, Chapter Algorithms for
Visualizing Large Networks, pages 525--49.
Chapman and Hall / CRC,  Computational Science.

Kamada T, Kawai S (1989):
An algorithm for drawing general undirected graphs.
\emph{Information Processing Letters},
\bold{31}, 7--15

Krahn U, Binder H, König J (2013):
A graphical tool for locating inconsistency in network meta-analyses.
\emph{BMC Medical Research Methodology},
\bold{13}, 35

Michailidis G, de Leeuw J (2001):
Data visualization through graph drawing.
\emph{Computational Statistics},
\bold{16}, 435--50

Rücker G, Schwarzer G (2016):
Automated drawing of network plots in network meta-analysis.
\emph{Research Synthesis Methods},
\bold{7}, 94--107
}
\seealso{
\code{\link{netmeta}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Ulrike
  Krahn \email{ulrike.krahn@bayer.com}, Jochem König
  \email{koenigjo@uni-mainz.de}, Guido Schwarzer
  \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.netcomparison.R
\name{forest.netcomparison}
\alias{forest.netcomparison}
\alias{plot.netcomparison}
\title{Forest plot for complex interventions in component network
meta-analysis}
\usage{
\method{forest}{netcomparison}(
  x,
  pooled = ifelse(x$random, "random", "common"),
  leftcols = c("studlab", "treat1", "treat2"),
  leftlabs = c("Comparison", "Trt 1", "Trt 2"),
  rightcols = c("effect", "ci", "statistic", "pval"),
  rightlabs = c(NA, NA, "z", "p-value"),
  nchar.comps = x$nchar.trts,
  digits = gs("digits.forest"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  smlab = NULL,
  backtransf = x$backtransf,
  lab.NA = gs("lab.NA"),
  equal.size = gs("equal.size"),
  ...
)

\method{plot}{netcomparison}(x, ...)
}
\arguments{
\item{x}{An object of class \code{netcomparison}.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{leftcols}{A character vector specifying (additional) columns
to be plotted on the left side of the forest plot or a logical
value (see \code{\link[meta]{forest.meta}} help page for details).}

\item{leftlabs}{A character vector specifying labels for
(additional) columns on left side of the forest plot (see
\code{\link[meta]{forest.meta}} help page for details).}

\item{rightcols}{A character vector specifying (additional) columns
to be plotted on the right side of the forest plot or a logical
value (see \code{\link[meta]{forest.meta}} help page for details).}

\item{rightlabs}{A character vector specifying labels for
(additional) columns on right side of the forest plot (see
\code{\link[meta]{forest.meta}} help page for details).}

\item{nchar.comps}{A numeric defining the minimum number of
characters used to create unique names for components.}

\item{digits}{Minimal number of significant digits for treatment
effects and confidence intervals, see \code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for tests
of overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall effects, see \code{print.default}.}

\item{smlab}{A label printed at top of figure. By default, text
indicating either common or random effects model is printed.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{lab.NA}{A character string to label missing values.}

\item{equal.size}{A logical indicating whether all squares should
be of equal size. Otherwise, the square size is proportional to
the precision of estimates.}

\item{\dots}{Additional arguments for \code{\link[meta]{forest.meta}}
function.}
}
\description{
Draws a forest plot in the active graphics window (using grid
graphics system).
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window. For more information see help page of
\code{\link[meta]{forest.meta}} function.
}
\examples{
# Examples: example(netcomparison)

}
\seealso{
\code{\link{netcomparison}}, \code{\link{netcomb}},
  \code{\link{discomb}}, \code{\link[meta]{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.netimpact.R
\name{print.netimpact}
\alias{print.netimpact}
\title{Print method for objects of class netimpact}
\usage{
\method{print}{netimpact}(
  x,
  common = x$x$common,
  random = x$x$random,
  digits = gs("digits.prop"),
  nchar.trts = x$nchar.trts,
  nchar.studlab = x$nchar.studlab,
  details.methods = gs("details"),
  legend = gs("legend"),
  legend.studlab = TRUE,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netimpact}.}

\item{common}{A logical indicating whether results for the common
effects model should be printed.}

\item{random}{A logical indicating whether results for the random
effects model should be printed.}

\item{digits}{Minimal number of significant digits.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{nchar.studlab}{A numeric defining the minimum number of
characters used to create unique study labels.}

\item{details.methods}{A logical specifying whether details on statistical
methods should be printed.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{legend.studlab}{A logical indicating whether a legend should
be printed for abbreviated study labels.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\description{
Print method for objects of class \code{netimpact}.
}
\examples{
# Examples: example(netimpact)

}
\seealso{
\code{\link{netimpact}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Linde2016.R
\docType{data}
\name{Linde2016}
\alias{Linde2016}
\title{Network meta-analysis of primary care depression treatments}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{id}}\tab Study ID \cr
\bold{\emph{author}}\tab First author \cr
\bold{\emph{year}}\tab Year of publication \cr
\bold{\emph{treat1}}\tab First treatment (abbreviated) \cr
\bold{\emph{treat2}}\tab Second treatment (abbreviated) \cr
\bold{\emph{treat1.long}}\tab First treatment \cr
\bold{\emph{treat2.long}}\tab Second treatment \cr
\bold{\emph{lnOR}}\tab Response after treatment (log odds ratio) \cr
\bold{\emph{selnOR}}\tab Standard error of log odds ratio \cr
\bold{\emph{resp1}}\tab Responder (first treatment) \cr
\bold{\emph{n1}}\tab Sample size (first treatment) \cr
\bold{\emph{resp2}}\tab Responder (second treatment) \cr
\bold{\emph{n2}}\tab Sample size (second treatment) \cr
}
}
\source{
Linde K, Rücker G, Schneider A et al. (2016):
Questionable assumptions hampered interpretation of a network
meta-analysis of primary care depression treatments.
\emph{Journal of Clinical Epidemiology},
\bold{71}, 86--96
}
\description{
Network meta-analysis of 22 treatments (including placebo and usual
care) for the primary care of depression.
}
\examples{
head(Linde2016)

# Example using pairwise(), netmeta() and netcomb():
# example(Linde2016, package = "metabook")
}
\seealso{
\code{\link{netmeta}}, \code{\link{netcomb}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Dong2013.R
\docType{data}
\name{Dong2013}
\alias{Dong2013}
\title{Network meta-analysis for chronic obstructive pulmonary disease}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{id}}\tab study ID \cr
\bold{\emph{treatment}}\tab treatment \cr
\bold{\emph{death}}\tab mortality \cr
\bold{\emph{randomized}}\tab number of individuals in treatment arm
}
}
\source{
Dong Y-H, Lin H-H, Shau W-Y, Wu Y-C, Chang C-H, Lai M-S (2013):
Comparative safety of inhaled medications in patients with chronic
obstructive pulmonary disease: systematic review and mixed
treatment comparison meta-analysis of randomised controlled trials.
\emph{Thorax},
\bold{68}, 48--56
}
\description{
Network meta-analysis comparing inhaled medications in patients
with chronic obstructive pulmonary disease.
}
\note{
The dataset Dong2013 is identical to dataset
\code{\link[metabook]{Dong2013}} in R package \bold{metabook}.
}
\examples{
head(Dong2013)

# Example using pairwise() and netmetabin():
# example(Dong2013, package = "metabook")
}
\seealso{
\code{\link[metabook]{Dong2013}},
  \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
  \code{\link{netmetabin}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/settings.netmeta.R
\name{settings.netmeta}
\alias{settings.netmeta}
\title{Print and change default network meta-analysis settings in R package
\bold{netmeta}}
\usage{
settings.netmeta(..., quietly = TRUE)
}
\arguments{
\item{...}{Arguments to change default settings.}

\item{quietly}{A logical indicating whether information on settings
should be printed.}
}
\description{
Print and change default settings to conduct and print or plot
network meta-analyses in R package \bold{netmeta}.
}
\seealso{
\code{\link[meta]{settings.meta}}, \code{\link[meta]{gs}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netcomparison.R
\name{netcomparison}
\alias{netcomparison}
\alias{print.netcomparison}
\title{Calculate comparison effects of two arbitrary complex interventions
in component network meta-analysis}
\usage{
netcomparison(
  x,
  treat1,
  treat2,
  common = x$common,
  random = x$random,
  level = x$level.ma,
  nchar.comps = x$nchar.comps,
  backtransf = x$backtransf,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{print}{netcomparison}(
  x,
  common = x$common,
  random = x$random,
  backtransf = x$backtransf,
  nchar.comps = x$nchar.comps,
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  big.mark = gs("big.mark"),
  legend = gs("legend"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netcomb} or \code{netcomparison}
(print function).}

\item{treat1}{A character vector defining the first complex
intervention(s).}

\item{treat2}{A character vector defining the second complex
intervention(s).}

\item{common}{A logical indicating whether results for common
effects model should be conducted.}

\item{random}{A logical indicating whether results for random
effects model should be conducted.}

\item{level}{The level used to calculate confidence intervals for
combinations of components.}

\item{nchar.comps}{A numeric defining the minimum number of
characters used to create unique names for components (see
Details).}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. If \code{backtransf=TRUE}, results
for \code{sm="OR"} are printed as odds ratios rather than log
odds ratios.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z-value
of test for overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for
p-values, see \code{print.default}.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
combination effect should be printed according to JAMA reporting
standards.}

\item{big.mark}{A character used as thousands separator.}

\item{legend}{A logical indicating whether a legend should be
printed.}
}
\value{
A list is returned by the function \code{netcomparison} with the
following elements:
\item{comparison}{Comparison.}
\item{TE.common, TE.random}{A vector of comparison effects (common
  and random effects model).}
\item{seTE.common, seTE.random}{A vector with corresponding standard
  errors (common and random effects model).}
\item{lower.common, lower.random}{A vector with lower confidence
  limits for comparisons (common and random effects model).}
\item{upper.common, upper.random}{A vector with upper confidence
  limits for comparisons (common and random effects model).}
\item{statistic.common, statistic.random}{A vector with z-values for
  the overall effect of comparisons (common and random effects
  model).}
\item{pval.common, pval.random}{A vector with p-values for the
  overall effect of comparisons (common and random effects model).}
\item{trts}{Treatments included in comparisons.}
\item{comps}{Components included in comparisons.}
\item{treat1, treat2}{A defined above.}
\item{common, random}{A defined above.}
\item{level, nchar.comps, backtransf, x}{A defined above.}
\item{B.matrix}{B matrix.}
\item{C.matrix}{C matrix.}
}
\description{
Calculate comparison effects of two arbitrary complex interventions
(i.e., combinations of several components) in component network
meta-analysis.
}
\details{
R functions \code{\link{netcomb}} and \code{\link{discomb}}
calculate effects for individual components and complex
interventions present in the component network meta-analysis
(CNMA). This function can be used to calculate the effect for
comparisons of two arbitrary complex interventions defined by
arguments \code{treat1} and \code{treat2}.

All complex interventions occuring in the network are considered
for the first complex intervention if argument \code{treat1} is
missing. The reference group defined in the (C)NMA is used as
second complex intervention if argument \code{treat2} is
missing. The first complex intervention in the (C)NMA is used if
the reference group is not defined.

The following matrices are needed to calculate comparison effects
of arbitrary complex interventions, (Rücker et al., 2020, Section
3.2):
\itemize{
\item B matrix describing how comparisons are composed by complex
  intervetions,
\item C matrix describing how the complex interventions are
  composed by the components.
}
Internally, both matrices are constructed based on arguments
\code{x}, \code{treat1} and \code{treat2}.

By default, component names are not abbreviated in
printouts. However, in order to get more concise printouts,
argument \code{nchar.comps} can be used to define the minimum
number of characters for abbreviated component names (see
\code{\link{abbreviate}}, argument \code{minlength}). R function
\code{\link{treats}} is utilised internally to create abbreviated
component names.
}
\note{
R function \code{\link{netcomplex}} can be used to calculate the
effect for arbitrary complex interventions in a component network
meta-analysis.
}
\examples{
# Only consider studies including Face-to-face PST (to reduce
# runtime of example)
#
face <- subset(Linde2016, id \%in\% c(16, 24, 49, 118))

# Use pairwise() to transform data to comparison-based format
#
pw <- pairwise(treat = int,
  event = resp, n = n, studlab = paste(author, year),
  data = face, reference = "plac", sm = "OR")

# Conduct random effects network meta-analysis
#
nma1 <- netmeta(pw, common = FALSE)

# Additive model for treatment components (with placebo as inactive
# treatment)
#
cnma1 <- netcomb(nma1, inactive = "placebo")

# Result for comparison Face-to-face PST vs TCA
netcomparison(cnma1, "Face-to-face PST", "TCA", nchar.comps = 4)
netcomparison(cnma1, "F", "T", nchar.comps = 4)

# Result for comparison Face-to-face PST vs TCA + Placebo
netcomparison(cnma1, "Face-to-face PST", "TCA + Plac", nchar.comps = 4)

\donttest{
# Forest plots
#
t1 <- c("F + TCA", "F + Plac", "SSRI + Plac + TCA")
t2 <- c("UC", "Plac", "UC")
#
netcomparison(cnma1, t1, t2)
#
forest(netcomparison(cnma1, t1, t2))
forest(netcomparison(cnma1, t1, t2), nchar.comps = 4)
forest(netcomparison(cnma1, c("F", "TCA"), "UC"), nchar.comps = 4)

# Artificial example
#
treat1 <- rep("A", 3)
treat2 <- c("B+C", "A+C", "C+D")
TE <- c(0, 1, 0)
seTE <- rep(1, 3)
# Conduct (C)NMA
nma2 <- netmeta(TE, seTE, treat1, treat2, random = FALSE)
cnma2 <- netcomb(nma2)

# Result for comparison A vs B + D
netcomparison(cnma2, "A", "B + D")
# Same results
netcomparison(cnma2, "A", "B+D")
netcomparison(cnma2, "A", "D+B")
netcomparison(cnma2, "a", "d+b")

# Generated B matrix
netcomparison(cnma2, "A", "B + D")$C.matrix
# Generated B matrix
netcomparison(cnma2, "A", "B + D")$B.matrix
}

}
\references{
Rücker G, Petropoulou M, Schwarzer G (2020):
Network meta-analysis of multicomponent interventions.
\emph{Biometrical Journal},
\bold{62}, 808--21
}
\seealso{
\code{\link{netcomb}}, \code{\link{discomb}},
  \code{\link{netcomplex}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netdistance.R
\name{netdistance.default}
\alias{netdistance.default}
\alias{netdistance}
\alias{netdistance.netmeta}
\alias{netdistance.netcomb}
\alias{print.netdistance}
\alias{netdistance.netconnection}
\title{Calculate distance matrix for an adjacency matrix}
\usage{
\method{netdistance}{default}(x, ...)

\method{netdistance}{netmeta}(x, sort = gs("sort.distance"), ...)

\method{netdistance}{netcomb}(x, sort = gs("sort.distance"), ...)

\method{netdistance}{netconnection}(x, ...)

\method{print}{netdistance}(x, lab.Inf = ".", ...)

netdistance(x, ...)
}
\arguments{
\item{x}{Either a netmeta or netcomb object or an adjacency matrix.}

\item{\dots}{Additional arguments.}

\item{sort}{A logical indicating whether to sort within (sub)networks by
treatment names.}

\item{lab.Inf}{A character string to label infinite values.}
}
\description{
Calculate distance matrix for an adjacency matrix based on distance
algorithm by Müller et al. (1987).
}
\examples{
data(smokingcessation)

pw1 <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")
nma1 <- netmeta(pw1, common = FALSE)

netdistance(nma1)

\donttest{
pw2 <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))
nma2 <- netmeta(pw2)

netdistance(nma2)
netdistance(nma2$A.matrix)
}

}
\references{
Müller WR, Szymanski K, Knop JV, and Trinajstic N (1987):
An algorithm for construction of the molecular distance matrix.
\emph{Journal of Computational Chemistry},
\bold{8}, 170--73
}
\seealso{
\code{\link{netmeta}}, \code{\link{netconnection}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
  Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subgroup.netmeta.R
\name{subgroup.netmeta}
\alias{subgroup.netmeta}
\alias{subgroup}
\alias{print.subgroup.netmeta}
\title{Subgroup analysis for network meta-analysis}
\usage{
\method{subgroup}{netmeta}(
  x,
  subgroup,
  only.connected = FALSE,
  common = x$common,
  random = x$random,
  method.tau = x$method.tau,
  level.ma = x$level.ma,
  backtransf = x$backtransf,
  nchar.trts = x$nchar.trts,
  ...
)

subgroup(x, ...)

\method{print}{subgroup.netmeta}(
  x,
  common = x$x$common,
  random = x$x$random,
  backtransf = x$x$backtransf,
  nchar.trts = x$x$nchar.trts,
  digits = gs("digits"),
  digits.se = gs("digits.se"),
  digits.Q = gs("digits.Q"),
  digits.pval.Q = gs("digits.pval.Q"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  big.mark = gs("big.mark"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  print.se = !backtransf,
  print.tau2 = gs("print.tau2"),
  print.tau = gs("print.tau"),
  print.Q = gs("print.Q"),
  text.tau2 = gs("text.tau2"),
  text.tau = gs("text.tau"),
  details.methods = gs("details"),
  legend = gs("legend"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta} (or \code{subgroup.netmeta}).}

\item{subgroup}{A vector defining the subgroups considered in the
network meta-analysis.}

\item{only.connected}{A logical indicating whether networks of subgroups
must be connected.}

\item{common}{A logical indicating whether results for common
effect subgroup network meta-analysis should be printed.}

\item{random}{A logical indicating whether results for random
effects subgroup network meta-analysis should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau}. Either \code{"DL"}, \code{"REML"}, or
\code{"ML"}, can be abbreviated.}

\item{level.ma}{The level used to calculate confidence intervals
for network estimates.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf = TRUE}, results for \code{sm = "OR"} are
presented as odds ratios rather than log odds ratios, for
example.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names
(see \code{\link{netmeta}}).}

\item{\dots}{Additional arguments.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
errors.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistic Q, see \code{print.default}.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity test, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance \eqn{\tau^2}, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for
\eqn{\tau}, the square root of the between-study variance
\eqn{\tau^2}.}

\item{big.mark}{A character used as thousands separator.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{print.se}{A logical specifying whether standard errors should be
printed.}

\item{print.tau2}{A logical specifying whether between-study
variance \eqn{\tau^2} should be printed.}

\item{print.tau}{A logical specifying whether \eqn{\tau}, the
square root of the between-study variance \eqn{\tau^2}, should be
printed.}

\item{print.Q}{A logical value indicating whether to print the
results of the test of heterogeneity.}

\item{text.tau2}{Text printed to identify between-study variance
\eqn{\tau^2}.}

\item{text.tau}{Text printed to identify \eqn{\tau}, the square
root of the between-study variance \eqn{\tau^2}.}

\item{details.methods}{A logical specifying whether details on statistical
methods should be printed.}

\item{legend}{A logical indicating whether a legend should be
printed.}
}
\value{
An object of class \code{"subgroup.netmeta"} with corresponding \code{print}
  and \code{forest} function.
}
\description{
Subgroup analysis for objects of class \code{netmeta}.
}
\examples{
\donttest{
data("Senn2013")
# Add variable with (fictitious) risk of bias values
Senn2013$rob <- NA
set.seed(1909)
for (i in unique(Senn2013$studlab))
  Senn2013$rob[Senn2013$studlab == i] <- sample(1:3, 1)
Senn2013$rob <- factor(Senn2013$rob, levels = 1:3,
  labels = c("low", "moderate", "high"))
# Conduct network meta-analysis
net <- netmeta(TE, seTE, treat1.long, treat2.long, studlab,
  data = Senn2013, sm = "MD", reference = "plac", nchar.trts = 4)
# Conduct subgroup network meta-analysis
subgroup(net, rob, common = FALSE)
}

}
\seealso{
\code{\link{forest.subgroup.netmeta}}, \code{\link{forest.netmeta}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.netconnection.R
\name{summary.netconnection}
\alias{summary.netconnection}
\alias{print.summary.netconnection}
\title{Summary method for objects of class netconnection}
\usage{
\method{summary}{netconnection}(object, ...)

\method{print}{summary.netconnection}(x, ...)
}
\arguments{
\item{object}{An object of class \code{netconnection}.}

\item{\dots}{Additional arguments (passed on to
\code{\link{print.netconnection}}.}

\item{x}{An object of class \code{summary.netconnection}.}
}
\description{
Summary method for objects of class \code{netconnection} to print
list of studies in subnetworks.
}
\examples{
# Artificial example with two subnetworks
#
t1 <- c("G", "B", "B", "D", "A", "F")
t2 <- c("B", "C", "E", "E", "H", "A")
#
nc2 <- netconnection(t1, t2)
print(nc2, details = TRUE)

summary(nc2)

}
\seealso{
\code{\link{netconnection}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hatmatrix.R
\name{hatmatrix}
\alias{hatmatrix}
\alias{print.hatmatrix}
\title{Derive hat matrix from network meta-analysis}
\usage{
hatmatrix(
  x,
  method = "Ruecker",
  type,
  common = x$common,
  random = x$random,
  nchar.trts = x$nchar.trts,
  nchar.studlab = x$nchar.studlab
)

\method{print}{hatmatrix}(
  x,
  common = x$x$common,
  random = x$x$random,
  nchar.trts = x$x$nchar.trts,
  nchar.studlab = x$x$nchar.studlab,
  digits = gs("digits"),
  legend = gs("legend"),
  legend.studlab = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{\link{netmeta}} object.}

\item{method}{A character string indicating which method is used to
derive the hat matrix. Either \code{"Ruecker"}, \code{"Krahn"} or
\code{"Davies"} (can be abbreviated, see Details).}

\item{type}{A character string indicating which specific hat matrix
should be derived (can be abbreviated, see Details).}

\item{common}{A logical indicating whether a hat matrix should be
printed for the common effects network meta-analysis.}

\item{random}{A logical indicating whether a hat matrix should be
printed for the random effects network meta-analysis.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{nchar.studlab}{A numeric defining the minimum number of
characters used to create unique study labels.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{legend.studlab}{A logical indicating whether a legend should
be printed for abbreviated study labels.}

\item{\dots}{Additional arguments (ignored).}
}
\value{
A list with two hat matrices: \code{common} (common effects model)
and \code{random} (random effects model).
}
\description{
Auxiliary function to derive hat matrix from network meta-analysis
}
\details{
This auxiliary function can be used to derive various hat matrices
from a network meta-analysis object.

\subsection{Hat matrix by Rücker (2012)}{

This hat matrix is estimated if \code{method = "Ruecker"}.

Let \emph{n} be the number of different treatments (nodes,
vertices) in a network and let \emph{m} be the number of existing
comparisons (edges) between the treatments. If there are only
two-arm studies, \emph{m} is equal to the number of studies,
\emph{k}. Let seTE.adj.common and seTE.adj.random be the vectors of
adjusted standard errors under the common and random effects model
(see \code{\link{netmeta}}). Let \strong{W} be the \emph{m} x
\emph{m} diagonal matrix that contains the inverse variance 1 /
seTE.adj.common\eqn{^2} or 1 / seTE.adj.random\eqn{^2}.

The given comparisons define the network structure. Therefrom an
\emph{m} x \emph{n} design matrix X (edge-vertex incidence matrix) is
formed; for more precise information, see Rücker (2012). Moreover,
the \emph{n} x \emph{n} Laplacian matrix \strong{L} and its
Moore-Penrose pseudoinverse \strong{L}\eqn{^+} are calculated (both
matrices play an important role in graph theory and electrical
network theory). Using these matrices, the variances based on both
direct and indirect comparisons can be estimated. The hat matrix
\strong{H} is estimated by \strong{H =
XL}\eqn{^+}\strong{X}\eqn{^T}\strong{W = X(X}\eqn{^T}\strong{W
X)}\eqn{^+}\strong{X}\eqn{^T}\strong{W}.
}

\subsection{Hat matrices by Krahn et al. (2013)}{

One of the following hat matrices is estimated if \code{method
= "Krahn"}.

Use of \code{type = "design"} (default) results in a hat matrix of
dimension \emph{n(n-1)/2} x \emph{s}, where \emph{s} is the sum of
the number of independent comparisons from each design.

Use of \code{type = "studies"} results in a hat matrix of dimension
\emph{n(n-1)/2} x \emph{l}, where \emph{l} is the number of
independent pairwise comparisons, i.e., a three-arm study
contributes two pairwise comparisons.
}

\subsection{Hat matrices by Davies et al. (2022)}{

One of three hat matrices is estimated if \code{method = "Davies"}.

Here, we focus on the hat matrix of the aggregate (two-step)
version of the graph theoretical NMA model. In the first step, a
pairwise meta-analysis is performed across each edge using the
adjusted weights (these account for correlations due to multi-arm
trials). From this we obtain direct treatment effect estimates (and
corresponding aggregate weights) associated with each edge. In step
two, we combine these direct estimates in a network meta analysis
to obtain the network estimates.  This is done using weighted least
squares regression. The hat matrix associated with this second step
is called the \emph{aggregate hat matrix}.

All three versions of the aggregate hat matrix contain the same
information: the second two can be derived directly from the
first. They differ in their dimensionality.

Each row of the hat matrix that represents a treatment comparison
(\emph{ij}) describes the flow of evidence through each edge for
that comparison. This defines a directed acyclic 'flow graph' from
node \emph{i} to node \emph{j}.

(1) Use of \code{type = "short"} (default) results in a hat matrix
of dimension \emph{e} x \emph{e}, where \emph{e} is the number of
(unique) edges (direct comparisons) in the network. This is the
aggregate hat matrix described in Davies et al. (2022). Each row
and column represents a pair of treatments for which there is at
least one direct comparison.

(2) Use of \code{type = "long"} results in a hat matrix of
dimension \emph{n(n-1)/2} x \emph{e}. There is a row for every
possible pair of treatments in the network - regardless of whether
there is direct evidence for this comparison. Each column
represents a pair of treatments for which there is at least one
direct comparison. The extra rows can be calculated from the short
hat matrix using consistency equations.


(3) Use of \code{type = "full"} results in a hat matrix of
dimension \emph{n(n-1)/2} x \emph{n(n-1)/2}. In comparison to the
long hat matrix, columns of zeroes are added for comparisons that
do not have any direct evidence. Therefore, there is a row and
column for every pair of treatments in the network. This hat matrix
is used to calculate the transition matrices for the random walk in
\code{\link{netcontrib}}.
}
}
\examples{
# Only consider first ten studies for concise output
first10 <- subset(Dong2013, id <= 10)
pw <- pairwise(treatment, death, randomized, studlab = id,
  data = first10, sm = "OR")
nma <- netmeta(pw, common = FALSE)

hatmatrix(nma)
hatmatrix(nma, method = "k")
hatmatrix(nma, method = "k", type = "studies")
hatmatrix(nma, method = "d")
hatmatrix(nma, method = "d", type = "long")
hatmatrix(nma, method = "d", type = "full")

}
\references{
Davies AL, Papakonstantinou T, Nikolakopoulou A, Rücker G, Galla T
(2022):
Network meta-analysis and random walks.
\emph{Statistics in Medicine},
\bold{41}, 2091--2114

Krahn U, Binder H, König J (2013):
A graphical tool for locating inconsistency in network meta-analyses.
\emph{BMC Medical Research Methodology},
\bold{13}, 35

Rücker G (2012):
Network meta-analysis, electrical networks and graph theory.
\emph{Research Synthesis Methods},
\bold{3}, 312--24
}
\seealso{
\code{\link{netmeta}}, \code{\link{netcontrib}},
  \code{\link{netheat}}, \code{\link[metabook]{Dong2013}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gemtc2netmeta.R
\name{gemtc2netmeta}
\alias{gemtc2netmeta}
\title{Create a netmeta object from a gemtc object}
\usage{
gemtc2netmeta(
  x,
  keep.samples = TRUE,
  level = gs("level"),
  level.ma = gs("level.ma"),
  reference.group = levels(x$model$network$treatments$id)[1],
  baseline.reference = gs("baseline.reference"),
  small.values = gs("small.values"),
  all.treatments = gs("all.treatments"),
  seq = gs("seq"),
  backtransf = gs("backtransf"),
  nchar.trts = gs("nchar.trts"),
  nchar.studlab = gs("nchar.studlab")
)
}
\arguments{
\item{x}{A \code{gemtc} object created with R package \bold{gemtc}.}

\item{keep.samples}{A logical indicating whether to keep the generated
samples.}

\item{level}{The level used to calculate confidence intervals for
individual comparisons.}

\item{level.ma}{The level used to calculate confidence intervals
for network estimates.}

\item{reference.group}{Reference treatment (first treatment is used
if argument is missing).}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect (passed on to
\code{\link{netrank}}, can be abbreviated.}

\item{all.treatments}{A logical or \code{"NULL"}. If \code{TRUE},
matrices with all treatment effects, and confidence limits will
be printed.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments in printouts.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf = TRUE}, results for \code{sm = "OR"} are
presented as odds ratios rather than log odds ratios, for
example.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{nchar.studlab}{A numeric defining the minimum number of
characters used to create unique study labels.}
}
\value{
A netmeta object with additional class "netmeta.gemtc".
}
\description{
Auxiliary function to create a netmeta object from a gemtc object
(experimental feature).
}
\examples{
\donttest{
if (requireNamespace("gemtc", quietly = TRUE)) {
library("gemtc")
# Load the example network and generate a consistency model:
model <- mtc.model(smoking, type="consistency")
results <- mtc.run(model, thin = 10)
detach("package:gemtc")

# Print results in netmeta layout
mtc <- gemtc2netmeta(results)
mtc
# SUCRAs
netrank(mtc)
# Rankogram
rankogram(mtc)
}
}

}
\seealso{
\code{\link{netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netcontrib.R
\name{netcontrib}
\alias{netcontrib}
\alias{print.netcontrib}
\title{Contribution matrix in network meta-analysis}
\usage{
netcontrib(
  x,
  method = "shortestpath",
  hatmatrix.F1000 = FALSE,
  common = x$common,
  random = x$random,
  study = FALSE,
  path = FALSE,
  nchar.trts = x$nchar.trts,
  warn.deprecated = gs("warn.deprecated"),
  verbose = FALSE,
  ...
)

\method{print}{netcontrib}(
  x,
  common = x$x$common,
  random = x$x$random,
  digits = 4,
  nchar.trts = x$nchar.trts,
  legend = gs("legend"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta} or \code{netcontrib}.}

\item{method}{A character string indicating which method is to
calculate the contribution matrix. Either \code{"randomwalk"},
\code{"shortestpath"}, \code{"cccp"}, or \code{"pseudoinverse"},
can be abbreviated.}

\item{hatmatrix.F1000}{A logical indicating whether hat matrix
given in F1000 article should be used for \code{method =
"shortestpath"}.}

\item{common}{A logical indicating whether a contribution matrix
should be printed for the common effects network meta-analysis.}

\item{random}{A logical indicating whether a contribution matrix
should be printed for the random effects network meta-analysis.}

\item{study}{A logical indicating whether contributions of individual studies
 for each comparison should be returned (currently only supported for the
\code{"shortestpath"} method).}

\item{path}{A logical indicating whether contributions of individual paths
for each comparison should be returned (currently only supported for the
\code{"shortestpath"} method).}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{verbose}{A logical indicating whether progress information
should be printed.}

\item{\dots}{Additional arguments.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{legend}{A logical indicating whether a legend should be
printed.}
}
\value{
An object of class \code{netcontrib} with corresponding
\code{print} function. The object is a list containing the
following components:
\item{common}{Numeric matrix of percentage contributions of direct
  comparisons for each network comparison for the common effects
  model.}
\item{random}{Numeric matrix of percentage contributions of direct
  comparisons for each network comparison for the random effects
  model.}
\item{study.common}{Data frame containing study contributions for each
  comparison for the common effects model.}
\item{study.random}{Data frame containing study contributions for each
  comparison for the random effects model.}
\item{path.common}{Data frame containing path contributions for each
  comparison for the common effects model.}
\item{path.random}{Data frame containing path contributions for each
  comparison for the random effects model.}
\item{x}{As defined above.}
\item{tictoc.common}{Computation times under common effects model
  (if R package \bold{tictoc} is installed).}
\item{tictoc.random}{Computation times under random effects model
  (if R package \bold{tictoc} is installed).}
with the contribution matrices for common and random NMA. Each
matrix has the percentage contributions of each direct comparison
as columns for each network comparison, direct or indirect as rows.
}
\description{
This function generates the contribution of direct comparisons to
every network treatment comparison. The output is a matrix where
rows represent network treatment effects and columns represent the
contribution of direct treatment effects.
}
\details{
In network meta-analysis (NMA), it is important to assess the
influence of limitations or other characteristics of individual
studies on the estimates obtained from the network. To this end,
the contribution matrix shows how much each direct treatment effect
contributes to each treatment effect estimate from network
meta-analysis.

We use ideas from graph theory to derive the proportion that is
contributed by each direct treatment effect using R package
\bold{igraph}. We start
with the 'projection' matrix in a two-step network meta-analysis model,
called the H matrix, which is analogous to the hat matrix in a
linear regression model. H entries are translated to proportion
contributions based on the observation that the rows of H can be
interpreted as flow networks. A stream is defined as the
composition of a path and its associated flow (Papakonstantinou et
al., 2018).

To account for multi-arm trials, we use the H matrix from a
two-step (aggregate) version of the graph theoretical NMA model
(Davies et al., 2022). This H matrix can be obtained from
\code{\link{hatmatrix}} with argument \code{method = "davies"}.

Four methods are implemented to estimate the proportion
contributions:

(1) If argument \code{method = "randomwalk"}, an analytical
random-walk (RW) approach is used (Davies et al., 2022). Here, the
"full" version of the aggregate H matrix (\code{\link{hatmatrix}}
with arguments \code{method = "davies"} and \code{type = "full"})
is used to define RW transition matrices. For each pair of
treatments (ij) in the network, the elements in the corresponding
row of H-full define a transition matrix from node i to node j. We
use the \bold{igraph} package to find every (directed) path from
node i to node j. The flow through each path is then equal to the
probability that a walker takes that path. This is simply the
product of the transition probabilities associated with each edge
along the path.

(2) If argument \code{method = "shortestpath"}, an iterative
algorithm is used (Papakonstantinou et al., 2018). Broadly
speaking, each iteration of the algorithm consists of the following
steps:
\itemize{
\item A path in the evidence flow network is selected.
\item The minimum flow through the edges making up the path is
  identified. This is assigned as the flow associated with the
  path.
\item The flow of the path is subtracted from the values of
  flow in the edges that make up that path. This means that the edge
  corresponding to the minimum flow in that path is removed from the
  graph.
\item A new path is then selected from the remaining graph. The process
  repeats until all the evidence flow in the edges has been assigned to a
  path.
}
In the original F1000 paper (Papakonstantinou et al., 2018), the
hat matrix used did not account for correlations due to multi-arm
trials. For reproducibility the result of this version can be
obtained by specifying \code{hatmatrix.F1000 = TRUE} for
\code{method = "shortestpath"}. For other purposes, this method is
not recommended.

(3) If argument \code{method = "cccp"}, contributions are estimated
using \code{\link[cccp]{l1}} from R package \bold{cccp} which must be
available (Rücker et al., 2024).

(4) If argument \code{method = "pseudoinverse"}, contributions are
derived from an L2 solution based on a Moore-Penrose pseudoinverse
(Rücker et al., 2024).

Once the streams have been identified by any method, the proportion
contribution of each direct comparison is equal to the sum over the
flow of evidence in each path containing that edge divided by the
number of edges that make up that path.

By default, treatment names are not abbreviated in
printouts. However, in order to get more concise printouts,
argument \code{nchar.trts} can be used to define the minimum number
of characters for abbreviated treatment names (see
\code{\link{abbreviate}}, argument \code{minlength}). R function
\code{\link{treats}} is utilised internally to create abbreviated
treatment names.

Calculation of network contributions can be compute-intensive for
the random-walk approach in large networks. Crude information on
the computation progress is printed if argument \code{verbose} is
\code{TRUE}. In addition, computation times are printed if R
package \bold{tictoc} is installed.
}
\examples{
\donttest{
# Use the Woods dataset
#
pw <- pairwise(treatment, event = r, n = N,
  studlab = author, data = Woods2010, sm = "OR")

nma <- netmeta(pw)

# Shortest path approach (default)
netcontrib(nma)
}

\dontrun{ 
# Random walk approach
netcontrib(nma, method = "r")
}

}
\references{
Davies AL, Papakonstantinou T, Nikolakopoulou A, Rücker G, Galla T
(2022):
Network meta-analysis and random walks.
\emph{Statistics in Medicine},
\bold{41}, 2091--2114

Papakonstantinou T, Nikolakopoulou A, Rücker G, Chaimani A, Schwarzer G,
Egger M, Salanti G (2018):
Estimating the contribution of studies in network meta-analysis:
paths, flows and streams.
\emph{F1000Research}

Rücker G, Papakonstantinou T, Nikolakopoulou A, Schwarzer G, Galla T,
Davies AL (2024):
Shortest path or random walks? A framework for path weights in network
meta-analysis.
\emph{Statistics in Medicine}.
}
\seealso{
\code{\link{netmeta}}, \code{\link[metabook]{Woods2010}}
}
\author{
Theodoros Papakonstantinou \email{dev@tpapak.com}, Annabel
  Davies \email{annabel.davies@manchester.ac.uk}
}
\keyword{contribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.netbind.R
\name{forest.netbind}
\alias{forest.netbind}
\alias{plot.netbind}
\title{Forest plot showing results of two or more network meta-analyses}
\usage{
\method{forest}{netbind}(
  x,
  pooled = ifelse(x$x$random, "random", "common"),
  equal.size = gs("equal.size"),
  leftcols = "studlab",
  leftlabs = "Treatment",
  rightcols = c("effect", "ci"),
  rightlabs = NULL,
  subset.treatments,
  digits = gs("digits.forest"),
  digits.prop = max(gs("digits.pval") - 2, 2),
  backtransf = x$backtransf,
  lab.NA = gs("lab.NA"),
  smlab,
  ...
)

\method{plot}{netbind}(x, ...)
}
\arguments{
\item{x}{An object of class \code{netbind}.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{equal.size}{A logical indicating whether all squares should
be of equal size. Otherwise, the square size is proportional to
the precision of estimates.}

\item{leftcols}{A character vector specifying columns to be plotted
on the left side of the forest plot (see Details).}

\item{leftlabs}{A character vector specifying labels for columns on
left side of the forest plot.}

\item{rightcols}{A character vector specifying columns to be
plotted on the right side of the forest plot (see Details).}

\item{rightlabs}{A character vector specifying labels for columns
on right side of the forest plot.}

\item{subset.treatments}{A character vector specifying treatments
to show in forest plot as comparators to the reference.}

\item{digits}{Minimal number of significant digits for treatment
effects and confidence intervals, see \code{print.default}.}

\item{digits.prop}{Minimal number of significant digits for the
direct evidence proportion.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{lab.NA}{A character string to label missing values.}

\item{smlab}{A label printed at top of figure. By default, text
indicating either common or random effects model is printed.}

\item{\dots}{Additional arguments for \code{\link[meta]{forest.meta}}
function.}
}
\description{
Forest plot to show network estimates of two or more network
meta-analyses.
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window.

The arguments \code{leftcols} and \code{rightcols} can be used to
specify columns which are plotted on the left and right side of the
forest plot, respectively. If argument \code{rightcols} is
\code{FALSE}, no columns will be plotted on the right side.

For more information see help page of \code{\link[meta]{forest.meta}}
function.
}
\examples{
# Examples: example(netbind)

}
\seealso{
\code{\link{netbind}}, \code{\link{netcomb}},
  \code{\link[meta]{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netposet.R
\name{netposet}
\alias{netposet}
\alias{print.netposet}
\title{Partial order of treatments in network meta-analysis}
\usage{
netposet(
  ...,
  outcomes,
  treatments,
  small.values,
  common,
  random,
  fixed,
  comb.fixed,
  comb.random
)

\method{print}{netposet}(x, pooled = ifelse(x$random, "random", "common"), ...)
}
\arguments{
\item{\dots}{See details.}

\item{outcomes}{A character vector with outcome names.}

\item{treatments}{A character vector with treatment names.}

\item{small.values}{See details.}

\item{common}{A logical indicating whether to show results for the
common effects model.}

\item{random}{A logical indicating whether to show results for the
random effects model.}

\item{fixed}{Ignored deprecated argument (replaced by
\code{common}).}

\item{comb.fixed}{Ignored deprecated argument (replaced by
\code{common}).}

\item{comb.random}{Ignored deprecated argument (replaced by
\code{random}).}

\item{x}{An object of class \code{netposet}.}

\item{pooled}{A character string indicating whether Hasse diagram
should be drawn for common (\code{"common"}) or random effects
model (\code{"random"}). Can be abbreviated.}
}
\value{
An object of class \code{netposet} with corresponding \code{print},
\code{plot}, and \code{hasse} functions. The object is a list
containing the following components:
\item{P.common}{Ranking matrix with rows corresponding to treatments
  and columns corresponding to outcomes (common effects model).}
\item{M0.common}{Hasse matrix skipping unnecessary paths (common
  effects model).}
\item{M.common}{"Full" Hasse matrix (common effects model).}
\item{O.common}{Matrix with information about partial ordering
  (common effects model).}
\item{P.random}{Ranking matrix with rows corresponding to
  treatments and columns corresponding to outcomes (random effects
  model).}
\item{M0.random}{Hasse matrix skipping unnecessary paths (random
  effects model).}
\item{M.random}{"Full" Hasse matrix (random effects model).}
\item{O.random}{Matrix with information about partial ordering
  (random effects model).}
\item{small.values, common, random}{As.defined above.}
\item{call}{Function call.}
\item{version}{Version of R package netmeta used to create object.}
}
\description{
Partial order of treatments in network meta-analysis. The set of
treatments in a network is called a partially ordered set (in
short, a \emph{poset}), if different outcomes provide different
treatment ranking lists.
}
\details{
In network meta-analysis, frequently different outcomes are
considered which may each provide a different ordering of
treatments. The concept of a partially ordered set (in short, a
\emph{poset}, Carlsen & Bruggemann, 2014) of treatments can be used
to gain further insights in situations with apparently conflicting
orderings. This implementation for rankings in network meta-analyis
is described in Rücker & Schwarzer (2017).

In function \code{netposet}, argument \code{\dots{}} can be any of the
following:
\itemize{
\item arbitrary number of \code{netrank} objects providing
  P-scores;
\item arbitrary number of \code{netmeta} objects;
\item single ranking matrix with each column providing P-scores
  (Rücker & Schwarzer 2015) or SUCRA values (Salanti et al. 2011)
  for an outcome and rows corresponding to treatments.
}
Note, albeit in general a ranking matrix is not constrained to have
values between 0 and 1, \code{netposet} stops with an error in this
case as this function expects a matrix with P-scores or SUCRA
values.

Argument \code{outcomes} can be used to label outcomes. If argument
\code{outcomes} is missing,
\itemize{
\item column names of the ranking matrix are used as outcome labels
  (if first argument is a ranking matrix and column names are
  available);
\item capital letters 'A', 'B', \dots{} are used as outcome labels
  and a corresponding warning is printed.
}

Argument \code{treatments} can be used to provide treatment labels
if the first argument is a ranking matrix. If argument
\code{treatment} is missing,
\itemize{
\item row names of the ranking matrix are used as treatment labels
  (if available);
\item letters 'a', 'b', \dots{} are used as treatment labels and a
  corresponding warning is printed.
}
If argument \code{\dots{}} consists of \code{netmeta} objects,
\code{netrank} is called internally to calculate P-scores. In this
case, argument \code{small.values} can be used to specify for each
outcome whether small values are beneficial (\code{"desirable"}) or
harmfull (\code{"undesirable"}); see \code{\link{netrank}}. This
argument is ignored for a ranking matrix and \code{netrank}
objects.

Arguments \code{common} and \code{random} can be used to define
whether results should be printed and plotted for common and random
effects model. If netmeta and netrank objects are provided in
argument \code{\dots{}}, values for \code{common} and \code{random}
within these objects are considered; if these values are not
unique, argument \code{common} or \code{random} are set to
\code{TRUE}.

In function \code{print.netposet}, argument \code{\dots{}} is
passed on to the printing function.
}
\examples{
\donttest{
# Define order of treatments in depression data set Linde2015
#
trts <- c("TCA", "SSRI", "SNRI", "NRI",
  "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")

# Outcome labels
#
outcomes <- c("Early response", "Early remission")

# (1) Early response
#
pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma1 <- netmeta(pw1, common = FALSE,
  seq = trts, ref = "Placebo", small.values = "undesirable")

# (2) Early remission
#
pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma2 <- netmeta(pw2, common = FALSE,
  seq = trts, ref = "Placebo", small.values = "undesirable")

# Partial order of treatment rankings (two outcomes)
#
po12 <- netposet(netrank(nma1), netrank(nma2), outcomes = outcomes)

# Scatter plot
#
plot(po12)

# Biplot (same scatter plot as only two outcomes considered)
#
plot(po12, "biplot")

# Hasse diagram for outcomes early response and early remission
#
\dontrun{
if (requireNamespace("Rgraphviz", quietly = TRUE))
  hasse(po12)
}

# Scatter plot for outcomes early response and early remission
#
oldpar <- par(pty = "s")
plot(po12)
par(oldpar)


# Consider five outcomes
#
# Outcome labels
#
outcomes <- c("Early response", "Early remission",
  "Lost to follow-up", "Lost to follow-up due to AEs",
   "Adverse events (AEs)")

# (3) Loss to follow-up
#
pw3 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(loss1, loss2, loss3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma3 <- netmeta(pw3, common = FALSE,
  seq = trts, ref = "Placebo", small.values = "desirable")

# (4) Loss to follow-up due to adverse events
#
pw4 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(loss.ae1, loss.ae2, loss.ae3), n = list(n1, n2, n3),
  studlab = id, data = subset(Linde2015, id != 55), sm = "OR")
#
nma4 <- netmeta(pw4, common = FALSE,
  seq = trts, ref = "Placebo", small.values = "desirable")

# (5) Adverse events
#
pw5 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(ae1, ae2, ae3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma5 <- netmeta(pw5, common = FALSE,
  seq = trts, ref = "Placebo", small.values = "desirable")

# Partial order of treatment rankings (based on netrank() objects)
#
po.ranks <- netposet(netrank(nma1), netrank(nma2),
  netrank(nma3), netrank(nma4), netrank(nma5), outcomes = outcomes)

# Same result (based on netmeta() objects)
#
po.nets <- netposet(nma1, nma2, nma3, nma4, nma5,
  outcomes = outcomes)
#
all.equal(po.ranks, po.nets)

# Print matrix with P-scores (random effects model)
#
po.nets$P.random

\dontrun{
# Hasse diagram for all outcomes (random effects model)
#
if (requireNamespace("Rgraphviz", quietly = TRUE))
  hasse(po.ranks)
}
}

# Example using ranking matrix with P-scores
#
# Ribassin-Majed L, Marguet S, Lee A.W., et al. (2017):
# What is the best treatment of locally advanced nasopharyngeal
# carcinoma? An individual patient data network meta-analysis.
# Journal of Clinical Oncology, 35, 498-505
#
outcomes <- c("OS", "PFS", "LC", "DC")
treatments <- c("RT", "IC-RT", "IC-CRT", "CRT",
  "CRT-AC", "RT-AC", "IC-RT-AC")
#
# P-scores (from Table 1)
#
pscore.os  <- c(15, 33, 63, 70, 96, 28, 45) / 100
pscore.pfs <- c( 4, 46, 79, 52, 94, 36, 39) / 100
pscore.lc  <- c( 9, 27, 47, 37, 82, 58, 90) / 100
pscore.dc  <- c(16, 76, 95, 48, 72, 32, 10) / 100
#
pscore.matrix <- data.frame(pscore.os, pscore.pfs, pscore.lc, pscore.dc)
rownames(pscore.matrix) <- treatments
colnames(pscore.matrix) <- outcomes
pscore.matrix
#
po <- netposet(pscore.matrix)
po12 <- netposet(pscore.matrix[, 1:2])
po
po12
#
\dontrun{
if (requireNamespace("Rgraphviz", quietly = TRUE)) {
  hasse(po)
  hasse(po12)
}
}
#
oldpar <- par(pty = "s")
plot(po12)
par(oldpar)

}
\references{
Carlsen L, Bruggemann R (2014):
Partial order methodology: a valuable tool in chemometrics.
\emph{Journal of Chemometrics},
\bold{28}, 226--34

Rücker G, Schwarzer G (2015):
Ranking treatments in frequentist network meta-analysis works
without resampling methods.
\emph{BMC Medical Research Methodology},
\bold{15}, 58

Rücker G, Schwarzer G (2017):
Resolve conflicting rankings of outcomes in network meta-analysis:
Partial ordering of treatments.
\emph{Research Synthesis Methods},
\bold{8}, 526--36

Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
\code{\link{netmeta}}, \code{\link{netrank}},
  \code{\link{plot.netrank}}, \code{\link{hasse}},
  \code{\link{plot.netposet}}, \code{\link[metabook]{Linde2015}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.netsplit.R
\name{plot.netsplit}
\alias{plot.netsplit}
\title{Scatter plot to evaluate local inconsistency in network meta-analysis}
\usage{
\method{plot}{netsplit}(
  x,
  type = "Bland-Altman",
  pooled = ifelse(x$x$random, "random", "common"),
  subset = NULL,
  only.reference = x$only.reference,
  level = x$level.ma,
  vertical = TRUE,
  size.points = NULL,
  points.only.inc = FALSE,
  col = "gray50",
  col.inc = "red",
  labels = TRUE,
  labels.only.inc = FALSE,
  size.labels = 5,
  col.labels = col,
  col.labels.inc = col.inc,
  max_overlap = 10,
  nchar.trts = x$nchar.trts,
  text.pooled,
  main,
  xlab = NULL,
  ylab = NULL,
  xlim = NULL,
  ylim = NULL,
  tag = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netsplit}.}

\item{type}{A character string indicating which figure type is to be used.
Either \code{"Bland-Altman"} or \code{"Wilson"}, can be abbreviated.
See Details.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{subset}{An optional logical vector specifying a subset of
comparisons to consider (must be of same length as the total number of
comparisons).}

\item{only.reference}{A logical indicating whether only comparisons
with the reference group should be considered.}

\item{level}{The level used to calculate confidence intervals for local
local inconsistency.}

\item{vertical}{A logical indicating whether confidence intervals should be
plotted vertically or horizontally; only considered if argument
\code{type = "Wilson"}. The interpretation is the same.}

\item{size.points}{A single numeric defining the point size. If \code{NULL},
point sizes are proportional to the proportion of indirect evidence.}

\item{points.only.inc}{A logical indicating whether (larger) points should
be only shown for comparisons with local inconsistency.}

\item{col}{Colour of points and confidence intervals.}

\item{col.inc}{Colour of points and confidence intervals for comparisons
with local inconsistency.}

\item{labels}{A logical value indicating whether to show comparisons labels,
or an expression or character vector specifying the labels to use.}

\item{labels.only.inc}{A logical indicating whether labels are only shown
for comparisons with local inconsistency.}

\item{size.labels}{A single numeric defining the size of comparison labels.}

\item{col.labels}{Colour of comparison labels.}

\item{col.labels.inc}{Colour of labels for comparisons with local
inconsistency.}

\item{max_overlap}{An integer specifying the maximum number of overlapping
labels allowed in the scatter plot.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names in comparisons.}

\item{text.pooled}{A character string used to describe the statistical model.}

\item{main}{Main title.}

\item{xlab}{A label for the x-axis.}

\item{ylab}{A label for the y-axis.}

\item{xlim}{The x limits (min, max) of the plot.}

\item{ylim}{The y limits (min, max) of the plot.}

\item{tag}{The text for the tag label (see \code{\link[ggplot2]{labs}}).}

\item{\dots}{Additional arguments (ignored).}
}
\value{
An object of class \code{"plot.netsplit"}. The object is a list containing
the following components:
\item{plot}{A ggplot2 object.}
\item{data}{Data set used to create the plot.}
}
\description{
Scatter plot to show the difference between direct and indirect evidence in
network meta-analysis to assess local inconsistency.
}
\details{
A scatter plot is drawn in the active graphics window.

By default (argument \code{type = "Bland-Altman"}), a variant of the
Bland-Altman plot is shown (Bland & Altman, 1995). In this case, the
network estimates are shown on the horizontal axis and the difference between
direct and indirect effect estimates on the vertical axis. The confidence
intervals are those from the differences.

If argument \code{type = "Wilson"}, a scatter plot with direct effect
estimates on the horizontal and indirect estimates on the vertical axis
is produced (Wilson et al., 2026). The confidence intervals are
calculated from the indirect (argument \code{vertical = TRUE}) or direct
effect estimates and the standard error of the difference between indirect
and direct effect estimates.

For both plot types, the dashed green line corresponds to comparisons with
equal direct and indirect effect estimates.
}
\examples{
# Examples: example(netsplit)

}
\references{
Bland JM, Altman DG (1995):
Comparing methods of measurement: Why plotting difference against standard
method is misleading.
\emph{The Lancet},
\bold{346}, 1085--87

Wilson H, Schönstein A, Robson S, Bonofiglio F (2026):
A novel approach for visualizing local consistency in
network meta-analysis.
\emph{Research Synthesis Methods},
1--15
}
\seealso{
\code{\link{forest.netsplit}}
}
\author{
Federico Bonofiglio \email{bonostat@gmx.de},
  Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.netsplit.R
\name{forest.netsplit}
\alias{forest.netsplit}
\title{Forest plot for direct and indirect evidence}
\usage{
\method{forest}{netsplit}(
  x,
  pooled = ifelse(x$x$random, "random", "common"),
  show = x$show,
  subgroup = "comparison",
  overall = x$overall,
  direct = x$direct,
  indirect = x$indirect,
  prediction = x$prediction,
  only.reference = x$only.reference,
  sortvar = NULL,
  subset = NULL,
  text.overall = "Network estimate",
  text.direct = "Direct estimate",
  text.indirect = "Indirect estimate",
  text.predict = "Prediction interval",
  type.overall,
  type.direct,
  type.indirect,
  col.square = "gray",
  col.square.lines = col.square,
  col.inside = "white",
  col.diamond = "gray",
  col.diamond.lines = "black",
  col.predict = "red",
  col.predict.lines = "black",
  col.subgroup = "black",
  equal.size = TRUE,
  leftcols,
  leftlabs,
  rightcols = c("effect", "ci"),
  rightlabs = NULL,
  digits = gs("digits.forest"),
  digits.prop = max(gs("digits.pval") - 2, 2),
  backtransf = x$backtransf,
  lab.NA = "",
  smlab,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netsplit}.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be plotted. Can be abbreviated.}

\item{show}{A character string indicating which comparisons should
be printed (see Details).}

\item{subgroup}{A character string indicating which layout should
be used in forest plot: subgroups by comparisons
(\code{"comparison"}) or subgroups by estimates
(\code{"estimate"}). Can be abbreviated.}

\item{overall}{A logical indicating whether network meta-analysis
estimates should be printed.}

\item{direct}{A logical indicating whether direct estimates should
be printed.}

\item{indirect}{A logical indicating whether indirect estimates
should be printed.}

\item{prediction}{A logical indicating whether prediction intervals
should be printed.}

\item{only.reference}{A logical indicating whether only comparisons
with the reference group should be printed.}

\item{sortvar}{An optional vector used to sort comparisons (must be
of same length as the total number of comparisons).}

\item{subset}{An optional logical vector specifying a subset of
comparisons to print (must be of same length as the total number of
comparisons).}

\item{text.overall}{A character string used in the plot to label
the network estimates.}

\item{text.direct}{A character string used in the plot to label the
direct estimates.}

\item{text.indirect}{A character string used in the plot to label
the indirect estimates.}

\item{text.predict}{A character string used in the plot to label
the prediction interval.}

\item{type.overall}{A character string specifying how to plot
treatment effects and confidence intervals for the overall
network evidence.}

\item{type.direct}{A character string specifying how to plot
treatment effects and confidence intervals for the direct
evidence.}

\item{type.indirect}{A character string specifying how to plot
treatment effects and confidence intervals for the indirect
evidence.}

\item{col.square}{The colour for squares.}

\item{col.square.lines}{The colour for the outer lines of squares.}

\item{col.inside}{The colour for results and confidence limits if
confidence limits are completely within squares squares.}

\item{col.diamond}{The colour of diamonds.}

\item{col.diamond.lines}{The colour of the outer lines of diamonds.}

\item{col.predict}{Background colour of prediction intervals.}

\item{col.predict.lines}{Colour of outer lines of prediction
intervals.}

\item{col.subgroup}{The colour to print information on subgroups, i.e.,
pairwise comparisons.}

\item{equal.size}{A logical indicating whether all squares should
be of equal size. Otherwise, the square size is proportional to
the precision of estimates.}

\item{leftcols}{A character vector specifying columns to be plotted
on the left side of the forest plot (see Details).}

\item{leftlabs}{A character vector specifying labels for columns on
left side of the forest plot.}

\item{rightcols}{A character vector specifying columns to be
plotted on the right side of the forest plot (see Details).}

\item{rightlabs}{A character vector specifying labels for columns
on right side of the forest plot.}

\item{digits}{Minimal number of significant digits for treatment
effects and confidence intervals, see \code{print.default}.}

\item{digits.prop}{Minimal number of significant digits for the
direct evidence proportion.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{lab.NA}{A character string to label missing values.}

\item{smlab}{A label printed at top of figure. By default, text
indicating either common or random effects model is printed.}

\item{\dots}{Additional arguments for \code{\link[meta]{forest.meta}}
function.}
}
\description{
Forest plot to show direct and indirect evidence in network
meta-analysis. Furthermore, estimates from network meta-analysis
as well as prediction intervals can be printed.
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window.

The arguments \code{leftcols} and \code{rightcols} can be used to
specify columns which are plotted on the left and right side of the
forest plot, respectively. If argument \code{rightcols} is
\code{FALSE}, no columns will be plotted on the right side.

If direct estimates are included in the forest plot (\code{direct =
TRUE}, default), the following columns will be printed on the left
side of the forest plot: the comparisons (column \code{"studlab"}
in \code{\link[meta]{forest.meta}}), number of pairwise comparisons
(\code{"k"}), direct evidence proportion (\code{"k"}), and
I\eqn{^2} from pairwise comparison (\code{"I2"}).

If direct estimates are not included in the forest plot
(\code{direct = FALSE}), only the comparisons (\code{"studlab"})
are printed on the left side of the forest plot.

For more information see help page of \code{\link[meta]{forest.meta}}
function.

Argument \code{show} determines which comparisons are printed:
\tabular{ll}{
\dQuote{all} \tab All comparisons \cr
\dQuote{both} \tab Only comparisons contributing both direct and
  indirect evidence \cr
\dQuote{with.direct} \tab Comparisons providing direct evidence \cr
\dQuote{direct.only} \tab Comparisons providing only direct
  evidence \cr
\dQuote{indirect.only} \tab Comparisons providing only indirect
  evidence
}
}
\examples{
# Examples: example(netsplit)

}
\seealso{
\code{\link[meta]{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatplot.R
\name{heatplot}
\alias{heatplot}
\title{Generic function for heat plots}
\usage{
heatplot(x, ...)
}
\arguments{
\item{x}{An R object.}

\item{\dots}{Additional arguments.}
}
\description{
Generic function for heat plots
}
\details{
For more details, look at the following function to generate heat
plots:
\itemize{
\item \code{\link{heatplot.netmeta}}
\item \code{\link[crossnma]{heatplot.crossnma}} (if installed)
}
}
\examples{
# Only consider first five studies (to reduce runtime of example)
#
studies <- unique(Senn2013$study)
Senn2013.5 <- subset(Senn2013, study \%in\% studies[1:5])

# Transform data from long arm-based to contrast-based format
#
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013.5,
  varnames = c("MD", "seMD"))

# Conduct random effects network meta-analysis with
# placebo as reference treatment
#
nma <- netmeta(pw, common = FALSE, reference = "plac")

# Generate a heat plot (with abbreviated treatment labels)
#
heatplot(nma, nchar.trts = 4) 

}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de
}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netgraph.netconnection.R
\name{netgraph.netconnection}
\alias{netgraph.netconnection}
\title{Network graph for objects of class netconnection}
\usage{
\method{netgraph}{netconnection}(
  x,
  seq,
  col = seq_len(x$n.subnets),
  reference.group = NULL,
  plastic = gs("plastic"),
  start.layout = "circle",
  iterate = FALSE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netconnection}.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments arrangement (anticlockwise if \code{start.layout =
"circle"}).}

\item{col}{A single color (or vector of colors) for lines
connecting treatments if argument \code{plastic = FALSE}
(see Details).}

\item{reference.group}{Reference treatment (only relevant for
disconnected networks).}

\item{plastic}{A logical indicating whether the appearance of the
comparisons should be in '3D look'.}

\item{start.layout}{A character string indicating which starting
layout is used (see \code{\link{netgraph.netmeta}}).}

\item{iterate}{A logical indicating whether the stress majorization
algorithm is carried out for optimization of the layout
(see \code{\link{netgraph.netmeta}}).}

\item{\dots}{Additional arguments passed on to
\code{\link{netgraph.netmeta}} (see Details).}
}
\description{
This function generates a graph of the evidence network.
}
\details{
Argument \code{col} can be a single color for all lines / edges, a vector
of length equal to the number of edges, or a vector of length equal
to the number of subnetworks. Argument \code{reference.group} is
only considered in disconnected networks, i.e., if more than one
(sub)network exists, and if argument \code{col} provides colors for
subnetworks. In this case, the first color provided in argument
\code{col} defines the color for the subnetwork with the reference
treatment.
}
\examples{
# Artificial example with two subnetworks
#
t1 <- c("G", "B", "B", "D", "A", "F")
t2 <- c("B", "C", "E", "E", "H", "A")
#
nc1 <- netconnection(t1, t2)
print(nc1, details = TRUE)

netgraph(nc1, points = TRUE, adj = 0.5, bg.points = "lightgray")
netgraph(nc1, points = TRUE, adj = 0.5, bg.points = "lightgray",
  plastic = TRUE)

}
\seealso{
\code{\link{netconnection}}, \code{\link{netgraph.netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netmeasures.R
\name{netmeasures}
\alias{netmeasures}
\title{Measures for characterizing a network meta-analysis}
\usage{
netmeasures(
  x,
  random = x$random | !missing(tau.preset),
  tau.preset = x$tau.preset,
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{random}{A logical indicating whether random effects model
should be used to calculate network measures.}

\item{tau.preset}{An optional value for the square-root of the
between-study variance \eqn{\tau^2}.}

\item{warn}{A logical indicating whether warnings should be
printed.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
A list containing the following components:
\item{random, tau.preset}{As defined above.}
\item{proportion}{A named vector of the direct evidence proportion
  of each network estimate.}
\item{meanpath}{A named vector of the mean path length of each
  network estimate.}
\item{minpar}{A named vector of the minimal parallelism on
  aggregated level of each network estimate.}
\item{minpar.study}{A named vector of the minimal parallelism on
  study level of each network estimate.}
\item{H.tilde }{Design-based hat matrix with information on
  absolute evidence flow per design. The number of rows is equal to
  the number of possible pairwise treatment comparisons and the
  number of columns is equal to the number of designs.}
}
\description{
This function provides measures for quantifying the direct evidence
proportion, the mean path length and the minimal parallelism (the
latter on aggregated and study level) of mixed treatment
comparisons (network estimates) as well as the evidence flow per
design, see König et al. (2013).  These measures support the
critical evaluation of the network meta-analysis results by
rendering transparent the process of data pooling.
}
\details{
The direct evidence proportion gives the absolute contribution of
direct effect estimates combined for two-arm and multi-arm studies
to one network estimate.

Concerning indirectness, comparisons with a mean path length beyond
two should be interpreted with particular caution, as more than two
direct comparisons have to be combined serially on average.

Large indices of parallelism, either on study-level or on
aggregated level, can be considered as supporting the validity of a
network meta-analysis if there is only a small amount of
heterogeneity.

The network estimates for two treatments are linear combinations of
direct effect estimates comparing these or other treatments. The
linear coefficients can be seen as the generalization of weights
known from classical meta-analysis. These coefficients are given in
the projection matrix \eqn{H} of the underlying model. For
multi-arm studies, the coefficients depend on the choice of the
study-specific baseline treatment, but the absolute flow of
evidence can be made explicit for each design as shown in König et
al. (2013) and is given in \code{H.tilde}.

All measures are calculated based on the common effects
meta-analysis by default. In the case that in function
\code{netmeta} the argument \code{random = TRUE}, all measures
are calculated for a random effects model. The value of the
square-root of the between-study variance \eqn{\tau^2} can also be
prespecified by argument \code{tau.preset} in function
\code{netmeta}.
}
\examples{
data(smokingcessation)

# Transform data from arm-based format to contrast-based format
#
pw <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")

# Conduct network meta-analysis
#
nma <- netmeta(pw)

# Calculate measures based on a common effects model
#        
nm <- netmeasures(nma)

# Plot of minimal parallelism versus mean path length
#
plot(nm$meanpath, nm$minpar, type = "n",
  xlab = "Mean path length", ylab = "Minimal parallelism")
text(nm$meanpath, nm$minpar, names(nm$meanpath), cex = 0.8)

}
\references{
König J, Krahn U, Binder H (2013):
Visualizing the flow of evidence in network meta-analysis and
characterizing mixed treatment comparisons.
\emph{Statistics in Medicine},
\bold{32}, 5414--29
}
\seealso{
\link{netmeta}
}
\author{
Ulrike Krahn \email{ulrike.krahn@bayer.com}, Jochem König
  \email{koenigjo@uni-mainz.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.netmeta.R
\name{forest.netmeta}
\alias{forest.netmeta}
\alias{plot.netmeta}
\title{Forest plot for network meta-analysis}
\usage{
\method{forest}{netmeta}(
  x,
  pooled = ifelse(x$random, "random", "common"),
  reference.group = x$reference.group,
  baseline.reference = x$baseline.reference,
  labels = x$trts,
  equal.size = gs("equal.size"),
  leftcols = "studlab",
  leftlabs,
  rightcols = c("effect", "ci"),
  rightlabs,
  digits = gs("digits.forest"),
  small.values = x$small.values,
  nsim = gs("nsim"),
  digits.prop = 2,
  smlab = NULL,
  sortvar = x$seq,
  overall.hetstat = gs("overall.hetstat"),
  print.tau2 = gs("forest.tau2"),
  print.tau = gs("forest.tau"),
  print.I2 = gs("forest.I2"),
  backtransf = x$backtransf,
  lab.NA = gs("lab.NA"),
  add.data,
  addrows.below.overall = if (x$overall.hetstat) 2 else gs("addrows.below.overall"),
  drop.reference.group = gs("drop.reference.group"),
  col.subgroup = "black",
  print.subgroup.name = FALSE,
  ...
)

\method{plot}{netmeta}(x, ...)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model
(\code{"random"}) should be plotted. Can be abbreviated.}

\item{reference.group}{Reference treatment(s).}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa.}

\item{labels}{An optional vector with treatment labels.}

\item{equal.size}{A logical indicating whether all squares should
be of equal size. Otherwise, the square size is proportional to
the precision of estimates.}

\item{leftcols}{A character vector specifying columns to be plotted
on the left side of the forest plot or a logical value (see
Details).}

\item{leftlabs}{A character vector specifying labels for
(additional) columns on left side of the forest plot (see
Details).}

\item{rightcols}{A character vector specifying columns to be
plotted on the right side of the forest plot or a logical value
(see Details).}

\item{rightlabs}{A character vector specifying labels for
(additional) columns on right side of the forest plot (see
Details).}

\item{digits}{Minimal number of significant digits for treatment
effects and confidence intervals, see \code{print.default}.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect, can be abbreviated; see
\code{\link{netrank}}.}

\item{nsim}{Number of repetitions to calculate SUCRAs.}

\item{digits.prop}{Minimal number of significant digits for
P-scores, SUCRAs and direct evidence proportions, see
\code{\link{print.default}} and \code{\link{netrank}}.}

\item{smlab}{A label printed at top of figure. By default, text
indicating either common or random effects model is printed.}

\item{sortvar}{An optional vector used to sort treatments (must be
of same length as the total number of treatments).}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{print.tau2}{A logical value indicating whether to print the
value of the between-study variance \eqn{\tau^2}.}

\item{print.tau}{A logical value indicating whether to print
\eqn{\tau}, the square root of the between-study variance
\eqn{\tau^2}.}

\item{print.I2}{A logical value indicating whether to print the
value of the I-squared statistic.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{lab.NA}{A character string to label missing values.}

\item{add.data}{An optional data frame with additional columns to
print in forest plot (see Details).}

\item{addrows.below.overall}{A numeric value indicating how many
empty rows are printed between meta-analysis results and
heterogeneity statistics.}

\item{drop.reference.group}{A logical indicating whether the
reference group should be printed in the forest plot.}

\item{col.subgroup}{The colour to print information on subgroups.}

\item{print.subgroup.name}{A logical indicating whether the name of
the grouping variable should be printed in front of the group
labels.}

\item{\dots}{Additional arguments for \code{\link[meta]{forest.meta}}
function.}
}
\description{
Draws a forest plot in the active graphics window (using grid
graphics system).
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window.

Argument \code{sortvar} can be either a numeric or character vector
with length of number of treatments. If \code{sortvar} is numeric
the \code{\link[base]{order}} function is utilised internally to
determine the order of values. If \code{sortvar} is character it
must be a permutation of the treatment names. It is also possible
to provide either \code{sortvar = Pscore}, \code{sortvar =
"Pscore"}, \code{sortvar = -Pscore}, or \code{sortvar = "-Pscore"}
in order to sort treatments according to the ranking generated by
\code{\link{netrank}} which is called internally. It is also
possible to use "SUCRA" instead of "Pscore". Similar expressions
are possible to sort by treatment comparisons (\code{sortvar = TE},
etc.), standard error (\code{sortvar = seTE}), number of studies
with direct treatment comparisons (\code{sortvar = k}), and direct
evidence proportion (\code{sortvar = prop.direct}, see also
\code{\link{netmeasures}}).

The arguments \code{leftcols} and \code{rightcols} can be used to
specify columns which are plotted on the left and right side of the
forest plot, respectively. The following columns are available:
\tabular{ll}{
\bold{Name} \tab \bold{Definition} \cr
\code{"studlab"} \tab Treatments \cr
\code{"TE"} \tab Network estimates (either from common or random
  effects model) \cr
\code{"seTE"} \tab Corresponding standard errors \cr
\code{"Pscore"} \tab P-scores (see \code{\link{netrank}}) \cr
\code{"SUCRA"} \tab SUCRAs (see \code{\link{netrank}}) \cr
\code{"n.trts"} \tab Number of participants per treatment arm \cr
\code{"k"} \tab Number of studies in pairwise comparisons \cr
\code{"prop.direct"} \tab Direct evidence proportions (see
  \code{\link{netmeasures}}) \cr
\code{"effect"} \tab (Back-transformed) network estimates \cr
\code{"ci"} \tab Confidence intervals \cr
\code{"effect.ci"} \tab (Back-transformed) network estimates and
  confidence intervals
}

As a sidenote, the rather odd column name \code{"studlab"} to
describe the treatment comparisons comes from internally calling
\code{\link[meta]{forest.meta}} which uses study labels as the essential
information.

Argument \code{add.data} can be used to add additional columns to
the forest plot. This argument must be a data frame with row names
equal to the treatment names in R object \code{x}, i.e.,
\code{x$trts}.

See help page of \code{\link[meta]{forest.meta}} for more information on
the generation of forest plots and additional arguments.
}
\examples{
# Examples: example(netmeta)

}
\seealso{
\code{\link[meta]{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netmeta.R
\name{netmeta}
\alias{netmeta}
\title{Network meta-analysis using graph-theoretical method}
\usage{
netmeta(
  TE,
  seTE,
  treat1,
  treat2,
  studlab,
  data = NULL,
  subset = NULL,
  correlated,
  sm,
  level = gs("level"),
  level.ma = gs("level.ma"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  prediction = gs("prediction"),
  level.predict = gs("level.predict"),
  reference.group,
  baseline.reference = gs("baseline.reference"),
  small.values = gs("small.values"),
  all.treatments = gs("all.treatments"),
  seq = gs("seq"),
  method.tau = gs("method.tau.netmeta"),
  tau.preset = NULL,
  tol.multiarm = gs("tol.multiarm"),
  tol.multiarm.se = gs("tol.multiarm.se"),
  details.chkmultiarm = gs("details.chkmultiarm"),
  sep.trts = gs("sep.trts"),
  nchar.trts = gs("nchar.trts"),
  nchar.studlab = gs("nchar.studlab"),
  func.inverse = invmat,
  n1 = NULL,
  n2 = NULL,
  event1 = NULL,
  event2 = NULL,
  incr = NULL,
  mean1 = NULL,
  mean2 = NULL,
  sd1 = NULL,
  sd2 = NULL,
  time1 = NULL,
  time2 = NULL,
  overall.hetstat = gs("overall.hetstat"),
  backtransf = gs("backtransf"),
  title = gs("title"),
  keepdata = gs("keepdata"),
  keeprma = gs("keeprma"),
  control = NULL,
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  nchar = nchar.trts,
  ...
)
}
\arguments{
\item{TE}{Estimate of treatment effect, i.e. difference between
first and second treatment (e.g. log odds ratio, mean difference,
or log hazard ratio). Or an R object created with
\code{\link[meta]{pairwise}}.}

\item{seTE}{Standard error of treatment estimate.}

\item{treat1}{Label/Number for first treatment.}

\item{treat2}{Label/Number for second treatment.}

\item{studlab}{An optional - but important! - vector with study
labels (see Details).}

\item{data}{An optional data frame containing the study
information.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{correlated}{An optional logical vector specifying whether
treatment arms of a multi-arm study are correlated (see Details).}

\item{sm}{A character string indicating underlying summary measure,
e.g., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"},
\code{"HR"}, \code{"MD"}, \code{"SMD"}, or \code{"ROM"}.}

\item{level}{The level used to calculate confidence intervals for
individual comparisons.}

\item{level.ma}{The level used to calculate confidence intervals
for network estimates.}

\item{common}{A logical indicating whether a common effects network
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects network
meta-analysis should be conducted.}

\item{prediction}{A logical indicating whether prediction intervals
should be printed.}

\item{level.predict}{The level used to calculate prediction
intervals for a new study.}

\item{reference.group}{Reference treatment (first treatment is used
if argument is missing).}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect (passed on to
\code{\link{netrank}}, can be abbreviated.}

\item{all.treatments}{A logical or \code{"NULL"}. If \code{TRUE},
matrices with all treatment effects, and confidence limits will
be printed.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments in printouts.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau}. Either \code{"DL"}, \code{"REML"}, or
\code{"ML"}, can be abbreviated.}

\item{tau.preset}{An optional value for manually setting the
square-root of the between-study variance \eqn{\tau^2}.}

\item{tol.multiarm}{A numeric for the tolerance for consistency of
treatment estimates in multi-arm studies which are consistent by
design.}

\item{tol.multiarm.se}{A numeric for the tolerance for consistency
of standard errors in multi-arm studies which are consistent by
design. This check is not conducted if the argument is
\code{NULL}.}

\item{details.chkmultiarm}{A logical indicating whether treatment
estimates and / or variances of multi-arm studies with
inconsistent results or negative multi-arm variances should be
printed.}

\item{sep.trts}{A character used in comparison names as separator
between treatment labels.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{nchar.studlab}{A numeric defining the minimum number of
characters used to create unique study labels.}

\item{func.inverse}{R function used to calculate the pseudoinverse
of the Laplacian matrix L (see Details).}

\item{n1}{Number of observations in first treatment group.}

\item{n2}{Number of observations in second treatment group.}

\item{event1}{Number of events in first treatment group.}

\item{event2}{Number of events in second treatment group.}

\item{incr}{Numerical value added to cell frequencies to construct the
covariance matrix used to calculate the REML or ML estimate of the
between-study variance \eqn{\tau^2} (only for binary or count outcomes).}

\item{mean1}{Mean in first treatment group.}

\item{mean2}{Mean in second treatment group.}

\item{sd1}{Standard deviation in first treatment group.}

\item{sd2}{Standard deviation in second treatment group.}

\item{time1}{Person time at risk in first treatment group.}

\item{time2}{Person time at risk in second treatment group.}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf = TRUE}, results for \code{sm = "OR"} are
presented as odds ratios rather than log odds ratios, for
example.}

\item{title}{Title of meta-analysis / systematic review.}

\item{keepdata}{A logical indicating whether original data(set)
should be kept in netmeta object.}

\item{keeprma}{A logical indicating whether \code{\link[metafor]{rma.mv}}
object should be stored.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.mv}}.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if studies are excluded from meta-analysis due to zero
standard errors).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{nchar}{Deprecated argument (replaced by \code{nchar.trts}).}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of class \code{netmeta} with corresponding \code{print},
\code{summary}, \code{forest}, and \code{netrank} functions. The
object is a list containing the following components:
\item{studlab, treat1, treat2, TE, seTE}{As defined above.}
\item{seTE.adj.common, seTE.adj.random}{Standard error of treatment
  estimate, adjusted for multi-arm studies.}
\item{design}{Design of study providing pairwise comparison.}
\item{n1, n2, event1, event2, incr}{As defined above.}
\item{mean1, mean2, sd1, sd2, time1, time2}{As defined above.}
\item{sd1, sd2, time1, time2}{As defined above.}
\item{k}{Total number of studies.}
\item{m}{Total number of pairwise comparisons.}
\item{n}{Total number of treatments.}
\item{d}{Total number of designs (corresponding to the unique set
  of treatments compared within studies).}
\item{trts}{Treatments included in network meta-analysis.}
\item{k.trts}{Number of studies evaluating a treatment.}
\item{n.trts}{Number of observations receiving a treatment (if
  arguments \code{n1} and \code{n2} are provided).}
\item{events.trts}{Number of events observed for a treatment (if
  arguments \code{event1} and \code{event2} are provided).}
\item{multiarm}{Logical vector to identify pairwise comparisons
  from multi-arm studies.}
\item{n.arms}{Number of treatment arms in study providing pairwise
  comparison.}
\item{studies}{Vector with unique study labels.}
\item{narms}{Number of arms for each study.}
\item{designs}{Vector with unique designs present in the network. A
  design corresponds to the set of treatments compared within a
  study.}
\item{comparisons}{Vector with unique direct comparisons present in the
  network.}
\item{TE.nma.common, TE.nma.random}{A vector of length \emph{m} of
  consistent treatment effects estimated by network meta-analysis
  (nma) (common / random effects model).}
\item{seTE.nma.common, seTE.nma.random}{A vector of length \emph{m}
  of effective standard errors estimated by network meta-analysis
  (common / random effects model).}
\item{lower.nma.common, lower.nma.random}{A vector of length
  \emph{m} of lower confidence interval limits for consistent
  treatment effects estimated by network meta-analysis (common
  effects / random effects model).}
\item{upper.nma.common, upper.nma.random}{A vector of length
  \emph{m} of upper confidence interval limits for the consistent
  treatment effects estimated by network meta-analysis (common
  effects / random effects model).}
\item{statistic.nma.common, statistic.nma.random}{A vector of length
  \emph{m} of z-values for test of treatment effect for individual
  comparisons (common / random effects model).}
\item{pval.nma.common, pval.nma.random}{A vector of length \emph{m}
  of p-values for test of treatment effect for individual
  comparisons (common / random effects model).}
\item{leverage.common}{A vector of length \emph{m} of leverages,
  interpretable as factors by which variances are reduced using
  information from the whole network.}
\item{w.common, w.random}{A vector of length \emph{m} of weights of
  individual studies (common / random effects model).}
\item{Q.common}{A vector of length \emph{m} of contributions to
  total heterogeneity / inconsistency statistic.}
\item{TE.common, TE.random}{\emph{n}x\emph{n} matrix with estimated
  overall treatment effects (common / random effects model).}
\item{seTE.common, seTE.random}{\emph{n}x\emph{n} matrix with
  standard errors (common / random effects model).}
\item{lower.common, upper.common, lower.random,
  upper.random}{\emph{n}x\emph{n} matrices with lower and upper
  confidence interval limits (common / random effects
  model).}
\item{statistic.common, pval.common, statistic.random,
  pval.random}{\emph{n}x\emph{n} matrices with z-value and p-value
  for test of overall treatment effect (common / random
  effects model).}
\item{seTE.predict}{\emph{n}x\emph{n} matrix with standard errors
  for prediction intervals.}
\item{lower.predict, upper.predict}{\emph{n}x\emph{n} matrices with
  lower and upper prediction interval limits.}
\item{prop.direct.common, prop.direct.random}{A named vector of the
  direct evidence proportion of each network estimate. (common
  effects / random effects model).}
\item{TE.direct.common, TE.direct.random}{\emph{n}x\emph{n} matrix
  with estimated treatment effects from direct evidence (common
  effects / random effects model).}
\item{seTE.direct.common, seTE.direct.random}{\emph{n}x\emph{n}
  matrix with estimated standard errors from direct evidence (common
  effects / random effects model).}
\item{lower.direct.common, upper.direct.common, lower.direct.random,
  upper.direct.random}{\emph{n}x\emph{n} matrices with lower and upper
  confidence interval limits from direct evidence (common / random
  effects model).}
\item{statistic.direct.common, pval.direct.common, statistic.direct.random,
  pval.direct.random}{
  \emph{n}x\emph{n} matrices with z-value and p-value for test of overall
  treatment effect from direct evidence (common / random effects model).}
\item{TE.indirect.common, TE.indirect.random}{\emph{n}x\emph{n}
  matrix with estimated treatment effects from indirect evidence
  (common / random effects model).}
\item{seTE.indirect.common, seTE.indirect.random}{\emph{n}x\emph{n}
  matrix with estimated standard errors from indirect evidence
  (common / random effects model).}
\item{lower.indirect.common, upper.indirect.common,
  lower.indirect.random, upper.indirect.random}{
  \emph{n}x\emph{n} matrices with lower and upper confidence interval limits
  from indirect evidence (common / random effects model).}
\item{statistic.indirect.common, pval.indirect.common,
  statistic.indirect.random, pval.indirect.random}{
  \emph{n}x\emph{n} matrices with z-value and p-value for test of overall
  treatment effect from indirect evidence (common / random effects model).}
\item{Q}{Overall heterogeneity / inconsistency statistic.}
\item{df.Q}{Degrees of freedom for test of heterogeneity /
  inconsistency.}
\item{pval.Q}{P-value for test of heterogeneity / inconsistency.}
\item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
  limits.}
\item{tau}{Square-root of between-study variance.}
\item{Q.heterogeneity}{Overall heterogeneity statistic.}
\item{df.Q.heterogeneity}{Degrees of freedom for test of overall
  heterogeneity.}
\item{pval.Q.heterogeneity}{P-value for test of overall
  heterogeneity.}
\item{Q.inconsistency}{Overall inconsistency statistic.}
\item{df.Q.inconsistency}{Degrees of freedom for test of overall
  inconsistency.}
\item{pval.Q.inconsistency}{P-value for test of overall
  inconsistency.}
\item{Q.decomp}{Data frame with columns 'treat1', 'treat2', 'Q',
  'df' and 'pval.Q', providing heterogeneity statistics for each
  pairwise meta-analysis of direct comparisons.}
\item{A.matrix}{Adjacency matrix (\emph{n}x\emph{n}).}
\item{X.matrix}{Design matrix (\emph{m}x\emph{n}).}
\item{B.matrix}{Edge-vertex incidence matrix (\emph{m}x\emph{n}).}
\item{L.matrix.common, L.matrix.random}{Laplacian matrix
  (\emph{n}x\emph{n}).}
\item{Lplus.matrix.common, Lplus.matrix.random}{Moore-Penrose
  pseudoinverse of the Laplacian matrix (\emph{n}x\emph{n}).}
\item{Q.matrix}{Matrix of heterogeneity statistics for pairwise
  meta-analyses, where direct comparisons exist
  (\emph{n}x\emph{n}).}
\item{G.matrix}{Matrix with variances and covariances of
  comparisons (\emph{m}x\emph{m}). G is defined as
  \strong{BL+B^t}.}
\item{H.matrix.common, H.matrix.random}{Hat matrix
  (\emph{m}x\emph{m}), defined as \strong{H = GW = BL+B^tW}.}
\item{n.matrix}{\emph{n}x\emph{n} matrix with number of
  observations in direct comparisons (if arguments \code{n1} and
  \code{n2} are provided).}
\item{events.matrix}{\emph{n}x\emph{n} matrix with number of events
  in direct comparisons (if arguments \code{event1} and
  \code{event2} are provided).}
\item{P.common, P.random}{\emph{n}x\emph{n} matrix with direct
  evidence proportions (common / random effects model).}
\item{Cov.common}{Variance-covariance matrix (common effects model)}
\item{Cov.random}{Variance-covariance matrix (random effects model)}
\item{sm, level, level.ma}{As defined above.}
\item{common, random}{As defined above.}
\item{prediction, level.predict}{As defined above.}
\item{reference.group, baseline.reference, small.values,
  all.treatments}{As defined above.}
\item{seq, tau.preset, tol.multiarm, tol.multiarm.se}{As defined
  above.}
\item{details.chkmultiarm, sep.trts, nchar.trts}{As defined above.}
\item{backtransf, title, warn, warn.deprecated}{As defined above.}
\item{call}{Function call.}
\item{version}{Version of R package netmeta used to create object.}

In addition, the following component is stored if \bold{metafor} is
used to calculate the between-study variance and argument
\code{keeprma = TRUE}:
\item{rma.tau}{R object created with \code{\link[metafor]{rma.mv}}.}
}
\description{
Network meta-analysis is a generalisation of pairwise meta-analysis
that compares all pairs of treatments within a number of treatments
for the same condition. The graph-theoretical approach for network
meta-analysis uses methods that were originally developed in
electrical network theory. It has been found to be equivalent to
the frequentist approach to network meta-analysis which is based on
weighted least squares regression (Rücker, 2012). An extension for
multi-arm studies with correlated treatment arms is also implemented
(Rücker & Schwarzer, 2025, unpublished).
}
\details{
Network meta-analysis using R package \bold{netmeta} is described
in detail in Schwarzer et al. (2015), Chapter 8.

\subsection{Network meta-analysis model}{
Let \emph{n} be the number of different treatments (nodes,
vertices) in a network and let \emph{m} be the number of existing
comparisons (edges) between the treatments. If there are only
two-arm studies, \emph{m} is the number of studies. Let TE and seTE
be the vectors of observed effects and their standard errors. Let W
be the \emph{m}x\emph{m} diagonal matrix that contains the inverse
variance 1 / seTE^2.

The given comparisons define the network structure. Therefrom an
\emph{m}x\emph{n} design matrix X (edge-vertex incidence matrix) is
formed; for more precise information, see Rücker (2012). Moreover,
the \emph{n}x\emph{n} Laplacian matrix L and its Moore-Penrose
pseudoinverse L+ are calculated (both matrices play an important
role in graph theory and electrical network theory). Using these
matrices, the variances based on both direct and indirect
comparisons can be estimated. Moreover, the hat matrix H can be
estimated by \strong{H = XL+X^tW = X(X^t W X)^+X^tW} and finally
consistent treatment effects can be estimated by applying the hat
matrix to the observed (potentially inconsistent) effects. H is a
projection matrix which maps the observed effects onto the
consistent (n-1)-dimensional subspace. This is the Aitken estimator
(Senn et al., 2013). As in pairwise meta-analysis, the Q statistic
measures the deviation from consistency. Q can be separated into
parts for each pairwise meta-analysis and a part for remaining
inconsistency between comparisons.

A simple random effects model assuming that a constant
heterogeneity variance is added to each comparison of the network
can be defined via a generalised methods of moments estimate of the
between-studies variance \eqn{\tau^2} (Jackson et al., 2012). This
is added to the observed sampling variance \code{seTE^2} of each
comparison in the network (before appropriate adjustment for
multi-arm studies). Then, as in standard pairwise meta-analysis,
the procedure is repeated with the resulting enlarged standard
errors.

For the random-effects model, the direct treatment estimates are
based on the common between-study variance \eqn{\tau^2} from the
network meta-analysis.
}

\subsection{Multi-arm studies}{
Often multi-arm studies are included in a network meta-analysis.
In multi-arm studies, the treatment effects on different
comparisons are not independent, but correlated. This is accounted
for by reweighting all comparisons of each multi-arm study. The
method is described in Rücker (2012) and Rücker and Schwarzer
(2014).

Comparisons belonging to multi-arm studies are identified by
identical study labels (argument \code{studlab}). It is therefore
important to use identical study labels for all comparisons
belonging to the same multi-arm study, e.g., study label
"Willms1999" for the three-arm study in the data example (Senn et
al., 2013). The function netmeta then automatically accounts for
within-study correlation by reweighting all comparisons of each
multi-arm study.

Note, all pairwise comparisons must be provided for a multi-arm
study. Consider a multi-arm study of \emph{p} treatments with known
variances. For this study, treatment effects and standard errors
must be provided for each of \emph{p}(\emph{p} - 1) / 2 possible
comparisons. For instance, a three-arm study contributes three
pairwise comparisons, a four-arm study even six pairwise
comparisons. Function \code{\link[meta]{pairwise}} automatically
calculates all pairwise comparisons for multi-arm studies.
}

\subsection{Studies with correlated treatment arms}{
The network meta-analysis method described in Rücker (2012) assumes that the
treatment arms of a study are independent. This assumption is justified for
parallel design trials and leads to variance consistency in multi-arm trials.
However, this assumption is violated for trials with correlated arms, for
example, split-body trials. For these trials, the variance of a contrast is
not the sum of the arm-based variances, but comes with a correlation term
which may lead to the violation of variance consistency. Using argument
\code{correlated}, an alternative method is used for studies with correlated
treatment arms (Rücker & Schwarzer, 2025, unpublished).

Argument \code{correlated}, which must be of the same length as arguments
\code{TE}, \code{seTE}, etc., is a logical vector which must be TRUE for
trials with correlated arms and FALSE otherwise. This can be easily done
using a variable with design information like
\code{correlated = design != "Parallel"}.
}

\subsection{Data formats}{
Data entry for this function is in \emph{contrast-based} format,
that is, data are given as contrasts (differences) between two
treatments (argument \code{TE}) with standard error (argument
\code{seTE}). In principle, meta-analysis functions from R package
\bold{meta}, e.g. \code{\link[meta]{metabin}} for binary outcomes or
\code{\link[meta]{metacont}} for continuous outcomes, can be used to
calculate treatment effects separately for each treatment
comparison which is a rather tedious enterprise. If data are
provided in \emph{arm-based} format, that is, data are given for
each treatment arm separately (e.g. number of events and
participants for binary outcomes), a much more convenient way to
transform data into contrast-based form is available. Function
\code{\link[meta]{pairwise}} can automatically transform data with binary
outcomes (using the \code{\link[meta]{metabin}} function from R package
\bold{meta}), continuous outcomes (\code{\link[meta]{metacont}}
function), incidence rates (\code{\link[meta]{metainc}} function), and
generic outcomes (\code{\link[meta]{metagen}} function). Additional
arguments of these functions can be provided (see help page of
function \code{\link[meta]{pairwise}}).
}

\subsection{Additional comments}{
Internally, both common and random effects models are calculated
regardless of values chosen for arguments \code{common} and
\code{random}. Accordingly, the network estimates for the random
effects model can be extracted from component \code{TE.random} of
an object of class \code{"netmeta"} even if argument \code{random =
FALSE}. However, all functions in R package \bold{netmeta} will
adequately consider the values for \code{common} and
\code{random}. E.g. function \code{\link{print.summary.netmeta}}
will not print results for the random effects model if \code{random
= FALSE}.

By default, treatment names are not abbreviated in
printouts. However, in order to get more concise printouts,
argument \code{nchar.trts} can be used to define the minimum number
of characters for abbreviated treatment names (see
\code{\link{abbreviate}}, argument \code{minlength}). R function
\code{\link{treats}} is utilised internally to create abbreviated
treatment names.

Names of treatment comparisons are created by concatenating
treatment labels of pairwise comparisons using \code{sep.trts} as
separator (see \code{\link{paste}}). These comparison names are
used in the covariance matrices \code{Cov.common} and
\code{Cov.random} and in some R functions, e.g,
\code{\link{decomp.design}}. By default, a colon is used as the
separator. If any treatment label contains a colon the following
characters are used as separator (in consecutive order):
\code{"-"}, \code{"_"}, \code{"/"}, \code{"+"}, \code{"."},
\code{"|"}, and \code{"*"}. If all of these characters are used in
treatment labels, a corresponding error message is printed asking
the user to specify a different separator.
}
}
\note{
R function \code{\link[metafor]{rma.mv}} from R package
\pkg{metafor} (Viechtbauer 2010) is called internally to estimate
the between-study variance \eqn{\tau^2} for the (restricted)
maximum likelihood method. For binary outcomes, incidence rates,
and the mean difference, the variance-covariance matrix is
calculated if arguments \code{event1}, \code{event2}, \code{n1},
and \code{n2} (binary outcomes); \code{event1}, \code{event2},
\code{time1}, and \code{time2} (incidence rates); \code{n1},
\code{n2}, \code{sd1}, and \code{sd2} (mean difference) are
provided. For datasets preprocessed with \code{\link[meta]{pairwise}}
the respective variables are selected automatically.
}
\examples{
#
# 1) Smoking cessation example
#

data(smokingcessation)

# Transform data from wide arm-based to contrast-based format
#
pw1 <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")

# Conduct random effects network meta-analysis
#
nma1 <- netmeta(pw1, common = FALSE)
nma1

# Draw network graphs
#
netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.25)
tname <- c("No intervention", "Self-help",
  "Individual counselling", "Group counselling")
netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.25, labels = tname)

# Forest plot
#
forest(nma1)

\donttest{
#
# 2) Diabetes example
#

# Transform data from long arm-based to contrast-based format
#
pw2 <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))

# Conduct common effects network meta-analysis
#
nma2 <- netmeta(pw2, random = FALSE)
nma2

\dontrun{
# Senn (2013) data set in comparison-based format
data(Senn2013)
#
nma2.2 <- netmeta(TE, seTE, treat1, treat2, studlab,
  data = Senn2013, sm = "MD", random = FALSE)
nma2.2
rm(Senn2013)
}

nma2$Q.decomp

# Comparison with reference group
#
print(nma2, reference = "plac")
forest(nma2, reference = "plac")

# Print detailed results
#
snma2 <- summary(nma2)
print(snma2, digits = 3)

# Only show individual study results for multi-arm studies
#
print(snma2, digits = 3, truncate = multiarm)

# Only show first three individual study results
#
print(snma2, digits = 3, truncate = 1:3)

# Only show individual study results for 'Kim 2007' and 'Willms 1999'
#
print(snma2, digits = 3, truncate = c("Kim 2007", "Willms 1999"))

# Only show individual study results for studies starting with the
# letter "W"
#
print(snma2, ref = "plac", digits = 3,
  truncate = substring(studlab, 1, 1) == "W")

# Conduct random effects network meta-analysis
#
nma3 <- netmeta(pw2, common = FALSE, reference = "plac")
nma3
forest(nma3, xlim = c(-1.5, 1), xlab = "HbA1c difference")

# Add column with P-Scores on right side of forest plot
#
forest(nma3, xlim = c(-1.5, 1),
  xlab = "HbA1c difference",
  rightcols = c("effect", "ci", "Pscore"),
  just.addcols = "right")

# Add column with P-Scores on left side of forest plot
#
forest(nma3, xlim = c(-1.5, 1),
  xlab = "HbA1c difference",
  leftcols = c("studlab", "Pscore"),
  just.addcols = "right")

# Sort forest plot by descending P-Score
#
forest(nma3, xlim = c(-1.5, 1),
  xlab = "HbA1c difference",
  rightcols = c("effect", "ci", "Pscore"),
  just.addcols = "right",
  sortvar = -Pscore)

# Sort by and print number of studies with direct treatment comparisons
#
forest(nma3, xlim = c(-1.5, 1),
  xlab = "HbA1c difference",
  leftcols = c("studlab", "k"),
  leftlabs = c("Contrast\nto Placebo", "Direct\nComparisons"),
  sortvar = -k,
  smlab = "Random Effects Model")

# Change printing order of treatments with placebo last and use
# long treatment names
#
trts <- c("acar", "benf", "metf", "migl", "piog",
  "rosi", "sita", "sulf", "vild", "plac")
#
nma4 <- netmeta(pw2, common = FALSE, reference = "plac", seq = trts)
print(nma4, digits = 2)

#
# 3) Dietary fat example
#

data(dietaryfat)

# Transform data from arm-based format to contrast-based format
# Using incidence rate ratios (sm = "IRR") as effect measure.
# Note, the argument 'sm' is not necessary as this is the default
# in R function metainc() called internally
#
pw5 <- pairwise(list(treat1, treat2, treat3),
  list(d1, d2, d3), time = list(years1, years2, years3),
  studlab = ID, data = dietaryfat, sm = "IRR")
pw5

# Conduct network meta-analysis
#
nma5 <- netmeta(pw5)
nma5

# Conduct network meta-analysis using incidence rate differences
# (sm = "IRD")
#
pw6 <- pairwise(list(treat1, treat2, treat3),
  list(d1, d2, d3), time = list(years1, years2, years3),
  studlab = ID, data = dietaryfat, sm = "IRD")
nma6 <- netmeta(pw6)
nma6

# Draw network graph
#
netgraph(nma5, points = TRUE, cex.points = 3, cex = 1.25,
  labels = c("Control","Diet", "Diet 2"))
}

}
\references{
Jackson D, White IR, Riley RD (2012):
Quantifying the impact of between-study heterogeneity in
multivariate meta-analyses.
\emph{Statistics in Medicine},
\bold{31}, 3805--20

Rücker G (2012):
Network meta-analysis, electrical networks and graph theory.
\emph{Research Synthesis Methods},
\bold{3}, 312--24

Rücker G, Schwarzer G (2014):
Reduce dimension or reduce weights? Comparing two approaches to
multi-arm studies in network meta-analysis.
\emph{Statistics in Medicine},
\bold{33}, 4353--69

Schwarzer G, Carpenter JR, Rücker G (2015):
\emph{Meta-Analysis with R (Use R!)}.
Springer International Publishing, Switzerland

Senn S, Gavini F, Magrez D, Scheen A (2013):
Issues in performing a network meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{22}, 169--89

Viechtbauer W (2010):
Conducting Meta-Analyses in R with the metafor Package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48
}
\seealso{
\code{\link[meta]{pairwise}}, \code{\link{forest.netmeta}},
  \code{\link{netrank}}, \code{\link[meta]{metagen}},
  \code{\link{smokingcessation}}, \code{\link{Senn2013}},
  \code{\link{dietaryfat}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netmetabin.R
\name{netmetabin}
\alias{netmetabin}
\title{Network meta-analysis of binary outcome data}
\usage{
netmetabin(
  event1,
  n1,
  event2,
  n2,
  treat1,
  treat2,
  studlab,
  data = NULL,
  subset = NULL,
  sm,
  method = "MH",
  cc.pooled = FALSE,
  incr,
  method.incr,
  allstudies,
  level = gs("level"),
  level.ma = gs("level.ma"),
  common = gs("common"),
  random = method \%in\% c("Inverse", "LRP") & (gs("random") | !is.null(tau.preset)),
  prediction = gs("prediction"),
  level.predict = gs("level.predict"),
  reference.group,
  baseline.reference = gs("baseline.reference"),
  small.values = gs("small.values"),
  all.treatments = gs("all.treatments"),
  seq = gs("seq"),
  tau.preset = NULL,
  tol.multiarm = 0.001,
  tol.multiarm.se = NULL,
  details.chkmultiarm = FALSE,
  details.chkdata = TRUE,
  sep.trts = ":",
  nchar.trts = 666,
  func.inverse = invmat,
  overall.hetstat = gs("overall.hetstat"),
  backtransf = gs("backtransf"),
  title = gs("title"),
  keepdata = gs("keepdata"),
  addincr,
  allincr,
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{event1}{Number of events (first treatment).}

\item{n1}{Number of observations (first treatment).}

\item{event2}{Number of events (second treatment).}

\item{n2}{Number of observations (second treatment)}

\item{treat1}{Label/Number for first treatment.}

\item{treat2}{Label/Number for second treatment.}

\item{studlab}{An optional - but important! - vector with study
labels (see Details).}

\item{data}{An optional data frame containing the study
information.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{sm}{A character string indicating underlying summary measure,
i.e., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"}.}

\item{method}{A character string indicating which method is to be
used for pooling of studies. One of \code{"MH"},
\code{"NCH"}, \code{"LRP"}, or \code{"Inverse"}, can be abbreviated.}

\item{cc.pooled}{A logical indicating whether \code{incr} should be
used as a continuity correction, when calculating the network
meta-analysis estimates (ignored for penalised logistic regression and
inverse variance method).}

\item{incr}{A numerical value which is added to each cell count,
i.e., to the numbers of events and non-events, of all treatment
arms in studies with zero events or non-events in any of the
treatment arms ("continuity correction").}

\item{method.incr}{A character string indicating which continuity
correction method should be used (\code{"only0"},
\code{"if0all"}, or \code{"all"}), see \code{\link[meta]{metabin}}.}

\item{allstudies}{A logical indicating whether studies with zero
events or non-events in all treatment arms should be included in
an inverse variance meta-analysis (applies only if \code{method =
"Inverse"} and \code{sm} is equal to either \code{"RR"} or
\code{"OR"}).}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{level.ma}{The level used to calculate confidence intervals
for network estimates.}

\item{common}{A logical indicating whether a common effects network
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects network
meta-analysis should be conducted.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed (only considered if \code{method =
"Inverse"}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study (only considered if \code{method =
"Inverse"}).}

\item{reference.group}{Reference treatment.}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect (passed on to
\code{\link{netrank}}, can be abbreviated.}

\item{all.treatments}{A logical or \code{"NULL"}. If \code{TRUE},
matrices with all treatment effects, and confidence limits will
be printed.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments in printouts.}

\item{tau.preset}{An optional value for manually setting the
square-root of the between-study variance \eqn{\tau^2} (only
considered if \code{method = "Inverse"}).}

\item{tol.multiarm}{A numeric for the tolerance for consistency of
treatment estimates in multi-arm studies which are consistent by
design (only considered if \code{method = "Inverse"}).}

\item{tol.multiarm.se}{A numeric for the tolerance for consistency
of standard errors in multi-arm studies which are consistent by
design (only considered if the argument is not \code{NULL} and
\code{method = "Inverse"}).}

\item{details.chkmultiarm}{A logical indicating whether treatment
estimates and / or variances of multi-arm studies with
inconsistent results or negative multi-arm variances should be
printed (only considered if \code{method = "Inverse"}).}

\item{details.chkdata}{A logical indicating whether number of
events and participants of studies with inconsistent data should
be printed.}

\item{sep.trts}{A character used in comparison names as separator
between treatment labels.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{func.inverse}{R function used to calculate the pseudoinverse
of the Laplacian matrix L (see \code{\link{netmeta}}).}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf = TRUE}, results for \code{sm = "OR"} are
presented as odds ratios rather than log odds ratios, for
example.}

\item{title}{Title of meta-analysis / systematic review.}

\item{keepdata}{A logical indicating whether original data(set)
should be kept in netmeta object.}

\item{addincr}{Deprecated argument (replaced by 'method.incr');
see \code{\link[meta]{metabin}}.}

\item{allincr}{Deprecated argument (replaced by 'method.incr');
see \code{\link[meta]{metabin}}.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if studies are excluded from meta-analysis due to zero
standard errors).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of class \code{netmetabin} and \code{netmeta} with
corresponding \code{print}, \code{summary}, \code{forest}, and
\code{netrank} functions. The object is a list containing the
following components:
\item{studlab, treat1, treat2}{As defined above.}
\item{n1, n2, event1, event2}{As defined above.}
\item{TE}{Estimate of treatment effect, i.e. difference between
  first and second treatment (e.g. log odds ratio).}
\item{seTE}{Standard error of treatment estimate.}
\item{k}{Total number of studies.}
\item{m}{Total number of pairwise comparisons.}
\item{n}{Total number of treatments.}
\item{d}{Total number of designs (corresponding to the unique set
  of treatments compared within studies).}
\item{trts}{Treatments included in network meta-analysis.}
\item{k.trts}{Number of studies evaluating a treatment.}
\item{n.trts}{Number of observations receiving a treatment.}
\item{events.trts}{Number of events observed for a treatment.}
\item{studies}{Study labels coerced into a factor with its levels
  sorted alphabetically.}
\item{narms}{Number of arms for each study.}
\item{designs}{Unique list of designs present in the network. A
  design corresponds to the set of treatments compared within a
  study.}
\item{TE.common, seTE.common}{\emph{n}x\emph{n} matrix with estimated
  overall treatment effects and standard errors for common effects
  model.}
\item{lower.common, upper.common}{\emph{n}x\emph{n} matrices with
  lower and upper confidence interval limits for common effects
  model.}
\item{statistic.common, pval.common}{\emph{n}x\emph{n} matrices with
  z-value and p-value for test of overall treatment effect under
  common effects model.}
\item{TE.random, seTE.random}{\emph{n}x\emph{n} matrix with
  estimated overall treatment effects and standard errors for
  random effects model (only available if \code{method =
  "Inverse"}).}
\item{lower.random, upper.random}{\emph{n}x\emph{n} matrices with
  lower and upper confidence interval limits for random effects
  model (only available if \code{method = "Inverse"}).}
\item{statistic.random, pval.random}{\emph{n}x\emph{n} matrices
  with z-value and p-value for test of overall treatment effect
  under random effects model (only available if \code{method =
  "Inverse"}).}
\item{TE.direct.common, seTE.direct.common}{\emph{n}x\emph{n} matrix
  with estimated treatment effects and standard errors from direct
  evidence under common effects model.}
\item{lower.direct.common, upper.direct.common}{\emph{n}x\emph{n}
  matrices with lower and upper confidence interval limits from
  direct evidence under common effects model.}
\item{statistic.direct.common, pval.direct.common}{\emph{n}x\emph{n}
  matrices with z-value and p-value for test of overall treatment
  effect from direct evidence under common effects model.}
\item{TE.direct.random, seTE.direct.random}{\emph{n}x\emph{n}
  matrix with estimated treatment effects and standard errors from
  direct evidence under random effects model (only available if
  \code{method = "Inverse"}).}
\item{lower.direct.random, upper.direct.random}{\emph{n}x\emph{n}
  matrices with lower and upper confidence interval limits from
  direct evidence under random effects model (only available if
  \code{method = "Inverse"}).}
\item{statistic.direct.random,
  pval.direct.random}{\emph{n}x\emph{n} matrices with z-value and
  p-value for test of overall treatment effect from direct evidence
  under random effects model (only available if \code{method =
  "Inverse"}).}
\item{Q}{Overall heterogeneity / inconsistency statistic. (only
  available if \code{method = "Inverse"})}
\item{df.Q}{Degrees of freedom for test of heterogeneity /
  inconsistency.}
\item{pval.Q}{P-value for test of heterogeneity / inconsistency.}
\item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
  limits (only available if \code{method = "Inverse"}).}
\item{tau}{Square-root of between-study variance (only available if
  \code{method = "Inverse"}).}
\item{Q.heterogeneity}{Overall heterogeneity statistic. (only
  available if \code{method = "Inverse"})}
\item{df.Q.heterogeneity}{Degrees of freedom for test of overall
  heterogeneity.}
\item{pval.Q.heterogeneity}{P-value for test of overall
  heterogeneity.}
\item{Q.inconsistency}{Overall inconsistency statistic.}
\item{df.Q.inconsistency}{Degrees of freedom for test of overall
  inconsistency.}
\item{pval.Q.inconsistency}{P-value for test of overall
  inconsistency.}
\item{A.matrix}{Adjacency matrix (\emph{n}x\emph{n}).}
\item{H.matrix.common}{Hat matrix (\emph{m}x\emph{m})}
\item{n.matrix}{\emph{n}x\emph{n} matrix with number of
  observations in direct comparisons.}
\item{events.matrix}{\emph{n}x\emph{n} matrix with number of events
  in direct comparisons.}
\item{sm, method, level, level.ma}{As defined above.}
\item{incr, method.incr, allstudies, cc.pooled}{As defined
  above.}
\item{addincr, allincr}{As defined above.}
\item{common, random}{As defined above.}
\item{prediction, level.predict}{As defined above.}
\item{reference.group, baseline.reference, small.values, all.treatments}{As
  defined above.}
\item{seq, tau.preset, tol.multiarm, tol.multiarm.se}{As defined
  above.}
\item{details.chkmultiarm, details.chkdata}{As defined above.}
\item{sep.trts, nchar.trts, overall.hetstat}{As defined above.}
\item{backtransf, title, warn, warn.deprecated}{As defined above.}
\item{data}{Dataset (in contrast-based format).}
\item{data.design}{List with data in arm-based format (each list
  element corresponds to a single design).}
\item{call}{Function call.}
\item{version}{Version of R package netmeta used to create object.}
}
\description{
Provides four models for the network meta-analysis of binary data
(Mantel-Haenszel method, based on the non-central hypergeometric
distribution, penalised logistic regression, and the
inverse variance method).
}
\details{
This function implements four models for the network meta-analysis
of binary data:
\itemize{
\item The Mantel-Haenszel network meta-analysis model, as described
  in Efthimiou et al. (2019) (\code{method = "MH"});
\item a network meta-analysis model using the non-central
  hypergeometric distribution with the Breslow approximation, as
  described in Stijnen et al. (2010) (\code{method = "NCH"});
\item a logistic regression with penalised likelihood, as described
  in Evrenoglou et al. (2022) (\code{method = "LRP"});
\item the inverse variance method for network meta-analysis
  (\code{method = "Inverse"}), also provided by
  \code{\link{netmeta}}.
}

Comparisons belonging to multi-arm studies are identified by
identical study labels (argument \code{studlab}). It is therefore
important to use identical study labels for all comparisons
belonging to the same multi-arm study.

Data entry for this function is in \emph{contrast-based} format,
that is, each line of the data corresponds to a single pairwise
comparison between two treatments (arguments \code{treat1},
\code{treat2}, \code{event1}, \code{n1}, \code{event2}, and
\code{n2}). If data are provided in \emph{arm-based} format, that
is, number of events and participants are given for each treatment
arm separately, function \code{\link[meta]{pairwise}} can be used to
transform the data to \emph{contrast-based} format (see help page
of function \code{\link[meta]{pairwise}}).

Note, all pairwise comparisons must be provided for a multi-arm
study. Consider a multi-arm study of \emph{p} treatments with known
variances. For this study, the number of events and observations
must be provided for each treatment, for each of \emph{p}(\emph{p}
- 1) / 2 possible comparisons in separate lines in the data. For
instance, a three-arm study contributes three pairwise comparisons,
a four-arm study even six pairwise comparisons. Function
\code{\link[meta]{pairwise}} automatically calculates all pairwise
comparisons for multi-arm studies.

For \code{method = "Inverse"}, both common and random effects
models are calculated regardless of values chosen for arguments
\code{common} and \code{random}. Accordingly, the network estimates
for the random effects model can be extracted from component
\code{TE.random} of an object of class \code{"netmeta"} even if
argument \code{random = FALSE}.  However, all functions in R
package \bold{netmeta} will adequately consider the values for
\code{common} and \code{random}. E.g. function
\code{\link{print.summary.netmeta}} will not print results for the
random effects model if \code{random = FALSE}.

For the random-effects model, the direct treatment estimates are
based on the common between-study variance \eqn{\tau^2} from the
network meta-analysis.

For \code{method = "MH"} and \code{method = "NCH"}, only a common
effects model is available.

By default, treatment names are not abbreviated in
printouts. However, in order to get more concise printouts,
argument \code{nchar.trts} can be used to define the minimum number
of characters for abbreviated treatment names (see
\code{\link{abbreviate}}, argument \code{minlength}). R function
\code{\link{treats}} is utilised internally to create abbreviated
treatment names.

Names of treatment comparisons are created by concatenating
treatment labels of pairwise comparisons using \code{sep.trts} as
separator (see \code{\link{paste}}). These comparison names are
used in the covariance matrices \code{Cov.common} and
\code{Cov.random} and in some R functions, e.g,
\code{\link{decomp.design}}. By default, a colon is used as the
separator. If any treatment label contains a colon the following
characters are used as separator (in consecutive order):
\code{"-"}, \code{"_"}, \code{"/"}, \code{"+"}, \code{"."},
\code{"|"}, and \code{"*"}. If all of these characters are used in
treatment labels, a corresponding error message is printed asking
the user to specify a different separator.
}
\examples{
\donttest{
# Only consider first four studies (to reduce runtime of example)
#
first4 <- subset(Dong2013, id <= 4)

# Transform data from long arm-based format to contrast-based
# format. Argument 'sm' has to be used for odds ratio as summary
# measure; by default the risk ratio is used in the metabin
# function called internally.
#
pw1 <- pairwise(treatment, death, randomized, studlab = id,
  data = first4, sm = "OR")

# Conduct Mantel-Haenszel network meta-analysis (without continuity
# correction)
#
nb1 <- netmetabin(pw1, ref = "plac")
nb1

# Obtain the league table
#
netleague(nb1)

# Mantel-Haenszel network meta-analysis for the whole
# dataset: example(Dong2013)
}

}
\references{
Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
(2019):
A Mantel-Haenszel model for network meta-analysis of rare events.
\emph{Statistics in Medicine},
\bold{38}, 2992--3012

Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A (2022):
Network Meta-analysis of rare events using penalized likelihood regression.
\emph{Statistics in Medicine},
\bold{41}, 5203--19.

Senn S, Gavini F, Magrez D, Scheen A (2013):
Issues in performing a network meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{22}, 169--89

Stijnen T, Hamza TH, Ozdemir P (2010):
Random effects meta-analysis of event outcome in the framework of
the generalized linear mixed model with applications in sparse
data.
\emph{Statistics in Medicine},
\bold{29}, 3046--67
}
\seealso{
\code{\link[meta]{pairwise}}, \code{\link{netmeta}},
  \code{\link[metabook]{Dong2013}}
}
\author{
Orestis Efthimiou \email{oremiou@gmail.com}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Theodoros Evrenoglou \email{theodoros.evrenoglou@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.rankogram.R
\name{summary.rankogram}
\alias{summary.rankogram}
\alias{print.summary.rankogram}
\title{Summary method for objects of class rankogram}
\usage{
\method{summary}{rankogram}(object, ...)

\method{print}{summary.rankogram}(
  x,
  common = x$common,
  random = x$random,
  sort = TRUE,
  nchar.trts = x$nchar.trts,
  digits = gs("digits.prop"),
  digits.mean = 2,
  details.methods = gs("details"),
  legend = gs("legend"),
  ...
)
}
\arguments{
\item{object}{An object of class \code{rankogram}.}

\item{\dots}{Additional arguments (passed on to
\code{\link{print.rankogram}}.}

\item{x}{An object of class \code{summary.rankogram}.}

\item{common}{A logical indicating to print ranking probabilities
and SUCRAs for the common effects model.}

\item{random}{A logical indicating to print ranking probabilities
and SUCRAs for the random effects model.}

\item{sort}{A logical indicating whether treatments should be
sorted by decreasing SUCRAs.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{digits}{Minimal number of significant digits for ranking probabilities
and SUCRAs, see \code{\link{print.default}}.}

\item{digits.mean}{Minimal number of significant digits for mean ranks, see
\code{\link{print.default}}.}

\item{details.methods}{A logical specifying whether details on statistical
methods should be printed.}

\item{legend}{A logical indicating whether a legend should be
printed.}
}
\description{
Summary method for objects of class \code{rankogram} to print
list of studies in subnetworks.
}
\examples{
# Examples:
# example(rankogram.netmeta)

}
\seealso{
\code{\link{rankogram}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Dogliotti2014.R
\docType{data}
\name{Dogliotti2014}
\alias{Dogliotti2014}
\title{Network meta-analysis of antithrombotic treatments in patients with
non-valvular atrial fibrillation}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{id}}\tab study ID \cr
\bold{\emph{treatment}}\tab treatment \cr
\bold{\emph{stroke}}\tab number of strokes \cr
\bold{\emph{total}}\tab number of individuals in treatment arm
}
}
\source{
Dogliotti A, Paolasso E, Giugliano RP (2014):
Current and new oral antithrombotics in non-valvular atrial
fibrillation: a network meta-analysis of 79 808 patients.
\emph{Heart},
\bold{100}, 396--405
}
\description{
This dataset comes from a systematic review aiming to determine
the effects of eight antithrombotic treatments in reducing the
incidence of major thrombotic events in patients with non-valvular
atrial fibrillation (Dogliotti et al., 2014). The review included 20
studies (79,808 participants), four of which were three-arm
studies. The primary outcome is stroke reduction.
}
\note{
The dataset Dogliotti2014 is identical to dataset
\code{\link[metabook]{Dogliotti2014}} in R package \bold{metabook}.
}
\examples{
head(Dogliotti2014)

# Example using pairwise(), netmeta() and netmetabin():
# example(Dogliotti2014, package = "metabook")
}
\seealso{
\code{\link[metabook]{Dogliotti2014}},
  \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
  \code{\link{netmetabin}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatplot.netmeta.R
\name{heatplot.netmeta}
\alias{heatplot.netmeta}
\title{Heat Plot}
\usage{
\method{heatplot}{netmeta}(
  x,
  pooled = ifelse(x$random, "random", "common"),
  seq = x$seq,
  nchar.trts = x$nchar.trts,
  low.colour = "red",
  mid.colour = "white",
  high.colour = "springgreen4",
  size = 6,
  size.trt = 16,
  size.axis = 12,
  digits = gs("digits.forest"),
  backtransf = x$backtransf,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model
(\code{"random"}) should be plotted. Can be abbreviated.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments in rows and columns of the heat plot.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{low.colour}{A string indicating the colour of low relative
treatment effects for the heat plot (e.g odds ratio of ~0.5)}

\item{mid.colour}{A string indicating the colour of null relative
treatment effects for the heat plot (e.g odds ratio of ~1.0).}

\item{high.colour}{A string indicating the colour of high relative
treatment effects for the heat plot (e.g odds ratio of ~2.0).}

\item{size}{The size of cell entries with the relative treatment
effect and confidence intervals.}

\item{size.trt}{The size of treatment names placed on the top and
left of the plot.}

\item{size.axis}{The size of labels on the top and left of the plot}

\item{digits}{Minimal number of significant digits for treatment
effects and confidence intervals, see \code{print.default}.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{\dots}{Additional arguments.}
}
\value{
League heat plot, where a color scale is used to represent the
values of relative treatment effects.
}
\description{
Produces a heat plot containing treatment estimates with confidence
intervals for all possible pairwise comparisons.
}
\examples{
# Only consider first five studies (to reduce runtime of example)
#
studies <- unique(Senn2013$study)
Senn2013.5 <- subset(Senn2013, study \%in\% studies[1:5])

# Transform data from long arm-based to contrast-based format
#
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013.5,
  varnames = c("MD", "seMD"))

# Conduct random effects network meta-analysis with
# placebo as reference treatment
#
nma <- netmeta(pw, common = FALSE, reference = "plac")

# Generate a heat plot (with abbreviated treatment labels)
#
heatplot(nma, nchar.trts = 4) 

}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netmatrix.R
\name{netmatrix}
\alias{netmatrix}
\title{Create a matrix with additional information for pairwise
comparisons}
\usage{
netmatrix(
  x,
  var,
  levels,
  labels = levels,
  func = "mode",
  ties.method = "random"
)
}
\arguments{
\item{x}{A \code{\link{netmeta}} object.}

\item{var}{Variable with additional information.}

\item{levels}{An optional vector of the values that \code{var}
might have taken (see \code{\link{factor}}).}

\item{labels}{An optional vector with labels for \code{var} (see
\code{\link{factor}}).}

\item{func}{A character string with the function name to summarize
values within pairwise comparisons; see Details.}

\item{ties.method}{A character string describing how ties are
handled if \code{func = "mode"}; see Details.}
}
\value{
A matrix with the same row and column names as the adjacency matrix
\code{x$A.matrix}.
}
\description{
Auxiliary function to create a matrix with additional information
for pairwise comparisons
}
\details{
For each pairwise comparison, unique values will be calculated for
the variable \code{var} based on the argument \code{func}: "mode"
(most common value), "min" (minimum value), "max", "mean",
"median", and "sum". In order to determine the most common value,
the argument \code{ties.method} can be used in the case of ties
with "first" meaning that the first / smallest value will be
selected; similar for "last" (last / largest value) and "random"
(random selection).
}
\examples{
data(smokingcessation)
# Add variable with (fictitious) risk of bias values
# with 1 = "low risk" and 2 = "high risk"
#
smokingcessation$rob <- rep(1:2, 12)

pw <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")
nma <- netmeta(pw, common = FALSE, ref = "A")

# Generate network graph with information on risk of bias
#
col.rob <- netmatrix(nma, rob, ties.method = "last",
  levels = 1:2, labels = c("green", "yellow"))
#
netgraph(nma, col = col.rob, cex.points = 5,
  bg.points = "gray", adj = 0.5,
  pos.number.of.studies = c(0.5, 0.25, 0.5, 0.5, 0.25, 0.5))

netgraph(nma, col = col.rob, cex.points = n.trts,
  bg.points = "blue", labels = paste0(trts, " (n=", n.trts, ")"),
  pos.number.of.studies = c(0.5, 0.25, 0.5, 0.5, 0.25, 0.5),
  offset = c(0.05, 0.035, 0.05, 0.025))

}
\seealso{
\code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netgraph.R
\name{netgraph}
\alias{netgraph}
\title{Generic function for network graphs}
\usage{
netgraph(x, ...)
}
\arguments{
\item{x}{An R object.}

\item{\dots}{Additional arguments.}
}
\description{
Generic function for network graphs
}
\details{
For more details, look at the following functions to generate
network graphs:
\itemize{
\item \code{\link{netgraph.netmeta}}
\item \code{\link{netgraph.netimpact}}
\item \code{\link{netgraph.netconnection}}
\item \code{\link{netgraph.netcomb}}
\item \code{\link{netgraph.discomb}}
}
}
\examples{
# Examples: example(netgraph.netmeta)

}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de
}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/invmat.R
\name{invmat}
\alias{invmat}
\title{Moore-Penrose Pseudoinverse of a Matrix}
\usage{
invmat(X)
}
\arguments{
\item{X}{A square matrix.}
}
\value{
The Moore-Penrose pseudoinverse for matrix \strong{X}.
}
\description{
Calculates the Moore-Penrose pseudoinverse of a square matrix
\strong{X}.
}
\details{
This function is used by default in R package \strong{netmeta} to
calculate the Moore-Penrose pseudoinverse \strong{L\eqn{^+}} of the
Laplacian matrix \strong{L} (Rücker, 2012):

\strong{L\eqn{^+} = (X - J / \emph{n})\eqn{^{-1}} + J / \emph{n}}
with identity matrix \strong{J} of dimension \emph{n}x\emph{n}.
}
\examples{
\donttest{
data(smokingcessation)

pw <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")
net <- netmeta(pw)

L1 <- net$L.matrix.common
L2 <- invmat(net$Lplus.matrix.common)
all.equal(round(L1, 10), round(L2, 10))
}

}
\references{
Rücker G (2012):
Network meta-analysis, electrical networks and graph theory.
\emph{Research Synthesis Methods},
\bold{3}, 312--24
}
\seealso{
\code{\link{netmeta}}, \code{\link{solve}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido Schwarzer
  \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netpairwise.R
\name{netpairwise}
\alias{netpairwise}
\alias{netpairwise.netmeta}
\alias{netpairwise.netmetabin}
\alias{print.netpairwise}
\alias{summary.netpairwise}
\alias{print.summary.netpairwise}
\alias{forest.netpairwise}
\alias{plot.netpairwise}
\alias{funnel.netpairwise}
\alias{radial.netpairwise}
\alias{baujat.netpairwise}
\alias{metabias.netpairwise}
\alias{print.metabias.netpairwise}
\alias{trimfill.netpairwise}
\alias{print.trimfill.netpairwise}
\alias{metainf.netpairwise}
\alias{print.metainf.netpairwise}
\alias{metacum.netpairwise}
\alias{print.metacum.netpairwise}
\alias{metareg.netpairwise}
\alias{print.metareg.netpairwise}
\title{Conduct pairwise meta-analyses for all comparisons with direct
evidence in a network meta-analysis}
\usage{
netpairwise(x, ...)

\method{netpairwise}{netmeta}(
  x,
  separate = FALSE,
  common = x$common,
  random = x$random,
  level = x$level,
  level.ma = x$level.ma,
  prediction = x$prediction,
  level.predict = x$level.predict,
  reference.group = if (missing(order)) x$reference.group else "",
  baseline.reference = x$baseline.reference,
  method.tau = x$method.tau,
  order = NULL,
  sep.trts = x$sep.trts,
  nchar.trts = x$nchar.trts,
  backtransf = x$backtransf,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{netpairwise}{netmetabin}(
  x,
  separate = FALSE,
  common = x$common,
  random = x$random,
  level = x$level,
  level.ma = x$level.ma,
  prediction = x$prediction,
  level.predict = x$level.predict,
  reference.group = if (missing(order)) x$reference.group else "",
  baseline.reference = x$baseline.reference,
  method = x$method,
  incr = x$incr,
  method.incr = x$method.incr,
  allstudies = x$allstudies,
  method.tau = x$method.tau,
  order = NULL,
  sep.trts = x$sep.trts,
  nchar.trts = x$nchar.trts,
  backtransf = x$backtransf,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{print}{netpairwise}(x, ...)

\method{summary}{netpairwise}(object, ...)

\method{print}{summary.netpairwise}(x, ...)

\method{forest}{netpairwise}(x, ...)

\method{plot}{netpairwise}(x, ...)

\method{funnel}{netpairwise}(x, k.min = 3, ...)

\method{radial}{netpairwise}(x, k.min = 3, ...)

\method{baujat}{netpairwise}(x, k.min = 3, ...)

\method{metabias}{netpairwise}(x, k.min = 10, ...)

\method{print}{metabias.netpairwise}(x, ...)

\method{trimfill}{netpairwise}(x, k.min = 3, ...)

\method{print}{trimfill.netpairwise}(x, ...)

\method{metainf}{netpairwise}(x, k.min = 2, ...)

\method{print}{metainf.netpairwise}(x, ...)

\method{metacum}{netpairwise}(x, k.min = 2, ...)

\method{print}{metacum.netpairwise}(x, ...)

\method{metareg}{netpairwise}(x, ..., k.min = 2)

\method{print}{metareg.netpairwise}(x, ...)
}
\arguments{
\item{x}{An object of class \code{netmeta} or \code{netpairwise}.}

\item{\dots}{Additional arguments (passed on to \code{metagen} or
print functions and to catch deprecated arguments).}

\item{separate}{A logical indicating whether results for pairwise
comparisons should be printed as separate meta-analyses or as
subgroups which is more concise.}

\item{common}{A logical indicating whether a common effects network
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects network
meta-analysis should be conducted.}

\item{level}{The level used to calculate confidence intervals for
individual comparisons.}

\item{level.ma}{The level used to calculate confidence intervals
for pooled estimates.}

\item{prediction}{A logical indicating whether prediction intervals
should be printed.}

\item{level.predict}{The level used to calculate prediction
intervals for a new study.}

\item{reference.group}{Reference treatment.}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau}. Either \code{"DL"}, \code{"REML"}, or
\code{"ML"}, can be abbreviated.}

\item{order}{An optional character or numerical vector specifying
the order of treatments.}

\item{sep.trts}{A character used in comparison names as separator
between treatment labels.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots. If
\code{backtransf = TRUE}, results for \code{sm = "OR"} are
presented as odds ratios rather than log odds ratios, for
example.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{method}{A character string indicating which method is to be
used for pooling of studies, see \code{\link[meta]{metabin}}.}

\item{incr}{A numerical value which is added to cell counts,
see \code{\link[meta]{metabin}}.}

\item{method.incr}{A character string indicating which continuity
correction method should be used (\code{"only0"},
\code{"if0all"}, or \code{"all"}), see \code{\link[meta]{metabin}}.}

\item{allstudies}{A logical indicating whether studies with zero
events or non-events in all treatment arms should be included in
the meta-analysis, see \code{\link[meta]{metabin}}.}

\item{object}{An object of class \code{netpairwise}.}

\item{k.min}{Minimum number of studies in pairwise comparison to
show funnel plot, radial plot or conduct test for funnel plot
asymmetry.}
}
\value{
Either a single \code{\link[meta]{metagen}} object with pairwise
comparisons as subgroups or a list with \code{\link[meta]{metagen}}
objects for each direct pairwise comparison.
}
\description{
Conduct pairwise meta-analyses for all comparisons with direct
evidence in a network meta-analysis.
}
\details{
Conduct pairwise meta-analyses for all comparisons with direct
evidence in a network meta-analysis. In contrast to
\code{\link{netmeta}} and \code{\link{netsplit}}, unadjusted
standard errors are used in the calculations and the between-study
heterogeneity variance is allowed to differ between comparisons.

The R function \code{\link[meta]{metagen}} is called internally.
}
\note{
This function must not be confused with \code{\link[meta]{pairwise}}
which can be used as a pre-processing step to convert data from
arm-based to contrast-based format by calculating all pairwise
comparisons within a study.
}
\examples{
oldsets <- settings.meta(digits = 2, digits.tau2 = 2, digits.tau = 2)

data(smokingcessation)

# Transform data from arm-based format to contrast-based format
#
pw1 <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")

# Conduct random effects network meta-analysis
#
nma1 <- netmeta(pw1, common = FALSE)

# Calculate and print concise results for all pairwise
# meta-analyses
#
np1 <- netpairwise(nma1)
np1
print(np1, details.method = FALSE)

\donttest{
# Random effects model
#
pw2 <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))
nma2 <- netmeta(pw2, common = FALSE, reference = "plac")

# Calculate and print concise results for all pairwise
# meta-analyses
#
np2 <- netpairwise(nma2)
np2
print(np2, details.method = FALSE)

forest(np2)

# Print detailed information for each pairwise comparison
#
np3 <- netpairwise(nma2, separate = TRUE)
forest(np3)
funnel(np3)
radial(np3)
funnel(np3, k.min = 1)
}

settings.meta(oldsets)

}
\seealso{
\code{\link{netmeta}}, \code{\link{netsplit}},
  \code{\link[meta]{pairwise}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treats.R
\name{treats}
\alias{treats}
\alias{comps}
\title{Abbreviate treatment names}
\usage{
treats(x, nchar.trts = 8, row = TRUE)

comps(x, trts, sep.trts, nchar.trts = 8, row = TRUE)
}
\arguments{
\item{x}{A vector with treatment or comparison names or a matrix
with treatment or comparison names as row and / or column names.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{row}{A logical indicating whether row or column names should
be used (only considered if argument \code{x} is a matrix).}

\item{trts}{A character vector with treatment names.}

\item{sep.trts}{A character used in comparison names as separator
between treatment labels.}
}
\description{
Auxiliary functions to create uniquely abbreviated treatment names.
}
\details{
These auxiliary functions can be used to create uniquely
abbreviated treatment names (and are used internally in several R
functions for this purpose).

In order to construct uniquely abbreviated treatment names,
\code{treats} uses \code{\link{substring}} to extract the first
\code{nchar.trts} characters. If these abbreviated treatment names
are not unique, \code{\link{abbreviate}} with argument
\code{minlength = nchar.trts} is used.

In order to construct comparisons with uniquely abbreviated
treatment names, \code{comps} calls \code{treats} internally.
}
\examples{
\donttest{
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))
nma <- netmeta(pw)

# Full treatment names
#
nma$trts

# Treatment names with four characters
#
treats(nma$trts, nchar.trts = 4)

# With two characters
#
treats(nma$trts, nchar.trts = 2)

# With one character (if possible)
#
treats(nma$trts, nchar.trts = 1)

# Full comparison names
#
nma$comparisons

# Abbreviated comparison names
#
with(nma, comps(comparisons, trts, sep.trts, nchar = 4))
}

}
\seealso{
\code{\link{netmeta}}, \code{\link{print.netmeta}},
  \code{\link{print.summary.netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.netbind.R
\name{print.netbind}
\alias{print.netbind}
\title{Print method for objects of class netbind}
\usage{
\method{print}{netbind}(
  x,
  common = x$x$common,
  random = x$x$random,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netbind} or
\code{summary.netbind}.}

\item{common}{A logical indicating whether results for the common
effects model should be printed.}

\item{random}{A logical indicating whether results for the random
effects model should be printed.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\description{
Print method for objects of class \code{netbind}.
}
\examples{
# Examples: example(netbind)

}
\seealso{
\code{\link{netbind}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smokingcessation.R
\docType{data}
\name{smokingcessation}
\alias{smokingcessation}
\title{Network meta-analysis of interventions for smoking cessation}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{event1}}\tab number of individuals with successful
  smoking cessation in arm 1 \cr
\bold{\emph{n1}}\tab number of individuals in arm 1 \cr
\bold{\emph{event2}}\tab number of individuals with successful
  smoking cessation in arm 2 \cr
\bold{\emph{n2}}\tab number of individuals in arm 2 \cr
\bold{\emph{event3}}\tab number of individuals with successful
  smoking cessation in arm 3 \cr
\bold{\emph{n3}}\tab number of individuals in arm 3 \cr
\bold{\emph{treat1}}\tab treatment 1 \cr
\bold{\emph{treat2}}\tab treatment 2 \cr \bold{\emph{treat3}}\tab
  treatment 3
}
}
\source{
Dias S, Welton NJ, Sutton AJ, Caldwell DM, Lu G and Ades AE (2013):
Evidence Synthesis for Decision Making 4: Inconsistency in networks
of evidence based on randomized controlled trials.
\emph{Medical Decision Making},
\bold{33}, 641--56
}
\description{
Network meta-analysis comparing the effects of a number of
interventions for smoking cessation.

These data are used as an example in Dias et al. (2013), page 651.
}
\examples{
data(smokingcessation)
head(smokingcessation)

# Examples: example(netmeta)
}
\seealso{
\code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
  \code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netleague.R
\name{netleague}
\alias{netleague}
\alias{print.netleague}
\title{Create league table with network meta-analysis results}
\usage{
netleague(
  x,
  y,
  common = x$common,
  random = x$random,
  seq = x$seq,
  ci = TRUE,
  backtransf = x$backtransf,
  direct,
  nchar.trts = x$nchar.trts,
  digits = gs("digits"),
  big.mark = gs("big.mark"),
  text.NA = ".",
  bracket = gs("CIbracket"),
  separator = gs("CIseparator"),
  lower.blank = gs("CIlower.blank"),
  upper.blank = gs("CIupper.blank"),
  writexl = !missing(path),
  path = "leaguetable.xlsx",
  overwrite = FALSE,
  details = gs("details"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{print}{netleague}(
  x,
  common = x$x$common,
  random = x$x$random,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of created with \code{netmeta}, \code{netcomb},
\code{discomb}, or \code{netleague} (mandatory).}

\item{y}{An object of class \code{netmeta}, \code{netcomb}, or \code{discomb}
(optional).}

\item{common}{A logical indicating whether a league table should be
printed for the common effects network meta-analysis.}

\item{random}{A logical indicating whether a league table should be
printed for the random effects network meta-analysis.}

\item{seq}{A character or numerical vector specifying the sequence
of treatments in rows and columns of a league table.}

\item{ci}{A logical indicating whether confidence intervals should
be shown.}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. If \code{backtransf = TRUE}, results
for \code{sm = "OR"} are printed as odds ratios rather than log
odds ratios, for example.}

\item{direct}{A logical indicating whether league tables with
network estimates or estimates from direct comparisons
should be shown if (i) argument \code{y} is provided and the input for
arguments \code{x} and \code{y} was created with \code{netmeta} or (ii)
input for argument \code{x} was created with \code{netmeta} or
\code{netcomb}.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{big.mark}{A character used as thousands separator.}

\item{text.NA}{A character string to label missing values.}

\item{bracket}{A character with bracket symbol to print lower
confidence interval: "[", "(", "\{", "".}

\item{separator}{A character string with information on separator
between lower and upper confidence interval.}

\item{lower.blank}{A logical indicating whether blanks between left
bracket and lower confidence limit should be printed.}

\item{upper.blank}{A logical indicating whether blanks between
separator and upper confidence limit should be printed.}

\item{writexl}{A logical indicating whether an Excel file should be
created (R package \bold{writexl} must be available).}

\item{path}{A character string specifying the filename of the Excel
file.}

\item{overwrite}{A logical indicating whether an existing Excel
file should be overwritten.}

\item{details}{A logical specifying whether details on rows and columns
should be printed.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (passed on to \code{write_xlsx}
to create Excel file).}
}
\value{
An object of class \code{netleague} with corresponding \code{print}
function if \code{writexl = FALSE}. The object is a list containing
the league tables in list elements 'common' and 'random'. An Excel
file is created if \code{writexl = TRUE}. In this case, \code{NULL}
is returned in R.
}
\description{
A league table is a square matrix showing all pairwise comparisons
in a network meta-analysis. Typically, both treatment estimates and
confidence intervals are shown.
}
\details{
A league table is a square matrix showing all pairwise comparisons
in a network meta-analysis (Hutton et al., 2015). Typically, both
treatment estimates and confidence intervals are shown.

If argument \code{y} is not provided, the league table contains
\itemize{
\item the network estimates in the lower triangle and the direct treatment
  estimates from pairwise comparisons in the upper triangle, if the input
  for argument \code{x} was created with \code{netmeta};
\item the network estimates from the component network meta-analysis in the
  lower triangle, if the input for argument \code{x} was created with
  \code{netcomb}, and, in the upper triangle, (i) the network estimates from
  the network meta-analysis, if argument \code{direct = FALSE}, or
  (ii)) the direct treatment estimates from pairwise comparisons, if
  argument \code{direct = TRUE};
\item the network estimates from the component network meta-analysis in the
  lower and upper triangle, if the input for argument \code{x} was created
  with \code{discomb}.
}
Note, for the random-effects model, the direct treatment estimates are based
on the common between-study variance \eqn{\tau^2} from the network
meta-analysis, i.e. the square of list element \code{x$tau}.

If argument \code{y} is provided, the league table contains
information on treatment comparisons from (component) network meta-analysis
object \code{x} in the lower triangle and from (component) network
meta-analysis object \code{y} in the upper triangle. This is, for
example, useful to print information on efficacy and safety in the
same league table. If argument \code{direct = TRUE}, direct estimates are
shown both in the lower and upper triangle if the input for arguments
\code{x} and \code{y} was created with \code{netmeta}.

By default, an R object with the league tables is generated. Alternatively,
an Excel file is created if argument \code{writexl = TRUE}.

This implementation reports pairwise comparisons of the treatment
in the column versus the treatment in the row in the lower triangle
and row versus column in the upper triangle. This is a common
presentation for network meta-analyses which allows to easily
compare direction and magnitude of treatment effects. For example,
given treatments A, B, and C, the results reported in the first row
and second column as well as second row and first column are from
the pairwise comparison A versus B. Note, this presentation is
different from the printout of a network meta-analysis object which
reports opposite pairwise comparisons in the lower and upper
triangle, e.g., A versus B in the first row and second column and B
versus A in the second row and first column.

If the same (component) network meta-analysis object is used for arguments
\code{x} and \code{y}, reciprocal treatment estimates will be shown
in the upper triangle (see examples), e.g., the comparison B versus
A.

R function \code{\link{netrank}} can be used to change the order of
rows and columns in the league table (see examples).
}
\examples{
# Network meta-analysis of count mortality statistics
#
pw0 <- pairwise(treatment, event = r, n = N,
  studlab = author, data = Woods2010, sm = "OR",
  reference.group = "Placebo")
nma0 <- netmeta(pw0)

oldopts <- options(width = 100)

# League table for common and random effects model with
# - network estimates in lower triangle
# - direct estimates in upper triangle
#
netleague(nma0, digits = 2, bracket = "(", separator = " - ")

\donttest{
# League table for common effects model
#
netleague(nma0, random = FALSE, digits = 2)

# Change order of treatments according to treatment ranking (random
# effects model)
#
netleague(nma0, common = FALSE, digits = 2, seq = netrank(nma0))
#
print(netrank(nma0), common = FALSE)

# Create a CSV file with league table for random effects model
#
league0 <- netleague(nma0, digits = 2, bracket = "(", separator = " to ")
#
write.table(league0$random, file = "league0-random.csv",
  row.names = FALSE, col.names = FALSE, sep = ",")
#
# Create Excel files with league tables
# (if R package writexl is available)
#
netleague(nma0, digits = 2, bracket = "(", separator = " to ",
          path = tempfile(fileext = ".xlsx"))

# Define order of treatments in depression dataset Linde2015
#
trts <- c("TCA", "SSRI", "SNRI", "NRI",
  "Low-dose SARI", "NaSSa", "rMAO-A", "Hypericum", "Placebo")

# Outcome labels
#
outcomes <- c("Early response", "Early remission")

# (1) Early response
#
pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma1 <- netmeta(pw1, common = FALSE,
  seq = trts, ref = "Placebo", small = "undesirable")

# (2) Early remission
#
pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")
#
nma2 <- netmeta(pw2, common = FALSE,
  seq = trts, ref = "Placebo", small = "undesirable")

options(width = 200)
netleague(nma1, digits = 2)

netleague(nma1, digits = 2, ci = FALSE)
netleague(nma2, digits = 2, ci = FALSE)

# League table for two outcomes with
# - network estimates of first outcome in lower triangle
# - network estimates of second outcome in upper triangle
#
netleague(nma1, nma2, digits = 2, ci = FALSE)

netleague(nma1, nma2, seq = netrank(nma1), ci = FALSE)
netleague(nma1, nma2, seq = netrank(nma2), ci = FALSE)

netrank(nma1)
netrank(nma2)

# Report results for network meta-analysis twice
#
netleague(nma1, nma1, seq = netrank(nma1), ci = FALSE,
  backtransf = FALSE)
netleague(nma1, nma1, seq = netrank(nma1), ci = FALSE,
  backtransf = FALSE, direct = TRUE)
}

options(oldopts)

}
\references{
Hutton B, Salanti G, Caldwell DM, et al. (2015):
The PRISMA Extension Statement for Reporting of Systematic Reviews
Incorporating Network Meta-analyses of Health Care Interventions:
Checklist and Explanations.
\emph{Annals of Internal Medicine},
\bold{162}, 777
}
\seealso{
\code{\link{netmeta}}, \code{\link{netcomb}},
  \code{\link{discomb}}, \code{\link{netposet}}, \code{\link{netrank}},
  \code{\link[metabook]{Woods2010}},
  \code{\link[metabook]{Linde2015}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}, Gerta
  Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomp.design.R
\name{decomp.design}
\alias{decomp.design}
\title{Design-based decomposition of Cochran's Q in network meta-analysis}
\usage{
decomp.design(
  x,
  tau.preset = x$tau.preset,
  warn = gs("warn"),
  nchar.trts = x$nchar.trts
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{tau.preset}{An optional value for the square-root of the
between-study variance \eqn{\tau^2} (see Details).}

\item{warn}{A logical indicating whether warnings should be
printed.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}
}
\value{
Network meta-analysis with a single design: \code{NULL}. Otherwise,
a list containing the following components:
\item{Q.decomp}{Data frame with Q statistics (variable \code{Q})
  based on the common effects model to assess the
  homogeneity/consistency in the whole network, within designs, and
  between designs. Corresponding degrees of freedom (\code{df}) and
  p-values (\code{p.val}) are also given.}
\item{Q.het.design}{Data frame with design-specific decomposition
  of the within-designs Q statistic (\code{Q}) of the common effects
  model, corresponding degrees of freedom (\code{df}) and p-values
  (\code{p.val}) are given.}
\item{Q.inc.detach}{Data frame with between-designs Q statistics
  (\code{Q}) of the common effects model after detaching of single
  designs, corresponding degrees of freedom (\code{df}) and
  p-values (\code{p.val}) are given.}
\item{Q.inc.design}{A named vector with contributions of single
  designs to the between design Q statistic given in
  \code{Q.decomp}.}
\item{Q.inc.random}{Data frame with between-designs Q statistic
  (\code{Q}) based on a random effects model with square-root of
  between-study variance \code{tau.within} estimated embedded in a
  full design-by-treatment interaction model, corresponding degrees
  of freedom (\code{df}) and p-value (\code{p.val}).}
\item{Q.inc.random.preset}{Data frame with between-designs Q
  statistic (\code{Q}) based on a random effects model with
  prespecified square-root of between-study variance
  \code{tau.preset} in the case if argument \code{tau.preset} is
  not NULL, corresponding degrees of freedom (\code{df}) and
  p-value (\code{p.val}).}
\item{Q.inc.design.random.preset}{A named vector with contributions
  of single designs to the between design Q statistic based on a
  random effects model with prespecified square-root of
  between-study variance \code{tau.preset} in the case if argument
  \code{tau.preset} is given.}
\item{residuals.inc.detach}{Matrix with residuals,
  i.e. design-specific direct estimates minus the corresponding
  network estimates after detaching the design of the column.}
\item{residuals.inc.detach.random.preset}{Matrix with residuals
  analogous to \code{residuals.inc.detach} but based on a random
  effects model with prespecified square-root of between-study
  variance \code{tau.preset} in the case if argument
  \code{tau.preset} is not NULL.}
\item{call}{Function call.}
\item{version}{Version of R package netmeta used to create object.}
}
\description{
This function performs a design-based decomposition of Cochran's Q
for assessing the homogeneity in the whole network, the homogeneity
within designs, and the homogeneity/consistency between designs. It
allows also an assessment of the consistency assumption after
detaching the effect of single designs.
}
\details{
In the context of network meta-analysis and the assessment of the
homogeneity and consistency assumption, a generalised Cochran's Q
statistic for multivariate meta-analysis can be used as shown in
Krahn et al. (2013). This Q statistic can be decomposed in a sum
of within-design Q statistics and one between-designs Q statistic
that incorporates the concept of design inconsistency, see Higgins
et al. (2012).

For assessing the inconsistency in a random effects model, the
between-designs Q statistic can be calculated based on a full
design-by-treatment interaction random effects model (see Higgins
et al., 2012). This Q statistic will be automatically given in the
output (\eqn{\tau^2} estimated by the method of moments (see Jackson
et al., 2012). Alternatively, the square-root of the between-study
variance can be prespecified by argument \code{tau.preset} to
obtain a between-designs Q statistic (in \code{Q.inc.random}), its
design-specific contributions \code{Q.inc.design.random.preset}) as
well as residuals after detaching of single designs
(\code{residuals.inc.detach.random.preset}).

Since an inconsistent treatment effect of one design can
simultaneously inflate several residuals, Krahn et al. (2013)
suggest for locating the inconsistency in a network to fit a set of
extended models allowing for example for a deviating effect of each
study design in turn. The recalculated between-designs Q statistics
are given in list component \code{Q.inc.detach}. The change of the
inconsistency contribution of single designs can be investigated in
more detail by a net heat plot (see function
\link{netheat}). Designs where only one treatment is involved in
other designs of the network or where the removal of corresponding
studies would lead to a splitting of the network do not contribute
to the inconsistency assessment. These designs are not included in
\code{Q.inc.detach}.
}
\examples{
\donttest{
# Only consider first five studies (to reduce runtime of example)
#
studies <- unique(Senn2013$study)
Senn2013.5 <- subset(Senn2013, study \%in\% studies[1:5])

# Transform data from long arm-based to contrast-based format
#
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013.5,
  varnames = c("MD", "seMD"))

# Conduct network meta-analysis
#
nma <- netmeta(pw)

# Decomposition of Cochran's Q
#
decomp.design(nma)
}

}
\references{
Higgins JPT, Jackson D, Barrett JK, Lu G, Ades AE, White IR (2012):
Consistency and inconsistency in network meta-analysis: concepts
and models for multi-arm studies.
\emph{Research Synthesis Methods},
\bold{3}, 98--110

Krahn U, Binder H, König J (2013):
A graphical tool for locating inconsistency in network meta-analyses.
\emph{BMC Medical Research Methodology},
\bold{13}, 35

Jackson D, White IR and Riley RD (2012):
Quantifying the impact of between-study heterogeneity in
multivariate meta-analyses.
\emph{Statistics in Medicine},
\bold{31}, 3805--20
}
\seealso{
\link{netmeta}, \link{netheat}
}
\author{
Ulrike Krahn \email{ulrike.krahn@bayer.com}, Jochem König
  \email{koenigjo@uni-mainz.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netmetareg.R
\name{netmetareg.netmeta}
\alias{netmetareg.netmeta}
\alias{netmetareg}
\alias{netmetareg.default}
\alias{print.netmetareg}
\title{Network meta-regression with a single continuous or binary covariate}
\usage{
\method{netmetareg}{netmeta}(
  x,
  covar = NULL,
  consistency = TRUE,
  assumption = "independent",
  method.tau = if (!x$random) "FE" else "REML",
  level = x$level.ma,
  reference.group = x$reference.group,
  direction1 = NULL,
  direction2 = NULL,
  nchar.trts = x$nchar.trts,
  ...
)

netmetareg(x, ...)

\method{netmetareg}{default}(x, ...)

\method{print}{netmetareg}(
  x,
  digits = gs("digits"),
  digits.se = gs("digits.se"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  print.se = FALSE,
  details.methods = TRUE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{covar}{Continuous or binary covariate.}

\item{consistency}{A logical indicating whether a consistency or
unrelated mean interaction effect (UMIE) model should be assumed.}

\item{assumption}{A character string indicating which assumption is done
for the covariate; either "independent" or "common" (can be abbreviated).}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance tau-squared. Either
\code{"FE"}, \code{"REML"}, or \code{"ML"}.}

\item{level}{The level used to calculate confidence intervals for regression
coefficients.}

\item{reference.group}{Reference treatment.}

\item{direction1}{Directionality parameter for the first treatment group
when using UMIE models.}

\item{direction2}{Directionality parameter for the second treatment group
when using UMIE models.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{\dots}{Additional arguments passed to \code{\link[metafor]{rma.mv}}.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
errors.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-value, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall treatment effect, see \code{print.default}.}

\item{print.se}{A logical specifying whether standard errors should be
printed.}

\item{details.methods}{A logical specifying whether details on statistical
methods should be printed.}
}
\value{
An object of class \code{c("netmetareg", "rma.mv", "rma")}. Please
look at the help page of R function \code{\link[metafor]{rma.mv}}
for more details on the output from this function.

In addition, a list \code{.netmeta} is added to the output containing
the following components:
\item{x, covar, method.tau}{As defined above.}
\item{dots}{Information provided in argument '\dots{}'.}
\item{call}{Function call.}
\item{version}{Version of R package \bold{netmeta} used to create
  object.}
\item{version.metafor}{Version of R package \bold{metafor} used to
  create object.}
}
\description{
Network meta-regression with a single continuous or binary covariate for
objects of class \code{netmeta} (Kwarteng et al., 2026). This is a wrapper
function for \code{\link[metafor]{rma.mv}} in the R package \bold{metafor}
(Viechtbauer 2010).
}
\details{
This R function is a wrapper function for \code{\link[metafor]{rma.mv}} in
the R package \bold{metafor} (Viechtbauer 2010).

Note, results are not back-transformed in printouts of network meta-analyses
using summary measures with transformations, e.g., log risk ratios are
printed instead of the risk ratio if argument \code{sm = "RR"}.

Argument '\dots{}' can be used to pass additional arguments to R
function \code{\link[metafor]{rma.mv}}. For example, argument
\code{control} to provide a list of control values for the
iterative estimation algorithm. See help page of R function
\code{\link[metafor]{rma.mv}} for more details.
}
\examples{
#
# 1) Smoking cessation example
#
data(smokingcessation)
# Add variable with (fictitious) risk of bias values
# with 1 = "low risk" and 2 = "high risk"
#
smokingcessation$rob <- rep(1:2, 12)

pw1 <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")

# Run network meta-analysis (NMA)
nma1 <- netmeta(pw1, common = FALSE, ref = "A")

# Network meta-regression with continuous covariate and assumption of
# independent slopes
nr1 <- netmetareg(nma1, rob)
nr1

\donttest{
# 2) Pain Prevention of Propofol Injection Example (Jalota2011)
#
# Create pairwise object
pw2 <- pairwise(treat = trt, event =  pain, n = n,
  studlab = id, data = Jalota2011, sm = "OR")

# Run network meta-analysis (NMA)
nma2 <- netmeta(pw2, common = FALSE, ref = "Hand vein")

# NMR with independent and consistent assumption (default)
nmr2_i <- netmetareg(nma2, covar = seTE, consistency = TRUE,
  assumption = "i")
# NMR with common and consistent assumption
nmr2_c <- netmetareg(nma2, covar = seTE, consistency = TRUE,
  assumption = "c")
# NMR with independent UMIE model (no consistency in interactions) using
# default treatment order
nmr2_i_umie <- netmetareg(nma2, covar = seTE, consistency = FALSE,
  assumption = "i")

# 3) Physical therapy example (Hong 2015)
#
dat3 <- Hong2015
# Externally create a customizable treatment order indicator variable,
# which will later be modified
dat3$trtnum <- as.numeric(as.factor(dat3$trt))
dat3 <- do.call(rbind, lapply(split(dat3, dat3$id),
  function(x) {
    x$direction <- ifelse(x$trtnum == min(x$trtnum), 0, 1)
  x}))

pw3 <- pairwise(treat = trt, mean = mean_pain, n = n, sd = sd_pain,
  studlab=id, data = dat3, sm = "MD")
# Create difference variable for covariate
pw3$disability_diff <- pw3$mean_disability1 - pw3$mean_disability2

# Update directionality parameters so that they will exclude imputed values
sel.NA <- is.na(pw3$disability_diff)
pw3$direction1[sel.NA] <- 0
pw3$direction2[sel.NA] <- 0

# Mean imputations for missing covariate values
pw3$disability_diff[sel.NA] <- mean(pw3$disability_diff, na.rm = TRUE)

# NMA
nma3 <- netmeta(pw3, common = FALSE, ref = "No treatment")

# NMR with independent and consistent assumption (default)
nmr3_i <- netmetareg(nma3, covar = disability_diff, consistency = TRUE,
  assumption = "i")
# NMR with common and consistent assumption
nmr3_c <- netmetareg(nma3, covar = disability_diff, consistency = TRUE,
  assumption = "c")
# NMR with independent UMIE model (no consistency in interactions) using
# default treatment order variable
nmr3_i_umie <- netmetareg(nma3, covar = disability_diff, consistency = FALSE,
  assumption = "i")
# NMR with independent UMIE model (no consistency in interactions) using
# custom treatment order variable which excludes the mean imputations from
# interaction estimation
nmr3_i_umie_noimp <- netmetareg(nma3, covar = disability_diff,
  consistency = FALSE, assumption = "i",
   direction1 = "direction1", direction2 = "direction2")
# NMR with common UMIE model (no consistency in interactions) using default
# treatment order variable
nmr3_c_umie <- netmetareg(nma3, covar = disability_diff, consistency = FALSE,
  assumption = "c")
# NMR with common UMIE model (no consistency in interactions) using custom
# treatment order variable which excludes the mean imputations from
# interaction estimation
nmr3_c_umie_noimp <- netmetareg(nma3, covar = disability_diff,
  consistency = FALSE, assumption = "c",
  direction1 = "direction1", direction2 = "direction2")
}

}
\references{
Kwarteng N, Evrenoglou T, Mueller J, Elsaesser M, Schramm E, Schwarzer G,
Nikolakopoulou A (2026):
Illustrating the assumptions of meta-regression in treatment networks.
Preprint available at \emph{Research Square},
\doi{10.21203/rs.3.rs-8235913/v1}

Viechtbauer W (2010):
Conducting Meta-Analyses in R with the Metafor Package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48
}
\seealso{
\code{\link{netmeta}}
}
\author{
Nana-adjoa Kwarteng
  \email{nana-adjoa.kwarteng@uniklinik-freiburg.de},
  Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dietaryfat.R
\docType{data}
\name{dietaryfat}
\alias{dietaryfat}
\title{Network meta-analysis of dietary fat}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{treat1}}\tab treatment 1 \cr
\bold{\emph{treat2}}\tab treatment 2 \cr
\bold{\emph{treat3}}\tab treatment 3 \cr
\bold{\emph{years1}}\tab person years arm 1 \cr
\bold{\emph{years2}}\tab person years arm 2 \cr
\bold{\emph{years3}}\tab person years arm 3 \cr
\bold{\emph{d1}}\tab events (deaths) arm 1 \cr
\bold{\emph{d2}}\tab events (deaths) arm 2 \cr
\bold{\emph{d3}}\tab events (deaths) arm 3 \cr
\bold{\emph{ID}}\tab study ID
}
}
\source{
Dias S, Sutton AJ, Ades AE and Welton NJ (2013):
Evidence synthesis for decision making 2: A generalized linear
modeling framework for pairwise and network meta-analysis of
randomized controlled trials.
\emph{Medical Decision Making},
\bold{33}, 607--17
}
\description{
Network meta-analysis comparing the effects of two diets to control
on mortality.

The data are rates, given as the number of deaths and person-years.
These data are used as an example in the supplemental material of
Dias et al. (2013).
}
\examples{
data(dietaryfat)
head(dietaryfat)

# Examples: example(netmeta)
}
\seealso{
\code{\link[meta]{pairwise}}, \code{\link[meta]{metainc}},
\code{\link{netmeta}}, \code{\link{netgraph.netmeta}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netimpact.R
\name{netimpact}
\alias{netimpact}
\title{Determine the importance of individual studies in network
meta-analysis}
\usage{
netimpact(
  x,
  seTE.ignore = 1000 * max(x$seTE, na.rm = TRUE),
  event.ignore = 0.01,
  nchar.trts = x$nchar.trts,
  nchar.studlab = x$nchar.studlab,
  verbose = FALSE
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{seTE.ignore}{Assumed (large) standard error in order to
mimicking the removal of individual studies from the network
meta-analysis (ignored for \code{\link{netmetabin}} objects).}

\item{event.ignore}{Assumed event number mimicking the removal of
individual studies from the network meta-analysis (considered for
\code{\link{netmetabin}} objects).}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{nchar.studlab}{A numeric defining the minimum number of
characters used to create unique study labels.}

\item{verbose}{A logical indicating whether information on the
estimation progress should be printed.}
}
\value{
An object of class \code{"netimpact"} with corresponding
\code{netgraph} and \code{print} function. The object is a list
containing the following components:
\item{impact.common}{A matrix with contributions of individual
  studies (columns) to comparisons (rows) under the common effects
  model.}
\item{impact.random}{A matrix with contributions of individual
  studies (columns) to comparisons (rows) under the random effects
  model.}
\item{ignored.comparisons}{List with comparisons of ignored study.}
\item{seTE.ignore, event.ignore, x}{As defined above.}
\item{nets}{List of all network meta-analyses (removing a single
  study).}
\item{version}{Version of R package netmeta used to create object.}
}
\description{
This function measures the importance of individual studies in
network meta-analysis by the reduction of the precision if the
study is removed / ignored from the network (Rücker et al., 2020).
}
\examples{
# Only consider first two studies (to reduce runtime of example)
#
studies <- unique(Franchini2012$Study)
pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
  n = list(n1, n2, n3),
  mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
  data = subset(Franchini2012, Study \%in\% studies[1:2]),
  studlab = Study)

nma1 <- netmeta(pw1)
ni1 <- netimpact(nma1, verbose = TRUE)
ni1

netgraph(ni1)

\donttest{
netgraph(ni1, plastic = TRUE)

pw2 <- pairwise(list(Treatment1, Treatment2, Treatment3),
  n = list(n1, n2, n3),
  mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
  data = Franchini2012,
  studlab = Study)

nma2 <- netmeta(pw2)
ni2 <- netimpact(nma2, verbose = TRUE)
netgraph(ni2)
}

}
\references{
Rücker G, Nikolakopoulou A, Papakonstantinou T, Salanti G, Riley
RD, Schwarzer G (2020):
The statistical importance of a study for a network meta-analysis
estimate.
\emph{BMC Medical Research Methodology},
\bold{20}, 190
}
\seealso{
\code{\link{netmeta}}, \code{\link{netmetabin}},
  \code{\link{netgraph.netimpact}}, \code{\link{print.netimpact}},
  \code{\link[metabook]{Franchini2012}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netheat.R
\name{netheat}
\alias{netheat}
\title{Net heat plot}
\usage{
netheat(
  x,
  random = FALSE,
  tau.preset = NULL,
  showall = TRUE,
  nchar.trts = x$nchar.trts,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{random}{A logical indicating whether the net heat plot should
be based on a random effects model.}

\item{tau.preset}{An optional value for the square-root of the
between-study variance \eqn{\tau^2} for a random effects model on
which the net heat plot will be based.}

\item{showall}{A logical indicating whether results should be shown
for all designs or only a sensible subset (see Details).}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{\dots}{Additional arguments.}
}
\description{
This function creates a net heat plot, a graphical tool for
locating inconsistency in network meta-analyses.
}
\details{
The net heat plot is a matrix visualization proposed by Krahn et
al. (2013) that highlights hot spots of inconsistency between
specific direct evidence in the whole network and renders
transparent possible drivers.

In this plot, the area of a gray square displays the contribution
of the direct estimate of one design in the column to a network
estimate in a row. In combination, the colors show the detailed
change in inconsistency when relaxing the assumption of consistency
for the effects of single designs. The colors on the diagonal
represent the inconsistency contribution of the corresponding
design. The colors on the off-diagonal are associated with the
change in inconsistency between direct and indirect evidence in a
network estimate in the row after relaxing the consistency
assumption for the effect of one design in the column. Cool colors
indicate an increase and warm colors a decrease: the stronger the
intensity of the color, the greater the difference between the
inconsistency before and after the detachment. So, a blue colored
element indicates that the evidence of the design in the column
supports the evidence in the row. A clustering procedure is applied
to the heat matrix in order to find warm colored hot spots of
inconsistency. In the case that the colors of a column
corresponding to design \eqn{d} are identical to the colors on the
diagonal, the detaching of the effect of design \eqn{d} dissolves
the total inconsistency in the network.

The pairwise contrasts corresponding to designs of three- or
multi-arm studies are marked by '_' following the treatments of the
design.

Designs where only one treatment is involved in other designs of
the network or where the removal of corresponding studies would
lead to a splitting of the network do not contribute to the
inconsistency assessment. By default (\code{showall = TRUE}), these
designs are not incorporated into the net heat plot. If
\code{showall = FALSE}, additional designs with minimal
contribution to the inconsistency Q statistic are not incorporated
(i.e., designs with \code{abs(Q.inc.design)} \code{<=}
\code{.Machine$double.eps^0.5)}.).

In the case of \code{random = TRUE}, the net heat plot is based on
a random effects model generalised for multivariate meta-analysis
in which the between-study variance \eqn{\tau^2} is estimated by the
method of moments (see Jackson et al., 2012) and embedded in a full
design-by-treatment interaction model (see Higgins et al., 2012).
}
\examples{
# Only consider first five studies (to reduce runtime of example)
#
studies <- unique(Senn2013$study)
Senn2013.5 <- subset(Senn2013, study \%in\% studies[1:5])

# Transform data from long arm-based to contrast-based format
#
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013.5,
  varnames = c("MD", "seMD"))

# Conduct common effects network meta-analysis with
# placebo as reference treatment
#
nma1 <- netmeta(pw, random = FALSE, reference = "plac")

# Generate a net heat plot based on the common effects model
#
netheat(nma1) 

\donttest{
# Conduct common and random effects network meta-analysis with placebo as
# reference treatment
#
nma2 <- netmeta(pw, reference = "plac")
        
# Generate a net heat plot based on the random effects model
#
netheat(nma2) 
}

}
\references{
Krahn U, Binder H, König J (2013):
A graphical tool for locating inconsistency in network meta-analyses.
\emph{BMC Medical Research Methodology},
\bold{13}, 35

Jackson D, White IR and Riley RD (2012):
Quantifying the impact of between-study heterogeneity in
multivariate meta-analyses.
\emph{Statistics in Medicine},
\bold{31}, 3805--20

Higgins JPT, Jackson D, Barrett JK, Lu G, Ades AE, White IR (2012):
Consistency and inconsistency in network meta-analysis: concepts
and models for multi-arm studies.
\emph{Research Synthesis Methods},
\bold{3}, 98--110
}
\seealso{
\link{netmeta}
}
\author{
Ulrike Krahn \email{ulrike.krahn@bayer.com}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netconnection.R
\name{netconnection.default}
\alias{netconnection.default}
\alias{netconnection}
\alias{netconnection.pairwise}
\alias{print.netconnection}
\alias{netconnection.netmeta}
\alias{netconnection.netcomb}
\title{Get information on network connectivity (number of subnetworks,
distance matrix)}
\usage{
\method{netconnection}{default}(
  data = NULL,
  treat1,
  treat2,
  studlab = NULL,
  subset = NULL,
  sep.trts = ":",
  nchar.trts = 666,
  title = "",
  details.disconnected = FALSE,
  warn = FALSE,
  ...
)

\method{netconnection}{pairwise}(
  data,
  treat1,
  treat2,
  studlab = NULL,
  subset = NULL,
  sep.trts = ":",
  drop.NA = TRUE,
  nchar.trts = 666,
  title = "",
  details.disconnected = FALSE,
  warn = FALSE,
  ...
)

\method{netconnection}{netmeta}(
  data,
  sep.trts = data$sep.trts,
  nchar.trts = data$nchar.trts,
  title = data$title,
  details.disconnected = FALSE,
  warn = FALSE,
  ...
)

\method{netconnection}{netcomb}(
  data,
  sep.trts = data$sep.trts,
  nchar.trts,
  title = data$title,
  details.disconnected = FALSE,
  warn = FALSE,
  ...
)

\method{print}{netconnection}(
  x,
  digits = max(4, .Options$digits - 3),
  nchar.trts = x$nchar.trts,
  details = FALSE,
  details.disconnected = x$details.disconnected,
  ...
)

netconnection(data, ...)
}
\arguments{
\item{data}{A data frame, e.g., created with
\code{\link[meta]{pairwise}}.}

\item{treat1}{Label / number for first treatment (ignored if
\code{data} was created with \code{\link[meta]{pairwise}}).}

\item{treat2}{Label / number for second treatment (ignored if
\code{data} was created with \code{\link[meta]{pairwise}}).}

\item{studlab}{Study labels (ignored if \code{data} was created
with \code{\link[meta]{pairwise}}).}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{sep.trts}{A character used in comparison names as separator
between treatment labels.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{title}{Title of meta-analysis / systematic review.}

\item{details.disconnected}{A logical indicating whether to print
more details for disconnected networks.}

\item{warn}{A logical indicating whether warnings should be
printed.}

\item{\dots}{Additional arguments (ignored at the moment)}

\item{drop.NA}{A logical indicating whether comparisons with missing
treatment estimates or standard errors should be considered.}

\item{x}{An object of class \code{netconnection}.}

\item{digits}{Minimal number of significant digits, see
\code{\link{print.default}}.}

\item{details}{A logical indicating whether to print the distance
matrix.}
}
\value{
An object of class \code{netconnection} with corresponding
\code{print} function. The object is a list containing the
following components:
\item{treat1, treat2, studlab, title, warn, nchar.trts}{As defined
  above.}
\item{k}{Total number of studies.}
\item{m}{Total number of pairwise comparisons.}
\item{n}{Total number of treatments.}
\item{n.subnets}{Number of subnetworks; equal to 1 for a fully
  connected network.}
\item{D.matrix}{Distance matrix.}
\item{A.matrix}{Adjacency matrix.}
\item{L.matrix}{Laplace matrix.}
\item{call}{Function call.}
\item{version}{Version of R package netmeta used to create object.}
}
\description{
To determine the network structure and to test whether a given
network is fully connected. Network information is provided as a
triple of vectors \code{treat1}, \code{treat2}, and \code{studlab}
where each row corresponds to an existing pairwise treatment
comparison (\code{treat1}, \code{treat2}) in a study
(\code{studlab}). The function calculates the number of subnetworks
(connectivity components; value of 1 corresponds to a fully
connected network) and the distance matrix (in block-diagonal form
in the case of subnetworks). If some treatments are combinations of
other treatments or have common components, an analysis based on
the additive network meta-analysis model might be possible, see
\link{discomb} function.
}
\examples{
\donttest{
# Transform data from long arm-based to contrast-based format
#
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))

nc1 <- netconnection(pw)
nc1

# Extract number of (sub)networks
#
nc1$n.subnets

# Extract distance matrix
#
nc1$D.matrix

# Artificial example with two subnetworks
#
t1 <- c("G", "B", "B", "D", "A", "F")
t2 <- c("B", "C", "E", "E", "H", "A")
#
nc2 <- netconnection(t1, t2)
print(nc2, details = TRUE)

# Number of subnetworks
#
nc2$n.subnets

# Extract distance matrix
#
nc2$D.matrix

# Conduct network meta-analysis (results in an error message due to
# unconnected network)
try(nma <- netmeta(1:6, 1:6, t1, t2, 1:6))

# Conduct network meta-analysis on first subnetwork
#
nma.1 <- netmeta(1:6, 1:6, t1, t2, 1:6, subset = nc2$subnet == 1)

# Conduct network meta-analysis on second subnetwork
#
nma.2 <- netmeta(1:6, 1:6, t1, t2, 1:6, subset = nc2$subnet == 2)

nma.1
nma.2
}

}
\seealso{
\code{\link{netmeta}}, \code{\link{netdistance}},
  \code{\link{discomb}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createC.R
\name{createC}
\alias{createC}
\alias{createC.matrix}
\alias{createC.netcomb}
\alias{createC.netmeta}
\alias{createC.netconnection}
\alias{createC.default}
\alias{combinations}
\title{Auxiliary functions for component network meta-analysis}
\usage{
createC(x, ...)

\method{createC}{matrix}(
  x,
  comb.ia,
  inactive = NULL,
  sep.comps = gs("sep.comps"),
  sep.ia = gs("sep.ia"),
  ...
)

\method{createC}{netcomb}(x, comb.ia = NULL, inactive = NULL, sep.ia = x$sep.ia, ...)

\method{createC}{netmeta}(x, inactive = NULL, sep.comps = gs("sep.comps"), ...)

\method{createC}{netconnection}(x, inactive = NULL, sep.comps = gs("sep.comps"), ...)

\method{createC}{default}(x, n = 1, ...)

combinations(x, n = NULL)
}
\arguments{
\item{x}{A \code{\link{netcomb}}, \code{\link{netmeta}} or
\code{\link{netconnection}} object, a matrix or the number of components.}

\item{\dots}{Additional arguments.}

\item{comb.ia}{A character vector specifying treatment combinations which
will be considered as interactions.}

\item{inactive}{A character string defining the inactive treatment
component (see \code{\link{netcomb}}).}

\item{sep.comps}{A single character to define separator between
treatment components.}

\item{sep.ia}{A single character to define separator for interactions.}

\item{n}{A single number specifying the number of components in combinations.}
}
\value{
R function \code{createC} returns a combination matrix / C matrix with
studies in rows and component and interaction terms in columns.

R function \code{combinations} returns a character vector with combinations.
}
\description{
Auxiliary functions to (i) create a combination / C matrix with information
on treatment combinations and interaction terms and (ii) a vector with all
combinations in a component network meta-analysis.
}
\examples{
# Examples: example(netcomb)

}
\seealso{
\code{\link{netcomb}}, \code{\link{discomb}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.netmetareg.R
\name{summary.netmetareg}
\alias{summary.netmetareg}
\alias{print.summary.netmetareg}
\title{Summary method for objects of class netmetareg}
\usage{
\method{summary}{netmetareg}(object, ...)

\method{print}{summary.netmetareg}(
  x,
  digits = gs("digits"),
  digits.se = gs("digits.se"),
  big.mark = gs("big.mark"),
  ...
)
}
\arguments{
\item{object}{An object of class \code{netmetareg}.}

\item{\dots}{Additional arguments (ignored).}

\item{x}{An object of class \code{summary.netmetareg}.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
errors.}

\item{big.mark}{A character used as thousands separator.}
}
\description{
Summary method for objects of class \code{netmetareg} to print
list of studies in subnetworks.
}
\examples{
\dontrun{
data(smokingcessation)
# Add variable with (fictitious) risk of bias values
# with 1 = "low risk" and 2 = "high risk"
#
smokingcessation$rob <- rep(1:2, 12)

pw <- pairwise(list(treat1, treat2, treat3),
  event = list(event1, event2, event3), n = list(n1, n2, n3),
  data = smokingcessation, sm = "OR")

nma <- netmeta(pw, common = FALSE, ref = "A")

# Network meta-regression with continuous covariate and assumption of
# independent slopes
nr <- netmetareg(nma, rob)
nr

summary(nr1)
}

}
\seealso{
\code{\link{netmetareg}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metabias.netmeta.R
\name{metabias.netmeta}
\alias{metabias.netmeta}
\title{Test of funnel plot asymmetry in network meta-analysis}
\usage{
\method{metabias}{netmeta}(
  x,
  order,
  pooled = ifelse(x$random, "random", "common"),
  method.bias = "Egger",
  lump.comparator = gs("lump.comparator"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta}.}

\item{order}{A mandatory character or numerical vector specifying
the order of treatments or list of comparators (see Details).}

\item{pooled}{A character string indicating whether results for the
common (\code{"common"}) or random effects model (\code{"random"})
should be used in test of funnel plot asymmetry. Can be
abbreviated.}

\item{method.bias}{A character vector indicating which test(s) for
funnel plot asymmatrx to use. Admissible values are
\code{"Begg"}, \code{"Egger"}, and \code{"Thompson"}, can be
abbreviated. See function \code{\link[meta]{metabias.meta}}.}

\item{lump.comparator}{A logical indicating whether comparators
should be lumped, e.g., to specify inactive treatments.
information on direct comparisons should be added to the plot.}

\item{\dots}{Additional arguments (passed on to
\code{\link[meta]{metabias.meta}}).}
}
\value{
A list with class \code{metabias} containing the following
components if a test for funnel plot asymmetry is conducted:
\item{statistic}{Test statistic.}
\item{df}{The degrees of freedom of the test statistic in
  the case that it follows a t distribution.}
\item{pval}{The p-value for the test.}
\item{estimate}{Estimates used to calculate test statisic.}
\item{method}{A character string indicating what type of test was
  used.}
\item{title}{Title of Cochrane review.}
\item{complab}{Comparison label.}
\item{outclab}{Outcome label.}
\item{var.model}{A character string indicating whether none,
  multiplicative, or additive residual heterogeneity variance was
  assumed.}
\item{method.bias}{As defined above.}
\item{x}{Network meta-analysis object.}
\item{version}{Version of R package \bold{meta} used to create
  object.}
\item{version.netmeta}{Version of R package \bold{netmeta} used to
  create object.}

Or a list with the following elements if test is not conducted due
to the number of studies:
\item{k}{Number of comparisons.}
\item{k.min}{Minimum number of comparisons to perform test for
  funnel plot asymmetry.}
\item{version}{Version of R package \bold{meta} used to create
  object.}
\item{version.netmeta}{Version of R package \bold{netmeta} used to
  create object.}
}
\description{
Test of funnel plot asymmetry in network meta-analysis
}
\details{
Test of funnel plot asymmetry in network meta-analysis

Argument \code{order} is mandatory to determine the order of
treatments (Chaimani et al., 2013):

\emph{\dQuote{[...] investigators should order
the treatments in a meaningful way and make assumptions about how
small studies differ from large ones. For example, if they
anticipate that newer treatments are favored in small trials, then
they could name the treatments from oldest to newest so that all
comparisons refer to \sQuote{old versus new intervention}. Other
possibilities include defining the comparisons so that all refer to
an active treatment versus placebo or sponsored versus
non-sponsored intervention.}}

Alternatively, it is possible to only provide a single or few
treatment name(s) in argument \code{order} to define the
comparator(s). In this case only comparisons with this / these
treatment(s) will be considered. If argument \code{lump.comparator}
is \code{TRUE}, all comparators will be lumped into a single
group.
}
\examples{
\donttest{
# Transform data from long arm-based to contrast-based format
#
pw <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"))

# Conduct network meta-analysis
#
nma <- netmeta(pw)

# Test for asymmetry in 'comparison-adjusted' funnel plot not
# conducted as argument 'order' is missing
#
try(metabias(nma))

# Test for funnel plot asymmetry comparing active treatments with
# placebo
metabias(nma, order = "pl")

# Rank test
#
metabias(nma, order = "pl", method.bias = "Begg")


# Test for funnel plot asymmetry based on (non-sensical) alphabetic
# order of treatments with placebo as last treatment
#
ord <- c("a", "b", "me", "mi", "pi", "r", "si", "su", "v", "pl")
metabias(nma, order = ord)
}

}
\references{
Chaimani A & Salanti G (2012):
Using network meta-analysis to evaluate the existence of
small-study effects in a network of interventions.
\emph{Research Synthesis Methods},
\bold{3}, 161--76

Chaimani A, Higgins JP, Mavridis D, Spyridonos P, Salanti G (2013):
Graphical tools for network meta-analysis in STATA.
PLOS ONE,
\bold{8}, e76654
}
\seealso{
\code{\link{netmeta}}, \code{\link{funnel.netmeta}},
  \code{\link[meta]{metabias}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.pairwise.R
\name{merge.pairwise}
\alias{merge.pairwise}
\title{Merge pairwise object with additional data}
\usage{
\method{merge}{pairwise}(x, y, all.x = TRUE, ...)
}
\arguments{
\item{x}{An object of class \code{pairwise}.}

\item{y}{A data frame or an object of class \code{netconnection}.}

\item{all.x}{A logical indicating whether to keep all observations from the
pairwise object, i.e., also include observations not belonging to a
subnetwork due to missing estimates or standard errors.}

\item{\dots}{Other arguments (passed on to \code{merge}).}
}
\value{
An object of class \code{pairwise}.
}
\description{
Merge pairwise object with additional data, e.g., information on
network connectivity.
}
\examples{
# Transform data from long arm-based format to contrast-based
# format Argument 'sm' has to be used for odds ratio as summary
# measure; by default the risk ratio is used in the metabin
# function called internally.
#
pw <- pairwise(treatment, event = r, n = N,
  studlab = author, data = Woods2010, sm = "OR",
  append = FALSE)
head(pw)

# Add information on network connectivity
nc <- netconnection(pw)
pw.nc <- merge(pw, nc)
head(pw.nc)

}
\seealso{
\code{\link[meta]{pairwise}}, \code{\link{netconnection}},
  \code{\link[metabook]{Woods2010}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gurusamy2011.R
\docType{data}
\name{Gurusamy2011}
\alias{Gurusamy2011}
\title{Network meta-analysis on blood loss during liver transplantation}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study information (first author, year) \cr
\bold{\emph{treatment}}\tab treatment \cr
\bold{\emph{death}}\tab mortality at 60 days post-transplantation
  \cr
\bold{\emph{n}}\tab number of individuals in treatment arm
}
}
\source{
Gurusamy KS, Pissanou T, Pikhart H, Vaughan J, Burroughs AK,
Davidson BR (2011):
Methods to decrease blood loss and transfusion requirements for
liver transplantation.
\emph{Cochrane Database of Systematic Reviews},
CD009052
}
\description{
Network meta-analysis comparing the effects of a number of
interventions for decreasing blood loss and blood transfusion
requirements during liver transplantation.
}
\note{
The dataset Gurusamy2011 is identical to dataset
\code{\link[metabook]{Gurusamy2011}} in R package \bold{metabook}.
}
\examples{
head(Gurusamy2011)

# Example using pairwise() and netmetabin():
# example(Gurusamy2011, package = "metabook")
}
\seealso{
\code{\link[metabook]{Gurusamy2011}},
  \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
  \code{\link{netmetabin}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.netmeta.R
\name{summary.netmeta}
\alias{summary.netmeta}
\title{Summary method for objects of class netmeta}
\usage{
\method{summary}{netmeta}(
  object,
  common = object$common,
  random = object$random,
  prediction = object$prediction,
  reference.group = object$reference.group,
  baseline.reference = object$baseline.reference,
  all.treatments = object$all.treatments,
  overall.hetstat = object$overall.hetstat,
  backtransf = object$backtransf,
  nchar.trts = object$nchar.trts,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{object}{An object of class \code{netmeta}.}

\item{common}{A logical indicating whether results for the common
effects model should be printed.}

\item{random}{A logical indicating whether results for the random
effects model should be printed.}

\item{prediction}{A logical indicating whether prediction intervals
should be printed.}

\item{reference.group}{Reference treatment.}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment (default) or vice versa. This argument is
only considered if \code{reference.group} has been specified.}

\item{all.treatments}{A logical or \code{"NULL"}. If \code{TRUE},
matrices with all treatment effects, and confidence limits will
be printed.}

\item{overall.hetstat}{A logical indicating whether to print heterogeneity
measures.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and forest plots.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names (see Details).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
A list of class "summary.netmeta" is returned with the following elements:

\item{k}{Total number of studies.}
\item{m}{Total number of pairwise comparisons.}
\item{n}{Total number of treatments.}
\item{d}{Total number of designs (corresponding to the unique set
  of treatments compared within studies).}

\item{trts}{Treatments included in network meta-analysis.}
\item{k.trts}{Number of studies evaluating a treatment.}
\item{n.trts}{Number of observations receiving a treatment (if
  available).}
\item{events.trts}{Number of events observed for a treatment (if
  available).}

\item{studies}{Study labels coerced into a factor with its levels
  sorted alphabetically.}
\item{narms}{Number of arms for each study.}

\item{designs}{Vector with unique designs present in the network. A
  design corresponds to the set of treatments compared within a
  study.}
\item{comparisons}{Vector with unique direct comparisons present in the
  network.}

\item{comparison}{Results for pairwise comparisons (data frame with
  columns studlab, treat1, treat2, TE, seTE, lower, upper, z, p).}
\item{comparison.nma.common}{Results for pairwise comparisons based
  on common effects model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p, leverage).}
\item{comparison.nma.random}{Results for pairwise comparisons based
  on random effects model (data frame with columns studlab, treat1,
  treat2, TE, seTE, lower, upper, z, p).}

\item{common}{Results for common effects model (a list with elements
  TE, seTE, lower, upper, z, p).}
\item{random}{Results for random effects model (a list with
  elements TE, seTE, lower, upper, z, p).}
\item{predict}{Prediction intervals (a list with elements seTE,
  lower, upper).}

\item{Q}{Overall heterogeneity / inconsistency statistic.}
\item{df.Q}{Degrees of freedom for test of heterogeneity /
  inconsistency.}
\item{pval.Q}{P-value for test of heterogeneity / inconsistency.}
\item{I2, lower.I2, upper.I2}{I-squared, lower and upper confidence
  limits.}
\item{tau}{Square-root of between-study variance.}

\item{Q.heterogeneity}{Overall heterogeneity statistic.}
\item{df.Q.heterogeneity}{Degrees of freedom for test of overall
  heterogeneity.}
\item{pval.Q.heterogeneity}{P-value for test of overall
  heterogeneity.}

\item{Q.inconsistency}{Overall inconsistency statistic.}
\item{df.Q.inconsistency}{Degrees of freedom for test of overall
  inconsistency.}
\item{pval.Q.inconsistency}{P-value for test of overall
  inconsistency.}

\item{Q.decomp}{Data frame with columns 'treat1', 'treat2', 'Q',
  'df' and 'pval.Q', providing heterogeneity statistics for each
  pairwise meta-analysis of direct comparisons.}

\item{sm}{A character string indicating underlying summary
  measure.}
\item{method}{A character string indicating which method is to be
  used for pooling of studies.}
\item{level}{The level used to calculate confidence intervals for
  individual studies.}
\item{level.ma}{The level used to calculate confidence intervals
  for pooled estimates.}
\item{level.predict}{The level used to calculate prediction
  intervals for a new study.}

\item{ci.lab}{Label for confidence interval.}

\item{incr}{Numerical value added to cell frequencies
  (if applicable).}
\item{method.incr}{A character string indicating which continuity
  correction method was used (if applicable).}
\item{allstudies}{A logical indicating whether studies with zero
  events or non-events in all treatment arms should be included in
  an inverse variance meta-analysis (if applicable).}
\item{cc.pooled}{A logical indicating whether \code{incr} should be
  used as a continuity correction (if applicable).}

\item{reference.group, baseline.reference}{As defined above.}
\item{all.treatments}{As defined above.}
\item{seq}{A character specifying the sequence of treatments.}

\item{tau.preset}{An optional value for the square-root of the
  between-study variance \eqn{\tau^2}.}

\item{sep.trts}{A character used in comparison names as separator
  between treatment labels.}
\item{nchar.trts}{A numeric defining the minimum number of
  characters used to create unique treatment names.}

\item{prediction, overall.hetstat, backtransf}{As defined above.}
 
\item{title}{Title of meta-analysis / systematic review.}
\item{call}{Function call.}
\item{version}{Version of R package \bold{netmeta} used to create object.}
}
\description{
Summary method for objects of class \code{netmeta}.
}
\examples{
# Examples: example(netmeta)

}
\seealso{
\code{\link{netmeta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nettable.R
\name{nettable}
\alias{nettable}
\alias{print.nettable}
\title{Table with network meta-analysis results}
\usage{
nettable(
  ...,
  name = NULL,
  method = NULL,
  order = NULL,
  common,
  random,
  upper = TRUE,
  reference.group = NULL,
  baseline.reference = NULL,
  backtransf = NULL,
  nchar.trts = if (writexl) 666 else NULL,
  digits = gs("digits"),
  digits.I2 = gs("digits.I2"),
  digits.pval = gs("digits.pval"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  big.mark = gs("big.mark"),
  text.NA = ".",
  bracket = gs("CIbracket"),
  separator = gs("CIseparator"),
  lower.blank = gs("CIlower.blank"),
  upper.blank = gs("CIupper.blank"),
  tol.direct = 5e-04,
  writexl = !missing(path),
  path = "nettable.xlsx",
  overwrite = FALSE,
  warn = FALSE,
  verbose = FALSE
)

\method{print}{nettable}(x, common = x$x$common, random = x$x$random, legend = gs("legend"), ...)
}
\arguments{
\item{\dots}{Any number of network meta-analysis objects or a
single list with network meta-analyses.}

\item{name}{An optional character vector providing descriptive
names for network meta-analysis objects.}

\item{method}{A character string indicating which method to split
direct and indirect evidence is to be used. Either
\code{"Back-calculation"} or \code{"SIDDE"}, can be abbreviated.
See Details.}

\item{order}{A optional character or numerical vector specifying
the order of treatments in comparisons.}

\item{common}{A logical indicating whether table for the common
effects network meta-analysis should be printed.}

\item{random}{A logical indicating whether table for the random
effects network meta-analysis should be printed.}

\item{upper}{A logical indicating whether treatment comparisons
should be selected from the lower or upper triangle of the
treatment effect matrices (see list elements \code{TE.common} and
\code{TE.random} in the \code{netmeta} object). Ignored if
argument \code{order} is provided.}

\item{reference.group}{Reference treatment. Ignored if argument
\code{order} is provided.}

\item{baseline.reference}{A logical indicating whether results
should be expressed as comparisons of other treatments versus the
reference treatment or vice versa. This argument is only
considered if \code{reference.group} is not equal to \code{""}
and argument\code{order} is not provided.}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. For example, if \code{backtransf =
TRUE}, results for \code{sm = "OR"} are printed as odds ratios
rather than log odds ratios.}

\item{nchar.trts}{A numeric defining the minimum number of
characters used to create unique treatment names.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.I2}{Minimal number of significant digits for I-squared
statistic, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of test of agreement between direct and indirect evidence, see
\code{print.default}.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{big.mark}{A character used as thousands separator.}

\item{text.NA}{A character string specifying text printed for
missing values.}

\item{bracket}{A character with bracket symbol to print lower
confidence interval: "[", "(", "\{", "".}

\item{separator}{A character string with information on separator
between lower and upper confidence interval.}

\item{lower.blank}{A logical indicating whether blanks between left
bracket and lower confidence limit should be printed.}

\item{upper.blank}{A logical indicating whether blanks between
separator and upper confidence limit should be printed.}

\item{tol.direct}{A numeric defining the maximum deviation of the
direct evidence proportion from 0 or 1 to classify a comparison
as providing only indirect or direct evidence, respectively.}

\item{writexl}{A logical indicating whether an Excel file should be
created (R package \bold{writexl} must be available).}

\item{path}{A character string specifying the filename of the Excel
file.}

\item{overwrite}{A logical indicating whether an existing Excel
file should be overwritten.}

\item{warn}{A logical indicating whether warnings should be
printed.}

\item{verbose}{A logical indicating whether progress information
should be printed.}

\item{x}{An object of class \code{nettable}.}

\item{legend}{A logical indicating whether a legend should be
printed for abbreviated treatment names.}
}
\value{
An object of class \code{nettable} with corresponding \code{print}
function if argument \code{writexl = FALSE}. The object is a list
containing the network tables in list elements 'common' and
'random'. An Excel file is created if \code{writexl = TRUE}. In
this case, \code{NULL} is returned in R.
}
\description{
Construct a table with network, direct and indirect estimates from
one or more network meta-analyses.
}
\details{
Construct a table with network, direct and indirect estimates from
one or more network meta-analyses. The table looks very similar to
the statistical part of a GRADE table for a network meta-analysis
(Puhan et al., 2014).

By default, an R object with the network tables is
generated. Alternatively, an Excel file is created if argument
\code{writexl = TRUE}.

Two methods to derive indirect estimates are available:
\itemize{
\item Separate Indirect from Direct Evidence (SIDE) using a
  back-calculation method (\code{method = "Back-calculation"})
  based on the \emph{direct evidence proportion} to calculate the
  indirect evidence (König et al., 2013);
\item Separate Indirect from Direct Design Evidence (SIDDE) as
  described in Efthimiou et al. (2019).
}

Note, for the back-calculation method, indirect treatment estimates
are already calculated in \code{\link{netmeta}} and this function
combines and prints these estimates in a user-friendly
way. Furthermore, this method is not available for the
Mantel-Haenszel and non-central hypergeometric distribution
approach implemented in \code{\link{netmetabin}}.

For the random-effects model, the direct treatment estimates are
based on the common between-study variance \eqn{\tau^2} from the
network meta-analysis, i.e. the square of list element
\code{x$tau}.

The SIDDE approach can be compute-intensive in large
networks. Crude information on the computation progress is printed
for SIDDE if argument \code{verbose} is \code{TRUE}.
}
\examples{
pw <- pairwise(treatment, event = r, n = N,
  studlab = author, data = Woods2010, sm = "OR")
#
nma <- netmeta(pw)
#
nt <- nettable(nma, digits = 2)
nt

\donttest{
print(nt, common = FALSE)
print(nt, random = FALSE)

# Create a CSV file with network table from random effects model
#
table <- nettable(nma, digits = 2, bracket = "(", separator = " to ")
#
write.table(table$random, file = "table-random.csv",
  row.names = FALSE, col.names = TRUE, sep = ",")
#
# Create Excel files with network tables
# (if R package writexl is available)
#
if (requireNamespace("writexl", quietly = TRUE))
  nettable(nma, digits = 2, bracket = "(", separator = " to ",
           path = tempfile(fileext = ".xlsx"))
}

}
\references{
Dias S, Welton NJ, Caldwell DM, Ades AE (2010):
Checking consistency in mixed treatment comparison meta-analysis.
\emph{Statistics in Medicine},
\bold{29}, 932--44

Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
(2019):
A Mantel-Haenszel model for network meta-analysis of rare events.
\emph{Statistics in Medicine},
\bold{38}, 2992--3012

König J, Krahn U, Binder H (2013):
Visualizing the flow of evidence in network meta-analysis and
characterizing mixed treatment comparisons.
\emph{Statistics in Medicine},
\bold{32}, 5414--29

Puhan MA, Schünemann HJ, Murad MH, et al. (2014):
A GRADE working group approach for rating the quality of treatment
effect estimates from network meta-analysis.
\emph{British Medical Journal},
\bold{349}, g5630
}
\seealso{
\code{\link{netsplit}}, \code{\link{netmeta}},
  \code{\link{netmetabin}}, \code{\link{netmeasures}},
  \code{\link[metabook]{Woods2010}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Baker2009.R
\docType{data}
\name{Baker2009}
\alias{Baker2009}
\title{Network meta-analysis of pharmacologic treatments for chronic
obstructive pulmonary disease}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{year}}\tab year of publication \cr
\bold{\emph{id}}\tab study ID \cr
\bold{\emph{treatment}}\tab treatment \cr
\bold{\emph{exac}}\tab one or more episodes of COPD exacerbation
  \cr
\bold{\emph{total}}\tab number of individuals in treatment arm
}
}
\source{
Baker WL, Baker EL, Coleman CI (2009):
Pharmacologic Treatments for Chronic Obstructive Pulmonary Disease:
A Mixed-Treatment Comparison Meta-analysis.
\emph{Pharmacotherapy: The Journal of Human Pharmacology and Drug
Therapy},
\bold{29}, 891--905
}
\description{
This dataset comes from a systematic review of randomized
controlled trials on pharmacologic treatments for chronic
obstructive pulmonary disease (COPD) (Baker et al., 2009).

The primary outcome, occurrence of one or more episodes of COPD
exacerbation, is binary (yes / no). For this outcome, five drug
treatments (fluticasone, budesonide, salmeterol, formoterol,
tiotropium) and two combinations (fluticasone + salmeterol,
budesonide + formoterol) were compared to placebo. The authors
considered the two combinations as separate treatments instead of
evaluating the individual components.
}
\note{
The dataset Baker2009 is identical to dataset
\code{\link[metabook]{Baker2009}} in R package \bold{metabook}.
}
\examples{
head(Baker2009)

# Example using pairwise(), netmeta() and netcomb():
# example(Baker2009, package = "metabook")
}
\seealso{
\code{\link[metabook]{Baker2009}},
  \code{\link[meta]{pairwise}}, \code{\link[meta]{metabin}},
  \code{\link{netmeta}}, \code{\link{netcomb}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.netposet.R
\name{plot.netposet}
\alias{plot.netposet}
\title{Scatter plot or biplot showing partially order of treatment ranks}
\usage{
\method{plot}{netposet}(
  x,
  plottype = "scatter",
  pooled = ifelse(x$random, "random", "common"),
  dim = "2d",
  sel.x = 1,
  sel.y = 2,
  sel.z = 3,
  cex = 1,
  col = "black",
  cex.text = cex,
  col.text = col,
  adj.x = 0,
  adj.y = 1,
  offset.x = 0.005,
  offset.y = -0.005,
  pch = NULL,
  cex.points = cex,
  col.points = col,
  col.lines = "black",
  lty.lines = 1,
  lwd.lines = 1,
  arrows = FALSE,
  length = 0.05,
  grid = TRUE,
  col.grid = "gray",
  lty.grid = 2,
  lwd.grid = 1,
  ...
)
}
\arguments{
\item{x}{An object of class \code{netmeta} (mandatory).}

\item{plottype}{A character string indicating whether a scatter
plot or biplot should be produced, either \code{"scatter"} or
\code{"biplot"}. Can be abbreviated.}

\item{pooled}{A character string indicating whether scatter plot
should be drawn for common (\code{"common"}) or random effects
model (\code{"random"}). Can be abbreviated.}

\item{dim}{A character string indicating whether a 2- or
3-dimensional plot should be produced, either \code{"2d"} or
\code{"3d"}. Can be abbreviated.}

\item{sel.x}{A numeric specifying number of outcome to use for the
x-axis in a scatterplot (argument is not considered for a
biplot).}

\item{sel.y}{A numeric specifying number of outcome to use for the
y-axis in a scatterplot (argument is not considered for a
biplot).}

\item{sel.z}{A numeric specifying number of outcome to use for the
z-axis in a scatterplot (argument is not considered for a
biplot).}

\item{cex}{The magnification to be used for treatment labels and
points.}

\item{col}{Colour(s) of treatment labels and points.}

\item{cex.text}{The magnification to be used for treatment labels.}

\item{col.text}{Colour(s) of treatment labels.}

\item{adj.x}{Value(s) in [0, 1] to specify adjustment of treatment
labels on x-axis (only considered in 2-D plots); see
\code{\link[graphics]{text}}.}

\item{adj.y}{Value(s) in [0, 1] to specify adjustment of treatment
labels on y-axis (only considered in 2-D plots); see
\code{\link[graphics]{text}}.}

\item{offset.x}{Offset(s) of treatment labels on x-axis (only
considered in 2-D plots).}

\item{offset.y}{Offset(s) of treatment labels on y-axis (only
considered in 2-D plots).}

\item{pch}{Plot symbol(s) for points; no points printed if equal to
\code{NULL}.}

\item{cex.points}{Magnification(s) to be used for points.}

\item{col.points}{Colour(s) of points.}

\item{col.lines}{Line colour.}

\item{lty.lines}{Line type.}

\item{lwd.lines}{Line width.}

\item{arrows}{A logical indicating whether arrows should be printed
(only considered in 2-D plots).}

\item{length}{Length of arrows; see \code{\link[graphics]{arrows}}.}

\item{grid}{A logical indicating whether grid lines should be added
to plot.}

\item{col.grid}{Colour of grid lines.}

\item{lty.grid}{Line type of grid lines.}

\item{lwd.grid}{Line width of grid lines.}

\item{\dots}{Additional graphical arguments.}
}
\description{
This function generates a scatter plot or biplot of P-scores with
an overlay describing partial order of treatment ranks.
}
\details{
By default (arguments \code{plottype = "scatter"} and \code{dim =
"2d"}), a scatter plot is created showing P-scores (see
\code{\link{netrank}}) for the first two outcomes considered in the
generation of a partially ordered set of treatment ranks (using
\code{\link{netposet}}). In addition to the P-scores, the partially
order of treatment ranks is shown as lines connecting treatments
which is analogous to a Hasse diagram. If argument \code{dim =
"3d"}), a 3-D scatter plot is generated showing P-scores for the
first three outcomes.

To overcome the restriction of two or three dimension, a biplot
(Gabriel, 1971) can be generated using argument \code{plottype =
"biplot"}. This is essentially a scatter plot using the first two
(\code{dim = "2d"}) or three (\code{dim = "3d"}) components in a
principal components analysis (using
\code{\link[stats]{prcomp}}). Note, if only two / three outcomes
are considered in a \code{netposet} object, a 2-D / 3-D scatter
plot is generated instead of a biplot as a principal component
analysis is superfluous in such a situation.

Arguments \code{sel.x} and \code{sel.y} can be used to select
different outcomes to show on x- and y-axis in a 2-D scatter plot;
argument \code{sel.z} can be used accordingly in a 3-D scatter
plot. These arguments are ignored for a biplot.

Note, in order to generate 3-D plots (argument \code{dim = "3d"}),
R package \bold{rgl} is necessary. Note, under macOS the X.Org X
Window System must be available (see
\url{https://www.xquartz.org}).
}
\examples{
# Examples: example(netposet)

}
\references{
Carlsen L, Bruggemann R (2014):
Partial order methodology: a valuable tool in chemometrics.
\emph{Journal of Chemometrics},
\bold{28}, 226--34

Gabriel KR (1971):
The biplot graphic display of matrices with application to
principal component analysis.
\emph{Biometrika},
\bold{58}, 453--67
}
\seealso{
\code{\link{netposet}}, \code{\link{hasse}},
  \code{\link{netrank}}, \code{\link{netmeta}},
  \code{\link[metabook]{Linde2015}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.netconnection.R
\name{as.data.frame.netconnection}
\alias{as.data.frame.netconnection}
\title{Create a data frame from an object of class netconnection}
\usage{
\method{as.data.frame}{netconnection}(x, ...)
}
\arguments{
\item{x}{An object of class \code{netconnection}.}

\item{\dots}{Additional arguments (ignored).}
}
\value{
A data frame is returned by the function
  \code{as.data.frame}.
}
\description{
The \code{as.data.frame} method returns a data frame containing
information on membership of studies / pairwise comparisons to a
(sub)network.
}
\examples{
# Artificial example with two subnetworks
#
t1 <- c("G", "B", "B", "D", "A", "F")
t2 <- c("B", "C", "E", "E", "H", "A")
#
nc <- netconnection(t1, t2)
print(nc, details = TRUE)

as.data.frame(nc)

}
\seealso{
\code{\link{netconnection}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
